      subroutine cmpt_intrps(hkl_max, interpolants,
     $     pixels, dimem, ctf_val, abs_ctf,
     $     theta, phi, omega, x, y, mag)
      
      implicit none
      
      include '../include/sizes.inc'
      include '../include/info.inc'
      include '../include/com_01.inc'
#include "./map_type.inc"
      
      integer, intent(in) :: hkl_max, dimem
      real, intent(in) :: pixels(dimem), theta, phi, omega, x, y, mag
      real, intent(in) :: ctf_val(0:nfc_2d_row, -nfc_2d_col:nfc_2d_col)
      real, intent(in) :: abs_ctf(0:nfc_2d_row, -nfc_2d_col:nfc_2d_col)
      FMAP_TYPE, intent(inout)::
     $     interpolants(3,-hkl_max:hkl_max,-hkl_max:hkl_max,0:hkl_max)
      
      integer symm, i, num_symm_local, alloc_stat
      
      real du, dv, num_annulus, center(2), imgmagfactor,
     $     rot_mat(3,3), ruv_mat_1(3,2,3), ruv_mat_2(3,2,3),
     $     ruv_sup_1(3,3), ruv_sup_2(3,3), icos_rot(3,3,60),
     $     theta_symm(60), phi_symm(60), omega_symm(60)

      real, allocatable    :: rffta_omega(:)
      complex, allocatable :: fc_2d(:,:)
      
      num_symm_local = num_symm

      allocate(rffta_omega(nrffta_omega),
     $     fc_2d(0:nfc_2d_row, -nfc_2d_col:nfc_2d_col),
     $     stat = alloc_stat)
      
      if(alloc_stat > 0) then
         call error_stop ('P3DR', 'cmpt_intrps',
     $        'Error in allocating memory for orient')
      endif
      
C     Initialize FFT workspace rffta_omega
      call vrffti( nrffta_col, rffta_omega )

C     calculate the radius in FFT space for 3D DFT
      num_annulus = real(nrffta_row)*PIXSIZ_OUT/res_max 
      if (num_annulus >= nrffta_row/2.0) then
         call error_stop('P3DR', 'cmpt_intrps',
     +        'Resolution is too high, should less than 2*PIXEL_SIZE')
      endif
      
      center(1)    = x
      center(2)    = y
            
C     Adjust the imgmagfactor according to its own pixel_size
      imgmagfactor = mag * pixsiz_out / angstrom_per_pix
            
C     Calculate set of equivalent orientations based on application of
C     symmetry operations. Choices include icoshedral, eightfold,
C     dihedral, and asymmetric
      
      if(symm_code == 532) then
         call gen5rot(icos_rot)
         num_symm_local = 5
         do i=1,5
            call equivalent_view(theta, phi, omega, 
     $           icos_rot(1,1,i), theta_symm(i),
     $           phi_symm(i), omega_symm(i))
         enddo
      elseif (symm_code == 8) then
         call eight_symmetry(theta_symm, phi_symm, omega_symm,
     $        theta, phi, omega)
      elseif (symm_code == 5 .or. symm_code == 52 .or.
     $        symm_code == 2 .or. symm_code == 222 .or.
     $        symm_code == 3 .or. symm_code == 32 .or.
     $        symm_code == 6 .or. symm_code == 622 .or.
     $        symm_code == 7 .or. symm_code == 72 .or.
     $        symm_code == 8 .or. symm_code == 822) then
         call dihedral_symmetry(theta_symm, phi_symm, omega_symm,
     $        theta, phi, omega, symm_code, num_symm)
      elseif (symm_code == 1) then
         theta_symm(1) = theta
         phi_symm(1)   = phi
         omega_symm(1) = omega
      else
         call error_stop('P3DR', 'cmpt_intrps',
     $        'Error: unsupported symmetry')
      endif
      
C     Compute Discrete Fourier Transform of pixel values:
      call fft_2dfft(pixels, npix_row, npix_col, center,
     $     nrffta_omega, rffta_omega, nrffta_row, 
     $     nrffta_col, nfc_2d_row, nfc_2d_col, fc_2d)
      du = center(1) - real(int(center(1)))
      dv = center(2) - real(int(center(2)))
      call shift_fft(fc_2d, nfc_2d_row, nfc_2d_col, du, dv)
            
C     Apply CTF correction
      IF(CTFMODE > 0 .and. CTFMODE <= 4) then
         fc_2d = fc_2d * ctf_val ! Fortran 90 array syntax
      endif
            
C     Accumulate contributions to interpolants arrays from all of the
C     symmetry related views of the particle image
      
      do symm = 1, num_symm_local
         
C     Set the rotation matrices which transform the plane containing
C     the projection of the particle to the (X,Y,Z) `standard space'
C     and get the inverse rot_mat
         
         call crowther_to_matrix(theta_symm(symm),
     $        -phi_symm(symm), -omega_symm(symm), rot_mat)
         
C     Set the matrix which is used to determine the intersection of
C     grid line in the standard 3-d space with the rotated place
C     containing the Fourier coefficients of the pixels
         
C     call rottrans_1( rot_mat, ruv_mat_1, ruv_sup_1, -0.5 )
C     call rottrans_1( rot_mat, ruv_mat_2, ruv_sup_2,  0.5 )
         call rottrans_1( rot_mat, ruv_mat_1, ruv_sup_1, -0.999 )
         call rottrans_1( rot_mat, ruv_mat_2, ruv_sup_2,  0.999 )
         
         call matrixentries_slab( nfc_2d_row, nfc_2d_col, fc_2d,
     $        hkl_max, interpolants,
     $        rot_mat, ruv_mat_1, ruv_mat_2, ruv_sup_1, ruv_sup_2,
     $        num_annulus, abs_ctf, imgmagfactor, mapmagfactor)
         
      enddo
      
      deallocate(rffta_omega, fc_2d)
      
      return
      end subroutine cmpt_intrps
