      subroutine dihedral_symmetry(theta_symm, phi_symm, omega_symm,
     $     theta, phi, omega, symm_code, num_symm)

C     For a given orientation, generate the full set of dihedral
C     symmetry related orientations

      implicit none

      integer, intent(in) :: num_symm, symm_code
      real, intent(in)    :: theta, phi, omega
      real, intent(out)   :: theta_symm(num_symm), phi_symm(num_symm),
     $     omega_symm(num_symm)

      integer i, half_symm
      real unit, sin_theta, sin_phi, cos_theta, cos_phi, 
     $     x, y, z, tmp_theta, tmp_phi,
     $     rad_to_deg, deg_to_rad
      
      rad_to_deg = 180.0/(4.0*atan(1.0))
      deg_to_rad = 1.0/rad_to_deg

C     Generate equivalent orientations in upper half
      if (symm_code <= 8) then
         half_symm = num_symm
      else
         half_symm = num_symm/2
      endif
      unit = 360.0 / real(half_symm)

      do i = 1, half_symm
         theta_symm(i) = theta
         phi_symm(i)   = phi + real(i-1)*unit
         omega_symm(i) = omega
      enddo

c     Generate equivalent orientations in lower half
      if (symm_code >= 32) then
         sin_theta = sin(theta*deg_to_rad)
         sin_phi   = sin(phi*deg_to_rad)
         cos_theta = cos(theta*deg_to_rad)
         cos_phi   = cos(phi*deg_to_rad)

         x = sin_theta * cos_phi
         y = sin_theta * sin_phi
         z = cos_theta

         x = - x
         z = - z

         tmp_theta = acos(z) * rad_to_deg
         if (x == 0 .and. y == 0) then
            tmp_phi = phi
         else
            tmp_phi = atan2(y, x) * rad_to_deg
         endif

         do i = half_symm+1, num_symm
            theta_symm(i) = tmp_theta
            phi_symm(i)   = tmp_phi + real(i-half_symm-1)*unit
            omega_symm(i) = mod(omega+180.0, 360.0)
         enddo
      endif

C     Get phi into range (-180,180)
      do i = 1, num_symm
         phi_symm(i) = mod(phi_symm(i), 360.0)
         if (phi_symm(i) > 180.0) then
            phi_symm(i) = phi_symm(i) - 360.0
         endif
         if (phi_symm(i) < -180.0) then
            phi_symm(i) = phi_symm(i) + 360.0
         endif
      enddo

      return
      end subroutine dihedral_symmetry
