      subroutine gen5rot(rot)                              
C
C     Modified from G. Bricogne's program genrot in the mrc package.
C     Generates the fivefold rotation matrices for 532 symmetry in the
C     orthnormal frame based on three dyads .
C
C     Author: Steve Fuller 10/29/89
C
C     Modified: Robert Sinkovits 6/27/06 - general cleanup and bringing
C     in line with Fortran 90 standard
C     Modified: Robert Sinkovits 12/8/06 - use save attribute so that
C     rotation matrices are only calculated the first time that gen5rot
C     is called.

      real, intent(out) :: rot(3,3,5)
      
      real, save  :: save_rot(3,3,5)
      logical, save :: first_time = .true.
      
      double precision drot(3,3), p5(3,3), p5inv(3,3), s5(3,3,5)
      double precision theta, pi, tpiov5, root5, xx5, cos5, sin5
      double precision sine, cosine
      integer k, n

      if(first_time) then
         first_time = .false.
         pi     = acos(-1.0d0)
         tpiov5 = pi/2.5         
         root5 = sqrt(5.0)
         xx5  = 0.5/root5
         cos5 = sqrt(0.5+xx5)
         sin5 = sqrt(0.5-xx5)

         p5 = 0.0               ! Fortran 90 array syntax
         p5inv = 0.0            ! Fortran 90 array syntax
         s5 = 0.0               ! Fortran 90 array syntax
         
C     Generate 5-fold matrices
         theta = -tpiov5
         do k = 1,5
            theta  = theta+tpiov5
            cosine = cos(theta)
            sine   = sin(theta)
            s5(1,1,k) = cosine
            s5(2,1,k) = sine
            s5(1,2,k) = -sine
            s5(2,2,k) = cosine
            s5(3,3,k) = 1.0
         enddo
         
C     Generate  xz-fivefold transition matrices
         p5(1,1) = cos5
         p5(3,1) = sin5
         p5(2,2) = 1.0
         p5(1,3) = -sin5
         p5(3,3) = cos5
         p5inv(1,1) = cos5
         p5inv(3,1) = -sin5
         p5inv(2,2) = 1.0
         p5inv(1,3) = sin5
         p5inv(3,3) = cos5
         
         do n=1,5
            drot = matmul(matmul(p5,s5(1:3,1:3,n)),p5inv)
            save_rot(1:3,1:3,n) = real(drot(1:3,1:3))
         enddo
         
      endif

      rot = save_rot            ! Fortran 90 array syntax
         
      return
      end subroutine gen5rot
