      subroutine noise_suppression(interpolants, hkl_max, 
     $     convert_to_invang, nfsc, fsc_freq, fsc_value)

C     Apply the Rosenthal and Henderson, JMB 333 721-745 (2003) noise
C     suppression algorithm to the Fourier transform of the
C     map. convert_to_invang is used to convert index-space to 1/A

      implicit none
#include "./map_type.inc"

      integer, intent(in) :: hkl_max, nfsc
      real, intent(in) :: convert_to_invang
      real, intent(in) :: fsc_freq(nfsc), fsc_value(nfsc)
      FMAP_TYPE, intent(inout) ::
     +     interpolants(3,-hkl_max:hkl_max,-hkl_max:hkl_max,0:hkl_max)

      integer locate, j, ih, ik, il
      real noise_factor, myfreq, myfsc, delta_lower, delta_total

      do il = 0, hkl_max
         do ik = -hkl_max, hkl_max
            do ih = -hkl_max, hkl_max
               
               myfreq = sqrt(real(ih*ih+ik*ik+il*il))*convert_to_invang
               j = locate(fsc_freq, nfsc, myfreq)
               if (j == 0) then
                  myfsc = fsc_value(1)
               else if (j == nfsc) then
                  myfsc = fsc_value(nfsc)
               else
                  delta_lower = myfreq - fsc_freq(j)
                  delta_total = fsc_freq(j+1) - fsc_freq(j)
                  myfsc = fsc_value(j) + (delta_lower/delta_total)*
     $                 (fsc_value(j+1)-fsc_value(j))
               endif
               
               myfsc = max(0.0,myfsc)
               noise_factor = sqrt((2.0*myfsc)/(1.0+myfsc))
               
               interpolants(2,ih,ik,il) = interpolants(2,ih,ik,il)*
     $              noise_factor
               interpolants(3,ih,ik,il) = interpolants(3,ih,ik,il)*
     $              noise_factor
            enddo
         enddo
      enddo
      
      return
      end subroutine noise_suppression
