      subroutine rearrange_2_slab(min_xyz, max_xyz, rrnum,
     +     F_VRU_TMP, min_x, max_x, F_XK_dZ )
C                                                            REARRANGE_2
C **********************************************************************
C *                                                               *
C * Subroutine: rearrange_2_slab                                  *
C * Author: Robert E. Lynch (rel@cs.purdue.edu)                   *
C * Date:   Oct. 10, 1998                                         *
C * Used in:  exchange_2_m_slab -- The subroutine for exchanging  *
C *             data for 1-D Fourier synthesis, "piecewise        *
C *             constant" model.                                  *
C * Function: Rearrange receive buffer so that the data are in    *
C *           right order.                                        *
C * Usage:                                                        *
C *   call rearrange_2_slab( min_xyz, max_xyz, num_hkl,           *
C *          F_VRU_TMP, min_x,max_x, min_y,max_y, min_z,max_z,    *
C *          F_XK_dZ )                                            *
C * Input:                                                        *
C *   MIN_XYZ, MAX_XYZ (INTEGER*4) -- Minimum and maximum of X,   *
C *                                   Y, Z                        *
C *   NUM_HKL (INTEGER*4) -- Total number of H,K, or L indices; a *
C *     dimension of arrays F_VRU_TMP, F_XK_dZ, F_XL_dY, and      *
C *     F_YH_dZ                                                   *
C *   F_VRU_TMP (REAL*4) -- Output from ALL-T0-ALL exchange       *
C *   MIN_X, MAX_X, MIN_Y, MAX_Y, MIN_Z, MAX_Z (INTEGER*4) --     *
C *     first and last X,Y,Z coordinates                          *
C * Output:                                                       *
C *   F_XK_dZ (REAL*4) -- values having X transformed from H,     *
C *     some of Z transfomred from L, and K-Fourier coefficients  *
C *                                                               *
C * Algorithm:                                                    *
C *   This subroutine moves data from F_VRU_TMP to F_XK_dZ,       *
C *   F_XL_dY and F_YH_dZ. Data in F_VRU_TMP are obtained by a    *
C *   global exchange among all the nodes. F_VRU_TMP does not     *
C *   have a uniform multi-dimensional array structure; the array *
C *   is filled as:                                               *
C *        F_VRU_TMP( MIN_XYZ:MAX_XYZ,                            *
C *                   FRST_LST_RR_OUT(1,1):FRST_LST_RR_OUT(2,1)   *
C *                   FRST_LST_U(1,IAM):FRST_LST_U(2:IAM)         *
C *                                                               *
C *                   followed by                                 *
C *                                                               *
C *                   MIN_XYZ:MAX_XYZ,                            *
C *                   FRST_LST_RR_OUT(1,2):FRST_LST_RR_OUT(2,2)   *
C *                   FRST_LST_U(1,IAM):FRST_LST_U(2:IAM)         *
C *                                                               *
C *                   etc.                                        *
C *                                                               *
C *   and the value of                                            *
C *        FRST_LST_RR_OUT(1,N):FRST_LST_RR_OUT(2,N)              *
C *   is different for different values of node number N. All of  *
C *   them are included so that F_VRU_TMP contains values for:    *
C *       all X: min_xyz:max_xyz                                  *
C *       all K: RR for 1:num_hkl                                 *
C *       some Z: for FRST_LST_U(1,iam):for FRST_LST_U            *
C *                                                               *
C *   This subroutine needs to combine all RRs and separate       *
C *   1:MAX_AXIS into three arrays, so that the data are stored   *
C *   in F_XK_dZ, F_XL_dY and F_YH_dZ. It calls subroutine        *
C *   MOVE_DATA_2 to rearrange the data.                          *
C *                                                               *
C * Modification history:                                         *
C *     - Hong Lin, Version 2.0, Oct. 8, 1999                     *
C *       Correct some errors.                                    *
C *     - Hong Lin, Version 2.0, Dec.10, 1999                     *
C *       Delete variable MAX_AXIS.                               *
C *   - Yongchang Ji, Final Version, Aug.2000 -- Jan. 2002             *

C *                                                               *
C *****************************************************************
C From COMMONs ---------------------------------------------------------
C  IAM          INTEGER*4; node number; in a COMMON in ALLPROGS.inc.
 
C  NUM_NODES    INTEGER*4; number of nodes; in a COMMON in ALLPROGS.inc.
 
C  FRST_LST_RR_OUT(1:2,0:(NUM_NODES-1) INTEGER*4; first and last values
C          of RR; in a COMMON in SMALL_ARRAY.inc:
C                   RR .eq. K
 
C  FRST_LST_U(1:2,0:(NUM_NODES-1) INTEGER*4; first and last values of U;
C          in a COMMON in SMALL_ARRAY.inc:
C                    U .eq. Z
 
C  RDISPLS(0:NUM_NODES) INTEGER*4; offset in receive buffer of items
C          received by IAM from NODE; in a COMMON in SMALL_ARRAY.inc.
C ----------------------------------------------------------------------
C
      implicit none
      include '../include/allprogs.inc'
 
C     Subroutine arguments
      integer min_xyz, max_xyz, rrnum, min_x, max_x
      real F_VRU_TMP( int8(-min_xyz+max_xyz+1)*rrnum
     +                *(-frst_lst_U(1,iam)+frst_lst_U(2,iam)+1) )
      real F_XK_dZ( min_x:max_x, rrnum,
     +              FRST_LST_U(1,IAM):FRST_LST_U(2,IAM))

C     Local variables
      integer node, num_x
 
      num_x = - min_x + max_x + 1
      
      do node = 0, num_nodes - 1
         call move_data_2_slab( min_xyz, max_xyz,
     +        frst_lst_RR_out(1,node), frst_lst_RR_out(2,node),
     +        frst_lst_U(1,iam), frst_lst_U(2,iam),
     +        F_VRU_TMP(rdispls(node) + 1),
     +        min_x, max_x, rrnum, num_x, F_XK_dZ )
      enddo
 
      return
      end subroutine rearrange_2_slab
