      subroutine calc_avgs (nids_tot, mags, pft_ccs, prj_ccs, cmp_ccs)

C     Compute average and standard deviations of array mags, cc_pft,
C     cc_prj, cc_cmp. Print results to stdout
C     
C     D. Belnap, July 1996
C     Modified 23-Apr-1998 by T. Baker
C     21-Jul-2006, R. Sinkovits, general cleanup

      implicit none

      integer*4, intent(in) :: nids_tot
      real*4, intent(in) :: mags(nids_tot)
      real*4, intent(in) :: pft_ccs(nids_tot)
      real*4, intent(in) :: prj_ccs(nids_tot)
      real*4, intent(in) :: cmp_ccs(nids_tot)

      real*4 mags_avg, mags_std
      real*4 pftcc_avg, pftcc_std
      real*4 prjcc_avg, prjcc_std
      real*4 cmpcc_avg, cmpcc_std
      integer*4 id

      if(nids_tot .le. 0) return

      write(*,*) '================================================'
      write(*,*) ' compute average mag factor and coefficients'
      write(*,*) ' number of particles = ', nids_tot
      write(*,*) '================================================'

C     Initialize parameters
      mags_avg  = 0.0
      mags_std  = 0.0
      pftcc_avg = 0.0
      pftcc_std = 0.0
      prjcc_avg = 0.0
      prjcc_std = 0.0
      cmpcc_avg = 0.0
      cmpcc_std = 0.0

C     Calculate averages
      do id = 1,nids_tot
         mags_avg  = mags_avg  + mags(id)
         pftcc_avg = pftcc_avg + pft_ccs(id)
         prjcc_avg = prjcc_avg + prj_ccs(id)
         cmpcc_avg = cmpcc_avg + cmp_ccs(id)
      enddo
      mags_avg  = mags_avg  / float(nids_tot)
      pftcc_avg = pftcc_avg / float(nids_tot)
      prjcc_avg = prjcc_avg / float(nids_tot)
      cmpcc_avg = cmpcc_avg / float(nids_tot)
      
C     Calculate standard deviations
      if(nids_tot .gt. 1) then
        do id = 1,nids_tot
          mags_std  =  mags_std  + (mags(id)   - mags_avg)**2
          pftcc_std =  pftcc_std + (pft_ccs(id) - pftcc_avg)**2
          prjcc_std =  prjcc_std + (prj_ccs(id) - prjcc_avg)**2
          cmpcc_std =  cmpcc_std + (cmp_ccs(id) - cmpcc_avg)**2
        enddo
        mags_std  = sqrt( mags_std  / float(nids_tot-1) )
        pftcc_std = sqrt( pftcc_std / float(nids_tot-1) )
        prjcc_std = sqrt( prjcc_std / float(nids_tot-1) )
        cmpcc_std = sqrt( cmpcc_std / float(nids_tot-1) )
      endif

      write(*,*) '                     average  std. dev.'
      write(*,*) '---------------------------------------'
      write(*,'(a,2(f10.3))') ' mag factor      ', mags_avg,  mags_std
      write(*,'(a,2(f10.3))') ' pft coefficient ', pftcc_avg, pftcc_std
      write(*,'(a,2(f10.3))') ' prj coefficient ', prjcc_avg, prjcc_std
      write(*,'(a,2(f10.3))') ' cmp coefficient ', cmpcc_avg, cmpcc_std

      return
      end subroutine calc_avgs
