      subroutine cc_radii (pmap, fltpmap, polar_prj, nrot, nannuli,
     $     ann_hi, ccrad, verbose)

C     Calculate the real-space correlation coefficients as a function of
C     annulus (radius).
C
C     ccrad(1,n) = correlation coefficient: pmap vs. polar_prj
C     ccrad(2,n) = correlation coefficient: filtered pmap vs. polar_prj
C
C     February 1993  purdue university vax 8550, T. Baker
C     Modified  4-jun-1998 by TSB (renamed cc_radii)
C     Modified 21-Jul-2006, R. Sinkovits, general cleanup
      
      implicit none
      
      integer*4, intent(in) :: nrot, nannuli, ann_hi, verbose
      real*4, intent(in)    :: pmap(nrot,nannuli)
      real*4, intent(in)    :: fltpmap(nrot,nannuli)
      real*4, intent(in)    :: polar_prj(nrot,nannuli)
      real*4, intent(out)   :: ccrad(2,nannuli)

      integer*4 i, ii, j, ann_hi_local
      real*8    sum12(2,nannuli), sq1(nannuli)
      real*8    sq2(nannuli), sq3(nannuli)
      real*4    div1, div2, div3
      real*4    avg1(nannuli), avg2(nannuli), avg3(nannuli)

      ann_hi_local = ann_hi
      if(verbose .ge. 1) ann_hi_local = nannuli

      do i = 1,min(nannuli, ann_hi_local+1)
         avg1(i) = 0.0
         avg2(i) = 0.0
         avg3(i) = 0.0
         sq1(i)  = 0.0
         sq2(i)  = 0.0
         sq3(i)  = 0.0

         do ii = 1,2
            ccrad(ii,i) = 0.0
            sum12(ii,i) = 0.0
         enddo

         do j = 1,nrot
            sum12(1,i) = sum12(1,i) + pmap(j,i)*polar_prj(j,i)
            sum12(2,i) = sum12(2,i) + fltpmap(j,i)*polar_prj(j,i)
            avg1(i)    = avg1(i) + pmap(j,i)
            avg2(i)    = avg2(i) + polar_prj(j,i)
            avg3(i)    = avg3(i) + fltpmap(j,i)
            sq1(i)     = sq1(i) + pmap(j,i)**2
            sq2(i)     = sq2(i) + polar_prj(j,i)**2
            sq3(i)     = sq3(i) + fltpmap(j,i)**2
         enddo

         avg1(i) = avg1(i)/real(nrot)
         avg2(i) = avg2(i)/real(nrot)
         avg3(i) = avg3(i)/real(nrot)
         div1    = real(sq1(i)) - real(nrot)*(avg1(i)**2)
         div2    = real(sq2(i)) - real(nrot)*(avg2(i)**2)
         div3    = real(sq3(i)) - real(nrot)*(avg3(i)**2)
         div1    = sign(sqrt(abs(div1)), div1)
         div2    = sign(sqrt(abs(div2)), div2)
         div3    = sign(sqrt(abs(div3)), div3)
         if(div1*div2.le.0.0) then
            ccrad(1,i) = 0.0
         else
            ccrad(1,i) = real((sum12(1,i)-avg1(i)*avg2(i)*real(nrot)) /
     $           (div1*div2))
         endif
         if(div3*div2.le.0.0) then
            ccrad(2,i) = 0.0
         else
            ccrad(2,i) = real((sum12(2,i)-avg3(i)*avg2(i)*real(nrot)) /
     $           (div3*div2))
         endif
      enddo
      
      return
      end subroutine cc_radii
