      subroutine get_thephi (pftimg, pftprj, nrot, nannuli,
     $     nview_mod, jcut, pixres_min, pixres_max,
     $     cc_pft, ann_lo, ann_hi, var, index)
      
C     determines the best theta,phi orientation
C     
C     may 1992  purdue university vax 8550, t. baker
C     modified 18-aug-1993 by t. baker.
C     modified xx_apr-1996 by dmb (fixed bug in extended output).
C     modified 23-apr-1998 by t. baker (renamed from pftcc_gettp)
C     modified 14-aug-2006, r sinkovits, general cleanup

      implicit none
      
      integer, intent(in)  :: jcut, nrot, nannuli, nview_mod
      integer, intent(in)  :: ann_lo, ann_hi
      real, intent(in)     :: pftimg((nrot/4),nannuli)
      real, intent(in)     :: pftprj((nrot/4),nannuli,nview_mod)
      real, intent(in)     :: pixres_min, pixres_max
      logical, intent(in)    :: var((nrot/4),nannuli)

      integer, intent(out) :: index
      real, intent(out)    :: cc_pft

      integer j, id, jn_lb, jn_ub, ir, r
      logical all_var_true
      real  sumx, sumy, sumxx, sumyy, sumxy
      real  ccor, npts, div1, div2, twopir, pi
      
      index = 1
      pi = acos(-1.0)
      cc_pft = -huge(cc_pft)
      
C     Determine whether or not all elements of array var are true. If
C     so, inner most computational loop can be streamlined

      all_var_true = all(var)   ! Fortran 90 intrinsic function

      do id = 1,nview_mod
         npts  = 0.0
         sumxx = 0.0
         sumyy = 0.0
         sumxy = 0.0
         sumx  = 0.0
         sumy  = 0.0
         do ir = ann_lo,ann_hi
            r      = min(nannuli, ir + 1)
            twopir = 2.0*pi*float(ir)

            jn_lb = max(1, jcut, ceiling(twopir/pixres_min))
            jn_ub = min(nrot/4-1, floor(twopir/pixres_max))

            if(all_var_true) then
               do j = jn_lb+1, jn_ub+1
                  sumxx = sumxx + pftimg(j,r)*pftimg(j,r)
                  sumyy = sumyy + pftprj(j,r,id)*pftprj(j,r,id)
                  sumxy = sumxy + pftimg(j,r)*pftprj(j,r,id)
                  sumx  = sumx  + pftimg(j,r)
                  sumy  = sumy  + pftprj(j,r,id)
               enddo
               npts = npts + (jn_ub - jn_lb + 1)
            else
               do j = jn_lb+1, jn_ub+1
                  if(var(j,r)) then
                     npts = npts + 1.0
                     sumxx = sumxx + pftimg(j,r)*pftimg(j,r)
                     sumyy = sumyy + pftprj(j,r,id)*pftprj(j,r,id)
                     sumxy = sumxy + pftimg(j,r)*pftprj(j,r,id)
                     sumx  = sumx  + pftimg(j,r)
                     sumy  = sumy  + pftprj(j,r,id)
                  endif
               enddo
            endif

         enddo

         div1 = npts*sumxx - sumx*sumx
         div2 = npts*sumyy - sumy*sumy
         ccor = (npts*sumxy - sumx*sumy)/sqrt(div1*div2)

         if(ccor > cc_pft) then
            cc_pft = ccor
            index = id
         endif
      enddo

      return
      end subroutine get_thephi
