      subroutine get_tpo_g (ncol, nrow, nrot, nannuli, nview_mod, 
     1     jcut, xcen, ycen, xcen_model, ycen_model, radius, del_ang, 
     2     pixres_min, pixres_max, ann_lo, ann_hi, var,
     3     mod_ts, mod_ps, the_best, phi_best, ome_best, 
     4     pftprj, prjs, cc_pft, cc_prj, 
     5     r4_img, r4_pimg, pfft, prj, polar_prj, pftimg, 
     6     quick_omega)

C     Determine theta and phi of model projection that best matches
C     particle image. Return orientation, correlation coefficients, and
C     hand-adjusted projection. Although the arrays r4_pimg, pfft,
C     polar_prj, and pftimg all have intent(out), these are just work
C     space arrays declared in calling routine and return values are
C     ignored.

      implicit none

      integer, intent(in)  :: ncol, nrow, nrot, nannuli, jcut
      integer, intent(in)  :: ann_lo, ann_hi, nview_mod, quick_omega
      real, intent(in)     :: xcen, ycen, xcen_model, ycen_model
      real, intent(in)     :: radius, del_ang, pixres_min, pixres_max
      real, intent(in)     :: pftprj(nrot/4,nannuli,nview_mod)
      real, intent(in)     :: mod_ts(nview_mod), mod_ps(nview_mod)
      real, intent(in)     :: prjs(ncol,nrow,nview_mod)
      real, intent(in)     :: r4_img(ncol,nrow)
      logical, intent(in)  :: var(nrot/4,nannuli)

      real, intent(out)    :: the_best, phi_best, ome_best
      real, intent(out)    :: cc_pft, cc_prj
      real, intent(out)    :: prj(ncol,nrow)
      real, intent(out)    :: r4_pimg(nrot,nannuli)
      complex, intent(out) :: pfft(nrot,nannuli)
      real, intent(out)    :: polar_prj(nrot,nannuli)
      real, intent(out)    :: pftimg((nrot/4),nannuli)

      integer i, j, index
      real    pi
      logical flip

      pi = acos(-1.0)

C     Calculate polar map of image
      call map_polar (r4_img, ncol, nrow, r4_pimg, nrot, nannuli, xcen,
     $     ycen, 1.0, radius, del_ang)

C     Get transform of polar map
      call pmap_fft (r4_pimg, pfft, nrot, nannuli)

C     Calculate amplitude of transformed polar map
      do i = 1, nannuli
          do j = 1,(nrot/4)
             pftimg(j,i) = abs(pfft(j,i))
        enddo
      enddo

C     Determine theta and absolute value of phi for image
      call get_thephi(pftimg, pftprj, nrot, nannuli,
     $     nview_mod, jcut, pixres_min, pixres_max, cc_pft,
     $     ann_lo,ann_hi,var, index)

      the_best = mod_ts(index)
      phi_best = mod_ps(index)

      prj(:,:) = prjs(:,:,index) !f90 array syntax

C     Calculate polar map of projection
      call map_polar(prj, ncol, nrow, polar_prj, nrot, nannuli,
     $     xcen_model, ycen_model, 1.0, radius, del_ang)

C     Determine omega and sign of phi for image
      if(quick_omega == 1) then
         call get_phiomega_quick(r4_pimg, polar_prj, nrot, nannuli,
     $        del_ang, ome_best, ann_lo, ann_hi, flip, cc_prj)
      else
         call get_phiomega(r4_pimg, polar_prj, nrot, nannuli,
     $        del_ang, ome_best, ann_lo, ann_hi, flip, cc_prj)
      endif

C     Correct orientation if hand flip detected in phi/omega determination
      if(flip) then
         the_best = 180.0 - the_best
         if(the_best == 0.0 .or. the_best == 180.0) then
            phi_best = 0.0
         else
            phi_best = phi_best + 180.0
         endif
         if(the_best .ne. 180.0) ome_best = ome_best + 180.0
         call map_xflip(prj, ncol, nrow)
      endif

      return
      end subroutine get_tpo_g
