      subroutine get_xy (fft1, fft2, xcen, ycen, idim1, idim2,
     1     res_max, fft_origx, fft_origy, flag,
     2     ipx, ipy, peakx1, peakx2, peaky1, peaky2, step1, step2)

c     routine for estimating the origins of icosahedral particles by
c     cross-correlation with circularly-averaged reference.
c     
c     note: "res_min" here = 1.0 (no choice allowed)
c     
c     may 1992  purdue university vax 8550, t. baker
c     modified 22-aug-2006, r sinkovits cleanup and rid of em_wrt

      implicit none

      integer*4, intent(in)  :: idim1, idim2
      complex, intent(in)    :: fft1(idim1,idim2), fft2(idim1,idim2)
      real*4, intent(in)     :: res_max, fft_origx, fft_origy

      integer*4, intent(out) :: flag
      integer*4, intent(out) :: ipx, ipy
      real*4, intent(out)    :: xcen, ycen
      real*4, intent(out)    :: peakx1, peakx2, peaky1, peaky2
      real*4, intent(out)    :: step1, step2

      real*4 res_min
      integer*4 alloc_stat
      real*4, allocatable :: cc_map(:,:)

      allocate(cc_map(idim1, idim2), stat = alloc_stat)
      if(alloc_stat .gt. 0) then
         write(*,*)'not enough memory for cc_map in get_xy()'
         stop
      endif

      res_min = 1.0
      call ccf_fft (fft1, fft2, cc_map, idim1, idim2, res_min, res_max)

      call rearrange_fft_style(idim1, idim2, cc_map)

      call pftcc_peak (cc_map, idim1, idim2, xcen, ycen, fft_origx,
     $     fft_origy, flag, ipx, ipy, peakx1, peakx2, peaky1, peaky2,
     $     step1, step2)
      
      deallocate(cc_map)
     
      return
      end subroutine get_xy
