      subroutine key_info (ncol, nrow, nsec, ncolbin, nrowbin, nsecbin,
     $     nrot, nannuli, npix, compare_mode, filt_fac,
     $     map_filename, inunit)

C     Routine for reading and setting program input parameters. Keyword
C     data and list of particle parameter files first read using
C     routines ppft_info() and pfile_info(). key_info then sets
C     defaults, calculated derived quantities, and tests input to make
C     sure that valid values were used.
      
      implicit none

      include '../include/sizes.inc'
      include '../include/info.inc'

      integer, intent(in)  :: inunit
      integer, intent(out) :: ncol, nrow, nsec
      integer, intent(out) :: ncolbin, nrowbin, nsecbin
      integer, intent(out) :: nrot, nannuli, npix, compare_mode
      real, intent(out)    :: filt_fac(10)
      character(len=1)     :: rw(2)
      character(len=filename_len), intent(out) :: map_filename

      include '../include/pifhead.inc'
      type (pifhead_t) prjhead, pfthead

      include '../include/infohead.inc'
      type(infohead_t) infohead
      
      common/pft_com/ mode, bin, verbose, mag_cen, mag_step, 
     $     mag_num, mag_norm, xcen_model, ycen_model, sym, dthe, 
     $     dphi_90, pftrad_lo, pftrad_hi, jcut, pixres_min, pixres_max,
     $     ann_lo, ann_hi, varopt, sigcut, idim1,
     $     idim2, radius, del_ang, nview_mod,
     $     quick_omega

      integer bin, quick_omega, nview_mod, ierr, ios
      integer i, n_power2, mode, verbose, mag_num, mag_norm
      integer idim1, idim2, fft_nfac, jcut, ann_lo, ann_hi, sym
      integer nrotd, nannulid, nview_modd

      real    del_ang, radius , pftrad_lo, pftrad_hi, pftrad_step
      real    pi, del_pixres
      real    pixres_min, pixres_max, res_tst, sigcut, mag_lo, mag_hi
      real    dang, dthe, dphi_90, test, mag_cen, mag_step
      real    res_lo, res_hi, xcen_model, ycen_model

      character(len=filename_len) :: pft_filename, prj_filename
      character(len=512) :: pftrads_fname, pftres1_fname, pftres2_fname

      logical   varopt
            
C     *** Read keyword data and use structure elements to set variables
      
      call ppft_info(infohead, inunit)
      mode          = infohead%input_mode
      bin           = infohead%bin_factor
      sym           = infohead%symmetry
      dang          = infohead%delta_theta
      compare_mode  = infohead%image_projection_compare_method
      verbose       = infohead%verbose
      per_ptle_ctf  = infoHead%per_ptle_ctf
      quick_omega   = infohead%quick_omega
      filt_fac(1)   = infohead%filter_factor_1
      pftrad_lo     = infohead%pftrad_lo
      pftrad_hi     = infohead%pftrad_hi
      pftrad_step   = infohead%pftrad_step
      ann_lo        = int(infohead%annulus_low)
      ann_hi        = int(infohead%annulus_high)
      res_lo        = infohead%resolution_low
      res_hi        = infohead%resolution_high
      jcut          = infohead%jcut
      sigcut        = infohead%sigcut
      mag_cen       = infohead%mag_cen
      mag_step      = infohead%mag_step
      mag_num       = infohead%mag_num
      mag_norm      = infohead%mag_norm      
      map_filename  = infohead%model_filename
      prj_filename  = infohead%prj_filename
      pft_filename  = infohead%pft_filename
      pftrads_fname = infohead%pftrads_filename
      pftres1_fname = infohead%pftres1_filename
      pftres2_fname = infohead%pftres2_filename

C     Make sure that input map exists and can be opened
      open(unit=18, file=map_filename, status='old',
     $     form='unformatted', iostat=ios)
      close(18)
      if(ios .ne. 0) then
         write(*,*) 
         write(*,*) 'input map not found: ', trim(map_filename)
         call error_stop('PPFT', 'key_info',
     $        'Unable to open map')
         write(*,*) 
      endif

C     *** Override invalid values for parameter

C     Valid values for mode are (-2, -1, 1, 2, 3)
      if(mode < -2 .or. mode > 3 .or. mode == 0) mode = 1

C     Symmetry group must be greater than zero
      if(sym <= 0) sym = 532

C     Bin factor must be 1 or 2
      if(bin.ne.2) bin = 1 

C     Compare mode must be in range 1 to 3
      if(compare_mode < 1 .or. compare_mode > 3) compare_mode = 0

C     Delta angle must be positive and greater than 0.1 degrees. After
C     correcting for invalid values, use dang to set step sizes in theta
C     and phi directions
      if(dang <= 0.0) dang = 1.0
      if(dang <  0.1) dang = 0.5
      dthe    = dang
      dphi_90 = dang

C     Set defaults for file names
      if(prj_filename  == ' ')  prj_filename = 'pft.prjs'
      if(pft_filename  == ' ')  pft_filename = 'pft.pfts'
      if(pftrads_fname == ' ')  pftrads_fname = 'empft.rads'
      if(pftres1_fname == ' ')  pftres1_fname = 'empft.res1'
      if(pftres2_fname == ' ')  pftres2_fname = 'empft.res2'

C     verbose must be in range -1 to 2
      if(verbose < -1 .or.  verbose > 2) verbose = 0

C     pftrad_lo must be at least 1.0
      if(pftrad_lo < 1.0)  pftrad_lo = 1.0

C     pftrad_step must be positive
      if(pftrad_step <= 0.0) pftrad_step = 1.0

C     Set jcut and varopt parameters for experimental, untested feature
      if(jcut == 0) jcut = 1
      if(jcut <  0) jcut = 0
      if(sigcut > 0.0) then
         varopt = .true.
      else
         varopt = .false.
      endif

      
c     *************************************************************************
c     *  input line #1: mode, bin, sym, dang, compare_mode, verbose,          *
c     *  filt_fac (3i,f,2i,f)                                                 *
c     *                                                                       *
c     *  mode  =  1 global orientation search mode (no input origins)         *
c     *        = -1 global mode (use stored pfts/prjs; no input origins)      *
c     *        =  2 global mode (use input origins)                           *
c     *        = -2 global mode (use stored pfts/prjs; use input origins)     *
c     *  bin   = 1 binfactor of 1 (no data compression)                       *
c     *        = 2 binfactor of 2 (compresses 3d data by 8x; 2d data by 4x)   *
c     *  verbose                                                              *
c     *     = -1 skip calculations in global_cc. used only for random         *
c     *          model calculations where cmp_cc is not needed                *
c     *     = 0 minimal output (refined orientations and origins)             *
c     *     = 1 correlation coefficients written out as functions of radius   *
c     *          and resolution in output files empft.* (* = rads,res1,res2)  *
c     *     = 2 gives verbose=1 output and correlation coefficients for each  *
c     *             possible view                                             *
c     *  compare_mode = 0  no ctf correction                                  *
c     *        = 1  multiply model projections by ctf                         *
c     *        = 2  multiply raw images by ctf (steve fuller method)          *
c     *        = 3  multiply raw images by inverse ctf (xing zhang method)    *
c     *************************************************************************
      if(filt_fac(1).le.0.0.or.filt_fac(1).gt.1.0) filt_fac(1) = 0.1
      if(compare_mode.eq.3) then
         print *, 'filt_fac factor set at - ', filt_fac(1)
      endif

C     Determine the number of model projections
      call get_nview_mod (sym, dthe, dphi_90, nview_mod)
      
C     If mode is positive, get (ncol, nrow, nsec) from map
C     header. Otherwise, retrieve problem size from file containing
C     precalculated projections. In the later case, note that nsec has
C     to be decremented by one since the last slice is the circular
C     average
      
      if(mode > 0) then

         call get_map_dimensions(map_filename, ncol, nrow, nsec, 
     $        angstrom_per_pix, ierr)
         if(ierr > 0) then
            call error_stop('PPFT', 'key_info',
     +           'Error in 3D map mode, it should be 20 or 40')
         endif

      else

         prjhead%lunum = 3
         rw(1) = 'r'
         rw(2) = achar(0)
         call pif_open(prj_filename, prjhead%lunum, rw)
         call pif_read_gh(prjhead)
         if(prjhead%mode .ne. 21) then
            print *, 'invalid pif mode for prj file'
            call tsbend
         endif
         prjhead%imgnum = 1
         call pif_read_dh (prjhead)
         if((prjhead%rad_lo .ne. pftrad_lo) .or.
     $        (prjhead%rad_hi .ne. pftrad_hi)) then
            print *, '*********************************************'
            print *, '* mismatch between entered pftrad_lo/hi and *'
            print *, '* that contained in - ',prj_filename
            print *, '* entered--> ', pftrad_lo,pftrad_hi
            print *, '* found----> ', prjhead%rad_lo,prjhead%rad_hi
            print *, '*********************************************'
            call tsbend
         endif
         ncol = prjhead%numcols
         nrow = prjhead%numrows
         nsec = prjhead%numsecs - 1 ! last section is circular average
         pfthead%lunum = 4
         rw(1) = 'r'
         rw(2) = achar(0)
         call pif_open (pft_filename, pfthead%lunum, rw)
         call pif_read_gh (pfthead)
         if(pfthead%mode.ne.22) then
            print *, 'invalid pif mode for pft file'
            call tsbend
         endif
         pfthead%imgnum = 1
         call pif_read_dh (pfthead)
         if((pfthead%rad_lo .ne. pftrad_lo) .or.
     $        (pfthead%rad_hi .ne. pftrad_hi)) then
            print *, '*********************************************'
            print *, '* mismatch between entered pftrad_lo/hi and *'
            print *, '* that contained in - ',pft_filename
            print *, '* entered--> ', pftrad_lo,pftrad_hi
            print *, '* found----> ', pfthead%rad_lo,pfthead%rad_hi
            print *, '*********************************************'
            call tsbend
         endif
         nrotd      = pfthead%numcols
         nannulid   = pfthead%numrows
         nview_modd = pfthead%numsecs

      endif

C     Get information from particle parameter files
      call pfile_info(inunit)

C     Make sure that input map and images have the same linear size
      if (ncol .ne. npix_col) then
          print *, 'ERROR. map size:', ncol, 'images size:', npix_col
          call error_stop('PPFT', 'key_info',
     +     'Reference map and input projections are not the same size!')
      endif
         
C     Make sure that pftrad_hi is both positive and no larger than the
C     smallest dimension of the map

      radius = (real(min(ncol-1,nrow-1))/2.0)
      if(pftrad_hi > radius)  pftrad_hi = radius
      if(pftrad_hi < 0.0)     pftrad_hi = radius

C     Determine the number of annuli
      nannuli = int((pftrad_hi - pftrad_lo + 1.0)/pftrad_step)

C     Calculate NROT, the number of rotational orientations used when
C     determining omega. NROT will be the smallest power of two greater
C     than or equal to test

      pi = acos(-1.0)
      test = 2.0*pftrad_hi*pi
      n_power2 = ceiling(log(test)/log(2.0))
      nrot = 2**n_power2
      del_ang  = 360.0/real(nrot)


C     Double size of FFT (not sure why this is being done here)
      fft_nfac = 2  ! doubles default fft size for cc function
      call fft_setdim_def_same (ncol, nrow, idim1, idim2, fft_nfac)
      
C     Comparing values of nrot, nannuli, and nview_mod calculated in
C     this routine to the values retrieved from the prjs/pfts file to
C     make sure that they are consistent.

      if(mode <= 0) then
         if(nrotd*4 .ne. nrot .or. nannulid .ne. nannuli 
     $        .or. nview_modd .ne. nview_mod) then
            print *, 'nrot,nannuli,nview_mod - ', 
     $           nrot,nannuli,nview_mod
            print *, 'nrotd,nannulid,nview_modd - ', nrotd,nannulid
            print *,  nview_modd
            print *, 'mismatch between pft file and batch job params'
            call tsbend
         endif
      endif
      
      pftrad_step = 1.0
      del_pixres  = 0.025
      if(jcut >= nrot/4)   jcut  = (nrot/4) - 1
      if(pixsiz_out <= 0.0) pixsiz_out = 1.0 ! pixel units assumed
      res_tst = real(idim1/2)*pixsiz_out
      if(res_lo <= 0.0 .or. res_lo > res_tst) res_lo = res_tst
      res_tst = 2.0*pixsiz_out
      if(res_hi < res_tst) res_hi = res_tst
      pixres_min = res_lo/pixsiz_out
      pixres_max = res_hi/pixsiz_out
      res_min    = real(idim1)/pixres_min
      res_max    = real(idim1)/pixres_max
      
C     Open additional output files and write header information of
C     verbose level is positive

      if(verbose > 0) then
         open (unit=8, file=pftres1_fname)
         write(8,'(2a,/,a)') 'correlation coefficients as function ',
     $        ' of resolution (1/pixel) for unmasked data',
     $        ' (computed after x,y origin refinement)'
         write(8,'(a,10f7.3)')  ' img', (i*del_pixres, i=1,10)
         write(8,1000)
         open (unit=9, file=pftrads_fname)
         write(9,'(2a,/,a)') 
     $        ' correlation coefficients as a function',
     $        ' of radius', ' (computed after x,y origin refinement)'
         open (unit=10, file=pftres2_fname)
         write(10,'(2a,/,a)') 
     $        ' correlation coefficients as function',
     $        ' of resolution (1/pixel) for ann_lo to ann_hi masked',
     $        ' data (computed after x,y origin refinement)'
         write(10,'(a,10f7.3)')  ' img', (i*del_pixres, i=1,10)
         write(10,1000)
      endif
      
      if(mag_num <= 0) mag_num = 1
      if(mod(mag_num,2) == 0) mag_num = mag_num + 1
      if(iabs(mode) == 1 .and. mag_cen <= 0.0) then
         mag_cen = 1.0
         print *,'global search mode - you have no ',
     $        'mag values to read in'
      endif

C     Adjust parameters to account for binning
      ncolbin = (ncol/bin) + 1
      nrowbin = (nrow/bin) + 1
      nsecbin = (nsec/bin) + 1
      if(mod(ncolbin,2) == 0) ncolbin = ncolbin - 1
      if(mod(nrowbin,2) == 0) nrowbin = nrowbin - 1
      if(mod(nsecbin,2) == 0) nsecbin = nsecbin - 1
      nrot       = nrot/bin
      idim1      = idim1/bin
      idim2      = idim2/bin
      nannuli    = nannuli/bin
      npix       = nrot*nannuli/4
      del_ang    = del_ang*real(bin)
      ann_lo     = ann_lo/bin
      ann_hi     = ann_hi/bin
      radius     = real(nannuli)
      pftrad_lo  = pftrad_lo/real(bin)
      if(pftrad_lo < 1.0) pftrad_lo = 1.0
      pftrad_hi  = pftrad_hi/real(bin)
      xcen_model = (ncolbin+1)/2
      ycen_model = (nrowbin+1)/2
      if(ann_lo <  1)       ann_lo = 0
      if(ann_lo >= nannuli) ann_lo = nannuli - 1
      if(ann_hi <= 0)       ann_hi = nannuli - 1
      if(ann_hi >= nannuli) ann_hi = nannuli - 1


      write(*,1001) ' PFT parameters'
      write(*,1001) ' ---------------------------------------------'
      write(*,1001) ' bin factor          ', bin
      write(*,1001) ' symmetry            ', sym
      if(mode == 3) then
         write(*,1001) ' # model views/window', nview_mod
      else
         write(*,1001) ' # model views/asu   ', nview_mod
      endif
      write(*,1001) ' fft_dim1,fft_dim2   ', idim1, idim2
      write(*,1002) ' res_lo,res_hi       ', res_lo, res_hi
      write(*,1002) ' res_min,res_max     ', res_min, res_max
      write(*,1002) ' pixres_min/max      ', pixres_min, pixres_max
      write(*,1002) ' pixel size          ', pixsiz_out
      write(*,1001) ' nannuli             ', nannuli
      write(*,1002) ' pftrad_lo,pftrad_hi ', pftrad_lo, pftrad_hi
      write(*,1001) ' ann_lo,ann_hi       ', ann_lo, ann_hi
      write(*,1001) ' nrot                ', nrot
      write(*,1002) ' delta omega         ', del_ang
      write(*,1002) ' model xcen,ycen     ', xcen_model, ycen_model
      write(*,1002) ' mag(_cen,_step,_num)', mag_cen,mag_step,mag_num
      if(mag_cen > 0.0) then
         mag_lo = mag_cen - (((mag_num-1)/2)*mag_step)
         mag_hi = mag_cen + (((mag_num-1)/2)*mag_step)
         write(*,1002) ' mag range(lo,hi)    ', mag_lo, mag_hi
      else
         write(*,'(2a)') ' mag factor centered on input values from',
     $        ' data file'
      endif
      write(*,1001) ' ----------------------------------------------'
      
 1000 format(74('-'))
 1001 format(a,2i9)
 1002 format(a,2f9.3,i9)

      return
      end subroutine key_info
