      SUBROUTINE MAP_PRJ_SLICE(MAP2D, MAP_SLICE, NCOL, NROW, NSEC,
     $     THETA, PHI, OMEGA, IZ)
      
C     Generate 2D projection of the real space map corresponding to
C     orientation specified by (theta, phi, omega).
C     
C     JANUARY 1986, PURDUE UNIVERSITY, VAX 11/750, L. A. MATTERN.
C     MODIFIED 11-DEC-1997 BY T. BAKER.
C     Modified 18-aug-2006, R Sinkovits. Eliminate intermediate array
C       PROJN and add contributions to projection directly to MAP2D

      IMPLICIT NONE

      INTEGER*4, INTENT(IN) :: NCOL, NROW, NSEC, IZ
      REAL*4, INTENT(IN)    :: THETA, PHI, OMEGA
      REAL*4, INTENT(IN)    :: MAP_SLICE(NCOL,NROW)
      REAL*4, INTENT(INOUT) :: MAP2D(NCOL,NROW)

      INTEGER*4 IX, IY, JX, JY
      REAL*4    COSTH, SINTH, COSPH, SINPH, COSOM, SINOM
      REAL*4    RSQ, YSQ, ZSQ, RMAXSQ
      REAL*4    XBIT, YBIT, X1, Y1, X, Y, Z
      REAL*4    T1, T2, T3, T4, T5, T6, T7, T8, T9
      REAL*4    XCEN, YCEN, ZCEN
      REAL*8    DEG_TO_RAD
      
      DEG_TO_RAD = ACOS(-1.0d0)/180.0d0

      XCEN = REAL((NCOL+1)/2)
      YCEN = REAL((NROW+1)/2)
      ZCEN = REAL((NSEC+1)/2)
      RMAXSQ = (MIN(XCEN-1, YCEN-1, ZCEN-1))**2

C     COMPUTE ELEMENTS OF ROTATION MATRIX

      COSTH = REAL(COS(THETA*deg_to_rad))
      SINTH = REAL(SIN(THETA*deg_to_rad))
      COSPH = REAL(COS(PHI*deg_to_rad))
      SINPH = REAL(SIN(PHI*deg_to_rad))
      COSOM = REAL(COS(OMEGA*deg_to_rad))
      SINOM = REAL(SIN(OMEGA*deg_to_rad))
      
      T1 = COSTH*COSPH*COSOM + SINPH*SINOM
      T2 = COSTH*SINPH*COSOM - COSPH*SINOM
      T3 = -SINTH*COSOM
      T4 = COSTH*COSPH*SINOM - SINPH*COSOM
      T5 = COSTH*SINPH*SINOM + COSPH*COSOM
      T6 = -SINTH*SINOM
      T7 = SINTH*COSPH
      T8 = SINTH*SINPH
      T9 = COSTH

C     Calculate the projection of the map section

      Z   = REAL(IZ) - ZCEN
      ZSQ = Z*Z
      IF(ZSQ > RMAXSQ) RETURN

      DO IY = 1,NROW
         Y   = REAL(IY) - YCEN
         YSQ = Y*Y
         RSQ = ZSQ + YSQ
         IF(RSQ > RMAXSQ) CYCLE
         DO IX = 1,NCOL
            X   = REAL(IX) - XCEN
            RSQ = ZSQ + YSQ + X*X
            IF(RSQ > RMAXSQ) CYCLE
            X1   = T1*X + T2*Y + T3*Z + XCEN
            Y1   = T4*X + T5*Y + T6*Z + YCEN
            JX   = INT(X1)
            JY   = INT(Y1)
            IF(  JX <= 0 .OR. JX >= NCOL .OR. 
     1           JY <= 0 .OR. JY >= NROW) CYCLE

            XBIT = X1 - JX
            YBIT = Y1 - JY
            
            MAP2D(JX,JY)     = (1.-XBIT)*(1.-YBIT)
     1           * MAP_SLICE(IX,IY) + MAP2D(JX,JY)
            MAP2D(JX+1,JY)   = XBIT*(1.-YBIT)
     1           * MAP_SLICE(IX,IY) + MAP2D(JX+1,JY)
            MAP2D(JX,JY+1)   = (1.-XBIT)*YBIT 
     1           * MAP_SLICE(IX,IY) + MAP2D(JX,JY+1)
            MAP2D(JX+1,JY+1) = XBIT*YBIT
     1           * MAP_SLICE(IX,IY) + MAP2D(JX+1,JY+1)
         ENDDO
      ENDDO
      
      RETURN
      END SUBROUTINE MAP_PRJ_SLICE
