      subroutine pftcc_peak(cc_map, idim1, idim2, xcen, ycen, fft_origx,
     $     fft_origy, flag, ipx_inbox, ipy_inbox, peakx1, peakx2, 
     $     peaky1, peaky2, step1, step2)

C     Determine the location of the peak in cross-correlation map
C     cc_map. The location of the maximum value in array cc_map (within
C     a smaller box centered on the array) is first identified. A more
C     through search over a smaller region of cc_map surrounding this
C     point is then performed.
      
      implicit none
      
      integer, parameter :: icor=15, jcor=15
      integer, parameter :: n_steps=2, n_incr=5
      integer, parameter :: kx=3, ky=3
      integer, parameter :: iopt=0
      
      integer, parameter :: icor2=(icor+1)/2, jcor2=(jcor+1)/2
      integer, parameter :: nxest=icor+kx+1, nyest=jcor+ky+1
      integer, parameter :: nxestp=nxest + n_steps*n_incr
      integer, parameter :: nyestp=nyest + n_steps*n_incr
      integer, parameter :: kwrk=3+icor+jcor+nxestp+nyestp
      integer, parameter :: lwrk=4+nxestp*(jcor+2*kx+5) + 
     $     nyestp*(2*ky+5) + icor*(kx+1)+jcor*(ky+1) + max(jcor,nxestp)

      real, intent(in)     :: cc_map(idim1, idim2)
      real, intent(in)     :: fft_origx, fft_origy
      integer, intent(in)  :: idim1, idim2
      integer, intent(out) :: flag, ipx_inbox, ipy_inbox
      real, intent(out)    :: xcen, ycen
      real, intent(out)    :: peakx1, peakx2, peaky1, peaky2
      real, intent(out)    :: step1, step2

      real surf_fit(icor*jcor), tx(nxestp), ty(nyestp)
      real c((nxestp-kx-1)*(nyestp-ky-1)), wrk(lwrk)
      integer iwrk(kwrk)

      integer nxest_arg, nyest_arg
      integer ix, iy, i, j, k
      
      integer ipx_outbox, ipy_outbox
      integer cenx, ceny, incx, incy
      real    amps(icor,jcor), peak_inbox, peak_outbox
      real    halfx, halfy, peakx, peaky
      
      integer nx, ny, ier, indx
      real    fit_max, xb, xe, yb, ye, s, fp
      real    step, rangex, rangey
      real    xcoord(icor), ycoord(jcor)

      real    grid_xcen, grid_ycen

C     Center of image in fourier domain
      halfx = real(idim1)/2.0
      halfy = real(idim2)/2.0

C     Determine center of box
      cenx = (idim1+1)/2
      ceny = (idim2+1)/2
      
C     Determine the location of the array element with the maximum value
C     in a box of size (0.3*idim1, 0.3*idim2) centered on cc_map
C     array. Initializations of ip[x|y]_[in|out]box and peak_[in|out]box
C     done to ensure that subroutine returns early if unable to find a
C     maximum anywhere

      incx  =  int(0.3 * ((idim1+1)/2))
      incy  =  int(0.3 * ((idim2+1)/2))
      ipx_inbox = 2
      ipy_inbox = 2
      peak_inbox = -huge(peak_inbox)/2.0
      do j = ceny-incy, ceny+incy
         do i = cenx-incx, cenx+incx
            if(cc_map(i,j) > peak_inbox) then
               ipx_inbox = i
               ipy_inbox = j
               peak_inbox = cc_map(i,j)
            endif
         enddo
      enddo

C     Determine the location of the array element with the maximum value
C     in a box of size (0.9*idim1, 0.9*idim2) centered on cc_map array

      incx  =  int(0.9 * ((idim1+1)/2))
      incy  =  int(0.9 * ((idim2+1)/2))
      ipx_outbox = 3
      ipy_outbox = 3
      peak_outbox = -huge(peak_outbox)/3.0
      do j = ceny-incy, ceny+incy
         do i = cenx-incx, cenx+incx
            if(cc_map(i,j) > peak_outbox) then
               ipx_outbox = i
               ipy_outbox = j
               peak_outbox = cc_map(i,j)
            endif
         enddo
      enddo


C     Test to see if the results from the searches over the smaller and
C     larger boxes are the same. If they differ, flag this particle as
C     having a center that is more than 30% from the center of the box.

      if(peak_inbox == peak_outbox) then
         flag = 0
!GCA store the peak, so that can be returned if surface fit fails
         grid_xcen = ipx_inbox
         grid_ycen = ipy_inbox
      else
         xcen = ipx_outbox
         ycen = ipy_outbox
         flag = -99
         return
      endif

C     Use the location of the peak in the smaller box centered on array
C     cc_map as starting point for a more throrough search for the peak.

      do j = 1,jcor
         iy = ipy_inbox + (j-jcor2)
         if(iy <= 0)    iy = iy + idim2
         if(iy > idim2)  iy = iy - idim2
         ycoord(j) = iy
         do i = 1,icor
            ix = ipx_inbox + (i-icor2)
            if(ix <= 0)    ix = ix + idim1
            if(ix > idim1) ix = ix - idim1
            if (j == 1) xcoord(i) = ix
            amps(j,i) = cc_map(ix,iy) !mbs 10/26/05
         enddo
      enddo
      
      xb = xcoord(1)
      xe = xcoord(icor)
      yb = ycoord(1)
      ye = ycoord(jcor)
      
      s = 0.00001
      
      step = 1.

      nxest_arg = nxest
      nyest_arg = nyest
      
      ier = 2
      do while(ier.ne.0) 
         
         call regrid(iopt, icor, xcoord, jcor, ycoord, amps, xb, xe,
     $        yb, ye, kx, ky, s, nxest_arg, nyest_arg, nx, tx, ny, ty,
     $        c, fp, wrk, lwrk, iwrk, kwrk, ier)
         
         if(ier == 10) then
            write(*,*)"check dimensions etc. in regrid call"
            stop
         end if
         if(ier == -2) then
            if(s > 1.e-6) then
               write(*,*)"smoothing factor too big, dividing by 10"
               s = s/10.
            else
               ier = 0
            end if
         else if(ier == -1) then
            ier = 0
         else if(ier == 1) then
            nxest_arg = nxest_arg + n_incr
            nyest_arg = nyest_arg + n_incr
            if(nxest_arg > nxestp .or. nyest_arg > nyestp) then
               write(*,*)"Cannot expand nxest or nyest"
               stop
            else
               write(*,*)"nxest or nyest too small, increasing by 5"
            endif

         else if(ier == 2 .or. ier == 3) then
            write(*,*)"smoothing factor too small, increasing 2x"
            s = s*2.
         end if
      end do
      
      do k = 1, 2
         call bispev(tx, nx, ty, ny, c, kx, ky, xcoord, icor, ycoord,
     $        jcor, surf_fit, wrk, lwrk, iwrk, kwrk, ier)
         if(ier == 10) then
            write(*,*)"check dimensions etc. in bispev call"
            stop
         end if
         
c     initialize parameters
         fit_max = -1.e38
         peakx         = -10000.
         peaky         = -10000.
         do j = 1, jcor
            do i = 1, icor
               if(surf_fit(j+(i-1)*icor) > fit_max) then
                  fit_max = surf_fit(j+(i-1)*icor)
                  peakx = xcoord(i)
                  peaky = ycoord(j)
               end if
            end do
         end do

         if(k == 1) then
            peakx1 = peakx
            peaky1 = peaky
            step1 = step
         else
            peakx2 = peakx
            peaky2 = peaky
            step2 = step
         endif

         if(peakx .ne. -10000. .and. peaky .ne. -10000.) then
            step = step/10.
            rangex = step*(icor2-1)
            rangey = step*(jcor2-1)
            i = 1
            
            xcen = peakx - rangex
            do indx = 1, 2*icor2 - 1
               xcoord(i) = xcen
               xcen = xcen + step
               i = i + 1
            enddo
            i = 1
            ycen = peaky - rangey
            do indx = 1, 2*jcor2 - 1
               ycoord(i) = ycen
               ycen = ycen + step
               i = i + 1
            enddo
         else
            write(*,*) 'WARNING [pftcc_peak]: surface fit failed,
     &         max not found'
            xcen = fft_origx - (grid_xcen - halfx - 1)
            ycen = fft_origy - (grid_ycen - halfy - 1)
            return
         end if
      end do
      
      xcen = fft_origx - (peakx - halfx - 1) !mbs 10/25/05
      ycen = fft_origy - (peaky - halfy - 1) !mbs 10/25/05
      
      
      return
      end subroutine pftcc_peak
