      SUBROUTINE PFTCC_RES (FFT1, FFT3, FFT4, FFT5, IDIM1, IDIM2, CC,
     $     DEL_PIXRES)

C     Calculate the radially averaged correlation coefficients

      IMPLICIT NONE

      INTEGER*4, INTENT(IN) :: IDIM1, IDIM2
      complex, INTENT(IN)   :: FFT1(IDIM1,IDIM2), FFT3(IDIM1,IDIM2)
      complex, INTENT(IN)   :: FFT4(IDIM1,IDIM2), FFT5(IDIM1,IDIM2)
      REAL*4, INTENT(IN)    :: DEL_PIXRES
      REAL*4, INTENT(OUT)   :: CC(2,20)

      INTEGER*4 IX, IY, BIN, IXTEST
      INTEGER*4 I, IX2, IY2
      REAL*4    RES, YSQ
      REAL*4    SUM13(20), SUM45(20), PTS(20)
      REAL*4    AVG1(20), AVG3(20), AVG4(20), AVG5(20)
      REAL*4    SQ1(20), SQ3(20), SQ4(20), SQ5(20)
      REAL*4    DIV1, DIV3, DIV4, DIV5, AMP1, AMP3, AMP4, AMP5

      IXTEST  = IDIM1/2

C     Initialize arrays using f90 array syntax
      CC    = 0.0
      PTS   = 0.0
      SUM13 = 0.0
      SUM45 = 0.0
      AVG1  = 0.0
      AVG3  = 0.0
      AVG4  = 0.0
      AVG5  = 0.0
      SQ1   = 0.0
      SQ3   = 0.0
      SQ4   = 0.0
      SQ5   = 0.0
      
      IY2 = (IDIM2/2) + 1
      DO IY = 1,IY2
         YSQ = FLOAT(IY-1)**2
         IX2 = IDIM1
         IF(IY == 1 .OR. IY == IY2) THEN
            IX2 = IDIM1/2
         ENDIF
         DO IX = 1,IX2
            IF(IY == 1 .AND. IX == 1) CYCLE ! Skip IX=IY=1 case
            IF(IX > IXTEST) THEN
               RES = SQRT ( YSQ + FLOAT(IDIM1-IX+1)**2 ) 
            ELSE
               RES = SQRT ( YSQ + FLOAT(IX-1)**2 )
            ENDIF
            BIN        = INT( RES/(IDIM1*DEL_PIXRES) ) + 1
            
            IF(BIN <= 20) THEN
               AMP1 = ABS(FFT1(IX,IY))
               AMP3 = ABS(FFT3(IX,IY))
               AMP4 = ABS(FFT4(IX,IY))
               AMP5 = ABS(FFT5(IX,IY))
               PTS(BIN)   = PTS(BIN) + 1.0
               SUM13(BIN) = SUM13(BIN) + AMP1*AMP3
               SUM45(BIN) = SUM45(BIN) + AMP4*AMP5
               AVG1(BIN)  = AVG1(BIN) + AMP1
               AVG3(BIN)  = AVG3(BIN) + AMP3
               AVG4(BIN)  = AVG4(BIN) + AMP4
               AVG5(BIN)  = AVG5(BIN) + AMP5
               SQ1(BIN)   = SQ1(BIN) + AMP1**2
               SQ3(BIN)   = SQ3(BIN) + AMP3**2
               SQ4(BIN)   = SQ4(BIN) + AMP4**2
               SQ5(BIN)   = SQ5(BIN) + AMP5**2
            ENDIF

         ENDDO
      ENDDO
      
      DO I = 1,20
         IF(PTS(I) <= 0.0) THEN
            AVG1(I) = 0.0
            AVG3(I) = 0.0
            AVG4(I) = 0.0
            AVG5(I) = 0.0
         ELSE
            AVG1(I) = AVG1(I)/PTS(I)
            AVG3(I) = AVG3(I)/PTS(I)
            AVG4(I) = AVG4(I)/PTS(I)
            AVG5(I) = AVG5(I)/PTS(I)
         ENDIF
         DIV1 = SQ1(I) - PTS(I)*(AVG1(I)**2)
         DIV3 = SQ3(I) - PTS(I)*(AVG3(I)**2)
         DIV4 = SQ4(I) - PTS(I)*(AVG4(I)**2)
         DIV5 = SQ5(I) - PTS(I)*(AVG5(I)**2)
         DIV1 = sign(SQRT(abs(DIV1)), DIV1)
         DIV3 = sign(SQRT(abs(DIV3)), DIV3)
         DIV4 = sign(SQRT(abs(DIV4)), DIV4)
         DIV5 = sign(SQRT(abs(DIV5)), DIV5)
         IF(DIV1*DIV3 <= 0.0) THEN
            CC(1,I) = 0.0
         ELSE
            CC(1,I) = (SUM13(I)-AVG1(I)*AVG3(I)*PTS(I))/(DIV1*DIV3)
         ENDIF
         IF(DIV4*DIV5 <= 0.0) THEN
            CC(2,I) = 0.0
         ELSE
            CC(2,I) = (SUM45(I)-AVG4(I)*AVG5(I)*PTS(I))/(DIV4*DIV5)
         ENDIF
      ENDDO

      RETURN
      END SUBROUTINE PFTCC_RES
