      subroutine ppft_info(infoHead, inunit)
      implicit none

      include '../include/infohead.inc'
      type(infohead_t), intent(out) :: infoHead
      integer, intent(in) :: inunit

      character(len=26)  :: upper_case, lower_case
      character(len=257) :: lineorig
      character(len=257) :: keyword
      character(len=1)   :: tab, spc
      
      integer char, ios, ic, i, j, loc
      
      data upper_case /'ABCDEFGHIJKLMNOPQRSTUVWXYZ'/
      data lower_case /'abcdefghijklmnopqrstuvwxyz'/

C     Set default values
      infoHead%per_ptle_ctf = 0
      infoHead%quick_omega = 1
      infoHead%threedr_mode     = 0
      infoHead%input_mode   = 1
      infoHead%input_mode_cc   = 1
      infoHead%phase_weight   = 0.0
      infoHead%bin_factor   = 1
      infoHead%symmetry     = 532
      infoHead%delta_theta  = 1.0
      infoHead%image_projection_compare_method = 3
      infoHead%ctf_mode     = 1
      infoHead%ctf_filter_mode = 0
      infoHead%verbose      = -1
      infoHead%bg_out       = 0
      infoHead%lg_out       = 0
      infoHead%filter_factor_1 = 0.0
      infoHead%filter_factor_2 = 0.0
      infoHead%pftrad_lo    = 0.0
      infoHead%pftrad_hi    = 0.0
      infoHead%pftrad_step  = 0.0
      infoHead%annulus_low  = 0.0
      infoHead%annulus_high = 0.0
      infoHead%radius       = 0.0
      infoHead%temperature_factor  = 0.0
      infoHead%resolution_out  = 50.0
      infoHead%resolution_low  = 300.0
      infoHead%resolution_high = 50.0
      infoHead%units_out    = 1
      infoHead%mp_out       = 0
      infoHead%jcut         = 0
      infoHead%sigcut       = 0.0
      infoHead%delta_xy     = 0.5
      infoHead%num_delta_xy = 7.0
      infoHead%delta_omega  = 0.5
      infoHead%num_delta_omega  = 7.0
      infoHead%map_pixsiz = 0.0
      infoHead%mag_cen  = 0.0
      infoHead%mag_step = 0.0
      infoHead%mag_num  = 0
      infoHead%mag_norm = 0
      infoHead%map_dim  = 0
      infoHead%max_origin_shift = 0.3
      infoHead%ico_spread= 1.0
      infoHead%minimum   = 0.0

      infoHead%model_filename = ''
      infoHead%prj_filename   = ''
      infoHead%pft_filename   = ''
      infoHead%pftrads_filename = 'pftrads.dat'
      infoHead%pftres1_filename = 'pftres1.dat'
      infoHead%pftres2_filename = 'pftres2.dat'
      infoHead%bg_filename = 'bg.dat'
      infoHead%title = 'pftsearch_title'

C     Define tab and spc using ascii code - tab=9, spc=32
      tab = achar(9)
      spc = achar(32)

      do i = 1, 10000
         
C     Read line as character string. Exit loop if EOF reached
         read(inunit, '(A)', iostat=ios) lineorig
         if(ios < 0) exit ! Break out of loop

C     Convert all tabs to spaces
         do ic=1,len(lineorig)
            if(lineorig(ic:ic) == tab) then
               lineorig(ic:ic) = ' '
            endif
         enddo
         
C     Convert all '=' to spaces. This makes it possible to parse lines
C     of both the form "key = value" and "key value"
         do ic=1,len(lineorig)
            if(lineorig(ic:ic) == '=') then
               lineorig(ic:ic) = ' '
            endif
         enddo

C     Get rid of leading whitespace
         lineorig = adjustl(lineorig)

C     Go to next loop iteration if line is blank or comment
         if(lineorig(1:1) == ' ') cycle
         if(lineorig(1:1) == '#') cycle
         
C     Get rid of trailing comments
         loc = scan(lineorig,'#')
         if(loc > 0) then
            do j=loc,257
               lineorig(j:j) = ' '
            enddo
         endif

C     Set keyword
         keyword = lineorig
         
C     Use f90 INDEX function to change keyword to upper case
         do ic = 1, 256
            char = index (lower_case, keyword(ic:ic))
            if (char .ne. 0) then
               keyword(ic:ic) = upper_case(char:char)
            endif
         enddo

C     Get rid of leading keyword from line
         do ic=1,len_trim(lineorig)
            if(lineorig(ic:ic) == ' ') then
               exit
            else
               lineorig(ic:ic) = ' '
            endif
         enddo
         
C     Process line based on value of keyword. Note that we are using a
C     slightly uncommon Fortran construct where a string is used in
C     place of the unit number in the read statement
         
         if(keyword(1:5) == 'THREE') then
            read(lineorig,*) infoHead%threedr_mode
         else if(keyword(1:12) == 'PER_PTLE_CTF') then
            read(lineorig,*) infoHead%per_ptle_ctf
         else if(keyword(1:7) == 'QUICK_O') then
            read(lineorig,*) infoHead%quick_omega
         else if(keyword(1:9) == 'ANNULUS_H') then
            read(lineorig,*) infoHead%annulus_high
         else if(keyword(1:9) == 'ANNULUS_L') then
            read(lineorig,*) infoHead%annulus_low
         else if(keyword(1:4) == 'BG_F') then
            infoHead%bg_filename = adjustl(lineorig)
         else if(keyword(1:4) == 'BG_O') then
            read(lineorig,*) infoHead%bg_out
         else if(keyword(1:3) == 'BIN') then
            read(lineorig,*) infoHead%bin_factor
         else if(keyword(1:5) == 'CTF_F') then
            read(lineorig,*) infoHead%ctf_filter_mode
         else if(keyword(1:5) == 'CTF_M') then
            read(lineorig,*) infoHead%ctf_mode
         else if(keyword(1:7) == 'DELTA_O') then
            read(lineorig,*) infoHead%delta_omega
         else if(keyword(1:7) == 'DELTA_T') then
            read(lineorig,*) infoHead%delta_theta
         else if(keyword(1:7) == 'DELTA_X') then
            read(lineorig,*) infoHead%delta_xy
         else if(keyword(1:15) == 'FILTER_FACTOR_1') then
            read(lineorig,*) infoHead%filter_factor_1
         else if(keyword(1:15) == 'FILTER_FACTOR_2') then
            read(lineorig,*) infoHead%filter_factor_2
         else if(keyword(1:3) == 'ICO') then
            read(lineorig,*) infoHead%ico_spread
         else if(keyword(1:3) == 'IMA') then
            read(lineorig,*) infoHead%image_projection_compare_method
         else if(keyword(1:10) == 'INPUT_MODE') then
            read(lineorig,*) infoHead%input_mode
         else if(keyword(1:13) == 'INPUT_MODE_CC') then
            read(lineorig,*) infoHead%input_mode_cc
         else if(keyword(1:3) == 'JCU') then
            read(lineorig,*) infoHead%jcut
         else if(keyword(1:3) == 'LG_') then
            read(lineorig,*) infoHead%lg_out
         else if(keyword(1:5) == 'MAG_C') then
            read(lineorig,*) infoHead%mag_cen
         else if(keyword(1:5) == 'MAP_D') then
            read(lineorig,*) infoHead%map_dim
         else if(keyword(1:6) == 'MAG_NU') then
            read(lineorig,*) infoHead%mag_num
         else if(keyword(1:6) == 'MAG_NO') then
            read(lineorig,*) infoHead%mag_norm
         else if(keyword(1:5) == 'MAG_S') then
            read(lineorig,*) infoHead%mag_step
         else if(keyword(1:5) == 'MAP_P') then
            read(lineorig,*) infoHead%map_pixsiz
         else if(keyword(1:3) == 'MAX') then
            read(lineorig,*) infoHead%max_origin_shift
         else if(keyword(1:3) == 'MIN') then
            read(lineorig,*) infoHead%minimum
         else if(keyword(1:3) == 'MOD') then
            infoHead%model_filename = adjustl(lineorig)
         else if(keyword(1:3) == 'MP_') then
            read(lineorig,*) infoHead%mp_out
         else if(keyword(1:11) == 'NUM_DELTA_O') then
            read(lineorig,*) infoHead%num_delta_omega
         else if(keyword(1:11) == 'NUM_DELTA_X') then
            read(lineorig,*) infoHead%num_delta_xy
         else if(keyword(1:5) == 'PFT_F') then
            infoHead%pft_filename = adjustl(lineorig)
         else if(keyword(1:8) == 'PFTRAD_L') then
            read(lineorig,*) infoHead%pftrad_lo
         else if(keyword(1:8) == 'PFTRAD_H') then
            read(lineorig,*) infoHead%pftrad_hi
         else if(keyword(1:8) == 'PFTRAD_S') then
            read(lineorig,*) infoHead%pftrad_step
         else if(keyword(1:7) == 'PFTRADS') then
            infoHead%pftrads_filename = adjustl(lineorig)
         else if(keyword(1:7) == 'PFTRES1') then
            infoHead%pftres1_filename = adjustl(lineorig)
         else if(keyword(1:7) == 'PFTRES2') then
            infoHead%pftres2_filename = adjustl(lineorig)
         else if(keyword(1:2) == 'PH') then
            read(lineorig,*) infoHead%phase_weight
         else if(keyword(1:5) == 'PRJ_F') then
            infoHead%prj_filename = adjustl(lineorig)
         else if(keyword(1:3) == 'RAD') then
            read(lineorig,*) infoHead%radius
         else if(keyword(1:12) == 'RESOLUTION_H') then
            read(lineorig,*) infoHead%resolution_high
         else if(keyword(1:12) == 'RESOLUTION_L') then
            read(lineorig,*) infoHead%resolution_low
         else if(keyword(1:12) == 'RESOLUTION_O') then
            read(lineorig,*) infoHead%resolution_out
         else if(keyword(1:3) == 'SIG') then
            read(lineorig,*) infoHead%sigcut
         else if(keyword(1:3) == 'SYM') then
            read(lineorig,*) infoHead%symmetry
         else if(keyword(1:3) == 'TEM') then
            read(lineorig,*) infoHead%temperature_factor
         else if(keyword(1:2) == 'TI') then
            infoHead%title = adjustl(lineorig)
         else if(keyword(1:3) == 'UNI') then
            read(lineorig,*) infoHead%units_out
         else if(keyword(1:3) == 'VER') then
            read(lineorig,*) infoHead%verbose
         else if(keyword(1:3) == 'END' .or. 
     $           keyword(1:9) == 'INPUTPARA') then
            exit
         else
            write(*,*) 'I do not understand this line ', keyword
            stop
         endif

      enddo

      call list_key_head(infoHead)

      return 
      end subroutine ppft_info
