      program psf
      implicit none
      
      character(len=256) :: infile

#ifdef MPI
      include 'mpif.h'
#endif

      include '../include/sizes.inc'
      INCLUDE '../include/allprogs.inc'
      INCLUDE '../include/info.inc'
      include '../include/mem_time.inc'
      
      character(len=13) :: subname  = 'Main program'

      real center(2)

      logical :: confine_3D = .false.
      
      integer first_iq, last_iq, max_iq_slab,
     +     first_z,last_z, first_hk,last_hk, max_hk_slab, num_hkl,
     +     den_min_x, den_max_x, den_min_y, den_max_y,
     +     den_min_z, den_max_z,
     +     count_map3d, count_fft3d, count_fft_klh, 
     +     count_f_hkl, count_temp,
     +     min_xyz, max_xyz, hkl_max, max_z_slab,
     +     first_rr_out, last_rr_out, max_rr_out_slab,
     +     first_u, last_u, max_u_slab,
     +     first_y, last_y, max_y_slab

      integer nrffts_1_row, nrffts_1_col, nrffts_1_omega,
     +     nrffts_2_row, nrffts_2_col, nrffts_2_omega

      integer nrffta_row, nrffta_col, nfc_2d_row, nfc_2d_col,
     +        nrffta_omega

      integer npix_col_even, npix_row_even, npix_sec_even
      integer npix_col_odd,  npix_row_odd,  npix_sec_odd

      integer imax_annul, i, alloc_stat, ierr, mcw, nsteps

      real pixsize_even, pixsize_odd, mem_fft_klh, mem_temp,
     +     mapprjbig, max_annul

      real, allocatable :: map3d(:,:,:), rffta_omega(:)

      complex, allocatable :: f_hkl_even(:,:,:), f_hkl_odd(:,:,:),
     +     fft3d_even(:), fft3d_odd(:), fft_klh(:,:,:)
    
      character(len=filename_len) :: fileeven, fileodd

      double precision elapsed_time
      integer clock1, clock2, clock3, clock4
      
      progname = 'PSF '
      
#ifdef MPI
      call MPI_INIT ( mpi_err )
      if (mpi_err.ne.0) call error_stop(progname, subname, 
     +     'Error in MPI_INIT')
      call MPI_COMM_RANK ( MPI_COMM_WORLD, iam, mpi_err )
      if (mpi_err.ne.0) call error_stop(progname, subname, 
     +     'Error in MPI_COMM_RANK')
      call MPI_COMM_SIZE ( MPI_COMM_WORLD, num_nodes, mpi_err )
      if (mpi_err.ne.0) call error_stop(progname, subname, 
     +     'Error in MPI_COMM_SIZE')
      if (num_nodes > max_numnodes) then
         write(*, 9001) num_nodes, max_numnodes
 9001    format(//' Requested number of nodes = NUM_NODES    = ',i10/
     +        ' Available number of nodes = MAX_NUMNODES = ',i10/)
         call error_stop(progname, subname,
     +        'NUM_NODES requested exceeds MAX_NUMNODES')
      endif
#else
      iam = 0
      num_nodes = 1
#endif
      
      call system_clock(clock1)

C     Root process reads data from stdin, gets dimensions of even and
C     odd maps, and performs tests to make sure that parameters have
C     reasonable values

      if (iam == 0) then
         write(*,*) 'PSF: Parallel structure factor'
         write(*,*) 'Parallel version run on ', num_nodes , ' CPUs'
         write(*,*) 'For doc see: http://cryoem.ucsd.edu/programs.shtm'
         write(*,*) 

         call parse_cmd_line(infile)
         
         if(len_trim(infile) == 0) then
            read(*,'(A)') fileeven
            read(*,'(A)') fileodd
            read(*,*) res_max, res_min, nsteps, pixsiz_out
         else
            open(file=infile, unit=17)
            read(17,'(A)') fileeven
            read(17,'(A)') fileodd
            read(17,*) res_max, res_min, nsteps, pixsiz_out
            close(17)
         endif
         
         write(*,'(A,i5)')     'Number of Nodes      ', num_nodes
         write(*,'(2A)')       'Even map filename    ', trim(fileeven)
         write(*,'(2A)')       'Odd map filename     ', trim(fileodd)
         write(*,'(A,f6.2)')   'Res max              ', res_max
         write(*,'(A,f6.2)')   'Res min              ', res_min
         write(*,'(A,i5)')     '#points in FSC curve ', nsteps
         write(*,'(A,f7.4,/)') 'Pixel size           ', pixsiz_out

         zero_fill = 1.0
         NFILES    = 1
         mapmagfactor = 1.0 

         call get_map_dimensions(fileeven, npix_col_even, npix_row_even,
     $        npix_sec_even, pixsize_even, ierr)
         if(ierr > 0) then
            call error_stop(progname, subname,
     +           'Error in even 3D map mode, it should be 20 or 40')
         endif
         
         call get_map_dimensions(fileodd, npix_col_odd, npix_row_odd,
     $        npix_sec_odd, pixsize_odd, ierr)
         if(ierr > 0) then
            call error_stop(progname, subname,
     +           'Error in odd 3D map mode, it should be 20 or 40')
         endif

         if (abs(pixsize_odd - pixsize_even) > 10E-5) then
            write(*,*) 'Warning: Pixel sizes from map headers differ'
            write(*,*) '         pixel size odd map  = ', pixsize_odd
            write(*,*) '         pixel size even map = ', pixsize_even
         else if (abs(pixsize_odd - pixsiz_out) >= 10E-5) then
            write(*,*) 'Warning: Map pixel size differs from PSF input'
            write(*,*) '         pixel size odd map  = ', pixsize_odd
            write(*,*) '         pixel size even map = ', pixsize_even
         endif

         if ( npix_col_even .ne. npix_col_odd .or.
     $        npix_row_even .ne. npix_row_odd .or.
     $        npix_sec_even .ne. npix_sec_odd) then
            call error_stop(progname, subname,
     +           'Two 3D density map have different size!')
         else
            npix_col = npix_col_even
            npix_row = npix_row_even
            npix_sec = npix_sec_even
         endif

         if(res_max >= res_min) then
            call error_stop(progname, subname,
     +           'res_max is greater than res_min')
         endif
         if(res_max < 2*pixsiz_out) then
            call error_stop(progname, subname,
     +           'Nyquist condition violated')
         endif
         if(nsteps <= 0) then
            call error_stop(progname, subname,
     +           'Number of points in FSC curve <= 0')
         endif
                  
      endif

#ifdef MPI
      mcw = mpi_comm_world
      call mpi_bcast(npix_col,     1, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(npix_row,     1, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(res_max,      1, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(res_min,      1, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(nsteps,       1, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(pixsiz_out,   1, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(zero_fill,    1, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(nfiles,       1, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(mapmagfactor, 1, mpi_real,    0, mcw, mpi_err)
#endif

C     Initialize parameters and arrays
      call intlz_params(hkl_max, num_hkl, min_xyz, max_xyz, 
     +     nrffta_row, nrffta_col, nrffta_omega, nfc_2d_row, 
     +     nfc_2d_col, nrffts_1_row, nrffts_1_col, 
     +     nrffts_1_omega, nrffts_2_row, nrffts_2_col, 
     +     nrffts_2_omega, den_min_x, den_max_x, den_min_y, 
     +     den_max_y, den_min_z, den_max_z,
     +     confine_3D)
      
      call intlz_arrays(hkl_max, min_xyz, max_xyz, min_z, max_z,
     +     first_hk, last_hk, max_hk_slab, first_iq, last_iq,
     +     max_iq_slab, first_rr_out, last_rr_out, max_rr_out_slab,
     +     first_u, last_u, max_u_slab,
     +     first_y, last_y, max_y_slab, first_z, last_z, max_z_slab,
     +     npix_row)

      if(iam == 0) then
         print *, ' zero_fill after  call of INTLZ_PARAMS =',zero_fill
         print *, ' nrffta_row after call of INTLZ_PARAMS =',nrffta_row
      endif

#ifdef MPI
      call mpi_barrier(mcw, mpi_err)
#endif

      call system_clock(clock2)
      t_init = elapsed_time(clock1, clock2)
      
C     Perform initializations for FFT calculations
      allocate(rffta_omega(nrffta_omega), stat = alloc_stat)
      if(alloc_stat > 0) call error_stop (progname, subname,
     +     'Error in allocating memory for nrffta_omega')
      call vrffti( nrffta_col, rffta_omega )

      max_annul = real(nrffta_row)*pixsiz_out/res_max
      imax_annul = min(int(max_annul)+1,hkl_max)

      count_map3d = npix_col * npix_row * max_y_slab
      mem_map3d = 4 * count_map3d
      count_fft3d = (hkl_max+1)* num_hkl * (last_y - first_y + 1)
      mem_fft3d = 8 * count_fft3d

      allocate(map3d(npix_col, npix_row, max_y_slab),
     +     fft3d_even(count_fft3d), fft3d_odd(count_fft3d),
     +     stat = alloc_stat)
      if(alloc_stat > 0) then
         call error_stop(progname, subname,
     +        'Unable to allocate arrays map3d, fft3d_even/odd')
      else
         mem_total = mem_total + mem_map3d + 2*mem_fft3d
      endif

C     Read even map and calculate FFT
      call read_map(fileeven, map3d, npix_col, npix_row,
     +     max_ y_slab, mapprjbig)
      do i = 0, last_y - first_y
         center(1) = (npix_row-1)/2.0
         center(2) = (npix_col-1)/2.0
         call fft_2dfft(map3d(1,1,i+1), 
     +        npix_row, npix_col, center,
     +        nrffta_omega, rffta_omega, nrffta_row, nrffta_col,
     +        hkl_max, hkl_max, fft3d_even(i*(hkl_max+1)*num_hkl+1))
      enddo

C     Read odd map and calculate FFT
      call read_map(fileodd, map3d, npix_col, npix_row,
     +     max_ y_slab, mapprjbig)
      do i = 0, last_y - first_y
         center(1) = (npix_row-1)/2.0
         center(2) = (npix_col-1)/2.0
         call fft_2dfft(map3d(1,1,i+1), 
     +        npix_row, npix_col, center,
     +        nrffta_omega, rffta_omega, nrffta_row, nrffta_col,
     +        hkl_max, hkl_max, fft3d_odd(i*(hkl_max+1)*num_hkl+1))
      enddo
      
      deallocate(map3d)
      mem_total = mem_total - mem_map3d

#ifdef MPI
      call mpi_barrier(mcw, mpi_err)
#endif

C     Perform data rearrangements and communications for parallel runs

#ifdef MPI
      if (num_nodes > 1) then
         count_fft_klh = (hkl_max+1)*num_hkl*(last_y - first_y + 1)
         mem_fft_klh = 8*count_fft_klh
         count_temp = (hkl_max+1)* num_hkl * (last_y - first_y + 1)
         mem_temp = 8 * count_temp
         count_fft3d = (last_iq-first_iq+1)*num_hkl*npix_row
         mem_fft3d = 8*count_fft3d

         allocate(fft_klh(0:hkl_max, -hkl_max:hkl_max, 
     +        first_y:last_y), stat = alloc_stat)
         if(alloc_stat > 0) then
            call error_stop (progname, subname,
     +           '3D density fft exch memory 1 allocation failed')
         else
            mem_total = mem_total + mem_fft_klh
         endif

         call arrange_3d_1(fft3d_even, fft_klh, num_nodes, hkl_max,
     +        first_y, last_y, frst_lst_iq)
         deallocate(fft3d_even)
         mem_total = mem_total - mem_temp
         allocate(fft3d_even(count_fft3d), stat = alloc_stat)
         if(alloc_stat > 0) then
            call error_stop (progname, subname,
     +           'Failed to allocate fft3d_even')
         else
            mem_total = mem_total + mem_fft3d
            mem_peak  = max(mem_peak, mem_total)
         endif
         call exch_3d_1(fft_klh, hkl_max, fft3d_even, npix_row)

         call arrange_3d_1(fft3d_odd, fft_klh, num_nodes, hkl_max,
     +        first_y, last_y, frst_lst_iq)
         deallocate(fft3d_odd)
         mem_total = mem_total - mem_temp
         allocate(fft3d_odd(count_fft3d), stat = alloc_stat)
         if(alloc_stat > 0) then
            call error_stop (progname, subname,
     +           'Failed to allocate fft3d_odd')
         else
            mem_total = mem_total + mem_fft3d
            mem_peak  = max(mem_peak, mem_total)
         endif
         call exch_3d_1(fft_klh, hkl_max, fft3d_odd, npix_row)
         
         deallocate(fft_klh)
         mem_total = mem_total - mem_fft_klh
      endif
#endif

      count_f_hkl = (last_iq-first_iq+1)* num_hkl**2
      mem_f_hkl = 8*count_f_hkl

C     Complete calculation of 3D FFT for even map
      allocate(f_hkl_even(-hkl_max:hkl_max, -hkl_max:hkl_max,
     +     first_iq:last_iq),
     +     stat = alloc_stat)
      if(alloc_stat > 0) then
         call error_stop (progname, subname,
     +        '3D density fft 1dfft allocation failed')
      else
         mem_total = mem_total + mem_f_hkl 
         mem_peak  = max(mem_peak, mem_total)
      endif
      call cfft_1d(fft3d_even, f_hkl_even, hkl_max, npix_row,
     +     nrffta_omega, rffta_omega)
      deallocate(fft3d_even)
      mem_total = mem_total - mem_fft3d

C     Complete calculation of 3D FFT for odd map
      allocate(f_hkl_odd(-hkl_max:hkl_max, -hkl_max:hkl_max,
     +     first_iq:last_iq),
     +     stat = alloc_stat)
      if(alloc_stat > 0) then
         call error_stop (progname, subname,
     +        '3D density fft 1dfft allocation failed')
      else
         mem_total = mem_total + mem_f_hkl 
         mem_peak  = max(mem_peak, mem_total)
      endif
      call cfft_1d(fft3d_odd, f_hkl_odd, hkl_max, npix_row,
     +     nrffta_omega, rffta_omega)
      deallocate(fft3d_odd)
      mem_total = mem_total - mem_fft3d

#ifdef MPI
      call mpi_barrier(mcw, mpi_err)
#endif
      
      call system_clock(clock3)
      t_3dfft = elapsed_time(clock2, clock3)

C     Compute S-factors
      call comp_sfactor(iam, f_hkl_even, f_hkl_odd, hkl_max, 
     +     first_iq, last_iq, res_max, res_min,
     +     nrffta_row, pixsiz_out, nsteps, mem_cmpt_intrps)

      mem_peak  = max(mem_peak, mem_total+mem_cmpt_intrps)
      deallocate(f_hkl_even, f_hkl_odd)
      mem_total = mem_total - 2 * mem_f_hkl

#ifdef MPI
      call mpi_barrier(mcw, mpi_err)
#endif

      call system_clock(clock4)
      t_interp = elapsed_time(clock3, clock4)
      
C     Print timing and memory usage information
      if (iam == 0) then
         time = t_init + t_3dfft + t_interp
         write(*,40) iam, t_init, t_3dfft, t_interp, time
 40      format (/ 'Node ', i3,
     +        ' Time used in initialization:      ', f15.4, ' ' / 8x,
     +        ' Time used in read map & 3DFFT:    ', f15.4, ' ' / 8x,
     +        ' Time used in computing S-factors: ', f15.4, ' ' / 8x,
     +        ' Total time used:                  ', f15.4)
         write(*,60) iam, mem_peak, mem_map3d, 
     +        mem_fft3d, mem_cmpt_intrps
 60      format(/ 'Node ', i3, ' Memory allocation in BYTES:', /
     +        8x, ' Peak memory allocated   =', f18.0 /
     +        8x, ' Memory for 3D Map       =', f18.0 /
     +        8x, ' Memory for 3D FFT       =', f18.0 /
     +        8x, ' Memory used in CMPCC    =', f18.0 )
         write(*,*) '**********finished nicely**********'
      endif

#ifdef MPI
      call mpi_finalize(mpi_err)
#endif
      
      end program psf
