      subroutine comp_sfactor(iam, f_1, f_2, hkl_max, 
     +     first_iq, last_iq, res_max, res_min,
     +     nrffta_row, pixelsize, res_step, mem_cmpt)

C     Calculates the structure factors from the 3D FTs (f_1 and f_2) of
C     two maps. Typically these correspond to the even and odd maps
C     generated by the program P3DR. Since this routine writes out the
C     results, no values are returned to the calling program except for
C     the memory usage (mem_cmpt).
C#    lines commented this way calculate bins with equal number of components   
      implicit none
      
#ifdef MPI
      include 'mpif.h'
#endif

      integer, intent(in) :: nrffta_row, res_step, hkl_max, iam
      integer, intent(in) :: first_iq, last_iq
      real,    intent(in) :: res_max, res_min, pixelsize
      complex, intent(in) :: f_1(-hkl_max:hkl_max, -hkl_max:hkl_max,
     +     first_iq:last_iq)
      complex, intent(in) :: f_2(-hkl_max:hkl_max, -hkl_max:hkl_max,
     +     first_iq:last_iq)
      
      double precision, intent(out) :: mem_cmpt
      
      INTEGER   NTOT(res_step), sNTOT(res_step)

      DOUBLE PRECISION    F1SUM(res_step), sF1SUM(res_step),
     +     F2SUM(res_step), sF2SUM(res_step),
     +     SUMDAB(res_step), sSUMDAB(res_step),
     +     AVG_PDIF(res_step), sAVG_PDIF(res_step),
     +     F1SQSUM(res_step), sF1SQSUM(res_step),
     +     F2SQSUM(res_step), sF2SQSUM(res_step),
     +     F1_F2SUM(res_step), sF1_F2SUM(res_step),
     +     F1F2SUM(res_step), sF1F2SUM(res_step),
     +     REF1F2SUM(res_step), sREF1F2SUM(res_step)
      
      REAL res(res_step), CCOEF(res_step), RFAC, Rphase, rad_to_deg
      REAL FSC(res_step)
      
      integer i, h, k, l, bin, mpi_err
      real max_annulsq, min_annulsq, dstarsq, res_inc
      real PDIF, DENOM1, DENOM2, PHASE1, PHASE2
      real APART1, APART2, BPART1, BPART2, AMP1, AMP2
      
      rad_to_deg = 180./ (4.0 * atan(1.0))
      
      mem_cmpt =  res_step * ( 12*4 + 8*8)
C#      max_annulsq = (real(nrffta_row)*pixelsize/res_max)**3
C#      min_annulsq = (real(nrffta_row)*pixelsize/res_min)**3
      max_annulsq = (real(nrffta_row)*pixelsize/res_max)
      min_annulsq = (real(nrffta_row)*pixelsize/res_min)
      res_inc   = (max_annulsq - min_annulsq)/res_step
      
      DO I = 1,res_step         ! INITIALIZE ARRAYS
C#         res(I)=nrffta_row*pixelsize/((min_annulsq+i*res_inc)**(1./3))
         res(I)=nrffta_row*pixelsize/((min_annulsq+i*res_inc))
         NTOT(I)     =   0
         AVG_PDIF(I) = 0.0
         F1_F2SUM(I) = 0.0
         F1SUM(I)    = 0.0
         F2SUM(I)    = 0.0
         F1F2SUM(I)  = 0.0
         F1SQSUM(I)  = 0.0
         F2SQSUM(I)  = 0.0
         SUMDAB(I)    = 0.0
         REF1F2SUM(I) = 0.0
      ENDDO
      
      do l = first_iq, last_iq
         do k = -hkl_max, hkl_max
            do h = -hkl_max, hkl_max
C#               dstarsq = (sqrt(real(h*h + k*k + l*l)))**3
               dstarsq = (sqrt(real(h*h + k*k + l*l)))
               if(dstarsq >= min_annulsq .and.
     +              dstarsq <= max_annulsq) then
                  bin = int((dstarsq - min_annulsq)/res_inc) + 1  
                  if (bin > res_step) bin = res_step
                  
                  apart1 = real(f_1(h,k,l))
                  bpart1 = aimag(f_1(h,k,l))
                  apart2 = real(f_2(h,k,l))
                  bpart2 = aimag(f_2(h,k,l))

                  amp1 = sqrt(apart1*apart1 + bpart1*bpart1)
                  amp2 = sqrt(apart2*apart2 + bpart2*bpart2)

                  if(amp1 == 0.0) then
                     phase1 = 0.0
                  else
                     phase1 = atan2(bpart1,apart1) * rad_to_deg
                  endif

                  if(amp2 == 0.0) then
                     phase2 = 0.0
                  else
                     phase2 = atan2(bpart2,apart2) * rad_to_deg
                  endif
                  
                  PDIF   = ABS(PHASE1 - PHASE2) 
                  IF(PDIF > 180.0) PDIF = 360.0 - PDIF
                  AVG_PDIF(BIN) = AVG_PDIF(BIN) + PDIF
                  F1_F2SUM(BIN) = F1_F2SUM(BIN) + ABS(AMP1 - AMP2)
                  REF1F2SUM(BIN)  = REF1F2SUM(BIN) + 
     +                                  (apart1*apart2+bpart1*bpart2)
                  F1SUM(BIN)    = F1SUM(BIN) + AMP1
                  F2SUM(BIN)    = F2SUM(BIN) + AMP2
                  F1SQSUM(BIN)  = F1SQSUM(BIN) + AMP1*AMP1
                  F2SQSUM(BIN)  = F2SQSUM(BIN) + AMP2*AMP2
                  F1F2SUM(BIN)  = F1F2SUM(BIN) + AMP1*AMP2
                  SUMDAB(BIN)   = SUMDAB(BIN) + 
     +                 sqrt((APART1 - APART2)**2 + (BPART1 - BPART2)**2)
                  NTOT(BIN)     = NTOT(BIN) + 1
               endif
            enddo
         enddo
      enddo

#ifdef MPI
      call mpi_reduce(NTOT, sNTOT, res_step, MPI_INTEGER, 
     +     MPI_SUM, 0, mpi_comm_world, mpi_err)
      if( mpi_err .ne. 0 ) call error_stop('PSF','comp_sfactor',
     +     'Error in calling MPI_reduce')
      call mpi_reduce(AVG_PDIF, sAVG_PDIF, res_step, 
     +     MPI_DOUBLE_PRECISION,MPI_SUM, 0, mpi_comm_world, mpi_err)
      if( mpi_err .ne. 0 ) call error_stop('PSF','comp_sfactor',
     +     'Error in calling MPI_reduce')
      call mpi_reduce(F1_F2SUM, sF1_F2SUM, res_step, 
     +     MPI_DOUBLE_PRECISION,MPI_SUM, 0, mpi_comm_world, mpi_err)
      if( mpi_err .ne. 0 ) call error_stop('PSF','comp_sfactor',
     +     'Error in calling MPI_reduce')
      call mpi_reduce(F1SUM, sF1SUM, res_step, MPI_DOUBLE_PRECISION,
     +     MPI_SUM, 0, mpi_comm_world, mpi_err)
      if( mpi_err .ne. 0 ) call error_stop('PSF','comp_sfactor',
     +     'Error in calling MPI_reduce')
      call mpi_reduce(F2SUM, sF2SUM, res_step,MPI_DOUBLE_PRECISION, 
     +     MPI_SUM, 0, mpi_comm_world, mpi_err)
      if( mpi_err .ne. 0 ) call error_stop('PSF','comp_sfactor',
     +     'Error in calling MPI_reduce')
      call mpi_reduce(F1SQSUM, sF1SQSUM, res_step, 
     +     MPI_DOUBLE_PRECISION, MPI_SUM, 0, mpi_comm_world, mpi_err)
      if( mpi_err .ne. 0 ) call error_stop('PSF','comp_sfactor',
     +     'Error in calling MPI_reduce')
      call mpi_reduce(F2SQSUM, sF2SQSUM, res_step, 
     +     MPI_DOUBLE_PRECISION, MPI_SUM, 0, mpi_comm_world, mpi_err)
      if( mpi_err .ne. 0 ) call error_stop('PSF','comp_sfactor',
     +     'Error in calling MPI_reduce')
      call mpi_reduce(F1F2SUM, sF1F2SUM, res_step, 
     +     MPI_DOUBLE_PRECISION, MPI_SUM, 0, mpi_comm_world, mpi_err)
      if( mpi_err .ne. 0 ) call error_stop('PSF','comp_sfactor',
     +     'Error in calling MPI_reduce')
      call mpi_reduce(SUMDAB,sSUMDAB,res_step,MPI_DOUBLE_PRECISION,
     +     MPI_SUM, 0, mpi_comm_world, mpi_err)
      if( mpi_err .ne. 0 ) call error_stop('PSF','comp_sfactor',
     +     'Error in calling MPI_reduce')
      call mpi_reduce(REF1F2SUM, sREF1F2SUM, res_step, 
     +     MPI_DOUBLE_PRECISION, MPI_SUM, 0, mpi_comm_world, mpi_err)
      if( mpi_err .ne. 0 ) call error_stop('PSF','comp_sfactor',
     +     'Error in calling MPI_reduce')
#else
      sNTOT     = NTOT
      sAVG_PDIF = AVG_PDIF
      sF1_F2SUM = F1_F2SUM
      sF1SUM    = F1SUM
      sF2SUM    = F2SUM
      sF1SQSUM  = F1SQSUM
      sF2SQSUM  = F2SQSUM
      sF1F2SUM  = F1F2SUM
      sSUMDAB   = SUMDAB
      sREF1F2SUM  = REF1F2SUM
#endif      

      if (iam == 0) then
         Write(*,'(/,2A)') '   Bin#  Resolution  observ  ',
     +        '   R-SF       CC       FSC     R-phase  phase-diff',
     +        '  ------  --------  -------- ', 
     +        ' --------  --------  --------  --------  -------- '
         WRITE(*,'(I8, F10.2, I10, 5F10.3)') 0, res_min, 0, 0.0, 
     +        1.0, 1.0, 0.0, 0.0
         
         DO I = 1, res_step
            CCOEF(I) = 1.0
            FSC(I) = 1.0
            IF(sNTOT(I) > 0) THEN
               DENOM1    = REAL(sF1SQSUM(I) - (sF1SUM(I)**2)/sNTOT(I))
               DENOM2    = REAL(sF2SQSUM(I) - (sF2SUM(I)**2)/sNTOT(I))
               IF(DENOM1 > 0.0 .AND. DENOM2 > 0.0) THEN
                  CCOEF(I) = REAL((sF1F2SUM(I) - 
     +                 (sF1SUM(I)*sF2SUM(I)/sNTOT(I)))/
     +                 ( SQRT(DENOM1) * SQRT(DENOM2) ))
               ENDIF
               IF(sF1SQSUM(I) > 0.0 .AND. sF2SQSUM(I) > 0.0) THEN
                  FSC(I) = REAL(sREF1F2SUM(I) /
     +                 SQRT(sF1SQSUM(I) * sF2SQSUM(I)))
               ENDIF
               RFAC     = REAL(sF1_F2SUM(I)/sF1SUM(I))
               Rphase   = REAL(sSUMDAB(I)/ sF1SUM(I))
               AVG_PDIF(I) = sAVG_PDIF(I)/REAL(sNTOT(I))
            ENDIF
            
            WRITE(*,'(I8, F10.2, I10, 5F10.3)') I, res(I), sNTOT(I), 
     +           RFAC, CCOEF(I), FSC(I), Rphase, AVG_PDIF(I)
         ENDDO
      endif
      
C     R-SF = sum(F1-kF2) / sum F1
C     S1 = sum(<F1>-F1)(<F2>-F2)
C     S2 = sum(<F1>-F1)^2
C     S3 = sum(<F2>-F2)^2
C     CC  = S1 / sqrt(S2*S3)
C     FSC = sum{re{F1*conj{F2}}} / sqrt{sum{|F1|**2} * sum{|F2|**2}}
C     A = F1 cos(phi1*deg_to_rad) - F2 cos(phi2*deg_to_rad)
C     B = F2 sin(phi2*deg_to_rad) - F1 sin(phi1*deg_to_rad)
C     R-phase =  sum(sqrt(A^2+B^2)) / sum(f1)
C     phase-diff = sum |phi1-phi2| / # observations

      end subroutine comp_sfactor
