#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "../include/common.h"
#include "../include/libPIF.h"

int main(int argc, char **argv) {
  forPifHead_t  thisHead;
  int mode, ncol, nrow, nsec, ncenx, nceny, ncenz, start=1;
  int i, j, k, p, ir, indx, zero=0;
  int *nann;
  int ndx[27], count;
  float *map3d, *sph3d, r, r2, x, y, z;
  double delta, *avgann;

  if(argc < 2) {
    printf("\nUSAGE: gensp imgname\n\n");
    printf("Generate a spherically averaged PIF map\n");
    printf("Required command line argument\n");
    printf("  imgname - name of PIF map or image stack\n\n");
    exit(0);
  }

  thisHead.luNum = 4;
  pif_open(argv[1], &thisHead.luNum, "r");
  pif_read_gh(&thisHead);

  mode = thisHead.mode;
  ncol = thisHead.numCols;
  nrow = thisHead.numRows;
  nsec = thisHead.numSecs;
  ncenx = ncol/2; // center for count starting from 0
  nceny = nrow/2; // center for count starting from 0
  ncenz = nsec/2; // center for count starting from 0

  if(mode != 20 && mode != 40 && mode != 46 && mode != 6) {
    printf("Invalid map mode %d - mode must equal 6, 20, 40, or 46\n", mode);
    exit(0);
  }

  map3d       = (float *) malloc(sizeof(float)*ncol*nrow*nsec);
  sph3d       = (float *) malloc(sizeof(float)*ncol*nrow*nsec);
  avgann      = (double *) malloc(sizeof(double)*(ncenx+1));
  nann        = (int *) malloc(sizeof(int)*(ncenx+1));

  thisHead.imgNum = 1; // Image numbering starts at 1
  pif_read_dh(&thisHead);
  thisHead.luNum = 4;
    
  if(mode == 20) {
    pif_read_mapi2(&thisHead, map3d, &start, &nsec, &ncol, &nrow, &nsec);
    pif_close(&thisHead.luNum);
  }
  else {
    printf("PIF file mode must be 20\n");
    pif_close(&thisHead.luNum);
    exit(0);
  }
  
  // Flip sign for pixel intensities
  for(i=0; i<ncol*nrow*nsec; i++)
    map3d[i] = -map3d[i];
    
  for(ir=0; ir<=ncenx; ir++) {
    avgann[ir]   = 0.0;
    nann[ir]     = 0;
  }
  
  // Calculate radial density profile
  for(i=0; i<ncol; i++) {
    for(j=0; j<nrow; j++) {
      for(k=0; k<nsec; k++) {
	indx = i*nrow*nsec + j*nsec + k;
	x = i - ncenx;
	y = j - nceny;
	z = k - ncenz;
	r2 = x*x + y*y + z*z;
	ir = sqrt(fabs(r2));
	if(ir <= ncenx) {
	  nann[ir]++;
	  avgann[ir] += map3d[indx];
	}
      }
    }
  }

  for(ir=0; ir<=ncenx; ir++)
    if(nann[ir] > 0)
      avgann[ir] = avgann[ir]/nann[ir];
    else
      avgann[ir] = 0.0;
  
  // Reuse array and initialize to zero
  for(i=0; i<ncol; i++) {
    for(j=0; j<nrow; j++) {
      for(k=0; k<nsec; k++) {
	indx = i*nrow*nsec + j*nsec + k;
	map3d[indx] = 0.0;
      }
    }
  }
  
  // Radial smoothing
  for(i=0; i<ncol; i++) {
    for(j=0; j<nrow; j++) {
      for(k=0; k<nsec; k++) {
	indx = i*nrow*nsec + j*nsec + k;
	x = i - ncenx;
	y = j - nceny;
	z = k - ncenz;
	r2 = x*x + y*y + z*z;
	r = sqrt(fabs(r2));
	ir = r;
	delta = r - ir;
	if(ir < ncenx-1)
	  map3d[indx] = (1.0 - delta)*avgann[ir] + delta*avgann[ir+1];
      }
    }
  }
  
  // Voxel averaging
  for(i=0; i<ncol; i++) {
    for(j=0; j<nrow; j++) {
      for(k=0; k<nsec; k++) {
	indx = i*nrow*nsec + j*nsec + k;
	
	ndx[0]  = (i-1)*nrow*nsec + (j-1)*nsec + k-1;
	ndx[1]  = (i-1)*nrow*nsec + (j-1)*nsec + k-1;
	ndx[2]  = (i-1)*nrow*nsec + (j-1)*nsec + k-1;
	ndx[3]  = (i-1)*nrow*nsec + (j)*nsec + k;
	ndx[4]  = (i-1)*nrow*nsec + (j)*nsec + k;
	ndx[5]  = (i-1)*nrow*nsec + (j)*nsec + k;
	ndx[6]  = (i-1)*nrow*nsec + (j+1)*nsec + k+1;
	ndx[7]  = (i-1)*nrow*nsec + (j+1)*nsec + k+1;
	ndx[8]  = (i-1)*nrow*nsec + (j+1)*nsec + k+1;
	ndx[9] = (i)*nrow*nsec + (j)*nsec + k-1;
	ndx[10] = (i)*nrow*nsec + (j)*nsec + k-1;
	ndx[11] = (i)*nrow*nsec + (j)*nsec + k-1;
	ndx[12] = (i)*nrow*nsec + (j)*nsec + k;
	ndx[13] = (i)*nrow*nsec + (j)*nsec + k;
	ndx[14] = (i)*nrow*nsec + (j)*nsec + k;
	ndx[15] = (i)*nrow*nsec + (j)*nsec + k+1;
	ndx[16] = (i)*nrow*nsec + (j)*nsec + k+1;
	ndx[17] = (i)*nrow*nsec + (j)*nsec + k+1;
	ndx[18] = (i+1)*nrow*nsec + (j)*nsec + k-1;
	ndx[19] = (i+1)*nrow*nsec + (j)*nsec + k-1;
	ndx[20] = (i+1)*nrow*nsec + (j)*nsec + k-1;
	ndx[21] = (i+1)*nrow*nsec + (j)*nsec + k;
	ndx[22] = (i+1)*nrow*nsec + (j)*nsec + k;
	ndx[23] = (i+1)*nrow*nsec + (j)*nsec + k;
	ndx[24] = (i+1)*nrow*nsec + (j)*nsec + k+1;
	ndx[25] = (i+1)*nrow*nsec + (j)*nsec + k+1;
	ndx[26] = (i+1)*nrow*nsec + (j)*nsec + k+1;
	
	indx = ndx[13];
	sph3d[indx] = 0.0;
	count = 0;
	for (p=0; p<27; p++) {
	  if(ndx[p] >= 0 && ndx[p] < ncol*nrow*nsec) {
	    sph3d[indx] += map3d[ndx[p]];
	    count++;
	  }
	}
	sph3d[indx] /= count;
	
      }
    }
  }
  
  // Dump out smoothed map
  pif_open("spherical.pif", &thisHead.luNum,"w");
  pif_write_gh(&thisHead);
  pif_write_dh(&thisHead);
  pif_write_map_sections(&thisHead, sph3d, &mode, 
			 &ncol, &nrow, &nsec, &zero, &nsec);
  pif_close(&thisHead.luNum);
  free(sph3d);
  return 0;
}
