#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "../include/common.h"
#include "../include/libPIF.h"

int main(int argc, char **argv) {
  forPifHead_t  thisHead;
  unsigned char *byte_buffer;
  int mode, ncol, nrow, nsec, ncenx, nceny, ncenz, start=1;
  int i, j, k, m, ir, irr, indx, outer_loc, inner_loc, count;
  int ip, jp, kp, indxp, nradx;
  int maxavg_loc, maxprod_loc, maxrprod_loc;
  int first_flip, nimg, nskip, *ncumm;
  float irw, irrw, fir;
  float *map3d, r2, bkg, x, y, z, phi, deg2rad;
  double rw_sum, rw_rsum, tw_sum, tw_rsum, sum_ratio, rsum_ratio;
  double avg, std, numerator, denominator, sum, sum2;
  double maxavg_val, maxprod_val, maxrprod_val, cc;
  double *sumann, *sum2ann, *avgann, *stdann;
  double *reldev, *prodann, *rprodann, *avgcumm, *nann;

  deg2rad = 360.0/(8.0*atan(1.0));

  if(argc < 2) {
    printf("\nUSAGE:  imgstats imgname [nskip]\n\n");
    printf("Prints out statistics on a PIF map or image stack. For image\n");
    printf("stacks, statistics provided individually for each image\n\n");
    printf("Required command line argument\n");
    printf("  imgname - name of PIF map or image stack\n\n");
    printf("Optional command line argument\n");
    printf("  nskip   - number of annuli from center of particle to skip\n");
    printf("            when determining peak density and other stats.\n");
    printf("            By default, nskip == 10\n");
    exit(0);
  }
  
  if(argc >= 3)
    nskip = strtod(argv[2], '\0');
  else
    nskip = 10;

  thisHead.luNum = 4;
  pif_open(argv[1], &thisHead.luNum, "r");
  pif_read_gh(&thisHead);

  nimg = thisHead.nimg;
  mode = thisHead.mode;
  ncol = thisHead.numCols;
  nrow = thisHead.numRows;
  nsec = thisHead.numSecs;
  ncenx = ncol/2; // center for count starting from 0
  nceny = nrow/2; // center for count starting from 0
  ncenz = nsec/2; // center for count starting from 0
  nradx = ncenx+1;  // number of radial samples

  if(mode != 20 && mode != 40 && mode != 46 && mode != 6) {
    printf("Invalid map mode %d - mode must equal 6, 20, 40, or 46\n", mode);
    exit(0);
  }

  byte_buffer = (unsigned char *) malloc(sizeof(unsigned char)*ncol*nrow);
  map3d       = (float *) malloc(sizeof(float)*ncol*nrow*nsec);
  sumann      = (double *) malloc(sizeof(double)*nradx);
  sum2ann     = (double *) malloc(sizeof(double)*nradx);
  avgann      = (double *) malloc(sizeof(double)*nradx);
  avgcumm     = (double *) malloc(sizeof(double)*nradx);
  stdann      = (double *) malloc(sizeof(double)*nradx);
  prodann     = (double *) malloc(sizeof(double)*nradx);
  rprodann    = (double *) malloc(sizeof(double)*nradx);
  reldev      = (double *) malloc(sizeof(double)*nradx);
  nann        = (double *) malloc(sizeof(double)*nradx);
  ncumm       = (int *) malloc(sizeof(int)*nradx);

  for(m=0; m<nimg; m++) {

    thisHead.imgNum = m+1; // Image numbering starts at 1
    pif_read_dh(&thisHead);
    thisHead.luNum = 4;
    bkg = thisHead.img_bkg;
    
    if(mode == 20)
      pif_read_mapi2(&thisHead, map3d, &start, &nsec, &ncol, &nrow, &nsec);
    else if(mode == 40)
      pif_read_mapi4(&thisHead, map3d, &start, &nsec, &ncol, &nrow, &nsec);
    else if(mode == 46) {
      pif_read_boxi4(&thisHead, map3d);
      for(i=0; i<ncol*nrow; i++)
	map3d[i] -= bkg;
    }
    else if(mode == 6) {
      pif_read_byte_image(&thisHead.luNum, byte_buffer,
			  &thisHead.imgOffset, &ncol, &nrow, &nsec);
      for(i=0; i<ncol*nrow; i++) {
	map3d[i] = byte_buffer[i];
	map3d[i] -= bkg;
      }
    }
      
    // Flip sign for pixel intensities
    for(i=0; i<ncol*nrow*nsec; i++)
      map3d[i] = -map3d[i];
    
    // Initialize the accumulators before processing each image
    sum = 0.0;
    sum2 = 0.0;
    count = 0;
    rw_sum = 0.0;
    rw_rsum = 0.0;
    tw_sum = 0.0;
    tw_rsum = 0.0;
    for(ir=0; ir<nradx; ir++) {
      sumann[ir]   = 0.0;
      sum2ann[ir]  = 0.0;
      avgann[ir]   = 0.0;
      avgcumm[ir]  = 0.0;
      stdann[ir]   = 0.0;
      prodann[ir]  = 0.0;
      rprodann[ir] = 0.0;
      nann[ir]     = 0.0;
      ncumm[ir]    = 0;
    }

    for(i=0; i<ncol; i++) {
      for(j=0; j<nrow; j++) {
	for(k=0; k<nsec; k++) {
	  indx = i*nrow*nsec + j*nsec + k;
	  x = i - ncenx;
	  y = j - nceny;
	  z = k - ncenz;

	  r2 = x*x + y*y + z*z;
// the following two lines implement nearest neighbor interpolation
      ir = (int)(sqrt(r2) + 0.5);
      fir = (float) ir;
// the following two lines implement linear interpolation
//      fir = sqrt(r2);
//      ir = (int)fir;
      irr = ir + 1;
      irrw = fir - ir;
      irw = 1. - irrw;
      
	  if(irr < nradx) {
	    count++;
	    sum += map3d[indx];
	    sum2 += map3d[indx]*map3d[indx];
	    nann[ir] += irw;
	    sumann[ir] += irw*map3d[indx];
	    sum2ann[ir] += irw*irw*map3d[indx]*map3d[indx];
	    nann[irr] += irrw;
	    sumann[irr] += irrw*map3d[indx];
	    sum2ann[irr] += irrw*irrw*map3d[indx]*map3d[indx];
	    
	    // Calculating wedge statistics for central section
	    if(k == ncenz && map3d[indx] > 0.0) {
	      phi = deg2rad*atan2(y,x);
	      if(abs(phi) < 20.0) { // right wedge
		rw_sum  += map3d[indx];
		rw_rsum += ir*map3d[indx];
	      }
	      if(phi > 70.0 && phi < 110.0) { // top wedge
		tw_sum  += map3d[indx];
		tw_rsum += ir*map3d[indx];
	      }
	    }
	  }
	  
	}
      }
    }

    sum_ratio = tw_sum/rw_sum;
    rsum_ratio = tw_rsum/rw_rsum;

    // Calculating cummulative statistics
    for(ir=0; ir<nradx; ir++) {
      for(i=ir; i<nradx; i++) {
	ncumm[i] += nann[ir];
	avgcumm[i] += sumann[ir];
      }
    }
    
    for(ir=0; ir<nradx; ir++) {
      if(ncumm[ir] > 0)
	avgcumm[ir] /= ncumm[ir];
      else
	avgcumm[ir] = 0.0;
    }

    // Global statistics
    avg = sum/count;
    numerator = sum2 - (sum*sum)/count;
    denominator = count-1;
    std = sqrt(numerator/denominator);

    // Asymmetry measure (CC between map and inverse)
    numerator = 0.0;
    denominator = 0.0;

    for(i=0; i<ncol; i++) {
      for(j=0; j<nrow; j++) {
	for(k=0; k<nsec; k++) {
	  indx = i*nrow*nsec + j*nsec + k;
	  r2 = (i-ncenx)*(i-ncenx) + (j-nceny)*(j-nceny) + (k-ncenz)*(k-ncenz);
	  ir = (int)(sqrt(r2)+0.5);
	  if(ir <nradx) {
	    kp = nsec - 1 - k;
	    jp = nrow - 1 - j;
	    ip = ncol - 1 - i;
	    indxp = ip*nrow*nsec + jp*nsec + kp;
	    numerator += (map3d[indx] - avg) * (map3d[indxp] - avg);
	    denominator += (map3d[indx] - avg) * (map3d[indx] - avg);
	  }
	  
	}
      }
    }

    cc = numerator/denominator;
    
    maxavg_loc   = nskip;
    maxavg_val   = -1.0e10;
    maxprod_loc  = nskip;
    maxprod_val  = -1.0e10;
    maxrprod_loc = nskip;
    maxrprod_val = -1.0e10;
    
    // Statistics by annulus
    for(ir=0; ir<nradx; ir++) {
      if(nann[ir] > 0.)
	avgann[ir] = sumann[ir]/nann[ir];
      else
	avgann[ir] = 0.0;
      
      if(nann[ir] > 1.) {
	numerator = sum2ann[ir] - (sumann[ir]*sumann[ir])/nann[ir];
	denominator = nann[ir]-1;
	stdann[ir] = sqrt(fabs(numerator/denominator));
      } else
	stdann[ir] = 0.0;
      
      if(avgann[ir] > maxavg_val && ir > nskip) {
	maxavg_val = avgann[ir];
	maxavg_loc = ir;
      }

      prodann[ir]  = stdann[ir]*avgann[ir];
      rprodann[ir] = ir*stdann[ir]*avgann[ir];
      
      if(prodann[ir] > maxprod_val && ir > nskip) {
	maxprod_val = prodann[ir];
	maxprod_loc = ir;
      }

      if(rprodann[ir] > maxrprod_val && ir > nskip) {
	maxrprod_val = rprodann[ir];
	maxrprod_loc = ir;
      }
      
    }

    // Get relative deviation of annulus avg from cumm average
    for(ir=0; ir<nradx; ir++) {
      if(fabs(avgcumm[ir]) > 0.001)
	reldev[ir] = fabs((avgann[ir]-avgcumm[ir])/avgcumm[ir]);
      else
	reldev[ir] = 0.0;
    }

    
    // Identify local maxima on either side of peak density
    outer_loc = maxavg_loc;
    for (ir=maxavg_loc+1; ir<nradx; ir++) {
      if(avgann[ir] > avgann[ir-1])
	break;
      outer_loc = ir;
    }
    
    inner_loc = maxavg_loc;
    for (ir=maxavg_loc-1; ir>nskip; ir--) {
      if(avgann[ir] > avgann[ir+1])
	break;
      inner_loc = ir;
    }
    
    first_flip = maxavg_loc;
    for (ir=maxavg_loc+1; ir<nradx; ir++) {
      if(avgann[ir] < 0)
	break;
      first_flip = ir;
    }

    printf("# ----- imgstats results -----\n");
    printf("#\n");
    printf("# Note that densities have already been adjusted so that\n");
    printf("# positive values correspond to higher density\n");
    printf("#\n");
    printf("# --- Global map statistics ---\n");
    printf("# Image mode = %d\n", mode);
    printf("# numCols = %d\n", ncol);
    printf("# numRows = %d\n", nrow);
    printf("# numSecs = %d\n", nsec);
    printf("# global avg = %10.3f\n", avg);
    printf("# global std = %10.3f\n", std);
    printf("# Correlation between map and inverse = %8.3f\n", cc);
    printf("# right wedge sum over positive density   = %10.3f\n", rw_sum);
    printf("# right wedge r*sum over positive density = %10.3f\n", rw_rsum);
    printf("# top wedge sum over positive density     = %10.3f\n", tw_sum);
    printf("# top wedge r*sum over positive density   = %10.3f\n", tw_rsum);
    printf("# top/right sum ratio    = %8.3f\n", sum_ratio);
    printf("# top/right r*sum ratio  = %8.3f\n", rsum_ratio);
    printf("#\n");
    printf("# start radius for peak finding = %10d\n", nskip);
    printf("# radius at peak density        = %10d\n", maxavg_loc);
    printf("# avg at peak density           = %10.3f\n", avgann[maxavg_loc]);
    printf("# std at peak density           = %10.3f\n", stdann[maxavg_loc]);
    printf("# radius at max(avg*std)        = %10d\n", maxprod_loc);
    printf("# max(avg*std)                  = %10.3f\n", prodann[maxprod_loc]);
    printf("# radius at max(r*avg*std)      = %10d\n", maxrprod_loc);
    printf("# max(r*avg*std)                = %10.3f\n", rprodann[maxrprod_loc]);
    printf("# first local peak  < peak_loc  = %10d\n", inner_loc);
    printf("# first local_peak  > peak_loc  = %10d\n", outer_loc);
    printf("# first sign change > peak_loc  = %10d\n", first_flip);
    printf("#\n");
    printf("# --- Statistics by spherical annulus ---\n");
    printf("# r(pix)   count        avg        std    |std*avg|    cummAvg    reldev\n");
    for(ir=0; ir<=ncenx; ir++)
      printf(" %5d   %8d  %10.5f %10.5f %10.5f %10.5f %10.5f\n", ir, (int)(nann[ir]+0.5), 
	     avgann[ir], stdann[ir], prodann[ir], avgcumm[ir], reldev[ir]);
    
  }

  pif_close(&thisHead.luNum);
  return 0;
}
