C *****************************************************************************
C *                                                                           *
C *    THIS PROGRAM READS SEVERAL PIFMAPS INTO CORE MEMORY, CALCULATE         *
C *         MEAN AND STD OF EACH VOXEL.                                       *
C *    WRITE OUT TWO FILES:    ONE IS MEAN VALUES OF THOSE MAPS               *
C *                            SECOND IS THE STD VALUES OF THOSE MAPS         *
C *    THE OUTPUT TWO FILES ARE USED FOR T-MAP CALCULATION                    *
C *    LOGICAL UNITS: 7 = REFERENCE 3D MAP                                    *
C *                   8 = OUTPUT MEAN PIFMAP FILE                             *
C *                   9 = OUTPUT STD PIFMAP FILE                              *
C *                                                                           *
C *    WEI ZHANG, JUN. 7, 2000                                                *
C *    modified by WEI ZHANG, sep. 27, 2000, variance is calculated for       *
C *                small number of NUMMAPS                                    *
C *                                                                           *
C *****************************************************************************
C
      program mergemaps
      IMPLICIT NONE

      include '../include/pifhead.inc'
      type(pifhead_t) MAPHEAD, OUTHEAD1, OUTHEAD2
      COMMON /PIFHEAD/   MAPHEAD, OUTHEAD1, OUTHEAD2

      character(len=1) :: rw(2)
      INTEGER*4 NCOL, NROW, NSEC, NUMMAPS, I
      CHARACTER MAP_FILENAME*64
      real*4, allocatable :: map3d(:), map_merge(:)

 10   WRITE(6,'(''$THE NUMBER OF PIFMAPS: '')')
      READ (5,*,ERR=10) NUMMAPS

      MAPHEAD%LUNUM = 7      ! UNIT FOR INPUT IMAGE FILE

C  *****************************
C  * LOOP FOR ALL 3D MAP FILES *
C  *****************************
      DO I = 1, NUMMAPS 
 11      WRITE(6,'(''$PIFMAP FILENAME: '')')
         READ (5,'(A)',ERR=11) MAP_FILENAME

         rw(1) = 'r'
         rw(2) = achar(0)
         CALL PIF_OPEN (MAP_FILENAME, MAPHEAD%LUNUM, rw)
         CALL PIF_READ_GH (MAPHEAD)
         ncol = mapHead%numCols  
         nrow = mapHead%numRows 
         nsec = mapHead%numSecs 
         print *, ncol, nrow, nsec
         IF(I.EQ.1) THEN
C     *************************************************************
C  * ALLOCATE VIRTUAL MEMORY FOR MAP AND MASK FILE             *
C  * READ 3D MAP INTO CORE MEMORY WITH SURFACE DENSITY IMPOSED *
C  *************************************************************
            allocate(map_merge(ncol*nrow*nsec), map3d(ncol*nrow*nsec))
            
         ENDIF

         CALL MAP_READ_CALC (MAP3D, MAP_MERGE, NCOL, NROW, NSEC, 
     $        I, NUMMAPS)

C     **********************
C     *  Close some files  *
C     **********************
         call pif_close(mapHead%luNum)
      ENDDO                     ! LOOP FOR ALL PIFMAP FILES

      call pif_close(OUTHEAD2%luNum)

C     ******************
C     *  Free Memmory  *  
C     ******************
      deallocate(map_merge, map3d)

      CALL TSBEND
      END program mergemaps


C  *********************************************************************
C  *                                                                   *
C  *         *****     SUBROUTINE MAP_READ_CALC      *****             *
C  *                                                                   *
C  *    READ 3D MAP DATA FROM FILE, PACK INTO "MAP" ARRAY,             * 
C  *              AND CALCULATE SUM AND SQUARE SUM.                    *   
C  *                                                                   *
C  *    ADAPED FROM SOUBROUTINE MAP_FILL_3D                            *
C  *                                                                   *
C  *********************************************************************

      SUBROUTINE MAP_READ_CALC (MAP_3D, MAP_MERGE, NCOL, NROW,
     $     NSEC,FLAG, NUMMAPS)
                              
      IMPLICIT NONE

      include '../include/pifhead.inc'
      type(pifhead_t) MAPHEAD, OUTHEAD1, OUTHEAD2
      COMMON /PIFHEAD/   MAPHEAD, OUTHEAD1, OUTHEAD2

      INTEGER*4 NCOL, NROW, NSEC, I, J, K, FLAG, NUMMAPS
      REAL*4    MAP_3D(NCOL,NROW,NSEC), MAP_MERGE(NCOL,NROW,NSEC)
      CHARACTER map_mergemapname*64
      character(len=1) :: rw(2)
      
      PRINT *, '**************************'
      PRINT *, '* Load and Write 3D map  *'
      PRINT *, '**************************'

      print *, 'sub: ', ncol, nrow, nsec, flag, NUMMAPS

      IF(FLAG.EQ.1) THEN
         DO I=1,NCOL
            DO J=1,NROW
               DO K=1,NSEC
                  MAP_3D(I,J,K)=0.0
                  MAP_MERGE(I,J,K)=0.0
               ENDDO
            ENDDO
         ENDDO
      ENDIF

      IF(FLAG.EQ.NUMMAPS) THEN
 13      WRITE(6,'(''$MAP_MERGE PIFMAP FILENAME: '')')
         READ (5,'(A)',ERR=13) MAP_MERGEMAPNAME

C     ********************************************************** 
C     *    SET UP PIF FORMAT MAP OUTPUT FILE OF THE MERGED MAP    *
C     **********************************************************
         CALL PIF_INIT_HEAD (OUTHEAD2)
         OUTHEAD2%LUNUM = 9
         rw(1) = 'w'
         rw(2) = '+'
         CALL PIF_OPEN (MAP_MERGEMAPNAME, OUTHEAD2%LUNUM, rw)
         OUTHEAD2%BINFACTOR  =   1
         OUTHEAD2%NIMG       =   1
         OUTHEAD2%MODE       =  20
         
         OUTHEAD2%NUMCOLS = NCOL ! UNBINNED SIZE
         OUTHEAD2%NUMROWS = NROW ! UNBINNED SIZE
         OUTHEAD2%NUMSECS = NSEC ! UNBINNED SIZE
         
C     MANY PARAMETERS IN THE MAPHEAD ARE NOT SETUP, SHOULD CHECK IF
C     THESE PARAMETERS WILL INFLUENCE LATER CALCULATION
         
         OUTHEAD2%A_CELL      = NCOL 
         OUTHEAD2%B_CELL      = NROW 
         OUTHEAD2%C_CELL      = NSEC
         OUTHEAD2%DELA       = MAPHEAD%DELA        
         OUTHEAD2%DELB        = MAPHEAD%DELB         
         OUTHEAD2%DELC       = MAPHEAD%DELC        
         OUTHEAD2%DATAR4_MIN       = MAPHEAD%DATAR4_MIN   
         OUTHEAD2%DATAR4_MAX          = MAPHEAD%DATAR4_MAX             
         OUTHEAD2%DATAR4_std       = MAPHEAD%DATAR4_std 
         OUTHEAD2%DATAR4_avg          = MAPHEAD%DATAR4_avg             
         OUTHEAD2%SCALEF       = MAPHEAD%SCALEF       
         OUTHEAD2%pixelSize       = MAPHEAD%pixelSize    
         OUTHEAD2%pixSizUnits        = MAPHEAD%pixSizUnits  
         DO I =1,80
            OUTHEAD2%DESCRIPTIONSTR(I) =  MAPHEAD%DESCRIPTIONSTR(I)
         ENDDO
         CALL PIF_WRITE_GH (OUTHEAD2) 
         OUTHEAD2%IMGNUM = 1
         CALL PIF_WRITE_DH (OUTHEAD2)                        
      ENDIF

      IF(FLAG.EQ.1) THEN
         CALL pif_read_WMAPI2(mapHead, MAP_MERGE, ncol, nrow, NSEC)
      else
         CALL pif_read_WMAPI2(mapHead, MAP_3D, ncol, nrow, NSEC)
      endif

      DO K = 1,NSEC
         DO J = 1,NROW
            DO I = 1,NCOL
               IF(FLAG.GT.1.and.MAP_3D(I,J,K).LT.MAP_MERGE(I,J,K)) THEN
                  MAP_MERGE(I,J,K) = MAP_3D(I,J,K)
               endif
            ENDDO     
         ENDDO
      ENDDO

      IF(FLAG.EQ.NUMMAPS) THEN
         call pif_write_map_sections(outhead2, map_merge,
     $        outhead2%mode, ncol, nrow, nsec, 0, nsec)
      endif

      RETURN
      END
