      program pifmapdel
      implicit none

C     Mask a map based on a PDB file
C     Author: W. Zhang, June 19, 2002
C     Modified: RS Sinkovits, November, 2010
      
      include '../include/pifhead.inc'
      type(pifhead_t) outhead, inhead

      character outmap*64, inmap*64, pdbfile*64, a1*64, a2*64
      character(len=1) rw(2)
      character(len=20) tempstr
      integer   i, j, k, ncol, nrow, nsec, behavior
      integer   ix1, ix2, iy1, iy2, iz1, iz2
      integer   iargc
      real      border, xx, yy, zz, pixelsize
      real      rx, ry, rz, r
      real*4, allocatable :: map3d(:,:,:)
      real*8 atof_fstr
      integer, allocatable :: mask(:,:,:)

C     --- Read parameters from stdin
      if(iargc() < 6) then
 10      write(*,'(a)', advance='no') 'border (in pixel float):  '
         read (*,*, err=10) border
 11      write(*,'(a)', advance='no') 'input PIF map:            '
         read (*,*, err=11) inmap
 12      write(*,'(a)', advance='no') 'output PIF map:           '
         read (*,*, err=12) outmap
 13      write(*,'(a)', advance='no') 'pixel size (Ang):         '
         read (*, * ,err=13) pixelsize
 14      write(*,'(a)', advance='no') 'pdb filename:             '
         read (*,*, err=14) pdbfile
 15      write(*,'(a)', advance='no') '1=mask, 2=delete density: '
         read (*,*, err=15) behavior
      else
         call getarg(1, tempstr)
         border = real(atof_fstr(tempstr))

         call getarg(2, inmap)
         call getarg(3, outmap)

         call getarg(4, tempstr)
         pixelsize = real(atof_fstr(tempstr))

         call getarg(5, pdbfile)

         call getarg(6, tempstr)
         behavior = int(atof_fstr(tempstr))
      endif

C     --- Read input map header and data
      inhead%lunum = 7      ! unit for input image file
      rw(1) = 'r'
      rw(2) = achar(0)
      call pif_open(inmap, inhead%lunum, rw)
      call pif_read_gh(inhead)
      ncol = inhead%numcols
      nrow = inhead%numrows
      nsec = inhead%numsecs
      inhead%imgnum = 1
      call pif_read_dh(inhead)

      allocate(map3d(ncol,nrow,nsec))
      allocate(mask(ncol,nrow,nsec))

      do i=1,ncol
         do j=1,nrow
            do k=1,nsec
               mask(i,j,k) = 1
            enddo
         enddo
      enddo

      call pif_read_wmapi2(inhead, map3d, ncol, nrow, nsec)
      call pif_close(inhead%lunum)

c     --- Open output map / fill and write header
      outhead%lunum = 11
      rw(1) = 'w'
      rw(2) = '+'
      call pif_open(outmap, outhead%lunum, rw)
      outhead = inhead          ! Structure assignment

      outhead%binfactor  =   1
      outhead%nimg       =   1
      outhead%mode       =  20
      outhead%numcols     = ncol ! unbinned size
      outhead%numrows     = nrow ! unbinned size
      outhead%numsecs     = nsec ! unbinned size

      call pif_write_gh(outhead) 
      outhead%imgnum = 1
      call pif_write_dh(outhead)
      
C     --- Apply PDB coordinates to map
      open (unit=2, file=pdbfile, err=996)
 19   read(2, '(a30,3f8.3,a12)',end=900, err=996) a1,
     $     xx, yy, zz, a2
      
c     Convert coordinates to pixel numbers
      ix1 = int(xx/pixelsize - border + (ncol+1)/2)
      ix2 = int(xx/pixelsize + border + (ncol+1)/2 + 0.5)
      iy1 = int(yy/pixelsize - border + (nrow+1)/2)
      iy2 = int(yy/pixelsize + border + (nrow+1)/2 + 0.5)
      iz1 = int(zz/pixelsize - border + (nsec+1)/2)
      iz2 = int(zz/pixelsize + border + (nsec+1)/2 + 0.5)

      if(ix1 < 1)    ix1 = 1
      if(iy1 < 1)    iy1 = 1 
      if(iz1 < 1)    iz1 = 1
      if(ix2 > ncol) ix2 = ncol
      if(iy2 > nrow) iy2 = nrow
      if(iz2 > nsec) iz2 = nsec

      do i = ix1, ix2
         rx = (i - (xx/pixelsize+(ncol+1)/2))**2
         do j = iy1, iy2
            ry = (j - (yy/pixelsize+(nrow+1)/2))**2
            do k = iz1, iz2
               rz = (k - (zz/pixelsize+(nsec+1)/2))**2
               r = sqrt(rx + ry + rz)
               if (r <= border) then
                  mask(i,j,k) = 0
               endif
            enddo
         enddo
      enddo
      goto 19

 900  continue


      do i=1,ncol
         do j=1,nrow
            do k=1,nsec
               if(mask(i,j,k) == 1 .and. behavior==1) then
                  map3d(i,j,k) = 0.0
               else if (mask(i,j,k) == 0 .and. behavior==2) then
                  map3d(i,j,k) = 0.0
               endif
            enddo
         enddo
      enddo

C     --- Done with PDB - write map and wrap things up
      call pif_write_map_sections(outhead, map3d, outhead%mode,
     $     ncol, nrow, nsec, 0, nsec)

 996  write(*,*) 'pifmapmask exiting normally'
      close(unit=2)
      call pif_close(outhead%lunum)
      deallocate(map3d)

      end program pifmapdel
