// prjcompare.c
// 
// Rob Ashmore UCSD Oct 2005
// Modernized and integrated with baker software (R Sinkovits Oct 2010)

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "../include/common.h"
#include "../include/libPIF.h"

typedef struct __imgHead_t__ {
  int    header;
  int    image;
} imgHead_t;

pifHead_t pifHead;


int main(int argc, char **argv)
{
  char      infile1[128], infile2[128], outfile[128];
  int       zero=0, one=1;
  int       i, j, r, binCol, binRow, binSec;
  int       imgs, totImgs;
  int       idim1, idim2, fft_nfac;
  int       nrot, nannuli, jn, ir, ann_lo, ann_hi;
  int       n_power2;
  float     npts, tot, test;
  float     ccor, xcen, ycen, radius, del_ang, rad_min;
  float     *float_buffer1, *float_buffer2, *float_buffer3, *tmpF;
  float     *pfft1, *pfft2, *fft1, *fft2, *pimg1, *pimg2;
  float     *pft_img1, *pft_img2;
  fString_t fName, readWrite;
  forPifHead_t  thisHead1, thisHead2, thisHead3;

  if(argc < 4) {
    printf("\n");
    printf("USAGE: prjcompare infile1 infile2 outfile\n\n");
    printf("  Calculate the differences between two boxed image stacks\n\n");
    printf("  infile1: first image stack\n");
    printf("  infile2: second image stack\n");
    printf("  outfile: difference image stack (image1 - image2)\n");
    exit(0);
  }
  
  sprintf(infile1,"%s",argv[1]);
  sprintf(infile2,"%s",argv[2]);
  sprintf(outfile,"%s",argv[3]);
  
  fName.addr     = (char *)malloc(sizeof(char)*255);
  readWrite.addr = (char *)malloc(sizeof(char)*64);

  /* Read first file */
  sprintf(fName.addr,"%s",infile1);
  sprintf(readWrite.addr,"r");
  i= thisHead1.luNum = 4;
  pif_open(fName.addr, &i, readWrite.addr);

  pif_read_gh(&thisHead1);
  if(thisHead1.mode != 46) {
      printf("prjcompare for mode==46 files only.\n");
      exit(0);
  }

  if(thisHead1.binFactor==2) {
      binSec  = thisHead1.numSecs/2+1;  
      binRow  = thisHead1.numRows/2+1;
      binCol  = thisHead1.numCols/2+1;
      if((binSec%2)==0)
	binSec--;
      if((binRow%2)==0)
	binRow--;
      if((binCol%2)==0)
	binCol--;
  }
  else {
    binCol = thisHead1.numCols;
    binRow = thisHead1.numRows;
    binSec = thisHead1.numSecs;
  }

  float_buffer1 = (float *)malloc(sizeof(float)*binCol*binRow*binSec);

  /* Read PRJ file */
  sprintf(fName.addr,"%s",infile2);
  sprintf(readWrite.addr,"r");
  i = thisHead2.luNum = 5;
  pif_open(fName.addr, &i, readWrite.addr);

  pif_read_gh(&thisHead2);

  if(thisHead2.mode != 46) {
    printf("prjcompare for mode==46 files only.\n");
    exit(0);
  }

  if(thisHead2.binFactor==2) {
    binSec  = thisHead2.numSecs/2+1;  
    binRow  = thisHead2.numRows/2+1;
    binCol  = thisHead2.numCols/2+1;
    if((binSec%2)==0)
      binSec--;
    if((binRow%2)==0)
      binRow--;
    if((binCol%2)==0)
      binCol--;
  }
  else {
    binCol = thisHead2.numCols;
    binRow = thisHead2.numRows;
    binSec = thisHead2.numSecs;
  }

  float_buffer2 = (float *)malloc(sizeof(float)*binCol*binRow*binSec);

  totImgs = MIN(thisHead1.nimg,thisHead2.nimg);

  /***********************/
  /* Now subtracted file */
  /***********************/
  printf("\nSubtracted image file written: %s\n\n", outfile);
  pif_copy_head(&thisHead1,&thisHead3);
  float_buffer3 = (float *)malloc(sizeof(float)*binCol*binRow*totImgs);
  
  sprintf(fName.addr,"%s",outfile);
  sprintf(readWrite.addr,"w");
  i = thisHead3.luNum = 7;
  thisHead3.nimg = totImgs;
  pif_open(fName.addr, &i, readWrite.addr);
  
  pif_write_gh(&thisHead3); 
  
  printf("Results:\n\n");
  printf("Lower average PFT deviation indicates greater correlation\n\n");
  for(imgs=0; imgs < totImgs; imgs++) {
    thisHead1.imgNum = imgs+1;
    pif_read_dh(&thisHead1);
    
    thisHead1.datar4_min = thisHead1.datar4_max = 0.0;
    pif_read_boxi4(&thisHead1, float_buffer1);
    
    thisHead2.imgNum = imgs+1;
    pif_read_dh(&thisHead2);
    
    thisHead2.datar4_min = thisHead2.datar4_max = 0.0;
    pif_read_boxi4(&thisHead2, float_buffer2);
    
    // Write subtracted file ?
    tmpF = float_buffer3+(imgs*binCol*binRow);
    for(i=0; i<(binRow*binCol); i++)
      if(float_buffer2[i]<240)
	tmpF[i] = float_buffer1[i];
      else
	tmpF[i] = 0.0;
    
      /***********************/
      /* Now compare the two */
      /***********************/
      /* Init vars */
      xcen = (binCol+1)/2;
      ycen = (binRow+1)/2;
      rad_min = 1.0;
      //Ca   Why do we double the size of the fft here?????
      //! DOUBLES DEFAULT FFT SIZE FOR CC FUNCTION
      fft_nfac = 2;
      fft_setdim_def_same_(&binCol, &binRow, &idim1, &idim2, &fft_nfac);
      
      radius = binCol/2.0;
      nannuli = radius;
      ann_lo = 5;
      ann_hi = ((float)(binCol/2.0) * 0.8);
      
      /* ! need sufficient sampling at radius */
      test     = 2.0*radius*M_PI;
      n_power2 = 0;
      while(test > 1.0) {
	n_power2 ++;
	test     = test/2.0;
      }
      nrot     = pow((double)2.0,(double)n_power2);
      del_ang  = 360.0/nrot;
      
      fft1     = (float *)malloc(sizeof(float)*idim1*idim2*2);
      fft2     = (float *)malloc(sizeof(float)*idim1*idim2*2);
      pfft1    = (float *)malloc(sizeof(float)*idim1*idim2*2);
      pfft2    = (float *)malloc(sizeof(float)*idim1*idim2*2);
      pimg1    = (float *)malloc(sizeof(float)*nrot*nannuli);
      pimg2    = (float *)malloc(sizeof(float)*nrot*nannuli);
      pft_img1 = (float *)malloc(sizeof(float)*nrot*nannuli);
      pft_img2 = (float *)malloc(sizeof(float)*nrot*nannuli);
      
      r4_img_fft_(float_buffer1, &binCol, &binRow, 
		  fft1, &idim1, &idim2,
		  &thisHead1.img_bkg);
      
      r4_img_fft_(float_buffer2, &binCol, &binRow, 
		  fft2, &idim1, &idim2,
		  &thisHead2.img_bkg);
      
      map_polar_(float_buffer1, &binCol, &binRow, pimg1, 
		 &nrot, &nannuli, &xcen, &ycen, 
		 &rad_min, &radius, &del_ang);
      
      pmap_fft_(pimg1, pfft1, &nrot, &nannuli);
      
      map_polar_(float_buffer2, &binCol, &binRow, pimg2, 
		 &nrot, &nannuli, &xcen, &ycen, 
		 &rad_min, &radius, &del_ang);
      
      pmap_fft_(pimg2, pfft2, &nrot, &nannuli);
      
      //! CONVERT PFFT A & B PARTS TO AMPLITUDES
      for(i=0;i< nannuli; i++)
	for(j=0;j<(nrot/4);j++)
	  pft_img1[(j*nannuli)+i] = sqrt((pfft1[(2*(j*nannuli))+i+1]*
					  pfft1[(2*(j*nannuli))+i+1]) + 
					 (pfft1[(2*(j*nannuli))+i]*
					  pfft1[(2*(j*nannuli))+i]));
      
      for(i=0;i< nannuli; i++)
	for(j=0;j<(nrot/4);j++)
	  pft_img2[(j*nannuli)+i] = sqrt((pfft2[(2*(j*nannuli))+i+1]*
					  pfft2[(2*(j*nannuli))+i+1]) + 
					 (pfft2[(2*(j*nannuli))+i]*
					  pfft2[(2*(j*nannuli))+i]));
      
      npts = 0.0;
      tot  = 0.0;
      for(ir=ann_lo;ir<ann_hi;ir++) {
	if(nannuli < ir)
	  r = nannuli-1;
	else
	  r = ir;
	for(jn=0; jn<(nrot/4); jn++) {
	  j = jn;
	  npts = npts + 1.0;
	  tot  = tot + 
	    (pft_img1[(j*nannuli)+r]-pft_img2[(j*nannuli)+r]) *
	    (pft_img1[(j*nannuli)+r]-pft_img2[(j*nannuli)+r]);
	}
      }
      
      ccor = tot / npts;
      
      printf("Image[%d]  Average PFT deviation == [%f]\n",
	     imgs+1, sqrt((double)ccor));
      
      free(fft1);
      free(fft2);
      free(pfft1);
      free(pfft2);
      free(pimg1);
      free(pimg2);
      free(pft_img1);
      free(pft_img2);
  }
  
  /***************************/
  /* Write subtracted file ? */
  /***************************/
  calcMapStats(float_buffer3, 
	       &thisHead3.img_bkg, 
	       &thisHead3.datar4_min,
	       &thisHead3.datar4_max, 
	       &thisHead3.datar4_avg, 
	       &thisHead3.datar4_std,
	       &binCol, &binRow, &totImgs);
  determine_scaling(&thisHead3, float_buffer3,
		    &binCol, &binRow, &totImgs);
  pif_write_gh(&thisHead3);
  
  printf("Writing image ");
  fflush(stdout);
  for(imgs=0;imgs < totImgs;imgs++) {
    printf("[%d] .. ",imgs+1);
    fflush(stdout);
    
    tmpF = float_buffer3+(imgs*binCol*binRow);
    calcMapStats(tmpF, 
		 &thisHead3.img_bkg, 
		 &thisHead3.datar4_min,
		 &thisHead3.datar4_max, 
		 &thisHead3.datar4_avg, 
		 &thisHead3.datar4_std,
		 &binCol, &binRow, &binSec);
    
    thisHead3.imgNum = imgs+1;
    pif_write_dh(&thisHead3);
    
    pif_write_map_sections(&thisHead3, tmpF,
			   &(thisHead3.mode), &(thisHead3.numCols), 
			   &(thisHead3.numRows), &(thisHead3.numSecs), 
			   &zero, &one);
    
  }
  
  printf("Done.\n");
  fflush(stdout);
  
  pif_close(&thisHead1.luNum);
  pif_close(&thisHead2.luNum);
  pif_close(&thisHead3.luNum);
  return 0;
  
}
