#!/usr/bin/perl
use warnings;
use strict;

print << "EOF";

This program takes a list of xmgrace project files (.agr) for
individual FSC curves and combines them into a single file that
displays multiple curves.

Order of curves as entered on command line is preserved in merged file
and standard UNIX wildcards can be used to specify multiple files. The
.agr extension must be explicitly provided.

Note that this program is specific for xmgrace FSC curves generated by
auto3dem and cannot be used for general .agr files

EOF

print "Enter list of xmgrace project files to be merged:\n";
my $list = <>;
my @inagr = glob("$list");
print "\n";

print "Enter name of xmgrace project file containing merged data:\n";
my $outagr = <>;

print "\n";

if(@inagr <= 0) {
    print "No files were specified - exiting program\n";
    exit;
}
my $errflag = 0;
foreach my $file (@inagr) {
    unless (-r $file) {
	print "File $file is not readable\n";
	$errflag = 1;
    }
}
if($errflag) {
    print "Unable to read files - exiting programs\n";
    exit;
}

my @lines = ();
my $file_count = 0;
my $world_xmax = 0;

# Slurp in all of first project file and FSC data from subsequent
# files, trip iteration number from title, and look for maximum value
# of x-axis range

foreach my $file (@inagr) {
    open(FH, "$file");
    my $found_fsc = 0;
    $file_count++;

    while(<FH>) {
	if(/start\s+fsc/i) {
	    $found_fsc = 1;
	}
	if(/^&/) {
	    last;
	}
	if(/world\s+xmax/i) {
	    s/\s+$//;
	    my (undef, undef, undef, $x) = split(/\s+/);
	    if($x > $world_xmax) {
		$world_xmax = $x;
	    }
	}
	if(/\s+title\s+\".*\"/) {
	    my (undef, undef, $title) = split(/\s+/);
	    $title =~ s/\"//g;
	    $_ = "\@    title \"$title\"\n";
	}
	s/#\@ s1 legend/\@ s1 legend/;

	if($file_count == 1 || $found_fsc) {
	    s/s1/s${file_count}/i;
	    push(@lines, $_);
	}
    }
    close(FH);
}

# Write out new project file containing multiple FSC curves
open(OUT, ">$outagr");
foreach my $line (@lines) {
    if($line =~ /world\s+xmax/i) {
	print OUT "\@    world xmax $world_xmax\n";
    } else {
	print OUT $line;
    }
}
print OUT "&\n";
close(OUT);
