#!/usr/bin/perl

use strict;
use warnings;
use FindBin qw($Bin);
use lib "$Bin/modules";

use get_info;

# flesh_out generates a complete particle parameter file from the
# header lines of the original parameter file. The corresponding PIF
# file is first queried to determine the box radius and the number of
# images. A particle record is then written out for each image using
# dummy values for the orientations angles and correlation
# coefficients and the box center for the image origins. A default
# value of 1.0 is used for the magnification unless specified on the
# command line.
#
# Usage:
# flesh_out filename [mag]

my ($dat, $mag) = @ARGV;

unless(defined $dat) {
    print "Usage: flesh_out file [mag]\n";
    exit(0);
}

unless(defined $mag) {
    $mag = "1.0";
}

my $pif    = get_img_name($dat);
my $numimg = get_numimages($pif);
my $boxrad = get_boxrad($pif);

my $new_dat = "${dat}_new";
open(ND, ">$new_dat");
open(OD, "$dat");

my $line = <OD>;
print ND $line;
$line = <OD>;
print ND $line;

$boxrad = sprintf("%.1f", $boxrad);

for (my $i=1; $i<= $numimg; $i++) {
    $i = sprintf("%4d", $i);
    print ND "$i, 0.0, 0.0, 0.0, $boxrad, $boxrad, $mag, 0.0, 0.0, 0.0\n";
}

close(OD);
close(ND);

rename $new_dat, $dat;
