#!/usr/bin/perl
use strict;
use warnings;
use FindBin qw($Bin);
use lib "$Bin/modules";

# This script flips the handedness of the particle image parameter
# files by applying the following transformation to the orientations
# 
# (theta,phi,omega) --> (theta,-phi,omeaga+180)
#
# Converted files are written to a new directory named FLIPPED.
#
# Program can be called in two ways
#
# Option 1 - argument list contains files to be converted
# handflip.pl set1.dat_001 set2.dat_001 set3.dat_001
# handflip.pl set?.dat_001
# handflip.pl *001
#
# Option 2 - single argument, a file containing a list of the files to
# be converted, one per line.
# handflip.pl list

my (@infile) = @ARGV;

my $dir = "FLIPPED";
mkdir $dir;

# Test for option 2 - file contains listing of files to convert
@infile = get_list(@infile);

# Convert files
foreach my $infile (@infile) {
    my $outfile = "$dir/$infile";
    write_flipped_file($infile, $outfile);
}

# --------------------------------------------------------------------

sub write_flipped_file {

    # Print out particle parameter file in which orientations have
    # been modified to get opposite handedness

    my ($infile, $outfile) = @_;
    open(IN, $infile);
    open(OUT, ">$outfile");

    my $header_flag = 0;

    while (<IN>) {
	
	# Print out the header lines and skip to next record
	if(/pif/i) {
	    $header_flag = 1;
	    print OUT $_;
	    next;
	}
	if($header_flag) {
	    $header_flag = 0;
	    print OUT $_;
	    next;
	}

	# Get rid of all whitespace
	s/\s//g;
	
	# Parse record
	# @p = (id, theta, phi, omega, ...)
	my @p = split(/,/);
	
	# If this is not a particle record, print and go to next record
	unless(@p >= 8) {
	    print OUT "$_\n";
	    next;
	}
	
###	# Transform phi: phi --> -phi
###	if($p[2] != 0) {
###	    $p[2] = -$p[2];
###	}
	
	# Transform omega: omega --> omega + 180
	$p[3] += 180;
	if($p[3] > 360.0) {
	    $p[3] -= 360;
	}
	
	if(@p >= 10) {
	    printf OUT "%7d,%8.3f,%8.3f,%8.3f,%8.3f,", @p[0,1,2,3,4];
	    printf OUT "%7.3f,%6.3f,%6.3f,%6.3f,%6.3f\n", @p[5,6,7,8,9];
	} else {
	    printf OUT "%7d,%8.3f,%8.3f,%8.3f,%8.3f,", @p[0,1,2,3,4];
	    printf OUT "%7.3f,%6.3f,%10.3f, , ,\n", @p[5,6,7];
	}
    }
    
    close(IN);
    close(OUT);
    return;
}

sub get_list {

    # Test array @infile to see if it consists of a single argument
    # which is the name of a file listing files to be converted

    my (@infile) = @_;

    if(@infile == 1) {
	my $count_max = 0;
	my @list = ();
	
	open(IN, $infile[0]);
	while(<IN>) {
	    s/\s//g; # Get rid of all whitespace
	    my @p = split(/,/);
	    if(@p > $count_max) {
		$count_max = @p;
	    }
	    if(@p == 1 && $p[0] =~ /\w/) {
		push(@list, $p[0]);
	    }
	}
	close(IN);
	
	if($count_max == 1) {
	    @infile = @list;
	}
    }
    return @infile;
}
