#!/usr/bin/perl

use strict;
use warnings;

my ($infile, $boxrad) = @ARGV;
unless(scalar(@ARGV) == 2) {
    print "\n";
    print "    Usage:   leginon_to_bcrd infile boxrad\n";
    print "    infile:  Leginon format box coordinates file\n";
    print "    boxrad:  Box radius\n";
    exit(0);
}

unless(-r $infile) {
    print "Unable to open input file $infile\n";
    exit(0);
}

my %x;
my %y;
my %count;
open(FH, "$infile");

# Loop over records in Leginon particle file
while(<FH>) {
    if($. == 1) {
	next;
    }

    s/^\s+//;
    s/\s+$//;

    my ($x, $y, $name) = split;

    if (defined $count{$name}) {
	$count{$name}++;
    } else {
	$count{$name} = 1;
    }

    my $i = $count{$name} - 1;
    $x{$name}[$i] = $x;
    $y{$name}[$i] = $y;
}

close(FH);

# Print out BCRD files
foreach my $key (keys %count) {
    my $bcrd = "${key}.bcrd";
    open(BCRD, ">$bcrd");
    print BCRD "BCRD V2.0\n";
    print BCRD "0\n";
    print BCRD "$count{$key}\n";
    print BCRD "$boxrad\n";
    print BCRD "$boxrad\n";
    for (my $i=0; $i<$count{$key}; $i++) {
	print BCRD "$x{$key}[$i] $y{$key}[$i]\n";
    }
    close(BCRD);
}
