use strict;
use warnings;
use lib do { __FILE__ =~ m|^(.*?)[^/]*$|; "$1"; };

sub make_grace_fscfile {
    my ($xref, $yref, $fsctitle, $iter, $fscagr) = @_;

    my $count = @{$xref};
    my $xmax = $$xref[$count-1];
    $xmax = (int($xmax*100) + 1)/100.0;

    open(AGR, ">$fscagr");
    print AGR << "EOF";

# Grace project file
# 
\@version 50005
\@page size 640, 480
\@page scroll 5%
\@page inout 5%
\@link page off
\@map font 0 to "Helvetica", "Helvetica"
\@reference date 0
\@date wrap off
\@date wrap year 1950
\@g0 stacked false
\@g0 hidden false
\@g0 type XY
\@g0 bar hgap 0.000000
\@g0 fixedpoint off
\@g0 fixedpoint type 0
\@g0 fixedpoint xy 0.000000, 0.000000
\@g0 fixedpoint format general general
\@g0 fixedpoint prec 6, 6
\@with g0
\@    world xmin 0
\@    world xmax $xmax
\@    world ymin 0
\@    world ymax 1.05
\@    stack world 0, 0, 0, 0
\@    view xmin 0.150000
\@    view xmax 1.150000
\@    view ymin 0.150000
\@    view ymax 0.850000
\@    title font 0
\@    title size 1.250000
\@    title color 1
\@    subtitle ""
\@    subtitle font 0
\@    subtitle size 1.000000
\@    subtitle color 1
\@    xaxes scale Normal
\@    xaxes invert off
\@    yaxes scale Normal
\@    yaxes invert off
\@    xaxis  on
\@    xaxis  type zero false
\@    xaxis  offset 0.000000, 0.000000
\@    xaxis  bar on
\@    xaxis  bar color 1
\@    xaxis  bar linestyle 1
\@    xaxis  bar linewidth 1.0
\@    xaxis  label "Spatial frequency (1/\\#{00C5})"
\@    xaxis  label layout para
\@    xaxis  label place auto
\@    xaxis  label char size 1.250000
\@    xaxis  label font 0
\@    xaxis  label color 1
\@    xaxis  label place auto
\@    xaxis  tick on
\@    xaxis  tick default 6
\@    xaxis  tick place rounded true
\@    xaxis  tick major 0.05
\@    xaxis  tick major size 1
\@    xaxis  tick major color 1
\@    xaxis  tick major linewidth 1.0
\@    xaxis  tick major linestyle 1.000000
\@    xaxis  tick major grid off
\@    xaxis  tick minor ticks 4
\@    xaxis  tick minor size 0.5
\@    xaxis  tick minor color 1
\@    xaxis  tick minor linewidth 1.0
\@    xaxis  tick minor linestyle 1.000000
\@    xaxis  tick minor grid off
\@    xaxis  tick out
\@    xaxis  tick place normal
\@    xaxis  tick type auto
\@    xaxis  ticklabel on
\@    xaxis  ticklabel prec 5
\@    xaxis  ticklabel format general
\@    xaxis  ticklabel append ""
\@    xaxis  ticklabel prepend ""
\@    xaxis  ticklabel angle 0
\@    xaxis  ticklabel skip 0
\@    xaxis  ticklabel stagger 0
\@    xaxis  ticklabel place normal
\@    xaxis  ticklabel offset 0.000000, 0.000000
\@    xaxis  ticklabel sign normal
\@    xaxis  ticklabel start type auto
\@    xaxis  ticklabel start 0.000000
\@    xaxis  ticklabel stop type auto
\@    xaxis  ticklabel stop 0.000000
\@    xaxis  ticklabel char size 1.000000
\@    xaxis  ticklabel font 0
\@    xaxis  ticklabel color 1
\@    xaxis  ticklabel type auto
\@    yaxis  on
\@    yaxis  type zero false
\@    yaxis  offset 0.000000, 0.000000
\@    yaxis  bar on
\@    yaxis  bar color 1
\@    yaxis  bar linestyle 1
\@    yaxis  bar linewidth 1.0
\@    yaxis  label "FSC"
\@    yaxis  label layout para
\@    yaxis  label place spec
\@    yaxis  label place 0.000000, 0.101000
\@    yaxis  label char size 1.250000
\@    yaxis  label font 0
\@    yaxis  label color 1
\@    yaxis  label place normal
\@    yaxis  tick on
\@    yaxis  tick default 6
\@    yaxis  tick place rounded true
\@    yaxis  tick major size 1
\@    yaxis  tick major color 1
\@    yaxis  tick major linewidth 1.0
\@    yaxis  tick major linestyle 1.000000
\@    yaxis  tick major grid off
\@    yaxis  tick minor size 1
\@    yaxis  tick minor color 1
\@    yaxis  tick minor linewidth 1.0
\@    yaxis  tick minor linestyle 1.000000
\@    yaxis  tick minor grid off
\@    yaxis  tick out
\@    yaxis  tick place normal
\@    yaxis  tick type auto
\@    yaxis  ticklabel on
\@    yaxis  ticklabel prec 5
\@    yaxis  ticklabel format general
\@    yaxis  ticklabel append ""
\@    yaxis  ticklabel prepend ""
\@    yaxis  ticklabel angle 0
\@    yaxis  ticklabel skip 0
\@    yaxis  ticklabel stagger 0
\@    yaxis  ticklabel place normal
\@    yaxis  ticklabel offset 0.000000, 0.000000
\@    yaxis  ticklabel sign normal
\@    yaxis  ticklabel start type auto
\@    yaxis  ticklabel start 0.000000
\@    yaxis  ticklabel stop type auto
\@    yaxis  ticklabel stop 0.000000
\@    yaxis  ticklabel char size 1.000000
\@    yaxis  ticklabel font 0
\@    yaxis  ticklabel color 1
\@    yaxis  ticklabel type auto
\@    altxaxis  off
\@    altyaxis  off
\@    legend on
\@    legend loctype view
\@    legend x1 0.17
\@    legend y1 0.40
\@    legend box color 1
\@    legend box pattern 1
\@    legend box linewidth 1.0
\@    legend box linestyle 1
\@    legend box fill color 0
\@    legend box fill pattern 1
\@    legend font 0
\@    legend char size 1.000000
\@    legend color 1
\@    legend length 4
\@    legend vgap 1
\@    legend hgap 1
\@    legend invert false
\@    frame type 0
\@    frame linestyle 1
\@    frame linewidth 1.0
\@    frame color 1
\@    frame pattern 1
\@    frame background color 0
\@    frame background pattern 0
\@    s0 line linestyle 2
\@    s0 line color 1
\@    s1 line linestyle 1
\@    s1 line color 1
@    s2 line linestyle 1
@    s2 line color 2
@    s3 line linestyle 1
@    s3 line color 3
@    s4 line linestyle 1
@    s4 line color 4
@    s5 line linestyle 1
@    s5 line color 5
@    s6 line linestyle 1
@    s6 line color 6
@    s7 line linestyle 1
@    s7 line color 7
@    s8 line linestyle 1
@    s8 line color 8
@    s9 line linestyle 1
@    s9 line color 9
@    s10 line linestyle 1
@    s10 line color 10
@    title "$fsctitle"

\@ target G0.S0
\@ type XY
                0.0  0.5
                1.0  0.5

EOF
    print AGR "# Start FSC data\n";
    print AGR "#\@ s1 legend \"iteration $iter\"\n";
    print AGR "\@ target G0.S1\n";
    print AGR "\@ type XY\n";
    print AGR "\n";

    for (my $i=0; $i<$count; $i++) {
	print AGR "$$xref[$i] $$yref[$i]\n";
    }
    
    print AGR "\&\n";
    
    close(AGR);
}
1;
