use strict;
use warnings;
use lib do { __FILE__ =~ m|^(.*?)[^/]*$|; "$1"; };

sub sendmail {

    my ($msgfile, $subj, $recipient, @attach) = @_;

    unless(defined $msgfile && defined $subj && defined $recipient) {
	print "Valid message file, subject, and recipient not provided\n";
	print "email notification not sent\n";
	return;
    }

    my $mutt = `which mutt`;
    chomp($mutt);
    if($mutt =~ /command not found/i) {
	print "Cannot find mutt mail program\n";
	print "email notification not sent\n";
	return;
    }

    # Change commas and semicolons to spaces in list of email addresses. 
    # E.g. change "a@b,c@d;e@f" to "a@b c@d e@f"
    $recipient =~ s/,/ /g;
    $recipient =~ s/;/ /g;

    # Quick test for valid email address(es). Won't catch all badly formed
    # addresses, but will at least get the low-hanging fruit
    my @good_addresses = ();
    foreach my $address (split(/\s+/, $recipient)) {
	if(wellformed_email($address)) {
	    push(@good_addresses, $address);
	} else {
	    print "Bad address found: $address\n";
	}
    }

    if(@good_addresses > 0) {
	$recipient = join(' ', @good_addresses);
	$recipient =~ s/\@/\\@/g;
    } else {
	print "No valid email addresses provided\n";
	print "email notification not sent\n";
	return;
    }

    my $cmd = "$mutt -s '$subj' ";
    foreach my $attach (@attach) {
	$cmd .= "-a $attach ";
    }
    $cmd .= "$recipient < $msgfile";

    system($cmd);
    return;
}

sub wellformed_email {

    # This routine returns true if the email address is properly
    # formed. As currently written, it is strongly biased towards
    # false positives rather than false negatives and is used
    # essentially as a sanity check on an address. Note that although
    # an address may be properly formed, it does not mean that it
    # necessarily mean that it is a valid address.
    #
    # A small number of valid, but unusual addresses will be
    # rejected. These include addresses with quotes, embedded spaces,
    # and square brackets. For example "a b c"@xyz.com or
    # abc@[120.121.122.123]


    my ($address) = @_;
    my $valid = 1; # Assume valid by default

    # Step 1 - make sure address is of form string1@string2
    unless($address =~ /^(.*)\@(.*)$/) {
	return(0);
    }

    # Step 2 - make sure that each string contains only allowed characters
    my ($string1, $string2) = ($1, $2);
    unless($string1 =~ /^[a-zA-Z0-9_\.\-]+$/ &&
	   $string2 =~ /^[a-zA-Z0-9_\.\-]+$/) {
	return(0);
    }

    # Step 3 - neither string should start or end with '.'
    if($string1 =~ /^\./ || $string1 =~ /\.$/ ||
       $string2 =~ /^\./ || $string2 =~ /\.$/) {
	return(0);
    }

    # If we made it this far, email address is valid
    return(1);
}
1;
