use strict;
use warnings;
use lib do { __FILE__ =~ m|^(.*?)[^/]*$|; "$1"; };

sub write_header {

    # Purpose: write out header information
    #
    # Input parameters:
    #   $ncpu     - number of CPUs
    #   $nodefile - file containing list of nodes
    #   auto_ref  - reference to hash of control parameters
    #
    # Return values:
    #   None

    my ($ncpu, $nodefile, $auto_ref, $p3dr_ref, $po2r_ref, 
	$pcut_ref, $psf_ref, $ppft_ref) = @_;
    
    my $date = localtime();
    print OUT "Job started at $date\n";
    print OUT "Number of CPUs: $ncpu\n";

    if(defined $nodefile && $nodefile !~ /^none$/i) {
	print OUT "nodefile: $nodefile\n";
	open(NF, $nodefile);
	while(<NF>) {
	    print OUT "   nodefile line $.:\t$_";
	}
	close(NF);
	print OUT "\n";
    }

    print OUT "Auto3dem control parameters\n";
    foreach my $key (sort keys %{$auto_ref}) {
	print OUT "   $key\t$$auto_ref{$key}\n";
    }
    print OUT "\n";

    print OUT "Executables\n";
    print OUT "   PO2R = $$po2r_ref{bin}\n";
    print OUT "   P3DR = $$p3dr_ref{bin}\n";
    print OUT "   PSF  = $$psf_ref{bin}\n";
    print OUT "   PCUT = $$pcut_ref{bin}\n";
    print OUT "   PPFT = $$ppft_ref{bin}\n";
    print OUT "\n";

}
1;
