#!/usr/bin/perl
use strict;
use warnings;

unless (@ARGV == 2) {
    print "Simple cleanup of PDB file. Retain only records that\n";
    print "begin with ATOM, REMARK, and TER. Also rename all chain\n";
    print "designators to 'A'. This script may be useful for getting\n";
    print "PDB file into format that can be used by ccp4i\n\n";
    print "Usage: pdbclean.pl old.pdb new.pdb\n\n";
    print "old.pdb    : original PDB file\n";
    print "new.pdb    : transformed PDB file\n";
    print "\n";
    exit(0);
}

unless(-r $ARGV[0]) {
    print "Unable to open input file $ARGV[0]\n";
    exit(0);
}

open(OLD, "$ARGV[0]");
open(NEW, ">$ARGV[1]");

while(<OLD>) {
    # Keep only ATOM, REMARK, and TER
    unless(/^\s*ATOM/i || /^\s*TER/i || /^\s*REMARK/i) {
	next;
    }

    # Change all chains to "A"
    s/([a-zA-Z]{3}\s+)[a-zA-Z0-9](\s+\d)/$1A$2/;

    print NEW;
}

close(OLD);
close(NEW);
