#!/usr/bin/perl
use strict;
use warnings;

unless (@ARGV == 3) {
    print "Combine two PDB files. It is your responsibility\n";
    print "to make sure that the coordinates are correctly\n";
    print "specified relative to each other.\n\n";
    print "Note that all chains will be labeled as 'A' to avoid\n";
    print "problems with reused chain designators. You may need to\n";
    print "manually edit the PDB file if you want to distinguish chains\n\n";
    print "Usage: pdbcomb.pl 60mer.pdb fab.pdb new.pdb\n";
    print "\n";
    print "60mer.pdb : capsid PDB file\n";
    print "fab.pdb   : Fab PDB with coor relative to capsid\n";
    print "new.pdb   : Combined PDB file\n";
    print "\n";
    exit(0);
}

open(CAP, "$ARGV[0]");
open(FAB, "$ARGV[1]");
open(NEW, ">$ARGV[2]");

unless(-r $ARGV[0]) {
    print "Unable to open input file $ARGV[0]\n";
    exit(0);
}

unless(-r $ARGV[1]) {
    print "Unable to open input file $ARGV[0]\n";
    exit(0);
}


while(<CAP>) {
    print NEW;
}

while(<FAB>) {
    # Keep only ATOM and TER
    unless(/^\s*ATOM/i || /^\s*TER/i) {
	next;
    }

    # Change all chains to "A"
    s/([a-zA-Z]{3}\s+)[a-zA-Z0-9](\s+\d)/$1A$2/;

    print NEW;
}

close(CAP);
close(FAB);
close(NEW);
