#!/usr/bin/perl
use strict;
use warnings;
use FindBin qw($Bin);
use lib "$Bin/modules";

# This script flips the handedness of the particle image parameter
# files by applying the following transformation to the orientations
# 
# (theta,phi,omega) --> (theta,-phi,omeaga+180)
#
# Converted files are written to a new directory named FLIPPED.
#
# Program can be called in two ways
#
# Option 1 - argument list contains files to be converted
# tweak.pl set1.dat_001 set2.dat_001 set3.dat_001
# tweak.pl set?.dat_001
# tweak.pl *001
#
# Option 2 - single argument, a file containing a list of the files to
# be converted, one per line.
# tweak.pl list

my (@infile) = @ARGV;

my $dir = "TWEAKED";
mkdir $dir;

my $randmag = 1.0;
srand(1);

# Test for option 2 - file contains listing of files to convert
@infile = get_list(@infile);

# Convert files
foreach my $infile (@infile) {
    my $outfile = "$dir/$infile";
    write_tweaked_file($infile, $outfile, $randmag);
}

# --------------------------------------------------------------------

sub write_tweaked_file {

    # Print out particle parameter file in which orientations have
    # been perturbed

    my ($infile, $outfile, $randmag) = @_;
    open(IN, $infile);
    open(OUT, ">$outfile");

    my $header_flag = 0;

    while (<IN>) {
	
	# Print out the header lines and skip to next record
	if(/pif/i) {
	    $header_flag = 1;
	    print OUT $_;
	    next;
	}
	if($header_flag) {
	    $header_flag = 0;
	    print OUT $_;
	    next;
	}

	# Get rid of all whitespace
	s/\s//g;
	
	# Parse record
	# @p = (id, theta, phi, omega, ...)
	my @p = split(/,/);
	$p[1] += $randmag - 2*rand($randmag);
	$p[2] += $randmag - 2*rand($randmag);
	$p[3] += $randmag - 2*rand($randmag);

	$p[1] = sprintf "%8.3f", $p[1];
	$p[2] = sprintf "%8.3f", $p[2];
	$p[3] = sprintf "%8.3f", $p[3];
	
	my $str = join ', ', @p;
	print OUT "$str\n";
    }
    
    close(IN);
    close(OUT);
    return;
}

sub get_list {

    # Test array @infile to see if it consists of a single argument
    # which is the name of a file listing files to be converted

    my (@infile) = @_;

    if(@infile == 1) {
	my $count_max = 0;
	my @list = ();
	
	open(IN, $infile[0]);
	while(<IN>) {
	    s/\s//g; # Get rid of all whitespace
	    my @p = split(/,/);
	    if(@p > $count_max) {
		$count_max = @p;
	    }
	    if(@p == 1 && $p[0] =~ /\w/) {
		push(@list, $p[0]);
	    }
	}
	close(IN);
	
	if($count_max == 1) {
	    @infile = @list;
	}
    }
    return @infile;
}
