#!/usr/bin/perl

use strict;
use warnings;

my @required_modules = ('Cwd', 'File::Copy', 'Math::Trig');
my @optional_modules = ('GD::Graph::lines');

printf "*** Checking perl installation\n\n";
printf "   %s %vd\n", "Running perl version", $^V;

my $have_all = 1;
foreach my $module (@required_modules) {
    eval "require $module";
    if($@) {
	print "   Perl installation missing required module $module\n";
	$have_all = 0;
    }
}
if($have_all) {
    print "   Have all required Perl modules\n";
}

$have_all = 1;
foreach my $module (@optional_modules) {
    eval "require $module";
    if($@) {
	print "   Perl installation missing optional module $module\n";
	$have_all = 0;
    }
}
if($have_all) {
    print "   Have all optional Perl modules\n";
}
print "\n";


print "*** Checking MPI installation\n\n";

my %result;
my @mpi_components = ('mpif90', 'mpicc', 'mpirun');
foreach my $component (@mpi_components) {
    print "   Looking for $component\n";
    $result{$component} = `which $component`;
    chomp($result{$component});
    if($result{$component} =~ /^\//) {
	print "   $component found at $result{$component}\n";
    } else {
	print "   $component not found\n";
    }
    print "\n";
}

print "*** Checking compilers\n\n";

if($result{mpif90} =~ /^\//) {
    open(MPIF90, $result{mpif90});
    while(<MPIF90>) {
	if(/^F90=/ || /^F90BASE=/) {
	    chomp;
	    my (undef, $compiler) = split(/=/);
	    $compiler =~ s/\"//g;
	    print "   mpif90 f90 compiler = $compiler\n";
	    my $result = `which $compiler`;
	    chomp($result);
	    if($result =~ /^\//) {
		print "   $compiler found at $result\n";
	    } else {
		print "   $compiler not found\n";
	    }
	    last;
	}
    }
    close(MPIF90);
}

print "\n";

if($result{mpicc} =~ /^\//) {
    open(MPICC, $result{mpicc});
    while(<MPICC>) {
	if(/^CC=/ || /^CCBASE=/) {
	    chomp;
	    my (undef, $compiler) = split(/=/);
	    $compiler =~ s/\"//g;
	    print "   mpicc C compiler = $compiler\n";
	    my $result = `which $compiler`;
	    chomp($result);
	    if($result =~ /^\//) {
		print "   $compiler found at $result\n";
	    } else {
		print "   $compiler not found\n";
	    }
	    exit;
	}
    }
    close(MPICC);
}
