#include "DM3io.h"

int   dm3type2bytes( int t) {
    int nb = 0;

    switch ( t ) {
        case 2: nb = 2; break;  // SHORT
        case 3: nb = 4; break;  // LONG
        case 4: nb = 2; break;  // USHORT
        case 5: nb = 4; break;  // ULONG
        case 6: nb = 4; break;  // FLOAT
        case 7: nb = 8; break;  // DOUBLE
        case 8: nb = 1; break;  // BOOLEAN
        case 9: nb = 1; break;  // CHAR
        case 10: nb = 1; break; // OCTET 
        default:
            fprintf(stderr, "Error(dm3type2bytes): Can not interpret data type %d!\n", t);
    }
	
    return(nb);
}

double   struct2double(char* c, int* data_type, int i)
{
    double  val = 0;
    char *j = c;
    int nb = dm3type2bytes(data_type[i]);
    j += nb*i;

    switch ( data_type[i] ) {
        case 2: val = *((short *) j); break;
        case 3: val = *((int *) j); break;
        case 4: val = *((unsigned short *) j); break;
        case 5: val = *((unsigned int *) j); break;
        case 6: val = *((float *) j); break;
        case 7: val = *((double *) j); break;
        case 8: val = *j; break;
        case 9: val = *j; break;
        case 10: val = *j; break;
        default:
            fprintf(stderr, "Error(struct2double): Can not interpret type %d!\n", data_type[i]);
    }
	
    return(val);
}

double   value2double(char* c, int data_type)
{
    double  val = 0;

    switch ( data_type ) {
        case 2: val = *((short *) c); break;
        case 3: val = *((int *) c); break;
        case 4: val = *((unsigned short *) c); break;
        case 5: val = *((unsigned int *) c); break;
        case 6: val = *((float *) c); break;
        case 7: val = *((double *) c); break;
        case 8: val = *c; break;
        case 9: val = *c; break;
        case 10: val = *c; break;
        default:
            fprintf(stderr, "Error(value2double): Can not interpret type %d!\n", data_type);
    }
	
    return(val);
}

double   value2long(char* c, int data_type)
{
    long  val = 0;

    switch ( data_type ) {
        case 2: val = *((short *) c); break;
        case 3: val = *((int *) c); break;
        case 4: val = *((unsigned short *) c); break;
        case 5: val = *((unsigned int *) c); break;
        case 8: val = *c; break;
        case 9: val = *c; break;
        case 10: val = *c; break;
        case 6:
        case 7:
            fprintf(stderr, "Error(value2long): data type %d can not be converted to long!\n", data_type); break;
        default:
            fprintf(stderr, "Error(value2long): Can not interpret type %d!\n", data_type);
	}
	
	return(val);
}

void  swap(char* v, unsigned long n) {

    char  t;
    int   i;
    for ( i=0; i<n/2; i++ ) {
        t = v[i];
        v[i] = v[n-1-i];
        v[n-1-i] = t;
    }
}

int  StructType_read(int doswap, DM3Struct * st, FILE* fid){

  int intbuf;
  int i;

  // read name length  
  if (fread((char *)&intbuf,4,1,fid) < 1) return(-1);
  if (doswap==1) swap((char *) &intbuf, 4);
//  printf("length of name: %d \n",intbuf);
  st->name_len = intbuf;
  // read number of fields  
  if (fread((char *)&intbuf,4,1,fid) < 1) return(-1);
  if (doswap==1) swap((char *) &intbuf, 4);
//  printf("number of fields: %d \n",intbuf);
  st->nfields = intbuf;
  st->fname_len = (int *) malloc(sizeof(int)*st->nfields);
  st->fdata_type = (int *) malloc(sizeof(int)*st->nfields);
  // read name_len and data type for all fields
  for (i=0; i<st->nfields; i++) {
      if (fread((char *)&intbuf,4,1,fid) < 1) return(-1);
      if (doswap==1) swap((char *) &intbuf, 4);
//      printf("length of field name: %d \n",intbuf);
      st->fname_len[i] = intbuf;
      if (fread((char *)&intbuf,4,1,fid) < 1) return(-1);
      if (doswap==1) swap((char *) &intbuf, 4);
//      printf("field data type: %d \n",intbuf);
      st->fdata_type[i] = intbuf;
  }
  // calculate size of data
  int datasize = 0;
  for ( i=0; i<st->nfields; i++) {
      datasize += dm3type2bytes(st->fdata_type[i]);
  }
  st->datasize = datasize;
  
  return(0);
}

int  StructType_data_read(int dataswap, DM3Struct * st, char * data, FILE* fid){

  char c8buf[8];
  int i;

  char *j = data;
  int nb;
  for ( i=0; i<st->nfields; i++) {
      nb = dm3type2bytes(st->fdata_type[i]);
      if (fread(c8buf,nb,1,fid) < 1) return(-1);
      if (dataswap==1 && nb>1) swap(c8buf, nb);
//      printf("s%d: %lf\n",i,value2double(c8buf,st->fdata_type[i]));
      memcpy(j, c8buf, nb);
      j += nb;
  }

  return(0);
}

int  ArrayType_read(int doswap, DM3Array * at, FILE* fid){

  int intbuf;

  // read data type
  if (fread((char *)&intbuf,4,1,fid) < 1) return(-1);
  if (doswap==1) swap((char *) &intbuf, 4);
//  printf("data type: %d \n",intbuf);
  at->data_type = intbuf;
  if (at->data_type >10){
      if (at->data_type == 15) { //struct element
          at->strct = (DM3Struct *) malloc(sizeof(DM3Struct));
          StructType_read(doswap, at->strct, fid);
          at->elem_size = at->strct->datasize;
      } else {
          fprintf(stderr,"ERROR (ArrayType_read): Can not yet read array with data type %d!\n",at->data_type);
          exit(-1);
      }
  } else {
      at->elem_size = dm3type2bytes(at->data_type);
  }
  // read number of elements  
  if (fread((char *)&intbuf,4,1,fid) < 1) return(-1);
  if (doswap==1) swap((char *) &intbuf, 4);
//  printf("number of elements: %d \n",intbuf);
  at->nelems = intbuf;
  at->datasize = at->nelems*at->elem_size;
  // get offset for direct access to the array data
  at->offset = ftell(fid);
  

  return(0);
}

int  ArrayType_data_read(int store_array, int dataswap, DM3Array * at, char * data, FILE* fid){

  char c8buf[8];
  int i, nb;

  if ( at->elem_size > 8){
      fprintf(stderr,"ERROR (ArrayType_data_read): Can not yet read data from complex array (%d>8)!\n",at->elem_size);
      exit(-1);
  }

  if (store_array) {
//  if (at->nelems > 20) printf("Number of elements: %d\n",at->nelems);
      char *j = data;
      nb = at->elem_size;
      for ( i=0; i<at->nelems; i++) {
          if (fread(c8buf,nb,1,fid) < 1) return(-1);
          if (dataswap==1 && nb>1) swap(c8buf, nb);
//          if (at->nelems<21) printf("a%d: %lf\n",i,value2double(c8buf,at->data_type));
          memcpy(j, c8buf, nb);
          j += nb;
      }
  } else {  //skip the data
      fseek(fid, (long) at->datasize, SEEK_CUR );
  }

  return(0);
}

int  TagType_read(int store_array, int doswap, int dataswap, DM3TagType* tt, FILE* fid){

  char c8buf[8];
  int intbuf, nb, datasize;

  // check delimiter
  if (fread(c8buf,4,1,fid) < 1) return(-1);
//  printf("delimiter: %s \n",c8buf);
  if (strncmp(c8buf, "%%%%",4)){
      fprintf(stderr,"ERROR (TagType_read): Tag Type delimiter is not correct (%s instead of %%%%%%%%)!\n",c8buf);
      exit(-1);
  }
  // read definition length  
  if (fread((char *)&intbuf,4,1,fid) < 1) return(-1);
  if (doswap==1) swap((char *) &intbuf, 4);
//  printf("length of definition: %d \n",intbuf);
  tt->def_len = intbuf;
  // read data type  
  if (fread((char *)&intbuf,4,1,fid) < 1) return(-1);
  if (doswap==1) swap((char *) &intbuf, 4);
  tt->data_type = intbuf;
  if (tt->data_type <= 10) { //simple data type
//      printf("simple data type: %d \n",tt->data_type);
      nb = dm3type2bytes(tt->data_type);
      tt->data = (char *) malloc(nb);
      if (fread(c8buf,nb,1,fid) < 1) return(-1);
      if (dataswap==1 && nb>1) swap(c8buf, nb);
//      printf("value: %lf\n",value2double(c8buf,tt->data_type));
      memcpy(tt->data, c8buf, nb);
  } else if (tt->data_type == 15) { //struct data type
//      printf("struct data type: %d \n",tt->data_type);
      tt->type = (char *) malloc(sizeof(DM3Struct));
      StructType_read(doswap, (DM3Struct *) tt->type, fid);
      datasize = ((DM3Struct *) tt->type)->datasize;
      tt->data = (char *) malloc(datasize);
      StructType_data_read(dataswap, (DM3Struct *) tt->type, tt->data, fid);
  } else if (tt->data_type == 18) { //string data type
      fprintf(stderr,"ERROR (TagType_read): String data type (%d) not yet supported!\n",tt->data_type);
      exit(-1);
  } else if (tt->data_type == 20) { //array data type
//      printf("array data type: %d \n",tt->data_type);
      tt->type = (char *) malloc(sizeof(DM3Array));
      ArrayType_read(doswap, (DM3Array *) tt->type, fid);
      datasize = ((DM3Array *) tt->type)->datasize;
      if (store_array) {
          tt->data = (char *) malloc(datasize);
          ((DM3Array *) tt->type)->offset = 0;
      }
//      printf("%ld\n",((DM3Array *) tt->type)->offset);
      ArrayType_data_read(store_array, dataswap, (DM3Array *) tt->type, tt->data, fid);
  }
  return(0);
}

int  TagEntry_read(int store_array, int doswap, int dataswap, DM3TagEntry* te, FILE* fid){

  char cbuf;
  short unsigned int shortbuf;

  if (fread((char *)&cbuf,1,1,fid) < 1) return(-1);
  te->isdata = (int) cbuf;
//  printf("is data: %d \n",te->isdata);
  // read length of label  
  if (fread((char *)&shortbuf,2,1,fid) < 1) return(-1);
  if (doswap==1) swap((char *) &shortbuf, 2);
//  printf("length of label: %d \n",shortbuf);
  te->label_len = shortbuf;
  // read label
  if (te->label_len > 0) {
      if (fread(te->label,te->label_len,1,fid) < 1) return(-1);
//      printf("label: %s \n",te->label);
  } //else printf("----------------\n");
  if (te->isdata == 21) {  //is data
//      printf("(TagType)\n");
      te->inst = (char *) malloc(sizeof(DM3TagType));
      TagType_read(store_array, doswap, dataswap, (DM3TagType *) te->inst, fid);
  
  } else { //is a TagGroup
//      printf("(TagGroup) \n");
      te->inst = (char *) malloc(sizeof(DM3TagGroup));
      TagGroup_read(store_array, doswap, dataswap, (DM3TagGroup *) te->inst, fid);
  }
  return(0);
}

int  TagGroup_read(int store_array, int doswap, int dataswap, DM3TagGroup* tg, FILE* fid){

  char cbuf;
  int intbuf;
  int i;
  if (fread((char *)&cbuf,1,1,fid) < 1) return(-1);
  tg->issorted = (int) cbuf;
//  printf("is sorted: %d \n",tg->issorted);
  if (fread((char *)&cbuf,1,1,fid) < 1) return(-1);
  tg->isopen = (int) cbuf;
//  printf("is open: %d \n",tg->isopen);
  // read number of Tag Entries  
  if (fread((char *)&intbuf,4,1,fid) < 1) return(-1);
  if (doswap==1) swap((char *) &intbuf, 4);
//  printf("number of tags: %d \n",intbuf);
  tg->ntags = intbuf;
//  printf(" (%d)\n",tg->ntags);
  tg->tag = (DM3TagEntry **) malloc(sizeof(DM3TagEntry*)*tg->ntags);
  for (i=0; i<tg->ntags; i++) {
      tg->tag[i] = (DM3TagEntry *) malloc(sizeof(DM3TagEntry));
//      if (tg->ntags > 1) printf("Tag %d of %d\n",i+1,tg->ntags);
      TagEntry_read(store_array, doswap, dataswap, tg->tag[i], fid);
  }
  
  return(0);
}

DM3TagEntry_nlist * TagEntry_find(char * target_label, DM3TagGroup * tgroup) {

//GCA: the solution with iterative struct is complicated by the loop over the tags
//     for each group. To rey alternative solutions to probe the tag tree
  DM3TagEntry_nlist * tenl_head = NULL;
  DM3TagEntry_nlist * tenl = NULL;
  DM3TagGroup * tg;

  int i;
  for ( i=0; i<tgroup->ntags; i++) {
//      printf("%d of %d: %s - %d\n",i+1,tgroup->ntags,tgroup->tag[i]->label,tgroup->tag[i]->isdata);
      if (strcmp(tgroup->tag[i]->label, target_label) == 0) {
//          printf("Found %s!\n",tgroup->tag[i]->label);
          if (tenl == NULL) {
              tenl = (DM3TagEntry_nlist *) malloc(sizeof(DM3TagEntry_nlist));
              tenl->tag = tgroup->tag[i];
              tenl->next = NULL;
              if (tenl_head == NULL) tenl_head = tenl;
          } else {
              tenl->next = (DM3TagEntry_nlist *) malloc(sizeof(DM3TagEntry_nlist));
              tenl = tenl->next;
              tenl->tag = tgroup->tag[i];
              tenl->next = NULL;
          }
          
      } else if ( tgroup->tag[i]->isdata != 21 ) {
          tg = (DM3TagGroup *) tgroup->tag[i]->inst;
          if (tenl == NULL) {
              tenl = TagEntry_find(target_label, tg);
              if (tenl_head == NULL) tenl_head = tenl;
              if (tenl != NULL) {
                  while (tenl->next !=NULL) tenl = tenl->next;
              }
          } else {
              tenl->next = TagEntry_find(target_label, tg);
              if (tenl->next != NULL) {
                  while (tenl->next->next !=NULL) tenl = tenl->next;
              }
          }
      }
  }
  
  return(tenl_head);
}

void TagGroup_show(DM3TagGroup * tg){

  int i;
  for ( i=0; i<tg->ntags; i++) {
      if(tg->tag[i]->isdata != 21) 
          printf("-%s ",tg->tag[i]->label);
      else
          printf("*%s ",tg->tag[i]->label);
  }
  printf("\n");
  for ( i=0; i<tg->ntags; i++) {
      if(tg->tag[i]->isdata != 21) {
          printf(" -- %s --\n",tg->tag[i]->label);
          TagGroup_show((DM3TagGroup *) tg->tag[i]->inst);
      }
  }
}

DM3TagEntry ** GetImageData_info( int * n, DM3Micrograph_header * mgh){

  int j;
  char target_label[MAXSTR];
  
  strcpy(target_label,"ImageData");

  DM3TagEntry ** te = NULL;

  DM3TagEntry_nlist * tl = TagEntry_find(target_label,&mgh->root_group);
  DM3TagEntry_nlist * tn = NULL;
  
// count how many found
  int nimg = 0;
  DM3TagEntry_nlist * t = tl;
  while (t != NULL) {
      nimg += 1;
      t = t->next;
  }
  if (nimg>0){
//      printf("Found %d images\n",nimg);
      t = tl;
      te = malloc(sizeof(DM3TagEntry *)* nimg);
      for (j=0; j<nimg; j++){
          te[j] = t->tag;
          t = t->next;
      }
     // clean up the nested list
      t = tl;
      for (j=0; j<nimg; j++){
          tn = t->next;
          free(t);
          t = tn;
      }
  }
  *n = nimg;
  return(te);
}

long GetIntValue(DM3TagType * tt){
   return(value2long(tt->data,tt->data_type));
}

long double GetFloatValue(DM3TagType * tt){
   return(value2double(tt->data,tt->data_type));
}

long double GetFloatStruct(DM3TagType * tt, int i){
   return(struct2double(tt->data,((DM3Struct *)tt->type)->fdata_type,i));
}

// GCA: this function is not used because the number associated to each type is different 
//      than the one used for the array data
int GetImageDataType( DM3TagEntry * imgtag){

  int data_type = 0;
  int i;
  if (strcmp(imgtag->label,"ImageData") !=0) {
      fprintf(stderr,"ERROR (GetImageDataType): input tag has label %s: ImageData required!\n",imgtag->label);
      return(-1);
  }
  if ( imgtag->isdata == 21 ) return(0);
  DM3TagGroup * tg = (DM3TagGroup *) imgtag->inst;
  for (i=0; i<tg->ntags; i++){
      if (strcmp(tg->tag[i]->label,"DataType")==0) {
          return((int)GetIntValue((DM3TagType *) tg->tag[i]->inst));
      }
  }
  return(data_type);           
}

int GetDataType( DM3TagEntry * imgtag){

  int data_type = 0;
  int i;
  if (strcmp(imgtag->label,"ImageData") !=0) {
      fprintf(stderr,"ERROR (GetDataType): input tag has label %s: ImageData required!\n",imgtag->label);
      return(-1);
  }
  if ( imgtag->isdata == 21 ) return(0);
  DM3TagGroup * tg = (DM3TagGroup *) imgtag->inst;
  for (i=0; i<tg->ntags; i++){
      if (strcmp(tg->tag[i]->label,"Data")==0) {
          DM3TagType * tt = (DM3TagType *) tg->tag[i]->inst;
          return( ((DM3Array *)tt->type)->data_type);
      }
  }
  return(data_type);           
}

int GetDataOffset( DM3TagEntry * imgtag){

  long offset = 0;
  int i;
  if (strcmp(imgtag->label,"ImageData") !=0) {
      fprintf(stderr,"ERROR (GetDataOffset): input tag has label %s: ImageData required!\n",imgtag->label);
      return(-1);
  }
  if ( imgtag->isdata == 21 ) return(0);
  DM3TagGroup * tg = (DM3TagGroup *) imgtag->inst;
  for (i=0; i<tg->ntags; i++){
      if (strcmp(tg->tag[i]->label,"Data")==0) {
          DM3TagType * tt = (DM3TagType *) tg->tag[i]->inst;
          return( ((DM3Array *)tt->type)->offset);
      }
  }
  return(offset);           
}

int GetPixelDepth( DM3TagEntry * imgtag){

  int pixdepth = 0;
  int i;
  if (strcmp(imgtag->label,"ImageData") !=0) {
      fprintf(stderr,"ERROR (GetPixelDepth): input tag has label %s: ImageData required!\n",imgtag->label);
      return(-1);
  }
  if ( imgtag->isdata == 21 ) return(0);
  DM3TagGroup * tg = (DM3TagGroup *) imgtag->inst;
  for (i=0; i<tg->ntags; i++){
      if (strcmp(tg->tag[i]->label,"PixelDepth")==0) {
          return((int)GetIntValue((DM3TagType *) tg->tag[i]->inst));
      }
  }
  return(pixdepth);           
}

float GetMagnification( DM3TagGroup * tg, DM3TagEntry * te){

  int i;
  float mag = -1;
  float curr_mag;

  if (strcmp(te->label,"ImageData")!=0) {
      fprintf(stderr,"ERROR (GetMagnification): reference tag has label %s: ImageData required!\n",te->label);
      return(mag);
  }
  int image_found = 0;
  DM3TagEntry * imgtag_te = NULL;
  DM3TagGroup * tgn = NULL;
  
  for (i=0; i<tg->ntags; i++) {
//      printf("%s\n",tg->tag[i]->label);
      if (tg->tag[i]->isdata !=21 && tg->tag[i] == te) image_found = 1;
  }
  if ( image_found ) {
      imgtag_te = NULL;
      for (i=0; i<tg->ntags; i++)
          if (strcmp(tg->tag[i]->label,"ImageTags")==0)
              imgtag_te = tg->tag[i];
      if (imgtag_te !=NULL) {
          tgn = (DM3TagGroup *) imgtag_te->inst;
          imgtag_te = NULL;
          for (i=0; i<tgn->ntags; i++){
              if (tgn->tag[i]->isdata !=21 && strcmp(tgn->tag[i]->label,"Microscope Info")==0) {
                  imgtag_te = tgn->tag[i];
                  break;
              }
          }
          if (imgtag_te !=NULL) {
              tgn = (DM3TagGroup *) imgtag_te->inst;
              imgtag_te = NULL;
              for (i=0; i<tgn->ntags; i++) {
                  if (strcmp(tgn->tag[i]->label,"Indicated Magnification")==0){
                      imgtag_te = tgn->tag[i];
                      break;
                  }
              }
              if (imgtag_te !=NULL) {
                  curr_mag = (float)GetFloatValue((DM3TagType *) imgtag_te->inst);
                  return(curr_mag);
              }
          }
      }
  } else { //keep scanning
      for (i=0; i<tg->ntags; i++) {
          if ( tg->tag[i]->isdata !=21 ) {
              tgn = (DM3TagGroup *) tg->tag[i]->inst;
              curr_mag = GetMagnification(tgn, te);
              if (curr_mag >0) return(curr_mag);  
          }
      }
  }        
  return(mag);

}

float GetCCDPixelSize( DM3TagGroup * tg, DM3TagEntry * te){

  int i;
  float sampl = -1;
  float curr_sampl;

  if (strcmp(te->label,"ImageData")!=0) {
      fprintf(stderr,"ERROR (GetCCDPixelSize): reference tag has label %s: ImageData required!\n",te->label);
      return(sampl);
  }
  int image_found = 0;
  DM3TagEntry * imgtag_te = NULL;
  DM3TagGroup * tgn = NULL;
  
  for (i=0; i<tg->ntags; i++) {
//      printf("%s\n",tg->tag[i]->label);
      if (tg->tag[i]->isdata !=21 && tg->tag[i] == te) image_found = 1;
  }
  if ( image_found ) {
      imgtag_te = NULL;
      for (i=0; i<tg->ntags; i++)
          if (strcmp(tg->tag[i]->label,"ImageTags")==0)
              imgtag_te = tg->tag[i];
      if (imgtag_te !=NULL) {
          tgn = (DM3TagGroup *) imgtag_te->inst;
          imgtag_te = NULL;
          for (i=0; i<tgn->ntags; i++){
              if (tgn->tag[i]->isdata !=21 && strcmp(tgn->tag[i]->label,"Acquisition")==0) {
                  imgtag_te = tgn->tag[i];
                  break;
              }
          }
          if (imgtag_te !=NULL) {
              tgn = (DM3TagGroup *) imgtag_te->inst;
              imgtag_te = NULL;
              for (i=0; i<tgn->ntags; i++){
                  if (tgn->tag[i]->isdata !=21 && strcmp(tgn->tag[i]->label,"Device")==0) {
                      imgtag_te = tgn->tag[i];
                      break;
                  }
              }
              if (imgtag_te !=NULL) {
                  tgn = (DM3TagGroup *) imgtag_te->inst;
                  imgtag_te = NULL;
                  for (i=0; i<tgn->ntags; i++){
                      if (tgn->tag[i]->isdata !=21 && strcmp(tgn->tag[i]->label,"CCD")==0) {
                          imgtag_te = tgn->tag[i];
                          break;
                      }
                  }
                  if (imgtag_te !=NULL) {
                      tgn = (DM3TagGroup *) imgtag_te->inst;
                      imgtag_te = NULL;
                      for (i=0; i<tgn->ntags; i++) {
                          if (strcmp(tgn->tag[i]->label,"Pixel Size (um)")==0){
                              imgtag_te = tgn->tag[i];
                              break;
                          }
                      }
                      if (imgtag_te !=NULL) {
                          DM3TagType * pixel_tag = (DM3TagType *) imgtag_te->inst;
                          if (pixel_tag->data_type!=15) {
                              fprintf(stderr,"ERROR (GetCCDPixelSize): CCD Pixel Size Tag is %d: 15 expected!\n",pixel_tag->data_type);
                              return(sampl);
                          }
//                          DM3Struct * pixel_strct = (DM3Struct *) pixel_tag->type;
//                          printf("Number of fields: %d\n",pixel_strct->nfields);
                          //GCA: there are two fields, not clear why: at the moment read only the first one
                          curr_sampl = (float)GetFloatStruct((DM3TagType *) imgtag_te->inst, 0);
                          return(curr_sampl);
                      }
                  }
              }
          }
      }
  } else { //keep scanning
      for (i=0; i<tg->ntags; i++) {
          if ( tg->tag[i]->isdata !=21 ) {
              tgn = (DM3TagGroup *) tg->tag[i]->inst;
              curr_sampl = GetCCDPixelSize(tgn, te);
              if (curr_sampl >0) return(curr_sampl);  
          }
      }
  }        
  return(sampl);

}

Mg_Dim * GetDimensions( DM3TagEntry * te){

  int i;
  Mg_Dim * mgdim = NULL;

  if (strcmp(te->label,"Dimensions")!=0 && strcmp(te->label,"ImageData")!=0) {
      fprintf(stderr,"ERROR (GetDimensions): input tag has label %s: Dimensions or ImageData required!\n",te->label);
      return(mgdim);
  }
  if ( te->isdata == 21 ) return(mgdim);
  DM3TagGroup * tg = (DM3TagGroup *) te->inst;
  if (strcmp(te->label,"Dimensions") == 0 ){
      if (tg->ntags != 2) {
//      fprintf(stderr,"Warning: this is not a micrograph (%d dimensions)!\n",tg->ntags);
          return(mgdim);
      }
  
      mgdim = (Mg_Dim *) malloc(sizeof(Mg_Dim));
      mgdim->x = (int)GetIntValue((DM3TagType *) tg->tag[0]->inst);
      mgdim->y = (int)GetIntValue((DM3TagType *) tg->tag[1]->inst);

  } else if (strcmp(te->label,"ImageData") == 0 ){
      DM3TagEntry * tdata;
      for (i=0; i<tg->ntags; i++){
          if (strcmp(tg->tag[i]->label,"Dimensions")==0) {
              tdata = tg->tag[i];
              mgdim = GetDimensions(tdata);
          }
      }
  }

  return(mgdim);

}

long GetSize( DM3TagEntry * imgtag){

  Mg_Dim * mgdim = NULL;
  long size = 0;
  int i;
  if (strcmp(imgtag->label,"ImageData") !=0) {
      fprintf(stderr,"ERROR (GetSize): input tag has label %s: ImageData required!\n",imgtag->label);
      return(-1);
  }
  if ( imgtag->isdata == 21 ) return(0);
  DM3TagGroup * tg = (DM3TagGroup *) imgtag->inst;
  DM3TagEntry * tdata;
  for (i=0; i<tg->ntags; i++){
      if (strcmp(tg->tag[i]->label,"Dimensions")==0) {
          tdata = tg->tag[i];
          mgdim = GetDimensions(tdata);
          if (mgdim!=NULL) size = mgdim->x*mgdim->y;
      }
  }
  if (mgdim != NULL) free(mgdim);
  return(size);           
}

DM3TagEntry * KeepLargestImage(int nimg, DM3TagEntry ** imdata){

  int i;
  DM3TagEntry * largest_img = NULL;
  DM3TagEntry * curr_img = NULL;
  long largest_img_npixels = 0;
  long npixels;
//  int pixel_depth;
   
  for (i=0; i<nimg; i++) {
      curr_img = imdata[i];
      npixels = GetSize(curr_img);
//      pixel_depth = GetPixelDepth(curr_img);
      if (npixels> largest_img_npixels) {
          largest_img_npixels = npixels;
//          printf("%d - npixels: %ld\n",i, largest_img_npixels);
          largest_img = curr_img;
      }
  }
  
  return(largest_img);
}

DM3Micrograph * DM3Micrograph_new() {

  DM3Micrograph * mg = (DM3Micrograph *) malloc(sizeof(DM3Micrograph));

  if (mg==NULL) {
      printf("DM3Micrograph_new() - Can not allocate memory for the micrograph!\n");
      return mg;     
  }

  memset(mg->filename,'\0',1024);

//  mg->file_id = NULL;
  mg->header = NULL;
  mg->data = NULL;
  
  return mg;
}

void DM3Micrograph_delete( DM3Micrograph *mg){
  if(mg->header) free(mg->header);
  if(mg->data) free(mg->data);
//  if(mg->file_id) fclose(mg->file_id);
  free(mg);
}

DM3Micrograph_header * DM3Micrograph_header_new() {

  DM3Micrograph_header * mgh = (DM3Micrograph_header *) malloc(sizeof(DM3Micrograph_header));

  if (mgh==NULL) {
      printf("DM3Micrograph_header_new() - Can not allocate memory for the micrograph header!\n");
      return mgh;     
  }


  return mgh;
}


int DM3Micrograph_read ( DM3Micrograph *mg ){

  int ret = 0;
  
  DM3Micrograph_read_header( mg );
  DM3Micrograph_read_data( mg );


  return ret;
}

int DM3Micrograph_read_header ( DM3Micrograph *mg ){

  int intbuf;
  int ret = 0;
  
  int header_swap = 0;
  
  if (mg->header == NULL) mg->header = (DM3Micrograph_header *) DM3Micrograph_header_new();
  else return (-1);

  FILE* fid = fopen(mg->filename, "r");
  if (fid == NULL) {
      fprintf(stderr,"ERROR (DM3Micrograph_read_header): can not open file %s!\n",mg->filename);
      return(-1);
  }
  
  DM3Micrograph_header * mgh = mg->header;
  
  // read DM version  
  if (fread((char *)&intbuf,4,1,fid) < 1) return(-1);
  if (intbuf > 3) {
      header_swap = 1;
      swap((char *) &intbuf, 4);
  }
  int version = intbuf;
//  printf("version: %d \n",version);
  if (version != 3 ) {
      fprintf(stderr,"ERROR (DM3Micrograph_read_header): %s does not seem to be a DM3 file!\n",mg->filename);
      fclose(fid);
      return(-1);
  }
  // read file size  
  if (fread((char *)&intbuf,4,1,fid) < 1) return(-1);
  if (header_swap==1) swap((char *) &intbuf, 4);
//  printf("file size: %d bytes\n",intbuf);
  
  // read endianess of data  
  if (fread((char *)&intbuf,4,1,fid) < 1) return(-1);
  if (header_swap==1) swap((char *) &intbuf, 4);
//  printf("little endian: %d \n",intbuf);
  int little_endian = intbuf;
  if (little_endian < 0 || little_endian > 1) {
      fprintf(stderr,"ERROR (DM3Micrograph_read_header): endianess test on %s has failed!\n",mg->filename);
      fclose(fid);
      return(-1);
  }
  // determine if data are in the same endianess as the machine reading them
  int data_swap = 0;
  if (little_endian==0) data_swap = header_swap;
  else if( header_swap == 0) data_swap = 1;

  mgh->header_swap = header_swap;
  mgh->data_swap = data_swap;
  
  // read from file and store all the tags, but avoid to store arrays
  TagGroup_read(0, header_swap, data_swap, &mgh->root_group, fid);
  
  fclose(fid);

  // get information about size and format of image
  int nid;
  DM3TagEntry ** imdata = GetImageData_info(&nid,mgh);
  DM3TagEntry * chosen_img = KeepLargestImage(nid, imdata);
  Mg_Dim * mgdim = GetDimensions(chosen_img);
  mgh->nx = mgdim->x;
  mgh->ny = mgdim->y;
//  printf("Image dimension: %d x %d\n",mgh->nx,mgh->ny);
  mgh->data_type = GetDataType(chosen_img);
//  printf("Number of bytes per pixel: %d\n",dm3type2bytes(mgh->data_type));

  // get info on microscope settings, in order to calculate sampling
  float mag = GetMagnification(&mgh->root_group, chosen_img);
//  printf("Magnification: %f\n",mag);
  float ccd_pixel_size = GetCCDPixelSize(&mgh->root_group, chosen_img);
//  printf("CCD pixel size: %f um\n",ccd_pixel_size);
  if (mag >0 && ccd_pixel_size >0)
      mgh->dx = mgh->dy = ccd_pixel_size*10000./mag;
  else {
      printf("Warning: could not determine nominal pixel size from magnification (%f) and CCD pixel size (%f)\n",mag,ccd_pixel_size);
      mgh->dx = mgh->dy = 1.0; 
  }
  printf("Nominal pixel size: %f angstroms\n",mgh->dx);
  mgh->data_offset = GetDataOffset(chosen_img);
//  TagGroup_show(&mgh->root_group);
  

  return(ret);
}

int DM3Micrograph_read_data ( DM3Micrograph *mg ){

  int ret = 0;
  int i;
  
  if (mg->header == NULL) DM3Micrograph_read_header( mg );

  FILE* fid = fopen(mg->filename, "r");
  if (fid == NULL) {
      fprintf(stderr,"ERROR (DM3Micrograph_read_data): can not open file %s!\n",mg->filename);
      return(-1);
  }
  
  DM3Micrograph_header * mgh = mg->header;
  
  if (mgh->data_offset <= 0) {
      fprintf(stderr,"ERROR (DM3Micrograph_read_data): can not determine offset to data in file %s!\n",mg->filename);
      return(-1);
  }
  fseek(fid, mgh->data_offset, SEEK_SET );

  // read data from file
  int nbytes = dm3type2bytes(mgh->data_type);
  long datasize = mgh->nx*mgh->ny*nbytes;
  mg->data = (char *) malloc(datasize);
  
  if (fread(mg->data,datasize,1,fid) < 1) return(-1);
  if (mgh->data_swap==1 && nbytes>1) {
      char *j = mg->data;
      for ( i=0; i<mgh->nx*mgh->ny; i++) {
          swap(j, nbytes);
          j += nbytes;
      }
  }
  
/*  char *j = mg->data;
  for ( i=0; i<mgh->nx*mgh->ny; i++) {
      if (fread(c8buf,nbytes,1,fid) < 1) return(-1);
      if (mgh->data_swap==1 && nbytes>1) swap(c8buf, nbytes);
      memcpy(j, c8buf, nbytes);
      j += nbytes;
  }
*/
  fclose(fid);

  return ret;
}

