#ifndef __DM3io_h__
#define __DM3io_h__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAXLEV 64
#define MAXSTR 256

typedef struct {
   int x;
   int y;
} Mg_Dim;

typedef struct {
   float x;
   float y;
} Mg_Sampl;

typedef struct {

    int name_len;
    int nfields;
    long datasize;
    int * fname_len;
    int * fdata_type;
    
} DM3Struct;

typedef struct {

    int data_type;
    long offset;
    int nelems;
    int elem_size;
    long datasize;
    DM3Struct * strct;
        
} DM3Array;

typedef struct {

    int def_len;
    int data_type;
    char * type;
    char * data;
    
} DM3TagType;

typedef struct {

    int isdata;
    int label_len;
    char label[MAXSTR];
    char * inst;
    
} DM3TagEntry;

typedef struct DM3TagEntry_nlist{
   DM3TagEntry * tag;
   struct DM3TagEntry_nlist * next;
} DM3TagEntry_nlist;

typedef struct {

    int issorted;
    int isopen;
    int ntags;
    DM3TagEntry ** tag;
    
} DM3TagGroup;

typedef struct {

    int data_swap;
    int header_swap;    
    long data_offset;        // offset to chosen image
    int data_type;
    int nx;                  // number of pixels along x
    int ny;                  // number of pixels along y
    float dx;                // sampling along x
    float dy;                // sampling along y
    DM3TagGroup root_group;
    
} DM3Micrograph_header;


typedef struct {

//    FILE * file_id;
    char filename[1024];
    
    DM3Micrograph_header * header;
    char * data;

} DM3Micrograph;

DM3Micrograph * DM3Micrograph_new();
void DM3Micrograph_delete( DM3Micrograph *mg);

DM3Micrograph_header * DM3Micrograph_header_new();
//void DM3Micrograph_header_delete( DM3Micrograph_header *mgg);

int DM3Micrograph_read ( DM3Micrograph *mg );
//int DM3Micrograph_write( DM3Micrograph *mg, char *fname );

int DM3Micrograph_open( DM3Micrograph *mg );
int DM3Micrograph_close( DM3Micrograph *mg );
int DM3Micrograph_read_header( DM3Micrograph *mg );
//int DM3Micrograph_write_header( DM3Micrograph *mg, char *fname );
int DM3Micrograph_read_data( DM3Micrograph *mg );
//int DM3Micrograph_write_data( DM3Micrograph *mg, char *fname );

int  TagGroup_read(int store_array, int doswap, int dataswap, DM3TagGroup* tg, FILE* fid);

#endif
