#include "PIFconvert.h"

int is_big_endian(){
   int x = 1;
   return ( (*(char*)&x) == 0 );
} 

int   type2piftype( int dt) {
    int t = -1;

    switch ( dt ) {
//        case UBYTE:  t = ; break;  
        case BYTE:   t = 0; break;  
//        case USHORT: t = ; break; 
        case SHORT:  t = 1; break;  
//        case UINT:   t = ; break;  
        case INT:    t = 2; break;  
        case FLOAT:  t = 9; break;  
//        case DOUBLE: t = ; break; 
        default:
            fprintf(stderr, "Error(type2piftype): Can not convert data type %d!\n", dt);
    }
	
    return(t);
}

float Pifscale_factor(PIFData_header * dh) {
  // Calculate the factor needed to convert out floatIntegers into floats
  // NOTE: at the moment it ignores the statistics of the data
  // WARNING: at the moment the routine is on hold, and it returns always 1
  //   This is due to inconsistencies in the current library libPIF.c used
  //   by the other programs (POR, PPFT and P3DR). Specifically, the library uses the
  //   recirpocal of the scale factor to get back the float values, but the reciprocal
  //   can be zero in many cases.
  //   This also means that all the floatint parameters in the data header are
  //   WRONGLY assigned. Also, the data in mode 2 will be interpreted as INT, while
  //   any mode floatint will be WRONG (only mode 9 will work as a float).

  double scale_factor = 1.0;
  return(scale_factor);
  //GCA: ignored thereafter
  int minVal=99999, maxVal=-99999;

  // determine minimum float
  minVal = MIN(minVal, dh->background);
  minVal = MIN(minVal,(int)(dh->a_cell));
  minVal = MIN(minVal,(int)(dh->b_cell));
  minVal = MIN(minVal,(int)(dh->c_cell));
  minVal = MIN(minVal,(int)(dh->alpha));
  minVal = MIN(minVal,(int)(dh->beta));
  minVal = MIN(minVal,(int)(dh->gamma));
  minVal = MIN(minVal,(int)(dh->mrc_mapc));
  minVal = MIN(minVal,(int)(dh->mrc_mapr));
  minVal = MIN(minVal,(int)(dh->mrc_maps));
  minVal = MIN(minVal, dh->data_imin);
  minVal = MIN(minVal, dh->data_imax);
  minVal = MIN(minVal,(int)(dh->data_avg));
  minVal = MIN(minVal,(int)(dh->data_std));
  minVal = MIN(minVal,(int)(dh->aoverb));
  minVal = MIN(minVal,(int)(dh->abang));
  minVal = MIN(minVal,(int)(dh->dela));
  minVal = MIN(minVal,(int)(dh->delb));
  minVal = MIN(minVal,(int)(dh->delc));
  minVal = MIN(minVal,(int)(dh->delta_theta));
  minVal = MIN(minVal,(int)(dh->delta_phi90));
  minVal = MIN(minVal,(int)(dh->a_star));
  minVal = MIN(minVal,(int)(dh->b_star));
  minVal = MIN(minVal,(int)(dh->c_star));
  minVal = MIN(minVal,(int)(dh->alp_star));
  minVal = MIN(minVal,(int)(dh->bet_star));
  minVal = MIN(minVal,(int)(dh->gam_star));
  minVal = MIN(minVal,(int)(dh->pixel_size));
  minVal = MIN(minVal,(int)(dh->resout));
  minVal = MIN(minVal,(int)(dh->invtempfac));
  minVal = MIN(minVal,(int)(dh->wiener_value));
  minVal = MIN(minVal,(int)(dh->data_rmin));
  minVal = MIN(minVal,(int)(dh->data_rmax));

  // determine maximum float
  maxVal = MAX(maxVal, dh->background);
  maxVal = MAX(maxVal,(int)(dh->a_cell));
  maxVal = MAX(maxVal,(int)(dh->b_cell));
  maxVal = MAX(maxVal,(int)(dh->c_cell));
  maxVal = MAX(maxVal,(int)(dh->alpha));
  maxVal = MAX(maxVal,(int)(dh->beta));
  maxVal = MAX(maxVal,(int)(dh->gamma));
  maxVal = MAX(maxVal,(int)(dh->mrc_mapc));
  maxVal = MAX(maxVal,(int)(dh->mrc_mapr));
  maxVal = MAX(maxVal,(int)(dh->mrc_maps));
  maxVal = MAX(maxVal, dh->data_imin);
  maxVal = MAX(maxVal, dh->data_imax);
  maxVal = MAX(maxVal,(int)(dh->data_avg));
  maxVal = MAX(maxVal,(int)(dh->data_std));
  maxVal = MAX(maxVal,(int)(dh->aoverb));
  maxVal = MAX(maxVal,(int)(dh->abang));
  maxVal = MAX(maxVal,(int)(dh->dela));
  maxVal = MAX(maxVal,(int)(dh->delb));
  maxVal = MAX(maxVal,(int)(dh->delc));
  maxVal = MAX(maxVal,(int)(dh->delta_theta));
  maxVal = MAX(maxVal,(int)(dh->delta_phi90));
  maxVal = MAX(maxVal,(int)(dh->a_star));
  maxVal = MAX(maxVal,(int)(dh->b_star));
  maxVal = MAX(maxVal,(int)(dh->c_star));
  maxVal = MAX(maxVal,(int)(dh->alp_star));
  maxVal = MAX(maxVal,(int)(dh->bet_star));
  maxVal = MAX(maxVal,(int)(dh->gam_star));
  maxVal = MAX(maxVal,(int)(dh->pixel_size));
  maxVal = MAX(maxVal,(int)(dh->resout));
  maxVal = MAX(maxVal,(int)(dh->invtempfac));
  maxVal = MAX(maxVal,(int)(dh->wiener_value));
  maxVal = MAX(maxVal,(int)(dh->data_rmin));
  maxVal = MAX(maxVal,(int)(dh->data_rmax));

  // round the integer at the end
  maxVal +=1;
  minVal -= 1;
  
  // Get greatest absolute value
//  minVal = fabs((float)minVal);
  minVal = abs(minVal);
  maxVal = MAX(minVal, maxVal);

  // Give a little extra so we're guaranteed NOT to run over the top
  maxVal = maxVal*1.05;

  scale_factor = (double)INT_MAX / (double)maxVal;

  return(scale_factor);
}


int PIFMicrograph_import ( Micrograph *mg, PIFMicrograph *pifmg ) {

    int ret = 0;
   
    if (pifmg == NULL || mg == NULL) {
        fprintf(stderr,"Error(PIFMicrograph_import): data not abvailable for conversion!\n");
        return(1);
    }

    PIFGlobal_header * pgh = pifmg->header->globalhead;
    PIFData_header *   pdh = pifmg->header->datahead;
    pgh->floatscale_factor = 1.0;
    pgh->nimg = 1;
    pgh->big_endian = is_big_endian();
    pgh->htype = 1;
    pgh->numcols = mg->header->nx;
    pgh->numrows = mg->header->ny;
    pgh->numsecs = 1;
    pgh->data_type = type2piftype(mg->header->data_type);
    
    pdh->numcols = mg->header->nx;
    pdh->numrows = mg->header->ny;
    pdh->numsecs = 1;
    pdh->data_type = type2piftype(mg->header->data_type);
    pdh->pack_radius = MIN(mg->header->nx,mg->header->ny)/2;
    
    pdh->bin_factor = 1;
    pdh->pixel_size = mg->header->dx;
    if ( mg->header->dy != mg->header->dx)
        fprintf(stderr,"Error(PIFMicrograph_import): pixel size is different along x and y. Stored only x value!\n");
    pdh->pixel_units = 0;

    pifmg->data = mg->data;

    pgh->floatscale_factor = Pifscale_factor(pifmg->header->datahead);
    pgh->intscale_factor = pgh->floatscale_factor;
        
    return(ret);

}
