#include "PIFio.h"

int   piftype2bytes( int t) {

    int nb = 0;

    switch ( t ) {
        case  0: nb = 1; break;  // BYTE
        case  1: nb = 2; break;  // SHORT
        case  2: nb = 4; break;  // FLOAT
        case  3: nb = 4; break;  // COMPLEX SHORT
        case  4: nb = 8; break;  // COMPLEX FLOAT
        case  6: nb = 1; break;  // BYTE BOX
        case  7: nb = 2; break;  // SHORT FLOAT
        case  8: nb = 4; break;  // COMPLEX SHORT FLOAT
        case  9: nb = 4; break;  // FLOAT
        case 10: nb = 8; break;  // COMPLEX FLOAT
        case 20: nb = 2; break;  // SHORT FLOAT 3D
        case 21: nb = 4; break;  // FLOAT 3D
        case 46: nb = 4; break;  // FLOAT BOX
        default:
            fprintf(stderr, "Error(piftype2bytes): Can not interpret data type %d!\n", t);
    }
    return(nb);
}


void PIFwriteScaleFactor(double fsf, char *arr) {

    sprintf(arr, "  %13e", 1L/fsf);
    arr[15] ='\0';
}


PIFMicrograph * PIFMicrograph_new() {

  PIFMicrograph * mg = (PIFMicrograph *) malloc(sizeof(PIFMicrograph));

  if (mg==NULL) {
      printf("PIFMicrograph_new() - Can not allocate memory for the micrograph!\n");
      return mg;     
  }

  memset(mg->filename,'\0',1024);

  mg->header = (PIFMicrograph_header *) malloc(sizeof(PIFMicrograph_header));
  mg->header->globalhead = (PIFGlobal_header *) malloc(sizeof(PIFGlobal_header));
  mg->header->datahead = (PIFData_header *) malloc(sizeof(PIFData_header));

  // initialize default parameters
  PIFGlobal_header * pgh = mg->header->globalhead;
  pgh->fileid[0] = pgh->fileid[4] = 8;
  pgh->fileid[1] = pgh->fileid[2] = pgh->fileid[3] = 0;
  pgh->fileid[5] = pgh->fileid[6] = pgh->fileid[7] = 0;

  mg->data = NULL;
  
  return mg;
}


void PIFMicrograph_delete( PIFMicrograph *mg){
  if(mg->header) free(mg->header);
  if(mg->data) free(mg->data);
  free(mg);
}


PIFMicrograph_header * PIFMicrograph_header_new() {

  PIFMicrograph_header * mgh = (PIFMicrograph_header *) malloc(sizeof(PIFMicrograph_header));

  if (mgh==NULL) {
      printf("PIFMicrograph_header_new() - Can not allocate memory for the micrograph header!\n");
      return mgh;     
  }
  
  mgh->globalhead = (PIFGlobal_header *) malloc(sizeof(PIFGlobal_header));

  if (mgh->globalhead==NULL) {
      printf("PIFMicrograph_header_new() - Can not allocate memory for the micrograph global header!\n");
      return mgh;     
  }

  return mgh;
}


int PIFMicrograph_write_global_header ( PIFMicrograph *mg, FILE *fid ){

  int ret = 0;
  int just_opened = 0;
  char head_label[32] ="AUTO3DEM I/O library";
  
    
  char scalefac[16];
  
  if (mg == NULL || mg->header == NULL ) {
      fprintf(stderr,"ERROR (PIFMicrograph_write_header): data not available for writing!\n");
      return(-1);
  }

  if (fid == NULL) {
      fid = fopen(mg->filename, "w");
      just_opened = 1;
  }
  if (fid == NULL) {
      fprintf(stderr,"ERROR (PIFMicrograph_write_header): can not open file %s!\n",mg->filename);
      return(-1);
  }
  
  PIFMicrograph_header * mgh = mg->header;
  
  // write global header
  PIFGlobal_header * gh = mgh->globalhead;
  
  char ghbuffer[512];
  memset(ghbuffer, 0, 512);
  
  char * j = ghbuffer;
  memcpy(j,gh->fileid,8);
  j += 8;
  PIFwriteScaleFactor(gh->floatscale_factor, scalefac);  
  memcpy(j,scalefac,16);
  j += 16;
  memcpy(j,&gh->nimg,4);
  j += 4;
  memcpy(j,&gh->big_endian,4);
  j += 4;
  memcpy(j,head_label,32);
  j += 32;
  memcpy(j,&gh->htype,4);
  j += 4;
  memcpy(j,&gh->numcols,4);
  j += 4;
  memcpy(j,&gh->numrows,4);
  j += 4;
  memcpy(j,&gh->numsecs,4);
  j += 4;
  memcpy(j,&gh->data_type,4);
  j += 4;
  memcpy(j,&gh->even,4);
  j += 4;
  memcpy(j,&gh->mrcx,4);
  j += 4;
  memcpy(j,&gh->mrcy,4);
  j += 4;
  memcpy(j,&gh->mrcz,4);
  j += 4;
  PIFwriteScaleFactor(gh->intscale_factor, scalefac);  
  memcpy(j,scalefac,16);
  j += 16;
  memcpy(j,&gh->a0,13);
  j += 13;
  memcpy(j,&gh->a1,13);
  j += 13;
  memcpy(j,&gh->a2,13);
  j += 13;
  memcpy(j,&gh->a3,13);
  j += 13;
  memcpy(j,&gh->a4,13);
  j += 13;
  memcpy(j,&gh->a5,13);
  j += 13;
  
  fwrite(ghbuffer, 1, 512, fid);
  
  // close file, if opened here
  if (just_opened == 1) fclose(fid);

  return(ret);
}


int PIFMicrograph_write_data( PIFMicrograph *mg, FILE *fid){
/* write data together with its own header */

  int ret = 0;
  int just_opened = 0;
  int i;
  
  if (mg == NULL || mg->header == NULL || mg->data == NULL) {
      fprintf(stderr,"ERROR (PIFMicrograph_write_data): data not available for writing!\n");
      return(-1);
  }

  if (fid == NULL) {
      fid = fopen(mg->filename, "r+");
      just_opened = 1;
  }
  if (fid == NULL) {
      fprintf(stderr,"ERROR (PIFMicrograph_write_data): can not open file %s!\n",mg->filename);
      return(-1);
  }

  PIFMicrograph_header * mgh = mg->header;
  
  // write data header
  PIFData_header * dh = mgh->datahead;
  PIFGlobal_header * gh = mgh->globalhead;
  float scalefac = gh->floatscale_factor;
  
  // point to the end of the global header
  fseek(fid,512L,SEEK_SET);
  
  int intval;
  char dhbuffer[512];
  char * j;
  
  memset(dhbuffer, 0, 512);
  j = dhbuffer;

  memcpy(j,&dh->numcols,4);
  j += 4;
  memcpy(j,&dh->numrows,4);
  j += 4;
  memcpy(j,&dh->numsecs,4);
  j += 4;
  memcpy(j,&dh->data_type,4);
  j += 4;
  //GCS: apparently this value is not converted
  // to investigate further
//  intval = (int) scalefac*dh->background;
  intval = (int) dh->background;
  memcpy(j,&intval,4);
  j += 4;
  memcpy(j,&dh->pack_radius,4);
  j += 4;
  memcpy(j,&dh->mrc_nxstart,4);
  j += 4;
  memcpy(j,&dh->mrc_nystart,4);
  j += 4;
  memcpy(j,&dh->mrc_nzstart,4);
  j += 4;
  memcpy(j,&dh->mrc_mx,4);
  j += 4;
  memcpy(j,&dh->mrc_my,4);
  j += 4;
  memcpy(j,&dh->mrc_mz,4);
  j += 4;
  intval = (int) scalefac*dh->a_cell;
  memcpy(j,&intval,4);
  j += 4;
  intval = (int) scalefac*dh->b_cell;
  memcpy(j,&intval,4);
  j += 4;
  intval = (int) scalefac*dh->c_cell;
  memcpy(j,&intval,4);
  j += 4;
  intval = (int) scalefac*dh->alpha;
  memcpy(j,&intval,4);
  j += 4;
  intval = (int) scalefac*dh->beta;
  memcpy(j,&intval,4);
  j += 4;
  intval = (int) scalefac*dh->gamma;
  memcpy(j,&intval,4);
  j += 4;
  intval = (int) scalefac*dh->mrc_mapc;
  memcpy(j,&intval,4);
  j += 4;
  intval = (int) scalefac*dh->mrc_mapr;
  memcpy(j,&intval,4);
  j += 4;
  intval = (int) scalefac*dh->mrc_maps;
  memcpy(j,&intval,4);
  j += 4;
  memcpy(j,&dh->data_imin,4);
  j += 4;
  memcpy(j,&dh->data_imax,4);
  j += 4;
  intval = (int) scalefac*dh->data_avg;
  memcpy(j,&intval,4);
  j += 4;
  intval = (int) scalefac*dh->data_std;
  memcpy(j,&intval,4);
  j += 4;
  memcpy(j,&dh->mrc_ispg,4);
  j += 4;
  memcpy(j,&dh->mrc_nsymbt,4);
  j += 4;
  memcpy(j,&dh->mrc_xorigin,4);
  j += 4;
  memcpy(j,&dh->mrc_yorigin,4);
  j += 4;
  memcpy(j,dh->description,80);
  j += 80;
  memcpy(j,dh->time,32);
  j += 32;
  memcpy(j,dh->micrograph,16);
  j += 16;
  memcpy(j,dh->scan,8);
  j += 8;
  intval = (int) scalefac*dh->aoverb;
  memcpy(j,&intval,4);
  j += 4;
  intval = (int) scalefac*dh->abang;
  memcpy(j,&intval,4);
  j += 4;
  intval = (int) scalefac*dh->dela;
  memcpy(j,&intval,4);
  j += 4;
  intval = (int) scalefac*dh->delb;
  memcpy(j,&intval,4);
  j += 4;
  intval = (int) scalefac*dh->delc;
  memcpy(j,&intval,4);
  j += 4;
  for ( i=0; i<6; i++) {
      memcpy(j,&dh->t_matrix[i],4);
      j += 4;
  }
  intval = (int) scalefac*dh->delta_theta;
  memcpy(j,&intval,4);
  j += 4;
  intval = (int) scalefac*dh->delta_phi90;
  memcpy(j,&intval,4);
  j += 4;
  memcpy(j,&dh->symmetry,4);
  j += 4;
  memcpy(j,&dh->bin_factor,4);
  j += 4;
  intval = (int) scalefac*dh->a_star;
  memcpy(j,&intval,4);
  j += 4;
  intval = (int) scalefac*dh->b_star;
  memcpy(j,&intval,4);
  j += 4;
  intval = (int) scalefac*dh->c_star;
  memcpy(j,&intval,4);
  j += 4;
  intval = (int) scalefac*dh->alp_star;
  memcpy(j,&intval,4);
  j += 4;
  intval = (int) scalefac*dh->bet_star;
  memcpy(j,&intval,4);
  j += 4;
  intval = (int) scalefac*dh->gam_star;
  memcpy(j,&intval,4);
  j += 4;
  intval = (int) scalefac*dh->pixel_size;
  memcpy(j,&intval,4);
  j += 4;
  memcpy(j,&dh->pixel_units,4);
  j += 4;
  intval = (int) scalefac*dh->resout;
  memcpy(j,&intval,4);
  j += 4;
  memcpy(j,&dh->ctf_mode,4);
  j += 4;
  intval = (int) scalefac*dh->invtempfac;
  memcpy(j,&intval,4);
  j += 4;
  intval = (int) scalefac*dh->wiener_value;
  memcpy(j,&intval,4);
  j += 4;
  memcpy(j,&dh->border_type,4);
  j += 4;
  memcpy(j,&dh->border_width,4);
  j += 4;
  intval = (int) scalefac*dh->data_rmin;
  memcpy(j,&intval,4);
  j += 4;
  intval = (int) scalefac*dh->data_rmax;
  memcpy(j,&intval,4);
  j += 4;
  memcpy(j,&dh->radlow,4);
  j += 4;
  memcpy(j,&dh->radhi,4);
  j += 4;
  memcpy(j,&dh->trans_x,4);
  j += 4;
  memcpy(j,&dh->trans_y,4);
  j += 4;
  memcpy(j,&dh->trans_z,4);
  j += 4;
  memcpy(j,&dh->is_truncated,4);
  j += 4;
  memcpy(j,&dh->origin_x,4);
  j += 4;
  memcpy(j,&dh->origin_y,4);
  j += 4;
  memcpy(j,&dh->origin_z,4);
  j += 4;
  memcpy(j,dh->future,100);
  j += 100;
  
  fwrite(dhbuffer, 1, 512, fid);
  
  // write data
  long datasize = dh->numcols*dh->numrows*dh->numsecs*piftype2bytes(dh->data_type);
  fwrite(mg->data, 1, datasize, fid);
 
  // close file, if opened here
  if (just_opened == 1) fclose(fid);
 
  return(ret);
}

int PIFMicrograph_write ( PIFMicrograph *mg ){

  int ret = 0;
  
  FILE* fid = fopen(mg->filename, "w");
  if (fid == NULL) {
      fprintf(stderr,"ERROR (PIFMicrograph_write): can not create file %s!\n",mg->filename);
      return(-1);
  }
  
  PIFMicrograph_write_global_header( mg, fid );
  PIFMicrograph_write_data( mg, fid );

  fclose(fid);
 
  return ret;
}



