#ifndef __PIFio_h__
#define __PIFio_h__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

typedef struct {

    char  fileid[8];              // magic number
//    char  floatscale_factor[16];  // scale factor for floating values
    double floatscale_factor;      // scale factor for floating values
    int   nimg;                   // number of images (for stacks)
    int   big_endian;             // 0: little endian; >0: big endian
    char  version[32];            // version of I/O library
    int   htype;                  // 1: all data sections have same size
    int   numcols;                // number of columns
    int   numrows;                // number of rows
    int   numsecs;                // number of sections
    int   data_type;              // data type
    int   even;
    int   mrcx;
    int   mrcy;
    int   mrcz;
//    char  intscale_factor[16];     // scale factor used only for mode 20
    double intscale_factor;        // scale factor used only for mode 20
    char  a0[13];
    char  a1[13];
    char  a2[13];
    char  a3[13];
    char  a4[13];
    char  a5[13];
    char  future[318];    
    
} PIFGlobal_header;

typedef struct {

    int   numcols;           // number of columns
    int   numrows;           // number of rows
    int   numsecs;           // number of sections
    int   data_type;
    float background;
    int   pack_radius;
    int   mrc_nxstart;
    int   mrc_nystart;
    int   mrc_nzstart;
    int   mrc_mx;
    int   mrc_my;
    int   mrc_mz;
    float a_cell;
    float b_cell;
    float c_cell;
    float alpha;
    float beta;
    float gamma;
    float mrc_mapc;
    float mrc_mapr;
    float mrc_maps;
    int   data_imin;
    int   data_imax;
    float data_avg;
    float data_std;
    int   mrc_ispg;
    int   mrc_nsymbt;
    int   mrc_xorigin;
    int   mrc_yorigin;
    char  description[80];
    char  time[32];
    char  micrograph[16];
    char  scan[8];
    float aoverb;
    float abang;
    float dela;
    float delb;
    float delc;
    int   t_matrix[6];
    float delta_theta;
    float delta_phi90;
    int   symmetry;
    int   bin_factor;
    float a_star;
    float b_star;
    float c_star;
    float alp_star;
    float bet_star;
    float gam_star;
    float pixel_size;
    int   pixel_units;          //0= angstroms; 1=nm
    float resout;
    int   ctf_mode;
    float invtempfac;
    float wiener_value;
    int   border_type;
    int   border_width;
    float data_rmin;
    float data_rmax;
    int   radlow;
    int   radhi;
    int   trans_x;
    int   trans_y;
    int   trans_z;
    int   is_truncated;
    int   origin_x;
    int   origin_y;
    int   origin_z;
    char  future[100];
        
} PIFData_header;

typedef struct {

    PIFGlobal_header * globalhead;
    PIFData_header   * datahead;

} PIFMicrograph_header;


typedef struct {

    char filename[1024];
    
    PIFMicrograph_header * header;
    char * data;

} PIFMicrograph;

PIFMicrograph * PIFMicrograph_new();
void PIFMicrograph_delete( PIFMicrograph *mg);

PIFMicrograph_header * PIFMicrograph_header_new();
//void PIFMicrograph_header_delete( PIFMicrograph_header *mgg);

//int PIFMicrograph_read ( PIFMicrograph *mg );
int PIFMicrograph_write( PIFMicrograph *mg );

//int PIFMicrograph_open( PIFMicrograph *mg );
//int PIFMicrograph_close( PIFMicrograph *mg );
//int PIFMicrograph_read_header( PIFMicrograph *mg );
int PIFMicrograph_write_header( PIFMicrograph *mg, FILE *fid );
//int PIFMicrograph_read_data( PIFMicrograph *mg );
int PIFMicrograph_write_data( PIFMicrograph *mg, FILE *fid );

#endif
