/******************************************************************
 *                                                                *
 *                *****     DM3 -> PIF      *****                 *
 *                                                                *
 *              Convert DM3 files to PIF format FILE              *
 *                                                                *
 ******************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
//#include <ctype.h>
//#include <math.h>

#include "image_types.h"
//void BinPixels(UBYTE *buffer, int *dx, int *dy, int binFactor);


void get_output_name_from_input(char* outfn, char* infn, char* insfx, char* outsfx) {
// get the output name from the input file and append given suffix, replacing the
// input one, if present

  char delim ='.';
  char sep = '/';
  char *lastdot, *lastsep;
   
  int has_given_ext = 0;
  strcpy(outfn,infn);
  const char *ext = strrchr(outfn, delim);
  if (ext && ext != outfn) {
     if (strcmp(ext+1,insfx) == 0) {
        has_given_ext = 1;
     }
  }
// remove the extension (generic routine to remove extensions)
  if (has_given_ext) {
    lastdot = strrchr(outfn, delim);
    lastsep = (sep == 0) ? NULL : strrchr (outfn, sep);

    // If it has an extension separator.

    if (lastdot != NULL) {
        // and it's before the extenstion separator.

        if (lastsep != NULL) {
            if (lastsep < lastdot) {
                // then remove it.

                *lastdot = '\0';
            }
        } else {
            // Has extension separator with no path separator.

            *lastdot = '\0';
        }
    }
  
  }
  int n = strlen(outfn);
  if (outfn[n-1] != delim) outfn[n++] = delim;
  outfn[n] = '\0';
  strcat(outfn,outsfx);
}


/************
    main
************/

int main(int argc, char **argv)
{
  char *check_arg;
  char banner_string[] = "DM32PIF";
  char input_dm3[1024];
  char output_pif[1024];
  char infmt[4] = "dm3";
  char outfmt[4] = "pif";
  int  bin = 1;

/****** verify proper usage ***********/

  if(argc < 2 || argc > 4) {/* usage */
    printf("Usage: dm32pif <dm3 file> [pif file] [bin factor]\n");
    printf("If pif file not provided, obtain name from input file\n");
    printf("Image will be binned only if bin factor provided\n");
    exit(0);
  }/* usage */

  banner(banner_string);
  printf("********    DM32PIF    ********\n");
  printf("Convert dm3 files to pif format\n\n");
  

/****** parse input arguments ***********/

  strcpy(input_dm3,argv[1]);
  if (argc == 4 ) {
      strcpy(output_pif,argv[2]);
      bin = strtol(argv[3],&check_arg,0);
  }
  if (argc == 3) {
      // check if second argument is output name or bin factor
      bin = (int) strtol(argv[2],&check_arg,0);
      if (strlen(check_arg) == 0) {
          get_output_name_from_input(output_pif,input_dm3,"dm3","pif");
      } else {
          strcpy(output_pif,argv[2]);
          bin = 1;
      }
  }
  if (argc == 2) {
          // get the output name from the input file
          get_output_name_from_input(output_pif,input_dm3,"dm3","pif");
  }          
      
  printf("\n");
  printf("Input file  = %s\n", input_dm3);
  printf("Output file = %s\n", output_pif);
  if (bin>1) {
      printf("Bin factor  = %d\n", bin);
  } else {
      printf("No binnig applied\n");
  }
  printf("\n");

 
/****** Read input dat and write to file in new format ***********/
 
  Micrograph * mg;
  if ((mg = Micrograph_new()) == NULL) return(1);
  strcpy(mg->filename,input_dm3);
  strcpy(mg->input_format,infmt);
  strcpy(mg->output_format,outfmt);

  if ( Micrograph_read (mg, bin) ) return(1);
  if ( Micrograph_write(mg, output_pif) ) return(1);
  Micrograph_delete(mg);
  
  printf("Done!\n");
  return(0);
}
