
C  ******************************************************************
C  *                                                                *
C  *                 *****      MRC-> PIF      *****                *
C  *                                                                *
C  *    LOGICAL UNITS: 7    = INPUT MRC FILE                        *
C  *                   11   = OUTPUT  PIFMAP FILE                   *
C  *    MBS, Jan. 2003                                              *
C  *    Latest changes MBS, Sep. 2003 (Made output of odd dimns)    *
C  *                                                                *
C  ******************************************************************
C
      implicit none
C
      include '../include/pifhead.inc'
      type(pifhead_t) PIFhead
c
      integer iargc
      integer*4 i, k, kstep, j, l, m, n
      integer*4 ncol, nrow, nsec
      integer*4 ncol_n, nrow_n, nsec_n
      integer*4 percent
      integer*4 nfilsz,kspg,kbs
      character(len=1) rw(2)
      character MRCname*256,PIFname*256
      character fullname*256
      character two_d*1
      logical   is2D, forceOdd

      integer alloc_stat
      integer*1, allocatable :: map_in_bt(:)
      integer*2, allocatable :: map_in_st(:)
      real*4, allocatable    :: map_in(:)
      real*4, allocatable    :: map_out(:)

      integer*4 iunit_in,iunit_out
      integer*4 jmode,ilen_mrc, ilen_pif
      integer*4 kxyz(3),mxyz(3)
      integer*8 imgOffset
      integer*4 img_size
      integer*4 img_plus_header
      integer*4 nbytes
      integer*4 skip_fl /0/
      real*4    dmin, dmax, dmean
      real*4    glob_max,glob_min,glob_scale
      real*4    add_data_factor, scale_data_factor, max_int
      real*4    new_glob_max
      common//ncol,nrow,nsec
      equivalence (ncol,kxyz)
      
C     -----------------------------------------------------------
C     Standard MRC imcom:
      character*4  cstamp/'MAP'/
      integer*4 karch(5), labls(20,10,5), lstream(12), mapcrs(3,5)
      integer*4 mode(5), nb(5), nbsym(5), ncrs(3,5), ncrst(3,5)      
      integer*4 nlab(5), nspg(5), nxyz(3,5)
      integer*4 nbhdr, nbw, nbw3, nbl, numopen
      logical   flag(5), old(5), nocon(5), unknown(5)
      real*4    cel(6,5), denmmm(3,5), rstamp, stuff(25,5)
      real*4    origin(3,5), stamp(2,5), rms(5)

      common/imcom/ lstream, nbhdr, nbw, nbw3, nb, nbl, ncrs, mode,
     $     ncrst, nxyz, cel, mapcrs, denmmm, nspg, nbsym, stuff,
     $     origin, stamp, rms, nlab, labls, numopen, karch,
     $     nocon, flag, old, unknown
      
      equivalence      (cstamp,rstamp)
C     -----------------------------------------------------------

      iunit_in   =  7
      iunit_out  =  11
      glob_max   = -1.e38
      glob_min   =  1.e38
      glob_scale =  1.e38

      call banner('MRC2PIF\0')  
      write(*,*)'********    MRC2PIF    ********'
      write(*,*)'*** Maps and stacks of images
     1 are forced to ODD dimension ****'

c
c----Read input data from user or optionally from command line
c

      if(iargc() < 2) then
 12      write(*,'(a,$)') 'MRC (CCP4) Filename (e.g. file.mrc): '
         read (*,'(a)', err=12) MRCname
 13      write(*,'(a,$)') 'PIF Filename (e.g. file.pif): '
         read (*,'(a)', err=13) PIFname
      else
         call getarg(1,MRCname)
         call getarg(2,PIFname)
      endif

      ilen_mrc = min(len_trim(MRCname), 256)
      if(ilen_mrc.gt.256) then
         write(*,*)'MRC (CCP4) file name too long'
         stop
      else
         do i = ilen_mrc+1, 256
            MRCname(i:i) = ' '
         end do
      end if
      
      ilen_pif = min(len_trim(PIFname), 256)
      if(ilen_pif.gt.256) then
         write(*,*)'PIF file name too long'
         stop
      else
         do i = ilen_pif+1, 256
            PIFname(i:i) = ' '
         end do
      end if

      call pif_init_head (PIFhead)      

      call imopen(iunit_in,MRCname(1:ilen_mrc),'ro')
        call qqinq(lstream(iunit_in),MRCname(1:ilen_mrc),
     1    fullname,nfilsz)
      call imclose(iunit_in)
      call imopen(iunit_in,MRCname(1:ilen_mrc),'ro')
c      call imopen(iunit_in,MRCname(1:ilen_mrc),'old')
c     
c   read header of the map files, determine memory requirements
c
c       irdhdr skips symmetry bytes 
      call irdhdr(iunit_in,kxyz,mxyz,jmode,dmin,dmax,dmean)

      if      (jmode > 2) then
         write(*,*)'Not dealing with transforms!'
         stop
      else if (jmode == 0) then
         nbytes = 1
      else if (jmode == 1) then
         nbytes = 2
      else if (jmode == 2) then
         nbytes = 4
      else
         write(*,*)"Mode not implemented yet:",jmode
         stop
      end if
c       If CCP4 header was incorrectly written 
c       (e.g. with nbsym=0 but they are actually present in the file), 
c       skip the difference between calculated and actual file size
      img_size = ncol*nrow*nsec*nbytes + 1024   !Image + header
      if(img_size .lt. nfilsz) then
         call irtsym(iunit_in,kspg,kbs)
         if(kbs .eq. 0) then
            kbs = nfilsz-img_size
            kspg = 1
            call qskip(lstream(iunit_in),kbs)
            skip_fl = 1
         end if
      end if

      if(nsec > 1) then
         write(*,'(a,$)')"2D/3D [2D]? "
         read (*,'(a)')two_d
      end if

      if  ((two_d .ne. 'n') .and.
     1         (two_d .ne. 'N') .and.
     2         (two_d .ne. '3')) then
         is2D = .true.
         if (nsec>1) then
             forceOdd = .true.                           !boxed images
         else
             forceOdd = .false.                          !micrograph
         end if
      else
         is2D = .false.
         forceOdd = .true.                               !map
      end if

C Determine the output mode
      if     (jmode == 0) then
         if(nsec > 1) then
            if  (is2D) then
               jmode = 6                                 !boxed images
            else
               jmode = 20                                !3d map, put it in I2
            end if
         else
            jmode = 0                                    !scanned(?) image
         end if
      else if(jmode == 1) then
         if(nsec > 1) then
            if  (is2D) then
             jmode = 7                                 !boxed images
            else
               jmode = 20                                !3d map
            end if
         else
            jmode = 1                                    !2d image
         end if
      else if(jmode == 2) then
         if(nsec > 1) then
            if  (is2D) then
             jmode = 46                                !Set of two_d R4 boxes
            else
             jmode = 20                                !3d map, put it in I2
            end if
         else
            jmode = 2 !46                                    !2d image
         end if
      else
         write(*,*)'Wrong mode:',jmode
         stop
      end if
      write(*,*)
      write(*,*)
      write(*,*)' PIF output mode:', jmode


C  *************************************************************
C  * Allocate core memory for input image/map
C  *************************************************************

C     Always real
      allocate(map_in(ncol*(nrow+2)), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate MAP_IN'
         stop
      endif

      ncol_n = ncol
      nrow_n = nrow
      nsec_n = nsec
      if (forceOdd) then
         if((ncol/2)*2 == ncol) then
            Pifhead%mrcX = 1
            ncol_n = ncol + 1
         end if
         if((nrow/2)*2 == nrow) then
            Pifhead%mrcY = 1
            nrow_n = nrow + 1
         end if
C      Make map of odd dimensions, otherwise leave number of images as input
         if(.not. is2D .and. (nsec/2)*2 == nsec) then
            Pifhead%mrcZ = 1
c            Pifhead%even = 1
            nsec_n = nsec + 1
         end if
      end if

C     Always real
      allocate(map_out(ncol_n*(nrow_n+2)), stat=alloc_stat)
      if (alloc_stat > 0) then
         write(*,*) 'Unable to allocate MAP_OUT'
         stop
      endif

      if(jmode == 0 .or. jmode == 6) then
         nbytes = 1
         allocate(map_in_bt(ncol_n*nrow_n), stat=alloc_stat)
         if (alloc_stat > 0) then
            write(*,*) 'Unable to allocate MAP_IN_BT'
            stop
         endif
      else if(jmode == 1 .or. jmode == 7) then
         allocate(map_in_st(ncol_n*nrow_n), stat=alloc_stat)
         if (alloc_stat > 0) then
            write(*,*) 'Unable to allocate MAP_IN_ST'
            stop
         endif
         nbytes = 2
      end if
      
C  *****************************************
C  *    Set up pif format map output file  *
C  *****************************************
       
      Pifhead%lunum = iunit_out
      rw(1)  = 'w'
      rw(2)  = '+'
      call pif_open (PIFname(1:ilen_pif), Pifhead%lunum, rw)
      Pifhead%binfactor  =  1
      Pifhead%mode       = jmode

      Pifhead%numcols    = ncol_n                    ! unbinned size
      Pifhead%numrows    = nrow_n                    ! unbinned size

      if ( Pifhead%mode .eq. 46 .or.
     1 Pifhead%mode .eq. 6 .or.
     2 Pifhead%mode .eq. 7 ) then ! dgm addition
        Pifhead%packRadius = ( ncol_n - 1 ) / 2
      endif

      if  (is2D) then
         Pifhead%numsecs = 1                                ! 2D case
      else
         Pifhead%numsecs    = nsec_n                    ! 3D
      end if

C   Many parameters in the PIFhead are not setup, should check if
C   these parameters will influence later calculations
 
      Pifhead%a_cell     = ncol 
      Pifhead%b_cell     = nrow 
      Pifhead%c_cell     = nsec

      if  (is2D) then
         Pifhead%nimg = nsec_n
      else
         Pifhead%nimg = 1                               !MRC has usually one image/file
      end if
c      write(*,*)two_d,pifhead%nimg,nsec_n             !Debug

C     
C     Read all sections from the input map files
C     
      write (*,*)
      write (*,*)' Read MRC map in core, write PIF map out...'

      kstep = nint(nsec/10.0)
      if(kstep.le.0) kstep = 1

      imgOffset = 1024
      img_size = ncol_n*nrow_n
      img_plus_header = nbytes*img_size + 512
      
c       Initialize map_out
      do i = 1, img_size
         map_out(i) = 0.
      end do
      
c       Let libPIF know that you're writing floats out (zero means float,
c       1 means i2 integers)
      if(pifhead%mode == 20) then
         call pif_set_i2Map(1)
      else
         call pif_set_i2Map(0)
      endif

c       Determine scaling first
      do k = 1,nsec
         call irdsec(iunit_in,map_in,*99)
c       Stuff input map into the output of possibly different dimensions:
         j = 0
         l = 0
         m = 0
         n = 0
         do i = 1, img_size
            j = j + 1
            if(j > ncol_n) then
             j = 1
             l = l + 1
            end if
            if(j <= ncol) then
             m = m + 1
             if(m > ncol) then
                m = 1
                n = n + 1
                if(n >= nrow) n=nrow-1
             end if
             map_out(j+l*ncol_n) = map_in(m+n*ncol)
            end if
         end do
         call determine_scaling(PIFhead,map_out,ncol_n,nrow_n,1)
         if(Pifhead%SCALEF < glob_scale)glob_scale = Pifhead%SCALEF
         if(Pifhead%datar4_max > glob_max)glob_max = Pifhead%datar4_max
         if(Pifhead%datar4_min < glob_min)glob_min = Pifhead%datar4_min
      end do
c       Close and reopen MRC file
      call imclose(iunit_in)
      call imopen(iunit_in,MRCname(1:ilen_mrc),'ro')
c       Skips symmetry bytes as well
      call irdhdr(iunit_in,kxyz,mxyz,jmode,dmin,dmax,dmean)
c       If CCP4 header was incorrectly written 
c       (e.g. with nbsym=0 but they are actually present in the file), 
c       skip the difference between calculated and actual file size
      if(skip_fl .ne. 0) then
         call qskip(lstream(iunit_in),kbs)
         write (*,*)kbs," symmetry bytes skipped"
      end if
      Pifhead%datai4_max = int(glob_max)
      Pifhead%datai4_min = int(glob_min)
      Pifhead%datar4_max = glob_max
      Pifhead%datar4_min = glob_min
      Pifhead%SCALEF     = glob_scale
      Pifhead%recipFloatScaleFactor = 1./Pifhead%SCALEF
      scale_data_factor = 1.
      add_data_factor = 0.
      if ( nbytes == 1 ) then
          Pifhead%SCALEF     = 1.
          Pifhead%recipFloatScaleFactor = 1.
      endif
c scale float data to use all the positive integer range used to store them
      if (Pifhead%mode==2 .or. Pifhead%mode==46 
     1 .or. Pifhead%mode==20) then !float data to int
          if ( Pifhead%mode==2 .or. Pifhead%mode==46 ) then
              max_int = 2147483647.
          else
              max_int = 32767.
          end if
          add_data_factor = - glob_min
          new_glob_max = (glob_max - glob_min)
          scale_data_factor = max_int / (new_glob_max*Pifhead%SCALEF)
     1        / 1.05
          Pifhead%datai4_max = int(new_glob_max*scale_data_factor)
          Pifhead%datai4_min = 0
          Pifhead%datar4_max = new_glob_max*scale_data_factor
          Pifhead%datar4_min = 0
      end if
      call pif_write_gh (PIFhead) 
c      write (*,*)'scale_data_factor', scale_data_factor
c      write (*,*)'scale',Pifhead%SCALEF,'min',glob_min,'max',glob_max    !Debug

      Pifhead%imgnum = 1
      call pif_write_dh (PIFhead)
c      call pif_list_head (PIFhead)

c       Main loop
      do k = 1,nsec_n
         if(k <= nsec) then
            call irdsec(iunit_in,map_in,*99)
c       Stuff input map into the output of possibly different dimensions:
            j = 0
            l = 0
            m = 0
            n = 0
            do i = 1, img_size
             j = j + 1
             if(j > ncol_n) then
                j = 1
                l = l + 1
             end if
             if(j <= ncol .and. l <= nrow) then
                m = m + 1
                if(m > ncol) then
                   m = 1
                   n = n + 1
                   if(n > nrow) n=nrow
                end if
                map_out(j+l*ncol_n) = (add_data_factor+map_in(m+n*ncol))
     1                                *scale_data_factor
             end if
            end do
            if  (is2D) then
             Pifhead%imgNum  =  k
             
             if(Pifhead%mode == 0 .or. Pifhead%mode == 6) then
                do i = 1, img_size
                   map_in_bt(i) =  int(map_out(i))+128 ! add 128 to convert to unsigned
                end do
                call pif_write_byte_image (Pifhead%luNum,map_in_bt,
     1               imgOffset,ncol_n,nrow_n,1)
             else if((Pifhead%mode == 1) .or. (Pifhead%mode == 7)) then
                do i = 1, img_size
                   map_in_st(i) = int(map_out(i))
                end do
                call pif_write_short_image(Pifhead%luNum,map_in_st,
     1             imgOffset,ncol_n,nrow_n,1)
             else if((Pifhead%mode == 2) .or. (Pifhead%mode == 46)) then
                call pif_write_map_sections(PIFhead, map_out, 
     $               PIFhead%mode, PIFhead%numCols, PIFhead%numRows,
     $               PIFhead%numSecs, 0, 1)
             end if
             imgOffset = imgOffset + img_plus_header !Calculate new image offset
             if(k .ne. nsec_n) then
                Pifhead%imgNum  =  k + 1 !Put right image number
                call pif_write_dh (PIFhead) !Write header for next image
             end if
            else
             call pif_write_map_sections(PIFhead, map_out, PIFhead%mode,
     $              ncol_n, nrow_n, nsec_n, k-1, 1)
            end if
            if(mod(k,kstep) == 0) then
             percent = nint(k*100./nsec)
             write(*,'("+",i3,''% complete'')') percent
            end if
         else                  !Write out an extra section to make it odd
            call pif_write_map_sections(PIFhead, map_out, PIFhead%mode,
     $           ncol_n, nrow_n, nsec_n, k-1, 1)
            if(mod(k,kstep) == 0) then
             percent = nint(k*100./nsec)
             write(*,'("+",i3,''% complete'')') percent
            end if
         end if
      end do

      write(*,*)'Done'

C     Close files
      call pif_close(Pifhead%lunum)
      call imclose(iunit_in)

C     Free memory
      if(allocated(map_in))    deallocate(map_in)
      if(allocated(map_out))   deallocate(map_out)
      if(allocated(map_in_st)) deallocate(map_in_st)
      if(allocated(map_in_bt)) deallocate(map_in_bt)
      
C     If irdsec fails, code automatically jumps to label 99
C     Jump over this line if program terminates normally

      go to 9999
 99   write(*,*)"Premature end of file on input"

 9999 end
       
