C  ******************************************************************
C  *                                                                *
C  *                 *****     PIF  -> CCP4     *****               *
C  *                                                                *
C  *    LOGICAL UNITS: 7    = INPUT PIF FILE                        *
C  *                   11   = OUTPUT  CCP4 FILE                      *
C  *    MBS, Jan. 2003                                              *
C  *    Latest changes MBS, Sep. 2003 (Made output of odd dimns)    *
C  *                                                                *
C  ******************************************************************
C
      implicit none
C
      include '../include/pifhead.inc'
      type (pifhead_t) PIFhead
c
      real*8 atof_fstr
      
      character(len=1) :: rw(2)
      integer iargc
      integer alloc_stat
      integer*4 i, j, k, l, m, n, kstep
      integer*4 ncol, nrow, nsec
      integer*4 nx, ny, nz
      integer*8 imgOffset
      integer*4 img_size
      integer*4 img_plus_header
      integer*4 percent
      integer*4 kspg,kbs
      integer*4 invert_flag
      character(len=1) :: yes
      character(len=10) :: spxlsize
      character(len=256) :: CCP4name, PIFname

CCC      byte, allocatable      :: map_in_bt(:)
      integer*4, allocatable :: map_in_st(:)
      real*4, allocatable    :: map_in(:)
      real*4, allocatable    :: map_out(:)

      integer*4 iunit_in,iunit_out
      integer*4 jmode, ilen_mrc, ilen_pif
      integer*4 nxyzst(3), mxyz(3)
      real*4    dmin, dmax, dmean
      real*4    tmin, tmax, tmean
      real*4    pxlsize
      common//nx,ny,nz
      equivalence (nx,nxyzst)
      integer*4 nbytes
      integer*4 labels(20,10)
      character(len=80) :: label
      real*4    cell(6)

C     -----------------------------------------------------------
C     Standard MRC imcom:
      character*4  cstamp/'MAP'/
      integer*4 karch(5), labls(20,10,5), lstream(12), mapcrs(3,5)
      integer*4 mode(5), nb(5), nbsym(5), ncrs(3,5), ncrst(3,5)      
      integer*4 nlab(5), nspg(5), nxyz(3,5)
      integer*4 nbhdr, nbw, nbw3, nbl, numopen
      logical   flag(5), old(5), nocon(5), unknown(5)
      real*4    cel(6,5), denmmm(3,5), rstamp, stuff(25,5)
      real*4    origin(3,5), stamp(2,5), rms(5)

      common/imcom/ lstream, nbhdr, nbw, nbw3, nb, nbl, ncrs, mode,
     $     ncrst, nxyz, cel, mapcrs, denmmm, nspg, nbsym, stuff,
     $     origin, stamp, rms, nlab, labls, numopen, karch,
     $     nocon, flag, old, unknown
      
      equivalence      (cstamp,rstamp)
C     -----------------------------------------------------------

      iunit_in  = 7
      iunit_out = 11

      write(*,'(a)')'********    PIF2CCP4    ********'
      write(*,'(a)')
     1    '***   Will reduce images/maps to original dimensions  ****'

c
c----Read input data from user
c
      if(iargc() < 1) then
 12      write(*,'(a,$)') 'PIF Filename: '
         read (*,'(a)', err=12) PIFname
      else
         call getarg(1,PIFname)
      endif
      
      if(iargc() < 2) then
 13      write(*,'(a,$)') 'CCP4 Filename: '
         read (*,'(a)', err=13) CCP4name
      else
         call getarg(2,CCP4name)
      endif
      
      if(iargc() < 3) then
         write(*,'(a,$)') 'Pixel size <1.0> A/pxl: '
         read (*,*) pxlsize
         if(pxlsize == 0.0) pxlsize = 1.0
      else
         call getarg(3,spxlsize)
         pxlsize = real(atof_fstr(spxlsize))
      endif
      
      invert_flag = 1
      if(iargc() < 4) then
         write(*,'(a,$)') 'Invert contrast (y or n)? <y> '
         read (*,'(a)') yes
      else
         call getarg(4,yes)
      endif
      
      if(yes == 'n' .or. yes == 'N') invert_flag = 0

      ilen_pif = len_trim(PIFname)
      if(ilen_pif > 256) then
         write(*,*)'PIF file name too long'
         stop
      else
         do i = ilen_pif+1, 256
            PIFname(i:i) = ' '
         end do
      end if

      ilen_mrc = len_trim(CCP4name)
      if(ilen_mrc > 256) then
         write(*,*)'CCP4 file name too long'
         stop
      else
         do i = ilen_mrc+1, 256
            CCP4name(i:i) = ' '
         end do
      end if
      
      write(*,*) 'PIFname:  ', trim(PIFname)
      write(*,*) 'CCP4name: ', trim(CCP4name)
      write(*,*) 'pxlsize:  ', pxlsize
      write(*,*) 'yes:      ', yes

      rw(1) = 'r' 
      Pifhead%lunum = iunit_in

      rw(1) = 'r' 
      rw(2) = achar(0)
      call pif_open (PIFname(1:ilen_pif), Pifhead%lunum, rw)
c     
c   read PIF header, determine memory requirements
c
      call pif_read_gh (PIFhead)

      if(Pifhead%htype .ne. 1) then
         write(*,*)"Could not convert stack of different sizes"
         stop
      end if
      ncol = Pifhead%numcols
      nrow = Pifhead%numrows
      nsec = Pifhead%numsecs
      jmode = Pifhead%mode
      if(Pifhead%mrcX == 1) then
         ncol = ncol - 1
      end if
      if(Pifhead%mrcY == 1) then
         nrow = nrow - 1
      end if
      if(Pifhead%mrcZ == 1) then
         nsec = nsec - 1
      end if

      if((jmode .ne. 20) .and.(jmode .ne. 21)) nsec = Pifhead%nimg
      nx = ncol
      ny = nrow
      nz = nsec
         
      if(jmode > 4) then
         if(     jmode ==  6) then
            jmode = 0
         else if(jmode ==  7) then
            jmode = 1
         else if(jmode ==  8) then
            jmode = 3
         else if(jmode ==  9) then
            jmode = 2
         else if(jmode == 10) then
            jmode = 4
         else if(jmode == 20) then
            jmode = 2
         else if(jmode == 21) then
            jmode = 2
         else if(jmode == 46) then
            jmode = 2
         else
            write(*,*)"Could not convert files with that mode:", jmode
            stop
         end if
      end if

C  *************************************************************
C  * Allocate core memory for maps: output map is always r*4 (CCP4 req);
C  * Input size depends on the PIF mode
C  *************************************************************

      allocate(map_in(Pifhead%numcols*Pifhead%numrows),stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate map_in'
         stop
      endif

      allocate(map_out(ncol*nrow), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate map_out'
         stop
      endif

      map_in = 0.0              ! f90 array syntax
      map_out = 0.0             ! f90 array syntax

      if(jmode .ne. 2) then
         if(jmode == 0) then
CCC   nbytes = 1
CCC   allocate(map_in_bt(Pifhead%numcols*Pifhead%numrows),
CCC   1           stat=alloc_stat)
CCC   if(alloc_stat > 0) then
CCC   write(*,*) 'Unable to allocate map_in_bt'
CCC   stop
CCC   endif
         else if (jmode == 1) then
            allocate(map_in_st(Pifhead%numcols*Pifhead%numrows),
     1           stat=alloc_stat)
            if(alloc_stat > 0) then
               write(*,*) 'Unable to allocate map_in_st'
               stop
            endif
            nbytes = 2
CCC   else if (jmode == 3) then
CCC   write(*,*)"Warning: not fully tested yet!"
CCC   map_out_p = malloc (4*(ncol+2)*nrow)
CCC   nbytes = 4
CCC   else if (jmode == 4) then
CCC   write(*,*)"Warning: not fully tested yet!"
CCC   map_out_p = malloc (8*(ncol+2)*nrow)
CCC   nbytes = 8
         else
            write(*,*)"Mode not implemented yet:",jmode
            stop
         end if
      end if
CCC   if (map_out_p ==0) then
CCC   write(*,'(a)')'Could not allocate memory for CCP4'
CCC   stop
CCC   end if
      kstep = nint(nsec/10.0)
      if(kstep.le.0) kstep = 1
      Pifhead%imgnum = 1
      call pif_read_dh (PIFhead)


C  ******************************************
C  *    Set up CCP4 format map output file   *
C  ******************************************
      call imopen(iunit_out,CCP4name(1:ilen_mrc),'unknown')

CCC   call date(dat)
CCC   call time(tim)
CCC   encode(80,1500,label) dat,tim
CCC   1500      format('Created with PIF2CCP4',7x,a9,2x,a8)
      
      label = 'Created with PIF2CCP4'

      dmin = 0.
      dmax = 0.
      dmean = 0.
      if(Pifhead%isTrunc > 0) then
         mxyz(1)    = Pifhead%origX
         mxyz(2)    = Pifhead%origY
         mxyz(3)    = Pifhead%origZ
      else
         mxyz(1)    = nx
         mxyz(2)    = ny
         mxyz(3)    = nz
      end if
      call icrhdr(iunit_out,nxyzst,mxyz,jmode,labels,0)
c       Put symmetry group as 1 and symmetry bytes number as 80
      kspg  = 1
      kbs   = 80
C      Change unit cell dimensions
C      Change start column, row and section ##
c       Take data from pif header
      if(Pifhead%isTrunc > 0) then
         Pifhead%transX = Pifhead%transX + 1     !It looks better in O with the shift somehow
         Pifhead%transY = Pifhead%transY + 1
         Pifhead%transZ = Pifhead%transZ + 1
         ncrst(1,1) = Pifhead%transX - (Pifhead%origX+1)/2
         ncrst(2,1) = Pifhead%transY - (Pifhead%origY+1)/2
         ncrst(3,1) = Pifhead%transZ - (Pifhead%origZ+1)/2
         cell(1)    = Pifhead%origX*pxlsize
         cell(2)    = Pifhead%origY*pxlsize
         cell(3)    = Pifhead%origZ*pxlsize
      else
         ncrst(1,1) = Pifhead%transX - nx/2
         ncrst(2,1) = Pifhead%transY - ny/2
         ncrst(3,1) = Pifhead%transZ - nz/2
         cell(1)    = nx*pxlsize
         cell(2)    = ny*pxlsize
         cell(3)    = nz*pxlsize
      end if
      cell(4) = 90.
      cell(5) = 90.
      cell(6) = 90.
        call ialsym(iunit_out,kspg,kbs)
      call ialcel(iunit_out,cell)
C      Change origin
      call ialorg(iunit_out,0.,0.,0.)
c$$$      call ialorg(iunit_out,Pifhead%origX,Pifhead%origY,Pifhead%origZ)

      rms(lstream(iunit_out)) = Pifhead%dataR4_std

      call iwrhdr(iunit_out,label,1,dmin,dmax,dmean)

c      call pif_list_head (PIFhead)
C     
C     Read all sections from the input map files
C     
      write (*,*)
      write (*,*)' Read PIF in core, write CCP4 out...'

      dmin  =  1.e10
      dmax  = -1.e10
      dmean =  0.
      imgOffset = 1024
      img_size = Pifhead%numcols*Pifhead%numrows
      img_plus_header = nbytes*img_size + 512
      do k = 1,nsec
            Pifhead%imgNum  =  k
         if     ((Pifhead%mode == 0) .or.
     1         (Pifhead%mode == 6)) then
CCC   call pif_read_byte_image(Pifhead%luNum,map_in_bt,
CCC   1         imgOffset,Pifhead%numcols,Pifhead%numrows,1)
         else if((Pifhead%mode == 1) .or.
     1         (Pifhead%mode == 7)) then
            call pif_read_short_image(Pifhead%luNum,map_in_st,
     1         imgOffset,Pifhead%numcols,Pifhead%numrows,1)
         else if((Pifhead%mode == 2) .or.
     1         (Pifhead%mode == 9)) then
            call pif_read_imgi4 (PIFhead,map_in,
     1         Pifhead%numcols,Pifhead%numrows,k)
         else if((Pifhead%mode == 3) .or.
     1         (Pifhead%mode == 8)) then
            call pif_read_mapi2(PIFhead,map_in,k,k,
     1         Pifhead%numcols+1,Pifhead%numrows,Pifhead%numsecs)      !FFT      
         else if((Pifhead%mode == 4) .or.
     1         (Pifhead%mode == 10)) then
            call pif_read_mapi4(PIFhead,map_in,k,k,
     1         Pifhead%numcols+1,Pifhead%numrows,Pifhead%numsecs) !FFT
         else if(Pifhead%mode == 20) then
            call pif_read_mapi2(PIFhead,map_in,k,k,
     1         Pifhead%numcols,Pifhead%numrows,Pifhead%numsecs)
         else if((Pifhead%mode == 21) .or.
     1         (Pifhead%mode == 22)) then
            call pif_read_mapi4(PIFhead,map_in,k,k,
     1         Pifhead%numcols,Pifhead%numrows,Pifhead%numsecs)
         else if(Pifhead%mode == 46) then
            call pif_read_boxi4(PIFhead,map_in)
         else
            write(*,*)"Wrong mode",jmode
            stop
         end if
         imgOffset = imgOffset + img_plus_header !Calculate new image offset

         if(jmode == 0) then
CCC            do i = 1, img_size
CCC             map_in(i) = map_in_bt(i)
CCC            end do
         else if(jmode == 1) then
            do i = 1, img_size
             map_in(i) = map_in_st(i)
            end do
         end if
c       Stuff input map into the output of possibly smaller dimensions:
            j = 0
            l = 0
            m = 0
            n = 0
            do i = 1, img_size
             m = m + 1
             if(m > Pifhead%numcols) then
                m = 1
                n = n + 1
             end if
             if(m.le.ncol .and. n.le.nrow) then
                j = j + 1
                if(j > ncol) then
                   j = 1
                   l = l + 1
                   if(l.ge.nrow) l=nrow-1
                end if
                if(invert_flag == 0) then
                   map_out(j+l*ncol) = map_in(m+n*Pifhead%numcols)
                else
                   map_out(j+l*ncol) = -map_in(m+n*Pifhead%numcols)
                end if
             end if
            end do

         if(jmode .le. 2) then
            call iclden(map_out,ncol,nrow,1,ncol,1,nrow,
     1         tmin,tmax,tmean)
         else
            call iclcdn(map_out,ncol/2+1,nrow,1,ncol/2,1,nrow,
     1         tmin,tmax,tmean)
         end if
c$$$void calcMapStats_(float *inbuffer, float *img_bkg, float *map_min,
c$$$        float *map_max, float *img_avg, float *img_std,
c$$$        int *width, int *height, int *nSecs)

         dmin = min(dmin,tmin)
         dmax = max(dmax,tmax)
         dmean = dmean + tmean
         call iwrsec(iunit_out,map_out)
         if(mod(k,kstep) == 0) then
            percent = nint(k*100./nsec)
            write(*,'("+",i3,''% complete'')') percent
         end if
      end do
      dmean = dmean/nsec
      call iwrhdr(iunit_out,label,-1,dmin,dmax,dmean)

      write(*,*)'Done'

      if (allocated(map_out))   deallocate(map_out)
CCC   if allocated(map_in_bt) deallocate(map_in_bt)
      if (allocated(map_in_st)) deallocate(map_in_st)
      
C     Close files
      call pif_close(Pifhead%lunum)
      call imclose(iunit_out)

      end
