C  ******************************************************************
C  *                                                                *
C  *                 *****     PIF  -> MRC      *****               *
C  *                                                                *
C  *    LOGICAL UNITS: 7    = INPUT PIF FILE                        *
C  *                   11   = OUTPUT  MRC FILE                      *
C  *    MBS, Jan. 2003                                              *
C  *    Latest changes MBS, Sep. 2003 (Made output of odd dimns)    *
C  *                                                                *
C  ******************************************************************
C
      implicit none
C
      include '../include/pifhead.inc'
      type(pifhead_t) Pifhead
c
      character(len=1) rw(2)
      integer iargc
      integer*4 i, j, k, l, m, n, kstep
      integer*4 ncol, nrow, nsec
      integer*4 nx, ny, nz
      integer*8 imgOffset
      integer*4 img_size
      integer*4 img_plus_header
      integer*4 percent
      character MRCname*64,PIFname*64

      integer   alloc_stat
      integer*1, allocatable :: map_in_bt(:)
      integer*2, allocatable :: map_in_st(:)
      real*4, allocatable    :: map_in(:)
      real*4, allocatable    :: map_out(:)

      integer*4 iunit_in, iunit_out
      integer*4 mode, ilen_mrc, ilen_pif
      integer*4 nxyz(3)
      integer*4 nxyzst(3)
      data      nxyzst/3*0/
      real*4    dmin, dmax, dmean
      real*4    tmin, tmax, tmean
      common//nx,ny,nz
      equivalence (nx,nxyz)
      integer*4 nbytes
      integer*4 labels(20,10)
      character label*80
      
      iunit_in  = 7
      iunit_out = 11

      write(*,*)'********    PIF2MRC    ********'
      write(*,*)
     1    '***   Will reduce images/maps to original dimensions  ****'

c
c----Read input data from user
c

      if(iargc() < 2) then
 12      write(*,'(a,$)') 'PIF Filename (e.g. file.pif): '
         read (*,'(a)', err=12) PIFname
 13      write(*,'(a,$)') 'MRC Filename (e.g. file.mrc): '
         read (*,'(a)', err=13) MRCname
      else
         call getarg(1,PIFname)
         call getarg(2,MRCname)
      endif

      ilen_pif = min(len_trim(PIFname), 64)
      if(ilen_pif.gt.64) then
         write(*,*)'PIF file name too long'
         stop
      else
         do i = ilen_pif+1, 64
            PIFname(i:i) = ' '
         end do
      end if

      ilen_mrc = min(len_trim(MRCname), 64)
      if(ilen_mrc.gt.64) then
         write(*,*)'MRC file name too long'
         stop
      else
         do i = ilen_mrc+1, 64
            MRCname(i:i) = ' '
         end do
      end if
      
      Pifhead%lunum = iunit_in
      rw(1) = 'r' 
      rw(2) = achar(0)
      call pif_open (PIFname(1:ilen_pif), Pifhead%lunum, rw)
c     
c   read PIF header, determine memory requirements
c
      call pif_read_gh (Pifhead)

      if(Pifhead%htype .ne. 1) then
         write(*,*)"Could not convert stack of different sizes"
         stop
      end if
      ncol = Pifhead%numcols
      nrow = Pifhead%numrows
      nsec = Pifhead%numsecs
      mode = Pifhead%mode
c      if(Pifhead%even == 1) then
      if(Pifhead%mrcX .eq. 1) then
         ncol = ncol - 1
      end if
c      if(Pifhead%even == 1) then
      if(Pifhead%mrcY .eq. 1) then
         nrow = nrow - 1
      end if
c      if(Pifhead%even == 1) then
      if(Pifhead%mrcZ == 1) then
         nsec = nsec - 1
      end if

      if((mode .ne. 20) .and.(mode .ne. 21)) nsec = Pifhead%nimg
      nx = ncol
      ny = nrow
      nz = nsec
         
      if(mode > 4) then
         if(     mode ==  6) then
            mode = 0
         else if(mode ==  7) then
            mode = 1
         else if(mode ==  8) then
            mode = 3
         else if(mode ==  9) then
            mode = 2
         else if(mode == 10) then
            mode = 4
         else if(mode == 20) then
            mode = 2
         else if(mode == 21) then
            mode = 2
         else if(mode == 46) then
            mode = 2
         else
            write(*,*)"Could not convert files with that mode:", mode
            stop
         end if
      end if

C  *************************************************************
C  * Allocate core memory for maps: output map is always r*4 (MRC req);
C  * Input size depends on the PIF mode
C  *************************************************************

      allocate(map_in(Pifhead%numcols*Pifhead%numrows), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate MAP_IN'
         stop
      endif

      allocate(map_out(ncol*nrow), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate MAP_OUT'
         stop
      endif

      nbytes = 0
      if(mode .ne. 2) then
         if(mode == 0) then
            nbytes = 1
            allocate(map_in_bt(Pifhead%numcols*Pifhead%numrows),
     $           stat=alloc_stat)
            if (alloc_stat > 0) then
               write(*,'(a)')'Could not allocate memory for PIF'
               stop
            end if
         else if (mode == 1) then
            allocate(map_in_st(Pifhead%numcols*Pifhead%numrows),
     $           stat=alloc_stat)
            if (alloc_stat > 0) then
               write(*,'(a)')'Could not allocate memory for PIF'
               stop
            end if
            nbytes = 2
         else if (mode == 3) then
            write(*,*)"Warning: not fully tested yet!"
            allocate(map_out((ncol+2)*nrow))
            nbytes = 4
         else if (mode == 4) then
            write(*,*)"Warning: not fully tested yet!"
            allocate(map_out(2*(ncol+2)*nrow))
            nbytes = 8
         else
            write(*,*)"Mode not implemented yet:",mode
            stop
         end if
      end if

      kstep = nint(nsec/10.0)
      if(kstep.le.0) kstep = 1


C  ******************************************
C  *    Set up MRC format map output file   *
C  ******************************************
      write(*,*)
      write(*,*) 'Ignore "UNKNOWN status" warning from MRC library'
      write(*,*)
      call imopen(iunit_out,MRCname(1:ilen_mrc),'unknown')
      call icrhdr(iunit_out,nxyz,nxyz,mode,labels,0)

      label = 'Created with PIF2MRC'

      dmin = 0.
      dmax = 0.
      dmean = 0.

      call iwrhdr(iunit_out,label,-1,dmin,dmax,dmean)

C     Read all sections from the input map files

      write (*,*)
      write (*,*)' Read PIF in core, write MRC out...'

      dmin  =  1.e10
      dmax  = -1.e10
      dmean =  0.
      imgOffset = 1024
      img_size = Pifhead%numcols*Pifhead%numrows
      img_plus_header = nbytes*img_size + 512
      do k = 1,nsec
         Pifhead%imgNum  =  k
         if     (Pifhead%mode == 0 .or.Pifhead%mode == 6) then
            call pif_read_byte_image(Pifhead%luNum,map_in_bt,
     1           imgOffset,Pifhead%numcols,Pifhead%numrows,1)
         else if(Pifhead%mode == 1 .or. Pifhead%mode == 7) then
            call pif_read_short_image(Pifhead%luNum,map_in_st,
     1           imgOffset,Pifhead%numcols,Pifhead%numrows,1)
         else if(Pifhead%mode == 2 .or. Pifhead%mode == 9) then
            call pif_read_imgi4 (Pifhead,map_in,
     1           Pifhead%numcols,Pifhead%numrows,k)
         else if(Pifhead%mode == 3 .or. Pifhead%mode == 8) then
            call pif_read_mapi2(Pifhead,map_in,k,k,
     1           Pifhead%numcols+1,Pifhead%numrows,Pifhead%numsecs) !FFT      
         else if(Pifhead%mode == 4 .or. Pifhead%mode == 10) then
            call pif_read_mapi4(Pifhead,map_in,k,k,
     1           Pifhead%numcols+1,Pifhead%numrows,Pifhead%numsecs) !FFT
         else if(Pifhead%mode == 20) then
            call pif_read_mapi2(Pifhead,map_in,k,k,
     1         Pifhead%numcols,Pifhead%numrows,Pifhead%numsecs)
         else if(Pifhead%mode == 21 .or. Pifhead%mode == 22) then
            call pif_read_mapi4(Pifhead,map_in,k,k,
     1         Pifhead%numcols,Pifhead%numrows,Pifhead%numsecs)
         else if(Pifhead%mode == 46) then
            call pif_read_boxi4(Pifhead,map_in)
         else
            write(*,*)"Wrong mode",mode
            stop
         end if
         imgOffset = imgOffset + img_plus_header !Calculate new image offset

         if(mode == 0) then
            do i = 1, img_size
               map_in(i) = map_in_bt(i)
            end do
         else if(mode == 1) then
            do i = 1, img_size
             map_in(i) = map_in_st(i)
            end do
         end if
c       Stuff input map into the output of possibly smaller dimensions:
            j = 0
            l = 0
            m = 0
            n = 0
            do i = 1, img_size
             m = m + 1
             if(m > Pifhead%numcols) then
                m = 1
                n = n + 1
             end if
             if(m <= ncol .and. n <= nrow) then
                j = j + 1
                if(j > ncol) then
                   j = 1
                   l = l + 1
                   if(l >= nrow) l=nrow-1
                end if
                map_out(j+l*ncol) = map_in(m+n*Pifhead%numcols)
             end if
            end do

         if(mode <=  2) then
            call iclden(map_out,ncol,nrow,1,ncol,1,nrow,
     $           tmin,tmax,tmean)
         else
            call iclcdn(map_out,ncol/2+1,nrow,1,ncol/2,1,nrow,
     $           tmin,tmax,tmean)
         end if
         dmin = min(dmin,tmin)
         dmax = max(dmax,tmax)
         dmean = dmean + tmean
         call iwrsec(iunit_out,map_out)
         if(mod(k,kstep) == 0) then
            percent = nint(k*100./nsec)
            write(*,'("+",i3,''% complete'')') percent
         end if
      end do
      dmean = dmean/nsec
      call iwrhdr(iunit_out,label,-1,dmin,dmax,dmean)

      write(*,*)'Done'

C     Free memory
      if(allocated(map_in))    deallocate(map_in)
      if(allocated(map_out))   deallocate(map_out)
      if(allocated(map_in_st)) deallocate(map_in_st)
      if(allocated(map_in_bt)) deallocate(map_in_bt)

C     Close files
      call pif_close(Pifhead%lunum)
      call imclose(iunit_out)

      end
