/******************************************************************************
** FileIO.c
** ========

Current : JF Conway     Fri 03-Mar-2000

Description
===========
File IO routines for CTFmix.

******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include "array.h"               // Array utility routines
#include "FileIO.h"
#include "pif.h"

// Local function declarations
// ===========================

/******************************************************************************
** OpenOutputPIFfile
** =================
*/
FILE *OpenOutputPIFfile ( char *fname
                        , PIF_FILE_HEADER  *pifHeader
                        , int dx, int dy
                        , int mode
			, int numImages
			)
{
  FILE *fptr;
  *pifHeader = PIF_NewPifFileHeader();
  pifHeader->numImages        = numImages;
  pifHeader->hType            = 1;	// all the same
  pifHeader->nx               = dx;
  pifHeader->ny               = dy;
  pifHeader->nz               = 1;
  pifHeader->mode             = mode;

  if((fptr = PIF_OpenForWrite(fname, pifHeader)) == NULL)
    return(NULL);

  return(fptr);
}
/******************************************************************************
** AppendPIFImage
** ==============
*/
int AppendPIFimage ( FILE *fptr
                   , PIF_FILE_HEADER  *pifHeader
                   , int mode
                   , void *array
                   , int dx, int dy
                   )
{
  PIF_IMAGE_HEADER imgHeader = PIF_NewPifImageHeader();
  int error;
  const long size = ((long) dx) * ((long) dy);
  float mean, stdv, min, max;

  if(pifHeader->hType == 1)  {	// all images the same mode and size
    if(mode != pifHeader->mode || dx != pifHeader->nx || dy != pifHeader->ny)  {
      printf("ERROR in AppendPIFimage: image does not match PIF file header\n");
      printf("   PIF file header has htype=1, so all images have constant mode and size.\n");
      if(mode != pifHeader->mode)
        printf("  Image mode (%d-%s) does not match PIF file header (%d-%s)\n"
                , mode, PIF_ModeString(mode)
                , pifHeader->mode, PIF_ModeString(pifHeader->mode)
                );
      else
        printf("  Image size (%d,%d) does not match PIF file header (%d,%d)\n"
                , dx, dy
                , pifHeader->nx, pifHeader->ny
                );
      return(ERROR);
    }
  }

  switch(mode)  {
    case kPIF_MODE_INT1:
    case kPIF_MODE_BOXED:
      {
        unsigned char lmin, lmax;
        GetStatsUByteArray(array, size, &lmin, &lmax, &mean, &stdv);
        min = lmin;
        max = lmax;
      }
      imgHeader.bkgnd = * ((unsigned char *) array);   // should bkgnd be float???
      break;
    case kPIF_MODE_INT2:
      {
        short lmin, lmax;
        GetStatsWordArray(array, size, &lmin, &lmax, &mean, &stdv);
        min = lmin;
        max = lmax;
      }
      imgHeader.bkgnd = * ((short *) array);   // should bkgnd be float???
      break;
    case kPIF_MODE_FLOATINT4:
    case kPIF_MODE_FLOAT4:
    case kPIF_MODE_FLOATINT2:
      GetStatsFloatArray(array, size, &min, &max, &mean, &stdv);
      imgHeader.bkgnd = * ((float *) array);   // should bkgnd be float???
      break;
    default:
      printf("ERROR in AppendPIFimage - mode %d (%s) not supported\n"
                          , imgHeader.mode
                          , PIF_ModeString(imgHeader.mode)
                          );
      return(ERROR);
  }

  imgHeader.nx   = imgHeader.mx = dx;
  imgHeader.ny   = imgHeader.my = dy;
  imgHeader.nz   = imgHeader.mz = 1;
  imgHeader.mode = mode;
  imgHeader.min  = min;
  imgHeader.max  = max;
  imgHeader.mean = mean;
  imgHeader.stdDev = stdv;

  error = PIF_WriteImage(fptr, pifHeader, &imgHeader, array);

  if(error != PIF_OK)
    return(ERROR);  
  return(OK);
}
