/****************************************************************************
** pif_private.h
** =============

This version: 03-Mar-2000 (v1.4.7)
James Conway, LMES/IBS, Grenoble, France (email: James.Conway@ibs.fr)
Modified from source code provided by Rob Ashmore, Purdue University, Indiana.

I got sick of all the function prototypes in PIF.c, so I broke them out into
this file.

****************************************************************************/

#ifndef __PIF_IO_PRIVATE__      // Make sure we only include this file once
#define __PIF_IO_PRIVATE__


#include "pif.h"

// Useful type abbreviations
// =========================

typedef unsigned char UCHAR;
typedef unsigned long ULONG;

/****************************************************************************
        P R I V A T E   F U N C T I O N S
        * 1) Miscellaneous
          3) I/O Routines - Write
          4) I/O Routines - Misc
          5) Datatype conversion

1) Miscellaneous
      TestIntIs32Bits
      EndianModeMismatch
      FindImageRawDepth
      FindImageFinalDepth
      FindImageDimensions
      CalcImageRawSizeInBytes
      CalcImageFinalSizeInBytes
      CopyPad
      StripPad
*/

static int    TestIntIs32Bits(void);
static int    EndianModeMismatch(const PIF_FILE_HEADER *pifHeader);
static int    FindImageRawDepth(int mode);
static
int    FindImageFinalDepth(int mode);
static
ULONG  FindImageDimensions(const PIF_IMAGE_HEADER *imgHeader);
static
ULONG  CalcImageRawSizeInBytes(const PIF_IMAGE_HEADER *imgHeader);
static
ULONG  CalcImageFinalSizeInBytes(const PIF_IMAGE_HEADER *imgHeader);
static
void   CopyPad(char *dest, const char *src, const int count);
static
void   StripPad(char *str);

/****************************************************************************
3) I/O Routines - Write
      WriteFileHeader
      WriteDepthCueColorMap
      WriteImageHeader
      WriteImageData
      WriteImageDataFloat
      ConvertAndWriteImageData
      ConvertAndWriteImageDataFloat
*/
static
int    WriteFileHeader( FILE *fptr                              // file pointer
                      , const PIF_FILE_HEADER *pifHeader        // file header info
                      );
static
int    WriteDepthCueColorMap( FILE *fptr                        // file pointer
                            , const PIF_FILE_HEADER *pifHeader  // file info
                            );
static
int    WriteImageHeader( FILE *fptr                             // file pointer
                       , const PIF_FILE_HEADER  *pifHeader      // file header info
                       , const PIF_IMAGE_HEADER *imgHeader      // image header info
                       );
static
int    WriteImageData( FILE *fptr                               // file pointer
                     , const PIF_FILE_HEADER  *pifHeader        // file header info
                     , const PIF_IMAGE_HEADER *imgHeader        // image header info
                     , void *buffer                             // image data
                     );
static
int    WriteImageDataFloat( FILE *fptr                          // file pointer
                          , const PIF_FILE_HEADER  *pifHeader   // file header info
                          , const PIF_IMAGE_HEADER *imgHeader   // image header info
                          , float *buffer                       // image data
                          );
static
int    ConvertAndWriteImageData( FILE *fptr
                               , const PIF_FILE_HEADER  *pifHeader
                               , const PIF_IMAGE_HEADER *imgHeader
                               , const ULONG count
                               , void *buffer
                               );
static
int    ConvertAndWriteImageDataFloat( FILE *fptr
                                    , const PIF_FILE_HEADER  *pifHeader
                                    , const PIF_IMAGE_HEADER *imgHeader
                                    , const ULONG count
                                    , float *buffer
                                    );

/****************************************************************************
4) I/O Routines - Misc
      WriteBufferToFile
*/

static
int    WriteBufferToFile( FILE *fptr
                        , void *buffer
                        , ULONG bytes
                        );

/****************************************************************************
5) Datatype conversion
      Char4toInt4
      Char4toFloatInt4
      Int4toChar4
      FloatInt4toChar4
      Char16toDouble
      ArraySwapBytesInt2
      SwapBytesInt4
      ArrayInt2toFloat4
      ArrayInt1toFloat4_Simple
      ArrayInt2toFloat4_Simple
      ArrayInt4toFloat4
      ArrayFloat4toInt4
      ArrayFloat4toInt2
      ArrayFloat4toInt1
*/

static
int    Char4toInt4( UCHAR const *cptr                   // 4-character array
                  , const PIF_FILE_HEADER *pifHeader    // file header info
                  );
static
float  Char4toFloatInt4( UCHAR const *cptr              // 4-character array
                       , const PIF_FILE_HEADER *pifHeader// file header info
                       );
static
void   Int4toChar4( int intValue                        // integer
                  , UCHAR *char4                        // place for output
                  );
static
void   FloatInt4toChar4( float floatValue               // float
                       , const PIF_FILE_HEADER *pifHeader// file header info
                       , UCHAR *char4                   // place for output
                       );
static
double Char16toDouble(char const *cptr);                // 16-character array
static
void   ArraySwapBytesInt2( void *wptr                   // array of 2-byte ints
                         , long count                   // number
                         );
static
int    SwapBytesInt4( int reverse );                    // 4-byte int
static
void   ArrayInt2toFloat4( void *array
                        , const ULONG count
                        , const PIF_FILE_HEADER *pifHeader
                        );

static
void   ArrayInt1toFloat4_Simple( void *array
                               , register const ULONG count
                               );
static
void   ArrayInt2toFloat4_Simple( void *array
                               , register const ULONG count
                               );
static
void   ArrayInt4toFloat4( void *array
                        , const ULONG count
                        , const PIF_FILE_HEADER *pifHeader
                        );
static
void   ArrayFloat4toInt4( register int *dest
                        , ULONG count
                        , float scale
                        , const float *src
                        );
static
void   ArrayFloat4toInt2( register short *dest
                        , ULONG count
                        , float scale
                        , const float *src
                        );
static
void   ArrayFloat4toInt1( register UCHAR *dest
                        , ULONG count
                        , float scale
                        , register const float *src
                        );


#endif

/***************************************************************************/
