/*****************************************************************************
** tiff2pif.c
** ==========
Convert tiff files (from Zeiss Phodis scanner) to PIF.

14-Jul-1997 - Add option to 'bin' down
19-Oct-2000 - Convert TIFF2BP to TIFF2PIF

*****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <math.h>

#include "FileIO.h"
#include "pif.h"
#include "TIFF_io.h"

#define VERSION "1.2"

//#define MAX(x,y)  (((x)>(y)) ? (x) : (y))  
//#define MIN(x,y)  (((x)<(y)) ? (x) : (y))

#define EQU(x,y) (((x)!=NULL &&(y)!=NULL &&(strcmp((x),(y))==0)))


#define FALSE (0)
#define TRUE (!FALSE)

//typedef unsigned char UBYTE;

int main(int argc, char **argv);
static
int  WritePIF(UBYTE *buffer, int dx, int dy, int bytesPerPixel, 
	      char *passFname);
static
int  ReadTiff(UBYTE **buffer, int *dx, int *dy, int *bytesPerPixel,
	      char *passFname);
//static
//void ConvertOD(UBYTE *buffer, const int dx, const int dy);
static
void BinPixels(UBYTE *buffer, int *dx, int *dy, int binFactor);
static
void ShortBinPixels(unsigned short *buffer, int *dx, int *dy, int binFactor);

/*****************************************************************************
** main
** ====
*/
int main(int argc, char **argv)
{
  UBYTE *buffer;
  int    i, invert, dx, dy, bytesPerPixel;
  char   pifStr[1024];
  char   *s1;
  int    binFactor;

  if(argc < 2) {/* usage */
    printf("tiff2pif error - too few args.\n");
    printf("Usage: tiff2pif tiff_file [pif_file] [binning]\n");
    printf("If pif_file not provided, obtain name from TIF file base\n");
    printf("If binning not provided, will be prompted to enter\n");
    exit(0);
  }/* usage */

  printf("tiff2pif - convert tiff format into PIF format\n");
  printf("==============================================\n");
  printf("Version %s compiled %s %s\n", VERSION, __DATE__, __TIME__);
  printf("JF Conway, LMES/IBS/Grenoble - James.Conway@ibs.fr\n");
  printf("\n");

  /* 1st command line arg - TIF file name */
  if(ReadTiff(&buffer, &dx, &dy, &bytesPerPixel, argv[1]) == 0) {
    exit(0);
  }

  /* 2nd command line arg (optional) - PIF file name
     if not provided, construct pif name from tif name */
  if(argc >= 3) {
    sprintf(pifStr,"%s",argv[2]);
  } else {
    sprintf(pifStr,"%s",argv[1]);
    s1 = (char *)strrchr(pifStr,'.');
    if(s1) {
      sprintf(s1,".pif");
    } else {
      sprintf(pifStr,"%s.pif",argv[1]);
    }
  }
  
  /* 3rd command line arg (optional) - binning factor
     if not provided, prompt user to enter value */
  if(argc >= 4) {
    binFactor = atoi(argv[3]);
  } else {
    printf("Bin pixels by integer factor of 1,2,3,...> ");
    scanf("%d", &binFactor);
  }

  if(binFactor <= 0) {
    printf("Invalid choice of pixel binning - must be positive integer\n");
    exit(0);
  }

  if(binFactor == 1)  {
    printf("- No 'binning' done\n");
  }

  if(argc >= 5) {
    invert = atoi(argv[4]);
  } else {
    printf("Invert image contrast? 1=invert, 0=no invert: ");
    scanf("%d", &invert);
  }

  printf("\n");
  printf("Input file  = %s\n", argv[1]);
  printf("Output file = %s\n", pifStr);
  printf("Bin factor  = %d\n", binFactor);
  printf("Invert      = %d\n", invert);
  printf("\n");
  
  //  ConvertOD(buffer,  dx,  dy);  // optional
  if(bytesPerPixel==2) {
    ShortBinPixels((unsigned short *)buffer, &dx, &dy, binFactor);  // optional
    int vmax, vmin;
    unsigned short *p = (unsigned short *)buffer;
    vmax = 0;
    vmin = 65535;
    for (i=0; i<dx*dy; i++) {
        if ((int)p[i] > vmax ) vmax = (int) p[i];
        else if ((int)p[i] < vmin ) vmin = (int)p[i];
    }
    short *pout = (short *)buffer;
    for (i=0; i<dx*dy; i++)
            pout[i] = (short) (32767./(float)(vmax-vmin)*(float)((int) p[i] - vmax));
//GCA: this would be the correct conversion to short, but robem only allows positive values
//            pout[i] = (short) (65535./(float)(vmax-vmin)*(float)((int) p[i] - vmax)+32767.);
  } else {
    BinPixels(buffer, &dx, &dy, binFactor);  // optional
  }

  if(invert >= 1) {
    if(bytesPerPixel==2) {
        short *p = (short *)buffer;
        for (i=0; i<dx*dy; i++)
            p[i] *= -1;
    } else {
        for (i=0; i<dx*dy; i++)
            buffer[i] *= -1;
    }
  }

  WritePIF(buffer, dx, dy, bytesPerPixel, pifStr);

  free(buffer);

  return(0);
}
/*********************************************************************
** WritePIF
** ========
*/
static
int WritePIF(UBYTE *buffer, int dx, int dy, int bytesPerPixel,
	     char *passFname)
{
  FILE *fptr;
  PIF_FILE_HEADER pifHeader;
  int mode;

  if(bytesPerPixel == 1)
    mode = kPIF_MODE_INT1;
  else if(bytesPerPixel == 2)
    mode = kPIF_MODE_INT2;
  else
    {      
      perror("I don't understand this mode.\n");
      exit(0);
    }

  do  {
    char fname[1024];
#ifdef OLDWAY
    printf("PIF filename ? ");
    scanf("%s", fname);
#else
    sprintf(fname,"%s",passFname);
#endif
    if((fptr = OpenOutputPIFfile(fname, &pifHeader, dx, dy, mode, 1)) == NULL)
      printf("Error opening PIF file for output\n");
    else
      break;
  } while(1);

  printf("- Writing PIF...\n");
  if(AppendPIFimage(fptr, &pifHeader, mode, buffer, dx, dy) != OK)  {
    printf("Error writing PIF file\n");
    fclose(fptr);
    return(0);
  }

  fclose(fptr);
  return(1);
}
/*********************************************************************
** ConvertOD
** =========
0 maps to 0, 255 maps to 255, OD is log(1/(255-pixel+1)).
First off, invert the input value (255-pixel), then add one to the transmission
value to get a scale 1-256, then we can take logs and avoid log of 0. And
conveniently, log(1) = 0 so one end of the output scale is nicely set. The
(negative) scale factor then returns us to 0-255 but inverts, and so we must
undo this inversion by subtracting from 255 because the OD scale runs backwards
to the transmission scale.
Also, rather than use log10 for each pixel, we create a 256-entry lookup table.
*/
/*static
void ConvertOD(UBYTE *buffer, const int dx, const int dy)
{
  char instr[1000];
  const double scale = 255.0 / log10(1.0/256.0); // rescale back to 0-255
  int i, x, y, tenth, nextPercent;
  UBYTE table[256];

  printf("Convert to OD values (y/n) ? ");
  scanf("%s", instr);
  if(toupper(instr[0]) != 'Y')  {
    printf("- No OD conversion done\n");
    return;
  }

// Set up table for mapping 8-bit tranmission value onto 8-bit OD value

  for(i=0; i<256; i++)
    table[i] = 255 - (UBYTE) (scale * log10(1.0 / (double) (255 - i + 1)));

// Convert transmissions into OD's

  printf("- OD conversion beginning:");
  tenth = dy / 10 + 1;	// make sure 10% or more, but not less
  nextPercent = 0;
  for(y=0; y<dy; y++)  {
    if(y >= nextPercent)  {
      printf(" %d%%", (y*100)/dy);
      fflush(stdout);
      nextPercent += tenth;
    }
    for(x=0; x<dx; x++, buffer++)
      *buffer = table[*buffer];
  }
  printf(" 100%%\n");
}
*/
/*********************************************************************
** BinPixels
** =========
Simple averaging of 2x2, 3x3, 4x4, ... into singple 'bins. We do this in
software because the Zeiss Scanner software does not do it properly in the
Y-direction.
*/
static
void BinPixels(UBYTE *buffer, int *dx, int *dy, int binFactor)
{
  int binSize, finalDx, finalDy, sum;
  int x, y, xx, yy, tenth, nextPercent;
  unsigned long offset;
  UBYTE *bufIn, *bufOut;


// Bin down - only keep complete 'bin'ed' rows and columns

  finalDx = (*dx) / binFactor;
  finalDy = (*dy) / binFactor;
  binSize = binFactor * binFactor;

  printf("- Image size of (%d,%d) scaled down by %d to (%d,%d)\n"
			, *dx, *dy
			, binFactor
			, finalDx, finalDy
			);
  printf("- Binning beginning:");
  tenth = finalDy / 10 + 1;	// make sure 10% or more, but not less
  nextPercent = 0;
  bufOut = buffer;	// overwrite input array

  for(y=0; y<finalDy; y++)  {
    if(y >= nextPercent)  {
      printf(" %d%%", (y*100)/finalDy);
      fflush(stdout);
      nextPercent += tenth;
    }
    bufIn = buffer + y*binFactor*(*dx);		// start of row
    for(x=0; x<finalDx; x++, bufIn+=binFactor)  {
      sum = 0;
      offset = 0;
      for(yy=0; yy<binFactor; yy++, offset+=(*dx))
        for(xx=0; xx<binFactor; xx++)
          sum += *(bufIn+offset+xx);
      *bufOut++ = (sum + binSize/2) / binSize;  // round value with binSize/2
    }    // along binned rows
  }    // down binned columns
  printf(" 100%%\n");
  *dx = finalDx;
  *dy = finalDy;
}
/*********************************************************************
** ShortBinPixels
** =========
Simple averaging of 2x2, 3x3, 4x4, ... into singple 'bins. We do this in
software because the Zeiss Scanner software does not do it properly in the
Y-direction.
*/
static
void ShortBinPixels(unsigned short *buffer, int *dx, int *dy, int binFactor)
{
  int binSize, finalDx, finalDy, sum;
  int x, y, xx, yy, tenth, nextPercent;
  unsigned long offset;
  unsigned short *bufIn, *bufOut;


// Bin down - only keep complete 'bin'ed' rows and columns

  finalDx = (*dx) / binFactor;
  finalDy = (*dy) / binFactor;
  binSize = binFactor * binFactor;

  printf("- Image size of (%d,%d) scaled down by %d to (%d,%d)\n"
			, *dx, *dy
			, binFactor
			, finalDx, finalDy
			);
  printf("- Binning beginning:");
  tenth = finalDy / 10 + 1;	// make sure 10% or more, but not less
  nextPercent = 0;
  bufOut = buffer;	// overwrite input array

  for(y=0; y<finalDy; y++)  {
    if(y >= nextPercent)  {
      printf(" %d%%", (y*100)/finalDy);
      fflush(stdout);
      nextPercent += tenth;
    }
    bufIn = buffer + y*binFactor*(*dx);		// start of row
    for(x=0; x<finalDx; x++, bufIn+=binFactor)  {
      sum = 0;
      offset = 0;
      for(yy=0; yy<binFactor; yy++, offset+=(*dx))
        for(xx=0; xx<binFactor; xx++)
          sum += *(bufIn+offset+xx);
      *bufOut++ = (sum + binSize/2) / binSize;  // round value with binSize/2
    }    // along binned rows
  }    // down binned columns
  printf(" 100%%\n");
  *dx = finalDx;
  *dy = finalDy;
}
/*********************************************************************
** ReadTiff
** ========
*/
static
int ReadTiff(UBYTE **buffer, int *dx, int *dy, int *bytesPerPixel,
	     char *passFname)
{
  FILE *fptr;

  do  {
    char  fname[1024];
#ifdef OLDWAY
    printf("Tiff filename ? ");
    scanf("%s", fname);
#else
    sprintf(fname,"%s",passFname);
#endif
    if((fptr = fopen(fname, "r")) == NULL)
      perror("Error opening TIFF file for input");
    else
      break;
  } while(1);

  TIFF_ReadImageHeader(fptr, dx, dy, bytesPerPixel);

// Allocate memory

  {
    const size_t bytes = sizeof(UBYTE) * ((*dx)*(*dy)) * (*bytesPerPixel);
    *buffer = (UBYTE *) malloc(bytes);
    if(*buffer == NULL)  {
      printf("Can't allocate [%d] sized buffer \n", (int) bytes);
      fclose(fptr);
      return(0);
    }
  }

  TIFF_ReadImageData(fptr, *buffer, *dx, *dy, *bytesPerPixel);
  fclose(fptr);
  return(1);
}
/****************************************************************************/
