/* File ctfDisp.h */
/*******************************************************************
 * CTFDISP - 
 *
 * Include file for ctfDisp.c that includes global definitions
 * MACROS and forward declarations for ctfDisp.
 *
 * Rob Ashmore
 * Biology Department
 * Purdue University
 * Februrary 1996
 *
 *******************************************************************/
#include <string.h>
#include <stdio.h>                              /* For printf and so on. */
#include <math.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>

#include <Xm/MessageB.h>                /* For message box stuff */
#include <Xm/Label.h>
#include <Xm/DialogS.h>
#include <Xm/Text.h>
#include <Mrm/MrmAppl.h>                /* Motif Toolkit and MRM */

#include <X11/cursorfont.h>                /* For watch cursor */
#include <Xm/FileSB.h>

#include "../include/libPIF.h"


/**************************
 *  TYPEDEFS              *
 **************************/

/* CURVE struct */
typedef struct __curve_t__
  {/* CURVE_T */
    float afractScrollValue;
    float resolutionScrollValue;
    float avoltageScrollValue;
    float spaberrationScrollValue;
    float chaberrationScrollValue;
    float spCoherenceScrollValue;
    float tmCoherenceScrollValue;
    float majDefocusScrollValue;
    float minDefocusScrollValue;
    float majAngScrollValue;
    float wienerScrollValue;
    float defocusScrollValue;
    float pfractScrollValue;
    int   spCoExp;
    float spCoFactor;
    int   isParamFile;
    char  paramFname[256];
    int   numParticles;
    float percPartTot;
  }/* CURVE_T */ curve_t;

/* CTF struct */
typedef struct __ctf_t__
  {/* CTF_T */
    float fudge;
    float disp_tmp;
    float tmp;
    int iy1;
    int iy2;
    float fy1;
    float fy2;
  }/* CTF_T */ ctf_t;

/* Our colormap struct */
typedef struct __map_t__
  {/* MAP_T */
    int x;
    int y;
    int color;
  }/* MAP_T */ map_t;

typedef unsigned long	X_PIXEL;

/* Struct for Purdue INT*2 file header */
typedef struct _purdueHead_t
  {/* PURDUEHEAD_T */
    short numCols,numRows;
    short img_bkg;
    int   nimg,nrec_last;
    int   bimg_sizes;
    int   img_min,img_max;
    float img_avg,img_std;
    int   file_id1, file_id2;
    short junk;
  }/* PURDUEHEAD_T */ purdueHead_t;




/**************************
 *  DEFINES               *
 **************************/

/* We need to have some default image buffer sizes etc.. for when the
 * program starts up.  Change these as needed dynamically during
 * program execution */
#define NUMCOLS 2048	     /* Default image size */
#define NUMROWS 2048
#define DEF_DISPLAY_SIZE 1024 /* Default display size */
#define MAXSIZE 2048         /* Maximum width or height in pixels for image */
#define MAXBOX 1000
#define CTF_WIDTH  512
#define CTF_HEIGHT 512
#define BOX_WIDTH  500
#define BOX_HEIGHT 500
#define DENOMINATOR 2147483647    /* For random number generator conversion */

#define MAXCURVE 256 /* Max value allowed as argument to XQueryColors */

#define PI        3.1415926535
#define	THISTWOPI 6.283185307         

#define PROGRAM_NAME		"CTFDISP"
#define FONT "fixed"


/* Now our ctfDisp colors */
#define WHITE       0           /* Drawing colors */
#define BLACK       1
#define RED         2
#define GREEN       3
#define BLUE        4
#define YELLOW      5
#define PLACEHOLDER 6
#define ORANGE      7
#define GRAY        8
#define ENTRY       0           /* Widget for color map entry */
#define SEAGREEN    200


#define IMAGE    16
#define PORTABLE 32

#define AFRACT        1
#define PFRACT        2 
#define	RESOLUTION    3
#define	ABERRATION_S  4
#define	ABERRATION_C  5
#define	AVOLTAGE      6
#define	DEFOCUS       7
#define COHERENCE_S   8
#define COHERENCE_T   9
#define WIENER        10
#define PIXSIZE       11

#define	ALL	 1
#define	FEG	 2
#define	EM300    3
#define	EM420W   4
#define	EM420L   5
#define FEG300	 6
#define	USER     10
#define JEOL_EM_2010F_HT_FEG   20
#define JEOL_EM_2010F_UHR_FEG  21
#define JEOL1010               22
#define Hitachi_H9000          23
#define Hitachi_HF2000	       24
#define FEI_Tecnai_G2_Polara   25
#define FEI_Tecnai_G2_Sphera   26


#define	APPLYFFT  11
#define	BANDPASS  12
#define SUMCURVE  13
#define DRAG	  14
#define CLEARCURVE 15
#define DRAGUPDATE 16
#define TOPCURVE   17
#define ALWAYSDRAW 18
#define USESUM     19
#define MAJDEFOCUS	 22
#define MINDEFOCUS	 23
#define MAJANG		 24
#define COHERENCE_S_EXP	 25
#define USERANG		 26
#define USERDEF		 27
#define CTFCMETH1	 28
#define CTFCMETH2	 29
#define CTFCMETH3	 30
#define CTFCMETH4	 31
#define CTFCMETH5	 32


#define NOPASS     0
#define PASSBAND   1
#define PASSCTF    2
#define LOWPASS    1 /* LOW and HI PASSCTF/PASS not used as of 10/28/96 */
#define HIPASS     2
#define LOWPASSCTF 4 
#define HIPASSCTF  5

#define FFT	        1
#define CTF	        2
#define BACK	        3
#define BACKSTRETCH	4

#define R40ANG		8
#define R20ANG		7
#define R10ANG		6
#define R5ANG	        5
#define R4ANG	        4
#define R3ANG	        3
#define R2ANG	        2
#define R1ANG	        1


#define CTFCFF           35
#define CTFCORRECT	 36
#define SECONDDISPLAY	 37
#define RESODRAW	 38
#define PARAM            39
#define ACTF		 42
#define PCTF		 43
#define ACTFPCTF	 44
#define	APPLYAMP	 45
#define	APPLYPHASE	 46
#define	NONE		 47
#define	APPLYAMPPHASE    48
#define	APPLYNONE	 49
#define	CORRECTAMPPHASE	 50
#define	CORRECTAMP	 51
#define	CORRECTPHASE	 52
#define	CORRECTNONE	 53
#define CTFCFF2          54
#define IMAGELOCAL	 55
#define PFTSEARCH	 56
#define EM3DR		 57
#define PARAMFILE	 58
#define PARAMNP		 59








/* Define the following so we don't define twice.  e.g. have everyone
 * call this include only if the following is undefined.
 */
#define MACROS_DEFINED 1




/* ************************************** *
 * RAW Xlib definitions                   *
 * ************************************** */
Arg arglist[32];
X_PIXEL		 black_pixel;
unsigned long    *pixels_return;
unsigned long    *colors_return;
Visual		 * visual = NULL;
int		 visual_depth = 0;
Colormap	 color_map = (Colormap) 0;

int display_width, display_height;
int ctf_width,     ctf_height;
int buffer_width,  buffer_height;
int image_width,   image_height;  // pixels
int file_width,    file_height;  //  pixels

int		image_colors;
GC		color_gc = (GC) 0;      /* Graphics contexts */
GC		test_color_gc = (GC) 0;
GC		hist_color_gc = (GC) 0;
GC		editBox_gc = (GC) 0;
GC		ctf_image_gc = (GC) 0;      
GC              draw_gc[MAXCURVE];
int             draw_gc_pos[MAXCURVE];
GC		image_gc = (GC) 0;      
GC		image_gcSEAGREEN = (GC) 0;
GC		image_gcGREEN = (GC) 0;
GC		image_gcRED = (GC) 0;
GC		image_gcBLUE = (GC) 0;
GC		image_gcBLACK = (GC) 0;
GC		image_gcYELLOW = (GC) 0;
GC		image_gcORANGE = (GC) 0;
GC		imageCTF_gcSEAGREEN = (GC) 0;
GC		imageCTF_gcGREEN = (GC) 0;
GC		imageCTF_gcRED = (GC) 0;
GC		imageCTF_gcBLUE = (GC) 0;
GC		imageCTF_gcBLACK = (GC) 0;
GC		imageCTF_gcGRAY = (GC) 0;
GC              imageCTF_colors[MAXCURVE];
GC              textGC = 0;

char path[1024];
char		        * display_name = NULL;
int      		i, j;
int		        x= 0;
int		        y= 0;
unsigned long	        gc_mask;
XGCValues		gc_values;
XColor	                * random_defs= NULL;
XColor	                * color_defs= NULL;
XColor			* gen_color_defs = NULL;
float                   pixelColor;
XEvent                  event;
unsigned long           event_mask;

XFontStruct *fontstruct; 	/* Font descriptor */



/*****************
 * IMAGE BUFFERS *
 *****************/
float *fftImage_buffer;
float *fftImage_junk;
unsigned short *raw_fft_image_buffer1;       
unsigned short *back_fft_image_buffer1;       
unsigned short *back_stretch_fft_image_buffer1;       
unsigned short *fft_image_buffer1;       
unsigned short *int_image_buffer1;       /* For passing to FORTRAN routines */
unsigned short *ctf2D_image_buffer1;
unsigned short *resoImage;

int ncol,nrow,fftWidth,fftHeight;

BYTE *mirror_image_buffer1;
BYTE *image_buffer1;
XImage *image_image1;
Window image_window1;


BYTE *ctf_image_buffer1;
XImage *ctf_image_image1;
Window ctf_image_window1;


ctf_t ctf_vals[MAXCURVE+1][5048]; /* Holds the calculated CTF function */ 
curve_t curve[MAXCURVE+1]; 
int curCtf; 
int ctfDrawn[MAXCURVE+1];
int alwaysDraw[MAXCURVE+1];
int useSum[MAXCURVE+1];
float weightings[MAXCURVE+1];
float maxParamWeight;
int resoDraw;

int histogram[512];

char outStr[1024];

int ctfMode;      /* For passing to ctf_multiply() */
int ctfMode_div;  /* For passing to ctf_divide() */
int applyAmp,applyPhase;



/* Global display and applications context pointers*/
Display         *display;		/* Display variable */
XtAppContext    app_context;		/* application context */

/* All of our widgets */
static Widget toplevel_widget, main_window_widget;

static Widget dragUpdateWidget,
  bandPassWidget,
  applyFFTwidget,
  sumCurveWidget,
  clearCurveWidget,
  alwaysDrawWidget,
  useSumWidget,
  paramFileW,
  paramNPW,
  topCurveTextWidget,
  curveTextWidget,
  drawingWidget,
  ctfDrawingWidget,
  afractScroll,
  afractValue,
  pfractScroll,
  pfractValue,
  resolutionScroll,
  resolutionValue,
  avoltageScroll,
  avoltageValue,
  spaberrationScroll,
  spaberrationValue,
  chaberrationScroll,
  chaberrationValue,
  spCoherenceScroll,
  spCoherenceValue,
  spCoExpLabelW,
  tmCoherenceScroll,
  tmCoherenceValue,
  majDefocusScroll,
  majDefocusValue,
  minDefocusScroll,
  minDefocusValue,
  majAngScroll,
  majAngValue,
  userAngScroll,
  userAngValue,
  userDefocusValue,
  wienerScroll,
  wienerValue,
  pixSizeValue,
  defocusLabel_widget,
  defocusScroll,
  defocusValue,
  ctfCorrectFFW,
  ctfCorrectFF2W,
  ctfCorrectFF2LabelW,
  ctfcMethNoneW,
  ctfcMeth1W,
  ctfcMeth2W,
  ctfcMeth3W,
  ctfcMeth4W,
  ctfcMeth5W,
  infoBox;
Widget ctfscreen,resoExample_screen,resoExampleLabel_widget,ctfCorrectScreen;
Widget resoButtonW[10];
Widget fileSel_widget;     /* Holds the file selection widget */
Widget fileDia_widget;     /* Holds the file selection dialog widget */
Widget actfW,
       pctfW,
       actfPctfW;          /* Holds ACTF/PCTF radio box button widget # */
Widget secondDisplayButton;


/* The UIL file hierarchy */
static MrmHierarchy s_MrmHierarchy;     /* MRM database hierarchy ID */
/* static MrmType *dummy_class; */           /* and class variable. */
static MrmType dummy_class;            /* and class variable. */
static char *db_filename_vec[] =        /* Mrm.heirachy file list. */
{
    "/doitFromMain/ctfDisp.uid" /* There is only one UID file for */
};                                    /* this application. */              

static int db_filename_num =
                (sizeof db_filename_vec / sizeof db_filename_vec [0]);


/* Cursor thigs */
Cursor cross_cursor;
Cursor watch_cursor;
XSetWindowAttributes attrs;
Pixmap                  shape_pixmap,         /* Cursor pixmaps */
                        mask_pixmap, 
                        zoomPixmap;





/***************************************
 * Generic C language work areas       *
 ***************************************/
fString_t fileName;                 /* What file do they want? */
fString_t image_fileName;           /* What file is the image? */

int tryColors;                  /* Keep trying until you get as many entries 
				 * in the colormap as you can */

int choice;			/* Pass this to the fortran code */

char stufBuf[NUMCOLS];          /* Init the screen */
short stufIntBuf[NUMCOLS];      /* Init the input buffer */

purdueHead_t purdueHeader;

FILE          *fIn;
int            numCols, numRows;
int            iRec;
int xoffset, yoffset;
int virXoffset,virYoffset;
int em_wrt,lun;
int noCallbackScroll;
int minOrig,maxOrig;
int saveMin,saveMax;
float scaleOrig;
int topCurve;                    /* Max curve to calc/plot */
int readWrite;
int writeType,readType;
int fileType;
int globAsk;
int gFinished;
int headOff,yOff,thisOffset;
int sysError;
int bandPass;
int dragUpdate;
int imageType;

int largeImage;
int inited;
int endianSwitch;
int applyFFTswitch;
int sumCurveSwitch;
int drawPointWhere,drawPWX;
int firstTime;
char motionNumStr[256];

int resXlo,resXhi;                  /* For low/hi band pass CTF filters */
int middlePass;
float scrollResolutionExample;
int   resoY;
int toggleThis;

int   filter[10];                   /* For ctf_multiply() */
float filt_fac[5]; 

float afractScrollValue, 
   resolutionScrollValue, 
   avoltageScrollValue, 
   spaberrationScrollValue, 
   chaberrationScrollValue, 
   spCoherenceScrollValue, 
   tmCoherenceScrollValue, 
   majDefocusScrollValue,
   minDefocusScrollValue,
   majAngScrollValue,
   wienerScrollValue,
   pixSizeTextValue,
   defocusScrollValue, 
   pfractScrollValue;
int spCoExp;
float spCoFactor;         /* Convert input into Rads */

int defocusMin,  defocusMax;

float defocus,
   cs,
   cc,
   deltaE,
   deltaF,
   voltage,
   pfract,
   afract,
   wavelength,
   res_limit; 

float sizeFactor,recipSizeFactor;   /* Corrections based on ZOOMiness */

float hiFFTscroll,loFFTscroll;

float userAngScrollValue;

int secondDisplay;
int firstCtf2D= TRUE;
char versionStr[1024];

/* ctfCorrect stuff */
int ctfMeth1,ctfMeth2,ctfMeth3,ctfMeth4,ctfMeth5;
int ctfCorrectHeight,middleLine,topLine;
int dispCtfCorrect;
float maxCTFCorrectYvalue;
float fudgeFactor;   


/* Command line stuff */
int preLoadSwitch;
char preLoadStr[256];


/********************************************************/
/*  FORWARD DECLARATIONS                                */
/********************************************************/
extern void img_fft_fill();
extern void fft_2D();
extern void fft_2d_rob_();
extern void piraddeg_();
extern float wavel_();
extern float ctf_();
extern void focus_astig_();
extern void fft_hiplop_();
extern void fft_flt_();
void handle_cr();
void     XlibInitialize();
void     userDefaults();
extern void vt_cls();  
void mainCtfDisp_sub_UNIX(); 
/* int mainCtfDisp_sub();  */
void init_globals();
void s_error();

void gotFilename();
int askThem(char *infoStr, int systemModal, int dialogType);
void cancel_fileSel();

void filterFiles();

void stuffImage();
void handle_cr(Widget widget);
void differentEndianCtf();
int calcBackground();
void setCurveVals(int doTextWidgets);
void DrawResoExample(int thisY);
void calcCtf(int currentCtf);
void checkAutomatic();


/*********************
 * CALLBACKS         *
 *********************/
void plotWindow_create();
void exit_activate();
void DrawWindow();
void formResize();
void plotClick();
static void plotMotion();
void imageClick();
void infoCancel();
void infoOK();
void ctf_activate();
void ctf_exit_activate();
void ctfWindow_create();
void DrawCTFWindow();
void ctfFormResize();
void ctfScrollCreate();
void ctfChange();
void ctfValueCreate();
void ctfValueChange();
void toggle_change();
void actfPctf_change();
void createTog();
void curveText_create();
void curveText_changed();
void curveArrowActivate();
void ctf_reset_activate();
void display_activate();
void reso_reset_activate();
void reso_button_create();
void read_activate();
void resoExample_activate();
void resoExample_deactivate();
void resoExampleChange();
void resoExampleLabel_create();
void defocusArrowActivate();
void textActivate();
void ctf_pointFB_activate();
void defocusSchertzerActivate();
void spCoArrowActivate();
void ctf_ioparms_activate();
void fftMinMaxChange();
void bandPass_flip_activate();
void ctfCorrect_activate();
void ctfCorrect_deactivate();
void ctfCorrect_tryit_activate();
void ctfCorrect_toggle_change();
void zoomActivate();
void secondDisplay_activate();


/* The names and addresses of things that Mrm.has to bind.  The names do
 * not have to be in alphabetical order.  */

static MrmRegisterArg reglist[] = {
    {"secondDisplay_activate", (caddr_t) secondDisplay_activate},
    {"zoomActivate", (caddr_t) zoomActivate},
    {"ctfCorrect_toggle_change", (caddr_t) ctfCorrect_toggle_change},
    {"ctfCorrect_tryit_activate", (caddr_t) ctfCorrect_tryit_activate},
    {"ctfCorrect_deactivate", (caddr_t) ctfCorrect_deactivate},
    {"ctfCorrect_activate", (caddr_t) ctfCorrect_activate},
    {"bandPass_flip_activate", (caddr_t) bandPass_flip_activate},
    {"fftMinMaxChange", (caddr_t) fftMinMaxChange},
    {"ctf_ioparms_activate", (caddr_t) ctf_ioparms_activate},
    {"spCoArrowActivate", (caddr_t) spCoArrowActivate},
    {"defocusSchertzerActivate", (caddr_t) defocusSchertzerActivate},
    {"ctf_pointFB_activate", (caddr_t) ctf_pointFB_activate},
    {"defocusArrowActivate", (caddr_t) defocusArrowActivate},
    {"resoExampleLabel_create", (caddr_t) resoExampleLabel_create},
    {"resoExampleChange", (caddr_t) resoExampleChange},
    {"resoExample_deactivate", (caddr_t) resoExample_deactivate},
    {"resoExample_activate", (caddr_t) resoExample_activate},
    {"reso_button_create", (caddr_t) reso_button_create},
    {"reso_reset_activate", (caddr_t) reso_reset_activate},
    {"display_activate", (caddr_t) display_activate},
    {"ctf_reset_activate", (caddr_t) ctf_reset_activate},
    {"curveArrowActivate", (caddr_t) curveArrowActivate},
    {"curveText_changed", (caddr_t) curveText_changed},
    {"curveText_create", (caddr_t) curveText_create},
    {"createTog", (caddr_t) createTog},
    {"actfPctf_change", (caddr_t) actfPctf_change},
    {"toggle_change", (caddr_t) toggle_change},
    {"ctfValueCreate", (caddr_t) ctfValueCreate},
    {"ctfValueChange", (caddr_t) ctfValueChange},
    {"ctfChange", (caddr_t) ctfChange},
    {"ctfScrollCreate", (caddr_t) ctfScrollCreate},
    {"ctfFormResize", (caddr_t) ctfFormResize},
    {"DrawCTFWindow", (caddr_t) DrawCTFWindow},
    {"ctfWindow_create", (caddr_t) ctfWindow_create},
    {"ctf_exit_activate", (caddr_t) ctf_exit_activate},
    {"ctf_activate", (caddr_t) ctf_activate},
    {"textActivate", (caddr_t) textActivate},
    {"infoCancel", (caddr_t) infoCancel},
    {"infoOK", (caddr_t) infoOK},
    {"plotClick", (caddr_t) plotClick},
    {"plotMotion", (caddr_t) plotMotion},
    {"imageClick", (caddr_t) imageClick},
    {"formResize", (caddr_t) formResize},
    {"DrawWindow", (caddr_t) DrawWindow},
    {"read_activate", (caddr_t) read_activate},
    {"exit_activate", (caddr_t) exit_activate},
    {"plotWindow_create", (caddr_t) plotWindow_create}
};

static int reglist_num = (sizeof reglist / sizeof reglist [0]);


/* Translate keys for TEXT widgets */
char rwa_translation_table[] = 
"<Key>Return : textActivate()";

XtActionsRec rwa_action_table[] = 
    {
      {"textActivate",     (XtActionProc)textActivate},
    };




/* End of ctfDisp.h */
