
C  ********************************************************************
C  *                                                                  *
C  *            *****     SUBROUTINE IMG_FFT_FILL_ROB     *****       *
C  *                                                                  *
C  *    LOAD NCOL X NROW "IMAGE" INTO "FFT" ARRAY TO COMPUTE FFT      *
C  *                                                                  *
C  *    MAY 1987  PURDUE UNIVERSITY VAX 11/750, T. BAKER.             *
C  *    DECEMBER 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.          *
C  *    LAST MODIFIED  7-AUG-1990 BY T. BAKER.                        *
C  ********************************************************************
C
      SUBROUTINE IMG_FFT_FILL_ROB (IMG, NCOL, NROW, FFT,IDIM1,IDIM2,
     1           imgBkgP)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, IDIM1, IDIM2, I, J, img_bkg, imgBkgP
      INTEGER*2 IMG(NCOL,NROW)
      REAL*4    FFT(2*IDIM1,IDIM2), TMP

      img_bkg = imgBkgP

      FFT = 0.0 ! Fortran 90 array syntax

      DO J = 1,NROW
        DO I = 1,NCOL
          TMP            = FLOAT(IMG(I,J) - IMG_BKG)
          FFT((2*I)-1,J) = TMP
        ENDDO
      ENDDO

      RETURN
      END

C  ---------------------------------------------------------------------------
C
C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE FFT_HIPLOP     *****               *
C  *                                                                          *
C  *    APPLY HI/LOW-PASS FILTER TO FFT DATA                                  *
C  *                                                                          *
C  *    RES_MAX = RESOLUTION IN # OF TRANSFORM SAMPLE POINTS BELOW WHICH FFT  *
C  *              IS GIVEN FULL WEIGHT (WEIGHT = 1.0)                         *
C  *    RES_MIN = RESOLUTION IN # OF TRANSFORM SAMPLE POINTS ABOVE WHICH FFT  *
C  *              IS GIVEN FULL WEIGHT (WEIGHT = 1.0)                         *
C  *    CUT_MAX & CUT_MIN not yet implemented                                 *
C  *                                                                          *
C  *    This routine is based on the routines fft_hip() and fft_lop() in      *
C  *    fftlib.f                                                              *
C  *                                                                          *
C  *    May 2000  PURDUE UNIVERSITY, RWA.                                     *
C  ****************************************************************************
C
      SUBROUTINE FFT_HIPLOP (FFT, IDIM1, IDIM2, RES_Min, RES_Max,
     1       CUT_Min, CUT_Max, INVERTSWITCH, FFT_FULL)
C
      IMPLICIT NONE
C
      INTEGER*4 IDIM1, IDIM2, IX, IY, IY2, ICOUNT, XTEST, YTEST
      REAL*4    FFT(1), RES_MAX, CUT_MAX, DELSQmax, MAX_SQ, YSQ, RSQ
      REAL*4    RES_Min, CUT_Min, DELSQmin, Min_SQ
      REAL*4    WEIGHT
      LOGICAL   FFT_FULL, invertSwitch
C  ----------------------------------------------------------------------------
C
      MAX_SQ = RES_MAX*RES_MAX
      DELSQmax  = CUT_MAX**2
      Min_SQ = RES_Min*RES_Min
      DELSQmin  = CUT_Min**2
      XTEST  = IDIM1/2
      YTEST  = IDIM2/2
      ICOUNT = -1
C
      IF(FFT_FULL) THEN
         IY2 = IDIM2
       ELSE
         IY2 = YTEST
      ENDIF
C
      DO IY = 1,IY2                        ! PROCESS FFT A ROW AT A TIME
        IF(IY.GT.YTEST) THEN
           YSQ = FLOAT(IY-IDIM2-1)**2
         ELSE
           YSQ = FLOAT(IY-1)**2
        ENDIF
        DO IX = 1,IDIM1                  ! PROCESS CURRENT FFT ROW
          ICOUNT = ICOUNT + 2
          IF(IX.LE.XTEST) THEN
             RSQ = YSQ + FLOAT(IX-1)**2            ! RIGHT HALF OF FFT
           ELSE
             RSQ = YSQ + FLOAT(IX-IDIM1-1)**2            ! LEFT HALF OF FFT
          ENDIF

Ca       Now let's test everything 
          if(.not.invertSwitch) then  ! Nornmal
            if(RSQ.gt.MAX_SQ .or.rsq.lt.min_sq) then      ! WEIGHT < 1.0
              if(DELSQmax.EQ.0.0 .and. DELSQmin.eq.0.0) then
                 WEIGHT = 0.0                        ! SHARP CUTOFF
               else 
Ca   Need to add code to do sloping cutoff
Ca                 if( (RES_MAX .gt. 0.0).and. 
Ca      1         (((sqrt(RSQ)-RES_MAX)**2/DELSQ).lt.50.0)) then
Ca                    WEIGHT = EXP(-((SQRT(RSQ)-RES_MAX)**2/DELSQ))
Ca                  else 
Ca                    if( RSQ/DELSQ .lt. 50.0) then
Ca                    WEIGHT = EXP(-RSQ/DELSQ)
Ca                  else
Ca                    WEIGHT = 0.0
Ca                  end if
Ca                 endif
                weight = 0.0  ! For now ...
              endif
              FFT(ICOUNT)   = WEIGHT*FFT(ICOUNT)
              FFT(ICOUNT+1) = WEIGHT*FFT(ICOUNT+1)
            ENDIF
          else   ! else it is invertSwitch 
            if(RSQ.lt.max_sq .and.rsq.gt.min_sq) then      ! WEIGHT < 1.0
              if(DELSQmax.EQ.0.0 .and. DELSQmin.eq.0.0) then
                 WEIGHT = 0.0                        ! SHARP CUTOFF
               else 
Ca   Need to add code to do sloping cutoff
Ca                 if( (RES_MAX .gt. 0.0).and. 
Ca      1         (((sqrt(RSQ)-RES_MAX)**2/DELSQ).lt.50.0)) then
Ca                    WEIGHT = EXP(-((SQRT(RSQ)-RES_MAX)**2/DELSQ))
Ca                  else 
Ca                    if( RSQ/DELSQ .lt. 50.0) then
Ca                    WEIGHT = EXP(-RSQ/DELSQ)
Ca                  else
Ca                    WEIGHT = 0.0
Ca                  end if
Ca                 endif
                weight = 0.0  ! For now ...
              endif
              FFT(ICOUNT)   = WEIGHT*FFT(ICOUNT)
              FFT(ICOUNT+1) = WEIGHT*FFT(ICOUNT+1)
            ENDIF
          endif  ! If .not. invertSwitch
        ENDDO
      ENDDO
C
      RETURN
      END
