      program emmapzoom
      
C     ZOOMS PORTION OR WHOLE 3D MAP UP OR DOWN.
C     
C     THIS PROGRAM IS A HYBRID OF EMMAP3DT AND EMSF3DBT.
C     OUTPUT IS A 3D MAP FILE IN PIF FORMAT
C     
C     UNITS: 1 = 3D MAP INPUT DATA FILE
C     2 = 3D I*2 MAP OUTPUT FILE
C     
C     OCTOBER 1998 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER
C     
C     MODIFIED  1-NOV-1998 BY TSB (Merge old EMMAP3DT & EMSF3DBT)
C     MODIFIED 16-NOV-2001 BY TSB (Linux version)
C     Modified 4/1/2010 By RSS (general cleanup)
      
      IMPLICIT NONE
      
      include '../include/pifhead.inc'
      TYPE(PIFHEAD_T)  MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD
      COMMON /PIFHEAD/ MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD
      
      INTEGER*4 NCOL, NROW, NSEC, HMAX, KMAX, LMAX, D(5), NSF
      INTEGER*4 NUM_UNIQUE, KMIN, NCD, NEF, T_MATRIX(2,3), TABLE_SIZE
      REAL*4    A11, A21, A22, A31, A32, A33, DSQMIN, DSQMAX, SCALE_FAC
      REAL*4    A_CELL, B_CELL, C_CELL, ALP, BET, GAM, VOL, VOL_STAR
      REAL*4    A_STAR, B_STAR, C_STAR, ALP_STAR, BET_STAR, GAM_STAR
      REAL*4    TRIG_TABLE(1001), DCELL(9), MAP_MIN, MAP_MAX
      REAL*4    XMIN, YMIN, ZMIN, XMAX, YMAX, ZMAX, DX, DY, DZ
      character(len=1) MAP_TITLE(80)
      CHARACTER MAP_FILENAME*256
      integer alloc_stat
      
      INTEGER*4, allocatable :: HKL(:)
      real*4, allocatable :: mapin(:), array(:)
      REAL*4, allocatable :: ARRAY2(:), TABLE1(:), TABLE2(:)
      REAL*4, allocatable :: MAPOUT(:), AP(:), BP(:)
      REAL*4, allocatable :: SINHX(:), COSHX(:), SINKY(:), COSKY(:)
      
      PRINT *, 'EMMAPZOOM: Zoom 3D MAP larger or smaller'
      call banner('EMMAPZOOM\0')
      PRINT *
      
      CALL INFO_EMMA (NCOL, NROW, NSEC, HMAX, KMAX, LMAX, NSF, A11, A21,
     $     A22, A31, A32, A33, A_CELL, B_CELL, C_CELL, ALP, BET, GAM,
     $     VOL, A_STAR, B_STAR, C_STAR, ALP_STAR, BET_STAR, GAM_STAR,
     $     VOL_STAR, DSQMIN, DSQMAX, SCALE_FAC, MAP_FILENAME, MAP_TITLE,
     $     TRIG_TABLE, DCELL, DX, DY, DZ, XMIN, YMIN, ZMIN, XMAX, YMAX,
     $     ZMAX)
      
      allocate(mapin(ncol*nrow*nsec), array((ncol+4)*nrow*nsec),
     $     stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate MAPIN and ARRAY in EMMAPZOOM'
         call tsbend
      endif
      
C     ----- READ IN THE ENTIRE 3D MAP
      call pif_read_mapi2(maphead, mapin, 1, nsec, ncol, nrow, nsec)
      
C     ----- CALCULATE FOURIER TRANSFORMS ON Z AND X OF NSEC SECTIONS
C GCA  ERROR: following routine expects array of size (ncol+2)*nrow*nsec
      CALL MAP3DT_PASS1 (MAPIN, ARRAY, NCOL, NROW, NSEC, D, HMAX)
      deallocate(mapin)
      CALL PIF_CLOSE (MAPHEAD%LUNUM) ! CLOSE AND SAVE INPUT 3D MAP FILE
      
      allocate(ARRAY2(2*NSEC*NROW*(HMAX+1)), HKL(3*NSF),
     $     AP(NSF), BP(NSF), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate ARRAY2, HKL, AP, BP'
         call tsbend
      endif
      
      CALL MAP3DT_STORE (ARRAY, NCOL, NROW, NSEC, HMAX, KMAX, ARRAY2)
      
C     ----- READ INTERMEDIATE RESULTS AND CALCULATE FOURIER TRANSFORMS ON Y
C GCA  ERROR: following routine expects both arrays of size 2*NSEC*(2*KMAX+1)*(HMAX+1)
      CALL MAP3DT_PASS2 (ARRAY, ARRAY2, NSEC, HMAX, KMAX, D)
C GCA  ERROR: following routine expects complex array of size(NSEC,HMAX+1,(2*KMAX+1))      
      CALL SFLOAD (ARRAY, HKL, AP, BP, NSEC, HMAX, KMAX, LMAX, NSF, A11,
     $     A21, A22, A31, A32, A33, DSQMIN, DSQMAX, SCALE_FAC,
     $     NUM_UNIQUE)
      
      CALL RIP (NCOL, NROW, NSEC, NCD, NEF, HKL, NUM_UNIQUE, AP, BP, DX,
     $     DY, DZ, XMIN, YMIN, ZMIN, XMAX, YMAX, ZMAX, DCELL, HMAX, 
     $     KMIN, KMAX)
      
      TABLE_SIZE = NSEC*NCD
      
      allocate(MAPOUT(NCOL*NROW*NSEC), TABLE2(NEF), 
     $     SINHX(NCOL*(HMAX+1)), COSHX(NCOL*(HMAX+1)),
     $     SINKY(NROW*(KMAX-KMIN+1)), COSKY(NROW*(KMAX-KMIN+1)),
     $     TABLE1(TABLE_SIZE), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate MAPOUT etc.'
         call tsbend
      endif
      
C     ----- CALCULATE FOURIER Transform, SECTION BY SECTION
      CALL FOURIER (MAPOUT, TRIG_TABLE, TABLE_SIZE, NCD, NEF, TABLE1,
     $     TABLE2, NCOL, NROW, NSEC, AP, BP, HKL, SINHX, COSHX, SINKY,
     $     COSKY, MAP_MIN, MAP_MAX, NUM_UNIQUE, DX, DY, DZ, XMIN, YMIN,
     $     ZMIN, HMAX, KMIN, KMAX)
      
      deallocate(AP, BP, SINHX, COSHX, SINKY, COSKY, TABLE1, TABLE2)
      
      CALL MAP_OUT (MAPOUT, NCOL, NROW, NSEC, A_CELL, B_CELL, C_CELL,
     $     GAM, T_MATRIX, MAP_MIN, MAP_MAX, MAP_FILENAME,
     $     MAP_TITLE)
      
      deallocate(mapout, array, array2, hkl)
      
      CALL TSBEND
      END program emmapzoom
      
      
      
      SUBROUTINE INFO_EMMA(NCOL, NROW, NSEC, HMAX, KMAX, LMAX, NSF, A11,
     $     A21, A22, A31, A32, A33, A_CELL, B_CELL, C_CELL, ALP, BET,
     $     GAM, VOL, A_STAR, B_STAR, C_STAR, ALP_STAR, BET_STAR,
     $     GAM_STAR ,VOL_STAR, DSQMIN, DSQMAX, SCALE_FAC, MAP_FILENAME,
     $     MAP_TITLE ,TRIG_TABLE, DCELL, DX, DY, DZ, XMIN, YMIN, ZMIN,
     $     XMAX, YMAX ,ZMAX)
      
C     Read in control parameters
      
      IMPLICIT NONE
      
      include '../include/pifhead.inc'
      TYPE(PIFHEAD_T)  MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD
      COMMON /PIFHEAD/ MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD
      
      INTEGER*4 IH, IK, IL
      INTEGER*4 I, NSF
      INTEGER*4 HMAX, KMAX, LMAX, NCOL, NROW, NSEC, NC2, NR2, NS2
      REAL*4    V, DX, DY, DZ, XMIN, YMIN, ZMIN, XMAX, YMAX, ZMAX
      REAL*4    AA(3,3), BB(3,3)
      REAL*4    DCELL(9), A_CELL, B_CELL, C_CELL, ALP, BET, GAM, VOL
      REAL*4    A_STAR, B_STAR, C_STAR, VOL_STAR, TRIG_TABLE(1001)
      REAL*4    ALP_STAR, BET_STAR, GAM_STAR, RCELL(9)
      REAL*4    A11, A21, A22, A31, A32, A33, DSQMIN, DSQMAX, SCALE_FAC
      REAL*4    PI, RADIAN, DEGREE, RESLO, RESHI, PIXSIZ_OLD, PIXSIZ_NEW
      REAL*4    COSALP, SINALP, COSBET, SINBET, COSGAM, SINGAM
      REAL*4    COSALP_STAR, COSBET_STAR, COSGAM_STAR
      REAL*4    SINALP_STAR, SINBET_STAR, SINGAM_STAR
      REAL*4    AST, BST, CST, XSQ, YSQ, ZSQ, DSTAR
      REAL*8    DPI, DRADIAN, DDEGREE, ARG, DEL_ARG
      real*8    deg_to_rad
      CHARACTER MAP_FILENAME*256
      character(len=1) RW(2), MAP_TITLE(80)
      
      deg_to_rad = acos(-1.0d0)/180.0d0
      
      NSF = 0                   ! SET COUNTER FOR # STRUCTURES FACTORS CALCULATED
      CALL PIRADDEG (PI, RADIAN, DEGREE)
      
C     ----- OPEN MAP FILE
      WRITE(*,'(A)',advance='no') 'Input 3D map filename: '
      READ (*,'(A)',ERR=999,END=999) MAP_FILENAME
      MAPHEAD%LUNUM = 1
      RW(1)  = 'r'
      RW(2) = achar(0)
      CALL PIF_OPEN (MAP_FILENAME, MAPHEAD%LUNUM, RW) 
      CALL PIF_READ_GH (MAPHEAD)
      IF(MAPHEAD%MODE == 20) THEN
         MAPHEAD%IMGNUM = 1     ! LOAD UP "IMG" #1
         CALL PIF_READ_DH (MAPHEAD)
         NCOL   = MAPHEAD%NUMCOLS
         NROW   = MAPHEAD%NUMROWS
         NSEC   = MAPHEAD%NUMSECS
         A_CELL = MAPHEAD%A_CELL
         B_CELL = MAPHEAD%B_CELL
         C_CELL = MAPHEAD%C_CELL
      ELSE
         PRINT *, 'Invalid PIF mode for 3D MAP'
         CALL TSBEND
      ENDIF
      
C     ----- ENTER NAME OF FINAL MAP FILE
      WRITE(*,'(A)',advance='no') 'Output 3D map filename: '
      READ (*,'(A)') MAP_FILENAME
      WRITE(*,'(A)',advance='no') 'Title: '
      READ (*,'(80A1)') MAP_TITLE
      
C     ---- CHECK INPUT FOR NCOL = EVEN RESTRICTION
C     ----- AND ALSO THAT NO PRIMES > 19 ARE FACTORS OF THE DIMENSIONS
      CALL CHK_PRIME (NCOL, NC2, .TRUE.)
      CALL CHK_PRIME (NROW, NR2, .FALSE.)
      CALL CHK_PRIME (NSEC, NS2, .FALSE.)
      WRITE(6,'(A,3I5)') ' 3D MAP: input dimensions =', NCOL, NROW, NSEC
      WRITE(6,'(A,3I5)') ' 3D MAP:   new dimensions =', NC2, NR2, NS2
      NCOL = NC2
      NROW = NR2
      NSEC = NS2
      PRINT *, ' Enter pixsiz(new & old) and res limits as pixels, ',
     1     'Angstroms or nms'
      WRITE(*,'(2A)',advance='no') 'PIXSIZ_OLD,PIXSIZ_NEW,RES_HI',
     1     ' <CR>=1.0,1.0,2.00001: '
      READ (*,*) PIXSIZ_OLD, PIXSIZ_NEW, RESHI
      IF(PIXSIZ_OLD <= 0.0) PIXSIZ_OLD = 1.0
      IF(PIXSIZ_NEW <= 0.0) PIXSIZ_NEW = 1.0
      IF(RESHI <= 0.0)           RESHI = 2.00001*PIXSIZ_OLD
      RESLO     = 0.0
      SCALE_FAC = 1.0
      DSQMIN    = 0.0
      DSQMAX    = 1.0/(RESHI**2)
      WRITE(*,'(A)',advance='no') 'XMIN,XMAX,YMIN,YMAX,ZMIN,ZMAX: '
      READ (*,*) XMIN, XMAX, YMIN, YMAX, ZMIN, ZMAX
      IF(XMAX == 0.0) XMAX = 1.0
      IF(YMAX == 0.0) YMAX = 1.0
      IF(ZMAX == 0.0) ZMAX = 1.0
      DX = PIXSIZ_NEW
      IF(DX <= 0.0) DX = 1.0
      DY = DX
      DZ = DX
      
C     HARDWIRE THESE FOR NOW
      
      A_CELL = NC2*PIXSIZ_OLD   !!! FIX: these are WRONG !!!!!
      B_CELL = NR2*PIXSIZ_OLD
      C_CELL = NS2*PIXSIZ_OLD
      ALP    = 90.0
      BET    = 90.0
      GAM    = 90.0
      CALL REAL_TO_REC (A_CELL,B_CELL,C_CELL,ALP,BET,GAM,VOL,
     1     A_STAR,B_STAR,C_STAR,VOL_STAR,ALP_STAR,BET_STAR,GAM_STAR)
      
      WRITE(*,1000) A_CELL,B_CELL,C_CELL,ALP,BET,GAM,RESHI,RESLO
      WRITE(*,'(T12,A,F10.2)') ' Fourier scaled by', SCALE_FAC
      
C     ----- CALCULATE RECIPROCAL LATTICE PARAMETERS
C     ----- (SEE INTERNATIONAL TABLES FOR CRYSTALLOGRAPHY, P.13)
      COSALP      = REAL(COS(ALP*deg_to_rad))
      SINALP      = REAL(SIN(ALP*deg_to_rad))
      COSBET      = REAL(COS(BET*deg_to_rad))
      SINBET      = REAL(SIN(BET*deg_to_rad))
      COSGAM      = REAL(COS(GAM*deg_to_rad))
      SINGAM      = REAL(SIN(GAM*deg_to_rad))
      COSALP_STAR = (COSBET*COSGAM-COSALP)/(SINBET*SINGAM)
      COSBET_STAR = (COSGAM*COSALP-COSBET)/(SINGAM*SINALP)
      COSGAM_STAR = (COSALP*COSBET-COSGAM)/(SINALP*SINBET)
      SINALP_STAR = SQRT(1.0-COSALP_STAR**2)
      SINBET_STAR = SQRT(1.0-COSBET_STAR**2)
      SINGAM_STAR = SQRT(1.0-COSGAM_STAR**2)
      AST         = 1.0/(A_CELL*SINBET*SINGAM_STAR)
      BST         = 1.0/(B_CELL*SINGAM*SINALP_STAR)
      CST         = 1.0/(C_CELL*SINALP*SINBET_STAR)
      
C     ----- CALCULATE MATRIX FOR D* TESTS
      A11 =  BST*SINGAM_STAR*SINALP
      A21 = -BST*SINGAM_STAR*COSALP
      A22 =  CST*SINBET_STAR
      A31 =  BST*COSGAM_STAR
      A32 =  CST*COSBET_STAR
      A33 =  AST
      
C     ----- CALCULATE LIMITS ON H, K, AND L
      HMAX = INT(SINALP_STAR/(AST*RESHI*SQRT(SINALP_STAR**2 + 
     1     (SINBET_STAR*COSGAM)**2 + (SINGAM_STAR*COSBET)**2 + 
     2     2.0*COSALP_STAR*(2.0*COSBET_STAR*COSGAM_STAR +
     3     SINBET_STAR*COSBET*SINGAM_STAR*COSGAM) - 
     4     2.0*(COSBET_STAR**2+COSGAM_STAR**2))))
      KMAX = INT(SINBET_STAR/(BST*RESHI*SQRT(SINBET_STAR**2 + 
     1     (SINGAM_STAR*COSALP)**2 + (SINALP_STAR*COSGAM)**2 +
     2     2.0*COSBET_STAR*(2.0*COSGAM_STAR*COSALP_STAR +
     3     SINGAM_STAR*COSGAM*SINALP_STAR*COSALP) - 
     4     2.0*(COSALP_STAR**2+COSGAM_STAR**2))))
      LMAX = INT(SINGAM_STAR/(CST*RESHI*SQRT(SINGAM_STAR**2 +
     1     (SINBET_STAR*COSALP)**2 + (SINALP_STAR*COSBET)**2 +
     2     2.0*COSGAM_STAR*(2.0*COSBET_STAR*COSALP_STAR +
     3     SINBET_STAR*COSBET*SINALP_STAR*COSALP) -
     4     2.0*(COSBET_STAR**2+COSALP_STAR**2))))
      
      WRITE(*,1010) NC2, NR2, NS2, HMAX, KMAX, LMAX
      
      SCALE_FAC = SCALE_FAC/REAL(NC2*NR2*NS2)
      
C     ----- CHECK INPUT FOR POSSIBLE PROGRAM OVERFLOWS 
      IF(NC2 < 2*HMAX+1) THEN
         WRITE(*,1020) HMAX, NC2
         CALL TSBEND
      ENDIF
      IF(NR2 < 2*KMAX+1) THEN
         WRITE(*,1030) KMAX, NR2
         CALL TSBEND
      ENDIF
      IF(NS2 < 2*LMAX+1) THEN
         WRITE(*,1040) LMAX, NS2
         CALL TSBEND
      ENDIF
      
C     ----- DETERMINE MAXIMUM SIZE OF NSF WITHIN RESOLUTION LIMITS
      DO IK = -KMAX,KMAX
         XSQ = (REAL(IK)*A11)**2
         DO IL = -LMAX,LMAX
            YSQ = (REAL(IK)*A21 + REAL(IL)*A22)**2 
            DO IH = 1,HMAX+1
               ZSQ = (REAL(IH-1)*A33 + REAL(IK)*A31 + REAL(IL)*A32)**2
               DSTAR = XSQ + YSQ + ZSQ
               IF(DSTAR <= DSQMAX.AND.DSTAR >= DSQMIN) NSF = NSF + 1
            ENDDO
         ENDDO
      ENDDO
      
      WRITE(*,'(12X,A,I9)') 'Max # of structure factors computed =', NSF
      
      CALL PIRADDEG_D (DPI, DRADIAN, DDEGREE)
      DEL_ARG = 2.0*DPI/1000.0
      ARG     = 0.D0
      
      DO I = 1,1001
         TRIG_TABLE(I) = COS(SNGL(ARG))
         ARG           = ARG + DEL_ARG
      ENDDO
      
      DCELL(1) = A_CELL
      DCELL(2) = B_CELL
      DCELL(3) = C_CELL
      DCELL(4) = ALP
      DCELL(5) = BET
      DCELL(6) = GAM
      IF(DCELL(4) >= 1.0) THEN
         DO I = 4,6
            DCELL(I) = REAL(COS(DCELL(I)/DDEGREE))
         ENDDO
      ENDIF
      CALL SF3DBT_STOAA (DCELL, AA)
      CALL SF3DBT_MI3 (AA, BB, V)
      V = SQRT(V)
      RCELL(1) = SQRT(BB(1,1))
      RCELL(2) = SQRT(BB(2,2))      
      RCELL(3) = SQRT(BB(3,3))
      RCELL(4) = BB(2,3)/(RCELL(2)*RCELL(3))
      RCELL(5) = BB(1,3)/(RCELL(1)*RCELL(3))
      RCELL(6) = BB(1,2)/(RCELL(1)*RCELL(2))
      DO I = 4,6
         DCELL(I+3) = REAL(ATAN2(SQRT(1.-DCELL(I)*DCELL(I)),DCELL(I))
     1       *DDEGREE)
         RCELL(I+3) = REAL(ATAN2(SQRT(1.-RCELL(I)*RCELL(I)),RCELL(I))
     1       *DDEGREE)
      ENDDO
      IF(DCELL(1) < 1.0) THEN
         V = 1.0/V
         DO I = 1,9
            CALL SF3DBT_ROT (DCELL(I), RCELL(I))
            CALL SF3DBT_ROT ( AA(I,1),  BB(I,1))
         ENDDO
      ENDIF
      
      PRINT *, 'V,VOL,VOL_STAR', V,VOL,VOL_STAR
      WRITE(*,1050) DCELL
      WRITE(*,1060) RCELL
      DX = DX/DCELL(1)
      DY = DY/DCELL(2)
      DZ = DZ/DCELL(3)
      
      CALL CHECKUP (XMIN, XMAX, DX)
      CALL CHECKUP (YMIN, YMAX, DY)
      CALL CHECKUP (ZMIN, ZMAX, DZ)
      RETURN
 999  CALL TSBEND
      
 1000 FORMAT(/,20X,'STRUCTURE FACTOR CALCULATION',/,20X,28('-'),/,12X,
     1     '    A =',F7.2,'     B =',F7.2,'      C =',F7.2,/,12X,
     2     'ALPHA =',F7.2,'  BETA =',F7.2,'  GAMMA =',F7.2,/,12X,
     3     'Resolution limits are',F7.2,' to ',F7.2)
 1010 FORMAT(12X,'Input cell divisions are',3I5,' pixels',/,12X,
     1     'Limits on H, K and L are',3I5)
 1020 FORMAT(' HMAX out of range.  HMAX equals',I5,' NCOL equals',I5)
 1030 FORMAT(' KMAX out of range.  KMAX equals',I5,' NROW equals',I5)
 1040 FORMAT(' LMAX out of range.  LMAX equals',I5,' NSEC equals',I5)
 1050 FORMAT(/,5X,'A',7X,'B',7X,'C',5X,'COS ALP',2X,'COS BET',2X,
     1     'COS GAM',3X,'ALP',4X,'BET',4X,'GAM',/' ',3F8.2,3F9.5,3F7.2)
 1060 FORMAT(/,5X,'A*',6X,'B*',6X,'C*',4X,'COS ALP*',1X,'COS BET*',1X,
     1     'COS GAM*',3X,'ALP*',3X,'BET*',3X,'GAM*',/  ' ',
     2     3F8.5,3F9.5,1X,3F7.2)

      END subroutine info_emma
      


      SUBROUTINE RIP (NCOL, NROW, NSEC, NCD, NEF, HKL, NUM_UNIQUE, AP,
     $     BP, DX, DY, DZ, XMIN, YMIN, ZMIN, XMAX, YMAX, ZMAX, DCELL,
     $     HMAX, KMIN, KMAX)

C     Routine called by emsf3dbt program

      IMPLICIT NONE

      INTEGER*4 NUM_UNIQUE
      INTEGER*4 NCOL, NROW, NSEC, HMAX, KMAX, KMIN, NN, NCD, NEF
      INTEGER*4 HKL(3,NUM_UNIQUE), J, K, L
      REAL*4    XMIN, YMIN, ZMIN, XMAX, YMAX, ZMAX, DX, DY, DZ, DCELL(9)
      REAL*4    APART, BPART
      REAL*4    AP(NUM_UNIQUE), BP(NUM_UNIQUE), FJX, FKX, FLX
      real*8    deg_to_rad
      
      character(len=1) xl, yl, zl
      DATA      XL/'X'/, YL/'Y'/, ZL/'Z'/
      
      deg_to_rad = acos(-1.0d0)/180.0d0
      
      HMAX   = 0
      KMAX   = 0
      KMIN   = 0
      
      FJX = DX*DCELL(1)
      FKX = DY*DCELL(2)
      FLX = DZ*DCELL(3)
      WRITE(*,1000) XL, XMIN, XMAX, DX, FJX, YL, YMIN, YMAX, DY, FKX,
     1     ZL, ZMIN, ZMAX, DZ, FLX
      
C     ----- SET NEW MAP DIMENSIONS
      NCOL = INT((XMAX-XMIN)/DX+1.5)
      NROW = INT((YMAX-YMIN)/DY+1.5)
      NSEC = INT((ZMAX-ZMIN)/DZ+1.5)
      WRITE(*,'(3(A,I6))') ' MAP dimensions:', NCOL, ' by', NROW,
     1     ' by', NSEC
      
      DO NN = 1,NUM_UNIQUE
         J     = HKL(1,NN)
         K     = HKL(2,NN)
         L     = HKL(3,NN)
         APART = REAL(AP(NN)*COS(deg_to_rad*BP(NN)))
         BPART = REAL(AP(NN)*SIN(deg_to_rad*BP(NN)))
         
         IF(HKL(1,NN)) 680,650,710
 650     IF(HKL(2,NN)) 690,660,710
 660     IF(HKL(3,NN)) 700,670,710
 670     GO TO 710
 680     J = -J
 690     K = -K
 700     L = -L
         BPART = -BPART
 710     IF(APART.NE.0.0 .OR. BPART.NE.0.0) THEN
            IF(J > HMAX) HMAX = J
            IF(K > KMAX) KMAX = K
            IF(K < KMIN) KMIN = K
         ENDIF
         HKL(1,NN) = J
         HKL(2,NN) = K
         HKL(3,NN) = L
         AP(NN)    = APART
         BP(NN)    = BPART
      ENDDO
      
      NCD = (HMAX+1)*(KMAX-KMIN+1)*2
      NEF = (HMAX+1)*NROW*2
      WRITE(*,1020) NCD, NEF
      RETURN
      
 1000 FORMAT(' ',A2,'Limits: ',F8.5,' to',F8.5,' in steps of',
     1     F8.5,' (',F6.3,' nms)')
 1020 FORMAT(' The CD table requires',I6,' memory locations'/,
     1     ' The EF table requires',I6,' memory locations'/)

      END subroutine rip
      


      SUBROUTINE FOURIER (MAP, TRIG_TABLE, TABLE_SIZE, NCD, NEF, TABLE1,
     $     TABLE2, NCOL, NROW, NSEC, AP, BP, HKL, SINHX, COSHX, SINKY,
     $     COSKY, MAP_MIN, MAP_MAX, NSF, DX, DY, DZ, XMIN, YMIN, ZMIN,
     $     HMAX, KMIN, KMAX)

C     FOURIER SUMMATION ROUTINE - THIS IS A MAJOR REVISION OF THE
C     SUBROUTINE "SIG100" OF THE PROGRAM FOUR3D.

      IMPLICIT NONE

      INTEGER*4 TABLE_SIZE, NCOL, NROW, NSEC, NEF, NSF
      INTEGER*4 I, J, K, M, N, CE, CF, LOCEI, CC, CD
      INTEGER*4 HMAX, KMAX, KMIN, NH, NK, NCD, ISEC
      integer*4 SECTION_THIS_LOOP
      INTEGER*4 HKL(3,NSF), NSECS_THIS_LOOP
      REAL*4    TABLE1(TABLE_SIZE), TABLE2(NEF), TRIG_TABLE(1001)
      REAL*4    SINKY(NROW*(KMAX-KMIN+1)), COSKY(NROW*(KMAX-KMIN+1))
      REAL*4    SINHX(NCOL*(HMAX+1)), COSHX(NCOL*(HMAX+1))
      REAL*4    MAP_MIN, MAP_MAX, XMIN, YMIN, ZMIN, DX, DY, DZ
      REAL*4    AP(NSF), BP(NSF), MAP(NCOL,NROW,NSEC)
      REAL*4    ARG, SINARG, COSARG, X, Y, Z, DEL_ARG

      MAP_MIN         = HUGE(MAP_MIN)
      MAP_MAX         = -HUGE(MAP_MAX)
      ISEC            = 0
      NH              = HMAX + 1            ! # H INDICES
      NK              = KMAX - KMIN + 1      ! # OF K INDICES
      NSECS_THIS_LOOP = TABLE_SIZE/NCD      ! # SECTIONS HANDLED EACH LOOP
      Z               = ZMIN
      PRINT *, ' '
      WRITE(*,'(A)') '   Set up SINHX,COSHX tables'

C     ----- SET UP SINHX,COSHX TABLES
      I       = 1
      DEL_ARG = 0.0
      X       = 0.0
      DO N = 1,NH
        ARG  = X
        DO M = 1,NCOL
          CALL SF3DBT_GETARGS (ARG, SINARG, COSARG, TRIG_TABLE)
          SINHX(I) = SINARG
          COSHX(I) = COSARG
          I        = I + 1
          ARG      = ARG + DEL_ARG
        ENDDO
        DEL_ARG = DEL_ARG + DX
        X       = X + XMIN
      ENDDO
      WRITE(*,'(A)') '   Set up SINHX,COSHX tables'

C     ----- SET UP SINKY,COSKY TABLES
      I       = 1
      Y       = REAL(KMIN)*YMIN
      DEL_ARG = REAL(KMIN)*DY
      DO K = 1,NK
         ARG  = Y
         DO N = 1,NROW
            CALL SF3DBT_GETARGS (ARG, SINARG, COSARG, TRIG_TABLE)
            SINKY(I) = SINARG
            COSKY(I) = COSARG
            I        = I + 1
            ARG      = ARG + DEL_ARG
         ENDDO
         Y       = Y + YMIN
         DEL_ARG = DEL_ARG + DY
      ENDDO
      
 100  IF(NSECS_THIS_LOOP > NSEC-ISEC) NSECS_THIS_LOOP = NSEC - ISEC

C     ----- INITIALIZE TABLE
      WRITE(*,'(A)') '   Set up TABLE1            '
      DO I = 1,TABLE_SIZE
         TABLE1(I) = 0.0
      ENDDO

C     ----- SET UP TABLE
      DO I = 1,NSF
         CC      = HKL(1,I)*NK + (HKL(2,I) - KMIN) + 1
         CD      = CC + NCD/2
         ARG     = HKL(3,I)*Z
         DEL_ARG = HKL(3,I)*DZ
         DO J = 1,NSECS_THIS_LOOP
            CALL SF3DBT_GETARGS (ARG, SINARG, COSARG, TRIG_TABLE)
            TABLE1(CC) = TABLE1(CC) + AP(I)*COSARG + BP(I)*SINARG
            TABLE1(CD) = TABLE1(CD) - AP(I)*SINARG + BP(I)*COSARG
            CC         = CC + NCD
            CD         = CC + NCD/2
            ARG        = ARG + DEL_ARG
         ENDDO
      ENDDO
      
      CC = 1
      CD = CC + NCD/2
      DO SECTION_THIS_LOOP = 1,NSECS_THIS_LOOP
         ISEC = ISEC + 1
         if(section_this_loop < nsecs_this_loop) then
            if(mod(section_this_loop,10) == 0) then
               WRITE(*,'(A,I4)') 'Fourier section', ISEC
            endif
         endif
         DO J = 1,NEF
            TABLE2(J) = 0.0
         ENDDO
         LOCEI = 1
         DO J = 1,NH
            I = 1
            DO K = 1,NK
               CE = LOCEI
               CF = CE + NEF/2
               DO N = 1,NROW
                  TABLE2(CE) = TABLE2(CE) + TABLE1(CC)*COSKY(I)
     1                 + TABLE1(CD)*SINKY(I)
                  TABLE2(CF) = TABLE2(CF) - TABLE1(CC)*SINKY(I)
     1                 + TABLE1(CD)*COSKY(I)
                  CE = CE + HMAX + 1
                  CF = CE + NEF/2
                  I  = I + 1
               ENDDO
               CC    = CC + 1
               CD    = CC + NCD/2
            ENDDO
            LOCEI = LOCEI + 1
         ENDDO
         
         CE = 1
         CF = NEF/2 + 1
         DO J = 1,NROW
            DO K = 1,NCOL       ! ZERO ROW OF MAP
               MAP(K,J,ISEC) = 0.0
            ENDDO
            I = 1
            DO N = 1,NH
               DO M = 1,NCOL
                  MAP(M,J,ISEC) = MAP(M,J,ISEC) + TABLE2(CE)*COSHX(I)
     1                 + TABLE2(CF)*SINHX(I)
                  I      = I + 1
               ENDDO
               CE = CE + 1
               CF = CE + NEF/2
            ENDDO
            DO M = 1,NCOL
               MAP_MIN = AMIN1(MAP_MIN,MAP(M,J,ISEC))
               MAP_MAX = AMAX1(MAP_MAX,MAP(M,J,ISEC))
            ENDDO
         ENDDO
         CC   = CC + NCD/2
         CD   = CC + NCD/2
         Z    = Z + DZ
      ENDDO                     ! DO SECTION_THIS_LOOP = 1,NSECS_THIS_LOOP
      IF(ISEC < NSEC) GO TO 100
      
      CLOSE (UNIT=3, STATUS='DELETE')
      RETURN
      END subroutine fourier



      SUBROUTINE CHECKUP (XMIN, XMAX, DX)

      IMPLICIT NONE

      INTEGER*4 NX
      REAL*4    XMIN, XMAX, DX, DIFFX

 100  DIFFX = XMAX-XMIN
      IF(DIFFX == 0.0) THEN                  ! COMPUTE ONLY 1 SECTION
        DX     = .000001
        RETURN
      ENDIF
      IF(DIFFX < 0.0) THEN                  ! SWITCH XMIN AND XMAX
        XMIN = XMAX
        XMAX = XMIN-DIFFX
        GO TO 100
      ENDIF
      IF(DIFFX > 1.0) THEN                  ! TRAP FOR XMAX-XMIN > 1.0
        XMAX = XMIN+1.0
      ENDIF
      IF(DX <= 0.0) THEN
        XMAX = XMIN
        GO TO 100
      ENDIF
      IF(DIFFX == 1.0) THEN                  ! SET FOR INTEGRAL SAMPLING
        NX   = NINT(DIFFX/DX)
        DX   = DIFFX/NX
        XMAX = XMIN+DIFFX-DX
      ENDIF

      RETURN
      END subroutine checkup



      SUBROUTINE MAP_OUT (MAP, NCOL, NROW, NSEC, A_CELL, B_CELL, C_CELL,
     $     GAM, T_MATRIX, MAP_MIN, MAP_MAX, MAP_FILENAME,
     $     MAP_TITLE)

C     Write the map file to disk

      IMPLICIT NONE

      include '../include/pifhead.inc'
      TYPE(PIFHEAD_T) MAPHEAD

      INTEGER*4 NCOL, NROW, NSEC, I, T_MATRIX(6)
      REAL*4    MAP(NCOL,NROW,NSEC), SCALE1, SCALE2
      REAL*4    A_CELL, B_CELL, C_CELL, GAM
      REAL*4    MAP_MIN, MAP_MAX, MAP_SCALEF
      CHARACTER MAP_FILENAME*256
      character(len=1) RW(2), MAP_TITLE(80)

      WRITE(*,'(2(A,E13.6),/)') ' MIN =', MAP_MIN, ' MAX =', MAP_MAX

C     ----- FINAL OUTPUT OF 3D MAP
      CALL PIF_INIT_HEAD (MAPHEAD)
      DO I =1,80
        MAPHEAD%DESCRIPTIONSTR(I) = MAP_TITLE(I)
      ENDDO
      MAPHEAD%LUNUM = 2
      RW(1)  = 'w'
      RW(2) = achar(0)
      CALL PIF_OPEN (MAP_FILENAME, MAPHEAD%LUNUM, RW)
      MAPHEAD%MODE      =   20
      MAPHEAD%NUMCOLS   = NCOL
      MAPHEAD%NUMROWS   = NROW
      MAPHEAD%NUMSECS   = NSEC
      SCALE1            = 1.01*ABS(MAP_MIN/REAL(-32767))
      SCALE2            = 1.01*ABS(MAP_MAX/REAL(32767))
      MAP_SCALEF        = 1.0/MAX(SCALE1,SCALE2)
      MAPHEAD%SCALEF    = MAP_SCALEF
      MAPHEAD%MAP_ABANG = GAM
      MAPHEAD%AOVERB    = 1.0
      MAPHEAD%A_CELL    = A_CELL                        !!! TIMES PIXSIZ
      MAPHEAD%B_CELL    = B_CELL
      MAPHEAD%C_CELL    = C_CELL
      MAPHEAD%DELA      = A_CELL/REAL(NCOL)
      MAPHEAD%DELB      = B_CELL/REAL(NROW)
      MAPHEAD%DELC      = C_CELL/REAL(NSEC)
      DO I = 1,6
         MAPHEAD%T_MATRIX(I)  = T_MATRIX(I)
      ENDDO
      CALL PIF_WRITE_GH (MAPHEAD)
      MAPHEAD%IMGNUM = 1
      CALL PIF_WRITE_DH (MAPHEAD)
      
      PRINT *, '* Write out final 3D MAP *'
      call pif_write_map_sections(maphead, map,
     $     maphead%mode, ncol, nrow, nsec, 0, nsec)
      
      CALL PIF_CLOSE (MAPHEAD%LUNUM)
      
      RETURN
      END subroutine map_out



      SUBROUTINE SFLOAD (ARRAY, HKL, AP, BP, NSEC, HMAX, KMAX, LMAX, 
     $     NSF, A11, A21, A22, A31, A32, A33, DSQMIN, DSQMAX, SCALE_FAC,
     $     NUM_UNIQUE)

C     LOAD STRUCTURE FACTOR DATA INTO ARRAYS

      IMPLICIT NONE

      INTEGER*4 NSEC, HMAX, KMAX, LMAX, NSF, NUM, NUM_UNIQUE
      INTEGER*4 IH, IK, IL, JL, HH, KK, HKL(3,NSF)
      REAL*4    A11, A21, A22, A31, A32, A33, DSQMIN, DSQMAX, SCALE_FAC
      REAL*4    PI, RADIAN, DEGREE, XSQ, YSQ, ZSQ, DSTAR, MODF, PHASE
      REAL*4    APART, BPART, AP(NSF), BP(NSF)
      COMPLEX   ARRAY(NSEC,HMAX+1,(2*KMAX+1))
      
      CALL PIRADDEG (PI, RADIAN, DEGREE)
      NUM = 0                   ! SF COUNTER
      
      DO IH = (HMAX+1),2,-1     ! L < 0 INDICES
         HH  = -(IH - 1)
         DO IK = (2*KMAX+1),1,-1
            KK  = -(IK - KMAX - 1)
            XSQ = (REAL(KK)*A11)**2
            DO IL = -1,-LMAX,-1
               JL = IL + 1 + NSEC
               YSQ = (REAL(KK)*A21 + REAL(IL)*A22)**2
               ZSQ = (REAL(HH)*A33 + REAL(KK)*A31 + REAL(IL)*A32)**2
               DSTAR = XSQ + YSQ + ZSQ
               IF(DSTAR > DSQMAX .OR. DSTAR < DSQMIN) CYCLE
               APART      =  REAL(ARRAY(JL,IH,IK))
               BPART      = -AIMAG(ARRAY(JL,IH,IK))
               MODF       = SQRT(APART**2+BPART**2) * SCALE_FAC * 2.0
               PHASE      = 0.0
               IF(MODF.NE.0.0)  PHASE = ATAN2(BPART,APART)*DEGREE
               IF(PHASE < 0.0)  PHASE = PHASE + 360.0
               NUM        = NUM + 1
               HKL(1,NUM) = HH
               HKL(2,NUM) = KK
               HKL(3,NUM) = -IL
               AP(NUM)    = MODF
               BP(NUM)    = 360.0-PHASE
            ENDDO
         ENDDO
      ENDDO
      
      DO IH = 1,HMAX+1          ! L  >  0 INDICES
         HH  = IH - 1
         DO IK = 1,(2*KMAX+1)
            KK  = IK - KMAX - 1
            XSQ = (REAL(KK)*A11)**2
            DO IL = 0,LMAX
               IF(IL == 0) THEN
                  IF(HH == 0 .AND. KK > 0) CYCLE
               ENDIF
               YSQ = (REAL(KK)*A21 + REAL(IL)*A22)**2
               ZSQ = (REAL(HH)*A33 + REAL(KK)*A31 + REAL(IL)*A32)**2
               DSTAR = XSQ + YSQ + ZSQ
               IF(DSTAR > DSQMAX .OR. DSTAR < DSQMIN) CYCLE
               JL         = IL + 1
               APART      =   REAL(ARRAY(JL,IH,IK))
               BPART      = -AIMAG(ARRAY(JL,IH,IK))
               MODF       = SQRT(APART**2+BPART**2) * SCALE_FAC * 2.0
               PHASE      = 0.0
               IF(MODF.NE.0.0)  PHASE = ATAN2(BPART,APART)*DEGREE
               IF(PHASE < 0.0)  PHASE = PHASE + 360.0
               NUM        = NUM + 1
               HKL(1,NUM) = HH
               HKL(2,NUM) = KK
               HKL(3,NUM) = IL
               AP(NUM)    = MODF
               BP(NUM)    = PHASE
            ENDDO
         ENDDO
      ENDDO
      
      WRITE(*,'(A,11X,A,I9)') '+',
     1     'Number of unique structure factors  =', NUM
      NUM_UNIQUE = NUM
      
      RETURN
      END subroutine sfload
