C  ********************************************************************
C  *                                                                  *
C  *                       *****  EMSF.FOR  *****                     *
C  *                                                                  *
C  *    MULTIPURPOSE PROGRAM FOR EXAMINING AND/OR MANIPULATING 2D OR  *
C  *     3D STRUCTURE FACTOR DATA.                                    *
C  *                                                                  *
C  *    TOUCH-KEY OPERATIONS:                                         *
C  *    ---------------------                                         *
C  *   -A   ADD/SUBTRACT structure factor data                        *
C  *    B   BACK-transform 2D or 3D SF data to give 2D or 3D MAP      *
C  *   -C   CIRCULAR average for 1D plot of INTS, LOG INTS, or AMPS.  *
C  *    F   Refine 2-,3-,4-,6-fold phase origins                      *
C  *    I   IMPOSE plane group symmetries                             *
C  *   -H   Compute HISTOGRAM of structure factor data values         *
C  *    K   FIX values in SF data                                     *
C  *   -L   CHECK/CHANGE header parameters                            *
C  *   -N   Add random NOISE to structure factor data                 *
C  *    R   R-factor comparisons of two SF data sets.                 *
C  *    S   Get STATISTICS on structure factor data                   *
C  *                                                                  *
C  *    O   OPEN new SF file                                          *
C  *    W   WRITE out the current SF data to disk                     *
C  *    E   EXIT program                                              *
C  *    ?   INFO on EMSF program                                      *
C  *                                                                  *
C  *    APRIL 1989 PURDUE UNIVERSITY VAX 8550, T. BAKER.              *
C  *                                                                  *
C  *    MODIFIED 12-DEC-1997  BY TSB (incorporated ICTF and other RHC *
C  *                                  routines).                      *
C  *    MODIFIED 15-DEC-1998  BY RWA (Port to UNIX; added PIF I/O).   *
C  *    MODIFIED 30-DEC-1999 by TSB (Keep track of SCALEF).           *
C  *    MODIFIED 29-NOV-2001 by TSB/RWA (Linux version)               *
C  *    Modified  5-Dec-2001 by TSB (Add SF3DBT capability).          *
C  ********************************************************************
C
      IMPLICIT NONE
C
      INTEGER*4 LUN_IN, LUN_OUT, NSF_stored, OVERFLOW
      PARAMETER (OVERFLOW=1000)
C
      INTEGER*4 NLIM, SF_TYPE, SF_TITLE(18), T_MATRIX(2,3), nsf_in
      INTEGER*4 EM_NDIM, EM_ERR, FILE_ID, MAP_TITLE(18), PixSizUnits
      REAL*4    SF_SCALEF, SF_SUMFSQ, SF_DSTR_MIN, SF_DSTR_MAX
      REAL*4    A_STAR, B_STAR, C_STAR, ALP_STAR, BET_STAR, GAM_STAR
      REAL*4    AX, AY, BX, BY, CX, CY, TILTAXIS, TILTANG, pixelSize
      REAL*4    A_CELL, B_CELL, C_CELL, ALP, BET, GAM, VOL, VOL_STAR
      REAL*4    MAP_MIN, MAP_MAX, AOVERB, MAP_ABANG, DSTR_MAX_user
      CHARACTER IKEY*1, SF_FILENAME*256
      integer alloc_stat
      DATA      LUN_IN, LUN_OUT / 1, 2 /

      INTEGER*4, allocatable :: HKLS(:)
      REAL*4, allocatable :: AMPS(:), PHIS(:), SIGS(:), DSTRS(:)

C       -----------------------------------------------------------------------
C
      PRINT *, 'EMSF: SF data manipulations'
Ca      PRINT *, '      5-Dec-2001 [TSB]'
      call banner('EMSF\0')
      PRINT *
  100      print *, 'EMSF Options'
      print *, '--------------------------------------------'
      print *, 'A)  MERGE SF data'
      print *, 'B)  BACK-transform 2D/3D SFs to 2D/3D MAP'
CCC      print *, 'C)  1D CIRCULAR average (I/LOG I/A)'
      print *, 'F)  F Refine 2,3,4,6-fold origins (2D SFs)'
      print *, 'I)  I IMPOSE plane group symmetries (2D SFs)'
      print *, 'K)  Fix values in SF data'
      print *, 'R)  R-factors comparisons'
CCC      print *, 'S)  Get STATISTICS'
      print *, ' '
      print *, 'O)  OPEN new SF file'
      print *, 'W)  WRITE SF data to disk'
      print *, 'E)  EXIT program'
CCC      print *, '?)  INFO on running EMSF program'
C
      WRITE(*,'(A)',advance='no') 'Enter Choice: '
      READ (*, '(A)') IKEY
      CALL STRING_UPPER (IKEY)
C  ************************************************************************
C  *    FOR ALL OPTIONS EXCEPT "O", "E" AND "?" TEST IF A FILE IS OPEN    *
C  *     IF NOT, THEN GO TO ERROR MESSAGE AT STATEMENT 900                *
C  ************************************************************************
      IF(IKEY.NE.'E'.AND.IKEY.NE.'O'.AND.IKEY.NE.'?') THEN
CCC        IF(FLAG.EQ.0) GO TO 900
      ENDIF
C
C       -----------------------------------------------------------------------
C       | OPTION O: OPEN A NEW SF DATA FILE |
C       -------------------------------------
      IF(IKEY.EQ.'O') THEN
         PRINT *
         EM_ERR = 1
         DO WHILE (EM_ERR.NE.0)
           SF_FILENAME = ' '
           CALL SF_OPEN (LUN_IN, NSF_stored, SF_TYPE,
     1     SF_FILENAME,
     2     A_STAR, B_STAR, C_STAR, ALP_STAR, BET_STAR, GAM_STAR,
     3     SF_SCALEF,
     4     A_CELL, B_CELL, C_CELL, ALP, BET, GAM,
     5     EM_NDIM, EM_ERR, t_matrix, pixelSize, PixSizUnits)
         ENDDO
         NLIM   = NSF_stored + OVERFLOW

         allocate(hkls(3*nlim), amps(nlim), phis(nlim), sigs(nlim),
     $        dstrs(nlim), stat=alloc_stat)
         if(alloc_stat > 0) then
            write(*,*) 'Unable to allocate HKLS,AMPS,PHIS,SIGS,DSTRS'
            call tsbend
         endif

         CALL SF_FILL (EM_ERR, SF_TYPE,
     1   NSF_stored, HKLS, AMPS, PHIS, SIGS, DSTRS, SF_DSTR_MIN,
     2   SF_DSTR_MAX, SF_SCALEF, SF_SUMFSQ, DSTR_MAX_user, nsf_in)
         CALL HITCR
         GO TO 100
C       -----------------------------------------------------------------------
C       | OPTION A: |
C       --------------------------------------
       ELSE IF (IKEY.EQ.'A') THEN
         GO TO 999
C       -----------------------------------------------------------------------
C       | OPTION B: BACK TRANSFORM SF DATA TO GIVE MAP |
C       ------------------------------------------------
       ELSE IF (IKEY.EQ.'B') THEN
          IF(EM_NDIM.EQ.2) THEN
             CALL EMSF_2DBACKT (SF_TYPE, NSF_stored, HKLS, 
     1       AMPS, PHIS, T_MATRIX, MAP_MIN, MAP_MAX, AOVERB,
     2       ALP_STAR, BET_STAR, GAM_STAR, A_STAR, B_STAR, C_STAR,
     3       VOL, VOL_STAR, MAP_TITLE, A_CELL, B_CELL, C_CELL,  
     4       MAP_ABANG, ALP, BET, GAM)
           ELSE IF(EM_NDIM.EQ.3) THEN
             CALL EMSF_3DBACKT (SF_TYPE, NSF_stored, HKLS, 
     1       AMPS, PHIS, T_MATRIX, MAP_MIN, MAP_MAX, 
     3       VOL, VOL_STAR, A_CELL, B_CELL, C_CELL,  
     4       ALP, BET, GAM, PixSizUnits, nsf_in)
          ENDIF
C       -----------------------------------------------------------------------
C       | OPTION C: CIRCULAR AVERAGE 1D PLOT OF INTS, LOG INTS, OR AMPS |
C       -----------------------------------------------------------------
       ELSE IF (IKEY.EQ.'C') THEN
CCC         CALL SF_1D (NSF, AMPS, PHIS, DSTRS)
C       -----------------------------------------------------------------------
C       | OPTION E: EXIT THE PROGRAM |
C       ------------------------------
       ELSE IF (IKEY.EQ.'E') THEN
         CALL TSBEND
C       -----------------------------------------------------------------------
C       | OPTION F: REFINE 2-,3-,4-,6-FOLD PHASE ORIGINS |
C       --------------------------------------------------
       ELSE IF (IKEY.EQ.'F') THEN
         CALL EMSF_ORIGINS (EM_NDIM, SF_TYPE, NSF_stored, HKLS, AMPS,
     1   PHIS, SF_SUMFSQ, T_MATRIX)
C       -----------------------------------------------------------------------
C       | OPTION H: PLOT HISTOGRAM OF SF DATA VALUES |
C       ----------------------------------------------
       ELSE IF (IKEY.EQ.'H') THEN
         GO TO 999
C       -----------------------------------------------------------------------
C       | OPTION I: IMPOSE plane group symmetries |
C       -------------------------------------------
       ELSE IF (IKEY.EQ.'I') THEN
         CALL EMSF_SYM (EM_NDIM, SF_TYPE, NLIM, NSF_stored, HKLS, AMPS,
     1   PHIS, SIGS, DSTRS, GAM_STAR, T_MATRIX)
C       -----------------------------------------------------------------------
C       | OPTION K: FIX SF VALUES |
C       ---------------------------
       ELSE IF (IKEY.EQ.'K') THEN
          CALL EMSF_FIX (EM_NDIM, NSF_stored, HKLS, AMPS, PHIS, SIGS,
     $         DSTRS,SF_TYPE, SF_SCALEF, SF_DSTR_MIN, SF_DSTR_MAX,
     $         T_MATRIX)
C       -----------------------------------------------------------------------
C       | OPTION L: CHECK/CHANGE HEADER PARAMETERS |
C       --------------------------------------------
       ELSE IF (IKEY.EQ.'L') THEN
         GO TO 999
C       -----------------------------------------------------------------------
C       | OPTION M: |
C       -----------------------------------------------
       ELSE IF (IKEY.EQ.'M') THEN
         GO TO 999
C       -----------------------------------------------------------------------
C       | OPTION N: ADD RANDOM NOISE TO SF DATA |
C       -----------------------------------------
       ELSE IF (IKEY.EQ.'N') THEN
         GO TO 999
C       -----------------------------------------------------------------------
C       | OPTION R: R-FACTOR COMPARISONS OF 2 SF DATA SETS |
C       ----------------------------------------------------
       ELSE IF (IKEY.EQ.'R') THEN      
         CALL EMSF_COMPARE (SF_TYPE, NSF_stored, HKLS, AMPS, 
     1   PHIS, SF_DSTR_MAX)
C       -----------------------------------------------------------------------
C       | OPTION S: GET STATISTICS ON SF DATA |
C       ---------------------------------------
       ELSE IF (IKEY.EQ.'S') THEN
CCC         CALL EMSF_STATS (NSF_stored, AMPS, PHIS,
CCC      1   DSTRS, SF_DSTR_MAX, SF_SCALEF)
C       -----------------------------------------------------------------------
C       | OPTION W: WRITE OUT THE SF DATA TO A DISK FILE |
C       --------------------------------------------------
       ELSE IF (IKEY.EQ.'W') THEN
         CALL SF_WRITE_pif (FILE_ID, EM_NDIM, SF_TYPE, SF_FILENAME,
     1   SF_TITLE, SF_SCALEF, SF_DSTR_MIN, SF_DSTR_MAX, LUN_OUT, 
     2   NSF_stored, HKLS, AMPS, PHIS, SIGS, 
     3   DSTRS, T_MATRIX, A_STAR, B_STAR, C_STAR, ALP_STAR, 
     4   BET_STAR, GAM_STAR, AX, AY, BX, BY, CX, CY, TILTAXIS, TILTANG,
     5   a_cell,b_cell,c_cell,alp,bet,gam,vol,vol_star, nsf_in)

C       -----------------------------------------------------------------------
C       | OPTION Y: |
C       -------------
       ELSE IF (IKEY.EQ.'Y') THEN
         GO TO 999
C --------------------------------------------------------------------
       ELSE IF (IKEY.EQ.'?') THEN
CCC         CALL SCR_DISPLAY_HELP ('SYS$DOCLIB:EMSF.DOC')
      ENDIF
      GO TO 100
C  ***********************************
C  *    HERE IF MAP FILE NOT OPEN    *
C  ***********************************
      GO TO 100
C
  999      CALL BELL (2)
      PRINT *, '************************************'
      PRINT *, '***** OPTION NOT YET AVAILABLE *****'
      PRINT *, '************************************'
      CALL HITCR
      GO TO 100

      if(allocated(hkls))  deallocate(hkls)
      if(allocated(amps))  deallocate(amps)
      if(allocated(phis))  deallocate(phis)
      if(allocated(sigs) ) deallocate(sigs)
      if(allocated(dstrs)) deallocate(dstrs)

      END

C  ****************************************************************************
C  *                                                                          *
C  *    EMSF subroutines used by EMSF                                         *
C  *                                                                          *
C  *    EMSF_2DBACKT      - Fourier synthesis of 2D SF data.                    *
C  *    EMSF_3DBACKT      - Fourier synthesis of 3D SF data.                    *
C  *    EMSF_COMPARE      - Compare SF data sets.                               *
C  *    EMSF_FIX      - Change SF values.                                   *
C  *    EMSF_ICTF      - Apply inverse CTF to SF values.                     *
C  *    EMSF_ORIGINS      - Refine 2-,3-,4-, or 6-fold phase origins.           *
C  *    EMSF_STATS      - Compute SF data statistics.                         *
C  *    EMSF_SYM      - Impose plane group symmetries.                      *
C  *                                                                          *
C  *    MARCH 1992  PURDUE UNIVERSITY VAX 8550, T. BAKER.                     *
C  ****************************************************************************
C
C
C

C  ****************************************************************************
C  *                                                                          *
C  *               *****     SUBROUTINE EMSF_2DBACKT     *****                *
C  *                                                                          *
C  *    ROUTINE TO COMPUTE FOURIER SYNTHESIS OF TWO-DIMENSIONAL STRUCTURE     *
C  *     FACTOR DATA USING A 64x64 OR 128x128 FFT ROUTINE.                    *
C  *                                                                          *
C  *    FEBRUARY 1980  BRANDEIS UNIVERSITY PDP 11/40, T. BAKER  (FOUR2D).     *
C  *    SEPTEMBER 1982  BRANDEIS UNIVERSITY VAX 11/780, T. BAKER.             *
C  *    JANUARY 1983  PURDUE UNIVERSITY VAX 11/750, T. BAKER (EMSF2DBT).      *
C  *    AUGUST 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER (EMSF2DBT).         *
C  *                                                                          *
C  *    MARCH 1992  PURDUE UNIVERSITY VAX 8550, T. BAKER (EMSF_BACKT).        *
C  *    LAST MODIFIED 12-DEC-1997 BY T. BAKER.                                *
C  ****************************************************************************
C
      SUBROUTINE EMSF_2DBACKT (SF_TYPE, NSF, HKLS, AMPS, PHIS,
     1 T_MATRIX, MAP_MIN, MAP_MAX, AOVERB, ALP_STAR, BET_STAR, 
     2 GAM_STAR, A_STAR, B_STAR, C_STAR, VOL, VOL_STAR, MAP_TITLE,
     3 A_CELL, B_CELL, C_CELL, MAP_ABANG, ALP, BET, GAM)
C
      IMPLICIT NONE
C
      INTEGER*4 NSF, NN(2), N1, N2, I, SF_TYPE, T_MATRIX(2,3)
      INTEGER*4 MAP_TITLE(18)
      INTEGER*4 HKLS(3,NSF)
      REAL*4    A_CELL, B_CELL, C_CELL, MAP_ABANG, ALP, BET, GAM
      REAL*4    MAP_MIN, MAP_MAX, AMPS(NSF), PHIS(NSF)
      REAL*4    AOVERB, ALP_STAR, BET_STAR, GAM_STAR
      REAL*4    A_STAR, B_STAR, C_STAR, VOL, VOL_STAR
      integer   alloc_stat
      real*4, allocatable :: array(:)

C  ----------------------------------------------------------------------------
C
      PRINT *, 'EMSF_2DBACKT: Back transform 2D SF data --> 2D MAP'
      PRINT *
C
 100  WRITE(6,'(2A)',advance='no') 
     1     '$COMPUTE 64x64<CR>, 128x128(1) or 256x256(2)',
     1     ' MAP? : '
      READ (5,*,ERR=100) I
      IF(I.EQ.1) THEN
         NN(1) = 128
         NN(2) = 128
       ELSE IF (I.EQ.2) THEN
         NN(1) = 256
         NN(2) = 256
       ELSE
         NN(1) = 64
         NN(2) = 64
      ENDIF
      N1 = NN(1)
      N2 = NN(2)

      allocate(array(2*nn(1)*nn(2)), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate ARRAY in EMSF_2DBACKT'
         call tsbend
      endif

      CALL SF_BT_LOAD (SF_TYPE,NSF,HKLS,AMPS,PHIS,N1,N2,ARRAY,T_MATRIX)
C  **************************************************
C  *    BACK TRANSFORM THE STRUCTURE FACTOR DATA    *
C  **************************************************
      CALL FOURT (ARRAY, NN, 2, -1, 1, 0)
      CALL SF_BT_MINMAX (N1, N2, ARRAY, MAP_MIN, MAP_MAX)
      CALL SF_BT_MAPOUT (N1, N2, ARRAY, MAP_MIN, MAP_MAX,
     1 T_MATRIX, A_CELL, B_CELL, C_CELL, MAP_ABANG, ALP, BET, GAM,
     2 AOVERB, ALP_STAR, BET_STAR, GAM_STAR, A_STAR, B_STAR, C_STAR,
     3 VOL, VOL_STAR, MAP_TITLE)

      deallocate(array)

      RETURN
      END

C  ************************************************************************
C  *                                                                      *
C  *             *****     SUBROUTINE EMSF_3DBACKT     *****              *
C  *                                                                      *
C  *    USES UNIVERSITY OF CANTERBURY FOURIER PROGRAM                     *
C  *                                                                      *
C  *    THIS IS A MODIFIED VERSION OF FOUR1.F4 FROM CYNDY STAUFFACHER     *
C  *     WHICH RAN ON THE BRANDEIS DEC10 CAMPUS COMPUTER WHICH WAS        *
C  *     REWRITTEN FROM FORDAP (A.ZALKIN)                                 *
C  *                                                                      *
C  *    OUTPUT OF THIS SUBROUTINE IS A 3D PIF FORMAT MAP FILE.            *
C  *                                                                      *
C  *    UNITS: = 3D PIF FORMAT 3D MAP OUTPUT FILE                         *
C  *                                                                      *
C  *    APRIL 1981  BRANDEIS UNIVERSITY PDP 11/40, T. BAKER               *
C  *    SEPTEMBER 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER            *
C  *    APRIL 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER                  *
C  *                                                                      *
C  *    MODIFIED  5-AUG-1998 BY TSB (Compute MAP in core memory).         *
C  *                                (Output PIF format MAP).              *
C  *    MODIFIED 12-AUG-1998 BY TSB (Fast input of SF data).              *
C  *    MODIFIED  3-NOV-1998 BY TSB/RWA (PIF format SF data).             *
C  *    MODIFIED 23-DEC-1998 BY TSB/RWA (Scale MAP by 2.0).               *
C  *    MODIFIED 21-JUL-1999 BY TSB/RWA (Fix SCALE1 problem).             *
C  *    MODIFIED 29-NOV-2001 BY TSB (Linux version).                      *
C  *    Modified  5-Dec-2001 by TSB (Move SF3DBT subroutines to SFLIB).   *
C  ************************************************************************
C
      SUBROUTINE EMSF_3DBACKT (SF_TYPE, NSF_stored, HKLS,
     1 AMPS, PHIS, T_MATRIX, MAP_MIN, MAP_MAX,
     3 VOL, VOL_STAR, A_CELL, B_CELL, C_CELL, ALP, BET, GAM,
     4 PixSizUnits, nsf_in)
C
      IMPLICIT NONE
C
      COMMON /CONST/ TRGTBL, XMIN, YMIN, ZMIN, XMAX, YMAX, ZMAX,
     1 DX, DY, DZ, DCELL, ITRANS, HMAX, KMAX, KMIN, RCELL, SCALE,
     2 TS, U, UINV, V
C
      INTEGER*4 NEQUIV, NSF_stored, DIV, NCOL, NROW, NSEC, NCD, NEF
      INTEGER*4 PixSizUnits, SF_TYPE, T_MATRIX(2,3), nsf_in
      INTEGER*4 HKLS(3,NSF_stored)
      INTEGER*4 ITRANS, HMAX, KMAX, KMIN
      INTEGER*4 NORIEN, TABLE_SIZE, NREF_DUMP, NUM_UNIQUE
      REAL*4    XMIN, YMIN, ZMIN, XMAX, YMAX, ZMAX, DX, DY, DZ
      REAL*4    DCELL(9), RCELL(9), SCALE, TS(3,24), U(3,3)
      REAL*4    UINV(3,3), V, TRGTBL(1001)
      REAL*4    FS(3,3,24), DETU, AA(3,3), BB(3,3)
      REAL*4    A_CELL, B_CELL, C_CELL, ALP, BET, GAM, VOL, VOL_STAR
      REAL*4    MAP_MIN, MAP_MAX
      REAL*4    AMPS(NSF_stored), PHIS(NSF_stored)
      CHARACTER MAP_FILENAME*256
      character(len=1) PIF_MAP_TITLE(80)

      integer   alloc_stat
      INTEGER*4, allocatable :: HH(:), KK(:), LL(:)
      REAL*4, allocatable :: MAP(:), AP(:), BP(:), TABLE1(:), TABLE2(:)
      REAL*4, allocatable :: SINHX(:), COSHX(:), SINKY(:), COSKY(:)

C  ----------------------------------------------------------------------------
C
      PRINT *, 'EMSF_3DBACKT: Back transform 3D SF data --> 3D MAP'
      PRINT *
C  *************************************
C  *    ENTER NAME OF FINAL MAP FILE   *
C  *************************************
      WRITE(*,'(A)',advance='no') 'Filename for output 3D MAP: '
      READ (*,'(A)') MAP_FILENAME
      WRITE(*,'(A)',advance='no') 'Title: '
      READ (*,'(80A1)') PIF_MAP_TITLE
C
      CALL SF3DBT_INFO (NEQUIV, FS, AA, BB, A_CELL, B_CELL, C_CELL,
     1 ALP, BET, GAM, VOL, VOL_STAR, NREF_DUMP)
      IF(ITRANS.GE.1) CALL SF3DBT_TRANS (DETU, AA, PixSizUnits)
C  **************************************************************************
C  *     SET FOURIER LIMITS AND WRITE DATA FILE CONTAINING EXPANDED DATA    *
C  **************************************************************************

      allocate(HH(nsf_in*NEQUIV), KK(nsf_in*NEQUIV),
     $     LL(nsf_in*NEQUIV), AP(nsf_in*NEQUIV),
     $     BP(nsf_in*NEQUIV), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate HH, KK, LL, AP, BP'
         call tsbend
      endif

      CALL SF3DBT_INPUT (NCOL, NROW, NSEC, NEQUIV, NORIEN, NCD, NEF, 
     1 FS, HKLS, AMPS, PHIS, nsf_in, AP, BP, HH, KK, LL, SF_TYPE, 
     2 NREF_DUMP, NUM_UNIQUE, PixSizUnits)

      allocate(MAP(NCOL*NROW*NSEC), TABLE2(NEF), SINHX(NCOL*(HMAX+1)),
     $     COSHX(NCOL*(HMAX+1)), SINKY(NROW*(KMAX-KMIN+1)),
     $     COSKY(NROW*(KMAX-KMIN+1)), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate MAP etc. in EMSF_3DBACKT'
         call tsbend
      endif

      DIV        = 1
      TABLE_SIZE = NSEC*NCD/DIV

      allocate(TABLE1(table_size), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate TABLE1  in EMSF_3DBACKT'
         call tsbend
      endif

C  ***********************************************
C  *    CALCULATE FOURIER, SECTION BY SECTION    *
C  ***********************************************
      CALL SF3DBT_FOURIER (MAP, TABLE_SIZE, NCD, NEF, TABLE1, TABLE2, 
     1 NCOL, NROW, NSEC, AP, BP, HH, KK, LL, SINHX, COSHX, SINKY, 
     2 COSKY, MAP_MIN, MAP_MAX, NUM_UNIQUE, NEQUIV)

      deallocate(HH, KK, LL, AP, BP, SINHX, COSHX, SINKY,
     $     COSKY, TABLE1, TABLE2)

      CALL SF3DBT_OUT (MAP, NORIEN, NCOL, NROW, NSEC, A_CELL, 
     1 B_CELL, C_CELL, ALP, BET, GAM, T_MATRIX, MAP_MIN, MAP_MAX, 
     2 MAP_FILENAME, PIF_MAP_TITLE, PixSizUnits)

      deallocate(map)

      RETURN
      END

C  ********************************************************************
C  *                                                                  *
C  *            *****     SUBROUTINE EMSF_COMPARE     *****           *
C  *                                                                  *
C  *    HAS SEVERAL FUNCTIONS TO COMPARE SF DATA SETS.                *
C  *                                                                  *
C  *    A  COMPARE TWO SF DATA SETS.                                  *
C  *    B  COMPARE R-FACTOR BETWEEN TWO SF DATA SETS.                 *
C  *                                                                  *
C  *    FEBRUARY 1990  PURDUE UNIVERSITY VAX 8550, T. BAKER.          *
C  ********************************************************************
C
      SUBROUTINE EMSF_COMPARE (SF_TYPE, NSF, HKLS, AMPS, PHIS, 
     1 SF_DSTR_MAX)
C                 
      IMPLICIT NONE
C
      INTEGER*4 NSF, SF_TYPE
      INTEGER*4 HKLS(3,NSF)
      REAL*4    AMPS(NSF), PHIS(NSF), SCALEF
      REAL*4    SF_DSTR_MAX
      REAL*4    A_CELL, B_CELL, C_CELL, ALP, BET, GAM
      REAL*4    A_STAR, B_STAR, C_STAR, ALP_STAR, BET_STAR, GAM_STAR
      CHARACTER IKEY*1
C -----------------------------------------------------------------------------
C
  100      print *, ' '
      print *, 'OPTION R: EMSF_COMPARE TUOCH-KEY MENU'
      print *, '-------------------------------------'
      print *, 'A  Compute scale factor between two data sets'
      print *, 'B  Compute R-factor between two data sets'
      print *, 'E  EXIT subroutine EMSF_COMPARE'
      print *, 'TOUCH-KEY OPTION:'
  200      print *, ' '
      READ (*, '(A)')IKEY

      CALL STRING_UPPER (IKEY)
      IF(IKEY.NE.'E') THEN
         IF(IKEY.EQ.'A') THEN
            CALL SF_SCALEFAC (NSF, 
     1      SF_TYPE, HKLS, AMPS, PHIS, SF_DSTR_MAX, A_CELL, 
     2      B_CELL, C_CELL, ALP, BET, GAM, A_STAR, B_STAR, C_STAR, 
     3      ALP_STAR, BET_STAR, GAM_STAR, SCALEF)
          ELSE IF(IKEY.EQ.'B') THEN
            CALL SF_RFACTOR (NSF,
     1      SF_TYPE, SF_DSTR_MAX, HKLS, AMPS, PHIS, A_STAR,
     2      B_STAR, C_STAR, ALP_STAR, BET_STAR, GAM_STAR,
     3      A_CELL, B_CELL, C_CELL, ALP, BET, GAM)
          ELSE
            print *, '**********************************'
            print *, '* NOT A VALID OPTION - TRY AGAIN *'
            print *, '**********************************'
            GO TO 200      
         ENDIF
       ELSE
         RETURN
      ENDIF
      GO TO 100
C
      END

C  ********************************************************************
C  *                                                                  *
C  *              *****     SUBROUTINE EMSF_FIX     *****             *
C  *                                                                  *
C  *    SEVERAL FUNCTIONS TO MANIPULATE SF VALUES.                    *
C  *                                                                  *
C  *    A MULTIPLY SF DATA BY A CONSTANT.                             *
C  *    B REVERSE PHASES WITHIN RANGE RES_MIN TO RES_MAX.             *
C  *    C APPLY CONTRAST TRANSFER FUNCTION (CTF)                      *
C  *    D APPLY SCALE FACTORS AS A FUNCTION OF D*                     *
C  *    H HYBRIDIZE PHASE DATA FROM OTHER FILE                        *
C  *    I APPLY INVERSE CONTRAST TRANSFER FUNCTION (ICTF)             *
C  *    S APPLY A PHASE SHIFT TO THE SF DATA.                         *
C  *    T APPLY TEMPERATURE FACTOR TO SF DATA                         *
C  *    Z ADD ZERO SF                                                 *
C  *                                                                  *
C  *    FEBRUARY 1990  PURDUE UNIVERSITY VAX 8550, T. BAKER.          *
C  *    MODIFIED 20-Jan-1992 BY H. CHENG.                             *
C  *    LAST MODIFIED 25-NOV-1997 BY T. BAKER.                        *
C  ********************************************************************
C
      SUBROUTINE EMSF_FIX (EM_NDIM, NSF, HKLS, AMPS, PHIS, SIGS, DSTRS, 
     1 SF_TYPE, SF_SCALEF, SF_DSTR_MIN, SF_DSTR_MAX, T_MATRIX)
C
      IMPLICIT NONE
C
      INTEGER*4 NSF, N1, N2, NUMCHARI4, SF_TYPE, EM_NDIM
      INTEGER*4 T_MATRIX(2,3)
      INTEGER*4 HKLS(3,NSF)
      REAL*4    AMPS(NSF), PHIS(NSF), SIGS(NSF), DSTRS(NSF), SCALEF
      REAL*4    TEMPFAC, RES_MIN, RES_MAX, PIX_SIZE, SF_SCALEF
      REAL*4    SF_DSTR_MIN, SF_DSTR_MAX, XSHFT, YSHFT, ZSHFT
      CHARACTER IKEY*1
      LOGICAL   ICTF
C ----------------------------------------------------------------------
C
 100      print *, ' '
      print *, 'OPTION K: EMSF_FIX MENU'
      print *, '------------------------------------------'
      print *, 'A  Multiply SF data by a constant'
      print *, 'B  Invert phases (contrast)'
CCC      print *, 'C  Apply CONTRAST TRANSFER FUNCTION (CTF)'
CCC      print *, 'F  APPLY scaling factors as function of d*'
CCC      print *, 'H  Hybridize PHASE data from other file'
      print *, 'I  Apply INVERSE CTF (ICTF)'
      print *, 'S  SHIFT phase origin '
      print *, 'T  Apply TEMPERATURE factor to SF data'
CCC      print *, 'Z  Add zero SF'
      print *, 'E  EXIT subroutine EMSF_FIX'
      print *, 'TOUCH-KEY OPTION:_'
 200      print *, ' '
      READ (*, '(A)')IKEY

      CALL STRING_UPPER (IKEY)
      IF(IKEY.NE.'E') THEN
         IF(IKEY.EQ.'A') THEN
 300        WRITE(6,'(A)') '$SCALE FACTOR (<CR>=1.0): '
            READ (5,*,ERR=300) SCALEF
            IF(SCALEF.EQ.0.0) RETURN
            CALL SF_RESCALE(SF_TYPE,SF_SCALEF,SCALEF,NSF,AMPS,PHIS,SIGS)
          ELSE IF(IKEY.EQ.'B') THEN
            N1 = NUMCHARI4(NINT(SF_DSTR_MIN)) + 6
            N2 = NUMCHARI4(NINT(SF_DSTR_MAX)) + 6
 400        WRITE(6,'(A,F12.5,A,F12.5,A)') '$RES_MIN,RES_MAX (<CR>=',
     1      SF_DSTR_MIN, ',', SF_DSTR_MAX, '): '
            READ (5,*,ERR=400) RES_MIN, RES_MAX
            IF(RES_MIN.LE.0.0) RES_MIN = SF_DSTR_MIN
            IF(RES_MAX.LE.0.0) RES_MAX = SF_DSTR_MAX
            CALL SF_FLIPPHASE (SF_TYPE,NSF,AMPS,PHIS,DSTRS,
     $           RES_MIN,RES_MAX)
CRHC          ELSE IF(IKEY.EQ.'C') THEN
CRHC            ICTF = .FALSE.
CRHC            CALL EMSF_ICTF (NSF, HKLS, AMPS, PHIS, DSTRS, ICTF)
CRHC          ELSE IF(IKEY.EQ.'F') THEN
CRHC            CALL SF_FUNC_SCL (NSF, HKLS, AMPS, PHIS, SIGS, DSTRS)
          ELSE IF(IKEY.EQ.'I') THEN
            ICTF = .TRUE.
            CALL EMSF_ICTF (SF_TYPE, NSF, AMPS, PHIS, DSTRS, 
     1        ICTF, SF_DSTR_MIN, SF_DSTR_MAX)
CRHC          ELSE IF(IKEY.EQ.'H') THEN
CRHC            CALL SF_CMB_PH (NSF, HKLS, AMPS, PHIS, SIGS, DSTRS,
CRHC      1      SF_DSTR_MIN, SF_DSTR_MAX) 
          ELSE IF(IKEY.EQ.'S') THEN
            IF(EM_NDIM.EQ.2) THEN
  601               WRITE(*,'(A)') '$XSHIFT,YSHIFT (2F): '
               READ (*,*,ERR=601) XSHFT, YSHFT
               CALL SF_HKL (NSF, HKLS, 1, T_MATRIX)
             ELSE IF(EM_NDIM.EQ.3) THEN
  701               WRITE(*,'(A)') '$XSHIFT,YSHIFT,ZSHIFT (3F): '
               READ (*,*,ERR=701) XSHFT, YSHFT, ZSHFT
            ENDIF
            CALL SF_PSHIFT (EM_NDIM, SF_TYPE, NSF, HKLS, AMPS, PHIS, 
     1                     XSHFT, YSHFT, ZSHFT)
            IF(EM_NDIM.EQ.2) CALL SF_HKL (NSF, HKLS, -1, T_MATRIX)
         ELSE IF(IKEY.EQ.'T') THEN
            PRINT *, '*************************************************'
            PRINT *, '* ENTER RES (in inverse pixels, nm, or Angstr): *'
            PRINT *, '*  At this resolution amplitudes drop to 1/exp  *'
            PRINT *, '*************************************************'
 500            WRITE(*,'(A)') '$RES[1/pixel], pix_size[A] (<CR>=1.0): '
            READ (*,*,ERR=500) TEMPFAC, PIX_SIZE
            IF(TEMPFAC.LE.0.0) TEMPFAC = 1.0
            PRINT *, ' TEMP_FACT: ', TEMPFAC, '[1/pixel]'
            IF(PIX_SIZE.GT.0.) THEN 
                PRINT *
                PRINT *, 'XTAL_BFAC: ', (2.0/TEMPFAC/pix_size)**2
                CALL HITCR
            ENDIF
            CALL SF_TFAC(SF_TYPE, NSF, HKLS, AMPS, PHIS, DSTRS, TEMPFAC)
CRHC          ELSE IF(IKEY.EQ.'Z') THEN
CRHC            CALL SF_ADDZERO (NSF,AMPS,PHIS,DSTRS,RES_MIN,RES_MAX,HKLS)
          ELSE
            PRINT *, '**********************************'
            PRINT *, '* NOT A VALID OPTION - TRY AGAIN *'
            PRINT *, '**********************************'
            GO TO 200
         ENDIF
       ELSE
         RETURN
      ENDIF
      GO TO 100
C
      END

C  **************************************************************************
C  *                                                                        *
C  *                 *****     SUBROUTINE EMSF_ICTF     *****               *
C  *                                                                        *
C  *    APPLY INVERSE CONTRAST TRANSFER FUNCTION TO STRUCTURE FACTOR DATA   *
C  *                                                                        *
C  *      InverseCTF applies an inverse CTF to the Fourier tranform data in   *
C  *      the form of structure factors.  This routine gives you a plot       *
C  *      to actually check the graphics feature of the amplitude before      *
C  *      and after multiplying the function of contrast transfer.            *
C  *                                                                        *
C  *    UNITS OF VARIABLES:                                                 *
C  *                                                                        *
C  *      DEFOCUS (nanometers)                                              *
C  *      Cs  (spherical aberration coefficient) (mm)                        *
C  *      VOLTAGE (volts)                                                   *
C  *      WAVELENGTH (picometers)                                           *
C  *                                                                        *
C  *    MARCH 1990  PURDUE UNIVERSITY VAX 8550, R. H. CHENG                 *
C  *    LAST MODIFIED 25-NOV-1997 BY  T. BAKER                                  *
C  **************************************************************************
C
      SUBROUTINE EMSF_ICTF (SF_TYPE, NSF, AMPS, PHIS, DSTRS,
     1     ICTF, SF_DSTR_MIN, SF_DSTR_MAX)
C
      IMPLICIT NONE
C
      INTEGER*4 NSF, SF_TYPE
      REAL*4    AMPS(NSF), PHIS(NSF), DSTRS(NSF), DSTR_LO
      REAL*4    DEFOCUS, AFRACT, PFRACT, RES_LIMIT_READ
      REAL*4    CS, VOLTAGE, WAVELENGTH, WAVE, WAVEL
      REAL*4    RES_LIMIT
      REAL*4    PI, TWOPI, RADIAN, DEGREE, CTF_MAX
      REAL*4    ZEROES(0:5), TMP1, TMP2, MAG_RD, SCAN_STEP_RD
      REAL*4    SCAN_STEP, MAG, PIX_STEP, DSTR_HI
      REAL*4    SF_DSTR_MIN, SF_DSTR_MAX
      CHARACTER IKEY*1
      LOGICAL   RSLRD, ICTF
      DATA      MAG/49000./,DEFOCUS/1000./,PIX_STEP/.51/
      DATA      SCAN_STEP/25./,PFRACT/1./,AFRACT/0.07/
      DATA        VOLTAGE/200000./, CS/2.0/
      SAVE      MAG, SCAN_STEP, VOLTAGE

      integer   alloc_stat
      real*4, allocatable :: bin1(:), bin2(:), avg0(:), avg1(:)

C  ---------------------------------------------------------------------
C
C  *****************************************************
C  *    SET UP INITIAL VALUES FOR PROGRAM VARIABLES    *
C  *****************************************************
      CALL PIRADDEG (PI, RADIAN, DEGREE)
      TWOPI          = 2.0*PI
      RES_LIMIT_READ = -1.0
      RSLRD             = .FALSE.
      WAVELENGTH     = 1000.0 * WAVEL(VOLTAGE)
C
 100  CALL EMSF_CTF_ZEROES (WAVELENGTH, ZEROES, CTF_MAX,
     1     AFRACT, PFRACT, CS, WAVE, DEFOCUS, TWOPI)
      RES_LIMIT = ZEROES(1)*PIX_STEP
      IF(.NOT.RSLRD.OR.RES_LIMIT_READ.EQ.-1.) THEN
        RES_LIMIT = ZEROES(1)*PIX_STEP
      ENDIF
c cc      CTF_MAX = PIX_STEP / CTF_MAX
C
 200  PRINT *
      WRITE(*,2002) PIX_STEP, DEFOCUS, VOLTAGE, PFRACT, AFRACT, 
     1 PIX_STEP/RES_LIMIT
 2002 FORMAT (6X,'PIXEL SIZE  DEFOCUS  VOLTAGE PFRACT AFRACT RES_LIMIT
     1 (1st zero)',/,6X,66('-'),/,6X,F10.4,2F9.1,2F7.2,F9.4,' (nm)') 
C
      CALL EMSF_ICTF_MENU
      READ (*,'(A)')IKEY
      CALL STRING_UPPER (IKEY)
C
      IF(IKEY.EQ.'A') THEN                              ! SET Cs VALUE
         CALL CTF_CS_COEF (CS)
       ELSE IF(IKEY.EQ.'E') THEN
         PRINT *
         RETURN
       ELSE IF(IKEY.EQ.'D') THEN                     ! SET DEFOCUS LEVEL
         CALL CTF_DEFOCUS (DEFOCUS)
       ELSE IF(IKEY.EQ.'G') THEN
         GO TO 2001
       ELSE IF(IKEY.EQ.'M') THEN                     ! SET MAGNIFICATION
 110      WRITE(*,1100) MAG, SCAN_STEP
          READ(*,*,ERR=110) MAG_RD, SCAN_STEP_RD
          IF(SCAN_STEP_RD.NE.SCAN_STEP.AND.SCAN_STEP_RD.GT.0.)
     1    SCAN_STEP = SCAN_STEP_RD
          IF(MAG_RD.NE.MAG.AND.MAG_RD.GT.0.) MAG = MAG_RD
 120      WRITE(*,1200) (1000.0*SCAN_STEP)/MAG
          READ(5,*,ERR=120) PIX_STEP
          IF(PIX_STEP.LE.0.) PIX_STEP = 1000.*SCAN_STEP/MAG
       ELSE IF(IKEY.EQ.'R') THEN
         RSLRD         = .FALSE.
 210         WRITE(*,2100) 1.0/RES_LIMIT
         READ(5,*,ERR=210) RES_LIMIT_READ
         IF(RES_LIMIT_READ.EQ.0.0) THEN
            RES_LIMIT_READ = RES_LIMIT
          ELSE
            RES_LIMIT_READ = 1/RES_LIMIT_READ
         ENDIF
         IF(RES_LIMIT_READ.GT.0..AND.RES_LIMIT_READ
     1                  .LE.ZEROES(1)*PIX_STEP) THEN      
           RES_LIMIT = RES_LIMIT_READ
          ELSE IF(RES_LIMIT_READ.GT.ZEROES(1)*PIX_STEP) THEN
           PRINT *, 'RES_LIMT > 1st ZERO; NOT AVAILABLE NOW'
           CALL TSBEND
c D           CALL GET_YESNO(OVERSL,.FALSE.)
c D           IF(.NOT.OVERSL) CALL TSBEND
         ENDIF
         IF(RES_LIMIT_READ.NE.RES_LIMIT) RSLRD = .TRUE.
       ELSE IF(IKEY.EQ.'V') THEN                        ! SET VOLTAGE
         CALL CTF_VOLTAGE (VOLTAGE, WAVELENGTH)
       ELSE IF(IKEY.EQ.'Z') THEN                 ! SET AMPLITUDE FRACTION
         CALL CTF_SETFRACT (PFRACT, AFRACT)
      ENDIF
      PRINT *
      GO TO 100
C
 2001      PRINT *
      WAVE = WAVELENGTH/1000.0                  ! CONVERT TO NM UNITS
      CS   = CS*1000000.0                          ! CONVERT TO NM UNITS
C
C  ****************************************************
C  *    DETERMINE THE STEP SIZE IN RECIPROCAL UNITS   *
C  ****************************************************
      DSTR_LO = SF_DSTR_MIN
      DSTR_HI = RES_LIMIT
      DSTR_HI      = AMIN1(DSTR_HI,SF_DSTR_MAX)
C
      PRINT *, '***********************'
      PRINT *, '* Enter range of data *'
      PRINT *, '***********************'
C
      TMP1 = 0.0
      TMP2 = 0.0
 300  WRITE(*,'(A,F8.5,A,F8.5,A)') ' DSTR_LO,DSTR_HI (<CR>=',
     1     SF_DSTR_MIN, ',', DSTR_HI, '): '
      READ (*,*,ERR=300) TMP1, TMP2
C
      IF(TMP1.GT.0..AND.TMP1.GT.SF_DSTR_MIN) DSTR_LO = TMP1
      IF(.NOT.ICTF) THEN
        IF(TMP2.GT.0.) DSTR_HI = TMP2
      ELSE
        IF(TMP2.GT.0..AND.TMP2.LT.SF_DSTR_MAX) THEN
          DSTR_HI = TMP2
         ELSE IF(TMP2.GT.SF_DSTR_MAX) THEN
          PRINT *, 'DSTR_HI RESET: TO MAXIUM OF SF W/in 1ST ZERO'
       ENDIF
      ENDIF
C
      PRINT *,'********************************************'
      WRITE(*,'(A,F8.5,A,F8.5,A)') '* DSTR_LO,DSTR_HI =', SF_DSTR_MIN, 
     1 ',', DSTR_HI, ' *'
      PRINT *,'********************************************'

      allocate(bin1(nsf), bin2(nsf), avg0(nsf), avg1(nsf), 
     $     stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate BIN1, BIN2, AVG0, AVG1'
         call tsbend
      endif

      CALL EMSF_MULTICTF (SF_TYPE, NSF, AMPS, PHIS,DSTRS, DSTR_LO,
     1     DSTR_HI, BIN1, BIN2, ICTF, 
     2     CTF_MAX, AFRACT, PFRACT, 
     3     CS, WAVE, DEFOCUS, TWOPI, PIX_STEP)
      
      deallocate(bin1, bin2, avg0, avg1)

      CS = CS/1000000.0                        ! Return to mm units
      GO TO 200
C
 1100 FORMAT('$MAG, SCAN_STEP (<CR>=',F8.1,', ',F5.1,' MICRON): ') 
 1200 FORMAT('$PIXEL SIZE (<CR>= ',F7.4,'nm): ') 
 2100 FORMAT('$RES_LIMIT (<CR>= ',F7.4,' nm): ')
      END

C  ****************************************************************************
C  *                                                                          *
C  *                 *****   SUBROUTINE EMSF_ICTF_MENU     *****              *
C  *                                                                          *
C  *    SEPTEMBER 1988  PURDUE UNIVERSITY VAX 8550, T. BAKER                  *
C  *    MODIFIED 25-SEP-90 BY  R. H. CHENG                                    *
C  *    LAST MODIFIED 25-NOV-1997 BY  T. BAKER                                *
C  ****************************************************************************
C
      SUBROUTINE EMSF_ICTF_MENU
C
      IMPLICIT NONE
C  ----------------------------------------------------------------------------
C
      print *, 'MENU: APPLY INVERSE CTF'
      print *, '-----------------------------------------------'
      print *, 'A  Set spherical ABERRATION coefficient value'
      print *, 'D  Set DEFOCUS value'
      print *, 'G  GO and apply function to SF'
      print *, 'R  Set RESOLUTION limit'
      print *, 'M  Set MAGNIFICATION or PIXEL SIZE'
      print *, 'V  Set acceleration VOLTAGE value'
      print *, 'Z  Set phase/amplitude fractional contributions'
      print *, 'E  Return EMSF_FIX subroutine'
      WRITE(*,'(A)',advance='no') 'Choice: '
C
      RETURN
      END

C  ********************************************************************
C  *                                                                  *
C  *            *****     SUBROUTINE EMSF_ORIGINS     *****           *
C  *                                                                  *
C  *    REFINE 2-,3-,4-, OR 6-FOLD PHASE ORIGINS.                     *
C  *                                                                  *
C  *    A  REFINE TWO-FOLD PHASE ORIGIN.                              *
C  *    B  REFINE THREE-FOLD PHASE ORIGIN.                            *
C  *    C  REFINE FOUR-FOLD PHASE ORIGIN.                             *
C  *    D  REFINE SIX-FOLD PHASE ORIGIN.                              *
C  *                                                                  *
C  *    MARCH 1992  PURDUE UNIVERSITY VAX 8550, T. BAKER.             *
C  *    LAST MODIFIED 12-MAR-1992 BY T. BAKER.                        *
C  ********************************************************************
C
      SUBROUTINE EMSF_ORIGINS (EM_NDIM, SF_TYPE, NSF, HKLS, AMPS, 
     1 PHIS, SF_SUMFSQ, T_MATRIX)
C                 
      IMPLICIT NONE
C
      INTEGER*4   NSF, TYPE, SF_TYPE, EM_NDIM
      INTEGER*4   T_MATRIX(2,3)
      INTEGER*4   HKLS(3,NSF)
      REAL*4      AMPS(NSF), PHIS(NSF), XSHFT, YSHFT, SF_SUMFSQ
      CHARACTER*1 IKEY
      
      integer    alloc_stat
      integer*4, allocatable :: tmp(:), flag(:)
      
C     --------------------------------------------------------------

      IF(EM_NDIM.NE.2) THEN
         PRINT *, '*******************************************'
         PRINT *, '* Option not available for 3D SF data set *'
         PRINT *, '*******************************************'
         CALL HITCR
         RETURN
      ENDIF
      TYPE = SF_TYPE
      IF(TYPE.EQ.0) CALL SF_ABTOAP (SF_TYPE, NSF, AMPS, PHIS)

      allocate(tmp(nsf), flag(nsf), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate TMP and FLAG'
         call tsbend
      endif

 100  PRINT *
      print *, 'OPTION F: EMSF_ORIGINS MENU'
      print *, '-------------------------------'
      print *, 'A  Refine two-fold origin'
      print *, 'B  Refine three-fold origin'
      print *, 'C  Refine four-fold origin'
      print *, 'D  Refine six-fold origin'
      print *, 'E  EXIT subroutine EMSF_ORIGINS'

 200  WRITE(*,'(A)',advance='no') 'Choice: '
      READ (*,'(A)')IKEY
      CALL STRING_UPPER (IKEY)
      IF(IKEY.EQ.'E') THEN
         deallocate(tmp, flag)
         IF(TYPE.EQ.0) CALL SF_APTOAB (SF_TYPE, NSF, AMPS, PHIS)
         RETURN
      ENDIF
C
      IF(IKEY.EQ.'A') THEN
         CALL SF_ORIGIN2 (EM_NDIM, SF_TYPE, NSF, HKLS, AMPS, PHIS, 
     1   XSHFT, YSHFT, TMP, SF_SUMFSQ, T_MATRIX)
       ELSE IF(IKEY.EQ.'B') THEN
         CALL SF_ORIGIN3 (EM_NDIM, SF_TYPE, NSF, HKLS, AMPS, PHIS, 
     1   XSHFT, YSHFT, TMP, T_MATRIX)
       ELSE IF(IKEY.EQ.'C') THEN
         CALL SF_ORIGIN4 (EM_NDIM, SF_TYPE, FLAG, NSF, HKLS, 
     1   AMPS, PHIS, XSHFT, YSHFT, TMP, T_MATRIX)
       ELSE IF(IKEY.EQ.'D') THEN
         CALL SF_ORIGIN3 (6, NSF, HKLS, AMPS, PHIS, XSHFT, YSHFT,
     1   TMP, T_MATRIX)
       ELSE
         GO TO 200
      ENDIF
      GO TO 100
C
      END

C  ********************************************************************
C  *                                                                  *
C  *             *****     SUBROUTINE EMSF_STATS     *****            *
C  *                                                                  *
C  *    HAS SEVERAL FUNCTIONS TO COMPUTE SF DATA STATISTICS.          *
C  *                                                                  *
C  *    A  COMPUTE WILSON PLOT.                                       *
C  *                                                                  *
C  *    JULY 1991  PURDUE UNIVERSITY VAX 8550, T. BAKER.              *
C  ********************************************************************
C
      SUBROUTINE EMSF_STATS (NSF, AMPS, DSTRS, SF_DSTR_MAX, 
     1 SF_SCALEF)
C                 
      IMPLICIT NONE
C
      INTEGER*4 NSF, NBINS
      REAL*4    AMPS(NSF), DSTRS(NSF), SF_SCALEF, SF_DSTR_MAX
      CHARACTER*1 IKEY
C -----------------------------------------------------------------------------
C
  100      print *, ' '
      print *, 'OPTION S: EMSF_STATS MENU'
      print *, '---------------------------------'
      print *, 'A  Compute Wilson plot of SF data'
      print *, 'E  EXIT subroutine EMSF_STATS'
  200      WRITE(*,'(A)',advance='no') 'Choice: '
      READ (*,'(A)')IKEY
C
      CALL STRING_UPPER (IKEY)
      IF(IKEY.NE.'E') THEN
         IF(IKEY.EQ.'A') THEN
 300        WRITE(6,'(A)') '$NBINS (I) <CR>=10: '
            READ (5,*,ERR=300) NBINS
            IF(NBINS.LE.0) NBINS = 10
            CALL SF_STATS (NSF, AMPS, DSTRS, NBINS, SF_DSTR_MAX, 
     1      SF_SCALEF)
            CALL HITCR
          ELSE
            print *, '**********************************'
            print *, '* NOT A VALID OPTION - TRY AGAIN *'
            print *, '**********************************'
            GO TO 200      
         ENDIF
       ELSE
         RETURN
      ENDIF
      GO TO 100
C
      END

C  ********************************************************************
C  *                                                                  *
C  *              *****     SUBROUTINE EMSF_SYM     *****             *
C  *                                                                  *
C  *    IMPOSE PLANE GROUP SYMMETRIES.                                *
C  *                                                                  *
C  *    NOTE: ONLY SOME OF THE 16 NON-p1 GROUPS AVAILABLE NOW.        *
C  *                                                                  *
C  *    MARCH 1992  PURDUE UNIVERSITY VAX 8550, T. BAKER.             *
C  *    LAST MODIFIED 19-NOV-1997 BY T. BAKER.                        *
C  ********************************************************************
C
      SUBROUTINE EMSF_SYM (EM_NDIM, SF_TYPE, NLIM, NSF, HKLS, AMPS, 
     1 PHIS, SIGS, DSTRS, GAM_STAR, T_MATRIX)
C                 
      IMPLICIT NONE
C
      INTEGER*4   NLIM, NSF, TYPE, NREJ, SF_TYPE, EM_NDIM
      INTEGER*4   T_MATRIX(2,3)
      INTEGER*4   HKLS(3,NLIM)
      REAL*4      AMPS(NLIM), PHIS(NLIM), SIGS(NLIM), DSTRS(NLIM)
      REAL*4      GAM_STAR
      CHARACTER*1 IKEY

      integer    alloc_stat
      integer*4, allocatable :: flag(:)

C -----------------------------------------------------------------------------
C
      IF(EM_NDIM.NE.2) THEN
        PRINT *, '*******************************************'
        PRINT *, '* Option not available for 3D SF data set *'
        PRINT *, '*******************************************'
        CALL HITCR
        RETURN
      ENDIF
      TYPE = SF_TYPE
CCC      IF(TYPE.EQ.0) CALL SF_ABTOAP (SF_TYPE, NSF, AMPS, PHIS)

      allocate(flag(nlim), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate FLAG'
         call tsbend
      endif

 100  print *, ' '
      print *, 'OPTION I: EMSF_SYM MENU'
      print *, '-----------------------------'
      print *, 'A  p2     H  pgg     N (p3m1)'
      print *, 'B (pm)    I  cmm     O (p31m)'
      print *, 'C (pg)    J  p4      P  p6'
      print *, 'D (cm)    K  p4mm    Q  p6m'
      print *, 'F (pmm)   L  p4g'
      print *, 'G (pmg)   M  (p3)'
      print *, 'E  EXIT subroutine EMSF_SYM'
 200  WRITE(*,'(A)',advance='no') 'Choice: '
      READ (*,'(A)')IKEY
      CALL STRING_UPPER (IKEY)
      IF(IKEY.EQ.'E') THEN
         deallocate(flag)
CCC   IF(TYPE.EQ.0) CALL SF_APTOAB (SF_TYPE, NSF, AMPS, PHIS)
        RETURN
      ENDIF
      CALL SF_HKL (NLIM, HKLS, 1, T_MATRIX)
      IF(IKEY.EQ.'A') THEN
         CALL SF_SYM_P2 (SF_TYPE, NSF, AMPS, PHIS)
       ELSE IF(IKEY.EQ.'H') THEN
         CALL SF_SYM_PGG (SF_TYPE, NLIM, NSF, HKLS, AMPS, PHIS, SIGS, 
     1   DSTRS, FLAG, GAM_STAR)
       ELSE IF(IKEY.EQ.'I') THEN
         CALL SF_SYM_CMM (SF_TYPE, NLIM, NSF, HKLS, AMPS, PHIS, SIGS, 
     1   DSTRS, FLAG, NREJ)
       ELSE IF(IKEY.EQ.'J') THEN
         CALL SF_SYM_P4  (SF_TYPE, NLIM, NSF, HKLS, AMPS, PHIS, SIGS, 
     1   DSTRS, FLAG, 90., 1.)
       ELSE IF(IKEY.EQ.'K') THEN
         CALL SF_SYM_P4M (SF_TYPE, NLIM, NSF, HKLS, AMPS, PHIS, SIGS, 
     1   DSTRS, FLAG)
       ELSE IF(IKEY.EQ.'L') THEN
         CALL SF_SYM_P4G (SF_TYPE, NLIM, NSF, HKLS, AMPS, PHIS, SIGS, 
     1   DSTRS, FLAG)
       ELSE IF(IKEY.EQ.'P') THEN
         CALL SF_SYM_P6  (SF_TYPE, NLIM, NSF, HKLS, AMPS, PHIS, SIGS, 
     1   DSTRS, FLAG)
       ELSE IF(IKEY.EQ.'Q') THEN
          CALL SF_SYM_P6M (NLIM, NSF, HKLS, AMPS, PHIS, SIGS, 
     $         DSTRS, FLAG)
       ELSE
         print *, 'Options in "()" NOT available'
         CALL SF_HKL (NLIM, HKLS, -1, T_MATRIX)
         GO TO 200
      ENDIF
      CALL SF_HKL (NLIM, HKLS, -1, T_MATRIX)
      GO TO 100
C
      END

C  *********************************************************************
C  *                                                                   *
C  *              *****   SUBROUTINE CTF_CS_COEF     *****             *
C  *                                                                   *
C  *    ENTER SPHERICAL ABERRATION COEFFICIENT IN MM                   *
C  *                                                                   *
C  *    SEPTEMBER 1988  PURDUE UNIVERSITY VAX 8550, T. BAKER           *
C  *    LAST MODIFIED 25-NOV-1997 BY T. BAKER                          *
C  *********************************************************************
C
      SUBROUTINE CTF_CS_COEF (CS)
C
      IMPLICIT NONE
C
      REAL*4 CS
C  ----------------------------------------------------------------------------
C
      PRINT *
C
 100  WRITE(*,'(2A)',advance='no') 
     1     'Coefficient of spherical abberation (Cs)',
     1     ' in mm (<CR>=2.0): '
      READ (*,*,ERR=100) CS
      IF(CS.LE.0.0) CS = 2.0
C
      RETURN
      END

C  **************************************************************
C  *                                                            *
C  *            *****   SUBROUTINE CTF_DEFOCUS     *****        *
C  *                                                            *
C  *    ENTER DEFOCUS VALUE IN NM (NEGATIVE = OVERFOCUS)        *
C  *                                                            *
C  *    SEPTEMBER 1988  PURDUE UNIVERSITY VAX 8550, T. BAKER    *
C  *    LAST MODIFIED 25-NOV-1997 BY T. BAKER                   *
C  **************************************************************
C
      SUBROUTINE CTF_DEFOCUS (DEFOCUS)
C
      IMPLICIT NONE
C
      REAL*4 DEFOCUS
C  ----------------------------------------------------------------------------
C
      PRINT *
C
      PRINT *, '************************************************'
      PRINT *, '* DEFOCUS SETTING: Positive value = underfocus *'
      PRINT *, '*                  Negative value = overfocus  *'
      PRINT *, '************************************************'
 100  WRITE(*,'(A)',advance='no') 'Defocus in nm (<CR>=1000.0): '
      READ (*,*,ERR=100) DEFOCUS
      IF(DEFOCUS.EQ.0.0) DEFOCUS = 1000.0
C
      RETURN
      END

C  *************************************************************************
C  *                                                                       *
C  *                *****   SUBROUTINE CTF_VOLTAGE     *****               *
C  *                                                                       *
C  *    ENTER ACCELERATING VOLTAGE IN KILOVOLTS                            *
C  *                                                                       *
C  *    THE ELECTRON WAVELENGTH IS CALCULATED APPLYING THE RELATAVISTIC    *
C  *     CORRECTION                                                        *
C  *                                                                       *
C  *    SEPTEMBER 1988  PURDUE UNIVERSITY VAX 8550, T. BAKER               *
C  *    LAST MODIFIED 25-NOV-1997 BY T. BAKER                              *
C  *************************************************************************
C
      SUBROUTINE CTF_VOLTAGE (VOLTAGE, WAVELENGTH)
C
      IMPLICIT NONE
C
      REAL*4 VOLTAGE, VOLTAGE1, WAVELENGTH, WAVEL
C  ----------------------------------------------------------------------------
C
      PRINT *
C
 100  WRITE(*,'(A,F5.1,A)',advance='no') 
     1     'Accelerating voltage in KV (<CR>= ',
     1     VOLTAGE/1000.0, '): '
      READ (*,*,ERR=100) VOLTAGE1
      IF(VOLTAGE1.GT.0.0.AND.VOLTAGE1.NE.VOLTAGE) THEN
        VOLTAGE = VOLTAGE1*1000.0
           ENDIF
      WAVELENGTH = WAVEL(VOLTAGE) * 1000.0      ! In picometers
C
      RETURN
      END

C  *************************************************************************
C  *                                                                       *
C  *                *****   SUBROUTINE CTF_SETFRACT     *****              *
C  *                                                                       *
C  *    SET FRACTIONAL AMPLITUDE/PHASE CONTRIBUTIONS.                      *
C  *                                                                       *
C  *      DEFAULTS = PFRACT = 1.0, AFRACT = 0.0.                           *
C  *                                                                       *
C  *    FEBRUARY 1990  PURDUE UNIVERSITY VAX 8550, T. BAKER, H. CHENG.     *
C  *    LAST MODIFIED 25-NOV-1997 BY T. BAKER                              *
C  *************************************************************************
C
      SUBROUTINE CTF_SETFRACT (PFRACT, AFRACT)
C
      IMPLICIT NONE

      INTEGER*4 I, J, MAXI
      REAL*4    PFRACT, AFRACT, TEST, MAX
      real*8    deg_to_rad
C  ----------------------------------------------------------------------------
C
      deg_to_rad = acos(-1.0d0)/180.0d0
      PRINT *
C
 100  WRITE(*,'(A)',advance='no') 
     1     'Fraction of amp contrast (Range = 0.0-1.0): '
      READ (*,*,ERR=100) AFRACT
      IF(AFRACT.LE.0.0) THEN
         PFRACT = 1.0
         AFRACT = 0.0
       ELSE IF (AFRACT.GT.1.0) THEN
         PRINT *, '****************'
         PRINT *, '* Out of range *'
         PRINT *, '****************'
         GO TO 100
       ELSE
         PFRACT = 1.0      
         DO J = 1,3
           MAXI   = 0
           MAX    = -HUGE(MAX)
           DO I = 0,359
             TEST = REAL(PFRACT*SIN(deg_to_rad*REAL(I)) + 
     1             AFRACT*COS(deg_to_rad*REAL(I)))
             IF(ABS(TEST).GT.MAX) THEN
               MAX  = TEST
               MAXI = I
             ENDIF
           ENDDO
           PFRACT = REAL(((1.0-MAX)/SIN(deg_to_rad*REAL(MAXI)))+PFRACT)
         ENDDO
      ENDIF
C
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *                *****   SUBROUTINE EMSF_CTF_ZEROES     *****              *
C  *                                                                          *
C  *    COMPUTE THE POSITIONS OF THE ZEROES IN THE CTF.                       *
C  *     ALSO LIST THE FIRST MAXIMUM OF THE CTF FOR UNDERFOCUS CONDITIONS.    *
C  *                                                                          *
C  *    FROM MISELL'S BOOK ON IMAGE PROCESSING                            *
C  *     EQUATION 3.28, PAGE 51, PRACT. METH. ELEC. MICROSC. 7 (1978)         *
C  *                                                                          *
C  *    SEPTEMBER 1988  PURDUE UNIVERSITY VAX 8550, T. BAKER                  *
C  *    MODIFIED 21-JUL-91 BY H. CHENG                                        *
C  *    LAST MODIFIED 25-NOV-1997 BY T. BAKER                                 *
C  ****************************************************************************
C
      SUBROUTINE EMSF_CTF_ZEROES (WAVELENGTH, ZEROES, CTF_MAX,
     1 AFRACT, PFRACT, CS, WAVE, DEFOCUS, TWOPI)
C
      IMPLICIT NONE
C
      INTEGER*4 N
      REAL*4    WAVELENGTH,DEFOCUS,AFRACT,PFRACT,TWOPI
      REAL*4    WAVE, CS, ZEROES(0:5), CTF_MAX, XTMP, TMP1, TMP2
      REAL*4    TMP
C  ---------------------------------------------------------------------
C
      PRINT *
      PRINT *, '*********************'
      PRINT *, '* Zeroes in the CTF *'
      PRINT *, '*********************'
C
      WAVE = WAVELENGTH
      XTMP = CS*(WAVE**3)
      WRITE(*,'(/,A,/)') ' ---------------------------------------'
      WRITE(*,'(A,/)') ' TAKING ACCOUNT OF AMPLITUDE CONTRAST'
      WRITE(*,'(2A,/,A,F12.4,A,/)') ' sqrt( atan(-1.0*Afract/Pfract) +',
     1 ' wave/pi/defocus) / WAVE', ' DEFOCUS: ', DEFOCUS, 'NM'
c
      DO 100 N = 0,4
        TMP1 = DEFOCUS**2 * (WAVE**2)                ! ?? approximation 
        TMP2 = 2.0*N*CS*(WAVE**3)*1000.0
        IF(DEFOCUS.LT.0.) TMP2 = -TMP2
        IF(TMP1.GE.TMP2) THEN
           TMP1 = SQRT(TMP1-TMP2)
         ELSE
           TMP1 = SQRT(TMP1+TMP2)
        ENDIF
        TMP2 = DEFOCUS * WAVE
        IF(TMP2.GE.TMP1) THEN
           ZEROES(N) = SQRT((TMP2-TMP1)/(CS*WAVE**3))
         ELSE
           ZEROES(N) = SQRT((TMP2+TMP1)/(CS*WAVE**3))
        ENDIF
c        print *, n, ZEROES(N) 
c --------------------------------------------------
c     taking account of Amplitude contrast
c   overwite previous single-sine terms of zeroes
c --------------------------------------------------
        TMP = atan(-1.*AFRACT/PFRACT) + n*(twopi/2.) 
        IF (TMP.LT.0.) GO TO 100
        zeroes(n) = sqrt(TMP * wave/(twopi/2.)/defocus * 1000.)/wave
        WRITE(*,'(F8.4,A,I5,F8.4,F8.2,A)') AFRACT, ' Afract: ', N, 
     1  ZEROES(N), 1./ZEROES(N), ' nm'
 100      CONTINUE
C
C  *******************************************
C  *    MAXIMUM UNDERFOCUS PHASE CONTRAST    *
C  *******************************************
      IF(DEFOCUS.GE.0.) THEN
        CTF_MAX = SQRT (2.0*WAVE*DEFOCUS*0.001)
        WRITE(*,'(A,F10.3,A)') ' First maximum of CTF at 1/(', 
     1  CTF_MAX, ' nm)'
      ENDIF
C
      CALL HITCR
      RETURN
      END

C
C  *      *****   SUBROUTINE SF_MULTICTF   *****
C  *
C  *    APPLY ICTF CUTOFF, TEMPERATURE FACTOR 
C  *    SMOOTH THE CORRECTION CURVE
C  *    CREATE/PRINT 1D-AMPLITUDE PLOT FILE 
C  *
C  *    SEPTEMBER 1990  PURDUE UNIVERSITY VAX 8550, H. CHENG
C  *    MODIFIED 25-SEP-90 BY  R. H. CHENG 
C  *    LAST MODIFIED 30-JAN-92 BY  R.H. CHENG 
C
      SUBROUTINE EMSF_MULTICTF (SF_TYPE, NSF, AMPS, PHIS, DSTRS, 
     1 RES_MIN, RES_MAX, BIN1, BIN2, ICTF, CTF_MAX, 
     2 AFRACT, PFRACT, CS, WAVE, DEFOCUS, TWOPI, PIX_STEP)
C
      IMPLICIT NONE
C
      INTEGER*4 SF_TYPE, NSF, NCUT, I
      REAL*4    AMPS(NSF),PHIS(NSF),DSTRS(NSF),PIX_STEP,RES_MIN,RES_MAX
      REAL*4    AFRACT, PFRACT, CS, WAVE, RES
      REAL*4    DEFOCUS, TMP, TWOPI, RESNM
      REAL*4    BIN1(NSF),BIN2(NSF)
      REAL*4    CTF_STEP,lsprs, hsprs
      REAL*4    WEIGHT 
      REAL*4    CTF_MAX
      REAL*4    PASLOW, PASSHI, LOP_SET, HIP_SET, LPDELSQ, HPDELSQ
      real*8    deg_to_rad
      LOGICAL   ICTF, ZERO, LOWPASS, HIPASS
CCC      LOGICAL   ZEROTEMP
C  ---------------------------------------------------------------------
C
      deg_to_rad = acos(-1.0d0)/180.0d0

      ZERO = .FALSE.
      LOWPASS = .FALSE.
      HIPASS = .FALSE.
      IF(ICTF) THEN
        LSPRS    = 1.0
        HSPRS    = 1.0
        PASSHI   = 0.0
        PASLOW   = 0.0
        LOWPASS  = .FALSE. 
        HIPASS   = .FALSE.
 15     WRITE(*,1500)
 1500   FORMAT(' MAX_ICTF FACTOR LOW/HIGH UP-LIMIT (<CR>=2.25,2.25)',/,
     1       '$(Type -1.0,-1.0 for NO UP-LIMIT): ')
        READ(*,*,ERR=15) PASLOW, PASSHI
        IF(PASSHI.EQ.0.) PASSHI = 2.250
        IF(PASLOW.EQ.0.) PASLOW = 2.250
        PRINT 55, ' UP-LIMIT OF ICFT FACTOR: ', PASSHI, PASLOW
        IF(PASLOW.LT.0.) THEN
           LOWPASS = .TRUE.
           PRINT *, 'ICTF> NO UP-LIMIT of ICTF set to LOW resolution '
        ENDIF
        IF(PASSHI.LT.0.) THEN
           HIPASS = .TRUE.
           PRINT *, 'ICTF> NO UP-LIMIT of ICTF set to HIGH resolution '
        ENDIF
        LOP_SET = 0.0
        HIP_SET = 0.0
        CTF_MAX = 1.0/ CTF_MAX                              ! 1/nm
        PRINT *, 'FIRST CTF_MAX (1/nm): ', CTF_MAX
C
        IF(.NOT.HIPASS) THEN
           PRINT *                                          ! 1/pix
           CTF_STEP = (RES_MAX-RES_MIN)/1000.               
           RES      = RES_MIN                               ! 1/pix
C
 777            RES = RES + CTF_STEP                             ! 1/pix
           IF(RES.GT.RES_MAX) GOTO 778 
           RESNM = RES / PIX_STEP                           ! 1/nm
           CALL CTFCALC (RESNM, TMP, AFRACT, PFRACT, CS, WAVE,
     1     DEFOCUS, TWOPI)
           TMP = 1./ TMP                                    ! ICTF
C -- DEBUG 
CDEBUG           WRITE(*,'(''+ICTF,RES, LOP_SET,HIP_SET (1/pix): '', 
CDEBUG      1         E12.5,3F8.4)') TMP, RES, LOP_SET, HIP_SET
CDEBUG
           IF(LOWPASS) GOTO 666
           IF(TMP.LT.PASLOW .OR. RESNM.GT.CTF_MAX) GOTO 666
              LOP_SET = RES                                    ! 1/pix
 666           IF(TMP.LT.PASSHI .OR. RESNM.LT.CTF_MAX) GOTO 777
              HIP_SET = RES                                    ! 1/pix
              WRITE(*,'(2A,E12.5,3F8.4)') '+ICTF,RES, LOP_SET,HIP_SET',
     1        ' (1/pix): ', TMP, RES, LOP_SET, HIP_SET
              PRINT *
C
 778           IF(LOP_SET.EQ.0.) THEN
              LOWPASS = .TRUE.
              PRINT *,' NO ICTF FACTOR OVER THE LIMIT OUT TO RES_MIN'
            ELSE
              LPDELSQ = ((LOP_SET - RES_MIN)/PIX_STEP)**2     !sq(1/nm)
              IF(LPDELSQ.EQ.0.) THEN                     ! RESET 
                 LOWPASS = .TRUE. 
                 PRINT *, ' RESET TO LOWPASS... LOP_SET = RES_MIN'
               ELSE
                 LOP_SET = LOP_SET / PIX_STEP                    ! 1/nm
                 PRINT 55,' LOP_SET, LPDELSQ (1/nm):  ',LOP_SET,LPDELSQ
              ENDIF
              PRINT *, 'SUPRESSING FACTOR FOR LOW END (<CR>=2.0): '
              READ (*,*) LSPRS
              IF (LSPRS.LE.0.) LSPRS = 2.0
           ENDIF
           IF(HIP_SET.EQ.0.) THEN 
              HIPASS  = .TRUE. 
              PRINT *,' NO ICTF FACTOR OVER THE LIMIT W/IN RES_MAX'
            ELSE
              HPDELSQ = ((HIP_SET - RES_MAX)/PIX_STEP)**2     !sq(1/nm)
              IF(HPDELSQ.EQ.0.) THEN                          ! RESET 
                 HIPASS  = .TRUE. 
                 PRINT *, ' RESET TO HIPASS... HIP_SET = RES_MAX'
               ELSE
                 HIP_SET = HIP_SET / PIX_STEP                    ! 1/nm
                 PRINT 55,' HIP_SET, HPDELSQ (1/nm):  ',HIP_SET,HPDELSQ
              ENDIF
              PRINT *, 'SUPRESSING FACTOR FOR HIGH END (<CR>=2.0): '
              READ (*,*) HSPRS
              IF (HSPRS.LE.0.) HSPRS = 2.0
           ENDIF
           IF(LOWPASS .AND. HIPASS) THEN
              PRINT *,' No ICTF factor over limit within working range'
           ENDIF
        ENDIF 
C        CALL POLYSMTH ()
C        CALL HITCR
      ENDIF
 55      FORMAT(A,2F8.5) 
C         ---------  end of if(ICTF)/SETUP ----------
C
      DO 10000 I = 1,NSF                ! GO FOR INDIVIDUAL SF's !
        RES = DSTRS(I)
        IF(RES.GE.RES_MIN.AND.RES.LE.RES_MAX) THEN
          RES = RES / PIX_STEP                   ! PIXEL to NM 
          CALL CTFCALC (RES, TMP, AFRACT, PFRACT, CS, WAVE,
     1    DEFOCUS, TWOPI)
          BIN1(I) = TMP                           ! binned CTF factors
C
C ----- START WITH INVERSING CTF FACTORS !
C
          IF(ICTF) THEN
            IF(TMP.NE.0.) THEN
              TMP = 1.0 / TMP                     ! ICTF
CCC              IF(ZEROTEMP) TMP = TMP*EXP(-TEMPFAC*RES**2)
              IF(.NOT.LOWPASS .AND. RES.LT.LOP_SET) THEN    !1/nm
                 NCUT = NCUT + 1
CCC                  WEIGHT = EXP(-((RES-LOP_SET)**2/LPDELSQ)) 
                 WEIGHT = EXP(-(lsprs*(RES-LOP_SET)**2/LPDELSQ)) 
                 TMP = TMP * WEIGHT 
              ENDIF 
              IF(.NOT.HIPASS .AND. RES.GT.HIP_SET) THEN
                 NCUT = NCUT + 1
                 WEIGHT = EXP(-(hsprs*(RES-HIP_SET)**2/HPDELSQ)) 
                 TMP = TMP * WEIGHT 
              ENDIF 
             ELSE
              PRINT *, ' CTF> ZERO FOUND! '
              PRINT *, ' Reset(s) ICTF factor @Resolution (1/nm)', RES
              TMP = 0.
              NCUT = NCUT + 1
            ENDIF                           ! END OF CTF=0 CHECK
          ENDIF                             ! END OF ICTF FILTER 
        BIN2(I) = TMP
C
        IF(SF_TYPE.EQ.0) THEN               ! (A + iB)
          AMPS(I) = AMPS(I)*TMP
          PHIS(I) = PHIS(I)*TMP
         ELSE                               ! (PHASE + AMP)
          AMPS(I) = AMPS(I)*TMP
C d           AMPS(I) = AMPS(I)*COS(deg_to_rad*PHIS(I))*TMP
C d          PHIS(I) = AMPS(I)*SIN(deg_to_rad*PHIS(I))*TMP
        ENDIF                               ! END OF APPLYING TMP/SF's
       ENDIF                                ! END OF RESOLUTION FILTER 
C
       IF(ZERO.AND.(RES.LT.RES_MIN.OR.RES.GT.RES_MAX)) THEN
         IF(SF_TYPE.EQ.0) THEN
            AMPS(I) = 0.
            PHIS(I) = 0.
          ELSE
            AMPS(I) = 0.
         ENDIF
       ENDIF
10000      CONTINUE
C      --------  end of NSF loop   --------
C
      IF(ICTF) THEN
        IF(NCUT.LE.0) THEN
           WRITE(6,'(A)') ' ALL ICTF ARE UNDER PASSHI, NO RESET. '
         ELSE
            WRITE(6,'(I7,A,F6.2,A)') NCUT, ' SF were reset ~ ', 
     1           100.*real(NCUT)/real(NSF), '%'
        ENDIF
      ENDIF
      CALL HITCR
C
CRHC      CALL SINCHK (10, 11, ICTF, NSF, DSTRS, BIN1, BIN2)
C      CALL CTF1PS (21, DSTRS, BIN2, BIN1, NSF)
C
C  ********************
C  *    ONE-D PLOT    *
C  ********************
CRHC      PRINT *, 'CREATE/PRINT 1D-AMPLITUDE PLOT FILE Y/<N>: '
CRHC      CALL GET_YESNO (ONEDPLOT,.FALSE.)
CRHC      IF(ONEDPLOT) THEN
CRHC         CALL ONEDASK (NSF,HKLS,AMPS,PHIS,DSTRS,RES_MIN,RES_MAX,CTF_MAX)
CRHC      ENDIF
C
      RETURN
      END

C 
      SUBROUTINE CTFCALC (RES, TMP, AFRACT, PFRACT, CS, WAVE,
     1 DEFOCUS, TWOPI)
C
C      25-JAN-92   R.H. CHENG 
C
      IMPLICIT NONE
C
      REAL*4  AFRACT, PFRACT, CS, WAVE, ALPHA, RES
      REAL*4  CHI_ALPHA, TMP, TWOPI, DEFOCUS
C  ----------------------------------------------------------------------------
C
      ALPHA     = WAVE * RES 
      CHI_ALPHA = (TWOPI/WAVE)
     1          *( (DEFOCUS                   ! FLIPPED, UNDER_F AS '+'
     2          *(ALPHA**2)/2.0) - (CS*(ALPHA**4)/4.0)  )
      TMP       = PFRACT * SIN(CHI_ALPHA) + AFRACT * COS(CHI_ALPHA) 
C
      RETURN 
      END 

      SUBROUTINE SINCHK (LUN1, LUN2, ICTF, NSF, DSTRS, BIN1, BIN2)
C
      IMPLICIT  NONE
      INTEGER   LUN1, LUN2, LUN3, NSF, I
      REAL      DSTRS(NSF), BIN1(NSF), BIN2(NSF)
      CHARACTER TITLE*80
      LOGICAL   ICTF
C  --------------------------------------------------------------
      LUN3 = LUN2 + 1
      OPEN(LUN1,STATUS='NEW',FORM='unformatted',FILE='sys$tmp:ctf.tmp')
      OPEN(LUN2,STATUS='NEW',FORM='unformatted',FILE='sys$tmp:iGO.tmp')
      OPEN(LUN3,STATUS='NEW',FORM='unformatted',FILE='sys$tmp:iCTF.tmp')
      IF(ICTF) THEN
         TITLE = 'PLOT OF INVERSE CONTRAST TRANSFER FUNCTION'
       ELSE
         TITLE = 'PLOT OF CONTRAST TRANSFER FUNCTION'
      ENDIF
c      do i = 1, nsf
c         write (8,'(2f)') dstr(i), bin1(i) 
c      enddo
c      call sort2 (nsf, dstr, bin1)
c      call sort2 (nsf, dstr, bin2)
c      write(8,'(/,i,/,'' after sorting'',/)') nsf
c      do i = 1, nsf
c         write (8,'(2f)') dstr(i), bin1(i) 
c      enddo
      WRITE(LUN1) TITLE
      WRITE(LUN2) TITLE
      WRITE(LUN1) NSF/5
      WRITE(LUN2) NSF/5
      WRITE(LUN1) (DSTRS(I), I=1,NSF,5)
      WRITE(LUN2) (DSTRS(I), I=1,NSF,5)
      WRITE(LUN1) (BIN1(I), I=1,NSF,5)
      WRITE(LUN2) (BIN2(I), I=1,NSF,5)
c
      CLOSE(LUN1)
      CLOSE(LUN2)
C
      WRITE(LUN3) TITLE
      WRITE(LUN3) NSF
      WRITE(LUN3) (DSTRS(I), I=1,NSF)
      WRITE(LUN3) (1./(BIN1(I)+.0000001), I=1,NSF)
      CLOSE(LUN3)
C
CC  FOR ASCII CHECK
      DO I = 1,NSF,25
        IF(BIN1(I).NE.0.)THEN 
          WRITE (29,'(2F12.6)') DSTRS(I), BIN1(I)
          WRITE (39,'(2F12.6)') DSTRS(I), 1./BIN1(I)
C        ELSE
C          WRITE (39,'(2F12.5)') DSTRS(I), 1./(BIN1(I)+.000001)
        ENDIF
      ENDDO
      DO I = 1,NSF,12
        IF(BIN1(I).NE.0.) WRITE (40,'(2F12.6)') DSTRS(I), 1./BIN1(I)
      ENDDO
      DO I = 1,NSF,6
        IF(BIN1(I).NE.0.) WRITE (41,'(2F12.6)') DSTRS(I), 1./BIN1(I)
      ENDDO
C
      RETURN
      END
