C  ************************************************************************
C  *                                                                      *
C  *                   *****     EMSF3DBT.FOR     *****                   *
C  *                                                                      *
C  *    UNIVERSITY OF CANTERBURY FOURIER PROGRAM                          *
C  *                                                                      *
C  *    THIS IS A MODIFIED VERSION OF FOUR1.F4 FROM CYNDY STAUFFACHER     *
C  *     WHICH RAN ON THE BRANDEIS DEC10 CAMPUS COMPUTER WHICH WAS        *
C  *     REWRITTEN FROM FORDAP (A.ZALKIN)                                 *
C  *                                                                      *
C  *    THE OUTPUT OF THIS PROGRAM IS A 3D MAP FILE IN THE I*2 FORMAT     *
C  *     COMPATIBLE FOR "EMMAP" TYPE PROGRAMS.                            *
C  *                                                                      *
C  *    UNITS: 2 = SF INPUT DATA FILE                                     *
C  *           3 = 3D I*2 MAP OUTPUT FILE                                 *
C  *                                                                      *
C  *    APRIL 1981  BRANDEIS UNIVERSITY PDP 11/40, T. BAKER               *
C  *    SEPTEMBER 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER            *
C  *    APRIL 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER                  *
C  *                                                                      *
C  *    MODIFIED  5-AUG-1998 BY TSB (Compute MAP in core memory).         *
C  *                                (Output PIF format MAP).              *
C  *    MODIFIED 12-AUG-1998 BY TSB (Fast input of SF data).              *
C  *    MODIFIED  3-NOV-1998 BY TSB/RWA (PIF format SF data).             *
C  *    MODIFIED 23-DEC-1998 BY TSB/RWA (Scale MAP by 2.0).               *
C  *    LAST MODIFIED 21-JUL-1999 BY TSB/RWA (Fix SCALE1 problem).        *
C  ************************************************************************
C
      IMPLICIT NONE
C
      INTEGER*4 NEQUIV, NSF, DIV, NCOL, NROW, NSEC, NCD, NEF
      COMMON /CONST/ TRGTBL, XMIN, YMIN, ZMIN, XMAX, YMAX, ZMAX,
     1 DX, DY, DZ, DCELL, ITRANS, HMAX, KMAX, KMIN, RCELL, SCALE,
     2 TS, U, UINV
      INTEGER*4 SF_TYPE, T_MATRIX(2,3), nsf_in
      INTEGER*4 EM_NDIM, EM_ERR, LUN, ITRANS, HMAX, KMAX, KMIN
      INTEGER*4 NORIEN, TABLE_SIZE, NREF_DUMP, NUM_UNIQUE
      REAL*4    XMIN, YMIN, ZMIN, XMAX, YMAX, ZMAX, DX, DY, DZ
      REAL*4    DCELL(9), RCELL(9), SCALE, TS(3,24), U(3,3)
      REAL*4    UINV(3,3), TRGTBL(1001), DSTR_MAX, SF_SUMFSQ
      REAL*4    FS(3,3,24), DETU, AA(3,3), BB(3,3)
      REAL*4    A_STAR, B_STAR, C_STAR, ALP_STAR, BET_STAR, GAM_STAR
      REAL*4    A_CELL, B_CELL, C_CELL, ALP, BET, GAM, VOL, VOL_STAR
      REAL*4    MAP_MIN, MAP_MAX, SF_SCALEF, SF_DSTR_MIN, SF_DSTR_MAX
      CHARACTER MAP_FILENAME*256, SF_FILENAME*256
      character(len=1) MAP_TITLE(80)
      integer   alloc_stat

      integer*4, allocatable :: hkl(:), hh(:), kk(:), ll(:)
      real*4, allocatable :: amp(:), phi(:), sig(:), dstr(:)
      real*4, allocatable :: ap(:), bp(:), map(:), table1(:), table2(:)
      real*4, allocatable :: sinhx(:), coshx(:), sinky(:), cosky(:)

      integer*4 pixSizUnits, pixelSize

C  ----------------------------------------------------------------------------
C
      PRINT *, 'EMSF3DBT: Back transform 3D SFs to produce 3D MAP'
Ca      PRINT *, '          21-Jul-99 [TSB/RWA]'
      call banner('EMSF3DBT\0')
      PRINT *
C  *************************
C  *    OPEN 3D SF FILE    *
C  *************************
      LUN    = 2
      EM_ERR = 1
      DO WHILE(EM_ERR.NE.0)
        SF_FILENAME = ' '
        CALL sf_open (LUN, NSF, SF_TYPE,
     1  SF_FILENAME, A_STAR, B_STAR, C_STAR, ALP_STAR, BET_STAR, 
     2  GAM_STAR, 
     3  SF_SCALEF, A_CELL, B_CELL, C_CELL, ALP, BET, GAM, 
     4  EM_NDIM, EM_ERR, DSTR_MAX, t_matrix, 
     5  pixelSize, pixSizUnits)
      ENDDO

      allocate(hkl(3*nsf), amp(nsf), phi(nsf), sig(nsf),
     $     dstr(nsf), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate HKL, AMP, PHI, SIG, DSTR'
         call tsbend
      endif

Ca      CALL SF_FILL2 (FILE_ID, EM_ERR, SF_TYPE, LUN, NSF, HKL, AMP, PHI, 
Ca      1 SIG, DSTR, SF_DSTR_MIN, SF_DSTR_MAX, SF_SCALEF, SF_SUMFSQ,
Ca      2 DSTR_MAX)
Ca      CALL sf_fill2_pif (FILE_ID, EM_ERR, SF_TYPE, LUN, NSF, HKL, AMP, 
Ca      1 PHI, SIG, DSTR, SF_DSTR_MIN, SF_DSTR_MAX, SF_SCALEF, SF_SUMFSQ,
Ca      2 DSTR_MAX)
      CALL sf_fill (EM_ERR, SF_TYPE, NSF, HKL, AMP, 
     1 PHI, SIG, DSTR, SF_DSTR_MIN, SF_DSTR_MAX, SF_SCALEF, SF_SUMFSQ,
     2 DSTR_MAX, nsf_in)

      deallocate(sig, dstr)

      CLOSE (UNIT=2, STATUS='KEEP')
C
      IF(EM_NDIM.NE.3) THEN                        ! TRAP FOR 2D SF DATA
        PRINT *, '************************************'
        PRINT *, '* SF file does not contain 3D data *'
        PRINT *, '*           EXIT program           *'
        CALL TSBEND
      ENDIF
C  *************************************
C  *    ENTER NAME OF FINAL MAP FILE   *
C  *************************************
      WRITE(*,'(A)') '$Filename for output 3D MAP: '
      READ (*,'(A)') MAP_FILENAME
      WRITE(*,'(A)') '$Title: '
      READ (*,'(80A1)') MAP_TITLE
C
      CALL SF3DBT_INFO (NEQUIV, FS, AA, BB, A_CELL, B_CELL, C_CELL,
     1 ALP, BET, GAM, VOL, VOL_STAR, NREF_DUMP)
      IF(ITRANS.GE.1) CALL SF3DBT_TRANS (DETU, AA)
C  **************************************************************************
C  *     SET FOURIER LIMITS AND WRITE DATA FILE CONTAINING EXPANDED DATA    *
C  **************************************************************************

      allocate(hh(nsf*nequiv), kk(nsf*nequiv), ll(nsf*nequiv),
     $     ap(nsf*nequiv), bp(nsf*nequiv), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate HH, KK, LL, AP, BP'
         call tsbend
      endif

      CALL SF3DBT_RIP (NCOL, NROW, NSEC, NEQUIV, NORIEN, NCD, NEF, FS, 
     1 HKL, AMP, PHI, NSF, AP, BP, HH, KK, LL, SF_TYPE, NREF_DUMP,
     2 NUM_UNIQUE)
C

      deallocate(hkl, amp, phi)

      allocate(map(ncol*nrow*nsec), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate MAP'
         call tsbend
      endif

      CLOSE (UNIT=1, STATUS='KEEP')

      allocate(table2(nef), sinhx(ncol*(hmax+1)), coshx(ncol*(hmax+1)),
     $     sinky(nrow*(kmax-kmin+1)), cosky(nrow*(kmax-kmin+1)),
     $     stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate TABLE2 ...'
         call tsbend
      endif

      DIV        = 1
      TABLE_SIZE = NSEC*NCD/DIV

      allocate(table1(table_size), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate TABLE1'
         call tsbend
      endif

C  ***********************************************
C  *    CALCULATE FOURIER, SECTION BY SECTION    *
C  ***********************************************
      CALL SF3DBT_FOURIER (MAP, TABLE_SIZE, NCD, NEF, TABLE1, TABLE2, 
     1 NCOL, NROW, NSEC, AP, BP, HH, KK, LL, SINHX, COSHX, SINKY, 
     2 COSKY, MAP_MIN, MAP_MAX, NUM_UNIQUE, NEQUIV)

      deallocate(SINHX, COSHX, SINKY, COSKY, TABLE1, TABLE2)

      CALL SF3DBT_OUT (MAP, NORIEN, NCOL, NROW, NSEC, A_CELL, 
     1 B_CELL, C_CELL, ALP, BET, GAM, T_MATRIX, MAP_MIN, MAP_MAX, 
     2 MAP_FILENAME, MAP_TITLE, 1)

      deallocate(map)

      CALL TSBEND
      END

C  **************************************************************************
C  *                                                                        *
C  *                *****     SUBROUTINE SF3DBT_RIP     *****               *
C  *                                                                        *
C  *    ROUTINE CALLED BY "EMSF3DBT" PROGRAM                                *
C  *                                                                        *
C  *    LAST MODIFIED  5-AUG-1998 BY TSB (Eliminate need for EMSF3DBT.TMP   *
C  *     scratch file for storing intermediate SF data).                    *
C  **************************************************************************
C
      SUBROUTINE SF3DBT_RIP (NCOL, NROW, NSEC, NEQUIV, NORIEN, NCD,
     1 NEF, FS, HKL, AMP, PHI, NSF, AP, BP, HH, KK, LL, SF_TYPE,
     2 NREF_DUMP, NUM_UNIQUE)
C
      IMPLICIT NONE
C
      INTEGER*4 NEQUIV, NSF
      COMMON /CONST/ TRIG_TABLE, XMIN, YMIN, ZMIN, XMAX, YMAX, ZMAX,
     1 DX, DY, DZ, DCELL, ITRANS, HMAX, KMAX, KMIN, RCELL, SCALE,
     2 TS, U, UINV
      REAL*4    XMIN, YMIN, ZMIN, XMAX, YMAX, ZMAX, DX, DY, DZ
      REAL*4    DCELL(9), RCELL(9), SCALE, TS(3,24), U(3,3)
      REAL*4    UINV(3,3), FS(3,3,24)
      REAL*4    TRIG_TABLE(1001), XSIN(16)
      REAL*4    AMP(NSF), PHI(NSF), HKLTMP(3)
      REAL*4    AP(NSF*NEQUIV), BP(NSF*NEQUIV)
      REAL*4    FJX, FKX, FLX
      REAL*4    APART, BPART, APARTS(24), BPARTS(24)
      real*8    deg_to_rad
      INTEGER*4 M1(24,3), M2(24,3), M3(24,3), SF_TYPE, NUM_UNIQUE
      INTEGER*4 NEXPAN, NTOTAL, NU, I, M, N
      INTEGER*4 BUFFER, IFIXR, NUCALC, MTX(24), MTY(24), MTZ(24)
      INTEGER*4 NCOL, NROW, NSEC, ITRANS, HMAX, KMAX, KMIN
      INTEGER*4 NN, NREF_DUMP, NORIEN, NCD, NEF
      INTEGER*4 HH(NSF*NEQUIV), KK(NSF*NEQUIV), LL(NSF*NEQUIV)
      INTEGER*4 J(24), K(24), L(24), HKL(3,NSF)
      DATA XSIN/0.,.5,.866025,1.,.866025,.5,0.,-.5,-.866025,-1.,
     1  -.866025,-.5,0.,.5,.866025,1./

      character(len=1) XL, YL, ZL
      character(len=2) AXIS(3)
      DATA AXIS/' Z',' Y',' X'/,   XL/'X'/,YL/'Y'/,ZL/'Z'/
C  ----------------------------------------------------------------------------
C
      deg_to_rad = acos(-1.0d0)/180.0d0

      NEXPAN = 0
      NTOTAL = 0
      BUFFER = 0
      HMAX   = 0
      KMAX   = 0
      KMIN   = 0
      IF(ITRANS.LE.1) THEN
         PRINT *, '****************************************************'
         PRINT *, '* INPUT PARAMETER DEFAULTS:                        *'
         PRINT *, '* Portion of cell desired XMIN,XMAX = 0.0,1.0      *'
         PRINT *, '*                         YMIN,YMAX = 0.0,1.0      *'
         PRINT *, '*                         ZMIN,ZMAX = 0.0,1.0      *'
         PRINT *, '* Pixel sizes              DX,DY,DZ = 1.0,1.0,1.0  *'
         PRINT *, '*  If no values entered for DY,DZ, default to DX   *'
         PRINT *, '* Orientation                NORIEN =   1 (Z-secs) *'
         PRINT *, '*  (Default is Z-sections)   NORIEN =  10 (Y-secs) *'
         PRINT *, '*                            NORIEN = 100 (X-secs) *'
         PRINT *, '****************************************************'
         WRITE(*,'(A)') '$XMIN,XMAX,YMIN,YMAX,ZMIN,ZMAX: '
         READ (*,*) XMIN, XMAX, YMIN, YMAX, ZMIN, ZMAX
         IF(XMAX.EQ.0.0) XMAX = 1.0
         IF(YMAX.EQ.0.0) YMAX = 1.0
         IF(ZMAX.EQ.0.0) ZMAX = 1.0
         WRITE(*,'(A)') '$DX,DY,DZ (<CR>=1.0,1.0,1.0): '
         READ (*,*) DX,   DY,   DZ
         IF(DX.LE.0.0) DX = 1.0
         IF(DY.LE.0.0) DY = DX
         IF(DZ.LE.0.0) DZ = DX
         WRITE(*,'(A)') '$NORIEN (<CR>=1): '
         READ (*,*) NORIEN
         IF(NORIEN.LE.0 .OR. NORIEN.GT.100) NORIEN = 1
         DX = DX/DCELL(1)
         DY = DY/DCELL(2)
         DZ = DZ/DCELL(3)
      ELSE
         NORIEN = 100
      ENDIF
      CALL SF3DBT_CHEKUP (XMIN, XMAX, NORIEN, 100, DX)
      CALL SF3DBT_CHEKUP (YMIN, YMAX, NORIEN,  10, DY)
      CALL SF3DBT_CHEKUP (ZMIN, ZMAX, NORIEN,   1, DZ)
      FJX = DX*DCELL(1)
      FKX = DY*DCELL(2)
      FLX = DZ*DCELL(3)
      I   = INT(ALOG10(REAL(NORIEN)+.1)+1.)
      WRITE(*,1010) AXIS(I)
      WRITE(*,1020) XL, XMIN, XMAX, DX, FJX, YL, YMIN, YMAX, DY, FKX,
     1  ZL, ZMIN, ZMAX, DZ, FLX
      IF(NORIEN.LT.10) THEN                         ! Z SECTIONS
C         CALL SF3DBT_ROT (XMIN,YMIN)                  ! SWITCH X & Y
C         CALL SF3DBT_ROT (XMAX,YMAX)
C         CALL SF3DBT_ROT (DX,DY)
       ELSE IF(NORIEN.EQ.10) THEN                  ! Y SECTIONS
         CALL SF3DBT_ROT (YMIN,ZMIN)
         CALL SF3DBT_ROT (YMAX,ZMAX)
         CALL SF3DBT_ROT (DY,DZ)
       ELSE IF(NORIEN.GT.10) THEN                  ! X SECTIONS
         CALL SF3DBT_ROT (XMIN,ZMIN)
         CALL SF3DBT_ROT (XMAX,ZMAX)
         CALL SF3DBT_ROT (DX,DZ)
      ENDIF
      DO I = 1,NEQUIV
        DO M = 1,3
          M1(I,M) = INT(FS(1,M,I))
          M2(I,M) = INT(FS(2,M,I))
          M3(I,M) = INT(FS(3,M,I))
        ENDDO
        IF(M1(I,1)+M2(I,1).EQ.0) THEN
          IF(M1(I,1))         290,320,290
        ENDIF
        IF(M1(I,2)+M2(I,2)) 320,280,320
  280        IF(M1(I,2))         290,320,290
  290        IF(M1(I,1)+M2(I,2)) 310,300,310
  300        CALL SF3DBT_ROT_I4 (M1(I,2),M2(I,1))
        GO TO 320
  310        CALL SF3DBT_ROT_I4 (M1(I,1),M2(I,2))
  320        MTX(I) = NUCALC(IFIXR(12.*TS(1,I)))
        MTY(I) = NUCALC(IFIXR(12.*TS(2,I)))
        MTZ(I) = NUCALC(IFIXR(12.*TS(3,I)))
      ENDDO
      NCOL = INT((XMAX-XMIN)/DX+1.5)
      NROW = INT((YMAX-YMIN)/DY+1.5)
      NSEC = INT((ZMAX-ZMIN)/DZ+1.5)
      WRITE(*,'(3(A,I6))') ' MAP dimensions:', NCOL, ' by', NROW, 
     1 ' by', NSEC
      IF(NREF_DUMP.GT.0) WRITE(6,1040)
      IF(ITRANS.GE.1) WRITE(6,1050)
C  *****************************************************************
C  *    CHANGES MADE TO READ HKL, AMP AND PHASE AND CONVERT TO     *
C  *      APARTS AND BPARTS   3-SEP-84  TSB                        *
C  *****************************************************************
      NN = 0
  400      NN = NN+1
      IF(NN.GT.NSF) GO TO 900
      IF(SF_TYPE.EQ.1) THEN
         APART = REAL(AMP(NN)*COS(deg_to_rad*PHI(NN)))
         BPART = REAL(AMP(NN)*SIN(deg_to_rad*PHI(NN)))
       ELSE
         APART = AMP(NN)
         BPART = PHI(NN)
      ENDIF
      IF(APART.EQ.0.0.AND.BPART.EQ.0.0) GO TO 400
      HKLTMP(1) = HKL(1,NN)
      HKLTMP(2) = HKL(2,NN)
      HKLTMP(3) = HKL(3,NN)
      NTOTAL    = NTOTAL + 1
C
      DO N = 1,NEQUIV
        J(N) = M1(N,1)*HKL(1,NN)+M2(N,1)*HKL(2,NN)+M3(N,1)*HKL(3,NN)
        K(N) = M1(N,2)*HKL(1,NN)+M2(N,2)*HKL(2,NN)+M3(N,2)*HKL(3,NN)
        L(N) = M1(N,3)*HKL(1,NN)+M2(N,3)*HKL(2,NN)+M3(N,3)*HKL(3,NN)
        NU   = NUCALC(J(N)*MTX(N)+K(N)*MTY(N)+L(N)*MTZ(N)) + 1
        APARTS(N) = APART*XSIN(NU + 3) - BPART*XSIN(NU)
        BPARTS(N) = BPART*XSIN(NU + 3) + APART*XSIN(NU)
        IF(ITRANS.GE.1) THEN
          FJX  = J(N)
          FKX  = K(N)
          FLX  = L(N)
          J(N) = IFIXR( U(1,1)*FJX + U(1,2)*FKX + U(1,3)*FLX )
          K(N) = IFIXR( U(2,1)*FJX + U(2,2)*FKX + U(2,3)*FLX )
          L(N) = IFIXR( U(3,1)*FJX + U(3,2)*FKX + U(3,3)*FLX )
        ENDIF
        IF(NORIEN.LT.10) THEN                   ! Z SECTIONS
C           CALL SF3DBT_ROT_I2 (J(N),K(N))
         ELSE IF(NORIEN.EQ.10) THEN                  ! Y SECTIONS
           CALL SF3DBT_ROT_I2 (K(N),L(N))
         ELSE IF(NORIEN.GT.10) THEN                  ! X SECTIONS
           CALL SF3DBT_ROT_I2 (J(N),L(N))
        ENDIF
        IF(J(N))680,650,710
  650        IF(K(N))690,660,710
  660        IF(L(N))700,670,710
  670        APARTS(N) = .5*APARTS(N)
        BPARTS(N) = .5*BPARTS(N)
        GO TO 710
  680        J(N) = -J(N)
  690        K(N) = -K(N)
  700        L(N) = -L(N)
        BPARTS(N) = -BPARTS(N)
  710        IF(APARTS(N).NE.0.0.OR.BPARTS(N).NE.0.0) THEN
          IF(J(N).GT.HMAX) HMAX = J(N)
          IF(K(N).GT.KMAX) KMAX = K(N)
          IF(K(N).LT.KMIN) KMIN = K(N)
        ENDIF
      ENDDO
      DO 860 I = 1,NEQUIV
        IF(APARTS(I).EQ.0.0.AND.BPARTS(I).EQ.0.0) GO TO 860
        M = I+1
        IF(M.LE.NEQUIV) THEN
          DO N = M,NEQUIV
           IF(APARTS(N).EQ.0.0) THEN
            IF(BPARTS(N).NE.0.0) THEN
             IF(J(I).EQ.J(N).AND.K(I).EQ.K(N).AND.L(I).EQ.L(N)) THEN
              APARTS(N) = 0.0
              BPARTS(N) = 0.0
             ENDIF
            ENDIF
           ENDIF
          ENDDO
        ENDIF
        NEXPAN     = NEXPAN + 1
        BUFFER     = BUFFER + 1
        HH(BUFFER) = J(I)
        KK(BUFFER) = K(I)
        LL(BUFFER) = L(I)
        AP(BUFFER) = APARTS(I)
        BP(BUFFER) = BPARTS(I)
        IF(NREF_DUMP.GT.0) THEN
          IF(NORIEN.LT.10) THEN
C             WRITE(*,2020) K(I),J(I),L(I),APARTS(I),BPARTS(I)
             WRITE(*,2020) J(I),K(I),L(I),APARTS(I),BPARTS(I)
           ELSE IF(NORIEN.EQ.10) THEN
             WRITE(*,2020) J(I),L(I),K(I),APARTS(I),BPARTS(I)
           ELSE IF(NORIEN.GT.10) THEN
             WRITE(*,2020) L(I),K(I),J(I),APARTS(I),BPARTS(I)
          ENDIF
        ENDIF
  860      CONTINUE
      IF(NREF_DUMP.GT.0) NREF_DUMP = NREF_DUMP - 1
      GO TO 400
C
  900      NCD = (HMAX+1)*(KMAX-KMIN+1)*2
      NEF = (HMAX+1)*NROW*2
      IF(NTOTAL.LE.0.OR.NEXPAN.LE.0) THEN
        PRINT *, '***********************************************'
        PRINT *, '* No data available for this summation...EXIT *'
        PRINT *, '* NTOTAL,NEXPAN =', NTOTAL, NEXPAN
        CALL TSBEND
      ENDIF
      WRITE(*,1060) NTOTAL, NEXPAN
      WRITE(*,1070) NCD, NEF
      NUM_UNIQUE = NEXPAN
      RETURN
C
 1010      FORMAT(' Compute sections along the',A2,' axis')
 1020      FORMAT(' ',A2,'Limits: ',F8.5,' to',F8.5,' in steps of',
     1  F8.5,' (',F6.3,' nms)')
 1040      FORMAT(' ',2X,'H',3X,'K',3X,'L',11X,'A',10X,'B'/)
 1050      FORMAT(20X,'(The indices are transformed to the new cell)')
 1060      FORMAT(' The',I8,' independent reflections have been',
     1  ' expanded to',I8,' terms')
 1070      FORMAT(' The CD table requires',I6,' memory locations'/,
     1  ' The EF table requires',I6,' memory locations'/)
 2020      FORMAT(3I4,2F10.2)
      END
