// FIXPIF - application for editing PIF header files
//
//    Rob Ashmore
//    Biology Department
//    Purdue University
//    Sept 1998

#define VERSION "2.00_20021127"

#include "../include/common.h"
#include "fixpif.h"
#include "../include/libUtil.h"
#include <math.h>

char rwa_translation_table[] = 
  "<Key>Return : imgNumActivate()";

XtActionsRec rwa_action_table[] = 
  {
    {"imgNumActivate",     (XtActionProc)imgNumActivate},
  };


// ---------------------------------------------------------------------
// OS transfer point.  The main routine does all the one-time setup and
// then calls XtAppMainLoop
// ---------------------------------------------------------------------

int main(int argc, char **argv)
{
  String fallbacks[] =
    { "*fontList:-dec-terminal-bold-r-normal--14-140-75-75-c-80-iso8859-1=tag",
      NULL
    };
  
  banner("fixpif");

  // Init
  currentHeader = 0;
  numImages     = 1;
  fileIsOpen    = FALSE;
  dirty         = FALSE;
  
  glbFields[0]  = "Num Images";
  glbFields[1]  = "ScaleF";
  glbFields[2]  = "Recip ScaleF";
  glbFields[3]  = "Htype";
  glbFields[4]  = "Cols";
  glbFields[5]  = "Rows";
  glbFields[6]  = "Secs";
  glbFields[7]  = "Mode";
  glbFields[8]  = "EndianNess";
  glbFields[9]  = "Title";
  glbFields[10] = "Even";
  glbFields[11] = "mrcX";
  glbFields[12] = "mrcY";
  glbFields[13] = "mrcZ";
  glbFields[14] = "int4scaleF";
  
  imgFields[0]  = "Image Num";
  imgFields[1]  = "pifMode";
  imgFields[2]  = "Background";
  imgFields[3]  = "i4Min";
  imgFields[4]  = "i4Max";
  imgFields[5]  = "r4Min";
  imgFields[6]  = "r4Max";
  imgFields[7]  = "Avg";
  imgFields[8]  = "Std";
  imgFields[9]  = "Cols";
  imgFields[10] = "Rows";
  imgFields[11] = "Secs";
  imgFields[12] = "Description";
  imgFields[13] = "aoverb";
  imgFields[14] = "map_abang";
  imgFields[15] = "a_cell";
  imgFields[16] = "b_cell";
  imgFields[17] = "c_cell";
  imgFields[18] = "alp";
  imgFields[19] = "bet";
  imgFields[20] = "gam";
  imgFields[21] = "dela";
  imgFields[22] = "delb";
  imgFields[23] = "delc";
  imgFields[24] = "a_star";
  imgFields[25] = "b_star";
  imgFields[26] = "c_star";
  imgFields[27] = "alp_star";
  imgFields[28] = "bet_star";
  imgFields[29] = "gam_star";
  imgFields[30] = "pixelSize";
  imgFields[31] = "t_matrix[0]";
  imgFields[32] = "t_matrix[1]";
  imgFields[33] = "t_matrix[2]";
  imgFields[34] = "t_matrix[3]";
  imgFields[35] = "t_matrix[4]";
  imgFields[36] = "t_matrix[5]";
  imgFields[37] = "PackRadius";
  imgFields[38] = "binFactor";
  imgFields[39] = "dthe";
  imgFields[40] = "dphi_90";
  imgFields[41] = "symmetry";
  imgFields[42] = "pixSizUnits";
  imgFields[43] = "res_out";
  imgFields[44] = "ctfMode";
  imgFields[45] = "tempFactor";
  imgFields[46] = "wiener";
  imgFields[47] = "borderType";
  imgFields[48] = "borderWidth";
  imgFields[49] = "rad_lo";
  imgFields[50] = "rad_hi";
  imgFields[51] = "transX";
  imgFields[52] = "transY";
  imgFields[53] = "transZ";
  imgFields[54] = "isTrunc";
  imgFields[55] = "origX";
  imgFields[56] = "origY";
  imgFields[57] = "origZ";
  imgFields[58] = "pifVersion";
 
  fileName.addr = (char *)calloc(2048,sizeof(char));
  fileName.length = 2048;
  
  XtToolkitInitialize();
  app_context = XtCreateApplicationContext();
  
  XtAppSetFallbackResources(app_context, fallbacks);
  
  // Open the display
  dsply = XtOpenDisplay(app_context, NULL, "fixpif",
			"fixpif", NULL, 0, &argc, argv);
  
  // Add our remapping of keys to widgets
  XtAppAddActions(app_context,rwa_action_table, 1); 
  
  // Explicitly call this class "fixpif"

  topLevelW = XtVaAppCreateShell(argv[0], "fixpif",
				       topLevelShellWidgetClass, 
				       dsply, NULL);
  
  create_windows();
  text_create(headNumTextW, NULL, NULL);
  text_create(globalFormW, NULL, NULL);
  text_create(imageFormW, NULL, NULL);

  XtUnmanageChild(XmMessageBoxGetChild(infoBoxW,XmDIALOG_HELP_BUTTON));

  // Manage the main part and realize everything. The interface comes
  // up on the display now. Manage all of the screens now so we can
  // alter what we need to in XLibInitialize() - Unmanage the unneeded
  // ones thereafter.

  XtManageChild(mainWindowW);
  XFlush(dsply);
  XtUnmanageChild(imageFormW);

  XlibInitialize();

  // Sit around forever waiting to process X-events. We never leave
  // XtAppMainLoop. From here on, we only execute our callback routines.
  XtAppMainLoop(app_context);

  return(0);

}



void XlibInitialize()
{
  // function name: XlibInitialize
  // Initialize for Xlib calls.

  // Enable save unders and backing store
  if(XDoesSaveUnders(XScreenOfDisplay(dsply, DefaultScreen(dsply))) &&
     XDoesBackingStore(XScreenOfDisplay(dsply, DefaultScreen(dsply)))) {
    attrs.save_under = TRUE;
    attrs.backing_store = WhenMapped;
    printf("Enabling 'Backing Store' and 'SaveUnder' features\n");
  }

  XFlush (dsply);
  
} // end function XlibInitialize



void s_error(char *problem_string)
{
  // function name: s_error
  // All errors are fatal.

  printf("%s\n", problem_string);
  exit(0);

} // end function s_error



void raiseIt(Widget w)
{
  // function name: raiseIt

  // Do everything necessary to raise a window to the foreground
  // on the display.

  XtManageChild(w);
  XtPopup(XtParent(w), XtGrabNone);

  XmProcessTraversal(w, XmTRAVERSE_CURRENT); 
  XtRealizeWidget(w);  
  XRaiseWindow(dsply, XtWindow(XtParent(w)));
  XFlush(dsply);

} // end function raiseIt



void stuffGlobal()
{
  // function name: stuffGlobal
  
  XtUnmanageChild(imageFormW);
  XtManageChild(globalFormW);
  
  pif_read_gh(&pifGHeader);
  numImages     = pifGHeader.nimg;
  currentHeader = 0;

  XmTextSetString(headNumTextW, "Glob");
  TextSetInt(glbHeadTextW[0], pifGHeader.nimg);

  sprintf(outStr,"%E", pifGHeader.floatScaleFactor);
  XmTextSetString(glbHeadTextW[1], outStr);

  sprintf(outStr,"%E", pifGHeader.recipFloatScaleFactor);
  XmTextSetString(glbHeadTextW[2], outStr);

  TextSetInt(glbHeadTextW[3],      pifGHeader.htype);
  TextSetInt(glbHeadTextW[4],      pifGHeader.numCols);
  TextSetInt(glbHeadTextW[5],      pifGHeader.numRows);
  TextSetInt(glbHeadTextW[6],      pifGHeader.numSecs);
  TextSetInt(glbHeadTextW[7],      pifGHeader.mode);
  TextSetInt(glbHeadTextW[8],      pifGHeader.endianNess);
  XmTextSetString(glbHeadTextW[9], pifGHeader.versionStr);
  TextSetInt(glbHeadTextW[10],     pifGHeader.even);
  TextSetInt(glbHeadTextW[11],     pifGHeader.mrcX);
  TextSetInt(glbHeadTextW[12],     pifGHeader.mrcY);
  TextSetInt(glbHeadTextW[13],     pifGHeader.mrcZ);
  TextSetFlt(glbHeadTextW[14],     pifGHeader.int4scaleF);

  XFlush(dsply);

  dirty = FALSE;

} // end function stuffGlobal



void stuffImage(int imgNum)
{
  // function name: stuffImage
  
  XtManageChild(imageFormW);
  XtUnmanageChild(globalFormW);
  
  pifDHeader.imgNum = imgNum;
  pif_read_dh(&pifDHeader);

  TextSetInt(headNumTextW, imgNum);

  TextSetInt(imgHeadTextW[0],  pifDHeader.imgNum);
  TextSetInt(imgHeadTextW[1],  pifDHeader.mode);
  TextSetFlt(imgHeadTextW[2],  pifDHeader.img_bkg);
  TextSetInt(imgHeadTextW[3],  pifDHeader.datai4_min);
  TextSetInt(imgHeadTextW[4],  pifDHeader.datai4_max);
  TextSetFlt(imgHeadTextW[5],  pifDHeader.datar4_min);
  TextSetFlt(imgHeadTextW[6],  pifDHeader.datar4_max);
  TextSetFlt(imgHeadTextW[7],  pifDHeader.datar4_avg);
  TextSetFlt(imgHeadTextW[8],  pifDHeader.datar4_std);
  TextSetInt(imgHeadTextW[9],  pifDHeader.numCols);
  TextSetInt(imgHeadTextW[10], pifDHeader.numRows);
  TextSetInt(imgHeadTextW[11], pifDHeader.numSecs);
  XmTextSetString(imgHeadTextW[12], pifDHeader.descriptionStr);
  TextSetFlt(imgHeadTextW[13], pifDHeader.aoverb);
  TextSetFlt(imgHeadTextW[14], pifDHeader.map_abang);
  TextSetFlt(imgHeadTextW[15], pifDHeader.a_cell);
  TextSetFlt(imgHeadTextW[16], pifDHeader.b_cell);
  TextSetFlt(imgHeadTextW[17], pifDHeader.c_cell);
  TextSetFlt(imgHeadTextW[18], pifDHeader.alp);
  TextSetFlt(imgHeadTextW[19], pifDHeader.bet);
  TextSetFlt(imgHeadTextW[20], pifDHeader.gam);
  TextSetFlt(imgHeadTextW[21], pifDHeader.dela);
  TextSetFlt(imgHeadTextW[22], pifDHeader.delb);
  TextSetFlt(imgHeadTextW[23], pifDHeader.delc);
  TextSetFlt(imgHeadTextW[24], pifDHeader.a_star);
  TextSetFlt(imgHeadTextW[25], pifDHeader.b_star);
  TextSetFlt(imgHeadTextW[26], pifDHeader.c_star);
  TextSetFlt(imgHeadTextW[27], pifDHeader.alp_star);
  TextSetFlt(imgHeadTextW[28], pifDHeader.bet_star);
  TextSetFlt(imgHeadTextW[29], pifDHeader.gam_star);
  TextSetFlt(imgHeadTextW[30], pifDHeader.pixelSize);
  TextSetInt(imgHeadTextW[31], pifDHeader.t_matrix[0]);
  TextSetInt(imgHeadTextW[32], pifDHeader.t_matrix[1]);
  TextSetInt(imgHeadTextW[33], pifDHeader.t_matrix[2]);
  TextSetInt(imgHeadTextW[34], pifDHeader.t_matrix[3]);
  TextSetInt(imgHeadTextW[35], pifDHeader.t_matrix[4]);
  TextSetInt(imgHeadTextW[36], pifDHeader.t_matrix[5]);
  TextSetInt(imgHeadTextW[37], pifDHeader.packRadius);
  TextSetInt(imgHeadTextW[38], pifDHeader.binFactor);
  TextSetFlt(imgHeadTextW[39], pifDHeader.dthe);
  TextSetFlt(imgHeadTextW[40], pifDHeader.dphi_90);
  TextSetInt(imgHeadTextW[41], pifDHeader.symmetry);
  TextSetInt(imgHeadTextW[42], pifDHeader.pixSizUnits);
  TextSetFlt(imgHeadTextW[43], pifDHeader.res_out);
  TextSetInt(imgHeadTextW[44], pifDHeader.ctfMode);
  TextSetFlt(imgHeadTextW[45], pifDHeader.tempFactor);
  TextSetFlt(imgHeadTextW[46], pifDHeader.wiener);
  TextSetInt(imgHeadTextW[47], pifDHeader.borderType);
  TextSetInt(imgHeadTextW[48], pifDHeader.borderWidth);
  TextSetInt(imgHeadTextW[49], pifDHeader.rad_lo);
  TextSetInt(imgHeadTextW[50], pifDHeader.rad_hi);
  TextSetInt(imgHeadTextW[51], pifDHeader.transX);
  TextSetInt(imgHeadTextW[52], pifDHeader.transY);
  TextSetInt(imgHeadTextW[53], pifDHeader.transZ);
  TextSetInt(imgHeadTextW[54], pifDHeader.isTrunc);
  TextSetInt(imgHeadTextW[55], pifDHeader.origX);
  TextSetInt(imgHeadTextW[56], pifDHeader.origY);
  TextSetInt(imgHeadTextW[57], pifDHeader.origZ);
  TextSetFlt(imgHeadTextW[58], pifDHeader.pifVersion);

  XFlush(dsply);
  dirty = FALSE;

} // end function stuffImage



void getImage(int imgNum)
{
  // function name: getImage

  char *thisStr;
  int i;

  pifDHeader.mode       = strtod(XmTextGetString(imgHeadTextW[1]), 0);
  pifDHeader.img_bkg    = strtod(XmTextGetString(imgHeadTextW[2]), 0);
  pifDHeader.datai4_min = strtod(XmTextGetString(imgHeadTextW[3]), 0);
  pifDHeader.datai4_max = strtod(XmTextGetString(imgHeadTextW[4]), 0);
  pifDHeader.datar4_min = strtod(XmTextGetString(imgHeadTextW[5]), 0);
  pifDHeader.datar4_max = strtod(XmTextGetString(imgHeadTextW[6]), 0);
  pifDHeader.datar4_avg = strtod(XmTextGetString(imgHeadTextW[7]), 0);
  pifDHeader.datar4_std = strtod(XmTextGetString(imgHeadTextW[8]), 0);
  pifDHeader.numCols    = strtod(XmTextGetString(imgHeadTextW[9]), 0);
  pifDHeader.numRows    = strtod(XmTextGetString(imgHeadTextW[10]), 0);
  pifDHeader.numSecs    = strtod(XmTextGetString(imgHeadTextW[11]), 0);

  thisStr= XmTextGetString(imgHeadTextW[12]);
  thisStr[79]= (char)'\0';
  sprintf(pifDHeader.descriptionStr,"%s",thisStr);
  XtFree(thisStr);

  pifDHeader.aoverb    = strtod(XmTextGetString(imgHeadTextW[13]), 0);
  pifDHeader.map_abang = strtod(XmTextGetString(imgHeadTextW[14]), 0);
  pifDHeader.a_cell    = strtod(XmTextGetString(imgHeadTextW[15]), 0);
  pifDHeader.b_cell    = strtod(XmTextGetString(imgHeadTextW[16]), 0);
  pifDHeader.c_cell    = strtod(XmTextGetString(imgHeadTextW[17]), 0);
  pifDHeader.alp       = strtod(XmTextGetString(imgHeadTextW[18]), 0);
  pifDHeader.bet       = strtod(XmTextGetString(imgHeadTextW[19]), 0);
  pifDHeader.gam       = strtod(XmTextGetString(imgHeadTextW[20]), 0);
  pifDHeader.dela      = strtod(XmTextGetString(imgHeadTextW[21]), 0);
  pifDHeader.delb      = strtod(XmTextGetString(imgHeadTextW[22]), 0);
  pifDHeader.delc      = strtod(XmTextGetString(imgHeadTextW[23]), 0);
  pifDHeader.a_star    = strtod(XmTextGetString(imgHeadTextW[24]), 0);
  pifDHeader.b_star    = strtod(XmTextGetString(imgHeadTextW[25]), 0);
  pifDHeader.c_star    = strtod(XmTextGetString(imgHeadTextW[26]), 0);
  pifDHeader.alp_star  = strtod(XmTextGetString(imgHeadTextW[27]), 0);
  pifDHeader.bet_star  = strtod(XmTextGetString(imgHeadTextW[28]), 0);
  pifDHeader.gam_star  = strtod(XmTextGetString(imgHeadTextW[29]), 0);
  pifDHeader.pixelSize = strtod(XmTextGetString(imgHeadTextW[30]), 0);

  for(i=0; i<6; i++)
    pifDHeader.t_matrix[i] = strtod(XmTextGetString(imgHeadTextW[(31+i)]), 0);

  pifDHeader.packRadius  = strtod(XmTextGetString(imgHeadTextW[37]), 0);
  pifDHeader.binFactor   = strtod(XmTextGetString(imgHeadTextW[38]), 0);
  pifDHeader.dthe        = strtod(XmTextGetString(imgHeadTextW[39]), 0);
  pifDHeader.dphi_90     = strtod(XmTextGetString(imgHeadTextW[40]), 0);
  pifDHeader.symmetry    = strtod(XmTextGetString(imgHeadTextW[41]), 0);
  pifDHeader.pixSizUnits = strtod(XmTextGetString(imgHeadTextW[42]), 0);
  pifDHeader.res_out     = strtod(XmTextGetString(imgHeadTextW[43]), 0);
  pifDHeader.ctfMode     = strtod(XmTextGetString(imgHeadTextW[44]), 0);
  pifDHeader.tempFactor  = strtod(XmTextGetString(imgHeadTextW[45]), 0);
  pifDHeader.wiener      = strtod(XmTextGetString(imgHeadTextW[46]), 0);
  pifDHeader.borderType  = strtod(XmTextGetString(imgHeadTextW[47]), 0);
  pifDHeader.borderWidth = strtod(XmTextGetString(imgHeadTextW[48]), 0);
  pifDHeader.rad_lo      = strtod(XmTextGetString(imgHeadTextW[49]), 0);
  pifDHeader.rad_hi      = strtod(XmTextGetString(imgHeadTextW[50]), 0);
  pifDHeader.transX      = strtod(XmTextGetString(imgHeadTextW[51]), 0);
  pifDHeader.transY      = strtod(XmTextGetString(imgHeadTextW[52]), 0);
  pifDHeader.transZ      = strtod(XmTextGetString(imgHeadTextW[53]), 0);
  pifDHeader.isTrunc     = strtod(XmTextGetString(imgHeadTextW[54]), 0);
  pifDHeader.origX       = strtod(XmTextGetString(imgHeadTextW[55]), 0);
  pifDHeader.origY       = strtod(XmTextGetString(imgHeadTextW[56]), 0);
  pifDHeader.origZ       = strtod(XmTextGetString(imgHeadTextW[57]), 0);
  pifDHeader.pifVersion  = strtod(XmTextGetString(imgHeadTextW[58]), 0);

  XFlush(dsply);

} // end function getImage



void getGlobal(int imgNum)
{
  // function name: getGlobal

  char *thisStr;

  pifGHeader.nimg                 = strtod(XmTextGetString(glbHeadTextW[0]), 0);
  pifGHeader.floatScaleFactor     = strtod(XmTextGetString(glbHeadTextW[1]), 0);
  pifGHeader.recipFloatScaleFactor= strtod(XmTextGetString(glbHeadTextW[2]), 0);
  pifGHeader.htype                = strtod(XmTextGetString(glbHeadTextW[3]), 0);
  pifGHeader.numCols              = strtod(XmTextGetString(glbHeadTextW[4]), 0);
  pifGHeader.numRows              = strtod(XmTextGetString(glbHeadTextW[5]), 0);
  pifGHeader.numSecs              = strtod(XmTextGetString(glbHeadTextW[6]), 0);
  pifGHeader.mode                 = strtod(XmTextGetString(glbHeadTextW[7]), 0);
  pifGHeader.endianNess           = strtod(XmTextGetString(glbHeadTextW[8]), 0);

  thisStr= XmTextGetString(glbHeadTextW[9]);
  thisStr[79]= '\0';
  sprintf(pifGHeader.versionStr,"%s",thisStr);
  XtFree(thisStr);

  pifGHeader.even       = strtod(XmTextGetString(glbHeadTextW[10]), 0);
  pifGHeader.mrcX       = strtod(XmTextGetString(glbHeadTextW[11]), 0);
  pifGHeader.mrcY       = strtod(XmTextGetString(glbHeadTextW[12]), 0);
  pifGHeader.mrcZ       = strtod(XmTextGetString(glbHeadTextW[13]), 0);
  pifGHeader.int4scaleF = strtod(XmTextGetString(glbHeadTextW[14]), 0);

  XFlush(dsply);

} // end function getGlobal



void read_pif()
{
  // function name: read_pif
  // function name: Read image(s) in pif format.

  int i=4;
  fString_t rWrite;

  if(fileIsOpen)
    pif_close(&i); // Close it before we go on

  pifGHeader.luNum = i;
  pifDHeader.luNum = i;
  rWrite.addr = malloc(sizeof(char)*4);
  rWrite.length = 1;
  sprintf(rWrite.addr,"r+");
  pif_open(fileName.addr, &i, rWrite.addr);

  fileIsOpen = TRUE;

  // Now stuff text widgets with what you've found
  stuffGlobal();

} // end function read_pif



void gotFilename(Widget w, char *client_data,
		 XmFileSelectionBoxCallbackStruct *callback_data)
{
  // function name: gotFilename
  // Get the filename they specified in file selection widget. Branch
  // according to what operation they want performed on this filename.
  
  XmString simpleString;

  // Pop down the file selection widget
  XtUnmanageChild(fileSelW);
  XFlush(dsply);
  
  // Get the filename
  if(!XmStringGetLtoR(callback_data->value, XmSTRING_DEFAULT_CHARSET, 
		      &(fileName.addr)))
    printf("Error extracting filename.\n");
  
  // Read the file
  read_pif();
  
  // Give feedback on main window title bar
  sprintf(outStr,"FixPIF: [%s] - %d images total.",fileName.addr,
	  pifGHeader.nimg);
  simpleString = XmStringCreateLocalized(outStr);
  XtSetArg(arglst[0], XmNdialogTitle, simpleString );
  XtSetArg(arglst[1], XmNrecomputeSize, FALSE );
  XtSetValues(mainWindowW, arglst, 2); 
  XFlush(dsply);
  XmStringFree(simpleString); 
  XFlush(dsply);
  
} // end function gotFilename



void cancel_fileSel(Widget widget, char *tag,
		    XmFileSelectionBoxCallbackStruct *callback_data)
{
  // function name: cancel_fileSel
  // They want to abort the selection of a file name.

  XtUnmanageChild(fileSelW);

} // end function cancel_fileSel



void imgNumActivate(Widget widget, XtPointer client_data, 
		    XtPointer call_data)
{
  // function name: imgNumActivate
  // When the "ImgNum" button is pressed redisplay the window
  // using the new imgNum.

  char *thisStr;

  if(fileIsOpen) {

    if(dirty) { // Check to see if we need to write first
      if(askThem("Do you want to save your changes to this header?",
		 XmDIALOG_QUESTION))
	write_activate(NULL, NULL, NULL);
    }

    thisStr= XmTextGetString(headNumTextW);
    currentHeader = strtod(thisStr,0);
    XtFree(thisStr);
    
    // Clip it accordingly
    currentHeader = MIN(currentHeader, numImages);
    currentHeader = MAX(currentHeader, 0);
    
    sprintf(outStr,"%d",currentHeader);
    XmTextSetString(headNumTextW, outStr);
    
    if(currentHeader==0)
      stuffGlobal();
    else 
      stuffImage(currentHeader);
    
  }
  else
    askThem("You must open a file first.", XmDIALOG_INFORMATION);
  
} // end function imgNumActivate



void text_changed(Widget widget, XtPointer client_data, XtPointer call_data)
{
  // function name: text_changed

  dirty = TRUE;

} // end function text_changed



void text_create(Widget widget, XtPointer client_data, XtPointer call_data)
{
  // function name: text_create
  // Create all of the text widgets needed for each PIF header attribute.

  char numStr[20];
  int i, totX;
  Arg arglist[9];
  XFontStruct *font;
  XmFontList font_list;
  XmStringCharSet cs= "ISOLatin1";

  // Load all of these guys with proportional "fixed" font
  font = XLoadQueryFont(dsply, "fixed");
  font_list = XmStringCreateFontList(font, cs); 
  
  totX= 0;
  
  if(widget == headNumTextW) { // Header number text widget
    handle_cr(widget);
  }
  else if(widget == globalFormW) { // global form
    
    for(i=0; i<MAXGLBFIELDS; i++) { // Do all fields
      XtSetArg(arglist[0], XmNx, 130);
      XtSetArg(arglist[1], XmNy, (50 + i*30));
      XtSetArg(arglist[2], XmNcolumns, 50);
      XtSetArg(arglist[3], XmNfontList, font_list);      
      XtSetArg(arglist[4], XmNeditable, FALSE);      
      sprintf(numStr,"Number%d",i);
      
      glbHeadTextW[i] = XmCreateText(widget, numStr, arglist, 3);
      XtAddCallback(glbHeadTextW[i], XmNvalueChangedCallback, 
		    (XtCallbackProc) text_changed, NULL);
      XtManageChild(glbHeadTextW[i]);
      
      XtSetArg(arglist[0], XmNx, 5);
      XtSetArg(arglist[1], XmNy, (50 + (i*30)));
      XtSetArg(arglist[2], XmNalignment, XmALIGNMENT_BEGINNING);
      XtSetArg(arglist[3], XmNlabelString, 
	       XmStringLtoRCreate(glbFields[i], cs));
      
      XtManageChild(XmCreateLabel(widget, "", arglist, 4));  
    }
  }
  else if (widget == imageFormW) { // Image header form
    Arg arglst[32];
    XmString tcs;
    XtCallbackRec callBck[5];    // Our callbacks
    
    // Setup the file selection widget - do this only once
    fileSelW = (Widget)XmCreateFileSelectionDialog(mainWindowW,
							 "Get file",NULL,0);
    tcs = XmStringCreateLocalized("*.*");
    XtSetArg(arglst[0], XmNx, 10);
    XtSetArg(arglst[1], XmNy, 10 );
    XtSetArg(arglst[2], XmNdirMask, tcs);  
    XtSetArg(arglst[3], XmNdefaultPosition, FALSE); 
    XtSetArg(arglst[4], XmNfontList, fixpifFontList); 
    XtSetArg(arglst[5], XmNlabelFontList,  fixpifFontList);
    XtSetArg(arglst[6], XmNbuttonFontList,  fixpifFontList);
    XtSetValues(fileSelW, arglst, 7);
    
    // Add callbacks to file selection widget
    callBck[0].callback= (XtCallbackProc)gotFilename;
    callBck[0].closure= 0;
    callBck[1].callback= 0;
    callBck[1].closure= 0;
    XtAddCallbacks(fileSelW, XmNokCallback, callBck);  
    callBck[0].callback= (XtCallbackProc)cancel_fileSel;
    callBck[0].closure= 0;
    callBck[1].callback= 0;
    callBck[1].closure= 0;
    XtAddCallbacks(fileSelW, XmNcancelCallback, callBck);  
    XFlush(dsply);
    XmStringFree(tcs);
    
    totX= 0;
    for(i=0; i<MAXIMGFIELDS; i++) { // Do all fields
      if(i < 23)
	XtSetArg(arglist[1], XmNy, (10 + i*30));
      else { // DO next column
	if(i < 47) { // second row
	  totX = 380;
	  XtSetArg(arglist[1], XmNy, (10 + ((i-22)*30)));
	}
	else { // third row
	  totX = 650;
	  XtSetArg(arglist[1], XmNy, (10 + ((i-46)*30)));
	}
      }
      XtSetArg(arglist[0], XmNx, (totX + 130) );
      XtSetArg(arglist[2], XmNcolumns, 20);
      XtSetArg(arglist[3], XmNfontList, font_list);      
      XtSetArg(arglist[4], XmNeditable, FALSE);      
      sprintf(numStr,"Number%d",i);
      
      imgHeadTextW[i] = XmCreateText(widget, numStr, arglist, 3);
      XtAddCallback(imgHeadTextW[i], XmNvalueChangedCallback, 
		    (XtCallbackProc) text_changed, NULL);
      XtManageChild(imgHeadTextW[i]);
      
      if(i < 23)
	XtSetArg(arglist[1], XmNy, (10 + i*30));
      else { // DO next column
	if(i < 47) { // second row
	  totX = 380;
	  XtSetArg(arglist[1], XmNy, (10 + ((i-22)*30)));
	}
	else { // third row
	  totX = 700;
	  XtSetArg(arglist[1], XmNy, (10 + ((i-46)*30)));
	}
      }
      XtSetArg(arglist[0], XmNx, (totX + 5) );
      XtSetArg(arglist[2], XmNalignment, XmALIGNMENT_BEGINNING);
      XtSetArg(arglist[3], XmNlabelString, 
	       XmStringLtoRCreate(imgFields[i], cs));
      
      XtManageChild(XmCreateLabel(widget, "", arglist, 4));
      
    }
  }
  
} // end function text_create



int askThem(char *infoStr, int dialogType)
{
  // function name: askThem
  // This is the generic display a message and get them to respond
  // screen. We get a return value from globAsk which is set by
  // infoOK() or infoCancel() (see below).

  XmString simpleString;

  // Init and try to position the window
  globAsk = -99;
  XFlush(dsply);
  simpleString = XmStringCreateLocalized(infoStr);
  XtSetArg(arglst[0], XmNmessageString, simpleString);
  XtSetArg(arglst[1], XmNdialogType, dialogType);
  XtSetArg(arglst[2], XmNx, 150);
  XtSetArg(arglst[3], XmNy, 150);
  XtSetValues(infoBoxW, arglst, 4);
  XFlush(dsply);
  XmStringFree(simpleString); 
 
  // Init it and pop it up sit here and wait until we get 
  // something back
  gFinished= FALSE;
  XtManageChild(infoBoxW);
  XtPopup(XtParent(infoBoxW), XtGrabNone);

  XmProcessTraversal(infoBoxW, XmTRAVERSE_CURRENT); 
  XtRealizeWidget(infoBoxW);  
  XFlush(dsply);

  // gFinished is set in the infoOK() or infoCancel() callback
  while(!gFinished)
    XtAppProcessEvent(app_context, XtIMAll);

  // Pop it down and return their response
  XtUnmanageChild(infoBoxW);
  XtPopdown(XtParent(infoBoxW));
  XtSetArg(arglst[0], XmNdialogStyle, XmDIALOG_MODELESS);
  XtSetValues(XtParent(infoBoxW), arglst, 1);
  XFlush(dsply);

  return(globAsk);

} // end function askThem



void infoOK(Widget widget, XtPointer client_data, XtPointer call_data)
{
  // function name: infoOK
  // If the OK button is pressed from the infoscreen return here. Set
  // gFinished to cause the value of globAsk be return to the starting
  // point (see askThem() above).

  globAsk = TRUE;
  XtUnmanageChild(infoBoxW);
  gFinished= TRUE;

} // end function infoOK



void infoCancel(Widget widget, XtPointer client_data, XtPointer call_data)
{
  // function name: infoCancel
  // If the CANCEL button is pressed from the infoscreen return here.
  // Set gFinished to cause the value of globAsk be return to the
  // starting point (see askThem() above).

  globAsk = FALSE;
  XtUnmanageChild(infoBoxW);
  gFinished= TRUE;

} // end function infoCancel



void read_activate(Widget widget, XtPointer client_data, XtPointer call_data)
{
  // function name: read_activate
  // They selected the "Read" submenu. Branch on the appropriate.

  raiseIt(fileSelW); 

} // end function read_activate



void write_all_activate(Widget widget, XtPointer client_data, 
			XtPointer call_data)
{
  // function name: write_all_activate

  int i, saveCurrentHeader;

  saveCurrentHeader = currentHeader;

  for(i=0; i<=numImages; i++) {
    currentHeader = i;
    sprintf(outStr,"%d",currentHeader);
    XmTextSetString(headNumTextW, outStr);
    
    if(currentHeader==0)
      stuffGlobal();
    else 
      stuffImage(currentHeader);
    
    if(currentHeader > 0) { // image
      pifDHeader.imgNum = currentHeader;
      getImage(currentHeader);
      pif_write_dh(&pifDHeader);
    }
    else {
      getGlobal(currentHeader);
      pif_write_gh(&pifGHeader);
    }
  }
  
  currentHeader = saveCurrentHeader;
  sprintf(outStr,"%d",currentHeader);
  XmTextSetString(headNumTextW, outStr);
  
  if(currentHeader==0)
    stuffGlobal();
  else 
    stuffImage(currentHeader);
  
  dirty = FALSE;
  
} // end function write_all_activate



void write_activate(Widget widget, XtPointer client_data, XtPointer call_data)
{
  // function name: write_activate

  if(currentHeader > 0) { // image
    pifDHeader.imgNum = currentHeader;
    getImage(currentHeader);
    pif_write_dh(&pifDHeader);
  }
  else { // global
    getGlobal(currentHeader);
    pif_write_gh(&pifGHeader);
  }

  dirty = FALSE;

} // end function write_activate



void exit_activate(Widget widget, XtPointer client_data, XtPointer call_data)
{
  // function name: exit_activate
  // User wants to exit the program - ask again to make sure.
  
  exit(0);
  
} // end function exit_activate



void headNumArrow_activate(Widget widget, XtPointer client_data, 
			   XtPointer call_data)
{
  // function name: headNumArrow_activate
  // They want to expand the histogram on the Alter Colormap screen.

  char *thisStr;

  if(fileIsOpen) {
    
    if(dirty) { // Check to see if we need to write first
      if(askThem("Do you want to save your changes to this header?",
		 XmDIALOG_QUESTION))
	write_activate(NULL, NULL, NULL);
    }
    
    thisStr = XmTextGetString(headNumTextW);
    currentHeader = (int)strtod(thisStr,0);
    XtFree(thisStr);
    
    if(widget == headNumUpW)
      currentHeader++;
    else if(widget == headNumDownW)
      currentHeader--;
    
    // Clip it accordingly
    currentHeader = MIN(currentHeader, numImages);
    currentHeader = MAX(currentHeader, 0);

    sprintf(outStr,"%d",currentHeader);
    XmTextSetString(headNumTextW, outStr);
    
    if(currentHeader == 0)
      stuffGlobal();
    else 
      stuffImage(currentHeader);
  }
  else
    askThem("You must open a file first", XmDIALOG_INFORMATION);
  
} // end function headNumArrow_activate



void TextSetInt(Widget w, int i)
{
  // function name: TextSetInt
  // Set text string based in integer input and optionally flush
  // display after setting string. Do nothing if useGUI not set.

  char str[128];

  sprintf(str, "%d", i);
  XmTextSetString(w, str);

} // end function TextSetInt



void TextSetFlt(Widget w, float f)
{
  // function name: TextSetFlt
  // Set text string based in integer input and optionally flush
  // display after setting string. Do nothing if useGUI not set.

  char str[128];

  sprintf(str, "%f", f);
  XmTextSetString(w, str);

} // end function TextSetFlt



void handle_cr(Widget widget)
{
  // function name: handle_cr
  // This isn't really a callback but it is called by a callback. This
  // routine changes the translation table interpretations for the
  // simple text widgets so it knows what to do when it sees a
  // carriage return.

  XtTranslations parsed_t_table;

  if(widget==headNumTextW) {
    // Do translations for study text widget
    // Modify this widgets translation table
    parsed_t_table = XtParseTranslationTable(rwa_translation_table);
    XtOverrideTranslations(widget, parsed_t_table);
  }
  
} // end function handle_cr
