      subroutine calc_mod_tps (mod_ts, mod_ps, the1, the2, phi1, 
     1 del_the, del_phi_90, nview_mod, sym)

C     Calculates theta and phi orientation angles within the search
C     window for computing projections and pft data from the model.
C
C     Notes - the step size in phi is scaled by 1/sin(theta) to ensure
C     an even sampling of the asymmetric unit; at theta = 0 or 180, phi
C     arbitrarily set to 0
C
C     D. Belnap, NIH, 30-Jan-1997
C     Misha and Rob Dec 2003 - changed the line below to prevent
C       iterating outside the asu.
C       nphi = nint( (find_phimax(sym,theta)-phi1) / dphi )  +  1
C     R. Sinkovits, general cleanup and commenting. Removed section of
C       code responsible for doing calculations in refine mode since PFT
C       search is used purely for search mode. Earlier revisions 
C       still contain old code. 21-Jul-2006

      implicit none

      integer*4, intent(in)    :: sym, nview_mod
      real*4, intent(in)       :: the1, the2, phi1
      real*4, intent(in)       :: del_the, del_phi_90      
      real*4, intent(out)      :: mod_ts(nview_mod), mod_ps(nview_mod)

      integer*4 nthe, nphi
      integer*4 num, ithe, iphi
      real*4    del_phi, the, ph, find_phimax, tol
      real*8 deg_to_rad
      parameter (tol = 0.0001)

      deg_to_rad = acos(-1.0d0)/180.0d0
      num = 0
      the = the1

      nthe = nint((the2-the1)/del_the) + 1
      do ithe = 1,nthe
         if (abs(the) < tol .or. abs(the-180) < tol) then
            num         = num + 1
            mod_ts(num) = the
            mod_ps(num) = 0.0
         else
            del_phi = real(del_phi_90 / sin(the*deg_to_rad))
            nphi = int((find_phimax(sym,the)-phi1)/del_phi) + 1
            ph = phi1
            do iphi = 1,nphi
               num         = num + 1
               mod_ts(num) = the
               mod_ps(num) = ph
               ph          = ph + del_phi
            enddo
         endif
         if (sym == 532 .and. abs(the-69.09484255211070) < tol) then
            the = 69.0 + del_the
         else
            the = the + del_the
         endif
      enddo
      
      return
      end subroutine calc_mod_tps
