      SUBROUTINE cfft_1d(CF_HKZ, CF_KLH, hkl_max, npix_row,
     +             nrffta_omega, rffta_omega)

C******************************************************** FFT_HKZ_TO_KLH
C*Purpose:  Given a complex array CF_KZH, transform along the real axis
C           Z to obtain the complex transform for all K and H.  The
C           original axis -ZMAX:ZMAX is enlarged to -LMAX:LMAX with
C           zero fill.
C*Tranform subroutine: 
C  VRFFTF (M,N,R,RT,MDIMR,WSAVE)
C
C  M       the number of (REAL) sequences to be transformed.
C  N       the length of the (REAL) sequences to be transformed.  The
C          method is most efficient when N is a product of small primes,
C          however n may be any positive integer.
C  R       a real two-dimensional array of size MDIMX x N containing the
C          the sequences to be transformed.  The sequences are stored
C          in the ROWS of R.  Thus, the I-th sequence to be transformed,
C          X(I,J), J=0,1,...,N-1, is stored as
C               R(I,J) = X(I,J-1) , J=1, 2, . . . , N.
C      !!NOTE!! in the application below, the array passed the COMPLEX
C               array C( -KMAX:(KMAX-1), 0:(2*LMAX-1) ), so
C                  M     .EQ. 2*2*KMAX
C                  N     .EQ. 2*LMAX
C                  MDIMR .EQ. 2*2*KMAX
C  RT      a real two-dimensional work array of size MDIMX x N.
C  MDIMR   the row (or first) dimension of the arrays R and RT exactly 
C          as they appear in the calling program.  This parameter is 
C          used to specify the variable dimension of these arrays.
C  WSAVE   a real one-dimensional work array which must be dimensioned
C          at least N+15.  The WSAVE array must be initialized by 
C          calling subroutine VRFFTI.  A different WSAVE array must be
C          used for each different value of N.  This initialization does
C          not have to be repeated so long as N remains unchanged.  The
C          same WSAVE array may be used by VRFFTF and VRFFTB.
C***********************************************************************
      IMPLICIT NONE
      INCLUDE '../include/allprogs.inc'
C Arguments 
      INTEGER*4 hkl_max, npix_row, nrffta_omega
      real*4    rffta_omega(nrffta_omega)
      complex*8 CF_HKZ( frst_lst_iq(1,iam):frst_lst_iq(2,iam),
     +                  -hkl_max:hkl_max, npix_row),
     +          CF_KLH( -hkl_max:hkl_max, -hkl_max:hkl_max,
     +                  frst_lst_iq(1,iam):frst_lst_iq(2,iam))
C Local variables
      INTEGER*4 H, K, L, Z, cen, num_hkl , L1, L2
      complex*8 C(-hkl_max:hkl_max, 0:2*hkl_max-1),
     +          RT(-hkl_max:hkl_max, 0:2*hkl_max-1) 
      REAL*4    ReRe, ReAIm, AImRe, AImAIm
C***********************************************************************
C
      num_hkl  =  2*hkl_max + 1
C
      DO H = frst_lst_iq(1,iam), frst_lst_iq(2,iam)
Cji move Z to fill into L, also correct center and fill zero.
Cji 1     cen     npix_row    1                       2*hkl_max
Cji L******c******R       ==> c******R00000000000L******
C
        cen = (npix_row -1)/2 + 1
        L1   = 0
        DO Z = cen, npix_row
          DO K = -hkl_max, hkl_max
            C( K, L1 ) = CF_HKZ( H, K, Z )
          ENDDO
          L1         = L1 + 1
        ENDDO

        L2 = num_hkl-2
        DO Z = cen-1, 1, -1
          DO K = -hkl_max, hkl_max
            C( K, L2 ) = CF_HKZ( H, K, Z )
          ENDDO
          L2         = L2 - 1
        ENDDO
 
        if ( npix_row .lt. num_hkl-1 ) then
C   Insert columns of zeros in these columns.
          DO L = L1,L2
            DO K = -hkl_max, hkl_max
              C( K, L ) = ( 0.0, 0.0 )
            ENDDO
          ENDDO
        ENDIF
 
C       Complex array C is processed by VRFFTF as a real array having
C       2*2*KMAX rows and 2*LMAX columns of real values.  Rows 1,3,5,...
C       are the real parts and rows 2,4,6,... are the imaginary parts
C       of the complex rows in C.

        CALL vrfftf(2*num_hkl,num_hkl-1,C,RT,2*num_hkl,rffta_omega)
 
C       The complex array C(-KMAX:KMAX,0:2*LMAX-1) contains the output.
C       Because the routine vrfftf processes REAL rows, two successive
C       rows contain values which have to be combined to obtain the
C       complex DFT.  Example with period 2*LMAX-1 = 8.  R(F(j)) denotes
C       the real part of the complex DFT and I(F(j)) denotes the
C       imaginary part.  I(F(0)) .EQ. I(F(4)) .EQ. 0.  Suppose
C       the computed REAL row(j), j odd, is 
C        RF(0),RF(1),IF(1) RF(2),IF(2),RF(3) IF(3),RF(4))
C       and the next row, j+1, is
C        RG(0),RG(1),IG(1) RG(2),IG(2),RG(3) IG(3),RG(4)
C       Because these are for real sequences, one has conjugate
C       symmetry:  Complex( j) = RF(j) + i IF(j)
C                  Complex(-j) = RF(j) - i IF(j)
C       Because of periodicity, RF(-4) = RF(4), both are real, so
C       there are no IF(4) or IF(-4); similarly for G.

C       Using ( real part      )  to denote the complex values, then
C             ( imaginary part )    
C       the first row gives the complex sequence
C       ... ( RF(2)) ( RF(1)) (RF(0)) (RF(1)) (RF(2)) (RF(3)) (RF(4))
C       ... (-IF(2)) (-IF(1)) (  0  ) (IF(1)) (IF(2)) (IF(3)) (  0  )
C       and the second row gives the complex sequence
C       ... ( RG(2)) ( RG(1)) (RG(0)) (RG(1)) (RG(2)) (RG(3)) (RG(4))
C       ... (-IG(2)) (-IG(1)) (  0  ) (IG(1)) (IG(2)) (IG(3)) (  0  )
C       But, when in a complex array, the second row (the G') are
C       multiplies by i = SQRT(-1).  Thus the `I' (which correspond
C       to the sine's) are multiplied by -1 and the first and second
C       rows of the G's are interchanged:
C       ... (+IG(2)) (+IG(1)) (  0  ) (-IG(1)) (-IG(2)) (-IG(3)) ( 0   )
C       ... ( RG(2)) ( RG(1)) (RG(0)) ( RG(1)) ( RG(2)) ( RG(3)) (RG(4))

C       Consequently we have to combine adjacent rows after the
C       processing by  vrfftf.  Here are a few of the combinations
C       which occur in the first row (REAL part) and the second
C       row (IMAGINARY part):
C       ... ( RF(1)+IG(1)) (RF(0)) (RF(1)-IG(1)) (RF(2)-IG(2))
C       ... (-IF(1)+RG(1)) (RG(0)) (IF(1)+RG(1)) (IF(2)+RG(2)) 
 
C       The preceding is an attempt to explain what the following
C       lines of the subroutine are trying to do.  At the end of this
C       subroutine are the comment lines of the routine  vrfftf.
C       Near the end of the documentation is a desciption of what
C       vrfftf computes for each line of REAL data.  So this
C       is what is done for the first, thrid, fifth, ... line
C       which correspong to the F's above and for the second, fourth,
C       ...., lines, one has to multiple by i = SQRT(-1) which
C       also interchanges the real and imaginary parts as I have
C       tried to descibe above (about the G's).  --- This documentation
C       given in P3DR, at the bottom of a subdirectory which begins
C       with `Vfftpk', which contains the code for the FFT's we are
C       using.

C       Form and transfer complex values to CF_KLH
 
Cji     complex*8 C(-hkl_max:hkl_max, 0:num_hkl-2),
C       For K = -KMAX+1, KMAX - 1, construct all columns L = 1:LMAX-1
Cji     Because we only do 1DFFT along Z direction, so we only consider
Cji     the symmetry along L direction.
        DO L = 1, hkl_max - 1
           DO K = -hkl_max , hkl_max
            ReRe   =   REAL(  C( K, 2*L - 1 ) )  !  cos( L 2pi K/N )
            ReAIm  =   REAL(  C( K, 2*L     ) )  !  sin( L 2pi K/N )
            AImRe  = - AIMAG( C( K, 2*L     ) )  ! -sin( L 2pi K/N )
            AImAIm =   AIMAG( C( K, 2*L - 1 ) )  !  cos( L 2pi K/N )
            CF_KLH( K, L, H ) = CMPLX( ReRe + AImRe,  ReAIm + AImAIm )
            CF_KLH( K,-L, H ) = CMPLX( ReRe - AImRe, -ReAIm + AImAIm )
          ENDDO
        ENDDO
C       For K = -KMAX, KMAX-1, construct column 0, LMAX, -LMAX
        DO K = -hkl_max, hkl_max
          CF_KLH( K, 0, H)    = C( K, 0 )
          CF_KLH( K, hkl_max, H) = C( K,num_hkl-2)
          CF_KLH( K,-hkl_max, H) = C( K,num_hkl-2)
        ENDDO

      ENDDO
 
      RETURN
      END SUBROUTINE cfft_1d
