      subroutine crowther_to_matrix(theta, phi, omega, rot)
C
C     Generates the rotation matrix corresponding to the Crowther angles
C     (theta, phi, omega)
C
C     Author: Steve Fuller 10/29/89
C
C     Modified: Robert Sinkovits 6/27/06 - general cleanup and
C     replacement of nonstandard sind()/cosd() functions with Fortran 90
C     sin()/cos()

      implicit none
      real*4, intent(out) :: rot(3,3)
      real*4, intent(in)  :: theta, phi, omega
      real*8 ct, st, cp, sp, co, so
      real*8 deg_to_rad

      deg_to_rad = acos(-1.0d0)/180.0d0

      st=sin(theta*deg_to_rad)
      ct=cos(theta*deg_to_rad)
      sp=sin(phi*deg_to_rad)
      cp=cos(phi*deg_to_rad)
      so=sin(omega*deg_to_rad)
      co=cos(omega*deg_to_rad)

      rot(1,1)=  real(so*ct*cp - co*sp)
      rot(1,2)= real(-so*ct*sp - co*cp)
      rot(1,3)= real(-so*st)
      rot(2,1)=  real(co*ct*cp + so*sp)
      rot(2,2)= real(-co*ct*sp + so*cp)
      rot(2,3)= real(-co*st)
      rot(3,1)=  real(st*cp)
      rot(3,2)= real(-st*sp)
      rot(3,3)=  real(ct)
 
      return
      end subroutine crowther_to_matrix
