      subroutine density_clear(rho, min_x, max_x, min_y, max_y, 
     +     den_min_x, den_max_x, f_z, l_z, density_map,
     +     map_min, map_max, border)

C     Generates final density map (density_map) from the calculated
C     density (rho). This routine is required because:
C
C     1. The dimensions of the final density map are not required to be
C     the same as those of the calculated density map.
C
C     2. Elements of the final density map further than den_max_x from
C     the origin are set to zero.
C
C     Hong Lin, 9/30/99 - original version
C     Yongchang Ji, 8/2000 - first modification
C     Yongchang Ji, 1/2002 - second modification
C     R Sinkovits, 7/10/06 - overhaul and cleanup

      implicit none

      integer, intent(in) :: min_x, max_x, min_y, max_y, border
      integer, intent(in) :: f_z, l_z, den_min_x, den_max_x
      real, intent(in)    :: rho(min_x:max_x, min_y:max_y, f_z:l_z)

      real, intent(out)   :: map_min, map_max
      real, intent(out)   :: density_map( den_min_x:den_max_x,
     $             den_min_x:den_max_x, f_z:l_z)

      integer col, row, sec, boxrad, boxrad2, inrad, inrad2
      real r, r2, delta, c2inv, soft

      if(border > 0.0) then
         c2inv  = 4.0/(border*border)
      else
         c2inv = 0.0
      endif

      boxrad  = den_max_x
      boxrad2 = boxrad*boxrad
      inrad   = den_max_x - border
      inrad2  = inrad*inrad

      map_min =  huge(map_min)
      map_max = -huge(map_max)
         
      do sec = max(f_z, den_min_x), min(l_z, den_max_x)
         do row = den_min_x, den_max_x
            do col = den_min_x, den_max_x
               
               r2 = sec*sec + row*row + col*col

               if(r2 >= boxrad2 .or. 
     $              row .gt. max_x .or. row .lt. min_x  .or. 
     $              col .gt. max_x .or. col .lt. min_x ) then
                  density_map(col,row,sec) = 0.
               else if(r2 > inrad2) then
                  r = sqrt(r2)
                  delta = (r - inrad)
                  soft = exp(-(delta*delta*c2inv))
                  density_map(col,row,sec) = rho(col,row,sec)*soft
               else
                  density_map(col,row,sec) = rho(col,row,sec)
               endif

               map_min = min(map_min,density_map(col,row,sec))
               map_max = max(map_max,density_map(col,row,sec))
                  
            enddo
         enddo
      enddo
            
      return
      end subroutine density_clear
