       subroutine disef_para(disef, IDIM1, IDIM2, funcweight,
     +     PIXSIZE, UNITS)

C     Calculates radially dependent weighting factor. Note that the
C     nature of the weighting factor depends on the value of the
C     parameter funcweight.
C
C     funcweight <= 10, disef(r) = r**funcweight, where r is the
C     distance in index space from the origin
C
C     funcweight > 10, disef(f) = exp(funcweight/400 * f**2), where f is
C     the spatial frequency
C
C     Note that this behavior is somewhat scary since the behavior of
C     disef changes abruptly at funcweight=10.0
C     
C     Author, Yongchang Ji, 8/2000-1/2002
C     Revised by Robert Sinkovits, 11/22/2006

      implicit none
      
      integer, intent(in) :: idim1, idim2, units
      real, intent(in) :: pixsize, funcweight
      real, intent(out) :: disef(0:idim1, -idim2:idim2)

      integer ix, iy
      real d_star_2, t, x, y, tmp
      
      
      if (funcweight <= 10.0)  then 

         do iy = -idim2, idim2
            do ix = 0, idim1
               x = real(ix)
               y = real(iy)
               t = x*x + y*y
               disef(ix,iy) = t**(funcweight/2.0)
            enddo
         enddo

      else
         
         if(units == 2) then
            tmp = 1.0/((idim1*2+1)*pixsize) ! already in nm units
         else
            tmp = 10.0/((idim1*2+1)*pixsize) ! convert to nm units
         endif

         do iy = -idim2, idim2
            do ix = 0, idim1
               x = real(ix)
               y = real(iy)
               d_star_2 = (x*x + y*y) * tmp * tmp
               disef(ix,iy) = exp ((funcweight/400.0) * d_star_2)
            enddo
         enddo
         
      endif
      
      return
      end subroutine disef_para
