      subroutine fft_2dfft(pixels, npix_row, npix_col, center,
     +             nrfft_omega, rfft_omega, nrfft_row, 
     +             nrfft_col, nfc_2d_row, 
     +             nfc_2d_col, fc_2d )
C
      implicit none
C
C Arguments
      integer*4 npix_row, npix_col, nrfft_row, nrfft_col, nrfft_omega,
     +        nfc_2d_row, nfc_2d_col
      complex fc_2d( 0:nfc_2d_row, -nfc_2d_col:nfc_2d_col )
      real*4  pixels(npix_row, npix_col), rfft_omega(nrfft_omega)
C
C Local
      integer*4 icol, irow, icol_a, icol_a_1, irow_a_1, irow_b_1_rfft,
     +        icol_c, icol_c_1_rfft, irow_c_1_rfft, irow_d_1,
     +        icol_rfft, h, k, num_cols_a, num_rows_a, num_rows_b,
     +        num_cols_c, num_rows_c, num_rows_d, nfc_2d_end,
     +        nrfft_d2,
     +        nrfft_trns_row, nrfft_trns_col

      real*4 rfft(nrfft_row,nrfft_col),rfft_trns(nrfft_col,nrfft_row)
      real*4    cc, ss, sc, cs, center(2)

      external vrffti, vrfftf
C**********************************************************************
C
      nrfft_trns_row = nrfft_col    
      nrfft_trns_col = nrfft_row
C
C     The PIXELS array is square.
C
C     The array, PIXELS( 1:NPIX_ROW 1:NPIX_COL) is     --------- cols
C     partitioned into 4 arrays A, B, C, D:            | C | B |
C         A has nonnegative u and v coordinates        ----------- v
C         B has negative u, nonnegative v              | D | A |    
C         C has negative u and v                       ----|----   
C         D has nonnegative u and negative v           |   |   |
C                                                      |   | 
C                                                    rows  u 
C 
C     Transfer and rearrange PIXELS to the larger work array RFFT
C     and fill the rest of RFFT with zeros.
C
C     After rearrangement, the pixel values are      --------------- v
C     stored in RFFT as indicated to the right:      | A | 0 | D |
C                                                    -------------
C                                                    | 0 | 0 | 0 |
C                                                    -------------
C                                                    | B | 0 | C |
C                                                    -------------
C                                                    |
C                                                    u
C
      irow_a_1   = int(center(1)) + 1
      icol_a_1   = int(center(2)) + 1
      num_rows_a = npix_row + 1 - irow_a_1
      num_cols_a = npix_col + 1 - icol_a_1
C 
C     The first row number of A in PIXELS is the same as the first row
C     number of D in PIXELS.
C
      irow_d_1   = irow_a_1
C 
C     The first column number of A in PIXELS is the same as the first
C     column number of B in PIXELS
C     The number of rows of D is the same as the number of rows of A
C
      num_rows_d = num_rows_a
C
C     The number of columns of B and A are the same.
C     Find the number of rows and columns of C --- these are each
C     one less than the first row and column number of A in PIXELS
C
      num_rows_c    = irow_a_1 - 1
      num_cols_c    = icol_a_1 - 1
C
C     Find the first row and column numbers of C in array RFFT.
C
      irow_c_1_rfft = nrfft_row + 1 - num_rows_c
      icol_c_1_rfft = nrfft_col + 1 - num_cols_c
C
C     The first column number of C in RFFT is the same as the first
C     column number of D in RFFT.
C     The number of the first row of C in RFFT is the same as the
C     number of the first row of B in RFFT
C 
C     The number of rows of B is the same as the number of rows of C
C
      num_rows_b    = num_rows_c
C
C     The first row of B in RFFT is the same as the first row of C
C
      irow_b_1_rfft = irow_c_1_rfft
C 
C     Zero RFFT
C
      do icol = 1, nrfft_col
         do irow = 1, nrfft_row
            rfft( irow, icol ) = 0.
         enddo
      enddo
C
C     Use the BLAS routine SCOPY to copy real vectors from one location
C     to another.
C     Syntax:
C       SCOPY(vector_length,input_vector,stride,output_vector,stride)
C
      icol_rfft = 0
      do icol_a = icol_a_1, npix_col
        icol_rfft = icol_rfft + 1
C
C       Copy a column of A in PIXELS to a column of A in RFFT.
C
        call scopy( num_rows_a, pixels(irow_a_1,icol_a), 1,
     +                          rfft(1,icol_rfft),  1 )
C
C       Copy a column of B in PIXELS to a column of B in RFFT.
C
        call scopy( num_rows_b, pixels(1,icol_a), 1,
     +                          rfft(irow_b_1_rfft,icol_rfft), 1 )
      enddo
C
      icol_rfft = icol_c_1_rfft - 1
      do icol_c = 1, num_cols_c
        icol_rfft = icol_rfft + 1
C
C       Copy a column of C in PIXELS to a column of C in RFFT.
C
        call scopy( num_rows_c, pixels(1,icol_c), 1,
     +                          rfft(irow_c_1_rfft,icol_rfft), 1 )
C
C       Copy a column of D in PIXELS to a column of D in RFFT.
C
        call scopy( num_rows_d, pixels(irow_d_1,icol_c), 1,
     +                          rfft(1,icol_rfft), 1 )
      enddo
C 
C     Carry out real FFT analysis (foward transformation) of each row
C     of  RFFT.
C
C     Transformation parameters, RFFT_OMEGA, were computed the first
C     time this routine (FFTANALY) is called.  The values in RFFT_OMEGA
C     must not change between successive calls of VRFFTF unless the
C     size of RFFT changes.  The input is the array RFFT of rearranged
C     pixel values and a work array RFFT_TRNS.  The transformation is
C     with respect to the column index, V, which becomes the index K of
C     the Fourier coefficients.  The Fourier coefficents are returned in
C     RFFT whose column coordinate is now K and whose row coordinate
C     remains U.
C 
      if( npix_row .lt. nrfft_row ) then
C
C       Array RFFT has more rows than PIXELS; array RFFT contains
C       rows of zeros --- do not transform the rows of zeros.
C                                                            CALL VRFFTF
C                                                            ***********
C       Find Fourier coefficients of rows containing A and D.
C
        call vrfftf(num_rows_a, nrfft_col,
     +              rfft(1,1), rfft_trns(1,1),
     +              nrfft_col, rfft_omega)
C
C       Find Fourier coefficients of rows containing B and C.
C
        call vrfftf(num_rows_c, nrfft_col,
     +              rfft(irow_c_1_rfft,1), rfft_trns(irow_c_1_rfft,1),
     +              nrfft_col, rfft_omega)
      else
C
C       Array RFFT has same number of rows as PIXELS.  Find Fourier
C       coefficients of each row of RFFT
C
        call vrfftf(nrfft_row, nrfft_col,
     +              rfft(1,1), rfft_trns(1,1),
     +              nrfft_col, rfft_omega)
      endif
C 
C     See NOTE #2 at end of this subroutine.
C 
C     Transpose RFFT and store the transpose in RFFT_TRNS.  The column
C     index of RFFT_TRNS is U and its row index is K.
C
      do irow = 1, nrfft_row
         do icol = 1, nrfft_col
            rfft_trns(icol,irow) = rfft(irow,icol)
         enddo
      enddo
C     
C     Carry out a real FFT analysis (foward transformation) of each row
C     of RFFT_TRNS, i.e., with respect to U and the index of the Fourier
C     coefficients is H.  The Fourier coefficients are returned in
C     RFFT_TRNS whose column coordinate is now H and row index is K.
C
C                                                            CALL VRFFTF
C                                                            ***********
      call vrfftf(nrfft_trns_row, nrfft_trns_col,
     +            rfft_trns(1,1), rfft(1,1),
     +            nrfft_col, rfft_omega)
C 
C     Zero the complex output array                  ---------- k
C     FC_2D( 0:NFC_2d_ROW, -NFC_2D_COL:NFC_2D_COL )  |   |   |
C                                                    ----|----
C                                                        |
C                                                        h
C 
      do icol = -nfc_2d_col, nfc_2d_col
         do irow = 0, nfc_2d_row
            fc_2d( irow, icol ) = ( 0., 0. )
         enddo
      enddo
C     
C     The complex array FC_2D( 0:NFC_2D_ROW, -NFC_2D_COL:NFC_2D_COL )
C     will contain the complex valued Fourier coefficients of the
C     real values which were originally stored in RFFT (obtained
C     by rearranging the pixel values and zero fill).  The current
C     real values in the array RFFT_TRNS(1:NRFFT_COL, 1:NRFFT_ROW)
C     have to be combined to get the complex valued Fourier
C     coefficients.  The Fourier coefficients are periodic with periods
C     COL_PERIOD = 2*NFC_2D_COL and ROW_PERIOD = 2*NFC_2D_ROW
C
C     See NOTE #3 at end of this subroutine.
C
C     Set the value of the complex Fourier coefficient at the origin.
C
      fc_2d( 0, 0 ) = cmplx( rfft_trns( 1, 1), 0.)
C
      nrfft_d2 = nrfft_trns_col / 2
C
CXXXXXif( 2*nrfft_d2 .eq. nrfft_trns_col ) then
CXXXXXIn this version, only even NRFFT_TRNS_COL is allowed.
C       There is an even number of rows and colums in RFFT.
C       NFC_2D_END is equal to NRFFT_D2
C
        nfc_2d_end = nrfft_d2
C
C       Set complex value at end of H-axis.
C
        fc_2d(nrfft_d2, 0) = cmplx( rfft_trns(1,nrfft_trns_col), 0.)
C 
C       Set complex values at four corners; use periodicity.
C
        fc_2d(0, nrfft_d2) = cmplx(rfft_trns(nrfft_trns_row,1), 0.)
        fc_2d(0,-nrfft_d2) = fc_2d(0, nrfft_d2)
        fc_2d(nrfft_d2, nrfft_d2)
     +    = cmplx(rfft_trns(nrfft_trns_row,nrfft_trns_col), 0.)
        fc_2d(nrfft_d2,-nrfft_d2) = fc_2d(nrfft_d2, nrfft_d2)

C       Set rest of values along along H = 0 and H = NRFFT_D2
C
        do k = 1, nrfft_d2 - 1
          cc                    = rfft_trns( 2*k,   1 )
          sc                    = rfft_trns( 2*k+1, 1 )
          fc_2d( 0,  k )        = cmplx( cc, sc )
C
C         FFT of real function is conjugate symmetric.
C
          fc_2d( 0, -k )        = cmplx( cc,-sc )

          cc                    = rfft_trns( 2*k,   nrfft_row )
          sc                    = rfft_trns( 2*k+1, nrfft_row )
          fc_2d( nrfft_d2, k ) = cmplx( cc, sc )
C
C         FFT of real function is conjugate symmetric.
C
          fc_2d( nrfft_d2,-k ) = cmplx( cc,-sc )
       enddo

C       Set rest of values along along K = 0 and K =   NRFFT_D2
C       and use periodicity to set values along  K = - NRFFT_D2
C
        do h = 1, (nrfft_d2 - 1)
          cc                    = rfft_trns( 1, 2*h   )
          cs                    = rfft_trns( 1, 2*h+1 )
          fc_2d( h, 0 )         = cmplx( cc, cs )
          cc                    = rfft_trns( nrfft_col, 2*h   )
          cs                    = rfft_trns( nrfft_col, 2*h+1 )
          fc_2d( h, nrfft_d2 ) = cmplx( cc, cs )
C
C         Impose periodic conditions.
C
          fc_2d( h,-nrfft_d2 ) = cmplx( cc, cs )
       enddo

C       Fill FC_2D for  H =  1, 2, ... ,  NRFFT_D2 - 1
C                       K =  1, 2, ... ,  NRFFT_D2 - 1
C                       K = -1,-2, ... , -NRFFT_D2 + 1
 
       do k = 1, (nrfft_d2 - 1)
          do h = 1, (nrfft_d2 - 1)
             cc            = rfft_trns( 2*k,   2*h   )
             ss            = rfft_trns( 2*k+1, 2*h+1 )
             sc            = rfft_trns( 2*k+1, 2*h   )
             cs            = rfft_trns( 2*k,   2*h+1 )
             fc_2d( h, k ) = cmplx( cc - ss,  sc + cs )
             fc_2d( h,-k ) = cmplx( cc + ss, -sc + cs )
          enddo
       enddo
       
      return
      end subroutine fft_2dfft
