      SUBROUTINE FFT_HIP (FFT, IDIM1, IDIM2, RES_MIN, CUT_MIN, FFT_FULL)

C     APPLY HI-PASS EXPONENTIAL FILTER TO FFT DATA                          
C     
C     RES_MIN = RESOLUTION IN # OF TRANSFORM SAMPLE POINTS ABOVE WHICH FFT  
C               IS GIVEN FULL WEIGHT (WEIGHT = 1.0)                         
C     CUT_MIN = # OF TRANSFORM SAMPLE POINTS BELOW RES_MIN BELOW WHICH FFT  
C               IS GIVEN ZERO WEIGHT (WEIGHT = 0.0)                         
C     
C     BETWEEN RES_MIN-CUT_MIN AND RES_MIN THE WEIGHT OF THE FILTER RISES    
C     EXPONENTIALLY FROM 0.0 TO 1.0                                        
C     
C     AUGUST 1985  PURDUE UNIVERSITY VAX 11/750, T. BAKER.                  
C     SEPTEMBER 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.                 
C     LAST MODIFIED 12-AUG-1999 BY TSB (Fix XTEST,YTEST problems).          
C     Modified 8-aug-2006 by R Sinkovits, general cleanup and modernization
      
      IMPLICIT NONE
      
      INTEGER*4, intent(in) :: IDIM1, IDIM2
      real*4, intent(in)    :: RES_MIN, CUT_MIN
      LOGICAL, intent(in)   :: FFT_FULL
      complex, intent(inout) :: fft(idim1,idim2)
      
      integer*4 IX, IY, IY2, XTEST, YTEST
      REAL*4    MAX_SQ, YSQ, RSQ, DELSQ, WEIGHT
      REAL*4    MIN_SQ, TMP
      
      TMP    = RES_MIN - CUT_MIN
      MIN_SQ = TMP*TMP
      MAX_SQ = RES_MIN*RES_MIN
      DELSQ  = CUT_MIN**2
      XTEST  = IDIM1/2
      YTEST  = IDIM2/2
      
      IF(FFT_FULL) THEN
         IY2 = IDIM2
      ELSE
         IY2 = YTEST
      ENDIF
      
      DO IY = 1,IY2             ! PROCESS FFT A ROW AT A TIME
         IF(IY > YTEST) THEN
            YSQ = FLOAT(IY-IDIM2-1)**2
         ELSE
            YSQ = FLOAT(IY-1)**2
         ENDIF
         DO IX = 1,IDIM1        ! PROCESS CURRENT ROW
            IF(IX <= XTEST) THEN
               RSQ = YSQ + FLOAT(IX-1)**2 ! RIGHT HALF OF FFT
            ELSE
               RSQ = YSQ + FLOAT(IX-IDIM1-1)**2 ! LEFT HALF OF FFT
            ENDIF
            IF(RSQ < MAX_SQ) THEN ! WEIGHT < 1.0
               IF(DELSQ == 0.0) THEN
                  WEIGHT = 0.0  ! SHARP CUTOFF
               ELSE 
                  IF(TMP <= 0.0) THEN
                     WEIGHT = 1.0 - EXP(-(6.0*RSQ)/DELSQ)
                  ELSE
                     IF(RSQ < MIN_SQ) THEN
                        WEIGHT = 0.0
                     ELSE
                        WEIGHT = 1.0 - EXP(-(6.0*
     1                       (SQRT(RSQ)-TMP)**2/DELSQ))
                     ENDIF
                  ENDIF
               ENDIF
               FFT(ix,iy)   = WEIGHT*FFT(ix,iy)
            ENDIF
         ENDDO
      ENDDO
      
      RETURN
      END subroutine fft_hip
