      SUBROUTINE FFT_LOP (FFT, IDIM1, IDIM2, RES_MAX, CUT_MAX, FFT_FULL)
      
C     APPLY LOW-PASS EXPONENTIAL FILTER TO FFT DATA                         
C     
C     RES_MAX = RESOLUTION IN # OF TRANSFORM SAMPLE POINTS BELOW WHICH FFT  
C               IS GIVEN FULL WEIGHT (WEIGHT = 1.0)                         
C     CUT_MAX = # OF TRANSFORM SAMPLE POINTS BEYOND RES_MAX AT WHICH FFT    
C               IS REDUCED TO A WEIGHT = 1/EXP                              
C     
C     JANUARY 1985  PURDUE UNIVERSITY VAX 11/750, T. BAKER.                 
C     SEPTEMBER 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.                 
C     MODIFIED IN FFT_LOP TO AVOID FLOATING UNDER AND OVERFLOW PROBLEMS     
C         SDF 15-SEP-1992                                                     
C     MODIFIED 12-AUG-1999 BY TSB (Fix XTEST,YTEST problems).
C     Modified 8-aug-2006 by R Sinkovits, general cleanup and modernization
      
      IMPLICIT NONE

      INTEGER*4, intent(in) :: IDIM1, IDIM2
      REAL*4, intent(in)    :: RES_MAX, CUT_MAX
      LOGICAL, intent(in)   :: FFT_FULL
      complex, intent(inout) :: fft(idim1,idim2)
      
      integer*4 IX, IY, IY2, XTEST, YTEST
      real*4    MAX_SQ, YSQ, RSQ, DELSQ, WEIGHT
      
      MAX_SQ = RES_MAX*RES_MAX
      DELSQ  = CUT_MAX**2
      XTEST  = IDIM1/2
      YTEST  = IDIM2/2
      
      IF(FFT_FULL) THEN
         IY2 = IDIM2
      ELSE
         IY2 = YTEST
      ENDIF
      
      DO IY = 1,IY2             ! PROCESS FFT A ROW AT A TIME
         IF(IY > YTEST) THEN
            YSQ = FLOAT(IY-IDIM2-1)**2
         ELSE
            YSQ = FLOAT(IY-1)**2
         ENDIF
         DO IX = 1,IDIM1        ! PROCESS CURRENT FFT ROW
            IF(IX <= XTEST) THEN
               RSQ = YSQ + FLOAT(IX-1)**2 ! RIGHT HALF OF FFT
            ELSE
               RSQ = YSQ + FLOAT(IX-IDIM1-1)**2 ! LEFT HALF OF FFT
            ENDIF
            if(RSQ > MAX_SQ) then ! WEIGHT < 1.0
               if(DELSQ == 0.0) then
                  WEIGHT = 0.0  ! SHARP CUTOFF
               else 
                  if((RES_MAX > 0.0) .and. 
     1                 (((sqrt(RSQ)-RES_MAX)**2/DELSQ) < 50.0)) then
                     WEIGHT = EXP(-((SQRT(RSQ)-RES_MAX)**2/DELSQ))
                  else 
                     if(RSQ/DELSQ < 50.0) then
                        WEIGHT = EXP(-RSQ/DELSQ)
                     else
                        WEIGHT = 0.0
                     end if
                  endif
               endif
               FFT(ix,iy)   = WEIGHT*FFT(ix,iy)
            ENDIF
         ENDDO
      ENDDO
      
      RETURN
      END subroutine fft_lop
