      subroutine focus_astig (focus, focus_maj, focus_min, psi, phi)

C     Compute defocus value for astigmatic data for a particular
C     direction with respect to the x,y image plane
C
C     Note: for non-astigmatic images (i.e. focus_maj = focus_min), at each
C     radius of the image fft, the defocus remains constant.
C
C     focus_maj = defocus value (in nms) along major axis.
C     focus_min = defocus value (in nms) along minor axis.
C     phi       = angle (in radians) specifying the direction of focus_maj
C                 relative to the x-axis of the digitized image
C     psi       = angle (in radians) of a particular fourier component
C                 whose defocus value is to be calculated
C
C     August 1999 Purdue University Alphastation 500/400, T. Baker.
C     Modified  6-aug-1999 by T. Baker
C     General cleanup, 11/15/2006 by Robert Sinkovits

      implicit none

      real, intent(in)  :: focus_maj, focus_min, psi, phi
      real, intent(out) :: focus
      real ang

      if(focus_maj == focus_min) then
         focus = focus_maj
      else
         ang   = psi - phi
         focus = focus_maj*cos(ang)**2 + focus_min*sin(ang)**2
      endif

      return
      end subroutine focus_astig
