      subroutine gen_read_pixel_frame(fname, pix_file_unit, indx,
     $     npix_col, npix_row, npix_col_bin, npix_row_bin, binfactor,
     $     pixels, err_code)


C     Read a single pixel frame from a boxed image file and store in a
C     real array. Allowed image file formats are currently restricted to
C     the following
C     
C     mode=0  - char  (1 byte)
C     mode=1  - short (2 bytes)
C     mode=6  - char  (2 bytes)
C     mode=46 - int   (4 bytes)
C
C     This routine is adapated from Hong Lin's rpifimag_f and has been
C     modified so as to be easier to maintain and more straightforward
C     to call directly without having to worry about using common blocks.
C
C     fname              file name (w/ pif or mrc extension)
C     pix_file_unit      logical unit for opening file
C     indx               index for image in stack
C     npix_col           number of pixels in column
C     npix_row           number of pixels in row
C     pixels             Array of pixel values
C     err_code           Error code

      implicit none
      include '../include/pifhead.inc'

      character(len=256), intent(in) :: fname
      character(len=1) :: rw(2)
      integer, intent(in)    :: pix_file_unit, indx, npix_col, npix_row
      integer, intent(in)    :: npix_col_bin, npix_row_bin, binfactor
      real, intent(out)      :: pixels(npix_col_bin*npix_row_bin)
      integer, intent(out)   :: err_code

      type(pifhead_t) imghead
      character pixels_buf_byte(npix_col*npix_row)
      character(len=256) :: tmpname
      integer*2 pixels_buf_int2(npix_col*npix_row)
      integer   i4, k, npix_col_mrc, npix_row_mrc, npix_sec_mrc
      real      mrcslice(npix_col*npix_row), bkg
      real      full(npix_col*npix_row)

      character(len=256) :: fullname
      integer :: skip, img_size, jmode, i, j, l, m, n
      integer :: kxyz(3), mxyz(3), nfilsz, kspg, kbs, nbytes
      integer :: ip1, im1, jp1, jm1, oddfactor, count
      integer :: skip_fl=0
      real*4  :: dmin, dmax, dmean, tmp(9)

C     -----------------------------------------------------------
C     Standard MRC imcom:
      character*4  cstamp/'MAP'/
      integer*4 karch(5), labls(20,10,5), lstream(12), mapcrs(3,5)
      integer*4 mode(5), nb(5), nbsym(5), ncrs(3,5), ncrst(3,5)      
      integer*4 nlab(5), nspg(5), nxyz(3,5)
      integer*4 nbhdr, nbw, nbw3, nbl, numopen
      logical   flag(5), old(5), nocon(5), unknown(5)
      real*4    cel(6,5), denmmm(3,5), rstamp, stuff(25,5)
      real*4    origin(3,5), stamp(2,5), rms(5)

      common/imcom/ lstream, nbhdr, nbw, nbw3, nb, nbl, ncrs, mode,
     $     ncrst, nxyz, cel, mapcrs, denmmm, nspg, nbsym, stuff,
     $     origin, stamp, rms, nlab, labls, numopen, karch,
     $     nocon, flag, old, unknown
      
      equivalence      (cstamp,rstamp)
C     -----------------------------------------------------------

C     Initialize err_code to failure. Will be overwritten if successful
      err_code = 1

C     Sometimes need to cleanup the string before attempting to use
      do k=1,256
         tmpname(k:k) = ' '
      enddo
      do k=1,256
         if(ichar(fname(k:k)) > 32 .and. ichar(fname(k:k)) <127 ) then
            tmpname(k:k) = fname(k:k)
         endif
      enddo

      if(index(tmpname,'.pif') > 0) then

         rw(1) = 'r'
         rw(2) = achar(0)
         call pif_open(tmpname, pix_file_unit, rw)
         imghead%lunum = pix_file_unit
         call pif_read_gh(imghead)

C     Test for invalid particle image
         if(indx > imghead%nimg .or. indx <= 0) then
            return
         endif

         imghead%imgnum = indx
         call pif_read_dh(imghead)
         imghead%imgnum = indx

C        Process byte image
         if (imghead%mode == 0 .or. imghead%mode == 6) then
            call pif_read_byte_image (imghead%luNum, pixels_buf_byte,
     $           imghead%imgOffset, imghead%numCols, imghead%numRows, 1)
            i4 = ichar(pixels_buf_byte(1))
            if(i4 < 0) i4 = i4 + 256
            bkg = real(i4)
            do k=1,npix_col*npix_row
               i4 = ichar(pixels_buf_byte(k))
               if(i4 < 0) i4 = i4 + 256
               full(k) = real(i4) - bkg
            enddo
            err_code = 0
         endif
         
C        Process int*2 images
         if (imghead%mode == 1 .or. imghead%mode == 7) then
            call pif_read_short_image (imghead%luNum, pixels_buf_int2,
     $           imghead%imgOffset, imghead%numCols, imghead%numRows, 1)
            i4 = int(pixels_buf_int2(1))
C            if(i4 < 0) i4 = i4 + 256*256
            bkg = real(i4)
            do k=1,npix_col*npix_row
               i4 = int(pixels_buf_int2(k))
C               if(i4 < 0) i4 = i4 + 256*256
               full(k) = real(i4) - bkg
            enddo
            err_code = 0
         endif

C        Process int*4 images
         if (imghead%mode == 46) then
            call pif_read_boxi4(imghead, full)
            bkg = real(full(1))
            do k=1,npix_col*npix_row
               full(k) = full(k) - bkg
            enddo
            err_code = 0
         endif

         call pif_close(pix_file_unit)
         
      else if (index(tmpname,'.mrc') > 0) then
         
C        Get the file size
         call imopen(pix_file_unit, tmpname, 'ro')
         call qqinq(lstream(pix_file_unit), tmpname, fullname, nfilsz)
         call imclose(pix_file_unit)
         
C        Read header and determine if we need to account for symmetry bytes
         call imopen(pix_file_unit, tmpname, 'ro')
         call irdhdr(pix_file_unit, kxyz, mxyz, jmode, 
     $        dmin, dmax, dmean)

         npix_col_mrc = kxyz(1)
         npix_row_mrc = kxyz(2)
         npix_sec_mrc = kxyz(3)

C     Test for invalid particle image
         if(indx > npix_sec_mrc .or. indx <= 0) then
            return              
         endif
         
         if (jmode > 2) then
            write(*,*)'get_mrc_slice: not for use with transforms!'
            stop
         else if (jmode == 0) then
            nbytes = 1
         else if (jmode == 1) then
            nbytes = 2
         else if (jmode == 2) then
            nbytes = 4
         else
            write(*,*) "Error - gen_read_pixel_frame: "
            write(*,*) "MRC mode not implemented yet:", jmode
            stop
         end if

         img_size = npix_col_mrc*npix_row_mrc*npix_sec_mrc*nbytes + 1024
         if(img_size < nfilsz) then
            call irtsym(pix_file_unit, kspg, kbs)
            if(kbs == 0) then
               kbs = nfilsz-img_size
               kspg = 1
               skip_fl = 1
            end if
         end if
         call imclose(pix_file_unit)
         
C        Calculate number of bytes to skip
         skip = (indx-1)*npix_col_mrc*npix_row_mrc*nbytes
         if(skip_fl == 1) then
            skip = skip + kbs
         endif
         write(*,*) 'skip = ', skip

C        Read in appropriate slice from MRC file and load into pixels array
         call imopen(pix_file_unit, tmpname, 'ro')
         call irdhdr(pix_file_unit, kxyz, mxyz, jmode, 
     $        dmin, dmax, dmean)
         call qskip(lstream(pix_file_unit), skip)
         call irdsec(pix_file_unit, mrcslice, *99)
         call imclose(pix_file_unit)
      
         do j=1,npix_col*npix_row
            full(j) = 0.0
         enddo
         
         j = 0
         l = 0
         m = 0
         n = 0
         do i = 1,npix_col*npix_row
            j = j + 1
            if(j > npix_col) then
               j = 1
               l = l + 1
            end if
            if(j <= npix_col_mrc .and. l <= npix_row_mrc) then
               m = m + 1
               if(m > npix_col_mrc) then
                  m = 1
                  n = n + 1
                  if(n > npix_row_mrc) n = npix_row_mrc
               end if
               full(j+l*npix_col) = mrcslice(m+n*npix_col_mrc)
            end if
         end do

         bkg = full(1)
         do k=1,npix_col*npix_row
            full(k) = full(k) - bkg
         enddo

         err_code = 0
         go to 9999
 99      write(*,*)"Premature end of file on input"
         return

      endif

C     Now handle binning of data. Note that full resolution pixel frame
C     has already been background subtracted and that the dimensions of
C     the binned image are guaranteed to be odd.

 9999 if(binfactor == 2) then
         
C        Calculate "oddfactor"
         if(mod(npix_col/2 +1, 2) == 0) then
            oddfactor = 1
         else
            oddfactor = 0
         endif

         count = 0

         do j = 1+oddfactor, npix_row-oddfactor, 2
            do i = 1+oddfactor, npix_col-oddfactor, 2

               ip1 = i+1
               im1 = i-1
               jp1 = j+1
               jm1 = j-1

               if(ip1 > npix_col) ip1 = npix_col
               if(im1 < 1)        im1 = 1
               if(jp1 > npix_row) ip1 = npix_row
               if(jm1 < 1)        jm1 = 1
              
               tmp(1) = full(im1 + (jm1-1)*npix_col)
               tmp(2) = full(i   + (jm1-1)*npix_col)
               tmp(3) = full(ip1 + (jm1-1)*npix_col)
               tmp(4) = full(im1 + (j-1)*npix_col)
               tmp(5) = full(i   + (j-1)*npix_col)
               tmp(6) = full(ip1 + (j-1)*npix_col)
               tmp(7) = full(im1 + (jp1-1)*npix_col)
               tmp(8) = full(i   + (jp1-1)*npix_col)
               tmp(9) = full(ip1 + (jp1-1)*npix_col)

               if(i == 1) then
                  tmp(1) = 0.0
                  tmp(4) = 0.0
                  tmp(7) = 0.0
               endif
               if(i == npix_col) then
                  tmp(3) = 0.0
                  tmp(6) = 0.0
                  tmp(9) = 0.0
               endif
               if(j == 1) then
                  tmp(1) = 0.0
                  tmp(2) = 0.0
                  tmp(3) = 0.0
               endif
               if(j == npix_row) then
                  tmp(7) = 0.0
                  tmp(8) = 0.0
                  tmp(9) = 0.0
               endif

               count = count + 1

               pixels(count) = 0.0625*(tmp(1)+tmp(3)+tmp(7)+tmp(9)) +
     $              0.125*(tmp(2)+tmp(4)+tmp(6)+tmp(8)) +
     $              0.25*tmp(5)

            enddo
         enddo
      else
         do i = 1,npix_col*npix_row
            pixels(i) = full(i)
         enddo
      endif

      return
      end subroutine gen_read_pixel_frame
