      subroutine get_nview_mod (sym, dthe, dphi_90, nview_mod)

C     Calculates the number of distinct values of theta and the total
C     number of (theta, phi) orientations that will be used to sample the
C     asymmetric unit (ASU).
C
C     Arguments:
C     sym - symmetry group
C     dthe - step size for incrementing theta
C     dphi_90 - step size for incrementing phi at theta=90
C     nview_mod - number of points covering the ASU
C
C     Purdue University dec 3000, january 1994, D. Belnap
C     24-Dec-1997 by T. Baker (use nview_mod variable)
C     Misha and Rob dec 2003 - changed the line below to prevent
C       iterating outside the asu.
C       nphi = nint( (find_phimax(sym,theta)-phi1) / dphi )  +  1
C     19-Jul-2006 by R. Sinkovits, cleanup and commenting

      implicit none

      integer*4, intent(in)  ::  sym
      integer*4, intent(out) ::  nview_mod
      real*4, intent(in)     ::  dthe, dphi_90

      integer*4 i, nphi, nthe
      real*4 theta, the1, the2, phi1, dphi, find_phimax, tol
      real*8 deg_to_rad
      parameter (tol = 0.0001)

      deg_to_rad = acos(-1.0d0)/180.0d0

C     Get the upper and lower bounds for theta and the lower bound for
C     phi in the ASU for symmetry group sym
      call get_t1t2p1 (sym, the1, the2, phi1)

C     Calculate number of distinct thetas
      nthe      = nint((the2-the1)/dthe) + 1

C     Calculate the total number of views by summing over the number of
C     distinct values of phi at each theta. At theta equal to 0 or 180,
C     have only a single value of phi. Note that tests are done using
C     tolerances rather than floating point equalities.

      nview_mod = 0
      theta     = the1
      do i = 1,nthe
         if (abs(theta) < tol .or. abs(theta-180) < tol) then
            nphi = 1
         else
            dphi = real(dphi_90 / sin(theta*deg_to_rad))
            nphi =  int( (find_phimax(sym,theta)-phi1) / dphi )  +  1
         endif
         nview_mod = nview_mod + nphi
 
         if (sym == 532 .and. abs(theta-69.09484255211070) < tol) then
            theta = 69.0 + dthe
         else
            theta = theta + dthe
         endif
      enddo
      return
      
      end subroutine get_nview_mod
