      subroutine get_ptle_ioomctf(nparam, filename_len, param_fnames,
     $     oddeven, nptles, nbox, indx, theta, phi, omega, x, y, mag,
     $     delf_maj, delf_min, ang_maj)
      
C     Get indices origins, orientations, magnifications, and
C     per-particle ctf parameters (ioomctf) for multiple files
C
C     nparam         Number of parameter files
C     filename_len   Maximum number of characters in file name
C     param_fnames   Array of file names
C     oddeven        Flag indicating whether 'odd', 'even', or all images
C                    should be counted. Note that designation of even or
C                    odd is continuous across all files
C     nptles         Total number of particle images
C     nbox           Box size for particle images
C     indx           Array of particle indices used to number images
C                    in the boxed image files
C     theta          Theta orientation angles
C     phi            Phi orientation angles
C     omega          Omega orientation angles
C     x              x-coordinates of ptle origins
C     y              y-coordinates of ptle origins
C     mag            Magnifications
C     delf_maj       Major axis defocus for ptle
C     delf_min       Minor axis defocus for ptle
C     ang_maj        Astigmatism angle for ptle

      integer, intent(in) :: nparam, filename_len, oddeven, nptles
      integer, intent(in) :: nbox(nparam)
      character(len=filename_len), intent(in) :: param_fnames(nparam)

      integer, intent(out) :: indx(nptles)
      real, intent(out)    :: theta(nptles), phi(nptles), omega(nptles)
      real, intent(out)    :: x(nptles), y(nptles), mag(nptles)
      real, intent(out)    :: delf_maj(nptles), delf_min(nptles)
      real, intent(out)    :: ang_maj(nptles)

      integer i, indx_temp, ios, count, header_flag
      integer, save :: skipone = 1
      real theta_temp, phi_temp, omega_temp, x_temp, y_temp, mag_temp
      real cc1, cc2, cc3, delf_maj_temp, delf_min_temp, ang_maj_temp
      character(len=filename_len) ::tempstr

      count = 0
      do i=1,nparam
         open(unit=17, file=param_fnames(i))
         header_flag = 0

         do
            read(17,'(A)', iostat=ios) tempstr
            if(ios .ne. 0) exit ! break out of loop
            
            tempstr = adjustl(tempstr)
            if(tempstr(1:1) == ' ') cycle ! skip blank lines
            
            if(index(tempstr,'.pif') > 0 .or. 
     $           index(tempstr,'.mrc') > 0) then
               header_flag = 1
            else if(header_flag == 1) then
               header_flag = 0
            else
               read(tempstr,*,iostat=ios) indx_temp, theta_temp, 
     $              phi_temp, omega_temp, 
     $              x_temp, y_temp, mag_temp, cc1, cc2, cc3,
     $              delf_maj_temp, delf_min_temp, ang_maj_temp

               if(  (oddeven .ne. 1 .and. oddeven .ne. 2) .or.
     $              (oddeven == 1 .and. skipone == 1) .or.
     $              (oddeven == 2 .and. skipone == 0) ) then
                  count = count + 1
                  
                  if(x_temp <= 1)      x_temp = nbox(i)/2
                  if(y_temp <= 1)      y_temp = nbox(i)/2
                  if(mag_temp <= 0.01) mag_temp = 1.0
                  
                  indx(count)     = indx_temp
                  theta(count)    = theta_temp
                  phi(count)      = phi_temp
                  omega(count)    = omega_temp
                  x(count)        = x_temp
                  y(count)        = y_temp
                  mag(count)      = mag_temp
                  delf_maj(count) = delf_maj_temp
                  delf_min(count) = delf_min_temp
                  ang_maj(count)  = ang_maj_temp
               endif
               
               skipone = mod(skipone+1, 2) 
            endif
         enddo
         
         close(17)
      enddo
      
      return
      end subroutine get_ptle_ioomctf
