      subroutine get_t1t2p1 (sym, the1, the2, phi1)

C     Determines the range of allowed values of theta and the lower
C     bound on phi within the asymmetric unit for a given point group
C     symmetry. The upper bound on phi is NOT returned since for some
C     symmetries (e.g. 532) it is a function of theta.
C
C     sym - symmetry point group
C     the1 - lower bound on theta in the ASU
C     the2 - upper bound on theta    "
C     phi1 - lower bound on phi      "
C
C     Purdue University dec 3000, Jan 1994, T. Baker and D. Belnap       
C     added sym = 2-20, 30 Jan 1997, D. Belnap                           
C     for sym=1-20, the2=90 & phi1=-360/(2*sym), 13 May 1998, D. Belnap
C     21-Jul-2006, General cleanup R. Sinkovits

      implicit none

      integer*4, intent(in) :: sym
      real*4, intent(out)   :: the1, the2, phi1

      the2 = 90.0
      
      if (sym.ge.1 .and. sym.le.31) then
         the1 =  0.0
         phi1 = -360.0/(2*float(sym))
      else if (sym.eq.532) then
         the1 = 69.09484255211070
         phi1 =  0.0
      else
         the1 =  0.0
         phi1 =  0.0
      endif

      return
      end subroutine get_t1t2p1
