      SUBROUTINE imgcomport(fft2, hkl_max, fft1,
     +     disef, abs_ctf, rmin, rmax, retvalue, funcmode)
      
C     compare the two complex image with radius from rmin to rmax
C     disef is the weight which apply to the distance according 
C     to the radius and funcweight
C     funcmode = 0, Use CC method with real part & aimag part 
C     together as two real
C     funcmode = 1, Use distance method
C     funcmode = 2, Use amplitute method
C     funcmode = 3, Use phase method
      
      IMPLICIT NONE
      
      integer, intent(in) :: hkl_max, funcmode
      complex, intent(in) :: FFT1(0:hkl_max, -hkl_max:hkl_max)
      complex, intent(in) :: FFT2(0:hkl_max, -hkl_max:hkl_max)
      real,    intent(in) :: disef(0:hkl_max, -hkl_max:hkl_max)
      real,    intent(in) :: abs_ctf(0:hkl_max, -hkl_max:hkl_max)
      real, intent(in)    :: rmin, rmax
      real, intent(out)   :: retvalue

      integer rmaxi, i, j, count
      real dis, tt1, tt2, tmp, pi, twopi
      complex tt
      double precision rpart1, rpart2, ipart1, ipart2, rf1sum, rf2sum,
     $     rf1f2sum, rf1sqsum, rf2sqsum, dvalue

C     Used in latest funcmode tests
      double precision xy_term, xx_term, yy_term

      pi = 4.0 * atan(1.0)
      twopi = 2.0 * pi

      retvalue = 1.0E+37
      count = 0
      dvalue = 0.0
      rmaxi = min(hkl_max,int(rmax)+1)
      
      if (funcmode == 0) then  

C     funcmode = 0, Use CC method

         rf1f2sum =0.0
         rf1sqsum =0.0
         rf2sqsum =0.0
         rf1sum   =0.0
         rf2sum   =0.0
         Do j = -rmaxi, rmaxi
            tmp = j*j
            if ((tmp <= rmax**2) .and. (tmp >= rmin**2)) then
               count = count + 1
               rpart1 = real(FFT1(0,j)) * disef(0,j)
               rpart2 = real(FFT2(0,j)) * disef(0,j)
               ipart1 = aimag(FFT1(0,j)) * disef(0,j)
               ipart2 = aimag(FFT2(0,j)) * disef(0,j)
               
               rf1sum   = rf1sum + (rpart1 + ipart1)
               rf2sum   = rf2sum + (rpart2 + ipart2)
               rf1f2sum = rf1f2sum + (rpart1*rpart2+ipart1*ipart2)
               rf1sqsum = rf1sqsum + (rpart1*rpart1+ipart1*ipart1)
               rf2sqsum = rf2sqsum + (rpart2*rpart2+ipart2*ipart2)
            endif
            Do i = 1, rmaxi
               tmp = i*i+j*j
               if ((tmp <= rmax**2) .and. (tmp >= rmin**2)) then
                  count = count + 2
                  rpart1 = real(FFT1(i,j)) * disef(i,j)
                  rpart2 = real(FFT2(i,j)) * disef(i,j)
                  ipart1 = aimag(FFT1(i,j)) * disef(i,j)
                  ipart2 = aimag(FFT2(i,j)) * disef(i,j)
                  
                  rf1sum   = rf1sum + 2*(rpart1 + ipart1)
                  rf2sum   = rf2sum + 2*(rpart2 + ipart2)
                  rf1f2sum = rf1f2sum + 2*(rpart1*rpart2+ipart1*ipart2)
                  rf1sqsum = rf1sqsum + 2*(rpart1*rpart1+ipart1*ipart1)
                  rf2sqsum = rf2sqsum + 2*(rpart2*rpart2+ipart2*ipart2)
               endif
            enddo
         enddo
         
         if (count > 0) then
            tt1=real((rf1sqsum-(rf1sum**2)/count)*
     +           (rf2sqsum-(rf2sum**2)/count))
            tt2=real(abs(rf1f2sum-(rf1sum*rf2sum)/count))
            if (tt2 > 0) then
               retvalue = sqrt(abs(tt1))/tt2
            endif
            if (retvalue < 1 .and. retvalue > 0) retvalue = 1/retvalue
         endif
         
      elseif (funcmode == 1) then

C     funcmode = 1, Use distance method
C     distance: sqrt((A1-A2)**2 + (b1-b2)**2)
C     if try to reduce the computation time of  sqrt.
C     distance: (A1-A2)**2 + (b1-b2)**2
C     but from my test, it not good to resolution.

         Do j = -rmaxi, rmaxi
            tmp = j*j
            if ((tmp <= rmax**2) .and. (tmp >= rmin**2)) then
               count = count + 1
               tt = FFT1(0,j) - FFT2(0,j)
               dis = sqrt(real(tt)**2 + aimag(tt)**2)
               dvalue = dvalue + dis * disef(0,j) * 
     +              (abs_ctf(0,j)+0.01)
            endif
            Do i = 1, rmaxi
               tmp = i*i+j*j
                  if ((tmp <= rmax**2) .and. (tmp >= rmin**2)) then
                     count = count + 2
                     tt = FFT1(i,j) - FFT2(i,j)
                     dis = sqrt(real(tt)**2 + aimag(tt)**2)
                     dvalue = dvalue + 2 * dis * disef(i,j) * 
     +                (abs_ctf(i,j)+0.01)
                  endif
            enddo
         enddo

         if (count >= 1) retvalue = real(dvalue / count)
         
      elseif (funcmode == 2) then
         
C     funcmode = 2, Use amplitute method
C     only module: |sqrt(A1**2+b1**2) - sqrt(A2**2+b2**2)|
C     
         Do j = -rmaxi, rmaxi
            tmp = j*j
            if ((tmp <= rmax**2) .and. (tmp >= rmin**2)) then
               count = count + 1
               dvalue = dvalue + disef(0,j) * (abs_ctf(0,j)+0.01) *
     +              abs(sqrt(real(FFT1(0,j))**2+aimag(FFT1(0,j))**2)
     +              -sqrt(real(FFT2(0,j))**2+aimag(FFT2(0,j))**2))
            endif
            Do i = 1, rmaxi
               tmp = i*i+j*j
               if ((tmp <= rmax**2) .and. (tmp >= rmin**2)) then
                  count = count + 2
                  dvalue = dvalue + 2 * disef(i,j) * 
     +                 (abs_ctf(i,j)+0.01) *
     +                 abs(sqrt(real(FFT1(i,j))**2+aimag(FFT1(i,j))**2)
     +                 -sqrt(real(FFT2(i,j))**2+aimag(FFT2(i,j))**2))
               endif
            enddo
         enddo

         if (count > 0) retvalue = real(dvalue / count)

      elseif (funcmode == 3) then
         
C     funcmode = 3, Use phase method
C     only phase: |atan(b1/|A1|) - atan(b2/|A2|)|
         
         Do j = -rmaxi, rmaxi
            tmp = j*j
            if ((tmp <= rmax**2) .and. (tmp >= rmin**2)) then
               count = count + 1
               tt1 = 0.0
               if (real(FFT1(0,j)).eq.0.0 ) then
                  if (aimag(FFT1(0,j)).gt.0.0) tt1 = pi / 2
                  if (aimag(FFT1(0,j)).lt.0.0) tt1 = 3 * pi / 2
               else 
                  tt1 = atan2(aimag(FFT1(0,j)),real(FFT1(0,j)))
                  tt1 = AMOD(tt1, twopi)
                  if (tt1 < 0.0) tt1 = twopi + tt1
               endif
               
               tt2 = 0.0
               if (real(FFT2(0,j)).eq.0.0 ) then
                  if (aimag(FFT2(0,j)).gt.0.0) tt2 = pi / 2
                  if (aimag(FFT2(0,j)).lt.0.0) tt2 = 3 * pi / 2
               else 
                  tt2 = atan2(aimag(FFT2(0,j)),real(FFT2(0,j)))
                  tt2 = AMOD(tt2, twopi)
                  if (tt2 < 0.0) tt2 = twopi + tt2
               endif
               
               if (abs(tt1-tt2) > pi) then
                  dvalue = dvalue + disef(0,j)*
     +                 (twopi - abs(tt1-tt2))
               else
                  dvalue = dvalue + disef(0,j)*(abs(tt1-tt2))
               endif

            endif
            Do i = 1, rmaxi
               tmp = i*i+j*j
               if ((tmp <= rmax**2) .and. (tmp >= rmin**2)) then
                  count = count + 2
                  tt1 = 0.0
                  if (real(FFT1(i,j)) == 0.0 ) then
                     if (aimag(FFT1(i,j)) > 0.0) tt1 = pi / 2
                     if (aimag(FFT1(i,j)) < 0.0) tt1 = 3 * pi / 2
                  else 
                     tt1 = atan2(aimag(FFT1(i,j)),real(FFT1(i,j)))
                     tt1 = AMOD(tt1, twopi)
                     if (tt1 < 0.0) tt1 = twopi + tt1
                  endif

                  tt2 = 0.0
                  if (real(FFT2(i,j)) == 0.0 ) then
                     if (aimag(FFT2(i,j)) > 0.0) tt2 = pi / 2
                     if (aimag(FFT2(i,j)) < 0.0) tt2 = 3 * pi / 2
                  else 
                     tt2 = atan2(aimag(FFT2(i,j)),real(FFT2(i,j)))
                     tt2 = AMOD(tt2, twopi)
                     if (tt2 < 0.0) tt2 = twopi + tt2
                  endif

                  if (abs(tt1-tt2) > pi) then
                     dvalue = dvalue + 2 * disef(i,j)*
     +                    (twopi - abs(tt1-tt2))
                  else
                     dvalue = dvalue + 2 * disef(i,j)*(abs(tt1-tt2))
                  endif

               endif
            enddo
         enddo

         if (count > 0) retvalue = real(dvalue / count)

      elseif (funcmode == 4) then

C     First attempt at implementing correlation coefficient correctly.
C     The weighting function disef is not used. Note that since we're
C     looking for the orientation that minimizes the return value from
C     imgcomport, we'll return the inverse of the correlation
C     coefficient. In the rare event that the correlation coefficient is
C     identically zero, return a very large value.

C     The generalization of the CC for complex data is itself complex
C
C                Sum(Xi Yi*)
C     CC = -------------------------
C          Sqrt(SumXi^2)Sqrt(SumYi^2)

C     For the case though of FTs of real data, the imaginary components
C     will cancel out. At negative wavevector, the corresponding term in
C     the numerator will be Yi Xi*. Since F(-k) = F(k)* for FTs of real
C     data, we get (Xi Yi* + Xi* Yi)

         xy_term = 0.0
         xx_term = 0.0
         yy_term = 0.0

         do j = -rmaxi, rmaxi
            tmp = j*j
            if ((tmp <= rmax**2) .and. (tmp >= rmin**2)) then
               xx_term = xx_term + real(fft1(0,j))*real(fft1(0,j))
     +              + aimag(fft1(0,j))*aimag(fft1(0,j))
               yy_term = yy_term + real(fft2(0,j))*real(fft2(0,j))
     +              + aimag(fft2(0,j))*aimag(fft2(0,j))
               xy_term = xy_term + real(fft1(0,j))*real(fft2(0,j))
     +              + aimag(fft1(0,j))*aimag(fft2(0,j))
            endif
            do i = 1, rmaxi
               tmp = i*i + j*j
               if ((tmp <= rmax**2) .and. (tmp >= rmin**2)) then
                  xx_term = xx_term + 2*real(fft1(i,j))*real(fft1(i,j))
     +                 + aimag(fft1(i,j))*aimag(fft1(i,j))
                  yy_term = yy_term + 2*real(fft2(i,j))*real(fft2(i,j))
     +                 + aimag(fft2(i,j))*aimag(fft2(i,j))
                  xy_term = xy_term + 2*real(fft1(i,j))*real(fft2(i,j))
     +                 + aimag(fft1(i,j))*aimag(fft2(i,j))
               endif
            enddo
            retvalue = real(xy_term/sqrt(xx_term * yy_term))
            if(retvalue == 0.0) then
               retvalue = 1.0E6
            else
               retvalue = 1.0/retvalue + 100000.0
            endif
         enddo

      else
         call error_stop('POR','imgcomport',
     +        'Wrong funcmode, it must be 0,1,2,3,4!')
      endif

      RETURN
      END subroutine imgcomport
