      subroutine info(inunit)

      implicit none
      integer, intent(in) :: inunit

      include '../include/sizes.inc'
      include '../include/allprogs.inc'
      include '../include/info.inc'
      
      character(len=26)  :: upper_case, lower_case
      character(len=257) :: lineorig
      character(len=257) :: refine_mode_string
      character(len=4)   :: keyword
      character(len=4)   :: subname
      character(len=1)   :: tab
      
      integer char, ios, ic, i, j, loc
      
      real res_tmp
      
      logical have_ctfm, have_inpu, have_resm

      data subname / 'INFO' /
      data upper_case /'ABCDEFGHIJKLMNOPQRSTUVWXYZ'/
      data lower_case /'abcdefghijklmnopqrstuvwxyz'/

C     Set default values
      zero_fill       = 1.0
      num_symm        = 1
      symm_code       = 1
      num_den_pix     = 0
      mapmagfactor    = 1.0
      outmode         = 20
      res_max         = 0.1
      res_min         = 1.e10
      dangle          = 1.0
      nangle          = 2
      funcmode        = 0
      funcweight      = 0.0
      dcenter         = 0.0
      ncenter         = -1 
      oddeven         = 0
      quick_search    = 1
      apo_border      = 0
      per_ptle_ctf    = 0
      nmagf           = 0
      magfstep        = 0.0
      magfreset       = 0
      magfcalib       = 0
      handtest        = 1
      gangle          = 2.
      refine_mode     = PLOCAL

C     Set flags for essential parameters to false
      have_ctfm = .false.
      have_inpu = .false.
      have_resm = .false.
      
C     Set file names to blank
      do i = 1, filename_len
         out_file_name(i:i) = ' '
         fsc_file_name(i:i) = ' '
         in_3d_name(i:i)    = ' '
      enddo
      
C     Define tab - 9 is the ascii code for tab character
      tab = achar(9)

      write(*,*) '-- Listing of data from program input file --'
      
      do i = 1, 10000
         
C     Read line as character string. Exit loop if EOF reached
         read(inunit, '(A)', iostat=ios) lineorig
         if(ios < 0) exit ! Break out of loop

C     Convert all tabs to spaces
         do ic=1,len(lineorig)
            if(lineorig(ic:ic) == tab) then
               lineorig(ic:ic) = ' '
            endif
         enddo
         
C     Get rid of leading whitespace
         lineorig = adjustl(lineorig)

C     Go to next loop iteration if line is blank or comment
         if(lineorig(1:1) == ' ') cycle
         if(lineorig(1:1) == '#') cycle

C     Get rid of trailing comments
         loc = scan(lineorig,'#')
         if(loc > 0) then
            do j=loc,257
               lineorig(j:j) = ' '
            enddo
         endif

C     Set keyword
         keyword = lineorig(1:4)
         
C     Use f90 INDEX function to change keyword to upper case
         do ic = 1, 4
            char = index (lower_case, keyword(ic:ic))
            if (char .ne. 0) then
               keyword(ic:ic) = upper_case(char:char)
            endif
         enddo

C     Get rid of leading keyword from line
         do ic=1,len_trim(lineorig)
            if(lineorig(ic:ic) == ' ') then
               exit
            else
               lineorig(ic:ic) = ' '
            endif
         enddo
         
C     Process line based on value of keyword. Note that we are using a
C     slightly uncommon Fortran construct where a string is used in
C     place of the unit number in the read statement

         if(keyword == 'INPU') then
            have_inpu = .true.
            call pfile_info(inunit)
            exit
         endif

         if(keyword == 'FSCF') then
            fsc_file_name = adjustl(lineorig)
            if(len_trim(fsc_file_name) > filename_len) then
               call error_stop(progname, subname,
     $              'fsc_file_name too long')
            endif
            write(*,*) 'Keyword FSCF'
            write(*,*) '  fsc_file_name ', trim(adjustl(fsc_file_name))
         endif

         if(keyword == 'OUTF') then
            out_file_name = adjustl(lineorig)
            if(len_trim(out_file_name) > filename_len) then
               call error_stop(progname, subname,
     $              'out_file_name too long')
            endif
            write(*,*) 'Keyword OUTF'
            write(*,*) '  out_file_name ', trim(adjustl(out_file_name))
         endif
               
         if(keyword == 'IN3D') then
            in_3d_name = adjustl(lineorig)
            if(len_trim(in_3d_name) > filename_len) then
               call error_stop(progname, subname,
     $              'in_3d_name too long')
            endif
            write(*,*) 'Keyword IN3D'
            write(*,*) '  in_3d_name ',    trim(adjustl(in_3d_name)) 
         endif
            
         if(keyword == 'QUIC') then
            read(lineorig,*) quick_search
            write(*,*) 'Keyword QUIC'
            write(*,*) '  quick_search ',  quick_search
         endif
            
         if(keyword == 'MODE') then
            read(lineorig,*) refine_mode_string
            refine_mode_string = adjustl(refine_mode_string)
            write(*,*) 'Keyword MODE'
            select case (refine_mode_string(1:1))
            case ('l')
               refine_mode = PLOCAL
            case ('g')
               refine_mode = PGLOBAL
            case ('m')
               refine_mode = PMAG
            case ('t')
               refine_mode = PTICOS_EQUIV
            case default
               call error_stop(progname, subname,
     $              'unrecognized mode parameter')
            end select
            write(*,*) '  mode ',  refine_mode
         endif
            
         if(keyword == 'GLOB') then
            read(lineorig,*) gangle
            if(gangle <= 0) gangle = 2.0
            write(*,*) 'Keyword Glob_anglestep'
            write(*,*) '  gangle ',  gangle
         endif
            
         if(keyword == 'PER_') then
            read(lineorig,*) per_ptle_ctf
            write(*,*) 'Keyword PER_'
            write(*,*) '  per_ptle_ctf ',  per_ptle_ctf
         endif
                        
         if(keyword == 'APO_') then
            read(lineorig,*) apo_border
            write(*,*) 'Keyword APO_BORDER'
            write(*,*) '  apo_border ',  apo_border
         endif
            
         if(keyword == 'SYMM') then
            read(lineorig,*) symm_code
            call symmcode(symm_code, num_symm)
            write(*,*) 'Keyword SYMM'
            write(*,*) '  symm_code ',     symm_code
         endif

         if(keyword == 'OUTM') then
            read(lineorig,*) outmode
            if(outmode .ne. 10 .and. outmode .ne. 20 .and.
     $           outmode .ne. 30 .and. outmode .ne. 40) then
               call error_stop(progname,subname,
     +              'Invalid value provided for outmode')
            endif
            write(*,*) 'Keyword OUTM'
            write(*,*) '  outmode ',       outmode 
         endif

         if(keyword == 'RAD_') then
            read(lineorig,*) rad_proj_in
            if(rad_proj_in <= 0 .or. rad_proj_in > 1.0) then
               call error_stop(progname,subname,
     +              'Invalid value provided for rad_proj_in')
            endif
            write(*,*) 'Keyword RAD_'
            write(*,*) '  rad_proj_in ',   rad_proj_in
         endif

         if(keyword == 'OUTP') then
            read(lineorig,*) min_x, max_x, min_y, max_y,
     $           min_z, max_z
            write(*,*) 'Keyword OUTP'
            write(*,*) '  min_x ',         min_x
            write(*,*) '  max_x ',         max_x
            write(*,*) '  min_y ',         min_y
            write(*,*) '  max_y ',         max_y
            write(*,*) '  min_z ',         min_z
            write(*,*) '  max_z ',         max_z
         endif

         if(keyword == 'MAPS') then
            read(lineorig,*) mapmagfactor, num_den_pix
            if(mapmagfactor <= 0) then
               mapmagfactor = 1.0
            endif
            if(num_den_pix <= 0) then
               num_den_pix = 0
            endif
            if(num_den_pix > 0 .and. mod(num_den_pix,2) == 0) then
               call error_stop(progname, subname,
     $              'If num_den_pix > 0, then it must be odd')
            endif
            write(*,*) 'Keyword MAPS'
            write(*,*) '  mapmagfactor ',  mapmagfactor 
            write(*,*) '  num_den_pix ',   num_den_pix
         endif

         if(keyword == 'ODDE') then
            read(lineorig,*) oddeven
            if(oddeven < 0 .and. oddeven > 2) then
               call error_stop(progname, subname,
     $              'oddeven must be equal to 0, 1, or 2')
            endif
            write(*,*) 'Keyword ODDE'
            write(*,*) '  oddeven ',       oddeven 
         endif

         if(keyword == 'ZERO') then
            read(lineorig,*) zero_fill
            if(zero_fill < 1.0) then
               call error_stop(progname, subname,
     $              'zero_fill must be greater than or equal to 1')
            endif
            write(*,*) 'Keyword ZERO'
            write(*,*) '  zero_fill ',     zero_fill
         endif

         if(keyword == 'RESM') then
            have_resm = .true.
            read(lineorig,*) res_max, res_min
            if(res_min < res_max) then
               res_tmp = res_min
               res_min = res_max
               res_max = res_tmp
            endif
            if(res_max .le. 0) then
               call error_stop(progname, subname,
     $              'res_max must be positive')
            endif
            write(*,*) 'Keyword RESM'
            write(*,*) '  res_max ',       res_max
            write(*,*) '  res_min ',       res_min
         endif

         if(keyword == 'CTFM') then
            have_ctfm = .true.
            read(lineorig,*) ctfmode, filter, tempfac, 
     $           ctf_ff1, ctf_ff2
            if(ctfmode < 0 .or. ctfmode > 4) ctfmode = 0
            if(filter <= 0 .or. filter > 5) filter =1
            if(ctf_ff1 <= 0 .or. ctf_ff1 > 1.0) ctf_ff1 = 0.05
            if(ctf_ff2 <= 0 .or. ctf_ff2 > 1.0) ctf_ff2 = 0.1
            write(*,*) 'Keyword CTFM'
            write(*,*) '  ctfmode ',       ctfmode
            write(*,*) '  filter ',        filter
            write(*,*) '  tempfac ',       tempfac
            write(*,*) '  ctf_ff1 ',       ctf_ff1
            write(*,*) '  ctf_ff2 ',       ctf_ff2
         endif

         if(keyword == 'DANG') then
            read(lineorig,*) dangle, nangle, funcmode, funcweight
            if(dangle < 0) dangle = 1.0
            if(nangle < 0) nangle = 2
            if(funcmode < 0 .or. funcmode > 4) funcmode = 0
            if(funcweight < 0) funcweight = 0.0
            if(ncenter < 0) then
               ncenter = nangle
               dcenter = dangle
            endif
            write(*,*) 'Keyword DANG'
            write(*,*) '  dangle ',        dangle
            write(*,*) '  nangle ',        nangle
            write(*,*) '  funcmode ',      funcmode
            write(*,*) '  funcweight ',    funcweight
         endif

         if(keyword == 'DCEN') then
            read(lineorig,*) dcenter, ncenter
            if(dcenter < 0) dcenter = 1.0
            if(ncenter < 0) ncenter = 2
            write(*,*) 'Keyword DCEN'
            write(*,*) '  dcenter ',       dcenter
            write(*,*) '  ncenter ',       ncenter
         endif

         if(keyword == 'HAND') then
            read(lineorig,*) handtest
            if(handtest .ne. 0) then
               handtest = 1
            endif
            write(*,*) 'Keyword HANDTEST'
            write(*,*) '  handtest ',      handtest
         endif

         if(keyword == 'MAGR') then
            read(lineorig,*) nmagf, magfstep, magfreset, magfcalib
            if(nmagf < 0) then
               call error_stop(progname, subname,
     $              'nmagf must be non-negative')
            endif
            if(magfstep < 0) then
               call error_stop(progname, subname,
     $              'magref_step must be positive')
            endif
            if(magfreset < 0 .or. magfreset > 1) then
               call error_stop(progname, subname,
     $              'magref_reset must be either 0 or 1')
            endif
            if(magfcalib < 0 .or. magfcalib > 1) then
               call error_stop(progname, subname,
     $              'magref_calibrate must be either 0 or 1')
            endif
            write(*,*) 'Keyword MAGREF'
            write(*,*) '  nmagref      ',      nmagf
            write(*,*) '  magref_step  ',      magfstep
            write(*,*) '  magref_reset ',      magfreset
            write(*,*) '  magref_calibrate ',  magfcalib
         endif

      enddo
      
C     Make sure that all essential keywords have been found
      if(.not. (have_ctfm .and. have_inpu
     $   .and. have_resm)) then
         call error_stop(progname, subname,
     $        'Missing essential keywords CTFM, INPU, and RESM')
      endif
      
      return 
      end subroutine info
