      subroutine intlz_arrays(hkl_max, min_xyz, max_xyz, min_z, max_z,
     $     first_hk, last_hk, max_hk_slab, first_iq, last_iq,
     $     max_iq_slab, first_rr_out, last_rr_out, max_rr_out_slab,
     $     first_u, last_u, max_u_slab, first_y, last_y, max_y_slab,
     $     first_z, last_z, max_z_slab, npix_row)

C     Initializes the arrays which specify the partitioning of data
C     structures across multiple nodes. Also initialize the scalars
C     describing the local chunk of data assigned to a node. This latter
C     operation is simply a done as a convenience so that a node can
C     refer to its local chunk of data without requiring access to any
C     MPI variables.
C
C     For each partitioning, the following arrays and variables are set
C
C       first_xxx    - local starting index
C       last_xxx     - local ending index
C       max_xxx_slab - maximum number of indices assigned to any node
C       frst_lst_xxx - array of starting/ending indices on all nodes
C
C     where xxx is hk, iq, rr_out, u, y, or z

C     Author: Robert E. Lynch, 1997
C     Modified: Hong Lin, 7/6/1998 
C     Modified: Hong Lin, 8/18/1998 
C     Modified: Hong Lin, 10/7/1999 
C     Modified: Robert Lynch, 8/2000
C     Modified: Yongchang Ji, 8/2000 - 1/2002
C     Modified: Robert Sinkovits, 11/28/2006, cleanup and reorganization

      implicit none
      include '../include/allprogs.inc'

      integer, intent(in) :: hkl_max, min_xyz, max_xyz, 
     $     min_z, max_z, npix_row

      integer, intent(out) :: first_hk, last_hk, max_hk_slab, 
     $     first_iq, last_iq, max_iq_slab, 
     $     first_rr_out, last_rr_out, max_rr_out_slab, 
     $     first_u, last_u, max_u_slab, 
     $     first_y, last_y, max_y_slab, 
     $     first_z, last_z, max_z_slab

      integer node

      call set_indices(num_nodes, 0, hkl_max, 
     $     frst_lst_iq, max_iq_slab)

      call set_indices(num_nodes, 1, npix_row, 
     $     frst_lst_y, max_y_slab)

      call set_indices(num_nodes, min_xyz, max_xyz, 
     $     frst_lst_u, max_u_slab)

      call set_indices(num_nodes, min_z, max_z, 
     $     frst_lst_z, max_z_slab)

      max_rr_out_slab = 0
      do node = 0, num_nodes - 1
        frst_lst_rr_out(1,node) = 2 * frst_lst_iq(1,node)
        frst_lst_rr_out(2,node) = 2 * frst_lst_iq(2,node) + 1

        if (node == 0) then
           frst_lst_rr_out(1,0) = 1
        endif
        if (node == num_nodes - 1) then
           frst_lst_rr_out(2,node) = frst_lst_rr_out(2,node) -1
        endif

        max_rr_out_slab = max(max_rr_out_slab,
     +    -frst_lst_rr_out(1,node)+frst_lst_rr_out(2,node)+1)
      enddo

      do node = 0, num_nodes-1
        frst_lst_hk(1,node) = frst_lst_rr_out(1,node) 
        frst_lst_hk(2,node) = frst_lst_rr_out(2,node)
      enddo
      max_hk_slab = max_rr_out_slab

      first_hk       = frst_lst_hk(1,iam)
      last_hk        = frst_lst_hk(2,iam)
      first_iq       = frst_lst_iq(1,iam)
      last_iq        = frst_lst_iq(2,iam)
      first_rr_out   = frst_lst_rr_out(1,iam)
      last_rr_out    = frst_lst_rr_out(2,iam)

      first_u        = frst_lst_u(1,iam)
      last_u         = frst_lst_u(2,iam)
      first_y        = frst_lst_y(1,iam)
      last_y         = frst_lst_y(2,iam)
      first_z        = frst_lst_z(1,iam)
      last_z         = frst_lst_z(2,iam)
 
      return
      end subroutine intlz_arrays
