      subroutine map_polar (map1, ncol, nrow, map2, nang, nannuli, 
     1     xcen, ycen, rad_min, rad_max, del_ang)
      
C     Construct polar map from cartesian map
C
C     Input:
C       map1             - Cartesian map
C       ncol, nrow       - dimensions of Cartesian map
C       nang             - number of angular steps
C       nannuli          - number of annuli
C       xcen, ycen       - center of map
C       rad_min, rad_max - minimum and maximum annulus radii
C       del_ang          - angle for rotation matrix
C     
C     Output:
C       map2 - polar map

      implicit none
      
      integer*4, intent(in) :: nrow, ncol, nang, nannuli
      real*4, intent(in)    :: xcen, ycen, rad_min, rad_max, del_ang
      real*4, intent(in)    :: map1(ncol,nrow)
      real*4, intent(out)   :: map2(nang,nannuli)

      integer*4 ix, iy, ir, ia
      integer*4 alloc_stat
      real*4    a11, a12, a21, a22, d1, d2, d3, d4
      real*4    dx, dy, xhi, yhi, x, y, dr, rstep
      real*4, allocatable :: xgrid(:), ygrid(:)
      real*8 deg_to_rad
      
      deg_to_rad = acos(-1.0d0)/180.0d0
      
      allocate(xgrid(nang), ygrid(nang), stat = alloc_stat)
      if(alloc_stat .gt. 0) then
         write(*,*) 'memory allocation failed in map_polar'
         stop
      endif
      
      map2 = 0.0 !f90 array syntax
      
      xhi  = ncol - 1
      yhi  = nrow - 1
      
c     Set up rotation matrix
      a11 = real(cos(del_ang*deg_to_rad))
      a12 = real(sin(del_ang*deg_to_rad))
      a21 = -a12
      a22 =  a11
      
      y     = ycen
      rstep = 1.0
      if(nannuli.gt.1) rstep = (rad_max-rad_min)/float(nannuli-1)
      
      do ir = 1,nannuli
         dr = (ir - 1)*rstep + rad_min
         x  = xcen + dr
         call map_grid (x, y, nang, xcen, ycen, xgrid, ygrid,
     $        a11, a12, a21, a22)
         do ia = 1,nang
            if(  xgrid(ia) < 1.0 .or. xgrid(ia) > xhi .or.
     $           ygrid(ia) < 1.0 .or. ygrid(ia) > yhi) then
               map2(ia,ir) = 0.0
            else
               ix = int(xgrid(ia))
               iy = int(ygrid(ia))
               d1   = map1(ix  ,iy  )
               d2   = map1(ix  ,iy+1)
               d3   = map1(ix+1,iy+1)
               d4   = map1(ix+1,iy  )
               dx   = xgrid(ia) - ix
               dy   = ygrid(ia) - iy
               map2(ia,ir) = d1 - dx*(d1 - d4) - dy*(d1 - d2) +
     1              dx*dy*(d1 - d2 + d3 - d4)
            endif
         enddo
      enddo
      
      deallocate(xgrid, ygrid)
      
      return
      end subroutine map_polar
