      subroutine maptempfac(fft, lb_x, ub_x, lb_y, ub_y, lb_z, ub_z,
     $     tempfac, pixsize, units)

C     Apply temperature factor to an array containing discrete Fourier
C     transform. This routine has been generalized to handle arbitrarily
C     sized arrays of up to three dimensions, but uses the upper bound
C     of the first dimension to determine the conversion factor between
C     index space and spatial frequency.
      
      implicit none
      
      integer, intent(in) :: units, lb_x, ub_x, lb_y, ub_y, lb_z, ub_z
      real, intent(in) :: tempfac, pixsize
      complex, intent(inout) :: fft(lb_x:ub_x, lb_y:ub_y, lb_z:ub_z)
      
      integer ix, iy, iz
      real    d_star2, sumsq, t, tmp, tmp2
      
      if(units == 2) then
         tmp =  1.0/((ub_x*2+1)*pixsize) ! already in nm units
      else
         tmp = 10.0/((ub_x*2+1)*pixsize) ! convert to nm units
      endif
      
      tmp2 = tmp*tmp

      do iz = lb_z, ub_z
         do iy = lb_y, ub_y
            do ix = lb_x, ub_x

               sumsq = real(ix*ix + iy*iy + iz*iz)
               d_star2 = sumsq * tmp2
               t = exp ((tempfac/400.0) * d_star2)
               
               fft(ix,iy,iz) = fft(ix,iy,iz) * t

            enddo
         enddo
      enddo
      
      return
      end subroutine maptempfac
