      subroutine matrix_to_crowther(theta, phi, omega, rot)
C
C     Returns the Crowther angles (theta, phi, omega) corresponding to a
C     rotation matrix.
C          
C     Peter's derivation is based on crowther matrix as:
C
C     so*ct*cp - co*sp       -so*ct*sp - co*cp          -so*st
C     co*ct*cp + so*sp       -co*ct*sp + so*cp          -co*st
C     st*cp                  -st*sp                      ct 
C
C     Author: Steve Fuller 10/29/89
C
C     Modified: Robert Sinkovits 6/27/06 - general cleanup and
C     replacement of nonstandard acosd()/atan2d() functions with Fortran
C     90 acos()/atan2()

      implicit none

      real*4, intent(inout)  :: rot(3,3)
      real*4, intent(out) :: theta, phi, omega

      real*4 omega_minus_phi, omega_plus_phi
      real*4 toler/1.0e-6/
      real*8 deg_to_rad, rad_to_deg

      deg_to_rad = acos(-1.0d0)/180.0d0
      rad_to_deg = 1.0d0/deg_to_rad

C     Fix rotation matrix if not accurate by this method - bug fix
      if(abs(rot(3,3))+toler >= 1.0) then
          rot(3,3)=sign(1.0, rot(3,3))
      endif                                             
            
      theta = real(rad_to_deg*acos(rot(3,3)))

C     If theta is very close to zero, then there is flexibility in the
C     assignment of the angles omega and phi. In this case, arbitrarily
C     set omega to zero. Note also that all minus signs in atan2() are
C     required.

      if(sin(theta*deg_to_rad) > toler) then
         omega = real(rad_to_deg * atan2(-rot(1,3),-rot(2,3)))
         phi =   real(rad_to_deg * atan2(-rot(3,2),rot(3,1)))
      else  
         omega=0.0
         if(cos(theta*deg_to_rad) == 1.0) then
            omega_minus_phi = 
     C        real(rad_to_deg * atan2(rot(2,2),rot(2,1)))
            phi = omega - omega_minus_phi
         else
            omega_plus_phi= 
     C        real(rad_to_deg * atan2(rot(2,2),-rot(2,1)))
            phi=omega_plus_phi
         endif
      endif            

      return
      end subroutine matrix_to_crowther
