      subroutine polar_to_cosines(theta, phi, x)
C
C     Convert (theta, phi, omega), given in degrees, to direction
C     cosines. A tolerance is set to try to alleviate round off problems
C     and the total length of the direction cosine vector is normalized
C     to 1.0
C
C     Author: Steve Fuller 1/10/88
C
C     Modified: Robert Sinkovits 6/28/06 - general cleanup and bringing
C     in line with Fortran 90 standard

      implicit none

      real*4, intent(in)  :: theta, phi
      real*4, intent(out) :: x(3)

      real*4 toler/1.0e-8/
      real*8 deg_to_rad
      integer i

      deg_to_rad = acos(-1.0d0)/180.0d0

      x(3) = real(cos(theta*deg_to_rad))

      if(abs(x(3)-1.0) < toler) then
          x(2) = real(sin(phi*deg_to_rad))
          x(1) = real(sin(phi*deg_to_rad))
      else
          x(2) = real(sin(theta*deg_to_rad) * sin(phi*deg_to_rad))
          x(1) = real(sin(theta*deg_to_rad) * cos(phi*deg_to_rad))
      end if

      do i = 1,3
         if(abs(x(i)) < toler) x(i)=0.0
      end do

      call norm(x)

      return
      end subroutine polar_to_cosines
