      subroutine return_to_unit_triangle(theta, phi, omega)

C     Checks whether or not the input orientation (theta, phi, omega)
C     lies within the icosahedral ASU defined by the triangle formed
C     from two adjacent points of fivefold symmetry (90, +/- 31.7) and
C     the nearest three-fold axis (69.0, 0).
C
C     For orientations outside of the ASU, the equivalent point within
C     the ASU is determined by individually applying the 60 rotation
C     matrices.
C
C     NOTE - all angles are in degrees
C      
C     Author: Steve Fuller 12/4/85
C     
C     MRC conversion sdf - 17-jan-87
C     Revision for larger area of return sdf 28-jan-87
C     Modified 1-feb-87 sdf
C     Completely re-written - previous version was wrong sdf 12-jan88
C     
C     Modified: Robert Sinkovits 6/28/06 - general cleanup and bringing
C     in line with Fortran 90 standard

      implicit none

      real*4, intent(inout) :: theta, phi, omega

      integer imat
      logical good
      real*4 dcos(3), x(3), thetar, phir, omegar, icos_rot(3,3,60)

C     Initialize icosahedral rotation matricies
      call genrot(icos_rot)

C     Test the current orientation
      call polar_to_cosines(theta, phi, dcos)
      if(good(dcos)) return
      
C     Test the 60 equivalent orientations
      do imat = 1, 60
         call equivalent_view(theta, phi, omega,
     $        icos_rot(1,1,imat), thetar, phir, omegar)
         call polar_to_cosines(thetar, phir, x)
         if(good(x)) then
            theta = thetar
            phi   = phir
            omega = omegar
            return
         end if
      end do
      
      write(6,'('' return_to_unit_triangle '')')
      write(6,'('' no equivalent found '')')
      write(6,'('' input values '')')
      write(6,'('' theta '',f10.3)') theta
      write(6,'('' phi '',f10.3)') phi
      write(6,'('' omega '',f10.3)') omega
      write(6,'('' last matrix tried '',i2)') imat
      
      return
      end subroutine return_to_unit_triangle
