      subroutine set_indices(num_nodes, min_index, max_index,
     $     bounds, max_chunk)

C     Determine the decomposition of a chunk of data with starting and
C     ending indices min_index and max_index, respectively, across
C     a set of nodes. Assign work as evenly as possible, but distribute
C     extra pieces starting with node 1 (skipping node 0). Lower and
C     upper bounds for the node stored in bounds(1,node), bounds(2,node)
C
C     Also returns the size of the largest chunk of data.
C
C     For nodes which get a zero sized chunk of work, set the starting
C     and ending indices to 0 and -1, respectively.
C
C     Example: num_nodes = 5, min_index = 1, max_index = 12
C
C     node   bounds(1,node)   bounds(2,node)
C     0       1                 2
C     1       3                 5
C     2       6                 8
C     3       9                10
C     4      11                12

      implicit none
      
      integer, intent(in)  :: num_nodes, min_index, max_index
      integer, intent(out) :: bounds(2,0:num_nodes-1)
      integer, intent(out) :: max_chunk

      integer i, quotient, remainder, count(0:num_nodes-1), total

      total = (max_index - min_index + 1)
      quotient = total/num_nodes
      remainder = total - quotient*num_nodes

C     Give each node an even chunk of work
      do i=0,num_nodes-1
         count(i) = quotient
      enddo

C     Distribute the remainder evenly, starting at node 1
      do i=1,remainder
         count(i) = count(i) + 1
      enddo

C     Calculate the range of indices for each node
      bounds(1,0) = min_index
      bounds(2,0) = min_index + count(0) - 1
      do i=1,num_nodes-1
         bounds(1,i) = bounds(2,i-1) + 1
         bounds(2,i) = bounds(1,i) + count(i) - 1
      enddo

C     If the chunk size assigned to a node is zero, set the lower
C     bound to zero and upper bound to -1
      do i=0,num_nodes-1
         if(count(i) == 0) then
            bounds(1,i) = 0
            bounds(2,i) = -1
         endif
      enddo

C     Determine the size of the largest chunk of work
      max_chunk = maxval(count)

      return
      end subroutine set_indices
