      subroutine shift_fft(fft, nrow, ncol, dx, dy)

C     Calculate the FFT of the image shifted in real space by
C     (dx,dy). This is equivalent to multiplying the elements
C     of the original FFT by exp[2*pi*i (h*dx/Tx * k*dy/Ty)], where
C     Tx and Ty are the periods of the FFT in the h and k directions.

      implicit none

      integer*4, intent(in)    :: nrow, ncol
      real*4, intent(in)       :: dx, dy 
      complex, intent(inout)   :: fft(0:nrow, -ncol:ncol)

      integer h, k
      real twopi, f
      complex hfactor(0:nrow), kfactor(-ncol:ncol)

      twopi = 2.0*acos(-1.0)

C     Precompute shifts in the h direction
      do h = 0, nrow
         f = twopi*h*dx/(2.0*nrow)
         hfactor(h) = cmplx(cos(f), sin(f))
      enddo

C     Precompute shifts in the k direction
      do k = 0, ncol
         f = twopi*k*dy/(2.0*ncol)
         kfactor(k) = cmplx(cos(f), sin(f))
         kfactor(-k) = conjg(kfactor(k))
      enddo

C     Apply shifts to FFT
      do k = -ncol, ncol
         do h = 0, nrow
            fft(h,k) = fft(h,k) * hfactor(h)*kfactor(k)
         enddo
      enddo

C     Impose periodic boundary conditions
C     Note (8/25/06) Make sure that this is done correctly
      do h = 0, nrow
         fft(h,ncol)  = 0.5 * (fft(h,ncol) + fft(h,-ncol))
         fft(h,-ncol) = fft(h,ncol)
      enddo

      do k= 0, ncol
         fft(nrow,k) = 0.5 * (fft(nrow,k) + conjg(fft(nrow,-k)))
         fft(nrow,-k) = conjg(fft(nrow,k))
      enddo

      return
      end subroutine shift_fft
