      subroutine symmcode(symm_code, num_symm)

C     Author: Hong Lin (linh@cs.purdue.edu)                             
C     
C     Purpose: Returns the number of matrices used to replicate an
C     object with a given symmetry code
C
C     Input:  symm_code (Integer)                                       
C     Output: num_symm  (Integer)                                       
C     
C     Modification history:                                             
C     - Creation, Nov.22, 1999, Hong Lin.                             
C     - R.E.Lynch, Aug 1, 2000                                        
C       Changed name to symmcode from calculate_symmetry              
C       Added data subname                                            
C       Changed arguments in call of error_stop                       
C     - Yongchang Ji, Final Version, Aug.2000 -- Jan. 2002            
C     - R Sinkovits, cleanup and modernization 6/30/2006

      implicit none
      include '../include/allprogs.inc'

      integer, intent(in)  :: symm_code
      integer, intent(out) :: num_symm

      character(len=8) :: subname
      data subname / 'symmcode' /

      if (symm_code <= 8) then
         num_symm = symm_code
      elseif (symm_code == 532) then
         num_symm = 60
      elseif (symm_code == 222) then
         num_symm = 4
      elseif (symm_code == 32) then
         num_symm = 6
      elseif (symm_code == 52) then
         num_symm = 10
      elseif (symm_code == 622) then
         num_symm = 12
      elseif (symm_code == 72) then
         num_symm = 14
      elseif (symm_code == 822) then
         num_symm = 16
      else
         write(*,1000) symm_code
         call error_stop(progname,subname, 
     1        'Error in symmetry code!')
      endif

 1000 format(//' Error in value of symmetry code SYMM = ',i10/
     +     ' Acceptible values:',
     +     '    less-equal 8, 532, 222, 32, 52, 622, 72, and 822')
      
      return 
      end subroutine symmcode
