      subroutine to_asym_unit (symmetry, theta, phi, omega)
C
C     Calls routines to check if orientations are in the asymmetric
C     unit. If not, these routines calculate the equivalent view within
C     the asymmetric unit. For omega < 0, add 360 degrees; for phi > 360,
C     subtract 360 degrees.
C
C     NOTE - all angles are in degrees
C
C     Purdue University dec 3000, jan,jun 1994, D. Belnap
C     Added 222, 522, 622, & 722 symmetry, dec 1996, jan 1997, D. Belnap
C     Modified 22-jun-1998 by TSB (updated dihedral symmetries)
C     Modified 29-jun-1999 by D. Belnap (updated dihedral symmetries)
C     Modified 6/28/06 by R. Sinkovits - general cleanup

      implicit none

      integer*4, intent(in) :: symmetry
      real*4, intent(inout)   :: theta, phi, omega

      if (symmetry >= 2 .and. symmetry <= 31) then ! n-fold symmetries
         call to_nfold (theta, phi, omega, symmetry)
      else if (symmetry == 532) then
         call return_to_unit_triangle (theta, phi, omega)
      else if (symmetry == 222) then
         call to_n22 (theta, phi, omega, 2)
      else if (symmetry == 32) then
         call to_n22 (theta, phi, omega, 3)
      else if (symmetry == 422) then
         call to_n22 (theta, phi, omega, 4)
      else if (symmetry == 52) then
         call to_n22 (theta, phi, omega, 5)
      else if (symmetry == 622) then
         call to_n22 (theta, phi, omega, 6)
      else if (symmetry == 72) then
         call to_n22 (theta, phi, omega, 7)
      else if (symmetry == 822) then
         call to_n22 (theta, phi, omega, 8)
      else if (symmetry == 92) then
         call to_n22 (theta, phi, omega, 9)
      endif
      
      return
      end subroutine to_asym_unit
