      SUBROUTINE TRANS_LENGTH_ESSL(N_WANT, N_GET )

C***PURPOSE  Find the length, N_GET, of a DFT which is .GE. N_WANT for
C            which the ESSL DFT routines can compute a DFT
C***AUTHOR   Robert E. Lynch, Purdue Univ., West Lafayette, IN, 2000
C***DESCRIPTION

C *Usage:
C     INTEGER N_WANT, N_GET
C
C     CALL TRANS_LENGTH_ESSL( N_WANT, N_GET )

C *Arguments:
C    On Entry:
C        INTEGER N_WANT:  The desired length of a (1D) DFT transform
C
C    On Return:
C        INTEGER N_GET:   Length of a (1D) DFT transform which ESSL can
C                         compute. N_WANT is equal to
C                            (2**I2)*(3**I3)*(5**I5)*(7**I7)*(11**I11)
C                         with 1 <= I2 <=25,  0 <= I3 <= 2,
C                              0 <= I5,I7,I11 <= 1
C                         If N_WANT has this form, then N_GET is set
C                         equal to N_WANT.  Otherwise, N_GET is set
C                         equal to the next larger integer which does
C                         have this form.
C *Errors:
C    If N_WANT is not between  2  and  4620, which are the first and
C    last entries in the array NTABLE, then a message is printed,
C    ERROR_STOP is called, and execution terminates.  ESSL can
C    compute transforms with lengths up to 36700160.  One can increase
C    the number of entries in NTABLE to include a larger set of values
C    (see p. 742 of the 1997 edition of the ESSL manual); one could
C    also have ESSL compute an acceptible value provided one makes
C    certain error `recoverable' (see the manual).
C 
C *Method
C    Binary search in NTABLE
C 
C***ROUTINES CALLED (NONE; except, possibly, ERROR_STOP)
C***INCLUDE FILES   allprogs.inc
C***REVISION HISTORY (YYMMDD)
C   201201 Date initially written
C *   - Yongchang Ji, Final Version, Aug.2000 -- Jan. 2002             *
c *   - Robert Sinkovits, general cleanup 11/14/2006

      implicit none
      include '../include/allprogs.inc'

      integer, intent(in)  :: n_want
      integer, intent(out) :: n_get

      integer n, n_low, n_high, v, v_low, v_high, v_want, kount
      integer numb_trans_lengths, j
      parameter ( numb_trans_lengths =  140 )
      integer ntable( numb_trans_lengths )

c     ntable contains the first numb_trans_lengths transform lengths
c     which essl can use as a (1d) dft length.
      data (ntable(j),j=1,numb_trans_lengths)/
     +     2,    4,    6,    8,   10,   12,   14,   16,   18,   20,
     +    22,   24,   28,   30,   32,   36,   40,   42,   44,   48,
     +    56,   60,   64,   66,   70,   72,   80,   84,   88,   90,
     +    96,  110,  112,  120,  126,  128,  132,  140,  144,  154,
     +   160,  168,  176,  180,  192,  198,  210,  220,  224,  240,
     +   252,  256,  264,  280,  288,  308,  320,  330,  336,  352,
     +   360,  384,  396,  420,  440,  448,  462,  480,  504,  512,
     +   528,  560,  576,  616,  630,  640,  660,  672,  704,  720, 
     +   768,  770,  792,  840,  880,  896,  924,  960,  990, 1008,
     +  1024, 1056, 1120, 1152, 1232, 1260, 1280, 1320, 1344, 1386,
     +  1408, 1440, 1536, 1540, 1584, 1680, 1760, 1792, 1848, 1920,
     +  1980, 2016, 2048, 2112, 2240, 2304, 2310, 2464, 2520, 2560,
     +  2640, 2688, 2772, 2816, 2880, 3072, 3080, 3168, 3360, 3520,
     +  3584, 3696, 3840, 3960, 4032, 4096, 4224, 4480, 4608, 4620 /

      character(len=16) :: subname
      data subname / 'set_trans_length' /
 
c     use bisection method to find entry in ntable
      v_want = n_want
      n_low  = 1
      v_low  = ntable(n_low)
      n_high = numb_trans_lengths
      v_high = ntable(n_high)
      if( ( n_want < v_low ) .or. ( v_high < n_want ) ) then
        print 1030, n_want, v_low, v_high, v_high
 1030   format(' requested transform length = ',i5/
     +         ' acceptible values are between ', i5,' and ',i5/
     +         ' if values larger than ',i5,' are needed, then ',
     +         ' augment the array ntable')
        call exit(1)
      endif
 
      do kount = 1, numb_trans_lengths
        n = int( ( n_low + n_high ) / 2 )
        v = ntable(n)

        if( v == v_want ) then
          n_get = v
          exit ! fortran 90 statment to exit loop
        elseif( v < v_want ) then
          n_low = n
        elseif( v_want < v ) then
          n_high = n
        endif

        if( n_high - n_low == 1 ) then
          if( ntable(n_low) == v_want ) then
            n_get = v_want
          else
            n_get = ntable(n_high)
          endif
          exit ! fortran 90 statment to exit loop
        endif

      enddo

      return
      end subroutine trans_length_essl
