      subroutine exch_3d_1(F_TMP, hkl_max, FFT_3D, npix_row)

C     Send variable amount of data from all nodes in array f_tmp and
C     receive in array fft_3d. f_tmp contains slabs of size 
C     (0:hkl_max, -hkl_max:hkl_max, frst_lst_y(1,iam):frst_lst_y(2,iam)
C
C     Following pattern of communications used
C
C =====================================================================
C                                 p0      p1
C we exchange data ----    to  ----    -----
C                  |a1|        |a1|    | b1|
C                  |  |        |  |    |   |  
C                  ----        ----    -----
C                  | b1|       |a2|    | b2|
C                  |   |       |  |    |   |
C                  -----       |  |    |   |
C                  |c1|        ----    -----
C                  |  |        |a3|    | b3|
C                  ----        |  |    |   | 
C                              ----    -----

      implicit none

#ifdef MPI
      include 'mpif.h'
#endif

      include '../include/allprogs.inc'
 
      integer, intent(in) :: hkl_max, npix_row
      complex, intent(in) :: F_TMP(1) 

      complex, intent(out) :: FFT_3D(
     +     frst_lst_iq(1,iam):frst_lst_iq(2,iam), 
     +     -hkl_max:hkl_max, npix_row)

#ifdef MPI

      integer node, send_offset, recv_offset, send_factor, recv_factor

      if(num_nodes > 1) then
         send_offset = 0
         recv_offset = 0
         
         send_factor = ( 2*hkl_max + 1)
     +        * ( frst_lst_y(2,iam) - frst_lst_y(1,iam) + 1)
         recv_factor = ( 2*hkl_max + 1)
     +        * ( frst_lst_iq(2,iam) - frst_lst_iq(1,iam) + 1)
         
         do node = 0, num_nodes-1
            scounts(node) = send_factor
     +           * (frst_lst_iq(2,node)- frst_lst_iq(1,node) + 1)
            sdispls(node) = send_offset
            send_offset   = send_offset + scounts(node)
            
            rcounts(node) = recv_factor
     +           * (frst_lst_y(2,node) - frst_lst_y(1,node) + 1)
            rdispls(node) = recv_offset
            recv_offset   = recv_offset + rcounts(node)
         enddo
         
C     Send data from F_TMP; receive data in FFT_3D
         call mpi_alltoallv (F_TMP, scounts, sdispls,
     +        mpi_complex, FFT_3D, rcounts, rdispls, mpi_complex,
     +        mpi_comm_world, mpi_err)
         
         if(mpi_err .ne. 0) then
            call error_stop(progname, 'exch_3d_1',
     +           ' Error in alltoall communication !')
         endif
      endif

#else
      FFT_3D = cmplx(0,0)
      write(*,*) "WARNING[exch_3d_1]: nothing done in serial mode!" 

#endif
      
      return
      end subroutine exch_3d_1
