      subroutine gather3d(fft_klh, hkl_max, fft3d, fft_ann)
      
C     Collect the contributions (fft_hkl) from each process to construct
C     the full array (fft3d), which is then sent to all processes. The
C     gathering and distribution of data is accomplished with a call to
C     the single MPI routine mpi_allgatherv.

      implicit none
      
#ifdef MPI
      include 'mpif.h'
#endif

      include '../include/allprogs.inc'

      integer, intent(in) :: hkl_max, fft_ann
      complex, intent(in) :: fft_klh(-hkl_max:hkl_max, 
     +     -hkl_max:hkl_max, frst_lst_iq(1,iam):frst_lst_iq(2,iam))
      complex, intent(out) :: fft3d(-fft_ann:fft_ann, 
     +     -fft_ann:fft_ann, 0:fft_ann)
      
      integer num_ann, i, j, k
      character(len=10) :: subname = 'Gather3d'

#ifdef MPI
      integer alloc_stat, disp(0:num_nodes), sendcount(0:num_nodes)
      complex, allocatable :: tmp(:,:,:)
#endif

#ifdef MPI
      allocate(tmp(-fft_ann:fft_ann, -fft_ann:fft_ann, 
     +     frst_lst_iq(1,iam):frst_lst_iq(2,iam)),
     +     stat = alloc_stat)
      
      if(alloc_stat > 0) then
         call error_stop (progname, subname,  
     +        ' Memory allocation interpolants failed')
      endif 
      
      do i = frst_lst_iq(1,iam), frst_lst_iq(2,iam) 
         do j = -fft_ann, fft_ann  
            do k = -fft_ann, fft_ann  
               tmp(k,j,i)=fft_klh(k,j,i)
            enddo
         enddo
      enddo
      
      num_ann = 2 * fft_ann + 1
      do i = 0, num_nodes-1
         if (frst_lst_iq(2,i) <= fft_ann) then
            sendcount(i)=(frst_lst_iq(2,i)-frst_lst_iq(1,i)+1)
     +           * num_ann * num_ann
         elseif (frst_lst_iq(1,i)<= fft_ann) then
            sendcount(i)=(fft_ann-frst_lst_iq(1,i)+1)* num_ann * num_ann
         else 
            sendcount(i)=0
         endif
         
         if (i.eq.0) then
            disp(0) = 0
         else
            disp(i) = disp(i-1) + sendcount(i-1)
         endif
      enddo
      
      call mpi_allgatherv(tmp,sendcount(iam),mpi_complex,fft3d,
     +     sendcount,disp,mpi_complex,mpi_comm_world,mpi_err)
      if( mpi_err .ne. 0 ) call error_stop(progname,subname,
     +     'Error in calling MPI_allgatherv fft3d')
      
      deallocate(tmp)
#else
      do k = 0, fft_ann
         do j = -fft_ann, fft_ann
            do i = -fft_ann, fft_ann
               fft3d(i,j,k) = fft_klh(i,j,k)
            enddo
         enddo
      enddo
#endif
      
      return
      end subroutine gather3d
