      subroutine read_map(map_file, map3D, npix_col, npix_row, 
     $     max_slab, mapprjbig)
      
      implicit none
      
#ifdef MPI
      include 'mpif.h'
#endif

      include '../include/allprogs.inc'
      INCLUDE '../include/pifhead.inc'
      
      type (PIFHEAD_T) MAPHEAD
      
      integer, intent(in) :: npix_col, npix_row, max_slab
      character(len=256), intent(in) :: map_file
      character(len=1) :: rw(2)
      
      real*4, intent(out) :: map3D(npix_col, npix_row, max_slab)
      real*4, intent(out) :: mapprjbig
      
      double precision favg, fstd
      double precision mapprj(npix_col, npix_row)
      double precision fmax, fmin
      integer  count, outmode
      integer i, f_z, l_z, NCOL, NROW, NSEC, x, y, z
      character(len=14) :: subname
      data subname / 'Read_Map' /

#ifdef MPI
      integer stat(mpi_status_size)
#endif
      
      if (iam == 0) then
         
         MAPHEAD%LUNUM = 1
         rw(1) = 'r'
         rw(2) = achar(0)
         CALL PIF_OPEN (map_file, MAPHEAD%LUNUM, rw)
         CALL PIF_READ_GH (MAPHEAD)
         outmode = MAPHEAD%MODE

         if (outmode.NE.20 .and. outmode.NE.40) then
            call error_stop(progname,'read_map',
     +           'Error in 3D density map mode, it should be 20 or 40')
         endif

         MAPHEAD%IMGNUM = 1     ! LOAD UP "IMG" #1
         CALL PIF_READ_DH (MAPHEAD)
         NCOL = MAPHEAD%NUMCOLS
         NROW = MAPHEAD%NUMROWS
         NSEC = MAPHEAD%NUMSECS

         if(  NCOL .ne. npix_col .or. 
     +        NROW .ne. npix_row .or. 
     +        NSEC .ne. npix_col) then
            call error_stop(progname,subname,
     +           'Density map COL,ROW,SEC not consistent with img size')
         endif

         mapprj = 0.0 ! f90 array syntax

#ifdef MPI
         do i = num_nodes-1, 1, -1
            f_z = frst_lst_y(1,i)
            l_z = frst_lst_y(2,i)
            
            if (outmode == 20) then
               call PIF_READ_MAPI2(MAPHEAD,map3D,f_z,l_z,NCOL,NROW,NSEC)
            elseif (outmode == 40) then
               call PIF_READ_MAPI4(MAPHEAD,map3D,f_z,l_z,NCOL,NROW,NSEC)
            endif
            
            do z = l_z - f_z + 1, 1, -1
               mapprj = mapprj + map3D(:,:,z) ! f90 array syntax
            enddo
            
            call MPI_SEND( map3D, (l_z - f_z + 1)*NCOL*NROW,
     +           mpi_real, i, i, mpi_comm_world, mpi_err )
            if (mpi_err .ne. 0) call error_stop(progname, subname,
     +           'Terminated because of mpi_send error')
         enddo
#endif

         f_z = frst_lst_y(1,iam)
         l_z = frst_lst_y(2,iam)

         if (outmode == 20) then
            call PIF_READ_MAPI2(MAPHEAD,map3D,f_z,l_z,NCOL,NROW,NSEC)
         elseif (outmode == 40) then
            call PIF_READ_MAPI4(MAPHEAD,map3D,f_z,l_z,NCOL,NROW,NSEC)
         endif

         CALL PIF_CLOSE (1)

         do z = l_z - f_z + 1, 1, -1
            mapprj = mapprj + map3D(:,:,z) ! f90 array syntax
         enddo

         fmax      = mapprj((ncol+1)/2,(ncol+1)/2)
         fmin      = mapprj((ncol+1)/2,(ncol+1)/2)
         favg      = 0.0
         fstd      = 0.0
         count     = 0
         do y = 1, NROW
            do x = 1, NCOL
               if ((x-(NCOL+1)/2)**2+(y-(NROW+1)/2)**2 <= NCOL**2) then
                  favg = favg + mapprj(x,y)
                  fstd = fstd + mapprj(x,y)**2
                  if (mapprj(x,y) > fmax ) fmax = mapprj(x,y)
                  if (mapprj(x,y) < fmin ) fmin = mapprj(x,y)
                  count = count + 1
               endif
            enddo
         enddo
         if (count > 0) then
            favg = favg / count
            fstd = sqrt( abs( fstd/count - favg*favg ))
            mapprjbig = real(fstd)
         else
            mapprjbig = 1
         endif

      else 
#ifdef MPI
         f_z = frst_lst_y(1,iam)
         l_z = frst_lst_y(2,iam)
         call MPI_RECV( map3D,(l_z - f_z + 1)*Npix_Col*Npix_Row,
     +        mpi_real, 0, iam, mpi_comm_world, stat, mpi_err)
         if (mpi_err.ne.0) call error_stop(progname, subname,
     +        'Terminated because of mpi_recv error')
#endif
      endif

      return
      end subroutine read_map
