C  ****************************************************************************
C  *                                                                          *
C  *                     *****     FFTLIB1.SUBS     *****                     *
C  *                                                                          *
C  *    THIS IS A LIBRARY OF SUBROUTINES WHICH ARE USED TO MANIPULATE "FFT"   *
C  *     ARRAY DATA.  THE OBJECT CODE FOR ALL THE ROUTINES ARE KEPT IN THE    *
C  *     LIBRARY SYS$EMLIB:TSBLIB.OLB                                         *
C  *                                                                          *
C  *    LIST OF SUBROUTINES:                                                  *
C  *                                                                          *
C  *      FFT_1D, FFT_1DLOP, FFT_1DSETDIM_DEF, FFT_2D, FFT_2D_BACK, FFT_2DBT, *
C  *      FFT_CAVG, FFT_COPY1, FFT_FILL,                                      *
C  *      FFT_FLIP, FFT_FLT, FFT_FLTMSK, FFT_FLTLLMSK, FFT_HIP, FFT_IMG_FILL, *
C  *      FFT_MAP_FILL, FFT_LOP, FFT_MAP, FFT_MRCTRANSF, FFT_MULT,            *
C  *      FFT_OPEN, FFT_PSHIFT, FFT_SETDIM, FFT_SETDIM_DEF,                   *
C  *      FFT_SETDIM_DEF_SAME, FFT_SPT_BACK, FFT_SPT_BOX                      *
C  *      FFT_SPT_GRID, FFT_SPT_LAT,                                          *
C  *      FFT_SPT_NOBACK, FFT_STATS, FFT_TO_ATBT, FFT_TO_ATBT2, FFT_TYPE      *
C  *                                                                          *
C  *    APRIL 1988  PURDUE UNIVERSITY VAX 8550, T. BAKER.                     *
C  ****************************************************************************
C  ****************************************************************************
C  *                                                                          *
C  *                    *****     FFTLIB2.SUBS     *****                      *
C  *                                                                          *
C  *    THIS CONTAINS THE FORTRAN SOURCE CODE FOR A VARIETY OF FOURIER        *
C  *     TRANSFORM SUBROUTINES WHICH ARE CONTAINED IN THE OBJECT LIBRARY      *
C  *     "DEXTRO3:TSBLIB.OLB"                                                 *
C  *                                                                          *
C  *    NOTE: THESE SUBROUTINES SHOULD ALL BE COMPILED WITH DEFAULT INT*4     *
C  *                                                                          *
C  *    DECEMBER 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER                 *
C  ****************************************************************************
C
C
C

C  ***********************************************************************
C  *                                                                     *
C  *                *****     SUBROUTINE FFT_2D_ROB     *****            *
C  *                                                                     *
C  *    COMPUTE IN-CORE 2D FFT OF THE DATA LOADED INTO THE FULL-SIZE     *
C  *     "FFT" ARRAY.                                                    *
C  *                                                                     *
C  *    ARGUMENTS: ARRAY, NN, NN_LENGTH, KSIGN, IFORM, WORK.             *
C  *    ARRAY     = REAL OR COMPLEX ARRAY TO BE TRANSFORMED.             *
C  *    NN        = ARRAY CONTAINING THE TRANSFORM DIMENSIONS.           *
C  *    NN_LENGTH = LENGTH OF NN ARRAY (I.E. 2 FOR 2D ARRAY).            *
C  *    KSIGN     = +1 FOR FORWARD TRANSFORM.                            *
C  *              = -1 FOR REVERSE TRANSFORM.                            *
C  *    IFORM     =  0 FOR ALL REAL INPUT DATA.                          *
C  *              =  1 FOR ALL OTHER TYPES.                              *
C  *                                                                     *
C  *                                                                     *
C  *    MAY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.                  *
C  *    LAST MODIFIED 15-JAN-1992 BY T. BAKER.                           *
C  ***********************************************************************
C
C GCA: this routine is going to fail if the array WORK is really needed
C      (see requirements of FOURT subroutine)
      SUBROUTINE FFT_2D_ROB (FFT, IDIM1, IDIM2, ksign, iform)

      IMPLICIT NONE

      INTEGER*4 NN(2), IDIM1, IDIM2, I, J, ksign, iform
      REAL*4    FFT(2*IDIM1,IDIM2), WORK(1), div

      NN(1) = IDIM1
      NN(2) = IDIM2
      div   = REAL(IDIM1*IDIM2)

      CALL FOURT (FFT, NN, 2, ksign, iform, WORK)     ! TRANSFORM THE DATA

      if(ksign == -1) then
         DO J = 1,IDIM2
            DO I = 1,2*IDIM1
             FFT(I,J) = FFT(I,J)/div
            ENDDO
         ENDDO
      endif

      RETURN
      END subroutine fft_2d_rob



C  **********************************************************************
C  *                                                                    *
C  *          *****     SUBROUTINE FFT_2DBT_R4     *****                *
C  *                                                                    *
C  *    COMPUTE IN-CORE 2D REVERSE FFT OF THE FULL-SIZE "FFT" ARRAY     *
C  *                                                                    *
C  *    LOAD IDIM1 X IDIM2 "FFT" ARRAY INTO NCOL X NROW REAL*4 ARRAY.   *
C  *                                                                    *
C  *    AUGUST 1999 PURDUE UNIVERSITY VAX 8550, T. BAKER.               *
C  *    LAST MODIFIED 24-AUG-1999 BY T. BAKER (ADAPTED FROM FFT_2DBT).  *
C  **********************************************************************
C
C GCA: this routine is going to fail if the array WORK is really needed
C      (see requirements of FOURT subroutine)
      SUBROUTINE FFT_2DBT_R4 (FFT, IDIM1, IDIM2, MAP, NCOL, NROW,
     1     IMG_BKG)
C
      IMPLICIT NONE
C
      INTEGER*4 NN(2), IDIM1, IDIM2, I, J, NCOL, NROW
      real*4    IMG_BKG
      REAL*4    FFT(2*IDIM1,IDIM2), MAP(NCOL,NROW), WORK(1), DIV
      real*4, allocatable :: fft2(:,:)
      integer   alloc_stat

      NN(1)      = IDIM1
      NN(2)      = IDIM2
      DIV        = REAL(IDIM1)*REAL(IDIM2)

      allocate(fft2(2*idim1,idim2), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate memory in FFT_2DBT_R4'
         call tsbend
      endif

      fft2(1:2*idim1,1:idim2) = fft(1:2*idim1,1:idim2) ! Fortran 90 syntax

      CALL FOURT (FFT, NN, 2, -1, 1, WORK)               ! TRANSFORM THE DATA
      DO J = 1,IDIM2
        DO I = 1,2*IDIM1
          FFT(I,J) = FFT(I,J)/DIV
        ENDDO
      ENDDO

      DO J = 1,NROW
        DO I = 1,NCOL
          MAP(I,J) = FFT((2*I)-1,J) + IMG_BKG
        ENDDO
      ENDDO

      fft(1:2*idim1,1:idim2) = fft2(1:2*idim1,1:idim2) ! Fortran 90 syntax
      deallocate(fft2)

      RETURN
      END


C  **********************************************************************
C  *                                                                    *
C  *                  *****  SUBROUTINE FFT_COPY1  *****                *
C  *                                                                    *
C  *    COPY TOP HALF OF FFT2 ARRAY INTO FFT1 ARRAY.                    *
C  *                                                                    *
C  *    JUNE 1992  PURDUE UNIVERSITY VAX 8550, T. BAKER.                *
C  *    MODIFIED 12-JUN-1992 BY T. BAKER.                               *
C  *    LAST MODIFIED 27-FEB-1998 BY T. BAKER (Make UNIX compatible).   *
C  **********************************************************************
C
      SUBROUTINE FFT_COPY1 (FFT2, FFT1, IDIM1, IDIM2)
C
      IMPLICIT NONE
C
      INTEGER*4 I, J, IDIM1, IDIM2
      REAL*4    FFT1(2*IDIM1,((IDIM2/2)+1)), FFT2(2*IDIM1,IDIM2)

      DO J = 1,(IDIM2/2)+1
        DO I = 1,2*IDIM1
          FFT1(I,J) = FFT2(I,J)
        ENDDO
      ENDDO
C
      RETURN
      END


C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE FFT_FLIP     *****                 *
C  *                                                                          *
C  *    MULTIPLY ALL "B" PARTS OF THE FFT BY -1 TO EFFECTIVELY ROTATE THE     *
C  *     "IMAGE" BY 180 DEGREES.                                              *
C  *                                                                          *
C  *    NOTE: THE "IMAGE" IS EFFECTIVELY ROTATED WITHIN A FULL IDIM1 BY IDIM2 *
C  *          BOUNDARY, NOT WITHIN THE ORIGINAL NCOL BY NROW BOUNDARY.        *
C  *                                                                          *
C  *    MAY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.                       *
C  ****************************************************************************
C
      SUBROUTINE FFT_FLIP (FFT, IDIM1, IDIM2, FFT_FULL)

      IMPLICIT NONE

      INTEGER*4 I, IDIM1, IDIM2, ARRAY_SIZE
      REAL*4    FFT(1)
      LOGICAL   FFT_FULL

      IF(FFT_FULL) THEN
         ARRAY_SIZE = 2*IDIM1*IDIM2
       ELSE
         ARRAY_SIZE = IDIM1*(IDIM2 + 2)
      ENDIF

      DO I = 2,ARRAY_SIZE,2
        FFT(I) = -FFT(I)
      ENDDO

      RETURN
      END

C  **********************************************************************
C  *                                                                    *
C  *                *****     SUBROUTINE FFT_FLT     *****              *
C  *                                                                    *
C  *    APPLY HARD OR GAUSSIAN LOW-PASS AND/OR HI-PASS FOURIER CUTOFF   *
C  *                                                                    *
C  *    THIS ROUTINE SUPERCEDES THE OLD FFT_FLTHIP AND FFT_FLTLOP.      *
C  *                                                                    *
C  *    JANUARY 1992  PURDUE UNIVERSITY VAX 8550, T. BAKER.             *
C  *    LAST MODIFIED 27-FEB-1998 BY T. BAKER (Make UNIX compatible).   *
C  **********************************************************************
C
      SUBROUTINE FFT_FLT (FFT, IDIM1, IDIM2, RES_MIN, RES_MAX,
     1 CUT_MIN, CUT_MAX, FFT_FULL)

      IMPLICIT NONE

      INTEGER*4 IDIM1, IDIM2
      REAL*4    FFT(1), RES_MIN, RES_MAX, CUT_MIN, CUT_MAX
      LOGICAL   FFT_FULL

      IF(RES_MIN >  0.0) THEN
        CALL FFT_HIP (FFT, IDIM1, IDIM2, RES_MIN, CUT_MIN, FFT_FULL)
      ENDIF
      IF(RES_MAX > 0.0) THEN
        CALL FFT_LOP (FFT, IDIM1, IDIM2, RES_MAX, CUT_MAX, FFT_FULL)
      ENDIF

      RETURN
      END


C  ********************************************************************
C  *                                                                  *
C  *            *****     SUBROUTINE FFT_IMG_FILL     *****           *
C  *                                                                  *
C  *    LOAD IDIM1 X IDIM2 "FFT" ARRAY INTO NCOL X NROW "IMAGE".      *
C  *                                                                  *
C  *    JANUARY 1992  PURDUE UNIVERSITY VAX 8550, T. BAKER.           *
C  ********************************************************************
C
      SUBROUTINE FFT_IMG_FILL(FFT, IDIM1, IDIM2, IMG, NCOL, NROW,
     1     IMG_BKG)

      IMPLICIT NONE

      INTEGER*4 NCOL, NROW, IDIM1, IDIM2, I, J
      real*4    IMG_BKG,  IMG(NCOL,NROW)
      REAL*4    FFT(2*IDIM1,IDIM2)

      DO J = 1,NROW
        DO I = 1,NCOL
          IMG(I,J) = FFT((2*I)-1,J) + IMG_BKG
        ENDDO
      ENDDO

      RETURN
      END


C  **********************************************************************
C  *                                                                    *
C  *             *****     SUBROUTINE FFT_MAP_FILL     *****            *
C  *                                                                    *
C  *    LOAD IDIM1 X IDIM2 "FFT" ARRAY INTO NCOL X NROW "MAP" SECTION.  *
C  *                                                                    *
C  *    JANUARY 1992  PURDUE UNIVERSITY VAX 8550, T. BAKER.             *
C  *    LAST MODIFIED 2-SEP-1999 BY TSB (Rid of EM_WRT).                *
C  **********************************************************************
C
      SUBROUTINE FFT_MAP_FILL (FFT, IDIM1, IDIM2, MAP, NCOL, NROW,
     1     NSEC, SEC)

      IMPLICIT NONE

      INTEGER*4 NCOL, NROW, NSEC, SEC, IDIM1, IDIM2, I, J
      REAL*4    MAP(NCOL,NROW,NSEC), FFT(2*IDIM1,IDIM2)

      DO J = 1,NROW
        DO I = 1,NCOL
          MAP(I,J,SEC) = FFT((2*I)-1,J)
        ENDDO
      ENDDO

      RETURN
      END



C  *******************************************************************
C  *                                                                 *
C  *             *****     SUBROUTINE FFT_PSHIFT     *****           *
C  *                                                                 *
C  *    APPLY PHASE SHIFT TO FFT TO MOVE ORIGIN TO A POSITION        *
C  *     X_SHIFT,Y_SHIFT IN THE "IMAGE".                             *
C  *                                                                 *
C  *    MAY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.              *
C  *    LAST MODIFIED 12-AUG-1999 BY TSB (Fix YTEST problem).        *
C  *******************************************************************
C
      SUBROUTINE FFT_PSHIFT (FFT, IDIM1, IDIM2, X_SHIFT, Y_SHIFT,
     1 FFT_FULL)
C
      IMPLICIT NONE
C
      INTEGER*4 IX, IY, IDIM1, IDIM2, COUNT, IY2, YTEST
      REAL*4    FFT(1), XSHIFT, YSHIFT, DX, DY
      REAL*4    APART, BPART
      REAL*8    DPI, DTWOPI, DRADIAN, DDEGREE, DDX, DDY, DSHIFT
      REAL*4    X_SHIFT, Y_SHIFT, SIN_SHIFT, COS_SHIFT
      LOGICAL   FFT_FULL

      CALL PIRADDEG_D (DPI, DRADIAN, DDEGREE)
      DTWOPI = 2.0*DPI
      DDX    = X_SHIFT*DTWOPI/REAL(IDIM1)
      DDY    = Y_SHIFT*DTWOPI/REAL(IDIM2)
      DX = REAL(DDX)
      DY = REAL(DDY)
      YTEST = IDIM2/2
      COUNT = -1
C       
      IF(FFT_FULL) THEN
         IY2 = IDIM2
       ELSE
         IY2 = YTEST
      ENDIF
C
      DO IY = 1,IY2
        IF(IY.GT.YTEST) THEN
           YSHIFT = REAL(IY-IDIM2-1)*DY
         ELSE
           YSHIFT = REAL(IY-1)*DY
        ENDIF
        DO IX = 1,2*IDIM1,2
          COUNT = COUNT + 2
          IF(IX.LE.IDIM1) THEN
             XSHIFT = (REAL(IX-1)/2.0)*DX
           ELSE
             XSHIFT = (REAL(IX-1-(2*IDIM1))/2.0)*DX
          ENDIF
          DSHIFT       = XSHIFT + YSHIFT
          DSHIFT       = DMOD(DSHIFT,DTWOPI)
          SIN_SHIFT    = REAL(SIN(DSHIFT))
          COS_SHIFT    = REAL(COS(DSHIFT))
          APART        = FFT(COUNT)
          BPART        = FFT(COUNT+1)
          FFT(COUNT)   =  APART*COS_SHIFT + BPART*SIN_SHIFT
          FFT(COUNT+1) = -APART*SIN_SHIFT + BPART*COS_SHIFT
        ENDDO
      ENDDO
C
      RETURN
      END


C  **************************************************************************
C  *                                                                        *
C  *              *****     SUBROUTINE FFT_SETDIM_DEF     *****             *
C  *                                                                        *
C  *    SET UP DEFAULT FOURIER TRANSFORM DIMENSIONS.                        *
C  *                                                                        *
C  *    THE SIZE OF THE FFT DIMENSIONS (IDIM1 BY IDIM2) WILL BE THE         *
C  *     SMALLEST POWER OF 2 ALLOWED FOR THE NCOL BY NROW DIMENSIONS,       *
C  *     MULTIPLIED BY NFAC.                                                *
C  *                                                                        *
C  *    MAY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.                     *
C  **************************************************************************
C
      SUBROUTINE FFT_SETDIM_DEF (NCOL, NROW, IDIM1, IDIM2, NFAC)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, I, IDIM1, IDIM2, NFAC
      INTEGER*4 NUMCHARI4, N1, N2
      REAL*4    TEST
C  ----------------------------------------------------------------------------
C
C  ********************************************
C  *    DETERMINE MINIMUM ALLOWED FFT SIZE    *
C  ********************************************
      TEST = REAL(NCOL*(2**(NFAC-1)))
      I    = 0
      DO WHILE (TEST.GT.1.0)
        I    = I + 1
        TEST = TEST/2.0
      ENDDO
      IDIM1 = 2**I
C
      TEST = REAL(NROW*(2**(NFAC-1)))
      I    = 0
      DO WHILE (TEST.GT.1.0)
        I    = I + 1
        TEST = TEST/2.0
      ENDDO
      IDIM2 = 2**I
C
      N1 = NUMCHARI4(IDIM1) + 1
      N2 = NUMCHARI4(IDIM2) + 1
CCC      WRITE(*,'(A,I5,A,I5)') ' FFT dimensions are', IDIM1, ' by',
CCC      1 IDIM2
C
      RETURN
      END


C  ****************************************************************************
C  *                                                                          *
C  *                *****     SUBROUTINE FFT_TO_ATBT     *****                *
C  *                                                                          *
C  *    CUTS OUT A SQUARE REGION (BOUNDED BY A RADIUS = IRAD TRANSFORM PIXEL  *
C  *     UNITS) OF ANY "POWER-2" TRANSFORM AND APPLYS AN ORIGIN CORRECTION.   *
C  *                                                                          *
C  *    ARRAY DIMENSIONS: AT AND BT((IRAD+2)*2*IRAD+1), FFT(2*IDIM1,2*IDIM2)  *
C  *                                                                          *
C  *    THIS IS ADAPTED FROM "TRANSF" FROM THE ORIGINAL MRC PROGRAM "ROTFLT". *
C  *     IT IS USED IN SEVERAL PROGRAMS SUCH AS EMROTFLT, EMICOFV*, EMICOORG, *
C  *     AND EMICOGRAD AMONG OTHERS.                                          *
C  *                                                                          *
C  *    JULY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.                      *
C  *    Modified 2/8/11 by RSS to avoid unintended side effects,              *
C  *    namely modification of input array FFT                                *
C  ****************************************************************************
C
      SUBROUTINE FFT_TO_ATBT (FFT, IDIM1, IDIM2, AT, BT, IRAD, 
     1     FFT_ORIGX, FFT_ORIGY, FFT_FULL)

      IMPLICIT NONE
      
      logical, intent(in) :: fft_full
      integer, intent(in) :: idim1, idim2, irad
      real*4, intent(in)  :: fft_origx, fft_origy, fft(2*idim1, idim2)
      real*4, intent(out) :: AT((2*IRAD+1)*(IRAD+2))
      real*4, intent(out) :: BT((2*IRAD+1)*(IRAD+2))

      integer   I, J, K, IK, OFFSET
      real*4    PI, SHIFT, YSHIFT
      real*4    X, XSHIFT_STEP, YSHIFT_STEP, COS_SHIFT, SIN_SHIFT
      real*4    tcol(2*idim1)
      parameter (PI = 3.141592654)

      IF(.NOT.FFT_FULL) THEN
        CALL BELL (2)
        PRINT *, '*************************************************'
        PRINT *, '* Option does not work with HALF-SIZE FFT array *'
        PRINT *, '*                                               *'
        PRINT *, '* Program terminated in subroutine FFT_TO_ATBT  *'
        CALL TSBEND
      ENDIF

      OFFSET      = IDIM1 - 2*IRAD - 1
      XSHIFT_STEP = FFT_ORIGX*2.0*PI/REAL(IDIM1)
      YSHIFT_STEP = FFT_ORIGY*2.0*PI/REAL(IDIM1)

      IK = 1
      DO J = 1,(IRAD+2)
        YSHIFT = YSHIFT_STEP*(J-1)
        DO I = 1,IDIM1
           tcol(i)       = fft(i+idim1, j)
           tcol(i+idim1) = fft(i, j)
        ENDDO
        DO I = 1,(2*IRAD+1)
          X         = REAL(I-IRAD-1)
          SHIFT     = XSHIFT_STEP*X + YSHIFT
          COS_SHIFT = COS(SHIFT)
          SIN_SHIFT = SIN(SHIFT)
          K         = 2*I + OFFSET
          AT(IK)    =  tcol(K)*COS_SHIFT + tcol(K+1)*SIN_SHIFT
          BT(IK)    = -tcol(K)*SIN_SHIFT + tcol(K+1)*COS_SHIFT
          IK        = IK + 1
        ENDDO
      ENDDO

      RETURN

      END


C    **************************************************************************
C    *                                                                        *
C    *                  *****     SUBROUTINE FOURT     *****                  *
C    *                                                                        *
C    *    THE COOLEY-TUKEY FAST FOURIER TRANSFORM IN USASI BASIC FORTRAN      *
C    *                                                                        *
C    *    TRANSFORM(K1,K2,...) = SUM(DATA(J1,J2,...)*EXP(ISIGN*2*PI*SQRT(-1)  *
C    *     *((J1-1)*(K1-1)/NN(1)+(J2-1)*(K2-1)/NN(2)+...))), SUMMED FOR ALL   *
C    *     J1, K1 BETWEEN 1 AND NN(1), J2, K2 BETWEEN 1 AND NN(2), ETC.       *
C    *     THERE IS NO LIMIT TO THE NUMBER OF SUBSCRIPTS.  DATA IS A          *
C    *     MULTIDIMENSIONAL COMPLEX ARRAY WHOSE REAL AND IMAGINARY            *
C    *     PARTS ARE ADJACENT IN STORAGE, SUCH AS FORTRAN IV PLACES THEM.     *
C    *     IF ALL IMAGINARY PARTS ARE ZERO (DATA ARE DISGUISED REAL), SET     *
C    *     IFORM TO ZERO TO CUT THE RUNNING TIME BY UP TO FORTY PERCENT.      *
C    *     OTHERWISE, IFORM = +1.  THE LENGTHS OF ALL DIMENSIONS ARE          *
C    *     STORED IN ARRAY NN, OF LENGTH NDIM.  THEY MAY BE ANY POSITIVE      *
C    *     INTEGERS, THO THE PROGRAM RUNS FASTER ON COMPOSITE INTEGERS, AND   *
C    *     ESPECIALLY FAST ON NUMBERS RICH IN FACTORS OF TWO.  ISIGN IS +1    *
C    *     OR -1.  IF A -1 TRANSFORM IS FOLLOWED BY A +1 ONE (OR A +1         *
C    *     BY A -1) THE ORIGINAL DATA REAPPEAR, MULTIPLIED BY NTOT (=NN(1)*   *
C    *     NN(2)*...).  TRANSFORM VALUES ARE ALWAYS COMPLEX, AND ARE RETURNED *
C    *     IN ARRAY DATA, REPLACING THE INPUT.  IN ADDITION, IF ALL           *
C    *     DIMENSIONS ARE NOT POWERS OF TWO, ARRAY WORK MUST BE SUPPLIED,     *
C    *     COMPLEX OF LENGTH EQUAL TO THE LARGEST NON 2**K DIMENSION.         *
C    *     OTHERWISE, REPLACE WORK BY ZERO IN THE CALLING SEQUENCE.           *
C    *     NORMAL FORTRAN DATA ORDERING IS EXPECTED, FIRST SUBSCRIPT VARYING  *
C    *     FASTEST.  ALL SUBSCRIPTS BEGIN AT ONE.                             *
C    *                                                                        *
C    *    RUNNING TIME IS MUCH SHORTER THAN THE NAIVE NTOT**2, BEING          *
C    *     GIVEN BY THE FOLLOWING FORMULA.  DECOMPOSE NTOT INTO               *
C    *     2**K2 * 3**K3 * 5**K5 * ....  LET SUM2 = 2*K2, SUMF = 3*K3 + 5*K5  *
C    *     + ... AND NF = K3 + K5 + ....  THE TIME TAKEN BY A MULTI-          *
C    *     DIMENSIONAL TRANSFORM ON THESE NTOT DATA IS T = T0 + NTOT*(T1+     *
C    *     T2*SUM2+T3*SUMF+T4*NF).  ON THE CDC 3300 (FLOATING POINT ADD TIME  *
C    *     OF SIX MICROSECONDS), T = 3000 + NTOT*(500+43*SUM2+68*SUMF+        *
C    *     320*NF) MICROSECONDS ON COMPLEX DATA.  IN ADDITION, THE            *
C    *     ACCURACY IS GREATLY IMPROVED, AS THE RMS RELATIVE ERROR IS         *
C    *     BOUNDED BY 3*2**(-B)*SUM(FACTOR(J)**1.5), WHERE B IS THE NUMBER    *
C    *     OF BITS IN THE FLOATING POINT FRACTION AND FACTOR(J) ARE THE       *
C    *     PRIME FACTORS OF NTOT.                                             *
C    *                                                                        *
C    *    PROGRAM BY NORMAN BRENNER FROM THE BASIC PROGRAM BY CHARLES         *
C    *     RADER.  RALPH ALTER SUGGESTED THE IDEA FOR THE DIGIT REVERSAL.     *
C    *     MIT LINCOLN LABORATORY, AUGUST 1967.  THIS IS THE FASTEST AND MOST *
C    *     VERSATILE VERSION OF THE FFT KNOWN TO THE AUTHOR.  SHORTER PRO-    *
C    *     GRAMS FOUR1 AND FOUR2 RESTRICT DIMENSION LENGTHS TO POWERS OF TWO. *
C    *     SEE-- IEEE AUDIO TRANSACTIONS (JUNE 1967), SPECIAL ISSUE ON FFT.   *
C    *                                                                        *
C    *    THE DISCRETE FOURIER TRANSFORM PLACES THREE RESTRICTIONS UPON THE   *
C    *     DATA.                                                              *
C    *     1.  THE NUMBER OF INPUT DATA AND THE NUMBER OF TRANSFORM VALUES    *
C    *         MUST BE THE SAME.                                              *
C    *     2.  BOTH THE INPUT DATA AND THE TRANSFORM VALUES MUST REPRESENT    *
C    *         EQUISPACED POINTS IN THEIR RESPECTIVE DOMAINS OF TIME AND      *
C    *         FREQUENCY.  CALLING THESE SPACINGS DELTAT AND DELTAF, IT       *
C    *         MUST BE TRUE THAT DELTAF=2*PI/(NN(I)*DELTAT).  OF COURSE,      *
C    *         DELTAT NEED NOT BE THE SAME FOR EVERY DIMENSION.               *
C    *     3.  CONCEPTUALLY AT LEAST, THE INPUT DATA AND THE TRANSFORM        *
C    *         OUTPUT REPRESENT SINGLE CYCLES OF PERIODIC FUNCTIONS.          *
C    *                                                                        *
C    *     EXAMPLE 1.  THREE-DIMENSIONAL FORWARD FOURIER TRANSFORM OF A       *
C    *       COMPLEX ARRAY DIMENSIONED 32 BY 25 BY 13 IN FORTRAN IV.          *
C    *       DIMENSION DATA(32,25,13),WORK(50),NN(3)                          *
C    *       COMPLEX DATA                                                     *
C    *       DATA NN/32,25,13/                                                *
C    *       DO 1 I=1,32                                                      *
C    *         DO 1 J=1,25                                                    *
C    *           DO 1 K=1,13                                                  *
C    *    1        DATA(I,J,K)=COMPLEX VALUE                                  *
C    *       CALL FOURT(DATA,NN,3,-1,1,WORK)                                  *
C    *                                                                        *
C    *     EXAMPLE 2.  ONE-DIMENSIONAL FORWARD TRANSFORM OF A REAL ARRAY OF   *
C    *       LENGTH 64 IN FORTRAN II.                                         *
C    *       DIMENSION DATA(2,64)                                             *
C    *       DO 2 I=1,64                                                      *
C    *         DATA(1,I)=REAL PART                                            *
C    *    2    DATA(2,I)=0.                                                   *
C    *       CALL FOURT(DATA,64,1,-1,0,work)                                  *
C    **************************************************************************
C
C GCA: routine is called giving real or complex DATA array, which is treated as
C      of real values (1 element is real part, second imaginary part and so on)
      SUBROUTINE FOURT (DATA, NN, NDIM, KSIGN, IFORM, WORK)
C
      DIMENSION DATA(*), NN(1), IFACT(32), WORK(1)  
C  ----------------------------------------------------------------------------
C
      TWOPI = 6.283185307         
      IF(NDIM-1)920,1,1         
    1      NTOT = 2
      DO 2 IDIM = 1,NDIM
      IF(NN(IDIM))920,920,2
    2      NTOT = NTOT*NN(IDIM)
C  **************************************
C  *    MAIN LOOP FOR EACH DIMENSION    *
C  **************************************
      NP1 = 2
      DO 910 IDIM = 1,NDIM
      N   = NN(IDIM)
      NP2 = NP1*N
      IF(N-1)920,900,5
C  ******************
C  *    FACTOR N    *
C  ******************
    5      M     = N
      NTWO  = NP1
      IF    = 1
      IDIV  = 2
   10      IQUOT = M/IDIV
      IREM  = M-IDIV*IQUOT
      IF(IQUOT-IDIV)50,11,11
   11      IF(IREM)20,12,20
   12      NTWO = NTWO+NTWO
      M    = IQUOT
      GO TO 10
   20      IDIV  = 3
   30      IQUOT = M/IDIV
      IREM  = M-IDIV*IQUOT
      IF(IQUOT-IDIV)60,31,31
   31      IF(IREM)40,32,40
   32      IFACT(IF) = IDIV
      IF        = IF+1
      M         = IQUOT
      GO TO 30
   40      IDIV = IDIV+2
      GO TO 30
   50      IF(IREM)60,51,60
   51      NTWO = NTWO+NTWO
      GO TO 70
   60      IFACT(IF) = M
C  ************************************************************************
C  *    SEPARATE FOUR CASES--                                             *
C  *      1. COMPLEX TRANSFORM OR REAL TRANSFORM FOR THE 4TH, 5TH,ETC.    *
C  *         DIMENSIONS.                                                  *
C  *      2. REAL TRANSFORM FOR THE 2ND OR 3RD DIMENSION.  METHOD--       *
C  *         TRANSFORM HALF THE DATA, SUPPLYING THE OTHER HALF BY CON-    *
C  *         JUGATE SYMMETRY.                                             *
C  *      3. REAL TRANSFORM FOR THE 1ST DIMENSION, N ODD.  METHOD--       *
C  *         TRANSFORM HALF THE DATA AT EACH STAGE, SUPPLYING THE OTHER   *
C  *         HALF BY CONJUGATE SYMMETRY.                                  *
C  *      4. REAL TRANSFORM FOR THE 1ST DIMENSION, N EVEN.  METHOD--      *
C  *         TRANSFORM A COMPLEX ARRAY OF LENGTH N/2 WHOSE REAL PARTS     *
C  *         ARE THE EVEN NUMBERED REAL VALUES AND WHOSE IMAGINARY PARTS  *
C  *         ARE THE ODD NUMBERED REAL VALUES.  SEPARATE AND SUPPLY       *
C  *         THE SECOND HALF BY CONJUGATE SYMMETRY.                       *
C  ************************************************************************
   70      NON2  = NP1*(NP2/NTWO)
      ICASE = 1
      IF(IDIM-4) 71,90,90
   71      IF(IFORM) 72,72,90
   72      ICASE = 2
      IF(IDIM-1) 73,73,90
   73      ICASE = 3
       IF(NTWO-NP1) 90,90,74
   74      ICASE = 4
      NTWO = NTWO/2
      N    = N/2
      NP2  = NP2/2
      NTOT = NTOT/2
      I    = 3
      DO J = 2,NTOT
        DATA(J) = DATA(I)
        I       = I+2
      ENDDO
   90      I1RNG = NP1
      IF(ICASE-2) 100,95,100
   95      I1RNG = NP0*(1+NPREV/2)
C  ***********************************************************************
C  *    SHUFFLE ON THE FACTORS OF TWO IN N.  AS THE SHUFFLING            *
C  *    CAN BE DONE BY SIMPLE INTERCHANGE, NO WORKING ARRAY IS NEEDED    *
C  ***********************************************************************
  100      IF(NTWO-NP1) 600,600,110
  110      NP2HF = NP2/2
      J     = 1
      DO 150 I2 = 1,NP2,NON2 
        IF(J-I2) 120,130,130
  120        I1MAX = I2+NON2-2
        DO I1 = I2,I1MAX,2
          DO I3 = I1,NTOT,NP2
            J3         = J+I3-I2
            TEMPR      = DATA(I3)
            TEMPI      = DATA(I3+1)
            DATA(I3)   = DATA(J3)
            DATA(I3+1) = DATA(J3+1)
            DATA(J3)   = TEMPR
            DATA(J3+1) = TEMPI
          ENDDO
        ENDDO
  130        M = NP2HF
  140        IF(J-M) 150,150,145
  145        J = J-M
        M = M/2
        IF(M-NON2) 150,140,140
  150      J = J+M
C  ****************************************************************************
C  *    MAIN LOOP FOR FACTORS OF TWO.  PERFORM FOURIER TRANSFORMS OF          *
C  *     LENGTH FOUR, WITH ONE OF LENGTH TWO IF NEEDED.  THE TWIDDLE FACTOR   *
C  *     W=EXP(ISIGN*2*PI*SQRT(-1)*M/(4*MMAX)).  CHECK FOR W=ISIGN*SQRT(-1)   *
C  *     AND REPEAT FOR W=ISIGN*SQRT(-1)*CONJUGATE(W).                        *
C  ****************************************************************************
      NON2T = NON2+NON2
      IPAR  = NTWO/NP1
  310      IF(IPAR-2) 350,330,320
  320      IPAR = IPAR/4
      GO TO 310
  330      DO I1 = 1,I1RNG,2
        DO J3 = I1,NON2,NP1
          DO K1 = J3,NTOT,NON2T
            K2         = K1+NON2
            TEMPR      = DATA(K2)
            TEMPI      = DATA(K2+1)
            DATA(K2)   = DATA(K1)-TEMPR
            DATA(K2+1) = DATA(K1+1)-TEMPI
            DATA(K1)   = DATA(K1)+TEMPR
            DATA(K1+1) = DATA(K1+1)+TEMPI
          ENDDO
        ENDDO
      ENDDO
  350      MMAX = NON2
  360      IF(MMAX-NP2HF) 370,600,600
  370      LMAX = MAX0(NON2T,MMAX/2)
      IF(MMAX-NON2) 405,405,380
  380      THETA = -TWOPI*REAL(NON2)/REAL(4*MMAX)
      IF (KSIGN) 400,390,390
  390      THETA = -THETA
  400      WR    = COS(THETA)
      WI    = SIN(THETA)
      WSTPR = -2.*WI*WI
      WSTPI =  2.*WR*WI
  405      DO 570 L = NON2,LMAX,NON2T
      M = L
      IF(MMAX-NON2) 420,420,410
  410      W2R = WR*WR-WI*WI
      W2I = 2.*WR*WI
      W3R = W2R*WR-W2I*WI
      W3I = W2R*WI+W2I*WR
  420      DO 530 I1 = 1,I1RNG,2
      DO 530 J3 = I1,NON2,NP1
      KMIN = J3+IPAR*M
      IF(MMAX-NON2) 430,430,440
  430      KMIN  = J3
  440      KDIF  = IPAR*MMAX
  450      KSTEP = 4*KDIF
      DO 520 K1 = KMIN,NTOT,KSTEP
      K2 = K1+KDIF
      K3 = K2+KDIF
      K4 = K3+KDIF
      IF(MMAX-NON2) 460,460,480
  460      U1R = DATA(K1)+DATA(K2)
      U1I = DATA(K1+1)+DATA(K2+1)
      U2R = DATA(K3)+DATA(K4)
      U2I = DATA(K3+1)+DATA(K4+1)
      U3R = DATA(K1)-DATA(K2)
      U3I = DATA(K1+1)-DATA(K2+1)
      IF (KSIGN) 470,475,475
  470      U4R = DATA(K3+1)-DATA(K4+1)
      U4I = DATA(K4)-DATA(K3)
      GO TO 510
  475      U4R = DATA(K4+1)-DATA(K3+1)
      U4I = DATA(K3)-DATA(K4)
      GO TO 510
  480      T2R = W2R*DATA(K2)-W2I*DATA(K2+1)
      T2I = W2R*DATA(K2+1)+W2I*DATA(K2)
      T3R = WR*DATA(K3)-WI*DATA(K3+1)
      T3I = WR*DATA(K3+1)+WI*DATA(K3)
      T4R = W3R*DATA(K4)-W3I*DATA(K4+1)
      T4I = W3R*DATA(K4+1)+W3I*DATA(K4)
      U1R = DATA(K1)+T2R
      U1I = DATA(K1+1)+T2I
      U2R = T3R+T4R
      U2I = T3I+T4I
      U3R = DATA(K1)-T2R
      U3I = DATA(K1+1)-T2I
      IF (KSIGN) 490,500,500
  490      U4R = T3I-T4I
      U4I = T4R-T3R
      GO TO 510
  500      U4R = T4I-T3I
      U4I = T3R-T4R
  510      DATA(K1)   = U1R+U2R
      DATA(K1+1) = U1I+U2I
      DATA(K2)   = U3R+U4R
      DATA(K2+1) = U3I+U4I
      DATA(K3)   = U1R-U2R
      DATA(K3+1) = U1I-U2I
      DATA(K4)   = U3R-U4R
  520      DATA(K4+1) = U3I-U4I
      KMIN       = 4*(KMIN-J3)+J3
      KDIF       = KSTEP
      IF(KDIF-NP2) 450,530,530
  530      CONTINUE
      M = MMAX-M
      IF (KSIGN) 540,550,550
  540      TEMPR = WR
      WR    = -WI
      WI    = -TEMPR
      GO TO 560
  550      TEMPR = WR
      WR    = WI
      WI    = TEMPR
  560      IF(M-LMAX) 565,565,410
  565      TEMPR = WR
      WR    = WR*WSTPR-WI*WSTPI+WR
  570      WI    = WI*WSTPR+TEMPR*WSTPI+WI
      IPAR  = 3-IPAR
      MMAX  = MMAX+MMAX
      GO TO 360
  600      CALL L6TO9(DATA,KSIGN,WORK,NTWO,
     1  NP2,NP0,KSTEP,I1MAX,IFACT,
     1  NON2,IF,NP1,ICASE,THETA,TWOPI,WSTPR,WSTPI,WR,WI,I1RNG,
     2  I1,I3,NTOT,J3,TEMPR,N,KMIN,TEMPI,I2,J,I,IDIM)
C  ***************************************
C  *    END OF LOOP ON EACH DIMENSION    *
C  ***************************************
  900      NP0   = NP1
      NP1   = NP2
  910      NPREV = N
  920      RETURN
      END
C  ****************************************************************************
C  *                                                                          *
C  *                   *****     SUBROUTINE L6TO9     *****                   *
C  *                                                                          *
C  *    MAIN LOOP FOR FACTORS NOT EQUAL TO TWO.  APPLY THE TWIDDLE FACTOR     *
C  *     W=EXP(ISIGN*2*PI*SQRT(-1)*(J2-1)*(J1-J2)/(NP2*IFP1)), THEN           *
C  *     PERFORM A FOURIER TRANSFORM OF LENGTH IFACT(IF), MAKING USE OF       *
C  *     CONJUGATE SYMMETRIES.                                                *
C  ****************************************************************************
C
      SUBROUTINE L6TO9 (DATA, KSIGN, WORK, NTWO,
     1  NP2, NP0, KSTEP, I1MAX, IFACT, NON2, IF, NP1, ICASE,
     2  THETA, TWOPI, WSTPR, WSTPI, WR, WI, I1RNG, I1, I3, NTOT,
     3  J3, TEMPR, N, KMIN, TEMPI, I2, J, I, IDIM)

      DIMENSION DATA(*), IFACT(32), WORK(1)

      IF(NTWO-NP2)605,700,700      
  605      IFP1  = NON2  
      IF    = 1  
      NP1HF = NP1/2
  610      IFP2  = IFP1/IFACT(IF)  
      J1RNG = NP2  
      IF(ICASE-3)612,611,612
  611      J1RNG = (NP2+IFP1)/2   
      J2STP = NP2/IFACT(IF)  
      J1RG2 = (J2STP+IFP2)/2 
  612      J2MIN = 1+IFP2  
      IF(IFP1-NP2)615,640,640      
  615      DO 635 J2 = J2MIN,IFP1,IFP2    
        THETA = -TWOPI*REAL(J2-1)/REAL(NP2)  
        IF (KSIGN) 625,620,620
  620        THETA = -THETA  
  625        SINTH = SIN(THETA/2.)  
        WSTPR = -2.*SINTH*SINTH
        WSTPI = SIN(THETA)     
        WR    = WSTPR+1.
        WI    = WSTPI   
        J1MIN = J2+IFP1 
        DO 635 J1 = J1MIN,J1RNG,IFP1   
          I1MAX = J1+I1RNG-2     
          DO I1 = J1,I1MAX,2 
            DO I3 = I1,NTOT,NP2
            J3MAX = I3+IFP2-NP1    
              DO J3 = I3,J3MAX,NP1
                TEMPR = DATA(J3)
                DATA(J3) = DATA(J3)*WR-DATA(J3+1)*WI
                DATA(J3+1) = TEMPR*WI+DATA(J3+1)*WR
              ENDDO
            ENDDO
          ENDDO
        TEMPR = WR
        WR    = WR*WSTPR-WI*WSTPI+WR
  635      WI    = TEMPR*WSTPI+WI*WSTPR+WI
  640      THETA = -TWOPI/REAL(IFACT(IF))
      IF (KSIGN) 650,645,645
  645      THETA = -THETA
  650      SINTH = SIN(THETA/2.)
      WSTPR = -2.*SINTH*SINTH
      WSTPI = SIN(THETA)
      KSTEP = 2*N/IFACT(IF)
      KRANG = KSTEP*(IFACT(IF)/2)+1
      DO 698 I1 = 1,I1RNG,2
      DO 698 I3 = I1,NTOT,NP2
        DO 690 KMIN = 1,KRANG,KSTEP
        J1MAX = I3+J1RNG-IFP1
        DO 680 J1 = I3,J1MAX,IFP1
          J3MAX = J1+IFP2-NP1
          DO 680 J3 = J1,J3MAX,NP1
            J2MAX = J3+IFP1-IFP2
            K     = KMIN+(J3-J1+(J1-I3)/IFACT(IF))/NP1HF
            IF(KMIN-1)655,655,665
  655            SUMR = 0.0
            SUMI = 0.0
            DO J2 = J3,J2MAX,IFP2
              SUMR = SUMR+DATA(J2)
              SUMI = SUMI+DATA(J2+1)
            ENDDO
            WORK(K)   = SUMR
            WORK(K+1) = SUMI
            GO TO 680  
  665            KCONJ = K+2*(N-KMIN+1) 
            J2    = J2MAX   
            SUMR  = DATA(J2) 
            SUMI  = DATA(J2+1)
            OLDSR = 0.0
            OLDSI = 0.0
            J2    = J2-IFP2
  670            TEMPR = SUMR
            TEMPI = SUMI
            SUMR  = TWOWR*SUMR-OLDSR+DATA(J2)
            SUMI  = TWOWR*SUMI-OLDSI+DATA(J2+1)
            OLDSR = TEMPR
            OLDSI = TEMPI
            J2    = J2-IFP2
             IF(J2-J3)675,675,670
  675            TEMPR         = WR*SUMR-OLDSR+DATA(J2)
            TEMPI         = WI*SUMI
            WORK(K)       = TEMPR-TEMPI
            WORK(KCONJ)   = TEMPR+TEMPI
            TEMPR         = WR*SUMI-OLDSI+DATA(J2+1)
            TEMPI         = WI*SUMR
            WORK(K+1)     = TEMPR+TEMPI
            WORK(KCONJ+1) = TEMPR-TEMPI
  680        CONTINUE
        IF(KMIN-1)685,685,686
  685        WR = WSTPR+1.0
        WI = WSTPI
        GO TO 690
  686        TEMPR=WR
        WR    = WR*WSTPR-WI*WSTPI+WR
        WI    = TEMPR*WSTPI+WI*WSTPR+WI
  690      TWOWR = WR+WR
      IF(ICASE-3)692,691,692
  691      IF(IFP1-NP2)695,692,692
  692      K     = 1
      I2MAX = I3+NP2-NP1
      DO I2 = I3,I2MAX,NP1
        DATA(I2)   = WORK(K)
        DATA(I2+1) = WORK(K+1)
        K          = K+2
      ENDDO
      GO TO 698
C  *******************************************************************
C  *    COMPLETE A REAL TRANSFORM IN THE 1ST DIMENSION, N ODD, BY    *
C  *     CONJUGATE SYMMETRIES AT EACH STAGE.                         *
C  *******************************************************************
  695      J3MAX = I3+IFP2-NP1
      DO 697 J3 = I3,J3MAX,NP1
        J2MAX = J3+NP2-J2STP
        DO 697 J2 = J3,J2MAX,J2STP
          J1MAX = J2+J1RG2-IFP2
          J1CNJ = J3+J2MAX+J2STP-J2
          DO 697 J1 = J2,J1MAX,IFP2
            K          = 1+J1-I3
            DATA(J1)   = WORK(K)
            DATA(J1+1) = WORK(K+1)
            IF(J1-J2)697,697,696
  696            DATA(J1CNJ)   =  WORK(K)
            DATA(J1CNJ+1) = -WORK(K+1)
  697            J1CNJ         = J1CNJ-IFP2
  698      CONTINUE
      IF    = IF+1
      IFP1  = IFP2
      IF(IFP1-NP1)700,700,610
C  ********************************************************************
C  *    COMPLETE A REAL TRANSFORM IN THE 1ST DIMENSION, N EVEN, BY    *
C  *     CONJUGATE SYMMETRIES.                                        *
C  ********************************************************************
  700      GO TO (900,800,900,701),ICASE
  701      NHALF = N
      N     = N+N
      THETA = -TWOPI/REAL(N)
      IF (KSIGN) 703,702,702
  702      THETA = -THETA
  703      SINTH = SIN(THETA/2.)
      WSTPR = -2.*SINTH*SINTH
      WSTPI = SIN(THETA)
      WR    = WSTPR+1.0
      WI    = WSTPI
      IMIN  = 3
      JMIN  = 2*NHALF-1
      GO TO 725
  710      J = JMIN
      DO I = IMIN,NTOT,NP2
        SUMR      = (DATA(I)  +DATA(J))  /2.
        SUMI      = (DATA(I+1)+DATA(J+1))/2.
        DIFR      = (DATA(I)  -DATA(J))  /2.
        DIFI      = (DATA(I+1)-DATA(J+1))/2.
        TEMPR     = WR*SUMI+WI*DIFR
        TEMPI     = WI*SUMI-WR*DIFR
        DATA(I)   = SUMR+TEMPR
        DATA(I+1) = DIFI+TEMPI
        DATA(J)   = SUMR-TEMPR
        DATA(J+1) = -DIFI+TEMPI
        J         = J+NP2
      ENDDO
      IMIN  = IMIN+2
      JMIN  = JMIN-2
      TEMPR = WR
      WR    = WR*WSTPR-WI*WSTPI+WR
      WI    = TEMPR*WSTPI+WI*WSTPR+WI
  725      IF(IMIN-JMIN)710,730,740
  730      IF (KSIGN) 731,740,740
  731      DO I = IMIN,NTOT,NP2
        DATA(I+1) = -DATA(I+1)
      ENDDO
  740      NP2  = NP2+NP2
      NTOT = NTOT+NTOT
      J    = NTOT+1
      IMAX = NTOT/2+1
  745      IMIN = IMAX-2*NHALF
      I    = IMIN
      GO TO 755
  750      DATA(J)   =  DATA(I)
      DATA(J+1) = -DATA(I+1)
  755      I         = I+2
      J         = J-2
      IF(I-IMAX)750,760,760
  760      DATA(J)   = DATA(IMIN)-DATA(IMIN+1)
      DATA(J+1) = 0.0
      IF(I-J)770,780,780
  765      DATA(J)   = DATA(I)
      DATA(J+1) = DATA(I+1)
  770      I         = I-2
      J         = J-2
      IF(I-IMIN)775,775,765
  775      DATA(J)   = DATA(IMIN)+DATA(IMIN+1)
      DATA(J+1) = 0.0
      IMAX      = IMIN
      GO TO 745
  780      DATA(1) = DATA(1)+DATA(2)
      DATA(2) = 0.0
      GO TO 900
C  *******************************************************************
C  *    COMPLETE A REAL TRANSFORM FOR THE 2ND OR 3RD DIMENSION BY    *
C  *     CONJUGATE SYMMETRIES.                                       *
C  *******************************************************************
  800      IF(I1RNG-NP1)805,900,900
  805      DO I3 = 1,NTOT,NP2
        I2MAX = I3+NP2-NP1
        DO I2 = I3,I2MAX,NP1
          IMIN = I2+I1RNG
          IMAX = I2+NP1-2
          JMAX = 2*I3+NP1-IMIN
          IF(I2-I3)820,820,810
  810          JMAX = JMAX+NP2
  820          IF(IDIM-2)850,850,830
  830          J = JMAX+NP0
          DO I = IMIN,IMAX,2
            DATA(I)   =  DATA(J)
            DATA(I+1) = -DATA(J+1)
            J         = J-2
          ENDDO
  850          J = JMAX
          DO I = IMIN,IMAX,NP0
            DATA(I)   =  DATA(J)
            DATA(I+1) = -DATA(J+1)
            J         = J-NP0
          ENDDO
        ENDDO
      ENDDO
  900      RETURN
      END
C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE REALFT     *****                   *
C  *                                                                          *
C  *    REAL FOURIER TRANSFORM                                                *
C  *                                                                          *
C  *    GIVEN A REAL SEQUENCE OF LENGTH 2N THIS SUBROUTINE CALCULATES THE     *
C  *     UNIQUE PART OF THE FOURIER TRANSFORM. THE FOURIER TRANSFORM HAS      *
C  *     N + 1 UNIQUE REAL PARTS AND N - 1 UNIQUE IMIGINARY PARTS. SINCE      *
C  *     THE REAL PART AT X(N) IS FREQUENTLY OF INTEREST. THIS SUBROUTINE     *
C  *     STORES IT AT X(N) RATHER THAN IN Y(0). THERFORE X AND Y MUST BE      *
C  *     OF LENGTH N + 1 INSTEAD OF N. NOTE THAT THIS STORAGE ARRANGEMENT     *
C  *     IS DIFFERENT FROM THAT EMPLOYED BY THE HERMITIAN FOURIER             *
C  *     TRANSFORM SUBROUTINE                                                 *
C  *                                                                          *
C  *    FOR CONVENIENCE THE DATA IS PRESENTED IN TWO PARTS. THE FIRST         *
C  *     CONTAINING THE EVEN NUMBERED REAL TERMS AND THE SECOND CONTAINING    *
C  *     THE ODD NUMBERED TERMS (NUMBERING STARTING AT 0). ON RETURN THE      *
C  *     REAL PART OF THE TRANSFORM REPLACES THE EVEN TERMS AND THE           *
C  *     IMAGINARY PART OF THE TRANSFORM REPLACES THE ODD TERMS               *
C  ****************************************************************************
C
      SUBROUTINE REALFT (EVEN, ODD, N, DIM)
C
      REAL*4  EVEN(1), ODD(1)
      REAL*4  A, B, C, D, E, F, ANGLE, CO, TWOPI, TWON
      INTEGER N, DIM(5), NT, D2, D3, D4, D5
      INTEGER I, J, K, L, NOVER2, I0, I1, I2
C  ----------------------------------------------------------------------------
C
      TWOPI = 6.283185
      TWON  = REAL(2*N)
C
      CALL CMPLFT (EVEN, ODD, N, DIM)
      NT     = DIM(1)
      D2     = DIM(2)
      D3     = DIM(3)
      D4     = DIM(4)
      D5     = DIM(5)
      NOVER2 = N/2+1
C
      IF (NOVER2.LT.2) GO TO 104
      DO 103 I = 2,NOVER2
        ANGLE = TWOPI*REAL(I-1)/TWON
        CO    = COS(ANGLE)
        SI    = SIN(ANGLE)
        I0    = (I-1)*D2+1
        J     = (N+2-2*I)*D2
        DO 102 I1 = I0,NT,D3
          I2 = I1+D4
          DO 101 K = I1,I2,D5
            L       = K+J
            A       = (EVEN(L)+EVEN(K))/2.0
            C       = (EVEN(L)-EVEN(K))/2.0
            B       = (ODD(L)+ODD(K))/2.0
            D       = (ODD(L)-ODD(K))/2.0
            E       = C*SI+B*CO
            F       = C*CO-B*SI
            EVEN(K) = A+E
            EVEN(L) = A-E
            ODD(K)  = F-D
            ODD(L)  = F+D
  101          CONTINUE
  102        CONTINUE
  103      CONTINUE
C
  104      CONTINUE
      IF (N.LT.1) RETURN
      J = N*D2
      DO 105 I1 = 1,NT,D3
        I2 = I1+D4
        DO 105 K = I1,I2,D5
          L       = K+J
          EVEN(L) = EVEN(K)-ODD(K)
          ODD(L)  = 0.0
          EVEN(K) = EVEN(K)+ODD(K)
          ODD(K)  = 0.0
  105      CONTINUE
C
      RETURN
      END
C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE CMPLFT     *****                   *
C  *                                                                          *
C  *    COMPLEX FINITE DISCRETE FOURIER TRANSFORM                             *
C  *     TRANSFORMS ONE DIMENSION OF MULTI-DIMENSIONAL DATA                   *
C  *     MODIFIED BY L. F. TEN EYCK FROM A ONE-DIMENSIONAL VERSION WRITTEN    *
C  *     BY G. T. SANDE, 1969.                                                *
C  *                                                                          *
C  *    THIS PROGRAM CALCULATES THE TRANSFORM                                 *
C  *            (X(T) + I*Y(T))*(COS(2*PI*T/N) - I*SIN(2*PI*T/N))             *
C  *                                                                          *
C  *    INDEXING -- THE ARRANGEMENT OF THE MULTI-DIMENSIONAL DATA IS          *
C  *     SPECIFIED BY THE INTEGER ARRAY D, THE VALUES OF WHICH ARE USED AS    *
C  *     CONTROL PARAMETERS IN DO LOOPS.  WHEN IT IS DESIRED TO COVER ALL     *
C  *     ELEMENTS OF THE DATA FOR WHICH THE SUBSCRIPT BEING TRANSFORMED HAS   *
C  *     THE VALUE I0, THE FOLLOWING IS USED.                                 *
C  *                                                                          *
C  *             I1 = (I0 - 1)*D(2) + 1                                       *
C  *             DO 100 I2 = I1, D(1), D(3)                                   *
C  *             I3 = I2 + D(4) - 1                                           *
C  *             DO 100 I = I2, I3, D(5)                                      *
C  *                .                                                         *
C  *                .                                                         *
C  *         100 CONTINUE                                                     *
C  *                                                                          *
C  *    WITH THIS INDEXING IT IS POSSIBLE TO USE A NUMBER OF ARRANGEMENTS     *
C  *     OF THE DATA, INCLUDING NORMAL FORTRAN COMPLEX NUMBERS (D(5) = 2)     *
C  *     OR SEPARATE STORAGE OF REAL AND IMAGINARY PARTS.                     *
C  *                                                                          *
C  *    P MAX IS THE LARGEST PRIME FACTOR THAT WILL BE TOLERATED BY THIS      *
C  *     PROGRAM.                                                             *
C  *    TWO GRP IS THE LARGEST POWER OF TWO THAT IS TREATED AS A SPECIAL      *
C  *     CASE.                                                                *
C  ****************************************************************************
C
      SUBROUTINE CMPLFT (X, Y, N, D)
C
      INTEGER N, D(5)
      REAL*4  X(1), Y(1)
      LOGICAL ERROR
      INTEGER PMAX, PSYM, TWOGRP, FACTOR(15), SYM(15), UNSYM(15)
C  ----------------------------------------------------------------------------
C
      PMAX   = 19
      TWOGRP =  8
C
      IF (N.LE.1) GO TO 101
      CALL SRFP (N, PMAX, TWOGRP, FACTOR, SYM, PSYM, UNSYM, ERROR)
      IF (ERROR) GO TO 102
C
      CALL MDFTKD (N, FACTOR, D, X, Y)
      CALL DIPRP (N, SYM, PSYM, UNSYM, D, X, Y)
C
  101      CONTINUE
      RETURN
C
  102      CONTINUE
      WRITE(*,103) N
      STOP
C
  103      FORMAT (43H0INVALID NUMBER OF POINTS FOR CMPL FT.  N =,I10//)
C
      END
C  ****************************************************************************
C  *                                                                          *
C  *                    *****     SUBROUTINE SRFP     *****                   *
C  *                                                                          *
C  *    SYMMETRIZED REORDERING FACTORING SUBROUTINE                           *
C  ****************************************************************************
C
      SUBROUTINE SRFP (PTS,PMAX,TWOGRP,FACTOR,SYM,PSYM,UNSYM,ERROR)
C
      LOGICAL ERROR
      INTEGER PTS, PMAX, TWOGRP, PSYM, FACTOR(10), SYM(10), UNSYM(10)
      INTEGER PP(14), QQ(7), F, J, JJ, N, NEST, P, PTWO, Q, R
C  ----------------------------------------------------------------------------
C
      NEST = 14
C
      N    = PTS
      PSYM = 1
      F    = 2
      P    = 0
      Q    = 0
  101      CONTINUE
      IF (N.LE.1) GO TO 105
      DO 102 J = F,PMAX
        IF (N.EQ.(N/J)*J) GO TO 103
  102      CONTINUE
      GO TO 114
  103      CONTINUE
      IF (2*P+Q.GE.NEST) GO TO 115
      F = J
      N = N/F
      IF (N.EQ.(N/F)*F) GO TO 104
      Q     = Q+1
      QQ(Q) = F
      GO TO 101
  104      CONTINUE
      N     = N/F
      P     = P+1
      PP(P) = F
      PSYM  = PSYM*F
      GO TO 101
C
  105      CONTINUE
      R = 1
      IF (Q.EQ.0) R = 0
      IF (P.LT.1) GO TO 107
      DO 106 J = 1,P
        JJ         = P+1-J
        SYM(J)     = PP(JJ)
        FACTOR(J)  = PP(JJ)
        JJ         = P+Q+J
        FACTOR(JJ) = PP(J)
        JJ         = P+R+J
        SYM(JJ)    = PP(J)
  106      CONTINUE
  107      CONTINUE
      IF (Q.LT.1) GO TO 109
      DO 108 J = 1,Q
        JJ         = P+J
        UNSYM(J)   = QQ(J)
        FACTOR(JJ) = QQ(J)
  108      CONTINUE
      SYM(P+1) = PTS/PSYM**2
  109      CONTINUE
      JJ           = 2*P+Q
      FACTOR(JJ+1) = 0
      PTWO         = 1
      J            = 0
  110      CONTINUE
      J = J+1
      IF (FACTOR(J).EQ.0) GO TO 112
      IF (FACTOR(J).NE.2) GO TO 110
      PTWO      = PTWO*2
      FACTOR(J) = 1
      IF (PTWO.GE.TWOGRP) GO TO 111
      IF (FACTOR(J+1).EQ.2) GO TO 110
  111      CONTINUE
      FACTOR(J) = PTWO
      PTWO      = 1
      GO TO 110
  112      CONTINUE
      IF (P.EQ.0) R = 0
      JJ        = 2*P+R
      SYM(JJ+1) = 0
      IF (Q.LE.1) Q = 0
      UNSYM(Q+1) = 0
      ERROR      = .FALSE.
C
  113      CONTINUE
      RETURN
C
  114      CONTINUE
      WRITE(*,116) PMAX,PTS
      ERROR = .TRUE.
      GO TO 113
C
  115      CONTINUE
      WRITE(*,117) NEST,PTS
      ERROR = .TRUE.
      GO TO 113
C
  116      FORMAT (24H0LARGEST FACTOR EXCEEDS ,I3,7H.  N = ,I6,1H.)
  117      FORMAT (22H0FACTOR COUNT EXCEEDS ,I3,7H.  N = ,I6,1H.)
C
      END
C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE MDFTKD     *****                   *
C  *                                                                          *
C  *    MULTI-DIMENSIONAL COMPLEX FOURIER TRANSFORM KERNEL DRIVER             *
C  *                                                                          *
C  *      -To fix a SUBSCRIPT RANGE ERROR in EMMAP3DT, X(10),Y(10) array        *
C  *       declarations changed to X(1),Y(1); D. BELNAP/T. BAKER; 29-DEC-1993   *
C  ****************************************************************************
C
      SUBROUTINE MDFTKD (N,FACTOR,DIM,X,Y)
C
      INTEGER N, FACTOR(10), DIM(5), F, M, P, R, S
C      REAL X(10), Y(10)
      REAL X(1), Y(1)
C  ----------------------------------------------------------------------------
C
      S=DIM(2)
      F=0
      M=N
  101      CONTINUE
      F=F+1
      P=FACTOR(F)
      IF (P.EQ.0) RETURN
      M=M/P
      R=M*S
      IF (P.GT.8) GO TO 107
      GO TO (101,102,103,104,105,108,107,106), P
C
  102      CONTINUE
      CALL R2CFTK (N,M,X(1),Y(1),X(R+1),Y(R+1),DIM)
      GO TO 101
C
  103      CONTINUE
      CALL R3CFTK (N,M,X(1),Y(1),X(R+1),Y(R+1),X(2*R+1),Y(2*R+1),DIM)
      GO TO 101
C
  104      CONTINUE
      CALL R4CFTK (N,M,X(1),Y(1),X(R+1),Y(R+1),X(2*R+1),Y(2*R+1),
     1  X(3*R+1),Y(3*R+1),DIM)
      GO TO 101
C
  105      CONTINUE
      CALL R5CFTK (N,M,X(1),Y(1),X(R+1),Y(R+1),X(2*R+1),Y(2*R+1),
     1  X(3*R+1),Y(3*R+1),X(4*R+1),Y(4*R+1),DIM)
      GO TO 101
C
  106      CONTINUE
      CALL R8CFTK (N,M,X(1),Y(1),X(R+1),Y(R+1),X(2*R+1),Y(2*R+1),
     1    X(3*R+1),Y(3*R+1),X(4*R+1),Y(4*R+1),X(5*R+1),Y(5*R+1),
     2    X(6*R+1),Y(6*R+1),X(7*R+1),Y(7*R+1),DIM)
      GO TO 101
C
  107      CONTINUE
      CALL RPCFTK (N, M, P, R, X, Y, DIM)
      GO TO 101
C
  108      CONTINUE
      WRITE(*,109)
      RETURN
C
  109      FORMAT (34H0TRANSFER ERROR DETECTED IN MDFTKD,//)
C
      END
C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE R2CFTK     *****                   *
C  *                                                                          *
C  *    RADIX 2 MULTI-DIMENSIONAL COMPLEX FOURIER TRANSFORM KERNEL            *
C  *                                                                          *
C  *      -To fix a SUBSCRIPT RANGE ERROR in EMMAP3DT, X*(10),Y*(10) array      *
C  *       declarations changed to X*(1),Y*(1); D. BELNAP/T. BAKER; 29-DEC-1993 *
C  ****************************************************************************
C
      SUBROUTINE R2CFTK (N, M, X0, Y0, X1, Y1, DIM)
C
      INTEGER N, M, DIM(5)
C      REAL*4  X0(10), Y0(10), X1(10), Y1(10)
      REAL*4  X0(1), Y0(1), X1(1), Y1(1)
      LOGICAL FOLD, ZERO
      INTEGER J, K, K0, M2, MOVER2
      INTEGER K1, K2, KK, L, L1, MM2, NT, SIZE, SEP, NS
      REAL*4  ANGLE, C, IS, IU, RS, RU, S, TWOPI, FJM1, FM2
      DATA TWOPI/6.283185/
C  ----------------------------------------------------------------------------
C
      NT     = DIM(1)
      SEP    = DIM(2)
      L1     = DIM(3)
      SIZE   = DIM(4)-1
      K2     = DIM(5)
      NS     = N*SEP
      M2     = M*2
      FM2    = REAL(M2)
      MOVER2 = M/2+1
      MM2    = SEP*M2
C
      FJM1 = -1.0
      DO 108 J = 1,MOVER2
        FOLD  = J.GT.1.AND.2*J.LT.M+2
        K0    = (J-1)*SEP+1
        FJM1  = FJM1+1.0
        ANGLE = TWOPI*FJM1/FM2
        ZERO  = ANGLE.EQ.0.0
        IF (ZERO) GO TO 102
        C = COS(ANGLE)
        S = SIN(ANGLE)
        GO TO 102
  101        CONTINUE
        FOLD = .FALSE.
        K0   = (M+1-J)*SEP+1
        C    = -C
  102     CONTINUE
C
        DO 107 KK = K0,NS,MM2
          DO 106 L = KK,NT,L1
            K1 = L+SIZE
            DO 105 K = L,K1,K2
              RS    = X0(K)+X1(K)
              IS    = Y0(K)+Y1(K)
              RU    = X0(K)-X1(K)
              IU    = Y0(K)-Y1(K)
              X0(K) = RS
              Y0(K) = IS
              IF (ZERO) GO TO 103
              X1(K) = RU*C+IU*S
              Y1(K) = IU*C-RU*S
              GO TO 104
  103              CONTINUE
              X1(K) = RU
              Y1(K) = IU
  104              CONTINUE
  105            CONTINUE
  106          CONTINUE
  107        CONTINUE
        IF (FOLD) GO TO 101
  108      CONTINUE
C
      RETURN
      END
C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE R3CFTK     *****                   *
C  *                                                                          *
C  *    RADIX 3 MULTI-DIMENSIONAL COMPLEX FOURIER TRANSFORM KERNEL            *
C  *                                                                          *
C  *      -To fix a SUBSCRIPT RANGE ERROR in EMMAP3DT, X*(10),Y*(10) array      *
C  *       declarations changed to X*(1),Y*(1); D. BELNAP/T. BAKER; 29-DEC-1993 *
C  ****************************************************************************
C
      SUBROUTINE R3CFTK (N, M, X0, Y0, X1, Y1, X2, Y2, DIM)
C
      INTEGER N, M, DIM(5)
C      REAL*4  X0(10), Y0(10), X1(10), Y1(10), X2(10), Y2(10)
      REAL*4  X0(1), Y0(1), X1(1), Y1(1), X2(1), Y2(1)
      LOGICAL FOLD, ZERO
      INTEGER J, K, K0, M3, MOVER2
      INTEGER K1, K2, KK, L, L1, MM3, NT, SIZE, SEP, NS
      REAL*4  ANGLE, A, B, C1, C2, S1, S2, T, TWOPI, I0, I1, I2
      REAL*4  IA, IB, IS, R0, R1, R2, RA, RB, RS, FJM1, FM3
      DATA TWOPI/6.283185/,A/-0.5/,B/0.86602540/
C  ----------------------------------------------------------------------------
C
      NT     = DIM(1)
      SEP    = DIM(2)
      L1     = DIM(3)
      SIZE   = DIM(4)-1
      K2     = DIM(5)
      NS     = N*SEP
      M3     = M*3
      FM3    = REAL(M3)
      MM3    = SEP*M3
      MOVER2 = M/2+1
C
      FJM1 = -1.0
      DO 108 J = 1,MOVER2
        FOLD  = J.GT.1.AND.2*J.LT.M+2
        K0    = (J-1)*SEP+1
        FJM1  = FJM1+1.0
        ANGLE = TWOPI*FJM1/FM3
        ZERO  = ANGLE.EQ.0.0
        IF (ZERO) GO TO 102
        C1 = COS(ANGLE)
        S1 = SIN(ANGLE)
        C2 = C1*C1-S1*S1
        S2 = S1*C1+C1*S1
        GO TO 102
  101        CONTINUE
        FOLD = .FALSE.
        K0   = (M+1-J)*SEP+1
        T    = C1*A+S1*B
        S1   = C1*B-S1*A
        C1   = T
        T    = C2*A-S2*B
        S2   = -C2*B-S2*A
        C2   = T
  102        CONTINUE
C
        DO 107 KK = K0,NS,MM3
          DO 106 L = KK,NT,L1
            K1 = L+SIZE
            DO 105 K = L,K1,K2
              R0    = X0(K)
              I0    = Y0(K)
              RS    = X1(K)+X2(K)
              IS    = Y1(K)+Y2(K)
              X0(K) = R0+RS
              Y0(K) = I0+IS
              RA    = R0+RS*A
              IA    = I0+IS*A
              RB    = (X1(K)-X2(K))*B
              IB    = (Y1(K)-Y2(K))*B
              IF (ZERO) GO TO 103
              R1    = RA+IB
              I1    = IA-RB
              R2    = RA-IB
              I2    = IA+RB
              X1(K) = R1*C1+I1*S1
              Y1(K) = I1*C1-R1*S1
              X2(K) = R2*C2+I2*S2
              Y2(K) = I2*C2-R2*S2
              GO TO 104
  103              CONTINUE
              X1(K) = RA+IB
              Y1(K) = IA-RB
              X2(K) = RA-IB
              Y2(K) = IA+RB
  104              CONTINUE
  105            CONTINUE
  106          CONTINUE
  107        CONTINUE
        IF (FOLD) GO TO 101
  108      CONTINUE
C
      RETURN
      END
C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE R4CFTK     *****                   *
C  *                                                                          *
C  *    RADIX 4 MULTI-DIMENSIONAL COMPLEX FOURIER TRANSFORM KERNEL            *
C  *                                                                          *
C  *      -To fix a SUBSCRIPT RANGE ERROR in EMMAP3DT, X*(10),Y*(10) array      *
C  *       declarations changed to X*(1),Y*(1); D. BELNAP/T. BAKER; 29-DEC-1993 *
C  ****************************************************************************
C
      SUBROUTINE R4CFTK (N, M, X0, Y0, X1, Y1, X2, Y2, X3, Y3, DIM)
C
      INTEGER N, M, DIM(5)
C      REAL*4 X0(10), Y0(10), X1(10), Y1(10)
C      REAL*4 X2(10), Y2(10), X3(10), Y3(10)
      REAL*4 X0(1), Y0(1), X1(1), Y1(1)
      REAL*4 X2(1), Y2(1), X3(1), Y3(1)
      LOGICAL FOLD, ZERO
      INTEGER J, K, K0, M4, MOVER2
      INTEGER K1, K2, KK, L, L1, MM4, NT, SIZE, SEP, NS
      REAL*4 ANGLE, C1, C2, C3, S1, S2, S3, T, TWOPI
      REAL*4 I1, I2, I3, IS0, IS1, IU0, IU1, R1, R2, R3
      REAL*4 RS0, RS1, RU0, RU1, FJM1, FM4
      DATA TWOPI/6.283185/
C  ----------------------------------------------------------------------------
C
      NT     = DIM(1)
      SEP    = DIM(2)
      L1     = DIM(3)
      SIZE   = DIM(4)-1
      K2     = DIM(5)
      NS     = N*SEP
      M4     = M*4
      FM4    = REAL(M4)
      MM4    = SEP*M4
      MOVER2 = M/2+1
C
      FJM1 = -1.0
      DO 108 J = 1, MOVER2
        FOLD  = J.GT.1.AND.2*J.LT.M+2
        K0    = (J-1)*SEP+1
        FJM1  = FJM1+1.0
        ANGLE = TWOPI*FJM1/FM4
        ZERO  = ANGLE.EQ.0.0
        IF (ZERO) GO TO 102
        C1 = COS(ANGLE)
        S1 = SIN(ANGLE)
        C2 = C1*C1-S1*S1
        S2 = S1*C1+C1*S1
        C3 = C2*C1-S2*S1
        S3 = S2*C1+C2*S1
        GO TO 102
  101        CONTINUE
        FOLD = .FALSE.
        K0   = (M+1-J)*SEP+1
        T    = C1
        C1   = S1
        S1   = T
        C2   = -C2
        T    = C3
        C3   = -S3
        S3   = -T
  102        CONTINUE
C
        DO 107 KK = K0,NS,MM4
          DO 106 L = KK,NT,L1
            K1 = L+SIZE
            DO 105 K = L,K1,K2
              RS0   = X0(K)+X2(K)
              IS0   = Y0(K)+Y2(K)
              RU0   = X0(K)-X2(K)
              IU0   = Y0(K)-Y2(K)
              RS1   = X1(K)+X3(K)
              IS1   = Y1(K)+Y3(K)
              RU1   = X1(K)-X3(K)
              IU1   = Y1(K)-Y3(K)
              X0(K) = RS0+RS1
              Y0(K) = IS0+IS1
              IF (ZERO) GO TO 103
              R1  = RU0+IU1
              I1  = IU0-RU1
              R2  = RS0-RS1
              I2  = IS0-IS1
              R3  = RU0-IU1
              I3  = IU0+RU1
              X2(K) = R1*C1+I1*S1
              Y2(K) = I1*C1-R1*S1
              X1(K) = R2*C2+I2*S2
              Y1(K) = I2*C2-R2*S2
              X3(K) = R3*C3+I3*S3
              Y3(K) = I3*C3-R3*S3
              GO TO 104
  103              CONTINUE
              X2(K) = RU0+IU1
              Y2(K) = IU0-RU1
              X1(K) = RS0-RS1
              Y1(K) = IS0-IS1
              X3(K) = RU0-IU1
              Y3(K) = IU0+RU1
  104              CONTINUE
  105            CONTINUE
  106          CONTINUE
  107        CONTINUE
        IF (FOLD) GO TO 101
  108      CONTINUE
C
      RETURN
      END
C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE R5CFTK     *****                   *
C  *                                                                          *
C  *    RADIX 5 MULTI-DIMENSIONAL COMPLEX FOURIER TRANSFORM KERNEL            *
C  *                                                                          *
C  *      -To fix a SUBSCRIPT RANGE ERROR in EMMAP3DT, X*(10),Y*(10) array      *
C  *       declarations changed to X*(1),Y*(1); D. BELNAP/T. BAKER; 29-DEC-1993 *
C  ****************************************************************************
C
            SUBROUTINE R5CFTK (N,M,X0,Y0,X1,Y1,X2,Y2,X3,Y3,X4,Y4,DIM)
C
      INTEGER N, M, DIM(5)
C      REAL X0(10), Y0(10), X1(10), Y1(10), X2(10), Y2(10)
C      REAL X3(10), Y3(10), X4(10), Y4(10)
      REAL X0(1), Y0(1), X1(1), Y1(1), X2(1), Y2(1)
      REAL X3(1), Y3(1), X4(1), Y4(1)
      LOGICAL FOLD, ZERO
      INTEGER J, K, K0, M5, MOVER2
      INTEGER K1, K2, KK, L, L1, MM5, NT, SIZE, SEP, NS
      REAL*4 ANGLE, A1, A2, B1, B2, C1, C2, C3, C4, S1, S2, S3, S4
      REAL*4 R0, R1, R2, R3, R4, RA1, RA2, RB1, RB2, RS1, RS2
      REAL*4 RU1, RU2, I0, I1, I2, I3, I4, IA1, IA2, IB1, IB2
      REAL*4 IS1, IS2, IU1, IU2, T, TWOPI, FJM1, FM5
      DATA TWOPI/6.283185/,A1/0.30901699/,B1/0.95105652/,
     1  A2/-0.80901699/,B2/0.58778525/
C  ----------------------------------------------------------------------------
C
      NT     =  DIM(1)
      SEP    = DIM(2)
      L1     = DIM(3)
      SIZE   = DIM(4)-1
      K2     = DIM(5)
      NS     = N*SEP
      M5     = M*5
      FM5    = REAL(M5)
      MM5    = SEP*M5
      MOVER2 = M/2+1
C
      FJM1 = -1.0
      DO 108 J = 1,MOVER2
        FOLD  = J.GT.1.AND.2*J.LT.M+2
        K0    = (J-1)*SEP+1
        FJM1  = FJM1+1.0
        ANGLE = TWOPI*FJM1/FM5
        ZERO  = ANGLE.EQ.0.0
        IF (ZERO) GO TO 102
        C1 = COS(ANGLE)
        S1 = SIN(ANGLE)
        C2 = C1*C1-S1*S1
        S2 = S1*C1+C1*S1
        C3 = C2*C1-S2*S1
        S3 = S2*C1+C2*S1
        C4 = C2*C2-S2*S2
        S4 = S2*C2+C2*S2
        GO TO 102
  101        CONTINUE
        FOLD = .FALSE.
        K0   = (M+1-J)*SEP+1
        T    = C1*A1+S1*B1
        S1   = C1*B1-S1*A1
        C1   = T
        T    = C2*A2+S2*B2
        S2   = C2*B2-S2*A2
        C2   = T
        T    = C3*A2-S3*B2
        S3   = -C3*B2-S3*A2
        C3   = T
        T    = C4*A1-S4*B1
        S4   = -C4*B1-S4*A1
        C4   = T
  102        CONTINUE
C
        DO 107 KK = K0,NS,MM5
          DO 106 L = KK,NT,L1
            K1 = L+SIZE
            DO 105 K = L,K1,K2
              R0    = X0(K)
              I0    = Y0(K)
              RS1   = X1(K)+X4(K)
              IS1   = Y1(K)+Y4(K)
              RU1   = X1(K)-X4(K)
              IU1   = Y1(K)-Y4(K)
              RS2   = X2(K)+X3(K)
              IS2   = Y2(K)+Y3(K)
              RU2   = X2(K)-X3(K)
              IU2   = Y2(K)-Y3(K)
              X0(K) = R0+RS1+RS2
              Y0(K) = I0+IS1+IS2
              RA1   = R0+RS1*A1+RS2*A2
              IA1   = I0+IS1*A1+IS2*A2
              RA2   = R0+RS1*A2+RS2*A1
              IA2   = I0+IS1*A2+IS2*A1
              RB1   = RU1*B1+RU2*B2
              IB1   = IU1*B1+IU2*B2
              RB2   = RU1*B2-RU2*B1
              IB2   = IU1*B2-IU2*B1
              IF (ZERO) GO TO 103
              R1    = RA1+IB1
              I1    = IA1-RB1
              R2    = RA2+IB2
              I2    = IA2-RB2
              R3    = RA2-IB2
              I3    = IA2+RB2
              R4    = RA1-IB1
              I4    = IA1+RB1
              X1(K) = R1*C1+I1*S1
              Y1(K) = I1*C1-R1*S1
              X2(K) = R2*C2+I2*S2
              Y2(K) = I2*C2-R2*S2
              X3(K) = R3*C3+I3*S3
              Y3(K) = I3*C3-R3*S3
              X4(K) = R4*C4+I4*S4
              Y4(K) = I4*C4-R4*S4
              GO TO 104
  103              CONTINUE
              X1(K) = RA1+IB1
              Y1(K) = IA1-RB1
              X2(K) = RA2+IB2
              Y2(K) = IA2-RB2
              X3(K) = RA2-IB2
              Y3(K) = IA2+RB2
              X4(K) = RA1-IB1
              Y4(K) = IA1+RB1
  104              CONTINUE
  105            CONTINUE
  106          CONTINUE
  107        CONTINUE
        IF (FOLD) GO TO 101
  108      CONTINUE
C
      RETURN
      END
C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE R8CFTK     *****                   *
C  *                                                                          *
C  *    RADIX 8 MULTI-DIMENSIONAL COMPLEX FOURIER TRANSFORM KERNEL            *
C  *                                                                          *
C  *      -To fix a SUBSCRIPT RANGE ERROR in EMMAP3DT, X*(10),Y*(10) array      *
C  *       declarations changed to X*(1),Y*(1); D. BELNAP/T. BAKER; 29-DEC-1993 *
C  ****************************************************************************
C
      SUBROUTINE R8CFTK (N,M,X0,Y0,X1,Y1,X2,Y2,X3,Y3,X4,Y4,
     1  X5,Y5,X6,Y6,X7,Y7,DIM)
C
      INTEGER N, M, DIM(5)
      LOGICAL FOLD, ZERO
      INTEGER J, K, K0, M8, MOVER2
      INTEGER K1, K2, KK, L, L1, MM8, NT, SIZE, SEP, NS
C      REAL*4  X0(10), Y0(10), X1(10), Y1(10), X2(10), Y2(10)
C      REAL*4  X3(10), Y3(10), X4(10), Y4(10), X5(10), Y5(10)
C      REAL*4  X6(10), Y6(10), X7(10), Y7(10)
      REAL*4  X0(1), Y0(1), X1(1), Y1(1), X2(1), Y2(1)
      REAL*4  X3(1), Y3(1), X4(1), Y4(1), X5(1), Y5(1)
      REAL*4  X6(1), Y6(1), X7(1), Y7(1)
      REAL*4  ANGLE, C1, C2, C3, C4, C5, C6, C7, E
      REAL*4  S1, S2, S3, S4, S5, S6, S7, T, TWOPI, FJM1, FM8
      REAL*4  R1, R2, R3, R4, R5, R6, R7, RS0, RS1, RS2, RS3
      REAL*4  RU0, RU1, RU2, RU3, I1, I2, I3, I4, I5, I6, I7
      REAL*4  IS0, IS1, IS2, IS3, IU0, IU1, IU2, IU3
      REAL*4  RSS0, RSS1, RSU0, RSU1, RUS0, RUS1, RUU0, RUU1
      REAL*4  ISS0, ISS1, ISU0, ISU1, IUS0, IUS1, IUU0, IUU1
      DATA TWOPI/6.283185/, E/0.70710678/
C  ----------------------------------------------------------------------------
C
      NT     = DIM(1)
      SEP    = DIM(2)
      L1     = DIM(3)
      SIZE   = DIM(4)-1
      K2     = DIM(5)
      NS     = N*SEP
      M8     = M*8
      FM8    = REAL(M8)
      MM8    = SEP*M8
      MOVER2 = M/2+1
C
      FJM1 = -1.0
      DO 108 J = 1,MOVER2
        FOLD  = J.GT.1.AND.2*J.LT.M+2
        K0    = (J-1)*SEP+1
        FJM1  = FJM1+1.0
        ANGLE = TWOPI*FJM1/FM8
        ZERO  = ANGLE.EQ.0.0
        IF (ZERO) GO TO 102
        C1 = COS(ANGLE)
        S1 = SIN(ANGLE)
        C2 = C1*C1-S1*S1
        S2 = S1*C1+C1*S1
        C3 = C2*C1-S2*S1
        S3 = S2*C1+C2*S1
        C4 = C2*C2-S2*S2
        S4 = S2*C2+C2*S2
        C5 = C4*C1-S4*S1
        S5 = S4*C1+C4*S1
        C6 = C4*C2-S4*S2
        S6 = S4*C2+C4*S2
        C7 = C4*C3-S4*S3
        S7 = S4*C3+C4*S3
        GO TO 102
  101        CONTINUE
        FOLD = .FALSE.
        K0   = (M+1-J)*SEP+1
        T    = (C1+S1)*E
        S1   = (C1-S1)*E
        C1   = T
        T    = S2
        S2   = C2
        C2   = T
        T    = (-C3+S3)*E
        S3   = (C3+S3)*E
        C3   = T
        C4   = -C4
        T    = -(C5+S5)*E
        S5   = (-C5+S5)*E
        C5   = T
        T    = -S6
        S6   = -C6
        C6   = T
        T    = (C7-S7)*E
        S7   = -(C7+S7)*E
        C7   = T
  102        CONTINUE
C
        DO 107 KK=K0,NS,MM8
          DO 106 L=KK,NT,L1
            K1=L+SIZE
            DO 105 K=L,K1,K2
              RS0=X0(K)+X4(K)
              IS0=Y0(K)+Y4(K)
              RU0=X0(K)-X4(K)
              IU0=Y0(K)-Y4(K)
              RS1=X1(K)+X5(K)
              IS1=Y1(K)+Y5(K)
              RU1=X1(K)-X5(K)
              IU1=Y1(K)-Y5(K)
              RS2=X2(K)+X6(K)
              IS2=Y2(K)+Y6(K)
              RU2=X2(K)-X6(K)
              IU2=Y2(K)-Y6(K)
              RS3=X3(K)+X7(K)
              IS3=Y3(K)+Y7(K)
              RU3=X3(K)-X7(K)
              IU3=Y3(K)-Y7(K)
              RSS0=RS0+RS2
              ISS0=IS0+IS2
              RSU0=RS0-RS2
              ISU0=IS0-IS2
              RSS1=RS1+RS3
              ISS1=IS1+IS3
              RSU1=RS1-RS3
              ISU1=IS1-IS3
              RUS0=RU0-IU2
              IUS0=IU0+RU2
              RUU0=RU0+IU2
              IUU0=IU0-RU2
              RUS1=RU1-IU3
              IUS1=IU1+RU3
              RUU1=RU1+IU3
              IUU1=IU1-RU3
              T=(RUS1+IUS1)*E
              IUS1=(IUS1-RUS1)*E
              RUS1=T
              T=(RUU1+IUU1)*E
              IUU1=(IUU1-RUU1)*E
              RUU1=T
              X0(K)=RSS0+RSS1
              Y0(K)=ISS0+ISS1
              IF (ZERO) GO TO 103
              R1=RUU0+RUU1
              I1=IUU0+IUU1
              R2=RSU0+ISU1
              I2=ISU0-RSU1
              R3=RUS0+IUS1
              I3=IUS0-RUS1
              R4=RSS0-RSS1
              I4=ISS0-ISS1
              R5=RUU0-RUU1
              I5=IUU0-IUU1
              R6=RSU0-ISU1
              I6=ISU0+RSU1
              R7=RUS0-IUS1
              I7=IUS0+RUS1
              X4(K)=R1*C1+I1*S1
              Y4(K)=I1*C1-R1*S1
              X2(K)=R2*C2+I2*S2
              Y2(K)=I2*C2-R2*S2
              X6(K)=R3*C3+I3*S3
              Y6(K)=I3*C3-R3*S3
              X1(K)=R4*C4+I4*S4
              Y1(K)=I4*C4-R4*S4
              X5(K)=R5*C5+I5*S5
              Y5(K)=I5*C5-R5*S5
              X3(K)=R6*C6+I6*S6
              Y3(K)=I6*C6-R6*S6
              X7(K)=R7*C7+I7*S7
              Y7(K)=I7*C7-R7*S7
              GO TO 104
  103              CONTINUE
              X4(K)=RUU0+RUU1
              Y4(K)=IUU0+IUU1
              X2(K)=RSU0+ISU1
              Y2(K)=ISU0-RSU1
              X6(K)=RUS0+IUS1
              Y6(K)=IUS0-RUS1
              X1(K)=RSS0-RSS1
              Y1(K)=ISS0-ISS1
              X5(K)=RUU0-RUU1
              Y5(K)=IUU0-IUU1
              X3(K)=RSU0-ISU1
              Y3(K)=ISU0+RSU1
              X7(K)=RUS0-IUS1
              Y7(K)=IUS0+RUS1
  104              CONTINUE
  105            CONTINUE
  106          CONTINUE
  107        CONTINUE
        IF (FOLD) GO TO 101
  108      CONTINUE
C
      RETURN
      END
C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE RPCFTK     *****                   *
C  *                                                                          *
C  *    RADIX PRIME MULTI-DIMENSIONAL COMPLEX FOURIER TRANSFORM KERNEL        *
C  *                                                                          *
C  *      Modified 14 Jan 1994, by D. Belnap for the following reason:            *
C  *      EMMAP3DT crashed in this routine because the X and Y arrays were      *
C  *      two-dimensional and the input was one-dimensional.  I solved this     *
C  *      problem by changing X and Y to one-dimensional arrays.  This change   *
C  *      required that the index for the 1D array be computed from the 2D      *
C  *      array indices.  For 2D_ARRAY(R,P) and 1D_ARRAY(R*P)                   *
C  *                                                                          *
C  *                  NUM  =  (j - 1)R + i                              *
C  *                                                                          *
C  *      where NUM is the 1D index number and i,j the 2D indices.            *
C  *      (The 2D array code remains but is commented out.)                  *
C  ****************************************************************************
C
      SUBROUTINE RPCFTK (N, M, P, R, X, Y, DIM)
C
      IMPLICIT NONE
C
      INTEGER N, M, P, R, DIM(5)
C      REAL*4  X(R,P), Y(R,P)
      REAL*4  X(1), Y(1)
      REAL*4  ANGLE, IS, IU, RS, RU, T, TWOPI, XT, YT
      REAL*4  FU, FP, FJM1, FMP
      REAL*4  AA(9,9), BB(9, 9),  A(18), B(18), C(18), S(18)
      REAL*4  IA(9), IB(9), RA(9), RB(9)
      INTEGER J, JJ, K0, K, MOVER2, MP, PM, PP, U, V
      INTEGER K1, K2, KK, L, L1, MMP, NT, SIZE, SEP, NS
      LOGICAL FOLD, ZERO
      DATA TWOPI/6.283185/
      INTEGER*4  NUM1, NUM2
C  ----------------------------------------------------------------------------
C
      NT     = DIM(1)
      SEP    = DIM(2)
      L1     = DIM(3)
      SIZE   = DIM(4)-1
      K2     = DIM(5)
      NS     = N*SEP
      MOVER2 = M/2+1
      MP     = M*P
      FMP    = REAL(MP)
      MMP    = SEP*MP
      PP     = P/2
      PM     = P-1
      FP     = REAL(P)
      FU     = 0.0
      DO 101 U = 1,PP
        FU    =  FU+1.0
        ANGLE =  TWOPI*FU/FP
        JJ    =  P-U
        A(U)  =  COS(ANGLE)
        B(U)  =  SIN(ANGLE)
        A(JJ) =  A(U)
        B(JJ) = -B(U)
  101      CONTINUE
      DO 103 U = 1,PP
        DO 102 V = 1,PP
          JJ      = U*V-U*V/P*P
          AA(V,U) = A(JJ)
          BB(V,U) = B(JJ)
  102         CONTINUE
  103      CONTINUE
C
      FJM1 = -1.0
      DO 117 J = 1,MOVER2
        FOLD  = J.GT.1.AND.2*J.LT.M+2
        K0    = (J-1)*SEP+1
        FJM1  = FJM1+1.0
        ANGLE = TWOPI*FJM1/FMP
        ZERO  = ANGLE.EQ.0.0
        IF (ZERO) GO TO 107
        C(1) = COS(ANGLE)
        S(1) = SIN(ANGLE)
        DO 104 U = 2,PM
          C(U) = C(U-1)*C(1)-S(U-1)*S(1)
          S(U) = S(U-1)*C(1)+C(U-1)*S(1)
  104        CONTINUE
        GO TO 107
  105        CONTINUE
        FOLD = .FALSE.
        K0   = (M+1-J)*SEP+1
        DO 106 U = 1,PM
          T    =  C(U)*A(U)+S(U)*B(U)
          S(U) = -S(U)*A(U)+C(U)*B(U)
          C(U) =  T
  106        CONTINUE
  107        CONTINUE
C
        DO 116 KK = K0,NS,MMP
          DO 115 L = KK,NT,L1
            K1 = L+SIZE
            DO 114 K = L,K1,K2
C            *******************
C            *  X,Y array fix  *
C            *******************
C              XT = X(K,1)
C              YT = Y(K,1)
C            -------------------
            NUM1 = (1-1)*R + K
              XT = X(NUM1)
              YT = Y(NUM1)
C            ===================END OF FIX
C            *******************
C            *  X,Y array fix  *
C            *******************
C              RS = X(K,2)+X(K,P)
C              IS = Y(K,2)+Y(K,P)
C              RU = X(K,2)-X(K,P)
C              IU = Y(K,2)-Y(K,P)
C            ------------------
            NUM1 = (2-1)*R + K
            NUM2 = (P-1)*R + K
              RS = X(NUM1)+X(NUM2)
              IS = Y(NUM1)+Y(NUM2)
              RU = X(NUM1)-X(NUM2)
              IU = Y(NUM1)-Y(NUM2)
C            ====================END OF FIX
              DO 108 U = 1,PP
                RA(U) = XT+RS*AA(U,1)
                IA(U) = YT+IS*AA(U,1)
                RB(U) = RU*BB(U,1)
                IB(U) = IU*BB(U,1)
  108              CONTINUE
              XT = XT+RS
              YT = YT+IS
              DO 110 U = 2,PP
                JJ = P-U
C              *******************
C              *  X,Y array fix  *
C              *******************
C                RS = X(K,U+1)+X(K,JJ+1)
C                IS = Y(K,U+1)+Y(K,JJ+1)
C                RU = X(K,U+1)-X(K,JJ+1)
C                IU = Y(K,U+1)-Y(K,JJ+1)
C              -----------------------
              NUM1 = ((U+1)-1)*R + K
              NUM2 = ((JJ+1)-1)*R + K
                RS = X(NUM1)+X(NUM2)
                IS = Y(NUM1)+Y(NUM2)
                RU = X(NUM1)-X(NUM2)
                IU = Y(NUM1)-Y(NUM2)
C              ========================END OF FIX
                XT = XT+RS
                YT = YT+IS
                DO 109 V = 1,PP
                  RA(V) = RA(V)+RS*AA(V,U)
                  IA(V) = IA(V)+IS*AA(V,U)
                  RB(V) = RB(V)+RU*BB(V,U)
                  IB(V) = IB(V)+IU*BB(V,U)
  109                CONTINUE
  110              CONTINUE
C            *******************
C            *  X,Y array fix  *
C            *******************
C              X(K,1) = XT
C              Y(K,1) = YT
C            -------------------
            NUM1 = (1-1)*R + K
              X(NUM1) = XT
              Y(NUM1) = YT
C            ===================END OF FIX
              DO 113 U = 1,PP
                JJ = P-U
                IF (ZERO) GO TO 111
                XT        = RA(U)+IB(U)
                YT        = IA(U)-RB(U)
C              *******************
C              *  X,Y array fix  *
C              *******************
C                X(K,U+1)  = XT*C(U)+YT*S(U)
C                Y(K,U+1)  = YT*C(U)-XT*S(U)
C              ---------------------------
              NUM1 = ((U+1)-1)*R + K
                X(NUM1) = XT*C(U)+YT*S(U)
                Y(NUM1) = YT*C(U)-XT*S(U)
C              ========================END OF FIX
                XT        = RA(U)-IB(U)
                YT        = IA(U)+RB(U)
C              *******************
C              *  X,Y array fix  *
C              *******************
C                X(K,JJ+1) = XT*C(JJ)+YT*S(JJ)
C                Y(K,JJ+1) = YT*C(JJ)-XT*S(JJ)
C              -----------------------------
              NUM1 = ((JJ+1)-1)*R + K
                X(NUM1) = XT*C(JJ)+YT*S(JJ)
                Y(NUM1) = YT*C(JJ)-XT*S(JJ)
C              ===========================END OF FIX
                GO TO 112
  111                CONTINUE
C              *******************
C              *  X,Y array fix  *
C              *******************
C                X(K,U+1)  = RA(U)+IB(U)
C                Y(K,U+1)  = IA(U)-RB(U)
C                X(K,JJ+1) = RA(U)-IB(U)
C                Y(K,JJ+1) = IA(U)+RB(U)
C              -----------------------
              NUM1    = ((U+1)-1)*R + K
              NUM2    = ((JJ+1)-1)*R + K
                X(NUM1) = RA(U)+IB(U)
                Y(NUM1) = IA(U)-RB(U)
                X(NUM2) = RA(U)-IB(U)
                Y(NUM2) = IA(U)+RB(U)
C              =======================END OF FIX
  112                CONTINUE
  113              CONTINUE
  114            CONTINUE
  115          CONTINUE
  116        CONTINUE
        IF (FOLD) GO TO 105
  117      CONTINUE
C
      RETURN
C
      END
C  ****************************************************************************
C  *                                                                          *
C  *                   *****     SUBROUTINE DIPRP     *****                   *
C  *                                                                          *
C  *    DOUBLE IN PLACE REORDERING PROGRAMME                                  *
C  *                                                                          *
C  *      -To fix a SUBSCRIPT RANGE ERROR in EMMAP3DT, X(10),Y(10) array        *
C  *       declarations changed to X(1),Y(1); D. BELNAP/T. BAKER; 29-DEC-1993   *
C  ****************************************************************************
C
      SUBROUTINE DIPRP (PTS, SYM, PSYM, UNSYM, DIM, X, Y)
C
      INTEGER PTS, PSYM
C      REAL*4  X(10), Y(10), T
      REAL*4  X(1), Y(1), T
      INTEGER SYM(10), UNSYM(10), DIM(5), MODULO(14)
      INTEGER DK, JJ, KK, LK, MODS, MULT, NEST, PUNSYM, TEST
      INTEGER NT, SEP, DELTA, P, P0, P1, P2, P3, P4, P5, SIZE
      INTEGER S(14), U(14)
      INTEGER A, B, C, D, E, F, G, H, I, J, K, L, M, N
      INTEGER BS, CS, DS, ES, FS, GS, HS, IS, JS, KS, LS, MS, NS
      INTEGER AL, BL, CL, DL, EL, FL, GL, HL, IL, JL, KL, LL, ML, NL
      LOGICAL ONEMOD
      EQUIVALENCE (AL,U(1)), (BS,S(2)), (BL,U(2))
      EQUIVALENCE (CS,S(3)), (CL,U(3)), (DS,S(4)), (DL,U(4))
      EQUIVALENCE (ES,S(5)), (EL,U(5)), (FS,S(6)), (FL,U(6))
      EQUIVALENCE (GS,S(7)), (GL,U(7)), (HS,S(8)), (HL,U(8))
      EQUIVALENCE (IS,S(9)), (IL,U(9)), (JS,S(10)), (JL,U(10))
      EQUIVALENCE (KS,S(11)), (KL,U(11)), (LS,S(12)), (LL,U(12))
      EQUIVALENCE (MS,S(13)), (ML,U(13)), (NS,S(14)), (NL,U(14))
C  ----------------------------------------------------------------------------
C
      NEST = 14
      NT   = DIM(1)
      SEP  = DIM(2)
      P2   = DIM(3)
      SIZE = DIM(4)-1
      P4   = DIM(5)
      IF (SYM(1).EQ.0) GO TO 106
      DO 101 J = 1,NEST
        U(J) = 1
        S(J) = 1
  101      CONTINUE
      N = PTS
      DO 102 J = 1,NEST
        IF (SYM(J).EQ.0) GO TO 103
        JJ    = NEST+1-J
        U(JJ) = N
        S(JJ) = N/SYM(J)
        N     = N/SYM(J)
  102      CONTINUE
  103      CONTINUE
C
      JJ = 0
      DO 105 A = 1,AL
      DO 105 B = A,BL,BS
      DO 105 C = B,CL,CS
      DO 105 D = C,DL,DS
      DO 105 E = D,EL,ES
      DO 105 F = E,FL,FS
      DO 105 G = F,GL,GS
      DO 105 H = G,HL,HS
      DO 105 I = H,IL,IS
      DO 105 J = I,JL,JS
      DO 105 K = J,KL,KS
      DO 105 L = K,LL,LS
      DO 105 M = L,ML,MS
      DO 105 N = M,NL,NS
        JJ = JJ+1
        IF (JJ.GE.N) GO TO 105
        DELTA = (N-JJ)*SEP
        P1    = (JJ-1)*SEP+1
        DO 104 P0 = P1,NT,P2
          P3 = P0+SIZE
          DO 104 P = P0,P3,P4
            P5    = P+DELTA
            T     = X(P)
            X(P)  = X(P5)
            X(P5) = T
            T     = Y(P)
            Y(P)  = Y(P5)
            Y(P5) = T
  104        CONTINUE
  105      CONTINUE
C
  106      CONTINUE
      IF (UNSYM(1).EQ.0) GO TO 120
      PUNSYM = PTS/PSYM**2
      MULT   = PUNSYM/UNSYM(1)
      TEST   = (UNSYM(1)*UNSYM(2)-1)*MULT*PSYM
      LK     = MULT
      DK     = MULT
      DO 107 K = 2,NEST
        IF (UNSYM(K).EQ.0) GO TO 108
        LK   = LK*UNSYM(K-1)
        DK   = DK/UNSYM(K)
        U(K) = (LK-DK)*PSYM
        MODS = K
  107      CONTINUE
  108      CONTINUE
      ONEMOD = MODS.LT.3
      IF (ONEMOD) GO TO 110
      DO 109 J = 3,MODS
        JJ         = MODS+3-J
        MODULO(JJ) = U(J)
  109      CONTINUE
  110      CONTINUE
      MODULO(2) = U(2)
      JL        = (PUNSYM-3)*PSYM
      MS        = PUNSYM*PSYM
C
      DO 119 J = PSYM,JL,PSYM
        K = J
C
  111        CONTINUE
        K = K*MULT
        IF (ONEMOD) GO TO 113
        DO 112 I = 3,MODS
          K = K-(K/MODULO(I))*MODULO(I)
  112        CONTINUE
  113        CONTINUE
        IF (K.GE.TEST) GO TO 114
        K = K-(K/MODULO(2))*MODULO(2)
        GO TO 115
  114        CONTINUE
          K = K-(K/MODULO(2))*MODULO(2)+MODULO(2)
  115        CONTINUE
        IF (K.LT.J) GO TO 111
C
        IF (K.EQ.J) GO TO 118
        DELTA = (K-J)*SEP
        DO 117 L = 1,PSYM
          DO 116 M = L,PTS,MS
            P1 = (M+J-1)*SEP+1
            DO 116 P0 = P1,NT,P2
              P3 = P0+SIZE
              DO 116 JJ = P0,P3,P4
                KK    = JJ+DELTA
                T     = X(JJ)
                X(JJ) = X(KK)
                X(KK) = T
                T     = Y(JJ)
                Y(JJ) = Y(KK)
                Y(KK) = T
  116          CONTINUE
  117        CONTINUE
  118        CONTINUE
  119      CONTINUE
C
  120      CONTINUE
      RETURN
      END
