C  ********************************************************************
C  *                                                                  *
C  *              *****     SUBROUTINE R4_IMG_FFT     *****              *
C  *                                                                  *
C  *    LOAD NCOL X NROW "IMAGE" INTO "FFT" ARRAY AND COMPUTE FFT.    *
C  *                                                                  *
C  *    JUNE 1991  PURDUE UNIVERSITY VAX 8550, T. BAKER.              *
C  *    LAST MODIFIED 30-NOV-1999 BY TSB (Get rid of FFT_IEND).       *
C  ********************************************************************
C
      SUBROUTINE R4_IMG_FFT (IMG, NCOL, NROW, FFT, IDIM1, IDIM2,
     $     IMG_BKG)

      IMPLICIT NONE

      INTEGER*4 NCOL, NROW, IDIM1, IDIM2
      real*4    IMG_BKG, IMG(NCOL,NROW)
      REAL*4    FFT(2*IDIM1,IDIM2)

      CALL R4_IMG_FFT_FILL (IMG, NCOL, NROW, FFT, IDIM1, IDIM2, 
     $     IMG_BKG)
      CALL FFT_2D (FFT, IDIM1, IDIM2)

      RETURN
      END


C  ********************************************************************
C  *                                                                  *
C  *            *****     SUBROUTINE R4_IMG_FFT_FILL     *****           *
C  *                                                                  *
C  *    LOAD NCOL X NROW "IMAGE" INTO "FFT" ARRAY TO COMPUTE FFT      *
C  *                                                                  *
C  *    BEWARE: THE DATA ***MUST*** BE PROPERLY FLOATED BEFORE THE    *
C  *            DATA ARE FOURIER TRANSFORMED.                         *
C  *                                                                  *
C  *    MAY 1987  PURDUE UNIVERSITY VAX 11/750, T. BAKER.             *
C  *    DECEMBER 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.          *
C  *    LAST MODIFIED 30-NOV-1999 BY TSB (Get rid of FFT_IEND).       *
C  ********************************************************************
C
      SUBROUTINE R4_IMG_FFT_FILL (IMG, NCOL, NROW, FFT, IDIM1, IDIM2,
     1 IMG_BKG)

      IMPLICIT NONE

      INTEGER*4 NCOL, NROW, IDIM1, IDIM2, I, J
      real*4    IMG_BKG, IMG(NCOL,NROW), FFT(2*IDIM1,IDIM2)

      FFT = 0.0 ! Fortran 90 array syntax

      DO J = 1,NROW
        DO I = 1,NCOL
          FFT((2*I)-1,J) = IMG(I,J) - IMG_BKG         ! FLOATING OCCURS HERE
        ENDDO
      ENDDO

      RETURN
      END



C  ****************************************************************************
C  *                                                                          *
C  *                 *****     SUBROUTINE R4_IMG_SHIFT     *****              *
C  *                                                                          *
C  *    SHIFT DATA WITHIN IMG BY XSHIFT,YSHIFT.                               *
C  *                                                                          *
C  *    POINT LAYOUT FOR BI-LINEAR INTERPOLATION SCHEME :                     *
C  *                                                                          *
C  *          D4            D3                                                *
C  *                                                                          *
C  *                    + (dx,dy)                                             *
C  *                    .                                                     *
C  *          D1. . . . .   D2                                                *
C  *                                                             *
C  *    THE INTERPOLATION ALGORITHM IS COMPUTED AS FOLLOWS:                   *
C  *                                                                          *
C  *          D1(1-dX)(1-dY) + D2(dX)(1-dY) + D3(dX)(dY) + D4(1-dX)(dY)       *
C  *                                                                          *
C  *    WHICH REDUCES TO:                                                     *
C  *                                                                          *
C  *          D1 + (D2-D1)dx + (D4-D1)dy + (D1-D2+D3-D4)dxdy                  *
C  *                                                                          *
C  *    FEBRUARY 1989 PURDUE UNIVERSITY VAX 8550, T. BAKER                    *
C  ****************************************************************************
C

      SUBROUTINE R4_IMG_SHIFT (IMG, NCOL, NROW, XSHIFT, YSHIFT,
     1   IMG_MIN, IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG)
C
      IMPLICIT NONE
C
      REAL*4    IMG_AVG, IMG_STD
      REAL*4    X, Y, DX, DY, XSHIFT, YSHIFT, D1, D2, D3, D4
      INTEGER*4 NCOL, NROW, I, J, IX1, IX2, IY1, IY2
      real*4    IMG_MIN, IMG_MAX
      real*4    IMG_BKG, IMG(NCOL,NROW)
      LOGICAL   XSTART, XEND, YSTART, YEND

      real*4, allocatable :: IMG2(:,:)
      INTEGER   ALLOC_STAT

      allocate(img2(ncol,nrow), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate array IMG2 in R4_IMG_SHIFT'
      endif

      DO J = 1,NROW
        Y   = J - YSHIFT
        IY1 = INT(Y)
        IY2 = IY1 + 1
        DY  = Y - IY1
        IF(IY1.GE.1 .AND. IY1.LE.NROW) THEN
           YSTART = .TRUE.
         ELSE
           YSTART = .FALSE.
        ENDIF
        IF(IY2.GE.1 .AND. IY2.LE.NROW) THEN
           YEND = .TRUE.
         ELSE
           YEND = .FALSE.
        ENDIF
        DO I = 1,NCOL
          X   = I - XSHIFT
          IX1 = INT(X)
          IX2 = IX1 + 1
          DX  = X - IX1
          IF(IX1.GE.1 .AND. IX1.LE.NCOL) THEN
             XSTART = .TRUE.
           ELSE
             XSTART = .FALSE.
          ENDIF
          IF(IX2.GE.1 .AND. IX2.LE.NCOL) THEN
             XEND = .TRUE.
           ELSE
             XEND = .FALSE.
          ENDIF
          IF(XSTART.AND.YSTART) THEN
             D1 = IMG(IX1,IY1)
           ELSE
             D1 = IMG_BKG
          ENDIF
          IF(XEND.AND.YSTART) THEN
             D2 = IMG(IX2,IY1)
           ELSE
             D2 = IMG_BKG
          ENDIF
          IF(XEND.AND.YEND) THEN
             D3 = IMG(IX2,IY2)
           ELSE
             D3 = IMG_BKG
          ENDIF
          IF(XSTART.AND.YEND) THEN
             D4 = IMG(IX1,IY2)
           ELSE
             D4 = IMG_BKG
          ENDIF

          IMG2(I,J) = D1 + (D2-D1)*DX + (D4-D1)*DY
     1                  + (D1-D2+D3-D4)*DX*DY

        ENDDO
      ENDDO
C  ********************************
C  *    COPY "IMG2" INTO "IMG"    *
C  ********************************
      img(1:ncol,1:nrow) = img2(1:ncol,1:nrow) ! Fortran 90 syntax

      CALL R4_IMG_STATS (IMG, NCOL, NROW, IMG_MIN, IMG_MAX, IMG_AVG,
     1   IMG_STD, IMG_BKG)
C
      deallocate(img2)

      RETURN
      END

C  *******************************************************************
C  *                                                                 *
C  *             *****     SUBROUTINE R4_IMG_STATS     *****            *
C  *                                                                 *
C  *    DETERMINE IMG_BKG, IMG_MIN, IMG_MAX, IMG_AVG AND IMG_STD     *
C  *     OF "IMAGE" VALUES.                                          *
C  *                                                                 *
C  *    NOTE THAT THE VALUE DETERMINED FOR IMG_MIN IS THE MINIMUM    *
C  *     NON-ZERO VALUE IN THE IMG ARRAY                             *
C  *                                                                 *
C  *    APRIL 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER           *
C  *    AUGUST 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.           *
C  *******************************************************************
C
      SUBROUTINE R4_IMG_STATS (IMG, NCOL, NROW, IMG_MIN, IMG_MAX,
     1    IMG_AVG, IMG_STD, IMG_BKG)
C
      IMPLICIT NONE

      REAL*4    IMG_AVG, IMG_STD, TOT, SUMSQ, VAR, X
      real*4    IMG_MIN, IMG_MAX
      INTEGER*4 NCOL, NROW, ARRAY_SIZE, I
      real*4    IMG(1), IMG_BKG
C  ----------------------------------------------------------------------------
C
      ARRAY_SIZE = NCOL*NROW
      img_min = img(1)
      img_max = img(1)
      SUMSQ      =    0.0
      TOT        =    0.0
C
      WRITE(*,1000) 
C
      DO I = 1,ARRAY_SIZE
        X          = IMG(I)
        SUMSQ      = SUMSQ + X*X
        TOT        = TOT + X
        IF(IMG(I).NE.0) THEN
          IMG_MIN = MIN(IMG_MIN,IMG(I))
          IMG_MAX = MAX(IMG_MAX,IMG(I))
        ENDIF
      ENDDO
C
      IMG_AVG = TOT/REAL(ARRAY_SIZE)
      IMG_STD = 0.0
      VAR     = (SUMSQ/REAL(ARRAY_SIZE)) - IMG_AVG**2
      IF(VAR.GE.0.0) IMG_STD = SQRT(VAR)
C  *************************
C  *    COMPUTE IMG_BKG    *
C  *************************
      TOT = 0.0
      DO I = 1,NCOL                                    ! BOTTOM ROW
        TOT = TOT + IMG(I)
      ENDDO
      DO I = ARRAY_SIZE-NCOL+1,ARRAY_SIZE                  ! TOP ROW
        TOT = TOT + IMG(I)
      ENDDO
      DO I = NCOL+1,ARRAY_SIZE-NCOL,NCOL                  ! LEFT COLUMN
        TOT = TOT + IMG(I)
      ENDDO
      DO I = 2*NCOL,ARRAY_SIZE-NCOL,NCOL                  ! RIGHT COLUMN
        TOT = TOT + IMG(I)
      ENDDO
      IMG_BKG = TOT/(2*NCOL+2*(NROW-2))
C
      WRITE(*,1010) IMG_MIN,IMG_MAX,IMG_AVG,IMG_STD,IMG_BKG
      RETURN
C
 1000      FORMAT(
     1 /,' *****************************************************',
     2 /,' *   NON-ZERO                                        *',
     3 /,' *   MINIMUM   MAXIMUM   AVERAGE        STD     BKG  *',
     4 /,' * ------------------------------------------------- *',
     5 /,' *            ...  WAIT for answer  ...              *')
 1010      FORMAT('+*',f8.1,f10.1,2F12.2,f7.1,'  *',
     1 /,' *****************************************************',/)
C
      END


C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE R4_IMG_SYM     *****                  *
C  *                                                                          *
C  *    APPLY NROT ROTATIONAL SYMMETRY TO IMAGE AT XCEN, YCEN                 *
C  *                                                                          *
C  *    IMG1 AND IMG2 ARE THE VIRTUAL ARRAYS CONTAINING THE ORIGINAL AND      *
C  *     SYMMETRIZED IMAGE DATA RESPECTIVELY                                  *
C  *                                                                          *
C  *    LIMIT ON NROT = 100, IF NROT>100 THEN COMPUTE CIRCULAR AVERAGE        *
C  *                                                                          *
C  *    DECEMBER 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER.                *
C  *    DECEMBER 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.                  *
C  ****************************************************************************
C
      SUBROUTINE R4_IMG_SYM (IMG1, NCOL, NROW, XCEN, YCEN, NROT,
     1     IMG_BKG, rint, numAlloc)
C
      IMPLICIT NONE
C
      INTEGER*4 NROT, NCOL, NROW, numAlloc
      real*4    IMG_BKG
      real*4    IMG1(1)
      real*4, allocatable :: IMG2(:)
      REAL*4    XCEN, YCEN, rint(numAlloc)
      INTEGER   ALLOC_STAT
C  ----------------------------------------------------------------------------

      allocate(img2(ncol*nrow), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate array IMG2 in R4_IMG_AVERAGEFLT'
      endif

      IF(NROT.GT.100) THEN
         CALL R4_IMG_SYM_CAVG (IMG1, IMG2, NCOL, NROW, XCEN, YCEN,
     1        rint, numAlloc)
       ELSE
         CALL R4_IMG_SYM_RAVG (IMG1, IMG2, NCOL, NROW, XCEN, YCEN, NROT,
     1      IMG_BKG)
      ENDIF
C
      img1(1:ncol*nrow) = img2(1:ncol*nrow) ! Fortran 90 syntax
      deallocate(img2)

      RETURN
      END

C  *************************************************************
C  *                                                           *
C  *        *****     SUBROUTINE R4_IMG_SYM_CAVG     *****        *
C  *                                                           *
C  *    NROT > 100: COMPUTE CIRCULARLY AVERAGED IMAGE          *
C  *************************************************************
C
      SUBROUTINE R4_IMG_SYM_CAVG (IMG1, IMG2, NCOL, NROW, XCEN, YCEN, 
     $     rint, numAlloc)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, numAlloc
      INTEGER*4 INUM, I, J, IR
      real*4    INTERP
      real*4    IRNUM(10000)
      real*4    IMG1(NCOL,NROW), IMG2(NCOL,NROW)
      REAL*4    XCEN, YCEN, X, Y, YSQ, rint(numAlloc)
      PARAMETER (INTERP=2)
C  ----------------------------------------------------------------------------
C
      INUM = NINT(INTERP*SQRT(REAL(NROW)**2+REAL(NCOL)**2))
      DO I = 1,INUM            ! INITIALIZE 1D INTERPOLATION ARRAY
        RINT(I)  = 0.0
        IRNUM(I) = 0.0
      ENDDO
C  *************************************************************
C  *    STEP THROUGH IMAGE AND REDUCE TO 1D CIRCULAR AVERAGE   *
C  *************************************************************
      DO J = 1,NROW
        Y   = REAL(J)-YCEN
        YSQ = Y*Y
        DO I = 1,NCOL
          X         = REAL(I)-XCEN
          IR        = nint(INTERP*SQRT(X*X+YSQ))+1
          IRNUM(IR) = IRNUM(IR)+1.0
          RINT(IR)  = RINT(IR)+IMG1(I,J)
        ENDDO
      ENDDO
C  ********************************
C  *    NORMALIZE THE 1D ARRAY    *
C  ********************************
      DO I = 1,INUM
        IF(IRNUM(I).GT.1.0) RINT(I) = RINT(I)/IRNUM(I)
      ENDDO
C  **************************************************
C  *    FILL IMG2 WITH CIRCULARLY AVERAGED IMAGE    *
C  **************************************************
      DO J = 1,NROW
        Y   = REAL(J)-YCEN
        YSQ = Y*Y
        DO I = 1,NCOL
          X         = REAL(I)-XCEN
          IR        = nint(INTERP*SQRT(X*X+YSQ))+1
          IMG2(I,J) = RINT(IR)
        ENDDO
      ENDDO

      RETURN
      END

C  ***************************************************************
C  *                                                             *
C  *        *****     SUBROUTINE R4_IMG_SYM_RAVG     *****          *
C  *                                                             *
C  *    NROT < 101: COMPUTE ROTATIONAL AVERAGE OF IMAGE          *
C  *                                                             *
C  *    LAST MODIFIED 27-JUN-1991 BY T. BAKER.                   *
C  ***************************************************************
C
      SUBROUTINE R4_IMG_SYM_RAVG (IMG1, IMG2, NCOL, NROW, XCEN, YCEN,
     1   NROT, IMG_BKG)
C
      IMPLICIT NONE
C
      REAL*4    XCEN, YCEN, A11, A12, A21, A22, DENS, D1, D2, D3, D4
      REAL*4    DX, DY, TOT, XGRID(100), YGRID(100)
      INTEGER*4 NROT, NCOL, NROW, I, J, K
      real*4    IMG_BKG,IMG1(NCOL,NROW), IMG2(NCOL,NROW)
      real*8    deg_to_rad
      INTEGER*4 IX, IY, IOUT, NUM
      INTEGER*4 NGRID, ITST, ILST
      DATA IOUT/-9999/
C  ----------------------------------------------------------------------------
C
C  ********************************
C  *    SET UP ROTATION MATRIX    *
C  ********************************
      deg_to_rad = acos(-1.0d0)/180.0d0
      A11 = REAL(COS(deg_to_rad*360.0/REAL(NROT)))
      A12 = REAL(SIN(deg_to_rad*360.0/REAL(NROT)))
      A21 = -A12
      A22 =  A11
C
      DO J = 1,NCOL                  ! STEP THROUGH COLUMNS OF IMG1
        DO I = 1,NROW                  ! STEP THROUGH CURRENT COLUMN OF IMG1
          CALL SYM_GRID (I, J, NROT, XCEN, YCEN, XGRID, YGRID,
     $          A11, A12, A21, A22)
C           **********************************************************
C           *   CHECK NUMBER OF GRID POINTS WITHIN IMAGE BOUNDARY    *
C           **********************************************************
          NGRID = 1
          DO K = 2,NROT
            IF(XGRID(K).GE.1.0.AND.XGRID(K).LE.NCOL.AND.
     1        YGRID(K).GE.1.0.AND.YGRID(K).LE.NROW) THEN
               NGRID = NGRID+1
             ELSE
               XGRID(K) = -1.0
               YGRID(K) = -1.0
            ENDIF
          ENDDO
          IF(NGRID.EQ.1) THEN
             IMG2(J,I) = IOUT
           ELSE
             DENS  = IMG1(J,I)
             NGRID = 1
             DO K = 2,NROT
               IX = INT(XGRID(K))
               IY = INT(YGRID(K))
               IF(IX.GE.1.AND.IY.GE.1) THEN
                 NGRID = NGRID+1
                 D2   = IMG_BKG
                 D3   = IMG_BKG
                 D4   = IMG_BKG
                 D1   = IMG1(IX,IY)
                 IF(IX.LT.NCOL) D2 = IMG1(IX+1,IY)
                 IF(IY.LT.NROW.AND.IX.LT.NCOL) D3 = IMG1(IX+1,IY+1)
                 IF(IY.LT.NROW) D4 = IMG1(IX,IY+1)
                 DX   = XGRID(K)-IX
                 DY   = YGRID(K)-IY
                 DENS = DENS+D1+DX*(D2-D1)+DY*(D4-D1)+
     1                      DX*DY*(D1-D2+D3-D4)
               ENDIF
               ENDDO
             IMG2(J,I) = NINT(DENS/REAL(NGRID))
          ENDIF
        ENDDO
      ENDDO
C  *********************************************************
C  *    COMPUTE BACKGROUND VALUE FROM POINTS ON BOUNDARY   *
C  *********************************************************
      PRINT *, '*******************************************'
      PRINT *, '* Compute BACKGROUND of symmetrized image *'
      PRINT *, '*******************************************'
      TOT  = 0.0
      NUM  =   0
      DO J = 1,NROW                              ! STEP THROUGH ROW BY ROW
        ITST = -1                                   ! OUTSIDE BOUNDARY
        ILST =  0
        DO I = 1,NCOL                                 ! STEP THROUGH ROW J
          IF(IMG2(I,J).EQ.IOUT) THEN                       ! OUTSIDE BOUNDARY
             IF(ITST.EQ.1.AND.I.NE.ILST) THEN            ! JUST LEAVING SYM AREA
               NUM = NUM+1
               TOT = TOT+IMG1(I,J)                  ! PIXEL AT RIGHT BOUNDARY
               ITST = -1                         ! NOW OUTSIDE BOUNDARY
               GO TO 100                         ! STEP TO NEXT ROW (J+1)
             ENDIF
             ITST = -1                             ! OUTSIDE BOUNDARY
           ELSE                                    ! INSIDE BOUNDARY
             IF(ITST.EQ.-1.AND.I.GT.1) THEN           ! LEAVING OUTSIDE/ENTERING
               NUM  = NUM+1                       !         INSIDE
               ILST = I+1
               TOT  = TOT+IMG1(I-1,J)        ! PREVIOUS PIXEL WAS ON LEFT BOUNDARY
             ENDIF
             ITST = 1                                    ! INSIDE BOUNDARY
          ENDIF
        ENDDO
        IF(ILST.EQ.0) THEN
C           **********************************************
C           *    GET HERE ONLY WHEN ENTIRE ROW IS ALL    *
C           *     INSIDE OR ALL OUTSIDE THE BOUNDARY     *
C           **********************************************
          IF(ITST.EQ.1) THEN                     ! ENTIRE ROW INSIDE BOUNDARY
            IF(NUM.EQ.0) THEN                   ! FIRST ROW INSIDE BOUNDARY USES
               DO I = 1,NCOL                   !      ENTIRE ROW FOR BACKGROUND
                 NUM = NUM + 1
                 TOT = TOT + IMG1(I,J)
               ENDDO
             ELSE                         ! USE EDGE VALUES FOR BACKGROUND
               NUM = NUM + 2            
               TOT = TOT + IMG1(1,J) + IMG1(NCOL,J)
            ENDIF
          ENDIF
          IF(ITST.EQ.-1) THEN                    ! ENTIRE ROW OUTSIDE BOUNDARY
            IF(NUM.NE.0) THEN                             ! THIS IS LAST ROW
              DO I = 1,NCOL
                NUM = NUM + 1
                TOT = TOT + IMG1(I,J-1)
              ENDDO
            ENDIF
          ENDIF
        ENDIF
 100    continue
      ENDDO
      IMG_BKG = TOT/REAL(NUM)
C  ************************************************
C  *    REPLACE ALL VALUES = IOUT WITH IMG_BKG    *
C  ************************************************
      DO J = 1,NROW
        DO I = 1,NCOL
          IF(IMG2(I,J).EQ.IOUT) IMG2(I,J) = IMG_BKG
        ENDDO
      ENDDO

      RETURN
      END


C  *************************************************************
C  *                                                           *
C  *        *****     SUBROUTINE SYM_GRID         *****        *
C  *                                                           *
C  *    FIND POSITIONS OF SYMMETRY RELATED POINTS.             *
C  *    LIMIT: NROT<101                                        *
C  *                                                           *
C  *    MAY 1992  PURDUE UNIVERSITY VAX 8550, T. BAKER.        *
C  *    LAST MODIFIED 25-MAY-1992 BY T. BAKER.                 *
C  *************************************************************
C
      SUBROUTINE SYM_GRID(I, J, NROT, XCEN, YCEN, XGRID, YGRID,
     $     A11, A12, A21, A22)
      
      IMPLICIT NONE

      REAL*4    XCEN, YCEN, XGRID(100), YGRID(100)
      REAL*4    A11, A12, A21, A22
      INTEGER*4 I, J, K, NROT

      XGRID(1) = J                              ! CURRENT COLUMN POSITION
      YGRID(1) = I                              ! CURRENT ROW POSITION
      DO K = 2,NROT
        XGRID(K) = A11*(XGRID(K-1)-XCEN)+A12*(YGRID(K-1)-YCEN)+XCEN
        YGRID(K) = A21*(XGRID(K-1)-XCEN)+A22*(YGRID(K-1)-YCEN)+YCEN
      ENDDO

      RETURN
      END
