C  ********************************************************************
C  *                                                                  *
C  *              *****     SUBROUTINE MAP_FFT     *****              *
C  *                                                                  *
C  *    LOAD NCOL X NROW "MAP" INTO "FFT" ARRAY AND COMPUTE FFT.      *
C  *                                                                  *
C  *    JUNE 1991  PURDUE UNIVERSITY VAX 8550, T. BAKER.              *
C  *    LAST MODIFIED 30-NOV-1999 BY TSB (Get rid of FFT_IEND).       *
C  ********************************************************************
C
      SUBROUTINE MAP_FFT(MAP, NCOL, NROW, NSEC, SEC, FFT, IDIM1, IDIM2)

      IMPLICIT NONE

      INTEGER*4 NCOL, NROW, NSEC, SEC, IDIM1, IDIM2
      REAL*4    MAP(NCOL,NROW,NSEC), FFT(2*IDIM1,IDIM2)

      CALL MAP_FFT_FILL(MAP, NCOL, NROW, NSEC, SEC, FFT, IDIM1, IDIM2)
      CALL FFT_2D(FFT, IDIM1, IDIM2)

      RETURN
      END

C  **********************************************************************
C  *                                                                    *
C  *             *****     SUBROUTINE MAP_FFT_FILL     *****            *
C  *                                                                    *
C  *    LOAD NCOL X NROW "MAP" SECTION INTO "FFT" ARRAY TO COMPUTE FFT  *
C  *                                                                    *
C  *    BEWARE: IT IS ***IMPORTANT*** THAT THE DATA BE PROPERLY FLOATED *
C  *            BEFORE THE DATA ARE FOURIER TRANSFORMED.                *
C  *                                                                    *
C  *    MAY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER                  *
C  *    LAST MODIFIED 30-NOV-1999 BY TSB (Get rid of FFT_IEND).         *
C  **********************************************************************
C
      SUBROUTINE MAP_FFT_FILL(MAP, NCOL, NROW, NSEC, SEC, FFT, 
     $     IDIM1, IDIM2)

      IMPLICIT NONE

      INTEGER*4 NCOL, NROW, NSEC, SEC, IDIM1, IDIM2, I, J
      REAL*4    MAP(NCOL,NROW,NSEC), FFT(2*IDIM1,IDIM2), TMP

      FFT = 0.0 ! Fortran 90 array syntax

      DO J = 1,NROW
        DO I = 1,NCOL
          TMP            = MAP(I,J,SEC)
          FFT((2*I)-1,J) = TMP
        ENDDO
      ENDDO

      RETURN
      END

C  ****************************************************************************

      SUBROUTINE MAP_GET_ANGLE (T_MATRIX, A_CELL, B_CELL, C_CELL,
     1 MAP_ABANG, ALP, BET, GAM)

      IMPLICIT NONE

      INTEGER*4 T_MATRIX(2,3)
      REAL*4    V1, V2, A_CELL, B_CELL, C_CELL, MAP_ABANG
      REAL*4    ALP, BET, GAM, cosalp, cosbet, cosgam
      real*8    deg_to_rad, rad_to_deg

      deg_to_rad = acos(-1.0d0)/180.0d0
      rad_to_deg = 1.0d0/deg_to_rad
      cosalp = real(cos(deg_to_rad*alp))
      cosbet = real(cos(deg_to_rad*bet))
      cosgam = real(cos(deg_to_rad*gam))
      
      V1 = SQRT( (T_MATRIX(1,1)*A_CELL)**2 + 
     1          (T_MATRIX(1,2)*B_CELL)**2 +
     2          (T_MATRIX(1,3)*C_CELL)**2 )
      V2 = SQRT( (T_MATRIX(2,1)*A_CELL)**2 +
     1          (T_MATRIX(2,2)*B_CELL)**2 +
     2          (T_MATRIX(2,3)*C_CELL)**2 )
      MAP_ABANG = (T_MATRIX(1,1)*T_MATRIX(2,1)*A_CELL*A_CELL) +
     1       (T_MATRIX(1,1)*T_MATRIX(2,2)*A_CELL*B_CELL*COSGAM) +
     2       (T_MATRIX(1,1)*T_MATRIX(2,3)*A_CELL*C_CELL*COSBET) +
     3       (T_MATRIX(1,2)*T_MATRIX(2,2)*B_CELL*B_CELL) +
     4       (T_MATRIX(1,2)*T_MATRIX(2,1)*A_CELL*B_CELL*COSGAM) +
     5       (T_MATRIX(1,2)*T_MATRIX(2,3)*B_CELL*C_CELL*COSALP) +
     6       (T_MATRIX(1,3)*T_MATRIX(2,3)*C_CELL*C_CELL) +
     7       (T_MATRIX(1,3)*T_MATRIX(2,1)*A_CELL*C_CELL*COSBET) +
     8       (T_MATRIX(1,3)*T_MATRIX(2,2)*B_CELL*C_CELL*COSALP)
      MAP_ABANG = real(rad_to_deg * ACOS(MAP_ABANG/(V1*V2) ))

      RETURN
      END SUBROUTINE MAP_GET_ANGLE



C  **************************************************************************
C  *                                                                        *
C  *              *****     SUBROUTINE MAP_PEAKSEARCH     *****             *
C  *                                                                        *
C  *    ROUTINE TO SEARCH FOR ALL NON-ADJACENT "MAP" VALUES ABOVE LOWLIM.   *
C  *    NOTE THAT THIS ROUTINE ONLY WORKS AT PRESENT WITH 2D MAP ARRAYS.    *
C  *                                                                        *
C  *    PEAKS(1,NPEAK) = COLUMN POSITION OF "PEAK"                          *
C  *    PEAKS(2,NPEAK) = ROW POSITION OF "PEAK"                             *
C  *    PEAKS(3,NPEAK) = INTENSITY OF "PEAK"                                *
C  *                                                                        *
C  *    MAY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER                      *
C  **************************************************************************
C
      SUBROUTINE MAP_PEAKSEARCH (MAP, NCOL, NROW, PEAKS, NLIM, NPEAK,
     1                          LOWLIM)
C
      IMPLICIT NONE
C
      INTEGER*4 NLIM, NPEAK, COL, ROW, N1, N2, NUMCHARI4
      INTEGER*4 I, NCOL, NROW, J_LAST, J_NEXT
      REAL*4    PEAKS(3,NLIM), MAP(NCOL,NROW), TEST, LOWLIM
C  ----------------------------------------------------------------------------
C
      DO I = 1,NLIM
        PEAKS(1,I) = 0.0
        PEAKS(2,I) = 0.0
        PEAKS(3,I) = 0.0
      ENDDO
C
      N1 = NUMCHARI4(NINT(LOWLIM)) + 2

      NPEAK = 0                        ! RESET NPEAK COUNTER
C
      DO ROW = 1,NROW                        ! STEP THROUGH ROW BY ROW
        J_LAST = ROW - 1
        J_NEXT = ROW + 1
        DO COL = 1,NCOL                  ! STEP THROUGH CURRENT ROW
          TEST = MAP(COL,ROW)
          IF(TEST > LOWLIM) THEN
            IF(COL < NCOL) THEN
              IF(TEST <= MAP(COL+1,ROW)) GO TO 100
            ENDIF
            IF(COL > 1) THEN
              IF(TEST < MAP(COL-1,ROW)) GO TO 100
            ENDIF
            IF(J_NEXT <= NROW) THEN
              IF(TEST <= MAP(COL,J_NEXT)) GO TO 100
            ENDIF
            IF(J_LAST > 0) THEN
              IF(TEST < MAP(COL,J_LAST)) GO TO 100
            ENDIF
            NPEAK = NPEAK + 1
            IF(NPEAK > NLIM) THEN            ! TRAP FOR ARRAY OVERFLOW
              NPEAK = -1                        ! SIGNAL OVERFLOW
              CALL BELL (3)
              PRINT *, '***************************************'
              PRINT *, '* TOO MANY PEAK PIXELS - RESET LOWLIM *'
              PRINT *, '***************************************'
              RETURN
            ENDIF
            PEAKS(1,NPEAK) = REAL(COL)
            PEAKS(2,NPEAK) = REAL(ROW)
            PEAKS(3,NPEAK) = MAP(COL,ROW)
          ENDIF
  100       CONTINUE
        ENDDO
      ENDDO

      N2 = NUMCHARI4(NPEAK)

      RETURN
      END


C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE MAP_ROT     *****                  *
C  *                                                                          *
C  *    ROTATE 2D OR 3D MAP ABOUT ITS CENTER BY "RANGL" DEGREES.              *
C  *                                                                          *
C  *    MAY 1990  PURDUE UNIVERSITY VAX 8550 BY WDG, RHC.                     * 
C  *    CAPABILITY TO ROTATE MAP ABOUT X,Y,Z-AXIS ADDED.                      *
C  *                                                                          *
C  *    POINT LAYOUT FOR BI-LINEAR INTERPOLATION SCHEME :                     *
C  *                                                                          *
C  *          D4            D3                                                *
C  *                                                                          *
C  *                    + (dx,dy)                                             *
C  *                    .                                                     *
C  *          D1. . . . .   D2                                                *
C  *                                                             *
C  *    THE INTERPOLATION ALGORITHM IS COMPUTED AS FOLLOWS:                   *
C  *                                                                          *
C  *          D1(1-dX)(1-dY) + D2(dX)(1-dY) + D3(dX)(dY) + D4(1-dX)(dY)       *
C  *                                                                          *
C  *    WHICH REDUCES TO:                                                     *
C  *                                                                          *
C  *          D1 + (D2-D1)dx + (D4-D1)dy + (D1-D2+D3-D4)dxdy                  *
C  *                                                             *
C  *    AUGUST 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.                    *
C  *    MODIFIED 11-DEC-1997 BY T. BAKER.                                     *
C  *    LAST MODIFIED 25-FEB-1998 BY T. BAKER (Add UNIX compatibility).       *
C  ****************************************************************************
C
      SUBROUTINE MAP_ROT (MAP1, NCOL, NROW, NSEC, RANGL, AXIS)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, NSEC
      REAL*4    MAP1(NCOL, NROW, NSEC), RANGL
      CHARACTER AXIS*1
      real*4, allocatable :: map2(:), tmp(:)
      integer   alloc_stat

      IF(AXIS.EQ.'Z') THEN

         allocate(map2(ncol*nrow), stat=alloc_stat)
         if(alloc_stat > 0) then
            write(*,*) 'Unable to allocate MAP2 in MAP_ROT'
            call tsbend
         endif

         CALL MAP_DOLLAR_ROT (MAP1, MAP2, NCOL, NROW, NSEC, RANGL)
         deallocate(map2)
       ELSE IF(AXIS.EQ.'X') THEN
         IF(NCOL.NE.NROW .OR. NCOL.NE.NSEC .OR. NROW.NE.NSEC) THEN
           CALL BELL (2)
           PRINT *, '*************************************'
           PRINT *, '* Only able to handle CUBIC 3D maps *'
           PRINT *, '*************************************'
           CALL HITCR
         ENDIF
         allocate(map2(nrow*nsec), tmp(nrow), stat=alloc_stat)
         if(alloc_stat > 0) then
            write(*,*) 'Unable to allocate MAP2 and TMP in MAP_ROT'
            call tsbend
         endif
         CALL MAP_ROTX(MAP1, MAP2, NCOL, NROW, NSEC, RANGL)
         deallocate(map2, tmp)
       ELSE IF(AXIS.EQ.'Y') THEN
         IF(NCOL.NE.NROW .OR. NCOL.NE.NSEC .OR. NROW.NE.NSEC) THEN
           CALL BELL (2)
           PRINT *, '*************************************'
           PRINT *, '* Only able to handle CUBIC 3D maps *'
           PRINT *, '*************************************'
           CALL HITCR
         ENDIF
         allocate(map2(ncol*nsec), tmp(nsec), stat=alloc_stat)
         if(alloc_stat > 0) then
            write(*,*) 'Unable to allocate MAP2 and TMP in MAP_ROT'
            call tsbend
         endif
         CALL MAP_ROTY(MAP1, MAP2, NCOL, NROW, NSEC, RANGL)
         deallocate(map2, tmp)
      ENDIF
C
      RETURN
      END
C  ============================================================================
      SUBROUTINE MAP_DOLLAR_ROT (MAP1, MAP2, NCOL, NROW, NSEC, RANGL)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, NSEC, IX, IY, I, J, K, TEST
      REAL*4    XCEN, YCEN, A11, A12, A21, A22, PERCENT, DIVISOR
      REAL*4    D1, D2, D3, D4, DX, DY, X, Y, RANGL
      REAL*4    MAP1(NCOL,NROW,NSEC), MAP2(NCOL,NROW)
      real*8    deg_to_rad
      LOGICAL   NOTIFY
C  ----------------------------------------------------------------------------
C
      deg_to_rad = acos(-1.0d0)/180.0d0
      DIVISOR = 1.0
C
      TEST = NCOL*NROW*NSEC
      IF(TEST > 5000000) THEN
         NOTIFY = .TRUE.
         DIVISOR = REAL(NCOL)
         WRITE(*,'(1X, ''  0.0% complete'')')
       ELSE
         NOTIFY = .FALSE.
      ENDIF
C
      XCEN      = REAL(NCOL+1)/2.0
      YCEN      = REAL(NROW+1)/2.0
C  ********************************
C  *    SET UP ROTATION MATRIX    *
C  ********************************
      A11 = REAL(COS(RANGL*deg_to_rad))
      A12 = REAL(SIN(RANGL*deg_to_rad))
      A21 = -A12
      A22 =  A11
C
      DO K = 1,NSEC                  ! STEP THROUGH NSEC SECTIONS
        DO J = 1,NROW                  ! STEP THROUGH ROWS OF MAP2
          DO I = 1,NCOL            ! STEP THROUGH CURRENT ROW OF MAP2
            X = A11*(REAL(I)-XCEN)+A12*(REAL(J)-YCEN) + XCEN
            Y = A21*(REAL(I)-XCEN)+A22*(REAL(J)-YCEN) + YCEN
            IX = INT(X)
            IY = INT(Y)
            IF(IX >= 1.AND.IY >= 1.AND.IX <= NCOL.AND.IY <= NROW) THEN
               D2        = 0.0
               D3        = 0.0
               D4        = 0.0
               D1        = MAP1(IX,IY,K)
               IF(IX < NCOL) D2 = MAP1(IX+1,IY,K)
               IF(IX < NCOL.AND.IY < NROW) D3 = MAP1(IX+1,IY+1,K)
               IF(IY < NROW) D4 = MAP1(IX,IY+1,K)
               DX        = X-IX
               DY        = Y-IY
               MAP2(I,J) = D1+DX*(D2-D1)+DY*(D4-D1)+DX*DY*(D1-D2+D3-D4)
             ELSE
               MAP2(I,J) = 0.0
            ENDIF
          ENDDO
        ENDDO                                    ! LOOP OVER ROWS
C  **********************************************
C  *    STORE ROTATED SECTION BACK INTO MAP1    *
C  **********************************************
        DO J = 1,NROW
           DO I = 1,NCOL
              MAP1(I,J,K) = MAP2(I,J)
           ENDDO
        ENDDO
        IF(NOTIFY) THEN
           PERCENT = 100.0*(REAL(K)/DIVISOR)
           WRITE(*,'(''+'',F5.1)') PERCENT
        ENDIF
      ENDDO                     ! LOOP OVER SECTIONS

      RETURN
      END

C  *************************************************************************
C  *                                                                       *
C  *              *****     SUBROUTINE MAP_ROTX     *****                  *
C  *                                                                       *
C  *    IN CORE ROTATION OF 3D MAP ABOUT X-AXIS.                           *
C  *                                                                       *
C  *    MAY 1991  PURDUE UNIVERSITY VAX 8550 T. BAKER.                     * 
C  *    MODIFIED 11-DEC-1997 BY T. BAKER.                                  *
C  *    LAST MODIFIED 25-FEB-1998 BY T. BAKER (Add UNIX compatibility).    *
C  *************************************************************************
C
      SUBROUTINE MAP_ROTX (MAP1, MAP2, NCOL, NROW, NSEC, RANGL)
C
      IMPLICIT NONE
C
      INTEGER*8 TEST
      INTEGER*4 I, J, K, NCOL, NROW, NSEC, IX, IY
      REAL*4    MAP1(NCOL,NROW,NSEC), MAP2(NROW,NSEC), RANGL
      REAL*4    XCEN, YCEN, A11, A12, A21, A22, PERCENT, DIVISOR
      REAL*4    D1, D2, D3, D4, DX, DY, X, Y
      real*8    deg_to_rad
      LOGICAL   NOTIFY
C  ----------------------------------------------------------------------------
C
      deg_to_rad = acos(-1.0d0)/180.0d0

      DIVISOR = REAL(NCOL)
      TEST = NCOL*NROW*NSEC
      IF(TEST > 5000000) THEN
         NOTIFY = .TRUE.
         WRITE(*,'(1X, ''  0.0% complete'')')
       ELSE
         NOTIFY = .FALSE.
      ENDIF
C
      XCEN = REAL(NROW+1)/2.0
      YCEN = REAL(NCOL+1)/2.0
C  ********************************
C  *    SET UP ROTATION MATRIX    *
C  ********************************
      A11 = REAL(COS(RANGL*deg_to_rad))
      A12 = REAL(SIN(RANGL*deg_to_rad))
      A21 = -A12
      A22 =  A11
C
      DO K = 1,NCOL                        ! STEP THROUGH NCOL YZ SECTIONS
        DO J = 1,NSEC                        ! STEP THROUGH ROWS OF MAP2
          DO I = 1,NROW                  ! STEP THROUGH CURRENT MAP2 ROW 
            X = A11*(REAL(I)-XCEN)+A12*(REAL(J)-YCEN) + XCEN
            Y = A21*(REAL(I)-XCEN)+A22*(REAL(J)-YCEN) + YCEN
            IX = INT(X)
            IY = INT(Y)
            IF(IX >= 1 .AND. IY >= 1 .AND. IX <= NROW .AND.
     1        IY <= NSEC) THEN
               D2        = 0.0
               D3        = 0.0
               D4        = 0.0
               D1        = MAP1(K,IX,IY)
               IF(IX < NROW) D2 = MAP1(K,IX+1,IY)
               IF(IX < NROW.AND.IY < NSEC) D3 = MAP1(K,IX+1,IY+1)
               IF(IY < NSEC) D4 = MAP1(K,IX,IY+1)
               DX        = X - IX
               DY        = Y - IY
               MAP2(I,J) = D1+DX*(D2-D1)+DY*(D4-D1)+DX*DY*(D1-D2+D3-D4)
             ELSE
               MAP2(I,J) = 0.0
            ENDIF
          ENDDO
        ENDDO                                    ! LOOP OVER ROWS
C  **********************************************
C  *    STORE ROTATED SECTION BACK INTO MAP1    *
C  **********************************************
        DO J = 1,NSEC
          DO I = 1,NROW
            MAP1(K,I,J) = MAP2(I,J)
          ENDDO
        ENDDO
        IF(NOTIFY) THEN
          PERCENT = 100.0*(REAL(K)/DIVISOR)
          WRITE(*,'(''+'',F5.1)') PERCENT
        ENDIF
      ENDDO                                    ! LOOP OVER SECTIONS
C
      RETURN
      END


C  *************************************************************************
C  *                                                                       *
C  *              *****     SUBROUTINE MAP_ROTY     *****                  *
C  *                                                                       *
C  *    IN CORE ROTATION OF 3D MAP ABOUT Y-AXIS.                           *
C  *                                                                       *
C  *    MAY 1991  PURDUE UNIVERSITY VAX 8550 T. BAKER.                     * 
C  *    LAST MODIFIED 11-DEC-1997 BY T. BAKER.                             *
C  *************************************************************************
C
      SUBROUTINE MAP_ROTY (MAP1, MAP2, NCOL, NROW, NSEC, RANGL)
C
      IMPLICIT NONE
C
      INTEGER*8 TEST
      INTEGER*4 I, J, K, NCOL, NROW, NSEC, IX, IY
      REAL*4    MAP1(NCOL,NROW,NSEC), MAP2(NSEC,NCOL), RANGL
      REAL*4    XCEN, YCEN, A11, A12, A21, A22, PERCENT, DIVISOR
      REAL*4    D1, D2, D3, D4, DX, DY, X, Y
      real*8    deg_to_rad
      LOGICAL   NOTIFY
C  ----------------------------------------------------------------------------
C
      deg_to_rad = acos(-1.0d0)/180.0d0

      DIVISOR = REAL(NROW)
      TEST = NCOL*NROW*NSEC
      IF(TEST > 5000000) THEN
         NOTIFY = .TRUE.
         WRITE(*,'(1X, ''  0.0% complete'')')
       ELSE
         NOTIFY = .FALSE.
      ENDIF
C
      XCEN = REAL(NSEC+1)/2.0
      YCEN = REAL(NCOL+1)/2.0
C  ********************************
C  *    SET UP ROTATION MATRIX    *
C  ********************************
      A11 = REAL(COS(RANGL*deg_to_rad))
      A12 = REAL(SIN(RANGL*deg_to_rad))
      A21 = -A12
      A22 =  A11
C
      DO K = 1,NROW                        ! STEP THROUGH NROW ZX SECTIONS
        DO J = 1,NCOL                        ! STEP THROUGH ROWS OF MAP2
          DO I = 1,NSEC                  ! STEP THROUGH CURRENT MAP2 ROW 
            X = A11*(REAL(I)-XCEN)+A12*(REAL(J)-YCEN) + XCEN
            Y = A21*(REAL(I)-XCEN)+A22*(REAL(J)-YCEN) + YCEN
            IX = INT(X)
            IY = INT(Y)
            IF(IX >= 1 .AND. IY >= 1 .AND. IX <= NSEC .AND.
     1        IY <= NCOL) THEN
               D2        = 0.0
               D3        = 0.0
               D4        = 0.0
               D1        = MAP1(IY,K,IX)
               IF(IX < NSEC) D2 = MAP1(IY,K,IX+1)
               IF(IX < NSEC.AND.IY < NCOL) D3 = MAP1(IY+1,K,IX+1)
               IF(IY < NCOL) D4 = MAP1(IY+1,K,IX)
               DX        = X - IX
               DY        = Y - IY
               MAP2(I,J) = D1+DX*(D2-D1)+DY*(D4-D1)+DX*DY*(D1-D2+D3-D4)
             ELSE
               MAP2(I,J) = 0.0
            ENDIF
          ENDDO
        ENDDO                                    ! LOOP OVER ROWS
C  **********************************************
C  *    STORE ROTATED SECTION BACK INTO MAP1    *
C  **********************************************
        DO J = 1,NCOL
          DO I = 1,NSEC
            MAP1(J,K,I) = MAP2(I,J)
          ENDDO
        ENDDO
        IF(NOTIFY) THEN
          PERCENT = 100.0*(REAL(K)/DIVISOR)
          WRITE(*,'(''+'',F5.1)') PERCENT
        ENDIF
      ENDDO                                    ! LOOP OVER SECTIONS
C
      RETURN
      END


C  **************************************************************************
C  *                                                                        *
C  *               *****     SUBROUTINE MAP_SCALE     *****                 *
C  *                                                                        *
C  *    ROUTINE TO RESCALE "MAP" VALUES INSIDE THE RANGE MIN_OLD-MAX_OLD    *
C  *     TO MIN_NEW-MAX_NEW (INCORE).                                                *
C  *                                                                        *
C  *    WORKS WITH 2D OR 3D "MAP" DATA.                                     *
C  *                                                                        *
C  *    JANUARY 1985  PURDUE UNIVERSITY VAX 11/750, T. BAKER.               *
C  *      Modified screen output, 22 Sept 1994, by D. Belnap                *
C  *    LAST MODIFIED 27-FEB-1998 BY T. BAKER (Make UNIX compatible).       *
C  **************************************************************************
C
      SUBROUTINE MAP_SCALE (MAP, NCOL, NROW, NSEC, MIN_OLD,
     1 MAX_OLD, MIN_NEW, MAX_NEW, MAP_MIN, MAP_MAX, 
     2 MAP_AVG, MAP_STD)
C
      IMPLICIT NONE

      INTEGER*4 NCOL, NROW, NSEC, I, ARRAY_SIZE, MAP_INCORE
      REAL*4    MAP(1), MIN_OLD, MAX_OLD, MIN_NEW, MAX_NEW, DIV, KSTEP
      REAL*4    MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD
      REAL*4    SCALE, TMP2, SUMSQ, TOT, VAR, PERCENT
      LOGICAL   :: VERBOSE = .FALSE.

      integer alloc_stat
      real*4, allocatable :: tmp(:)
      integer*2, allocatable :: itmp(:)
C  ----------------------------------------------------------------------------
C
      allocate(itmp(ncol), stat=alloc_stat)
      allocate(tmp(ncol), stat=alloc_stat)

      ARRAY_SIZE = NCOL*NROW*NSEC
      DIV        = REAL(NSEC)
C
      IF(VERBOSE .AND. NSEC > 1) THEN
         PRINT *,'*****************************'
         PRINT *,'*  Rescaling 3D MAP values  *'
         PRINT *,'*****************************'
         PRINT *,'OLD values'
         PRINT *,'MIN_OLD,MAX_OLD =',MIN_OLD, MAX_OLD      
         PRINT *,'NEW values'
         PRINT *,'MIN_NEW,MAX_NEW =',MIN_NEW, MAX_NEW
         PERCENT = 0.0
         WRITE(*,'(1X,F5.1,''% done with 3D MAP'')') PERCENT
         DIV = REAL(ARRAY_SIZE)/10.
      ELSE
         DIV = REAL(ARRAY_SIZE)
      ENDIF
C
      SCALE = 1.0
      IF(MIN_OLD.NE.MAX_OLD) THEN
        SCALE = (MAX_NEW-MIN_NEW)/(MAX_OLD-MIN_OLD)
      ENDIF
      MAP_MIN = HUGE(MAP_MIN)
      MAP_MAX = -HUGE(MAP_MAX)
      SUMSQ   = 0.0
      TOT     = 0.0
C
      KSTEP = DIV
      DO I = 1,ARRAY_SIZE
        TMP2 = MAP(I)
        IF(TMP2 >= MIN_OLD.AND.TMP2 <= MAX_OLD) THEN
          TMP2   = MIN_NEW + (TMP2-MIN_OLD) * SCALE
          MAP(I) = TMP2
        ENDIF
        SUMSQ   = SUMSQ + TMP2*TMP2
        TOT     = TOT + TMP2
        MAP_MIN = AMIN1(MAP_MIN,TMP2)
        MAP_MAX = AMAX1(MAP_MAX,TMP2)
        IF(VERBOSE) THEN
          IF(REAL(I) >= KSTEP) THEN
            PERCENT = 100.0*REAL(I)/REAL(ARRAY_SIZE)
            WRITE(*,'(''+'',F5.1)') PERCENT
            KSTEP = KSTEP + DIV
          ENDIF
        ENDIF
      ENDDO
C
      MAP_AVG = TOT/DIV
      MAP_STD = 0.0
      VAR     = (SUMSQ/DIV) - (MAP_AVG*MAP_AVG)
      IF(VAR >= 0.0) MAP_STD = SQRT(VAR)
C
      deallocate(tmp, itmp)
      RETURN
      END



C  **********************************************************************
C  *                                                                    *
C  *              *****     SUBROUTINE MAP_STATS     *****              *
C  *                                                                    *
C  *    ROUTINE TO DETERMINE THE MIN/MAX, AVERAGE AND STD FOR VALUES    *
C  *     IN THE "MAP" ARRAY.                                            *
C  *                                                                    *
C  *    APRIL 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER.             *
C  **********************************************************************
C
      SUBROUTINE MAP_STATS (MAP, NCOL, NROW, NSEC, MAP_MIN,
     1 MAP_MAX, MAP_AVG, MAP_STD, EM_WRT)
C
      IMPLICIT NONE

      INTEGER*4 NCOL, NROW, NSEC, I, ARRAY_SIZE, KSTEP
      INTEGER*4 EM_WRT
      REAL*4    MAP(1), MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD
      REAL*4    VAR, X, DIV, PERCENT
      REAL*8    TOT, SUMSQ
      LOGICAL   NOTIFY
C  ----------------------------------------------------------------------------
C
      ARRAY_SIZE = NCOL*NROW*NSEC
C

      IF(EM_WRT.EQ.1) WRITE(*,1000) 
      IF(EM_WRT.EQ. 1. AND. ARRAY_SIZE > 5000000) THEN
         PERCENT = 0.0
         WRITE(*,'(''+'',F5.1,''% done with 3D MAP'')') PERCENT
         NOTIFY = .TRUE.
       ELSE
         NOTIFY = .FALSE.
      ENDIF
C
      MAP_MIN  = HUGE(MAP_MIN)
      MAP_MAX  = -HUGE(MAP_MAX)
      SUMSQ    = 0.0
      TOT      = 0.0
C
      KSTEP = NINT(REAL(ARRAY_SIZE)/10.0)
      DIV   = REAL(ARRAY_SIZE)
      DO I = 1,ARRAY_SIZE
        X       = MAP(I)
        SUMSQ   = SUMSQ + X**2
        TOT     = TOT + X
        MAP_MIN = AMIN1(MAP_MIN,X)
        MAP_MAX = AMAX1(MAP_MAX,X)
        IF(MOD(I,KSTEP).EQ.0) THEN
          IF(NOTIFY) THEN
            PERCENT = 100.0*REAL(I)/DIV
            WRITE(*,'(''+'',F5.1)') PERCENT
          ENDIF
        ENDIF
      ENDDO
C
      MAP_AVG = REAL(TOT/REAL(ARRAY_SIZE))
      MAP_STD = 0.0
      VAR     = REAL((SUMSQ/REAL(ARRAY_SIZE))-(MAP_AVG*MAP_AVG))
      IF(VAR >= 0.0) MAP_STD = SQRT(VAR)
      IF(EM_WRT.EQ.1) WRITE(*,1010) MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD
C
      RETURN
 1000 FORMAT('      MINIMUM     MAXIMUM        AVERAGE         STD',
     1     /,' -----------------------------------------------------',/)
 1010      FORMAT('+',2F12.1,2F15.2)
      END


C  *************************************************************************
C  *                                                                       *
C  *               *****     SUBROUTINE MAP3DT_STORE     *****             *
C  *                                                                       *
C  *    THIS ROUTINE, PREVIOUSLY CALLED MAP3DT_WRITEH, WRITES OUT COMPLEX  *
C  *     INTERMEDIATE RESULTS OF XZ TRANSFORM FROM "ARRAY" TO "ARRAY2".    *
C  *                                                                       *
C  *    FOR SPACE GROUPS HAVING UNIQUE REFLECTIONS WITH H < 0 AS WELL AS   *
C  *     H >= 0 .                                                          *
C  *************************************************************************
C
      SUBROUTINE MAP3DT_STORE (ARRAY,NCOL,NROW,NSEC,HMAX,KMAX,ARRAY2)
C
      IMPLICIT NONE
C
      INTEGER*4 H, K, L, OFFSET, HMAX, KMAX, NCOL, NROW, NSEC, IQ, IP
      COMPLEX   ARRAY(((NCOL+2)/2),NROW,NSEC)
      REAL*4    ARRAY2(2*NSEC*NROW*(HMAX+1))
C  ----------------------------------------------------------------------------
C
C  *************************
C  *    NEGATIVE H ONLY    *
C  *************************
      IQ = NROW - KMAX
  100      IP = IQ + 1
      IQ = IQ + NROW
      IF(IQ > NROW) GO TO 300
      OFFSET = 1
      DO L=1,(HMAX + 1)
        DO K=1,NSEC
          DO H = IP,IQ
            ARRAY2(OFFSET)   = REAL (ARRAY(L,H,K))
            ARRAY2(OFFSET+1) = AIMAG(ARRAY(L,H,K))
            OFFSET = OFFSET + 2
          ENDDO
        ENDDO
      ENDDO
      IF(IQ < NROW) GO TO 100
C  ****************************
C  *    H POSITIVE OR ZERO    *
C  ****************************
      IQ = 0
  200      IP = IQ + 1
      IQ = IQ + NROW
      IF(IQ > KMAX) IQ = KMAX + 1
      OFFSET = 1
      DO L = 1,(HMAX + 1)
        DO K = 1,NSEC
          DO H = IP,IQ
            ARRAY2(OFFSET)   = REAL (ARRAY(L,H,K))
            ARRAY2(OFFSET+1) = AIMAG(ARRAY(L,H,K))
            OFFSET = OFFSET + 2
          ENDDO
        ENDDO
      ENDDO
      IF(IQ <= KMAX) GO TO 200
      RETURN
C  **********************
C  *    SPANNING H=0    *
C  **********************
  300      IQ = IQ - NROW
      IF(IQ > KMAX) IQ = KMAX + 1
      OFFSET = 1
      DO L = 1,(HMAX + 1)
        DO K = 1,NSEC
          DO H = IP,NROW
            ARRAY2(OFFSET)   = REAL (ARRAY(L,H,K))
            ARRAY2(OFFSET+1) = AIMAG(ARRAY(L,H,K))
            OFFSET = OFFSET + 2
          ENDDO
          DO H = 1,IQ
            ARRAY2(OFFSET)   = REAL (ARRAY(L,H,K))
            ARRAY2(OFFSET+1) = AIMAG(ARRAY(L,H,K))
            OFFSET = OFFSET + 2
          ENDDO
        ENDDO
      ENDDO
      IF(IQ <= KMAX) GO TO 200
C
      RETURN
      END


C  **********************************************************************
C  *                                                                    *
C  *             *****     SUBROUTINE MAP3DT_PASS1     *****            *
C  *                                                                    *
C  *    THIS SUBROUTINE, MODIFIED FROM TEN EYCK'S READ Y, READS IN A    *
C  *     MAP AS Z-SECTIONS IN WHICH X VARIES MOST RAPIDLY AND Y VARIES  *
C  *     MOST SLOWLY.                                                   *
C  *                                                                    *
C  *    LAST MODIFIED 10-AUG-1998 BY TSB (Fill ARRAY and FFT).          *
C  **********************************************************************
C
      SUBROUTINE MAP3DT_PASS1 (MAP3D, ARRAY, NCOL, NROW, NSEC, D, HMAX)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, NSEC, I, J, K, HMAX, D(5), I1, I2, I3
      REAL*4    MAP3D(NCOL*NROW*NSEC), ARRAY((NCOL+2)*NROW*NSEC)
C  ----------------------------------------------------------------------------
C
      WRITE(*,'(A)') ' First Pass'
C
      I1 = 0
      DO K = 1,NSEC
        I2 = (K-1)*NROW*(NCOL+2)
        DO J = 1,NROW
          I3 = I2 + (J-1)*(NCOL+2)
          DO I = 1,NCOL
            I1 = I1 + 1
            I3 = I3 + 1
            ARRAY(I3) = MAP3D(I1)
          ENDDO
        ENDDO
      ENDDO
C
Ca We're overruning this guy in UNIX
Ca      D(1) = (NCOL+2)*NROW*NSEC
Ca  Initially this was the fix - wwhy??????????
Ca      D(1) = (NCOL+1)*NROW*NSEC
      D(1) = (NCOL+2)*NROW*NSEC
      D(2) = 2
      D(3) = D(1)
      D(4) = D(1)
      D(5) = NCOL + 2
      CALL REALFT (ARRAY(1), ARRAY(2), NCOL/2, D)
      D(2) = NCOL + 2
      D(3) = NROW*(NCOL+2)
      D(4) = 2*(HMAX+1)
      D(5) = 2
      CALL CMPLFT (ARRAY(1), ARRAY(2), NROW, D)
C
      RETURN
      END

C  **********************************************************************
C  *                                                                    *
C  *             *****     SUBROUTINE MAP3DT_PASS2     *****            *
C  *                                                                    *
C  *    THIS SUBROUTINE, MODIFIED FROM TEN EYCK'S READ_H, READS BACK    *
C  *     BACK THE COMPLEX INTERMEDIATE RESULTS OF XZ TRANSFORM AS H -   *
C  *     PLANES, WHEN THERE HAS BEEN NO TRANSPOSITION AND INTERLEAVING  *
C  *     IT ALLOWS NO SCRATCH LOCATION ON Y                             *
C  *                                                                    *
C  *    LAST MODIFIED 11-AUG-1998 BY TSB (Fill ARRAY and FFT).          *
C  **********************************************************************
C
      SUBROUTINE MAP3DT_PASS2 (ARRAY, ARRAY2, NSEC, HMAX, KMAX, D)
C
      IMPLICIT NONE
C
      INTEGER*4 NSEC, H, K, L, OFFSET, HMAX, KMAX, D(5), I1, I2, I3
      REAL*4    ARRAY (2*NSEC*(2*KMAX+1)*(HMAX+1))
      REAL*4    ARRAY2(2*NSEC*(2*KMAX+1)*(HMAX+1))
C  ----------------------------------------------------------------------------
C
      WRITE(*,'(A)') '+Second Pass'
C
      OFFSET = 1
      DO H = 1,(HMAX+1)
        I1 = (H-1)*2*NSEC
        DO L = 1,NSEC
          I2 = I1 + (L-1)*2 + 1
          DO K = 1,(2*KMAX+1)
            I3 = I2 + (K-1)*(HMAX+1)*2*NSEC
            ARRAY(I3)   = ARRAY2(OFFSET)
            ARRAY(I3+1) = ARRAY2(OFFSET+1)
            OFFSET = OFFSET + 2
          ENDDO
        ENDDO
      ENDDO
C
      D(1)  = 2*NSEC*(HMAX+1)*(2*KMAX+1)
      D(2)  = 2
      D(3)  = D(1)
      D(4)  = D(1)
      D(5)  = 2*NSEC
      CALL CMPLFT (ARRAY(1), ARRAY(2), NSEC, D)
C
      RETURN
      END
