C  ****************************************************************************
C  *                                                                          *
C  *                     *****     MISCLIB.SUBS     *****                     *
C  *                                                                          *
C  *    LIBRARY OF MISCELLANEOUS SUBROUTINES/FUNCTIONS:                       *
C  *                                                                          *
C  *      BELL          - RING TERMINAL BELL N TIMES                          *
C  *      BLANKL        - SEND BLANK LINES TO SYS$OUTPUT                      *
C  *                      IF #LINES < 1, BLANK CURRENT LINE W/O LINEFEED      *
C  *      BSL           - COMPUTE BESSEL FUNCTIONS                            *
C  *      CONTUR_*      - A SERIES OF SUBROUTINES USED FOR CONTOURING "MAPS"  *
C  *      CONVERSION FUNCTIONS - FUNCTION ROUTINES TO CONVERT BETWEEN         *
C  *                              VARIABLE TYPES                              *
C  *        I4I2        - CONVERT FROM INTEGER*4 TO INTEGER*2                 *
C  *        R4I2        - CONVERT FROM REAL*4 TO INTEGER*2                    *
C  *        R4I4        - CONVERT FROM REAL*4 TO INTEGER*4                    *
C  *      CHK_PRIME     - FACTOR "N"; FIND NUMBER "N_OUT" WITH NO PRIME >19   *
C  *      DECTOOCT      - RETURN OCTAL VALUE OF DECIMAL NUMBER                *
C  *      GET_YESNO     - RETURN LOGICAL IN RESPONSE TO Y/N QUESTION.         *
C  *      GNOI_R4_SIZE  - DETERMINE SIZE FOR GAUSS ARRAY.                     *
C  *      GNOI_R4_FILL  - FILL GAUSS ARRAY WITH NORMAL DISTRIBUTION.          *
C  *      HITCR         - HIT <CR> TO CONTINUE PROGRAM.                       *
C  *      HKLDSTAR      - COMPUTE DSTAR FOR REFLECTION H,K,L                  *
C  *      HKL_TO_HK     - TRANSFORM FROM HKL INDICIES TO H'K' USING THE       *
C  *                       INVERSE OF THE TRANSFORMATION MATRIX "T_MATRIX"    *
C  *      HK_TO_HKL     - TRANSFORM FROM H'K' INDICIES TO HKL                 *
C  *      I2_MEDIAN_3   - RETURN MEDIAN OF THREE I*2 NUMBERS                  *
C  *      LATCEN        - LATTICE CENTERING: FROM JOACHIM'S SPIDER ROUTINES   *
C  *      LATCEN2       - MODIFIED VERSION OF LATCEN                          *
C  *      NUMCHARI4     - DETERMINES THE NUMBER OF CHARACTERS IN AN INTEGER*4 *
C  *                       NUMBER.                                            *
C  *        NUM_DIGIT_R4  - FOR REAL*4 NUMBER DETERMINES NUMBER OF DIGITS TO    *
C  *                     LEFT OF DECIMAL POINT AND TOTAL DIGITS IN NUMBER   *
C  *      PIRADDEG      - GENERATE PI, RADIAN AND DEGREE TO MAXIMUM SINGLE    *
C  *                       PRECISION ACCURACY.                                *
C  *      PIRADDEG_D    - GENERATE PI, RADIAN AND DEGREE TO MAXIMUM DOUBLE    *
C  *                       PRECISION ACCURACY.                                *
C  *      REAL_TO_REC   - CONVERT REAL SPACE PARAMETERS TO RECIPROCAL         *
C  *                       LATTICE PARAMETERS.                                *
C  *      REC_TO_REAL   - CONVERT RECIPROCAL LATTICE PARAMETERS TO REAL       *
C  *                       SPACE PARAMETERS.                                  *
C  *      SEED          - GENERATES THE SEED VALUE FOR THE RANDOM NUMBER      *
C  *                       GENERATOR.                                         *
C  *      SOLV2D        - FROM JOACHIM'S SPIDER SOFTWARE.                     *
C  *      SWBYTE        - SWAP BYTES IN INTEGERS.                             *
C  *      TSBEND        - IDENTIFIES PROGRAM TERMINATION WITH A STRING OF *'s *
C  *      TSBEND_CLEAR  - TERMINATES PROGRAM AFTER CLEARING THE SCREEN.       *
C  *      WAVEL         - COMPUTE ELECTRON WAVELENGTH FROM VOLTS.             *
C  ****************************************************************************
C
C

C  **************************************************************
C  *                                                            *
C  *             *****     SUBROUTINE BELL     *****            *
C  *                                                            *
C  *    RING TERMINAL BELL N TIMES                              *
C  *                                                            *
C  *    JANUARY 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER    *
C  **************************************************************
C
      SUBROUTINE BELL (N)
C
      IMPLICIT NONE
C
      INTEGER*2 N, I
      character(len=1) A
C  ----------------------------------------------------------------------------
C
      A = achar(7)
      IF(N.LE.0) RETURN
      DO I = 1,N
        WRITE(*,'(A,A1)') '+', A
      ENDDO
C
      RETURN
      END

C  *****************************************************************
C  *                                                               *
C  *              *****     SUBROUTINE BLANKL     *****            *
C  *                                                               *
C  *    ROUTINE TO INSERT NLINES BLANK LINES INTO THE OUTPUT       *
C  *     IF NLINES < 1, BLANK OUT THE CURRENT LINE                 *
C  *                                                               *
C  *    MARCH 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER         *
C  *****************************************************************
C
      SUBROUTINE BLANKL (NLINES)
C
      IMPLICIT NONE
C
      INTEGER*2 I, NLINES
C  ----------------------------------------------------------------------------
C
      IF(NLINES.GT.0) THEN
         DO I = 1,NLINES
           WRITE(*,'(1X)')
         ENDDO
       ELSE
         WRITE(*,1000)
      ENDIF
C
      RETURN
 1000      FORMAT('+',79(' '))
      END

C  *****************************************************************
C  *                                                               *
C  *              *****     SUBROUTINE BSL     *****               *
C  *                                                               *
C  *    FUNCTION ROUTINES TO COMPUTE BESSEL FUNCTIONS              *
C  *                                                               *
C  *    JULY 1986  PURDUE UNIVERSITY VAX 11/750, T. BAKER          *
C  *    ORIGINAL ORIGIN UNKNOWN                                    *
C  *    USED IN PROGRAMS SUCH AS EMROTFLT                          *
C  *****************************************************************
C
C  ***********************
C  *    FUNCTION BSL0    *
C  ***********************
C
      FUNCTION BSL0 (X0)
C
      IMPLICIT NONE
C
      REAL*4 BSL0, X0, X1, X2, P, Q
C  ----------------------------------------------------------------------------
C
      IF(X0-4.0) 100, 200, 200
  100      X2   = (X0/4.0)**2
      BSL0 = ((((((-0.00050144*X2+0.00767719)*X2-0.07092536)*X2+
     1  0.44435843)*X2-1.7777561)*X2+3.9999973)*X2-3.9999999)*X2+1.0
      RETURN
C
  200      X1 = 4.0/X0
      X2 = X1**2
      P  = ((((-0.00000370*X2+0.0001736)*X2-.48761E-04)*X2+
     1    0.00017343)*X2-0.00175306)*X2+0.39894228
      Q  = (X1)*(((((0.00000323*X2-0.00001421)*X2+0.00003425)*X2-
     1    0.00008698)*X2+0.00045643)*X2-0.01246694)
      BSL0 = SQRT(X1)*(P*COS(X0-.7853981E+00)-Q*SIN(X0-.7853981E+00))
      RETURN
C
      END
C  ***********************
C  *    FUNCTION BSL1    *
C  ***********************
C
      FUNCTION BSL1 (X0)
C
      IMPLICIT NONE
C
      REAL*4 BSL1, X0, X1, X2, P, Q
C  ----------------------------------------------------------------------------
C
      IF(X0-4.0) 100, 200, 200
  100      X1   = X0/4.0
      X2   = X1**2
      BSL1 = X1*(((((((-0.0001290*X2+0.0022069)*X2-0.02366168)*X2+
     1      0.1777583)*X2-0.8888840)*X2+2.6666661)*X2-4.0)*X2+2.0)
      RETURN
C
  200      X1 = 4.0/X0
      X2 = X1**2
      P  = ((((0.0000042*X2-0.0000201)*X2+0.0000581)*X2-0.0002232)*
     1    X2+0.0029218 )*X2+0.3989423
      Q  = (X1)*(((((-0.0000037*X2+0.0000162)*X2-.398708E-04)*X2+
     1    0.0001065)*X2-0.0006390)*X2+0.0374008)
      BSL1 = SQRT(X1)*(P*SIN(X0-.7853981E+00)+Q*COS(X0-.7853981E+00))
      RETURN
C
      END



C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE CHK_PRIME     *****                *
C  *                                                                          *
C  *    CHECKS THE NUMBER "N" FOR THE LARGEST PRIME FACTOR.  IT THEN RETURNS  *
C  *     A NUMBER "N_OUT" WHICH IS THE NEXT SMALLEST NUMBER THAT DOESN'T HAVE *
C  *     A PRIME FACTOR GREATER THAN 19.                                      *
C  *                                                                          *
C  *    IF "EVEN" = .TRUE. THEN ONLY RETURN "N_OUT" THAT IS EVEN.             *
C  *                                                                          *
C  *    AUGUST 1998 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.         *
C  ****************************************************************************
C
      SUBROUTINE CHK_PRIME (N, N_OUT, EVEN)
C
      IMPLICIT NONE
C
      INTEGER*4 PRIMES(8), I, J, N, N_OUT, N_TEST
      LOGICAL   EVEN
      DATA PRIMES/2,3,5,7,11,13,17,19/
C  ----------------------------------------------------------------------------
C
      N_TEST = N
      IF(EVEN.AND.MOD(N_TEST,2).NE.0) N_TEST = N_TEST - 1
      J = N_TEST
C
   10      DO I=1,8
        IF(MOD(J,PRIMES(I)).EQ.0) THEN
           J = J/PRIMES(I)
           IF(J.EQ.1) THEN                ! NUMBER PASSES TESTS AND IS OK NOW
             N_OUT = N_TEST
             RETURN
           ENDIF
           GO TO 10
         ELSE
        ENDIF
      ENDDO
C  ************************************************
C  *    HERE IF NUMBER IS NOT COMPATIBLE          *
C  *    TRY REDUCING SIZE TO NEXT LOWER NUMBER    *
C  ************************************************
      N_TEST = N_TEST - 1
      IF(EVEN.AND.MOD(N_TEST,2).NE.0) N_TEST = N_TEST - 1
      J = N_TEST
      GO TO 10
C
      END


C  **********************************************************************
C  *                                                                    *
C  *               *****     SUBROUTINE GET_YESNO     *****             *
C  *                                                                    *
C  *    ACCEPT TOUCH-KEY RESPONSE TO YES/NO QUESTION.                   *
C  *    RETURN LOGICAL ANSWER.  DEFAULT (<CR>) ANSWER PASSED THROUGH    *
C  *     ARGUMENT LIST.                                                 *
C  *                                                                    *
C  *    NOVEMBER 1989   PURDUE UNIVERSITY VAX 8550, T. BAKER.           *
C  *    LAST MODIFIED 28-DEC-1999 by TSB (VT_READCHAR not working).     *
C  **********************************************************************
C
      SUBROUTINE GET_YESNO (ANSWER, DEFAULT)
C
      IMPLICIT NONE
C
      INTEGER   ICHR
      LOGICAL     ANSWER, DEFAULT
      CHARACTER*1 IKEY
C  ----------------------------------------------------------------------------
C
  100      READ (*,'(A1)') IKEY
      IF(IKEY.EQ.'Y' .OR. IKEY.EQ.'y') THEN 
         ANSWER = .TRUE.
       ELSE IF(IKEY.EQ.'N' .OR. IKEY.EQ.'n') THEN
         ANSWER = .FALSE.
       ELSE
         ICHR = ICHAR(IKEY)
ctsb         IF(ICHR.EQ.13 .OR. ICHR.EQ.221) THEN         ! Doesn't work 28-Dec-99 ???
         IF(ICHR.EQ.32) THEN
            ANSWER = DEFAULT
          ELSE
            PRINT *, '**********************************'
            PRINT *, '* Must respond with Y, N or <CR> *'
            PRINT *, '**********************************'
            WRITE(*,'(A)') '$Enter choice:'
            GO TO 100
         ENDIF
      ENDIF
C
      RETURN
      END


C  *******************************************************************
C  *                                                                 *
C  *               *****     SUBROUTINE HITCR     *****              *
C  *                                                                 *
C  *    RESUME PROGRAM AFTER ENTERING A <CR>                         *
C  *                                                                 *
C  *    JUNE 1986 T. BAKER, PURDUE UNIVERSITY                        *
C  *******************************************************************
C
      SUBROUTINE HITCR
C
      IMPLICIT NONE
C
      INTEGER*4 I
C  ----------------------------------------------------------------------------
C
      PRINT *, '********************************'
      PRINT *, '* Hit <CR> to continue program *'
      PRINT *, '********************************'
      READ (*,*,ERR=100) I
C
  100      RETURN
      END


C  **************************************************************************
C  *                                                                        *
C  *                  *****     SUBROUTINE HKL_TO_HK     *****              *
C  *                                                                        *
C  *    TRANSFORM FROM HKL INDICIES TO H'K' USING THE INVERSE OF THE        *
C  *       TRANSFORMATION MATRIX "T_MATRIX"                                 *
C  *                                                                        *
C  *     T (1,1),(1,2),(1,3) = INDICIES OF FIRST PRINCIPAL LATTICE VECTOR   *
C  *                              (NEAR HORIZONTAL)                         *
C  *     T (2,1),(2,2),(2,3) = INDICIES OF SECOND PRINCIPAL LATTICE VECTOR  *
C  *                              (NEAR VERTICAL)                           *
C  *                                                                        *
C  *    H, K, L = INPUT 3D MILLER INDICIES                                  *
C  *    IH, IK  = OUTPUT 2D MILLER INDICIES                                 *
C  *                                                                        *
C  *    JULY 1985  PURDUE UNIVERSITY VAX 11/750, T. BAKER                   *
C  **************************************************************************
C
      SUBROUTINE HKL_TO_HK (H, K, L, IH, IK, T_MATRIX)
C
      IMPLICIT NONE
C
      INTEGER*4 T_MATRIX(2,3)
      INTEGER*4 H, K, L, IH, IK
C  ----------------------------------------------------------------------------
C
      IH = H*T_MATRIX(1,1) + K*T_MATRIX(1,2) + L*T_MATRIX(1,3)
      IK = H*T_MATRIX(2,1) + K*T_MATRIX(2,2) + L*T_MATRIX(2,3)
C
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE HK_TO_HKL     *****                *
C  *                                                                          *
C  *    TRANSFORM FROM H'K' INDICIES TO HKL                                   *
C  *                                                                          *
C  *     T (1,1),(1,2),(1,3) = INDICIES OF FIRST PRINCIPAL LATTICE VECTOR     *
C  *                              (NEAR HORIZONTAL)                           *
C  *     T (2,1),(2,2),(2,3) = INDICIES OF SECOND PRINCIPAL LATTICE VECTOR    *
C  *                              (NEAR VERTICAL)                             *
C  *                                                                          *
C  *    IH, IK  = INPUT 2D MILLER INDICIES                                    *
C  *    H, K, L = OUTPUT 3D MILLER INDICIES                                   *
C  *                                                                          *
C  *    JULY 1985  PURDUE UNIVERSITY VAX 11/750, T. BAKER                     *
C  ****************************************************************************
C
      SUBROUTINE HK_TO_HKL (IH, IK, H, K, L, T_MATRIX)
C
      IMPLICIT NONE
C
      INTEGER*4 H, K, L, IH, IK
      INTEGER*4 T_MATRIX (2,3)
C  ----------------------------------------------------------------------------
C
      H = IH*T_MATRIX(1,1) + IK*T_MATRIX(2,1)
      K = IH*T_MATRIX(1,2) + IK*T_MATRIX(2,2)
      L = IH*T_MATRIX(1,3) + IK*T_MATRIX(2,3)
C
      RETURN
      END

C  *****************************************************
C  *                                                   *
C  *      *****     SUBROUTINE LATCEN2     *****       *
C  *                                                   *
C  *    ADOPTED FROM JOACHIM FRANK'S SPIDER SYSTEM     *
C  *    MODIFIED FEB 27, 1997, T. BAKER                *
C  *****************************************************
C
      SUBROUTINE LATCEN2 (IH1, IK1, X1, Y1, IH2, IK2, X2, Y2, AX, AY, 
     1 BX, BY, XCEN, YCEN)
C
      IMPLICIT NONE
C
      INTEGER*2 I, J
      INTEGER*4 IH1, IK1, IH2, IK2
      REAL*4    XM(2,3), YM(2,3), X1, Y1, X2, Y2, AX, AY, BX, BY, TST
      REAL*4        XCEN, YCEN
C  ----------------------------------------------------------------------------
C
C  *************************************
C  *    INITIALIZE XM, YM MATRICIES    *
C  *************************************
      DO I = 1,3
        DO J = 1,2
          XM(J,I) = 0.0
          YM(J,I) = 0.0
        ENDDO
      ENDDO
C
      XM(1,3) = IH1*(X1-XCEN) + IH2*(X2-XCEN) 
      XM(2,3) = IK1*(X1-XCEN) + IK2*(X2-XCEN) 
      XM(1,1) = IH1*IH1 + IH2*IH2
      XM(1,2) = IH1*IK1 + IH2*IK2 
      XM(2,1) = XM(1,2) 
      XM(2,2) = IK1*IK1 + IK2*IK2
C
      YM(1,3) = IH1*(Y1-YCEN) + IH2*(Y2-YCEN) 
      YM(2,3) = IK1*(Y1-YCEN) + IK2*(Y2-YCEN) 
      YM(1,1) = XM(1,1)
      YM(1,2) = XM(1,2)
      YM(2,1) = XM(2,1)
      YM(2,2) = XM(2,2)
C
      CALL SOLV2D (XM, TST)
      CALL SOLV2D (YM, TST)
      IF(TST.GE.0.5) THEN
        PRINT *, '**********************************'
        PRINT *, '* SINGULARITY IN EQUATIONS FOUND *'
        PRINT *, '**********************************'
        RETURN
      ENDIF
      AX = XM(1,3)
      BX = XM(2,3)
      AY = YM(1,3)
      BY = YM(2,3)
C
      RETURN
      END

C  ************************************************************************
C  *                                                                      *
C  *                 *****     FUNCTION NUMCHARI4     *****               *
C  *                                                                      *
C  *    DETERMINE THE NUMBER OF CHARACTERS IN THE INTEGER*4 NUMBER "N"    *
C  *                                                                      *
C  *    SEPTEMBER 1986  PURDUE UNIVERSITY VAX 11/750, T. BAKER            *
C  ************************************************************************
C
      FUNCTION NUMCHARI4 (N)
C
      IMPLICIT NONE
C
      REAL*4    REALNUM
      INTEGER*4 N, NUMCHARI4, NCHR
C  ----------------------------------------------------------------------------
C
      NCHR    = 0
      REALNUM = ABS(REAL(N))
      DO WHILE (REALNUM.GE.1.0)
        NCHR    = NCHR+1
        REALNUM = REALNUM/10.0
      ENDDO
      IF(N.LE.0) NCHR = NCHR+1
      NUMCHARI4 = NCHR
C
       RETURN
      END

C  ************************************************************************
C  *                                                                      *
C  *               *****    FUNCTION NUM_DIGIT_R4    *****                *
C  *                                                                      *
C  *    For a real number (REAL*4), determines the number of digits to    *
C  *      the left of the decimal point.  It then adds the number of        *
C  *      digits to the right (given) to give total digits in number.       *
C  *      Adapted from NUMCHARI2.                                      *
C  *                                                                      *
C  *    APRIL 1995  Purdue University,  D. Belnap                    *
C  *    LAST MODIFIED  3-DEC-1998 BY TSB (Fixed bug in count).            *
C  ************************************************************************
C
      FUNCTION NUM_DIGIT_R4 (REALNUM, RIGHT_DIGITS)
C
      IMPLICIT NONE
C
      REAL*4    DUMMY, REALNUM
      INTEGER*4 NUM_DIGIT_R4, N_DIGIT, RIGHT_DIGITS
C  ----------------------------------------------------------------------------
C
      N_DIGIT = 2                        ! DECIMAL POINT PLUS 1 LEFT DIGIT
      IF(REALNUM.LT.0.0) N_DIGIT = 3            ! MINUS SIGN
      DUMMY   = ABS(REALNUM)
C
      DO WHILE (DUMMY.GE.10.0)
        N_DIGIT = N_DIGIT + 1
        DUMMY   = DUMMY/10.0
      ENDDO
C
      IF (RIGHT_DIGITS.GT.0)  N_DIGIT = N_DIGIT + RIGHT_DIGITS
      NUM_DIGIT_R4 = N_DIGIT
C
      END

C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE PIRADDEG     *****                 *
C  *                                                                          *
C  *    ROUTINE TO COMPUTE PI, RADIAN AND DEGREE TO MAXIMUM SINGLE PRECISION  *
C  *      ACCURACY                                                            *
C  *                                                                          *
C  *    DECEMBER 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER                 *
C  ****************************************************************************
C
      SUBROUTINE PIRADDEG (PI, RADIAN, DEGREE)
C
      IMPLICIT NONE
C
      REAL*4 PI, RADIAN, DEGREE
C  ----------------------------------------------------------------------------
C
      PI     = 4.0*ATAN2(1.0,1.0)
      RADIAN = PI/180.0
      DEGREE = 1.0/RADIAN
C
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE PIRADDEG_D     *****               *
C  *                                                                          *
C  *    ROUTINE TO COMPUTE PI, RADIAN AND DEGREE TO MAXIMUM DOUBLE PRECISION  *
C  *      ACCURACY                                                            *
C  *                                                                          *
C  *    APRIL 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER                      *
C  ****************************************************************************
C
      SUBROUTINE PIRADDEG_D (DPI, DRADIAN, DDEGREE)
C
      IMPLICIT NONE
C
      REAL*8 DPI, DRADIAN, DDEGREE
C  ----------------------------------------------------------------------------
C
      DPI     = 4.0*ATAN2(1.0,1.0)
      DRADIAN = DPI/180.0
      DDEGREE = 1.0/DRADIAN
C
      RETURN
      END

C  ***********************************************************************
C  *                                                                     *
C  *              *****     SUBROUTINE REAL_TO_REC     *****             *
C  *                                                                     *
C  *    COMPUTE REAL SPACE PARAMETERS KNOWING THE RECIPROCAL LATTICE     *
C  *     PARAMETERS                                                      *
C  *                                                                     *
C  *    APRIL 1985  PURDUE UNIVERSITY VAX 11/750, T. BAKER               *
C  ***********************************************************************
C
      SUBROUTINE REAL_TO_REC (A_CELL,B_CELL,C_CELL,ALP,BET,GAM,VOL,
     1 A_STAR,B_STAR,C_STAR,VOL_STAR,ALP_STAR,BET_STAR,GAM_STAR)
C
      IMPLICIT NONE

      REAL*4    TEST(3,2), A_CELL, B_CELL, C_CELL, ALP, BET, GAM, VOL
      REAL*4    A_STAR, B_STAR, C_STAR, VOL_STAR
      REAL*4    ALP_STAR, BET_STAR, GAM_STAR
      real*4    cosalp, cosbet, cosgam, sinalp, sinbet, singam
      real*8    deg_to_rad, rad_to_deg
C  ----------------------------------------------------------------------------
C
      deg_to_rad = acos(-1.0d0)/180.0d0
      rad_to_deg = 1.0d0/deg_to_rad

      TEST(1,1) = A_CELL
      TEST(2,1) = B_CELL
      TEST(3,1) = C_CELL
      TEST(1,2) = ALP
      TEST(2,2) = BET
      TEST(3,2) = GAM
      IF(A_CELL.LE.0.0) A_CELL = -1.0
      IF(B_CELL.LE.0.0) B_CELL = -1.0
      IF(C_CELL.LE.0.0) C_CELL = -1.0
      IF(ALP.LE.0.0)       ALP = 90.0
      IF(BET.LE.0.0)       BET = 90.0
      IF(GAM.LE.0.0)       GAM = 90.0

      cosalp = real(cos(alp*deg_to_rad))
      cosbet = real(cos(bet*deg_to_rad))
      cosgam = real(cos(gam*deg_to_rad))
      sinalp = real(sin(alp*deg_to_rad))
      sinbet = real(sin(bet*deg_to_rad))
      singam = real(sin(gam*deg_to_rad))

      VOL = A_CELL*B_CELL*C_CELL * SQRT (1.0 + 
     1     2*COSALP*COSBET*COSGAM-COSALP**2
     2     -COSBET**2-COSGAM**2)
      A_STAR = B_CELL*C_CELL*SINALP/VOL
      B_STAR = C_CELL*A_CELL*SINBET/VOL
      C_STAR = A_CELL*B_CELL*SINGAM/VOL
      VOL    = ABS(VOL)
      IF(A_STAR.LT.0.0) A_STAR = 0.0
      IF(B_STAR.LT.0.0) B_STAR = 0.0
      IF(C_STAR.LT.0.0) C_STAR = 0.0
      ALP_STAR = real(rad_to_deg * ACOS((COSBET*COSGAM-COSALP) / 
     1                     (SINBET*SINGAM)))
      BET_STAR = real(rad_to_deg * ACOS((COSGAM*COSALP-COSBET) / 
     1                     (SINGAM*SINALP)))
      GAM_STAR = real(rad_to_deg * ACOS((COSALP*COSBET-COSGAM) / 
     1                     (SINALP*SINBET)))
      VOL_STAR = 1.0/VOL

      A_CELL = TEST(1,1)
      B_CELL = TEST(2,1)
      C_CELL = TEST(3,1)
      ALP    = TEST(1,2)
      BET    = TEST(2,2)
      GAM    = TEST(3,2)
C
      RETURN
      END

C  ***********************************************************************
C  *                                                                     *
C  *              *****     SUBROUTINE REC_TO_REAL     *****             *
C  *                                                                     *
C  *    COMPUTE RECIPROCAL LATTICE PARAMETERS KNOWING THE REAL SPACE     *
C  *     PARAMETERS                                                      *
C  *                                                                     *
C  *    JULY 1985  PURDUE UNIVERSITY VAX 11/750, T. BAKER                *
C  ***********************************************************************
C
      SUBROUTINE REC_TO_REAL (A_CELL,B_CELL,C_CELL,ALP,BET,GAM,VOL,
     1 A_STAR,B_STAR,C_STAR,VOL_STAR,ALP_STAR,BET_STAR,GAM_STAR)
C
      IMPLICIT NONE
C
      REAL*4    TEST(3,2), A_CELL, B_CELL, C_CELL, ALP, BET, GAM, VOL
      REAL*4    A_STAR, B_STAR, C_STAR, VOL_STAR
      REAL*4    ALP_STAR, BET_STAR, GAM_STAR
      real*8    deg_to_rad, rad_to_deg
      real*4    cosalp_star, cosbet_star, cosgam_star
      real*4    sinalp_star, sinbet_star, singam_star
C  ----------------------------------------------------------------------------
C
      deg_to_rad = acos(-1.0d0)/180.0d0
      rad_to_deg = 1.0d0/deg_to_rad

      TEST(1,1) = A_STAR
      TEST(2,1) = B_STAR
      TEST(3,1) = C_STAR
      TEST(1,2) = ALP_STAR
      TEST(2,2) = BET_STAR
      TEST(3,2) = GAM_STAR
      IF(A_STAR.LE.0.0)   A_STAR   = -1.0
      IF(B_STAR.LE.0.0)   B_STAR   = -1.0
      IF(C_STAR.LE.0.0)   C_STAR   = -1.0
      IF(ALP_STAR.LE.0.0) ALP_STAR = 90.0
      IF(BET_STAR.LE.0.0) BET_STAR = 90.0
      IF(GAM_STAR.LE.0.0) GAM_STAR = 90.0

      cosalp_star = real(cos(alp_star*deg_to_rad))
      cosbet_star = real(cos(bet_star*deg_to_rad))
      cosgam_star = real(cos(gam_star*deg_to_rad))
      sinalp_star = real(sin(alp_star*deg_to_rad))
      sinbet_star = real(sin(bet_star*deg_to_rad))
      singam_star = real(sin(gam_star*deg_to_rad))

      VOL_STAR = A_STAR*B_STAR*C_STAR * SQRT( 1.0 + 
     1     2*COSALP_STAR*COSBET_STAR*COSGAM_STAR
     1      -COSALP_STAR**2 
     1      -COSBET_STAR**2
     1      -COSGAM_STAR**2)
      A_CELL = B_STAR * C_STAR * SINALP_STAR/VOL_STAR
      B_CELL = C_STAR * A_STAR * SINBET_STAR/VOL_STAR
      C_CELL = A_STAR * B_STAR * SINGAM_STAR/VOL_STAR
      VOL_STAR = ABS(VOL_STAR)
      IF(A_CELL.LT.0.0) A_CELL = 0.0
      IF(B_CELL.LT.0.0) B_CELL = 0.0
      IF(C_CELL.LT.0.0) C_CELL = 0.0
      ALP = real(rad_to_deg*ACOS((COSBET_STAR*COSGAM_STAR-COSALP_STAR)/ 
     1            (SINBET_STAR*SINGAM_STAR)))
      BET = real(rad_to_deg*ACOS((COSGAM_STAR*COSALP_STAR-COSBET_STAR)/ 
     1            (SINGAM_STAR*SINALP_STAR)))
      GAM = real(rad_to_deg*ACOS((COSALP_STAR*COSBET_STAR-COSGAM_STAR)/ 
     1            (SINALP_STAR*SINBET_STAR)))
      VOL = 1.0/VOL_STAR
C
      A_STAR   = TEST(1,1)
      B_STAR   = TEST(2,1)
      C_STAR   = TEST(3,1)
      ALP_STAR = TEST(1,2)
      BET_STAR = TEST(2,2)
      GAM_STAR = TEST(3,2)
C
      RETURN
      END

C  ***************************************************************
C  *                                                             *
C  *            *****     SUBROUTINE SEED     *****              *
C  *                                                             *
C  *    GENERATES THE SEED FOR RANDOM NUMBER GENERATOR           *
C  *                                                             *
C  *    THE LARGEST INTEGER*4 NUMBER IN THE VAX IS 2147483647    *
C  *     2147483647/(24*60*60 = seconds in a day) = 24855        *
C  *                                                             *
C  *    LAST MODIFIED 27-JUL-86 BY T. BAKER                      *
C  ***************************************************************
C
      SUBROUTINE SEED (ISEED)
C
      IMPLICIT NONE
C
      REAL*4    SECNDS
      INTEGER*4 ISEED
C  ----------------------------------------------------------------------------
C
      ISEED = 24855 * INT(SECNDS(0.0))
C
      RETURN
      END

C  ****************************************************
C  *                                                  *
C  *      *****     SUBROUTINE SOLV2D     *****       *
C  *                                                  *
C  *    ADOPTED FROM JOACHIM FRANK'S SPIDER SYSTEM    *
C  ****************************************************
C
      SUBROUTINE SOLV2D (A, TST)
C
      IMPLICIT NONE
C
      REAL*4 A(2,3), TST, D, P, Q
C -----------------------------------------------------------------------------
C
      TST = 0.0
      D   = A(1,1)*A(2,2) - A(1,2)*A(2,1)
      P   = A(1,3)*A(2,2) - A(2,3)*A(1,2)
      Q   = A(2,3)*A(1,1) - A(1,3)*A(2,1)
      IF(D.NE.0.0) GO TO 100
      IF(P.EQ.0.0.AND.Q.EQ.0.0) GO TO 300
      GO TO 200
  100      IF(ABS(D).GE.(1.E-6)*ABS(P).AND.ABS(D).GE.(1.E-6)*ABS(Q))
     1 GO TO 400
  200      CONTINUE
      TST    = 1.0
  300      A(1,3) = 0.0
      A(2,3) = 0.0
      RETURN
  400      A(1,3) = P/D
      A(2,3) = Q/D
      RETURN
      END


C  **********************************************************************
C  *                                                                    *
C  *                *****     SUBROUTINE TSBEND     *****               *
C  *                                                                    *
C  *    MARKS THE END OF A PROGRAM BY TYPING 80 *'s ON SYS$OUTPUT       *
C  **********************************************************************
C
      SUBROUTINE TSBEND
C
C  ----------------------------------------------------------------------------
      WRITE(*,1000)
 1000      FORMAT(1X,80('*'))
      CALL EXIT
      END

C  ****************************************************************************
C  *                                                                          *
C  *                    *****     FUNCTION WAVEL     *****                    *
C  *                                                                          *
C  *    COMPUTE ELECTRON WAVELENGTH IN NANOMETERS GIVEN THE VOLTAGE.          *
C  *                                                                          *
C  *    AUGUST 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.         *
C  *    LAST MODIFIED 13-AUG-1999 BY T. BAKER                                 *
C  ****************************************************************************
C
      FUNCTION WAVEL (VOLTS)
C
      IMPLICIT NONE
C
      REAL*4 WAVEL, VOLTS
C  ----------------------------------------------------------------------------
C
      WAVEL = 1.23 / SQRT (VOLTS + (1.0E-6 * VOLTS**2))             ! IN NM
C
      RETURN
      END
