C      PEAKLIB.SUBS SUBROUTINES
C
C      PEAK_CEN      PEAK_CHK      PEAK_SORT
C      PEAK_SORT_HI      PEAK_SEARCH
C  ************************************************************************
C  *                                                                      *
C  *                *****     SUBROUTINE PEAK_CEN     *****               *
C  *                                                                      *
C  *    ROUTINE INTERPOLATES AROUND PEAK POSITIONS FOUND BY SUBROUTINE    *
C  *     PEAK_SEARCH TO DETERMINE ACCURATE PEAK POSITIONS                 *
C  *                                                                      *
C  *    DECEMBER 1985  PURDUE UNIVERSITY VAX 11/750, T. BAKER             *
C  *    LAST MODIFIED 10-JUL-86 BY T. BAKER                               *
C  ************************************************************************
C
      SUBROUTINE PEAK_CEN (PEAKS, NPEAK, IMG, NCOL, NROW)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, NPEAK, IPEAK
      INTEGER*4 IX, IY
      REAL*4 IDEN(7,7)
      INTEGER*4 X1, X2, Y1, Y2, NX, NY, XCEN, YCEN
      REAL*4    TOT, BACK, DSUM, DEN, PEAKS(3,NPEAK), XOFF, YOFF
      REAL*4    DX, DY,IMG(NCOL,NROW)
C  ----------------------------------------------------------------------------
C
C  *************************************************************************
C  *    STEP THROUGH SET OF "PEAKS" AND DETERMINE INTERPOLATED POSITION    *
C  *************************************************************************
      DO IPEAK = 1,NPEAK
        XCEN = NINT(PEAKS(1,IPEAK))
        YCEN = NINT(PEAKS(2,IPEAK))
        X1   = XCEN - 3
        Y1   = YCEN - 3
        X2   = XCEN + 3
        Y2   = YCEN + 3
        IF(X1.LE.0)    X1 = 1
        IF(Y1.LE.0)    Y1 = 1
        IF(X2.GT.NCOL) X2 = NCOL
        IF(Y2.GT.NROW) Y2 = NROW
        NX  = X2-X1+1
        NY  = Y2-Y1+1
        XOFF = REAL(NX-1)/2.0 - XCEN + X1
        YOFF = REAL(NY-1)/2.0 - YCEN + Y1
C      PRINT *, PEAKS(1,IPEAK), PEAKS(2,IPEAK), PEAKS(3,IPEAK)
C      PRINT *, X1,Y1, '   ', X2,Y2
C      WRITE (6,'('' XOFF,YOFF ='',2F10.2)') XOFF, YOFF
C
        TOT = 0.0
        DO IY = 1,NY                        ! COMPUTE BACKGROUND VALUE
          DO IX = 1,NX
            IDEN(IX,IY) = IMG(IX-1+X1,IY-1+Y1)
            TOT         = TOT + IDEN(IX,IY)
          ENDDO
        ENDDO
        BACK = TOT/REAL(NX*NY)
C         *******************
C  -------*    FIND SUMS    *
C         *******************
        DX    = 0.0
        DY    = 0.0
        DSUM  = 0.0
C
        DO IY = 1,NY
          DO IX = 1,NX
            DEN = IDEN(IX,IY)-BACK
            IF(DEN.GE.0.0) THEN
              DX = DX + DEN*IX
              DY = DY + DEN*IY
              DSUM = DSUM + DEN
            ENDIF
          ENDDO
C          WRITE(6,'(9I6)') (IDEN(IX,IY),IX=1,NX)
        ENDDO
        DX = DX/DSUM - REAL(NX+1)/2.0
        DY = DY/DSUM - REAL(NY+1)/2.0
        PEAKS(1,IPEAK) = PEAKS(1,IPEAK) + XOFF + DX
        PEAKS(2,IPEAK) = PEAKS(2,IPEAK) + YOFF + DY
C      WRITE (6,'('' DX,DY ='',2F10.2)') DX,DY
C      WRITE (6,'('' X,Y ='',2F10.2)') PEAKS(1,IPEAK),PEAKS(2,IPEAK)
      ENDDO
C
      RETURN
      END
C
C  *****************************************************************************
C  *                                                                           *
C  *                    *****     SUBROUTINE PEAK_CHK     *****                *
C  *                                                                           *
C  *    ROUTINE TO CHECK FOR PEAKS FOUND BY SUBROUTINE PEAK_SEARCH AND TO      *
C  *     FIND THE HIGHEST PEAK WITHIN A CIRCULAR WINDOW OF RADIUS "D"          *
C  *                                                                           *
C  *    NPEAK        = # OF PEAKS FOUND ABOVE A THRESHOLD PIXEL VALUE          *
C  *    NPEAK_UNIQUE = # OF UNIQUE PEAKS LEFT AFTER IMPOSING WINDOW CONSTRAINT *
C  *                                                                           *
C  *    PEAKS WHICH ARE FLAGGED, I.E. THOSE WHICH DO NOT HAVE THE HIGHEST      *
C  *     VALUE WITHIN A GIVEN WINDOW, ARE GIVEN A NEGATIVE PIXEL VALUE.        *
C  *                                                                           *
C  *    APRIL 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER                     *
C  *    LAST MODIFIED  1-SEP-88 BY T. BAKER                                    *
C  *****************************************************************************
C
      SUBROUTINE PEAK_CHK (PEAKS, NPEAK, NPEAK_UNIQUE, D)
C
      IMPLICIT NONE
C
      INTEGER*4 NPEAK, NPEAK_UNIQUE, I, J
      REAL*4    PEAKS(3,NPEAK), D, DSQ, TEST
C  ----------------------------------------------------------------------------
C
      IF(NPEAK.LT.2) THEN
        IF(NPEAK.LT.1) THEN
           PRINT *, '***************************'
           PRINT *, '* WARNING: No peaks found *'
           PRINT *, '***************************'
         ELSE
           PRINT *, '********************************'
           PRINT *, '* WARNING: Only one peak found *'
           PRINT *, '********************************'
        ENDIF
        RETURN
      ENDIF
C
      DSQ          = D*D
      NPEAK_UNIQUE = NPEAK
C
      DO I = 1,(NPEAK-1)
        IF(PEAKS(3,I).GE.0.0) THEN
          DO J = (I+1),NPEAK
            IF(PEAKS(3,J).GE.0.0) THEN
              TEST = (PEAKS(1,I)-PEAKS(1,J))**2 + 
     1              (PEAKS(2,I)-PEAKS(2,J))**2
              IF(TEST.LE.DSQ) THEN
                PEAKS(3,J)   = -PEAKS(3,J)      ! FLAG FAILED PEAK
                NPEAK_UNIQUE = NPEAK_UNIQUE - 1
              ENDIF
            ENDIF
          ENDDO
        ENDIF
      ENDDO
      WRITE(6,'(I6,A)') NPEAK_UNIQUE, ' unique peaks found'
      RETURN

      END
C
C  *****************************************************************
C  *                                                               *
C  *            *****     SUBROUTINE PEAK_SORT     *****           *
C  *                                                               *
C  *    ROUTINE TO SORT PEAKS FOUND BY SUBROUTINE PEAK_SEARCH SO   *
C  *      HIGHEST VALUES ARE AT THE TOP OF ARRAY "PEAKS"           *
C  *    THIS USES A FAST "BUBBLE-SORT" ROUTINE                     *
C  *                                                               *
C  *    APRIL 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER         *
C  *    LAST MODIFIED  23-DEC-85 BY T. BAKER                       *
C  *****************************************************************
C
      SUBROUTINE PEAK_SORT (PEAKS, NPEAK)
C
      IMPLICIT NONE
C
      INTEGER*4 NPEAK, IEND, I, J, K
      REAL*4    PEAKS(3,NPEAK), TMP
      LOGICAL*1 TEST
C  ----------------------------------------------------------------------------
C
      TEST = .FALSE.
      IEND = NPEAK-1
C
      DO WHILE (.NOT.TEST)
        TEST = .TRUE.
        DO I = 1,IEND
          IF(PEAKS(3,I).LT.PEAKS(3,I+1)) THEN
            TEST = .FALSE.
            DO K = 1,3
              TMP          = PEAKS(K,I)
              PEAKS(K,I)   = PEAKS(K,I+1)
              PEAKS(K,I+1) = TMP
            ENDDO
            J = I
            DO WHILE (J.GT.1.AND.PEAKS(3,J).GE.PEAKS(3,J-1))
              DO K = 1,3
                TMP          = PEAKS(K,J)
                PEAKS(K,J)   = PEAKS(K,J-1)
                PEAKS(K,J-1) = TMP
              ENDDO
              J = J-1
            ENDDO
          ENDIF
        ENDDO
        IEND = IEND-1
      ENDDO
C  ************************************************************
C  *    FINISHED SORTING: RECALCULATE VALUE OF NPEAK          *
C  *                                                          *
C  *    NPEAK WILL CHANGE IF THERE ARE FLAGGED PEAK VALUES    *
C  ************************************************************
      DO I = 1,NPEAK
        IF(PEAKS(3,I).LT.0.0) THEN
          NPEAK = I-1
          IF(NPEAK.LT.1) THEN
            PRINT *, '************************************'
            PRINT *, '* WARNING: No pixels pass the test *'
            PRINT *, '************************************'
          ENDIF
          RETURN
        ENDIF
      ENDDO
      RETURN
C
      END


C*******************************************************************************
C                                                                              *
C      Procedure : PEAK_SORT_HI                                               *
C                                                                             *
C      Object  : The Array PEAKS is sorted by such that the highest peak      *
C                  value is stored in the first array position (i.e. de-       *
C                  creasing  order, using a QUICKSORT algorithm, on a key      *
C                  being peaks(3,*).                                           *
C                                                                              *
C      Parameters :                                                           *
C            PEAKS             REAL*4(3,NPEAK)      Array to be sorted             *
C            NPEAK            INTEGER*4      SIZE of the Array              *
C                                                                              *
C      Authors  :                                                             *
C                 Eric P. Meyer [EPM], Purdue University 18-JUN-87            *
C                                                                              *
C      Release : V01.00      Initial Release                  02-JUL-87 EPM  *
C                 V01.01        Converted to two PEAK_SORTs     18-MAY-88 TSB  *
C                  W. Zhang, 11-FEB-1999: 
C                            Add more conditions in  line 3000 and 4000
C*******************************************************************************
C                                                                            
c
c       New Peak Sort Routine using QuickSort Algorithm
C
      SUBROUTINE PEAK_SORT_HI (PEAKS, NPEAK)
C
      IMPLICIT NONE
C
      INTEGER*4 NPEAK, MIN_QUICK_SORT_SIZE, STACK_SIZE
      PARAMETER (MIN_QUICK_SORT_SIZE = 9, STACK_SIZE = 1024)
      INTEGER*4 STACK_TOP, LEFT, RIGHT, INDEX_LEFT, INDEX_RIGHT
      INTEGER*4 INDEX, INDEX_2, STACK(STACK_SIZE,2)
      REAL*4    PEAKS(3,NPEAK), KEY, TMP(3), REC(3)
C  -----------------------------------------------------------------------------
C
      IF (NPEAK.LE.MIN_QUICK_SORT_SIZE) GO TO 9000
C
      STACK_TOP = 0
      LEFT      = 1
      RIGHT     = NPEAK
C
2000      INDEX_LEFT  = LEFT
      INDEX_RIGHT = RIGHT + 1
      KEY         = PEAKS(3,LEFT)
      REC(1)      = PEAKS(1,LEFT)
      REC(2)      = PEAKS(2,LEFT)
      REC(3)      = PEAKS(3,LEFT)
C                        
3000      INDEX_LEFT = INDEX_LEFT + 1
      IF (PEAKS(3,INDEX_LEFT).GT.KEY.AND.INDEX_LEFT
     1      .LE.NPEAK) GO TO 3000
C
4000      INDEX_RIGHT = INDEX_RIGHT - 1
      IF (PEAKS(3,INDEX_RIGHT).LT.KEY.AND.INDEX_RIGHT
     1      .GE.0) GO TO 4000
C   
      IF (INDEX_RIGHT.LE.INDEX_LEFT) THEN
        TMP(1)               = PEAKS(1,INDEX_RIGHT)
        TMP(2)               = PEAKS(2,INDEX_RIGHT)
        TMP(3)               = PEAKS(3,INDEX_RIGHT)
        PEAKS(1,INDEX_RIGHT) = PEAKS(1,LEFT)
        PEAKS(2,INDEX_RIGHT) = PEAKS(2,LEFT)
        PEAKS(3,INDEX_RIGHT) = PEAKS(3,LEFT)
        PEAKS(1,LEFT)        = TMP(1)
        PEAKS(2,LEFT)        = TMP(2)
        PEAKS(3,LEFT)        = TMP(3)
        GO TO 7000
      ENDIF
C
      TMP(1)               = PEAKS(1,INDEX_RIGHT)
      TMP(2)               = PEAKS(2,INDEX_RIGHT)
      TMP(3)               = PEAKS(3,INDEX_RIGHT)
      PEAKS(1,INDEX_RIGHT) = PEAKS(1,INDEX_LEFT)
      PEAKS(2,INDEX_RIGHT) = PEAKS(2,INDEX_LEFT)
      PEAKS(3,INDEX_RIGHT) = PEAKS(3,INDEX_LEFT)
      PEAKS(1,INDEX_LEFT)  = TMP(1)
      PEAKS(2,INDEX_LEFT)  = TMP(2)
      PEAKS(3,INDEX_LEFT)  = TMP(3)
      GO TO 3000
C
7000      IF (((RIGHT-INDEX_RIGHT).GE.(INDEX_RIGHT-LEFT)).AND.
     +         ((INDEX_RIGHT-LEFT).GT.MIN_QUICK_SORT_SIZE)) THEN
        STACK_TOP          = STACK_TOP + 1
        STACK(STACK_TOP,1) = INDEX_RIGHT + 1
        STACK(STACK_TOP,2) = RIGHT
        RIGHT              = INDEX_RIGHT - 1
        GO TO 2000
      ENDIF
      IF (((INDEX_RIGHT-LEFT).GT.(RIGHT-INDEX_RIGHT)).AND.
     +         ((RIGHT-INDEX_RIGHT).GT.MIN_QUICK_SORT_SIZE)) THEN
        STACK_TOP          = STACK_TOP + 1
        STACK(STACK_TOP,1) = LEFT
        STACK(STACK_TOP,2) = INDEX_RIGHT - 1
        LEFT               = INDEX_RIGHT - 1
        GO TO 2000
      ENDIF
      IF (((RIGHT-INDEX_RIGHT).GT.min_quick_sort_size).AND.
     +         (MIN_QUICK_SORT_SIZE.GE.(INDEX_RIGHT-LEFT))) THEN
        LEFT = INDEX_RIGHT + 1
        GO TO 2000
      ENDIF
      IF (((INDEX_RIGHT-LEFT).GT.MIN_QUICK_SORT_SIZE).AND.
     +         (MIN_QUICK_SORT_SIZE.GE.(RIGHT-INDEX_RIGHT))) THEN
        RIGHT = INDEX_RIGHT - 1
        GO TO 2000
      ENDIF
C
      IF (STACK_TOP.NE.0) THEN
        LEFT      = STACK(STACK_TOP,1)
        RIGHT     = STACK(STACK_TOP,2)
        STACK_TOP = STACK_TOP - 1
        GO TO 2000
      ENDIF
C
9000      DO INDEX = 2,NPEAK
        IF (PEAKS(3,INDEX - 1).LT.PEAKS(3,INDEX)) THEN
          KEY     = PEAKS(3,INDEX)
          REC(1)  = PEAKS(1,INDEX)
          REC(2)  = PEAKS(2,INDEX)
          REC(3)  = PEAKS(3,INDEX)
          INDEX_2 = INDEX - 1
8100          PEAKS(1,INDEX_2+1) = PEAKS(1,INDEX_2)
          PEAKS(2,INDEX_2+1) = PEAKS(2,INDEX_2)
          PEAKS(3,INDEX_2+1) = PEAKS(3,INDEX_2)
          INDEX_2            = INDEX_2 - 1
          IF (INDEX_2.GT.0.AND.
     1    (.NOT.(PEAKS(3,INDEX_2).GT.KEY))) GO TO 8100
          PEAKS(1,INDEX_2+1) = REC(1)
          PEAKS(2,INDEX_2+1) = REC(2)
          PEAKS(3,INDEX_2+1) = REC(3)
        ENDIF
      ENDDO
C
      RETURN
      END
C
C  **************************************************************************
C  *                                                                        *
C  *               *****     SUBROUTINE PEAK_SEARCH     *****               *
C  *                                                                        *
C  *    ROUTINE TO SEARCH FOR ALL NON-ADJACENT "IMG" VALUES ABOVE LOWLIM    *
C  *                                                                        *
C  *    PEAKS(1,NPEAK) = COLUMN POSITION OF "PEAK"                          *
C  *    PEAKS(2,NPEAK) = ROW POSITION OF "PEAK"                             *
C  *    PEAKS(3,NPEAK) = INTENSITY OF "PEAK"                                *
C  *                                                                        *
C  *    APRIL 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER                  *
C  *    LAST MODIFIED  1-SEP-88 BY T. BAKER                                 *
C  *    MODIFIED BY W. ZHANG, IMG(INTEGER*2-> REAL4                         *
C  **************************************************************************
C
      SUBROUTINE PEAK_SEARCH (IMG, NCOL, NROW, PEAKS, NLIM, NPEAK,
     1                       LOWLIM)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, NLIM, NPEAK, LOWLIM, COL, ROW
      REAL*4    ITEST
      INTEGER*4 J_LAST, J_NEXT
      REAL*4    PEAKS(3,NLIM),IMG(NCOL,NROW)
C  ----------------------------------------------------------------------------
C
      WRITE(6,*) ' # of non-adjacent pixels with intensities > ',
     1     LOWLIM, ' = Wait for answer'
C
      NPEAK = 0                        ! RESET NPEAK COUNTER
C
      DO ROW = 1,NROW                        ! STEP THROUGH ROW BY ROW
        J_LAST = ROW - 1
        J_NEXT = ROW + 1
        DO COL = 1,NCOL                  ! STEP THROUGH CURRENT ROW
          ITEST = IMG(COL,ROW)
          IF(ITEST.GT.LOWLIM) THEN
            IF(COL.LT.NCOL) THEN
              IF(ITEST.LE.IMG(COL+1,ROW)) GO TO 100
            ENDIF
            IF(COL.GT.1) THEN
              IF(ITEST.LT.IMG(COL-1,ROW)) GO TO 100
            ENDIF
            IF(J_NEXT.LE.NROW) THEN
              IF(ITEST.LE.IMG(COL,J_NEXT)) GO TO 100
            ENDIF
            IF(J_LAST.GT.0) THEN
              IF(ITEST.LT.IMG(COL,J_LAST)) GO TO 100
            ENDIF
            NPEAK = NPEAK+1
            IF(NPEAK.GT.NLIM) THEN            ! TRAP FOR ARRAY OVERFLOW
              NPEAK = -1                        ! SIGNAL OVERFLOW
              PRINT *, '***************************************'
              PRINT *, '* TOO MANY PEAK PIXELS - RESET LOWLIM *'
              PRINT *, '***************************************'
              RETURN
            ENDIF
            PEAKS(1,NPEAK) = REAL(COL)
            PEAKS(2,NPEAK) = REAL(ROW)
            PEAKS(3,NPEAK) = IMG(COL,ROW)
          ENDIF
  100       CONTINUE
        ENDDO
      ENDDO
C
      WRITE(6,*) '# of non-adjacent pixels with intensities > ',
     1     LOWLIM, ' = ', NPEAK
C
      RETURN
      END
