// PIF library I/O routines
// Author Robert W. Ashmore
// Mods by James Conway, 23&27 May, 1997, and March 2000.
// Additional mods by Robert Sinkovits, 2007+

#include <sys/types.h>
#include <string.h>
#include "../include/common.h"
#include "../include/libPIF.h"

#define VERSION "2.10_20030619 libPIF"
#define MAXLUNS 1000
#define OKREAD(x,y,z) if(x!=y){printf("[%s] [PIF] Error accessing file.\n",z);\
exit(0);}

static FILE *lun[MAXLUNS];
static int   endians[MAXLUNS];
static int convTsb= 0;
static int i2Map= TRUE;


void pif_tsbworld(int setValue) 
{
  // function name: pif_tsbworld
  // Setup for whether we read datat from disk in an XWindows type
  // format where the first row on the disk is actually the last row
  // in the data unit (convTsb = 1) OR the first row on the disk is
  // the first row in the data unit (convTsb = 0)

  convTsb = setValue;
} // end function pif_tsbworld

void pif_tsbworld_(int setValue) {
  pif_tsbworld(setValue);
}

void pif_tsbworld__(int setValue) {
  pif_tsbworld(setValue);
}



void calcMapStats(float *inbuffer, float *img_bkg, float *map_min,
		  float *map_max, float *img_avg, float *img_std,
		  int *width, int *height, int *nSecs) 
{
  // function name: calcMapStats
  // Calculates statistics from the MAP

  int i, j, k, wd, hi, ns;
  int tot=0;
  float totValues=0.0, pixelValue;
  double var=0.0, sumsq=0.0;

  wd = *width;
  hi = *height;
  ns = *nSecs;

  *img_avg = 0.0;
  *img_std = 0.0;
  *map_min = *map_max = inbuffer[0];

  // Accumulate intermediate values (Use non-zeros also)
  for(k=0; k<ns; k++) {
    for(i=0; i<hi; i++) {
      for(j=0; j<wd; j++) {
	pixelValue = inbuffer[k*hi*wd + i*wd + j];
	*map_min = MIN(*map_min, pixelValue);
	*map_max = MAX(*map_max, pixelValue);
	sumsq += (pixelValue * pixelValue);
	totValues +=  (float)pixelValue;
	tot++;
      }
    }
  }
  
  // Calc the background
  *img_bkg= inbuffer[2];   // Bkg in the first section

  // Calc stats
  *img_avg = totValues / (float)(wd * hi * ns);
  var = (sumsq / (float)(wd*hi*ns)) - (*img_avg * *img_avg);
  if(var > 0.0) *img_std = (float)sqrt(var);

} // calcMapStats

void calcmapstats(float *inbuffer, float *img_bkg, float *map_min,
		  float *map_max, float *img_avg, float *img_std,
		  int *width, int *height, int *nSecs) {
  calcMapStats(inbuffer, img_bkg, map_min, map_max, img_avg, img_std,
		width, height, nSecs);
}

void calcmapstats_(float *inbuffer, float *img_bkg, float *map_min,
		   float *map_max, float *img_avg, float *img_std,
		   int *width, int *height, int *nSecs) {
  calcMapStats(inbuffer, img_bkg, map_min, map_max, img_avg, img_std,
		width, height, nSecs);
}

void calcmapstats__(float *inbuffer, float *img_bkg, float *map_min,
		   float *map_max, float *img_avg, float *img_std,
		   int *width, int *height, int *nSecs) {
  calcMapStats(inbuffer, img_bkg, map_min, map_max, img_avg, img_std,
		width, height, nSecs);
}

void calcMapStats_(float *inbuffer, float *img_bkg, float *map_min,
		  float *map_max, float *img_avg, float *img_std,
		  int *width, int *height, int *nSecs) {
  calcMapStats(inbuffer, img_bkg, map_min,
		map_max, img_avg, img_std,
		width, height, nSecs);
}

void calcMapStats__(float *inbuffer, float *img_bkg, float *map_min,
		  float *map_max, float *img_avg, float *img_std,
		  int *width, int *height, int *nSecs) {
  calcMapStats(inbuffer, img_bkg, map_min,
		map_max, img_avg, img_std,
		width, height, nSecs);
}



void pif_set_i2map(int *value) 
{
  // function name: pif_set_i2map
  // Setup for I2 writing from fortran

  i2Map = *value;

} // end function pif_set_i2map

void pif_set_i2map_(int *value) {
  pif_set_i2map(value);
}

void pif_set_i2map__(int *value) {
  pif_set_i2map(value);
}



void determine_scaling(forPifHead_t *thisHead, float *inbuffer,
			int *ncol, int *nrow, int *nsec) 
{
  // function name: determine_scaling
  // Calculate the factor needed to convert out floatIntegers into
  // floats when we read this file later.  Just scan all of our values
  // and pick a value that maintains our dynamic range

  int minVal=99999, maxVal=-99999;
  float img_bkg;

  // calculate stats first
  calcMapStats(inbuffer, &img_bkg, 
	       &(thisHead->datar4_min), &(thisHead->datar4_max),
	       &(thisHead->datar4_avg), &(thisHead->datar4_std),
	       ncol, nrow, nsec);
  thisHead->datai4_min = thisHead->datar4_min;
  thisHead->datai4_max = thisHead->datar4_max;

  // Don't forget to account for alp,bet,gam etc...
  minVal = MIN(minVal, thisHead->datai4_min);
  minVal = MIN(minVal, thisHead->datai4_max);
  minVal = MIN(minVal,(int)(thisHead->datar4_min));
  minVal = MIN(minVal,(int)(thisHead->datar4_max));
  minVal = MIN(minVal,(int)(thisHead->datar4_avg));
  minVal = MIN(minVal,(int)(thisHead->datar4_std));
  minVal = MIN(minVal,(int)(thisHead->aoverb));
  minVal = MIN(minVal,(int)(thisHead->map_abang));
  minVal = MIN(minVal,(int)(thisHead->a_cell));
  minVal = MIN(minVal,(int)(thisHead->b_cell));
  minVal = MIN(minVal,(int)(thisHead->c_cell));
  minVal = MIN(minVal,(int)(thisHead->alp));
  minVal = MIN(minVal,(int)(thisHead->bet));
  minVal = MIN(minVal,(int)(thisHead->gam));
  minVal = MIN(minVal,(int)(thisHead->dela));
  minVal = MIN(minVal,(int)(thisHead->delb));
  minVal = MIN(minVal,(int)(thisHead->delc));
  minVal = MIN(minVal,(int)(thisHead->a_star));
  minVal = MIN(minVal,(int)(thisHead->b_star));
  minVal = MIN(minVal,(int)(thisHead->c_star));
  minVal = MIN(minVal,(int)(thisHead->alp_star));
  minVal = MIN(minVal,(int)(thisHead->bet_star));
  minVal = MIN(minVal,(int)(thisHead->gam_star));
  minVal = MIN(minVal,(int)(thisHead->pixelSize));
  minVal = MIN(minVal,(int)(thisHead->dthe));
  minVal = MIN(minVal,(int)(thisHead->dphi_90));
  minVal = MIN(minVal,(int)(thisHead->tempFactor));
  minVal = MIN(minVal,(int)(thisHead->wiener));

  maxVal = MAX(maxVal, thisHead->datai4_min);
  maxVal = MAX(maxVal, thisHead->datai4_max);
  maxVal = MAX(maxVal,(int)(thisHead->datar4_min));
  maxVal = MAX(maxVal,(int)(thisHead->datar4_max));
  maxVal = MAX(maxVal,(int)(thisHead->datar4_avg));
  maxVal = MAX(maxVal,(int)(thisHead->datar4_std));
  maxVal = MAX(maxVal,(int)(thisHead->aoverb));
  maxVal = MAX(maxVal,(int)(thisHead->map_abang));
  maxVal = MAX(maxVal,(int)(thisHead->a_cell));
  maxVal = MAX(maxVal,(int)(thisHead->b_cell));
  maxVal = MAX(maxVal,(int)(thisHead->c_cell));
  maxVal = MAX(maxVal,(int)(thisHead->alp));
  maxVal = MAX(maxVal,(int)(thisHead->bet));
  maxVal = MAX(maxVal,(int)(thisHead->gam));
  maxVal = MAX(maxVal,(int)(thisHead->dela));
  maxVal = MAX(maxVal,(int)(thisHead->delb));
  maxVal = MAX(maxVal,(int)(thisHead->delc));
  maxVal = MAX(maxVal,(int)(thisHead->a_star));
  maxVal = MAX(maxVal,(int)(thisHead->b_star));
  maxVal = MAX(maxVal,(int)(thisHead->c_star));
  maxVal = MAX(maxVal,(int)(thisHead->alp_star));
  maxVal = MAX(maxVal,(int)(thisHead->bet_star));
  maxVal = MAX(maxVal,(int)(thisHead->gam_star));
  maxVal = MAX(maxVal,(int)(thisHead->pixelSize));
  maxVal = MAX(maxVal,(int)(thisHead->dthe));
  maxVal = MAX(maxVal,(int)(thisHead->dphi_90));
  maxVal = MAX(maxVal,(int)(thisHead->tempFactor));
  maxVal = MAX(maxVal,(int)(thisHead->wiener));

  // Get greatest magnitude
  minVal = fabs((float)minVal);
  maxVal = MAX(minVal, maxVal);

  // Give a little extra so we're guaranteed NOT to run over the top
  if(i2Map) {
     if ( ((double)MAXSHORT / (double)maxVal) >1.05) maxVal = maxVal*1.05;
  } else {
     if ( ((double)MAXINT / (double)maxVal) >1.05) maxVal = maxVal*1.05;
  }
  
  // We're doing MAPs as shorts so make it smaller
  if(i2Map)
    thisHead->floatScaleFactor = (double)MAXSHORT / (double)maxVal;
  else
    thisHead->floatScaleFactor = (double)MAXINT / (double)maxVal;

  thisHead->int4scaleF = (double)MAXINT / (double)maxVal;

  thisHead->recipFloatScaleFactor = 1.0 / thisHead->floatScaleFactor; 

} // end function determine_scaling

void determine_scaling_(forPifHead_t *thisHead, float *inbuffer,
		       int *ncol, int *nrow, int *nsec) {
  determine_scaling(thisHead, inbuffer, ncol, nrow, nsec);
}

void determine_scaling__(forPifHead_t *thisHead, float *inbuffer,
		       int *ncol, int *nrow, int *nsec) {
  determine_scaling(thisHead, inbuffer, ncol, nrow, nsec);
}



void increment_fname_unix(char *inFname, char *outFname) 
{
  // function name: increment_fname_unix
  // Increment the number after the tilde '~' in the extension. Used by
  // pft.for to calculate the out filename.  YOU MUST HAVE 256 BYTES for
  // the OUTFNAME and it's recommended to have 256 for the inFname. UNIX
  // is (char *) VMS is (fString_t *)

  char *s1, *s2, tmpStr[512];
  int i, incrVal=1;
  
  for(i=0; i<256; i++) {
    tmpStr[i]= '\0';
    outFname[i]= '\0';
  }
  
  // FORTRAN doesn't always NULL terminate its strings esp. if they're
  // part of an array of strings. Let's just get what we need here
  
  for(i=0; i<256; i++)
    tmpStr[i]= inFname[i];
  
  tmpStr[255]= '\0';
  s1= strchr(tmpStr,' ');
  if(s1) *s1= '\0'; // Get rid of trailing blanks
  if(strlen(tmpStr) > 250) {
    printf("Error - Filename too long [%s]\n",inFname);
    exit(0);
  }
  s1= strrchr(tmpStr,'_');
  s2= strrchr(tmpStr,'.');
  if(s1 && (s1 > s2)) { // Find out the last value
    incrVal = (int)strtod((s1+1),0);
    incrVal++;
    *(s1)= '\0';   // Get rid of everything at and after the underscore
  }
  
  sprintf(outFname, "%s_%03d", tmpStr, incrVal); 
  
} // end function increment_fname_unix

void increment_fname_unix_(char *inFname, char *outFname) {
  increment_fname_unix(inFname, outFname);
}

void increment_fname_unix__(char *inFname, char *outFname) {
  increment_fname_unix(inFname, outFname);
}



void increment_fname(fString_t *inFname, fString_t *outFname) 
{
  // function name: increment_fname
  //
  // Increment the number after the tilde '~' in the extension. Used
  // by pft.for to calculate the out filename. YOU MUST HAVE 256 BYTES
  // for the OUTFNAME and it's recommended to have 256 for the
  // inFname. VMS sends funky string descriptor. UNIX uses char *
  
  increment_fname_unix((char *)inFname,(char *)outFname);

} // end function increment_fname

void increment_fname_(fString_t *inFname, fString_t *outFname) {
  increment_fname(inFname, outFname);
}

void increment_fname__(fString_t *inFname, fString_t *outFname) {
  increment_fname(inFname, outFname);
}



static int Util_IAmBigEndian(void)
{
  // function name: Util_IAmBigEndian
  //
  // Written by James Conway.  Auto-detect whether we're BIG or little
  // ENDIAN.  Return TRUE if this is a big-endian machine (byte order is
  // hi/lo), else FALSE (byte order is lo/hi)

  static int firstTime = TRUE;
  static int bigEndian;

  if(firstTime)  {
    union {
      unsigned char  bytes[2];
      unsigned short word;
    } byteWord;
    firstTime = FALSE;
    byteWord.word = 1;
    bigEndian = (byteWord.bytes[1] == 1);
  }
  return(bigEndian);

} // end function Util_IAmBigEndian

static int Util_IAmBigEndian_(void) {
  return(Util_IAmBigEndian());
}

static int Util_IAmBigEndian__(void) {
  return(Util_IAmBigEndian());
}



void differentEndian(BYTE *inStr, int length, int endianNess) 
{
  // function name: differentEndian
  // Convert this to a different ENDIAN bit of memory

  int i, lm1;
  BYTE thisStr[8];

  if((endianNess == 0) != (Util_IAmBigEndian() == 0)) {
    for(i=0; i<length; i++) 
      *(thisStr + i)= *(inStr + i);
    lm1= length - 1;
    for(i=0; i<length; i++) 
      *(inStr + i) = (*(thisStr + (lm1-i)) & 0377);
  }

} // end function differentEndian

void differentEndian_(BYTE *inStr, int length, int endianNess ) {
  differentEndian(inStr, length, endianNess );
}

void differentEndian__(BYTE *inStr, int length, int endianNess ) {
  differentEndian(inStr, length, endianNess );
}



void convertBackFloat(int thisInt, float *thisFloat, 
		      float recipFloatScaleFactor) 
{
  // function name: convertBackFloat
  // Convert the int passed in to the float passed. Use the
  // recipFloatScaleFactor read from the file.

#ifdef USETHISKLUDGE
  // We should be using 32768 (2^15) here instead of 30000
  if(i2Map && (thisInt >= 32768 || thisInt <= -32768)) // bump this up to 4 bytes
      *thisFloat = (float)(((float)thisInt) * recipFloatScaleFactor)/64000.0;
  else
#endif
    *thisFloat = (float)(thisInt * recipFloatScaleFactor);

} // end function convertBackFloat

void convertBackFloat_(int thisInt, float *thisFloat, 
		      float recipFloatScaleFactor) {
  convertBackFloat(thisInt, thisFloat, recipFloatScaleFactor);
}

void convertBackFloat__(int thisInt, float *thisFloat, 
		      float recipFloatScaleFactor) {
  convertBackFloat(thisInt, thisFloat, recipFloatScaleFactor);
}



void convertFloat(int *thisInt, float thisFloat, float floatScaleFactor) 
{
  // function name: convertFloat
  // Convert the float passed in to the int passed.  Use the
  // floatScaleFactor calculated by calcScaleFactor() below

#ifdef USETHISKLUDGE
  if(i2Map) // bump this up to 4 bytes
  // We should be using 32768 (2^15) here instead of 30000
    *thisInt = (int)((thisFloat * 64000.0) * floatScaleFactor);
  else
#endif
    *thisInt = thisFloat * floatScaleFactor;
  
} // end function convertFloat

void convertFloat_(int *thisInt, float thisFloat, float floatScaleFactor) {
  convertFloat(thisInt, thisFloat, floatScaleFactor);
}

void convertFloat__(int *thisInt, float thisFloat, float floatScaleFactor) {
  convertFloat(thisInt, thisFloat, floatScaleFactor);
}



void pif_close(int *luNum) 
{
  // function name: pif_close
  
  fclose(lun[*luNum]);
  
} // end function pif_close

void pif_close_(int *luNum) {
  pif_close(luNum);
}

void pif_close__(int *luNum) {
  pif_close(luNum);
}



void pif_open(char *fName, int *luNum, char *readWrite) 
{
  // function name: pif_open

  int i, leadingWS, thisPos;
  char *s, fin_name[210], r_w[4];

  s = (fName);
  fin_name[0] = '\0';
  // change the Fortran filename to C string
  leadingWS = TRUE;
  thisPos = 0;
  for(i=0; i < 200 ; i++) {
    if(*s != ' ' && leadingWS) leadingWS = FALSE;
    if(*s == ' ' && !leadingWS) break;
    fin_name[thisPos] = *s;
    if(*s != ' ') thisPos++; // Then we've got a good one
    s++;
  }
  fin_name[thisPos] = '\0';
  r_w[0] = *(readWrite);
  r_w[1] = *(readWrite+1);
  r_w[2] = '\0';

  if(*luNum >= MAXLUNS) {
    printf("PIF_OPEN Maximum LUN exceeded [%d]\n",*luNum);
    exit(0);
  }
  
  lun[*luNum] = fopen( fin_name, r_w );
  if((lun[*luNum])==0) {
    printf("****************************************\n");
    printf("* [PIF_OPEN] Error opening [%s] for [%s]\n\n", fin_name, r_w);
    printf("****************************************\n");
    exit(0);
  }
  
} // end function pif_open

void pif_open_(char *fName, int *luNum, char *readWrite) {
  pif_open(fName, luNum, readWrite);
}

void pif_open__(char *fName, int *luNum, char *readWrite) {
  pif_open(fName, luNum, readWrite);
}



int pif_read_gh(forPifHead_t *thisHead) 
{
  // function name: pif_read_gh
  // Read the PIF file's global header. Changed reading in of
  // FloatScaleFactor, R. Ashmore, D. Belnap, 30 Sep 1997
  
  char floatScaleStr[17], ctfStr[16], file_id[4], *s1;
  int endian;
  FILE *fIn;

  fIn = lun[thisHead->luNum];
  
  // [1] Read global file header
  if(fseek(fIn,0,0) != 0) {
    printf("[PIF_READ_GH] Error seeking file.\n");
    exit(0);
  }

  // Read the file type header - its there twice, check the 2nd one
  if(fread(&file_id[0],sizeof(file_id),1,fIn) != 1) {
    printf("[PIF_READ_GH] Error reading file.\n");
    exit(0);
  }
  if(fread(&file_id[0],sizeof(file_id),1,fIn) != 1) {
    printf("[PIF_READ_GH] Error reading file.\n");
    exit(0);
  }

  if(file_id[0]!=8 && file_id[1]!=0 && file_id[2]!=0 && file_id[3]!=0) {
    printf("[PIF_READ_GH] Don't understand file type [%d,%d,%d,%d]\n"
	   ,file_id[0] ,file_id[1] ,file_id[2] ,file_id[3] );
    return(FALSE);
  }
  
  fread(floatScaleStr,sizeof(char),16,fIn);
  floatScaleStr[16]= '\0';
  s1 = floatScaleStr;
  while(*s1==' ') s1++; // Get rid of white space
  thisHead->recipFloatScaleFactor = strtod(s1, 0);
  fflush(stdout);
  /* 24 */

  if(thisHead->recipFloatScaleFactor == 0.0) {
    printf("Invalid scaling factor[%f]\n", thisHead->recipFloatScaleFactor);
    thisHead->recipFloatScaleFactor= 1.0;
  }

  thisHead->floatScaleFactor = 1.0/thisHead->recipFloatScaleFactor;

  fread(&(thisHead->nimg), sizeof(int), 1, fIn);
  fread(&(thisHead->endianNess), sizeof(int), 1, fIn);
  endians[thisHead->luNum] = thisHead->endianNess != 0;
  endian = endians[thisHead->luNum];
  /* 32 */

  differentEndian((BYTE *)&(thisHead->nimg), 4, endian);

  fread(thisHead->versionStr, sizeof(char), 32, fIn); 
  thisHead->pifVersion = strtod(thisHead->versionStr, 0);
  /* 64 */

  fread(&(thisHead->htype), sizeof(int), 1, fIn);   
  fread(&(thisHead->numCols), sizeof(int), 1, fIn); 
  fread(&(thisHead->numRows), sizeof(int), 1, fIn); 
  fread(&(thisHead->numSecs), sizeof(int), 1, fIn); 
  differentEndian((BYTE *)&(thisHead->htype), 4, endian);
  differentEndian((BYTE *)&(thisHead->numCols), 4, endian);
  differentEndian((BYTE *)&(thisHead->numRows), 4, endian);
  differentEndian((BYTE *)&(thisHead->numSecs), 4, endian);
  if(thisHead->htype != 1) { // htype==1 means all images same size
    printf("[PIF_READ_GH] Don't understand this type [%d]\n",
	   thisHead->htype); 
    return(FALSE);
  }
  /* 80 */

  fread(&(thisHead->mode), sizeof(int), 1, fIn); 
  differentEndian((BYTE *)&(thisHead->mode), 4, endian);
  if(thisHead->mode==20)
    i2Map = TRUE;
  else
    i2Map = FALSE;
  /* 84 */

  fread(&(thisHead->even), sizeof(int), 1, fIn); 
  differentEndian((BYTE *)&(thisHead->even), 4, endian);
  fread(&(thisHead->mrcX), sizeof(int), 1, fIn); 
  differentEndian((BYTE *)&(thisHead->mrcX), 4, endian);
  fread(&(thisHead->mrcY), sizeof(int), 1, fIn); 
  differentEndian((BYTE *)&(thisHead->mrcY), 4, endian);
  fread(&(thisHead->mrcZ), sizeof(int), 1, fIn); 
  differentEndian((BYTE *)&(thisHead->mrcZ), 4, endian);
  /* 100 */

  fread(floatScaleStr, sizeof(char), 16, fIn);
  floatScaleStr[16]= '\0';
  s1 = floatScaleStr;
  while(*s1==' ') s1++; // Get rid of white space
  thisHead->int4scaleF = strtod(s1,0);
  fflush(stdout);
  /* 116 */

  fread(ctfStr, sizeof(char), 13, fIn);
  thisHead->a0 = strtod(ctfStr, 0);
  fread(ctfStr, sizeof(char), 13, fIn);
  thisHead->a1 = strtod(ctfStr, 0);
  fread(ctfStr, sizeof(char), 13, fIn);
  thisHead->a2 = strtod(ctfStr, 0);
  fread(ctfStr, sizeof(char), 13, fIn);
  thisHead->a3 = strtod(ctfStr, 0);
  fread(ctfStr, sizeof(char), 13, fIn);
  thisHead->a4 = strtod(ctfStr, 0);
  fread(ctfStr, sizeof(char), 13, fIn); 
  thisHead->a5 = strtod(ctfStr, 0);
  /* 194 */

  /* We have a historical bug whereby old PIF files had the
     numberOfImages variable stuffed here.  Fixed 3/97 */

  if(thisHead->mode == 6) // Packed boxes
    thisHead->numSecs = 1;
  
  return(TRUE);

} // end function pif_read_gh

int pif_read_gh_(forPifHead_t *thisHead) {
  return(pif_read_gh(thisHead));
}

int pif_read_gh__(forPifHead_t *thisHead) {
  return(pif_read_gh(thisHead));
}


int pif_read_dh(forPifHead_t *thisHead) 
{
  // function name: pif_read_dh
  // Read the PIF file's header for a specific image
  
  int  i, endian, dataSize;
  int  thisNcols, thisNrows, thisNsecs, localInt;
  int aoverbI, map_abangI, delaI, delbI, delcI;
  int dtheI, dphi_90I, tmpI;
  int mrc_nxstart,mrc_nystart,mrc_nzstart,
    mrc_mx,mrc_my,mrc_mz,mrc_xlength,mrc_ylength,
    mrc_zlength,mrc_alpha,mrc_beta,mrc_gamma,
    mrc_mapc,mrc_mapr,mrc_maps,mrc_min,mrc_max,
    mrc_mean,mrc_std,mrc_ispg,mrc_nsymbt,mrc_xorigin,
    mrc_yorigin;
  off_t seekPos, imageBytes;  
  float thisFSF;
  FILE *fIn;
  
  endian = endians[thisHead->luNum];
  fIn = lun[thisHead->luNum];

  // mode=30 use same method as mode=40 w/ double column size
  // mode=10 use same method as mode=20 w/ double column size
  
  // Set the data size based on file mode
  if(thisHead->mode == 46)
    dataSize = 4;
  else if(thisHead->mode == 40) // 4-byte 3D maps
    dataSize = 4;
  else if(thisHead->mode == 30) // 4-byte 3D fft map
    dataSize = 4;
  else if(thisHead->mode == 20) // 2-byte 3D density map
    dataSize = 2;
  else if(thisHead->mode == 10) // 2-byte 3D fft map
    dataSize = 2;
  else if(thisHead->mode == 1) 
    dataSize = 2;
  else if(thisHead->mode == 7)
    dataSize = 2;
  else
    dataSize = 1;
  
  thisFSF = thisHead->recipFloatScaleFactor;
  if(thisHead->mode == 20) // Get header values in greater precision
    thisFSF = 1.0 / thisHead->int4scaleF;
  
  if(thisHead->htype == 1) { // all images same size, we can fseek directly
    
    // We have a historical bug whereby old PIF files had the
    // numberOfImages variable stuffed here.  Fixed 3/97
    if(thisHead->mode == 6) 
      thisHead->numSecs = 1; // No sections for packed boxes
    
    imageBytes = (off_t) thisHead->numCols * thisHead->numRows * 
      thisHead->numSecs * dataSize;
    seekPos = 512 + ((thisHead->imgNum)-1) * (512 + imageBytes);
    
    fseek(fIn, seekPos, 0);
    OKREAD(fread(&thisNcols, sizeof(int), 1, fIn), 1,"PIF_READ_DH");
    fread(&thisNrows, sizeof(int), 1, fIn);
    fread(&thisNsecs, sizeof(int), 1, fIn);
    differentEndian((BYTE *)&thisNcols, 4, endian);
    differentEndian((BYTE *)&thisNrows, 4, endian);
    differentEndian((BYTE *)&thisNsecs, 4, endian);
    
    // We have a historical bug whereby old PIF files had the
    // numberOfImages variable stuffed here.  Fixed 3/97
    if(thisHead->mode == 6) 
      thisNsecs = 1; /* No sections for packed boxes */
    
    // Check that the size is what we expect (since htype==1)
    if(thisHead->mode != 31 &&
       (thisNcols != thisHead->numCols ||
	thisNrows != thisHead->numRows ||
	thisNsecs != thisHead->numSecs)) {
      printf("Error in pif_read_dh for image %d\n", thisHead->imgNum);
      printf("  - htype=1 implies all images should have the same size\n");
      printf("  - expected size is (%d,%d,%d)\n", 
	     thisHead->numCols, thisHead->numRows, thisHead->numSecs);
      printf("  - found size is    (%d,%d,%d) \n", 
	     thisNcols, thisNrows, thisNsecs);
      thisHead->imgOffset = seekPos + 512; // image starts after image header
      return(1);	/* error */
    }
  } else { // htype != 1, we must work through all the images
    seekPos = 512;            // global header
    fseek(fIn,seekPos,0);
    
    for(i=0; i<thisHead->imgNum; i++) { // Seek the one we want
      fread(&thisNcols, sizeof(int), 1, fIn);
      fread(&thisNrows, sizeof(int), 1, fIn); 
      fread(&thisNsecs, sizeof(int), 1, fIn); 
      differentEndian((BYTE *)&thisNcols, 4, endian);
      differentEndian((BYTE *)&thisNrows, 4, endian);
      differentEndian((BYTE *)&thisNsecs, 4, endian);
      
      // We have a historical bug whereby old PIF files had the
      // numberOfImages variable stuffed here.  Fixed 3/97 
      if(thisHead->mode == 6)  // No sections for packed boxes
	thisNsecs = 1;
      
      if((i+1) < thisHead->imgNum) {
	seekPos += 512 + (off_t)(thisNcols*thisNrows*thisNsecs*dataSize); 
	/* Header + image*/
	fseek(fIn,seekPos,0);
      }
    }
    thisHead->numCols = thisNcols;
    thisHead->numRows = thisNrows;
    thisHead->numSecs = thisNsecs;
  }
  
  thisHead->imgOffset = seekPos + 512;  // image starts after image header
  
  if(fIn != NULL) {
    
    // [2] Read header for each image
    fread(&(thisHead->mode), sizeof(int), 1, fIn); 
    differentEndian((BYTE *)&(thisHead->mode), 4, endian);
    fread(&localInt,sizeof(int), 1, fIn);        
    differentEndian((BYTE *)&localInt, 4, endian);
    if(thisHead->pifVersion >= 2.09)
      convertBackFloat(localInt, &(thisHead->img_bkg), thisFSF);
    else
      thisHead->img_bkg = localInt;
    
    fread(&(thisHead->packRadius), sizeof(int), 1, fIn);        
    differentEndian((BYTE *)&(thisHead->packRadius), 4, endian);
    
    fread(&mrc_nxstart,sizeof(int), 1, fIn);         
    differentEndian((BYTE *)&mrc_nxstart, 4, endian);
    fread(&mrc_nystart,sizeof(int), 1, fIn);         
    differentEndian((BYTE *)&mrc_nystart, 4, endian);
    fread(&mrc_nzstart,sizeof(int), 1, fIn);         
    differentEndian((BYTE *)&mrc_nzstart, 4, endian);
    fread(&mrc_mx,sizeof(int), 1, fIn);              
    differentEndian((BYTE *)&mrc_mx, 4, endian);
    fread(&mrc_my,sizeof(int), 1, fIn);              
    differentEndian((BYTE *)&mrc_my, 4, endian);
    fread(&mrc_mz,sizeof(int), 1, fIn);              
    differentEndian((BYTE *)&mrc_mz, 4, endian);
    fread(&mrc_xlength,sizeof(int), 1, fIn);       
    differentEndian((BYTE *)&mrc_xlength, 4, endian);
    fread(&mrc_ylength,sizeof(int), 1, fIn);       
    differentEndian((BYTE *)&mrc_ylength, 4, endian);
    fread(&mrc_zlength,sizeof(int), 1, fIn); 
    differentEndian((BYTE *)&mrc_zlength, 4, endian);
    
    convertBackFloat(mrc_xlength, &(thisHead->a_cell), thisFSF);
    convertBackFloat(mrc_ylength, &(thisHead->b_cell), thisFSF);
    convertBackFloat(mrc_zlength, &(thisHead->c_cell), thisFSF);

    fread(&mrc_alpha, sizeof(int), 1, fIn); 
    differentEndian((BYTE *)&mrc_alpha, 4, endian);
    fread(&mrc_beta, sizeof(int), 1, fIn);          
    differentEndian((BYTE *)&mrc_beta, 4, endian);
    fread(&mrc_gamma, sizeof(int), 1, fIn);         
    differentEndian((BYTE *)&mrc_gamma, 4, endian);
    
    convertBackFloat(mrc_alpha, &(thisHead->alp), thisFSF);
    convertBackFloat(mrc_beta,  &(thisHead->bet), thisFSF);
    convertBackFloat(mrc_gamma, &(thisHead->gam), thisFSF);
    
    fread(&mrc_mapc,sizeof(int), 1, fIn);            
    differentEndian((BYTE *)&mrc_mapc, 4, endian);
    fread(&mrc_mapr,sizeof(int), 1, fIn);            
    differentEndian((BYTE *)&mrc_mapr, 4, endian);
    fread(&mrc_maps,sizeof(int), 1, fIn);            
    differentEndian((BYTE *)&mrc_maps, 4, endian);
    fread(&mrc_min,sizeof(int), 1, fIn);           
    differentEndian((BYTE *)&mrc_min, 4, endian);
    fread(&mrc_max,sizeof(int), 1, fIn);           
    differentEndian((BYTE *)&mrc_max, 4, endian);
    fread(&mrc_mean,sizeof(int), 1, fIn);          
    differentEndian((BYTE *)&mrc_mean, 4, endian);
    fread(&mrc_std,sizeof(int), 1, fIn);       
    differentEndian((BYTE *)&mrc_std, 4, endian);
    
    thisHead->datai4_min = mrc_min;
    thisHead->datai4_max = mrc_max;
    
    convertBackFloat(mrc_mean, &(thisHead->datar4_avg), thisFSF);
    convertBackFloat(mrc_std, &(thisHead->datar4_std), thisFSF);
    
    fread(&mrc_ispg,sizeof(int), 1, fIn);               
    differentEndian((BYTE *)&mrc_ispg, 4, endian);
    fread(&mrc_nsymbt,sizeof(int), 1, fIn);            
    differentEndian((BYTE *)&mrc_nsymbt, 4, endian);
    fread(&mrc_xorigin,sizeof(float), 1, fIn);      
    differentEndian((BYTE *)&mrc_xorigin, 4, endian);
    fread(&mrc_yorigin,sizeof(float), 1, fIn);      
    differentEndian((BYTE *)&mrc_yorigin, 4, endian);
    
    fread(thisHead->descriptionStr, sizeof(char), 80, fIn); 
    if(EQU(thisHead->descriptionStr,""))
      sprintf(thisHead->descriptionStr,
	      "                                                                           ");
    
    fread(thisHead->timeStr, sizeof(char), 32, fIn);
    fread(thisHead->micrographStr, sizeof(char), 16, fIn);
    fread(thisHead->scanNumberStr, sizeof(char), 8, fIn);
    
    fread(&aoverbI, sizeof(int), 1, fIn);
    fread(&map_abangI, sizeof(int), 1, fIn);
    fread(&delaI, sizeof(int), 1, fIn);
    fread(&delbI, sizeof(int), 1, fIn);
    fread(&delcI, sizeof(int), 1, fIn);
    differentEndian((BYTE *)&aoverbI, 4, endian);
    differentEndian((BYTE *)&map_abangI, 4, endian);
    differentEndian((BYTE *)&delaI, 4, endian);
    differentEndian((BYTE *)&delbI, 4, endian);
    differentEndian((BYTE *)&delcI, 4, endian);
    
    fread(thisHead->t_matrix, sizeof(int), 6, fIn);        
    for(i=0; i<6; i++)
      differentEndian((BYTE *)&(thisHead->t_matrix[i]), 4, endian);
    
    convertBackFloat(aoverbI, &(thisHead->aoverb), thisFSF);
    convertBackFloat(map_abangI, &(thisHead->map_abang), thisFSF);
    convertBackFloat(delaI, &(thisHead->dela), thisFSF);
    convertBackFloat(delbI, &(thisHead->delb), thisFSF);
    convertBackFloat(delcI, &(thisHead->delc), thisFSF);
    
    fread(&dtheI, sizeof(int), 1, fIn);
    fread(&dphi_90I, sizeof(int), 1, fIn);
    differentEndian((BYTE *)&dtheI, 4, endian);
    differentEndian((BYTE *)&dphi_90I, 4, endian);
    convertBackFloat(dtheI, &(thisHead->dthe), thisFSF);
    convertBackFloat(dphi_90I, &(thisHead->dphi_90), thisFSF);
    
    fread(&(thisHead->symmetry), sizeof(int), 1, fIn);
    differentEndian((BYTE *)&(thisHead->symmetry), 4, endian);
    fread(&(thisHead->binFactor), sizeof(int), 1, fIn);
    differentEndian((BYTE *)&(thisHead->binFactor), 4, endian);

    fread(&tmpI, sizeof(int), 1, fIn);
    differentEndian((BYTE *)&tmpI, 4, endian);
    convertBackFloat(tmpI, &(thisHead->a_star), thisFSF);
    fread(&tmpI, sizeof(int), 1, fIn);         
    differentEndian((BYTE *)&tmpI, 4, endian);
    convertBackFloat(tmpI, &(thisHead->b_star), thisFSF);
    fread(&tmpI, sizeof(int), 1, fIn);         
    differentEndian((BYTE *)&tmpI, 4, endian);
    convertBackFloat(tmpI, &(thisHead->c_star), thisFSF);
    fread(&tmpI, sizeof(int), 1, fIn);         
    differentEndian((BYTE *)&tmpI, 4, endian);
    convertBackFloat(tmpI, &(thisHead->alp_star), thisFSF);
    fread(&tmpI, sizeof(int), 1, fIn);         
    differentEndian((BYTE *)&tmpI, 4, endian);
    convertBackFloat(tmpI, &(thisHead->bet_star), thisFSF);
    fread(&tmpI, sizeof(int), 1, fIn);         
    differentEndian((BYTE *)&tmpI, 4, endian);
    convertBackFloat(tmpI, &(thisHead->gam_star), thisFSF);
    fread(&tmpI, sizeof(int), 1, fIn);         
    differentEndian((BYTE *)&tmpI, 4, endian);
    convertBackFloat(tmpI, &(thisHead->pixelSize), thisFSF);
    
    fread(&(thisHead->pixSizUnits), sizeof(int), 1, fIn);         
    differentEndian((BYTE *)&(thisHead->pixSizUnits), 4, endian);
    fread(&tmpI, sizeof(int), 1, fIn);         
    differentEndian((BYTE *)&tmpI, 4, endian);
    convertBackFloat(tmpI, &(thisHead->res_out), thisFSF);
    fread(&(thisHead->ctfMode), sizeof(int), 1, fIn);         
    differentEndian((BYTE *)&(thisHead->ctfMode), 4, endian);
    fread(&tmpI, sizeof(int), 1, fIn);         
    differentEndian((BYTE *)&tmpI, 4, endian);
    convertBackFloat(tmpI, &(thisHead->tempFactor), thisFSF);
    fread(&tmpI, sizeof(int), 1, fIn);         
    differentEndian((BYTE *)&tmpI, 4, endian);
    convertBackFloat(tmpI, &(thisHead->wiener), thisFSF);
    fread(&(thisHead->borderType), sizeof(int), 1, fIn);         
    differentEndian((BYTE *)&(thisHead->borderType), 4, endian);
    fread(&(thisHead->borderWidth), sizeof(int), 1, fIn);         
    differentEndian((BYTE *)&(thisHead->borderWidth), 4, endian);
    
    fread(&tmpI, sizeof(int), 1, fIn);         
    differentEndian((BYTE *)&tmpI, 4, endian);
    convertBackFloat(tmpI, &(thisHead->datar4_min), thisFSF);
    fread(&tmpI, sizeof(int), 1, fIn);         
    differentEndian((BYTE *)&tmpI, 4, endian);
    convertBackFloat(tmpI, &(thisHead->datar4_max), thisFSF);
    
    fread(&(thisHead->rad_lo), sizeof(int), 1, fIn);         
    differentEndian((BYTE *)&(thisHead->rad_lo), 4, endian);
    fread(&(thisHead->rad_hi), sizeof(int), 1, fIn);         
    differentEndian((BYTE *)&(thisHead->rad_hi), 4, endian);
    
    fread(&(thisHead->transX), sizeof(int), 1, fIn);         
    differentEndian((BYTE *)&(thisHead->transX), 4, endian);
    fread(&(thisHead->transY), sizeof(int), 1, fIn);         
    differentEndian((BYTE *)&(thisHead->transY), 4, endian);
    fread(&(thisHead->transZ), sizeof(int), 1, fIn);         
    differentEndian((BYTE *)&(thisHead->transZ), 4, endian);
    fread(&(thisHead->isTrunc), sizeof(int), 1, fIn);         
    differentEndian((BYTE *)&(thisHead->isTrunc), 4, endian);
    fread(&(thisHead->origX), sizeof(int), 1, fIn);         
    differentEndian((BYTE *)&(thisHead->origX), 4, endian);
    fread(&(thisHead->origY), sizeof(int), 1, fIn);         
    differentEndian((BYTE *)&(thisHead->origY), 4, endian);
    fread(&(thisHead->origZ), sizeof(int), 1, fIn);         
    differentEndian((BYTE *)&(thisHead->origZ), 4, endian);  
  } else {
    printf("[PIF_READ_DH] Image [%d] unreadable.\n",thisHead->imgNum);
    return(1);
  }
  
  return(0);
  
} // end function pif_read_dh

int pif_read_dh_(forPifHead_t *thisHead) {
  return(pif_read_dh(thisHead));
}

int pif_read_dh__(forPifHead_t *thisHead) {
  return(pif_read_dh(thisHead));
}



void pif_read_byte_image(int *luNum, BYTE *byte_buffer, off_t *imgOffset,
			 int *numCols, int *numRows, int *numSecs) 
{
  // function name: pif_read_byte_image
  // Read the PIF file's BYTE image for a specific image

  FILE *fIn;
  fIn= lun[*luNum];
  
  fseek(fIn,*imgOffset,0);
#define NORMAL_WAY 1
#ifdef NORMAL_WAY
  OKREAD(fread(byte_buffer,sizeof(BYTE), 
	       ((*numCols)*(*numRows)*(*numSecs)),fIn),
	 ((*numCols)*(*numRows)*(*numSecs)),
	 "PIF_READ_BYTE_IMAGE");
#else
  fread(byte_buffer,sizeof(BYTE), 
	((*numCols)*(*numRows)*(*numSecs)),fIn);
#endif
  
} // end function pif_read_byte_image

void pif_read_byte_image_(int *luNum, BYTE *byte_buffer, off_t *imgOffset,
			 int *numCols, int *numRows, int *numSecs) {
  pif_read_byte_image(luNum, byte_buffer, imgOffset,
		       numCols, numRows, numSecs);
}

void pif_read_byte_image__(int *luNum, BYTE *byte_buffer, off_t *imgOffset,
			 int *numCols, int *numRows, int *numSecs) {
  pif_read_byte_image(luNum, byte_buffer, imgOffset,
		       numCols, numRows, numSecs);
}



int pif_read_float_image(int *luNum, float *in_float_buffer, off_t *imgOffset,
			  int *numCols, int *numRows, int *numSecs,
			  int *binSwitch, float *recipScaleF) 
{
  // pif_read_float_image
  // Read the PIF file's float image for a specific image and stuff it
  // in the float array passed in. JFC.3 27-May-97 Added error return
  // (0=ok, else error)

  FILE *fIn;
  int   i, j,k,l,m,n,indx,totBuffer, binStep, binSection;
  int   iM1,i0,iP1,jM1,jP1,oddFactor;
  int   lastSec,lastRow,lastCol;
  int   totPnts,lastBinSec,binCols,binRows;
  float totVals,tmpVals[9],backGround;
  int   *int_buffer[2],*thisIntBuffer, *bBuf;
  
  fIn= lun[*luNum];
  
  totBuffer = (*numCols)*(*numRows);
  
  if(*binSwitch==2) {/* They want to "sample" average */
    binStep = 2;
    binCols = (*numCols/2)+1;
    binRows = (*numRows/2)+1;
    oddFactor = 0;
    if((binCols % 2)==0) {/* Make odd */
      oddFactor = 1;
      binCols--;
    }
    if((binRows % 2)==0) 
      binRows--;
  } else {/* They DON'T want to bin the data */
    binSection = totBuffer;
    binStep = 1;
    lastSec = *numSecs;
    lastRow = *numRows;
    lastCol = *numCols;
    lastBinSec = 1;
  }
  
  int_buffer[0] = (int *)malloc(sizeof(int)*totBuffer);
  if(int_buffer[0] == NULL)  {
    printf("ERROR in pif_read_float_image - can't allocate int_buffer\n");
    return(1);
  }
  int_buffer[1] = (int *)malloc(sizeof(int)*totBuffer);
  if(int_buffer[1] == NULL)  {
    printf("ERROR in pif_read_float_image - can't allocate int_buffer\n");
    return(1);
  }
  
  if(*binSwitch == 2) {/* DO the "sample" average */
    
    /* Seek the image - for 2D images we just do this once. */
    if(fseek(fIn,(*imgOffset),0) != 0) {/* failed to find position */
      perror("ERROR in pif_read_float_image fseek");
      printf("ERROR seeking position %llu\n", (long long unsigned)*imgOffset);
      return(1);
    }
    
    if(fread(int_buffer[0],sizeof(int), totBuffer, fIn) != totBuffer) {
      perror("ERROR in pif_read_float_image fseek");
      printf("ERROR reading float image\n");
      return(1);
    }
    
    /* Init out buffer */
    for(i=0;i<totBuffer;i++) {                            
      differentEndian((BYTE *)&int_buffer[0][i],       
		       4, endians[*luNum]);             
    }                                                     
    bBuf = int_buffer[0];        /* Just use this one */
    backGround = (float)bBuf[0] * (*recipScaleF);
    
    for(i=0;i<(binRows);i++) // Do all rows
      for(j=0;j<(binCols);j++) // DO all cols
	in_float_buffer[(i*binCols)+j] = backGround;
    
    /* Now sample average */
    for(i=oddFactor;i<(*numRows-oddFactor);i+=binStep) {/* Do all rows */
      if(i==0)
	iM1 = i*(*numCols);
      else 
	iM1 = (i-1)*(*numCols);
      
      i0  = i*(*numCols);

      if((i+1)==*numRows)
	iP1 = i*(*numCols);
      else
	iP1 = (i+1)*(*numCols);

      for(j=oddFactor; j<(*numCols-oddFactor); j+=binStep) {/* DO all cols */
	if(j==0)
	  jM1 = j;
	else
	  jM1 = j - 1;

	jP1 = j + 1;
	tmpVals[0] = bBuf[iM1+jM1]*(*recipScaleF);
	tmpVals[1] = bBuf[iM1+j]*(*recipScaleF);
	tmpVals[2] = bBuf[iM1+jP1]*(*recipScaleF);
	tmpVals[3] = bBuf[i0+jM1]*(*recipScaleF);
	tmpVals[4] = bBuf[i0+j]*(*recipScaleF);
	tmpVals[5] = bBuf[i0+jP1]*(*recipScaleF);
	tmpVals[6] = bBuf[iP1+jM1]*(*recipScaleF);
	tmpVals[7] = bBuf[iP1+j]*(*recipScaleF);
	tmpVals[8] = bBuf[iP1+jP1]*(*recipScaleF);
	if((i)== 0) {/* Past rows */
	  tmpVals[0] = backGround;
	  tmpVals[1] = backGround;
	  tmpVals[2] = backGround;
	}
	if(i >= (*numRows-1)) {/* Past rows */
	  tmpVals[6] = backGround;
	  tmpVals[7] = backGround;
	  tmpVals[8] = backGround;
	}
	if(jM1 < 0) {/* Past cols */
	  tmpVals[0] = backGround;
	  tmpVals[3] = backGround;
	  tmpVals[6] = backGround;
	}
	if(jP1 >= *numCols) {/* Past cols */
	  tmpVals[2] = backGround;
	  tmpVals[5] = backGround;
	  tmpVals[8] = backGround;
	}
		  
	totVals = 0.0;
	for(indx=0; indx<9; indx++)
	  totVals += tmpVals[indx] * binVals2D[indx];
	
	in_float_buffer[((i/binStep)*(binCols))+(j/binStep)] = totVals;
	
	
      } /* DO all cols */
    } /* Do all rows */
    
  } else { /* Normal */
    for(i=0;i<(lastSec);i+= binStep) {/* Read all secs */
      
      /* Seek the image - for 2D images we just do this once and
       * i==0 there. */
      if(fseek(fIn,(*imgOffset+(i*totBuffer)),0) != 0) {
	/* failed to find position */
	perror("ERROR in pif_read_float_image fseek");
	printf("ERROR seeking position %ld\n", *imgOffset);
	return(1);	/* error */
      }
      
      if(fread(int_buffer[0],sizeof(int), totBuffer, fIn) 
	 != totBuffer) {
	perror("ERROR in pif_read_float_image fseek");
	printf("ERROR reading section %d\n", i+1);
	return(1);	/* error */
      }
      
      for(k=0; k<totBuffer; k++)
	differentEndian((BYTE *)&int_buffer[0][k], 4, endians[*luNum]);
      
      if(lastBinSec > 1) {/* Bin it */
	if(fread(int_buffer[1],sizeof(int), 
		 totBuffer, fIn) != totBuffer) {
	  perror("ERROR in pif_read_float_image fseek");
	  printf("ERROR reading section %d\n", i+1);
	  return(1);	/* error */
	}
	
	for(k=0;k<totBuffer;k++)
	  differentEndian((BYTE *)&int_buffer[1][k], 4, endians[*luNum]);
      }
      
      for(j=0; j<lastRow; j+= binStep) { // Rows
	for(k=0; k<lastCol; k+= binStep) { // Do all
	  totVals = 0.0;
	  totPnts = 0;
	  for(n=0;n<lastBinSec;n++) { // secs
	    thisIntBuffer = int_buffer[n];
	    for(l=0;l<binStep;l++) {
	      for(m=0;m<binStep;m++) { // Do local neighbors
		totVals += thisIntBuffer[(j+l)*(*numCols) + k+m]*(*recipScaleF);
		totPnts ++;
	      }/* Do local neighbors */
	    }
	  }
	  in_float_buffer[(((i/binStep)*binSection)+
			   ((j/binStep)*(*numCols/binStep))+
			   (k/binStep))]= totVals/totPnts;
	}
      }
      
    }/* Read all secs */
  }/* Normal */
  
  free(int_buffer[0]);
  free(int_buffer[1]);
  
  return(0);	/*  no error */
  
} // end function pif_read_float_image

int pif_read_float_image_(int *luNum, float *float_buffer, off_t *imgOffset,
			 int *numCols, int *numRows, int *numSecs,
			 int *binSwitch, float *recipScaleF) {
  return(pif_read_float_image(luNum, float_buffer, imgOffset,
			       numCols, numRows, numSecs,
			       binSwitch, recipScaleF));
}

int pif_read_float_image__(int *luNum, float *float_buffer, off_t *imgOffset,
			 int *numCols, int *numRows, int *numSecs,
			 int *binSwitch, float *recipScaleF) {
  return(pif_read_float_image(luNum, float_buffer, imgOffset,
			       numCols, numRows, numSecs,
			       binSwitch, recipScaleF));
}



void pif_read_short_image(int *luNum, short *short_buffer,
			  off_t *imgOffset, int *numCols, int *numRows, 
			  int *numSecs) 
{
  // function name: pif_read_short_image
  // Read the PIF file's SHORT image for a specific image

  int i, ntot;
  FILE *fIn;
  fIn = lun[*luNum];

  ntot = (*numCols) * (*numRows) * (*numSecs);

  fseek(fIn,*imgOffset,0);
  OKREAD(fread(short_buffer, sizeof(short), ntot, fIn), 
	 ntot, "PIF_READ_SHORT_IMAGE");
  
  for(i=0; i<ntot; i++)
    differentEndian((BYTE *)&short_buffer[i], 2, endians[*luNum]);
  
} // end function pif_read_short_image

void pif_read_short_image_(int *luNum, short *short_buffer,
			   off_t *imgOffset,
			   int *numCols, int *numRows, int *numSecs) {
  pif_read_short_image(luNum, short_buffer, imgOffset,
		       numCols, numRows, numSecs);
}

void pif_read_short_image__(int *luNum, short *short_buffer,
			    off_t *imgOffset,
			    int *numCols, int *numRows, int *numSecs) {
  pif_read_short_image(luNum, short_buffer, imgOffset,
		       numCols, numRows, numSecs);
}



void pif_read_wmapi2(forPifHead_t *thisHead, float *float_buffer,
		        int *ncol, int *nrow, int *nsec) 
{
  // function name: pif_read_wmapi2
  // Read entire Whole mapi2 into buffer

  int startSec=1, endSec;
  endSec = thisHead->numSecs;
  
  pif_read_mapi2(thisHead, float_buffer, &startSec, &endSec,
		 ncol, nrow, nsec);

} // end function pif_read_wmapi2

void pif_read_wmapi2_(forPifHead_t *thisHead, float *float_buffer,
		     int *ncol, int *nrow, int *nsec) {
  pif_read_wmapi2(thisHead, float_buffer, ncol, nrow, nsec);
}

void pif_read_wmapi2__(forPifHead_t *thisHead, float *float_buffer,
		     int *ncol, int *nrow, int *nsec) {
  pif_read_wmapi2(thisHead, float_buffer, ncol, nrow, nsec);
}



void pif_read_mapi2(forPifHead_t *thisHead, float *float_buffer,
		     int *sec1, int *sec2, int *ncol, int *nrow, int *nsec) 
{
  // function name: pif_read_mapi2
  // Read the PIF file's SHORT image for a floating point MAP

  int tmpSec1, tmpSec2, numSecs;
  int binCol,binRow,binSec,thisBin;
  int strtCol,strtRow,strtSec;
  int endCol,endRow,endSec;

  if(thisHead->recipFloatScaleFactor == 0.0)
    thisHead->recipFloatScaleFactor =  1.0 /  thisHead->floatScaleFactor ;
  if(thisHead->floatScaleFactor == 0.0)
    thisHead->floatScaleFactor =  1.0 /  thisHead->recipFloatScaleFactor ;

  tmpSec1 = MIN(*sec1, *sec2);
  tmpSec2 = MAX(*sec1, *sec2);

  if(tmpSec1 > thisHead->numSecs ||
     tmpSec2 > thisHead->numSecs ||
     tmpSec1 < 1 || tmpSec2 < 1 ) { /* Fatal error */
    printf("[PIF_READ_MAPI2() Invalid section range [%d][%d]\n",
	   tmpSec1,tmpSec2);
    exit(0);
  }

  numSecs = tmpSec2 - tmpSec1;
  
  numSecs++;  /* Always get the last one */

  binCol = thisHead->numCols;
  binRow = thisHead->numRows;
  binSec = thisHead->numSecs;

  if(*ncol == binCol || *nrow == binRow || *nsec == binSec)
    thisBin = 1;
  else
    thisBin = 2;
  
  /* Times 2 because we're short here */
  thisHead->imgOffset = 1024 + ((tmpSec1-1) * (off_t) (binCol * binRow * 2));
  
  strtCol = strtRow = strtSec = 1;
  endCol = binCol;
  endRow = binRow;
  endSec = numSecs;
  pif_read_map_short_image(&(thisHead->luNum),
			    float_buffer,
			    &(thisHead->imgOffset),
			    &binCol, &binRow, &numSecs,
			    &(thisHead->recipFloatScaleFactor),
			    &(thisHead->datar4_min),
			    &(thisHead->datar4_max),
			    &thisBin,
			    &strtCol, &strtRow, &strtSec,
			    &endCol,&endRow,&endSec);

} // end function pif_read_mapi2

void pif_read_mapi2_(forPifHead_t *thisHead, float *float_buffer,
		    int *sec1, int *sec2,
		    int *ncol, int *nrow, int *nsec) {
  pif_read_mapi2(thisHead, float_buffer, sec1, sec2, ncol, nrow, nsec);
}

void pif_read_mapi2__(forPifHead_t *thisHead, float *float_buffer,
		    int *sec1, int *sec2,
		    int *ncol, int *nrow, int *nsec) {
  pif_read_mapi2(thisHead, float_buffer, sec1, sec2, ncol, nrow, nsec);
}



void pif_read_mapi4(forPifHead_t *thisHead, float *float_buffer,
		     int *sec1, int *sec2, int *ncol, int *nrow, int *nsec)
{
  // function name: pif_read_mapi4
  // Read the PIF file's INT image for a floating point MAP

  int tmpSec1, tmpSec2, numSecs, binCol, binRow, binSec, thisBin;
  
  if(thisHead->recipFloatScaleFactor == 0.0)
    thisHead->recipFloatScaleFactor =  1.0 /  thisHead->floatScaleFactor ;
  if(thisHead->floatScaleFactor == 0.0)
    thisHead->floatScaleFactor =  1.0 /  thisHead->recipFloatScaleFactor ;
  
  tmpSec1 = MIN(*sec1, *sec2);
  tmpSec2 = MAX(*sec1, *sec2);
  
  if(tmpSec1 > thisHead->numSecs ||
     tmpSec2 > thisHead->numSecs ||
     tmpSec1 < 1 || tmpSec2 < 1 ) {
    printf("[PIF_READ_MAPI4() Invalid section range [%d][%d]\n",
	   tmpSec1,tmpSec2);
    exit(0);
  }
  
  numSecs = tmpSec2 - tmpSec1;
  
  binCol = thisHead->numCols;
  binRow = thisHead->numRows;
  binSec = thisHead->numSecs;
  
  if(*ncol == binCol && *nrow == binRow)
    thisBin = 1;
  else
    thisBin = 2;
  
  if((thisBin != thisHead->binFactor) &&
     (thisHead->mode==21 || thisHead->mode==22)) {/* Error */
    printf ("PIF_READ_MAPI4()  - PRJS/PFTS data\n");
    printf ("Incompatible bin factors BIN_STORED[%d] BIN_EXEC[%d]\n",
	    thisHead->binFactor,thisBin);
    exit(0);
  }
  
  /* Times 4 because we're INT here */
  thisHead->imgOffset = 1024 + ((tmpSec1-1) * (off_t) (binCol * binRow * 4));
  
  numSecs++;  /* Always get the last one */
  
  pif_read_map_int_image(&(thisHead->luNum),
			  float_buffer,
			  &(thisHead->imgOffset),
			  &binCol,
			  &binRow,
			  &numSecs,
			  &(thisHead->recipFloatScaleFactor),
			  &(thisHead->datar4_min),
			  &(thisHead->datar4_max),
			  &thisBin,
			  &(thisHead->mode)
			  );
  
} // end function pif_read_mapi4

void pif_read_mapi4_(forPifHead_t *thisHead, float *float_buffer,
                     int *sec1, int *sec2,
                     int *ncol, int *nrow, int *nsec) {
  pif_read_mapi4(thisHead, float_buffer, sec1, sec2,
		  ncol, nrow,  nsec);
}

void pif_read_mapi4__(forPifHead_t *thisHead, float *float_buffer,
                     int *sec1, int *sec2,
                     int *ncol, int *nrow, int *nsec) {
  pif_read_mapi4(thisHead, float_buffer, sec1, sec2,
		  ncol, nrow,  nsec);
}



void pif_read_map_short_image(int *luNum, float *float_buffer, off_t *imgOffset,
			      int *numCols, int *numRows, int *numSecs,
			      float *recipFloatScaleFactor,
			      float *minMapPixel, float *maxMapPixel,
			      int *binSwitch,
			      int *strtCol, int *strtRow, int *strtSec,
			      int *ncol, int *nrow, int *nsec) 
{
  // function name: pif_read_map_short_image
  // Read the PIF file's SHORT image for a floating point MAP

  int i,j,k,l,m,n,indx,rowCol,rowColBin;
  short *short_buffer[4];         /* Hard code this for only 2x2 binning now */
  short *sec1,*sec2,*sec3;
  short *thisShortBuffer;
  float thisFloat,totFloats;
  FILE *fIn;
  int totPnts, thisOffset,binCol,binRow,binSec,oddFactor;
  int binStep,secOut,rowOut,colOut,lastSec,lastRow,lastCol;
  int jM1,j0,jP1,kM1,kP1;
  float tmpVals[27];
  
  if(*binSwitch > 2) {/* Error */
    printf("PIF_READ_MAP_SHORT_IMAGE - Cannot bin data like this[%d]\n",
	   *binSwitch);
    exit(0);
  } else if(*binSwitch==2) { // Bin the data
    binStep = 2;
    lastSec = *numSecs - 2;
    lastRow = *numRows - 2;
    lastCol = *numCols - 2;
    binSec  = *numSecs/2+1;
    binRow  = *numRows/2+1;
    binCol  = *numCols/2+1;
    oddFactor = 0;
    if(binSec%2 == 0) binSec--;
    if(binRow%2 == 0) binRow--;
    if(binCol%2 == 0) { /* Make Odd */
      oddFactor = 1;
      binCol--;
    }
    rowColBin = (binCol) * (binRow);
  } else { // Don't bin the data 
    binStep = 1;
    lastSec = *numSecs;
    lastRow = *numRows;
    lastCol = *numCols;
    rowColBin = (*numCols/binStep) * (*numRows/binStep);
    
  }
  
  rowCol    = (*numCols) * (*numRows);
  sec1 = short_buffer[0] = (short *)malloc(sizeof(short)*rowCol);
  sec2 = short_buffer[1] = (short *)malloc(sizeof(short)*rowCol);
  sec3 = short_buffer[2] = (short *)malloc(sizeof(short)*rowCol);
  
  fIn = lun[*luNum];
  
  fseek(fIn,*imgOffset,0);
  
  if(*binSwitch == 2) {/*  Sample Average binning */
    
    if(*strtCol != 1 || *strtRow != 1 || *strtSec != 1 ||
       *ncol    != *numCols ||
       *nrow    != *numRows ||
       *nsec    != *numSecs ) {/* Can't do */
      printf("Don't know how to truncate the MAP with binned data.\n");
      printf("[P_R_M_S_I] [%d][%d][%d]-[%d][%d][%d]\n",
	     *strtCol,*strtRow,*strtSec,
	     *ncol,*nrow,*nsec);
      exit(0);
    }/* Can't do */

    for(i=oddFactor;i<(*numSecs-oddFactor-1);i+=binStep) {
      /* Do these sections */
      if(i==oddFactor) {/* Read all three */
	if(oddFactor==0) {/* Don't read first section */
	} else {/* Read it */
	  OKREAD(fread(short_buffer[0],sizeof(short), 
		       (rowCol),fIn),rowCol,
		 "PIF_READ_MAP_SHORT_IMAGE");
	  for(m=0;m<rowCol;m++)
	    differentEndian((BYTE *)&sec1[m],2, endians[*luNum]);
	}/* Read it */
	OKREAD(fread(short_buffer[1],sizeof(short),(rowCol),fIn),
	       rowCol, "PIF_READ_MAP_SHORT_IMAGE");
	for(m=0;m<rowCol;m++)
	  differentEndian((BYTE *)&sec2[m],2, endians[*luNum]);
	OKREAD(fread(short_buffer[2],sizeof(short),(rowCol),fIn),
	       rowCol, "PIF_READ_MAP_SHORT_IMAGE");
	for(m=0;m<rowCol;m++)
	  differentEndian((BYTE *)&sec3[m],2, endians[*luNum]);
      } else {/* Just read two */
	      /* Stuff from last time */
	for(m=0;m<rowCol;m++) sec1[m]= sec3[m];
	OKREAD(fread(short_buffer[1],sizeof(short),(rowCol),fIn),
	       rowCol, "PIF_READ_MAP_SHORT_IMAGE");
	for(m=0;m<rowCol;m++)
	  differentEndian((BYTE *)&sec2[m],2, endians[*luNum]);
	OKREAD(fread(short_buffer[2],sizeof(short),(rowCol),fIn),
	       rowCol, "PIF_READ_MAP_SHORT_IMAGE");
	for(m=0;m<rowCol;m++)
	  differentEndian((BYTE *)&sec3[m],2, endians[*luNum]);
      }/* Just read two */
      
      for(j=oddFactor;j<(*numRows-oddFactor-binStep);j+=binStep) {/* rows */
	jM1 = (j-1)*(*numCols);
	if(jM1 < 0) jM1 = 0;
	j0  = (j)*(*numCols);
	jP1 = (j+1)*(*numCols);
	for(k=oddFactor;k<(*numCols-oddFactor-binStep);k+=binStep) {/* cols */
	  kM1 = k - 1;
	  if(kM1 < 0) kM1 = 0;
	  kP1 = k + 1;
	  
	  tmpVals[0]  = sec1[jM1+kM1];
	  tmpVals[1]  = sec1[jM1+k];
	  tmpVals[2]  = sec1[jM1+kP1];
	  tmpVals[3]  = sec1[j0+kM1];
	  tmpVals[4]  = sec1[j0+k];
	  tmpVals[5]  = sec1[j0+kP1];
	  tmpVals[6]  = sec1[jP1+kM1];
	  tmpVals[7]  = sec1[jP1+k];
	  tmpVals[8]  = sec1[jP1+kP1];
	  tmpVals[9]  = sec2[jM1+kM1];
	  tmpVals[10] = sec2[jM1+k];
	  tmpVals[11] = sec2[jM1+kP1];
	  tmpVals[12] = sec2[j0+kM1];
	  tmpVals[13] = sec2[j0+k];
	  tmpVals[14] = sec2[j0+kP1];
	  tmpVals[15] = sec2[jP1+kM1];
	  tmpVals[16] = sec2[jP1+k];
	  tmpVals[17] = sec2[jP1+kP1];
	  tmpVals[18] = sec3[jM1+kM1];
	  tmpVals[19] = sec3[jM1+k];
	  tmpVals[20] = sec3[jM1+kP1];
	  tmpVals[21] = sec3[j0+kM1];
	  tmpVals[22] = sec3[j0+k];
	  tmpVals[23] = sec3[j0+kP1];
	  tmpVals[24] = sec3[jP1+kM1];
	  tmpVals[25] = sec3[jP1+k];
	  tmpVals[26] = sec3[jP1+kP1];
	  
	  if(j==0) {/* Off the bottom */
	    tmpVals[0]  = 0.0;
	    tmpVals[1]  = 0.0;
	    tmpVals[2]  = 0.0;
	    tmpVals[9]  = 0.0;
	    tmpVals[10] = 0.0;
	    tmpVals[11] = 0.0;
	    tmpVals[18] = 0.0;
	    tmpVals[19] = 0.0;
	    tmpVals[20] = 0.0;
	  }
	  if(k==0) {/* Off the bottom */
	    tmpVals[0]  = 0.0; 
	    tmpVals[3]  = 0.0;
	    tmpVals[6]  = 0.0;
	    tmpVals[9]  = 0.0;
	    tmpVals[12] = 0.0;
	    tmpVals[15] = 0.0;
	    tmpVals[18] = 0.0;
	    tmpVals[21] = 0.0;
	    tmpVals[24] = 0.0;
	  }
	  if(j==(*numRows-1)) {/* Off the Top */
	    tmpVals[6]  = 0.0;
	    tmpVals[7]  = 0.0;
	    tmpVals[8]  = 0.0;
	    tmpVals[15] = 0.0;
	    tmpVals[16] = 0.0;
	    tmpVals[17] = 0.0;
	    tmpVals[24] = 0.0;
	    tmpVals[25] = 0.0;
	    tmpVals[26] = 0.0;
	  }
	  if(k==(*numCols-1)) {/* Off the Top */
	    tmpVals[2]  = 0.0;
	    tmpVals[5]  = 0.0;
	    tmpVals[8]  = 0.0;
	    tmpVals[11] = 0.0;
	    tmpVals[14] = 0.0;
	    tmpVals[17] = 0.0;
	    tmpVals[20] = 0.0;
	    tmpVals[23] = 0.0;
	    tmpVals[26] = 0.0;
	  }
	  
	  totFloats = 0.0;
	  for(indx=0; indx<27; indx++)
	    totFloats += tmpVals[indx] * binVals3D[indx];
	  
	  /* Do we want to convert from TSB format for maps to robem format?? */
	  if(convTsb)
	    rowOut = ((lastRow-j-1)/binStep)*binCol;
	  else
	    rowOut = (j/binStep)*binCol;
	  
	  colOut = k/binStep;
	  
	  thisFloat= 
	    float_buffer[((i/binStep)*rowColBin)+
			 rowOut+colOut]= (float)
	    (((float)(totFloats)) *
	     (*recipFloatScaleFactor));
	  
	  *minMapPixel = MIN(*minMapPixel,thisFloat);
	  *maxMapPixel = MAX(*maxMapPixel,thisFloat);
	}/* Do columns */
	
      }/* Do rows */
      
    }/* Do these sections */
    
  } else {/* Normal */
    int maxColOut,maxRowOut,maxSecOut;
    
    rowColBin = (*ncol) * (*nrow);
    maxColOut = (*ncol);
    maxRowOut = ((*nrow - 1) / binStep)*(*ncol/binStep);
    maxSecOut = (int)((*nsec-1)/binStep)*rowColBin;
    
    /* Only do what we need for the truncate buffer case */
    lastSec = *nsec + (*strtSec-1);
    lastRow = *nrow + (*strtRow-1);
    lastCol = *ncol + (*strtCol-1);
    
    /* Don't read past what's on the disk */
    lastSec = MIN(lastSec, *numSecs);
    lastRow = MIN(lastRow, *numRows);
    lastCol = MIN(lastCol, *numCols);
    
    for(i=0; i<lastSec; i+=binStep) {/* Do all sections */

      for(l=0;l<binStep;l++) // Read all sections
	OKREAD(fread(short_buffer[l],sizeof(short), (rowCol),fIn),rowCol,
	       "PIF_READ_MAP_SHORT_IMAGE");
      
      /* Account for where they want to start on the disk */
      secOut = ((i-(*strtSec-1))/binStep)*rowColBin;
      
      secOut = MIN(MAX(0,secOut),maxSecOut); // Make sure we don't go negative
      
      for(j=0; j<lastRow; j+=binStep) {/* Rows */
	/* Do we want to convert from TSB format for maps to robem format?? */
	/* Account for where they want to start on the disk */

	if(convTsb) // Flip Y
	  rowOut = (((lastRow+(*strtRow-1))-j-1)/binStep)*(*ncol/binStep);
	else
	  rowOut = ((j-(*strtRow-1)) / binStep)*(*ncol/binStep);
	
	/* Make sure we don't overflow */
	rowOut = MIN(MAX(0,rowOut),maxRowOut);
	
	for(k=0; k<lastCol; k+=binStep) {/* Stuff the float buffer */
	  
	  colOut = k/binStep;
	  /* Account for where they want to start on the disk */
	  colOut -= (*strtCol-1);
	  /* Make sure we don't go negative */
	  colOut = MIN(MAX(0,colOut),maxColOut);
	  
	  if(binStep == 1) {/* Just do every pixel */
	    thisShortBuffer = short_buffer[0];
	    thisOffset = j*(*numCols) + k;
	    differentEndian((BYTE *)&thisShortBuffer[thisOffset], 
			     2, endians[*luNum]);
	    thisFloat =
	      float_buffer[(secOut+rowOut+colOut)]= 
	      ((float)(thisShortBuffer[thisOffset])) *
	       (*recipFloatScaleFactor);
	    
	  } else {/* Average pixels to "bin" them */
	    totFloats = 0.0;
	    totPnts = 0;
	    
	    for(l=0; l<binStep; l++) { // Do both sections
	      for(m=0; m<binStep; m++) { // Do both rows
		for(n=0; n<binStep; n++) { // Do both cols
		  thisOffset = (j+m)*(*numCols) + (k+n);
		  thisShortBuffer = short_buffer[l];
		  
		  differentEndian((BYTE *)&thisShortBuffer[thisOffset], 
				  2, endians[*luNum]);
		  totFloats +=
		    ((float)(thisShortBuffer[thisOffset])) *
		    (*recipFloatScaleFactor);
		  totPnts++;
		}
	      }
	    }
	    
	    thisFloat = 
	      float_buffer[((i/binStep)*rowColBin)+rowOut+colOut] = 
	      (totFloats/(float)totPnts);
	  }/* Average pixels to "bin" them */
	  
	  *minMapPixel = MIN(*minMapPixel,thisFloat);
	  *maxMapPixel = MAX(*maxMapPixel,thisFloat);
	}/* Stuff the float buffer */
	
      }/* Rows */
    }/* Do all sections */
    
  }/* Normal */
  
  free(short_buffer[0]);
  free(short_buffer[1]);
  free(short_buffer[2]);
  
} // end function pif_read_map_short_image

void pif_read_map_short_image_(int *luNum,
			      float *float_buffer,
			      off_t *imgOffset,
			      int *numCols, int *numRows, int *numSecs,
			      float *recipFloatScaleFactor,
			      float *minMapPixel, float *maxMapPixel,
			      int *binSwitch,
			      int *strtCol, int *strtRow, int *strtSec,
			      int *ncol, int *nrow, int *nsec ) {
  pif_read_map_short_image(luNum,
			    float_buffer,
			    imgOffset,
			    numCols, numRows, numSecs,
			    recipFloatScaleFactor,
			    minMapPixel, maxMapPixel,
			    binSwitch,strtCol,strtRow,strtSec,
			    ncol,nrow,nsec);
}

void pif_read_map_short_image__(int *luNum,
			      float *float_buffer,
			      off_t *imgOffset,
			      int *numCols, int *numRows, int *numSecs,
			      float *recipFloatScaleFactor,
			      float *minMapPixel, float *maxMapPixel,
			      int *binSwitch,
			      int *strtCol, int *strtRow, int *strtSec,
			      int *ncol, int *nrow, int *nsec ) {
  pif_read_map_short_image(luNum,
			    float_buffer,
			    imgOffset,
			    numCols, numRows, numSecs,
			    recipFloatScaleFactor,
			    minMapPixel, maxMapPixel,
			    binSwitch,strtCol,strtRow,strtSec,
			    ncol,nrow,nsec);
}



void pif_read_imgi4(forPifHead_t *thisHead, float *float_buffer,
		     int *ncol, int *nrow, int *thisImg) 
{
  // function name: pif_read_imgi4
  // Read the PIF file's SHORT image for a floating point MAP

  int tmpThisImg, numSecs, binCol, binRow, thisBin;
  
  if(thisHead->recipFloatScaleFactor == 0.0)
    thisHead->recipFloatScaleFactor =  1.0 /  thisHead->floatScaleFactor ;
  if(thisHead->floatScaleFactor == 0.0)
    thisHead->floatScaleFactor =  1.0 /  thisHead->recipFloatScaleFactor ;
  
  tmpThisImg = *thisImg;
  
  if(tmpThisImg > thisHead->nimg || tmpThisImg < 1 ) {
    printf("[PIF_READ_IMGI4() Invalid image [%d]\n", tmpThisImg);
    exit(0);
  }
  
  numSecs = 1;
  
  binCol = thisHead->numCols;
  binRow = thisHead->numRows;
  
  if(*ncol == thisHead->numCols && *nrow == thisHead->numRows )
    thisBin = 1;
  else
    thisBin = 2;
  
  // Multiply by 4 because we're INT here
  thisHead->imgOffset = 1024 + (512 * (tmpThisImg-1)) +
    (tmpThisImg - 1) * (off_t) (binCol * binRow * 4);
  
  pif_read_float_image(&(thisHead->luNum), float_buffer,
		       &(thisHead->imgOffset),
		       &(thisHead->numCols),
		       &(thisHead->numRows),
		       &numSecs, &thisBin,
		       &(thisHead->recipFloatScaleFactor));
  
  
} // end function pif_read_imgi4

void pif_read_imgi4_(forPifHead_t *thisHead, float *float_buffer,
		    int *ncol, int *nrow, int *thisImg) {
  pif_read_imgi4(thisHead, float_buffer, ncol, nrow, thisImg);
}

void pif_read_imgi4__(forPifHead_t *thisHead, float *float_buffer,
		    int *ncol, int *nrow, int *thisImg) {
  pif_read_imgi4(thisHead, float_buffer, ncol, nrow, thisImg);
}



void pif_read_map_int_image(int *luNum, float *float_buffer, off_t *imgOffset,
			    int *numCols, int *numRows, int *numSecs,
			    float *recipFloatScaleFactor,
			    float *minMapPixel, float *maxMapPixel,
			    int *binSwitch, int *mode) 
{
  // function name: pif_read_map_int_image
  // Read the PIF file's INT image for a floating point MAP

  int i,j,k,l,m,n,indx,rowCol,rowColBin;
  int *int_buffer[4];         /* Hard code this for only 2x2 binning now */
  int *sec1,*sec2,*sec3;
  int *thisIntBuffer;
  float thisFloat,totFloats;
  FILE *fIn;
  int totPnts, thisOffset,binCol,binRow,binSec,oddFactor;
  int binStep,rowOut,colOut,lastSec,lastRow,lastCol;
  int jM1,j0,jP1,kM1,kP1;
  float tmpVals[27];
  
  if(*binSwitch > 2) {
    printf("PIF_READ_MAP_INT_IMAGE - Cannot bin data like this[%d]\n",
	   *binSwitch);
    exit(0);
  } else if(*binSwitch==2) {/* They want to sample average bin the data */
    binStep = 2;
    lastSec = *numSecs - 2;
    lastRow = *numRows - 2;
    lastCol = *numCols - 2;
    if(*mode==20)
      binSec  = *numSecs/2+1;
    else
      binSec = *numSecs;

    binRow  = *numRows/2+1;
    binCol  = *numCols/2+1;
    oddFactor = 0;
    if(binSec%2 == 0) binSec--;
    if(binRow%2 == 0) binRow--;
    if(binCol%2 == 0) {/* Make Odd */
      oddFactor = 1;
      binCol--;
    }
    rowColBin = binCol * binRow;
  } else {/* They DON'T want to bin the data */
    binStep = 1;
    lastSec = *numSecs;
    binRow = lastRow = *numRows;
    binCol = lastCol = *numCols;
    rowColBin = (*numCols/binStep) * (*numRows/binStep);
  }
  
  rowCol    = (*numCols) * (*numRows);
  sec1 = int_buffer[0] = (int *)malloc(sizeof(int)*rowCol);
  sec2 = int_buffer[1] = (int *)malloc(sizeof(int)*rowCol);
  sec3 = int_buffer[2] = (int *)malloc(sizeof(int)*rowCol);
  
  fIn = lun[*luNum];
  
  fseek(fIn,*imgOffset,0);
  
  if(*binSwitch == 2) {/*  Sample Average binning */
    int lastSection;
    lastSection = (*numSecs-oddFactor-1);
    if(lastSection <= oddFactor)
      lastSection = oddFactor + 1;  /* Read at least one of these */
    for(i=oddFactor;i<(lastSection);i+=binStep) {/* Do these sections */
      if(i==oddFactor) {/* Read all three */
	if(oddFactor==0) {/* Don't read first section */
	} else {/* Read it */
	  OKREAD(fread(int_buffer[0],sizeof(int), 
		       (rowCol),fIn), rowCol,
		 "PIF_READ_MAP_INT_IMAGE");
	  for(m=0;m<rowCol;m++)
	    differentEndian((BYTE *)&sec1[m],4, endians[*luNum]);
	}/* Read it */
	OKREAD(fread(int_buffer[1],sizeof(int),(rowCol),fIn),
	       rowCol,
	       "PIF_READ_MAP_INT_IMAGE");
	for(m=0;m<rowCol;m++)
	  differentEndian((BYTE *)&sec2[m],4, endians[*luNum]);
	OKREAD(fread(int_buffer[2],sizeof(int),(rowCol),fIn),
	       rowCol,
	       "PIF_READ_MAP_INT_IMAGE");
	for(m=0;m<rowCol;m++)
	  differentEndian((BYTE *)&sec3[m],4, endians[*luNum]);
      } else {/* Just read two */
	/* Stuff from last time */
	for(m=0;m<rowCol;m++) sec1[m]= sec3[m];
	OKREAD(fread(int_buffer[1],sizeof(int),(rowCol),fIn),
	       rowCol, "PIF_READ_MAP_INT_IMAGE");
	for(m=0;m<rowCol;m++)
	  differentEndian((BYTE *)&sec2[m],4, endians[*luNum]);
	OKREAD(fread(int_buffer[2],sizeof(int),(rowCol),fIn),
	       rowCol,
	       "PIF_READ_MAP_INT_IMAGE");
	for(m=0;m<rowCol;m++)
	  differentEndian((BYTE *)&sec3[m],4, endians[*luNum]);
      }/* Just read two */
      
      for(j=oddFactor; j<(*numRows-oddFactor); j+=binStep) {/* Do rows */
	jM1 = (j-1)*(*numCols);
	if(jM1 < 0) jM1 = 0;
	j0  = (j)*(*numCols);
	jP1 = (j+1)*(*numCols);
	for(k=oddFactor; k<(*numCols-oddFactor); k+=binStep) {/* Do columns */
	  kM1 = k - 1;
	  kM1 = MAX(kM1, 0);
	  kP1 = k + 1;
	  
	  tmpVals[0]  = sec1[jM1+kM1];
	  tmpVals[1]  = sec1[jM1+k];
	  tmpVals[2]  = sec1[jM1+kP1];
	  tmpVals[3]  = sec1[j0+kM1];
	  tmpVals[4]  = sec1[j0+k];
	  tmpVals[5]  = sec1[j0+kP1];
	  tmpVals[6]  = sec1[jP1+kM1];
	  tmpVals[7]  = sec1[jP1+k];
	  tmpVals[8]  = sec1[jP1+kP1];
	  tmpVals[9]  = sec2[jM1+kM1];
	  tmpVals[10] = sec2[jM1+k];
	  tmpVals[11] = sec2[jM1+kP1];
	  tmpVals[12] = sec2[j0+kM1];
	  tmpVals[13] = sec2[j0+k];
	  tmpVals[14] = sec2[j0+kP1];
	  tmpVals[15] = sec2[jP1+kM1];
	  tmpVals[16] = sec2[jP1+k];
	  tmpVals[17] = sec2[jP1+kP1];
	  tmpVals[18] = sec3[jM1+kM1];
	  tmpVals[19] = sec3[jM1+k];
	  tmpVals[20] = sec3[jM1+kP1];
	  tmpVals[21] = sec3[j0+kM1];
	  tmpVals[22] = sec3[j0+k];
	  tmpVals[23] = sec3[j0+kP1];
	  tmpVals[24] = sec3[jP1+kM1];
	  tmpVals[25] = sec3[jP1+k];
	  tmpVals[26] = sec3[jP1+kP1];
	  
	  if(j==0) { // Off the bottom
	    tmpVals[0]  = 0.0;
	    tmpVals[1]  = 0.0;
	    tmpVals[2]  = 0.0;
	    tmpVals[9]  = 0.0;
	    tmpVals[10] = 0.0;
	    tmpVals[11] = 0.0;
	    tmpVals[18] = 0.0;
	    tmpVals[19] = 0.0;
	    tmpVals[20] = 0.0;
	  }
	  if(k==0) { // Off the bottom
	    tmpVals[0]  = 0.0;
	    tmpVals[3]  = 0.0;
	    tmpVals[6]  = 0.0;
	    tmpVals[9]  = 0.0;
	    tmpVals[12] = 0.0;
	    tmpVals[15] = 0.0;
	    tmpVals[18] = 0.0;
	    tmpVals[21] = 0.0;
	    tmpVals[24] = 0.0;
	  }
	  if(j==(*numRows-1)) { // Off the Top
	    tmpVals[6]  = 0.0;
	    tmpVals[7]  = 0.0;
	    tmpVals[8]  = 0.0;
	    tmpVals[15] = 0.0;
	    tmpVals[16] = 0.0;
	    tmpVals[17] = 0.0;
	    tmpVals[24] = 0.0;
	    tmpVals[25] = 0.0;
	    tmpVals[26] = 0.0;
	  }
	  if(k==(*numCols-1)) { // Off the Top
	    tmpVals[2]  = 0.0;
	    tmpVals[5]  = 0.0;
	    tmpVals[8]  = 0.0;
	    tmpVals[11] = 0.0;
	    tmpVals[14] = 0.0;
	    tmpVals[17] = 0.0;
	    tmpVals[20] = 0.0;
	    tmpVals[23] = 0.0;
	    tmpVals[26] = 0.0;
	  }
	  
	  totFloats = 0.0;
	  for(indx=0; indx<27; indx++)
	    totFloats += tmpVals[indx] * binVals3D[indx];
	  
	  // Do we want to convert from TSB format for maps to robem format?
	  if(convTsb)
	    rowOut = ((lastRow - j - 1) / binStep)*binCol;
	  else
	    rowOut = (j/binStep)*binCol;
	  
	  colOut = k/binStep;
	  
	  thisFloat = 
	    float_buffer[((i/binStep)*rowColBin)+
			 rowOut+colOut]= (float)
	    (((float)(totFloats)) *
	     (*recipFloatScaleFactor));
	  
	  *minMapPixel = MIN(*minMapPixel,thisFloat);
	  *maxMapPixel = MAX(*maxMapPixel,thisFloat);
	}/* Do columns */
	
      }/* Do rows */
      
    }/* Do these sections */
    
  } else {/* Normal */
    
    for(i=0;i<(lastSec);i+=binStep) {/* Do all sections */
      /* printf("Sec[%d]  rowcol = [%d] bS[%d]\n",
	 i,rowCol,binStep);   */
      for(l=0;l<binStep;l++) {/* REad all sections */
	OKREAD(fread(int_buffer[l],sizeof(int),(rowCol),fIn), 
	       rowCol, "PIF_READ_MAP_INT_IMAGE");
      }
      for(j=0;j<(lastRow);j+=binStep) {/* Rows */
	for(k=0;k<(lastCol);k+=binStep) {/* Stuff the float buffer */
	  
	  /* Do we want to convert from TSB format for maps to robem
	     format?? */
	  if(convTsb)
	    rowOut = ((lastRow - j - 1) / binStep)*(*numCols/binStep);
	  else
	    rowOut = (j/binStep)*(*numCols/binStep);
	  
	  colOut = k/binStep;
	  
	  if(binStep == 1) {/* Just do every pixel */
	    thisOffset = j*(*numCols) + k;
	    thisIntBuffer = int_buffer[0];
	    
	    differentEndian((BYTE *)&thisIntBuffer[thisOffset], 
			     4, endians[*luNum]);
	    thisFloat =
	      float_buffer[((i/binStep)*rowColBin)+
			   rowOut+colOut]= (float)
	      (float)
	      (((float)(thisIntBuffer[thisOffset])) *
	       (*recipFloatScaleFactor));
	    
	  } else { // Average pixels to "bin" them
	    totFloats = 0.0;
	    totPnts = 0;
	    
	    for(l=0; l<binStep; l++) { // Do both sections
	      for(m=0; m<binStep; m++) { // Do both rows
		for(n=0; n<binStep; n++) { // Do both cols
		  thisOffset = (j+m)*(*numCols) + (k+n);
		  thisIntBuffer = int_buffer[l];
		  differentEndian((BYTE *)&thisIntBuffer[thisOffset], 
				   4, endians[*luNum]);
		  totFloats +=
		    (float)
		    (((float)(thisIntBuffer[thisOffset])) *
		     (*recipFloatScaleFactor));
		  totPnts++;
		}
	      }
	    }
	    
	    thisFloat = float_buffer[((i/binStep)*rowColBin)+rowOut+colOut] = 
	      (float)(totFloats/(float)totPnts);
	  }
	  
	  *minMapPixel = MIN(*minMapPixel, thisFloat);
	  *maxMapPixel = MAX(*maxMapPixel, thisFloat);
	}/* Stuff the float buffer */
	
      }/* Rows */
    }/* Do all sections */
    
  }/* Normal */
  
  free(int_buffer[0]);
  free(int_buffer[1]);
  free(int_buffer[2]);
  
} // end function pif_read_map_int_image

void pif_read_map_int_image_(int *luNum, float *float_buffer,
			     off_t *imgOffset,
			     int *numCols, int *numRows, int *numSecs,
			     float *recipFloatScaleFactor,
			     float *minMapPixel, float *maxMapPixel,
			     int *binSwitch, int *mode) {
  pif_read_map_int_image(luNum, float_buffer, imgOffset,
			 numCols, numRows, numSecs,
			 recipFloatScaleFactor,
			 minMapPixel, maxMapPixel,
			 binSwitch, mode);
}

void pif_read_map_int_image__(int *luNum, float *float_buffer,
			      off_t *imgOffset,
			      int *numCols, int *numRows, int *numSecs,
			      float *recipFloatScaleFactor,
			      float *minMapPixel, float *maxMapPixel,
			      int *binSwitch, int *mode) {
  pif_read_map_int_image(luNum, float_buffer, imgOffset,
			 numCols, numRows, numSecs,
			 recipFloatScaleFactor,
			 minMapPixel, maxMapPixel,
			 binSwitch, mode);
}



void pif_read_sf(forPifHead_t *sfHead, strucFact_t *sfs) 
{
  // function name: pif_read_sf
  // Read the Structure factor array to disk

  int i, localInt;
  FILE *fIn;
  float maxF=-999999.9, minF=999999.9;
  
  fIn = lun[sfHead->luNum];  
  fseek(fIn,1024,0);  /* SF files always begin at 1024 */
  
  for(i=0; i<sfHead->numRows; i++) {
    OKREAD(fread(&(sfs[i].h),sizeof(short),1,fIn),1,"PIF_READ_SF");
    differentEndian((BYTE *)&(sfs[i].h),2, endians[sfHead->luNum]);
    fread(&(sfs[i].k),sizeof(short),1,fIn);  
    differentEndian((BYTE *)&(sfs[i].k),2, endians[sfHead->luNum]);
    fread(&(sfs[i].l),sizeof(short),1,fIn);  
    differentEndian((BYTE *)&(sfs[i].l),2, endians[sfHead->luNum]);
    
    fread(&localInt,sizeof(int),1,fIn);        
    differentEndian((BYTE *)&localInt, 4, endians[sfHead->luNum]);
    convertBackFloat(localInt,&(sfs[i].a_amp),
		     sfHead->recipFloatScaleFactor);
    
    fread(&localInt,sizeof(int),1,fIn);        
    differentEndian((BYTE *)&localInt, 4, endians[sfHead->luNum]);
    convertBackFloat(localInt,&(sfs[i].b_phi),
		     sfHead->recipFloatScaleFactor);
    
    fread(&localInt,sizeof(int),1,fIn);        
    differentEndian((BYTE *)&localInt, 4, endians[sfHead->luNum]);
    convertBackFloat(localInt,&(sfs[i].sig),
		     sfHead->recipFloatScaleFactor);
    
    fread(&localInt,sizeof(int),1,fIn);        
    differentEndian((BYTE *)&localInt, 4, endians[sfHead->luNum]);
    convertBackFloat(localInt,&(sfs[i].dstr),
		     sfHead->recipFloatScaleFactor);
    
    minF = MIN(sfs[i].a_amp, minF);
    minF = MIN(sfs[i].b_phi, minF);
    
    maxF = MAX(sfs[i].a_amp, maxF);
    maxF = MAX(sfs[i].b_phi, maxF);
  }
  
} // end function pif_read_sf

void pif_read_sf_(forPifHead_t *sfHead, strucFact_t *sfs) {
  pif_read_sf(sfHead, sfs);
}

void pif_read_sf__(forPifHead_t *sfHead, strucFact_t *sfs) {
  pif_read_sf(sfHead, sfs);
}



void pif_init_head(forPifHead_t *thisHead) 
{
  // function name: pif_init_head
  // Initialize this PIF header to defaults
  
  thisHead->luNum      = 0;
  thisHead->nimg       = 1;
  thisHead->imgNum     = 1;
  thisHead->floatScaleFactor = 1.0;
  thisHead->recipFloatScaleFactor = 1.0;
  thisHead->int4scaleF = 1.0; 
  thisHead->htype      = 1;
  thisHead->numCols    = 0;
  thisHead->numRows    = 0;
  thisHead->numSecs    = 1;
  thisHead->mode       = -99;    /*20 ==MAP 21 = PRJS 
				   22 = PFTS 6 = PACKED BOXES 
				   1 == Zeiss scan*/
  thisHead->endianNess = Util_IAmBigEndian();
  
  sprintf(thisHead->versionStr,
	  "Undefined                                                                    ");
  thisHead->img_bkg    = 0;
  thisHead->datai4_min    = 0;
  thisHead->datai4_max    = 0;
  thisHead->datar4_min    = 0.0;
  thisHead->datar4_max    = 0.0;
  thisHead->datar4_avg    = 0.0;
  thisHead->datar4_std    = 0.0;
  thisHead->imgOffset  = 0;
  sprintf(thisHead->descriptionStr, 
	  "Undefined                                                                    ");
  sprintf(thisHead->timeStr, 
	  "Undefined                                                                    ");
  sprintf(thisHead->micrographStr, 
	  "Undefined                                                                    ");
  sprintf(thisHead->scanNumberStr, 
	  "Undefined                                                                    ");
  thisHead->aoverb     = 1.0;
  thisHead->map_abang  = 90.0;
  thisHead->a_cell     = 0.0;
  thisHead->b_cell     = 0.0;
  thisHead->c_cell     = 0.0;
  thisHead->alp        = 90.0;
  thisHead->bet        = 90.0;
  thisHead->gam        = 90.0;
  thisHead->dela       = 1.0;
  thisHead->delb       = 1.0;
  thisHead->delc       = 1.0;
  thisHead->a_star     = 0.0;
  thisHead->b_star     = 0.0;
  thisHead->c_star     = 0.0;
  thisHead->alp_star   = 0.0;
  thisHead->bet_star   = 0.0;
  thisHead->gam_star   = 0.0;
  thisHead->pixelSize  = 0.0;
  thisHead->t_matrix[0]= 1;
  thisHead->t_matrix[1]= 0;
  thisHead->t_matrix[2]= 0;
  thisHead->t_matrix[3]= 1;
  thisHead->t_matrix[4]= 0;
  thisHead->t_matrix[5]= 0;
  thisHead->packRadius = 0;
  thisHead->binFactor  = 1;
  thisHead->dthe       = 1.0;
  thisHead->dphi_90    = 1.0;
  thisHead->symmetry   = 1;
  thisHead->pixSizUnits = 0;
  thisHead->res_out = 0.0;
  thisHead->ctfMode = 0;
  thisHead->tempFactor = 0.0;
  thisHead->wiener = 0.0;
  thisHead->borderType = 0;
  thisHead->borderWidth = 0;
  thisHead->rad_lo = 0;
  thisHead->rad_hi = 0;
  thisHead->even   = 0;
  thisHead->mrcX   = 0;
  thisHead->mrcY   = 0;
  thisHead->mrcZ   = 0;
  thisHead->transX   = 0;
  thisHead->transY   = 0;
  thisHead->transZ   = 0;
  thisHead->isTrunc   = 0;
  thisHead->origX    = 0;
  thisHead->origY    = 0;
  thisHead->origZ    = 0;
  thisHead->a0       = 0.0;
  thisHead->a1       = 0.0;
  thisHead->a2       = 0.0;
  thisHead->a3       = 0.0;
  thisHead->a4       = 0.0;
  thisHead->a5       = 0.0;
  
} // end function pif_init_head

void pif_init_head_(forPifHead_t *thisHead) {
  pif_init_head(thisHead);
}

void pif_init_head__(forPifHead_t *thisHead) {
  pif_init_head(thisHead);
}



void pif_list_head(forPifHead_t *thisHead) 
{
  // function name: pif_list_head
  // Write the PIF file's header
  
  printf("luNum             [%d]\n",thisHead->luNum);
  printf("nimg              [%d]\n",thisHead->nimg);
  printf("version           [%s]\n",thisHead->versionStr);
  printf("int4scaleF        [%f]\n",thisHead->int4scaleF);
  printf("scaleF            [%f]\n",thisHead->floatScaleFactor);
  printf("img_bkg           [%f]\n",thisHead->img_bkg);
  printf("datai4_min        [%d]\n",thisHead->datai4_min);
  printf("datai4_max        [%d]\n",thisHead->datai4_max);
  printf("datar4_avg        [%f]\n",thisHead->datar4_avg);
  printf("datar4_std        [%f]\n",thisHead->datar4_std);
  printf("numCols           [%d]\n",thisHead->numCols);
  printf("numRows           [%d]\n",thisHead->numRows);
  printf("numSecs           [%d]\n",thisHead->numSecs);
  printf("mode              [%d]\n",thisHead->mode);
  printf("htype             [%d]\n",thisHead->htype);
  printf("description       [%s]\n",thisHead->descriptionStr);
  printf("time              [%s]\n",thisHead->timeStr);        
  printf("micrograph        [%s]\n",thisHead->micrographStr);  
  printf("scanNumber        [%s]\n",thisHead->scanNumberStr);  
  printf("aoverb            [%f]\n",thisHead->aoverb);
  printf("map_abang         [%f]\n",thisHead->map_abang);
  printf("a_cell            [%f]\n",thisHead->a_cell);
  printf("b_cell            [%f]\n",thisHead->b_cell);
  printf("c_cell            [%f]\n",thisHead->c_cell);
  printf("alp               [%f]\n",thisHead->alp);
  printf("bet               [%f]\n",thisHead->bet);
  printf("gam               [%f]\n",thisHead->gam);
  printf("dela              [%f]\n",thisHead->dela);
  printf("delb              [%f]\n",thisHead->delb);
  printf("delc              [%f]\n",thisHead->delc);
  printf("a_star            [%f]\n",thisHead->a_star);
  printf("b_star            [%f]\n",thisHead->b_star);
  printf("c_star            [%f]\n",thisHead->c_star);
  printf("alp_star          [%f]\n",thisHead->alp_star);
  printf("bet_star          [%f]\n",thisHead->bet_star);
  printf("gam_star          [%f]\n",thisHead->gam_star);
  printf("pixelSize         [%f]\n",thisHead->pixelSize);
  printf("packRadius        [%d]\n",thisHead->packRadius);
  printf("binFactor         [%d]\n",thisHead->binFactor);
  printf("dthe              [%f]\n",thisHead->dthe);
  printf("dphi_90           [%f]\n",thisHead->dphi_90);
  printf("symmetry          [%d]\n",thisHead->symmetry);
  printf("pixSizUnits       [%d]\n",thisHead->pixSizUnits);
  printf("res_out           [%f]\n",thisHead->res_out);
  printf("ctfMode           [%d]\n",thisHead->ctfMode);
  printf("tempFactor        [%f]\n",thisHead->tempFactor);
  printf("wiener            [%f]\n",thisHead->wiener);
  printf("borderType        [%d]\n",thisHead->borderType);
  printf("borderWidth       [%d]\n",thisHead->borderWidth);
  printf("rad_hi            [%d]\n",thisHead->rad_hi);
  printf("rad_lo            [%d]\n",thisHead->rad_lo);
  printf("even              [%d]\n",thisHead->even);
  printf("mrcX              [%d]\n",thisHead->mrcX);
  printf("mrcY              [%d]\n",thisHead->mrcY);
  printf("mrcZ              [%d]\n",thisHead->mrcZ);
  printf("transX            [%d]\n",thisHead->transX);
  printf("transY            [%d]\n",thisHead->transY);
  printf("transZ            [%d]\n",thisHead->transZ);
  printf("isTrunc           [%d]\n",thisHead->isTrunc);
  printf("origX             [%d]\n",thisHead->origX);
  printf("origY             [%d]\n",thisHead->origY);
  printf("origZ             [%d]\n",thisHead->origZ);
  printf("a0                [%12E]\n",thisHead->a0);
  printf("a1                [%12E]\n",thisHead->a1);
  printf("a2                [%12E]\n",thisHead->a2);
  printf("a3                [%12E]\n",thisHead->a3);
  printf("a4                [%12E]\n",thisHead->a4);
  printf("a5                [%12E]\n",thisHead->a5);
  
} // end function pif_list_head

void pif_list_head_(forPifHead_t *thisHead) {
  pif_list_head(thisHead);
}

void pif_list_head__(forPifHead_t *thisHead) {
  pif_list_head(thisHead);
}



int pif_write_dh(forPifHead_t *thisHead) 
{
  // function name: pif_write_dh
  // Write data header for pif file

  char descStr[100], junkChar[512];
  int endian, dataSize, i, localInt, zero=0;
  int  thisNcols, thisNrows, thisNsecs;
  float thisFSF, zerof=0.0;
  FILE *fOut;

  if(thisHead->mode == 46)
    dataSize = 4;
  else if (thisHead->mode == 20)
    dataSize = 2;
  else if(thisHead->mode == 1)
    dataSize = 2;
  else if(thisHead->mode == 7)
    dataSize = 2;
  else
    dataSize = 1;
  
  thisHead->imgOffset = 512 +         // Global header
    (512 * ((thisHead->imgNum -1))) + // Data headers
    ((off_t)(thisHead->imgNum -1) *    // Image data
     thisHead->numCols *
     thisHead->numRows *
     thisHead->numSecs *
     dataSize);

  endian = endians[thisHead->luNum];

  fOut = lun[thisHead->luNum];
  if(fseek(fOut, thisHead->imgOffset, 0)==-1) {
    printf("DH fseek() error.  If VMS have the file attributes changed?");
    exit(0);
  }

  thisFSF = thisHead->floatScaleFactor;
  if(thisHead->mode == 20)
    thisFSF = thisHead->int4scaleF; // Get header values in greater precision

  thisNcols = thisHead->numCols;
  thisNrows = thisHead->numRows;
  thisNsecs = thisHead->numSecs;
  differentEndian((BYTE *)&thisNcols, 4, endian);
  fwrite(&thisNcols, sizeof(int),1,fOut);
  differentEndian((BYTE *)&thisNrows, 4, endian);
  fwrite(&thisNrows, sizeof(int),1,fOut);
  differentEndian((BYTE *)&thisNsecs, 4, endian);
  fwrite(&thisNsecs, sizeof(int), 1, fOut);
  /* 12 */
  
  /* [2] write header for this image */
  localInt = thisHead->mode;
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  if(thisHead->mode == 20)
    i2Map = TRUE;
  else
    i2Map = FALSE;
  
  convertFloat(&localInt, thisHead->img_bkg, thisFSF);
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  /* 20 */
  
  localInt = thisHead->packRadius;
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int),1,fOut);             
  
  fwrite(&zero, sizeof(int), 1, fOut); // mrc_nxstart  // Not used as of 12-17-97
  fwrite(&zero, sizeof(int), 1, fOut); // mrc_nystart
  fwrite(&zero, sizeof(int), 1, fOut); // mrc_nzstart
  fwrite(&zero, sizeof(int), 1, fOut); // mrc_mx
  fwrite(&zero, sizeof(int), 1, fOut); // mrc_ny
  fwrite(&zero, sizeof(int), 1, fOut); // mrc_nz
  /* 48 */
  
  convertFloat(&localInt, thisHead->a_cell, thisFSF);
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  convertFloat(&localInt, thisHead->b_cell, thisFSF);
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  convertFloat(&localInt, thisHead->c_cell, thisFSF);
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  /* 68 */
  
  convertFloat(&localInt, thisHead->alp, thisFSF);
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  convertFloat(&localInt, thisHead->bet, thisFSF);
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  convertFloat(&localInt, thisHead->gam, thisFSF);
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  
  fwrite(&zero, sizeof(int), 1, fOut); // mrc_mapc // Not used as of 12-17-97
  fwrite(&zero, sizeof(int), 1, fOut); // mrc_mapr
  fwrite(&zero, sizeof(int), 1, fOut); // mrc_maps
  /* 92 */
  
#ifdef NEEDTOWRITE_THESE_AS_FLOAT
  convertFloat(&localInt, thisHead->datai4_min, thisFSF);
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  convertFloat(&localInt, thisHead->datai4_max, thisFSF);
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);       
#else
  localInt = thisHead->datai4_min;
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  localInt = thisHead->datai4_max;
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);       
#endif /* #ifdef NEEDTOWRITE_THESE_AS_FLOAT */
  convertFloat(&localInt, thisHead->datar4_avg, thisFSF);
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  convertFloat(&localInt, thisHead->datar4_std, thisFSF);
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  
  fwrite(&zero, sizeof(int), 1, fOut); // mrc_ispg   // Not used as of 12-17-97
  fwrite(&zero, sizeof(int), 1, fOut); // mrc_nsymbt
  fwrite(&zerof, sizeof(float), 1, fOut); // mrc_xorigin
  fwrite(&zerof, sizeof(float), 1, fOut); // mrc_yorigin
  /* 124 */
  
  memcpy(descStr, thisHead->descriptionStr, 72);
  descStr[72] = '\0';
  fwrite(descStr, sizeof(char), 80, fOut);
  
  memcpy(descStr, thisHead->timeStr, 32);           
  descStr[31] = '\0';                   
  fwrite(descStr, sizeof(char), 32, fOut); 
  
  memcpy(descStr, thisHead->micrographStr, 16);     
  descStr[15] = '\0';                   
  fwrite(descStr, sizeof(char), 16, fOut); 
  
  memcpy(descStr, thisHead->scanNumberStr, 8);      
  descStr[7] = '\0';                    
  fwrite(descStr, sizeof(char), 8, fOut);  
  /* 260 */
  
  convertFloat(&localInt, thisHead->aoverb, thisFSF);
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  convertFloat(&localInt, thisHead->map_abang, thisFSF);
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  convertFloat(&localInt, thisHead->dela, thisFSF);
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  convertFloat(&localInt, thisHead->delb, thisFSF);
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  convertFloat(&localInt, thisHead->delc, thisFSF);
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  /* 280 */
  
  for(i=0;i<6;i++) {
    localInt = thisHead->t_matrix[i];
    differentEndian((BYTE *)&localInt, 4, endian);
    fwrite(&localInt, sizeof(int), 1, fOut);        
  }
  
  convertFloat(&localInt, thisHead->dthe, thisFSF);
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  convertFloat(&localInt, thisHead->dphi_90, thisFSF);
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  
  localInt = thisHead->symmetry;
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  
  localInt = thisHead->binFactor;
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  
  convertFloat(&localInt, thisHead->a_star, thisFSF);
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  convertFloat(&localInt, thisHead->b_star, thisFSF);
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  convertFloat(&localInt, thisHead->c_star, thisFSF);
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  convertFloat(&localInt, thisHead->alp_star, thisFSF);
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  convertFloat(&localInt, thisHead->bet_star, thisFSF);
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  convertFloat(&localInt, thisHead->gam_star, thisFSF);
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  convertFloat(&localInt, thisHead->pixelSize, thisFSF);
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             

  localInt = thisHead->pixSizUnits;
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  convertFloat(&localInt, thisHead->res_out, thisFSF);
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  localInt = thisHead->ctfMode;
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  convertFloat(&localInt, thisHead->tempFactor, thisFSF);
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  convertFloat(&localInt, thisHead->wiener, thisFSF);
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  localInt = thisHead->borderType;
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  localInt = thisHead->borderWidth;
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             

  convertFloat(&localInt, thisHead->datar4_min, thisFSF);
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int),1,fOut);             
  convertFloat(&localInt, thisHead->datar4_max, thisFSF);
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);       

  localInt = thisHead->rad_lo;
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  localInt = thisHead->rad_hi;
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             

  localInt = thisHead->transX;
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int),1,fOut);             
  localInt = thisHead->transY;
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int),1,fOut);             
  localInt = thisHead->transZ;
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int),1,fOut);             
  localInt = thisHead->isTrunc;
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int),1,fOut);             
  localInt = thisHead->origX;
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  localInt = thisHead->origY;
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             
  localInt = thisHead->origZ;
  differentEndian((BYTE *)&localInt, 4, endian);
  fwrite(&localInt, sizeof(int), 1, fOut);             

  /* Pad out to 512 bytes */
  for(i=0;i<512;i++)
    junkChar[i]= (char)0;
  fwrite(junkChar, sizeof(char), 100, fOut);
  
  return(0);
  
} // end function pif_write_dh

int pif_write_dh_(forPifHead_t *thisHead) {
  return(pif_write_dh(thisHead));
}

int pif_write_dh__(forPifHead_t *thisHead) {
  return(pif_write_dh(thisHead));
}



int pif_write_gh(forPifHead_t *thisHead) 
{
  // function name: pif_write_gh
  // Write PIF file global header

  char floatScaleStr[18], ctfStr[18], junkChar[512];
  int i, file_id, localInt;
  FILE *fPIF;

  fPIF = lun[thisHead->luNum];
  endians[thisHead->luNum] = thisHead->endianNess;

  if(thisHead->floatScaleFactor != 0.0)
    thisHead->recipFloatScaleFactor = 1.0 / thisHead->floatScaleFactor;

  if(fPIF != NULL) {
    
    if(fseek(fPIF,0,0) ==- 1) {
      printf("fseek() error.  If VMS have the file attributes changed?");
      exit(0);
    }
    
    /* [1] Write global file header */
    file_id = 8;                 /* means PIF EMPORTABLE*/
    differentEndian((BYTE *)&file_id, 4, 0); /* force little-endian */
    fwrite(&file_id,sizeof(int),1,fPIF);
    fwrite(&file_id,sizeof(int),1,fPIF);
    
    
    sprintf(floatScaleStr, "%16.9E", thisHead->recipFloatScaleFactor);
    fwrite(floatScaleStr,sizeof(char),16,fPIF);
    localInt = thisHead->nimg;
    differentEndian((BYTE *)&localInt, 4, endians[thisHead->luNum]);
    fwrite(&localInt,sizeof(int),1,fPIF);             
    /* 28 */      
    /* This is VAX or DECstation */
    localInt = thisHead->endianNess;
    differentEndian((BYTE *)&localInt, 4, endians[thisHead->luNum]);
    fwrite(&localInt,sizeof(int),1,fPIF);             
    
    sprintf(thisHead->versionStr, "%s", VERSION); 
    fwrite(thisHead->versionStr, sizeof(char), 32, fPIF);
    /* 64 */      
    
    localInt = thisHead->htype;
    differentEndian((BYTE *)&localInt, 4, endians[thisHead->luNum]);
    fwrite(&localInt, sizeof(int), 1, fPIF);             
    localInt = thisHead->numCols;
    differentEndian((BYTE *)&localInt, 4, endians[thisHead->luNum]);
    fwrite(&localInt, sizeof(int), 1, fPIF);             
    localInt = thisHead->numRows;
    differentEndian((BYTE *)&localInt, 4, endians[thisHead->luNum]);
    fwrite(&localInt, sizeof(int), 1, fPIF);             
    localInt = thisHead->numSecs;
    differentEndian((BYTE *)&localInt, 4, endians[thisHead->luNum]);
    fwrite(&localInt, sizeof(int), 1, fPIF);             
    
    localInt = thisHead->mode;
    differentEndian((BYTE *)&localInt, 4, endians[thisHead->luNum]);
    fwrite(&localInt, sizeof(int), 1, fPIF);             
    if(thisHead->mode == 20)
      i2Map = TRUE;
    else
      i2Map = FALSE;

    /* 84 */      
    
    localInt = thisHead->even;
    differentEndian((BYTE *)&localInt, 4, endians[thisHead->luNum]);
    fwrite(&localInt, sizeof(int), 1, fPIF);             
    localInt = thisHead->mrcX;
    differentEndian((BYTE *)&localInt, 4, endians[thisHead->luNum]);
    fwrite(&localInt, sizeof(int), 1, fPIF);             
    localInt = thisHead->mrcY;
    differentEndian((BYTE *)&localInt, 4, endians[thisHead->luNum]);
    fwrite(&localInt, sizeof(int), 1, fPIF);             
    localInt = thisHead->mrcZ;
    differentEndian((BYTE *)&localInt, 4, endians[thisHead->luNum]);
    fwrite(&localInt, sizeof(int), 1, fPIF);             
    /* 100 */      
    
    sprintf(floatScaleStr,"%16.9E", thisHead->int4scaleF);
    fwrite(floatScaleStr,sizeof(char),16,fPIF);
    /* 116 */
    
    sprintf(ctfStr, "%12E", thisHead->a0);
    fwrite(ctfStr,sizeof(char), 13, fPIF);
    sprintf(ctfStr, "%12E", thisHead->a1);
    fwrite(ctfStr,sizeof(char), 13, fPIF);
    sprintf(ctfStr, "%12E", thisHead->a2);
    fwrite(ctfStr,sizeof(char), 13, fPIF);
    sprintf(ctfStr, "%12E", thisHead->a3);
    fwrite(ctfStr,sizeof(char), 13, fPIF);
    sprintf(ctfStr, "%12E", thisHead->a4);
    fwrite(ctfStr,sizeof(char), 13, fPIF);
    sprintf(ctfStr, "%12E", thisHead->a5);
    fwrite(ctfStr, sizeof(char), 13, fPIF);
    /* 194 */
    
    /* Pad out to 512 bytes*/
    for(i=0; i<512; i++)
      junkChar[i]= (char)0;
    fwrite(junkChar, sizeof(char), 318, fPIF);    
  }
  
  return(0);
  
} // end function pif_write_gh

int pif_write_gh_(forPifHead_t *thisHead) {
  return(pif_write_gh(thisHead));
}

int pif_write_gh__(forPifHead_t *thisHead) {
  return(pif_write_gh(thisHead));
}



void pif_write_prji1(forPifHead_t *prjHead, float *float_buffer,
		      int *id, int *numCols, int *numRows,
		      float *sFac, float *min) 
{
  // function name: pif_write_prji1

  int i, j, yOut, offset, numSecs, ncol, nrow;
  BYTE *byte_buffer;
  float minP,maxP,thisFac;
  
  thisFac = *sFac;
  minP = *min;
  ncol = *numCols;
  nrow = *numRows;
  byte_buffer = (BYTE *)malloc((nrow * ncol)*sizeof(BYTE));
  if(!byte_buffer) {/* CAnt go on */
    printf("[PIF_WRITE_PRJI1] Cannot allocate [%d]\n",(nrow * ncol));
    exit(0);
  }
  
  if(prjHead->imgNum==1) {/* Do this from the first one */
    minP = maxP = float_buffer[0];
    for(i=0; i<nrow; i++) {
      yOut = i * (ncol);
      for(j=0; j<ncol; j++) {
	offset = yOut+j;
	minP = MIN(float_buffer[offset], minP);
	maxP = MAX(float_buffer[offset], maxP);
      }
    }
#ifdef USETHIS
    if(fabs(minP) > maxP)
      maxP = 0.0 - minP;
    else
      minP = 0.0 - maxP;

    minP *= 1.3;
    maxP *= 1.3;
#endif
    thisFac = *sFac = 250.0 / (maxP-minP);
    *min = minP;
  }
  
  for(i=0; i<nrow; i++) {
    yOut = i * (ncol);
    for(j=0; j<ncol; j++) {/* Do all */
      offset = yOut+j;
      byte_buffer[offset]= (BYTE)((float_buffer[offset]-minP) * thisFac);
    }
  }
  
  prjHead->imgNum    = *id;
  prjHead->img_bkg   = byte_buffer[0];
  /* Need a packRadius */
  prjHead->packRadius = (*numCols)/2;
  prjHead->numCols   = ncol;
  prjHead->numRows   = nrow;
  pif_write_dh (prjHead);
  
  prjHead->imgOffset += 512;
  numSecs = 1;
  pif_write_byte_image(&(prjHead->luNum), byte_buffer, &(prjHead->imgOffset),
			numCols, numRows, &numSecs);
  
  free(byte_buffer);
  
} // end function pif_write_prji1

void pif_write_prji1_(forPifHead_t *prjHead, float *float_buffer,
		     int *id, int *numCols, int *numRows,
		     float *sFac, float *min) {
  pif_write_prji1(prjHead, float_buffer, id, numCols, numRows,
		  sFac, min);
}

void pif_write_prji1__(forPifHead_t *prjHead, float *float_buffer,
		     int *id, int *numCols, int *numRows,
		     float *sFac, float *min) {
  pif_write_prji1(prjHead, float_buffer, id, numCols, numRows,
		  sFac, min);
}



void pif_write_byte_image(int *luNum, BYTE *byte_buffer, off_t *imgOffset,
			   int *numCols, int *numRows, int *numSecs) 
{
  // function name: pif_write_byte_image
  // Write the PIF file's BYTE image for a specific image

  FILE *fPIF;
  fPIF = lun[*luNum];
  
  fseek(fPIF, *imgOffset, 0);
  fwrite(byte_buffer, sizeof(BYTE), (*numCols)*(*numRows)*(*numSecs), fPIF);
  
} // end function pif_write_byte_image

void pif_write_byte_image_(int *luNum, BYTE *byte_buffer, off_t *imgOffset,
			  int *numCols, int *numRows, int *numSecs) {
  pif_write_byte_image(luNum, byte_buffer, imgOffset,
			numCols, numRows, numSecs);
}

void pif_write_byte_image__(int *luNum, BYTE *byte_buffer, off_t *imgOffset,
			  int *numCols, int *numRows, int *numSecs) {
  pif_write_byte_image(luNum, byte_buffer, imgOffset,
			numCols, numRows, numSecs);
}



void pif_write_byteshort_image(int *luNum, short *short_buffer, off_t *imgOffset,
				int *numCols, int *numRows, int *numSecs) 
{
  // function name: pif_write_byteshort_image
  // Write the PIF file's SHORT image in BYTE on disk for a specific image

  FILE *fPIF;
  BYTE *byte_buffer;
  int i, j, k, ncol, nrow, nsec, rowOut;
  float scaleFactor, minF, maxF;

  nsec = *numSecs;
  nrow = *numRows;
  ncol = *numCols;

  minF = maxF = (float)short_buffer[0];
  for(i=0; i<nsec; i++) {
    for(j=0; j<nrow; j++) {
      rowOut = i*ncol*nrow + j*ncol;
      for(k=0; k<ncol; k++) {
	minF = MIN(minF, (float)short_buffer[rowOut+k]);
	maxF = MAX(maxF, (float)short_buffer[rowOut+k]);
      }
    }
  }
  
  if(minF != maxF)
    scaleFactor = 254.0 / (maxF-minF);
  else
    scaleFactor = 1.0;

  byte_buffer = (BYTE *)malloc(sizeof(BYTE)*(ncol*nrow*nsec));
  for(i=0; i<nsec; i++) {
    for(j=0; j<nrow; j++) {
      rowOut = i*ncol*nrow + j*ncol;
      for(k=0;k<ncol;k++) {
	byte_buffer[rowOut+k]= (BYTE)(((float)short_buffer[rowOut-k]-
				       minF) * scaleFactor);
      }
    }
  }

  fPIF = lun[*luNum];

  fseek(fPIF, *imgOffset, 0);
  fwrite(byte_buffer, sizeof(BYTE), (*numCols)*(*numRows)*(*numSecs), fPIF);

  free(byte_buffer);

} // end function pif_write_byteshort_image

void pif_write_byteshort_image_(int *luNum, short *short_buffer, 
				off_t *imgOffset, int *numCols, 
				int *numRows, int *numSecs) {
  pif_write_byteshort_image(luNum, short_buffer, imgOffset,
			     numCols, numRows, numSecs);
}

void pif_write_byteshort_image__(int *luNum, short *short_buffer, 
				off_t *imgOffset, int *numCols, 
				int *numRows, int *numSecs) {
  pif_write_byteshort_image(luNum, short_buffer, imgOffset,
			     numCols, numRows, numSecs);
}



void pif_write_short_image(int *luNum, short *short_buffer,
			   off_t *imgOffset, int *numCols, int *numRows, 
			   int *numSecs) 
{
  // function name: pif_write_short_image
  // Write the PIF file's SHORT image for a specific image

  int i, j, k, m;
  short *local_short_buffer;
  FILE *fPIF;

  fPIF = lun[*luNum];

  fseek(fPIF, *imgOffset, 0);

  local_short_buffer = (short *)
    malloc(sizeof(short)*((*numCols) * (*numRows) * (*numSecs) ));

  for(i=0; i<(*numSecs); i++) {
    for(j=0; j<(*numRows); j++) {
      for(k=0; k<(*numCols); k++) {
	m = i*(*numRows)*(*numCols) + j*(*numCols) + k;
	local_short_buffer[m] = short_buffer[m];
	differentEndian((BYTE *)&local_short_buffer[m], 2, endians[*luNum]);
      }
    }
  }
  
  fwrite(local_short_buffer,sizeof(short),
	 ((*numCols)*(*numRows)*(*numSecs)), fPIF);
  
  free(local_short_buffer);
  
} // end function pif_write_short_image

void pif_write_short_image_(int *luNum, short *short_buffer,
			   off_t *imgOffset,
			   int *numCols, int *numRows, int *numSecs) {
  pif_write_short_image(luNum, short_buffer, imgOffset,
			numCols, numRows, numSecs);
}

void pif_write_short_image__(int *luNum, short *short_buffer,
			   off_t *imgOffset,
			   int *numCols, int *numRows, int *numSecs) {
  pif_write_short_image(luNum, short_buffer, imgOffset,
			numCols, numRows, numSecs);
}



void pif_write_rowi4(forPifHead_t *thisHead, float *float_buffer, 
		      int *nrot, int *nannuli,
		      int *whichAnnulus, int *whichImg) 
{
  // function name: pif_write_rowi4
  // Write this INTEGER*4 float to disk one row at a time

  off_t imgOffset;
  int thisNrot, thisAnnulus=1, nsec=1;

  imgOffset =  1024 + (((*whichAnnulus)-1)*(*nrot/4)*4) + 
    ((off_t)((*whichImg)-1)*(*nrot/4)*(*nannuli)*4);
  
  thisNrot = *nrot/4;
  pif_write_map_fltint(&(thisHead->luNum), float_buffer, 
		       &imgOffset, &thisNrot, &thisAnnulus, 
		       &nsec, &(thisHead->floatScaleFactor));

} // end function pif_write_rowi4

void pif_write_rowi4_(forPifHead_t *thisHead,  float *float_buffer, 
		     int *nrot,  int *nannuli,
		     int *whichAnnulus, int *whichImg) {
  pif_write_rowi4(thisHead, float_buffer, nrot, nannuli, 
		   whichAnnulus, whichImg);
}

void pif_write_rowi4__(forPifHead_t *thisHead,  float *float_buffer, 
		     int *nrot,  int *nannuli,
		     int *whichAnnulus, int *whichImg) {
  pif_write_rowi4(thisHead, float_buffer, nrot, nannuli, 
		   whichAnnulus, whichImg);
}



void pif_write_map_short_image(int *luNum, short *short_buffer,
				off_t *mapOffset, int *numCols) 
{
  // function name: pif_write_map_short_image
  // Write the PIF file's SHORT image for a floating point MAP

  int i;
  FILE *fIn;

  fIn = lun[*luNum];

  fseek(fIn,*mapOffset,0);
  for(i=0; i<(*numCols); i++)
    differentEndian((BYTE *)&short_buffer[i], 2, endians[*luNum]);

  OKREAD(fwrite(short_buffer,sizeof(short),(*numCols),fIn),
	     *numCols, "PIF_WRITE_MAP_SHORT_IMAGE");

} // end function pif_write_map_short_image

void pif_write_map_short_image_(int *luNum, short *short_buffer,
			       off_t *mapOffset, int *numCols) {
  pif_write_map_short_image(luNum, short_buffer, mapOffset, numCols);
}

void pif_write_map_short_image__(int *luNum, short *short_buffer,
			       off_t *mapOffset, int *numCols) {
  pif_write_map_short_image(luNum, short_buffer, mapOffset, numCols);
}



void pif_write_map_float_image(int *luNum, float *float_buffer,
				off_t *mapOffset,
				int *numCols, int *numRows, int *numSecs,
				float *floatScaleFactor) 
{
  // function name: pif_write_map_float_image
  // Write the PIF file's FLOAT image for entire floating point MAP

  int i,j,k,thisSec,thisRow;
  FILE *fOut;
  short *thisShort;
  int thisInt;
  float thisFloat;
  double thisDiff;

  fOut = lun[*luNum];

  thisShort = (short *)malloc(sizeof(short)*(*numCols+2));

  fseek(fOut,(*mapOffset),0);

  for(i=0; i<*numSecs; i++) {/* secs */
    thisSec = i * (*numRows) * (*numCols);
    for(j=0; j<*numRows; j++) {/* rows */
      /* It's already in TSB format  */
      thisRow = j * (*numCols);
      
      for(k=0;k<*numCols;k++) {/* Do all */
	thisFloat = (float)(float_buffer[(thisSec + 
					  thisRow + k)]*
			    (*floatScaleFactor));
	/* Mimic the fortran nint() rtn */
	thisInt = (int)thisFloat;
	thisDiff = (double)((double)thisFloat - (double)(thisInt));
	if(thisDiff >= 0.5) /* rounnd it */
	  thisShort[k] = (short)thisInt + 1;
	else if(thisDiff <= -0.5) /* rounnd it */
	  thisShort[k] = (short)thisInt - 1;
	else
	  thisShort[k] = (short)thisInt;
	
	differentEndian((BYTE *)&thisShort[k], 2, endians[*luNum]);
      }
      
      OKREAD(fwrite(thisShort,sizeof(short),(*numCols),fOut), *numCols,
	     "PIF_WRITE_MAP_FLOAT_IMAGE");
    }
  }
  
  free(thisShort);

} // end function pif_write_map_float_image

void pif_write_map_float_image_(int *luNum, float *float_buffer,
			       off_t *mapOffset,
			       int *numCols, int *numRows, int *numSecs,
			       float *floatScaleFactor) {
  pif_write_map_float_image(luNum, float_buffer, mapOffset,
			     numCols, numRows, numSecs,
			     floatScaleFactor);
}

void pif_write_map_float_image__(int *luNum, float *float_buffer,
			       off_t *mapOffset,
			       int *numCols, int *numRows, int *numSecs,
			       float *floatScaleFactor) {
  pif_write_map_float_image(luNum, float_buffer, mapOffset,
			     numCols, numRows, numSecs,
			     floatScaleFactor);
}



void pif_write_map_fltint(int *luNum, float *float_buffer, off_t *mapOffset,
			   int *numCols, int *numRows, int *numSecs,
			   float *floatScaleFactor) 
{
  // function name: pif_write_map_fltint
  //
  // Write the PIF file's FLOAT image for entire floating point
  // MAP. Convert floats to ints not shorts as is done in
  // pif_write_map_float_image() above.

  int i, j, k, thisSec, thisRow, tempInt;
  FILE *fOut;
  int *thisInt;
  float thisFloat;
  double thisDiff;

  fOut = lun[*luNum];

  thisInt = (int *)malloc(sizeof(int)*(*numCols+2));

  fseek(fOut,(*mapOffset),0);

  /* minInt = maxInt = (int)float_buffer[0]; */
  for(i=0;i<*numSecs;i++) { // secs
    thisSec = i * (*numRows) * (*numCols);
    for(j=0;j<*numRows;j++) { // rows
      /* It's already in TSB format  */
      thisRow = j * (*numCols);
      
      for(k=0; k<*numCols; k++) { // Do all
	thisFloat = (float)(float_buffer[(thisSec + 
					  thisRow + k)]*
			    (*floatScaleFactor));
	/* Mimic the fortran nint() rtn */
	tempInt = thisFloat;
	thisDiff = thisFloat - tempInt;
	
	if(thisDiff >= 0.5) // round it
	  thisInt[k] = tempInt + 1;
	else if(thisDiff <= -0.5) // round it
	  thisInt[k] = tempInt - 1;
	else
	  thisInt[k] = tempInt;
	
	if(thisFloat > 2147483647.0) { // Overflow
	  printf("[PIF_WRITE_MAP_FLTINT()] Overflow of int.[%f][%f]\n",
		 float_buffer[thisSec+thisRow+ k], *floatScaleFactor);
	  exit(0);
	}
	
	/*minInt = MIN(minInt,thisInt[k]);
	  maxInt = MAX(maxInt,thisInt[k]); */
	differentEndian((BYTE *)&thisInt[k], 4, endians[*luNum]);
      }
      
      OKREAD(fwrite(thisInt,sizeof(int),(*numCols),fOut), *numCols,
	     "PIF_WRITE_MAP_FLTINT");
    }
  }

  free(thisInt);

} // end function pif_write_map_fltint

void pif_write_map_fltint_(int *luNum, float *float_buffer, off_t *mapOffset,
			   int *numCols, int *numRows, int *numSecs,
			   float *floatScaleFactor) {
  pif_write_map_fltint(luNum, float_buffer, mapOffset,
			numCols, numRows, numSecs,
			floatScaleFactor);
}

void pif_write_map_fltint__(int *luNum, float *float_buffer, off_t *mapOffset,
			   int *numCols, int *numRows, int *numSecs,
			   float *floatScaleFactor) {
  pif_write_map_fltint(luNum, float_buffer, mapOffset,
			numCols, numRows, numSecs,
			floatScaleFactor);
}



void pif_write_sf(forPifHead_t *sfHead, strucFact_t *sfs) 
{
  // function name: pif_write_sf
  // Write the Structure factor array to disk

  int i, localInt;
  FILE *fOut;
  float minF,maxF;

  fOut = lun[sfHead->luNum];

  fseek(fOut,1024,0);  /* SF files always begin at 1024 */

  maxF= minF = sfs[0].a_amp;
  for(i=0;i<sfHead->numRows;i++) {
    
    OKREAD(fwrite(&(sfs[i].h),sizeof(short),1,fOut),1,"PIF_WRITE_SF");
    fwrite(&(sfs[i].k),sizeof(short),1,fOut);
    fwrite(&(sfs[i].l),sizeof(short),1,fOut);
    
    convertFloat(&localInt,sfs[i].a_amp,sfHead->floatScaleFactor);
    differentEndian((BYTE *)&localInt, 4, endians[sfHead->luNum]);
    fwrite(&localInt,sizeof(int),1,fOut);             
    
    convertFloat(&localInt,sfs[i].b_phi,sfHead->floatScaleFactor);
    differentEndian((BYTE *)&localInt, 4, endians[sfHead->luNum]);
    fwrite(&localInt,sizeof(int),1,fOut);             
    
    convertFloat(&localInt,sfs[i].sig,sfHead->floatScaleFactor);
    differentEndian((BYTE *)&localInt, 4, endians[sfHead->luNum]);
    fwrite(&localInt,sizeof(int),1,fOut);             
    
    convertFloat(&localInt,sfs[i].dstr,sfHead->floatScaleFactor);
    differentEndian((BYTE *)&localInt, 4, endians[sfHead->luNum]);
    fwrite(&localInt,sizeof(int),1,fOut);             
    
    minF = MIN(sfs[i].a_amp,minF);
    minF = MIN(sfs[i].b_phi,minF);
    
    maxF = MAX(sfs[i].a_amp,maxF);
    maxF = MAX(sfs[i].b_phi,maxF);
    
  }
  
  fseek(fOut,(512+92),0);  // Seek the data min/max fields
  convertFloat(&localInt,minF,sfHead->floatScaleFactor);
  differentEndian((BYTE *)&localInt, 4, endians[sfHead->luNum]);
  fwrite(&localInt,sizeof(int),1,fOut);             
  convertFloat(&localInt,maxF,sfHead->floatScaleFactor);
  differentEndian((BYTE *)&localInt, 4, endians[sfHead->luNum]);
  fwrite(&localInt,sizeof(int),1,fOut);

} // end function pif_write_sf

void pif_write_sf_(forPifHead_t *sfHead, strucFact_t *sfs) {
  pif_write_sf(sfHead, sfs);
}

void pif_write_sf__(forPifHead_t *sfHead, strucFact_t *sfs) {
  pif_write_sf(sfHead, sfs);
}



void pif_read_boxi4(forPifHead_t *boxHead, float *float_buffer) 
{
  // function name: pif_read_boxi4
  // Read in 4-byte boxed images
  // Gotten from Misha Sherman Jan 2003

  off_t boxOffset;
  int i,j;
  int luNum,numCols, numRows, numSecs,imgNum;
  float recipFloatScaleFactor;
  int localInt;
  int *rowInts;
  float localFloat;
  FILE *fIn;

  luNum   = boxHead->luNum;
  numCols = boxHead->numCols;
  numRows = boxHead->numRows;
  numSecs = 1;
  imgNum  = boxHead->imgNum;
  recipFloatScaleFactor  = boxHead->recipFloatScaleFactor;
  
  // Times 4 because we're INT here
  boxOffset = 512 + imgNum*512 + (off_t)(imgNum-1)*numCols*numRows*4; 

  rowInts = (int *)malloc(sizeof(int)*numCols);

  fIn = lun[luNum];

  fseek(fIn,boxOffset,0);

  for(i=0; i<numRows; i++) { // rows
    fread(rowInts, sizeof(int), numCols, fIn); 
    for(j=0; j<numCols; j++) { // cols
      localInt = rowInts[j];
      differentEndian((BYTE *)&localInt, 4, endians[luNum]);
      convertBackFloat(localInt, &localFloat, recipFloatScaleFactor);
      float_buffer[i*numCols + j]= localFloat;
    }
  }

  free(rowInts);

} // end function pif_read_boxi4

void pif_read_boxi4_(forPifHead_t *boxHead, float *float_buffer) {
  pif_read_boxi4(boxHead,float_buffer);
}

void pif_read_boxi4__(forPifHead_t *boxHead, float *float_buffer) {
  pif_read_boxi4(boxHead,float_buffer);
}



void pif_copy_head(forPifHead_t *fromHead, forPifHead_t *toHead) 
{
  // function name: pif_copy_head
  //
  // Copy one PIF header to another. Using the struct assignment is
  // less error prone than individually copying each element.

  *toHead = *fromHead; // struct assignment
  
} // end function pif_copy_head

void pif_copy_head_(forPifHead_t *fromHead,forPifHead_t *toHead) {
  pif_copy_head(fromHead,toHead);
}

void pif_copy_head__(forPifHead_t *fromHead,forPifHead_t *toHead) {
  pif_copy_head(fromHead,toHead);
}



void chk_file_exists(char *fname) 
{
  // Function name: chk_file_exists
  // Determine whether or not a file exists

  FILE *fIn;
  char inChar, sysCall[1024];
  int totBytes=0;

  fIn = fopen(fname,"r");
  if(fIn) {
    printf("\n\nFound file [%s]\n",fname);
    
    while(!feof(fIn)) { // Read in up to 10 bytes from file
      fread(&inChar,sizeof(BYTE),1,fIn);
      totBytes++;
      if(totBytes > 10) break;
    }
    
    fclose(fIn);
    if(totBytes > 8) { // Keep file
      printf("\n\n");
      printf("******************************************************\n");
      printf("You already have a non-empty file [%s]\n", fname);
      printf("Aborting program ...\n");
      printf("******************************************************\n\n");
      exit(0);
    }
    else { // Overwrite file
      printf("\n\n");
      printf("******************************************************\n");
      printf("You already have a file with minimal data [%s]\n", fname);
      printf("I'm blowing that file away ...\n");
      printf("******************************************************\n\n");
      sprintf(sysCall,"rm %s",fname);
      system(sysCall);
    }
    
  }
  
} // end function chk_file_exists

void chk_file_exists_(char *fname) {
  chk_file_exists(fname);
}

void chk_file_exists__(char *fname) {
  chk_file_exists(fname);
}


// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXx

void pif_write_map_sections(forPifHead_t *head, float *float_buffer,
			    int *mode, int *ncol, int *nrow, int *nsec,
			    int *secOffset, int *sections) 
{
  // function name: pif_write_map_sections
  // Write out one or more sections of an int*2 or int*4 map
  // Uses C convention for section offset. (1st section is offset 0)

  int thisBin;
  off_t offset;

  // Testing to make sure that ncol, nrow, and nsec are compatible
  // with the bin factor and map dimensions specified in the
  // header. Most of this is a stub for not yet implemented features.

  if(*ncol==head->numCols && *nrow==head->numRows && *nsec==head->numSecs) {
    thisBin = 1;
  } 
  else if (*ncol > head->numCols && *nrow > head->numRows &&
	   *nsec > head->numSecs ) {
    printf("PIF_WRITE_MAP_SECTIONS - ");
    printf("I don't know how to do this [%d][%d][%d][%d][%d][%d][%d]\n",
	   head->numCols, head->numRows, head->numSecs,
	   *ncol, *nrow, *nsec, head->binFactor);
    exit(0);
  } 
  else
    thisBin = 2;

  if(thisBin == 2) {
    printf("PIF_WRITE_MAP_SECTIONS() BIN not implemented on write yet... \n");
    exit(0);
  }

  if(*mode == 20) { // 2-byte maps
    offset = 1024 + 2*(off_t)(*secOffset)*(*ncol)*(*nrow);
    pif_write_map_float_image(&(head->luNum), float_buffer,
			      &offset, ncol, nrow, sections,
			      &(head->floatScaleFactor));
  }
  else if(*mode == 40) { // 4-byte maps
    offset = 1024 + 4*(off_t)(*secOffset)*(*nrow)*(*ncol);
    pif_write_map_fltint(&(head->luNum), float_buffer,
			 &offset, ncol, nrow, sections,
			 &(head->floatScaleFactor));
  } //??? RSS Is mode 6 correct ??????
  else if(*mode == 2 || *mode == 6 || *mode == 46) { // boxed images
    offset = 512 + head->imgNum*512 + (off_t)(head->imgNum-1)*(*ncol)*(*nrow)*4; 
    pif_write_map_fltint(&(head->luNum), float_buffer,
			 &offset, ncol, nrow, sections,
			 &(head->floatScaleFactor));
  }
  else {
    printf("PIF_WRITE_MAP_SECTIONS - mode (%d) must be ", *mode);
    printf("2, 6, 20, 40, or 46\n");
    exit(0);
  }
    
} // end function pif_write_map_sections

void pif_write_map_sections_(forPifHead_t *head, float *float_buffer,
			    int *mode, int *ncol, int *nrow, int *nsec,
			    int *secOffset, int *sections) {
  pif_write_map_sections(head, float_buffer, mode, 
			 ncol, nrow, nsec, secOffset, sections);
}

void pif_write_map_sections__(forPifHead_t *head, float *float_buffer,
			    int *mode, int *ncol, int *nrow, int *nsec,
			    int *secOffset, int *sections) {
  pif_write_map_sections(head, float_buffer, mode, 
			 ncol, nrow, nsec, secOffset, sections);
}
