/* File oned.h */
/*******************************************************************
 * ONED - 
 *
 * Include file for oned.c that includes global definitions
 * MACROS and forward declarations for oned.
 *
 * Tim Baker
 * Biology Department
 * Purdue University
 * January 1997
 *
 *******************************************************************/

#include <string.h>
#include <stdio.h>                              /* For printf and so on. */
#include <math.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>

#include "../include/libPIF.h"

#include <Xm/MessageB.h>                /* For message box stuff */
#include <Xm/Label.h>
#include <Xm/DialogS.h>
#include <Xm/Text.h>
#include <Xm/ScrollBar.h>
#include <Xm/ToggleB.h>
#include <Xm/PushB.h>
#include <Mrm/MrmAppl.h>                /* Motif Toolkit and MRM */

#include <X11/cursorfont.h>                /* For watch cursor */
#include <Xm/FileSB.h>

/* #include <Xm/String.h> */


/**************************
 *  TYPEDEFS              *
 **************************/

/* STruct for models */
typedef struct __model_t__
  {/* MODEL_T */
    float   ix1;
    float   ix2;
    float   ix3;
    int     iy1;
    int     iy2;
  }/* MODEL_T */ model_t;

typedef unsigned long	X_PIXEL;



/**************************
 *  DEFINES               *
 **************************/

/* We need to have some default image buffer sizes etc.. for when the
 * program starts up.  Change these as needed dynamically during
 * program execution */
#define NUMCOLS 2048	     /* Default image size */
#define NUMROWS 2048
/* #define DEF_DISPLAY_SIZE 900*/ /* Default display size */
#define DEF_DISPLAY_SIZE 512 /* Default display size */
#define MAXSIZE 2048         /* Maximum width or height in pixels for image */
#define MAXBOX 1000
#define ONED_WIDTH  512
#define ONED_HEIGHT 512
#define BOX_WIDTH  500
#define BOX_HEIGHT 500

#define PI        3.1415926535
#define	THISTWOPI 6.283185307         
#define DENOMINATOR 2147483647    /* For random number generator conversion */

#define PROGRAM_NAME		"ONED"
#define FONT "fixed"


/* Now our oned colors */
#define WHITE       0           /* Drawing colors */
#define BLACK       1
#define RED         2
#define GREEN       3
#define BLUE        4
#define PLACEHOLDER 5
#define ENTRY       0           /* Widget for color map entry */
#define YELLOW      6


/* Now define the values we defined in oned.uil */
#define ONED	   1
#define MAIN       2
#define MODEL      3
#define FFTSCREEN  4
#define FLOAT      5
#define TEMP	   6
#define FILTER	   7
#define NOISE	   8
#define NUMSAMP	   9
#define NUMMOTIF   10
#define SIGNOISE   11
#define ALLON      12
#define FILTER2	   13
#define	NOISESPEC  14


#define READING    100
#define WRITING    101


/* Define the following so we don't define twice.  e.g. have everyone
 * call this include only if the following is undefined.
 */
#define MACROS_DEFINED 1




/* ************************************** *
 * RAW Xlib definitions                   *
 * ************************************** */
Arg arglist[32];
X_PIXEL		black_pixel;
unsigned long             *pixels_return;
Visual		* visual = NULL;
int		visual_depth = 0;
Colormap		color_map = (Colormap) 0;
int              *plane_mask;
int              contig;

int 		display_width;
int		display_height;

int 		oned_width;
int		oned_height;

int 		buffer_width;
int		buffer_height;

int		image_width;		/* in pixels */
int		image_height;

int		image_colors;
GC		image_gc = (GC) 0;      
GC		image_gcWHITE = (GC) 0;
GC		image_gcYELLOW = (GC) 0;
GC		image_gcGREEN = (GC) 0;
GC		image_gcGREENDASH = (GC) 0;
GC		image_gcRED = (GC) 0;
GC		image_gcBLUE = (GC) 0;
GC		image_gcBLACK = (GC) 0;

unsigned long	        gc_mask;
XGCValues		gc_values;
XColor	                * color_defs= NULL;
float                   pixelColor;
unsigned long           event_mask;

int hiColor;

XFontStruct *fontstruct; 	/* Font descriptor */

/* Global display and applications context pointers*/
Display         *display;		/* Display variable */
XtAppContext    app_context;		/* application context */

/* The UIL file hierarchy */
static MrmHierarchy s_MrmHierarchy;     /* MRM database hierarchy ID */
/* static MrmType *dummy_class; */           /* and class variable. */
static MrmType dummy_class;            /* and class variable. */
static char *db_filename_vec[] =        /* Mrm.heirachy file list. */
  {"oned.uid" /* There is only one UID file for */
  };                                    /* this application. */              

static int db_filename_num =
                (sizeof db_filename_vec / sizeof db_filename_vec [0]);


/***************************************
 * Generic ONED application work areas *
 ***************************************/
int          i, j;               /* Generic loop counters */
int          inited;             /* Are the windows inited yet?? */
int          globAsk;            /* What do they want to do when asked? */
int          gFinished;          /* Let us know when they're finished from
				  * being asked */
int          curOned;            /* Curve stack pointer */
float        onedScrollValue;    /* Value of the scrollbar */
fString_t    fileName;           /* STRing descriptor for passing C strings
				  * to FORTRAN subroutines */
BYTE *image_buffer1;             /* buffer to display in the window */
XImage *image_image1;            /* X Windows logical image */
Window image_window1;            /* X Windows "window" the image is in */

int          modelState[12];     /* WHich models are being used? */
int          rememberState[12];  /* WHich models are being used? */
float        ampScrollValue[12]; /* What is the value of the amp 
				  * scroll [i] ? */
float        phaseScrollValue[12]; 
                                 /* What is the value of the phase
				  * scroll [i] ? */
int           wavnumScrollValue[12]; 
                                 /* What is the value of the wavnum
				  * scroll [i] ? */
int          tag[12];            /* tags to send to callbacks */
model_t      model[12*515];      /* Our models */
model_t      totModel[1515];     /* Our gold standard model */
model_t      perfCrystal[1515];  /* Our gold standard SAMP crystal noNoise */
model_t      sampCrystal[1515];  /* Our gold standard SAMPLED crystal */
model_t      fftCrystal[1515];   /* Our  FFT Transformed SAMPLED crystal */
model_t      backCrystal[1515];  /* Our backTransformed SAMPLED crystal */
model_t      filterRemember[515];/* Which were filtered before? */
float        tempFactors[515];   /* Calculated temp factors */
int          formUp;             /* Which form do we have on the screen? */
int          numFilter;          /* How many pixels ON in mask */
int          numFilter2;         /* How many pixels OFF in mask */
int          numSamps;           /* Number of samples/motif*/
float        numMotifs;          /* Number of motifs per crystal */
float        maxNumMotifs;       /* MAX Number of motifs per crystal */
float        davg,dfloat,dmin,dmax;  /* Globals for stats */
float        dfloatBack;         /* Float for back trans Crystal */
int          ndata;              /* Number of data points in model */
float        xscl;               /* Number of points to scale to */
int          xnp;                /* equivalent to numSamps/numMotifs */
int          filterSwitch;       /* Are we filtering ? */
int          floatSwitch;        /* Are we floating ? */
int          noiseSwitch;        /* Add noise? */
int          tempSwitch;         /* Add noise? */
int          allonSwitch;        /* All curves active? */
int          annulusSwitch;      /* Filter for resolution an annulus or REV */
float        numTemp;            /* How much temp */
float        numNoise;           /* How much noise */
float        numSigNoise;        /* What signal/nosie ? */
float        factor;             /* COnvert random number into 0.0-1.0 */
int          ranging;            /* Are they forming a range? */
int          startRange,endRange;/* Endpoints for range */
int          readWrite;          /* Are they reading or writing a file ? */
float        noiseSpectrum[515]; /* Noise calculated for screen */
float        noiseFactor[515];   /* Noise calculated for screen */
int          beginAnn,endAnn;    /* Annulus marks for our lo/hi pass filter*/
char         saveTempStr[64],
             saveSigStr[64];     /* Remember values for last time */
char         outStr[512];        /* Generic string */


/* All of our widgets */
static Widget toplevel_widget,          /* Root widget ID of our */
                                        /* application. */
  main_window_widget;                   /* Root widget ID of main */
                                        /* MRM fetch */
static Widget                           /* Our screens */
  drawingWidget,
  infoBox;
Widget onedscreen,fftscreen;
Widget fileSel_widget;     /* Holds the file selection widget */
static Widget ampScroll[12],ampLabel[12],       /* Application widgets */
              phaseScroll[12],phaseLabel[12],
              wavnumLabel[12];
static Widget modelToggle[12];
static Widget resetButton[12];
static Widget numSampWidget,numMotifWidget,numSampLabel,numMotifLabel;
static Widget filterLabel,filterScroll;
static Widget filter2Label,filter2Scroll, f1Label,f2Label;
static Widget sigNoiseLabel,sigNoiseScroll,noiseSpecToggleW;
static Widget tempLabel,tempScroll;
static Widget sigNoise2Label, allonToggleW;
Pixmap resetPixmap,noresetPixmap;


/********************************************************/
/*  FORWARD DECLARATIONS                                */
/********************************************************/
void            XlibInitialize();
static int      init_application();
void            init_globals();
void            s_error();
void            gotFilename();
static int      askThem();
void            cancel_fileSel();
void            filterFiles();
void            DrawFFTWindow();
void            oned_terp_c();
void            oned_fft_c();
extern void     fourt();
void            DrawFFTplot();
void            doFilter();
void            calcNoiseFactor(float *noiseFctr, int i);




/*********************
 * CALLBACKS         *
 *********************/
void plotWindow_create();
void exit_activate();
void DrawWindow();
void formResize();
void plotClick();
void imageClick();
void infoCancel();
void infoOK();
void new_activate();
void read_activate();
void save_activate();
void createScrolls();
void ampScrollChanged();
void phaseScrollChanged();
void modelChanged();
void toggle_activate();
void scrollChange();
void pageChange();
void reset_activate();


/* The names and addresses of things that Mrm.has to bind.  The names do
 * not have to be in alphabetical order.  */

static MrmRegisterArg reglist[] = {
    {"pageChange", (caddr_t) pageChange},
    {"scrollChange", (caddr_t) scrollChange},
    {"modelChanged", (caddr_t) modelChanged},
    {"phaseScrollChanged", (caddr_t) phaseScrollChanged},
    {"ampScrollChanged", (caddr_t) ampScrollChanged},
    {"createScrolls", (caddr_t) createScrolls},
    {"toggle_activate", (caddr_t) toggle_activate},
    {"reset_activate", (caddr_t) reset_activate},
    {"new_activate", (caddr_t) new_activate},
    {"read_activate", (caddr_t) read_activate},
    {"save_activate", (caddr_t) save_activate},
    {"infoCancel", (caddr_t) infoCancel},
    {"infoOK", (caddr_t) infoOK},
    {"imageClick", (caddr_t) imageClick},
    {"formResize", (caddr_t) formResize},
    {"DrawWindow", (caddr_t) DrawWindow},
    {"exit_activate", (caddr_t) exit_activate},
    {"plotWindow_create", (caddr_t) plotWindow_create}
};

static int reglist_num = (sizeof reglist / sizeof reglist [0]);
