C  ****************************************************************************
C  *                                                                          *
C  *                       *****     EMPRJ.FOR     *****                      *
C  *                                                                          *
C  *    PRODUCE PROJECTIONS FROM 3D MAP IN VIEW ORIENTATIONS AS GIVEN IN      *
C  *     PARAMETER FILE(S).                                                   *
C  *                                                                          *
C  *    THE OUTPUT DATA CAN BE DISPLAYED WITH THE PROGRAM "ROBEM".            *
C  *                                                                          *
C  *    LOGICAL UNITS: 1 = 3D MAP                                             *
C  *                   2 = INPUT PARAMETERS DATA FILE(S)                      *
C  *                   3 = PRJ DATABASE FILE (DEFAULT = EMPRJ.PIF)            *
C  *                                                                          *
C  *    FEBRUARY 1998 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.       *
C  *    MODIFIED  5-FEB-1998 BY TSB (Clean up).                               *
C  *    MODIFIED 25-FEB-1998 BY TSB (Unix compatibility).                     *
C  *    MODIFIED 23-Apr-1998 BY RWA - Added concept of BYTE output PIF        *
C  *       mode.  Toggle to "1" if you want packed box BYTE output instead    *
C  *       of default I4 pft compatible file.                                 *
C  *    MODIFIED 10-AUG-1999 BY TSB (Generate projections in ASU).            *
C  *    MODIFIED 19-AUG-1999 BY TSB (Fix bug in particle FFT origins).        *
C  *    MODIFIED  2-SEP-1999 BY TSB/RWA (Add CTF modulation capability).      *
C  *    LAST MODIFIED 28-DEC-1999 BY TSB (Add random views capability).       *
C  ****************************************************************************
C
      IMPLICIT NONE
C
      include '../include/sizes.inc'

      COMMON BIN, XCEN_MODEL, YCEN_MODEL
      COMMON/CTF_STUFF/ CTFMODE, UNITS, PIXSIZE, VOLTS, FOCUS_MAJ, 
     1 FOCUS_MIN,ANG_MAJ, AMP_FAC, Cs, TEMPFAC, WIENER
C
      INTEGER*4 BIN, NVIEWS, NCOL, NROW, NSEC, NFILES, SYM, UNITS
      INTEGER*4 NCOLBIN, NROWBIN, NSECBIN, outPifMode, CTFMODE
      INTEGER*4 IDIM1, IDIM2, viewCutoff, alloc_stat
      REAL*4    XCEN_MODEL, YCEN_MODEL, DANG, PIXSIZE, VOLTS, WIENER
      REAL*4    FOCUS_MAJ, FOCUS_MIN, ANG_MAJ, AMP_FAC, Cs, TEMPFAC
      real*4    minF,maxF
      CHARACTER(len=filename_len) :: PARAM_FNAMES(pfilesmx)
      CHARACTER(len=filename_len) :: PRJ_FILENAME

      REAL*4, allocatable :: MODEL_TS(:), MODEL_PS(:), MODEL_OS(:)
      real*4, allocatable :: model_dx(:), model_dy(:), MAP3D(:)
      REAL*4, allocatable :: PRJ_AVG(:), PRJ(:), FFT(:)
      INTEGER*4, allocatable :: ITMP(:)

C  ----------------------------------------------------------------------------
C
      PRINT *, 'EMPRJ: Compute PRJ database from 3D MAP'
Ca      PRINT *, '       28-Dec-1999 [TSB]'
      call banner('EMPRJ\0')
      PRINT *
      PRINT *, 'See Web-based documentation for tips on running program'
      PRINT *, 'http://cryoem.ucsd.edu/programs-old.shtm'
      PRINT *

      CALL INFO_EMPRJ (NCOL, NROW, NSEC, NCOLBIN, NROWBIN, NSECBIN,
     $     NFILES, NVIEWS, PARAM_FNAMES, outPifMode, SYM, DANG,
     $     viewCutoff, prj_filename)

      CALL FFT_SETDIM_DEF (NCOLBIN, NROWBIN, IDIM1, IDIM2, 1)

      allocate(model_dx(NVIEWS), model_dy(NVIEWS),
     $     MODEL_TS(NVIEWS), MODEL_PS(NVIEWS), MODEL_OS(NVIEWS),
     $     MAP3D(NCOLBIN*NROWBIN*NSECBIN), PRJ(NCOLBIN*NROWBIN), 
     $     PRJ_AVG(NCOLBIN*NROWBIN), 
     $     ITMP(NCOL), FFT(2*IDIM1*IDIM2), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate memory on EMPRJ'
         call tsbend
      endif

      minF = 99999.0
      maxF = -99999.0
      CALL GETTPO (NFILES, NVIEWS, MODEL_TS, MODEL_PS, MODEL_OS, 
     1     PARAM_FNAMES, SYM, DANG, model_dx, model_dy)

      CALL PRJS (NVIEWS, MODEL_TS, MODEL_PS, MODEL_OS, NCOLBIN, 
     1     NROWBIN, NSECBIN, MAP3D, PRJ, PRJ_AVG, ITMP, 
     2     outPifMode, FFT, IDIM1, IDIM2, minF, maxF, 
     2     viewCutoff, prj_filename, model_dx, model_dy)

      deallocate(model_dx, model_dy, model_ts, model_ps, model_os,
     $     map3d, prj, prj_avg, itmp, fft)

      CALL EXIT
      END


C  ****************************************************************************
C  *                                                                          *
C  *                    *****     SUBROUTINE PRJS     *****                   *
C  *                                                                          *
C  *    PROJECT 3D MAP ALONG THETA,PHI,OMEGA for NVIEWS VIEWS AND SAVE THE    *
C  *     2D PROJECTIONS AND THE AVERAGE PROJECTION IN A DATA FILE.            *
C  *                                                                          *
C  *    FEBRUARY 1998 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.       *
C  *    LAST MODIFIED  2-SEP-1999 BY TSB (Add CTF capabilities).              *
C  ****************************************************************************
C
      SUBROUTINE PRJS (NVIEWS, MODEL_TS, MODEL_PS, MODEL_OS, NCOLBIN, 
     1 NROWBIN, NSECBIN, MAP3D, PRJ, PRJ_AVG, ITMP, outPifMode,
     2 FFT, IDIM1, IDIM2, minF, maxF, viewCutoff, prj_filename,
     2 model_dx, model_dy)

      IMPLICIT NONE

      include '../include/sizes.inc'

      include '../include/pifhead.inc'
      TYPE(PIFHEAD_T)  MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD
      COMMON /PIFHEAD/ MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD

      COMMON BIN, XCEN_MODEL, YCEN_MODEL
C
      INTEGER*4 BIN, NCOLBIN, NROWBIN, NSECBIN, ITMP(NCOLBIN)
      INTEGER*4 NVIEWS, ID, outPifMode, SFAC, MINP, IDIM1, IDIM2
      REAL*4    XCEN_MODEL, YCEN_MODEL, MAP3D(NCOLBIN,NROWBIN,NSECBIN)
      REAL*4    MODEL_TS(NVIEWS), MODEL_PS(NVIEWS), MODEL_OS(NVIEWS)
      REAL*4    model_dx(nviews), model_dy(nviews)
      REAL*4    PRJ(NCOLBIN,NROWBIN)
      REAL*4    PRJ_AVG(NCOLBIN,NROWBIN), FFT(2*IDIM1,IDIM2)
        real*4    img_min,img_max,img_avg,img_std,img_bkg
      REAL*4    minF,maxF,xshift,yshift
      integer*4 viewCutoff
      CHARACTER(len=filename_len) :: PRJ_FILENAME
C  ----------------------------------------------------------------------------
C
      CALL PIF_READ_WMAPI2 (MAPHEAD, MAP3D, NCOLBIN, NROWBIN, NSECBIN)
      CALL PIF_CLOSE (1)

      prj_avg = 0.0 ! F90 array syntax
      WRITE(*,1000)

      DO ID = 1,NVIEWS
        WRITE(*,1010) ID, MODEL_TS(ID), MODEL_PS(ID), MODEL_OS(ID)
C         **************************************************************
C---------*    NOTE USE OF OMEGA = 90.0 TO GET CORRECT ORIENTATIONS    *
C         **************************************************************

C     NOTE - make sure that model_os is set correctly in gettpo
        call map_prj(prj, map3d, ncolbin, nrowbin, nsecbin, 
     $       model_ts(id), model_ps(id), 90.0+model_os(id),
     $       1.0, real(nsecbin))

Ca *******************************************************************
Ca  Is this the place to do the img_shift????
Ca *******************************************************************
        if(model_dx(id) .ne. 0.0) then
           xshift = model_dx(id) - (ncolbin/2) ! formerly (ncolbin/2+1)
           yshift = model_dy(id) - (nrowbin/2) ! formerly (nrowbin/2+1)
           print *, 'Shifting origin - ',xshift,yshift
           call r4_img_stats (prj, ncolbin, nrowbin, img_min, img_max, 
     1        img_avg, img_std, img_bkg)
           call r4_img_shift(prj, ncolbin,nrowbin,xshift,yshift,
     1        img_min,img_max,img_avg,img_std,img_bkg)
        endif
 
        CALL PRJS_STORE (PRJ, NCOLBIN, NROWBIN, ID, PRJ_AVG, NVIEWS, 
     1         ITMP, outPifMode, SFAC, MINP, FFT, IDIM1, IDIM2,
     1         minF, maxF, viewCutoff, prj_filename)
      ENDDO
C
      RETURN
C
 1000      FORMAT('  View # Theta    Phi  Omega',/,28('-'),/)
 1010      FORMAT ('+',I5,F8.2,F7.2,F7.2)
      END


C  ****************************************************************************
C  *                                                                          *
C  *                    *****     SUBROUTINE GETTPO     *****                 *
C  *                                                                          *
C  *    FILL UP THE PARAMETER ARRAYS BY READING T,P,O ORIENTATION DATA FROM   *
C  *     ALL PARAMETER FILES (UNIT = 2).                                      *
C  *                                                                          *
C  *    FEBRUARY 1998 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.       *
C  *    MODIFIED  5-FEB-1998 BY T. BAKER (Clean up).                          *
C  *    LAST MODIFIED 27-DEC-1999 BY TSB (Add random views capability).       *
C  ****************************************************************************
C
      SUBROUTINE GETTPO (NFILES, NVIEWS, MODEL_TS, MODEL_PS, MODEL_OS, 
     1 PARAM_FNAMES, SYM, DANG, model_dx, model_dy)
C      
      IMPLICIT NONE
      include '../include/sizes.inc'
C
      COMMON BIN, XCEN_MODEL, YCEN_MODEL
C
      INTEGER*4 ID, NFILES, NVIEWS, FILE, NN, SYM, BIN
      REAL*4    MODEL_TS(NVIEWS), MODEL_PS(NVIEWS), MODEL_OS(NVIEWS)
      REAL*4    MODEL_dx(NVIEWS), MODEL_dy(NVIEWS)
      REAL*4    DANG, XCEN_MODEL, YCEN_MODEL
      CHARACTER(len=filename_len) :: PARAM_FNAMES(pfilesmx)
C  ----------------------------------------------------------------------------
C
      IF(IABS(SYM).GT.0) GO TO 500
C
      NN = 1
      DO 200 FILE = 1,NFILES
        IF(FILE.GT.1) CLOSE (UNIT=2, STATUS='KEEP')
        OPEN (UNIT=2, FILE=PARAM_FNAMES(FILE), STATUS='OLD', ERR=996,
     1      ACCESS='SEQUENTIAL', FORM='FORMATTED', ACTION='READ')
Ca      1      ACCESS='SEQUENTIAL', FORM='FORMATTED', ACTION='READ', SHARED)
        READ (2,'(A)',ERR=997)                         ! DUMMY READ #1
        READ (2,'(A)',ERR=998)                        ! DUMMY READ #2
  100        READ (2,*,END=200,ERR=999) ID, MODEL_TS(NN), 
     1  MODEL_PS(NN), MODEL_OS(NN), model_dx(nn), model_dy(nn)
        NN = NN + 1
        GO TO 100
  200      ENDDO
      NN = NN -1
       RETURN
C
  500      CALL MODEL_TPS (MODEL_TS, MODEL_PS, DANG, NVIEWS, SYM)
      DO NN = 1,NVIEWS
         model_dx(nn) = 0.0  ! Not doing this
         model_dy(nn) = 0.0
        WRITE(11,1000) NN, MODEL_TS(NN), MODEL_PS(NN), 0.0, 
     1  XCEN_MODEL-1.0, YCEN_MODEL-1.0, 1.0, 1.0, 1.0, 1.0
      ENDDO
      CLOSE (UNIT=11, STATUS='KEEP')
      RETURN
C
  996      PRINT *, '***********************************'
      PRINT *, '* ERROR trying to OPEN PARAM file *'
      CALL TSBEND
  997      PRINT *, '***********************************************'
      PRINT *, '* GETTPO ERROR reading 1st line of PARAM file *'
      CALL TSBEND
  998      PRINT *, '***********************************************'
      PRINT *, '* GETTPO ERROR reading 2nd line of PARAM file *'
      CALL TSBEND
  999      PRINT *, '*********************************************'
      PRINT *, '* GETTPO ERROR reading data from PARAM file *'
      CALL TSBEND
C
 1000      FORMAT (1X,I6,',',4(F8.3,','),F7.3,',',3(F6.3,','),F6.3)
      END



C  ****************************************************************************
C  *                                                            *
C  *                 *****     SUBROUTINE MODEL_TPS     *****                 *
C  *                                                            *
C  *    Calculates THETA and PHI orientation angles within the search window  *
C  *     for computing projections from the model.                            *
C  *                                                                            *
C  *    DPHI = DANG / sin(THE), avoids oversampling near THE=0,180            *
C  *    If THE=0,180 then PHI is undefined, here it is set to 0.0.            *
C  *                                                            *
C  *    AUGUST 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.         *
C  *    MODIFIED 10-AUG-1999 BY TSB (Adapted from DMB's CALC_MOD_TPS).        *
C  *    LAST MODIFIED 27-DEC-1999 BY TSB (Add random views capability).       *
C  ****************************************************************************
C
      SUBROUTINE MODEL_TPS (MODEL_TS, MODEL_PS, DANG, NVIEWS, SYM)
C
      IMPLICIT NONE
C
      INTEGER*4 NTH, NPH, NVIEWS, SYM, NUM, ITHE, IPHI
      INTEGER*4 I, ISEED
      REAL*4    MODEL_TS(NVIEWS), MODEL_PS(NVIEWS), THE1, THE2, PHI1
      REAL*4    DANG, DPHI, THE, PH, FIND_PHIMAX, DUMMY
      real*8    deg_to_rad
C ----------------------------------------------------------------------------
C
      deg_to_rad = acos(-1.0d0)/180.0d0

      IF(SYM.LT.0) GO TO 100
C
      CALL GET_T1T2P1 (SYM, THE1, THE2, PHI1)
      NUM = 0
      THE = THE1
C
      NTH = NINT((THE2-THE1)/DANG) + 1
      DO ITHE = 1,NTH
        IF (THE.EQ.0.0 .OR. THE.EQ.180.0) THEN
           NUM           = NUM + 1
           MODEL_TS(NUM) = THE
           MODEL_PS(NUM) = 0.0
         ELSE
           DPHI = REAL(DANG / SIN(deg_to_rad*THE))
Ca  Misha and Rob - We changed the following line to prevent it from
Ca  iterating past the edge of the ASU.
Ca           NPH  = NINT((FIND_PHIMAX(SYM,THE)-PHI1)/DPHI) + 1
           NPH  = INT((FIND_PHIMAX(SYM,THE)-PHI1)/DPHI) + 1
           PH   = PHI1
           DO IPHI = 1,NPH
             NUM           = NUM + 1
             MODEL_TS(NUM) = THE
             MODEL_PS(NUM) = PH
             if(ph .gt. find_phimax(sym,the)-phi1) then
              print *, 'Over - ', ph,find_phimax(sym,the)-phi1
             endif
             PH            = PH + DPHI
           ENDDO
        ENDIF
        IF(SYM.EQ.532 .AND. THE.EQ.69.09484255211070) THEN
           THE = 69.0 + DANG                      ! DANG unlikely to be < 0.1
         ELSE
           THE = THE + DANG
        ENDIF
      ENDDO                                      !End do ITHE = 1,NTH
      if(num .ne. nviews) then
         print *, 'NVIEWS views not calculated!',num,nviews
      endif
      RETURN
C  **********************************************************************
C  *    HERE IF WANT TO GENERATE RANDOM VIEWS IN THE ASYMMETRIC UNIT    *
C  **********************************************************************
  100      CALL SEED (ISEED)
      DUMMY = 0.0
      DO I = 1,NVIEWS
        MODEL_TS(I) = RAN(ISEED)*360.0
        MODEL_PS(I) = RAN(ISEED)*360.0
        CALL TO_ASYM_UNIT (IABS(SYM), MODEL_TS(I), MODEL_PS(I), DUMMY)
      ENDDO
      RETURN
      END

