#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "3dgraphics.h"

void init_vertices() 
{
  // function name: init_vertices
  // Initialize the vertex coordinates and the composition of the
  // faces (i.e. which verticies define a face) for an icosahedron in
  // the standard cryoEM orientation

  int i;
  float vertices[12][3] = {
    {-0.525731112119133606, 0.0,   0.850650808352039932},
    {0.525731112119133606,  0.0,   0.850650808352039932},
    {-0.525731112119133606, 0.0,  -0.850650808352039932},
    {0.525731112119133606,  0.0,  -0.850650808352039932},
    {0.0,  0.850650808352039932,   0.525731112119133606},
    {0.0,  0.850650808352039932,  -0.525731112119133606},
    {0.0, -0.850650808352039932,   0.525731112119133606},
    {0.0, -0.850650808352039932,  -0.525731112119133606},
    {0.850650808352039932,   0.525731112119133606, 0.0},
    {-0.850650808352039932,  0.525731112119133606, 0.0},
    {0.850650808352039932,  -0.525731112119133606, 0.0},
    {-0.850650808352039932, -0.525731112119133606, 0.0}
  };
  int faces[20][3] = {
    {0, 4, 1},   {4, 8, 1},    {8, 10, 1},   {9, 11, 2},  
    {2, 7, 3},   {7, 10, 3},   {5, 2, 3},    {0,  9, 4},
    {9, 5, 4},   {9,  2, 5},   {0, 1, 6},    {7, 11, 6}, 
    {11, 0, 6},  {4,  5, 8},   {5, 3, 8},    {6,  1, 10},
    {7, 6, 10},  {8, 3, 10},   {7, 2, 11},   {9,  0, 11}
  };

  icosabox.num_points = 12;
  for (i=0;i<icosabox.num_points;i++) {
    icosabox.vertex[i].lx = vertices[i][0];
    icosabox.vertex[i].ly = vertices[i][1];
    icosabox.vertex[i].lz = vertices[i][2];
  }
  
  icosabox.num_faces= 20;
  for (i=0;i<icosabox.num_faces;i++) {
    icosabox.faces[i].x = faces[i][0];
    icosabox.faces[i].y = faces[i][1];
    icosabox.faces[i].z = faces[i][2];
  }
  
} // end function init_vertices


void rotz(float *x,float *y,float zang)
{
  float tx,ty;

  tx=(*x*cos(zang)-*y*sin(zang));
  ty=(*x*sin(zang)+*y*cos(zang));
  *y=ty;
  *x=tx;
}

void roty(float *x,float *z,float yang)
{
  float tx,tz;

  tx=(*x*cos(yang)+*z*sin(yang));
  tz=(*x*-sin(yang)+*z*cos(yang));
  *x=tx;
  *z=tz;
}

void rotx(float *y,float *z,float xang)
{
  float ty,tz;

  ty=(*y*cos(xang)-*z*sin(xang));
  tz=(*y*sin(xang)+*z*cos(xang));
  *y=ty;
  *z=tz;
}

float scncord(float x,float z,float dist)
{
  return(dist*x/(z+100));
}

void Convert3d_to_scrcords()
{
  int i;

  for (i=0;i<icosabox.num_points;i++) {
    icosabox.vertex[i].sx = 
      scncord(icosabox.vertex[i].lx,icosabox.vertex[i].lz,icosdist);
    icosabox.vertex[i].sy = 
      scncord(icosabox.vertex[i].ly,icosabox.vertex[i].lz,icosdist);
  }
}

void print_verticies() {
  int i;

  for (i=0;i<icosabox.num_points;i++) 
    printf ("%f %f %f\n",icosabox.vertex[i].lx,icosabox.vertex[i].ly,icosabox.vertex[i].lz);
  for (i=0;i<icosabox.num_faces;i++) 
    printf ("%d %d %d\n",icosabox.faces[i].x,icosabox.faces[i].y,icosabox.faces[i].z);

}

void scale_verticies(float scale) {
  int i;

  for (i=0;i<icosabox.num_points;i++) {
    icosabox.vertex[i].lx *= scale;
    icosabox.vertex[i].ly *= scale;
/*    icosabox.vertex[i].lz*=scale; */
  }
}
   
void translate_verticies(float translatex, float translatey) {
  int i;

  for (i=0;i<icosabox.num_points;i++) {
    icosabox.vertex[i].sx += translatex;
    icosabox.vertex[i].sy += translatey;
  }
}
