// File appodize.c adapted from softboxing.c
// This file just includes the impose soft border box functionality
// from softboxing.c
// W. Zhang, 1997/10/1
// last modified: wz 04/10/2000

#include "../include/common.h"
#include "../include/libPIF.h"
#include "../include/appodize.h"
#include "robem_def.h"

int calcBackground(BYTE *inBuf, int width, int height, int boxWidth,
		   int fileType, int bkgrndMethod, int boxMethod,
		   int packRadius, int boxRadius, int featherRadius,
		   float recipSizeFactor, int tryColors)
{
  // function name: calcBackground
  // Calculate statistics about the image.  Used by the
  // stuffDisplayBoxBuffer() routine(s) to mask the background of the
  // boxed images.
  
  int i, j, totPnts, xcen, ycen, iMycen, jMxcen, thisRadius, cenRadius;
  double absTmp, back, std, avg, var, sumSq, distnce;
  double cutOffLo, cutOffHi, tmp;
  BYTE b1;
  
  // Init the background value
  back = 0.0;
  if(fileType==PACKEDPORTABLE || fileType==REBOX) { // Already boxed
    cenRadius = thisRadius= packRadius;
    if(boxRadius > 0)
      thisRadius= boxRadius;
  }
  else { // Boxing from scratch
    thisRadius= boxRadius;
    cenRadius = featherRadius;
  }
  
  sumSq = 0.0;
  back = 0.0;
  if(bkgrndMethod==USEAVGPIXEL || fileType==REBOX) { // Average it
    if(fileType==REBOX ||
       ((boxMethod==USECIRCLE && fileType!=EMPORTABLE) &&
	(width==height))) { // Calc for circle boxes
      xcen = ycen = cenRadius;
      totPnts = 0;
      for(i=0; i<height; i++) {
	for(j=0; j<width; j++) {
	  b1= inBuf[(i*boxWidth)+j];
	  iMycen = (i*recipSizeFactor)-ycen;
	  jMxcen = (j*recipSizeFactor)-xcen;
	  distnce = sqrt((double)(iMycen*iMycen + jMxcen*jMxcen)); 
	  if(distnce>=thisRadius && ((distnce-1.0)<= thisRadius)) {
	    // found one on the thisRadius
	    tmp  =  (double)b1;
	    back += tmp;
	    totPnts++;
	    sumSq += (tmp*tmp);
	  }
	}
      }
      
      std     = 0.0;
      avg     = 0.0;
      if(totPnts > 0) { // OK to do
	avg = back/totPnts;
	var = (sumSq/totPnts) - (avg*avg);
	if(var >= 0.0) 
	  std = sqrt(var);
      }
      
      // ****************************************************
      // Throw out data points more than 2*STD from AVG
      // May 9, 2000 RWA
      // We need to trap here that out stdev is great enough to
      // justify this test.  For example with test data we may
      // get a perfect variance (== 0.0) and therefore a std==0.0
      // this throws out all data below and we end up with a background
      // of Zero (0.0). 

      if(std > (avg *.01)) { // StDev > 1 % of the avg pixel
	back    = 0.0;
	totPnts = 0;
	sumSq   = 0.0;
	
	// We need to throw away the "dark" stuff because that is
	// likely where the adjoining particle is contributing to our
	// concept of background

	cutOffLo= avg - (2.0*std);
	cutOffHi= avg + (2.0*std);

	for(i=0; i<height; i++) {
	  for(j=0; j<width; j++) {
	    b1 = inBuf[(i*boxWidth)+j];
	    tmp= (double)b1;
	    iMycen = (i*recipSizeFactor)-ycen;
	    jMxcen = (j*recipSizeFactor)-xcen;
	    distnce = (float)sqrt((double)(iMycen*iMycen + jMxcen*jMxcen)); 
	    if(distnce>=thisRadius && ((distnce-1.0)<= thisRadius)) {
	      // we've found one on the thisRadius
	      if(tmp < avg)
		absTmp = avg + (avg - tmp);
	      else 
		absTmp = tmp;

	      if(absTmp > cutOffLo && absTmp <=cutOffHi) {
		back   += tmp;
		totPnts++;
		sumSq  += tmp*tmp;
	      }
	    }
	  }
	}
	
	std = 0.0;
	avg = 0.0;
	if(totPnts > 0) { // OK to do
	  avg = back/totPnts;
	  var = sumSq/totPnts - avg*avg;
	  if(var >= 0.0) 
	    std = sqrt(var);
	}
      }
      
      back = (int)(avg);
    }
    else {  // get background value from perimeter of rectangle
      for(i=0; i<width; i++) 
	back += (float)inBuf[i] + (float)inBuf[((height-1)*boxWidth)+i];
      for(i=1; i<(height-1); i++)
	back += (float)inBuf[i*boxWidth] + 
	  (float)inBuf[(i*boxWidth)+(height-1)];

      back = back/(2*width + 2*height - 2);
    }
  }
  else if(bkgrndMethod==USERANDPIXEL) { // Pass back a random one
    float factor;
    factor= 1.0 / RAND_MAX;         // Get our fraction of the random number
    back= ((tryColors-7) * (random() * factor)); // Just get me a random number
  }
  
  return((int)back);
			    
} // end function calcBackground



int calcIntBackground(int *inBuf, int width, int height, int boxWidth,
		      int fileType, int bkgrndMethod, int boxMethod,
		      int packRadius, int boxRadius, int featherRadius,
		      float recipSizeFactor, int tryColors)
{
  // function name: calcIntBackground
  // Calculate statistics about the image. Used by the
  // stuffDisplayBoxBuffer() routine(s) to mask the background of the
  // boxed images.

  int i, j, totPnts, xcen, ycen, iMycen, jMxcen, thisRadius, cenRadius;
  float b1;
  double absTmp, back, std, avg, var, sumSq, distnce;
  double cutOffLo, cutOffHi, tmp;

  back = 0.0;
  if(fileType==PACKEDPORTABLE || fileType==REBOX) { // Already boxed
    cenRadius = thisRadius= packRadius;
    if(boxRadius > 0)
      thisRadius= boxRadius;
  }
  else { // Boxing from scratch
    thisRadius= boxRadius;
    cenRadius = featherRadius;
  }
  
  sumSq = 0.0;
  back = 0.0;
  if(bkgrndMethod==USEAVGPIXEL || fileType==REBOX) { // Average it
    if(fileType==REBOX ||
       ((boxMethod==USECIRCLE && fileType!=EMPORTABLE) &&
	(width==height))) { // Calc for circle boxes
      xcen = ycen = cenRadius;
      totPnts = 0;
      for(i=0;i<height;i++) {
	for(j=0;j<width;j++) {
	  b1= (float)inBuf[(i*boxWidth)+j];
	  iMycen = i*recipSizeFactor - ycen;
	  jMxcen = j*recipSizeFactor - xcen;
	  distnce = (float)sqrt((double)(iMycen*iMycen + jMxcen*jMxcen)); 
	  if(distnce>=thisRadius && ((distnce-1.0)<= thisRadius)) {
	    // we've found one on the thisRadius
	    tmp  =  b1;
	    back += tmp;
	    totPnts++;
	    sumSq += tmp*tmp;
	  }
	}
      }
      
      std     = 0.0;
      avg     = 0.0;
      if(totPnts > 0) { // OK to do
	avg     = back/totPnts;
	var     = sumSq/totPnts - avg*avg;
	if(var >= 0.0) 
	  std = sqrt(var);
      }
      
      // ***************************************************
      // Throw out data points more than 2*STD from AVG
      // May 9, 2000 RWA
      // We need to trap here that out stdev is great enough to
      // justify this test.  For example with test data we may
      // get a perfect variance (== 0.0) and therefore a std==0.0
      // this throws out all data below and we end up with a background
      // of Zero (0.0). 
      
      if(std > (avg *.01)) { // StDev > 1% of the avg pixel
	back   = 0.0;
	totPnts = 0;
	sumSq  = 0.0;
	
	// We need to throw away the "dark" stuff because that is
	// likely where the adjoining particle is contributing to our
	// concept of background

	cutOffLo= avg - (2.0*std);
	cutOffHi= avg + (2.0*std);

	for(i=0;i<height;i++) {
	  for(j=0;j<width;j++) {
	    b1 = (float)inBuf[(i*boxWidth)+j];
	    tmp= (float)b1;
	    iMycen = (i*recipSizeFactor)-ycen;
	    jMxcen = (j*recipSizeFactor)-xcen;
	    distnce = (float)sqrt((double)(iMycen*iMycen + jMxcen*jMxcen)); 
	    if(distnce>=thisRadius && ((distnce-1.0)<= thisRadius)) {
	      // we've found one on the thisRadius
	      if(tmp < avg)
		absTmp = avg + (avg - (float)tmp);
	      else absTmp = (float)tmp;

	      if(absTmp > cutOffLo && absTmp <=cutOffHi) {
		back   += tmp;
		totPnts++;
		sumSq  += (tmp*tmp);
	      }
	    }
	  }
	}

	std = 0.0;
	avg = 0.0;
	if(totPnts > 0) { // OK to do
	  avg = back/totPnts;
	  var = (sumSq/totPnts) - (avg*avg);
	  if(var >= 0.0) 
	    std = sqrt(var);
	}
      }
	  
      back = (int)(avg); 
    }
    else { // get background value from perimeter of rectangle
      for(i=0; i<width;i++)
	back += inBuf[i]          + inBuf[((height-1)*boxWidth)+i];
      for(i=1; i<(height-1);i++)
	back += inBuf[i*boxWidth] + inBuf[(i*boxWidth)+(height-1)];
      
      back  = back/(2*width + 2*height -2);
    }
  }
  else if(bkgrndMethod==USERANDPIXEL) { // Pass back a random one
    float factor;
    factor= 1.0 / RAND_MAX; // Get our fraction of the random number
    back= ((tryColors-7) * (random() * factor));  // Just get a random number
  }
  
  return((int)back);
			    
} // end function calcIntBackground



float calcFloatBackground(float *inBuf, int width, int height, int boxWidth,
			  int fileType, int bkgrndMethod, int boxMethod,
			  int packRadius, int boxRadius, int featherRadius,
			  float recipSizeFactor, int tryColors)
{
  // function name: calcFloatBackground
  // Calculate statistics about the image. Used by the
  // stuffDisplayBoxBuffer() routine(s) to mask the background of the
  // boxed images.

  int i, j, totPnts, xcen, ycen, iMycen, jMxcen, thisRadius, cenRadius;
  float b1, back;
  double absTmp, std, avg, var, sumSq, distnce;
  double cutOffLo, cutOffHi, tmp;

  back = 0.0;
  if(fileType==PACKEDPORTABLE || fileType==REBOX) { // Already boxed
    cenRadius = thisRadius= packRadius;
    if(boxRadius > 0)
      thisRadius= boxRadius;
  }
  else { // Boxing from scratch
    thisRadius= boxRadius;
    cenRadius = featherRadius;
  }

  sumSq = 0.0;
  back = 0.0;
  if(bkgrndMethod==USEAVGPIXEL || fileType==REBOX) { // Average it
    if(fileType==REBOX ||
       ((boxMethod==USECIRCLE && fileType!=EMPORTABLE) &&
	(width==height))) { // Calc for circle boxes
      xcen = ycen = cenRadius;
      totPnts = 0;
      for(i=0; i<height; i++) {
	for(j=0; j<width; j++) {
	  b1= inBuf[(i*boxWidth)+j];
	  iMycen = (i*recipSizeFactor)-ycen;
	  jMxcen = (j*recipSizeFactor)-xcen;
	  distnce = sqrt((double)(iMycen*iMycen + jMxcen*jMxcen)); 
	  if(distnce>=thisRadius && ((distnce-1.0)<= thisRadius)) {
	    // we've found one on the thisRadius
	    tmp  =  b1;
	    back += tmp;
	    totPnts++;
	    sumSq += tmp*tmp;
	  }
	}
      }

      std     = 0.0;
      avg     = 0.0;
      if(totPnts > 0) { // OK to do
	avg = back/(float)totPnts;
	var = (sumSq/(float)totPnts) - (avg*avg);
	if(var >= 0.0) 
	  std = sqrt(var);
      }
	  
      // ****************************************************
      // Throw out data points more than 2*STD from AVG
      // May 9, 2000 RWA
      // We need to trap here that out stdev is great enough to
      // justify this test.  For example with test data we may
      // get a perfect variance (== 0.0) and therefore a std==0.0
      // this throws out all data below and we end up with a background
      // of Zero (0.0). 

      if(std > (avg *.01)) { // StDev > 1% avg pixel
	back   = 0.0;
	totPnts = 0;
	sumSq  = 0.0;
	
	// We need to throw away the "dark" stuff because that is
	// likely where the adjoining particle is contributing to our
	// concept of background 

	cutOffLo= avg - (2.0*std);
	cutOffHi= avg + (2.0*std);

	for(i=0;i<height;i++) {
	  for(j=0;j<width;j++) {
	    b1 = inBuf[((i*boxWidth)+j)];
	    tmp= b1;
	    iMycen = (i*recipSizeFactor)-ycen;
	    jMxcen = (j*recipSizeFactor)-xcen;
	    distnce = sqrt((double)(iMycen*iMycen + jMxcen*jMxcen)); 
	    if(distnce>=thisRadius && ((distnce-1.0)<= thisRadius)) {
	      // we've found one on the thisRadius
	      if(tmp < avg)
		absTmp = avg + (avg - tmp);
	      else 
		absTmp = tmp;

	      if(absTmp > cutOffLo && absTmp <=cutOffHi) {
		back   += tmp;
		totPnts++;
		sumSq  += tmp*tmp;
	      }
	    }
	  }
	}

	std = 0.0;
	avg = 0.0;
	if(totPnts > 0) { // OK to do
	  avg = back/(float)totPnts;
	  var = sumSq/(float)totPnts - avg*avg;
	  if(var >= 0.0) 
	    std = sqrt(var);
	}
      }
      
      back = (float)(avg); 
    }
    else { // get background value from perimeter of rectangle
      for(i=0; i<width; i++)
	back += inBuf[i]          + inBuf[((height-1)*boxWidth)+i];
      for(i=0; i<height; i++)
	back += inBuf[i*boxWidth] + inBuf[(i*boxWidth)+(width-1)];
      
      back    = back/(2.0*width + 2.0*height - 2.0);
    }
  }
  else if(bkgrndMethod==USERANDPIXEL) { // Pass back a random one
    float factor;
    factor= 1.0 / RAND_MAX; // Get our fraction of the random number
    back= (tryColors-7.0) * (random() * factor); // Get a random number   
  }
  
  return((float)back);
			    
} // end function calcFloatBackground



void appodize_it(float *editBox_int_buffer, int outWidth, int outHeight, 
		 int box_width, int fileType, int bkgrndMethod, int boxMethod,
		 int packRadius, int boxRadius, int featherRadius,
		 float recipSizeFactor, float sizeFactor, int tryColors,
		 int featherMode, float *backChar, int thisBoxRadius,
		 int forceAvgBkg, float img_avg)
{
  // function name: appodize_it
  
  int k, l;
  
  // DO not do this here - we need the bkg from the outer
  // ice calculated in the rtn that calls us here.
  // Calc the background
  //
  // *backChar= (float)calcFloatBackground(editBox_int_buffer, 
  //				outWidth, outHeight, 
  //				box_width, fileType,
  //				bkgrndMethod, boxMethod,
  //				packRadius, boxRadius,
  //				featherRadius, recipSizeFactor,
  //				tryColors);

  if(forceAvgBkg)
    *backChar = img_avg;
  
  if(featherMode != OLD && (featherRadius!=boxRadius)) {
    // We want to do some kind of appodizing
    float *editBox_int_buffer_tmp1,*editBox_int_buffer_tmp2;
    editBox_int_buffer_tmp1 = (float *)malloc(outWidth*outHeight
					      *sizeof(float));
    editBox_int_buffer_tmp2 = (float *)malloc(outWidth*outHeight
					      *sizeof(float));

    for(k=0;k<outHeight;k++) { // Do the y direction
      int yOut,yOut2;
      // Where is the center?
      yOut  = k * box_width;
      yOut2 = k * outWidth;
      for(l=0; l<outWidth; l++) { // Stuff the outbuffer
	editBox_int_buffer_tmp1[yOut2+l]=
	  editBox_int_buffer[yOut+l];  
      }
    }
    
    Impose_Softbox(editBox_int_buffer_tmp1, 
		   editBox_int_buffer_tmp2, 
		   outWidth, outHeight, 
		   *backChar, featherMode, 
		   boxRadius, (featherRadius - boxRadius) );

    for(k=0; k<outHeight; k++) { // Do the y direction
      int yOut,yOut2;
      // Where is the center?
      yOut  = k * box_width;
      yOut2 = k * outWidth;
      for(l=0; l<outWidth; l++) { // Stuff the outbuffer
	editBox_int_buffer[yOut+l]=  
	  editBox_int_buffer_tmp2[yOut2+l];  
      }
    }
    free(editBox_int_buffer_tmp1);
    free(editBox_int_buffer_tmp2);
  }
  else { // DO it the OLD fashioned method of stuffing the corners
    
    // Draw the background  for all > radius in box
    for(k=0; k<outHeight; k++) { // Do the y direction
      int yOut, yCent, xCent;
      float ySq, xSq, dist;
      
      // Where is the center?
      yOut = k * box_width;
      yCent = thisBoxRadius*sizeFactor;
      xCent = thisBoxRadius*sizeFactor;
      xCent = (outWidth/2) * sizeFactor;
      yCent = (outHeight/2) * sizeFactor;
      
      // Stuff all > radius in background character
      for(l=0;l<outWidth;l++) { // Stuff the outbuffer
	
	ySq = (k - yCent) * (k - yCent);
	xSq = (l - xCent) * (l - xCent);
	
	dist = sqrt(ySq+xSq);
	
	if(bkgrndMethod==USERANDPIXEL)
	  *backChar= (float)calcFloatBackground(editBox_int_buffer, 
						outWidth, outHeight,
						box_width, fileType,
						bkgrndMethod, boxMethod,
						packRadius, boxRadius,
						featherRadius, recipSizeFactor,
						tryColors);
	if(dist > ((thisBoxRadius * sizeFactor)+1.0))
	  editBox_int_buffer[yOut+l]=  *backChar;
      }
    }
  }
  
} // end function appodize_it



float DumpFunc(int type, float dist, int BorderWidth)
{
  // function name: DumpFunc

  float ReturnValue;
  float temp1, temp2;
  
  // temp1 = ((float)BorderWidth - dist) /  (float) BorderWidth;
  // here at radius = r2, the dumping function =exp(-2.0) that is 0.14
  temp2 =  dist / BorderWidth;
  temp1 = temp2*(exp(-2.0)-1.0)+1.0;
  if (type == LINEAR)	
    if (temp1<0.0) temp1=0.0;	
  ReturnValue = temp1;
  if (type == EXPONENTIAL)
    ReturnValue = exp(-2.0 * temp2 );
  if (type == GAUSSIAN) 
    ReturnValue =  exp(-2.0 * temp2 * temp2);
  return ReturnValue;
  
} // end function DumpFunc



void Impose_Softbox(float *inBuffer, float *outBuffer, 
		    int ncol, int nrow, 
		    float img_bkg, int type, 
		    int beginRadius, int BorderWidth)
{
  //  function name: Impose_Softbox

  float   *p;
  int i, j, k1, xc, yc, radius2, Rx, Ry;
  double distance, PixelValue, dist1; 
  
  // (Xc, Yc) is the center of the image frame
  xc = ncol/2  ; 
  yc = nrow/2  ; 
  radius2 = beginRadius;
  
  // Masking data for each box type
  
  for (j=0; j<nrow ; ++j) {
    for (i=0; i<ncol; ++i) {
      Rx = i  - xc;
      Ry = j  - yc;
      distance = sqrt((float)(Rx*Rx + Ry*Ry));
      k1 = j * nrow  + i;
      p = outBuffer + k1;
      
      dist1 = distance - radius2;
      
      PixelValue = *(inBuffer + k1);
      
      if (dist1 <= -0.0 || BorderWidth==0) 	
	*p = PixelValue; 
      else
	*p = (PixelValue - img_bkg) * 
	  DumpFunc(type, dist1, BorderWidth) + img_bkg;
      
    }
  }
  
} // end function Impose_Softbox
