#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <Xm/Xm.h>
#include <Xm/GadgetP.h>
#include <Xm/PrimitiveP.h>
#include <Xm/MessageB.h>
#include <Xm/ScrollBar.h>
#include <Xm/ToggleB.h>
#include <Xm/Label.h>
#include <Xm/DialogS.h>
#include <Xm/Text.h>
#include <Xm/List.h>
#include <X11/cursorfont.h>
#include <Xm/FileSB.h>

#include "../include/common.h"
#include "../include/libPIF.h"
#include "../include/libUtil.h"
#include "../include/appodize.h"
#include "init_array.h"
#include "stats.h"
#include "lists.h"
#include "robem_def.h"
#include "robem.h"

char * emma_autobox(float * inCore_buffer, int* globAccum, box_t* box, int* compress_factor,
                          int* numBoxes, int thisRadius, int emptyBox, int nThins, int nDilates,
                          int file_height, int file_width, int fileType, 
	                      int bkgrndMethod, int boxMethod, int packRadius, 
	                      int boxRadius, int featherRadius, float recipSizeFactor, 
	                      int tryColors)
{
    
// fixed parameters
    float prag = 2.5;       /* Cross-point difference, changed from 1.5*/
    int dataRange = 128;    /* histogram range */
    float scaleRange = (float)(dataRange*2 - 1);
    
    
    int abh, abw, aba; // autoBox height, width and area
    int radi, tbSize;  // radius, total box size
    int cl, totSum1; 
    int i, j, k, yOut;
    int kj, j1n, i1, j1, avgx, avgy; 
    int ii, jj, ti, tj, clusterSize, sp;
    int startJ, endJ, startI, endI;
    int averageBorder;
    int *empty_ref_buffer,*empty_full_buffer;
    int *hist_p;
    unsigned short level;
    float *ab_buf, *prob_p, *sum_p;
    float averageArea, sum, avgEnhanceFactor, sum1, sum1_raw;
    float  *ab_buf_raw, *ab_buf_bin, *Accum, *auxAccum;
    float avgVal, minAvg, maxAvg, corr;
    float clusterSizeUplimit, clusterSizeDownlimit; 
    float img_bkg, minF, maxF, img_avg, img_std, scaleF;
    int *stack_x, *cluster_x, *stack_y, *cluster_y;
    int deltracker;

    char* error_msg = malloc(256);
 
    // **** Image compressed to particle  10 < radius < 20 pixels
    //      data are binned to get this compressed radius
    *compress_factor = (int) (thisRadius/20 + 1 );
    if (*compress_factor<1) *compress_factor = 1;
    
    radi = (thisRadius/(*compress_factor) + 0.5);
    tbSize = (radi*2)+1;
    abh = file_height/(*compress_factor);
    abw = file_width/(*compress_factor);
    aba = abh*abw;
    
//GCA: this part found as deleted in the original code.
//     Routine needs testing before being put back.
//    if(emptyBox) { // empty particles
    if(0) { // empty particles
      float dist, xSq, ySq;
      int maxDist, minDist, tbSizeHalf;
      
      // Build reference buffer
      empty_full_buffer = (int *)malloc(sizeof(int)*tbSize*tbSize);
      empty_ref_buffer  = (int *)malloc(sizeof(int)*tbSize*tbSize);
      tbSizeHalf = tbSize * 0.5;
      maxDist = radi*radi;
      minDist = maxDist * 0.60;
      
      for (i=0; i<tbSize; i++) {
	    ySq = (i-tbSizeHalf)*(i-tbSizeHalf);
	    for (j=0; j<tbSize; j++) {
	      xSq = (j-tbSizeHalf)*(j-tbSizeHalf);
	      dist = xSq + ySq;
	  
	  // Just give me a 1 or 0
	      if(dist > minDist && dist <= maxDist)
	        empty_ref_buffer[i*tbSize + j] = 1;
	      else
	        empty_ref_buffer[i*tbSize + j] = 0;

	      empty_full_buffer[i*tbSize + j] = (dist <= maxDist) ? 1 : 0;
	  
	    }
      }  
    }
    
    
    // **** allocate space for image buffers
    // ab_buf_raw: normalized binned image, normalized between 0 and half the dataRange
    // ab_buf_bin: binned image not normalized
    // ab_buf:     binned, circularly averaged and normalized image
    if((ab_buf_bin = (float *)malloc(aba*sizeof(float)))==(float *)NULL) {
	  strcpy(error_msg,"Err alloc bin image");
	  return(error_msg);
    }
    
    // **** allocate space for ab_buf_raw and ab_buf buffers
    if((ab_buf_raw = (float *)malloc(aba*sizeof(float)))==(float *)NULL) {
      free(ab_buf_bin);
	  strcpy(error_msg,"Err alloc ptle image");
	  return(error_msg);
    }
    
    if((ab_buf = (float *)calloc(aba,sizeof(float))) == (float *)NULL) {
      free(ab_buf_raw);
      free(ab_buf_bin);
	  strcpy(error_msg,"Err alloc ptle image");
	  return(error_msg);
    }
    
    // scale the input data
    calcStats(inCore_buffer, &img_bkg, &minF, &maxF, 
	      &img_avg, &img_std,
	      file_width, file_height, file_width, fileType, bkgrndMethod, boxMethod,
	      packRadius, boxRadius, featherRadius,
		  recipSizeFactor, tryColors);

    // calculate thresholds to determine outliers
    // pixels with values beyond 10 times the standard deviation will be truncated
    // in the buffer used to pick particles
    int thrfact = 10;  //threshold factor
    if (minF<img_avg-thrfact*img_std || maxF>img_avg+thrfact*img_std) {
       minF = MAX(minF,img_avg-thrfact*img_std);
       maxF = MIN(maxF,img_avg+thrfact*img_std);
    }
    
    // **** Load in autobox raw image
    float tmpf;
    int l, m;
    float newminF = MAXINT;
    float newmaxF = -MAXINT;
    for (j=0; j<abh; j++) {
      yOut = j*abw;
      for (k=0; k<abw; k++) {
         tmpf = 0.;
         for (l=0; l < (*compress_factor); l++)
            for (m=0; m < (*compress_factor); m++)
                tmpf+= MAX(MIN(inCore_buffer[(j*(*compress_factor)+l)*file_width+(k*(*compress_factor)+m)],maxF),minF);
                tmpf/=(*compress_factor)*(*compress_factor);
         ab_buf_bin[yOut+k] = tmpf;
         newminF=MIN(newminF,tmpf);
         newmaxF=MAX(newmaxF,tmpf);
      }
    }
// scale the raw buffer 
    scaleF = scaleRange/(newmaxF-newminF);
    for (j=0; j<aba; j++)
       ab_buf_raw[j] = 0.5 + ((ab_buf_bin[j]-newminF)*scaleF)/2.0;
    
    
    // *********** Histogram equalization
    if((prob_p=(float*)malloc(dataRange*sizeof(float)))==(float*)NULL ||
       (sum_p=(float*)malloc(dataRange*sizeof(float)))==(float*)NULL  ||
       (hist_p = (int *)malloc(dataRange*sizeof(int))) == (int *)NULL) {
	  strcpy(error_msg,"Err alloc buffers for Hist Equal.");
	  return(error_msg);
    }
    
    // initialize histogram
    init_intarr(hist_p, dataRange, 0);
    // initialize probability function
    init_floatarr(prob_p, dataRange, 0.0);
    // initialize cumulative probability
    init_floatarr(sum_p, dataRange, 0.0);
    
    for (k=0; k<abh*abw; k++)
      hist_p[(int)ab_buf_raw[k]]++;
    
    for (i=0; i<dataRange; i++)
      prob_p[i] = (float)hist_p[i]/(float)aba;
    
    for (i=0; i<dataRange; i++)
      for (j=0; j<=i; j++)
	    sum_p[i] += prob_p[j];
    
    // assign histogram level to each pixel in the raw buffer
    for (k=0; k<abh; k++) {
        kj = k*abw;
        for (j=0; j<abw; j++) {
	        level = (unsigned short)(0.5+(dataRange - 1)*
				 sum_p[(int)ab_buf_raw[kj+j]]);
	        if(level >= dataRange) {
	            printf("DING! Keep this test\n");
	            level = dataRange - 1;
	        }
	        ab_buf_raw[kj+j] = level;
        }
    }
    
    free(hist_p);
    free(prob_p); 
    free(sum_p);
    
    // Design and apply circular average to raw buffer.
    // The filter is designed in order to have at least three
    // independent measures with the size of a particle
    averageBorder = radi/3;
    
// circular filter: design
    float *avgkern;
    int avgsize = 2*averageBorder+1;
    if((avgkern=(float *)malloc(avgsize*avgsize*sizeof(float)))==(float *)NULL) {
	  strcpy(error_msg,"Err alloc avg kernel filters");
	  return(error_msg);
    }

    averageArea = 0;
    minAvg = maxAvg = 0;
    
    for (j=0; j<2*averageBorder+1; j++){
       for (k=0; k<2*averageBorder+1; k++){
          j1n = j*(2*averageBorder+1)+k;
          if ((k-averageBorder)*(k-averageBorder)+(j-averageBorder)*(j-averageBorder) <= averageBorder*averageBorder) {
              avgkern[j1n] = 1.;
              averageArea++;
          } else
              avgkern[j1n] = 0.;
       }     
    }

// circular filter: apply

    for (j=averageBorder; j<abh-averageBorder; j++) {
        for (i=averageBorder; i<abw-averageBorder; i++) {
            j1n = j*abw + i;
            sum = 0.;
            for (jj=j-averageBorder; jj<=j+averageBorder; jj++) {
                for (ii=i-averageBorder; ii<=i+averageBorder; ii++) {
                    j1 = jj*abw+ii;
                    i1 = (jj-j+averageBorder)*(2*averageBorder+1)+(ii-i+averageBorder);
                    sum += avgkern[i1]*ab_buf_raw[j1];
                }
            }
            avgVal = ab_buf[j1n] = sum/averageArea;
            if(maxAvg < avgVal) maxAvg = avgVal;
        }
    }

    free(avgkern);
    
    
    // ***** Enhance the contrast of averaged image
    avgEnhanceFactor = (dataRange-1.0)/(maxAvg - minAvg);  
    
    for (j=0; j<abh; j++) {
        yOut = j*abw;
        for (k=0; k<abw; k++)
	        ab_buf[yOut+k] = 
	          (int) (0.5 + avgEnhanceFactor*ab_buf[yOut+k]);
    }
    
    // ***** Local background substraction by cross-point method

    // initialize accumulation buffers    
    if((Accum=(float *)malloc(aba*sizeof(float)))==(float *)NULL) {
      free(ab_buf);
      free(ab_buf_raw);
      free(ab_buf_bin);
	  strcpy(error_msg,"Err alloc buffers for Accum");
	  return(error_msg);
    }
    
    if((auxAccum=(float *)malloc(aba*sizeof(float)))==(float *)NULL) {
      free(ab_buf);
      free(ab_buf_raw);
      free(ab_buf_bin);
      free(Accum);
      strcpy(error_msg,"Err alloc buffers for auxAccum");
      return(error_msg);
    }
    
    *numBoxes = 0;
    init_floatarr(Accum, abh*abw, 0.0);
    
    // Cross-Point Method - Pass 1
    // scan left to right, top to bottom
    
    for (j=radi+1; j<abh-radi-1; j++) {
       for (i=radi+1; i<abw-radi-1; i++) {
	      if(ab_buf[j*abw+i] - ab_buf[j*abw+i+radi+1] >= prag) {
	         if(ab_buf[j*abw+i+radi+1] - ab_buf[(j+radi+1)*abw+i+radi+1] >= prag) {
	            Accum[(j+radi+1)*abw+i+radi+1] = 1;
	         } else
	         if(ab_buf[j*abw+i+radi+1] - ab_buf[(j+radi+1)*abw+i+radi+1] <= -prag) {
	            Accum[j*abw+i+radi+1] = 1;
	         } else
	         if(ab_buf[j*abw+i+radi+1] - ab_buf[(j-radi-1)*abw+i+radi+1] >= prag) {
		        Accum[(j-radi-1)*abw+i+radi+1] = 1;
	         } else
		     if(ab_buf[j*abw+i+radi+1] - ab_buf[(j-radi-1)*abw+i+radi+1] <= -prag) {
		        Accum[j*abw+i+radi+1] = 1;
		     }
	      } else
	      if(ab_buf[j*abw+i] - ab_buf[j*abw+i+radi+1] <= -prag) {
	         if(ab_buf[j*abw+i] - ab_buf[(j+radi+1)*abw+i] >= prag) {
	            Accum[(j+radi+1)*abw+i] = 1;
	         } else 
	         if(ab_buf[j*abw+i] - ab_buf[(j+radi+1)*abw+i] <= -prag) {
		        Accum[j*abw+i] = 1;
	         } else
		     if(ab_buf[j*abw+i] - ab_buf[(j-radi-1)*abw+i] >= prag) {
		        Accum[(j-radi-1)*abw+i] = 1;
		     } else
		     if(ab_buf[j*abw+i] - ab_buf[(j-radi-1)*abw+i] <= -prag) {
		        Accum[j*abw+i] = 1;
		     }
	      }
       }  //end for
    } // end for
    
    // Cross-Point Method - Pass 2
    // scan top to bottom, left to right
    
    for (i=radi+1; i<abw-radi-1; i++) {
       for (j=radi+1; j<abh-radi-1; j++) {
	      if(ab_buf[j*abw+i] - ab_buf[(j+radi+1)*abw+i] >= prag) {
	         if(ab_buf[(j+radi+1)*abw+i] - ab_buf[(j+radi+1)*abw+i+radi+1] >= prag) {
	            Accum[(j+radi+1)*abw+i+radi+1] = 1;
	         } else
	         if(ab_buf[(j+radi+1)*abw+i] - ab_buf[(j+radi+1)*abw+i+radi+1] <= -prag) {
	            Accum[(j+radi+1)*abw+i] = 1;
	         } else
	         if(ab_buf[(j+radi+1)*abw+i] - ab_buf[(j+radi+1)*abw+i-radi-1] >= prag) {
		        Accum[(j+radi+1)*abw+i-radi-1] = 1;
	         } else
		     if(ab_buf[(j+radi+1)*abw+i] - ab_buf[(j+radi+1)*abw+i-radi-1] <= -prag) {
		        Accum[(j+radi+1)*abw+i] = 1;
		     }
	      } else
	      if(ab_buf[j*abw+i] - ab_buf[(j+radi+1)*abw+i] <= -prag) {
	         if(ab_buf[j*abw+i] - ab_buf[j*abw+i+radi+1] >= prag) {
	            Accum[j*abw+i+radi+1] = 1;
	         } else
	         if(ab_buf[j*abw+i] - ab_buf[j*abw+i+radi+1] <= -prag) {
		        Accum[j*abw+i] = 1;
	         } else
		     if(ab_buf[j*abw+i] - ab_buf[j*abw+i-radi-1] >= prag) {
		        Accum[j*abw+i-radi-1] = 1;
		     } else
		     if(ab_buf[j*abw+i] - ab_buf[j*abw+i-radi-1] <= -prag) {
		        Accum[j*abw+i] = 1;
		     }
	      }
       }
    }
    
    // Do thinning first
    
    int nt = nThins;
    // number of thinnings operations based on radius value
    // (negative values can only be given by command line)
    if (nt <0) nt = radi/5;
    for (k=0; k<nt; k++) { // Thinning layers
       for (j=1; j<abh-1; j++) {
	      for (i=1; i<abw-1; i++) {
	         auxAccum[j*abw+i] = Accum[j*abw+i];
	         if(Accum[j*abw+i]) {
	            if(Accum[j*abw+i-1]    + Accum[j*abw+i+1]     +
	              Accum[(j-1)*abw+i]   + Accum[(j+1)*abw+i]   +
	              Accum[(j-1)*abw+i-1] + Accum[(j-1)*abw+i+1] +
	              Accum[(j+1)*abw+i-1] + Accum[(j+1)*abw+i+1] < 8)
	               auxAccum[j*abw+i] = 0;
	         }
	      }
       }
      
      // Take the eroded data as gold std       
       for (j=1; j<abh-1; j++) 
	      for (i=1; i<abw-1; i++)
	         Accum[j*abw+i] = auxAccum[j*abw+i];
    }
    
    // Do dilation after
    
    int nd = nDilates;
    // number of dilations operations based on radius value
    // (negative values can only be given by command line)
    if (nd <0) nd = radi/5;
    for (k=0; k<nd; k++) { // Dilation layers
       for (j=1; j<abh-1; j++) {
	      for (i=1; i<abw-1; i++) {
	         auxAccum[j*abw+i] = Accum[j*abw+i];
	         if(!Accum[j*abw+i]) {
	            if(Accum[j*abw+i-1]    + Accum[j*abw+i+1]     +
	              Accum[(j-1)*abw+i]   + Accum[(j+1)*abw+i]   +
	              Accum[(j-1)*abw+i-1] + Accum[(j-1)*abw+i+1] +
	              Accum[(j+1)*abw+i-1] + Accum[(j+1)*abw+i+1] > 0)
	               auxAccum[j*abw+i] = 1;
	         }
          }
       }
      
       // Now take the dilated data as our gold standard
       for (j=1; j<abh-1; j++)
	      for (i=1; i<abw-1; i++)
	         Accum[j*abw+i] = auxAccum[j*abw+i];
      
    } // Dilation layers
    

//GCA: this part found as deleted in the original code.
//     Routine needs testing before being put back.
//    if(emptyBox) { // empty particles
    if(0) { // empty particles
      int m, n, totHits;
      for (j=1; j<abh-tbSize; j++) {
	     for(i=1; i<abw-tbSize; i++) {
	        totHits = 0;
	  
	        for (m=0; m<tbSize; m++)
	           for(n=0;n<tbSize;n++)
	              if(empty_ref_buffer[m*tbSize + n] == Accum[(j+m)*abw + i + n])
		             totHits++;
	  
	        if(totHits > ((tbSize*tbSize)*0.75)) { // Fill it up
//               sprintf(outStr,"Found empty particle at x[%d]y[%d]\n",i,j);
//               TextSet(timeLeftBoxW, outStr, TRUE);

	           for (m=0; m<tbSize; m++)
	              for (n=0; n<tbSize; n++)
		             if(empty_full_buffer[(m*tbSize)+n]==1) 
		                auxAccum[(j+m)*abw + i + n] = 2;

	           // Advance beyond 
	           i += tbSize;
	        } else { // Just do normal
	           if(auxAccum[j*abw+i] < 2)
	              auxAccum[j*abw+i] = Accum[j*abw+i];
	        }
	     }
      }
      
      // Now take the model enhanced data as our gold standard
      for (j=1; j<abh-1; j++) {
	     for (i=1; i<abw-1; i++)
	        if(auxAccum[j*abw+i] == 2)
	           Accum[j*abw+i] = 1;
	        else
	           Accum[j*abw+i] = auxAccum[j*abw+i];
      }
      
      free(empty_ref_buffer);
      free(empty_full_buffer);
    } // Do for empties
    
    free(auxAccum);
    
    // Store the clusters in the global buffer for display purposes
    for (cl=0; cl<aba; cl++)
       globAccum[cl] = Accum[cl];
    
    // ******** Start finding particle by Stack algorithm
    if((cluster_x = (int *)malloc(aba*sizeof(int))) == (int *)NULL ||
       (cluster_y = (int *)malloc(aba*sizeof(int))) == (int *)NULL)
    {
       free(Accum);
       free(ab_buf);
       free(ab_buf_bin);
       free(ab_buf_raw);
	   strcpy(error_msg,"Err alloc buffers for Clustering");
	   return(error_msg);
    }
    
    if((stack_x = (int *)malloc(aba*sizeof(int))) == (int *)NULL ||
       (stack_y = (int *)malloc(aba*sizeof(int))) == (int *)NULL) {
	   free(cluster_x);
	   free(cluster_y);
	   free(Accum);
       free(ab_buf);
       free(ab_buf_bin);
       free(ab_buf_raw);
	   strcpy(error_msg,"Err alloc buffers for Clustering");
	   return(error_msg);
    }

    // Cluster size analysis
    // upper limit: twice the circle area less twice the perimeter
    clusterSizeUplimit   = 2.0*3.14159*radi*radi - 4*3.14159*radi;
    // lower limit: 0.6 the circle area less twice the perimeter
    clusterSizeDownlimit = 0.6*3.14159*radi*radi - 4*3.14159*radi;
    // upper limit of squared bounding box, equal to twice the diameter
    float boundBoxLimit2 = 32.*radi*radi;
    
    deltracker = 0;    
    for (j=0; j<abh; j++) { // height
       for (i=0; i<abw; i++) { // width
	      if(Accum[j*abw+i]) { // accum test passed
	         sp = clusterSize = 0;
	  
	         // ---------- addPoint ------------------
	         cluster_x[clusterSize] = i;
	         cluster_y[clusterSize] = j;
	         clusterSize++;
             if(i>0 && j>0 && Accum[(j-1)*abw+i-1] == 1) {
	            stack_x[sp] = i-1;
	            stack_y[sp] = j-1;
	            sp++;
	            Accum[(j-1)*abw+i-1] = 2;
	         }
             if(i>0 && Accum[j*abw+i-1] == 1) {
	            stack_x[sp] = i-1;
	            stack_y[sp] = j;
	            sp++;
	            Accum[j*abw+i-1] = 2;
	         }
             if(i>0 && j<abh-1 && Accum[(j+1)*abw+i-1] == 1) {
	            stack_x[sp] = i-1;
	            stack_y[sp] = j+1;
	            sp++;
                Accum[(j+1)*abw+i-1] = 2;
	         }
             if(j>0 && Accum[(j-1)*abw+i] == 1) {
	            stack_x[sp] = i;
	            stack_y[sp] = j-1;
	            sp++;
                Accum[(j-1)*abw+i] = 2;
	         }
             if(j<abh-1 && Accum[(j+1)*abw+i] == 1) {
                stack_x[sp] = i;
                stack_y[sp] = j+1;
                sp++;
                Accum[(j+1)*abw+i] = 2;
	         }
             if(i<abw-1 && j>0 && Accum[(j-1)*abw+i+1] == 1) {
	            stack_x[sp] = i+1;
	            stack_y[sp] = j-1;
	            sp++;
	            Accum[(j-1)*abw+i+1] = 2;
             }
             if(i<abw-1 && Accum[j*abw+i+1] == 1) {
	            stack_x[sp] = i+1;
	            stack_y[sp] = j;
	            sp++;
	            Accum[j*abw+i+1] = 2;
	         }
             if(i<abw-1 && j<abh-1 && Accum[(j+1)*abw+i+1] == 1) {
	            stack_x[sp] = i+1;
	            stack_y[sp] = j+1;
	            sp++;
	            Accum[(j+1)*abw+i+1] = 2;
             }
	  
	         Accum[j*abw+i] = 0;
	  
             while(sp>0) { // sp > 0
	    
	            // --------- removeStackTop ----------------
	            ti = stack_x[sp-1];
	            tj = stack_y[sp-1];
	            sp --;
	    
	            // ---------- addPoint ------------------
	            cluster_x[clusterSize] = ti;
	            cluster_y[clusterSize] = tj;
        	    clusterSize++;
	            if(clusterSize >= aba) {
	               free(stack_x);
	               free(stack_y);
	               free(cluster_x);
	               free(cluster_y);
        	       free(Accum);
	               free(ab_buf);
	               strcpy(error_msg,"Error cluster too big");
	               return(error_msg);
	            }
                if(sp >= aba) {
	               free(stack_x);
	               free(stack_y);
	               free(cluster_x);
	               free(cluster_y);
        	       free(Accum);
	               free(ab_buf);
	               strcpy(error_msg,"Error stack too big");
	               return(error_msg);
	            }
	            if(ti>0 && tj>0) {
	               if(Accum[(tj-1)*abw+ti-1] == 1) {
		              stack_x[sp] = ti-1;
		              stack_y[sp] = tj-1;
		              sp++;
		              Accum[(tj-1)*abw+ti-1] = 2;
	               }
	            }
	            if(ti>0) {
	               if(Accum[tj*abw+ti-1] == 1) {
		              stack_x[sp] = ti-1;
		              stack_y[sp] = tj;
		              sp++;
		              Accum[tj*abw+ti-1] = 2;
	               }
	            }
                if(ti>0 && tj<abh-1) {
	               if(Accum[(tj+1)*abw+ti-1] == 1) {
		              stack_x[sp] = ti-1;
		              stack_y[sp] = tj+1;
		              sp++;
		              Accum[(tj+1)*abw+ti-1] = 2;
	               }
	            }
                if(tj>0) {
	               if(Accum[(tj-1)*abw+ti] == 1) {
		              stack_x[sp] = ti;
		              stack_y[sp] = tj-1;
		              sp++;
		              Accum[(tj-1)*abw+ti] = 2;
	               }
	            }
	            if(tj<abh-1) {
	               if(Accum[(tj+1)*abw+ti] == 1) {
		              stack_x[sp] = ti;
		              stack_y[sp] = tj+1;
		              sp++;
		              Accum[(tj+1)*abw+ti] = 2;
	               }
	            }
                if(ti<abw-1 && tj>0) {
	               if(Accum[(tj-1)*abw+ti+1] == 1) {
		              stack_x[sp] = ti+1;
		              stack_y[sp] = tj-1;
		              sp++;
		              Accum[(tj-1)*abw+ti+1] = 2;
	               }
	            }
	            if(ti<abw-1) {
	               if(Accum[tj*abw+ti+1] == 1) {
		              stack_x[sp] = ti+1;
		              stack_y[sp] = tj;
		              sp++;
                      Accum[tj*abw+ti+1] = 2;
                   }
                }
	            if(ti<abw-1 && tj<abh-1) {
	               if(Accum[(tj+1)*abw+ti+1] == 1) {
		              stack_x[sp] = ti+1;
		              stack_y[sp] = tj+1;
		              sp++;
		              Accum[(tj+1)*abw+ti+1] = 2;
	               }
	            }
	            Accum[tj*abw+ti] = 0;
             } // sp > 0
	  
             int minx = cluster_x[0];
             int maxx = cluster_x[0];
             int miny = cluster_y[0];
             int maxy = cluster_y[0];
             float sum2;
             int totSum2;
             int shell = MAX(0.2*radi,2.);
             
	         if(clusterSize > clusterSizeDownlimit &&
               clusterSize < clusterSizeUplimit ) {// clusterSize in limits
	            avgx = avgy = 0;
	            for (k=0; k<clusterSize; k++) {
	               avgx += cluster_x[k];
	               avgy += cluster_y[k];
	               if (cluster_x[k] < minx) minx = cluster_x[k];
	               else if (cluster_x[k] > maxx) maxx = cluster_x[k];
	               if (cluster_y[k] < miny) miny = cluster_y[k];
	               else if (cluster_y[k] > maxy) maxy = cluster_y[k];
	            }
	            avgx /= clusterSize; 
	            avgy /= clusterSize;
	    
	            startJ = avgy - radi;
	            endJ   = avgy + radi;
	            startI = avgx - radi;
	            endI   = avgx + radi;
                // consider only particles that are far from the edges, at least by the radius
	            if(startJ > radi && endJ <(abh-radi) &&
	              startI > radi && endI <(abw-radi) ) {
	      
	               // ---------- correlParticles 
                   sum2 = 0.;
                   totSum2 = 0;
                   sum1 = 0.;
                   sum1_raw = 0.;
                   totSum1 = 0;
                   // redefine box for shell analysis purposes
                   // since shell is less than radius, the 'if' condition above
                   // guarantees that box is within image boundaries
	               startJ = avgy - radi-shell;
	               endJ   = avgy + radi+shell;
	               startI = avgx - radi-shell;
	               endI   = avgx + radi+shell;
	      
	               for (jj=startJ; jj<endJ; jj++) {
		              for (ii=startI; ii<endI; ii++) {
		                 if((ii-avgx)*(ii-avgx)+(jj-avgy)*(jj-avgy) < (radi+shell)*(radi+shell)) {
		                  if((ii-avgx)*(ii-avgx)+(jj-avgy)*(jj-avgy) < radi*radi) {
                            sum1 += ab_buf[jj*abw+ii];
                            sum1_raw += ab_buf_bin[jj*abw+ii];
                            totSum1++;
		                  } else {
		                    sum2 += ab_buf[jj*abw+ii];
		                    totSum2++;
		                  }
                         }  
		              }
	               }
	            } else { 
	               // Make it a lousy correlation
                   sum1 = 0.0;
                   totSum1 = 1;
                   sum2 = 0.0;
                   totSum2 = 1;
	            }

                int bbsize = (maxx-minx)*(maxx-minx)+(maxy-miny)*(maxy-miny);
                // Threshold as difference between average inside (sum1/totSum1) and average within shell outside particle (sum2/sumTot2)
                corr = -(sum1/(float)totSum1-sum2/(float)totSum2);
                //GCA: the value of 2.5 should be converted in something optimized for the SNR of each micrograph
	            if(corr >= 2.5 && avgy < abh-radi && avgx < abw-radi && bbsize < boundBoxLimit2) {
	               if(*numBoxes < MAXBOX) {
	                  // the center is approximated because of the previous compression
		              box[*numBoxes].x = avgx * (*compress_factor);
		              box[*numBoxes].y = avgy * (*compress_factor);
		              box[*numBoxes].reference = FALSE;
		              box[*numBoxes].correlation = corr;
		              box[*numBoxes].avgdensity = sum1/(float)totSum1;
		              box[*numBoxes].raw_avgdensity = sum1_raw/(float)totSum1;
		              (*numBoxes) ++;
	               } else { // Overflow
		              printf(
		                "\n[AUTOBOX_ACTIVATE] Too many boxes [%d][%d]; box ignored.",*numBoxes,MAXBOX);
	               }
	            } else deltracker++;
	         }
	      }  // end accum test
       }  // end width
    }  // end height
    printf("Particles deleted by shell analysis: %d\n",deltracker);
    printf("Number of particles left: %d\n",*numBoxes);
    deltracker =0;
        
/**************** Iterative pruning of tails in histogram of correlation **********************/
    int corrRange;
    float minCorr, maxCorr, scaleCorr;
    float* boxcorr;
    
    corrRange = (*numBoxes)/2+1;
    // generate correlation histogram
    if((boxcorr=(float*)malloc((*numBoxes)*sizeof(float)))==(float*)NULL ||
       (hist_p = (int *)malloc(corrRange*sizeof(int))) == (int *)NULL) {
	    strcpy(error_msg,"Err alloc buffers for Corr Hist Equal.");
	    return(error_msg);
    }
    
    int purge_histo = 1;
    int hist_thresh, hist_window, hist_sum;
    int hist_min, hist_max;
    int refi;
    while (purge_histo) {
    
       corrRange = (*numBoxes)/2+1;
       minCorr = box[0].correlation;
       maxCorr = box[0].correlation;

       // initialize histogram
       init_intarr(hist_p, corrRange, 0);

       // calculate min and max corr 
       for (k=0; k<(*numBoxes); k++) {
          minCorr = MIN(minCorr, box[k].correlation);
          maxCorr = MAX(maxCorr, box[k].correlation);
       }
       if (maxCorr>minCorr)
           scaleCorr = (corrRange-1)/(maxCorr-minCorr);
       else
           scaleCorr = 1.;
       // scale to histogram range
       for (k=0; k<(*numBoxes); k++) {
	      boxcorr[k]= 0.5 + ((box[k].correlation-minCorr)*scaleCorr);
       }
    
       for (k=0; k<(*numBoxes); k++)
          hist_p[(int)boxcorr[k]]++;

       // detect and eliminate outliers at both sides
       // start from each side and sum values over a window:
       // if the sum does not reach the given threshold,
       // exclude particles with values up to that point
       
       // lower tail
       hist_min = 0;
       hist_max = corrRange-1;
       hist_window = MAX(2,corrRange/10);
       hist_thresh = hist_window;
       for (k=0; k<corrRange-hist_window-1; k++) {
          hist_sum = 0;
          for (j=k; j<k+hist_window; j++)
              hist_sum += hist_p[j];
          if (hist_sum < hist_thresh)
             hist_min = k;
          else
             break;
       }
       while (hist_p[hist_min] <1 && hist_min < corrRange-1) hist_min++;

       // upper tail
       hist_thresh = hist_window-1;
       for (k=corrRange-1; k>hist_window-1; k--) {
          hist_sum = 0;
          for (j=k; j>k-hist_window; j--)
              hist_sum += hist_p[j];
          if (hist_sum < hist_thresh)
             hist_max = k;
          else
             break;
       }
       while (hist_p[hist_max] <1 && hist_max > 0) hist_max--;

//GCA: at the moment the lower tail is not processed, and only outliers
//     with high correlation values are removed
       hist_min = 0;
//       hist_max = corrRange-1;  // to avoid upper tail processing too

       if ( hist_min>0 || hist_max <corrRange-1) {
          //eliminate outlier particles
          i = *numBoxes-1;
          while (i>=0 && (*numBoxes)>0) {
             refi = (int) boxcorr[i];
             if ( refi < hist_min || refi > hist_max) {
                deltracker++;
                for (j=i+1; j<*numBoxes; j++) {
                    box[j-1] = box[j];
                }
                (*numBoxes) --;
             }
             i--;     
          }
       } else
           purge_histo = 0;
    }

    printf("Particles deleted by histogram (upper) tail pruning: %d\n",deltracker);
    deltracker =0;

    free(boxcorr); 
    free(hist_p); 

/*********  screen particles by density *****************************/

    // screen particle by average density of raw, binned, data
    // only exclude very obvious contaminants
    float avgsum=0.;
    float avgsum2 = 0.;
    float stddev, avgthresh;
    // repeat three times, at most
    k = 3;
    int deleted;
    while (k > 0) {
       deleted = 0;
       avgsum=0.;
       avgsum2 = 0.;
       for (i=0; i<*numBoxes; i++) {
          avgsum += box[i].raw_avgdensity;
          avgsum2 += (box[i].raw_avgdensity*box[i].raw_avgdensity);
       }    
       stddev = sqrt((avgsum2 - ((avgsum*avgsum)/(*numBoxes)))/((*numBoxes) - 1));
       avgthresh = avgsum/(*numBoxes) - 3.*stddev;
       i = *numBoxes-1;
       while (i>=0) { 
           if (box[i].raw_avgdensity < avgthresh) {
               deltracker++;
               for (j=i+1; j<*numBoxes; j++) {
                   box[j-1] = box[j];
               }
               (*numBoxes) --;
               deleted ++;
           }
           i--;     
       }
       k--;
       if (deleted == 0) k = 0;
    }
    printf("Particles deleted by average density screening: %d\n",deltracker);
    deltracker =0;

    free(stack_x);
    free(stack_y);
    free(cluster_x);
    free(cluster_y);
    free(Accum);
    free(ab_buf_bin);
    free(ab_buf_raw);
    free(ab_buf);
    free(error_msg);
    
    printf("\nFinished emma-based automatic boxing\n");
    printf("Done.  [%d] boxes.\n",*numBoxes);
    return(NULL);
}
