#include "gui.h"

void dest_mirror(Widget widget, XtPointer client_data, 
				  XtPointer call_data) {
  // function name: dest_mirror
  // Physically apply mirror operation to map
  
  int i, j, k, secBegin, secEnd, rowBegin, rowEnd, colBegin, colEnd;
  float tempFloat;
  
  if(widget == invertXW) { // X --> -X
    for (k=0; k<origNumSecs; k++) {
      secBegin = k*numRows*numCols;
      for (j=0; j<numRows; j++) {
	rowBegin = j*numCols;
	for (i=0; i<numCols/2; i++) {
	  colBegin = secBegin + rowBegin + i;
	  colEnd   = secBegin + rowBegin + (numCols - 1 - i);
	  tempFloat = float_image_buffer1[colEnd];
	  float_image_buffer1[colEnd] = float_image_buffer1[colBegin];
	  float_image_buffer1[colBegin] = tempFloat;
	}
      }
    }
  }
  else if(widget == invertYW) { // Y --> -Y
    for (k=0; k<origNumSecs; k++) {
      secBegin = k*numRows*numCols;
      for (j=0; j<numRows/2; j++) {
	rowBegin = j*numCols;
	rowEnd   = ((numRows - 1 - j)*numCols);
	for (i=0; i<numCols; i++) {
	  colBegin = secBegin + rowBegin + i;
	  colEnd   = secBegin + rowEnd + i;
	  tempFloat = float_image_buffer1[colEnd];
	  float_image_buffer1[colEnd] = float_image_buffer1[colBegin];
	  float_image_buffer1[colBegin] = tempFloat;
	}
      }
    }
  }
  else if(widget == invertZW) { // Z --> -Z
    for (k=0; k<origNumSecs/2; k++) {
      secBegin = k*numRows*numCols;
      secEnd   = (origNumSecs - 1 - k) *numRows*numCols;
      for (j=0; j<numRows; j++) {
	rowBegin = j*numCols;
	for (i=0; i<numCols; i++) {
	  colBegin = secBegin + rowBegin + i;
	  colEnd   = secEnd + rowBegin + i;
	  tempFloat = float_image_buffer1[colEnd];
	  float_image_buffer1[colEnd] = float_image_buffer1[colBegin];
	  float_image_buffer1[colBegin] = tempFloat;
	}
      }
    }
  }
  
  // Their shaded views are no longer valid - show new sections
  shadedMaps= FALSE;
  dispSections = TRUE;
  XtSetArg(arglst[0], XmNlabelPixmap, sectionPixmap);
  XtSetArg(arglst[1], XmNselectPixmap, unsectionPixmap);
  XtSetArg(arglst[2], XmNrecomputeSize, FALSE);
  XtSetValues(sectionW, arglst, 3); 
  if(dispSections) {
    // Update the sections - if shaded views then they have to recalc
    // Now stuff the display buffer from our new FIXed map
    stuffMapImage(map_image_buffer, float_image_buffer1, 
		  file_width, file_height, nsecs, imgNum, 0);
    DrawWindow();
  }
  
} // end function dest_mirror



void create_invertscreen () {
  // function name: create_invertscreen
  // Create and initialize attributes of Invert 3D map screen
  
  XtSetArg(arglst[0], XmNautoUnmanage, 0);
  XtSetArg(arglst[1], XmNdefaultPosition, 0);
  XtSetArg(arglst[2], XmNdialogTitle, XmStringCreateLocalized("Invert 3D map"));
  XtSetArg(arglst[3], XmNdialogStyle, XmDIALOG_MODELESS);
  XtSetArg(arglst[4], XmNx, 700);
  XtSetArg(arglst[5], XmNy, 100);
  XtSetArg(arglst[6], XmNwidth, 300);
  XtSetArg(arglst[7], XmNheight, 170);
  invertScreenW = XmCreateFormDialog(toplevel_widget, "inv", arglst, 8);

  XtSetArg(arglst[0], XmNlabelType, XmPIXMAP);
  XtSetArg(arglst[1], XmNwidth, 38);
  XtSetArg(arglst[2], XmNheight, 32);
  XtSetArg(arglst[3], XmNx, 10);
  XtSetArg(arglst[4], XmNy, 130);
  invertExitW = XmCreatePushButton(invertScreenW, "inv_exit", arglst, 5);
  XtAddCallback(invertExitW, XmNactivateCallback, simple_deactivate, NULL);
  XtManageChild(invertExitW);

  XtSetArg(arglst[0], XmNx, 10);  // Common attribute for buttons

  XtSetArg(arglst[1], XmNlabelString, 
	   XmStringCreateLocalized("Reflect in yz plane (x -> -x) "));
  XtSetArg(arglst[2], XmNy, 10);
  invertXW = XmCreatePushButton(invertScreenW, "inv_x", arglst, 3);
  XtAddCallback(invertXW, XmNactivateCallback, dest_mirror, NULL);
  XtManageChild(invertXW);

  XtSetArg(arglst[1], XmNlabelString, 
	   XmStringCreateLocalized("Reflect in xz plane (y -> -x) "));
  XtSetArg(arglst[2], XmNy, 50);
  invertYW = XmCreatePushButton(invertScreenW, "inv_y", arglst, 3);
  XtAddCallback(invertYW, XmNactivateCallback, dest_mirror, NULL);
  XtManageChild(invertYW);

  XtSetArg(arglst[1], XmNlabelString, 
	   XmStringCreateLocalized("Reflect in xy plane (z -> -z) "));
  XtSetArg(arglst[2], XmNy, 90);
  invertZW = XmCreatePushButton(invertScreenW, "inv_z", arglst, 3);
  XtAddCallback(invertZW, XmNactivateCallback, dest_mirror, NULL);
  XtManageChild(invertZW);

} // end function create_invertscreen
