//   lists.c 
//
//   Description: Contains all the functions used in list management
//   Functions to add a member to a list, display the list, find the
//   number in a given range, and get the current node are included
//
//   Created By: Brant French
//   Last Modified:  5-11-98

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "robem_def.h"
#include "lists.h"
#include "../include/common.h"
#include "../include/libUtil.h"

INFO info;  // A global info structure



P_INFO *Add_Particle(P_INFO *head,int particle_number,
		     float theta,float phi,float omega,float x,float y,
		     float mag,float cc1,float cc2,float cc3,
		     int state, int ncol, int nrow)
{
  // function name: Add_Particle
  // Adds a Particle to end of the list.  Also sets all values to
  // those passed in by the function.

  int rank = 1;
  P_INFO *temp; // temp struct used to hold the current location in list

  if (head == NULL) {
    head = (P_INFO *)malloc(sizeof(P_INFO));
    temp = head;
  }
  else {
    temp = head;
    rank++;
    while (temp->next!=NULL) { // Find next position
      temp=temp->next;
      rank++;
    }
    temp->next = (P_INFO *)malloc(sizeof(P_INFO));
    temp = temp->next;
  }
  temp->particle_number=particle_number;
  temp->theta=theta; temp->phi=phi; temp->omega=omega; 
  temp->x=x; temp->y=y; temp->mag=mag;
  temp->origX = x;
  temp->origY = y;
  temp->deltaX = x - (ncol/2.0 - 0.5);
  temp->deltaY = y - (nrow/2.0 - 0.5);
  deltaMax = MAX(temp->deltaX, deltaMax);
  deltaMax = MAX(temp->deltaY, deltaMax);
  deltaMin = MIN(temp->deltaX, deltaMin);
  deltaMin = MIN(temp->deltaY, deltaMin);
  deltaXmin= MIN(temp->deltaX, deltaXmin);
  deltaYmin= MIN(temp->deltaY, deltaYmin);
  deltaXmax= MAX(temp->deltaX, deltaXmax);
  deltaYmax= MAX(temp->deltaY, deltaYmax);
  temp->distance = sqrt((temp->deltaX*temp->deltaX)+
			(temp->deltaY*temp->deltaY));
  distanceMin = MIN(distanceMin,temp->distance);
  distanceMax = MAX(distanceMax,temp->distance);
  temp->rank  = rank;
  temp->cc1=cc1; temp->cc2=cc2; temp->cc3=cc3;
  temp->toggle = state;  // Set particle to on/off
  if(particle_number < 0)
    temp->toggle &= INFILEBAD;
  temp->projection = NULL;
  temp->next=NULL;
  return(head);

} // end function Add_Particle



void Display_Particle_List(INFO head)
{
  // function name: Display_Particle_List
  // Displays all the links in the list.  Takes the list as input and
  // runs through until NULL is hit and then stops.

  P_INFO *temp=head.info;

  printf ("%s\n",head.filename);
  printf ("%f %f %f %f\n%f %f %f\n", head.a, head.b, head.c, head.d,
	  head.e, head.f, head.g);
  while (temp!=NULL) {
    printf ("%d %f %f %f %f %f %f %f %f %f\n",temp->particle_number,
	    temp->theta, temp->phi, temp->omega, temp->x,temp->y,
	    temp->mag, temp->cc1, temp->cc2, temp->cc3);
    temp=temp->next;
  }
} // end function Display_Particle_List



float find_max_height(P_INFO *head,float start,float end,float step,
		      int paramNum, int *badCount) 
{
  // function name: find_max_height
  // Finds the maximum height value in this given range of particles.

  float maxheight=-99999.9, x, height;
 
  for (x=start; x<end; x+=step) {
    height = find_number_in_range(head, x, x+step, paramNum, badCount);
    if (height+(*badCount) > maxheight)
      maxheight = height+(*badCount);
  }
  if(maxheight <= 0.0)
    maxheight = 1.0;
  return (maxheight);
  
} // end function find_max_height



int find_number_in_range(P_INFO *head, float start, float end,
			 int paramNum, int *badCount)
{
  // function name: find_number_in_range
  // Finds the number of particles in the range specified in the
  // parameter list. This function finds the number of particles cc1
  // values in the range start to end in O(n) time. It then returns
  // that number.

  P_INFO *temp = head;
  int count = 0;
  
  *badCount = 0;
  while (temp!=NULL) {
    if(paramNum==THETAT) {
      if ((temp->theta<end) && (temp->theta>=start)) {
        if (!temp->toggle)
            count++;
        else 
            (*badCount)++;
      }
    }
    else if(paramNum==PHIT) {
      if ((temp->phi<end) && (temp->phi>=start)) {
        if (!temp->toggle)
            count++;
        else 
            (*badCount)++;
      }
    }
    else if(paramNum==OMEGAT) {
      if ((temp->omega<end) && (temp->omega>=start)) {
        if (!temp->toggle)
            count++;
        else 
            (*badCount)++;
      }
    }
    else if(paramNum==XT) {
      if ((temp->x<end) && (temp->x>=start)) {
        if (!temp->toggle)
            count++;
        else 
            (*badCount)++;
      }
    }
    else if(paramNum==DELTAXT) {
      if ((temp->deltaX<end) && (temp->deltaX>=start)){
        if (!temp->toggle)
            count++;
        else 
            (*badCount)++;
      }
    }
    else if(paramNum==ABSDELTAXT) {
      if ((fabs(temp->deltaX)<end) && (fabs(temp->deltaX)>=start)){
        if (!temp->toggle)
            count++;
        else 
            (*badCount)++;
      }
    }
    else if(paramNum==YT) {
      if ((temp->y<end) && (temp->y>=start)) {
        if (!temp->toggle)
            count++;
        else 
            (*badCount)++;
      }
    }
    else if(paramNum==DELTAYT) {
      if ((temp->deltaY<end) && (temp->deltaY>=start)){
        if (!temp->toggle)
            count++;
        else 
            (*badCount)++;
      }
    }
    else if(paramNum==ABSDELTAYT) {
      if ((fabs(temp->deltaY)<end) && (fabs(temp->deltaY)>=start)) {
        if (!temp->toggle)
            count++;
        else 
            (*badCount)++;
      }
    }
    else if(paramNum==DISTANCET) {
      if ((temp->distance<end) && (temp->distance>=start)){
        if (!temp->toggle)
            count++;
        else 
            (*badCount)++;
      }
    }
    else if(paramNum==MAGT) {
      if ((temp->mag<end) && (temp->mag>=start)){
        if (temp->toggle & HISTMAGBAD)
            (*badCount)++;
        else  
            count++;
      }
    }
    else if(paramNum==PFTCC) {
      if ((temp->cc1<end) && (temp->cc1>=start)){
        if (temp->toggle & HISTPFTBAD)
            (*badCount)++;
        else  
            count++;
      }
    }
    else if(paramNum==PRJCC) {
      if ((temp->cc2<end) && (temp->cc2>=start)){
        if (temp->toggle & HISTPRJBAD)
            (*badCount)++;
        else  
            count++;
      }
    }
    else if(paramNum==CMPCC) {
      if ((temp->cc3<end) && (temp->cc3>=start)){
        if (temp->toggle & HISTCMPBAD)
            (*badCount)++;
        else  
            count++;
      }
    }

    temp=temp->next;
  }
  return (count);

} // end function find_number_in_range



void find_minmax(P_INFO *head)
{
  // function name: find_minmax

  P_INFO *temp = head;

  thetamin = head->theta;
  thetamax = head->theta;
  phimin   = head->phi;
  phimax  =  head->phi;
  omegamin = head->omega;
  omegamax = head->omega;
  xmin     = head->x;
  xmax     = head->x;
  ymin     = head->y;
  ymax     = head->y;
  magmin   = head->mag;
  magmax   = head->mag;
  cc1min   = head->cc1;
  cc1max   = head->cc1;
  cc2min   = head->cc2;
  cc2max   = head->cc2;
  cc3min   = head->cc3;
  cc3max   = head->cc3;

  while (temp != NULL) {
    if (!temp->toggle) {
      if (temp->theta < thetamin)  thetamin = temp->theta;
      if (temp->theta > thetamax)  thetamax = temp->theta;
      if (temp->phi < phimin)      phimin = temp->phi;
      if (temp->phi > phimax)      phimax = temp->phi;
      if (temp->omega < omegamin)  omegamin = temp->omega;
      if (temp->omega > omegamax)  omegamax = temp->omega;
      if (temp->x < xmin)          xmin = temp->x;
      if (temp->x > xmax)          xmax = temp->x;
      if (temp->y < ymin)          ymin = temp->y;
      if (temp->y > ymax)          ymax = temp->y;
      if (temp->mag < magmin)      magmin = temp->mag;
      if (temp->mag > magmax)      magmax = temp->mag;
      if (temp->cc1 < cc1min)      cc1min = temp->cc1;
      if (temp->cc1 > cc1max)      cc1max = temp->cc1;
      if (temp->cc2 < cc2min)      cc2min = temp->cc2;
      if (temp->cc2 > cc2max)      cc2max = temp->cc2;
      if (temp->cc3 < cc3min)      cc3min = temp->cc3;
      if (temp->cc3 > cc3max)      cc3max = temp->cc3;
    }
    temp = temp->next;
  }

} // end function find_minmax



void Get_Particle_Info(INFO *head, FILE *fIn, int ncol, int nrow)
{
  // function name: Get_Particle_Info
  // Gets an individual particle from the file specified by fIn and
  // places data into the list specified by head.

  // These variables are used to store the individual parameters that
  // can then be passed to the Add_Particle function

  char inStr [512], junk;
  int quit = 0, ijunk, tiltFormat, particle_number;
  float theta, phi, omega, f7;
  float x=1.0, y=1.0, mag=1.0, cc1=1.0, cc2=1.0, cc3=1.0;
  
  fscanf(fIn,"%s", head->filename); 
  tiltFormat= FALSE;
  if(EQU(head->filename,"TILT_FORMAT"))
    tiltFormat= TRUE;
  
  if(!tiltFormat) { // Get second line
    get_str(fIn,'\n',inStr);
    get_str(fIn,'\n',inStr);
    sscanf(inStr,"%f%c %d%c %f%c %f%c %f%c %f%c %f%c %f",
	   &theta,&junk,
	   &ijunk,&junk,
	   &phi,&junk,
	   &omega,&junk,&x,&junk,&y,&junk,
	   &mag,&junk,&f7);
  }
  
  head->a = theta; head->b = phi; head->c = omega; head->d = x;
  head->e = y; head->f = mag; head->g = f7;

  // OLDWAY Brantt
  // if (fscanf(fIn,"%d%c",&particle_number,&junk)!=2) quit = 1;
  // printf("particle number [%d]\n",particle_number);

  while (!quit) {
    if(tiltFormat) { // special
      fscanf(fIn,"%d%c %f%c %f%c %f",
	     &particle_number,&junk,&theta,&junk,&phi,&junk,&omega);
    }
    else { // Normal
      get_str(fIn,'\n',inStr);
      if(EQU(inStr,""))
	particle_number = -99;
      else
	sscanf(inStr,
	       "%d%c %f%c %f%c %f%c %f%c %f%c %f%c %f%c %f%c %f",
	       &particle_number, &junk,
	       &theta, &junk, &phi, &junk,
	       &omega, &junk,
	       &x, &junk, &y, &junk, &mag, &junk,
	       &cc1, &junk, &cc2, &junk, &cc3);
    }
    
    if(particle_number > 0 && particle_number < 999777)
      head->info = Add_Particle(head->info,particle_number,theta,phi,omega,
				x, y, mag,
				cc1, cc2, cc3, FALSE, ncol, nrow);
    if(feof(fIn))
      quit = TRUE;

    // OLDWAY Brant 
    // if (fscanf(fIn,"%d%c",&particle_number,&junk)!=2) quit = 1;
    // printf("222  particle number [%d]\n",particle_number);
  }

} // end function Get_Particle_Info



void add_missing_particles(INFO *head, int nimg, int ncol, int nrow)
{
  // function name: add_missing_particles
  // We've got to add in blank records for missing particles so the
  // progtram has something to use when displaying them.

  int i, foundIt;
  P_INFO *thisInfo;

  for (i=0; i<nimg; i++) { // Find them all

    foundIt = FALSE;
    thisInfo = head->info;
    while(thisInfo != NULL) { // Do all of them
      if(thisInfo->particle_number==(i+1)) { // Found it
	foundIt = TRUE;
	break;
      }
      thisInfo = thisInfo->next;
    }
    
    if(!foundIt) { // Then add it
      head->info = Add_Particle(head->info, (i+1),
				0.0, 0.0, 0.0,
				0.0, 0.0, 0.0,
				0.0, 0.0, 0.0,
				FALSE, ncol, nrow);
      
    }
  }

} // end function add_missing_particles



void Free_Particle_Info(INFO *head)
{
  // function name: Free_Particle_Info
  // Free the memory.

  P_INFO *thisInfo, *this2;

  thisInfo = head->info;
  while(thisInfo != NULL) { // Free all
    if(thisInfo->projection)
      free(thisInfo->projection);
    thisInfo->projection = NULL;
    this2 = thisInfo->next;
    free(thisInfo);
    thisInfo = this2;
  }
  head->info = NULL;

} // end function Free_Particle_Info



void sortParticles(P_INFO *head, int maxParts, int key, int incSort)
{
  // function name: sortParticles

  P_INFO *temp; // temp struct used to hold the current location in list

  int totParts=0, i, *fids, *ids;
  float *ts, *ps, *os, *xs, *ys, *mags;
  float *cc1s, *cc2s, *cc3s, *deltaXs, *deltaYs, *dists;

  fids    = (int *)malloc(sizeof(int)*maxParts);
  ids     = (int *)malloc(sizeof(int)*maxParts);
  ts      = (float *)malloc(sizeof(float)*maxParts);
  ps      = (float *)malloc(sizeof(float)*maxParts);
  os      = (float *)malloc(sizeof(float)*maxParts);
  xs      = (float *)malloc(sizeof(float)*maxParts);
  ys      = (float *)malloc(sizeof(float)*maxParts);
  mags    = (float *)malloc(sizeof(float)*maxParts);
  cc1s    = (float *)malloc(sizeof(float)*maxParts);
  cc2s    = (float *)malloc(sizeof(float)*maxParts);
  cc3s    = (float *)malloc(sizeof(float)*maxParts);
  deltaXs = (float *)malloc(sizeof(float)*maxParts);
  deltaYs = (float *)malloc(sizeof(float)*maxParts);
  dists   = (float *)malloc(sizeof(float)*maxParts);

  if (head != NULL) {
    temp = head;
    while (temp != NULL) { // Find next position
      fids[totParts]    = 1;
      ids[totParts]     = temp->particle_number;
      ts[totParts]      = temp->theta;
      ps[totParts]      = temp->phi;
      os[totParts]      = temp->omega;
      xs[totParts]      = temp->x;
      ys[totParts]      = temp->y;
      mags[totParts]    = temp->mag;
      cc1s[totParts]    = temp->cc1;
      cc2s[totParts]    = temp->cc2;
      cc3s[totParts]    = temp->cc3;
      deltaXs[totParts] = temp->deltaX;
      deltaYs[totParts] = temp->deltaY;
      dists[totParts]   = temp->distance;
      
      temp=temp->next;
      totParts++;
    }

    // Do the sort
    param_srt_rwa(&key, &totParts, fids, ids, ts, ps, os, xs, ys, mags,
		  cc1s, cc2s, cc3s, deltaXs, deltaYs, dists, &incSort);
    
    // Stuff the sort info backinto the linked list
    temp = head;
    while (temp!=NULL) { // Find next position
      for (i=0; i<totParts; i++)
	if(ids[i] == temp->particle_number) { // Found it
	  temp->rank = i+1;
	  break;
	}
      temp=temp->next;
    }
  }

  free(fids);
  free(ids);
  free(ts);
  free(ps);
  free(os);
  free(xs);
  free(ys);
  free(cc1s);
  free(cc2s);
  free(cc3s);
  free(deltaXs);
  free(deltaYs);
  free(dists);
  
} // end function sortParticles
