// Robem structure definitions

// Struct for rectangle boxes 
typedef struct __rect_t__ {
  int drawable;
  int x, y;
  int lowx, lowy;
  int width, height;
} rect_t;

// boxing hist 
typedef struct __bhist_t__ {
  float rangeVal;
  int hits;
} bhist_t;

// CTF struct 
typedef struct __ctf_t__ {
  int latX0, latY0;
  int latX1, latY1;
  float distA, distAsq;
  float distB, distBsq;
  float omega;
  int pickedPts;
} ctf_t;

// Our colormap struct 
typedef struct __map_t__ {
  int x, y;
  int color;
  int particle_number;
} map_t;

// Our lattice struct 
typedef struct __lattice_t__ {
  int x, y;
  int measuredX, measuredY;
} lattice_t;

// Our fullscreen struct 
typedef struct __full_t__ {
  int dispx, dispy;
  unsigned short *fullImage;
} full_t;

// Our circAvg struct 
typedef struct __circAvg_t__ {
  char  *mapName;
  float *rint;
  float *stDev;
  int   nterp;
} circAvg_t;

// Our box struct 
typedef struct __box_t__ {
  int  x, y;
  float *editBox_buffer;
  float *edit_mirror_buffer;
  int  boxZoom;
  int  inMemory;
  int  reference;
  int  nochange;
  float correlation;
  float avgdensity;
  float raw_avgdensity;
  float stdev;
} box_t;

typedef unsigned long	X_PIXEL;

// Struct for Purdue INT*2 file header 
typedef struct _purdueHead_t {
  short numCols,numRows,numSecs;
  float img_bkg;
  int   nimg, nrec_last;
  int   bimg_sizes;
  float img_min, img_max;
  float map_min, map_max;
  float img_avg, img_std;
  int   file_id1, file_id2;
  short junk;
} purdueHead_t;

// UNblob struct 
typedef struct __blobStructure_t__ {
  int id;
  int size;
  float minInt, maxInt;
} blobStructure_t;

typedef struct _drawCircAvg_t {
  char      fName[128];
  GC        colorGC;
  int       index;
  int       pixelColor;
} drawCircAvg_t;

// robem variable declarations

#define R4_MIN  -1.0e+38
#define R4_MAX   1.0e+38

/* ************************************** *
 * RAW Xlib definitions                   *
 * ************************************** */

Arg arglst[32];
X_PIXEL		black_pixel;
unsigned long  *pixels_return;
Visual	       *visual;
int		vdepth;
Colormap        color_map;

int defDispBufSize;  // Volatile DEF_DISPLAY_BUFFER_SIZE
int display_width, display_height; // in pixels?
int buffer_width,  buffer_height;  // in pixels?
int image_width,   image_height;   // in pixels
int file_width,    file_height;    // in pixels
int orig_file_width;               // Original file width

// Graphics contexts
GC color_gc,     rotflt_gc,     fft_gc,  
  fftContour_gc, emcoravg_gc,   circAvg_gc,     
  test_color_gc, hist_color_gc, editBox_gc,
  icosa_gc,      image_gc;      

GC image_gcLAT,      image_gcWHITE,  image_gcDOT,
  image_gcDOTgreen,  image_gcDOTred, image_gcGREEN,
  image_gcYELLOW,    image_gcRED,    image_rotfltgcRED,
  image_gcBLUE,      image_gcBLACK,  image_gcPLACEHOLDER,
  image_gcGREY[258], boxing_GC;

int		        x, y;
unsigned long	        gc_mask;
XGCValues		gc_values;
XColor	                *color_defs;
XColor	                *mirror_color_defs;
XColor			*gen_color_defs;
float                   pixelColor;
XEvent                  event;

XFontStruct *fontstruct; 	/* Font descriptor */

XFontStruct *robemFont; 	/* Global robem Font descriptor */
XmFontList robemFontList;


// CTF fit parameters
float adraValues[6], adrbValues[6];
float global_defocus;

/*****************
 * IMAGE BUFFERS *
 *****************/
float *float_image_buffer1;              /* For Map sections */
float *map_image_buffer;                 /* For depth cued MAP */
unsigned short *int_image_buffer1;       /* For passing to FORTRAN routines */
int nsecs;
float *rotMap, *tmpRotMap, *this_float_image_buffer1;
float *subtract_image_buffer1, *diffSection_buffer, *subtract_map_image_buffer;
float *subtract_image_buffer2, *subtract_map_image_buffer2;
float *subMapYmxb_buffer;
float *fftImage_gold;
float *fftImage_buffer2;
float *fftImage_fourAver;  /* Special buffer for fourAver */
float *paintMap;

float  *packImg[MAXBOX];
BYTE deletePackedImage[MAXBOX];
float maxPackPixel,minPackPixel;
float scalePackFactor;
float scaleBoxFactor;

float  *inCore_buffer;

float *fix2DinCore_buffer,          *fix2DinCore_diff;
float *fix2DpackImg_buffer[MAXBOX], *fix2DpackImg_diff[MAXBOX];

full_t fullScreen_buffer;

float *editBox_mirror_buffer;
float *editBox_undo_buffer;

float *emcoravg_image_buffer1;
float *emcoravg_image_buffer_disk;  /* Save what was read from disk */
float *emcoravg_image_buffer_avg;  /* Save the average */

float *imgconv_image_buffer1;
float *imgconv_image_buffer_disk;  /* Save what was read from disk */
float *imgconv_image_buffer_avg;  /* Save the average */


BYTE *mirror_image_buffer1;

/********************/
/* XWindows buffers */
/********************/
XImage *image_image1;
Window image_window1;
BYTE *image_buffer1;
BYTE *image_buffer1_disp;

BYTE *emcoravg_buffer1;
XImage *emcoravg_image1;
Window emcoravg_window1;

BYTE *circAvg_buffer1;
XImage *circAvg_image1;
Window circAvg_window1;

BYTE *rotflt_buffer1;
XImage *rotflt_image1;
Window rotflt_window1;

BYTE *fft_gold_buffer1;
BYTE *fft_buffer1, *fft_buffer2, *fft_display_buffer1;
XImage *fft_image1;
Window fft_window1;
BYTE *fft_contour_buffer;
BYTE *fft_contour_buffer2;
XImage *fftContour_image1;
Window fftContour_window1;

BYTE *color_buffer1;
XImage *color_image1;
Window color_window1;

BYTE *test_color_buffer1;
XImage *test_color_image1;
Window test_color_window1;

BYTE *hist_color_buffer1;
XImage *hist_color_image1;
Window hist_color_window1;

BYTE *editBox_display_buffer2;
BYTE *editBox_display_buffer;
float  *editBox_int_buffer;
XImage *editBox_image1;
Window editBox_window1;

BYTE *icosa_buffer1;
XImage *icosa_image1;
Window icosa_window1;

int *globAccum;      /* For autoboxing cluster display */
int compress_factor;
int firstAMMSTDEVtime;
int showBoxingSwitch,showBoxingLevel;  /* For boxing histogram */
float threshLo,threshHi;

int histogram[512];
int histoEqualize;    /* Histogram Equalize or not??  */

int movieSpeed;       /* How many loop through the sleep do they want? */
int flickSpeed;       /* How many loop through the sleep do they want? */

/* Global display and applications context pointers*/
Display         *dsply; 		/* Display variable */
XtAppContext    app_context;		/* application context */

// Cursor things
Cursor old_newXY_cursor, cross_cursor, old_cursor;
Cursor bogosity_cursor,  watch_cursor, zoomCursor, lanternCursor;
XSetWindowAttributes attrs;
Pixmap lanternPixmap, zoomPixmap,    lockedPixmap,     unlockedPixmap;
Pixmap resetPixmap,   noresetPixmap, sectionPixmap,    unsectionPixmap;
Pixmap playPixmap,    stopPixmap,    playingPixmap,    stoppedPixmap;
Pixmap moviePixmap,   unmoviePixmap, infiniPixmap,     noinfiniPixmap;
Pixmap rockPixmap,    norockPixmap,  flashLightPixmap, noflashLightPixmap;
Pixmap goPixmap,      nogoPixmap,    exitPixmap,       noexitPixmap;
Pixmap slowPixmap,    noslowPixmap,  helpPixmap,       nohelpPixmap;

/***************************************
 * Generic C language work areas       *
 ***************************************/
fString_t fileName;                 /* What file do they want? */
fString_t image_fileName;           /* What file is the image? */

int tryColors;                  /* Keep trying until you get as many entries 
				 * in the colormap as you can */

char stufBuf[NUMCOLS];          /* Init the screen */
short stufIntBuf[NUMCOLS];      /* Init the input buffer */
char stufBLKBuf[NUMCOLS];          /* Init the screen */
short stufBLKIntBuf[NUMCOLS];      /* Init the input buffer */


purdueHead_t purdueHeader;
forPifHead_t pifHeader;
char preLoadStr[256];
float shadeItDensity;
int  firstTimeLoHiRes;
float zoomItFactor;
char outStr[1024];
char versionStr[1024];
char contourStr[400];            /* Holds the contours being used */

int numCols, numRows;
int origWidthFromFile;           /* Remember this for pixelSize */
int fileBinCols,fileBinRows,     /* Remember these so we can write_pif */
    fileBinSecs,fileBinFactor;
int fileMrcEven, fileMrcX, fileMrcY, fileMrcZ;
int iRec;
int isBoxing, autoMinMaxThreshold;
int cycleRef;                          /* Autocycle reference correlate? */
int darkCircleRef;                     /* Use a dark circle for reference? */
int refReadFile;                       /* Use disk file reference? */
int fftMode;                           /* Are we looking at an FFT? */
int backTransform;                     /* Back transform FFT?? */
int pickLattice;                       /* Are we picking Lattice points? */
int layerLines;                        /* Are we picking Lines or points? */
int contour;                           /* Are we plotting the contour? */
int contourToggle;                     /* Toggle contour plotting on & off */
int filterToggle;                      /* Are we filtering the FFT? */
int drawLatToggle;                     /* Draw lattice lines or not */
int drawHiLoToggle;                    /* Draw hi/low pass filter ? */
int latValChanged;                     /* Are we changing from scrolls or 
					* mouse?? */
int autoZoom;
int lastNumBox;
int initLastNumBox;
int lastBoxX,lastBoxY;
int boxZoom;
float zoomFactor;                      /* Let's do this in float for more
					* granularity... */
float oldZoomFactor;
float oldSizeFactor;
int editBoxZoom;
int xoffset, yoffset;
int oldGlobX, oldGlobY, globX,globY;
int virXoffset,virYoffset;
int radiusImgNum,packRadius,borderWidth;
int boxRadius, emptyShellWidth, emmaRadius, featherRadius, imgNum;
int firstGridding;            /* Make a new grid first time empty boxing */
int boxRadiusDisp, emptyShellDisp, emmaRadiusDisp, 
  featherRadiusDisp, refineDisp;
int featherMode,backgroundMode,borderType;
int clusterDisp,referenceDisp;
int findBox;
int curBox,numBoxes,numPoly,numRects;
int fullScreen;                           /* They want to get back 
					   * to normal display */
int imgInCore;
int noCallbackScroll;
int boxMethod,dispMethod,histMethod,locked,bkgrndMethod;
int makeSquareBox;                        /* Force the box to be this size */ 
int readWrite,contrastStretch,fftMinMaxLock,disp1024fft;
int writeType,readType;
int fftType;
int preloadSwitch,normLinSwitch,shadeItSwitch,zoomItSwitch,doLinAlso,doFix;
int dumpNormStats, doNoise, doApplyCtf, doTrunc;
int dumpSpherAvg;
int autoBoxSwitch,boxitSwitch,doInterp,doOutFile;
int useGUI;
int reBoxSwitch;
char outFname[2048],dumpNormFile[1024],dumpSpherAvgFile[1024];
int liteSwitch;
int colorType,rampSwitch,rotfltSwitch,crysDispSwitch,rainbowSwitch;
int unblobSwitch;
int updateCrys;                           /* Auto update or not? */
int maxRadiusSwitch;
int fileType;
int blindWriting;
int globAsk,globAskAnswer;
int gFinished;
int lastPackedNumImgs;                   /* So we can free() intelligently */
int redLevel,greenLevel,blueLevel,brightLevel;
float histScaleFactor;
float histZoomFactor;
int scrollWidth,scrollHeight;
int headOff;
int sysError;
int paintSize;
int paintMapSwitch;
int circAvgSize,caOffsetX;
float minCA,maxCA,scaleCircFactor;
int numCircAvgDraw;
drawCircAvg_t drawCircs[5];
int stepCM;                               /* How many levels to step CM */
float backDispChar;                    /* Background color */
float  backChar;
int largeImage;
int inited;                                    /* Has the UIL been 
						* instantiated? */
int drawLite;                                  /* Don't draw so much on lite
						* executions */
int numExp;                                    /* Debug exposures */
int shiftFFT;
int fftMotionFeedback;                         /* Provide pointer cues? */
int scaleItSwitch;
int dispSections;
int minSurf,maxSurf,firstSec,lastSec;
float minFFTvalue,maxFFTvalue;                 /* Max and Min to display */
float globMinFFTvalue,globMaxFFTvalue;         /* Max and Min globals */
int tempFactor;                                /* FFt temp factor */
float tempFactors[1024][2048];                  /* Holds calculated tFac's */
float ctfValues[1024][2048];                    /* Holds the ctf Function */
float imageMaxFFTvalue;                        /* What was the MAX for img? */
float filtSizeValue;                           /* How large the FFT filter? */
int idim1,idim2;                               /* FFT dimensions */
int icut;                                      /* Hard(0) or SOft(1) cutoff in
						* fft filter ? */
int fftDispType;                               /* AMP Phase or Intensity */
int resoX,resoY;                               /* Resolution filter */
int lowResoFilter,hiResoFilter;                
int maxOrig,minOrig;
int applyCTF;                                  /* Apply the CTF to image? */
int ctfFilter;
int changeCTFtext;                             /* Did we chaneg the text in
						* response to the scrollbar?*/
int memoryMiser;                               /* Memory algorithms used? */
double xCen,yCen,zCen;                         /* x y z centers */
int largestDim;                                /* largest dimension of x,y,z */

/* Map globals */
int  numSecs,origNumSecs,mapNumSecs;           /* Number of map sections */
float maxMapPixel,minMapPixel,scaleMapFactor;
float cuedMaxPixel,cuedMinPixel,secMaxPixel,secMinPixel;
int makingMovie,rockIt,flashLightIt;
float map_min,map_max;
float surfaceDensity;
cue_t cuedMap[MAXSECTIONS];
double *rwa_tmp2d_map,*rwa_tmp_map;
int  secondMapShade, immediateMapShade, radialMapShade, stereoMapShade;
int immediateResource;          /* DO they want to inhibit immediate Update */
int  saveRadialMapShade;
int  radialProjections;
float globFract;
double globAngx,globAngy;
int shadedMaps;
BYTE globTilt;
double globTheta,globPhi,globOmega,
       globMaxRadius,globMinRadius,origGlobMaxRadius;
float globPixelSize;
int rotateXYZ;                  /* Do they want to rotate in X,Y,Z or 
				 * theta,phi,omega */
float aoverb,map_abang;
float deltaMapABang,deltaAoverB;
int whichStep,startMovie,endMovie;
int radialStepSize;
int expStep;
int dispIcosa;
float icosaScaleF;
int irev;
int mapStartImage,                 /* Remember these so we can use later. */
    mapSectionsPerRow;
int packStartImage,                /* Remember these so we can use later. */
    packSectionsPerRow;
float radialValue[258];
int lastDeleted;                   /* What was the last packed box they wanted
				    * deleted? */
float cosang, sinang;       /* degrees of tilt */
float contourLevel;         /* What contour for movie contour */
int totContours;            /* Holds contour values they want */
int conTours[50];
float deltaCont,breakCont;  /* Delta contours ; Break point value*/
int numCont;                /* Number of Contours */
int revRadProj;             /* Reverse contrast on Radial Projections? */
int radProjCS;              /* COntrast Stretch radial projections? */
int radialCS;               /* COntrast Stretch Radial Cues ?  */
int icosRadProj;            /* Do icosahedral radial projections? */
int emptyBox;               /* Are they trying to box empty particles? */

// Appodizing stuff
int appodizing, writeFFTType;

/* Circular Average stuff */
int currentCircAvg,selectedCircAvg;
circAvg_t circAvg[1000];
int caInvertContrast;       /* Invert contrast for CircAvg??? */

float sizeFactor,recipSizeFactor;   /* Corrections based on ZOOMiness */

box_t box[MAXBOX];
int   boxRefLast[MAXBOX];           /* Which boxes were used last time for
				     * reference correlation??? */
box_t poly[MAXBOX];
int   ref_box_flag;
rect_t rectBox;
map_t imgNums[MAXBOX];
int lastImg;

// RAMP colormap stuff
int firstBuildY, lastBuildY, buildingCmap;

/* Lattice Points for FFT */
int fftSize;                              /* What size FFT */
lattice_t  latPt[12];                      /* LAttice/CTF Points ONE & TWO */
int currentLatPt;                         /* Which one are we doing now? */
int pickedLatPts;                         /* Have they picked the first two? */
int numLatPts;                            /* Number of lattice points we have*/
float axL,ayL,bxL,byL;                    /* Lattice distances */
float omega;                              /* Lattice angle */
float anga,angb,ang,astar,bstar,gamstr;   /* Lattice params */
float angaSave,angbSave,angSave,
      astarSave, bstarSave, gamstrSave,omegaSave,
      axLSave,ayLSave,bxLSave,dstar,dstarSave,
      byLSave;                            /* Lattice params - Saved  */
float latScrollValue[12];                 /* Lattice params value from scroll*/
int latLineOrder;                         /* Order of latrtice line */
int averageFFT;                           /* Avergae or single fft?? */
int fftZoomFactor;                        /* FFT Zoom factor */
int fftZoomX,fftZoomY;                    /* X,y for fft zoom */
map_t fftMeasures[3];                     /* Measure x,y in FFT */
int fftBkgValue;                          /* FFT Background value for Image */

/* fourAver stuff */
float rasterStep;                         /* Raster Step size */
int backSize;                             /* How large the back transform */
float fourAverRadius;                     
float fourAverCut;                     
int refinePhaseOrigin,imposeSymmetry;
float phaseOrigX,phaseOrigY;
int deltaPhaseShift;
int writeFourAverSF;

/* CTF stuff */
int ctfMode;                              /* CTF correction in what mode? */
int stuffBackground;                      /* Stuff bkg after CTF correct? */
int drawCTF;                              /* Draw CTF or not */
int lopHipSwitch;                         /* Do LOP/HIP or not? */
int measureFFT;                           /* Measure FFT?? */
int ctfCurveNum;                          /* Which CTF is current */
int maxCtfCurve;                          /* What's the MAX ctf curve num? */
ctf_t ctfPts[MAXCTFS];                    /* Holds CTF curve params */
float ctfFocusMinvalue,ctfAngMajorValue,  /* Parameters for refining CTF */
      ctffocusMajValue,     
      ctfMajorDefocusValue,     
      ctfMinorDefocusValue,     
      scanStep, magnification,
      pixSize, kv,
      wavelength, spAberration,
      wiener, angMajor, ctfTFactor, ctfDstar;
int ctfDivideMode;
int doingAutoDefocusRefine, adrCircAvgDraw,adrSDraw,adrSminusBDraw,
    adrCtfsqDraw,adrDDraw,adrBDraw,adrZoomDraw;
int autoDefocus1Dmethod;
float  *ca1d, *ca1d_std;
float  *ca1d_save;
float  *ca1d_cnt,*adr_s,*adr_sminusb,*adr_ctfsq,*adr_d,*adr_b;
float  ca1d_avg;
int doingAuto2DDefocusRefine;
int drawAuto2Dctf,drawAuto2Dscurve,drawAuto2Dctflines;
float *auto2DdrawBuf,*auto2DdataBuf;
float auto2DdefMajor, auto2DdefMinor, auto2Dangle;
float  *a2dr_s,*a2dr_sminusb,*a2dr_ctfsq,*a2dr_d,*a2dr_b;
int a2drSDraw,a2drSminusBDraw,a2drCtfsqDraw,a2drDDraw,a2drBDraw;
double *poly_paras,*fftasum;
int hkl_max, order_b, order_d;
map_t scurveClick;                     /* Measure x,y in FFT */
char  gottenParamFname[1024];


/* FFT measurements */
float fftAngsPerPixel;                    /* How many angstroms per pixel? */
int fftMeasLineNum;                       /* What line num to dump to file? */
int calcFFT1DcircAvg;                     /* Give feedback? */
int disp1Dca;                             /* Disp buffer?? */
int ctfEnhance;                           /* Enhance CTF display ?? */
int bumpContrast;                         /* Bump up the contrast? */

/* ROTFLT stuff */
int idim1Rotflt;                          /* Rotflt size fft */
int lg_end;                               /* Symmetry Limit */
float *rho;                               /* Holds the backtrans */
int rotfltCrosshair;                      /* Plot center crosshair? */
int rotfltResiduals;                      /* Plot residuals contour? */
float powSpec[1000];                      /* The power Spectrum */
int radiusM1;
float yFactor,xFactor;                    /* PowSpec plotting params */
int nrot,nannuli;                         /* Symmetry equals # rotations 
					   * and nannuli*/
int firstRotFlt;                          /* Is this the first pass or what? */
int logROTFLTplot;                        /* Plot power spec in log? */
float fft_step, phi0;                     /* ROTFLT Params */
int  jzero;
float sump1,sump2,sump3;                  /* Azimuthal power stats */
float logCutoff;                          /* Cutoff log plot here */
int backTransROTFLT;                      /* Back transform it ??? */
int stretchRotflt;
float minRotfltScaleF;
int newStart;
float sigMax,sigMaxTot;
int iopt[5];                              /* Menu choices */
int newOrigin;                            /* Calc new origin ??? */
float winScale;                           /* Scale window search for origin */
float xorigSave,yorigSave;                /* Global saved areas */
int saveLogYFactor,saveLinYFactor;        /* Save the concept of yFactor for
					   * both linear and log plots */
float *sigma;                             /* Holds the residuals */


/* crysDisp stuff */
float xorig,yorig;                        /* What is refined origin ? */
int npixx, npixy, ncellx, ncelly;         /* Stuff for mapdsp_gplot_rob() */
float aDscale,gscale;
int x_size;                               /* x_size param */
float aoverbTemp,map_abangTemp;           /* Temp interface holders */
float *crys_float_image_buffer;           /* Put image in tsb format */
int  crysDisping;                         /* Are we trying to act like a
					   * crystal??? */
int  fourierAveragedCrysDisp;             /* Did we get this by Fourier
					     Averaging ??? */
int cdNumCols,cdNumRows,cdNumSecs;
int adim,bdim;
float gammaXtal;


/* UNBLOB stuff */
int minUnblob,maxUnblob;                  /* Contour range for unblob */
int unblobSize;                           /* Size of blobs to kill */
float *unblobBuffer;                      /* Holds unblobbed map */
int radiusUnblob;                         /* Unblob radius */
int cleanUnblob;                          /* Save the largest and kill rest? */

/* CUT stuff */
int cutMinValues[3],cutMaxValues[3];      /* Cut the MAP in x,y,z dimensions*/
float perMinValues[3],perMaxValues[3];    /* For percentage cut */
int reverseSections;                      /* Reverse Sections? */
int negUniverseCut;                       /* Cut out negative Universe */
int whichIcos;                            /* Which icos display */
int destroyIcos;


/* FIXIT stuff */
int maxFixValue,minFixValue,pageFixValue,
    incFixValue;                          /* New params for scroll bar */
int drawFix2Dgrid,drawFix2Dstats;
int flickerFix2D;
float fix2Davg[MAXREGIONS][MAXREGIONS];
float fix2Dstd[MAXREGIONS][MAXREGIONS];

/* FIX values stuff */
int fixMode;                              /* What method of fixing? */
int fixIcosFill;

/* MRC type image file stuff */
float floatScaleFactor;
float recipFloatScaleFactor;
float int4scaleF;
char title[512];
float junkFloat;
int htype,pifMode,nimg,nrec_last, asked;
int rec_length, nrecords, nrows_per_record;
int nrecords_per_row, numRecs, recLength, totLength;
int mrc_nx, mrc_ny, mrc_nz, mrc_mode;
float dela, delb, delc;
float dthe, dphi_90;
int binFactor;
int t_matrix[6];

// VOLUME stuff
float volumeFraction[1024], volumeCutoff[1024];
int   volumeBin[1024], volumeDisp, nbins;

// AUTOBOX stuff
int nThins, nDilates;        // Number of thinning/dilation layers
float prag;                  /* Cross point factor */
float refCutoff;             /* Cutoff to throw out ref correlations */
float refCutoffRatio;        /* Cutoff ratio to throw out ref correlations */
int refCircAvg;                            /* Circavg the ref? */
int refAutomate;                           /* Automate the ref? */
int boxitSize,boxitBoxSize,boxitFeatherSize;
char boxitFname[512];
int autoBoxSize,autoBoxBoxSize,autoBoxFeatherSize;
int autoBoxThin, autoBoxDilates, autoBoxIter;
float autoBoxThresh;
char refReadFileName[512];

// Pointer stuff
int pointColor;
float apOrigXtot,apOrigYtot;

// PARAM FILE stuff
int dispCenters;                           /* Should we disp param centers? */
int haveParamFile,projASU;
float pixSizeP,unitsP,volts,ampFacs,delf_maj,delf_min,ang_maj,cs_coeff;
int totParms;
int icoDispMode;
int whichParam;                  /* Which file param do they want to view? */
int symmetry;                    /* What symmetry do they want to run under? */
float phiMax, theta1, theta2;
int maxPhi;                      /* Max phi in projASU */
int nview;
float *thetas,*phis;
int drawIcos;                    /* Draw the icosahedron?? */
int findNearest;                 /* Find nearest particle??? */
int whichViewMove;               /* Disp movement or view info */
int useOmega;                    /* Use omega when drawing icosahedron? */
int incSort;

int idtoggle, thetatoggle, phitoggle, omegatoggle,
  xtoggle, ytoggle, deltaXtoggle, deltaYtoggle,
  absDeltaXtoggle, absDeltaYtoggle, distancetoggle,
  magtoggle, cc1toggle, cc2toggle, cc3toggle;

float   snumb; // starting value for number of ptles
float        startbox,endbox;    /* Start and End of the range */
int          maxYbox;            /* Max # of particles to disp in Y */
float 	     step; 		 /* Value for the step of the range */
int 	     yscale;		 /* A height scaling variable */
float        avgBadCount;        /* How many HIST values were bad? */
float 	     linelength,pixelstep;
float        ico_toolsScrollValue;    /* Value of the scrollbar */
int icosaMotioning;    /* Are we looking at motion events? */
char inParamFilename[256];
char outParamFilename[256];
char paramBoxesFilename[256];
char paramMapFilename[256];
char paramScanFilename[256];
char paramBatchJobFilename[256];
char paramBoxCoordsFilename[256];
char *batchListStr[MAXPACKEDFILES];
int totParamFiles;
int newXY;             /* Are they trying to locate a new x,y center? */
P_INFO *foundPart;     /* The one they found */
int icoSortLevel;      /* Shows progress of sorting */
int mapProj,mapProjContour;    /* Are we doing map Projections or not? */
float map_alp, map_bet, map_gam;
int mapCols, mapRows, mapSecs;
int scanCols,scanRows;
int projASUdeltaAng;
float allMax;
float deltaStdX,deltaStdY;
float minXYplot,maxXYplot;
int doManipPP,doCavgPP;
int cavgPPcenter;
float *cavgPPrint[MAXBOX];
int totCavgPPrint;

// CTf fit 1d and 2d
int    ctf1dCalcIt, ctf2dCalcIt;
float *allPackImg;  
float *ctf1dSaveFftAvg;
float *ctf2dSaveFftasum,*ctf2dSaveFftasumsq;

/*****************************************************
 * emmap3dt/emsf3dbt stuff 
 *****************************************************/
strucFact_t *sFacs;
strucFact_t *fa_sFacs;
forPifHead_t faHeader;
float vol,vol_star;
float a_cell,b_cell,c_cell;
int   nsf, origNsf;
int   calcLowResMap;
int   lowRes;
float *lowRes_buffer1;
int   lowResCols,lowResRows,lowResSecs;
float pixelSizeFactor,cutFactor;
int   calcSFs,calcBin,truePhi, forceByte,forceAvgBkg,minMaxColor;
int   forceAvgBkgSwitch;
float pixelSizeSave, cutMin0Save, cutMax0Save, cutMin1Save,
      cutMax1Save, cutMin2Save, cutMax2Save;


/* Subtract 3D Map stuff */
int subtractingMap,subMap2d3d;
char subtractFilename[256];
int sub2binCol,sub2binRow,sub2binSec;
float *mags,*coefs,*cntr1,*cntr2;
int   magchk,saveMagchk,numRad;
int   subCanDrawWindow;                 /* Suppress drawing at times */
int   submapFindRad;                    /* Draw radius or mag??? */
int   rdensit, wrdensit, ccoefit, wccoefit,s_wccoefit,flickerIt;
float *rdens,*wrdens,*ccoef,*wccoef,*s_wccoef;
float radstep,diffSecScrollValue,diffSec2ScrollValue;
float magSecScrollValue,threshScrollValue;
float radScaleM,radScaleB,saveBmag;
int   thickStepFR,thinStepFR;
char  RDoutStr[128],WRDoutStr[128],CCoutStr[128],
      WCCoutStr[128],S_WCCoutStr[128];
int   drawingFindRad,diffSection;
float saveMinMapPixel,saveMaxMapPixel;
int   saveSecondMap,showRadii,threshDoit,threshAbs;
int   icosaQuarter,findRadHist_toggle,findMagHist_toggle;
int   normalExcise;

// Imgconvstuff
char  imgconvFilename[256];
int   readImgconv, imgconvNcol, imgconvNrow;
float imgconvBkg;

// Emcoravgstuff
char  emcoravgFilename[256];
int   readEmcoravg, emcoravgNcol, emcoravgNrow;
int   coravgRadius, lastPlotClickX, lastPlotClickY;
int   useAvgCorAvg,showLowLim;
float emcoravgBkg, coravgLowLimValue;

// Normalize stuff
int   kindOfNormalize;
float normLinMean,normLinStd;

// Partproc stuff
int readPackedFile;

// Fix stuff
float batchFixStd;

// ApplyCTF stuff
int   applyCtfFilter, applyCtfMode;
float applyCtfWiener, applyCtfTFactor, applyCtfAngMajor,
  applyCtfPixelSize, applyCtfDefocus_in, applyCtfDefocus_minor_in,
  applyCtfAmpContrast_in, applyCtfSphericalAbbr,
  applyCtfVolts;  // int in volts NOT kiloVolts

// Noise stuff
int noiseType;
float noiseStd;

// Trunc stuff
int truncBegX, truncEndX, truncBegY, truncEndY, truncBegZ, truncEndZ;
float truncVal;

// Binning stuff
int doBinning;

// Interp stuff
float doInterpOrigSize, doInterpOutputSize;

// Model stuff
int    modelType, mbImmediate, isIcosRepl;
float *model_undo_buffer, modelFillValue;

// robem widget declarations

Widget  
  binningScreenW,  // Widgets for binning screen (see binning.c)
  binningDoW, binningHelpW, binningExitW,
  binningLabelW,
  binningRadioW,
  binningToggle2W, binningToggle3W, binningToggle4W,

  invertScreenW,   // Widgets for invert screen (see invert.c)
  invertExitW,
  invertXW, invertYW, invertZW,

  rotateScreenW,   // Widgets for rotate screen (see rotate.c)
  rotateDoW, rotateHelpW, rotateExitW,
  rotateThetaW, rotatePhiW, rotateOmegaW,
  rotateThetaLabelW, rotatePhiLabelW, rotateOmegaLabelW,

  normalScreenW,   // Widget for normalization screen (see normal.c)
  normalDoW, normalHelpW, normalExitW,
  normalMeanW, normalMeanLabelW,
  normalStdW, normalStdLabelW,

  rescaleScreenW,  // Widget for rescale screen (see rescale.c)
  rescaleExitW, rescaleDoW,
  rescaleOldMinW, rescaleOldMaxW,
  rescaleNewMinW, rescaleNewMaxW,

  statScreenW,     // Widget for statistics screen (see stat.c)
  statExitW, statTitleW, statTextW,
  statCalcW, statHeadW, statSaveW,

  circAvgScreenW,  // Widget for circular avg screen (see circavg.c)
  circAvgExitW,
  circAvgClearW, circAvgPrintW, circAvgPifoutW,
  circAvgLineW[4], circAvgLabelW[4], circAvgZoomW,
  circAvgListW,
  circAvgTextW,
  circAvgToggleW,
  circAvgW,
  circAvgScrollW,
  circAvgUpW, circAvgDownW,

  appodizeScreenW, // Widget for appodization screen (see appod.c)
  appodizeExitW, appodizeHelpW, appodizeDoW,
  appodBoxRadW, appodFeaRadW, appodBoxRadLW, appodFeaRadLW,
  apFeaLabelW, apFeaRadioW, apOldW, apExponentialW, apGaussianW, apLinearW,
  apBkgLabelW, apBkgRadioW, apPickW, apCircleW, apRectW,

  truncaddScreenW, // Widget for trunc/add screen (see truncadd.c)
  taTitleW, truncaddExitW, truncaddHelpW, truncaddDoW,
  taFillW, taFillLabelW,
  taDimXW, taDimYW, taDimZW,
  taBegXW, taBegYW, taBegZW, // truncadd text widgets for begin indices
  taEndXW, taEndYW, taEndZW, // truncadd text widgets for end indices
  taXLabelW, taYLabelW, taZLabelW, // labels for indices
  taFrontW, taBackW, taLeftW, taRightW, taTopW, taBottomW, taWholeW,
  gbsLabelW, gbsUpArrowW, gbsDownArrowW,

  autoDRScreenW, // Widget for auto defocus refinement screen (see autodr.c)
  autoDRHelpW, autoDRExitW,
  adrHeaderW, adraW[6], adrbW[6], adraLW[6], adrbLW[6],
  adrDefocusW, adrChisqW, adrDefocusLW, adrChisqLW,
  adrPlotLW, adrCircW, adrSW, adrSMBW, adrCtf2W, adrDW, adrBW,
  adrZoomW, adrHiResW, adrHiResLW,
  adrDoitW, adrDoit10W, adrabResetW, adrUpdateW, adrPIFW,

  dioScreenW, // Widget for icosahedral proj parameters (see dio.c)
  dioExitW, dioHelpW,
  dioThreshTextW, dioSlideTextW, dioThreshLabelW, dioSlideLabelW,
  dioControlsW, dioFaceW, dioAsuW, dioBulgeW, dioNormW, dioDestW,

  fix2DScreenW, // Widget for blemish removal (see fix2d.c)
  fix2DExitW, fix2DHelpW,
  fix2DRegionsTextW, fix2DRegionsLabelW,
  fix2DStdTextW, fix2DStdLabelW,
  fix2DDoW, fix2DPreviewW, fix2DRestoreW, fix2DFlickW,
  fix2DTitleW, fix2DGridW, fix2DStatsW,

  pointScreenW, // Widget for point screen
  pointLabelW[7],
  pointExitW, pointColorW, pointCenterW, pointHistW,

  manipPPScreenW, // Widget for manipulate ptle parameter screen
  manipPPExitW, manipPPHelpW, manipPPDoW,
  manipPPValueW, manipPPRadioW, manipPPRandW, manipPPUnkW,

  cavgPPScreenW, // Widget for cavg ptle parameter screen
  cavgPPExitW, cavgPPHelpW, cavgPPDoW,
  cavgPPAoptW, cavgPPAopt1W, cavgPPAopt2W,
  cavgPPCoptW, cavgPPCopt1W, cavgPPCopt2W,
  cavgPPNflW, cavgPPNftW,

  boxScreenW, // Widget for ptle boxing screen
  boxExitW, boxHelpW, boxStopW, boxcolorPushW,
  boxThickW, boxShowrefW, boxDispnumW,
  boxReffeedW, boxShowclW, boxCycleW,
  emmaRadiusTextW, emmaRadiusArrowUpW, emmaRadiusArrowDownW,
  emmaRadiusLabelW, emmaRadiusTogW,
  featherRadiusTextW, featherRadiusArrowUpW, featherRadiusArrowDownW,
  featherRadiusLabelW, featherRadiusTogW,
  boxRadiusTextW, boxRadiusArrowUpW, boxRadiusArrowDownW,
  boxRadiusLabelW, boxRadiusTogW,
  emptyShellTextW, emptyShellArrowUpW, emptyShellArrowDownW,
  emptyShellLabelW, emptyShellTogW,
  dilateLabelW, dilateScrollW, dilateValueW,
  thinLabelW, thinScrollW, thinValueW,
  timeLeftBoxW, squareTextW,
  boxTypeLabelW, boxRadioW, boxRectW, boxSquareW, boxCircleW, boxPolyW,
  autoBoxW, correlateButtonW, correlateButton2W, emptyBoxW, clearBoxW,
  autoBoxLabelW, refCorrLabelW, autoCorrLabelW,
  autoMinMaxLabelW, autoMinLabelW, autoMaxLabelW, 
  minBoxScrollW, maxBoxScrollW,
  autoMenuBarW, autoMenuEntryW, autoMenuW, 
  autoBycW, autoByaW, autoBysW, autoThreshW,
  showBoxingStatsToggleW, boxGrid2W,

  versionScreenW, // Widget for version screen
  versionExitW, versionStopW, versionLabelW,
  versionScrollW, versionTextW, versionUpW, versionDownW,

  fixScreenW, // Widget for fixing values in screen
  fixExitW, fixHelpW, fixFormW, fixDoW,
  fixControlsW, fixAddW, fixMulW, fixOvrW, fixInvW, fixShtW, fixGsnW, 
  fixMinW, fixMaxW, fixTextW, fixMinLW, fixMaxLW, fixTextLW, 
  fixBegXW, fixBegYW, fixBegZW, fixBegXLW, fixBegYLW, fixBegZLW,
  fixEndXW, fixEndYW, fixEndZW,
  fixGaussStdTextW, fixGaussStdLabelW,

  helpScreenW, // Widget for Help screen
  helpExitW, helpTextW, helpSaveW,

  statusScreenW,  // Widget for status screen
  statusTextW, statusStopW,

  infoAnswerScreenW, // Widget for info answer screen
  infoListW, infoPromptTextW,

  infoBoxW, // Widget for info message box

  colorScreenW, // Widget for colormap editor
  colorExitW,
  colorRadio1W, colorGnW, colorRvW, colorFtW, colorRnW, colorBdW,
  colorRdW, colorWrW, 
  colorRadio2W, noContrastW, useContrastW, histEqualW,
  colorLabelW[8], colorValueW[8], colorScrollW[8],
  stepCMLabelW, stepCMValueW, stepCMScrollW,
  radialValueW, colorHistUpW, colorHistDnW,
  colormapW, histW, colormapScrollW, colormapScrollDownW,
  testcolormapW, colorRampW, colorRbowW, 

  interpScreenW, // Widget for interpolation screen
  interpHelpW, interpExitW, interpDoW,
  interpOutTextW, interpOrigTextW, interpOutLabW, interpOrigLabW, 

  movementScreenW, // Widget for movement screen
  movementTextW, diamTextW, movementLabelW, moveExitW,

  fftFeedbackScreenW,  // Widget for FFT feedback screen
  fftFeedbackExitW, newXOrigW, newYOrigW, 
  apOrigDoW, apOrigResetW, apOrigLabel1W, apOrigLabel2W,
  fftnewXtextW, fftnewYtextW,
  fftFeedbackLabel1W, fftFeedbackLabel2W, fftFeedbackLabel3W,

  domegaScreenW, // Widget for delta omega PP screen
  domegaExitW, domegaHelpW, domegaLabelW, domegaTextW,
  domegaDoW, domegaOldW,

  dxyScreenW, // Widget for delta XY screen
  dxyExitW, dxyHelpW, dxyDoW,
  dxyXTextW, dxyYTextW, dxyXLabelW, dxyYLabelW,

  optionsScreenW, // Widget for option screen
  optionsExitW,
  minMapTextW, maxMapTextW, minMapLabelW, maxMapLabelW,
  optionsBkgW, optionsBkgrW, optionsAvgW, optionsRanW,
  optionsDspW, optionsCirW, optionsBoxW, optionsZoomW,
  optionsExW, optionsFftW,
  calcSfsW, calcBinW, truePhiW, forceByteW, forceAvgBkgW, minmaxColorW,

  fixitScreenW, // Widget for fixit screen
  fixitLabelW, fixitDoitW, fixitExitW,
  minFixTextW, maxFixTextW, pageFixTextW, incFixTextW,
  minFixLabelW, maxFixLabelW, pageFixLabelW, incFixLabelW,

  subScreenW, // Widget for subtractmap screen
  subExitW, subDoW, sub2dW, sub3dW, subRenderW, subHelpW,
  subIcosW, subS2ndW,
  subMagsecTW, subMagsecLW, subMagScrollW, subMagSaveW, subMagResetW,
  subMagLW, subRadFW, subRadrdW, subRadwrW, subRadccW, subRadwcW,
  subMagFW, subRadLW, subMagMinRadW, subMagMaxRadW,
  subMagFindW, subMagHistW, subMag1ResetW, subMag2ResetW, subMag3ResetW,
  subMagStartLW, subMagStartTW, subMagStepLW, subMagStepTW,
  subMagNumLW, subMagNumTW,
  subRadMinTW, subRadMaxTW, subRadMinLW, subRadMaxLW,
  subRadRadiiW, subEquationW, subRadBW, subRadDW, subRadRW,
  subRadMinUpW, subRadMinDownW, subRadMaxUpW, subRadMaxDownW,
  subSmMinLW, subSmMaxLW, subSmResetW,
  subFile1LW, subFile1TW, subFile2LW, subFile2TW, subFile2BW,
  subDiff1LW, subDiff1TW, subDiff2LW, subDiff2TW, subDiffBW,
  subDiffSW, subDiffS2W, subDiffSBW, subDiffRBW,
  subFlickSW, subFlickMW, subFlickIW, subFlickUpW, subFlickDownW,
  subFlickTypeW, subFlick12W, subFlick13W, subFlick23W,
  subThreshTW, subThreshLW, subThreshSW, subThreshDoW, subThreshAbsW,

  emcorAvgScreenW, // Widget for emcoravg screen
  emcorAvgExitW, emcorAvgTryW, emcorAvgSaveW,
  emcorAvgZLW, emcorAvgZTW, emcorAvgRLW, emcorAvgRTW,
  emcorAvgDrawW, emcorAvgUAW, emcorAvgDAW, emcorAvgFileW,
  emcorAvgLLLW, emcorAvgLLVW, emcorAvgLLSW, emcorAvgLLSTW, emcorAvgLLUTW,

  refScreenW, // Widget for reference based boxing screen
  refAllW, refDarkTW, refCircAvgW, refAutomateW,
  refReadfileW, refResetW, refRegridW, refTextW,
  refIterLW, refReadLW, refIterTW, refDoW,

  movconScreenW, // Widget for movie contour screen
  movconExitW, movconResetW, movconClearW, 
  movconUseW, movconWriteW, movconTryW,
  movconBrkW, movconTogW, movconTextW, movconBrkLW, movconConLW,
  movconScrollLW, movconNumLW, movconNumScrLW, movconDelLW, movconDelScrLW,
  movconScrollW, movconNumScrollW, movconConScrollW,

  mapShadeScreenW, // Widget for mapshade screen
  secondShadeW, immediateUpdateW, stereoShadeW, 
  radialShadeW, radProjShadeW, 
  mapShadeExitW, mapShadeStopW, mapShadeDoW, 
  mapShadeHelpW, mapShadeCueW, irevToggleW,
  radialCSToggleW, radialLabelW, radialTextW,
  radProjCSToggleW, radProjToggleW, icosRadProjToggleW,
  movieFormW, movieFormLW, numFramesLabelW, stepValueLabelW,
  stepTextW, frameTextW, stepToggleW, stepToggle2W,
  cutFormW, cutLabelMinW, cutLabelMaxW, cutLabelXW, cutLabelYW,
  cutLabelZW, cutMinValueW[3], cutMaxValueW[3], perValueW[3],
  cutFormLabelW, cutMinScrollW[3], cutMaxScrollW[3],
  maxxToggleW, maxyToggleW, maxzToggleW,
  unblobFormW, unblobSizeLW, unblobRadiusLW, unblobFillLW,
  unblobRadiusValueW,  unblobSizeValueW,  unblobFillValueW,
  unblobSizeScrollW, unblobRadiusScrollW, unblobToggleW,
  unblobGoldW, unblobResetW, unblobsizeToggleW, unblobradiusToggleW,
  mapIcosTogW, icosaUpW, icosaDownW,
  foldMenuBarW, foldMenuW, foldMenuEntryW, fold5W, fold3W, fold2W,
  hardRotateButtonW, sampleLW, sampleRenderW, sampleUW, sampleDW,
  loHiResW, volumeValueW, volumeLabelW, volumeToggleW,
  timeLeftLabelW,
  surfDensLabelW, surfDensScrollW, surfDensValueW, surfDensToggleW,
  fractLabelW, fractScrollW, fractValueW, fractToggleW,
  angxLabelW, angxScrollW, angxValueW, angxToggleW,
  angyLabelW, angyScrollW, angyValueW, angyToggleW,
  thetaButtonW, thetaScrollW, thetaValueW, thetaToggleW,
  phiButtonW, phiScrollW, phiValueW, phiToggleW,
  omegaButtonW, omegaScrollW, omegaValueW, omegaToggleW,
  minRadiusLabelW, minRadiusValueW, minRadiusScrollW, 
  maxRadiusLabelW, maxRadiusValueW, maxRadiusScrollW, maxRadiusToggleW,
  minRadiusAngW, maxRadiusAngW,
  pixelSizeLabelW, pixelSizeValueW, pixelSizeScrollW, pixelSizeToggleW,

  batchScreenW, // Widget for batch job screen
  batchModeW, batchBinW, batchSymW, batchDtheW, batchDphi90W,
  batchIlistW, batchRadLoW, batchRadHiW, batchRadStepW, batchAnnLoW, 
  batchAnnHiW, batchResLoW, batchResHiW, batchJCutW, batchSigW,
  batch3DMapW, batchPrjsFileW, batchPftsFileW,
  batchMagCenW, batchMagStepW, batchMagNumW, batchMagNormW,
  batchExitW, batchBrowseW, batchSubmitW, batchASUW,
  batchFileLabelW, batchFileTextW, batchHelpLabelW,
  batchTextW, batchParamFileW,

  modelScreenW, // Widgets for model builder screen
  modelExitW, modelHelpW, modelDoW, modelClearW, modelUndoW, modelSaveW,
  modelChooseLW, modelLabelW,
  mbImmediateUpdateW, mbReplW,
  modelMenuW, modelMenuBarW, modelMenuEntryW,
  modelCubeW, modelSphereW, modelTriW, modelCylW, modelIcosW,
  modelFillLW, modelFillTW,
  modelCenLW, modelXLW, modelYLW, modelZLW,
  mbCenterxTextW, mbCenteryTextW, mbCenterzTextW,
  mbCenterxScrollW, mbCenteryScrollW, mbCenterzScrollW,
  modelDimLW, modelWLW, modelHLW, modelDLW,
  mbDimensionxTextW, mbDimensionyTextW, mbDimensionzTextW,
  mbDimensionxScrollW, mbDimensionyScrollW, mbDimensionzScrollW,
  modelRotLW, modelThetaLW, modelPhiLW, modelOmegaLW,
  mbRotatetTextW, mbRotatepTextW, mbRotateoTextW,
  mbRotatetScrollW, mbRotatepScrollW, mbRotateoScrollW,

  fourFiltScreenW, // Widgets for Fourier Filter screen
  fourFiltExitW, fourFiltHelpW, fourFiltDoW, fourFiltLabelW, 
  fourFiltRNLW, fourFiltRXLW, resMinTextW, resMaxTextW,
  fourFiltCNLW, fourFiltCXLW, cutMinTextW, cutMaxTextW,

  crysDispScreenW, // Widgets for Crystal display screen
  crysDispResetW, crysDispExitW, crysDispImmW, crysDispABW,
  crysDispLabelW, 
  crysDispGamLW, map_abangTextW, mapScrollW, mapValueW,
  crysDispABLW, aoverbTextW, abScrollW, abValueW,
  crysDispZoomLW, cDispZoomArrowUpW, cDispZoomArrowDownW, scaleXvalueW,
  crysDispXLW, crysDispYLW, cDispSymmOrigXW, cDispSymmOrigYW,
  crysDispMenuW, crysDispMenuBarW, crysDispMenuEntryW,
  crysDispP1W, crysDispP2W, crysDispPmW, crysDispPgW, crysDispCmW,
  crysDispP2mmW, crysDispP2mgW, crysDispP2ggW, crysDispC2mmW, crysDispP4W,
  crysDispP4mmW, crysDispP4gmW, crysDispP3W, crysDispP3m1W, crysDispP31mW,
  crysDispP6W, crysDispP6mmW,
  crysDispXtalLW, crysDispNcellxLW, crysDispNcellyLW,
  crysDispAdimLW, crysDispBdimLW, crysDispGLW,
  ncellxScrollW, ncellyScrollW, adimScrollW, bdimScrollW, gammaXtalScrollW,
  ncellxvalueW,  ncellyvalueW, adimTextW, bdimTextW, gammaXtalTextW,

  imgconvScreenW, // Widgets for image convolution screen
  imgconvExitW, imgconvTryW, imgconvHelpW, imgconvFileW, imgconvLabelW, 

  partprocScreenW, // Widgets for particle processing screen
  paramFileLabelW, paramFileTextW, paramFileBrowseW,
  paramMapFileLabelW, paramMapFileTextW, paramMapFileBrowseW,
  paramScanFileLabelW, paramScanFileTextW, paramScanFileBrowseW,
  ppCavgBW, ppDoaBW, ppMppBW, ppDxyaBW, ppRbAllBW, ppRbOneBW, ppWnewBW,
  partprocExitW,
  parmheadW, phPixLW, phVoltLW, phAmpLW, phMajLW, phMinLW,
  phAngLW, phCsLW, phUnitLW,
  pixsizeW, unitsW, voltsW, ampfacsW, delfMajW, 
  delfMinW, angMajW, csCoefW,
  partprocMenuBarW,
  ppHelpMenuW, ppHelpMenuEntryW, ppHelpUnitBW, ppHelpXYBW, ppHelpHistBW,
  ppBatchMenuW, ppBatchMenuEntryW, ppBatchBatchBW,
  ppCalcMenuW, ppCalcMenuEntryW, ppCalcMoveBW, ppCalcViewBW, ppCalcProjBW,
  ppSymmMenuW, ppSymmMenuEntryW, ppSymm532BW, ppSymm52BW, ppSymm5BW,
  ppDispMenuW, ppDispMenuEntryW, ppDispHistBW, ppDispUnitBW, 
  ppDispSortBW, ppDispPlotBW, ppDispCavgBW,
  ppDispCentW, ppPickNewW, mapProjW, ppProjContW, projContourTextW,
  ppinTextW, ppinUpW, ppinDownW, icosaW, partPointTextW,
  parmTogsW, parmTogsLabelW, incDecW, idtW, thetatW, phitW, omegatW,
  xtW, deltaXTW, absDeltaXTW, ytW, deltaYTW, absDeltaYTW,
  distanceTW, magtW, pftccW, prjccW, cmpccW,
  histControlsW, histBinLW, histStepLW, histStartXLW, histEndXLW, histMaxLW,
  binTextW, histStepTextW, strtXPPW, endXPPW, maxYPPW,
  binPPUpW,  binPPDownW, stepPPUpW, stepPPDownW,
  projasuW, projasuTextW, projasuLabelW, projasuOneW,
  thetaProjLW, phiProjLW, omegaProjLW, thetaProjW, phiProjW, omegaProjW,
  ppMinMaxFormW, ppMinMaxLW, ppMinTextW, ppMaxTextW, 
  ppMinScrollW, ppMaxScrollW,

  rotfltScreenW, // Widgets for rotational filtering screen
  rotfltExitW, rotfltHelpW, rotfltWriteW,
  rotfltImgNumW, rotfltImgNumLW, rotfltImgnumArrowUpW, rotfltImgnumArrowDownW,
  rotfltDrawW,
  xscaleScrollW, yscaleScrollW, xscaleLabelW, yscaleLabelW,
  refineButtonW,
  nrotDescW, nrotScrollW, nrotLabelW,
  lgendDescW, lgendScrollW, lgendLabelW,
  nannuliDescW, nannuliScrollW, nannuliLabelW,
  phi0DescW, phi0ScrollW, phi0LabelW,
  fftStepLabelW, fftStepTextW, tryitButtonW,
  rotfltStretchW, backTtoggleW, rotfltXhairW, rotfltContW, jZerotoggleW,
  rotfltXorigLW, rotfltYorigLW, xorigTextW, yorigTextW,
  xorigUpArrowW, xorigDownArrowW, yorigUpArrowW, yorigDownArrowW,
  rotfltResetW, rotfltResetLW, refineResetLabelW, refineResetButtonW,
  rotfltRadioW, logROTFLTtoggleW, linROTFLTtoggleW,

  editBoxScreenW, // Widgets for editbox (movie) screen
  editBoxW, editBarW,
  boxUpW, boxDownW, editBoxTextW, movieDoneW, lightSourceLabelW, movieCBW,
  movieZLW, editBoxZoomTextW, editBoxZoomUpW, editBoxZoomDownW, movieSaveW,
  paintFormW, paintFormLW, editBoxPaintTextW, paintUpW, paintDownW,
  undoW, savePaintW, minLabelW, maxLabelW, movieMinW, movieMaxW,
  playFormW, movieArrowUpW, movieArrowDownW, flashLightW, rockW,
  playW, stopW, infiniW, 
  
  fourAverScreenW, // Widgets for fourier average screen
  fourAverExitW, fourAverHelpW, fourAverSfTogW,
  fourAverLastW, refineLabelW, imposeLabelW,
  refineMenuBarW, refineMenuW, refineMenuEntryW, 
  refine2W, refine3W, refine4W, refine6W,
  imposeMenuBarW, imposeMenuW, imposeMenuEntryW, 
  imposep2W, imposepmW, imposepgW, imposecmW, imposepmmW, imposepmgW, 
  imposepggW, imposecmmW, imposep4W, imposep4mW, imposep4gW, imposep3W, 
  imposep3m1W, imposep31mW, imposep6W, imposep6mW, 
  fourAverResetW, fourAverResetLW, phaseOrigLW,
  moveOrigXlabelW, moveOrigYlabelW, moveOrigXW, moveOrigYW, moveOrigBW,
  phasXupW, phasXdownW, phasYupW, phasYdownW,
  shiftOrigXW, shiftOrigYW, shiftOrigXlabelW, shiftOrigYlabelW, shiftOrigTW,
  fourAverRadiusLW, fourAverRadiusTW, fourAverCutLW, fourAverCutTW,
  fourAverRasterLW, fourAverRasterTW,

  auto2DDRScreenW, // Widgets for 2D auto defocus refinement
  auto2DDRHelpW, auto2DDRExitW, auto2DDRDoW, auto2DDRUpdateW,
  auto2DDRFMajLW, auto2DDRFMinLW, auto2DDRAngLW,
  auto2DDRResLW, auto2DDRChiLW,  a2ddrWarnW,
  a2ddrfjW, a2ddrfnW, a2ddragW, a2ddrtfW, a2ddrcsqW,
  a2ddrTog1W, a2ddrTog2W, a2ddrTog3W, a2ddrSW, 
  a2ddrSMBW, a2ddrCTF2W, a2ddrDW, a2ddrBW, 

  fftScreenW, // Widgets for FFT screen
  fftExitW, fftHelpW,
  fftDrawingW, contourW, fftFeedbackW, overlayLW, overlaySW,
  fftZoomToggleW, fftContourTW,   backFFTToggleW, fft1dcircW,
  drawFFTLiteButtonW, drawFFT2LiteButtonW,
  latResetLabelW, latResetButW,
  displayFFTFormW, // Display controls subform (upper right)
  linlogRadioW, linearFFTtoggleW, logFFTtoggleW, nonlinFFTtoggleW,
  fftsizeRadioW, a1024FFTW, a512FFTW, a256FFTW, a128FFTW,
  ampFormW, amplitudeToggleW, recalcToggleW,
  disp1024W, fftLockW, dispMinLW, dispMaxLW, dispTfacLW,
  minFFTscrollW, maxFFTscrollW, tfacFFTscrollW,
  minFFTW, maxFFTW, tfacFFTW, theominFFTW, theomaxFFTW,
  fftMinFixW, fftMaxFixW,
  fftMenuBarW, fftMenuW, fftMenuEntryW,
  fftLopHipBW, fftCtfEstBW, fftLatRefBW, fftRotFltBW, fftMeasBW,
  fftWriteFFTBW, fftWriteBackBW, fftWriteCtf1BW, fftWriteCtf2BW,
  fftWriteCtf3BW, fftWriteCtf4BW, fftReadFFTBW,
  fftMeasureFormW, // Measure controls (middle right)
  fftMeasurePSTextW, fftMeas1W, fftMeas2W, fftMeas3W, fftMeas4W,
  fftImagesFormW,  // Image control (bottom center)
  fftImagesTitleW, fftImgnumArrowUpW, fftImgnumArrowDownW,
  fftImgNumW, fftImgNumLW, fftAvgBW, fftStopW, fftCtfEnhW,
  fftFeedbackFormW, // Measurement feedback form
  fftFeedTitleW, fftMeasPixW, fftMeasAngW,
  lophipFormW, // Low/high pass controls
  lophipLoLW, lophipHiLW, lopRadiusTextW, hipRadiusTextW,
  lophipL1W, lophipL2W, lophipL3W, lophipL4W, lophipL5W,
  lopScrollW, hipScrollW, lopTextW, hipTextW,
  ctfContFormW, // CTF operations form
  ctfFullW, ctfEmptyW, ctfReadW, ctfAutoW, ctfAuto2W,
  ctfParamsW, // CTF parameters form
  ctfScanLW, ctfPixLW, ctfAmpLW, ctfVoltsLW, ctfCsLW, ctfWienerLW,
  ctfTfacLW, ctfModeLW, ctfWaveLW, ctfWaveW, ctfMagLW,
  ctfScanStepW, ctfPixSizeW, ctfKvW, ctfAmpFactorW, ctfSpAberW,
  ctfMajorDefocusW, ctfMinorDefocusW, ctfWienerW, ctfAngMajorW,
  ctfTFactorW, ctfModeW, ctfMagW, ctfApplyW,
  scopesMenuBarW, scopesMenuW, scopesMenuEntryW,
  fegBW, feg300BW, em420BW, em420lBW, em300BW, htfegBW, 
  uhrfegBW, jeol1010BW, h9000BW, hf2000BW, polaraBW, spheraBW, 
  ctfinvMenuBarW, ctfinvMenuW, ctfinvMenuEntryW,
  ctfinv1BW, ctfinv2BW, ctfinv3BW, ctfinv4BW, ctfinv5BW,
  ctfCurveFormW, // CTF node form
  ctfCurveTitleW, ctfCurveUpW, ctfCurveDownW,
  ctfMeasureLabelW, ctfMeasureLabel2W, ctfCurveTextW,
  ctfCurveOutBW, ctfCurveInBW,
  ctfScrollsFormW, // CTF scrolls form
  ctfForce1dTW, pickCTFtoggleW, ctfFlickerW, ctfScrollLW,
  ctfFminLW, ctfFmajLW, ctfAmajLW,
  ctfFminSW, ctfFmajSW, ctfAmajSW,
  ctfFminTW, ctfFmajTW, ctfAmajTW,
  latPointsFormW, // Lattice points form
  latPointsTitleW, latPointsRadioW, layerLinesTW, twodLinesTW,
  millerLabelW, lat1LabelW, lat2LabelW,
  lat1XtextW, lat1YtextW, lat2XtextW, lat2YtextW,
  pick2LW, drawLatToggleW,
  latTogglesFormW, // Lattice toggles form
  latTogglesTitleW, filterFFTtoggleW, filtDescW, filtSizeW,
  filtLabelW, hardSoftRadioW, hardToggleW, softToggleW,
  drawHiLoToggleW, latTogglesDoW,
  latticeScrollW[16], latticeLabel1W[16], latticeLabel2W[16],

  mainWindowW, // Widgets for main robem screen
  mainMenuBarW, mainToolFormW, drawingW, vScrollW, hScrollW,
  mfileMenuW, mfileMenuEntryW,                           // File entry
  mreadMenuW, mreadMenuEntryW,                           // Read entry
  readAllW, readMrcW, readSfW, readBoxW, readStateW,
  readArchiveW, readTestW,
  mwriteMenuW, mwriteMenuEntryW,                         // Write entry
  writeCircW, writeRectW, writeBoxW, writeScanW, 
  writeMapW, writeStateW, rawButtonW, pcxButtonW, 
  mappMenuW, mappMenuEntryW, appCircW, appRectW,         // Append entry
  exitButtonW,
  mmanMenuW, mmanMenuEntryW,                             // Manipulate entry
  fixButtonW, removeBlemishButtonW, rotateButtonW, truncaddButtonW,
  invertButtonW, rescaleButtonW, subtractMapButtonW, calcSFButtonW,
  editBoxButtonW, paintMapButtonW, normalizeW, normalizeBkgW,
  linGradW, checkBkgW, fourFiltW, addAverageW, interpolateW,
  binningW, modelW,
  mdispMenuW, mdispMenuEntryW,                           // Display entry
  colorButtonW, statsW, crysW, circAvgButtonW,  
  mapShadeButtonW, mapMovieButtonW,
  mfftMenuW, mfftMenuEntryW, fftButtonW,                 // FFT entry
  mccMenuW, mccMenuEntryW, emcorButtonW, imgconvButtonW, // CC entry
  mpartMenuW, mpartMenuEntryW, partButtonW,              // PartProc entry
  moptMenuW, moptMenuEntryW, optButtonW,                 // Option entry
  zoomArrowUpW, zoomArrowDownW, zoomTextW,  // Start toolbar widgets
  drawLiteButtonW, subToolFormW, mainRadioW,
  pointToolW, boxToolW, zoomToolW, sectionW, imgNumLabelW,
  imgNumTextW, imgnumArrowUpW, imgnumArrowDownW, lockW,
  boxInitLabelW, boxInitW, boxinitArrowUpW, boxinitArrowDownW,
  versionToolW, helpToolW,
  fileSelW,

  orderBkgW, orderSigW,
  widgetList[20], fixitW, toplevel_widget;

// robem function prototypes

extern void dph_nomem_tilt_fast(float *coremap, 
				float *origMap, 
				int ncol, int nrow, 
				float *dphshd_map, 
				BYTE shift, BYTE surf, 
				int sec1, int sec2, 
				int col1, int col2,
				int row1, int row2,
				int num_views, 
				int irev, float surf_dens, int useForMinMax,
				float *shadMap,
				double thetaX, double thetaY,
				double minRadius, double maxRadius,
				double xCen, double yCen, double zCen,
				float radialXcen, float radialYcen, 
				float radialZcen,
				int lowRes, int topRadialSec);

extern void rotx(float *y,float *z,float xang);
extern void roty(float *x,float *z,float yang);
extern void rotz(float *x,float *y,float zang);

void frr(float *x, float *y, float *z) ;

void init_intarr(int *arr, int count, int value);
void init_floatarr(float *arr, int count, float value);
void init_doublearr(double *arr, int count, double value);

void drawCtfCurve(float *farr, float shift, float minCA,
		  float scale, int xfac, GC color);

double wstrtod(Widget wid);

int getBorderType(int mode);

void writePCX(char *filename, int startImg, int endImg, int movie);

void farray_minmax(float *farr, float *min, float *max, int n);

void ctf_nodes_misha(float wavelength, float focus,
		     float cs_coef, float amp_fac, int node,
		     float *d_star);

void ctf_focus_misha (float wavelength, float *focus,
		      float cs_coef, float amp_fac, int node,
		      float d_star);

void convert_abpart(float apart, float bpart, 
		    float *amp, float *phi);

void fft_point (float *fft, int idim1, int idim2, int ix, int iy,
		float *radius, float *apart, float *bpart);

void fft_setdim_local (int ncol, int nrow, int *idim1, int *idim2);

void map_clear_local(float *map, int ncol, int nrow);



void createDefaultColormap();
void em_sleep(int i);
void calcCircleStats(float *inbuffer, 
		     float *img_bkg, float *img_min,
		     float *img_max, float *img_avg, float *img_std,
		     int width, int height, int boxWidth);

int interp2d();
double deg2rad(double degrees);
double rad2deg(double rads);
double rwalog2(float thisSizeFactor);
void calcFFT(int doFFT, int thisBox, int tryBackground);
void calcNewLat();

void get_nview_mod_(int *symm, float *dthe, float *dphi_90, int *nview);
void get_t1t2p1_(int *symm, float *the1, float *the2, float *phi1);
void calc_mod_tps_(float *thetas, float *phis, float *the1, float *the2, 
	      float *phi1, float *dthe,
	      float *dphi_90, int *view, int *symmetry);

void RwaActivateWidget();
void checkAutomatic();
void write_pif();

void clearMap(float *thisMap,int nCols,int nRows, int nSecs);

void mainRobem_sub_UNIX(unsigned short **int_buffer, 
			fString_t *fName,
			int *i, int *j);
void emmap_rob_shd1 (cue_t *dphshd_map, 
			    double *tmp2d, double *tmp,
			    int ncol, int nrow,
			    int sec1, int sec2, 
			    float fract, 
			    int useForMinMax, 
			    float globAngx, float stepVal,
			    float globAngy, int whichStep,
			    float passMin, int angx, int angy, int thisFract);
void map_1d_rob (float *map, int ncol, int nrow, int nsec, 
			float xcen, float ycen, float zcen,
			int interp, int nterp, float max_r, 
			float *rint, int *irnum, float *sum_rint,
			float *sumsq_rint, float *stdev);
void img_1d_rob(float *img,
		       int ncol, int nrow,
		       float xcen, float ycen,
		       int interp, int nterp,
		       float *rint, int *irnum,
		       float img_bkg);
void dph_rescale_rob (int ncol, int nrow, int num_views, 
			     cue_t *cuedMap, int tryColors,
			     int *radialStepSize);
void get_t1t2p1();
void calc_tp_rwa();
void calcMapStats();
void map2d_stats();
void fpoly_ctf(double  x, double  a[], double  *y, double  dyda[], 
		      int na, int order_poly1,
		      int order_poly2, double  wavelength, int idim_half, 
		      double  A, double  Cs,
		      double  pixel_size );

void unitCell_background();
void fftMeasureDraw(int button);
void findLatPt(int x, int y);
void calcDefocus(float *majorValue, float *minorValue);
void DrawThermometer();
void DrawFlickThermometer();
void findMaxMin(float *fftImg_buffer, float *maxF, float *minF, 
		float *offset, int fftWidth, int fftHeight);


void ctf_all_activate();

void phys_map_rotate(float *gold_buffer,
		     float *temp_buffer,
		     float theta, float phi, float omega);
void new_phys_map_rotate(float *gold_buffer,
			 float *temp_buffer,
			 float theta, float phi, float omega);

void calcCTF();
void calcMeanDefocus(float *meanValue);
int calcShortBackground();
void calcSizeFactor(float thisZoomFactor);
void evalSwitches(int argc, char **argv);
void readResourceFile();
void init_globals(int argc, char **argv);
void groundZero();
void XlibInitialize();
void s_error(char *problem_string);
int checkStatus(char *outStr);
int calcHistLength(int histVal);
void stuffImage(unsigned short *thisBuffer, int i, int j, 
		int scaleIt, int fullScreenFirst);
void stuffMapImage(float *map_img_buf, 
		   float *thisBuffer, 
		   int ncols, 
		   int nrows, 
		   int nsecs,
		   int startImg,
		   int passedOffset);
void stuffPackedImage(int width,int height,
		      float **pkImg, int nimg,
		      int startImg);
void drawContour(float contourLvl, 
		 int destructive, 
		 int thisCurBox,
		 Widget thisWidget,
		 float *thisFloatBuf,
		 int deltaX, int deltaY,
		 BYTE *this_display_buffer);
void stuffContourBuffer();
void pushBox(int x, int y, int findThisBox, int *thisBox);
void removeBox(int x, int y, int diskCoords);
void pushRectBox(int button, int x, int y);
void pushPolyBox(int x, int y);
void popPolyBox();
void filterFiles(char *filesStr, int tag);
void writeColormap(char *fName);
void readColormap(char *fName);
void pushColormap();
void popColormap();
void contrast_enhance(int min, int max);
void pointerLabelDraw(int x, int y);
void stuffDisplayBoxBuffer(int thisBox, char *filename, 
			   int thisZoomFactor);
void stuffRectangleBuffer(float *outBuf, char *filename);
void stuffDisplayMapBuffer(int thisSec, int thisZoomFactor, int boxOffset);
void stuffDisplayPackBoxBuffer(int thisSec, int thisZoomFactor,
			       int computeAppodize);
void handle_cr();
void writeBoxCoords(char *fName, int append);
void readBoxCoords(char *fName);
void readState(char *fName);
void writeState(char *fName);
void virtual_read_activate(int newX, int newY);
int askThem(char *infoStr, int dialogType);
int askThemAnswer(char *infoStr, int maxAnswer);
void read_batchjob(char *fName);
void read_param(char *fName, int readPackedFile);
void read_param_map(char *fName);
void read_param_scan(char *fName);
void write_param(char *fName, int flag);
void Single_Line(int whichParam,GC color);
void map2sfs(float resHiP);

void zoomText_change();
void button_zoom_activate();
void plotWindow_create();
void raiseIt(Widget w);
void DrawWindow();
void plotMotion(Widget	widget,
	caddr_t unused,   /* Client specified data */
	register XMotionEvent *pEvent);
void fftMotion(Widget	widget,
	caddr_t unused,   /* Client specified data */
	register XMotionEvent *pEvent);
void cancel_fileSel(Widget	widget,
		int    *tag,
		XmFileSelectionBoxCallbackStruct *callback_data);
void gotFilename (Widget   w,
		  char *client_data,
		  XmFileSelectionBoxCallbackStruct *callback_data);
void cmScrollArrowActivateButton();
void paintSizeArrowActivateButton();
void icosaArrowActivateButton();
void boxNumArrowActivateButton();
void boxInitArrowActivateButton();
void imgNumArrowActivateButton();
void fftImgNumArrowButton();
void radiusArrowActivateButton();
void gbsArrowActivateButton();
void ppinArrowButton();
void ppintext_changed();
void binPPArrowButton();
void binPPtext_changed();
void stepPPArrowButton();
void stepPPtext_changed();
void fft_step_changed();
void diffSec_really_doit();
void mapShade_activate_work();
void boxInitActivate(Widget widget, 
		     int *tag, 
		     XmAnyCallbackStruct *callback_data);
void imgNumActivate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void colormap_ramp_activate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void colormapMotion(Widget	widget,
		caddr_t unused,   /* Client specified data */
		register XMotionEvent *pEvent);
void circAvgMotion(Widget	widget,
		caddr_t unused,   /* Client specified data */
		register XMotionEvent *pEvent);
void histogram_activate();
void editBoxMotion(Widget	widget,
		caddr_t unused,   /* Client specified data */
		register XMotionEvent *pEvent);
void editBoxText_changed(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data); 

void context_help_screen_activate(Widget w, XtPointer cld, XtPointer cad);
void binning_activate(Widget w, XtPointer cld, XtPointer cad);
void simple_deactivate(Widget w, XtPointer cld, XtPointer cad);
void normalize_activate_2(Widget w, XtPointer cld, XtPointer cad);
void rescale_densities(Widget w, XtPointer cld, XtPointer cad);
void stat_calc_activate(Widget w, XtPointer cld, XtPointer cad);
void pifhead_activate(Widget w, XtPointer cld, XtPointer cad);
void save_title_activate(Widget w, XtPointer cld, XtPointer cad);
void circAvg_print_activate(Widget w, XtPointer cld, XtPointer cad);
void circAvg_pifOut_activate(Widget w, XtPointer cld, XtPointer cad);
void circAvgText_changed(Widget w, XtPointer cld, XtPointer cad);
void circAvg_change(Widget w, XtPointer cld, XtPointer cad);
void circAvg_list_proc(Widget w, XtPointer cld, XtPointer cad);
void circAvgArrowActivate(Widget w, XtPointer cld, XtPointer cad);
void DrawCircAvgWindow(Widget w, XtPointer cld, XtPointer cad);
void appodize_activate(Widget w, XtPointer cld, XtPointer cad);
void truncadd_half_activate(Widget w, XtPointer cld, XtPointer cad);
void truncadd_do_activate(Widget w, XtPointer cld, XtPointer cad);
void autoDR_activate(Widget w, XtPointer cld, XtPointer cad);
void autodr_deactivate(Widget w, XtPointer cld, XtPointer cad);
void adr_toggle_change(Widget w, XtPointer cld, XtPointer cad);
void adrab_reset_activate(Widget w, XtPointer cld, XtPointer cad);
void autoDR_update_activate(Widget w, XtPointer cld, XtPointer cad);
void autoDwritePIFhead_activate(Widget w, XtPointer cld, XtPointer cad);
void dio_change(Widget w, XtPointer cld, XtPointer cad);
void fix2d_deactivate(Widget w, XtPointer cld, XtPointer cad);
void fix2D_do_activate(Widget w, XtPointer cld, XtPointer cad);
void fix2D_flicker_activate(Widget w, XtPointer cld, XtPointer cad);
void fix2D_toggle_change(Widget w, XtPointer cld, XtPointer cad);
void point_color_activate(Widget w, XtPointer cld, XtPointer cad);
void drawCenterMarker(Widget w, XtPointer cld, XtPointer cad);
void histMode_change(Widget w, XtPointer cld, XtPointer cad);
void partproc_manip_work(Widget w, XtPointer cld, XtPointer cad);
void manipPP_change(Widget w, XtPointer cld, XtPointer cad);
void cavgPP_do_activate(Widget w, XtPointer cld, XtPointer cad);
void cavgPP_change(Widget w, XtPointer cld, XtPointer cad);
void box_change(Widget w, XtPointer cld, XtPointer cad);
void dispMode_change(Widget w, XtPointer cld, XtPointer cad);
void versionArrowActivate(Widget w, XtPointer cld, XtPointer cad);
void radiusText_changed(Widget w, XtPointer cld, XtPointer cad);
void boxcolor_change(Widget w, XtPointer cld, XtPointer cad);
void autobox_scroll_change(Widget w, XtPointer cld, XtPointer cad);
void boxMode_change(Widget w, XtPointer cld, XtPointer cad);
void autoBox_activate(Widget w, XtPointer cld, XtPointer cad);
void correlate_activate(Widget w, XtPointer cld, XtPointer cad);
void boxgrid2_activate(Widget w, XtPointer cld, XtPointer cad);
void squareText_changed(Widget w, XtPointer cld, XtPointer cad);
void version_scroll_activate(Widget w, XtPointer cld, XtPointer cad);
void fix_change(Widget w, XtPointer cld, XtPointer cad);
void fix_do_activate(Widget w, XtPointer cld, XtPointer cad);
void help_save_activate(Widget w, XtPointer cld, XtPointer cad);
void info_list_proc(Widget w, XtPointer cld, XtPointer cad);
void infoOK(Widget w, XtPointer cld, XtPointer cad);
void infoCancel(Widget w, XtPointer cld, XtPointer cad);
void colorScrollChange(Widget w, XtPointer cld, XtPointer cad);
void colormap_ramp_change(Widget w, XtPointer cld, XtPointer cad);
void DrawTestColormapWindow(Widget w, XtPointer cld, XtPointer cad);
void histArrowActivate(Widget w, XtPointer cld, XtPointer cad);
void DrawColormapWindow(Widget w, XtPointer cld, XtPointer cad);
void colorClick(Widget w, XtPointer cld, XtPointer cad);
void toggle_change(Widget w, XtPointer cld, XtPointer cad);
void interpolate_activate(Widget w, XtPointer cld, XtPointer cad);
void diamText_change(Widget w, XtPointer cld, XtPointer cad);
void apNewOrig_do_activate(Widget w, XtPointer cld, XtPointer cad);
void mapshade_deactivate(Widget w, XtPointer cld, XtPointer cad);
void mapShade_activate(Widget w, XtPointer cld, XtPointer cad);
void partproc_deltaomega_work(Widget w, XtPointer cld, XtPointer cad);
void partproc_deltaoldNew_work(Widget w, XtPointer cld, XtPointer cad);
void partproc_delta_work(Widget w, XtPointer cld, XtPointer cad);
void options_deactivate(Widget w, XtPointer cld, XtPointer cad);
void fixit_changed(Widget w, XtPointer cld, XtPointer cad);
void fixit_deactivate(Widget w, XtPointer cld, XtPointer cad);
void subtractmap_deactivate(Widget w, XtPointer cld, XtPointer cad);
void subtractMap_work(Widget w, XtPointer cld, XtPointer cad);
void subtractMap_2d3d_activate(Widget w, XtPointer cld, XtPointer cad);
void smMinMax_reset_activate(Widget w, XtPointer cld, XtPointer cad);
void subtractMap_flicker_arm(Widget w, XtPointer cld, XtPointer cad);
void subtractMap_flicker_disarm(Widget w, XtPointer cld, XtPointer cad);
void flickArrowActivate(Widget w, XtPointer cld, XtPointer cad);
void flicker_change(Widget w, XtPointer cld, XtPointer cad);
void subtractMap_diffSection(Widget w, XtPointer cld, XtPointer cad);
void diffSecSave_activate(Widget w, XtPointer cld, XtPointer cad);
void subtractMap_findMag_disp(Widget w, XtPointer cld, XtPointer cad);
void subtractMap_findRad_disp(Widget w, XtPointer cld, XtPointer cad);
void subtractMap_cc(Widget w, XtPointer cld, XtPointer cad);
void subMapShadeScreen_activate(Widget w, XtPointer cld, XtPointer cad);
void diffSec_scroll_change(Widget w, XtPointer cld, XtPointer cad);
void radTog_change(Widget w, XtPointer cld, XtPointer cad);
void subtractMap_findRad(Widget w, XtPointer cld, XtPointer cad);
void emcoravg_tryit_activate(Widget w, XtPointer cld, XtPointer cad);
void emcoravg_deactivate(Widget w, XtPointer cld, XtPointer cad);
void emcoravg_save_activate(Widget w, XtPointer cld, XtPointer cad);
void emcoravg_text_changed(Widget w, XtPointer cld, XtPointer cad);
void DrawEmcoravgWindow(Widget w, XtPointer cld, XtPointer cad);
void coravg_scroll_change(Widget w, XtPointer cld, XtPointer cad);
void ref_select_activate(Widget w, XtPointer cld, XtPointer cad);
void ref_deactivate(Widget w, XtPointer cld, XtPointer cad);
void movie_contour_state(Widget w, XtPointer cld, XtPointer cad);
void movieMinMax_change(Widget w, XtPointer cld, XtPointer cad);
void mapMovie_activate(Widget w, XtPointer cld, XtPointer cad);
void stepMode_change(Widget w, XtPointer cld, XtPointer cad);
void surfDensValueChange(Widget  widget, XtPointer cld, XtPointer cad);
void unblob_scroll_change(Widget  widget, XtPointer cld, XtPointer cad);
void stepMode_button_change(Widget  widget, XtPointer cld, XtPointer cad);
void unblob_text_change(Widget  widget, XtPointer cld, XtPointer cad);
void reverse_sections(Widget  widget, XtPointer cld, XtPointer cad);
void unblob_save_activate(Widget  widget, XtPointer cld, XtPointer cad);
void unblob_reset_activate(Widget  widget, XtPointer cld, XtPointer cad);
void foldAxis_activate(Widget  widget, XtPointer cld, XtPointer cad);
void rotate_do_activate(Widget  widget, XtPointer cld, XtPointer cad);
void samRenArrowActivate(Widget  widget, XtPointer cld, XtPointer cad);
void surfDensScrollChange(Widget  widget, XtPointer cld, XtPointer cad);
void rotate_xyz_activate(Widget  widget, XtPointer cld, XtPointer cad);
void batch_focus(Widget  widget, XtPointer cld, XtPointer cad);
void paramFile_browse_activate(Widget  widget, XtPointer cld, XtPointer cad);
void batchjob_allasu_activate(Widget  widget, XtPointer cld, XtPointer cad);
void batchjob_submit_activate(Widget  widget, XtPointer cld, XtPointer cad);
void batchparam_list_proc(Widget  widget, XtPointer cld, XtPointer cad);
void model_deactivate(Widget w, XtPointer cld, XtPointer cad);
void model_activate(Widget w, XtPointer cld, XtPointer cad);
void model_clear_activate(Widget w, XtPointer cld, XtPointer cad);
void model_undo_activate(Widget w, XtPointer cld, XtPointer cad);
void model_save_activate(Widget w, XtPointer cld, XtPointer cad);
void mb_scroll_change(Widget w, XtPointer cld, XtPointer cad);
void modelObject_change(Widget w, XtPointer cld, XtPointer cad);
void fourierFilt_work(Widget w, XtPointer cld, XtPointer cad);
void crysdisp_deactivate(Widget w, XtPointer cld, XtPointer cad);
void crysDisp_reset_activate(Widget w, XtPointer cld, XtPointer cad);
void crystalDisp_changed(Widget w, XtPointer cld, XtPointer cad);
void cDisp_newAdimBdim_changed(Widget w, XtPointer cld, XtPointer cad);
void crysDisp_scroll_change(Widget w, XtPointer cld, XtPointer cad);
void cDispSymm_changed(Widget w, XtPointer cld, XtPointer cad);
void imgconv_deactivate(Widget w, XtPointer cld, XtPointer cad);
void imgconv_tryit_activate(Widget w, XtPointer cld, XtPointer cad);
void cavgPP_activate(Widget w, XtPointer cld, XtPointer cad);
void partproc_rebox_activate(Widget w, XtPointer cld, XtPointer cad);
void views_activate(Widget w, XtPointer cld, XtPointer cad);
void projASU_activate(Widget w, XtPointer cld, XtPointer cad);
void symmetry_activate(Widget w, XtPointer cld, XtPointer cad);
void ico_hist_activate(Widget w, XtPointer cld, XtPointer cad);
void unitCell_activate(Widget w, XtPointer cld, XtPointer cad);
void xyPlot_activate(Widget w, XtPointer cld, XtPointer cad);
void plotCavgPP_activate(Widget w, XtPointer cld, XtPointer cad);
void partproc_sort_activate(Widget w, XtPointer cld, XtPointer cad);
void icosaClick(Widget w, XtPointer cld, XtPointer cad);
void DrawICOSWindow(Widget w, XtPointer cld, XtPointer cad);
void paramMode_change(Widget w, XtPointer cld, XtPointer cad);
void project_one(Widget w, XtPointer cld, XtPointer cad);
void ppMinMax_scroll_change(Widget w, XtPointer cld, XtPointer cad);
void rotflt_deactivate(Widget w, XtPointer cld, XtPointer cad);
void rotflt_write_activate(Widget w, XtPointer cld, XtPointer cad);
void DrawROTFLTWindow(Widget w, XtPointer cld, XtPointer cad);
void rotfltClick(Widget w, XtPointer cld, XtPointer cad);
void rotflt_change(Widget w, XtPointer cld, XtPointer cad);
void rotflt_forceTryit_activate(Widget w, XtPointer cld, XtPointer cad);
void editBoxClick(Widget w, XtPointer cld, XtPointer cad);
void DrawEditBoxWindow(Widget w, XtPointer cld, XtPointer cad);
void movie_deactivate(Widget w, XtPointer cld, XtPointer cad);
void movie_contour_activate(Widget w, XtPointer cld, XtPointer cad);
void editBoxZoomText_changed(Widget w, XtPointer cld, XtPointer cad);
void editBoxZoomArrowActivate(Widget w, XtPointer cld, XtPointer cad);
void editBox_save_activate(Widget w, XtPointer cld, XtPointer cad);
void editBoxPaintText_changed(Widget w, XtPointer cld, XtPointer cad);
void editBox_undo_activate(Widget w, XtPointer cld, XtPointer cad);
void mapMovie_show_activate(Widget w, XtPointer cld, XtPointer cad);
void fouraver_deactivate(Widget w, XtPointer cld, XtPointer cad);
void refine_changed(Widget w, XtPointer cld, XtPointer cad);
void impose_changed(Widget w, XtPointer cld, XtPointer cad);
void fourAver_resetSfs(Widget w, XtPointer cld, XtPointer cad);
void movePhaseOrigin_activate(Widget w, XtPointer cld, XtPointer cad);
void latText_changed(Widget w, XtPointer cld, XtPointer cad);
void auto2ddr_deactivate(Widget w, XtPointer cld, XtPointer cad);
void fft_deactivate(Widget w, XtPointer cld, XtPointer cad);
void auto2DDR_activate(Widget w, XtPointer cld, XtPointer cad);
void auto2DDR_update_activate(Widget w, XtPointer cld, XtPointer cad);
void auto2DDR_curves_activate(Widget w, XtPointer cld, XtPointer cad);
void auto2DDR_ctflines_activate(Widget w, XtPointer cld, XtPointer cad);
void auto2DDR_scurve_activate(Widget w, XtPointer cld, XtPointer cad);
void auto2DDR_drawmodified_activate(Widget w, XtPointer cld, XtPointer cad);
void DrawFFTWindow(Widget w, XtPointer cld, XtPointer cad);
void fftClick(Widget w, XtPointer cld, XtPointer cad);
void fft_change(Widget w, XtPointer cld, XtPointer cad);
void fft_change_toggle(Widget w, XtPointer cld, XtPointer cad);
void fft_1DCA_activate(Widget w, XtPointer cld, XtPointer cad);
void fixit_activate(Widget w, XtPointer cld, XtPointer cad);
void fft_mode_activate(Widget w, XtPointer cld, XtPointer cad);
void rotflt_screen_activate(Widget w, XtPointer cld, XtPointer cad);
void fft_writeFB_activate(Widget w, XtPointer cld, XtPointer cad);
void ctf_appodize_activate(Widget w, XtPointer cld, XtPointer cad);
void read_fftbuffer_activate(Widget w, XtPointer cld, XtPointer cad);
void fft_average_activate(Widget w, XtPointer cld, XtPointer cad);
void paramFileGenerate_activate(Widget w, XtPointer cld, XtPointer cad);
void autoDefocusRefine_activate(Widget w, XtPointer cld, XtPointer cad);
void auto2DDefocusRefine_activate(Widget w, XtPointer cld, XtPointer cad);
void ctf_reset_activate(Widget w, XtPointer cld, XtPointer cad);
void ctfArrowActivate(Widget w, XtPointer cld, XtPointer cad);
void ctf_ioparms_activate(Widget w, XtPointer cld, XtPointer cad);
void ctfText_changed(Widget w, XtPointer cld, XtPointer cad);
void drawFFTLiteActivate(Widget w, XtPointer cld, XtPointer cad);
void latReset_activate(Widget w, XtPointer cld, XtPointer cad);
void fourAver_manage(Widget w, XtPointer cld, XtPointer cad);
void plotChange(Widget w, XtPointer cld, XtPointer cad);
void formResize(Widget w, XtPointer cld, XtPointer cad);
void DrawWindowExpose(Widget w, XtPointer cld, XtPointer cad);
void plotClick(Widget w, XtPointer cld, XtPointer cad);
void fix_activate(Widget w, XtPointer cld, XtPointer cad);
void removeBlemish_activate(Widget w, XtPointer cld, XtPointer cad);
void truncadd_activate(Widget w, XtPointer cld, XtPointer cad);
void rescale_activate(Widget w, XtPointer cld, XtPointer cad);
void subtractMap_activate(Widget w, XtPointer cld, XtPointer cad);
void calcSF_activate(Widget w, XtPointer cld, XtPointer cad);
void editBox_activate(Widget w, XtPointer cld, XtPointer cad);
void normalize_activate(Widget w, XtPointer cld, XtPointer cad);
void linGrad_activate(Widget w, XtPointer cld, XtPointer cad);
void checkBkg_activate(Widget w, XtPointer cld, XtPointer cad);
void fourierFilt_activate(Widget w, XtPointer cld, XtPointer cad);
void addAverage_activate(Widget w, XtPointer cld, XtPointer cad);
void model_screen_activate(Widget w, XtPointer cld, XtPointer cad);
void crysDisp_screen_activate(Widget w, XtPointer cld, XtPointer cad);
void circAvg_screen_activate(Widget w, XtPointer cld, XtPointer cad);
void mapShadeScreen_activate(Widget w, XtPointer cld, XtPointer cad);
void fft_activate(Widget w, XtPointer cld, XtPointer cad);
void exit_activate(Widget w, XtPointer cld, XtPointer cad);
void read_activate(Widget w, XtPointer cld, XtPointer cad);
void write_activate(Widget w, XtPointer cld, XtPointer cad);
void drawLiteActivate(Widget w, XtPointer cld, XtPointer cad);
void toolMode_change(Widget w, XtPointer cld, XtPointer cad);
void imgNumText_changed(Widget w, XtPointer cld, XtPointer cad);
void simple_activate(Widget w, XtPointer cld, XtPointer cad);

void fft_create();
void ref_correlate_activate();
void mapMovieArrowActivate();
void fft_flicker_activate();
void calcLattice();
void latScrollChanged();
void latTextChanged();
void ctfTextChanged();
void lohiRadiusChanged();
void lopTextChanged();
void hipTextChanged();
void paramFileRead_work();
void fourAver_work();
void fftImgNum_cr_activate();
void rotfltArrowActivate(Widget w, XEvent *event);
void rotflt_tryit_activate();
void emcoravg_arrow_activate();
void stuffEmcoravgImage();
void stuffImgconvImage();
void read_imgconv(char *fName);
void read_emcoravg(char *fName);
void crysDisp_tryit_activate();
void crysDisp_cr_activate();
void cDisp_zoom_activate(Widget w, XEvent *event);
void unblob_tryit_activate();
void ctfCurve_cr_activate();
void surfDens_CR();
void histEqualize();
void calcSF_work(int thisCalcSFs);
void coravgArrowButton();
void phaseArrowButton();
void smMMaxArrowButton();
void subtractMap_doit();
void subtractMap_read();
void drawMinMaxRadius();
void diffSec_do();
void drawPointScreen();
float findRealValue();
void boxgrid_activate();
void paramFile_read_activate();
void paramScanFile_read_activate();
void projContour_cr_activate();
void projasu_cr_activate();
void paramMapFile_read_activate();
void projASUunitCell_work();
void DrawIcosahedron(float xrot,float yrot,float zrot,
		     int xoffset, int yoffset, 
		     float scaleFactor, Window thisWin);
void findNearestParticle(int *x, int *y,
			 P_INFO **foundPart,
			 int toggleFound,
			 int makeThisParticle,
			 int showPackedBox,
			 int buttonNum);
void convertThetaPhi(int x, int y, 
		     float *thisTheta, float *thisPhi);
void convertXY(int *convX, int *convY, float fX, float fY);
void icosaMotion(Widget	widget,
	caddr_t unused,   /* Client specified data */
	register XMotionEvent *pEvent);
void markXYplotOutliers();
void ppMinMax_change();
void ctfValue_change();
void movement_activate();
void find_nearest_neighbors(INFO *head);
void resolution_tryit_activate();


/* void sleep(); */
/* pid_t fork(); */
void vt_cls();  

void mainRobem_sub_UNIX(unsigned short **int_buffer, 
			fString_t *fName,
			int *i, int *j); 

int checkMemory(void *status, int line, char *file);

void rad_dens_fast_rob_C (float *coremap, 
			  int ncol, int nrow, int ndepth, 
			  float *buf,
			  float x_org, float y_org, float z_org, 
			  float radius, float smallestDens,
			  float theta, float phi,float omega,
			  int colOffset, int rowOffset,int largestDim,
			  int thisIrev);

void icosRadialProject (float *coremap, 
			       int ncol, int nrow, int ndepth, 
			       float *buf,
			       float x_org, float y_org, float z_org, 
			       float radius, float smallestDens,
			       float theta, float phi,float omega,
			       int colOffset, int rowOffset,
			       int thisIrev, int tag,
			       float thresh, float slid, int destroy,
			       float minRadius);

int callXdy (double x, double y, double z, 
		     double minRad, double maxRad);

void thetaPhi_dphshd_C (float *coremap,
			       float *origMap,
			       int ncol, int nrow, 
			       float *dphshd_map, 
			       BYTE shift, BYTE surf, 
			       int sec1, int sec2, 
			       int col1, int col2,
			       int row1, int row2,
			       int num_views,
			       int thisIrev, float surf_dens,
			       int useForMinMax,
			       float *shad_map,
			       double phi, double theta, double omega,
			       double minRadius, double maxRadius,
			       double xCen, double yCen, double zCen,
			       float radialXcen, float radialYcen, 
			       float radialZcen,
			       int lowRes, int topRadialSec, int step);

double getVoxel(float *coremap,
		       double thisCol, double thisRow, double thisSecn,
		       double theta, double phi, double omega,
		       int ncol, int nrow);

void newTranslateThetaPhiOmega(double *thisCol,
				      double *thisRow,
				      double *thisSecn,
				      double xCen,double yCen,double zCen,
				      int col,int row,int secn,
				      double t1,
				      double t2,
				      double t3,
				      double t4,
				      double t5,
				      double t6,
				      double t7,
				      double t8,
				      double t9
				      );

void emmap_rob_dphshd_C (float *map,
				float *final_map,
				float *radial_map,
				float *shad_map,
				float *origMap,
				int   numFrames,
				float surfaceDensity,
				int ncol, int nrow, 
				int firstStep, int nsec,
				int col1, int col2,
				int row1, int row2,
				char ikey,
				double phi, double thetaY,
				double minRadius, double maxRadius,
				double xCen, double yCen, double zCen,
				float radialXcen, float radialYcen, 
				float radialZcen,
				int lowRes, int topRadialSec);
void img_write_recl_open(fString_t *fileName, int *lun, 
				short *int_image_buffer1,
				int *image_width, int *image_height);

void   yji_ctf1dfit_( int *ncol, int *nrow, int *nimg, 
			     int *fftSize, float *pixelSize,
			     float *volts, float *res_hi, float *defMajor, 
			     float *ampContrast, float *Cs, 
			     float *allPackImg, float *chisq, 
			     float *a,
			     float *adr_s,
			     float *adr_sminusb,
			     float *adr_ctfsq,
			     float *adr_d,
			     float *adr_b,
			     float *ca1d,
			     int   *calcIt,
			     float *saveFftAvg
			     );
char *img_filename;
void img_open();
void img_pack_find_prompt();
void fft_latref2_index_ ();
void fft_latref2_spot_ ();
void yongning_routine_();
void emsf_backt_rob_(int *sf_type, int *em_ndim, int *nsf,
		     strucFact_t *sFacs,
		     float *map_min, float *map_max,
		     float *float_image_buffer1,
		     int *backSizeA,int *backSizeB);
void emsf_origins_rob_(int *em_ndim,int *sf_type,int *nsf,
			      float *sf_sumfsq, 
			      strucFact_t *sFacs, int *ikey);
void emsf_sym_rob_(int *em_ndim, int *sf_type, int *nlim, int *nsf, 
			 strucFact_t *sFacs,
			 float *gamstr, 
			 int *ikey, float *thisAoverb);
void map_rot_();
void img_filt_();
void fft_2d_();
void fft_dsp_rob_();
void fft_fltmsk_rob_();
void fft_2d_rob_();
void fft_copy1_();
void latcen_();
void latcen2_();
void emrotflt_main_(float *powSpec,
			   float *elipX,float *elipY,
			   int *irad,
			   int *idim1Rotflt,
			   float *phi0,float *ascal,float *bscal,
			   float *twodd,int *jzero,
			   int *radiusM1,int *max_bo,
			   float *fft_step,int *nannuli,
			   float *del_br,
			   float *rho,
			   float *del_lr,
			   int *iopt,
			   int *lg_start,int *lg_end,
			   float *xorig,float *yorig,
			   float *pow_scale, int *nrot,
			   int *numCols, int *numRows,
			   float *sump1, float *sump2, float *sump3,
			   float *fft_origx, float *fft_origy, 
			   float *winScale, float *sigma,
			   float *fft,
			   int *newStart, float *sigMax, float *sigMaxTot
			   );
void mapdsp_gplot_rob_(float *crys_float_image_buffer, 
			      unsigned short *disp_buffer, 
			      float *aDscale, 
			      int *cdNumCols, int *cdNumRows, int *cdNumSecs, 
			      int *npixx, int *npixy, 
			      float *xstrt, float *ystrt, 
			      int *ny, 
			      int *ncellx, int *ncelly, 
			      int *section,
			      float *new_min,
			      float *angle,
			      float *aoverbPass,
			      int *plotX,int *plotY,
			      int *adim, int *bdim, float *passGamma);
void img_lingrad_rob_(float *img, int *ncol, int *nrow,
			     int *irad, float *imgBkg, float *expFac);
void count_kill_blobs_rob_(int *bCnt, int *uniqB,
				  float *contMin, float *contMax,
				  int *ublobSiz,
				  int *killFlag, 
				  float *killValue, 
				  float *bgMin, float *bgMax,
				  int *status,
				  float *unblobBuf,
				  int *nc, int *nr, int *ns,
				  blobStructure_t *bL, int *radUB, int *invU
				  );
void getblobminmaxint_rob_();
void contur_a_rob_();
void contur_b_rob_();
void contur_c_rob_();
void contur_d_rob_();
void rad_dens_fast_rob_();
void ccf_refine_();
void ref_ccf_refine_();
void fft_avg_ref_();
void map_prj_();
void map_scale_();
void main_map3dt_();
void map3dt_info_rwa_();
void main_emsf_();
float wavel_();
void ccomap3ds_();
void cc_3d_(float *float_image_buffer1,
		   float *subtract_image_buffer1,
		   int *numCols, int *numRows, int *numSecs,
		   int *sub2binCol, int *sub2binRow, int *sub2binSec,
		   int *ichc,float *std,
		   float *xc, float *yc, float *zc,
		   float *magini,float *magstep,int *magchk,
		   float *radlo,float *radhi,float *radstep,float *bmag,
		   int *iix,int *iiy,int *iiz,
		   float *mags, float *coefs,
		   int *subMap2d3d, int *qtr_switch
		   );
void ccomap3cs_(float *thisBuf1,
		       float *thisBuf2,
		       int *numCols,int *numRows,int *sec1,
		       int *sub2binCol,int *sub2binRow,int *sec2,
		       int *nfit,
		       float *cntr1,float *cntr2,
		       float *map_std,
		       float *xc,float *yc,float *zc,
		       float *radlo,float *radhi,float *radstep,
		       float *bmag,
		       float *rdens,float *wrdens,float *ccoef,
		       float *wccoef,float *s_wccoef,int *nbins,
		       float *diffSecScrollValue,
		       float *diffSec2ScrollValue,
		       int *qtr_switch, int *subMap2d3d,
		       float *threshScrollValue,
		       int *threshAbs
		       );
void mag_write_();
void fft_lop_();
void fft_hip_();

void img_fill();
void copy_r4();
void fft_pshift();
void emfft_spt_rob1();
void emfft_spt_rob2();
void rob_sf_pshift();
void rec_to_real();
void fft_fltllmsk_rob();
void img_stats_rob();
void apply_ctf_rtn();
void fft_dsp_c(float *fft, BYTE *img, 
	       int idim1, int idim2,
	       int ncol, int nrow, 
	       float thresh_hi, float thresh_lo,
	       int hiPixel, int loPixel, 
	       int linear, int amps);
void convert_abpart(float apart, float bpart,
			   float *amp, float *phase);
int GetButtonVal(unsigned int b, int x1, int x2, int x3);

void map_vol_rwa(float *map, int ncol, int nrow, int nsec, float map_min,
		 float map_max, float *fraction, int *bin, int nbins,
		 float *cutoff, float minRad, float maxRad);

void add_map_padded(float *map1, int ncol1, int nrow1, int nsec1,
		    float *map2, int ncol2, int nrow2, int nsec2);
void set_boxgrid_default();

void createCursors();
void createPixmaps();
void applyPixmaps();
void enableSaveUnder();
void allocInitBuffers();
void updateCursor(Widget w, Cursor c);
void DrawX(Window w, GC color, int x, int y, int len);
void TextSet(Widget w, char *str, int flag);
void TextSetInt(Widget w, int i, int flag);
void TextSetFloat(Widget w, float f, int flag);
void add_gauss_noise(float *img, int n, float mean, float std, float nstd);
