      subroutine yji_ctf1dfit(npix_col, npix_row, num_img, fftsize,
     +     pixelSizeCall, voltage, res_hi, defMajorCall,
     +     ampContrastCall, CsCall, packImgCall, chisqCall, 
     +     aPass, adr_s, adr_sminusb, adr_ctfsq, adr_d, adr_b, 
     +     ca1d_pass, calcIt, saveFftAvg)

C     1. read images from one image file and do 2D FFT
C     2. average these 2D FFTs
C     3. calculate some stats and writeout

      implicit none

      include '../include/allprogs.inc'

      character*8   subname
      data subname / '    ' /

      integer mm, displayScale, calcIt
      integer alloc_stat1, alloc_stat2
      real  adr_s(1024), adr_sminusb(1024), ca1d_pass(1024)
      real  adr_ctfsq(1024), adr_d(1024), adr_b(1024) 
      real  center(2)
      real  saveFftAvg(1)

      real, allocatable :: rffta_omega(:)
      real, allocatable :: fftavg(:)
      complex, allocatable :: fft2d(:)
      
      double precision, allocatable :: poly_paras(:)
      double precision, allocatable :: ca1d_avg(:)
      double precision, allocatable :: ca1d_std(:)

      integer fftsize,m,n
      real    pixelSizeCall, voltage, res_hi
      real    pixels(npix_col,npix_row)
      real    aPass(13)
      real    defMajorCall
      real    ampContrastCall, CsCall, chisqCall
      real    packImgCall(npix_col, npix_row, num_img)
      integer npix_col, npix_row, num_img, nrffta_row,nrffta_col,
     +     nrffta_omega, nfc_2d_row, nfc_2d_col, hkl_min, hkl_max,
     +     i, j , mem_size, annlow, annhigh, ndata
      integer poly_orderb, poly_orderd, poly_order
      double precision pixel_size, defocus, res_low,
     +     fw, fw2, wavelength, a, cs, chisq
      real    img_avg, img_std, rtemp, rtemp1
      real    tmp5, B, D, theta, ctf, ctf2, pi
      
      
      PROGNAME = 'fftavgr'
      PI = 3.14159265358979323

      print *, 'col,row,nimg = ', npix_col,npix_row,num_img

      call TRANS_LENGTH_ESSL(fftsize, nrffta_row)
      print *, ' FFT size =',nrffta_row

      nfc_2d_row   = nrffta_row/2
      nfc_2d_col   = nfc_2d_row
      hkl_min      = - nfc_2d_row
      hkl_max      = nfc_2d_row
      nrffta_col   = nrffta_row
      nrffta_omega = nrffta_col + 15

      rtemp = pixelSizeCall
      pixel_size = rtemp
      print *, ' pixel_size =', pixel_size

      print *, ' voltage =', voltage

      rtemp = defMajorCall
      defocus = rtemp
      print *, ' guessed defocus value =', defocus

      print *, ' highest resolution =', res_hi

C     Percentage of Amplitude Contrast & Spherical aberration constant
      A = 0.05
      Cs = 20000000.0     

      rtemp = ampContrastCall
      rtemp1 = CsCall
      A   = rtemp
      Cs  = rtemp1
      print *, 'Percentage Amp Contrast =', rtemp, '  Cs =', rtemp1

      poly_orderb = 5
      poly_orderd = 5
      print *, ' Order of Background =', poly_orderb 
      print *, ' Order of signal strengh decay =', poly_orderd 

      poly_orderb  = poly_orderb + 1 
      poly_orderd  = poly_orderd + 1 
      poly_order  = poly_orderb + poly_orderd + 1
      allocate(poly_paras(poly_order), stat=alloc_stat1)

      do j = 1,poly_order-1
         poly_paras(j) = aPass(j)
      enddo

      print *, 'Initial Parameters B&D for polynomial functions:',
     +     (poly_paras(j),j=1,poly_order-1)
C     according to the subroutine, the last of parameter is used for
C     defocus value
      poly_paras(poly_order) = defocus

      allocate(rffta_omega(nrffta_omega), stat=alloc_stat1)
      if(alloc_stat1 > 0) call error(progname, subname,
     +     'Error in allocating memory for nrffta_omega')

      mem_size   = npix_col * npix_row 

      mem_size   = (nfc_2d_row +1) * (nfc_2d_col * 2 +1)
      allocate(fftavg(mem_size), stat=alloc_stat1)
      allocate(fft2d(mem_size), stat=alloc_stat2)
      if (alloc_stat1 > 0 .or. alloc_stat2 > 0) call error(
     +     progname, subname, 'fft space memory allocation failed')

      do i = 1, mem_size
         fftavg(i) = 0.
      enddo

C
C *********************************************************************
C *read each image do 2D DFT to fft2d(complex) and average to fftavg(real)*
C *********************************************************************
C
      call vrffti( nrffta_col, rffta_omega )
      
C     RWa Nov 2005 Try and optimize this - if we've calculated
C     this before then just use that - don't recalc every time 
      if(calcIt == 1) then
         do i = 1, num_img

            do m=1,npix_row
               do n=1,npix_col
                  pixels(n,m) = packImgCall(n,m,i) 
               enddo
            enddo

            center(1) = (npix_row-1)/2.0
            center(2) = (npix_col-1)/2.0
            call fft_2dfft(pixels, npix_row, npix_col, center, 
     +           nrffta_omega, rffta_omega, nrffta_row, nrffta_col,
     +           hkl_max, hkl_max, fft2d)

C     average them   
            do j = 1, mem_size
               fftavg(j) = fftavg(j) + 
     +              sqrt( real(fft2d(j))**2 + aimag(fft2d(j))**2)
               saveFftAvg(j) = fftavg(j)
            enddo

         enddo
      else
         do j = 1, mem_size
            fftavg(j) = saveFftAvg(j)
         enddo
      endif                     ! if(calcIt .eq. 1)

      deallocate(rffta_omega, fft2d)

C   allocate memory and caluate some stats
      allocate(CA1D_AVG(hkl_max), stat=alloc_stat1)
      allocate(CA1D_STD(hkl_max), stat=alloc_stat2)
      
C     call to calcuate 1D statistic
      call D_STATS(fftavg,hkl_max,IMG_AVG,IMG_STD,CA1D_AVG,CA1D_STD)
      
      deallocate(fftavg)

C     calculate ctf values
C     note some different with ctf_para.f, because of the unit
      wavelength = 12.264264967 / 
     +     sqrt (voltage + 0.9784663543E-6 * voltage**2 )
      fw2 = (defocus-sqrt(defocus*defocus-Cs*wavelength*2.0))/Cs
      fw = sqrt(fw2)
      annlow = nint(fw*(pixel_size*2*hkl_max)/wavelength)
      if(annlow < 1.0) annlow = 1.0
      res_low= 2.0*hkl_max*pixel_size/annlow
      annhigh = nint (2*hkl_max*pixel_size /res_hi)
      if(annhigh > float(hkl_max)) annhigh = float(hkl_max)
      ndata = annhigh + 1 - annlow
      
      print *, 'wavelength = ', wavelength
      print *, 'annlow     = ', annlow
      print *, 'annhigh    = ', annhigh
      print *, 'res_low    = ', res_low
      print *, 'res_hi     = ', res_hi
      print *, ' '

      call cmpt_fit1d(ca1d_AVG(annlow),ca1d_std(annlow), 
     +     ndata,annlow, annhigh,
     +     poly_paras, poly_order, poly_orderb, poly_orderd,
     +     wavelength, hkl_max, A, Cs, pixel_size, chisq)

      defocus = poly_paras(poly_order) /10000.

C     A second output file that has the approximation results:
      open (unit=14, file="1d_result.out", err=962)
      goto 963
 962  call error(progname, subname,
     +     'Error in opening output file')
 963  write(14,*) "X,Y,B+CTF^2*D,Y-B,B,CTF*CTF,D"

      do i=1,hkl_max
         tmp5 =2.0*(i-annlow)/(annhigh-annlow) - 1.0;
         
         B = poly_paras(1)
         B = B + poly_paras(2)*tmp5
         B = B + poly_paras(3)*(3.0*tmp5*tmp5-1.0)/2.0
         B = B + poly_paras(4)*(5.0*tmp5*tmp5*tmp5-3.0*tmp5)/2.0
         B = B + poly_paras(5)
     +        *(5.0*tmp5*tmp5*tmp5*tmp5-30.0*tmp5*tmp5+3.0)/8.0
         B = B + poly_paras(6)*(63.0*tmp5*tmp5*tmp5*tmp5
     +        *tmp5-70.0*tmp5*tmp5*tmp5+15.0*tmp5)/8.0
         
         D = poly_paras(7)
         D = D + poly_paras(8)*tmp5
         D = D + poly_paras(9)*(3.0*tmp5*tmp5-1.0)/2.0
         D = D + poly_paras(10)*(5.0*tmp5*tmp5*tmp5-3.0*tmp5)/2.0
         D = D + poly_paras(11)*(5.0*tmp5*tmp5*tmp5*tmp5
     +        -30.0*tmp5*tmp5+3.0)/8.0
         D = D + poly_paras(12)*(63.0*tmp5*tmp5*tmp5*tmp5
     +        *tmp5-70.0*tmp5*tmp5*tmp5+15.0*tmp5)/8.0
         
         fw =wavelength*i/(hkl_max*2.0* pixel_size)
         fw2= fw*fw
         theta = PI*2.0*(Cs*fw2*fw2/4.0-poly_paras(13)*fw2/2.0)/
     $        wavelength
         ctf = sqrt(1.0-A*A)*sin(theta)-A*cos(theta)
         ctf2 = ctf*ctf
C     Stuff XWindows display buffers
           
         displayScale = int(512.0 / float(fftsize)) 
         adr_s(i*displayScale)       = (B + ctf2 * D)
         adr_sminusb(i*displayScale) = adr_s(i*displayScale) - B
         adr_ctfsq(i*displayScale)   = ctf2
         adr_d(i*displayScale)       = D
         adr_b(i*displayScale)       = B
         ca1d_pass(i*displayScale)   = ca1d_avg(i)
         if (displayScale > 1 .and. i < 256) then
            do mm = 1, displayScale
               adr_s(i*displayScale+mm)       = (B + ctf2 * D)
               adr_sminusb(i*displayScale+mm) = adr_s(i*displayScale)-B
               adr_ctfsq(i*displayScale+mm)   = ctf2
               adr_d(i*displayScale+mm)       = D
               adr_b(i*displayScale+mm)       = B
               ca1d_pass(i*displayScale+mm)   = ca1d_avg(i)
            enddo
         endif

         write(14, 1011)i,ca1d_avg(i),B+ctf2*D,ca1d_avg(i)-B,B,ctf2,D
 1011    FORMAT (1X,I4,',',5(F10.6,','),F10.6)
      enddo
      close(UNIT=14)
      
      defMajorCall = defocus
      chisqCall = chisq

      do j = 1, poly_orderb+poly_orderd+1
         aPass(j) = poly_paras(j)
      enddo

      deallocate(ca1d_AVG, ca1d_std, poly_paras)

      end

      subroutine error(a,b,c)
      character*256 a,b,c
      print *, a,b,c
      return
      end

C **********************************************************************
C *              *****     SUBROUTINE D_STATS     *****                *
C *                                                                    *
C *    ROUTINE TO DETERMINE THE MIN/MAX, AVERAGE AND STD FOR VALUES    *
C *     IN THE "FFT" real ARRAY.                                       *
C *    the array is (0:hkl_max, -hkl_max:hkl_max) of real              *
C **********************************************************************

      SUBROUTINE  D_STATS( fftavg, hkl_max, MAP_AVG, MAP_STD, 
     +                     CA1D_AVG, CA1D_STD)

      IMPLICIT NONE

C     arguments
      integer hkl_max
      real fftavg(0:hkl_max, -hkl_max:hkl_max)    
      double precision  CA1D_AVG(hkl_max), CA1D_STD(hkl_max)
      real MAP_STD, MAP_AVG

C     local 
      REAL MAP_MIN, MAP_MAX, R, X, PNT, VAR
      REAL  CA1D_CNT(hkl_max)
      INTEGER  I, J, RI, array_size
      double precision TOT, SUMSQ

       MAP_MIN  = huge(map_min)
       MAP_MAX  = -huge(map_max)
       SUMSQ    = 0.0
       TOT      = 0.0

       DO I = 1, hkl_max
         CA1D_AVG(I) = 0.0
         CA1D_STD(I) = 0.0
         CA1D_CNT(I) = 0.0
       ENDDO

       DO I = 0, hkl_max
         DO J = - hkl_max, hkl_max
           X       = fftavg(I, J)
           SUMSQ   = SUMSQ + X**2
           TOT     = TOT + X
           MAP_MIN = AMIN1(MAP_MIN,X)
           MAP_MAX = AMAX1(MAP_MAX,X)

           R  = SQRT (float(I*I + J*J))
           RI = int(R)
        
           IF (RI < hkl_max .AND. RI.NE.0) THEN
             PNT = 1.0 - (R - RI)
             CA1D_AVG(RI) = CA1D_AVG(RI) + X*PNT
             CA1D_CNT(RI) = CA1D_CNT(RI) + PNT
             CA1D_STD(RI) = CA1D_STD(RI) + X*X*PNT
             CA1D_AVG(RI+1) = CA1D_AVG(RI+1) + X*(1.0-PNT)
             CA1D_CNT(RI+1) = CA1D_CNT(RI+1) + 1.0 - PNT
             CA1D_STD(RI+1) = CA1D_STD(RI+1) + X*X*(1.0-PNT)
           elseif (RI == hkl_max) THEN
             PNT = 1.0 - (R - RI)
             CA1D_AVG(RI) = CA1D_AVG(RI) + X*PNT
             CA1D_CNT(RI) = CA1D_CNT(RI) + PNT
             CA1D_STD(RI) = CA1D_STD(RI) + X*X*PNT
           ENDIF

         ENDDO
       ENDDO

       ARRAY_SIZE= (hkl_max +1)*(2*hkl_max + 1)
       MAP_AVG = TOT/FLOAT(ARRAY_SIZE)
       MAP_STD = 0.0
       VAR     = (SUMSQ/FLOAT(ARRAY_SIZE))-(MAP_AVG*MAP_AVG)
       IF(VAR.GE.0.0) MAP_STD = SQRT(VAR)
       WRITE(*,1000)
       WRITE(*,1001)
 1000  FORMAT('                  STATICS: averaged FFT')
 1001  FORMAT('FFTSIZE   MINIMUM     MAXIMUM        AVERAGE        STD',
     1 /,' ------------------------------------------------------',/)

       WRITE(*,*)hkl_max, MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD

       DO I = 1, hkl_max
         CA1D_AVG(I)=CA1D_AVG(I)/CA1D_CNT(I)
         CA1D_STD(I)=CA1D_STD(I)/CA1D_CNT(I)
         CA1D_STD(I)=CA1D_STD(I)-CA1D_AVG(I)*CA1D_AVG(I)
         IF(CA1D_STD(I).GE.0.0) THEN
           CA1D_STD(I)= SQRT (CA1D_STD(I))
         ELSE
           PRINT *, 'CHECK YOUR DATA!'
         ENDIF

         CA1D_AVG(I) = CA1D_AVG(I)/map_avg
         CA1D_STD(I) = CA1D_STD(I)/map_avg
         print *, I, CA1D_AVG(I), CA1D_STD(I)

       ENDDO

      return
      end
