      subroutine yji_ctf2dfit(poly_paras_call, npix_col,npix_row,
     +    num_img,fftSizeCall, pixelSizeCall, voltCall, res_hiCall, 
     +    defMajorCall, defMinorCall, angleCall, ampContrastCall, 
     +    CsCall, packImgCall, chisqCall, auto2DdrawBuf, 
     +    auto2DdataBuf, fftasumCall, hkl_maxCall, 
     +    order_bCall, order_dCall, calcIt, saveFftasum, saveFftasumsq)
C **********************************************************************
C *   1. read images from one image file and do 2D FFT                 *
C *   2. average these 2D FFTs                                         *
C *   3. calculate some stats and writeout                             *
C *                                                                    *
C **********************************************************************
      implicit none

      INCLUDE '../include/allprogs.inc'

      character*8   subname
      data progname / 'fftavgr' /
      data subname / '    ' /

C Local variables, used only in the main routine

      double precision amp, MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD

      real, allocatable :: rffta_omega(:)
      double precision, allocatable :: fftasum(:)
      double precision, allocatable :: fftasumsq(:)
      complex, allocatable :: fft2d(:)
      double precision, allocatable :: da2d(:)

      double precision, allocatable :: poly_paras(:)

      integer npix_col, npix_row, num_img, nrffta_row,nrffta_col,
     +   nrffta_omega, nfc_2d_row, nfc_2d_col, hkl_min, hkl_max,
     +   i, j , mem_size, array_size
      integer itemp, itemp2, poly_orderb, poly_orderd, poly_order
      integer fftSizeCall,m,n
      integer hkl_maxCall, order_bCall, order_dCall
      integer alloc_stat
      real    pixelSizeCall, voltCall, res_hiCall
      real    pixels(npix_col,npix_row)
      real    auto2DdrawBuf(512,256)
      real    auto2DdataBuf(512,256)
      double precision    poly_paras_call(1024)
      double precision    fftasumCall(1024*2048)
      real    defMajorCall, defMinorCall, angleCall
      real    ampContrastCall, CsCall, chisqCall
      real    packImgCall(npix_col, npix_row, num_img)
      double precision pixel_size, voltage, res_hi, wavelength,
     +                 a, cs, chisq, defmajor, defminor, angle
      real    rtemp, rtemp1, rtemp2
      integer calcIt
      real  saveFftasum(1), saveFftasumsq(1)
      
      real pi, center(2)

C     ------------------ Initialization --------------------

      PI = 3.14159265358979323

      print *, 'col,row,nimg = ', npix_col,npix_row,num_img

      itemp = fftSizeCall
      call TRANS_LENGTH_ESSL( itemp, nrffta_row )
      print *, ' FFT size =',nrffta_row

      nfc_2d_row   = nrffta_row/2
      nfc_2d_col   = nfc_2d_row
      hkl_min      = - nfc_2d_row
      hkl_max      = nfc_2d_row
      hkl_maxCall  = hkl_max
      nrffta_col   = nrffta_row
      nrffta_omega = nrffta_col + 15

      rtemp = pixelSizeCall
      pixel_size = rtemp
      print *, ' pixel_size =', pixel_size

      rtemp = voltCall
      voltage = rtemp
      print *, ' voltage =', voltage

      rtemp = defMajorCall
      rtemp1 = defMinorCall
      rtemp2 = angleCall
      defmajor = rtemp
      defminor = rtemp1
      angle    = rtemp2
      defmajor = rtemp * 10000.0
      defminor = rtemp1 * 10000.0
      print *,'guessed defocus value =',defmajor, defminor, angle

      rtemp = res_hiCall
      res_hi = rtemp
      print *, ' highest resolution =', res_hi

C     Percentage of Amplitude Contrast & Spherical aberration constant
      A = 0.05
      Cs = 20000000.0

      rtemp = ampContrastCall
      rtemp1 = CsCall
      A   = rtemp
      Cs  = rtemp1
      Cs = Cs * 10000000.0

      print *, 'Percentage Amp Contrast =', A, '  Cs =', Cs

C     Set the order of polynomial to fit the background B, 
C     and the order of polynomial to fit signal strengh decay D

      itemp = 5
      itemp2 = 5
      poly_orderb = itemp
      poly_orderd = itemp2
      order_bCall = poly_orderb
      order_dCall = poly_orderd
      print *, ' Order of Background =', poly_orderb 
      print *, ' Order of signal strengh decay =', poly_orderd 

C     Set the Initial Parameters for polynomial functions of B & D, 
C     we formulate the B and D in the order of x, 2nd order is y, 
C     Ord 2:a0 + a1y + a2yy + a3x + a4xy + a5xx  
C     Ord 3:a0 + a1y + a2yy + a3yyy + a4x + a5xy + a6xyy + a7xx + a8xxy + a9xxx

      poly_orderb  = (poly_orderb + 1) * (poly_orderb + 2) / 2
      poly_orderd  = (poly_orderd + 1) * (poly_orderd + 2) / 2
      poly_order   = poly_orderb + poly_orderd + 3
      allocate(poly_paras(poly_order), stat=alloc_stat)

      do j = 1, poly_order-3
          poly_paras(j) = 0.0
      enddo
      poly_paras(1) = 1.0
      poly_paras(poly_orderb+1) = 1.0

      poly_paras(poly_order-2) = defmajor
      poly_paras(poly_order-1) = defminor
      poly_paras(poly_order  ) = angle

      allocate(rffta_omega(nrffta_omega), stat=alloc_stat)
      if(alloc_stat > 0) then
         print *, 'Error in allocating memory for nrffta_omega'
      endif

      mem_size     = (nfc_2d_row +1) * (nfc_2d_col * 2 +1)
      array_size   = mem_size
      allocate(fftasum(mem_size), fftasumsq(mem_size), 
     $     fft2d(mem_size), da2d(mem_size), stat=alloc_stat)

      if (alloc_stat > 0) then
         print *, 'fft space memory allocation failed'
      endif

      do i = 1, mem_size
         fftasum(i)   = 0.
         fftasumsq(i) = 0.
      enddo

C
C *********************************************************************
C *read each image do 2D DFT to fft2d(complex) calculate fftasum and fftasumsq(real)*
C *********************************************************************

      call vrffti( nrffta_col, rffta_omega )
      itemp  = npix_row*npix_col

C     RWa Nov 2005 Try and optimize this - if we've calculated
C     this before then just use that - don't recalc every time 
      if(calcIt .eq. 1) then
         do i = 1, num_img

            do m=1,npix_row
               do n=1,npix_col
                  pixels(n,m) = packImgCall(n,m,i) 
               enddo
            enddo

C     do 2D DFT
            
            center(1) = (npix_row - 1)/2.0
            center(2) = (npix_col - 1)/2.0

            call fft_2dfft(pixels, npix_row, npix_col, center,
     +           nrffta_omega, rffta_omega, nrffta_row, nrffta_col,
     +           hkl_max, hkl_max, fft2d)

C     average them   
            do j = 1, array_size
               amp       = sqrt( real(fft2d(j))**2 + aimag(fft2d(j))**2)
               fftasum(j) = fftasum(j) + amp
               fftasumsq(j) = fftasumsq(j) + amp**2
               saveFftasum(j) = fftasum(j)
               saveFftasumsq(j) = fftasumsq(j)
            enddo

         enddo
      else
         do j = 1, array_size
            fftasum(j) = saveFftasum(j) 
            fftasumsq(j) = saveFftasumsq(j) 
         enddo
      endif   ! if(calcIt .eq. 1) 

      deallocate(rffta_omega, fft2d)

C     calculate some stats, we use fftasum to store avg, fftasumsq store std

      MAP_MIN = fftasum(1)/num_img
      MAP_MAX = fftasum(1)/num_img
      MAP_AVG = 0. 
      MAP_STD = 0.
      do i = 1, array_size
         fftasum(i) = fftasum(i) / num_img
         amp = fftasumsq(i)/num_img - fftasum(i)**2
         if ( amp .gt.0 ) then 
            fftasumsq(i) = sqrt(amp)
         else
            fftasumsq(i) = 0
         endif
         MAP_MIN = MIN(MAP_MIN, fftasum(i))
         MAP_MAX = MAX(MAP_MAX, fftasum(i))
         MAP_AVG = MAP_AVG + fftasum(i)
         MAP_STD = MAP_STD + fftasum(i)**2
      enddo

      MAP_AVG = MAP_AVG / array_size
      amp     = MAP_STD / array_size - MAP_AVG**2
      if (amp.gt.0 ) MAP_STD = sqrt(amp)

      do i = 1, array_size
        fftasum(i) = fftasum(i) / MAP_AVG
        fftasumsq(i) = fftasumsq(i) / MAP_AVG
        fftasumCall(i) = fftasum(i)
      enddo

       PRINT *, ' '
       WRITE(*,1000)
       WRITE(*,1001)
       WRITE(*,1003)
 1000  FORMAT('                  STATICS: averaged FFT')
 1001  FORMAT('FFTSIZE   MINIMUM     MAXIMUM        AVERAGE        STD')
 1003  FORMAT('-------------------------------------------------------')
       WRITE(*,1002)hkl_max, MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD
 1002 FORMAT (I5,2(2x, F10.3),5x, F10.3, 1x, F10.3)

C     calculate ctf values
C     note some different with ctf_para.f, because of the unit
      wavelength = 12.264264967 / 
     +             sqrt (voltage + 0.9784663543E-6 * voltage**2 )

C     wei findout the inner-most first zero  based on the maximum def value 

      deallocate(da2d)

      call cmpt_fit(fftasum, fftasumsq, hkl_max, res_hi,
     +              poly_paras, poly_order, poly_orderb, poly_orderd,
     +              wavelength, A, Cs, pixel_size, chisq,auto2DdrawBuf,
     +              auto2DdataBuf)

      defmajor = poly_paras(poly_order-2) /10000.
      defminor = poly_paras(poly_order-1) /10000.
      angle    = poly_paras(poly_order)

      call robStuffPolyParas(poly_paras_call, poly_paras, poly_order)

      angle    = angle * 180.0 / 3.14159265358979323
      if (defmajor.lt.defminor) then
         rtemp = defmajor
         defmajor = defminor
         defminor = rtemp
         if (angle.gt.0.0) then 
            angle = angle - 90.0
         else
            angle = angle + 90.0
         endif
      endif
	if (angle.lt.0.0) then
            angle = angle +360.
        endif

      deallocate(fftasum, fftasumsq, poly_paras)

      defMajorCall = defmajor
      defMinorCall = defminor
      angleCall = angle
      chisqCall = chisq

      print *, '**********finished nicely**********'

      return
      end subroutine yji_ctf2dfit


      subroutine robStuffPolyParas(poly_paras_call, poly_paras, 
     $     poly_order)
      integer i, poly_order
      double precision poly_paras(poly_order), poly_paras_call(1024)

      do i = 1, poly_order
         poly_paras_call(i) = poly_paras(i)
      enddo

      return
      end subroutine robStuffPolyParas


      subroutine robangle_curves(angdraw, angle, poly_paras,
     +     defmajor, defminor, hkl_max, orderb,
     +     orderd, fftasum, voltage, ampContrast, 
     +     Cs, pixel_size,
     +     sCall, sminusbCall, ctfsqCall, dCall, bCall)
      
      implicit none

      real theta, nu, pi, costh, sinth, defmajor, defminor
      real xx, yy, rx, ry, arg1, arg2, B, D, fw, fw2
      real ctf, ctf2, ampContrast, Cs
      integer  ii, orderb, orderd, irx, iry
      real def, xbit, ybit, s
      real angdraw, voltage, pixel_size
      real sCall(1024), sminusbCall(1024), ctfsqCall(1024)
      real dCall(1024), bCall(1024)
      integer hkl_max, kk, i, j, k
      double precision fftasum(1024*2048)
      double precision poly_paras(1024)
      double precision wavelength, angle

      pi = 3.14159265358979323
      wavelength = 12.264264967 / 
     +             sqrt (voltage + 0.9784663543E-6 * voltage**2 )

      theta   = (angdraw-angle)*pi/180.0
      costh   = cos(theta)
      sinth   = sin(theta)
      def     = 10000.0*(defmajor*costh*costh + defminor*sinth*sinth)

      xx = abs(cos(angdraw*pi/180.0))
      yy = abs(sin(angdraw*pi/180.0))

      do i=1,hkl_max
         rx = xx*i
         ry = yy*i
         kk=1

         B = 0.0
         arg1 = 1
         do j=1, orderb+1
            arg2 = 1
            do k =1, orderb-j+2
               B = B + poly_paras(kk)*arg1*arg2
               arg2 = arg2*ry
               kk = kk+1
            enddo
            arg1=arg1*rx
         enddo

         D=0.0
         arg1 = 1
         do j=1, orderd+1
            arg2 = 1
            do k =1, orderd-j+2
               D = D + poly_paras(kk)*arg1*arg2
               arg2 = arg2*ry
               kk = kk+1
            enddo
            arg1=arg1*rx
         enddo
         
         fw  = wavelength*i/(hkl_max*2.0*pixel_size)
         fw2 = fw*fw
         nu = pi*2.0*(Cs*fw2*fw2/4.0-def*fw2/2.0)/wavelength
         ctf = sqrt(1.0-ampContrast*ampContrast)*sin(nu) -
     $        ampContrast*cos(nu)
         ctf2 = ctf*ctf
         irx = rx
         iry = ry
         xbit = abs(rx - irx)
         ybit = abs(ry - iry)
         s = 0.0

         if(ry >= 0.0) then
            ii = (iry+hkl_max)*(hkl_max+1)+irx+1
            s = s + fftasum(ii)*(1.0-xbit)*(1.0-ybit)
            ii = (iry+1+hkl_max)*(hkl_max+1)+irx+1
            s = s + fftasum(ii)*(1.0-xbit)*ybit
            ii = (iry+hkl_max)*(hkl_max+1)+irx+1+1
            s = s + fftasum(ii)*xbit*(1.0-ybit)
            ii = (iry+1+hkl_max)*(hkl_max+1)+irx+1+1
            s = s + fftasum(ii)*xbit*ybit
         else
            ii = (iry+hkl_max)*(hkl_max+1)+irx+1
            s = s+ fftasum(ii)*(1.0-xbit)*(1.0-ybit)
            ii = (iry-1+hkl_max)*(hkl_max+1)+irx+1
            s = s + fftasum(ii)*(1.0-xbit)*ybit
            ii = (iry+hkl_max)*(hkl_max+1)+irx+1+1
            s = s + fftasum(ii)*xbit*(1.0-ybit)
            ii = (iry-1+hkl_max)*(hkl_max+1)+irx+1+1
            s = s + fftasum(ii)*xbit*ybit
         endif

         sCall(i) = s
         sminusbCall(i) = s - B
         ctfsqCall(i) = ctf2
         dCall(i) = D
         bCall(i) = B
      enddo
      
      return 
      end subroutine robangle_curves
