C  ********************************************************************
C  *                                                                  *
C  *                *****     EMCORAVG.FOR     *****                  *
C  *                                                                  *
C  * THIS PROGRAM IS A COMBINED VERSION OF EMCOR.FOR AND EMCORAVG.FOR *
C  * ---------------------------------------------------------------- *
C  * PART A:   AUTO AND CROSS CORRELATION PROGRAM.                    *
C  *                                                                  *     
C  *                                                                  *
C  *    LIMITS: UP TO 512x512 REFERENCE IMAGE, AND UP TO 1024x1024    *
C  *      MAIN IMAGE AND FFT.                                         *
C  *                                                                  *
C  *                   INPUT:  PIF FORMAT IMAGES AND REFERENCE IMAGES *
C  *                           PIF MODE IS 0, BOTH ARE SCANNED IMAGES *
C  *                                                                  *
C  *                           FOR CROSS CORRELATION: FFT SIZE IS     *
C  *                           DEPENDENT ONLY ON THE MAIN IMAGE SIZE, *
C  *                           THEREFORE, A USER NEEDS TO MAKE SURE   *
C  *                           THAT THE REFERENCE IMAGE FILENAME IS   *
C  *                           ENTERED FIRST AND IT IS SMALLER THAN   *
C  *                           THAT OF THE MAIN IMAGE.                *
C  *                                                                  *
C  *                   OUTPUT: PIF FORMAT AUTO/CROSS  CORRELATION     *
C  *                           IMAGE FILE, PIF MODE = 0               *
C  *                                                                  *
C  * PART B: SCAN THE CROSSCORRELATION IMAGE FOR PEAKS ABOVE LOWLIM   *
C  *         AND AVERAGING DATA FROM THE MAIN IMAGE USING WINDOWS THE *
C  *         SAME SIZE AS THE REFERENCE IMAGE.                        *
C  *                   OUTPUT: BEST PEAKS AND AVERAGED IMAGE          *
C  *                                                                  *
C  *    FEBRUARY 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER.        *
C  *    MAY 1988  PURDUE UNIVERSITY VAX 8550, T. BAKER.               *
C  *    MODIFIED  5-SEP-1990 BY T. BAKER.                             *
C  *    LAST MODIFIED  JAN-1999 BY W. ZHANG &  T. BAKER.              *
C  *                                                                  *
C  ********************************************************************

C  *****************************************************************
C  *                                                               *
C  *            *****     SUBROUTINE COR_FIRST     *****           *
C  *                                                               *
C  *    FIRST DIMENSION OF CORRELATION FFT ((IDIM2/2)+1 ROWS )     *
C  *****************************************************************
C
      SUBROUTINE COR_FIRST (IDIM1, IDIM2, ARRAY, A, B, C, icor_pass)
C
      IMPLICIT NONE
C
      COMMON    ICOR , LUN_IMG, LUN_REF, LUN_CCOUT
      
      INTEGER*4 ICOR , LUN_IMG, LUN_REF, LUN_CCOUT
      INTEGER*4 IDIM1, IDIM2,   L, J1, icor_pass
      INTEGER*4 J, KK, K, IY, ILIM, II, IYMAX, IRES
      REAL*4    ARRAY(2*IDIM1,((IDIM2/2)+1))
      REAL*4    A(IDIM1*2, IDIM2/2+1), B(IDIM1*2, IDIM2/2+1), C(IDIM1*2)
      REAL*4    WORK, TST, YSQ, DIMDIM, RESSQ, RESX, DIV
C  ----------------------------------------------------------------------------
C
C  ************************************************************
C  *    OPTION TO SET RESOLUTION LIMIT - - LOW-PASS FILTER    *
C  ************************************************************
      icor = icor_pass
      print *, 'icor = ', icor
      IYMAX = (IDIM2/2)+1
      RESX  = SQRT(REAL(IDIM1/2)**2.0+REAL(IDIM2/2)**2.0)
      IRES  = RESX + 0.5
      WRITE(6,1000) IRES
Ca      READ (5,'(I)') IRES
      IF(IRES.LE.0) THEN
         ILIM = 0
         IRES = RESX + 0.5
         GO TO 10
       ELSE
         ILIM = 1
      END IF
      IF(IRES.LT.IYMAX) IYMAX = IRES
      RESSQ = IRES*IRES
C  *******************************************************
C  *    TRANSFORM FIRST DIMENSION  ((IDIM2/2)+1 ROWS)    *
C  *******************************************************
   10      IY     = 0
      DIMDIM = REAL(IDIM1)*REAL(IDIM2)
      DIV    = REAL(IDIM2/2) + 1.0
C
        PRINT *, '**********************************'
        PRINT *, '* Back-Transform first dimension *'
        PRINT *, '**********************************'
C
      DO J = 1,((IDIM2/2)+1)
        IY = IY+1
        J1=((IDIM2/2)+1)+1-j
        IF(IY.LE.IYMAX.AND.IY.LE.IRES) THEN
C
           IF(ICOR.NE.1) THEN
C             
             DO II = 1,(2*IDIM1),2
               C(II)   = (A(II,J)*B(II,J)+A(II+1,J)*B(II+1,J))/DIMDIM
               C(II+1) = (A(II,J)*B(II+1,J)-A(II+1,J)*B(II,J))/DIMDIM
             ENDDO
           ELSE
             DO II = 1,(2*IDIM1),2
               C(II)   = (B(II,J)*A(II,J)+B(II+1,J)*B(II+1,J))/DIMDIM
               C(II+1) = 0.0
             ENDDO
          END IF
          
          IF(ILIM.EQ.1) THEN
            YSQ = (IY-1)**2.0                  ! RESOLUTION TEST
            DO K = 1,((IDIM1/2)+1)
              TST = YSQ+(K-1)**2.0
              IF(TST.GT.RESSQ) THEN
                KK      = 2*K-1
                C(KK)   = 0.0
                C(KK+1) = 0.0
                IF(K.NE.1) THEN
                  C((2*IDIM1+3)-KK) = 0.0
                  C((2*IDIM1+2)-KK) = 0.0
                ENDIF
              ENDIF
            ENDDO
          ENDIF
C           ************************************************
C  ---------*    TRANSFORM THE ROW AND STORE IT IN ARRAY   *
C           ************************************************
          CALL FOURT(C,IDIM1,1,-1,1,WORK)
          DO L = 1,IDIM1*2
            ARRAY(L,J) = C(L)
          ENDDO
        ELSE
          DO L = 1,IDIM1*2                        ! ZERO REST OF ARRAY
            ARRAY(L,J) = 0.0
          ENDDO
        ENDIF
      ENDDO
C
       
       CALL BLANKL (0)

      RETURN
C
 1000      FORMAT('$RES LIMIT IN TRANSFORM PIXELS (<CR>=',I4,'): ')
      END
C
C  ***********************************************************************
C  *                                                                     *
C  *              *****     SUBROUTINE COR_SECOND     *****              *
C  *                                                                     *
C  *    THIS COMPUTES THE SECOND DIMENSION OF THE FFT (IDIM1 COLUMNS)    *
C  ***********************************************************************
C
      SUBROUTINE COR_SECOND (IDIM1, IDIM2, ARRAY, MAP, IMG, B, 
     1     MAP_AVG, MAP_STD, img_min, img_max)
C
      IMPLICIT NONE

      COMMON    ICOR, LUN_IMG, LUN_REF, LUN_CCOUT
      
      INTEGER*4 ICOR, LUN_IMG, LUN_REF, LUN_CCOUT
      INTEGER*4 IDIM1, IDIM2, IMG_MIN, IMG_MAX
      INTEGER*4 I, II1, II2, J, JJ, K
      INTEGER*4 NCI2, NRI2, NBKG
      real*4    LUTHII, LUTLOI 
      real*4    IMG(IDIM1, IDIM2), img_bkg
      REAL*4    ARRAY(2*IDIM1,((IDIM2/2)+1)), MAP(IDIM1,IDIM2)
      REAL*4    B(IDIM2*2), SCALE, WORK, SUMSQ, TOT, VAR
      REAL*4    DIMDIM, MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD
      REAL*4    IMG_AVG, IMG_STD 
      LOGICAL   MAP_INCORE
C  ----------------------------------------------------------------------------
C
      PRINT *, '***********************************'
      PRINT *, '* Back-Transform second dimension *'
      PRINT *, '***********************************'
C
      DIMDIM = REAL(IDIM1)*REAL(IDIM2)
      SUMSQ  = 0.0
      TOT    = 0.0
C
      DO I = 1,IDIM1                  ! MAIN LOOP OF FFT
        II1 = 2*I - 1
        II2 = II1 + 1
        JJ  = 1
        DO J = 1,(IDIM2/2)+1
          B(JJ)   = ARRAY(II1,J)/DIMDIM
          B(JJ+1) = ARRAY(II2,J)/DIMDIM
          JJ      = JJ + 2
        ENDDO
C  ***************************
C  *    SET FRIEDEL MATES    *
C  ***************************
        K = IDIM2 - 1
        DO J = (IDIM2+3),(2*IDIM2),2
          B(J)   =  B(K)
          B(J+1) = -B(K+1)
          K      = K-2
        ENDDO
        CALL FOURT (B, IDIM2, 1, -1, 1, WORK)                  ! TRANSFORM COLUMN
        DO J = 1,IDIM2
          MAP(I,J) = B(2*J-1)
        ENDDO
      ENDDO                                          ! MAIN LOOP OF FFT
      CALL BLANKL (0)
C
        PRINT *, '********************************************'
        PRINT *, '* Find MIN/MAX values in correlation IMAGE *'
        PRINT *, '********************************************'

      MAP_INCORE = .TRUE.
      CALL MAP_STATS (MAP, IDIM1, IDIM2, 1, MAP_MIN, MAP_MAX,
     1 MAP_AVG, MAP_STD, 1)
C
        PRINT *, '***************************************************'
        PRINT *, '* Rescale correlation MAP to values between 6-255 *'
        PRINT *, '***************************************************'

      LUTLOI =   6.
      LUTHII = 255.
      SCALE = LUTHII-LUTLOI/(MAP_MAX-MAP_MIN)
      DO J = 1,IDIM2
        DO I = 1,IDIM1
          MAP(I,J) = LUTLOI + (MAP(I,J)-MAP_MIN)*SCALE
          SUMSQ    = SUMSQ  +  MAP(I,J)**2
          TOT      = TOT    +  MAP(I,J)
        ENDDO
      ENDDO
C
      PRINT *, '*************************************************'
      PRINT *, '* Write out correlation MAP as an IMAGE to disk *'
      PRINT *, '*************************************************'

       NCI2    =  IDIM1
       NRI2    =  IDIM2
       IMG_MIN = LUTLOI
       IMG_MAX = LUTHII
       IF(MAP_MIN.LE.0.0) THEN
          IMG_BKG = LUTLOI + NINT(-MAP_MIN*SCALE)
        ELSE
          IMG_BKG = LUTLOI
       ENDIF
       IMG_STD = 0.0
       IMG_AVG = TOT/(REAL(NCI2)*REAL(NRI2))
       VAR     = (SUMSQ/(REAL(NCI2)*REAL(NRI2))) - IMG_AVG**2
       IF(VAR.GE.0.0) IMG_STD = SQRT(VAR)
       NBKG = IMG_BKG

C  **************************************
C  *    WRITE OUT THE CORRELATION IMAGE *
C  **************************************
      DO I = 1, IDIM1
         DO J = 1, IDIM2
Ca            IMG(I,J) = NINT(MAP(I,J))
            IMG(I,J) = MAP(I,J)
         ENDDO
      ENDDO
        CALL IMG_STATS1 (IMG, IDIM1, IDIM2, IMG_MIN, IMG_MAX, 
     1            IMG_AVG, IMG_STD, IMG_BKG) 
      MAP_AVG = IMG_AVG
      MAP_STD = IMG_STD


Ca      CALL IMG_WRITE_FRAME(LUN_CCOUT, IMG,BYTE_BUFFER, IMG_NUM,
Ca      1  IDIM1,IDIM2,IMG_MIN, IMG_MAX,IMG_AVG, IMG_STD,CCOUT_Head%mode)


C
      RETURN
      END

 
      
C  ***************************************************************************
C  *                                                                         *
C  *                *****     SUBROUTINE CORAVG_LIM     *****                *
C  *                                                                         *
C  *    USE GRAPHICS DISPLAY TO ESTIMATE VALUE FOR LOWLIM.                   *
C  *                                                                         *
C  *    DECEMBER 1985  T. BAKER, PURDUE UNIVERSITY VAX 11/750                *
C  *    LAST MODIFIED 14-OCT-87 T. BAKER                                     *
C  *    MODIFIED BY W. ZHANG: IMG (INTEGER2-> REAL4)                         *
C  ***************************************************************************
C
      SUBROUTINE CORAVG_LIM(NLIM, PEAKS, NPEAK, IMG, CCNC, CCNR, LOWLIM)

      IMPLICIT NONE

      INTEGER*4 CCNC, CCNR, NLIM, LOWLIM, NPEAK
      INTEGER*4 EM_WRT
      INTEGER*4 I
      REAL*4    PEAKS(3,NLIM), IMG(CCNC,CCNR)
C  ----------------------------------------------------------------------------
C

C  *****************************************************************
C  *    ENTER VALUES FOR LOWLIM BY HAND                            *
C  *                                                               *
C  *    ESTIMATE A VALUE FOR LOWLIM FROM THE MEAN AND STD OF THE   *
C  *     CROSS-CORRELATION IMAGE                                   *
C  *****************************************************************
Ca Aren't we passing this in here???
Ca      LOWLIM = CC_AVG + 2*CC_STD
Ca      IF(LOWLIM.GT.255) LOWLIM = 255
  100      WRITE(6,'('' LOWLIM <CR>='',I4,'': '',$)') LOWLIM
Ca      READ (5,'(I)',ERR=100) I
      i = 0
      IF(I.GT.0) THEN
        IF(I.GT.255) I = 255
        LOWLIM = I
      ENDIF
      EM_WRT = 1
      CALL PEAK_SEARCH (IMG,CCNC,CCNR,PEAKS,NLIM,NPEAK,LOWLIM,EM_WRT)
      IF(NPEAK.LE.0) THEN
        LOWLIM = LOWLIM + 10
        if(lowlim .gt. 255) go to  200   ! ?????
        GO TO 100
      ENDIF
      CALL PEAK_SORT_HI (PEAKS, NPEAK)             ! SORT PIXELS > LOWLIM
  200      WRITE(6,'(''$LOWLIM OK? YES<CR> NO(1): '')')    ! CHECK IF LOWLIM IS OK
Ca      READ (5,'(I)',ERR=200) I
      i = 0
      IF(I.EQ.1) GO TO 100
      RETURN
C
      END


C  ***********************************************************************
C  *                                                                     *
C  *               *****     SUBROUTINE CORAVG_SUM     *****             *
C  *                                                                     *
C  *    ADDS TOGETHER WINDOWS NCOLMP x NROWMP AT POSITIONS IN IMAGE      *
C  *      GIVEN BY SUBROUTINE PEAK_SEARCH (EACH PEAK POSITION REFERS TO  *
C  *      THE LOWER LEFT CORNER OF THE WINDOW IN "IMG"                   *
C  *                                                                     *
C  *      MODIFICATIONS DONE TO ALLOW UNIT CELL INTERPOLATION BY C.HO    *
C  *                             SEPTEMBER 1986                          *
C  *    LAST MODIFIED 12-SEP-88 BY T. BAKER                              *
C  ***********************************************************************
C
      SUBROUTINE CORAVG_SUM (MAINIMG, NCOL, NROW, AVGIMG, MAP, NCOLMP,
     $     NROWMP, PEAKS, NPEAK_UNIQUE)
C
      IMPLICIT NONE

      COMMON  LUN_IMG, LUN_REF, LUN_CCOUT, LUN_AVGOUT

      INTEGER*4 LUN_IMG, LUN_REF, LUN_CCOUT, LUN_AVGOUT
      INTEGER*4 NCOL, NROW, NCOLMP, NROWMP, NAVG
      INTEGER*4 NPEAK_UNIQUE, ISTEP, IAVG
      INTEGER*4 IX, IY, ITEST1, ITEST2
      INTEGER*4 INDX, INDX1, INDY, INDY1, NUMCHARI4
      INTEGER*4 N2
      INTEGER*4 EM_WRT, I, J, K, I1, I2
      real*4    MAINIMG(NCOL,NROW)
      real*4    AVGIMG(NCOLMP,NROWMP)
       
      REAL*4    MAP(NCOLMP,NROWMP), PEAKS(3,NPEAK_UNIQUE)
      REAL*4    DX, DY, D1, D2, D3, D4, AVG
      REAL*4      MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD

      N2 = NUMCHARI4(NPEAK_UNIQUE)
  100      WRITE(6,'(''$RANGE OF CORRELATION PEAK POSITIONS TO USE TO
     1 COMPUTE AVERAGE (<CR>=1,'',I8,'': '')') NPEAK_UNIQUE
      i1 = 0
      i2 = 0
      IF(I1.EQ.0.AND.I2.EQ.0) THEN
        I1 = 1
        I2 = NPEAK_UNIQUE
      ENDIF
      IF(I1.LE.0 .OR. I1.GT.NPEAK_UNIQUE .OR. I2.LE.0
     1          .OR. I2.GT.NPEAK_UNIQUE) THEN
        CALL BELL (2)
        PRINT *, '*********************'
        PRINT *, '* Range not allowed *'
        PRINT *, '*********************'
        GO TO 100
      ENDIF
  200      WRITE(6,'(''$INCREMENT BETWEEN PEAKS USED TO SELECT CELLS
     1 FOR AVERAGING (<CR>=1): '')')
      istep = 0
      IF(ISTEP.LE.0) ISTEP = 1
      IF(ISTEP.GT.(I2-I1+1)) THEN
        CALL BELL (2)
        PRINT *, '****************************'
        PRINT *, '* Increment size too large *'
        PRINT *, '****************************'
        GO TO 200
      ENDIF
      IAVG =  NINT(REAL(I2 - I1 + 1)/REAL(ISTEP))

      WRITE(6,'(''$DISPLAY INTERMEDIATE RESULTS ON GRAPHICS
     1     NO<CR> YES(1): '')')

      PRINT*, '      N    TOT     IX     IY   PEAK'
      PRINT*, '-----------------------------------'
      PRINT *
      NAVG = 0

      DO 400 I = I1,NPEAK_UNIQUE,ISTEP
         NAVG   = NAVG + 1
         IX     = IFIX(PEAKS(1,I)) ! "LOWER" LEFT COL POSITION
         IY     = IFIX(PEAKS(2,I)) ! "LOWER" LEFT ROW POSITION
         DX     = PEAKS(1,I) - IX
         DY     = PEAKS(2,I) - IY
         ITEST1 = IX + NCOLMP - 1
         ITEST2 = IY + NROWMP - 1
         
C     ******************************
C------------*    SUM WINDOWS OF IMAGE    *
C     ******************************
         IF(ITEST1+1.GT.NCOL.OR.ITEST2+1.GT.NROW) THEN
            NAVG = NAVG - 1     ! WINDOW PARTIALLY OUTSIDE IMAGE BOUNDARY
            GO TO 400           ! SKIP THIS WINDOW
         ENDIF
         IF(NAVG.GT.IAVG) THEN
            NAVG = NAVG - 1
            GO TO 500
         ENDIF
         
         DO J = 1,NROWMP
            INDY  = IY + J - 1
            INDY1 = INDY + 1
            DO K = 1,NCOLMP
               INDX     = IX + K - 1
               INDX1    = INDX + 1
               D1       = MAINIMG(INDX, INDY)
               D2       = MAINIMG(INDX1,INDY)
               D3       = MAINIMG(INDX1,INDY1)
               D4       = MAINIMG(INDX, INDY1)
               MAP(K,J) = MAP(K,J) + D1 + DX*(D2-D1) + DY*(D4-D1) +
     1              DX*DY*(D1-D2+D3-D4)
            ENDDO
         ENDDO
         WRITE(6,'(''+'',4I7,F7.0)') I, NAVG, IY, IX, PEAKS(3,I)
 400  CONTINUE
      EM_WRT = 1
 500  CALL MAP_STATS (MAP, NCOLMP, NROWMP, 1, MAP_MIN,
     1     MAP_MAX, MAP_AVG, MAP_STD, EM_WRT)
      
      WRITE(6,'(''$RESCALE THE IMAGE? NO<CR> YES(1): '')')
      i = 0
      IF(I.EQ.1) THEN
         WRITE(6,'(''$ENTER NEW MIN,MAX: '')')
      ENDIF
      
C  ************************************************
C  *    CALCULATE BACKGROUND VALUE FOR "IMG1"     *
C  ************************************************
      LUN_AVGOUT=4

      WRITE(6,'(''$COMPUTE VARIANCE MAP? YES<CR> NO(1): '')')
      i = 0
      IF(I.EQ.1) RETURN
C
      AVG = REAL(NAVG)
C  **********************************************************************
C  *    NORMALIZE "MAP" AND STORE IN "AVGIMG".                          *
C  *    INITIALIZE "MAP" ARRAY FOR LATER USE TO STORE VARIANCE IMAGE    *
C  **********************************************************************
      DO J = 1,NROWMP
        DO I = 1,NCOLMP
          AVGIMG(I,J) = (MAP(I,J)/AVG)
          MAP(I,J)    = 0.0
        ENDDO
      ENDDO

      END
      
C  *******************************************************************
C  *                                                                 *
C  *             *****     SUBROUTINE IMG_STATS1     *****            *
C  *                                                                 *
C  *    DETERMINE IMG_BKG, IMG_MIN, IMG_MAX, IMG_AVG AND IMG_STD     *
C  *     OF "IMAGE" VALUES.                                          *
C  *                                                                 *
C  *    NOTE THAT THE VALUE DETERMINED FOR IMG_MIN IS THE MINIMUM    *
C  *     NON-ZERO VALUE IN THE IMG ARRAY                             *
C  *                                                                 *
C  *    APRIL 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER           *
C  *    AUGUST 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.           *
C  *    LAST MODIFIED 24-JUL-1990 BY T. BAKER                        *
C  *******************************************************************
C
      SUBROUTINE IMG_STATS1 (IMG, NCOL, NROW,IMG_MIN, IMG_MAX, 
     1            IMG_AVG, IMG_STD, IMG_BKG)

      IMPLICIT NONE

      REAL*4    TOT, SUMSQ, VAR, X
      INTEGER*4 NCOL, NROW, ARRAY_SIZE, I, IMG_MIN, IMG_MAX
      REAL*4 IMG_BKG
      REAL*4    IMG_AVG, IMG_STD
      REAL*4 IMG(NCOL*NROW)
C  ----------------------------------------------------------------------------
C
      ARRAY_SIZE = NCOL*NROW
      IMG_MIN    = HUGE(IMG_MIN)-1
      IMG_MAX    = -IMG_MIN
      SUMSQ      =    0.0
      TOT        =    0.0
C
      WRITE(*,1000) 
C
      DO I = 1,ARRAY_SIZE
        X          = REAL(IMG(I))
        SUMSQ      = SUMSQ + X*X
        TOT        = TOT + X
        IF(IMG(I).NE.0) THEN
          IMG_MIN = MIN(IMG_MIN,INT(IMG(I)))
          IMG_MAX = MAX(IMG_MAX,INT(IMG(I)))
        ENDIF
      ENDDO
C
      IMG_AVG = TOT/REAL(ARRAY_SIZE)
      IMG_STD = 0.0
      VAR     = (SUMSQ/REAL(ARRAY_SIZE)) - IMG_AVG**2
      IF(VAR.GE.0.0) IMG_STD = SQRT(VAR)
C  *************************
C  *    COMPUTE IMG_BKG    *
C  *************************
      TOT = 0.0
      DO I = 1,NCOL                                    ! BOTTOM ROW
        TOT = TOT + IMG(I)
      ENDDO
      DO I = ARRAY_SIZE-NCOL+1,ARRAY_SIZE                  ! TOP ROW
        TOT = TOT + IMG(I)
      ENDDO
      DO I = NCOL+1,ARRAY_SIZE-NCOL,NCOL                  ! LEFT COLUMN
        TOT = TOT + IMG(I)
      ENDDO
      DO I = 2*NCOL,ARRAY_SIZE-NCOL,NCOL                  ! RIGHT COLUMN
        TOT = TOT + IMG(I)
      ENDDO
      IMG_BKG = TOT/REAL(2*NCOL+2*(NROW-2))
C
      WRITE(*,1010) IMG_MIN,IMG_MAX,IMG_AVG,IMG_STD,IMG_BKG
      RETURN
C
 1000      FORMAT(
     1 /,' *****************************************************',
     2 /,' *   NON-ZERO                                        *',
     3 /,' *   MINIMUM   MAXIMUM   AVERAGE        STD     BKG  *',
     4 /,' * ------------------------------------------------- *',
     5 /,' *            ...  WAIT for answer  ...              *')
 1010      FORMAT('+*',I8,I10,3F12.2,'  *',
     1 /,' *****************************************************',/)
      END
