C  ********************************************************************
C  *                                                                  *
C  *            *****     SUBROUTINE MAP_FFT_FILL_tsbOrien     *****  *
C  *                                                                  *
C  *    LOAD NCOL X NROW "MAP" INTO "FFT" ARRAY TO COMPUTE FFT        *
C  *                                                                  *
C  *    NOTE that this is in TSB orientation - use                    *
C  *    r4_img_fft_fill() if you're inside robEM                      *
C  ********************************************************************

      SUBROUTINE MAP_FFT_FILL_TSBORIEN (img, NCOL, NROW, FFT,
     1           IDIM1,IDIM2,imgBkgP)

      IMPLICIT NONE

      INTEGER*4 NCOL, NROW, IDIM1, IDIM2, I, J
      integer*4 img_bkg,imgBkgP
      REAL*4    IMG(NCOL,NROW)
      REAL*4    FFT(2*IDIM1,IDIM2), TMP

      img_bkg   = imgBkgP
      FFT       = 0.0 ! Fortran 90 array syntax

      DO J = 1,NROW
        DO I = 1,NCOL
          TMP            = IMG(I,(nrow-J+1)) - real(IMG_BKG)
          FFT((2*I)-1,J) = TMP
        ENDDO
      ENDDO

      RETURN
      END subroutine map_fft_fill_tsborien


C  ****************************************************************************
C  *                                                                          *
C  *                 *****     SUBROUTINE FFT_FLTMSK_ROB *****                *
C  *                                                                          *
C  *    SUB-OPTION "B" OF EMFFT_FILTER SUBPROGRAM                             *
C  *                                                                          *
C  *    APPLY 2D RECIPROCAL LATTICE MASK WITH SHARP OR SOFT HOLE EDGES        *
C  *     RADIUS = RADIUS OF THE CIRCULAR MASK HOLE.                           *
C  *                                                                          *
C  *    AUGUST 1985  PURDUE UNIVERSITY VAX 11/750, T. BAKER.                  *
C  *    LAST MODIFIED  2-MAR-1992 BY T. BAKER.                                *
C  ****************************************************************************

      SUBROUTINE FFT_FLTMSK_ROB (FFT,MASK,IDIM1,IDIM2,axp,ayp,bxp,byp,
     1        radius,icut, lowResoFilter, hiResoFilter)

      IMPLICIT NONE

      INTEGER*4 I, II, J, JJ, IDIM1, IDIM2, NH, NK, NH1, NK1, NHOLE
      INTEGER*4 IX, IX1, IX2, IY, IY1, IY2, ICUT, IXTST, NHOLE_LIM
      PARAMETER (NHOLE_LIM = 260000)
      INTEGER*2 MASK(IDIM1,((IDIM2/2)+1))
      REAL*4    FFT(2*IDIM1,((IDIM2/2)+1)), RTST, XLIM, YLIM
      REAL*4    X(3,NHOLE_LIM), Y(3,NHOLE_LIM)
      REAL*4    XCEN, YCEN, X1, X2, Y1, Y2, XB, YB, RADIUS, RSQ, RYSQ
      real*4    ax, ay, bx, by, axp, ayp, bxp, byp, thisMaskVal
      integer*4 intYsq, intXsq, lowResoFilter, hiResoFilter

      RTST = RADIUS*RADIUS
      IF(ICUT.NE.1) ICUT = 0

C     TRAP FOR FLOATING DIVIDE BY ZERO
      ax = axp
      ay = ayp
      bx = bxp
      by = byp

      IF(abs(AX) < 0.0001) ax = 0.0001      
      IF(abs(AY) < 0.0001) ay = 0.0001      
      IF(abs(bx) < 0.0001) bx = 0.0001      
      IF(abs(by) < 0.0001) by = 0.0001      
      I    = 1.5*REAL(IDIM1)/AX
      J    = 1.5*REAL(IDIM1)/AY
      NH   = MIN(ABS(I),ABS(J))
      I    = 1.5*REAL(IDIM2)/BX
      J    = 1.5*REAL(IDIM2)/BY
      NK   = MIN(ABS(I),ABS(J))
      NH1  = (NH-1)/2
      NK1  = (NK-1)/2
      XLIM = REAL(IDIM1)
      YLIM = REAL(IDIM2/2)

C     DETERMINE CENTER AND BOUNDARIES FOR EACH HOLE
      NHOLE = 0
      DO I = 1,NK
         II = I-NK1-1
         XB = II*BX
         YB = II*BY
         DO J = 1,NH
            JJ   = J-NH1-1
            XCEN = XB+JJ*AX+1.0+(IDIM1/2)
            X1   = XCEN-RADIUS
            X2   = XCEN+RADIUS
            IF(X1 < 1.0)  X1 = 1.0
            IF(X2 < 1.0)  X2 = 1.0
            IF(X1 > XLIM) X1 = XLIM
            IF(X2 > XLIM) X2 = XLIM
            YCEN = YB+JJ*AY+1.0
            Y1   = YCEN-RADIUS
            Y2   = YCEN+RADIUS
            IF(Y1 < 1.0)  Y1 = 1.0
            IF(Y2 < 1.0)  Y2 = 1.0
            IF(Y1 > YLIM) Y1 = YLIM
            IF(Y2 > YLIM) Y2 = YLIM
            IF(X1.NE.X2.AND.Y1.NE.Y2) THEN
               NHOLE      = NHOLE+1
               X(1,NHOLE) = XCEN
               X(2,NHOLE) = X1
               X(3,NHOLE) = X2
               Y(1,NHOLE) = YCEN
               Y(2,NHOLE) = Y1
               Y(3,NHOLE) = Y2
            ENDIF
         ENDDO
      ENDDO
      

C     ZERO THE FILTER MASK 
      DO IY = 1,(IDIM2/2)+1
         DO IX = 1,IDIM1
            MASK(IX,IY) = 0
         ENDDO
      ENDDO
      
C     CONSTRUCT 2D LATTICE FILTER MASK
      DO I = 1,NHOLE
         IX1 = X(2,I)
         IX2 = X(3,I)
         IY1 = Y(2,I)
         IY2 = Y(3,I)
         DO IY = IY1,IY2
            intYsq = iy*iy
            RYSQ = (IY-Y(1,I))**2
            IF(RYSQ <= RTST) THEN
               DO IX = IX1,IX2
                  intXsq= (ix-257.0)*(ix-257.0)
                  if((intYsq+intXsq) >= hiResoFilter .and. 
     $                 (intYsq+intXsq) <= lowResoFilter ) then
                     RSQ = RYSQ + (IX-X(1,I))**2
                     IF(RSQ <= RTST) THEN
                        IF(ICUT.NE.1) THEN ! Hard cutoff
                           MASK(IX,IY) = 1000
                        ELSE               ! Soft cutoff
                           MASK(IX,IY) = NINT(1000.0*EXP(-RSQ/RTST))
                        ENDIF
                     ENDIF
                  endif
                  
               ENDDO
            ENDIF
         ENDDO
      ENDDO

C     APPLY FILTER MASK TO FFT
      IXTST = IDIM1/2
      DO IY = 1,(IDIM2/2)+1
        DO IX = 1,IDIM1
          IF(IX <= IXTST) THEN
             IX1 = 2*IX-1+IDIM1
           ELSE
             IX1 = 2*IX-1-IDIM1
          ENDIF
          IX2         = IX1+1
Ca ORIG          thisMaskVal = MASK(IX,IY)/1000.0
Ca New          thisMaskVal = MASK(idim1-IX,IY)/1000.0
Ca Works for OJ          thisMaskVal = MASK(IX,IY)/1000.0
          thisMaskVal = MASK((idim1-IX)+2,IY)/1000.0
          FFT(IX1,IY) = FFT(IX1,IY)*thisMaskVal
          FFT(IX2,IY) = FFT(IX2,IY)*thisMaskVal
        ENDDO
      ENDDO

C     Now do second half
      IXTST = IDIM1/2
      DO IY = (idim2/2)+2,(IDIM2)
        DO IX = 1,IDIM1
          IF(IX <= IXTST) THEN
             IX1 = 2*IX-1+IDIM1
           ELSE
             IX1 = 2*IX-1-IDIM1
          ENDIF
          IX2         = IX1+1
Ca OLDWAY
Ca          FFT(IX1,IY) = FFT(IX1,IY)*(MASK(IX,(idim2-IY))/1000.0)
Ca          FFT(IX2,IY) = FFT(IX2,IY)*(MASK(IX,(idim2-IY))/1000.0)
Ca          print *, ix1,ix2,iy, ((idim1-IX)+2),(idim2-IY)

CA OLDWAY non-flipped FFT
Ca ORIG          thisMaskVal = (MASK((idim1-IX)+2,(idim2-IY)+2)/1000.0)
Ca try 1          thisMaskVal = (MASK(IX,(idim2-IY)+2)/1000.0)
Ca Works for OJ          thisMaskVal = (MASK((idim1-IX)+2,(idim2-IY)+2)/1000.0)
          thisMaskVal = (MASK((IX),(idim2-IY)+2)/1000.0)
          FFT(IX1,IY) = FFT(IX1,IY)*thisMaskVal
          FFT(IX2,IY) = FFT(IX2,IY)*thisMaskVal

        ENDDO
      ENDDO

      RETURN
      END subroutine fft_fltmsk_rob

C  ***************************************************************
C  *                                                             *
C  *         *****     SUBROUTINE FFT_FLTLLMSK     *****         *
C  *                                                             *
C  *    SUB-OPTION "C" OF EMFFT_FILTER SUBPROGRAM                *
C  *                                                             *
C  *    APPLY 2D LAYERLINE MASK TO FFT                           *
C  *                                                             *
C  *    D IS THE PERPENDICULAR DISTANCE FROM THE POSITION X,Y    *
C  *     IN THE FFT TO THE EQUATORIAL LAYERLINE                  *
C  *                                                             *
C  *    NOTE: ONLY WORKS ON HALF-SIZE FFT ARRAYS.                *
C  *                                                             *
C  *    AUGUST 1985  PURDUE UNIVERSITY VAX 11/750, T. BAKER.     *
C  *    LAST MODIFIED  5-SEP-1989 BY T. BAKER.                   *
C  ***************************************************************

      SUBROUTINE FFT_FLTLLMSK_ROB (FFT, IDIM1, IDIM2, angle, 
     1        lowResoFilter, hiResoFilter, space, fall, icut)
C
      IMPLICIT NONE
C
      INTEGER*4 IDIM1, IDIM2, IXTST, I, IX, IY
      REAL*4    Y, D, SPACE, COSANG, TANANG, FALL, WEIGHT
      REAL*4    FFT(2*IDIM1,(IDIM2+1)), FRAC, ANGLE
      integer*4 intYsq,intXsq, icut
      integer*4 lowResoFilter, hiResoFilter
      integer*4 twoIdim1Mi, oppositeIY 

      COSANG = COS(ANGLE)
      TANANG = TAN(ANGLE)
      IXTST  = (IDIM1/2)

      DO IY = 1,(IDIM2/2)+1
C     DO IY = 1,(IDIM2)+1
        Y  = IY-1.0
        IX = 0
        if(iy > (idim2/2))then
           intYsq = (idim2-iy)*(idim2-iy)
        else
           intYsq = iy*iy
        endif
        DO I = 1,2*IDIM1,2                  ! PROCESS THE CURRENT ROW
Ca          We have to offset this for the upper half???
Ca              Do this only once???
           if(iy == ixtst .and. i == 1)then
            ix = 0
           endif
           IF(IX >= IXTST) IX = IX-IDIM1
           intXsq= (ix-256.0)*(ix-256.0)
           intXsq= ix*ix
Ca          Now trap to see if this is out of our resolution limit
           D = (Y-IX*TANANG)*COSANG
           IF(D < 0.0) D = D+1000*SPACE
           FRAC = D/SPACE
           FRAC = FRAC-INT(FRAC)
           IF(FRAC > 0.5) FRAC = 1.0-FRAC
Ca           Check to see if inside filtering            
           if((intYsq+intXsq) >= hiResoFilter .and. 
     1        (intYsq+intXsq) <= lowResoFilter ) then
            IF(ICUT.NE.1) THEN
               if((frac*space) <= fall) then
                  weight = 1.0             ! HARD CUTOFF
               else
                  weight = 0.0
               endif
            else
               WEIGHT      = EXP(-0.5*((FRAC*SPACE)/FALL)**2.0)
            endif
           else
            weight = 0.0
           endif

           if(weight >= 1.0) then
            weight = 1.0
           endif
C     Trap for ridiculously low values           
           if(weight <= 0.000000001) then
              weight = 0.0
           endif
           
           FFT(I,IY)   = FFT(I,IY)  *WEIGHT
           FFT(I+1,IY) = FFT(I+1,IY)*WEIGHT
           if(iy > 1) then
C     Now stuff the friedel mate in the FFT
              twoIdim1Mi = (2*idim1) -   i ! Current Row
              oppositeIY = (idim2 - IY)  +2 ! A row at a time
              FFT(twoIdim1Mi,oppositeIY)   = 
     1             FFT(twoIdim1Mi,oppositeIY)  *WEIGHT
              FFT(twoIdim1Mi+1,oppositeIY) = 
     1             FFT(twoIdim1Mi+1,oppositeIY)*WEIGHT
           endif
           
           IX          = IX+1
           
        ENDDO
      ENDDO

      RETURN
      END

C  ***************************************************************************
C  *                                                                         *
C  *               *****     SUBROUTINE MAPDSP_GPLOT_ROB *****               *
C  *                                                                         *
C  *    THIS INTERPOLATES AN NCOL X NROW GRID ONTO AN NPIXX X NPIXY GRID.    *
C  *     AND PLOTS TO THE LEXIDATA.                                          *
C  *                                                                         *
C  *    MODIFIED 19-OCT-1990 BY T. BAKER.                                    *
c  *    Converted to X-Windows 09-NOV-93 by David Belnap and Eric Eltzroth   *
C  ***************************************************************************

      SUBROUTINE MAPDSP_GPLOT_ROB (MAP, IMG, SCALE, NCOL, NROW, NSEC, 
     1    NPIXX, NPIXY, XSTRT, YSTRT, NY, NCELLX, NCELLY, SECTION,
     2    NEW_MIN, angle, aoverbPass, plotX, plotY, adim, bdim,
     3    gammaXtal)

      IMPLICIT NONE

      INTEGER*4 NPIX, NX, NYI2, NX1, NX2
      INTEGER*4 NCOL, NROW, NSEC, NPIXX, NPIXY, NY
      INTEGER*4 NCELLX, NCELLY, I, J, IX, IY, IX10, INDY, INDY1, irwa
      INTEGER*4 K, IX1, IX2, INDX1, INDX2, INDX11, INDX22
      INTEGER*4 NXMAX, JJ, ISHFT, intYstrt, intXstrt
      INTEGER*4 SECTION, plotX, plotY
      REAL*4    SCALE, COSANG, SINANG, XSHFT, XSTEP, YSTEP, TMP
      REAL*4    DY, DX, DX1, DX2, X1, X2, P11, P12, P21, P22, Y
      REAL*4    MAP(NCOL,NROW,NSEC), TMP2
      REAL*4    NEW_MIN, XSTRT, YSTRT, angle, aoverbPass
      real*4    aoverb, map_abang
      INTEGER*2 ITMP(5120), IBUF(5120), IMG(plotX,plotY)
      integer*4 window_w,window_h,adim,bdim
      real*4    gammaXtal,cosGamXtal,sinGamXtal
      LOGICAL   FILLED

      map_abang = angle
      aoverb = aoverbPass
      NYI2  = NY                  ! NEED INTEGER*2 VALUE FOR LX_LIM
      COSANG = COS(ANGLE)
      SINANG = SIN(ANGLE)

      cosGamXtal = COS(gammaXtal)
      sinGamXtal = SIN(gammaXtal)

      TMP   = COSANG*real(NY)

C     Kludge this for now - width and height of display window 
      window_w= plotX
      window_h= plotY

      NXMAX = WINDOW_W-NPIXX
      IX10  = 10*NCOL
      XSTEP = 1.0/SCALE
      YSTEP = REAL(NROW)/REAL(NPIXY)
      Y     = -YSTEP

Ca      We've got to get this part of the code working - kludge it
Ca      for now.  (e.g. Fast plot for 90 degrees)
Ca      IF(COSANG == 0.0) GO TO 200            ! TEST FOR ORTHOGONAL PLOT
      if(cosang .ne. cosang) go to 200
      if(cosang == 0.0) cosang= 0.0000001

      DO I = 1,NPIXY
         Y     = Y+YSTEP
         IY    = Y
         DY    = Y-IY
         INDY  = IY+1
         INDY1 = 1+MOD(INDY,NROW)
         XSHFT = COSANG*Y*SCALE/AOVERB
         IX    = NINT(XSHFT)
         DX1   = IX*XSTEP-IY*COSANG/AOVERB
         DX2   = DX1-COSANG/AOVERB
         X1    = -XSTEP+DX1
         X2    = -XSTEP+DX2

         DO J  = 1,NPIXX
            X1      = X1+XSTEP
            X2      = X2+XSTEP
            IX1     = X1+IX10
            IX2     = X2+IX10
            INDX1   = 1+MOD(IX1,NCOL)
            INDX2   = 1+MOD(IX2,NCOL)
            INDX11  = 1+MOD(INDX1,NCOL)
            INDX22  = 1+MOD(INDX2,NCOL)
            P11     = MAP(INDX1,INDY,SECTION)
            P12     = MAP(INDX2,INDY1,SECTION)
            P21     = MAP(INDX11,INDY,SECTION)
            P22     = MAP(INDX22,INDY1,SECTION)
            IX1     = IX1-IX10
            IX2     = IX2-IX10
            DX1     = X1-IX1
            DX2     = X2-IX2
            TMP2    = P11+DX1*(P21-P11+(DY*(P11-P21)))+
     1           DY*(P12-P11+DX2*(P22-P12))
            TMP2    = (TMP2-NEW_MIN)
            ITMP(J) = NINT(TMP2)
         ENDDO


C     Now fill the outBuffer with the image data
         FILLED = .FALSE.       ! SIGNIFY THAT IBUF IS UNFILLED
C     Try and see if this X direction thing is being rounded properly
C     NX     = NINT(XSTRT + REAL(IX))
         intXstrt  = nint(xstrt) + ix
         nx        = intXstrt
         
         DO J = 1,NCELLX

          NX = NX + adim 

          intYstrt = nint(ystrt) + I
          NY = intYstrt

          X1 = real(NX) - TMP

          DO JJ = 1,NCELLY

            IF(.NOT.FILLED) THEN
               do k=1,npixx
                  IBUF(K) = ITMP(K)
               enddo
              FILLED = .TRUE.                    ! SIGNIFY THAT IBUF IS FILLED
            ENDIF
            X1 = X1 + TMP

            NY = NY + ((bdim * sinang) * sinGamXtal)
Ca            /* Now account for angle for X also */
            if(jj > 1) then
             X1 = X1 + (adim * cosGamXtal)
            endif

            IF((NY >= 0).AND.(NY < WINDOW_H)) THEN  ! TEST NY WITHIN BOUNDS
              NX1 = NINT(X1)
                IF(X1 < 0.0) NX1 = X1-0.5
C               ! TEST FOR "X" WITHIN BOUNDARY
              IF(((NX1+NPIXX) >= 0) .and.((NX1-NPIXX) <= NXMAX)) THEN 
                  NPIX = NPIXX
                  IF(NX1 < 0) THEN
                    ISHFT = ABS(NX1)
                    NPIX = NPIXX+NX1
                    NX1  = 0
                    do k=1,npix
                       IBUF(K) = ITMP(K+ISHFT)
                    enddo
                    FILLED = .FALSE.
                    GO TO 100
                  ENDIF
                  IF(NX1 > NXMAX) NPIX = WINDOW_W-NX1
  100                  NX2  = NX1 - 1 + NPIX
                NYI2 = NY

Ca                  Now stuff our final buffer with the data
Ca                  call lx_lim(nx1,nyi2,nx2,nyi2)
Ca                  call lx_put(ibuf,npix)

                do irwa=1,npix
                  if(((irwa+nx1) >= 1).and.((nx1+irwa) <= plotX)) then
                    if((nyi2 >= 1).and.(nyi2 <= plotY)) then
                      img(irwa+nx1,nyi2) = ibuf(irwa)
                    endif
                  endif
                enddo

              ENDIF    ! TEST FOR "X" WITHIN BOUNDARY
            ENDIF  ! If Y is in bounds (window_h) ! TEST NY WITHIN BOUNDS
            ENDDO
        ENDDO
      ENDDO      !  DO I = 1,NPIXY  -  MAIN LOOP IN Y DIRECTION

      RETURN

C     FAST ORTHOGONAL PLOT
 200  DO J = 1,NPIXY
         Y     = Y+YSTEP
         IY    = Y
         DY    = Y-IY
         INDY  = IY+1
         INDY1 = 1+MOD(INDY,NROW)
         X1    = -XSTEP
         DO I = 1,NPIXX
            X1       = X1+XSTEP
            IX1      = X1+IX10
            INDX1    = 1+MOD(IX1,NCOL)
            INDX2    = 1+MOD(INDX1,NCOL)
            P11      = MAP(INDX1,INDY,SECTION)
            P12      = MAP(INDX1,INDY1,SECTION)
            P21      = MAP(INDX2,INDY,SECTION)
            P22      = MAP(INDX2,INDY1,SECTION)
            IX1      = IX1-IX10
            DX       = X1-IX1
            TMP2     = P11+DX*(P21-P11+(DY*(P11-P21)))+
     1           DY*(P12-P11+DX*(P22-P12))
            TMP2     = (TMP2-NEW_MIN)
            IMG(I,J) = nINT(TMP2)
         ENDDO
      ENDDO
      
      NX = NINT(XSTRT)
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *                 *****     SUBROUTINE IMG_LINGRAD_ROB *****               *
C  *                                                                          *
C  *      REMOVE LINEAR GRADIENT FROM IMG(NCOL,NROW) IMAGE ARRAY.             *
C  *                                                                          *
C  *    THIS SUBROUTINE WAS ORIGINALLY PART OF MDPP AND WAS WRITTEN (I        *
C  *     PRESUME) BY ROSS SMITH.  IT WAS REWRITTEN AROUND 1980 AT NIH FOR     *
C  *     PDP 11S, AND SUBSEQUENTLY ADAPTED TO THE VAX (WITH VERY LITTLE       *
C  *     ADDITIONAL MODIFICATION). -BENES TRUS                                *
C  *                                                                          *
C  *      NEW ADDITION: POINTS OUTSIDE OF A CIRCLE IN THE IMAGE MAY BE        *
C  *                  EXCLUDED.                                               *
C  *                                                                          *
C  *      IRAD    = RADIUS OF DATA TO INCLUDE IN GRADIENT CALCULATION.        *
C  *            = 0 IMPLIES CALCULATING A LINEAR GRADIENT OVER THE WHOLE      *
C  *                IMAGE. USES BIMG_CURRENT (CURRENT IMAGE IN PACK FILE      *
C  *                BEING WORKED ON).                                         *
C  *                                                                          *
C  *    16-JANUARY 1990  NIH, BENES TRUS.                                     *
C  *      LAST MODIFIED: 20th June, 1990  BY NRD@BRAGG.BIO.PURDUE.EDU         *
C  *                                                                          *
C  *    March 1998 RWA - I've had to multiply the IMG() pixel values by       *
C  *    100.0 to increase the dynamic range so that the TRUS*() rtns          *
C  *    work properly.  Trouble is we're passing in BYTE info in this         *
C  *    short buffer and only using 0-255 when TRUS expects 32000 short       *
C  *    values.  Changed it to pass in the expFac (expanding Factor).         *
C  *    Seems with packed boxes we can't use this large of an expanding       *
C  *    factor (100.0).  Let's try 10.0 * numCols/500.0 so a 5000x5000        *
C  *    scanned image woudl have our 100.0 factor and a 50x50 packed box      *
C  *    would have an expFac==1.0.                                            *
C  *                                                                          *
C  ****************************************************************************

      SUBROUTINE IMG_LINGRAD_ROB (IMG, NCOL, NROW, IRAD, imgBkg, expFac)

      IMPLICIT NONE 

      INTEGER*4    IRAD, NCOL, NROW, I, J, IC, JC
      INTEGER*4    NFIT, NNFIT, ISOL, IDSOL
      INTEGER*4    INUM, KWA(20)
      REAL*4       DIST, TMP, STD, AVG, STD2
      REAL*4       VAR, SUMSQ, TOTPTS, BACK, YSQ, RSQ1, RSQ2
      real*4       expFac
      real*4       new_bkg, imgBkg
      real*4       IMG(NCOL, NROW)
      DOUBLE PRECISION    Q(20,20),G(20),DET,T(20,21),XPIX

      RSQ1 = (REAL(IRAD)-1.0)**2
      RSQ2 = (REAL(IRAD))**2
      INUM=0
      NFIT=3
      NNFIT=(NFIT+1)
      
      DO I=1,NFIT
         G(I)=0.D0
         DO J=1,NFIT
            Q(I,J)=0.D0
         ENDDO
      ENDDO
      JC=NROW/2+1
      IC=NCOL/2+1

      DO J=1,NROW
        YSQ = (REAL(J-JC))**2
        DO I=1,NCOL
Ca  Let's expand this by expFac for more dynamic range
         XPIX= (IMG(I, J) * expFac)
         IF(IRAD == 0) THEN
            GOTO 27
          ELSE
            DIST = (REAL(I-IC))**2 + YSQ 
            IF(DIST > RSQ2) GOTO 7
         ENDIF
27        CONTINUE
        CALL TRUS_PLYFIT (I, J, XPIX)
        INUM=INUM+1
7       ENDDO
      ENDDO

      DO I=1,NFIT
         T(I,NNFIT)=G(I)
         DO J=1,NFIT
            T(I,J)=Q(I,J)
         ENDDO
      ENDDO
C
      CALL TRUS_MATDBL (ISOL, IDSOL, NFIT, -NNFIT, T, 20, KWA, DET)
C  ****************************************************************************
C  *    USE PLYOUT SUBROUTINE TO MODIFY ALL PIXELS OF IMG ARRAY (INSIDE RSQ2).*
C  *                                                                          *
C  *    IRAD<>0 : REMOVE GRADIENT FROM DATA INSIDE RSQ2, CALCULATE NEW_BKG.   *
C  ****************************************************************************
      BACK   = 0.0
      TOTPTS = 0.0
      SUMSQ  = 0.0
      avg    = 0.0
      IF (IRAD.NE.0) THEN
        DO J=1,NROW
          YSQ = (REAL(J-JC))**2
          DO I=1,NCOL
            DIST = (REAL(I-IC))**2 + YSQ 
            IF(DIST > RSQ2) GOTO 40
Ca  Let's expand this by expFac for more dynamic range
            XPIX = (IMG(I,J) * expFac)
            CALL TRUS_PLYOUT (I, J, XPIX)
            IMG(I,J)= XPIX            ! IMG NOW FLOATED AROUND 0     
            IF (DIST >= RSQ1) THEN
            TMP    = IMG(I,J)            ! NEW_BKG WILL BE NEAR 0
              BACK   = BACK+TMP
              TOTPTS = TOTPTS+1.0
              SUMSQ  = SUMSQ+TMP*TMP
            ENDIF      
40          ENDDO
        ENDDO
C
        STD     = 0.0
        if(totpts > 0) then
          AVG     = BACK/TOTPTS
          VAR     = (SUMSQ/TOTPTS) - (AVG*AVG)
          IF(VAR >= 0.0) STD = SQRT(VAR)
        endif
C  *****************************************************************
C  *    THROW OUT DATA POINTS OUTSIDE OF 2*STD FROM CALCULATION    *
C  *****************************************************************
         BACK   = 0.0
         TOTPTS = 0.0
         SUMSQ  = 0.0
        IF (STD > 0.0) THEN                                      
          STD2 = STD*2
          DO J = 1,NROW
            YSQ = (REAL(J-JC))**2
            DO I = 1,NCOL
              DIST = (REAL(I-IC))**2 + YSQ 
              IF (DIST >= RSQ1 .AND. DIST <= RSQ2) THEN
                TMP    = IMG(I,J)
                IF(ABS(TMP-AVG) < STD2) THEN     
                  BACK   = BACK+TMP
                      TOTPTS = TOTPTS+1.0
                    SUMSQ  = SUMSQ+TMP*TMP
                  ENDIF
                ENDIF
            ENDDO
          ENDDO
          STD = 0.0
          if(totpts > 0) then
            AVG = BACK/TOTPTS
            VAR = (SUMSQ/TOTPTS) - (AVG*AVG)
            IF(VAR >= 0.0) STD = SQRT(VAR)
          endif
        ENDIF ! IF (STD > 0.0) THEN             
          NEW_BKG = nINT(AVG)                  ! new img_bkg for image
C  *********************************************
C  *    REPLACE OLD BACKGROUND WITH NEW_BKG    *
C  *********************************************
        DO J = 1,NROW
          YSQ = (REAL(J-JC))**2
          DO I = 1,NCOL
            DIST = (REAL(I-IC))**2 + YSQ 
            IF(DIST <= RSQ2) THEN
             IMG(I,J) = (IMG(I,J)) + imgBkg - NEW_BKG ! UNFLOAT IMG
            ENDIF
Ca  Let's not clip these here - we'll scale it down in the routine
Ca  that called us ...      Maybe later it will be needed ???
Ca            IMG(I,J) = MIN(IMG(I,J),255)
Ca            IMG(I,J) = MAX(IMG(I,J),0)
          ENDDO
        ENDDO
C
      ELSE 
        DO J=1,NROW
          DO I=1,NCOL
            XPIX = (IMG(I, J) * expFac)
            CALL TRUS_PLYOUT (I, J, XPIX)
            IMG(I,J) = imgBkg + XPIX     
          ENDDO
        ENDDO
      ENDIF  ! IF (irad)             

      RETURN
      END subroutine img_lingrad_rob

C  ****************************************************************************
C  *                                                                          *
C  *                     *****     EMROTFLT.FOR     *****                     *
C  *                                                                          *
C  *    PROGRAM FOR COMPUTING ROTATIONALLY SYMMETRIZED AVERAGES.              *
C  *                                                                          *
C  *    WITH THE COMPUTED FFT OF AN IMAGE, THE PROGRAM LOCATES THE BEST       *
C  *    POSITION FOR THE POSTULATED ROTATIONAL SYMMETRY AXIS AND THEN         *
C  *    DECOMPOSES THE TRANSFORM INTO CYLINDRICAL HARMONICS ABOUT THIS        *
C  *    ORIGIN.  THOSE HARMONICS CONSISTENT WITH THE POSTULATED SYMMETRY OF   *
C  *    THE OBJECT ARE THEN COMBINED TO PRODUCE A ROTATIONALLY FILTERED       *
C  *    IMAGE.                                                                *
C  *                                                                          *
C  *    OPERATES ON 128x128 UP TO 1024x1024 FFT'S                             *
C  *     NANNULI (511):  NUMBER OF TRANSFORM ANNULI (ALSO THE MAXIMUM RADIUS  *
C  *                      OF THE TRANSFORM ACCESSIBLE)                        *
C  *     RHO (2*NLR-1,2*NLR-1): SIZE OF ROTATIONALLY AVERAGED MAP.            *
C  *     BSA (NANNULI*NLR): NUMBER OF BESSEL ARGUMENTS                        *
C  *                                                                          *
C  *    PARAMETERS AND ARRAYS USED:                                           *
C  *      NANNULI     = NUMBER OF ANNULI IN THE TRANSFORM                     *
C  *      NLR-1       = NUMBER OF RADIAL STEPS IN THE AVERAGE.                *
C  *      MAX_BO      = MAXIMUM NUMBER OF BESSEL ORDERS                       *
C  *      NANNULI*NLR = MAXIMUM NUMBER OF BESSEL ARGUMENTS                    *
C  *      ALG(NLR,MAX_BO), BLG(NLR,MAX_BO), RHO(2*NLR-1,2*NLR-1)              *
C  *                                                                          *
C  *    ORIGINAL MODIFICATION OF MRC(CAMBRIDGE) PROGRAM "ROTFLT" FOR USE ON   *
C  *     THE BRANDEIS PDP 11/40 COMPUTER (SEE ROTFLT.PDP)                     *
C  *    OCTOBER 1982  BRANDEIS UNIVERSITY VAX 11/780, T. BAKER.               *
C  *    DECEMBER 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER.                *
C  *    AUGUST 1991  PURDUE UNIVERSITY VAX 8550, T. BAKER.                    *
C  *    LAST MODIFIED 12-AUG-1991 BY T. BAKER.                                *
C  ****************************************************************************

      subroutine EMROTFLT_MAIN(pow_addr,ELIPX,ELIPY,IRAD,IDIM1,PHI0,
     1 ASCAL,BSCAL,TWODD,JZERO,radiusM1,MAX_BO,FFT_STEP,NANNULI,DEL_BR,
     2 rho_addr,DEL_LR,IOPT,LG_START,LG_END,XORIG,YORIG,POW_SCALE,
     3 nrotPass,ncol,nrow,sump1,sump2,sump3, fft_origxP,fft_origyP,
     4 winScale,sigma, fft, newStart, sigMax, sigMaxTot)

      IMPLICIT NONE

      INTEGER*4 NLR, MAX_BO, JZERO, IMX, IMY, NCOL, NROW
      COMMON /COMORG/ DXMIN, DYMIN, DELDX, DELDY, IMX, IMY
      COMMON /COMROT/ NROT, NMAX
      INTEGER*4 IDIM1, IDIM2, IRAD, NANNULI, NROT, NMAX, IOPT(4)
      integer*4 array_size, nrotpass
      integer*4 MAXBES, newStart
      INTEGER*4 LUN_IN, LG_START, LG_END, radiusM1
      REAL*4    FFT(2*IDIM1,IDIM1), sigMax, sigMaxTot
      REAL*4    DXMIN, DYMIN, DELDX, DELDY, DEL_BR, DEL_LR, FFT_STEP
      REAL*4    PHI0, TWODD, ASCAL, BSCAL, XORIG, YORIG, POW_SCALE
      REAL*4    ELIPX, ELIPY, sump1,sump2,sump3,fft_origxP, fft_origyP
      real*4    pow_addr(1000), winScale
      real*4    sigma(17,17)
      DATA      LUN_IN/1/
      real*4    rho_addr(2*ncol*nrow)
      real*4    fft_origx, fft_origy
      logical fft_full

      real*4, allocatable :: AF_ADDR(:), BF_ADDR(:)
      real*4, allocatable :: AT_ADDR(:), BT_ADDR(:)
      real*4, allocatable :: ALG_ADDR(:), BLG_ADDR(:)
      real*4, allocatable :: ABG_ADDR(:), BBG_ADDR(:)
      real*4, allocatable :: BSA_ADDR(:)
      integer*4, allocatable :: MAXB_ANNULUS_ADDR(:)
      integer alloc_stat

C     Let's init the global with what we've passed in.
      nrot = nrotPass
      fft_origx = xorig
      fft_origy = yorig

      ARRAY_SIZE = NCOL*NROW
      idim2 = idim1   ! We only pass one in here 

C     FFT Already done in robem
      FFT_FULL = .TRUE.
      
      CALL ROTFLT_INFO_ROB (ELIPX,ELIPY,IRAD,IDIM1,PHI0,ASCAL,BSCAL,
     1     TWODD,JZERO,radiusM1,MAX_BO,FFT_STEP,NANNULI,DEL_BR,DEL_LR,
     2     IOPT,LG_START,XORIG,YORIG,
     3     fft_origx, fft_origy, POW_SCALE,nlr)

      allocate(maxb_annulus_addr(nannuli), stat=alloc_stat)

      CALL ROTFLT_MAXBES_ROB (NLR, DEL_BR, DEL_LR, NANNULI,
     1     MAXBES, MAXB_ANNULUS_ADDR)

C  *******************************************************
C  *    READ IN THE APPROPRIATE PART OF THE TRANSFORM    *
C  *******************************************************
      
      allocate(at_addr((2*IRAD+1)*(IRAD+2)),
     $     bt_addr((2*IRAD+1)*(IRAD+2)), stat=alloc_stat)

      CALL FFT_TO_ATBT(FFT, IDIM1, IDIM2, AT_ADDR,
     1 BT_ADDR, IRAD, fft_origx, fft_origy, fft_full)

C  ****************************************
C  *    OPTION TO REFINE SYMMETRY AXIS    *
C  ****************************************
      IF(IOPT(1) == 1) THEN            ! OPTION TO REFINE SYMMETRY AXIS

         CALL ROTFLT_ORIGIN_ROB (ELIPX,ELIPY,AT_ADDR,BT_ADDR,
     1   IRAD,IDIM1,FFT_STEP,MAXB_ANNULUS_ADDR,NANNULI,
     2   winScale,sigma, newStart, sigMax, sigMaxTot)
         FFT_ORIGX = FFT_ORIGX + DXMIN + (IMX-1)*DELDX
         FFT_ORIGY = FFT_ORIGY + DYMIN + (IMY-1)*DELDY
         FFT_ORIGX = DXMIN + (IMX-1)*DELDX
         FFT_ORIGY = DYMIN + (IMY-1)*DELDY
       ELSE
         FFT_ORIGX = XORIG
         FFT_ORIGY = YORIG
      ENDIF

      IF(IOPT(2) == 1) CALL TSBEND            ! OPTION TO COMPUTE BIG G'S

C     COMPUTE BIG G'S FROM TRANSFORM AND PRINT POWER SPECTRUM
      allocate(abg_addr(nannuli*maxbes), bbg_addr(nannuli*maxbes),
     $     af_addr(maxbes), bf_addr(maxbes), stat=alloc_stat)

      CALL ROTFLT_BG_ROB (ELIPX, ELIPY, AT_ADDR, BT_ADDR,
     1 ABG_ADDR, BBG_ADDR, IRAD, IDIM1, PHI0,
     2 ASCAL, BSCAL, POW_ADDR, AF_ADDR, BF_ADDR,
     3 FFT_STEP, MAXB_ANNULUS_ADDR, NANNULI, DEL_BR, MAXBES,
     3 fft_origx, fft_origy)

      deallocate(maxb_annulus_addr)
      deallocate(af_addr)
      deallocate(bf_addr)
      if(allocated(at_addr)) deallocate(at_addr)
      if(allocated(bt_addr)) deallocate(bt_addr)

      CALL ROTFLT_POWSPEC_ROB (MAXBES, POW_ADDR, IOPT, POW_SCALE,
     1   sump1,sump2,sump3)

      IF(IOPT(4) == 1) CALL TSBEND      ! OPTION TO COMPUTE ROTATIONAL AVERAGE

      allocate(alg_addr(nlr*max_bo), blg_addr(nlr*max_bo),
     $     bsa_addr(nannuli*nlr), stat=alloc_stat)

      CALL ROTFLT_LG_ROB (ALG_ADDR, BLG_ADDR, ABG_ADDR,
     1 BBG_ADDR, BSA_ADDR, TWODD, NLR, MAX_BO, NANNULI,
     2 DEL_BR, MAXBES)

      deallocate(abg_addr, bbg_addr, bsa_addr)

C     COMPUTE A ROTATIONALLY SYMMETRIZED AVERAGE
      ARRAY_SIZE = 4*(2*NLR-1)**2

      CALL ROTFLT_FB_ROB (ALG_ADDR, BLG_ADDR,RHO_ADDR,
     1 JZERO, NLR, MAX_BO, LG_START, LG_END)

      deallocate(alg_addr, blg_addr)

C     Keep the C code happy
      fft_origxP = fft_origx
      fft_origyP = fft_origy
      radiusM1 = nlr

      return
      END

C  ****************************************************************************
C  *                                                                          *
C  *                *****     SUBROUTINE ROTFLT_INFO_ROB *****                *
C  *                                                                          *
C  *    INPUT VARIABLES (DEFAULTS):                                           *
C  *     NROT        (2)     = ROTATIONAL SYMMETRY                            *
C  *     NANNULI   (IDIM/4)  = # OF ANNULI USED FROM THE FFT                  *
C  *     FFT_STEP   (1.0)    = SPACING OF ANNULI IN TPU                       *
C  *     RADIUSM1            = # OF RADIAL STEPS IN THE RECONSTRUCTION        *
C  *     DEL_LR    (1.0)     = STEP SIZE IN THE RECONSTRUCTION (PIXELS)       *
C  *     ELIPX,ELIPY (1.,1.) = X,Y ELLIPTICAL DISTORTION SCALE FACTORS        *
C  *     ASCAL,BSCAL(1.0,0.0) = RADIAL WEIGHTING FACTORS WHICH MAY BE USED    *
C  *          TO INCREASE ARTIFICIALLY THE STRENGTHS OF THE HIGH              *
C  *          FREQUENCIES.  USE WITH CAUTION!!  ANNULUS NUMBER IR HAS A       *
C  *          FACTOR (ASCAL + IR*BSCAL) APPLIED.                              *
C  *     PHI0      (0.0) = ROTATION OF RECONSTRUCTION WRT ORIGINAL IMAGE      *
C  *     JZERO           = 0 INCLUDES G-ZERO IN THE RECONSTRUCTION            *
C  *                     = 1 OMITS G-ZERO FROM THE RECONSTRUCTION             *
C  *                                                                          *
C  *    LAST MODIFIED 12-AUG-1991 BY T. BAKER.                                *
C  ****************************************************************************

      SUBROUTINE ROTFLT_INFO_ROB (ELIPX, ELIPY, IRAD, IDIM, PHI0, ASCAL,
     $     BSCAL, TWODD, JZERO, radiusM1, MAX_BO, FFT_STEP, NANNULI,
     $     DEL_BR, DEL_LR, IOPT,LG_START, XORIG, YORIG,
     $     fft_origx, fft_origy, POW_SCALE, nlr)

      IMPLICIT NONE

      COMMON /COMROT/ NROT, NMAX
      INTEGER*4 MAX_BO, NLR, NANNULI, NROT, NMAX, JZERO
      INTEGER*4 IDIM, ITEST, IRAD, NSTEP, RADIUSM1
      INTEGER*4 IOPT(4), N1, N2, NUMCHARI4, ITMP1, LG_START
      REAL*4    PI, RADIAN, DEGREE, PHI0, A1, A2, A3, A4, ELIPX, ELIPY
      REAL*4    TWODD, DEL_BR, FFT_STEP, ASCAL, BSCAL, DEL_LR, TMP
      REAL*4    XORIG, YORIG, POW_SCALE, fft_origx, fft_origy

      CALL PIRADDEG (PI, RADIAN, DEGREE)

C     OBTAIN INPUT PARAMETERS
      IF(FFT_STEP <= 0.0) FFT_STEP = 1.0
      ITEST = nINT(((REAL(IDIM)/4.0)/fft_step)-1.0)
 100  IF(NROT <= 0) NROT = 2
      IF(JZERO.NE.1) JZERO = 0
      PHI0 = AMOD(PHI0,360.0)
      IF(NANNULI <= 0) NANNULI = ITEST
      IF(NANNULI > (ITEST*2)) THEN
         CALL BELL (2)
         PRINT *, NANNULI, ' annuli exceeds allowed limit of', ITEST*2
         nannuli = (itest*2)
         print *, 'Resetting to - ', nannuli, itest
C     call TSBEND
         
      ENDIF
      IRAD = INT(NANNULI*FFT_STEP+0.99)
      IF(IRAD > (fft_step*ITEST*2)) THEN
         nannuli = nannuli - 1
         print *, 'Resetting nannuli to - ', nannuli, itest
         print *, fft_step, irad
        GO TO 100
      ENDIF
      DEL_BR = FFT_STEP/REAL(IDIM)
C  **********************************************************************
C  *    SET UP THE # RADIAL STEPS AND STEPSIZE IN THE RECONSTRUCTION    *
C  *     AND INPUT ASCAL,BSCAL,ELIPX,ELIPY SCALE FACTORS.               *
C  **********************************************************************
      ELIPX = 1.0
      ELIPY = 1.0
      ASCAL = 1.0
      BSCAL = 0.0
      NSTEP = INT(FFT_ORIGX)
      a1 = 0.0
      a2 = 0.0
      a3 = 0.0
      a4 = 0.0
      IF(RADIUSM1 <= 0) RADIUSM1 = NSTEP
      IF(DEL_LR <= 0.0 .OR. DEL_LR > REAL(NSTEP)) THEN
        DEL_LR = REAL(NSTEP)/REAL(RADIUSM1) 
      ENDIF
      TWODD = 2.0*PI*FFT_STEP*DEL_LR/REAL(IDIM)
      IF(A1.NE.0.0 .OR. A2.NE.0.0) THEN
        ASCAL = A1
        BSCAL = A2
      ENDIF
      IF(A3.NE.0.0) ELIPX = A3
      IF(A4.NE.0.0) ELIPY = A4
C  ****************************************************************
C  *    DETERMINE THE MAXIMUM BESSEL ORDER THAT WILL BE NEEDED    *
C  ****************************************************************
      TMP    = 2.0*PI*RADIUSM1*DEL_LR*DEL_BR
      MAX_BO = ((INT(TMP*REAL(NANNULI) + 2.0))/NROT) + 1
      NLR    = RADIUSM1 + 1
      PHI0   = PHI0*RADIAN
C
      IF(IOPT(1).NE.1) THEN                        ! DO NOT REFINE ORIGIN
         IF(XORIG.NE.0.0 .OR. YORIG.NE.0.0) THEN
           XORIG = XORIG - FFT_ORIGX
           YORIG = YORIG - FFT_ORIGY
         ELSE
           XORIG = 0.0
           YORIG = 0.0
        ENDIF
      ENDIF
      IF(IOPT(2).NE.1) THEN                         ! COMPUTE POWER SPECTRUM
        IF(POW_SCALE == 0.0) POW_SCALE = 100.0
      ENDIF
      IF(IOPT(4) == 1) GO TO 900
      LG_START = 0
      IF(JZERO == 1) LG_START = NROT
      ITMP1  = (MAX_BO-1)*NROT
      N1     = NUMCHARI4(LG_START)
      N2     = NUMCHARI4(ITMP1)
      IF(LG_START == 0 .AND. JZERO == 1) LG_START = NROT

  900      RETURN
      END

C  **********************************************************************
C  *                                                                    *
C  *           *****     SUBROUTINE ROTFLT_MAXBES_ROB     *****         *
C  *                                                                    *
C  *    DETERMINE LARGEST BESSEL ORDER ON EACH ANNULUS.                 *
C  *                                                                    *
C  *    AUGUST 1991  PURDUE UNIVERSITY VAX/VMS 8550 T. BAKER.           *
C  **********************************************************************

      SUBROUTINE ROTFLT_MAXBES_ROB (NLR, DEL_BR, DEL_LR, NANNULI, 
     1     MAXBES, MAXB_ANNULUS)

      IMPLICIT NONE

      INTEGER*4 NLR, NANNULI, IR, MAXB_ANNULUS(NANNULI), MAXBES
      REAL*4    DEL_BR, DEL_LR, TMP, PI, RADIAN, DEGREE

      CALL PIRADDEG (PI, RADIAN, DEGREE)

      TMP = 2.0*PI*(NLR-1)*DEL_LR*DEL_BR      ! NLR-1 BECAUSE OF ROTFLT_INFO

      DO IR = 1,NANNULI
         MAXB_ANNULUS(IR) = 1 + INT(TMP*IR + 2.0)
      ENDDO

      MAXBES = MAXB_ANNULUS(NANNULI)

      RETURN
      END

C  **********************************************************************
C  *                                                                    *
C  *           *****     SUBROUTINE ROTFLT_ORIGIN_ROB     *****             *
C  *                                                                    *
C  *    FINDS THE BEST PHASE ORIGIN CORRELATED WITH THE N-FOLD          *
C  *      SYMMETRY.                                                     *
C  *                                                                    *
C  *    INPUT PARAMETERS (DEFAULTS):                                    *
C  *      DXMIN,DXMAX (-2.0,2.0) = RANGE OF SEARCH IN X DIRECTION       *
C  *          RELATIVE TO ORIGX, MEASURED IN DENSITOMETER STEPS.        *
C  *      DELDX (0.25) = INTERVAL OF SEARCH IN X DIRECTION MEASURED     *
C  *          IN DENSITOMETER STEPS.                                    *
C  *      DYMIN,DYMAX,DELDY (-2.0,2.0,0.25) = SIMILAR FOR Y DIRECTION   *
C  *      NB_STEP (5) = # OF TRANSFORM ANNULI TO BE LUMPED INTO A BAND  *
C  *          WHEN PRINTING RESIDUALS FROM SEARCH PROCEDURE.            *
C  *      NB_SKIP (0) = # OF BANDS OF DATA TO OMIT WHEN DETERMINING     *
C  *          THE BEST ORIGIN.                                          *
C  **********************************************************************

      SUBROUTINE ROTFLT_ORIGIN_ROB (ELIPX, ELIPY, AT, BT, IRAD, IDIM, 
     1 STEPBR, MAXB_ANNULUS, NANNULI, winScale, sigma, newStart, 
     1 sigMax, sigMaxTot)

      IMPLICIT NONE
      
      COMMON /COMORG/ DXMIN, DYMIN, DELDX, DELDY, IMX, IMY
      COMMON /COMROT/ NROT, NMAX
      INTEGER*4 IDIM, IRAD, NROT, NANNULI, NMAX, NN, NNSQ, newStart
      PARAMETER (NN=17, NNSQ=NN*NN)
      REAL*4    SIGMA(NN,NN), SIGA(NN,NN), SIGB(NN,NN), SIGR(NN,NN)
      REAL*4    DXMIN, DXMAX, DELDX, DYMIN, DYMAX, DELDY
      REAL*4    PI, RADIAN, DEGREE, ELIPX, ELIPY, ALPHA, STEPBR
      REAL*4    AT((2*IRAD+2)*(IRAD+2)), BT((2*IRAD+2)*(IRAD+2))
      REAL*4    A, B, A1, A2, A3, PIBYX, COSAL, SINAL, R, DELPHI, PHI
      REAL*4    X, Y, X1, Y1, Z1, X2, Y2, XBIT, YBIT, XA, YA, XB, YB
      REAL*4    DX, DY, BETA, CB, SB, XX, YY, B1, SIGMIN, SIGMAX
      real*4    winScale,sigMaxTot
      INTEGER*4 NB_STEP, NB_SKIP, NPHI, IPHI, M1, M2, N, N2, IX, IY
      INTEGER*4 IR, IRLAST, NB_SKIP_LIM, ISIG(NN), NX, NY, IMX, IMY
      INTEGER*4 I, J, IN, IDIAM, NN1, NN2, NUMCHARI4
      INTEGER*4 MAXB_ANNULUS(NANNULI)
      DATA SIGR/NNSQ*0.0/
      
      CALL PIRADDEG (PI, RADIAN, DEGREE)
      IDIAM = 2*IRAD+1
      PIBYX = 2.0*PI/REAL(IDIM)
      ALPHA = 2.0*PI/REAL(NROT)
      COSAL = COS(ALPHA)
      SINAL = SIN(ALPHA)

C     SET UP INPUT PARAMETERS
C     Let's pass into this rtn how to modify the search window
C     We'll use a param called 'winScale'
      a1 = 0.0
      a2 = 0.0
      a3 = 0.0
      DXMIN = -2.0 * winScale
      DXMAX =  2.0 * winScale
      DELDX =  0.25 * winScale
      
      a1 = 0.0
      a2 = 0.0
      a3 = 0.0
      DYMIN = -2.0 * winscale
      DYMAX =  2.0 * winscale
      DELDY =  0.25 * winscale
      IF(A1.NE.0.0.OR.A2.NE.0.0.OR.A3.NE.0.0) THEN
         DYMIN = A1
         DYMAX = A2
         DELDY = A3
      ENDIF
C     100      WRITE(6,1020) NANNULI
C     READ (5,'(I)',ERR=100) NB_STEP
 100  nb_step = 0
      IF(NB_STEP <= 0) NB_STEP = 5
      IF(NB_STEP > NANNULI) THEN
         CALL BELL (2)
         GO TO 100
      ENDIF
      NB_SKIP     = 0
      NB_SKIP_LIM = INT((REAL(NANNULI)/REAL(NB_STEP))+0.9)-1
      IF(NB_SKIP_LIM > 0) THEN
C     WRITE(6,1030) NB_SKIP_LIM
C     READ (5,'(I)') NB_SKIP
         nb_skip = 1
         IF(NB_SKIP < 0 .OR. NB_SKIP > NB_SKIP_LIM) NB_SKIP = 0
      ENDIF
      NX = (DXMAX-DXMIN+0.1)/DELDX+1
      NY = (DYMAX-DYMIN+0.1)/DELDY+1
      IF(NX > NN) NX = NN
      IF(NY > NN) NY = NN
      
C     STEP THROUGH ALL TRANSFORM ANNULI
      do j = 1,nn
         do i = 1,nn
            sigma(i,j) = 0.0
            isig(j) = 0
         enddo
      enddo
      
      IRLAST = (NB_SKIP*NB_STEP + 1)
      DO 300 IR = (NB_SKIP*NB_STEP + 1),NANNULI
         R      = IR*STEPBR
         NPHI   = 2*(MAXB_ANNULUS(IR) - 1)/NROT+2
         DELPHI = ALPHA/NPHI
         PHI    = 0.0
         DO IPHI = 1,NPHI
            DO J = 1,NN
               DO I = 1,NN      ! ZERO "SIGA" AND "SIGB" ARRAYS
                  sigr(i,j) = 0.0
                  SIGA(I,J) = 0.0
                  SIGB(I,J) = 0.0
               ENDDO
            ENDDO
            X = R*COS(PHI)
            Y = R*SIN(PHI)

C           STEP THROUGH N TIMES TO COVER N SECTORS OF THE TRANSFORM
            DO IN = 1,NROT
               X1 = X*ELIPX
               Y1 = Y*ELIPY
               Z1 = 1.0
               IF(Y1 < 0) THEN
                  Y1 = -Y1
                  X1 = -X1
                  Z1 = -1.0
               ENDIF
               X1 = X1+IRAD+1
               XBIT = X1-REAL(INT(X1))
               X2   = 1.0-XBIT
               YBIT = Y1-REAL(INT(Y1))
               Y2   = 1.0-YBIT
               N    = IDIAM*INT(Y1)+INT(X1)
               N2   = N+1
               M1   = N+IDIAM
               M2   = M1+1
C     if(n == 0)n = 1     ! Why do we have to do this???
               A    = X2*Y2*AT(N)+XBIT*Y2*AT(N2)+X2*YBIT*AT(M1)
     1              +XBIT*YBIT*AT(M2)
               B    = Z1*(X2*Y2*BT(N)+XBIT*Y2*BT(N2)+X2*YBIT*BT(M1)
     1              +XBIT*YBIT*BT(M2))
               XA   = PIBYX*X
               YA   = PIBYX*Y

C           STEP THROUGH ORIGIN REFINEMENT WINDOW
               DO IX = 1,NX
                  DX = DXMIN+(IX-1)*DELDX
                  XB = XA*DX
                  DO IY = 1,NY
                     DY   = DYMIN+(IY-1)*DELDY
                     YB   = YA*DY
                     BETA = XB+YB
                     CB   = COS(BETA)
                     SB   = SIN(BETA)
                     A1   =  A*CB+B*SB
                     B1   = -A*SB+B*CB
                     SIGR(IX,IY) = SIGR(IX,IY)+A1*A1+B1*B1
                     SIGA(IX,IY) = SIGA(IX,IY)+A1
                     SIGB(IX,IY) = SIGB(IX,IY)+B1
                     if(ix == 2 .and. iy == 2) then
                     endif
                  ENDDO
               ENDDO
               
C           STEP ROUND INTO NEXT SECTOR
               XX = X
               YY = Y
               X  = XX*COSAL-YY*SINAL
               Y  = XX*SINAL+YY*COSAL
            ENDDO
            DO IX = 1,NX
               DO IY = 1,NY
                  SIGR(IX,IY) = SIGR(IX,IY)-(SIGA(IX,IY)**2
     1                 +SIGB(IX,IY)**2)/NROT
               ENDDO
            ENDDO
            PHI = PHI+DELPHI
         ENDDO
         
         IF(MOD(IR,NB_STEP).NE.0 .AND. IR.NE.NANNULI) GO TO 300
         IF(IR.NE.NANNULI) THEN
            NN1 = NUMCHARI4(IR + 1 - NB_STEP)
            NN2 = NUMCHARI4(IR)
            WRITE(6,1040) (IR + 1 - NB_STEP), IR
         ELSE
            NN1 = NUMCHARI4(IRLAST)
            NN2 = NUMCHARI4(IR)
            WRITE(6,1040) IRLAST, IR
         ENDIF
         IRLAST = IR + 1
C     if(newStart == 1) then
         if(.true.) then
            SIGMAX = SIGR(1,1)
         endif
         DO IY = 1,NY
            DO IX = 1,NX
C     if(newStart == 1) then
               if(.true.) then
                  SIGMAX = AMAX1(SIGMAX,SIGR(IX,IY))
               endif
               IF(IR > NB_SKIP) THEN
                  SIGMA(IX,IY) = SIGMA(IX,IY) + SIGR(IX,IY)
               ENDIF
            ENDDO
         ENDDO
         DO IY = 1,NY
            DO IX = 1,NX
               ISIG(IX) = NINT(SIGR(IX,IY)*100.0/SIGMAX)
            ENDDO
            WRITE(6,'(1X,18I4)') (ISIG(IX),IX=1,NX)
         ENDDO
         DO J = 1,NN            ! ZERO "SIGR" ARRAY
            DO I = 1,NN
               SIGR(I,J) = 0.0
            ENDDO
         ENDDO
 300  CONTINUE

      WRITE(6,'('' ARRAY OF TOTAL RESIDUALS'')')
      SIGMIN = SIGMA(1,1)
      if(newStart == 1) then
         sigMaxTot = SIGMIN
      endif
      DO IY = 1,NY
         DO IX = 1,NX
            if(newStart == 1) then
               sigMaxTot = AMAX1(sigMaxTot,SIGMA(IX,IY))
            endif
            IF(SIGMIN >= SIGMA(IX,IY)) THEN
               SIGMIN = SIGMA(IX,IY)
               IMX    = IX
               IMY    = IY
            ENDIF
         ENDDO
      ENDDO
      DO IY = 1,NY
         DO IX = 1,NX
            ISIG(IX) = nINT(SIGMA(IX,IY)*100.0/sigMaxTot)
         ENDDO
         WRITE(6,'(1X,18I4)') (ISIG(IX),IX=1,NX)
      ENDDO
      WRITE(6,1050) IMX, IMY
      if((imx < nx).and.(imy < ny).and.(imx > 1).and.(imy > 1))then
         winScale = winScale * 0.5 ! Reduce window 
      endif
      
      RETURN
 1040 FORMAT(' RESIDUALS FOR ANNULI ',I8,' TO ',I8)
 1050 FORMAT(' Minimum value at IMX ',I5,' IMY ',I5)

      END subroutine rotflt_origin_rob

C  **************************************************************************
C  *                                                                        *
C  *               *****     SUBROUTINE ROTFLT_BG_ROB *****                 *
C  *                                                                        *
C  *    ROUTINE TO COMPUTE BIG G'S AROUND EACH ANNULUS IN THE TRANSFORM.    *
C  **************************************************************************

      SUBROUTINE ROTFLT_BG_ROB (ELIPX, ELIPY, AT, BT, ABG, BBG,
     $     IRAD,IDIM, PHI0, ASCAL, BSCAL, POW, AF, BF, STEPBR,
     $     MAXB_ANNULUS,NANNULI, DEL_BR, MAXBES, fft_origxP, fft_origyP)

      IMPLICIT NONE

      COMMON /COMROT/ NROT, NMAX
      INTEGER*4 IDIM, IRAD, NROT, NANNULI, NMAX, MAXBES
      REAL*4    AT((2*IRAD+2)*(IRAD+2)), BT((2*IRAD+2)*(IRAD+2))
      real*4    abg(nannuli,maxbes), bbg(nannuli,maxbes), pow(1000)
      REAL*4    PI, RADIAN, DEGREE, DEL_BR, DEL_BRSQ, ASCAL, BSCAL
      REAL*4    AF(MAXBES), BF(MAXBES), ELIPX, ELIPY, PHI0, RSC, DR
      REAL*4    PHAS, CF1, SF1, CPN1, SPN1, CPN0, SPN0, X2, Y2
      REAL*4    XBIT, YBIT, CP0, SP0, CP00, SP00, CPL, SPL, X, Y
      REAL*4    X1, Y1, Z1, R, STEPBR, PHIDEL, PIBY2, XA, YA
      REAL*4    CPHI0, SPHI0, CPH, SPH, A, B, A1, AB, BB
      INTEGER*4 I, J, IDIAM, IR, NPHI, IPHI, N, N2, M1, M2, IN1, IN
      INTEGER*4 MAXB_ANNULUS(NANNULI)
      real*4    fft_origxP, fft_origyP
      real*4    fft_origx, fft_origy
      LOGICAL   NOTIFY

      NOTIFY = .FALSE.

      fft_origx = fft_origxP
      fft_origy = fft_origyP

      CALL PIRADDEG (PI, RADIAN, DEGREE)
      PIBY2    = PI/2.0
      XA       = FFT_ORIGX*2.0*PI/REAL(IDIM)
      YA       = FFT_ORIGY*2.0*PI/REAL(IDIM)
      DEL_BRSQ = DEL_BR*DEL_BR
      CPHI0    = COS(PHI0)
      SPHI0    = SIN(PHI0)
      IDIAM    = 2*IRAD + 1

C     INITIALIZE ARRAYS
      DO J = 1,MAXBES
        POW(J) = 0.0
        DO I = 1,NANNULI
          ABG(I,J) = 0.0
          BBG(I,J) = 0.0
        ENDDO
      ENDDO

C     MAIN LOOP: STEP THROUGH ALL NANNULI ANNULI
      R = STEPBR
      DO IR = 1,NANNULI
         
         NMAX = MAXB_ANNULUS(IR)
         DO IN = 1,NMAX
            AF(IN) = 0.0
            BF(IN) = 0.0
         ENDDO
         
C        SOLVE FOR G'S AROUND THIS ANNULUS
         NPHI   = NMAX*NROT
         PHIDEL = PI/NPHI
         CP0    = CPHI0
         SP0    = SPHI0
         CP00   = 1.0
         SP00   = 0.0
         CPL    = COS(PHIDEL)
         SPL    = SIN(PHIDEL)

         DO IPHI = 1,NPHI
            X  = R*CP0
            Y  = R*SP0
            X1 = X*ELIPX
            Y1 = Y*ELIPY
            Z1 = 1.0
            IF(Y1 < 0) THEN
               Y1 = -Y1
               X1 = -X1
               Z1 = -1.0
            ENDIF
            X1   = X1 + IRAD + 1
            XBIT = X1 - REAL(INT(X1))
            X2   = 1.0 - XBIT
            YBIT = Y1 - REAL(INT(Y1))
            Y2   = 1.0 - YBIT
            N    = IDIAM*INT(Y1) + INT(X1)
            N2   = N + 1
            M1   = N + IDIAM
            M2   = M1 + 1
C     if(n == 0)n = 1  ! Why do we have to do this???
            A    = X2*Y2*AT(N)+XBIT*Y2*AT(N2)+X2*YBIT*AT(M1)
     1           +XBIT*YBIT*AT(M2)
            B    = Z1*(X2*Y2*BT(N)+XBIT*Y2*BT(N2)+X2*YBIT*BT(M1)
     1           +XBIT*YBIT*BT(M2))
            PHAS = XA*X + YA*Y
            CF1  = COS(PHAS)
            SF1  = SIN(PHAS)
            A1   =  A*CF1 + B*SF1
            B    = -A*SF1 + B*CF1
            A    = A1

            AF(1) = AF(1) + A
            CPN1  = 1.0
            SPN1  = 0.0
            
            DO IN = 2,NMAX,2
               IN1    = IN + 1
               CPN0   = CPN1*CP00 - SPN1*SP00
               SPN0   = SPN1*CP00 + CPN1*SP00
               AF(IN) = AF(IN) + B*SPN0
               BF(IN) = BF(IN) + B*CPN0
               IF(IN.NE.NMAX) THEN
                  CPN1    = CPN0*CP00 - SPN0*SP00
                  SPN1    = SPN0*CP00 + CPN0*SP00
                  AF(IN1) = AF(IN1) + A*CPN1
                  BF(IN1) = BF(IN1) - A*SPN1
               ENDIF
            ENDDO
            CPH  = CP0*CPL - SP0*SPL
            SPH  = SP0*CPL + CP0*SPL
            CP0  = CPH
            SP0  = SPH
            CPH  = CP00*CPL - SP00*SPL
            SPH  = SP00*CPL + CP00*SPL
            CP00 = CPH
            SP00 = SPH
         ENDDO

C        APPLY EXP(-I*N*PI/2) FACTOR, SUM POWER SPECTRUM ETC
         RSC = ASCAL + IR*BSCAL
         DR  = IR*DEL_BRSQ
         
         DO IN = 1,NMAX,2
            IN1          = IN + 1
            AB           = AF(IN)/NPHI*RSC
            BB           = BF(IN)/NPHI*RSC
            ABG(IR,IN)   = AB
            BBG(IR,IN)   = BB
            POW(IN)      = POW(IN) + (DR*(AB*AB + BB*BB))
            IF(IN.NE.NMAX) THEN
               AB            = AF(IN1)/NPHI*RSC
               BB            = BF(IN1)/NPHI*RSC
               ABG(IR,IN1)   =  BB
               BBG(IR,IN1)   = -AB
               POW(IN1)      = POW(IN1) + (DR*(AB*AB + BB*BB))
            ENDIF
            RSC = -RSC
         ENDDO
         R = R + STEPBR
      ENDDO
      
      RETURN
      END

C  ***********************************************************
C  *                                                         *
C  *      *****     SUBROUTINE ROTFLT_POWSPEC_ROB     *****  *
C  *                                                         *
C  *    ROUTINE TO LIST THE POWER SPECTRUM AT THE TERMINAL   *
C  ***********************************************************

      SUBROUTINE ROTFLT_POWSPEC_ROB (MAXBES, POW, IOPT, POW_SCALE,
     1       sump1,sump2,sump3)

      IMPLICIT NONE

      COMMON /COMROT/ NROT, NMAX
      INTEGER*4 NROT, MAXBES, NMAX, I, IPOW, NK1, IOPT(4)
      real*4, allocatable :: pow2_addr(:)
      integer   alloc_stat
      REAL*4    POW(MAXBES), SUMP1, SUMP2, SUMP3, PF, POW1, POW_SCALE

      POW1   = 1.0
      
      IF(POW_SCALE < 0.0) THEN
         IPOW = 1
      ELSE
         IPOW = 0
         POW1 = POW(1)/POW_SCALE
      ENDIF
      
      SUMP1 = 0.0
      PF    = 1.0
      DO I = 1,NMAX
         POW(I) = POW(I)/POW1*PF
         PF     = 2.0
         SUMP1  = SUMP1 + POW(I)
      ENDDO

      IF(IOPT(3) == 2 .OR.IOPT(3) == 3) THEN
C     OPEN (UNIT=7, NAME='EMROTFLT.TMP', STATUS='NEW',
C     1       ACCESS='SEQUENTIAL', FORM='FORMATTED')
C     WRITE(7,'(I5,F12.2)') ((I-1),POW(I),I=1,NMAX)
C     CLOSE (UNIT=7, DISP='KEEP')
      ENDIF

      NK1   = NROT+1
      SUMP3 = 0.0
      SUMP2 = POW(1)
      DO I = NK1,NMAX,NROT
         SUMP3 = SUMP3+POW(I)
         SUMP2 = SUMP2+POW(I)
      ENDDO
      SUMP2 = SUMP2/SUMP1*100.0
      SUMP3 = SUMP3/(SUMP1-POW(1))*100.
      IF(IOPT(3) == 1 .OR. IOPT(3) == 3) THEN
         allocate(pow2_addr(maxbes), stat=alloc_stat)
         CALL ROTFLT_POWPLOT_ROB (MAXBES, POW, POW2_ADDR)
         deallocate(pow2_addr)
      ELSE
      ENDIF
      
      RETURN
      END


C  *************************************************************
C  *                                                           *
C  *       *****     SUBROUTINE ROTFLT_POWPLOT_ROB     *****   *
C  *                                                           *
C  *    ROUTINE TO PLOT THE POWER SPECTRUM ON THE GRAPHICS     *
C  *     USING EITHER A LINEAR OR SEMI-LOGARITHMIC SCALE       *
C  *************************************************************

      SUBROUTINE ROTFLT_POWPLOT_ROB (MAXBES, POW, POW2)

      IMPLICIT NONE

      COMMON /COMROT/ NROT, NMAX
      INTEGER*4 NROT, MAXBES, NMAX, MMAX
      integer*4 lxtxt_size,lxtxt_col,lxtxt_x,lxtxt_y
      integer*4 lx_xlim
      INTEGER*4 I, J, IX, IX2, FACTOR
      INTEGER*4 ICOL1, ICOL2, ICOL3, ISFTX, ISFTY, IY, IY1, IY2
      INTEGER*4 ITYPE, YLIM, YTIC_STEP, NLOG, NCHAR
      REAL*4    POW(MAXBES), POW2(MAXBES), SCALEX, SCALEY, FNUM
      logical   lxdev_1280

      lxdev_1280 =  .true.      ! Just fake it out until we port this routine
      IF(LXDEV_1280) THEN
         FACTOR = 1
         YLIM   = 450
      ELSE
         FACTOR = 2
         YLIM   = 400
      ENDIF
      ICOL1 = 1
      ICOL2 = 2
      ICOL3 = 3
      LXTXT_COL  =    1
      LXTXT_SIZE =    1
      SCALEX     = 10.0/REAL(FACTOR)
      SCALEY     =  1.0/REAL(FACTOR)
      MMAX       = NMAX
      WRITE(6,'(''$REGULAR<CR> OR LOGARITHMIC(1) PLOT? : '')')
      READ (5,'(I8)') ITYPE
      IF(ITYPE.NE.1) THEN
         ITYPE     = 0
         NCHAR     = 3
         YTIC_STEP = NINT(SCALEY*REAL(YLIM)/10.0)
         NLOG      = (YLIM/YTIC_STEP)
      ELSE
         NCHAR = 5
         WRITE(6,'(''$NUMBER OF ORDERS OF MAGNITUDE FOR LOG PLOT
     1        <CR>=2: '')')
         READ (5,'(I8)') NLOG
         IF(NLOG <= 0) NLOG = 2
         YTIC_STEP = NINT(REAL(YLIM)/REAL(NLOG))
      ENDIF
 100  DO I = 1,NMAX
         IF(ITYPE == 0) THEN
            POW2(I) = (POW(I)*SCALEY*YLIM)/POW(1)
         ELSE
            POW2(I) = YLIM + YTIC_STEP*LOG10((POW(I)/POW(1)))
            IF(POW2(I) < 0) POW2(I) = 0.0
         ENDIF
      ENDDO
      
C     PLOT POWER SPECTRUM
      ISFTX  = 100
      ISFTY  = 100/REAL(FACTOR)
      
C     ABSCISSA
      IX = SCALEX*MMAX
      DO I = 1,MMAX             ! ABSCISSA TIC MARKS
         IX = SCALEX*(I-1) + ISFTX
      ENDDO
      
C     ORDINATE
      IX = SCALEX*MMAX
      DO I = 1,(NLOG-1)
         IY = ISFTY + (I*YTIC_STEP)
      ENDDO
      
C     ABSCISSA LABELS
      DO I = 1,(MMAX+1),5
         J  = I-1
         IF(J < 10) THEN
            LXTXT_X = SCALEX*(I-1) + ISFTX - 5
         ELSE IF(J < 100) THEN
            LXTXT_X = SCALEX*(I-1) + ISFTX - 10
         ELSE IF(J < 1000) THEN
            LXTXT_X = SCALEX*(I-1) + ISFTX - 15
         ENDIF
         LXTXT_Y = ISFTY - 10
      ENDDO
      
C     ORDINATE LABELS
      LXTXT_X = ISFTX - NCHAR*10*LXTXT_SIZE - 10
      DO I = 1,(NLOG+1)
         IF(ITYPE == 0) THEN
            J = I-1
            FNUM = J*0.1
            LXTXT_Y = ISFTY + J*YTIC_STEP + 5
         ELSE
            J    = 1-I
            FNUM = 10.0**J
            LXTXT_Y = ISFTY + YLIM + J*YTIC_STEP + 5
         ENDIF
      ENDDO
      
C  ***************************************************************
C  *    DRAW PLOT OF POWER SPECTRUM ON SEMI-LOGARITHMIC SCALE    *
C  ***************************************************************
      DO I = 1,MMAX
        IX  = ISFTX + SCALEX*(I-1)
        IX2 = ISFTX + SCALEX*I
        IY1 = ISFTY + NINT(POW2(I))
        IY2 = ISFTY + NINT(POW2(I+1))
      ENDDO

C     OPTION TO RESCALE PLOT
      READ (5,'(I8)') I
      IF(I.NE.1) RETURN
      WRITE(6,'(''$REGULAR<CR> OR LOGARITHMIC(1) PLOT? : '')')
      READ (5,'(I8)') ITYPE
  200      IF(ITYPE.NE.1) THEN
         ITYPE = 0
         WRITE(6,'(''$SCALEX,SCALEY,NMAX <CR>=1.0,1.0,'',I3,
     1   '': '')') NMAX
         READ (5,'(2F10.4,I8)') SCALEX, SCALEY, MMAX
         SCALEX = SCALEX/REAL(FACTOR)
         SCALEY = SCALEY/REAL(FACTOR)
       ELSE
         NCHAR = 5
         WRITE(6,'(''$NUMBER OF ORDERS OF MAGNITUDE FOR LOG PLOT
     1 <CR>=2: '')')
         READ (5,'(I8)') NLOG
         IF(NLOG <= 0) NLOG = 2
         YTIC_STEP = NINT(REAL(YLIM)/REAL(NLOG))
         WRITE(6,'(''$SCALEX,NMAX <CR>=1.0,'',I3,'': '')') NMAX
         READ (5,'(F10.4,I8)') SCALEX, MMAX
         SCALEX = SCALEX/REAL(FACTOR)
      ENDIF
      IF(SCALEX <= 0.0) THEN
        IF(SCALEX < 0.0) THEN
          CALL BELL (2)
        ENDIF
        SCALEX = 1.0
      ENDIF
      IF(SCALEY <= 0.0) THEN
        IF(SCALEY < 0.0) THEN
          CALL BELL (2)
        ENDIF
        SCALEY = 1.0
      ENDIF
      IF(MMAX <= 0.OR.MMAX > NMAX) MMAX = NMAX
      SCALEX = SCALEX*10.0
      lx_xlim= 99999  ! Just fake it out until we port this routine
      IF((SCALEX*MMAX+ISFTX) > (LX_XLIM-2)) THEN
        CALL BELL (2)
        GO TO 200
      ENDIF
      IF(ITYPE.NE.1) THEN
         NCHAR     = 3
         YTIC_STEP = NINT(SCALEY*REAL(YLIM)/10.0)
         NLOG      = YLIM/YTIC_STEP
      ENDIF
      GO TO 100

      END

C  **********************************************************************
C  *                                                                    *
C  *              *****     SUBROUTINE ROTFLT_LG_ROB     *****              *
C  *                                                                    *
C  *    COMPUTE LITTLE G'S FROM BIG G'S.                                *
C  **********************************************************************
C
      SUBROUTINE ROTFLT_LG_ROB (ALG, BLG, ABG, BBG, BSA, TWODD, NLR, 
     1                     MAX_BO, NANNULI, DEL_BR, MAXBES)

      IMPLICIT NONE

      COMMON /COMROT/ NROT, NMAX
      INTEGER*4 MAX_BO, LG_NUM, NROT, NANNULI, NLR, NMAX, NBN
      INTEGER*4 I, J, IN, IR, ILR, NN, ILR1, IARG, MAXBES
      REAL*4    PI, RADIAN, DEGREE, TWODD, DD, DEL_BR, BS
      REAL*4    ABG(NANNULI,MAXBES), BBG(NANNULI,MAXBES)
      REAL*4    BSA(NANNULI*NLR), ALG(NLR,MAX_BO), BLG(NLR,MAX_BO)
C      real*4    tmp1,tmp2,tmp3,tmp4,tmp5

      CALL PIRADDEG (PI, RADIAN, DEGREE)

      NBN = NANNULI*NLR               ! # OF BESSEL ARGUMENTS TO CALCULATE
C  ******************************************
C  *    INITIALIZATION OF LITTLE G'S ETC    *
C  ******************************************
      DO J = 1,MAX_BO
         DO I = 1,NLR
            ALG(I,J) = 0.0
            BLG(I,J) = 0.0
         ENDDO
      ENDDO
      DD = 2.0*PI*DEL_BR**2

      LG_NUM = 1
      DO IN = 1,NMAX,NROT
        CALL ROTFLT_FILBES_ROB (LG_NUM, BSA, NBN, TWODD, NROT)
        NN = IN - 1
C  ***************************************************************
C  *    HALVE FINAL ORDINATE FOR TRAPEZOIDAL RULE INTEGRATION    *
C  *    INITIAL ORDINATE AT BIGR=0 ALWAYS ZERO                   *
C  ***************************************************************
        ABG(NANNULI,IN) = 0.5*ABG(NANNULI,IN)
        BBG(NANNULI,IN) = 0.5*BBG(NANNULI,IN)
        DO ILR = 1,NLR
          ILR1 = ILR - 1
          IARG = ILR1 + 1
          DO IR = 1,NANNULI
Ca             I've not resolved this bug to my satisfaction!!!! RWA 10/3/97
Ca             print *, 'iarg= ', iarg, bsa(iarg)
Ca             print *, 'ir= ', ir
Ca             print *, 'dd= ', dd
Ca             print *, 'bs= ', bs
Ca            tmp1            = real(IR) * DD
Ca             print *, 'tmp1= ', tmp1
Ca            tmp2            = BSA(IARG)
Ca             print *, 'tmp2= ', tmp2
Ca            tmp3            = tmp3 * tmp1
Ca             print *, 'tmp3= ', tmp3
Ca            BS              = tmp3
            BS              = BSA(IARG) * IR * DD
Ca             print *, 'iarg22= ', iarg
            IARG            = IARG + ILR1
            ALG(ILR,LG_NUM) = ALG(ILR,LG_NUM) + ABG(IR,IN)*BS
            BLG(ILR,LG_NUM) = BLG(ILR,LG_NUM) + BBG(IR,IN)*BS
          ENDDO
        ENDDO
        LG_NUM  = LG_NUM + 1
      ENDDO

      RETURN
      END

C  **********************************************************************
C  *                                                                    *
C  *          *****      SUBROUTINE ROTFLT_FILBES_ROB *****             *
C  *                                                                    *
C  *    ROUTINE TO GENERATE POSITIVE BESSEL ORDERS FOR ALL REQUIRED     *
C  *     ARGUMENTS, CALLED BY ROTFLT_LG SUBROUTINE                      *
C  *                                                                    *
C  *    MILLER'S ALGORITHM IS USED:                                     *
C  *    UPWARDS RECURRENCE IS USED FOR ARGS. LARGER THAN BSL_ORDER N.   *
C  *    THE RECURRENCE RELATION IS:                                     *
C  *        BSA_(N+1)(ARG) = (2*N/ARG)*BSA_N(ARG) - BSA_(N-1)(ARG)      *
C  *    DOWNWARDS RECURRENCE IS USED FOR ARGS. LESS THAN BSL_ORDER.     *
C  *    IT IS FROM AN EVEN VALUE OF BSL_ORDER MM. THE BESSEL VALUE      *
C  *    IS NOMALIZED BY THE FORMULAR:                                   *
C  *        1 = BSL0(ARG) + 2*BSA_2(ARG) + 2*BSA_4(ARG)                 *
C  *            + 2*BSA_6(ARG) + 2*BSA_8(ARG) + ...                     *  
C  *                                                                    *
C  *    REFERENCE:                                                      *
C  *       W. H. PRESS, B. P. FLANNERY, S. A. TEUKOLSKY AND W. T.       *
C  *       VETTERLING(1989), "NUMERICAL RECIPES: THE ART OF SCIENTIFIC  *
C  *       COMPUTING (FORTRAN VERSION)", PAGE 175-176.                  * 
C  *                                                                    *
C  *   Replaces old rotflt_filbes() that was found to be buggy.         *
C  *   Found by Wei  Dec 1997    RW AShmore                             *
C  *                                                                    *
C  **********************************************************************

      SUBROUTINE ROTFLT_FILBES_ROB (LG_NUM, BSA, NBN, TWODD, NROT)

      IMPLICIT NONE

      INTEGER*4 LG_NUM, NBN, NROT, I, J, JMAX
      REAL*4    TWODD, BSA(NBN), ARG, BFA, BFB, RECIP, X, TEST
      REAL*4    BSL0, BSL1

C     THE FOLLOWING VARABLES ARE USED IN DOWNWARDS RECURRENCE
      INTEGER*4  MM, JJ, JSUM
      REAL*4  IACC, BIGNO, BIGNI
      REAL*4  BESSJ, SUM, BJP, BJ, BJM
      
      IACC = 40.0               ! MAKE IACC LARGER TO INCREASE ACCURACY
C     BIGNO = 1.E10
C     BIGNI = 1.E-10
      BIGNO = 1.E7              ! RWa - had to change these because SUM 
      BIGNI = 1.E-7             ! was overflowing ...

      ARG = 0.0
      IF(LG_NUM == 1) THEN
         DO I = 1,NBN                        ! J-ZERO BESSEL
           BSA(I) = BSL0(ARG)
           ARG    = ARG+TWODD
         ENDDO
       ELSE
         JMAX = ((LG_NUM-1)*NROT)-1

C     OLD WAY:   TEST = 0.825*(REAL(JMAX)-3.9)
C     SET BESSEL FUNCTION VALUE TO ZERO IF ARG IS VERY SMALL. 
C     THE ACCURACY IS INCREASED BY USING 0.6 INSTEAD OF 0.825.

         TEST = 0.6*(REAL(JMAX)-3.9)

         DO I = 1,NBN
           BFA    = BSL0(ARG)
           BSA(I) = BFA
           IF(ARG <= 0.000001) THEN
              BSA(I) = 0.0
            ELSE
              BFB   = BSL1(ARG)
              RECIP = 2.0/ARG

C OLD WAY: IF(ARG > TEST) THEN
C       UPWARDS RECURRENCE FROM BSL0 AND BSL1:

              IF(ARG > JMAX) THEN
                 DO J = 1,JMAX
                   X   = J*BFB*RECIP-BFA
                   BFA = BFB
                   BFB = X
                 ENDDO
                 BSA(I) = X

              ELSE IF (ARG > TEST) THEN

C       DOWNWARDS  RECURRENCE WHEN ARG < BSL_ORDER

                      MM = 2*((JMAX+INT(SQRT(IACC*JMAX)))/2)
C         
                      BESSJ=0.
                      JSUM=0
                      BJP=0.
                      BJ=1.
                
                      DO 300 JJ = MM, 1, -1  
                         BJM = JJ*RECIP*BJ-BJP
                         BJP = BJ
                         BJ = BJM
                   
C       RENORMALIZE TO PREVENT OVERFLOWS        
                         IF(ABS(BJ) > BIGNO) THEN       
                              BJ = BJ*BIGNI
                              BJP = BJP*BIGNI
                              BESSJ = BESSJ*BIGNI
                              SUM = SUM*BIGNI
                         ENDIF
                
                         IF (JSUM.NE.0) SUM = SUM + BJ
                         JSUM = 1 - JSUM
 
C       SAVE THE UNMORMALIZED VALUE             
                      IF (JJ == JMAX) BESSJ = BJP
  300                 CONTINUE
  
                      SUM = 2*SUM - BJ

C       PUT NORMALIZED VALUE INTO BSA ARRAY
                      BSA(I) = BESSJ/SUM
               
            ELSE
                 BSA(I) = 0.0
              ENDIF
           ENDIF
           ARG = ARG+TWODD
         ENDDO
      ENDIF

      RETURN
      END


C  ****************************************************************************
C  *                                                                          *
C  *                 *****     SUBROUTINE ROTFLT_FB_ROB *****                 *
C  *                                                                          *
C  *    FOURIER-BESSEL CALCULATION OF ROTATIONALLY SYMMETRIZED AVERAGE.       *
C  *                                                                          *
C  *    LAST MODIFIED  8-AUG-1991 BY T. BAKER.                                *
C  ****************************************************************************
C
      SUBROUTINE ROTFLT_FB_ROB (ALG, BLG, RHO, JZERO, NLR, MAX_BO,
     1 LG_START, LG_END)
C
      IMPLICIT NONE

      COMMON /COMROT/ NROT, NMAX
      INTEGER*4 MAX_BO, NROT, NLR, NMAX, JZERO, ITST, ILST
      INTEGER*4 NCOL, NROW, ITEST, NUM, IX, IY, RADIUS, IR1, IR2
      INTEGER*4 LG_NUM, LG_START, LG_END
      REAL*4    ALG(NLR,MAX_BO), BLG(NLR,MAX_BO), RHO(2*NLR-1,2*NLR-1)
      REAL*4    X, Y, YSQ, R, RSQ, RBIT, R2, C, C0, C1, S, S0, S1
      REAL*4    PHI, PHIN, ADD1, ADD2, OUTSIDE, TOT, RMAXSQ, AMIN
      REAL*4    AVG, PERCENT
      real*4    map_min, map_max
      LOGICAL   NOTIFY

      OUTSIDE = -123456789.0123
      NOTIFY = .FALSE.

      AMIN = 0.001*ABS(ALG(1,1))
      DO LG_NUM = 1,MAX_BO
         DO RADIUS = 1,NLR
            IF(ABS(ALG(RADIUS,LG_NUM)) < AMIN) ALG(RADIUS,LG_NUM) = 0.0
            IF(ABS(BLG(RADIUS,LG_NUM)) < AMIN) BLG(RADIUS,LG_NUM) = 0.0
         ENDDO
      ENDDO

      NCOL   = 2*NLR - 1
      NROW   = NCOL
      RMAXSQ = (REAL(NLR-1)-0.1)**2
C  ******************************************************
C  *    X,Y,R IN UNITS OF DELSR STEPS, NOT ANGSTROMS    *
C  ******************************************************
      Y = REAL(-NLR)
      IF(NOTIFY) THEN
      ENDIF
      DO IY = 1,NROW
        Y   = Y + 1.0
        YSQ = Y*Y
        X   = REAL(-NLR)
        DO IX = 1,NCOL
          RHO(IX,IY) = OUTSIDE
          X          = X + 1.0
          RSQ        = X*X + YSQ
          IF(RSQ <= RMAXSQ) THEN
            PHI = 0.0
            IF(RSQ.NE.0.0) PHI = ATAN2(Y,X)
            R      = SQRT(RSQ)
            RADIUS = INT(R)
            IR1    = RADIUS + 1
            IR2    = RADIUS + 2
            RBIT   = R - REAL(RADIUS)
            R2     = 1.0 - RBIT
            PHIN   = NROT*PHI
            C1     = COS(PHIN)
            S1     = SIN(PHIN)
            C0     = 1.0
            S0     = 0.0
Ca   Why is this done here????
            IF(JZERO == 0) RHO(IX,IY) = ALG(IR1,1)*R2+ALG(IR2,1)*RBIT
            if(jzero == 1) RHO(IX,IY) = ALG(IR1,LG_NUM)*R2+
     1             ALG(IR2,LG_NUM)*RBIT
            DO LG_NUM = 2,MAX_BO
              ITEST = (LG_NUM-1)*NROT
              IF(ITEST >= LG_START.AND.ITEST <= LG_END) THEN
                C    = C1*C0 - S1*S0
                S    = S1*C0 + C1*S0
                ADD1 = (ALG(IR1,LG_NUM)*R2+ALG(IR2,LG_NUM)*RBIT)*C*2.0
                ADD2 = (BLG(IR1,LG_NUM)*R2+BLG(IR2,LG_NUM)*RBIT)*S*2.0
                IF(ABS(ADD1) < AMIN) ADD1 = 0.0
                IF(ABS(ADD2) < AMIN) ADD2 = 0.0
                RHO(IX,IY) = RHO(IX,IY) + ADD1 - ADD2
                C0         = C
                S0         = S
              ENDIF
            ENDDO
          ENDIF
        ENDDO                                    ! STEP THROUGH COLUMN
        IF(NOTIFY) THEN
          PERCENT = 100.0*REAL(IY)/REAL(NROW)
        ENDIF
      ENDDO                                    ! STEP THROUGH ROW
C  **********************************
C  *    COMPUTE BACKGROUND VALUE    *
C  **********************************
      TOT  = 0.0
      NUM  =   0
      DO IY = 1,NROW
        ITST = -1
        ILST =  0
        DO IX = 1,NCOL
          IF(RHO(IX,IY) == OUTSIDE) THEN            ! OUTSIDE BOUNDARY
             IF(ITST == 1.AND.IX.NE.ILST) THEN      ! LEAVING INSIDE AREA
               NUM  = NUM + 1
               TOT  = TOT + RHO(IX-1,IY)
               ITST = -1
               GO TO 200
             ENDIF
             ITST = -1
           ELSE                              ! INSIDE BOUNDARY
             IF(ITST == -1) THEN                  ! ENTERING INSIDE AREA
               NUM  = NUM + 1
               ILST = IX
               TOT  = TOT + RHO(IX,IY)
             ENDIF
             ITST = 1
          ENDIF
        ENDDO
  200      ENDDO
      AVG = TOT/REAL(NUM)
C  *************************************************************************
C  *    REPLACE VALUES OUTSIDE CIRCULAR BOUNDARY WITH AVERAGE PERIMETER    *
C  *     VALUE AND DETERMINE MIN AND MAX MAP VALUES                        *
C  *************************************************************************

      MAP_MIN = HUGE(MAP_MIN)
      MAP_MAX = -HUGE(MAP_MAX)
      DO IY = 1,NROW
        DO IX = 1,NCOL
          IF(RHO(IX,IY) == OUTSIDE) RHO(IX,IY) = AVG
          MAP_MAX = AMAX1(MAP_MAX,RHO(IX,IY))
          MAP_MIN = AMIN1(MAP_MIN,RHO(IX,IY))
        ENDDO
      ENDDO
C  ******************************************************
C  *    STORE ROTATIONALLY AVERAGED DATA IN MAP FILE    *
C  ******************************************************

      RETURN
      END


C  ******************************************************************
C  *                                                                *
C  *            *****  SUBROUTINE fft_avg_ref  *****                *
C  *                                                                *
C  ******************************************************************

      SUBROUTINE FFT_AVG_REF (REF_IMG_ADDR, REF_FFT_ADDR, 
     1                  REF_BKG, NCOL, NROW, IDIM1, IDIM2)

      IMPLICIT NONE

      INTEGER*4 FFT_NFAC, IDIM1,IDIM2, REF_BKG, NCOL, NROW, img_bkg
      real*4    REF_IMG_ADDR(1), REF_FFT_ADDR(1)

      FFT_NFAC = 1
      CALL FFT_SETDIM_DEF_SAME (NCOL, NROW, IDIM1, IDIM2, FFT_NFAC)
      
      IMG_BKG = REF_BKG
      CALL R4_IMG_FFT_FILL(REF_IMG_ADDR, NCOL, NROW,
     1     REF_FFT_ADDR, IDIM1, IDIM2, img_bkg)
      CALL FFT_2D (REF_FFT_ADDR, IDIM1, IDIM2)
      
      RETURN
      
      END subroutine fft_avg_ref


C *****************************************************************************
C                                                                             *
C      Procedure : PEAK_SORT_HI                                               *
C                                                                             *
C      Object  : The Array PEAKS is sorted by such that the highest peak      *
C                  value is stored in the first array position (i.e. de-      *
C                  creasing  order, using a QUICKSORT algorithm, on a key     *
C                  being peaks(3,*).                                          *
C                                                                             *
C      Parameters :                                                           *
C            PEAKS             REAL*4(3,NPEAK)      Array to be sorted        *
C            NPEAK            INTEGER*4      SIZE of the Array                *
C                                                                             *
C      Authors  :                                                             *
C                 Eric P. Meyer [EPM], Purdue University 18-JUN-87            *
C                                                                             *
C      Release : V01.00      Initial Release                  02-JUL-87 EPM   *
C                 V01.01        Converted to two PEAK_SORTs     18-MAY-88 TSB *
C     Added code to trap for infinite looping past end of array.              *
C     9/24/97 RWA                                                             *
C                                                                             *
C     New Peak Sort Routine using QuickSort Algorithm                         *
C *****************************************************************************

      SUBROUTINE PEAK_SORT_HI_ROB (PEAKS, NPEAK)

      IMPLICIT NONE

      INTEGER*4 NPEAK, MIN_QUICK_SORT_SIZE, STACK_SIZE
      PARAMETER (MIN_QUICK_SORT_SIZE = 9, STACK_SIZE = 1024)
      INTEGER*4 STACK_TOP, LEFT, RIGHT, INDEX_LEFT, INDEX_RIGHT
      INTEGER*4 INDEX, INDEX_2, STACK(STACK_SIZE,2)
      REAL*4    PEAKS(3,NPEAK), KEY, TMP(3), REC(3)
      
      IF (NPEAK <= MIN_QUICK_SORT_SIZE) GO TO 9000
      
      STACK_TOP = 0
      LEFT      = 1
      RIGHT     = NPEAK
      
 2000 INDEX_LEFT  = LEFT
      INDEX_RIGHT = RIGHT + 1
      KEY         = PEAKS(3,LEFT)
      REC(1)      = PEAKS(1,LEFT)
      REC(2)      = PEAKS(2,LEFT)
      REC(3)      = PEAKS(3,LEFT)

 3000 INDEX_LEFT = INDEX_LEFT + 1
C     Give us an out if we've gone too far
      if(index_left > npeak) go to 4000
      IF (PEAKS(3,INDEX_LEFT) > KEY) GO TO 3000
      
 4000 INDEX_RIGHT = INDEX_RIGHT - 1
C     Give us an out if we've gone too far
      if(index_right > npeak) go to 5000
      IF (PEAKS(3,INDEX_RIGHT) < KEY) GO TO 4000
      
 5000 IF (INDEX_RIGHT <= INDEX_LEFT) THEN
         TMP(1)               = PEAKS(1,INDEX_RIGHT)
         TMP(2)               = PEAKS(2,INDEX_RIGHT)
         TMP(3)               = PEAKS(3,INDEX_RIGHT)
         PEAKS(1,INDEX_RIGHT) = PEAKS(1,LEFT)
         PEAKS(2,INDEX_RIGHT) = PEAKS(2,LEFT)
         PEAKS(3,INDEX_RIGHT) = PEAKS(3,LEFT)
         PEAKS(1,LEFT)        = TMP(1)
         PEAKS(2,LEFT)        = TMP(2)
         PEAKS(3,LEFT)        = TMP(3)
         GO TO 7000
      ENDIF
      
      TMP(1)               = PEAKS(1,INDEX_RIGHT)
      TMP(2)               = PEAKS(2,INDEX_RIGHT)
      TMP(3)               = PEAKS(3,INDEX_RIGHT)
      PEAKS(1,INDEX_RIGHT) = PEAKS(1,INDEX_LEFT)
      PEAKS(2,INDEX_RIGHT) = PEAKS(2,INDEX_LEFT)
      PEAKS(3,INDEX_RIGHT) = PEAKS(3,INDEX_LEFT)
      PEAKS(1,INDEX_LEFT)  = TMP(1)
      PEAKS(2,INDEX_LEFT)  = TMP(2)
      PEAKS(3,INDEX_LEFT)  = TMP(3)
      GO TO 3000

 7000 IF (RIGHT-INDEX_RIGHT >= INDEX_RIGHT-LEFT .AND.
     +     INDEX_RIGHT-LEFT >  MIN_QUICK_SORT_SIZE) THEN
         STACK_TOP          = STACK_TOP + 1
         STACK(STACK_TOP,1) = INDEX_RIGHT + 1
         STACK(STACK_TOP,2) = RIGHT
         RIGHT              = INDEX_RIGHT - 1
         GO TO 2000
      ENDIF
      IF (INDEX_RIGHT-LEFT   > RIGHT-INDEX_RIGHT .AND.
     +     RIGHT-INDEX_RIGHT > MIN_QUICK_SORT_SIZE) THEN
         STACK_TOP          = STACK_TOP + 1
         STACK(STACK_TOP,1) = LEFT
         STACK(STACK_TOP,2) = INDEX_RIGHT - 1
         LEFT               = INDEX_RIGHT - 1
         GO TO 2000
      ENDIF
      IF (RIGHT-INDEX_RIGHT    >  min_quick_sort_size .AND.
     +     MIN_QUICK_SORT_SIZE >= INDEX_RIGHT-LEFT) THEN
        LEFT = INDEX_RIGHT + 1
        GO TO 2000
      ENDIF
      IF (INDEX_RIGHT-LEFT     >  MIN_QUICK_SORT_SIZE .AND.
     +     MIN_QUICK_SORT_SIZE >= RIGHT-INDEX_RIGHT) THEN
         RIGHT = INDEX_RIGHT - 1
        GO TO 2000
      ENDIF

      IF (STACK_TOP.NE.0) THEN
         LEFT      = STACK(STACK_TOP,1)
         RIGHT     = STACK(STACK_TOP,2)
         STACK_TOP = STACK_TOP - 1
         GO TO 2000
      ENDIF

 9000 DO INDEX = 2,NPEAK
         IF (PEAKS(3,INDEX - 1) < PEAKS(3,INDEX)) THEN
            KEY     = PEAKS(3,INDEX)
            REC(1)  = PEAKS(1,INDEX)
            REC(2)  = PEAKS(2,INDEX)
            REC(3)  = PEAKS(3,INDEX)
            INDEX_2 = INDEX - 1
 8100       PEAKS(1,INDEX_2+1) = PEAKS(1,INDEX_2)
            PEAKS(2,INDEX_2+1) = PEAKS(2,INDEX_2)
            PEAKS(3,INDEX_2+1) = PEAKS(3,INDEX_2)
            INDEX_2            = INDEX_2 - 1
            IF (INDEX_2 > 0.AND.
     1           (.NOT.(PEAKS(3,INDEX_2) > KEY))) GO TO 8100
            PEAKS(1,INDEX_2+1) = REC(1)
            PEAKS(2,INDEX_2+1) = REC(2)
            PEAKS(3,INDEX_2+1) = REC(3)
         ENDIF
      ENDDO

      RETURN
      END

C  *****************************************************************************
C  *                                                                           *
C  *                    *****     SUBROUTINE PEAK_CHK_ROB ****                 *
C  *                                                                           *
C  *    ROUTINE TO CHECK FOR PEAKS FOUND BY SUBROUTINE PEAK_SEARCH AND TO      *
C  *     FIND THE HIGHEST PEAK WITHIN A CIRCULAR WINDOW OF RADIUS "D"          *
C  *                                                                           *
C  *    NPEAK        = # OF PEAKS FOUND ABOVE A THRESHOLD PIXEL VALUE          *
C  *    NPEAK_UNIQUE = # OF UNIQUE PEAKS LEFT AFTER IMPOSING WINDOW CONSTRAINT *
C  *                                                                           *
C  *    PEAKS WHICH ARE FLAGGED, I.E. THOSE WHICH DO NOT HAVE THE HIGHEST      *
C  *     VALUE WITHIN A GIVEN WINDOW, ARE GIVEN A NEGATIVE PIXEL VALUE.        *
C  *                                                                           *
C  *    APRIL 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER                     *
C  *    LAST MODIFIED  1-SEP-88 BY T. BAKER                                    *
C  *****************************************************************************

      SUBROUTINE PEAK_CHK_ROB (PEAKS, NPEAK, NPEAK_UNIQUE, D)

      IMPLICIT NONE

      INTEGER*4 NPEAK, NPEAK_UNIQUE, I, J
      REAL*4    PEAKS(3,NPEAK), D, DSQ, TEST

      DSQ          = D*D
      NPEAK_UNIQUE = NPEAK

      DO I = 1,(NPEAK-1)
        IF(PEAKS(3,I) >= 0.0) THEN
          DO J = (I+1),NPEAK
            IF(PEAKS(3,J) >= 0.0) THEN
              TEST = (PEAKS(1,I)-PEAKS(1,J))**2 + 
     1              (PEAKS(2,I)-PEAKS(2,J))**2
              IF(TEST <= DSQ) THEN
                PEAKS(3,J)   = -PEAKS(3,J)      ! FLAG FAILED PEAK
                NPEAK_UNIQUE = NPEAK_UNIQUE - 1
              ENDIF
            ENDIF
          ENDDO
        ENDIF
      ENDDO
      
      RETURN
      END

C  **************************************************************************
C  *                                                                        *
C  *                     *****     CONTUR.FOR     *****                     *
C  *                                                                        *
C  *    THIS CONTAINS A SERIES OF SUBROUTINES USED FOR CONTOURING "MAPS"    *
C  *                                                                        *
C  *    THE SUBROUTINE NAMES USED HERE ARE CHANGED FROM THE ORIGINAL NAMES: *
C  *                                                                        *
C  *        NEW NAME   ORIGINAL NAME                                        *
C  *        --------   -------------                                        *
C  *        CONTUR_A      AUX095                                            *
C  *        CONTUR_B      AUX098                                            *
C  *        CONTUR_C      AUX096                                            *
C  *        CONTUR_D      AUX097                                            *
C  *                                                                        *
C  *    THESE SUBROUTINES ARE CURRENTLY USED IN THE PROGRAMS EMMAPDSP,      *
C  *     EMMAPENV AND EMFFTLAT.                                             *
C  **************************************************************************

      SUBROUTINE CONTUR_A_ROB (MAP,IMG,XP,YP,NCOL,NROW,SCLX,SCLY,TOUR,
     1                    ix,iy,ix1,iy1,pltPt,
     2                    cosangp,sinangp,icxp,icyp)

      IMPLICIT NONE

      COMMON    /COM031/ IRP,JRP,ISP,JSP,NNX,NNY
       COMMON    /VPLT/ COSANG, SINANG, ICX, ICY
      INTEGER*4 NCOL, NROW, IRP, JRP, ISP, JSP, NNX, NNY, I, J
      INTEGER*4 ITEMP, JTEMP, II, JJ, CONTUR_B_ROB, PRINT, pltPt
      INTEGER*2 IMG(NCOL+1,NROW+1)
      REAL*4    MAP(NCOL+1,NROW+1), XP(NCOL+1), YP(NROW+1)
      integer*2 ix(40000),iy(40000),ix1(40000),iy1(40000)
      real*4    cosangp,sinangp
      integer*4 icxP,icyp
      REAL*4    COSANG, SINANG
      INTEGER*4 ICX, ICY
      REAL*4    SCLX, SCLY, TOUR

C     Stuff the common area with the values passed in 
      sinang = sinangp
      cosang = cosangp
      icx = icxp
      icy = icyp

      PRINT = -1
      DO I = 1,(NCOL+1)
         XP(I) = I-1
      ENDDO
      DO I = 1,(NROW+1)
         YP(I) = I-1
      ENDDO
      NNX  = NCOL+1
      NNY  = NROW+1

      DO J = 1,(NROW+1)
         DO I = 1,(NCOL+1)
            IMG(I,J) = 0
         ENDDO
      ENDDO

      DO I = 2,NNX
        JRP = 1
        JSP = 1
        IF(MAP(I,1) > TOUR) THEN
          J = I-1
          IF(MAP(J,1) <= TOUR) THEN
            IRP = I
            ISP = J
            CALL CONTUR_C_ROB (MAP,IMG,XP,YP,NCOL,NROW,PRINT,SCLX,SCLY,
     $           TOUR, ix,iy,ix1,iy1, pltPt)
            PRINT = -1
          ENDIF
        ENDIF
      ENDDO

      DO I = 2,NNY
         ISP = NNX
         IRP = NNX
         IF(MAP(NNX,I) > TOUR) THEN
            J = I-1
            IF(MAP(NNX,J) <= TOUR) THEN
               JRP = I
               JSP = J
               CALL CONTUR_C_ROB (MAP,IMG,XP,YP,NCOL,NROW,PRINT,SCLX
     $              ,SCLY,TOUR, ix,iy,ix1,iy1, pltPt)
               PRINT = -1
            ENDIF
         ENDIF
      ENDDO

      ITEMP = NNX-1
      DO I = 1,ITEMP
         JRP = NNY
         JSP = NNY
         II  = NNX-I
         IF(MAP(II,NNY) > TOUR) THEN
            J = II+1
            IF(MAP(J,NNY) <= TOUR) THEN
               IRP = II
               ISP = J
               CALL CONTUR_C_ROB (MAP,IMG,XP,YP,NCOL,NROW,PRINT,SCLX
     $              ,SCLY,TOUR,ix,iy,ix1,iy1, pltPt)
               PRINT = -1
            ENDIF
         ENDIF
      ENDDO

      JTEMP = NNY-1
      DO I = 1,JTEMP
         IRP = 1
         ISP = 1
         II = NNY-I
         IF(MAP(1,II) > TOUR) THEN
            J = II+1
            IF(MAP(1,J) <= TOUR) THEN
               JRP = II
               JSP = J
               CALL CONTUR_C_ROB (MAP,IMG,XP,YP,NCOL,NROW,PRINT,SCLX
     $              ,SCLY,TOUR,ix,iy,ix1,iy1, pltPt)
               PRINT = -1
            ENDIF
         ENDIF
      ENDDO

      DO I = 2,JTEMP
         DO J = 2,NNX
            JJ = J-1
            IF(MAP(J,I) > TOUR) THEN
               IF(MAP(JJ,I) <= TOUR) THEN
                  IF(CONTUR_B_ROB(IMG,NCOL,NROW,J,I,0).NE.1) THEN
                     IRP = J
                     ISP = JJ
                     JRP = I
                     JSP = I
                     CALL CONTUR_C_ROB(MAP, IMG, XP, YP, NCOL, NROW,
     $                    PRINT, SCLX, SCLY, TOUR, ix, iy, ix1, iy1,
     $                    pltPt)
                     PRINT = -1
                  ENDIF
               ENDIF
            ENDIF
         ENDDO
      ENDDO

      RETURN
      END


      FUNCTION CONTUR_B_ROB (IMG, NCOL, NROW, NI, NJ, IND)
      IMPLICIT NONE
      INTEGER*4 CONTUR_B_ROB, NI, NJ, IND, NCOL, NROW
      INTEGER*2 IMG(NCOL+1,NROW+1)
      CONTUR_B_ROB = 0
      IF(IMG(NI,NJ) == 1) THEN
        CONTUR_B_ROB = 1
      ENDIF
      IF(IND.NE.0) IMG(NI,NJ) = 1
      RETURN
      END


      SUBROUTINE CONTUR_C_ROB (MAP,IMG,XP,YP,NCOL,NROW,PRINT,SCLX,SCLY,
     1                    TOUR, ix,iy,ix1,iy1, pltPt)
C
      IMPLICIT NONE
C
      COMMON    /COM031/ IRP,JRP,ISP,JSP,NNX,NNY
      INTEGER*4 IT(9), JT(9), LOCATE, CONTUR_B_ROB, PRINT, NCOL, NROW
      INTEGER*4 IRP, JRP, ISP, JSP, NNX, NNY, IN, JN, INN, JNN, pltPt
      INTEGER*2 IMG(NCOL+1,NROW+1)
      REAL*4    MAP(NCOL+1,NROW+1), XP(NCOL+1), YP(NROW+1)
      REAL*4    TOUR, ZPP, SCLX, SCLY, VX, VY, HTM
      integer*2 ix(40000),iy(40000),ix1(40000),iy1(40000)
      LOGICAL   DTEST(9)
      DATA      IT / 0,1,1, 0,9,0,-1,-1,0/
      DATA      JT /-1,0,0,-1,9,1, 0, 0,1/
      DATA      DTEST /.FALSE.,.TRUE.,.FALSE.,.TRUE.,.FALSE.,
     1  .TRUE.,.FALSE.,.TRUE.,.FALSE./
C  ----------------------------------------------------------------------------
C
      GO TO 200
  100      IRP = IN
      JRP = JN
 200  CALL CONTUR_D_ROB (XP(IRP),YP(JRP),MAP(IRP,JRP),XP(ISP),YP(JSP),
     1     MAP(ISP,JSP), PRINT, SCLX, SCLY, TOUR, ix,iy,ix1,iy1, pltPt)
      PRINT = 1
      LOCATE = 3*(JRP-JSP)+IRP-ISP+5
      IN     = ISP+IT(LOCATE)
      JN     = JSP+JT(LOCATE)
      IF(IN <= NNX.AND.IN >= 1.AND.JN <= NNY.AND.JN >= 1) GO TO 300
      RETURN
C
  300      IF(LOCATE == 6) THEN
        IF(CONTUR_B_ROB(IMG,NCOL,NROW,IRP,JRP,1) == 1) RETURN
      ENDIF
C
      IF(.NOT.DTEST(LOCATE)) THEN
        ZPP = MAP(IN,JN)
        IF(ZPP > TOUR) GO TO 100
        ISP = IN
        JSP = JN
        GO TO 200
      ENDIF
C
      VX     = (XP(IRP)+XP(IN))*0.5
      VY     = (YP(JRP)+YP(JN))*0.5
      LOCATE = 3*(JRP-JN)+IRP-IN+5
      INN    = IN+IT(LOCATE)
      JNN    = JN+JT(LOCATE)
      HTM    = (MAP(IRP,JRP)+MAP(ISP,JSP)+MAP(IN,JN)+MAP(INN,JNN))/4.0
      IF(HTM > TOUR) GO TO 800
      CALL CONTUR_D_ROB (XP(IRP), YP(JRP), MAP(IRP,JRP), VX, VY, HTM,
     1            PRINT, SCLX, SCLY, TOUR, ix,iy,ix1,iy1, pltPt)
      IF(MAP(INN,JNN) <= TOUR) THEN
        ISP = INN
        JSP = JNN
        GO TO 200
      ENDIF
C
      CALL CONTUR_D_ROB (XP(INN), YP(JNN), MAP(INN,JNN), VX, VY, HTM,
     1            PRINT, SCLX, SCLY, TOUR, ix,iy,ix1,iy1, pltPt)
      IF(MAP(IN,JN) <= TOUR) THEN
        IRP = INN
        JRP = JNN
        ISP = IN
        JSP = JN
        GO TO 200
      ENDIF
      CALL CONTUR_D_ROB (XP(IN), YP(JN), MAP(IN,JN), VX, VY, HTM,
     1              PRINT, SCLX, SCLY, TOUR, ix,iy,ix1,iy1, pltPt)
      GO TO 100
  800      CONTINUE
      CALL CONTUR_D_ROB (VX, VY, HTM, XP(ISP), YP(JSP), MAP(ISP,JSP),
     1              PRINT, SCLX, SCLY, TOUR, ix,iy,ix1,iy1, pltPt)
      IF(MAP(IN,JN) > TOUR) GO TO 100
      CALL CONTUR_D_ROB (VX, VY, HTM, XP(IN), YP(JN), MAP(IN,JN),
     1              PRINT, SCLX, SCLY, TOUR, ix,iy,ix1,iy1, pltPt)
      IF(MAP(INN,JNN) > TOUR) GO TO 900
      CALL CONTUR_D_ROB (VX, VY, HTM, XP(INN), YP(JNN), MAP(INN,JNN),
     1              PRINT, SCLX, SCLY, TOUR, ix,iy,ix1,iy1, pltPt)
      ISP = INN
      JSP = JNN
      GO TO 200
  900      ISP = IN
      JSP = JN
      IRP = INN
      JRP = JNN
      GO TO 200
C
      END
C
C  ****************************************************************************
      SUBROUTINE CONTUR_D_ROB (XR,YR,HR,XS,YS,HS,IPLT,SCLX,SCLY,TOUR,
     1              ixp,iyp,ix1p,iy1p,pltPt)
C
      IMPLICIT NONE
C
      INTEGER*4 IPLT, pltPt
      integer*4 ix,iy,ix1,iy1
      INTEGER*2 ixp(40000),iyp(40000),ix1p(40000),iy1p(40000)
      REAL*4    XR, YR, XS, YS, FRAC, HR, HS, xplt, yplt
      REAL*4    TOUR, SCLX, SCLY

      FRAC  = (HR-TOUR)/(HR-HS)
      XPlt  = SCLX*(XR-FRAC*(XR-XS))
      YPlt  = SCLY*(YR-FRAC*(YR-YS))
      IF(IPLT == 1) THEN
         CALL MAPDSP_VECPLT_ROB (XPLT, YPLT, 2, ix,iy,ix1,iy1)
         ixp(pltPt)  = ix
         iyp(pltPt)  = iy
         ix1p(pltPt) = ix1
         iy1p(pltPt) = iy1
         if(pltPt < 39999) then
           pltPt = pltPt + 1
         endif
       ELSE
         CALL MAPDSP_VECPLT_ROB (XPLT, YPLT, 3, ix,iy,ix1,iy1)
      ENDIF
C
C
C
      RETURN
      END
C  ****************************************************************************
Ca *  SUBROUTINE  MAPDSP_VECPLT_ROB()
Ca *  I need to pass these values of ix,iy,ix1,iy1 back to the X WIndows
Ca *  So I can display them there...
Ca *
Ca ****************************************************************************
      SUBROUTINE MAPDSP_VECPLT_ROB(XP, YP, IPEN, ix, iy, ix1, iy1)

      IMPLICIT NONE
      SAVE      XN, YN

      COMMON    /VPLT/ COSANG, SINANG, ICX, ICY
      REAL*4    COSANG, SINANG, X, Y, XP, YP, XN, YN
      INTEGER*4 ICX, ICY, IBEN, IPEN
      INTEGER*4 IX, IY, IX1, IY1
C  ----------------------------------------------------------------------------
C
      IBEN = ABS(IPEN)
      GO TO (100,200,300) IBEN
  100      WRITE(6,1000) IPEN
      RETURN
  200      IX  = XN+ICX+YN*COSANG+0.5
      IY  = YN*SINANG+ICY+0.5
      IX1 = XP+ICX+YP*COSANG+0.5
      IY1 = YP*SINANG+ICY+0.5
  300      XN  = XP
      YN  = YP
      IX1 = XP+ICX+YP*COSANG+0.5
      IY1 = YP*SINANG+ICY+0.5
      IF(IPEN > 0) RETURN
      CALL BELL(1)
      X  = XN
      Y  = YN
      XN = 0
      YN = 0
C
         RETURN
 1000      FORMAT(' IPEN COMMAND ',I8,' IS ILLEGAL ')
      END



C  ******************************************************************
C  *                                                                *
C  *            *****     SUBROUTINE ccf_refine   *****             *
C  *                                                                *
C  ******************************************************************
      SUBROUTINE CCF_REFINE (IMG_ADDR, BKG, NCOL, NROW, DX, DY)
C
      IMPLICIT NONE

      COMMON /CCF_REFINE_COM/ RES_MIN, RES_MAX, X_SHIFT, Y_SHIFT
      real*4    BKG
      INTEGER*4 NCOL, NROW
      INTEGER*4 IDIM1, IDIM2, ARRAY_SIZE, NPEAK, MPEAK, NLIM
      INTEGER*4 FFT_NFAC
      real*4    IMG_ADDR(1)
      PARAMETER (NLIM=5000)
      REAL*4    PEAKS (3,NLIM)
      REAL*4    RES_MIN, RES_MAX
      REAL*4    X_SHIFT, Y_SHIFT
      REAL*4    MIN_OLD, MAX_OLD
      integer*4  DX, DY
      real*4    map_avg,map_std
      real*4    img_bkg, map_min, map_max
      real*4, allocatable :: fft1_addr(:), fft2_addr(:), cc_map(:)
      integer   alloc_stat
C  ----------------------------------------------------------------------------
C
      FFT_NFAC = 1                
      CALL FFT_SETDIM_DEF_SAME (NCOL,NROW,IDIM1,IDIM2,FFT_NFAC)

      allocate(fft1_addr(2*idim1*idim2), fft2_addr(2*idim1*idim2),
     $     stat=alloc_stat)

C      write(6, *) IDIM1,IDIM2, FFT1_ADDR, FFT2_ADDR
C       ************************************************************
C------*    SET CORRELATION RESOLUTION LIMITS: RES_MIN,RES_MAX    *
C       ************************************************************
C      RES_TMP1 = 0 
C      RES_TMP2 = SQRT(REAL(IDIM1/(2*FFT_NFAC))**2.0 +
C      1                  REAL(IDIM2/(2*FFT_NFAC))**2.0)
C      write(6,*) "res_tmp2 is:", res_tmp2
C      RES_MIN = RES_TMP1*FFT_NFAC
C      RES_MAX = RES_TMP2*FFT_NFAC/2
        RES_MIN = 1
        RES_MAX = 15 
C
C      SET IMAGE REFERENCE CENTER 
C
      X_SHIFT = 0 
      Y_SHIFT = 0 

      IMG_BKG = BKG
      CALL R4_IMG_FFT_FILL (IMG_ADDR, NCOL, NROW,
     1     FFT1_ADDR, IDIM1, IDIM2, img_bkg)
      CALL FFT_2D (FFT1_ADDR, IDIM1, IDIM2)

      ARRAY_SIZE = 2*IDIM1*IDIM2                        ! # LONG WORDS
      fft2_addr(1:array_size) = fft1_addr(1:array_size) ! F90 syntax
      CALL FFT_FLIP (FFT2_ADDR, IDIM1, IDIM2)
      X_SHIFT = REAL (IDIM1 - NCOL + 1)
      Y_SHIFT = REAL (IDIM2 - NROW + 1)
      CALL FFT_PSHIFT (FFT2_ADDR, IDIM1, IDIM2, X_SHIFT, Y_SHIFT)

      allocate(cc_map(idim1*idim2), stat=alloc_stat)

      CALL CCF_FFT (FFT1_ADDR, FFT2_ADDR, CC_MAP,
     1             IDIM1, IDIM2, RES_MIN, RES_MAX)
      MIN_OLD = MAP_MIN
      MAX_OLD = MAP_MAX
      CALL MAP_SCALE (CC_MAP, IDIM1, IDIM2, 1, MIN_OLD,
     1     MAX_OLD, 1.0, 255.0, map_min, map_max,
     2     map_avg, map_std)
C
      CALL MAP_PEAKSEARCH (CC_MAP, IDIM1, IDIM2, PEAKS, NLIM,
     1                    NPEAK, 230.0)
      CALL PEAK_SORT_HI_ROB (PEAKS, NPEAK)
      CALL PEAK_CHK_ROB  (PEAKS, NPEAK, MPEAK, 1.5)
      CALL PEAK_SORT_HI_ROB (PEAKS, NPEAK)
      CALL CCF_REFINE_PEAK (IDIM1, IDIM2, PEAKS, NPEAK, DX, DY )

      deallocate(fft1_addr, fft2_addr, cc_map)
      RETURN

      END
C  ******************************************************************
C  *                                                                *
C  *            *****  SUBROUTINE ref_ccf_refine  *****             *
C  *                                                                *
C  ******************************************************************
      SUBROUTINE REF_CCF_REFINE (REF_FFT_ADDR, IMG_ADDR, BKG, 
     1     NCOL, NROW, IDIM1, IDIM2, DX, DY)
C
      IMPLICIT NONE

      COMMON /CCF_REFINE_COM/ RES_MIN, RES_MAX, X_SHIFT, Y_SHIFT
      real*4, allocatable :: fft_addr(:)
      INTEGER*4 NCOL, NROW
      real*4    BKG
      INTEGER*4 IDIM1, IDIM2, ARRAY_SIZE, NPEAK, MPEAK, NLIM
      INTEGER*4 FFT_NFAC
      real*4, allocatable :: cc_map(:)
      real*4    IMG_ADDR(1)
      REAL*4    REF_FFT_ADDR(1)
      PARAMETER (NLIM=5000)
      REAL*4    PEAKS (3,NLIM)
      REAL*4    RES_MIN, RES_MAX
      REAL*4    X_SHIFT, Y_SHIFT
      REAL*4    MIN_OLD, MAX_OLD
      integer*4  DX, DY
      real*4    img_bkg,map_min,map_max
      integer   alloc_stat
C  ----------------------------------------------------------------------
C
      FFT_NFAC = 1                

      allocate(fft_addr(2*idim1*idim2), stat=alloc_stat)

C
C       ************************************************************
C------*    SET CORRELATION RESOLUTION LIMITS: RES_MIN,RES_MAX    *
C       ************************************************************
        RES_MIN = 1
        RES_MAX = 15 

C      SET IMAGE REFERENCE CENTER 

      X_SHIFT = 0 
      Y_SHIFT = 0 

      IMG_BKG = BKG
      CALL R4_IMG_FFT_FILL(IMG_ADDR, NCOL, NROW,
     1     FFT_ADDR, IDIM1, IDIM2, img_bkg)

      CALL FFT_2D (FFT_ADDR, IDIM1, IDIM2)

      ARRAY_SIZE = 2*IDIM1*IDIM2                           ! # LONG WORDS

      allocate(cc_map(idim1*idim2), stat=alloc_stat)

      CALL CCF_FFT (FFT_ADDR, REF_FFT_ADDR, CC_MAP,
     1             IDIM1, IDIM2, RES_MIN, RES_MAX)
      MIN_OLD = MAP_MIN
      MAX_OLD = MAP_MAX
Ca   New way of calling this - For some reason if we ever even call
Ca   this routine we crash in ccf_refine_peak() below.  NOT calling
Ca   map_scale(), however, doesn't seem to impact the results we want.
Ca   VERY strange!!!
Ca      CALL MAP_SCALE (CC_MAP, IDIM1, IDIM2, 1, MIN_OLD,
Ca      1               MAX_OLD, 1.0, 255.0, map_min, map_max,
Ca      2               map_avg, map_std)
C
      CALL MAP_PEAKSEARCH (CC_MAP, IDIM1, IDIM2, PEAKS, NLIM,
     1                    NPEAK, 230.0)
      CALL PEAK_SORT_HI_ROB (PEAKS, NPEAK)
      CALL PEAK_CHK_ROB  (PEAKS, NPEAK, MPEAK, 1.5)
      CALL PEAK_SORT_HI_ROB (PEAKS, NPEAK)
      CALL CCF_REFINE_PEAK (IDIM1, IDIM2, PEAKS, NPEAK, DX, DY )
      
      deallocate(fft_addr, cc_map)
      RETURN
C
      END
C
C  ******************************************************************
C  *                                                                *
C  *          *****     SUBROUTINE ccf_refine_peak     *****        *
C  *                                                                *
C  *    SUBROUTINE TO compute CORRELATION MAP PEAK ARRAY            *
C  ******************************************************************
C
      SUBROUTINE CCF_REFINE_PEAK (NCOL, NROW, PEAKS, NPEAK, DX, DY)

      IMPLICIT NONE

      INTEGER*4 NPEAK
      INTEGER*4 NCOL, NROW, DX, DY
      INTEGER*4 IPEAKX, IPEAKY
      REAL*4    PEAKS(3,NPEAK), HALFX, HALFY

      HALFX   = REAL(NCOL)/2.0
      HALFY   = REAL(NROW)/2.0
      IPEAKX = nINT(PEAKS(1,1)) - 1
      IPEAKY = nINT(PEAKS(2,1)) - 1
      IF(IPEAKX > INT(HALFX)) IPEAKX = IPEAKX - NCOL
      IF(IPEAKY > INT(HALFY)) IPEAKY = IPEAKY - NROW
      DX = -nINT(IPEAKX/2.0)
      DY = -nINT(IPEAKY/2.0)
      RETURN
      END

C
C  ********************************************************************
C  *                                                                  *
C  *            *****     SUBROUTINE IMG_FILT     *****               *
C  *                                                                  *
C  *    LOAD NCOL X NROW "IMAGE" INTO "FFT" ARRAY TO COMPUTE FFT      *
C  *    Then do either lopass or hipass filtering as needed and       *
C  *    back transform.                                               *
C  *                                                                  *
C  *    MAY 1987  PURDUE UNIVERSITY VAX 11/750, T. BAKER.             *
C  *    DECEMBER 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.          *
C  *    LAST MODIFIED  7-AUG-1990 BY T. BAKER.                        *
C  ********************************************************************
C
      SUBROUTINE IMG_FILT (IMG, NCOL, NROW, FFT, IDIM1, IDIM2,
     1     imgBkgP, cutmin, resmin, cutmax, resmax)

      IMPLICIT NONE

      INTEGER*4  NCOL, NROW, IDIM1, IDIM2, img_bkg, imgBkgP
      INTEGER*2  IMG(NCOL,NROW)
      real*4     cutmin, resmin, cutmax, resmax, FFT(2*IDIM1,IDIM2)
      logical    fft_full

      img_bkg = imgBkgP
      fft_full = .true.

      CALL R4_IMG_FFT(img, ncol, nrow, fft, idim1, idim2,
     1     img_bkg)
      
      CALL FFT_FLT(fft, idim1, idim2, resmin, resmax,
     1     cutmin, cutmax, fft_full)
      
      CALL FFT_2D_BACK(fft,idim1,idim2)
      
      CALL FFT_IMG_FILL(fft, idim1, idim2, img, ncol, nrow,
     1     img_bkg)
      
      RETURN
      END
C
C  *********************************************************************
C  *                                                                   *
C  *             *****     SUBROUTINE EMFFT_SPT_ROB1*****              *
C  *                                                                   *
C  *********************************************************************
C
      subroutine EMFFT_SPT_ROB1 (idim1, idim2,
     1        ax,ay,bx,by,nspot,x1p,x2p,y1p,y2p,ncol,nrow,
     2        rasterP, radiusP, cutP)

      IMPLICIT NONE

      COMMON    CUT, KBOX, RADIUS, RASTER, RESLIM, X1, X2, Y1, Y2
      INTEGER*4 IDIM1, IDIM2, NSPOT, KBOX
      INTEGER*4 NH, NK, AMPTYPE,ncol,nrow
      INTEGER*4    T_MATRIX(2,3)
      REAL*4    CUT, RADIUS, RASTER, RESLIM
      real*4    fft_origx, fft_origy, xshift_step,yshift_step
      REAL*4    X1, X2, Y1, Y2, PI, RADIAN, DEGREE
      REAL*4    x1p, x2p, y1p, y2p
      real*4    a_star,   b_star,   c_star, alp_star, bet_star, gam_star
      real*4    ax, ay, bx, by
      real*4    rasterP, radiusP, cutP
      LOGICAL   BACKGROUND, BOXES
C  ----------------------------------------------------------------------------
      x1 = x1p
      x2 = x2p
      y1 = y1p
      y2 = y2p

      CALL PIRADDEG (PI, RADIAN, DEGREE)

      fft_origx = real(ncol - 1)/2.0
      fft_origy = real(nrow - 1)/2.0
      Xshift_step = FFT_ORIGX*2.0*PI/REAL(IDIM1)
      Yshift_step = FFT_ORIGY*2.0*PI/REAL(IDIM2)

      background = .true.
      boxes      = .false.
      ampType = 0   ! Integrated
      kbox = 0      ! Circular
      radius = radiusP  ! Horizontal Radius
      cut    = cutP     ! Cut*radius stuff
      t_matrix(1,1) = 1
      t_matrix(1,2) = 0
      t_matrix(1,3) = 0
      t_matrix(2,1) = 0
      t_matrix(2,2) = 1
      t_matrix(2,3) = 0
      raster = rasterP
      reslim = (idim1*raster)/(sqrt(2.0*((idim1/2)**2)))

      CALL FFT_SPT_LAT_ROB (IDIM1, IDIM2, NSPOT, NH, NK, BACKGROUND,
     1 t_matrix, a_star, b_star, c_star, alp_star, bet_star, gam_star,
     2 ax, ay, bx, by)

      RETURN
      END
C
C  *********************************************************************
C  *                                                                   *
C  *             *****     SUBROUTINE EMFFT_SPT_ROB2 *****              *
C  *                                                                   *
C  *    INTEGRATES VALUES AROUND IDEAL LATTICE POINTS IN 128x128 UP    *
C  *     TO 1024x1024 SIZE FFTs                                        *
C  *                                                                   *
C  *    NOTE: AT THE MOMENT, THIS PROGRAM ONLY WORKS FOR FFTS WHERE    *
C  *     IDIM1 = IDIM2.  A WARNING MESSAGE IS GIVEN AND THE PROGRAM    *
C  *     TERMINATES IF IDIM1.NE.IDIM2                                  *
C  *                                                                   *
C  *    THE INTEGRATION REGION AROUND EACH SPOT MAY BE CIRCULAR OR     *
C  *     ELLIPTICAL                                                    *
C  *    AMPLITUDES WITHIN THE HOLES ARE GAUSSIAN WEIGHTED AND MAY BE   *
C  *     BACKGROUND SUBTRACTED                                         *
C  *                                                                   *
C  *    UNITS: 2 = EMFFTSPT.TMP (TEMP FILE CONTAINING SPOT STATS)      *
C  *           3 = FILE FOR OUTPUT OF STRUCTURE FACTOR DATA            *
C  *           4 = FFTLAT.DAT OR EMCORAVG.DAT (LATTICE PARAMETERS      *
C  *                FROM EMFFTLAT, EMLATREF OR EMCORAVG)               *
C  *                                                                   *
C  *     THIS ROUTINE IS A MAJOR REVISION OF SELEC1.TBI, ORIGINALLY    *
C  *      USED ON THE CAMBRIDGE IBM 370                                *
C  *                                                                   *
C  *    MAY 1981  BRANDEIS UNIVERSITY PDP 11/40, BY T. BAKER.          *
C  *    MAY 1984  PURDUE UNIVERSITY VAX 11/750, BY T. BAKER.           *
C  *    AUGUST 1988  PURDUE UNIVERSITY VAX 8550, BY T. BAKER.          *
C  *    LAST MODIFIED  5-MAR-1992 BY T. BAKER.                         *
C  *********************************************************************
C
      subroutine EMFFT_SPT_ROB2 (fft, idim1, idim2,
     1        ax,ay,bx,by,nspot,stFacs,x1p,x2p,y1p,y2p,
     2        ncol,nrow,
     3        a_star, b_star, c_star, alp_star, bet_star, gam_star,
     4        rasterP, radiusP, cutP, shiftFFT)
C
      IMPLICIT NONE
C
        include '../include/strucfac.inc'
C
      COMMON    CUT, KBOX, RADIUS, RASTER, RESLIM, X1, X2, Y1, Y2
      INTEGER*4 IDIM1, IDIM2, JDIM, NSPOT, KBOX, NBLOCK
      INTEGER*4 NH, NK, AMPTYPE
      integer*4, allocatable :: nbsmpl(:), nreject(:), nsmpl(:)
      integer*4, allocatable :: ih(:), ik(:)
      real*4,    allocatable :: gauss(:), xcen(:), ycen(:), wpower(:)
      real*4,    allocatable :: bsumi(:), std_back(:), backg(:)
      real*4,    allocatable :: x(:), y(:)
      INTEGER*4 T_MATRIX(2,3), ncol, nrow, shiftFFT
      real*4    fft_origx, fft_origy
      real*4    xshift_step,yshift_step
      REAL*4    CUT, RADIUS, RASTER, RESLIM
      REAL*4    X1, X2, Y1, Y2, PI, RADIAN, DEGREE
      REAL*4    x1p, x2p, y1p, y2p, x_shift, y_shift
      REAL*4    FFT(2*IDIM1,IDIM2+1)
      real*4    a_star,   b_star,   c_star, alp_star, bet_star, gam_star
      real*4    ax, ay, bx, by, tiltaxis, tiltang
      real*4, allocatable :: XT(:), YT(:), dstar(:)
      real*4    rasterP, radiusP, cutP
      LOGICAL   BACKGROUND, BOXES
      DATA      JDIM /16/
      type(strucfac_t) stFacs(nspot)
      integer   alloc_stat
C  ----------------------------------------------------------------------------

Ca      Shift the FFT ???
      if(shiftFFT == 1) then
         x_shift = real(idim1/2)
         y_shift = real(idim2/2)
         CALL FFT_PSHIFT (fft, idim1, idim2, x_shift, y_shift,
     1      .true.)
      endif

      x1 = x1p
      x2 = x2p
      y1 = y1p
      y2 = y2p

      CALL PIRADDEG (PI, RADIAN, DEGREE)

      fft_origx = real(ncol - 1)/2.0
      fft_origy = real(nrow - 1)/2.0
      XSHIFT_STEP = fft_origx*2.0*PI/REAL(IDIM1)
      YSHIFT_STEP = FFT_ORIGY*2.0*PI/REAL(IDIM2)

      background = .true.
      boxes      = .false.
      ampType = 0       ! Integrated
      kbox    = 0       ! Circular
      radius  = radiusP     ! Horizontal Radius
      cut     = cutP        ! Cut*radius stuff
      t_matrix(1,1) = 1
      t_matrix(1,2) = 0
      t_matrix(1,3) = 0
      t_matrix(2,1) = 0
      t_matrix(2,2) = 1
      t_matrix(2,3) = 0
      raster = rasterP
      reslim = (idim1*raster)/(sqrt(2.0*((idim1/2)**2)))

      CALL FFT_SPT_LAT_ROB (IDIM1, IDIM2, NSPOT, NH, NK, BACKGROUND,
     1 t_matrix, a_star, b_star, c_star, alp_star, bet_star, gam_star,
     2 ax, ay, bx, by)

      allocate(ih(nspot), ik(nspot), xt(nspot), yt(nspot),
     $     dstar(nspot), stat=alloc_stat)

      xt = 0.0 ! F90 array syntax
      yt = 0.0 ! F90 array syntax

      CALL FFT_SPT_GRID_ROB (IDIM1, IDIM2, JDIM, NSPOT, NBLOCK, 
     1 XT, YT, IH, IK, DSTAR, NH, NK,
     2 BACKGROUND, ax, ay, bx, by)

      allocate(nsmpl(nspot), gauss(nspot), wpower(nspot), 
     $     xcen(nspot), ycen(nspot), stat=alloc_stat)

      if(background) then
         allocate(nbsmpl(nspot), nreject(nspot), bsumi(nspot),
     $        std_back(nspot), backg(500*nspot), stat=alloc_stat)
      endif

      allocate(x(idim1*(jdim/2)), y(idim1*(jdim/2)), stat=alloc_stat)

      IF(BACKGROUND) THEN
         
         CALL FFT_SPT_BACK_ROB (FFT, IDIM1, IDIM2, JDIM, NSPOT, 
     1        NBLOCK, XT, YT, IH, IK, NSMPL,
     2        GAUSS, XCEN, YCEN, WPOWER,
     3        BSUMI, NBSMPL, BACKG, STD_BACK,
     4        NREJECT, DSTAR, X, Y, AMPTYPE,
     5        xshift_step, yshift_step, t_matrix, tiltaxis, tiltang,
     6        stFacs)
         
         deallocate (nbsmpl, nreject, bsumi, std_back, backg)
         
      ENDIF
      
      deallocate(ih, ik, xt, y, dstar)
      deallocate(nsmpl, gauss, wpower, xcen, ycen)

      RETURN
      END
C
C  *********************************************************************
C  *                                                                   *
C  *           *****     SUBROUTINE FFT_SPT_GRID_ROB *****             *
C  *                                                                   *
C  *    GENERATES INDICIES AND COORDINATES FOR ALL SPOTS WITHIN THE    *
C  *     RESOLUTION LIMIT                                              *
C  *                                                                   *
C  *********************************************************************
C
      SUBROUTINE FFT_SPT_GRID_ROB (IDIM1, IDIM2, JDIM, NSPOT, NBLOCK,
     1 XT, YT, IH, IK, DSTAR, NH, NK, BACKGROUND, AX, AY, BX, BY)

      IMPLICIT NONE

      COMMON    CUT, KBOX, RADIUS, RASTER, RESLIM, X1, X2, Y1, Y2
      INTEGER*4 I, IDIM1, IDIM2, JDIM, NSPOT, NBLOCK, NS, KBOX
      INTEGER*4 NH, NK, NH1, HH, H_INDEX, K_INDEX
      INTEGER*4 IH(NSPOT), IK(NSPOT)
      REAL*4    CUT, RADIUS, RASTER, RESLIM, X, XX, XTEST, Y, YY, R
      REAL*4    XT(NSPOT), YT(NSPOT), DSTAR(NSPOT), YMAX, YTS, Y1, Y2
      REAL*4    XB1, XB2, YB1, YB2, X1, X2, X_BOUNDS, Y_BOUNDS
      REAL*4    AX, AY, BX, BY
      LOGICAL   BACKGROUND
C  ----------------------------------------------------------------------------
C
Ca      SF Amplitude is derived from data integrated from circled
Ca      radius, RADIUS.
Ca
Ca      SF Amplitude is background subtracted value where background
Ca      is obtained from the SFs in an annulus with:
Ca
Ca               r1 = RADIUS - CUT
Ca               r2 = 2 * r1
Ca
Ca      Tim Baker April 2003
Ca
      IF(BACKGROUND) THEN
         X_BOUNDS = (IDIM1/2) - (2.0*CUT*RADIUS)
         Y_BOUNDS = (IDIM2/2) - (2.0*CUT*RADIUS)
       ELSE
         X_BOUNDS = (IDIM1/2) - (CUT*RADIUS)
         Y_BOUNDS = (IDIM2/2) - (CUT*RADIUS)
      ENDIF
      NS  = 0
      NH1 = (NH-1)/2
C  ******************************************
C  *    DETERMINE X AND Y CENTERS OF ALL    *
C  *    SPOTS WITHIN RESOLUTION LIMITS      *
C  ******************************************

      DO K_INDEX = 0,(NK-1)
        XB1 = K_INDEX*X2         ! XB1,YB1 REFER TO LOCATION WHERE SF DATA IS
        YB1 = K_INDEX*Y2         !  OBTAINED
        XB2 = K_INDEX*BX         ! XB2,YB2 REFER TO REFINED LATTICE FROM
        YB2 = K_INDEX*BY         !  ORIGINAL DATA
        DO HH = 1,NH
          H_INDEX = HH - NH1
          IF(K_INDEX.NE.0 .OR. H_INDEX > 0) THEN
            X  = XB1 + H_INDEX*X1                ! TO OBTAIN SF MEASUREMENTS
            IF(ABS(X) <= X_BOUNDS) THEN
              XX = XB2 + H_INDEX*AX         ! USED FOR DSTAR AND RESOLUTION TEST
              IF(ABS(XX) <= 0.00001) THEN
                 XTEST = HUGE(XTEST)
               ELSE
                 XTEST = RASTER*IDIM1/ABS(XX)
              ENDIF

              IF(XTEST >= RESLIM) THEN
                Y  = YB1 + H_INDEX*Y1                ! TO OBTAIN SF MEASUREMENTS
                IF(Y <= Y_BOUNDS) THEN
                  YY = YB2 + H_INDEX*AY  ! USED FOR DSTAR AND RESOLUTION TEST
                  R  = RASTER*IDIM1/SQRT(XX*XX+YY*YY)
                  IF(RESLIM <= R) THEN             ! RESOLUTION TEST
                    NS = NS+1                     ! COUNT # OF SPOTS WITHIN LIMITS

                    DSTAR(NS) = 1.0/R

                    IF(Y < 0.0) THEN             ! CHECK IF SPOT IS FRIEDEL MATE
                       XT(NS) = -X + 1.0 + IDIM1/2
                       YT(NS) = -Y + 1.0
                       IH(NS) = -H_INDEX
                       IK(NS) = -K_INDEX
                     ELSE
                       XT(NS) = X + 1.0 + IDIM1/2
                       YT(NS) = Y + 1.0
                       IH(NS) = H_INDEX
                       IK(NS) = K_INDEX
                    ENDIF

                  ENDIF                        ! RESOLUTION TEST
                ENDIF                              ! Y BOUNDARY TEST
              ENDIF                              ! RESOLUTION TEST
            ENDIF                              ! X BOUNDARY TEST
          ENDIF
          ENDDO
      ENDDO

      YMAX = -HUGE(YMAX)
      DO I = 1,NSPOT
        YTS = YT(I) + RADIUS*CUT*(KBOX + 1)
        IF(KBOX == 1) YTS = YT(I) + (CUT/2.0)
        YMAX = AMAX1(YMAX,YTS)
      ENDDO
      NBLOCK = (YMAX + (JDIM/2) - 0.5)/(JDIM/2)

      RETURN
      END
C
C  ****************************************************************************
C  *                                                                          *
C  *               *****     SUBROUTINE FFT_SPT_LAT_ROB *****                 *
C  *                                                                          *
C  *    GENERATES LATTICE COORDINATES AND DETERMINES THE NUMBER OF SPOTS      *
C  *                                                                          *
C  *    AX,AY,BX,BY ARE USED TO CALCULATE RESOLUTION VALUES AND THE LATTICE   *
C  *     PARAMETERS REFINED FROM THE ORIGINAL DATA (I.E. UNFORCED).           *
C  *                                                                          *
C  ****************************************************************************
C
      SUBROUTINE FFT_SPT_LAT_ROB (IDIM1, IDIM2, NSPOT, NH, NK, 
     1 BACKGROUND, T_MATRIX, A_STAR, B_STAR, C_STAR, ALP_STAR, 
     2 BET_STAR, GAM_STAR, AX, AY, BX, BY)

      IMPLICIT NONE

      COMMON    CUT, KBOX, RADIUS, RASTER, RESLIM, X1, X2, Y1, Y2
      INTEGER*4 IDIM1, IDIM2, NSPOT, NH, NK, NH1, HH, N, NUMCHARI4
      INTEGER*4 I, KBOX, H_INDEX, K_INDEX, T_MATRIX(2,3)
      REAL*4    PI, RADIAN, DEGREE, RASTER, RESLIM, BX_SPOT, BY_SPOT
      REAL*4    VSTAR1, VSTAR2, VSTAR(3), ANGSTR1, ANGSTR2, DIFNG
      REAL*4    Y, R, VX(3), VY(3), CUT, RADIUS, X1, X2, Y1, Y2
      REAL*4    X, XTEST, X_BOUNDS, Y_BOUNDS
      REAL*4    A_STAR,   B_STAR,   C_STAR, ALP_STAR, BET_STAR, GAM_STAR
      REAL*4    AX, AY, BX, BY
      LOGICAL   BACKGROUND

      CALL PIRADDEG (PI, RADIAN, DEGREE)

      VSTAR1 = SQRT(AX*AX+AY*AY)
      VSTAR2 = SQRT(BX*BX+BY*BY)
      DO I = 1,3
        IF(T_MATRIX(1,I).NE.0) THEN
          VSTAR(I) = VSTAR1/(RASTER*REAL(IDIM1))
        ENDIF
        IF(T_MATRIX(2,I).NE.0) THEN
          VSTAR(I) = VSTAR2/(RASTER*REAL(IDIM1))
        ENDIF
      ENDDO
      ANGSTR1  = ATAN2(AY,AX)*DEGREE
      ANGSTR2  = ATAN2(BY,BX)*DEGREE
      DIFNG = AMIN1(ABS(ANGSTR1-ANGSTR2),ABS(ANGSTR1-ANGSTR2+360.0))
      DIFNG = AMIN1(DIFNG,ABS(360.0-DIFNG))
      ALP_STAR = (T_MATRIX(1,2)+T_MATRIX(2,2)) * 
     1          (T_MATRIX(1,3)+T_MATRIX(2,3)) * DIFNG
      IF(ALP_STAR < 0.0) ALP_STAR = 180.0+ALP_STAR
      BET_STAR = (T_MATRIX(1,3)+T_MATRIX(2,3)) * 
     1          (T_MATRIX(1,1)+T_MATRIX(2,1)) * DIFNG
      IF(BET_STAR < 0.0) BET_STAR = 180.0+BET_STAR
      GAM_STAR = (T_MATRIX(1,1)+T_MATRIX(2,1)) *
     1          (T_MATRIX(1,2)+T_MATRIX(2,2)) * DIFNG
      IF(GAM_STAR < 0.0) GAM_STAR = 180.0+GAM_STAR
      A_STAR = VSTAR(1)
      B_STAR = VSTAR(2)
      C_STAR = VSTAR(3)
      WRITE(6,'(6F10.4)') A_STAR,   B_STAR,   C_STAR,
     1                 ALP_STAR, BET_STAR, GAM_STAR
      WRITE(6,1000) AX, AY, BX, BY, VSTAR1, VSTAR2, DIFNG
      DO I = 1,3
        IF(T_MATRIX(1,I).NE.0) THEN
          VX(I) = AX
          VY(I) = AY
        ENDIF
        IF(T_MATRIX(2,I).NE.0) THEN
          VX(I) = BX
          VY(I) = BY
        ENDIF
      ENDDO
      WRITE(6,'(6F10.4)') VX(1), VY(1), VX(2), VY(2), VX(3), VY(3)

      NSPOT = 0
      NK    = 1 +     (RASTER*REAL(IDIM1)/(RESLIM*VSTAR2))
      NH    = 1 + 2.0*(RASTER*REAL(IDIM1)/(RESLIM*VSTAR1))
      NH1   = (NH-1)/2
C  ******************************************
C  *    DETERMINE X AND Y CENTERS OF ALL    *
C  *    SPOTS WITHIN RESOLUTION LIMITS      *
C  ******************************************
      IF(BACKGROUND) THEN
         X_BOUNDS = (IDIM1/2) - (2.0*CUT*RADIUS)
         Y_BOUNDS = (IDIM2/2) - (2.0*CUT*RADIUS)
       ELSE
         X_BOUNDS = (IDIM1/2) - (CUT*RADIUS)
         Y_BOUNDS = (IDIM2/2) - (CUT*RADIUS)
      ENDIF

      DO K_INDEX = 0,(NK-1)
        BX_SPOT = K_INDEX*BX
        BY_SPOT = K_INDEX*BY
        DO HH = 1,NH
          H_INDEX = HH-NH1
          IF(K_INDEX.NE.0 .OR. H_INDEX > 0) THEN
            X = BX_SPOT + H_INDEX*AX
            IF(ABS(X) <= X_BOUNDS) THEN

              IF(ABS(X) <= 0.00001) THEN
                 XTEST = HUGE(XTEST)
               ELSE
                 XTEST = RASTER*IDIM1/ABS(X)
              ENDIF

              IF(XTEST >= RESLIM) THEN
                Y = BY_SPOT + H_INDEX*AY
                IF(Y <= Y_BOUNDS) THEN
                  R = RASTER*IDIM1/SQRT(X*X+Y*Y)
                  IF(RESLIM <= R) THEN             ! RESOLUTION TEST
                    NSPOT = NSPOT+1             ! COUNT # OF SPOTS WITHIN LIMITS
                  ENDIF
                ENDIF
              ENDIF
            ENDIF                        ! X BOUNDARY TEST
          ENDIF
          ENDDO
      ENDDO

      N = NUMCHARI4(NSPOT)
      RETURN

 1000      FORMAT(7X,'LATTICE PARAMETERS',//9X,'1 0',2F10.3,/9X,'0 1',
     1 2F10.3,/7X,'VSTAR1',F10.3,/7X,'VSTAR2',F10.3,
     2 /,7X,'ANGLE',F10.3)
      END

C  ****************************************************************
C  *                                                              *
C  *          *****     SUBROUTINE FFT_SPT_BACK_ROB     *****     *
C  *                                                              *
C  ****************************************************************
C
      SUBROUTINE FFT_SPT_BACK_ROB (FFT, IDIM1, IDIM2, JDIM, NSPOT, 
     1 NBLOCK, XT, YT, IH, IK, NSMPL, GAUSS, XCEN, YCEN, WPOWER, 
     2 BSUMI, NBSMPL, BACKG, STD_BACK, NREJECT, DSTAR, X, Y, AMPTYPE,
     3 XSHIFT_STEP, YSHIFT_STEP, T_MATRIX, TILTAXIS, TILTANG,stFacs)

      IMPLICIT NONE

      COMMON    CUT, KBOX, RADIUS, RASTER, RESLIM, X1, X2, Y1, Y2
      INTEGER*4 IDIM1, IDIM2, JDIM, KDIM, NBLOCK, KBOX, AMPTYPE
      INTEGER*4 I, IT1, IL, IB, J, JJ, NS, JY1, JY2, JX1, JX2
      INTEGER*4 MX, NSPOT, NSPOT1, NBACK, KK, T_MATRIX(2,3), twojji
      INTEGER*4 NSMPL(NSPOT), NBSMPL(NSPOT), NREJECT(NSPOT), IX, IY
      INTEGER*4 H, K, L, IH(NSPOT), IK(NSPOT)
      REAL*4    XT(NSPOT), YT(NSPOT), XCEN(NSPOT), YCEN(NSPOT)
      REAL*4    BSUMI(NSPOT), GAUSS(NSPOT), WPOWER(NSPOT), POW, WPOWB
      REAL*4    DSTAR(NSPOT), STD_BACK(NSPOT), BACKG(500,NSPOT), BACK
      REAL*4    GAU, PTOT, CUT, ACEN, PCEN, RADIUS, SHIFT, SUMSQ, STD
      REAL*4    X(IDIM1,JDIM/2), Y(IDIM1,JDIM/2), SF_SCALEF
      REAL*4    TWOJJ, RSS, XR, YR, R, APART, BPART, XN, YN, R1
      REAL*4    PI, TWOPI, DEGREE, RADIAN, RASTER, RESLIM
      REAL*4    X1, X2, Y1, Y2, FFT(2*IDIM1,IDIM2+1)
      REAL*4    XSHIFT_STEP, YSHIFT_STEP, TILTAXIS, TILTANG
      include '../include/strucfac.inc'
      type(strucfac_t) stFacs(nspot)

C  ******************************
C  *    INITIALIZE VARIABLES    *
C  ******************************
      IT1   = 0
      IL    = 0
      PTOT  = 0.0
      DO I = 1,NSPOT
         NSMPL(I)    =   0
         NBSMPL(I)   =   0
         NREJECT(I)  =   0
         XCEN(I)     = 0.0
         YCEN(I)     = 0.0
         WPOWER(I)   = 0.0
         BSUMI(I)    = 0.0
         GAUSS(I)    = 0.0
         STD_BACK(I) = 0.0
      ENDDO
      CALL PIRADDEG (PI, RADIAN, DEGREE)
      TWOPI = 2.0*PI

C  ***********************************************
C  *    READ DOWN TRANSFORM A BLOCK AT A TIME    *
C  ***********************************************
      KDIM = IDIM1/2
      KK   = 0
      DO IB = 1,NBLOCK

        DO J = 1,(JDIM/2)

          if(kk < -99) then
Ca
Ca            Get everything for the rest from the upper half
Ca
Ca            This rtn expects a HALF FFT so we've got to get our data
Ca            from the upper half...
Ca
             if(kk == 1) then
              kk = idim2
             else
              kk = kk - 1
             endif
          else
             KK = KK + 1
          endif
          DO JJ = 1,KDIM

            TWOJJ        = 2*JJ
            twojji       = twojj
            X(JJ+KDIM,J) = FFT(TWOJJi-1,KK)
            Y(JJ+KDIM,J) = FFT(TWOJJi,KK)
            X(JJ,J)      = FFT(IDIM1+TWOJJi-1,KK)
            Y(JJ,J)      = FFT(IDIM1+TWOJJi,KK)
            
Ca  We've got a disagreement here between decw_emfft/fftlib1.subs and
Ca  this code ...  The first KK==1 line looks just fine but everything
Ca  after that is different...  ???
Ca  I'm thinking the reason is that this routine was originally written
Ca  for 1/2 FFT's but we're passing a FULL FFT here... ???
Ca
Ca            if(x(jj,j) .ne. 0.0 .and. j <= 3.0) then
Ca            if(ib == 1 .and. j <= 9.0) then
Ca             print *, 'xy()xy()-',jj,j,X(JJ+KDIM,J),Y(JJ+KDIM,J),
Ca      1                 X(JJ,J),Y(JJ,J)
Ca            endif

          ENDDO
        ENDDO


C
        DO 500 NS = 1,NSPOT
          RSS = RADIUS*CUT
          JY1 = YT(NS) - RSS*2.0
          IF(JY1 > IT1+(JDIM/2)) GO TO 500
          JY2 = YT(NS) + RSS*2.0
          IF(JY2 <= IT1) GO TO 500
          IF(JY1 < IT1+1)        JY1 = IT1 + 1
          IF(JY2 > IT1+(JDIM/2)) JY2 = IT1 + (JDIM/2)

          JX1 = XT(NS) - RSS*2.0*(KBOX+1)
          JX2 = XT(NS) + RSS*2.0*(KBOX+1)
          IF(JX1 < 1)     JX1 = 1
          IF(JX2 > IDIM1) JX2 = IDIM1

          DO IY = JY1,JY2
            YR = (YT(NS)-IY)
            DO 300 IX = JX1,JX2
              XR = (XT(NS)-IX)/(KBOX+1)
              R  = SQRT(XR*XR+YR*YR)
C               ****************************************************
C---------------*    IF POINT OUTSIDE BACKGROUND AREA GO TO 300    *
C               ****************************************************
              IF(R > RSS*2.0) GO TO 300
              APART  = X(IX,IY-IT1)
              BPART  = Y(IX,IY-IT1)
            
              POW    = APART*APART + BPART*BPART
C               ***********************************************
C---------------*    CHECK IF POINT INSIDE BACKGROUND AREA    *
C               ***********************************************
              IF(R > RSS) THEN
                BSUMI(NS)            = BSUMI(NS) + POW
                NBSMPL(NS)           = NBSMPL(NS) + 1
                BACKG(NBSMPL(NS),NS) = POW
                GO TO 300
              ENDIF
C               *********************************
C  -------------*    POINT INSIDE MASK HOLE     *
C               *********************************
              R1           = R/RADIUS
              GAU          = (EXP(-0.5*R1*R1))**2
              GAUSS(NS)    = GAUSS(NS) + GAU
              WPOWER(NS)   = WPOWER(NS) + POW*GAU
              NSMPL(NS)    = NSMPL(NS) + 1
              IF(ABS(XR*(KBOX+1)) > 1.0) GO TO 300
              IF(ABS(YR) > 1.0) GO TO 300
              MX       = IX-1-IDIM1/2
              SHIFT    = XSHIFT_STEP*MX + YSHIFT_STEP*(IY-1)
              SHIFT    = AMOD(SHIFT,TWOPI)
              XN       =  APART*COS(SHIFT) + BPART*SIN(SHIFT)
              YN       = -APART*SIN(SHIFT) + BPART*COS(SHIFT)
              XCEN(NS) = XCEN(NS) + XN*(1.0-ABS(XR)*
     1                   (KBOX+1))*(1.0-ABS(YR))
              YCEN(NS) = YCEN(NS) + YN*(1.0-ABS(XR)*
     1                   (KBOX+1))*(1.0-ABS(YR))
  300            CONTINUE
          ENDDO
  500        CONTINUE                              ! MAIN LOOP OVER NSPOTS
        IT1 = IT1 + (JDIM/2)
      ENDDO
C
C  **********************************************
C  *    CHECK FOR BAD BACKGROUND MEASUREMENTS   *
C  **********************************************
      DO NS = 1,NSPOT
        IF(NBSMPL(NS).NE.0) THEN
          BACK  = BSUMI(NS)/NBSMPL(NS)
          SUMSQ = 0.0
          DO I = 1,NBSMPL(NS)
            SUMSQ = SUMSQ + (BACKG(I,NS)-BACK)**2
          ENDDO
          STD_BACK(NS) = SQRT(SUMSQ/(NBSMPL(NS)-1))
          NBACK     = 0
          BSUMI(NS) = 0.0                  ! REINITIALIZE THE BACKGROUND SUM
          DO I = 1,NBSMPL(NS)
            IF((BACKG(I,NS)-BACK) <= (2*STD_BACK(NS))) THEN
              NBACK     = NBACK+1
              BSUMI(NS) = BSUMI(NS)+BACKG(I,NS)
            ENDIF
          ENDDO
          NREJECT(NS) = NBSMPL(NS)-NBACK
          NBSMPL(NS)  = NBACK
        ENDIF
      ENDDO
C  ********************************
C  *    OUTPUT SF DATA TO FILE    *
C  ********************************
      TILTAXIS  = 0.0
      TILTANG   = 0.0
      SF_SCALEF = 1.0
      NSPOT1 = NSPOT+1
C  *****************************************************
C  *    CALCULATE AMPLITUDE AND PHASE FOR EACH SPOT    *
C  *     AND WRITE TO OUTPUT FILE                      *
C  *****************************************************
      DO NS = 1,NSPOT
        ACEN = SQRT(XCEN(NS)**2+YCEN(NS)**2)/1000.0          ! SCALE DOWN "ACEN"
        PCEN = 0.0
        IF(ACEN > 0.0.AND.XCEN(NS).NE.0.0) THEN
          PCEN = ATAN2(YCEN(NS),XCEN(NS))*DEGREE
          PCEN = AMOD(PCEN,360.0)
          IF(PCEN < 0.0) PCEN = PCEN + 360.0
        ENDIF
        BACK = BSUMI(NS)
        IF(NBSMPL(NS).NE.0) BACK = BACK/NBSMPL(NS)
        IF(GAUSS(NS).NE.0.0) WPOWER(NS) = WPOWER(NS)/GAUSS(NS)
        WPOWB = WPOWER(NS) - BACK
        IF(WPOWB < 0.0) WPOWB = 0.0
        WPOWB  = SQRT(WPOWB)/1000.0            ! PUT "WPOWB" ON SMALLER SCALE
        ACEN   = ACEN/100.0                    ! PUT "ACEN" ON SMALLER SCALE
        GAU    = SQRT(GAUSS(NS))
        BACK   = SQRT(BACK)/100.0            ! PUT "BACK" ON SMALLER SCALE
        STD    = SQRT(STD_BACK(NS))/100.0
        YT(NS) = YT(NS) - 1.0
        XT(NS) = XT(NS) - 1.0 - IDIM1/2
        CALL HK_TO_HKL (IH(NS), IK(NS), H, K, L, T_MATRIX)

        stFacs(ns)%h = h
        stFacs(ns)%k = k
        stFacs(ns)%l = l
        stFacs(ns)%phi = pcen
        stFacs(ns)%sig = back
        stFacs(ns)%dstr = dstar(ns)
        
        IF(AMPTYPE == 1) THEN
           stFacs(ns)%amp = acen
         ELSE
           stFacs(ns)%amp = wpowb
        ENDIF
      ENDDO

      RETURN
      END


C  ****************************************************************************
C  *                                                                          *
C  *                *****     SUBROUTINE EMSF_BACKT_ROB *****                 *
C  *                                                                          *
C  *    ROUTINE TO COMPUTE FOURIER SYNTHESIS OF TWO-DIMENSIONAL STRUCTURE     *
C  *     FACTOR DATA USING A 64x64 OR 128x128 FFT ROUTINE.                    *
C  *                                                                          *
C  *    FEBRUARY 1980  BRANDEIS UNIVERSITY PDP 11/40, T. BAKER  (FOUR2D).     *
C  *    SEPTEMBER 1982  BRANDEIS UNIVERSITY VAX 11/780, T. BAKER.             *
C  *    JANUARY 1983  PURDUE UNIVERSITY VAX 11/750, T. BAKER (EMSF2DBT).      *
C  *    AUGUST 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER (EMSF2DBT).         *
C  *                                                                          *
C  *    MARCH 1992  PURDUE UNIVERSITY VAX 8550, T. BAKER (EMSF_BACKT).        *
C  *    LAST MODIFIED 12-DEC-1997 BY T. BAKER.                                *
C  ****************************************************************************
C
      SUBROUTINE EMSF_BACKT_ROB (SF_TYPE, EM_NDIM, NSF, stFacs, MAP_MIN,
     $     MAP_MAX, array_addr, backSizeA ,backSizeB)

      IMPLICIT NONE

      INTEGER*4 NSF, NN(2), N1, N2, I, SF_TYPE, EM_NDIM, T_MATRIX(2,3)
      integer*4 backSizeA, backSizeB
      real*4    array_addr(2*backSizeA*backSizeB)
      INTEGER*4 HKL(3,NSF)
      REAL*4    MAP_MIN, MAP_MAX, AMPS(NSF), PHIS(NSF), work

      include '../include/strucfac.inc'
      type(strucfac_t) stFacs(NSF)

      IF(EM_NDIM.NE.2) THEN
        RETURN
      ENDIF
      t_matrix(1,1) = 1
      t_matrix(1,2) = 0
      t_matrix(1,3) = 0
      t_matrix(2,1) = 0
      t_matrix(2,2) = 1
      t_matrix(2,3) = 0
      DO I = 1,NSF
         hkl(1,i) = stFacs(i)%h
         hkl(2,i) = stFacs(i)%k
         hkl(3,i) = stFacs(i)%l
         amps(i)  = stFacs(i)%amp
         phis(i)  = stFacs(i)%phi
      enddo

Ca Doesn't the impose Symmetry and refine Phase Origin do this ???
Ca      Now apply the phase shift here ?????
Ca      xshft = 0.5
Ca      yshft = 0.5
Ca      zshft = 0.0
CA      CALL SF_PSHIFT(em_ndim,sf_type,nsf,hkl,amps,phis,
Ca      1              xshft,yshft,zshft)
 

      WRITE(6,'(''$COMPUTE 64x64<CR>, 128x128(1) or 256x256(2)
     1     MAP? : '')')
      nn(1) = backSizeA
      nn(2) = backSizeB
      N1 = NN(1)
      N2 = NN(2)

      array_addr = 0.0 ! F90 array syntax

      CALL SF_BT_LOAD (SF_TYPE,NSF,HKL,AMPS,PHIS,N1,N2,ARRAY_ADDR,
     1 T_MATRIX)

      CALL SF_BT_MINMAX (N1, N2, ARRAY_ADDR, MAP_MIN, MAP_MAX) !RSSXXX
C  **************************************************
C  *    BACK TRANSFORM THE STRUCTURE FACTOR DATA    *
C  **************************************************
      CALL FOURT (ARRAY_ADDR, NN, 2, -1, 1, work)
      CALL SF_BT_MINMAX (N1, N2, ARRAY_ADDR, MAP_MIN, MAP_MAX)

      RETURN
      END
C
C  ********************************************************************
C  *                                                                  *
C  *            *****     SUBROUTINE EMSF_ORIGINS     *****           *
C  *                                                                  *
C  *    REFINE 2-,3-,4-, OR 6-FOLD PHASE ORIGINS.                     *
C  *                                                                  *
C  *    A  REFINE TWO-FOLD PHASE ORIGIN.                              *
C  *    B  REFINE THREE-FOLD PHASE ORIGIN.                            *
C  *    C  REFINE FOUR-FOLD PHASE ORIGIN.                             *
C  *    D  REFINE SIX-FOLD PHASE ORIGIN.                              *
C  *                                                                  *
C  *    MARCH 1992  PURDUE UNIVERSITY VAX 8550, T. BAKER.             *
C  *    LAST MODIFIED 12-MAR-1992 BY T. BAKER.                        *
C  ********************************************************************
C
      SUBROUTINE EMSF_ORIGINS_ROB (EM_NDIM,SF_TYPE,NSF,
     1 SF_SUMFSQ, stFacs, ikey)
C                 
      IMPLICIT NONE

      real*4, allocatable :: tmp_addr(:)
      integer*4, allocatable :: flag_addr(:)
      INTEGER*4   NSF, TYPE, SF_TYPE, EM_NDIM
      INTEGER*4   T_MATRIX(2,3), i
      INTEGER*4   HKL(3,NSF)
      REAL*4      AMP(NSF), PHI(NSF), XSHFT, YSHFT, SF_SUMFSQ
      real*4      sf_scalef
      integer*4   ikey
      integer     alloc_stat

      include '../include/strucfac.inc'
      type(strucfac_t) stFacs(NSF)

      print *, 'Inside emsf_origins'
      IF(EM_NDIM.NE.2) THEN
        RETURN
      ENDIF

      t_matrix(1,1) = 1
      t_matrix(1,2) = 0
      t_matrix(1,3) = 0
      t_matrix(2,1) = 0
      t_matrix(2,2) = 1
      t_matrix(2,3) = 0
      sf_sumfsq = 0.0
      DO I = 1,NSF
         hkl(1,i) = stFacs(i)%h
         hkl(2,i) = stFacs(i)%k
         hkl(3,i) = stFacs(i)%l
         amp(i)  = stFacs(i)%amp
         phi(i)  = stFacs(i)%phi

Ca        Compute sf_sumfsq like sf_fill() does ...         
         sf_scalef = 1.0  ! For now - let's pass this in in PIF file.
         IF(SF_TYPE == 0) THEN
            AMP(I)    = AMP(I)*SF_SCALEF       ! APART, BPART
            PHI(I)    = PHI(I)*SF_SCALEF
            SF_SUMFSQ = SF_SUMFSQ + AMP(I)**2 + PHI(I)**2
         ELSE IF(SF_TYPE == 1) THEN
            AMP(I)    = AMP(I)*SF_SCALEF       ! AMP, PHASE
            SF_SUMFSQ = SF_SUMFSQ + AMP(I)**2
         ENDIF
         
      enddo

      TYPE = SF_TYPE
      IF(TYPE == 0) CALL SF_ABTOAP (SF_TYPE, NSF, AMP, PHI)
      print *, 'Allocating orig -->>',4*nsf

      allocate(tmp_addr(nsf), flag_addr(nsf), stat=alloc_stat)

      IF(IKEY == 1) THEN
         CALL SF_ORIGIN2_ROB (EM_NDIM, SF_TYPE, NSF, HKL, 
     1   AMP, PHI, XSHFT, YSHFT, TMP_ADDR, SF_SUMFSQ, T_MATRIX)
       ELSE IF(IKEY == 2) THEN
         CALL SF_ORIGIN3_ROB (3, EM_NDIM, SF_TYPE, NSF, HKL, 
     1   AMP, PHI, XSHFT, YSHFT, TMP_ADDR, T_MATRIX)
       ELSE IF(IKEY == 3) THEN
         CALL SF_ORIGIN4_ROB (EM_NDIM, SF_TYPE, FLAG_ADDR, 
     1   NSF, HKL, AMP, PHI, XSHFT, YSHFT, TMP_ADDR, T_MATRIX)
       ELSE IF(IKEY == 4) THEN
         CALL SF_ORIGIN3_ROB (6, em_ndim, sf_type, NSF, HKL, 
     1   AMP, PHI, XSHFT, YSHFT, TMP_ADDR, T_MATRIX)
      ENDIF

      DO I = 1,NSF
         stFacs(i)%h = hkl(1,i) 
         stFacs(i)%k = hkl(2,i) 
         stFacs(i)%l = hkl(3,i) 
         stFacs(i)%amp = amp(i)  
         stFacs(i)%phi = phi(i)  
      enddo

      deallocate(tmp_addr, flag_addr)
      IF(TYPE == 0) CALL SF_APTOAB (SF_TYPE, NSF, AMP, PHI)
      RETURN
C
      END
C
C  ********************************************************************
C  *                                                                  *
C  *              *****     SUBROUTINE EMSF_SYM     *****             *
C  *                                                                  *
C  *    IMPOSE PLANE GROUP SYMMETRIES.                                *
C  *                                                                  *
C  *    NOTE: ONLY SOME OF THE 16 NON-p1 GROUPS AVAILABLE NOW.        *
C  *                                                                  *
C  *    MARCH 1992  PURDUE UNIVERSITY VAX 8550, T. BAKER.             *
C  *    LAST MODIFIED 19-NOV-1997 BY T. BAKER.                        *
C  ********************************************************************
C
      SUBROUTINE EMSF_SYM_ROB(EM_NDIM, SF_TYPE, NLIM, NSF, stFacs,
     1 GAM_STAR, ikey, aoverb)

      IMPLICIT NONE

      integer*4, allocatable :: flag_addr(:)
      INTEGER*4   NLIM, NSF, TYPE, NREJ, SF_TYPE, EM_NDIM
      INTEGER*4   T_MATRIX(2,3), i
      INTEGER*4   HKL(3,NLIM)
      REAL*4      AMP(NLIM), PHI(NLIM), SIG(NLIM), DSTR(NLIM)
      REAL*4      GAM_STAR, aoverb
      integer*4   ikey
      integer     alloc_stat

      include '../include/strucfac.inc'
      type(strucfac_t) stFacs(NSF)

      IF(EM_NDIM.NE.2) THEN
        CALL BELL (2)
        RETURN
      ENDIF

      t_matrix(1,1) = 1
      t_matrix(1,2) = 0
      t_matrix(1,3) = 0
      t_matrix(2,1) = 0
      t_matrix(2,2) = 1
      t_matrix(2,3) = 0
      DO I = 1,NSF
         hkl(1,i) = stFacs(i)%h
         hkl(2,i) = stFacs(i)%k
         hkl(3,i) = stFacs(i)%l
         amp(i)  = stFacs(i)%amp
         phi(i)  = stFacs(i)%phi
         sig(i)   = stFacs(i)%sig
         dstr(i)  = stFacs(i)%dstr
      enddo

      TYPE = SF_TYPE
CCC      IF(TYPE == 0) CALL SF_ABTOAP (SF_TYPE, NSF, AMP, PHI)
      print *, 'Allocating flag - ', 4*nlim

      allocate(flag_addr(nlim), stat=alloc_stat)

      CALL SF_HKL (NLIM, HKL, 1, T_MATRIX)
      IF(IKEY == 1) THEN
         CALL SF_SYM_P2 (SF_TYPE, NSF, AMP, PHI)
       ELSE IF(IKEY == 8) THEN
         CALL SF_SYM_PGG (SF_TYPE, NLIM, NSF, HKL, AMP, PHI, SIG, 
     1   DSTR, FLAG_ADDR, GAM_STAR)
       ELSE IF(IKEY == 9) THEN
         CALL SF_SYM_CMM (SF_TYPE, NLIM, NSF, HKL, AMP, PHI, SIG, 
     1   DSTR, FLAG_ADDR, NREJ, gam_star)
       ELSE IF(IKEY == 10) THEN
         CALL SF_SYM_P4  (SF_TYPE, NLIM, NSF, HKL, AMP, PHI, SIG, 
     1   DSTR, FLAG_ADDR, gam_star, aoverb)
       ELSE IF(IKEY == 11) THEN
         CALL SF_SYM_P4M (SF_TYPE, NLIM, NSF, HKL, AMP, PHI, SIG, 
     1   DSTR, FLAG_ADDR)
       ELSE IF(IKEY == 12) THEN
         CALL SF_SYM_P4G (SF_TYPE, NLIM, NSF, HKL, AMP, PHI, SIG, 
     1   DSTR, FLAG_ADDR)
       ELSE IF(IKEY == 16) THEN
         CALL SF_SYM_P6  (SF_TYPE, NLIM, NSF, HKL, AMP, PHI, SIG, 
     1   DSTR, FLAG_ADDR, gam_star, aoverb)
       ELSE IF(IKEY == 17) THEN
         CALL SF_SYM_P6M (sf_type, NLIM, NSF, HKL, AMP, PHI, 
     1   SIG, DSTR, FLAG_ADDR, gam_star, aoverb)
       ELSE
         CALL BELL (2)
         CALL SF_HKL (NLIM, HKL, -1, T_MATRIX)
      ENDIF
      CALL SF_HKL (NLIM, HKL, -1, T_MATRIX)

      DO I = 1,NSF
         stFacs(i)%h = hkl(1,i) 
         stFacs(i)%k = hkl(2,i) 
         stFacs(i)%l = hkl(3,i) 
         stFacs(i)%amp = amp(i)  
         stFacs(i)%phi = phi(i)  
         stFacs(i)%sig = sig(i)   
         stFacs(i)%dstr = dstr(i)  
      enddo

      deallocate(flag_addr)
      RETURN

      END


C
C  **********************************************************
C  *                                                        *
C  *        *****     SUBROUTINE SF_ORIGIN2     *****       *
C  *                                                        *
C  *    FIND BEST TWO-FOLD ORIGIN IN 2-D CRYSTAL LATTICE    *
C  *                                                        *
C  *    LAST MODIFIED 12-MAR-1992, BY T. BAKER.             *
C  **********************************************************
C
      SUBROUTINE SF_ORIGIN2_ROB (EM_NDIM, SF_TYPE, NSF, 
     1 HKL, AMP, PHI, XSHFT, YSHFT, AMPSQ, SF_SUMFSQ, t_matrix)
C
      IMPLICIT NONE

      INTEGER*4 NSF, IR(21), I, IX, IY, EM_NDIM, SF_TYPE
      integer*4 T_MATRIX(2,3)
      INTEGER*4 HKL(3,NSF)
      REAL*4    AMP(NSF), PHI(NSF), AMPSQ(NSF), RES1, P, PHI2
      REAL*4    XSHFT, YSHFT, ZSHFT, DEL, XSH(21), YSH(21), R(21)
      REAL*4    PHDIFF, XMIN, YMIN, RMIN, SF_SUMFSQ, old_rmin

      CALL SF_HKL (NSF, HKL, 1, T_MATRIX)         ! APPLY T_MATRIX TO HKL DATA

      DEL = 0.0
      DO I = 1,NSF
        AMPSQ(I) = AMP(I)**2.0
      ENDDO
C
      old_rmin = 9999999.9
  100      CALL SF_SETSHIFT_ROB (XSHFT, YSHFT, DEL, RMIN, XSH, YSH)
      WRITE(*,'(/9X,A,/)') 'RESIDUAL MAP (ASSUMING TWO-FOLD AXIS)'
C
      DO IY = 1,21
        DO IX = 1,21
          RES1 = 0.0
          DO I = 1,NSF
            P = PHI(I) - (360.0*(HKL(1,I)*XSH(IX)+HKL(2,I)*YSH(IY)))
            P = AMOD(P,360.0)
            IF(P < 0.0) P = P + 360.0
            PHI2   = INT(P/180.0+0.5)*180.0
            PHDIFF = (P-PHI2)**2*AMPSQ(I)
            RES1   = RES1 + PHDIFF
          ENDDO
C
          RES1   = SQRT(RES1/SF_SUMFSQ)
          R(IX)  = RES1
          IR(IX) = RES1 + 0.5
          IF(RES1 < RMIN) THEN
            RMIN = RES1
            XMIN = XSH(IX)
            YMIN = YSH(IY)
          ENDIF
        ENDDO
        WRITE(*,'(10X,21I3)') (IR(IX),IX=1,21)
      ENDDO
      WRITE(*,1000) XMIN, YMIN, RMIN
C
      XSHFT = XMIN
      YSHFT = YMIN
      if (rmin < (old_rmin * 0.9995)) then
         old_rmin = rmin
         GO TO 100
      endif
C  **********************************************
C  *    APPLY CALCULATED PHASE SHIFT TO DATA    *
C  **********************************************
      CALL SF_PSHIFT (EM_NDIM, SF_TYPE, NSF, HKL, AMP, PHI, XSHFT, 
     1 YSHFT, ZSHFT)
      CALL SF_HKL (NSF, HKL, -1, T_MATRIX)      ! CONVERT BACK TO ORIGINAL HKL
      RETURN
C
 1000      FORMAT(/' Minimum residual at XMIN,YMIN =',2F8.3,' is',F6.2)
      END
C
C  ********************************************************************
C  *                                                                  *
C  *             *****     SUBROUTINE SF_ORIGIN3     *****            *
C  *                                                                  *
C  *    FIND BEST THREE OR SIX-FOLD ORIGIN IN 2D CRYSTAL LATTICE      *
C  *    ADAPTED FROM ORIG3 AND ORIG6 OF MRC PROCESSING SYSTEM         *
C  *                                                                  *
C  *    LAST MODIFIED 12-MAR-1992, BY T. BAKER.                       *
C  ********************************************************************
C
      SUBROUTINE SF_ORIGIN3_ROB (IOPT, EM_NDIM, SF_TYPE, NSF,
     1  HKL, AMP, PHI, XSHFT, YSHFT, P2, t_matrix)

      IMPLICIT NONE

      INTEGER*4 NSF, NX, NY, J, N, N1, M, EM_NDIM, SF_TYPE
        integer*4 T_MATRIX(2,3)
      REAL*4    AMP(NSF), PHI(NSF), P2(NSF), XSHFT, YSHFT, ZSHFT
      REAL*4    DEL, XSH(21), YSH(21), R(21), DJ, DN, ANG
      REAL*4    XMIN, YMIN, RMIN, DIFF, P, D, old_rmin
      INTEGER*4 KJ, KN, HJ, HN, IR(21), IOPT, KOUNT, HKL(3,NSF)

      old_rmin = 999999.9
      CALL SF_HKL (NSF, HKL, 1, T_MATRIX)         ! APPLY T_MATRIX TO HKL DATA
      DEL   = 0.0
   10      DIFF  = 0.0
      KOUNT = 0
      CALL SF_SETSHIFT_ROB (XSHFT, YSHFT, DEL, RMIN, XSH, YSH)
      IF(IOPT == 3) THEN
       ELSE IF(IOPT == 6) THEN
      ENDIF

      DO NY = 1,21
        DO NX = 1,21
          IF(KOUNT.NE.0) THEN
            KOUNT = 0
            DIFF  = 0.0
          ENDIF
          DO N = 1,NSF
            P = PHI(N)-360.0*(XSH(NX)*HKL(1,N)+YSH(NY)*HKL(2,N))
            P = AMOD(P,360.0)
            IF(P < 0.) P = P+360.0
            P2(N) = P
          ENDDO
C
          DO J = 1,(NSF-1)
            HJ = HKL(1,J)
            KJ = HKL(2,J)
            N1  = J+1
            DO 600 N = N1,NSF
              HN = HKL(1,N)
              KN = HKL(2,N)
              DJ = HJ*HJ+HJ*KJ+KJ*KJ
              DN = HN*HN+HN*KN+KN*KN
              IF(DJ.NE.DN) GO TO 600
              IF(DJ.NE.0.0) ANG = (HJ*HN+KJ*KN+0.5*(HN*KJ+HJ*KN))/DJ
              IF(IOPT == 6) GO TO 300
              IF(ANG == -0.5) GO TO 200
              IF(ANG == 0.5) GO TO 100
              GO TO 600
  100              D = P2(J)+P2(N)-360.0
              GO TO 500
  200              D = P2(J)-P2(N)
              GO TO 500
  300              ANG = ABS(ANG)
              IF(ANG == 0.5) GO TO 400
              GO TO 600
  400              D = P2(J)-P2(N)
  500              D = ABS(D)
              IF(D > 180.0) D = 360.0-D
              DIFF  = D+DIFF
              KOUNT = KOUNT+1
  600            CONTINUE
          ENDDO
          IF(KOUNT > 0) THEN
             R(NX) = DIFF/KOUNT
           ELSE
             R(NX) = 0.0
          ENDIF
          IR(NX) = R(NX)+0.5
          IF(RMIN >= R(NX)) THEN
            RMIN = R(NX)
            XMIN = XSH(NX)
            YMIN = YSH(NY)
          ENDIF
        ENDDO
        WRITE(*,'(10X,21I3)') (IR(M),M=1,21)
      ENDDO
      WRITE(*,1000) KOUNT, XMIN, YMIN, RMIN
      XSHFT = XMIN
      YSHFT = YMIN
      if (rmin < (old_rmin * 0.9995)) then
         old_rmin = rmin
         GO TO 10
      endif
C  **********************************************
C  *    APPLY CALCULATED PHASE SHIFT TO DATA    *
C  **********************************************
      CALL SF_PSHIFT (EM_NDIM, SF_TYPE, NSF, HKL, AMP, PHI, XSHFT, 
     1 YSHFT, ZSHFT)
      CALL SF_HKL (NSF, HKL, -1, T_MATRIX)      ! CONVERT BACK TO ORIGINAL HKL
      RETURN
C
 1000      FORMAT(/' Average phase error based on',I4,' comparisons',/
     1 ' Minimum residual at XMIN,YMIN =',2F8.3,' is',F6.2)
      END
C
C  ************************************************************
C  *                                                          *
C  *         *****     SUBROUTINE SF_ORIGIN4     *****        *
C  *                                                          *
C  *    FIND BEST FOUR-FOLD ORIGIN IN 2D CRYSTAL LATTICE      *
C  *    ADAPTED FROM ORIG4 OF MRC PROCESSING SYSTEM           *
C  *                                                          *
C  *    LAST MODIFIED 12-MAR-1992, BY T. BAKER.               *
C  ************************************************************
C
      SUBROUTINE SF_ORIGIN4_ROB (EM_NDIM, SF_TYPE, IFLAG, NSF, 
     1 HKL, AMP, PHI, XSHFT, YSHFT, P2, t_matrix)
C
      IMPLICIT NONE
C
      INTEGER*4 NSF, IFLAG(NSF), I, N, J, EM_NDIM, SF_TYPE
      integer*4 T_MATRIX(2,3)
      INTEGER*4 HJ, HN, HJN, KJ, KN, KJN, IR(21), KOUNT, NX, NY, N1
      INTEGER*4 HKL(3,NSF)
      REAL*4    AMP(NSF), PHI(NSF), P2(NSF), P, D
      REAL*4    XSHFT, YSHFT, ZSHFT, DEL, XSH(21), YSH(21), R(21)
      REAL*4    XMIN, YMIN, RMIN, DIFF, old_rmin

      old_rmin = 999999.9
      CALL SF_HKL (NSF, HKL, 1, T_MATRIX)         ! APPLY T_MATRIX TO HKL DATA
      DEL   = 0.0
   10      DIFF  = 0.0
      KOUNT = 0
      CALL SF_SETSHIFT_ROB (XSHFT, YSHFT, DEL, RMIN, XSH, YSH)
      WRITE(*,'(/,8X,A,/)') 'RESIDUAL MAP (ASSUMING FOUR-FOLD AXIS)'
C  ********************************
C  *    STORE HSQ+KSQ IN IFLAG    *
C  ********************************
      DO I = 1,NSF
        IFLAG(I) = HKL(1,I)*HKL(1,I)+HKL(2,I)*HKL(2,I)
      ENDDO
C
      DO NY = 1,21
        DO NX = 1,21
          IF(KOUNT.NE.0) THEN
            KOUNT = 0
            DIFF  = 0.0
          ENDIF
          DO N = 1,NSF
            P = PHI(N)-360.0*(XSH(NX)*HKL(1,N)+YSH(NY)*HKL(2,N))
            P = AMOD(P,360.0)
            IF(P < 0.) P = P+360.0
            P2(N) = P
          ENDDO
C
          DO J = 1,(NSF-1)
            HJ = HKL(1,J)
            KJ = HKL(2,J)
            N1  = J+1
            DO N = N1,NSF
              IF(IFLAG(J) == IFLAG(N)) THEN
                HN  = HKL(1,N)
                KN  = HKL(2,N)
                HJN = HJ*HN
                KJN = KJ*KN
                IF(HJN == -KJN) THEN
                  D = P2(J)-P2(N)
                  D = ABS(D)
                  IF(D > 180.0) D = 360.0-D
                  DIFF  = D+DIFF
                  KOUNT = KOUNT+1
                ENDIF
              ENDIF
            ENDDO
          ENDDO
C
          R(NX)  = DIFF/KOUNT
          IR(NX) = R(NX)+0.5
          IF(RMIN >= R(NX)) THEN
            RMIN = R(NX)
            XMIN = XSH(NX)
            YMIN = YSH(NY)
          ENDIF
        ENDDO
        WRITE(*,'(10X,21I3)') (IR(I),I=1,21)
      ENDDO
C
        WRITE(*,1000) KOUNT, XMIN, YMIN, RMIN
      XSHFT = XMIN
      YSHFT = YMIN
      if (rmin < (old_rmin * 0.9995)) then
         old_rmin = rmin
         GO TO 10
      endif
C  **********************************************
C  *    APPLY CALCULATED PHASE SHIFT TO DATA    *
C  **********************************************
      CALL SF_PSHIFT (EM_NDIM, SF_TYPE, NSF, HKL, AMP, PHI, XSHFT, 
     1 YSHFT, ZSHFT)
      CALL SF_HKL (NSF, HKL, -1, T_MATRIX)      ! CONVERT BACK TO ORIGINAL HKL
      RETURN
C
 1000      FORMAT(/' Average phase error based on',I4,' comparisons',/
     1 ' Minimum residual at XMIN,YMIN =',2F8.3,' is',F6.2)
        END
C
C  **********************************************************
C  *                                                        *
C  *       *****     SUBROUTINE SF_SETSHIFT     *****       *
C  *                                                        *
C  *    READ STARTING ORIGIN SHIFT AND INCREMENT            *
C  *     XSHFT,YSHFT = STARTING PHASE ORIGIN SHIFT          *
C  *     DEL = SIZE OF X AND Y INCREMENTS                   *
C  **********************************************************
C
      SUBROUTINE SF_SETSHIFT_ROB(XSHFT, YSHFT, DEL, RMIN, XSH, YSH)

      IMPLICIT NONE

      REAL*4    XSHFT, YSHFT, DEL, XSH(21), YSH(21), RMIN, TMP
      INTEGER*4 I
C  ----------------------------------------------------------------------------
C
      RMIN = 1.0E20
      IF(DEL == 0.0) THEN
         WRITE(*,'(A)') '$XSHFT,YSHFT,DSHFT <CR>=0.0,0.0,0.05: '
         xshft = 0.0
         yshft = 0.0
         del = 0.0
         IF(DEL <= 0.0) DEL = 0.05
       ELSE
         TMP = DEL/2.0
         WRITE(*,'(A,F8.5,A)') '$DSHFT <CR> = ', TMP, ': '
         del = 0.0
         IF(DEL <= 0.0) DEL = TMP
      ENDIF

      DO I = 1,21
Ca  DO math here to determine the (i-11) below based on the
Ca  size of the XSH ans YSH arrays - e.g. (i-11) because xsh(21)ysh(21)
        XSH(I) = XSHFT + (I-11)*DEL
        YSH(I) = YSHFT + (I-11)*DEL
      ENDDO
C
      RETURN
      END
C
C  ****************************************************************************
C  *                                                                          *
C  *             *****     SUBROUTINE FFT_LATREF2_INDEX     *****             *
C  *                                                                          *
C  ****************************************************************************
C
      SUBROUTINE FFT_LATREF2_INDEX (IX, IY, IDIM1, IDIM2, JH, JK,
     1 XTMP, YTMP, lx_ylim, AXOLD, AYOLD, BXOLD, BYOLD)

      IMPLICIT NONE

      REAL*4    XTMP, YTMP
      REAL*4    TMP, AXOLD, AYOLD, BXOLD, BYOLD
      INTEGER*4 IX, IY, JH, JK
      INTEGER*4 IDIM1, IDIM2, NPTS, JX, JY
      integer*4 lx_ylim, ix_large, iy_large

      ix_large = ix
      iy_large = iy
      JX   = IX - MOD(IX_large,(LX_YLIM/IDIM1))
      JY   = IY - MOD(IY_large,(LX_YLIM/IDIM2))
      
      XTMP = JX
      YTMP = JY

C  ******************************************
C  *    CALCULATE INDEX OF SELECTED SPOT    *
C  ******************************************
      TMP  = (((AXOLD*YTMP)-(AYOLD*XTMP))/
     1      ((BYOLD*AXOLD)-(BXOLD*AYOLD)))
      JK   = nINT(TMP)
      TMP  = ((XTMP-(JK*BXOLD))/(AXOLD))
      JH   = nINT(TMP)
      XTMP = JX*IDIM1/REAL(LX_YLIM)
      YTMP = JY*IDIM2/REAL(LX_YLIM)
      IX   = XTMP
      IY   = YTMP
      WRITE(6,1000) NPTS, JH, JK, IX, IY
C
      RETURN
 1000      FORMAT(/,' SPOT #',I2,' IS ',2I3,/,
     1 ' CENTRAL POINT TRANSFORM COORDINATES:',
     2 '  X=',I8,' Y=',I8,//11X,'AMPLITUDES',20X,'PHASES')
      END
C
C  ************************************************************
C  *                                                          *
C  *     *****     SUBROUTINE FFT_LATREF2_SPOT     *****      *
C  *                                                          *
C  *    DISPLAY 9X9 WINDOWS OF TRANSFORM AT SELECTED POINTS   *
C  ************************************************************
C
      SUBROUTINE FFT_LATREF2_SPOT (FFT, IDIM1, IDIM2, APART, BPART,
     1 IX, IY, fmax, degree,ncol,nrow )

      IMPLICIT NONE

      INTEGER*4 IX, IY, IAMP2(9,9), IFASE(9,9)
      INTEGER*4 IDIM1, IDIM2, KDIM, IAMP(9,9)
      INTEGER*4 I, J, NREC, IX1, IX2, IY1, IY2, IXX, IIX, IIY
      INTEGER*4 J2, JJ, ISWAP, MX, TWOJ, IAMPI, KK, ncol,nrow
      REAL*4    FFT(2*IDIM1,(IDIM2/2)+1), APART(IDIM1), BPART(IDIM1)
      REAL*4    SCALE1, SCALE2, FRIEDL, XTMP, YTMP, POW, PHA, SHIFT
      REAL*4    FMAX, DEGREE, TWOPI, pi , radian
      real*4    fft_origx, fft_origy
C  ----------------------------------------------------------------------------
C
      CALL PIRADDEG (PI, RADIAN, DEGREE)
      TWOPI = 2.0*PI

      SCALE1 = 999.0/FMAX
      SCALE2 = 99.0/999.0

C     CHECK IF HAVE SELECTED A FRIEDEL MATE
      IF(IY < 0) THEN
         IX     = -IX
         FRIEDL = -1.0
       ELSE
         FRIEDL =  1.0
      ENDIF
      IY  = ABS(IY) + 1
      IX  = IX+(IDIM1/2) + 1
      IX1 = IX - 4
      IX2 = IX + 4
      IF(IX1 < 1)     IX1 = 1
      IF(IX2 > IDIM1) IX2 = IDIM1
      IY1 = IY - 4
      IY2 = IY + 4
      IF(IY1 < 1)         IY1 = 1
      IF(IY2 > (IDIM2+1)) IY2 = (IDIM2+1)

C     INITIALIZE AMPLITUDE AND PHASE ARRAYS
      IAMP  = 0                 ! F90 array syntax
      IAMP2 = 0                 ! F90 array syntax
      IFASE = 0                 ! F90 array syntax
      
      kdim = idim1/2
      fft_origx = real(ncol - 1)/2.0
      fft_origy = real(nrow - 1)/2.0

      NREC = IY2 - IY1 + 1
      KK   = IY1 - 1
      IF(IY1 == 1) IY1 = IY2 - 8
      DO 700 I = 1,NREC
        KK    = KK + 1
        ISWAP = 1
        DO J = 1,KDIM
          TWOJ          = 2*J
          APART(J+KDIM) = FFT(TWOJ-1,KK)
          BPART(J+KDIM) = FFT(TWOJ,KK)
          APART(J)      = FFT(IDIM1+TWOJ-1,KK)
          BPART(J)      = FFT(IDIM1+TWOJ,KK)
        ENDDO
        IIY = 9 - NREC + I

C     FILL UP AMPLITUDE AND PHASE ARRAYS  
C     AMPLITUDES ARE SCALED SO SQRT(FSQMAX) = 99
        
 300    IXX = 0
        DO IIX = IX1,IX2
          IXX = IXX + 1
          MX  = IIX - 1 - (IDIM1/2)
          IF(MX >= (IDIM1/2)) MX = MX - IDIM1
          XTMP  = APART(IIX)
          YTMP  = BPART(IIX)
          SHIFT = FFT_ORIGX*MX + FFT_ORIGY*(IIY + IY1 - 1 - 1)
          PHA   = 0.0
          IF(XTMP.NE.0.0) PHA = ATAN2(YTMP,XTMP) - SHIFT
          PHA   = AMOD(PHA,TWOPI)
          PHA   = (PHA*DEGREE/10.0)*FRIEDL
          IF(PHA < 0.0) PHA = PHA + 36.0
          IFASE(IXX,IIY) = PHA + 0.5
          POW            = (XTMP*XTMP + YTMP*YTMP)
          IAMPI          = SQRT(POW)*SCALE1 + 0.5
          IAMP (IXX,IIY) = IAMPI
          IAMP2(IXX,IIY) = NINT(REAL(IAMPI)*SCALE2)
        ENDDO
C
        IF(ISWAP == -1.OR.I == 1) GO TO 700
        IIY = IIY-2*(I-1)
        IF(IIY > 0) THEN
          J2 = KDIM - 1
          DO J = 1,KDIM
            APART(KDIM+2-J) =  FFT(2*J-1,KK)
            BPART(KDIM+2-J) = -FFT(2*J,KK)
          ENDDO
          APART(1) = FFT(IDIM1+1,KK)
          BPART(1) = FFT(IDIM1+2,KK)
          JJ   = 2*IDIM1 + 1
          DO J = 1,J2
            JJ              = JJ - 2
            APART(KDIM+1+J) =  FFT(JJ,KK)
            BPART(KDIM+1+J) = -FFT(JJ+1,KK)
          ENDDO
          ISWAP = -1
          GO TO 300
        ENDIF
  700      CONTINUE
C  **************************
C  *    OUTPUT OF ARRAYS    *
C  **************************
      IF(FRIEDL == 1.0)  WRITE(6,1000) ((IAMP2(I,J),I=1,9),
     1 (IFASE(I,J),I=1,9),J=9,1,-1)
      IF(FRIEDL == -1.0) WRITE(6,1000) ((IAMP2(I,J),I=9,1,-1),
     1 (IFASE(I,J),I=9,1,-1),J=1,9)
      RETURN
C
 1000      FORMAT(1X,9I3,3X,9I3)
      END
C
C  ****************************************************************************
C  *                                                                          *
C  *                ***** SUBROUTINE ROB_SF_PSHIFT *****                      *
C  *                                                                          *
C  ****************************************************************************
C
      SUBROUTINE ROB_SF_PSHIFT (stFacs, nsf, moveX, moveY)

      IMPLICIT NONE

      INTEGER*4 NSF, I, SF_TYPE, EM_NDIM
      INTEGER*4 HKL(3,NSF)
      REAL*4    amps(nsf), phis(nsf), moveX, moveY, moveZ

      include '../include/strucfac.inc'
      type(strucfac_t) stFacs(NSF)

      sf_type = 1
      em_ndim = 2

      DO I = 1,NSF
         hkl(1,i) = stFacs(i)%h
         hkl(2,i) = stFacs(i)%k
         hkl(3,i) = stFacs(i)%l
         amps(i)  = stFacs(i)%amp
         phis(i)  = stFacs(i)%phi
      enddo

C     Now apply the phase shift here 
      moveZ = 0.0
      CALL SF_PSHIFT(em_ndim, sf_type, nsf, hkl, amps, phis,
     1              moveX, moveY, moveZ)

      DO I = 1,NSF
         stFacs(i)%h = hkl(1,i)
         stFacs(i)%k = hkl(2,i)
         stFacs(i)%l = hkl(3,i)
         stFacs(i)%amp = amps(i)
         stFacs(i)%phi = phis(i)
      enddo

      RETURN
      END



C  **********************************************************************
C  *                                                                    *
C  *              *****     SUBROUTINE MAP2D_STATS     *****            *
C  *                                                                    *
C  *    ROUTINE TO DETERMINE THE MIN/MAX, AVERAGE AND STD FOR VALUES    *
C  *     IN THE "MAP" ARRAY.                                            *
C  *    modified from MAP_STATS                                         *
C  **********************************************************************

      SUBROUTINE MAP2D_STATS (MAP, NCOL, NROW, MAP_MIN,MAP_MAX, MAP_AVG,
     $     MAP_STD, CA1D, CA1D_STD, CA1D_CNT,IDIM_MIN)
      
      IMPLICIT NONE

      INTEGER*4 NCOL, NROW, I, J, ARRAY_SIZE
      INTEGER*4 IDIM_MIN, RI
      REAL*4    MAP(NCOL,NROW), MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD
      REAL*4    VAR, X, R, PNT
      REAL*4    CA1D_CNT(IDIM_MIN/2), TMP1, TMP2
      REAL*4    TOT, SUMSQ, CA1D_STD(IDIM_MIN/2), CA1D(IDIM_MIN/2)
       
      ARRAY_SIZE = NCOL*NROW 

      MAP_MIN  = HUGE(MAP_MIN)
      MAP_MAX  = -HUGE(MAP_MAX)
      SUMSQ    = 0.0
      TOT      = 0.0

      DO I = 1, IDIM_MIN/2
        CA1D(I) = 0.0
        CA1D_STD(I) = 0.0
        CA1D_CNT(I) = 0
      ENDDO
       
      DO I = 1,NCOL
        DO J = 1, NROW
          X       = MAP(I, J)
          SUMSQ   = SUMSQ + X**2
          TOT     = TOT + X
          MAP_MIN = AMIN1(MAP_MIN,X)
          MAP_MAX = AMAX1(MAP_MAX,X)
          
          TMP1 = ABS(I - IDIM_MIN/2 -1)     
          TMP2 = ABS(J - IDIM_MIN/2 -1)  
          R = SQRT (TMP1*TMP1 + TMP2*TMP2)
           RI = R
            
          IF ((RI < IDIM_MIN/2).AND.(RI.NE.0)) THEN
            
            PNT = 1.0 - (R - RI)
            CA1D(RI) = CA1D(RI) + X*PNT
            CA1D_CNT(RI) = CA1D_CNT(RI) + PNT
            CA1D_STD(RI) = CA1D_STD(RI) + X*X*PNT
            CA1D(RI+1) = CA1D(RI+1) + X*(1.0-PNT)
            CA1D_CNT(RI+1) = CA1D_CNT(RI+1) + 1.0 - PNT
            CA1D_STD(RI+1) = CA1D_STD(RI+1) + X*X*(1.0-PNT)
          ENDIF
          IF (RI == IDIM_MIN/2) THEN
            
            PNT = 1.0 - (R - RI)
            CA1D(RI) = CA1D(RI) + X*PNT
            CA1D_CNT(RI) = CA1D_CNT(RI) + PNT
            CA1D_STD(RI) = CA1D_STD(RI) + X*X*PNT
            
          ENDIF
           
        ENDDO
      ENDDO 

      MAP_AVG = TOT/REAL(ARRAY_SIZE)
      MAP_STD = 0.0
      VAR     = (SUMSQ/REAL(ARRAY_SIZE))-(MAP_AVG*MAP_AVG)
      IF(VAR >= 0.0) MAP_STD = SQRT(VAR)

      DO I = 1, IDIM_MIN/2
         CA1D(I)=CA1D(I)/CA1D_CNT(I)
         CA1D_STD(I)=CA1D_STD(I)/CA1D_CNT(I)
         CA1D_STD(I)=CA1D_STD(I)-CA1D(I)*CA1D(I)
         IF(CA1D_STD(I) >= 0.0) THEN
            CA1D_STD(I)= SQRT (CA1D_STD(I))
         ENDIF
      ENDDO

      RETURN
      END


      
      subroutine mag_write(map, ncol, nrow, nsec, xc, yc, zc, mag,
     $     outmap)

c     1/20/92: Authored by Holland Cheng 
c     3/9/09:  Modified by Robert Sinkovits

      implicit none
      
      integer*4, intent(in) :: ncol, nrow, nsec
      real*4, intent(in)    :: map(ncol,nrow,nsec), xc, yc, zc, mag
      real*4, intent(out)   :: outmap(ncol,nrow,nsec)

      integer*4 i, j, k 
      real*4 a

      do k = 1,nsec
         do j = 1,nrow
            do i = 1,ncol
               call mag3d_intrp(a, map, ncol, nrow, nsec, 
     $              i, j, k, mag, xc, yc, zc)
               outmap(i,j,k) = a
            enddo
         enddo
      enddo

      return
      end subroutine mag_write
