C  ******************************************************************
C  *                                                                *
C  *                 *****      MRC-> PIF      *****                *
C  *                                                                *
C  *    LOGICAL UNITS: 7    = INPUT MRC FILE                        *
C  *                   11   = OUTPUT  PIFMAP FILE                   *
C  *    MBS, Jan. 2003                                              *
C  *    Latest changes MBS, Sep. 2003 (Made output of odd dimns)    *
C  *                                                                *
C  ******************************************************************
C
      subroutine read_mrc_file(mrcName, rBuf, two_d,nc,nr,ns)
      implicit none
C
      include '../include/pifhead.inc'
      type(pifhead_t) PIFhead
c
      integer*4 i, k, kstep, j, l, m, n
      integer*4 ncol, nrow, nsec
      integer*4 ncol_n, nrow_n, nsec_n
      integer*4 percent
      integer*4 nc,nr,ns
      integer alloc_stat
      real*4    rBuf(nc,nr,ns)
      integer*4 nfilsz,kspg,kbs
      character MRCname*256
      character fullname*256
      character two_d*1
      integer*2, allocatable :: map_in_st(:)
      real*4, allocatable    :: map_in(:)
      real*4, allocatable    :: map_out(:)

      integer*4 iunit_in,iunit_out
      integer*4 jmode,ilen_mrc
      integer*4 kxyz(3),mxyz(3)
      integer*8 imgOffset
      integer*4 img_size
      integer*4 img_plus_header
      integer*4 nbytes
      integer*4 skip_fl /0/
      real*4    dmin, dmax, dmean
      real*4    glob_max,glob_min,glob_scale
      common//ncol,nrow,nsec
      equivalence (ncol,kxyz)
      
C     -----------------------------------------------------------
C     Standard MRC imcom:
      character*4  cstamp/'MAP'/
      integer*4 karch(5), labls(20,10,5), lstream(12), mapcrs(3,5)
      integer*4 mode(5), nb(5), nbsym(5), ncrs(3,5), ncrst(3,5)      
      integer*4 nlab(5), nspg(5), nxyz(3,5)
      integer*4 nbhdr, nbw, nbw3, nbl, numopen
      logical   flag(5), old(5), nocon(5), unknown(5)
      real*4    cel(6,5), denmmm(3,5), rstamp, stuff(25,5)
      real*4    origin(3,5), stamp(2,5), rms(5)

      common/imcom/ lstream, nbhdr, nbw, nbw3, nb, nbl, ncrs, mode,
     $     ncrst, nxyz, cel, mapcrs, denmmm, nspg, nbsym, stuff,
     $     origin, stamp, rms, nlab, labls, numopen, karch,
     $     nocon, flag, old, unknown
      
      equivalence      (cstamp,rstamp)
C     -----------------------------------------------------------

      iunit_in   =  7
      iunit_out  =  11
      glob_max   = -1.e38
      glob_min   =  1.e38
      glob_scale =  1.e38

c
c----Read input data from user
c
      write(*,'(a)') 'MRC (CCP4) Filename: '
      ilen_mrc = min(len_trim(MRCname), 64)
      if(ilen_mrc>64) then
         write(*,*)'MRC (CCP4) file name too long'
         stop
      else
         do i = ilen_mrc+1, 64
            MRCname(i:i) = ' '
         end do
      end if
      
      call pif_init_head (PIFhead)      

      call imopen(iunit_in,MRCname(1:ilen_mrc),'ro')
        call qqinq(lstream(iunit_in),MRCname(1:ilen_mrc),
     1    fullname,nfilsz)
      call imclose(iunit_in)
      call imopen(iunit_in,MRCname(1:ilen_mrc),'ro')
c     
c   read header of the map files, determine memory requirements
c
c       irdhdr skips symmetry bytes 
      call irdhdr(iunit_in,kxyz,mxyz,jmode,dmin,dmax,dmean)

      if      (jmode > 2) then
         write(*,*)'Not dealing with transforms!'
         stop
      else if (jmode == 0) then
         nbytes = 1
      else if (jmode == 1) then
         nbytes = 2
      else if (jmode == 2) then
         nbytes = 4
      else
         write(*,*)"Mode not implemented yet:",jmode
         stop
      end if
c       If CCP4 header was incorrectly written 
c       (e.g. with nbsym=0 but they are actually present in the file), 
c       skip the difference between calculated and actual file size
      img_size = ncol*nrow*nsec*nbytes + 1024   !Image + header
      if(img_size < nfilsz) then
         call irtsym(iunit_in,kspg,kbs)
         if(kbs == 0) then
            kbs = nfilsz-img_size
            kspg = 1
            call qskip(lstream(iunit_in),kbs)
            skip_fl = 1
         end if
      end if

      if(nsec > 1) then
         write(*,'(a)')"2D/3D [2D]? "
Ca         read (*,'(a)')two_d
      end if
      if     (jmode == 0) then
         if(nsec > 1) then
            if  ((two_d .ne. 'n') .and.
     1         (two_d .ne. 'N') .and.
     2         (two_d .ne. '3')) then
               jmode = 6                                 !boxed images
            end if
         else
            jmode = 0                                    !scanned(?) image
         end if
      else if(jmode == 1) then
         if(nsec > 1) then
            if  ((two_d .ne. 'n') .and.
     1         (two_d .ne. 'N') .and.
     2         (two_d .ne. '3')) then
             jmode = 7                                 !boxed images
            else
               jmode = 20                                !3d map
            end if
         else
            jmode = 1                                    !2d image
         end if
      else if(jmode == 2) then
         if(nsec > 1) then
            if  ((two_d .ne. 'n') .and.
     1         (two_d .ne. 'N') .and.
     2         (two_d .ne. '3')) then
             jmode = 46                                !Set of two_d R4 boxes
            else
             jmode = 20                                !3d map, put it in I2
            end if
         else
            jmode = 2                                    !2d image
         end if
      else
         write(*,*)'Wrong mode:',jmode
         stop
      end if
      Write(*,*)"mode", jmode
      jmode = 46            !Set of 2D R4 boxes


C  *************************************************************
C  * Allocate core memory for input image/map
C  *************************************************************
      allocate(map_in(ncol*nrow), stat=alloc_stat) !Always real!
      if (alloc_stat > 0) then
         write(*,'(a)')'Could not allocate memory for the map'
         stop
      end if

      ncol_n = ncol
      nrow_n = nrow
      nsec_n = nsec
      if((ncol/2)*2 == ncol) then
         Pifhead%mrcX = 1
         ncol_n = ncol + 1
      end if
      if((nrow/2)*2 == nrow) then
         Pifhead%mrcY = 1
         nrow_n = nrow + 1
      end if
C      Make map of odd dimensions, otherwise leave number of images as input
      if((nsec/2)*2 == nsec.and. jmode == 20) then
         Pifhead%mrcZ = 1
c         Pifhead%even = 1
         nsec_n = nsec + 1
      end if

      allocate(map_out(ncol_n*nrow_n), stat=alloc_stat) !Always real!
      if (alloc_stat > 0) then
         write(*,'(a)')'Could not allocate memory for output map'
         stop
      end if

      if((jmode == 0) .or. (jmode == 6)) then
         nbytes = 1
crss         allocate(map_in_bt(ncol_n*nrow_n), stat=alloc_stat)
crss         if (alloc_stat > 0) then
crss            write(*,'(a)')'Could not allocate memory for PIF'
crss            stop
crss         end if
      else if((jmode == 1) .or.
     1      (jmode == 7)) then
         allocate(map_in_st(ncol_n*nrow_n), stat=alloc_stat)
         if (alloc_stat > 0) then
            write(*,'(a)')'Could not allocate memory for PIF'
            stop
         end if
         nbytes = 2
      end if
      
C  *****************************************
C  *    Set up pif format map output file  *
C  *****************************************
       
      Pifhead%lunum = iunit_out
      Pifhead%binfactor  =  1
      Pifhead%mode       = jmode
      print *, 'Mode found - ', jmode

      Pifhead%numcols    = ncol_n                    ! unbinned size
      Pifhead%numrows    = nrow_n                    ! unbinned size
      if  ((two_d .ne. 'n') .and.
     1    (two_d .ne. 'N') .and.
     2    (two_d .ne. '3')) then
         Pifhead%numsecs = 1                                ! 2D case
      else
         Pifhead%numsecs    = nsec_n                    ! 3D
      end if

C   Many parameters in the PIFhead are not setup, should check if
C   these parameters will influence later calculations
 
      Pifhead%a_cell     = ncol 
      Pifhead%b_cell     = nrow 
      Pifhead%c_cell     = nsec

      if  ((two_d .ne. 'n') .and.
     1    (two_d .ne. 'N') .and.
     2    (two_d .ne. '3')) then
         Pifhead%nimg = nsec_n
      else
         Pifhead%nimg = 1                               !MRC has usually one image/file
      end if
c      write(*,*)two_d,pifhead%nimg,nsec_n             !Debug

C     
C     Read all sections from the input map files
C     
      write (*,*)
      write (*,*)' Read MRC map in core, write PIF map out...'

      kstep = nint(nsec/10.0)
      if(kstep <= 0) kstep = 1

      imgOffset = 1024
      img_size = ncol_n*nrow_n
      img_plus_header = nbytes*img_size + 512
      
c       Initialize map_out
      do i = 1, img_size
         map_out(i) = 0.
      end do
      
Cac       Determine scaling first
Ca      do k = 1,nsec
Ca         call irdsec(iunit_in,map_in,*99)
Cac       Stuff input map into the output of possibly different dimensions:
Ca         j = 0
Ca         l = 0
Ca         m = 0
Ca         n = 0
Ca         do i = 1, img_size
Ca            j = j + 1
Ca            if(j>ncol_n) then
Ca             j = 1
Ca             l = l + 1
Ca            end if
Ca            if(j <= ncol) then
Ca             m = m + 1
Ca             if(m>ncol) then
Ca                m = 1
Ca                n = n + 1
Ca                if(n >= nrow) n=nrow-1
Ca             end if
Ca             map_out(j+l*ncol_n) = map_in(m+n*ncol)
Ca            end if
Ca         end do
CaCa         call determine_scaling(PIFhead,map_out,ncol_n,nrow_n,1)
CaCa         if(Pifhead%SCALEF < glob_scale)glob_scale = Pifhead%SCALEF
Ca         if(Pifhead%datar4_max > glob_max)glob_max = Pifhead%datar4_max
Ca         if(Pifhead%datar4_min < glob_min)glob_min = Pifhead%datar4_min
Ca      end do
c       Close and reopen MRC file
      call imclose(iunit_in)
      call imopen(iunit_in,MRCname(1:ilen_mrc),'ro')
c       Skips symmetry bytes as well
      call irdhdr(iunit_in,kxyz,mxyz,jmode,dmin,dmax,dmean)
c       If CCP4 header was incorrectly written 
c       (e.g. with nbsym=0 but they are actually present in the file), 
c       skip the difference between calculated and actual file size
      if(skip_fl .ne. 0) then
         call qskip(lstream(iunit_in),kbs)
         write (*,*)kbs," symmetry bytes skipped"
      end if
      Pifhead%datar4_max = glob_max
      Pifhead%datar4_min = glob_min
      Pifhead%SCALEF     = glob_scale
Ca      Pifhead%recipFloatScaleFactor = 1./Pifhead%SCALEF
Ca      call pif_write_gh (PIFhead) 
ccccc      write (*,*)'scale', glob_scale   !Debug

      Pifhead%imgnum = 1
Ca      call pif_write_dh (PIFhead)
c      call pif_list_head (PIFhead)

c       Main loop
      do k = 1,nsec_n
         if(k <= nsec) then
            call irdsec(iunit_in,map_in,*99)
c       Stuff input map into the output of possibly different dimensions:
            j = 0
            l = 0
            m = 0
            n = 0
            do i = 1, img_size
             j = j + 1
             if(j>ncol_n) then
                j = 1
                l = l + 1
             end if
             if(j <= ncol .and. l <= nrow) then
                m = m + 1
                if(m>ncol) then
                   m = 1
                   n = n + 1
                   if(n>nrow) n=nrow
                end if
                map_out(j+l*ncol_n) = map_in(m+n*ncol)
Ca                   rBuf((k*ncol*nrow)+(j+l*ncol_n)) = map_in(m+n*ncol)
                   rBuf(j,l,k) = map_in(m+n*ncol)
             end if
            end do
            if  ((two_d .ne. 'n') .and.
     1         (two_d .ne. 'N') .and.
     2         (two_d .ne. '3')) then
             Pifhead%imgNum  =  k
             
             if((jmode ==0) .or. (jmode ==6)) then
                do i = 1, img_size
crss                   map_in_bt(i) = map_out(i)
                end do
Ca                call pif_write_byte_image (Pifhead%luNum,map_in_bt,
Ca      1             imgOffset,ncol_n,nrow_n,1)
             else if((jmode == 1) .or. (jmode ==  7)) then
                do i = 1, img_size
                   map_in_st(i) = map_out(i)
                end do
Ca                call pif_write_short_image(Pifhead%luNum,map_in_st,
Ca      1             imgOffset,ncol_n,nrow_n,1)
             else if((jmode == 2) .or. (jmode == 46)) then
Ca                call pif_write_boxi4 (PIFhead,map_out)
             end if
Ca             imgOffset = imgOffset + img_plus_header !Calculate new image offset
Ca             if(k .ne. nsec_n) then
Ca                Pifhead%imgNum  =  k + 1 !Put right image number
Ca                call pif_write_dh (PIFhead) !Write header for next image
Ca             end if
            else
Ca             call pif_write_seci2(PIFhead,map_out,ncol_n,nrow_n,k)
            end if
            if(mod(k,kstep)==0) then
             percent = nint(k*100./nsec)
CCC             write(*,'("+",i3,''% complete'')') percent
            end if
         else                  !Write out an extra section to make it odd
Ca            call pif_write_seci2(PIFhead,map_out,ncol_n,nrow_n,k)
            if(mod(k,kstep)==0) then
             percent = nint(k*100./nsec)
CCC             write(*,'("+",i3,''% complete'')') percent
            end if
         end if
      end do

      write(*,*)'Done'
      call imclose(iunit_in)
      if (allocated(map_in_st)) deallocate(map_in_st)

      return
 99      write(*,*)"Premature end of file on input"
      return
      end
C ==========================================================


C  ******************************************************************
C  *                                                                *
C  *                 *****      MRC Header     *****                *
C  *                                                                *
C  ******************************************************************
C
      subroutine read_mrc_header(mrcName, pifHead, two_d)
      implicit none
C
      include '../include/pifhead.inc'
      type(pifhead_t) PIFhead
c
      integer*4 i, kstep
      integer*4 ncol, nrow, nsec
      integer*4 ncol_n, nrow_n, nsec_n
      integer*4 nfilsz,kspg,kbs
      character MRCname*256
      character fullname*256
      character two_d*1

      integer*4 iunit_in,iunit_out
      integer*4 jmode,ilen_mrc
      integer*4 kxyz(3),mxyz(3)
      integer*8 imgOffset
      integer*4 img_size
      integer*4 img_plus_header
      integer*4 nbytes
      integer*4 skip_fl /0/
      real*4    dmin, dmax, dmean
      real*4    glob_max,glob_min,glob_scale
      common//ncol,nrow,nsec
      equivalence (ncol,kxyz)
      
C       Standard MRC imcom:
      character*4     cstamp/'MAP'/

      integer*4      karch(5)
      integer*4      labls(20,10,5)
      integer*4      lstream(12)
      integer*4      mapcrs(3,5)
      integer*4      mode(5)
      integer*4      nb(5)
      integer*4      nbsym(5)
      integer*4      ncrs(3,5)
      integer*4      ncrst(3,5)      
      integer*4      nlab(5)
      integer*4      nspg(5)
      integer*4      nxyz(3,5)
      integer*4      nbhdr,nbw,nbw3,nbl,numopen

      logical            flag(5)
      logical            old(5)
      logical            nocon(5)
      logical            unknown(5)

      real*4            cel(6,5)
      real*4            denmmm(3,5)
      real*4          rstamp
      real*4            stuff(25,5)
      real*4            origin(3,5)
      real*4            stamp(2,5)
      real*4            rms(5)

      common/imcom/ lstream,nbhdr,nbw,nbw3,nb,nbl,ncrs,mode,
     $     ncrst,nxyz,cel,mapcrs,denmmm,nspg,nbsym,stuff,
     $     origin,stamp,rms,nlab,labls,numopen,karch,
     $     nocon,flag,old,unknown
C***  
C     What does it mean to use save attribute on common block?
C     gfortran complains - comment out for now
C        save    imcom 
C***
      equivalence      (cstamp,rstamp)




      iunit_in   =  7
      iunit_out  =  11
      glob_max   = -1.e38
      glob_min   =  1.e38
      glob_scale =  1.e38

c
c----Read input data from user
c
      write(*,'(a)') 'MRC (CCP4) Filename: '
      ilen_mrc = min(len_trim(MRCname), 64)
      if(ilen_mrc>64) then
         write(*,*)'MRC (CCP4) file name too long'
         stop
      else
         do i = ilen_mrc+1, 64
            MRCname(i:i) = ' '
         end do
      end if
      
      call pif_init_head (PIFhead)      

      call imopen(iunit_in,MRCname(1:ilen_mrc),'ro')
        call qqinq(lstream(iunit_in),MRCname(1:ilen_mrc),
     1    fullname,nfilsz)
      call imclose(iunit_in)
      call imopen(iunit_in,MRCname(1:ilen_mrc),'ro')
c     
c   read header of the map files, determine memory requirements
c
c       irdhdr skips symmetry bytes 
      call irdhdr(iunit_in,kxyz,mxyz,jmode,dmin,dmax,dmean)

      if      (jmode > 2) then
         write(*,*)'Not dealing with transforms!'
         stop
      else if (jmode == 0) then
         nbytes = 1
      else if (jmode == 1) then
         nbytes = 2
      else if (jmode == 2) then
         nbytes = 4
      else
         write(*,*)"Mode not implemented yet:",jmode
         stop
      end if
c       If CCP4 header was incorrectly written 
c       (e.g. with nbsym=0 but they are actually present in the file), 
c       skip the difference between calculated and actual file size
      img_size = ncol*nrow*nsec*nbytes + 1024   !Image + header
      if(img_size < nfilsz) then
         call irtsym(iunit_in,kspg,kbs)
         if(kbs == 0) then
            kbs = nfilsz-img_size
            kspg = 1
            call qskip(lstream(iunit_in),kbs)
            skip_fl = 1
         end if
      end if

      if(nsec > 1) then
         write(*,'(a)')"2D/3D [2D]? "
Ca         read (*,'(a)')two_d
      end if
      if     (jmode == 0) then
         if(nsec > 1) then
            if  ((two_d .ne. 'n') .and.
     1         (two_d .ne. 'N') .and.
     2         (two_d .ne. '3')) then
               jmode = 6                                 !boxed images
            end if
         else
            jmode = 0                                    !scanned(?) image
         end if
      else if(jmode == 1) then
         if(nsec > 1) then
            if  ((two_d .ne. 'n') .and.
     1         (two_d .ne. 'N') .and.
     2         (two_d .ne. '3')) then
             jmode = 7                                 !boxed images
            else
               jmode = 20                                !3d map
            end if
         else
            jmode = 1                                    !2d image
         end if
      else if(jmode == 2) then
         if(nsec > 1) then
            if  ((two_d .ne. 'n') .and.
     1         (two_d .ne. 'N') .and.
     2         (two_d .ne. '3')) then
             jmode = 46                                !Set of two_d R4 boxes
            else
             jmode = 20                                !3d map, put it in I2
            end if
         else
            jmode = 2                                    !2d image
         end if
      else
         write(*,*)'Wrong mode:',jmode
         stop
      end if
         Write(*,*)"mode", jmode
             jmode = 46                                !Set of 2D R4 boxes


      ncol_n = ncol
      nrow_n = nrow
      nsec_n = nsec
      if((ncol/2)*2 == ncol) then
         Pifhead%mrcX = 1
         ncol_n = ncol + 1
      end if
      if((nrow/2)*2 == nrow) then
         Pifhead%mrcY = 1
         nrow_n = nrow + 1
      end if
C      Make map of odd dimensions, otherwise leave number of images as input
      if((nsec/2)*2 == nsec.and. jmode == 20) then
         Pifhead%mrcZ = 1
c         Pifhead%even = 1
         nsec_n = nsec + 1
      end if

      if((jmode == 0) .or. (jmode == 6)) then
         nbytes = 1
      else if((jmode == 1) .or.
     1      (jmode == 7)) then
         nbytes = 2
      end if
      
      Pifhead%binfactor  =  1
      Pifhead%mode       = jmode

      Pifhead%numcols    = ncol_n                    ! unbinned size
      Pifhead%numrows    = nrow_n                    ! unbinned size
      if  ((two_d .ne. 'n') .and.
     1    (two_d .ne. 'N') .and.
     2    (two_d .ne. '3')) then
         Pifhead%numsecs = 1                                ! 2D case
      else
         Pifhead%numsecs    = nsec_n                    ! 3D
      end if

C   Many parameters in the PIFhead are not setup, should check if
C   these parameters will influence later calculations
 
      Pifhead%a_cell     = ncol 
      Pifhead%b_cell     = nrow 
      Pifhead%c_cell     = nsec

      if  ((two_d .ne. 'n') .and.
     1    (two_d .ne. 'N') .and.
     2    (two_d .ne. '3')) then
         Pifhead%nimg = nsec_n
      else
         Pifhead%nimg = 1                               !MRC has usually one image/file
      end if
c      write(*,*)two_d,pifhead%nimg,nsec_n             !Debug

C     
C     Read all sections from the input map files
C     
      write (*,*)
      write (*,*)' Read MRC map in core, write PIF map out...'

      kstep = nint(nsec/10.0)
      if(kstep <= 0) kstep = 1

      imgOffset = 1024
      img_size = ncol_n*nrow_n
      img_plus_header = nbytes*img_size + 512
      
c       Close and reopen MRC file
      call imclose(iunit_in)
      call imopen(iunit_in,MRCname(1:ilen_mrc),'ro')
c       Skips symmetry bytes as well
      call irdhdr(iunit_in,kxyz,mxyz,jmode,dmin,dmax,dmean)
c       If CCP4 header was incorrectly written 
c       (e.g. with nbsym=0 but they are actually present in the file), 
c       skip the difference between calculated and actual file size
      if(skip_fl .ne. 0) then
         call qskip(lstream(iunit_in),kbs)
         write (*,*)kbs," symmetry bytes skipped"
      end if
      Pifhead%datar4_max = glob_max
      Pifhead%datar4_min = glob_min
      Pifhead%SCALEF     = glob_scale

      Pifhead%imgnum = 1

      return
      write(*,*)"Premature end of file on input"
      return
      end
