C  ****************************************************************************
C  *                                                                          *
C  *                      *****     EMMAP3DT.FOR     *****                    *
C  *                                                                          *
C  *    CALCULATES 3D STRUCTURE FACTORS IN SPACE GROUP P1 FROM A 3D MAP       *
C  *                                                                          *
C  *    THIS PROGRAM IS ADAPTED FROM "FFTBAK", TENN EYCK'S P1 FOURIER         *
C  *     TRANSFORM PROGRAM.  THE PROGRAM EXPECTS AS INPUT A MAP WITH          *
C  *     Z-SECTIONS, IN WHICH X VARIES MOST RAPIDLY AND Y MOST SLOWLY.        *
C  *                                                                          *
C  *    LIMITATIONS: NCOL MUST BE EVEN AND NROW,NSEC MUST NOT HAVE ANY        *
C  *     FACTORS GREATER THAN 19                                              *
C  *                                                                          *
C  *    TENN EYCK'S FFT SUBROUTINES STORED IN [TSB.FOR]FFTLIB.F               *
C  *                                                                          *
C  *    OCTOBER 1981  BRANDEIS UNIVERSITY PDP 11/40, T. BAKER.                *
C  *    DECEMBER 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER.                *
C  *                                                                          *
C  *      Modified Dec 1993, Jan 1994 by D. Belnap                        *
C  *    MODIFIED  3-AUG-1998 BY T. BAKER. (Make PIF compatible).              *
C  *    MODIFIED  5-AUG-1998 BY TSB/RWA (Add new PIF_READ_TMAPI2 and making   *
C  *      use of in core 3D MAP calculations).                                *
C  *    MODIFIED 10-AUG-1998 BY TSB (FFT calculations all in core!).          *
C  *    LAST MODIFIED 12-AUG-1998 BY TSB (SORT not needed).                   *
C  *    TO DOS: 1. MAKE PIF FORMAT FOR STORING SF DATA.
C  ****************************************************************************
C
      SUBROUTINE MAIN_MAP3DT (NCOL, NROW, NSEC, MAP3D, stFacs, nsf,
     $     hmax, KMAX, LMAX)

      IMPLICIT NONE

      include '../include/pifhead.inc'
      include '../include/strucfac.inc'
      
      TYPE(PIFHEAD_T)  MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD
      COMMON /PIFHEAD/ MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD

      INTEGER*4 NCOL, NROW, NSEC, HMAX, KMAX, LMAX, D(5), NSF
      INTEGER*4, allocatable :: HKL(:)
      INTEGER*4 alloc_stat
      REAL*4    MAP3D(1)
      real*4, allocatable :: ARRAY(:), ARRAY2(:)
      real*4, allocatable :: AMP(:), PHI(:), DSTR(:)
      type(strucfac_t) stFacs(nsf)

      PRINT *, 'EMMAP3DT: Calculate 3D SFs in space group P1 from',
     1        ' a 3D MAP'
      PRINT *, '          13-Aug-98 [TSB]'
      PRINT *

      allocate(array((ncol+4)*nrow*nsec), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Could not allocate memory for ARRAY'
         call tsbend
      endif
      array = 0.0 ! F90 array syntax

C     FIRST PASS CALCULATES FOURIER TRANSFORMS ON Z AND X OF NSEC SECTIONS
C GCA  ERROR: following routine expects array of size (ncol+2)*nrow*nsec
      CALL MAP3DT_PASS1 (MAP3D, ARRAY, NCOL, NROW, NSEC, D, HMAX)

      allocate(array2(2*nsec*nrow*(hmax+1)), hkl(3*nsf),
     $     amp(nsf), phi(nsf), dstr(nsf), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Could not allocate memory for ARRAY2 ...'
         call tsbend
      endif

      CALL MAP3DT_STORE (ARRAY, NCOL, NROW, NSEC, HMAX, KMAX, ARRAY2)

C     SECOND PASS READS INTERMEDIATE RESULTS AND CALCS FOURIER TRANSFORM ON Y
C GCA  ERROR: following routine expects both arrays of size2*NSEC*(2*KMAX+1)*(HMAX+1)
      CALL MAP3DT_PASS2 (ARRAY, ARRAY2, NSEC, HMAX, KMAX, D)
C GCA ERROR: following routine expects a complex array with different size
C GCA commented until sorted out
      write(*,*) '[MAIN_MAP3DT]: Output not generated, sorry!'
C      CALL MAP3DT_SFOUT (ARRAY, HKL, AMP, PHI, DSTR, NSEC, HMAX, KMAX, 
C     $     LMAX, NSF, stFacs)

      deallocate(array, array2, hkl, amp, phi, dstr)

      END

C     
C  ****************************************************************
C  *                                                              *
C  *          *****     SUBROUTINE MAP3DT_INFO_rwa     *****          *
C  *                                                              *
C  *    ROUTINE TO READ IN CONTROL PARAMETERS                     *
C  ****************************************************************
C
      SUBROUTINE MAP3DT_INFO_RWA (NCOL, NROW, NSEC, HMAX, KMAX, LMAX, 
     1   NSF, a_cell, b_cell, c_cell, voxsiz, reslo, reshi, scaleP)
C
      IMPLICIT NONE
C
      COMMON    A11,A21,A22,A31,A32,A33,DSQMIN,DSQMAX,SCALE
      COMMON/SF/A_STAR,B_STAR,C_STAR,ALP_STAR,BET_STAR,GAM_STAR
      INTEGER*4 IH, IK, IL
      INTEGER*4 NSF
      INTEGER*4 HMAX, KMAX, LMAX, NCOL, NROW, NSEC, NC2, NR2, NS2
      REAL*4    A_CELL, B_CELL, C_CELL, ALP, BET, GAM, VOL
      REAL*4    A_STAR, B_STAR, C_STAR, VOL_STAR
      REAL*4    ALP_STAR, BET_STAR, GAM_STAR, scaleP
      REAL*4    A11, A21, A22, A31, A32, A33, DSQMIN, DSQMAX, SCALE
      REAL*4    PI, RADIAN, DEGREE, RESLO, RESHI, VOXSIZ
      REAL*4    COSALP, SINALP, COSBET, SINBET, COSGAM, SINGAM
      REAL*4    COSALP_STAR, COSBET_STAR, COSGAM_STAR
      REAL*4    SINALP_STAR, SINBET_STAR, SINGAM_STAR
      REAL*4    AST, BST, CST, TMP, XSQ, YSQ, ZSQ, DSTAR
      real*8    deg_to_rad
C  ----------------------------------------------------------------------------
C
      deg_to_rad = acos(-1.0d0)/180.0d0
      NSF = 0                  ! SET COUNTER FOR # STRUCTURES FACTORS CALCULATED
        CALL PIRADDEG (PI, RADIAN, DEGREE)
C  ********************************************************************
C  *    CHECK INPUT FOR NCOL = EVEN RESTRICTION                       *
C  *    AND ALSO THAT NO PRIMES > 19 ARE FACTORS OF THE DIMENSIONS    *
C  ********************************************************************
      CALL CHK_PRIME (NCOL, NC2, .TRUE.)
      CALL CHK_PRIME (NROW, NR2, .FALSE.)
      CALL CHK_PRIME (NSEC, NS2, .FALSE.)
      WRITE(6,'(A,3I5)') ' 3D MAP old dimensions =', NCOL, NROW, NSEC
      WRITE(6,'(A,3I5)') ' 3D MAP new dimensions =', NC2, NR2, NS2
      NCOL = NC2
      NROW = NR2
      NSEC = NS2
      PRINT *, ' Enter voxsiz and res limits as pixels, Angstrms or nms'
      WRITE(*,'(2A)') '$VOXSIZ,RES_LO,RES_HI,SCALE <CR>=1.0,',
     1 '1000.0,2.0001,1.0: '
      scale = scaleP
      IF(VOXSIZ <= 0.0) VOXSIZ = 1.0
      IF(RESHI <= 0.0)  RESHI = 2.0001*VOXSIZ
      IF(RESLO <= 0.0)  RESLO = 0.0
      IF(SCALE <= 0.0)  SCALE = 1.0
      IF(RESLO.NE.0.0 .AND. RESLO < RESHI) THEN
        TMP   = RESLO
        RESLO = RESHI
        RESHI = TMP
      ENDIF
      IF(RESLO == 0.0) THEN
         DSQMIN = 0.0
       ELSE
         DSQMIN = 1.0/(RESLO**2)
      ENDIF
      DSQMAX = 1.0/(RESHI**2)
C
C      HARDWIRE THESE FOR NOW
C
      A_CELL = NC2*VOXSIZ
      B_CELL = NR2*VOXSIZ
      C_CELL = NS2*VOXSIZ
      ALP    = 90.0
      BET    = 90.0
      GAM    = 90.0
C  **************************************************************
C  *    SET UP OUTPUT FILE FOR STRUCTURE FACTOR CALCULATIONS    *
C  **************************************************************
      CALL REAL_TO_REC (A_CELL,B_CELL,C_CELL,ALP,BET,GAM,VOL,
     1 A_STAR,B_STAR,C_STAR,VOL_STAR,ALP_STAR,BET_STAR,GAM_STAR)

      WRITE(*,1000) A_CELL,B_CELL,C_CELL,ALP,BET,GAM,RESHI,RESLO
      WRITE(*,'(T12,A,F10.2)') ' Fourier scaled by', SCALE
C  *************************************************************
C  *    CALCULATE RECIPROCAL LATTICE PARAMETERS                *
C  *    (SEE INTERNATIONAL TABLES FOR CRYSTALLOGRAPHY, P.13)   *
C  *************************************************************
      COSALP      = COS(ALP*deg_to_rad)
      SINALP      = SIN(ALP*deg_to_rad)
      COSBET      = COS(BET*deg_to_rad)
      SINBET      = SIN(BET*deg_to_rad)
      COSGAM      = COS(GAM*deg_to_rad)
      SINGAM      = SIN(GAM*deg_to_rad)
      COSALP_STAR = (COSBET*COSGAM-COSALP)/(SINBET*SINGAM)
      COSBET_STAR = (COSGAM*COSALP-COSBET)/(SINGAM*SINALP)
      COSGAM_STAR = (COSALP*COSBET-COSGAM)/(SINALP*SINBET)
      SINALP_STAR = SQRT(1.0-COSALP_STAR**2)
      SINBET_STAR = SQRT(1.0-COSBET_STAR**2)
      SINGAM_STAR = SQRT(1.0-COSGAM_STAR**2)
      AST         = 1.0/(A_CELL*SINBET*SINGAM_STAR)
      BST         = 1.0/(B_CELL*SINGAM*SINALP_STAR)
      CST         = 1.0/(C_CELL*SINALP*SINBET_STAR)
C  ***************************************
C  *    CALCULATE MATRIX FOR D* TESTS    *
C  ***************************************
      A11 =  BST*SINGAM_STAR*SINALP
      A21 = -BST*SINGAM_STAR*COSALP
      A22 =  CST*SINBET_STAR
      A31 =  BST*COSGAM_STAR
      A32 =  CST*COSBET_STAR
      A33 =  AST
C  ******************************************
C  *     CALCULATE LIMITS ON H, K, AND L    *
C  ******************************************
      HMAX = INT(SINALP_STAR/(AST*RESHI*SQRT(SINALP_STAR**2 + 
     1      (SINBET_STAR*COSGAM)**2 + (SINGAM_STAR*COSBET)**2 + 
     2      2.0*COSALP_STAR*(2.0*COSBET_STAR*COSGAM_STAR +
     3      SINBET_STAR*COSBET*SINGAM_STAR*COSGAM) - 
     4      2.0*(COSBET_STAR**2+COSGAM_STAR**2))))
      KMAX = INT(SINBET_STAR/(BST*RESHI*SQRT(SINBET_STAR**2 + 
     1      (SINGAM_STAR*COSALP)**2 + (SINALP_STAR*COSGAM)**2 +
     2      2.0*COSBET_STAR*(2.0*COSGAM_STAR*COSALP_STAR +
     3      SINGAM_STAR*COSGAM*SINALP_STAR*COSALP) - 
     4      2.0*(COSALP_STAR**2+COSGAM_STAR**2))))
      LMAX = INT(SINGAM_STAR/(CST*RESHI*SQRT(SINGAM_STAR**2 +
     1      (SINBET_STAR*COSALP)**2 + (SINALP_STAR*COSBET)**2 +
     2      2.0*COSGAM_STAR*(2.0*COSBET_STAR*COSALP_STAR +
     3      SINBET_STAR*COSBET*SINALP_STAR*COSALP) -
     4      2.0*(COSBET_STAR**2+COSALP_STAR**2))))

      WRITE(*,1010) NC2, NR2, NS2, HMAX, KMAX, LMAX

      SCALE  = SCALE/REAL(NC2*NR2*NS2)
C  ****************************************************
C  *    CHECK INPUT FOR POSSIBLE PROGRAM OVERFLOWS    *
C  ****************************************************
      IF(NC2 < 2*HMAX+1) THEN
        WRITE(*,1020) HMAX, NC2
        CALL TSBEND
      ENDIF
      IF(NR2 < 2*KMAX+1) THEN
        WRITE(*,1030) KMAX, NR2
        CALL TSBEND
      ENDIF
      IF(NS2 < 2*LMAX+1) THEN
        WRITE(*,1040) LMAX, NS2
        CALL TSBEND
      ENDIF
C  ***************************************************************
C  *    DETERMINE MAXIMUM SIZE OF NSF WITHIN RESOLUTION LIMITS   *
C  ***************************************************************
        DO IK = -KMAX,KMAX
          XSQ = (REAL(IK)*A11)**2
          DO IL = -LMAX,LMAX
            YSQ = (REAL(IK)*A21 + REAL(IL)*A22)**2 
            DO IH = 1,HMAX+1
              ZSQ = (REAL(IH-1)*A33 + REAL(IK)*A31 + REAL(IL)*A32)**2
              DSTAR = XSQ + YSQ + ZSQ
              IF(DSTAR <= DSQMAX.AND.DSTAR >= DSQMIN) NSF = NSF + 1
            ENDDO                               ! DO IH = 1,HMAX+1
          ENDDO                               ! DO IL = 1,(2*LMAX+1)
        ENDDO                                     ! DO IK = 1,(2*KMAX+1)

      WRITE(*,'(12X,A,I9)') 'Max # of structure factors computed =', NSF
      RETURN

 1000 FORMAT(/,20X,'STRUCTURE FACTOR CALCULATION',/,20X,28('-'),/,12X,
     1     '    A =',F7.2,'     B =',F7.2,'      C =',F7.2,/,12X,
     2     'ALPHA =',F7.2,'  BETA =',F7.2,'  GAMMA =',F7.2,/,12X,
     3     'Resolution limits are',F7.2,' to ',F7.2)
 1010 FORMAT(12X,'Input cell divisions are',3I5,' pixels',/,12X,
     1     'Limits on H, K and L are',3I5)
 1020 FORMAT(' HMAX out of range.  HMAX equals',I5,' NCOL equals',I5)
 1030 FORMAT(' KMAX out of range.  KMAX equals',I5,' NROW equals',I5)
 1040 FORMAT(' LMAX out of range.  LMAX equals',I5,' NSEC equals',I5)
      END
C
C  *******************************************************************
C  *                                                                 *
C  *           *****     SUBROUTINE MAP3DT_SFOUT     *****           *
C  *                                                                 *
C  *    THIS SUBROUTINE PRODUCES BINARY STRUCTURE FACTOR OUTPUT      *
C  *                                                                 *
C  *     THIS IS MODIFIED FROM TEN EYCK'S OUTSF AND PRODUCES         *
C  *     STRUCTURE FACTOR OUTPUT AMP, PHI, DSTAR(1/PIX;1/ANG/1/NMS)  *
C  *******************************************************************
C
      SUBROUTINE MAP3DT_SFOUT (ARRAY, HKL, AMP, PHI, DSTR, NSEC, HMAX, 
     1 KMAX, LMAX, NSF, stFacs)

      IMPLICIT NONE

C     Note - got rid of commented code that writes out sf file. No
C     longer needed and just make code more confusing. (see v3.15 for
C     deleted code)

      include '../include/pifhead.inc'
      TYPE(PIFHEAD_T)  MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD
      COMMON /PIFHEAD/ MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD

      COMMON    A11,A21,A22,A31,A32,A33,DSQMIN,DSQMAX,SCALE
      COMMON/SF/A_STAR,B_STAR,C_STAR,ALP_STAR,BET_STAR,GAM_STAR
      INTEGER*4 JJ, NSEC, HMAX, KMAX, LMAX, NSF, NUM
      INTEGER*4 IH, IK, IL, JL, HH, KK, HKL(3,NSF)
      REAL*4    A_STAR, B_STAR, C_STAR, ALP_STAR, BET_STAR, GAM_STAR
      REAL*4    A11, A21, A22, A31, A32, A33
      REAL*4    DSQMIN, DSQMAX, SCALE
      REAL*4    PI, RADIAN, DEGREE, XSQ, YSQ, ZSQ, DSTAR, MODF, PHASE
      REAL*4    APART, BPART, AMP(NSF), PHI(NSF), DSTR(NSF)
      COMPLEX   ARRAY(NSEC,HMAX+1,(2*KMAX+1))

      include '../include/strucfac.inc'
      type(strucfac_t) stFacs(nsf)

      CALL PIRADDEG (PI, RADIAN, DEGREE)
      NUM = 0                                          ! SF COUNTER

      DO IH = (HMAX+1),2,-1     ! L < 0 INDICES
         HH  = -(IH - 1)
         DO IK = (2*KMAX+1),1,-1
            KK  = -(IK - KMAX - 1)
            XSQ = (REAL(KK)*A11)**2
            DO IL = -1,-LMAX,-1
               JL = IL + 1 + NSEC
               YSQ = (REAL(KK)*A21 + REAL(IL)*A22)**2
               ZSQ = (REAL(HH)*A33 + REAL(KK)*A31 + REAL(IL)*A32)**2
               DSTAR = XSQ + YSQ + ZSQ
               IF(DSTAR > DSQMAX .OR. DSTAR < DSQMIN) CYCLE
               APART =   REAL(ARRAY(JL,IH,IK))
               BPART = -AIMAG(ARRAY(JL,IH,IK))
               MODF  = SQRT(APART**2+BPART**2)*SCALE
               PHASE = 0.0
               IF(MODF.NE.0.0)   PHASE = ATAN2(BPART,APART)*DEGREE
               IF(PHASE < 0.0)  PHASE = PHASE + 360.0
               DSTAR = SQRT(DSTAR)
               NUM = NUM + 1
               HKL(1,NUM) = HH
               HKL(2,NUM) = KK
               HKL(3,NUM) = -IL
               AMP(NUM)   = MODF
               PHI(NUM)   = 360.0-PHASE
               DSTR(NUM)  = DSTAR
            ENDDO
         ENDDO
      ENDDO

      DO IH = 1,HMAX+1          ! L > 0 INDICES
         HH  = IH - 1
         DO IK = 1,(2*KMAX+1)
            KK  = IK - KMAX - 1
            XSQ = (REAL(KK)*A11)**2
            DO IL = 0,LMAX
               IF(IL == 0) THEN
                  IF(HH == 0 .AND. KK > 0) CYCLE
               ENDIF
               YSQ = (REAL(KK)*A21 + REAL(IL)*A22)**2
               ZSQ = (REAL(HH)*A33 + REAL(KK)*A31 + REAL(IL)*A32)**2
               DSTAR = XSQ + YSQ + ZSQ
               IF(DSTAR > DSQMAX .OR. DSTAR < DSQMIN) CYCLE
               JL = IL + 1
               APART =   REAL(ARRAY(JL,IH,IK))
               BPART = -AIMAG(ARRAY(JL,IH,IK))
               MODF  = SQRT(APART**2+BPART**2)*SCALE
               PHASE = 0.0
               IF(MODF.NE.0.0)   PHASE = ATAN2(BPART,APART)*DEGREE
               IF(PHASE < 0.0)  PHASE = PHASE + 360.0
               DSTAR = SQRT(DSTAR)
               NUM = NUM + 1
               HKL(1,NUM) = HH
               HKL(2,NUM) = KK
               HKL(3,NUM) = IL
               AMP(NUM)   = MODF
               PHI(NUM)   = PHASE
               DSTR(NUM)  = DSTAR
            ENDDO
         ENDDO
      ENDDO

      WRITE(*,'(A,11X,A,I9)') '+',
     1     'Number of unique structure factors  =', NUM

C     Stuff the structure factors struct.
      do jj = 1,num
         stFacs(jj)%h = hkl(1,jj)
         stFacs(jj)%k = hkl(2,jj)
         stFacs(jj)%l = hkl(3,jj)
         stFacs(jj)%amp  = amp(jj)
         stFacs(jj)%phi  = phi(jj)
         stFacs(jj)%sig  = 0.0
         stFacs(jj)%dstr = dstr(jj)
      enddo

C     Stuff the new NSF
      nsf = num

      RETURN
      END


C  ************************************************************************
C  *                                                                      *
C  *                   *****     EMROBSF3DBT.FOR     *****                   *
C  *                                                                      *
C  *    UNIVERSITY OF CANTERBURY FOURIER PROGRAM                          *
C  *                                                                      *
C  *    THIS IS A MODIFIED VERSION OF FOUR1.F4 FROM CYNDY STAUFFACHER     *
C  *     WHICH RAN ON THE BRANDEIS DEC10 CAMPUS COMPUTER WHICH WAS        *
C  *     REWRITTEN FROM FORDAP (A.ZALKIN)                                 *
C  *                                                                      *
C  *    THE OUTPUT OF THIS PROGRAM IS A 3D MAP FILE IN THE I*2 FORMAT     *
C  *     COMPATIBLE FOR "EMMAP" TYPE PROGRAMS.                            *
C  *                                                                      *
C  *    UNITS: 2 = SF INPUT DATA FILE                                     *
C  *           3 = 3D I*2 MAP OUTPUT FILE                                 *
C  *                                                                      *
C  *    APRIL 1981  BRANDEIS UNIVERSITY PDP 11/40, T. BAKER               *
C  *    SEPTEMBER 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER            *
C  *    APRIL 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER                  *
C  *                                                                      *
C  *    MODIFIED  5-AUG-1998 BY TSB (Compute MAP in core memory).         *
C  *                                (Output PIF format MAP).              *
C  *    LAST MODIFIED 12-AUG-1998 BY TSB (Fast input of SF data).         *
C  ************************************************************************
C
      SUBROUTINE MAIN_EMSF (NCOL, NROW, NSEC, nsf, stFacs, map,
     1   a_cell, b_cell, c_cell, maxReso,
     2   xminP,xmaxP,yminP,ymaxP,zminP,zmaxP,dxP,dyP,dzP,norien)

      IMPLICIT NONE

      INTEGER*4 NEQUIV, NSF, DIV, NCOL, NROW, NSEC, NCD, NEF
      COMMON /CONST/ TRGTBL, XMIN, YMIN, ZMIN, XMAX, YMAX, ZMAX,
     1 DX, DY, DZ, DCELL, ITRANS, HMAX, KMAX, KMIN, RCELL, SCALE,
     2 TS, U, UINV, V
      INTEGER*4 SF_TYPE
      INTEGER*4 EM_NDIM, EM_ERR, LUN, ITRANS, HMAX, KMAX, KMIN
      INTEGER*4 NORIEN, TABLE_SIZE, NREF_DUMP, NUM_UNIQUE
      REAL*4    XMIN, YMIN, ZMIN, XMAX, YMAX, ZMAX, DX, DY, DZ
      REAL*4    XMINp, YMINp, ZMINp, XMAXp, YMAXp, ZMAXp, DXp, DYp, DZp
      REAL*4    DCELL(9), RCELL(9), SCALE, TS(3,24), U(3,3)
      REAL*4    UINV(3,3), V, TRGTBL(1001), DSTR_MAX, SF_SUMFSQ
      REAL*4    FS(3,3,24), DETU, AA(3,3), BB(3,3)
      REAL*4    A_STAR, B_STAR, C_STAR, ALP_STAR, BET_STAR, GAM_STAR
      REAL*4    maxReso
      REAL*4    A_CELL, B_CELL, C_CELL, ALP, BET, GAM, VOL, VOL_STAR
      REAL*4    MAP_MIN, MAP_MAX, SF_SCALEF, SF_DSTR_MIN, SF_DSTR_MAX
      real*4    MAP(ncol*nrow*nsec)
      CHARACTER SF_FILENAME*64
      INTEGER   alloc_stat

      include '../include/strucfac.inc'
      type(strucfac_t) stFacs(nsf)

      INTEGER*4, allocatable :: HKL(:), HH(:), KK(:), LL(:)
      REAL*4, allocatable :: AMP(:), PHI(:), SIG(:), DSTR(:)
      real*4, allocatable :: AP(:), BP(:)
      REAL*4, allocatable ::  TABLE1(:), TABLE2(:)
      REAL*4, allocatable :: SINHX(:), COSHX(:), SINKY(:), COSKY(:)

C  ----------------------------------------------------------------------------
C
      PRINT *, 'EMROBSF3DBT: Back transform 3D SFs to produce 3D MAP'
      PRINT *, '          12-Aug-98 [TSB]'
      PRINT *
C  *************************
C  *    OPEN 3D SF FILE    *
C  *************************
      EM_ERR = 1
      DO WHILE(EM_ERR.NE.0)
        SF_FILENAME = ' '

Ca      Kludge these for now ...
          alp = 0.0
          bet = 0.0
          gam = 0.0
C GCA: the routine does not read any file
          CALL SF_OPEN3_PIF_RWA (NSF, SF_TYPE, A_STAR, B_STAR,
     $         C_STAR, ALP_STAR, BET_STAR, GAM_STAR, SF_SCALEF, A_CELL,
     $         B_CELL, C_CELL, ALP, BET, GAM, VOL, VOL_STAR, EM_NDIM,
     $         EM_ERR, DSTR_MAX, maxReso)
      ENDDO

      allocate(hkl(3*nsf), amp(nsf), phi(nsf), sig(nsf), dstr(nsf),
     $     stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate hkl, amp, phi, sig, dstr'
         call tsbend
      endif

      amp  = 0.0 ! F90 array syntax
      phi  = 0.0 ! F90 array syntax
      sig  = 0.0 ! F90 array syntax
      dstr = 0.0 ! F90 array syntax
      
      CALL SF_FILL2_RWA (EM_ERR, SF_TYPE, NSF, HKL, AMP, PHI, SIG, DSTR,
     $     SF_DSTR_MIN, SF_DSTR_MAX, SF_SCALEF, SF_SUMFSQ, stFacs)
      deallocate(SIG, DSTR)     ! UNNEEDED

      IF(EM_NDIM.NE.3) THEN                        ! TRAP FOR 2D SF DATA
        PRINT *, '************************************'
        PRINT *, '* SF file does not contain 3D data *'
        PRINT *, '*           EXIT program           *'
        CALL TSBEND
      ENDIF
C  *************************************
C  *    ENTER NAME OF FINAL MAP FILE   *
C  *************************************
      WRITE(*,'(A)') '$Filename for output 3D MAP: '
      WRITE(*,'(A)') '$Title: '

      CALL ROBSF3DBT_INFO (NEQUIV, FS, AA, BB, A_CELL, B_CELL, C_CELL,
     1 ALP, BET, GAM, VOL, VOL_STAR, NREF_DUMP)
      IF(ITRANS >= 1) CALL ROBSF3DBT_TRANS (DETU, AA)
C  **************************************************************************
C  *     SET FOURIER LIMITS AND WRITE DATA FILE CONTAINING EXPANDED DATA    *
C  **************************************************************************

      allocate(HH(nsf*nequiv), KK(nsf*nequiv), LL(nsf*nequiv), 
     $     AP(nsf*nequiv), BP(nsf*nequiv), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate hh, kk, ll, ap, bp'
         call tsbend
      endif

      ap = 0.0 ! F90 array syntax
      bp = 0.0 ! F90 array syntax

      CALL ROBSF3DBT_RIP (NCOL, NROW, NSEC, NEQUIV, NORIEN, NCD, NEF,
     $     FS, HKL, AMP, PHI, NSF, AP, BP, HH, KK, LL, SF_TYPE,
     $     NREF_DUMP,xminP, xmaxP, yminP, ymaxP, zminP, zmaxP, dxP, dyP,
     $     dzP, NUM_UNIQUE)
C
      deallocate(HKL, AMP, PHI)
      
      map = 0.0 ! F90 array syntax

      allocate(table2(nef), sinhx(ncol*(hmax+1)), coshx(ncol*(hmax+1)),
     $     sinky(nrow*(kmax-kmin+1)), cosky(nrow*(kmax-kmin+1)),
     $     stat=alloc_stat)

      table2 = 0.0 ! F90 array syntax
      sinhx  = 0.0 ! F90 array syntax
      coshx  = 0.0 ! F90 array syntax
      sinky  = 0.0 ! F90 array syntax
      cosky  = 0.0 ! F90 array syntax

      DIV        = 1
      TABLE_SIZE = NSEC*NCD/DIV
      
      allocate(table1(table_size), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) '***************************************'
         write(*,*) '* Insufficient CORE memory for TABLES *'
         write(*,*) '*            EXIT PROGRAM             *'
         CALL TSBEND
      ENDIF

      table1 = 0.0 ! F90 array syntax

C  ***********************************************
C  *    CALCULATE FOURIER, SECTION BY SECTION    *
C  ***********************************************
      CALL ROBSF3DBT_FOURIER (MAP, TABLE_SIZE, NCD, NEF, TABLE1, TABLE2, 
     1 NCOL, NROW, NSEC, AP, BP, HH, KK, LL, SINHX, COSHX, SINKY, 
     2 COSKY, MAP_MIN, MAP_MAX, NUM_UNIQUE, NEQUIV)


      deallocate(sinhx, coshx, sinky, cosky, table1, table2)

      if(allocated(hh))  deallocate(hh)
      if(allocated(kk))  deallocate(kk)
      if(allocated(ll))  deallocate(ll)
      if(allocated(ap))  deallocate(ap)
      if(allocated(bp) ) deallocate(bp)

      END


C  *******************************************************************
C  *                                                                 *
C  *          *****     SUBROUTINE ROBSF3DBT_FOURIER     *****          *
C  *                                                                 *
C  *    FOURIER SUMMATION ROUTINE                                    *
C  *                                                                 *
C  *    THIS IS A MAJOR REVISION OF THE SUBROUTINE "SIG100"          *
C  *     OF THE PROGRAM FOUR3D.                                      *
C  *                                                                 *
C  *    LAST MODIFIED  5-AUG-1998 BY TSB (Store MAP in 3D array and  *
C  *     eliminate need for EMROBSF3DBT.SCR scratch file).              *
C  *******************************************************************

      SUBROUTINE ROBSF3DBT_FOURIER (MAP, TABLE_SIZE, NCD, NEF, TABLE1,
     1  TABLE2, NCOL, NROW, NSEC, AP, BP, HH, KK, LL, SINHX, COSHX, 
     2  SINKY, COSKY, MAP_MIN, MAP_MAX, NSF, NEQUIV)

      IMPLICIT NONE

      INTEGER*4 TABLE_SIZE, NCOL, NROW, NSEC, NEF, NSF, NEQUIV
      COMMON /CONST/ TRIG_TABLE, XMIN, YMIN, ZMIN, XMAX, YMAX, ZMAX,
     1 DX, DY, DZ, DCELL, ITRANS, HMAX, KMAX, KMIN, RCELL, SCALE,
     2 TS, U, UINV
      INTEGER*4 I, J, K, M, N, CE, CF, LOCEI, CC, CD
      INTEGER*4 HMAX, KMAX, KMIN, NH, NK, ITRANS, NCD, ISEC
      INTEGER*4 HH(NSF*NEQUIV), KK(NSF*NEQUIV), LL(NSF*NEQUIV)
      INTEGER*2 SECTION_THIS_LOOP, NSECS_THIS_LOOP
      REAL*4    TABLE1(TABLE_SIZE), TABLE2(NEF), TRIG_TABLE(1001)
      REAL*4    SINKY(NROW*(KMAX-KMIN+1)), COSKY(NROW*(KMAX-KMIN+1))
      REAL*4    SINHX(NCOL*(HMAX+1)), COSHX(NCOL*(HMAX+1))
      REAL*4    UINV(3,3), MAP_MIN, MAP_MAX
      REAL*4    XMIN, YMIN, ZMIN, XMAX, YMAX, ZMAX, DX, DY, DZ
      REAL*4    DCELL(9), RCELL(9), SCALE, TS(3,24), U(3,3)
      REAL*4    AP(NSF*NEQUIV), BP(NSF*NEQUIV), MAP(NCOL,NROW,NSEC)
      REAL*4    ARG, SINARG, COSARG, X, Y, Z, DEL_ARG

      MAP_MIN         = HUGE(MAP_MIN)
      MAP_MAX         = -HUGE(MAP_MAX)
      ISEC            = 0
      NH              = HMAX + 1            ! # H INDICES
      NK              = KMAX - KMIN + 1      ! # OF K INDICES
      NSECS_THIS_LOOP = TABLE_SIZE/NCD      ! # SECTIONS HANDLED EACH LOOP
      Z               = ZMIN
      PRINT *, ' '
      WRITE(*,'(A)') '+   Set up SINHX,COSHX tables'
C  ***********************************
C  *    SET UP SINHX,COSHX TABLES    *
C  ***********************************
      I       = 1
      DEL_ARG = 0.0
      X       = 0.0
      DO N = 1,NH
        ARG  = X
        DO M = 1,NCOL
          CALL SF3DBT_GETARGS (ARG, SINARG, COSARG, TRIG_TABLE)
          SINHX(I) = SINARG
          COSHX(I) = COSARG
          I        = I + 1
          ARG      = ARG + DEL_ARG
        ENDDO
        DEL_ARG = DEL_ARG + DX
        X       = X + XMIN
      ENDDO
      WRITE(*,'(A)') '+   Set up SINHX,COSHX tables'
C  ***********************************
C  *    SET UP SINKY,COSKY TABLES    *
C  ***********************************
      I       = 1
      Y       = REAL(KMIN)*YMIN
      DEL_ARG = REAL(KMIN)*DY
      DO K = 1,NK
        ARG  = Y
        DO N = 1,NROW
          CALL SF3DBT_GETARGS (ARG, SINARG, COSARG, TRIG_TABLE)
          SINKY(I) = SINARG
          COSKY(I) = COSARG
          I        = I + 1
          ARG      = ARG + DEL_ARG
        ENDDO
        Y       = Y + YMIN
        DEL_ARG = DEL_ARG + DY
      ENDDO

 100  IF(NSECS_THIS_LOOP > NSEC-ISEC) NSECS_THIS_LOOP = NSEC - ISEC
C  **************************
C  *    INITIALIZE TABLE    *
C  **************************
      WRITE(*,'(A)') '+   Set up TABLE1            '
      DO I = 1,TABLE_SIZE
        TABLE1(I) = 0.0
      ENDDO
C  **********************
C  *    SET UP TABLE    *
C  **********************
      DO I = 1,NSF
        CC      = HH(I)*NK + (KK(I) - KMIN) + 1
        CD      = CC + NCD/2
        ARG     = LL(I)*Z
        DEL_ARG = LL(I)*DZ
        DO J = 1,NSECS_THIS_LOOP
          CALL SF3DBT_GETARGS (ARG, SINARG, COSARG, TRIG_TABLE)

Ca          if(cc > table_size) cc = table_size
Ca          if(cd > table_size) cd = table_size

          TABLE1(CC) = TABLE1(CC) + AP(I)*COSARG + BP(I)*SINARG
          TABLE1(CD) = TABLE1(CD) - AP(I)*SINARG + BP(I)*COSARG
          CC         = CC + NCD
          CD         = CC + NCD/2
          ARG        = ARG + DEL_ARG
        ENDDO
      ENDDO

      CC = 1
      CD = CC + NCD/2
      DO SECTION_THIS_LOOP = 1,NSECS_THIS_LOOP
        ISEC = ISEC + 1
        WRITE(*,'(A,I4)') '+ ', ISEC
        DO J = 1,NEF
          TABLE2(J) = 0.0
        ENDDO
        LOCEI = 1
        DO J = 1,NH
          I = 1
          DO K = 1,NK
            CE = LOCEI
            CF = CE + NEF/2
            DO N = 1,NROW
              TABLE2(CE) = TABLE2(CE) + TABLE1(CC)*COSKY(I)
     1                               + TABLE1(CD)*SINKY(I)
              TABLE2(CF) = TABLE2(CF) - TABLE1(CC)*SINKY(I)
     1                               + TABLE1(CD)*COSKY(I)
              CE = CE + HMAX + 1
              CF = CE + NEF/2
              I  = I + 1
            ENDDO
            CC    = CC + 1
            CD    = CC + NCD/2
          ENDDO
          LOCEI = LOCEI + 1
        ENDDO

        CE = 1
        CF = NEF/2 + 1
        DO J = 1,NROW
          DO K = 1,NCOL                              ! ZERO ROW OF MAP
            MAP(K,J,ISEC) = 0.0
          ENDDO
          I = 1
          DO N = 1,NH
            DO M = 1,NCOL
              MAP(M,J,ISEC) = MAP(M,J,ISEC) + TABLE2(CE)*COSHX(I)
     1                                     + TABLE2(CF)*SINHX(I)
              I      = I + 1
            ENDDO
            CE = CE + 1
            CF = CE + NEF/2
          ENDDO
          DO M = 1,NCOL
            MAP_MIN = AMIN1(MAP_MIN,MAP(M,J,ISEC))
            MAP_MAX = AMAX1(MAP_MAX,MAP(M,J,ISEC))
          ENDDO
        ENDDO
        CC   = CC + NCD/2
        CD   = CC + NCD/2
        Z    = Z + DZ
      ENDDO                       ! DO SECTION_THIS_LOOP = 1,NSECS_THIS_LOOP
      IF(ISEC < NSEC) GO TO 100

      RETURN
      END


C
C  **************************************************************************
C  *                                                                        *
C  *                *****     SUBROUTINE ROBSF3DBT_RIP     *****               *
C  *                                                                        *
C  *    ROUTINE CALLED BY "EMROBSF3DBT" PROGRAM                                *
C  *                                                                        *
C  *    LAST MODIFIED  5-AUG-1998 BY TSB (Eliminate need for EMROBSF3DBT.TMP   *
C  *     scratch file for storing intermediate SF data).                    *
C  **************************************************************************
C
      SUBROUTINE ROBSF3DBT_RIP (NCOL, NROW, NSEC, NEQUIV, NORIEN, NCD,
     1 NEF, FS, HKL, AMP, PHI, NSF, AP, BP, HH, KK, LL, SF_TYPE,
     2 NREF_DUMP,xminP,xmaxP,yminP,ymaxP,zminP,zmaxP,dxP,dyP,dzP,
     2 NUM_UNIQUE)

      IMPLICIT NONE

      INTEGER*4 NEQUIV, NSF
      COMMON /CONST/ TRIG_TABLE, XMIN, YMIN, ZMIN, XMAX, YMAX, ZMAX,
     1 DX, DY, DZ, DCELL, ITRANS, HMAX, KMAX, KMIN, RCELL, SCALE,
     2 TS, U, UINV
      REAL*4    XMIN, YMIN, ZMIN, XMAX, YMAX, ZMAX, DX, DY, DZ
      REAL*4    XMINp, YMINp, ZMINp, XMAXp, YMAXp, ZMAXp
      REAL*4    DXp, DYp, DZp
      REAL*4    DCELL(9), RCELL(9), SCALE, TS(3,24), U(3,3)
      REAL*4    UINV(3,3), FS(3,3,24)
      REAL*4    TRIG_TABLE(1001), XSIN(16)
      REAL*4    AMP(NSF), PHI(NSF), HKLTMP(3)
      REAL*4    AP(NSF*NEQUIV), BP(NSF*NEQUIV)
      REAL*4    FJX, FKX, FLX
      REAL*4    APART, BPART, APARTS(24), BPARTS(24)
      real*8    deg_to_rad
      INTEGER*4 M1(24,3), M2(24,3), M3(24,3), SF_TYPE, NUM_UNIQUE
      INTEGER*4 NEXPAN, NTOTAL, NU, I, M, N
      INTEGER*4 BUFFER, IFIXR, NUCALC, MTX(24), MTY(24), MTZ(24)
      INTEGER*4 NCOL, NROW, NSEC, ITRANS, HMAX, KMAX, KMIN
      INTEGER*4 NN, NREF_DUMP, NORIEN, NCD, NEF
      INTEGER*4 HH(NSF*NEQUIV), KK(NSF*NEQUIV), LL(NSF*NEQUIV)
      INTEGER*4 J(24), K(24), L(24), HKL(3,NSF)
      DATA XSIN/0.,.5,.866025,1.,.866025,.5,0.,-.5,-.866025,-1.,
     1  -.866025,-.5,0.,.5,.866025,1./

      character(len=1) XL, YL, ZL
      character(len=2) AXIS(3)
      DATA AXIS/' Z',' Y',' X'/,   XL/'X'/,YL/'Y'/,ZL/'Z'/

      deg_to_rad = acos(-1.0d0)/180.0d0

      NEXPAN = 0
      NTOTAL = 0
      BUFFER = 0
      HMAX   = 0
      KMAX   = 0
      KMIN   = 0
      IF(ITRANS <= 1) THEN
         PRINT *, '****************************************************'
         PRINT *, '* INPUT PARAMETER DEFAULTS:                        *'
         PRINT *, '* Portion of cell desired XMIN,XMAX = 0.0,1.0      *'
         PRINT *, '*                         YMIN,YMAX = 0.0,1.0      *'
         PRINT *, '*                         ZMIN,ZMAX = 0.0,1.0      *'
         PRINT *, '* Pixel sizes              DX,DY,DZ = 1.0,1.0,1.0  *'
         PRINT *, '*  If no values entered for DY,DZ, default to DX   *'
         PRINT *, '* Orientation                NORIEN =   1 (Z-secs) *'
         PRINT *, '*  (Default is Z-sections)   NORIEN =  10 (Y-secs) *'
         PRINT *, '*                            NORIEN = 100 (X-secs) *'
         PRINT *, '****************************************************'
         WRITE(*,'(A)') '$XMIN,XMAX,YMIN,YMAX,ZMIN,ZMAX: '
         xmin = xminP
         xmax = xmaxP
         ymin = yminP
         ymax = ymaxP
         zmin = zminP
         zmax = zmaxP
         IF(XMAX == 0.0) XMAX = 1.0
         IF(YMAX == 0.0) YMAX = 1.0
         IF(ZMAX == 0.0) ZMAX = 1.0
         WRITE(*,'(A)') '$DX,DY,DZ (<CR>=1.0,1.0,1.0): '
         dx = dxP
         dy = dyP
         dz = dzP
         IF(DX <= 0.0) DX = 1.0
         IF(DY <= 0.0) DY = DX
         IF(DZ <= 0.0) DZ = DX
         WRITE(*,'(A)') '$NORIEN (<CR>=1): '
         IF(NORIEN <= 0 .OR. NORIEN > 100) NORIEN = 1
         DX = DX/DCELL(1)
         DY = DY/DCELL(2)
         DZ = DZ/DCELL(3)
      ELSE
         NORIEN = 100
      ENDIF
      CALL SF3DBT_CHEKUP (XMIN, XMAX, NORIEN, 100, DX)
      CALL SF3DBT_CHEKUP (YMIN, YMAX, NORIEN,  10, DY)
      CALL SF3DBT_CHEKUP (ZMIN, ZMAX, NORIEN,   1, DZ)
      FJX = DX*DCELL(1)
      FKX = DY*DCELL(2)
      FLX = DZ*DCELL(3)
      I   = ALOG10(REAL(NORIEN)+.1)+1.
      WRITE(*,1010) AXIS(I)
      WRITE(*,1020) XL, XMIN, XMAX, DX, FJX, YL, YMIN, YMAX, DY, FKX,
     1  ZL, ZMIN, ZMAX, DZ, FLX
      IF(NORIEN < 10) THEN                         ! Z SECTIONS
C         CALL SF3DBT_ROT (XMIN,YMIN)                  ! SWITCH X & Y
C         CALL SF3DBT_ROT (XMAX,YMAX)
C         CALL SF3DBT_ROT (DX,DY)
       ELSE IF(NORIEN == 10) THEN                  ! Y SECTIONS
         CALL SF3DBT_ROT (YMIN,ZMIN)
         CALL SF3DBT_ROT (YMAX,ZMAX)
         CALL SF3DBT_ROT (DY,DZ)
       ELSE IF(NORIEN > 10) THEN                  ! X SECTIONS
         CALL SF3DBT_ROT (XMIN,ZMIN)
         CALL SF3DBT_ROT (XMAX,ZMAX)
         CALL SF3DBT_ROT (DX,DZ)
      ENDIF
      DO I = 1,NEQUIV
        DO M = 1,3
          M1(I,M) = FS(1,M,I)
          M2(I,M) = FS(2,M,I)
          M3(I,M) = FS(3,M,I)
        ENDDO
        IF(M1(I,1)+M2(I,1) == 0) THEN
          IF(M1(I,1))         290,320,290
        ENDIF
        IF(M1(I,2)+M2(I,2)) 320,280,320
  280        IF(M1(I,2))         290,320,290
  290        IF(M1(I,1)+M2(I,2)) 310,300,310
  300        CALL SF3DBT_ROT_I4 (M1(I,2),M2(I,1))
        GO TO 320
  310        CALL SF3DBT_ROT_I4 (M1(I,1),M2(I,2))
  320        MTX(I) = NUCALC(IFIXR(12.*TS(1,I)))
        MTY(I) = NUCALC(IFIXR(12.*TS(2,I)))
        MTZ(I) = NUCALC(IFIXR(12.*TS(3,I)))
      ENDDO
      NCOL = (XMAX-XMIN)/DX+1.5
      NROW = (YMAX-YMIN)/DY+1.5
      NSEC = (ZMAX-ZMIN)/DZ+1.5
      WRITE(*,'(3(A,I6))') ' MAP dimensions:', NCOL, ' by', NROW, 
     1 ' by', NSEC
      IF(NREF_DUMP > 0) WRITE(6,1040)
      IF(ITRANS >= 1) WRITE(6,1050)
C  *****************************************************************
C  *    CHANGES MADE TO READ HKL, AMP AND PHASE AND CONVERT TO     *
C  *      APARTS AND BPARTS   3-SEP-84  TSB                        *
C  *****************************************************************
      NN = 0
  400      NN = NN+1
      IF(NN > NSF) GO TO 900
      IF(SF_TYPE == 1) THEN
         APART = AMP(NN)*COS(deg_to_rad*PHI(NN))
         BPART = AMP(NN)*SIN(deg_to_rad*PHI(NN))
       ELSE
         APART = AMP(NN)
         BPART = PHI(NN)
      ENDIF
      IF(APART == 0.0.AND.BPART == 0.0) GO TO 400
      HKLTMP(1) = HKL(1,NN)
      HKLTMP(2) = HKL(2,NN)
      HKLTMP(3) = HKL(3,NN)
      NTOTAL    = NTOTAL + 1

      DO N = 1,NEQUIV
        J(N) = M1(N,1)*HKL(1,NN)+M2(N,1)*HKL(2,NN)+M3(N,1)*HKL(3,NN)
        K(N) = M1(N,2)*HKL(1,NN)+M2(N,2)*HKL(2,NN)+M3(N,2)*HKL(3,NN)
        L(N) = M1(N,3)*HKL(1,NN)+M2(N,3)*HKL(2,NN)+M3(N,3)*HKL(3,NN)
        NU   = NUCALC(J(N)*MTX(N)+K(N)*MTY(N)+L(N)*MTZ(N)) + 1
        APARTS(N) = APART*XSIN(NU + 3) - BPART*XSIN(NU)
        BPARTS(N) = BPART*XSIN(NU + 3) + APART*XSIN(NU)
        IF(ITRANS >= 1) THEN
          FJX  = J(N)
          FKX  = K(N)
          FLX  = L(N)
          J(N) = IFIXR( U(1,1)*FJX + U(1,2)*FKX + U(1,3)*FLX )
          K(N) = IFIXR( U(2,1)*FJX + U(2,2)*FKX + U(2,3)*FLX )
          L(N) = IFIXR( U(3,1)*FJX + U(3,2)*FKX + U(3,3)*FLX )
        ENDIF
        IF(NORIEN < 10) THEN    ! Z SECTIONS
        ELSE IF(NORIEN == 10) THEN ! Y SECTIONS
           CALL SF3DBT_ROT_I2 (K(N),L(N))
        ELSE IF(NORIEN > 10) THEN ! X SECTIONS
           CALL SF3DBT_ROT_I2 (J(N),L(N))
        ENDIF
        IF(J(N))680,650,710
 650    IF(K(N))690,660,710
 660    IF(L(N))700,670,710
 670    APARTS(N) = .5*APARTS(N)
        BPARTS(N) = .5*BPARTS(N)
        GO TO 710
 680    J(N) = -J(N)
 690    K(N) = -K(N)
 700    L(N) = -L(N)
        BPARTS(N) = -BPARTS(N)
  710        IF(APARTS(N).NE.0.0.OR.BPARTS(N).NE.0.0) THEN
          IF(J(N) > HMAX) HMAX = J(N)
          IF(K(N) > KMAX) KMAX = K(N)
          IF(K(N) < KMIN) KMIN = K(N)
        ENDIF
      ENDDO
      DO 860 I = 1,NEQUIV
        IF(APARTS(I) == 0.0.AND.BPARTS(I) == 0.0) GO TO 860
        M = I+1
        IF(M <= NEQUIV) THEN
          DO N = M,NEQUIV
           IF(APARTS(N) == 0.0) THEN
            IF(BPARTS(N).NE.0.0) THEN
             IF(J(I) == J(N).AND.K(I) == K(N).AND.L(I) == L(N)) THEN
              APARTS(N) = 0.0
              BPARTS(N) = 0.0
             ENDIF
            ENDIF
           ENDIF
          ENDDO
        ENDIF
        NEXPAN     = NEXPAN + 1
        BUFFER     = BUFFER + 1
        HH(BUFFER) = J(I)
        KK(BUFFER) = K(I)
        LL(BUFFER) = L(I)
        AP(BUFFER) = APARTS(I)
        BP(BUFFER) = BPARTS(I)
        IF(NREF_DUMP > 0) THEN
          IF(NORIEN < 10) THEN
             WRITE(*,2020) J(I),K(I),L(I),APARTS(I),BPARTS(I)
           ELSE IF(NORIEN == 10) THEN
             WRITE(*,2020) J(I),L(I),K(I),APARTS(I),BPARTS(I)
           ELSE IF(NORIEN > 10) THEN
             WRITE(*,2020) L(I),K(I),J(I),APARTS(I),BPARTS(I)
          ENDIF
        ENDIF
  860      CONTINUE
      IF(NREF_DUMP > 0) NREF_DUMP = NREF_DUMP - 1
      GO TO 400

  900      NCD = (HMAX+1)*(KMAX-KMIN+1)*2
      NEF = (HMAX+1)*NROW*2
      IF(NTOTAL <= 0.OR.NEXPAN <= 0) THEN
        PRINT *, '***********************************************'
        PRINT *, '* No data available for this summation...EXIT *'
        PRINT *, '* NTOTAL,NEXPAN =', NTOTAL, NEXPAN
        CALL TSBEND
      ENDIF
      WRITE(*,1060) NTOTAL, NEXPAN
      WRITE(*,1070) NCD, NEF
      NUM_UNIQUE = NEXPAN
      RETURN

 1010      FORMAT(' Compute sections along the',A2,' axis')
 1020      FORMAT(' ',A2,'Limits: ',F8.5,' to',F8.5,' in steps of',
     1  F8.5,' (',F6.3,' nms)')
 1040      FORMAT('   H   K   L           A          B',/)
 1050      FORMAT(20X,'(The indices are transformed to the new cell)')
 1060      FORMAT(' The',I8,' independent reflections have been',
     1  ' expanded to',I8,' terms')
 1070      FORMAT(' The CD table requires',I6,' memory locations'/,
     1  ' The EF table requires',I6,' memory locations'/)
 2020      FORMAT(3I4,2F10.2)
      END
C
C  *************************************************************************
C  *                                                                       *
C  *               *****     SUBROUTINE ROBSF3DBT_INFO     *****              *
C  *                                                                       *
C  *   READ CONTROL PARAMETERS, CELL CONSTANTS, AND SYMMETRY CARDS         *
C  *   CALCULATE COSINE TABLE                                              *
C  *                                                                       *
C  *    LAST MODIFIED  5-AUG-1998 BY TSB (Rid of control parameter file).  *
C  *************************************************************************
C
      SUBROUTINE ROBSF3DBT_INFO (NEQUIV, FS, AA, BB, A_CELL, B_CELL, 
     1 C_CELL, ALP, BET, GAM, VOL, VOL_STAR, NREF_DUMP)

      IMPLICIT NONE

      COMMON /CONST/ TRIG_TABLE, XMIN, YMIN, ZMIN, XMAX, YMAX, ZMAX,
     1 DX, DY, DZ, DCELL, ITRANS, HMAX, KMAX, KMIN, RCELL, SCALE,
     2 TS, U, UINV, V
      INTEGER*4 NEQUIV, I, M, N, ITRANS, HMAX, KMAX, KMIN
      INTEGER*4 NREF_DUMP
      REAL*4    XMIN, YMIN, ZMIN, XMAX, YMAX, ZMAX, DX, DY, DZ, V
      REAL*4    DCELL(9), RCELL(9), TS(3,24), U(3,3), UINV(3,3), SCALE
      REAL*4    TRIG_TABLE(1001), FS(3,3,24), AA(3,3), BB(3,3)
      REAL*4    A_CELL, B_CELL, C_CELL, ALP, BET, GAM, VOL, VOL_STAR
      REAL*8    DPI, DRADIAN, DDEGREE, ARG, DEL_ARG

      CALL PIRADDEG_D (DPI, DRADIAN, DDEGREE)
      DEL_ARG = 2.0*DPI/1000.0
      ARG     = 0.D0

      DO I = 1,1001
        TRIG_TABLE(I) = COS(SNGL(ARG))
        ARG           = ARG + DEL_ARG
      ENDDO

      WRITE(*,'(A)') '$ITRANS, NEQUIV, SCALE <CR>=0,1,1.0'
      itrans = 0
      nequiv = 0.0
      scale = 0.0
      IF(ITRANS <= 0 .OR. ITRANS > 2) ITRANS = 0
      IF(NEQUIV <= 1 .OR. NEQUIV > 24) NEQUIV = 1
       IF(SCALE == 0.0) SCALE = 1.0
      DCELL(1) = A_CELL
      DCELL(2) = B_CELL
      DCELL(3) = C_CELL
      DCELL(4) = ALP
      DCELL(5) = BET
      DCELL(6) = GAM
      IF(DCELL(4) >= 1.0) THEN
        DO I = 4,6
          DCELL(I) = COS(DCELL(I)/DDEGREE)
        ENDDO
      ENDIF
      CALL SF3DBT_STOAA (DCELL, AA)
      CALL SF3DBT_MI3 (AA, BB, V)
      V = SQRT(V)
      RCELL(1) = SQRT(BB(1,1))
      RCELL(2) = SQRT(BB(2,2))      
      RCELL(3) = SQRT(BB(3,3))
      RCELL(4) = BB(2,3)/(RCELL(2)*RCELL(3))
      RCELL(5) = BB(1,3)/(RCELL(1)*RCELL(3))
      RCELL(6) = BB(1,2)/(RCELL(1)*RCELL(2))
      DO I = 4,6
        DCELL(I+3) = ATAN2(SQRT(1.-DCELL(I)*DCELL(I)),DCELL(I))*DDEGREE
        RCELL(I+3) = ATAN2(SQRT(1.-RCELL(I)*RCELL(I)),RCELL(I))*DDEGREE
      ENDDO
      IF(DCELL(1) < 1.0) THEN
        V = 1./V
        DO I = 1,9
          CALL SF3DBT_ROT (DCELL(I), RCELL(I))
          CALL SF3DBT_ROT ( AA(I,1),  BB(I,1))
        ENDDO
      ENDIF
      WRITE(*,'(A,/,A)') '(TS(J),(FS(I,J),I=1,3),J=1,3)',
     1 '$Defaults = 0.,1.,0.,0.,  0.,0.,1.,0.,  0.,0.,0.,1.: '
      IF(NEQUIV == 1) THEN
        FS(1,1,1) = 1.0
        FS(2,2,1) = 1.0
        FS(3,3,1) = 1.0
      ENDIF
      WRITE(*,'(A)') '$NREF_DUMP <CR>=0: '
      nref_dump = 0

      WRITE(*,'(A,I2,A,I2,A,F5.1)') ' ITRANS =', ITRANS, 
     1 '  NEQUIV =', NEQUIV, '  SCALE =', SCALE
      PRINT *, 'V,VOL,VOL_STAR', V,VOL,VOL_STAR
      WRITE(*,1010) DCELL
      WRITE(*,1020) RCELL
      WRITE(*,1030) NEQUIV
      WRITE(*,'(3(3X,F7.4,3F4.0))') ((TS(M,N),(FS(I,M,N),I=1,3),
     1                                M=1,3),N=1,NEQUIV)
      IF(ITRANS.NE.0) THEN
        PRINT *, 'Transform unit cell before Fourier summation'
      ENDIF
      RETURN

 1010      FORMAT(/,'     A       B       C     COS ALP  COS BET  ',
     1  'COS GAM   ALP    BET    GAM',/' ',3F8.2,3F9.5,3F7.2)
 1020      FORMAT(/,'     A*      B*      C*    COS ALP* COS BET* ',
     1   'COS GAM*   ALP*   BET*   GAM*',/  ' ',
     2   3F8.5,3F9.5,1X,3F7.2)
 1030      FORMAT(/' Number of equivalent reflections (HKL) is ',I3,//,
     1 '     TRANSFORMED X         TRANSFORMED Y',
     2 '         TRANSFORMED Z',/)
      END
C
C  ****************************************************************************
C  *                                                                          *
C  *              *****     SUBROUTINE ROBSF3DBT_TRANS     *****                 *
C  *                                                                          *
C  *    SECTIONS CALCULATED PARALLEL TO A GENERAL PLANE                       *
C  *     THE U MATRIX OR THE EQUATION FOR THE GENERAL PLANE IS SUPPLIED       *
C  *                                                                          *
C  *    ROUTINE CALLED BY "EMROBSF3DBT" PROGRAM                                  *
C  ****************************************************************************
C
      SUBROUTINE ROBSF3DBT_TRANS (DETU, AA)

      IMPLICIT NONE

      COMMON /CONST/ TRIG_TABLE, XMIN, YMIN, ZMIN, XMAX, YMAX,
     1 ZMAX, DX, DY, DZ, DCELL, ITRANS, HMAX, KMAX, KMIN, RCELL,
     2 SCALE, TS, U, UINV
      REAL*4    XMIN, YMIN, ZMIN, XMAX, YMAX, ZMAX, DX, DY, DZ
      REAL*4    DCELL(9), RCELL(9), SCALE, TS(3,24), U(3,3)
      REAL*4    DETU, AA(3,3), TRIG_TABLE(1001)
      REAL*4    DPLANE, PLANE(3), DDCELL, CELL(3), GNEW(3,3)
      REAL*4    XYZMIN(3), XYZMAX(3), DXYZ(3), UINV(3,3)
      INTEGER*4 ITRANS, HMAX, KMAX, KMIN, I, J, K, KK
      EQUIVALENCE  (XYZMIN,XMIN), (XYZMAX,XMAX), (DXYZ,DX)

      IF(ITRANS == 1) THEN
         WRITE(*,1000)
         DO I = 1,3
           DO J = 1,3
             GNEW(I,J) = 0.0
             DO K = 1,3
               DO KK = 1,3
                 GNEW(I,J) = GNEW(I,J) + U(I,K)*U(J,KK)*AA(K,KK)
               ENDDO
             ENDDO
           ENDDO
         ENDDO
         DCELL(1) = SQRT(GNEW(1,1))
         DCELL(2) = SQRT(GNEW(2,2))
         DCELL(3) = SQRT(GNEW(3,3))
         DCELL(6) = GNEW(1,2)/(DCELL(1)*DCELL(2))
         DCELL(5) = GNEW(1,3)/(DCELL(1)*DCELL(3))
         DCELL(4) = GNEW(2,3)/(DCELL(2)*DCELL(3))
       ELSE
         WRITE(*,1010) PLANE, DPLANE
         CALL SF3DBT_GP1 (PLANE, DPLANE, AA, U)
         DO I = 1,3
           WRITE(*,1020) I, XYZMIN(I), XYZMAX(I), DXYZ(I)
           DDCELL     = AMAX1(2.*ABS(XYZMIN(I)),2.*ABS(XYZMAX(I)),
     1                       10.,CELL(I))
           U(I,1)     = U(I,1)*DDCELL
           U(I,2)     = U(I,2)*DDCELL
           U(I,3)     = U(I,3)*DDCELL
           XYZMIN(I)  = XYZMIN(I)/DDCELL
           XYZMAX(I)  = XYZMAX(I)/DDCELL
           DXYZ(I)    = DXYZ(I)/DDCELL
           DCELL(I)   = DDCELL
           DCELL(I+3) = 0.0
         ENDDO
      ENDIF
      WRITE(*,1030) (DCELL(I),I=1,6)
      WRITE(*,1040) (I,(U(I,J),J=1,3),I=1,3)
      CALL SF3DBT_MI3 (U, UINV, DETU)
      WRITE(*,1050) DETU
      WRITE(*,1060) ((UINV(I,J),J=1,3),I=1,3)
      RETURN

 1000 FORMAT(' Transformation matrix read from control data file')
 1010 FORMAT(' Transformation matrix defined by the plane',/,11X,
     1     F10.5,' X +',F10.5,' Y +',F10.5,' Z =',F10.5)
 1020 FORMAT(' New axis',I2,' runs from',F10.5,' to',F10.5,
     1     ' in steps of',F10.5,' Angstroms')
 1030 FORMAT(' New cell constants are',3F12.3,3F12.5)
 1040 FORMAT(11X,'NEWA',I1,' =',F12.6,' A1 +',F12.6,' A2 +',F12.6,
     1     ' A3')
 1050 FORMAT(' The determinant of the U matrix is',F15.6)
 1060 FORMAT(' The matrix inverse to U is as follows',/,(11X,3F15.6))
      END


C  ******************************************************************
C  *                                                                *
C  *            *****     SUBROUTINE SF_OPEN3_PIF_rwa *****             *
C  *                                                                *
C  *    OPENS PIF SF FILE                                           *
C  *                                                                *
C  *    September 1998  PURDUE UNIVERSITY, RW Ashmore.              *
C  ******************************************************************
C
      SUBROUTINE SF_OPEN3_PIF_RWA (NSF, SF_TYPE, A_STAR, B_STAR, 
     $     C_STAR, ALP_STAR, BET_STAR, GAM_STAR, SF_SCALEF, A_CELL, 
     $     B_CELL, C_CELL, ALP, BET, GAM, VOL, VOL_STAR, EM_NDIM, 
     $     EM_ERR, DSTR_MAX, maxReso)

      IMPLICIT NONE

      include '../include/pifhead.inc'
      TYPE(PIFHEAD_T)  MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD
      COMMON /PIFHEAD/ MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD

      INTEGER*4 NSF, SF_TYPE
      INTEGER*4 EM_NDIM, EM_ERR
      REAL*4    A_STAR, B_STAR, C_STAR, ALP_STAR, BET_STAR, GAM_STAR
      REAL*4    SF_SCALEF
      REAL*4    A_CELL, B_CELL, C_CELL, ALP, BET, GAM, VOL, VOL_STAR
      REAL*4    DSTR_MAX, maxReso

      sf_type = 1
      em_ndim = 3

      a_star  = a_cell
      b_star  = b_cell
      c_star  = c_cell
      alp_star     = alp
      bet_star     = bet
      gam_star     = gam
      CALL REC_TO_REAL (A_CELL,B_CELL,C_CELL,ALP,BET,GAM,VOL,
     1 A_STAR,B_STAR,C_STAR,VOL_STAR,ALP_STAR,BET_STAR,GAM_STAR)
      sf_scalef = 1.0
      IF(SF_SCALEF == 0.0) SF_SCALEF = 1.0

C  ************************************************
C  *    READ THE STRUCTURE FACTOR DATA FILE TO    *
C  *     DETERMINE ARRAY SIZE REQUIREMENTS        *
C  ************************************************
      WRITE(*,'(A)') '$MAX resolution (pixels/Ang/nm) (F): '
        dstr_max = maxReso
      IF(DSTR_MAX > 0.0) THEN
         DSTR_MAX = 1.0/DSTR_MAX
       ELSE
         DSTR_MAX = HUGE(DSTR_MAX)
         RETURN
      ENDIF

      PRINT '(8X,A)', ' structure factors'

      PRINT '(A,I7)', '+', NSF
      EM_ERR = 0
      RETURN

      END

C  ********************************************************************
C  *                                                                  *
C  *               *****     SUBROUTINE SF_FILL2_rwa *****            *
C  *                                                                  *
C  *    READ SF DATA FROM UNIT LUN INTO ARRAYS.                       *
C  *                                                                  *
C  *    September 1998  PURDUE UNIVERSITY, RW Ashmore                 *
C  ********************************************************************

      SUBROUTINE SF_FILL2_RWA (EM_ERR, SF_TYPE, NSF, HKL, AMP, PHI, SIG,
     $     DSTR, SF_DSTR_MIN, SF_DSTR_MAX, SF_SCALEF, SF_SUMFSQ, stFacs)

      IMPLICIT NONE

      INTEGER*4 EM_ERR, NSF, I, SF_TYPE
      INTEGER*4 HKL(3,NSF)
      REAL*4    AMP(NSF), PHI(NSF), SIG(NSF), DSTR(NSF)
      REAL*4    SF_DSTR_MIN, SF_DSTR_MAX, SF_SUMFSQ, SF_SCALEF

      include '../include/pifhead.inc'
      TYPE(PIFHEAD_T)  MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD
      COMMON /PIFHEAD/ MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD

      include '../include/strucfac.inc'
      type(strucfac_t) stFacs(NSF)

      PRINT *, '*********************************'
      PRINT *, '* Read in STRUCTURE FACTOR data *'
      PRINT *, '*********************************'
      PRINT '(A, I8,A)', '   ', nsf, ' structure factors'
C
C  *******************************************
C  *    READ IN THE STRUCTURE FACTOR DATA    *
C  *******************************************
      SF_DSTR_MIN = HUGE(SF_DSTR_MIN)
      SF_DSTR_MAX = -HUGE(SF_DSTR_MAX)

      DO I = 1,NSF
         hkl(1,i) = stFacs(i)%h
         hkl(2,i) = stFacs(i)%k
         hkl(3,i) = stFacs(i)%l
         amp(i)   = stFacs(i)%amp
         phi(i)   = stFacs(i)%phi
         sig(i)   = stFacs(i)%sig
         dstr(i)  = stFacs(i)%dstr

        SF_DSTR_MIN = MIN(SF_DSTR_MIN,DSTR(I))
        SF_DSTR_MAX = MAX(SF_DSTR_MAX,DSTR(I))
        IF(SF_TYPE == 0) THEN
           AMP(I)    = AMP(I)*SF_SCALEF                  ! APART, BPART
           PHI(I)    = PHI(I)*SF_SCALEF
           SF_SUMFSQ = SF_SUMFSQ + AMP(I)**2 + PHI(I)**2
         ELSE IF(SF_TYPE == 1) THEN
           AMP(I)    = AMP(I)*SF_SCALEF                  ! AMP, PHASE
             SF_SUMFSQ = SF_SUMFSQ + AMP(I)**2
        ENDIF
        IF(HKL(3,I) < 0) THEN            ! MAKE SURE HKL DATA HAS L>=0
          HKL(1,I) = -HKL(1,I)
          HKL(2,I) = -HKL(2,I)
          HKL(3,I) = -HKL(3,I)
          IF(SF_TYPE == 0) THEN
             PHI(I) = -PHI(I)
           ELSE 
             PHI(I) = 360.0 - PHI(I)
          ENDIF
        ENDIF
        IF(MOD(I,10000) == 0) PRINT '(A,I7)', '+', I
      ENDDO

      PRINT '(A,I7)', '+', NSF
      WRITE (*,'(A,F10.5)') ' Maximum resolution =', SF_DSTR_MAX
      IF(SF_DSTR_MIN == 0.0) THEN
        WRITE (*,'(A,F10.5)') ' Minimum resolution =', SF_DSTR_MIN
       ELSE
        WRITE (*,'(A)') ' Minimum resolution =   INFINITY'
      ENDIF

      EM_ERR = 0                              ! SUCCESSFUL COMPLETION

      RETURN
      END
