Ca
Ca
Ca  This module modified April 1997 RW Ashmore for use with robem.
Ca
Ca

c==============================================================================
c Program UNBLOB
c ==============
c This version: 1.2    18-Dec-1992
c
c James F. Conway
c Bldg 6 Room B2-34
c Lab Structural Biology
c NIAMS/National Institutes of Health
c Bethesda, MD 20892
c
c The UNBLOB program should be linked with:
c      1) JFC_Map_IO.for:  Read/WriteMapHeader, Read/WriteMapSlice
c
c Compile options: EXTEND for lines longer than 72 columns
c
c This program reads 3D 'MAP'-format files and deletes blobs of density that
c are small and disconnected from the main virus capsid density. Two passes are
c made through the MAP file - one to catalogue the density blobs, the second to
c delete the ones below a specified volume. Remember that increasing protein
c density is usually represented in the MAP file by numbers negative and
c decreasing.
c
c Version History
c ===============
c v1.0
c ----
c First Release version
c
c v1.1
c ----
c  Speed enhancements to IO by specifying BLOCKSIZE and BLOCKCOUNT parameters
c in OPEN() statements. Improvements on Vax 4000-500 are:
c                                          CPU     Total
c                                        (secs)   (secs)
c 17,1 25-block MAP file: v1.0            67.29   144.85 (2 mins, 25 secs)
c                         v1.1            53.13    75.23 (1 min,  15 secs)
c                        % diff             79%      52%
c
c 137,421-block MAP file: v1.0           634.89   809.92 (13 mins, 30 secs)
c                         v1.1           604.38   446.72 ( 7 mins, 27 secs)
c                        % diff             95%      55%
c
c v1.2
c ----
c Program keeps track of min/max intensities for each blob and for background.
c From these, the new min/max values for the UNBLOB-ed MAP file can be found
c and written into the header when the new file is created. Note that the
c values in the old MAP file (produced by EMICOFB) may not always be correct!
c Also, blobId and blobSize arrays have been converted to a structure called
c blobList and the fields blobMin, blobMax have been added.
c
c Program Notes
c =============
c 'Voxels' are 26-connected, ie, a connection is considered to be made if any
c of the sides, edges or corners of two voxels are touching. This means that
c there are 26 potential connections that could be made between a voxel and its
c adjacent voxels.
c
c The program uses a structure/array (blobList) to record density blob
c information in the first pass through the MAP file. The fields are
c   1) blob id's, 2) blob sizes, 3) min blob density, 4) max blob density
c As each blob is encountered, an entry is made in the blobList array id field
c equal to the index of the entry. If a blob is later found to be connected to
c another blob, then the blobs are merged into one that is identified by the
c lowest id number. The entry for the higher numbered blob is replaced by the
c lower id so that it 'points' to the lower entry, and its corresponding size
c field is added to that of the lower id and then zeroed. It is possible that
c the lower id blob may be merged later with yet another blob that has a yet
c lower id, so that the entry pointing to it will now be pointing to a pointer
c to this new blob id. A 'unique' blob can be identified in one of two ways:
c      1) the entry points at itself (ie, blobList(entry)%id = entry), and
c      2) the size entry is non-zero.
c
c In the second pass, blobs are identified in the same manner and order as the
c first pass, and the blob sizes can be looked up by finding the 'unique'
c entry that the blob id entry ultimately points to. The corresponding size is
c then used to determine whether to delete the blob or not.
c
c Currently 50000 non-unique blob entries can be handled in the arrays. To 
c allow more blob entries, simply change the value of the BMAX parameter and
c recompile. (I know this is not the best way to use memory, ok?)
c
c MAP file formats are described in the TSB doc file EMICO_SYS.DOC.
c

c==============================================================================
c GetBlobMinMaxInt_rob
c ================
c Work out the min and max densities in the blobList. Ignore densities that
c are in blobs smaller than the size limit, and also include the 'killValue'
c if we are going to overwrite anything. If we don't find any blobs then 
c leave the minInt and maxInt values alone - they were read in originally
c from the map file header and should be valid.

      subroutine GETBLOBMINMAXINT_ROB(blobList, bCnt, blobSizeLim,
     $     killValue, minInt, maxInt)

      implicit none
c Variables passed in
      integer*4 bCnt, blobSizeLim
      real*4    killValue
      type blobstructure
        sequence
        integer*4 id
        integer*4 size
        real*4    minInt
        real*4    maxInt
      end type blobstructure
      type(blobstructure) blobList(bcnt)
c Variables returned
      real*4 minInt, maxInt
c Local variables
      integer*4 b
      logical firstFlag

      if(bCnt.eq.0)  return

      firstFlag = .true.
        minInt = 9999999.9
        maxInt =-99999999.9
      do b=1, bCnt
        if(blobList(b)%size.ge.blobSizeLim) then
          if(firstFlag) then            ! seed min/max variables
            firstFlag = .false.
            minInt = blobList(b)%minInt
            maxInt = blobList(b)%maxInt
          else
             if(blobList(b)%minInt .lt. minInt)  then
                minInt = blobList(b)%minInt
             endif
             if(blobList(b)%maxInt .lt. maxInt)  then
                maxInt = blobList(b)%maxInt
             endif
          end if
        end if
      end do

c Only include killValue if we are actually going to kill any blobs
      if(.not.firstFlag) then
        minInt = MIN(minInt, killValue)
        maxInt = MIN(maxInt, killValue)
      end if

      return
      end
c==============================================================================
c Count_Kill_Blobs_rob
c ================
c Pass through the map file and assign a blob id to each blob and count the
c number of 'voxels' in the blob. Unassigned blob voxels are numbered 0 and
c transparent voxels are 0. The index into the 'blobList' array is the id of
c the blob as it was initially encountered and is also recorded in the array.
c Blob ids number from 1 up. The size of the blob is kept in the 'size' field
c of the structure. If a blob is later found to join another blob, then the 
c blob id is changed to the lower id and the size is added to that of the 
c lower id. Min and Max densities are also kept in the array and are also 
c merged as the size field is merged.
c This routine operates in two modes: if killFlag is true, then delete blobs
c according to intensity and the contour limits passed in, and write the
c results to the outUnit file. But if killFlag is false, then just count
c blobs and blob sizes. Note that the subroutines Compare() and KillBlobs()
c are very similar - they are both working out what blobs are what - but
c Compare is building up the list of blobs and size, while KillBlobs is 
c figuring blob id's and killing those blobs that are smaller than the 
c limit.
Ca 6/18/97 RWA - See KillBlobs() for description of inverseUniverse Switch.

      subroutine COUNT_KILL_BLOBS_ROB(bCnt, uniqueBlobs , contour1,
     $     contour2, killFlag, killSize, killValue , bgMin, bgMax,
     $     status, mapBuffer, px, py , pz, blobList, radiusUnblob,
     $     inverseUniverse)

      implicit none
c Variables passed in
      integer*4 killFlag, killSize, uniqueBlobs
      real*4    contour1, contour2, killValue
      integer*4 px,py,pz,radiusUnblob, inverseUniverse
      real*4    mapBuffer(px,py,pz)
c Variables returned
      integer*4 bCnt, status
      real*4 bgMin, bgMax
      type blobstructure
        sequence
        integer*4 id
        integer*4 size
        real*4    minInt
        real*4    maxInt
      end type blobstructure
      type(blobstructure) blobList(bCnt)
c Local variables
      integer alloc_stat
      integer*4, allocatable :: lastSlice(:), thisSlice(:)
      real*4, allocatable    :: realSlice(:)
      integer*4, allocatable :: temp8(:)
      integer*4 z
      real*4    realTemp
      integer*4 totalBlobs, combinedBlobs
c Map file header data
      character title*72
      integer*4 dType, dx, dy, dz, mh_tmatrix(6)
      real*4  minIntensity, maxIntensity, scale, mh_lattice(8), 
     $     mh_steps(3)
      common /mapheader/ title, dx, dy, dz, dType, scale, minIntensity, 
     &                   maxIntensity
     &                  , mh_lattice, mh_steps, mh_tmatrix
c Functions

      dx = px       ! Get these globals from those passed in
      dy = py
      dz = pz

      if(contour1.gt.contour2) then      ! ensure contour1<contour2
Ca        temp = contour1
Ca        contour1 = contour2
Ca        contour2 = temp
        realTemp = contour1
        contour1 = contour2
        contour2 = realTemp
      end if

      allocate(lastSlice(dx*dy), thisSlice(dx*dy), realSlice(dx*dy),
     $     temp8(dx*dy), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate last,this,realSlice'
         go to 99
      endif


Ca      Always init out the bgMin and bgMax - memory not 
Ca      initialized otherwised.
        bgMax = -99999999.9
        bgMin =  99999999.9

        print *, 'Allocated -->>',lastSlice,thisSlice,realSlice

        lastSlice = 0        !F90 array syntax
        thisSlice = 0        !F90 array syntax
        realSlice = 0.0      !F90 array syntax

      totalBlobs    = 0
      combinedBlobs = 0

Ca      Cycle through all sections
      do z=1, dz
        CALL READMAPSLICE_ROB(dType, dx, dy, dz, scale, 
     &                   realSlice, mapBuffer, z)
        CALL BINARIZE(realSlice, thisSlice, dx, dy,
     &                  contour1, contour2, bgMin, bgMax)

        if(killFlag.eq.0) then
          CALL COMPARE(thisSlice, lastSlice, realSlice
     &            , dx, dy, blobList, totalBlobs, bCnt, combinedBlobs)
        else
          CALL KILLBLOBS(thisSlice, lastSlice, realSlice
     &            , dx, dy, dz, blobList, totalBlobs, bCnt, 
     &        killSize, killValue,
     &        radiusUnblob, z, inverseUniverse)
          CALL WRITEMAPSLICE_ROB(dType, dx, dy, dz, scale, 
     &                         realSlice, mapBuffer, z)
        endif
        temp8 = lastSlice       !F90 array syntax
        lastSlice = thisSlice   !F90 array syntax
        thisSlice = temp8       !F90 array syntax
        uniqueBlobs = totalBlobs - combinedBlobs
Ca        delta = SECNDS(start)
Ca        if(delta.lt.0.0 .or. delta.gt.5.0 .or. z.eq.dz) then
Ca          if(.not.killFlag) then
Ca            write(*, '(a,i<Int4Width(z)>,a,i<Int4Width(totalBlobs)>
Ca     &                  ,a,i<Int4Width(uniqueBlobs)>)') 
Ca     &                    '+Slice ', z
Ca     &                  , ', Total blobs = ', totalBlobs
Ca     &                  , ', Unique blobs = ', uniqueBlobs
Ca          else
Ca            write(*, '(a,i<Int4Width(z)>,a,i<Int4Width(totalBlobs)>)') 
Ca     &                    '+Slice ', z
Ca     &                  , ', Total blobs = ', totalBlobs
Ca          end if
Ca          start = SECNDS(0.0)
Ca        end if
      end do

      bCnt = totalBlobs

      deallocate(lastSlice, thisSlice, realSlice)

      status = 0

99      return
      end


c==============================================================================
c Binarize
c ========
c Copy real array data into integer array as either 0's (outside range of
c contour1 to contour2) or -1's (inside range).

      subroutine BINARIZE(arrayIn, arrayOut, dx, dy, contour1, contour2,
     $     bgMin, bgMax)

      implicit none
c Variables passed
      integer*4 dx, dy
      real*4  arrayIn(dx,dy), contour1, contour2
c Variables returned
      integer*4 arrayOut(dx,dy)
      real*4 bgMin, bgMax
c Local variables
      integer*4 x, y

      do y=1, dy
        do x=1, dx
           if((arrayIn(x,y).ge.contour1) .and. 
     $          (arrayIn(x,y).le.contour2)) then
              arrayOut(x,y) = -1 ! solid
           else
              if(arrayIn(x,y) .gt. bgMax) then
                 bgMax = arrayIn(x,y)
              endif
              if(arrayIn(x,y) .lt. bgMin) then
                 bgMin = arrayIn(x,y)
              endif
              arrayOut(x,y) = 0 ! transparent
           end if
        end do
      end do

      return
      end
c==============================================================================
c Compare
c =======
c Build up a list of blobs (groups of connected voxels) in the current 2d
c plane (new) that are connected to each other or to pre-found blobs in
c the prevoius plave (old). Check for voxels in the new plane that might
c connect several distinct blobs in the old plabe and coalesce then all
c in the blobList. Also, use the realData version of the new plane to test
c for min and max intensities for each blob as new voxels are added to them.

      subroutine COMPARE(new, old, realData, dx, dy, blobList, count,
     $     maxCnt, combinedBlobs)

      implicit none
c Variables passed in
      integer*4 dx, dy, maxCnt
      integer*4 new(dx,dy), old(dx,dy)
      real*4 realData(dx,dy)
c Variables returned
      integer*4 count
      integer*4 combinedBlobs
      type blobstructure
        sequence
        integer*4 id
        integer*4 size
        real*4    minInt
        real*4    maxInt
      end type blobstructure
      type(blobstructure) blobList(maxCnt)
c Local variables
      integer*4 blobs(27), bCnt, thisBlob, x, y

      do y=1, dy
        do x=1, dx
          if(new(x,y).lt.0) then            ! blob
            bCnt = 0
            CALL TESTCONNECTION(old, dx, dy, x, y, blobs, bCnt)
            CALL TESTCONNECTION(new, dx, dy, x, y, blobs, bCnt)
            if(bCnt.gt.0) then            ! old connection(s)
            CALL RESOLVECONNECTIONS(blobs, bCnt, blobList
     &                              , count, thisBlob, combinedBlobs)
            if(realData(x,y).lt.blobList(thisBlob)%minInt) then
              blobList(thisBlob)%minInt = realData(x,y)
            end if
            if(realData(x,y).gt.blobList(thisBlob)%maxInt) then
              blobList(thisBlob)%maxInt = realData(x,y)
            end if
            else                        ! unconnected blob
            if(count.eq.maxCnt) then
              write(*, '(a,i8)') ' Too many blobs - limit is ', maxCnt
              return
            end if
            count = count + 1            ! create new entry for it
            thisBlob = count
            blobList(thisBlob)%id = thisBlob
            blobList(thisBlob)%size = 0
            blobList(thisBlob)%minInt = realData(x,y)
            blobList(thisBlob)%maxInt = realData(x,y)
            end if
            blobList(thisBlob)%size = blobList(thisBlob)%size + 1
            new(x,y) = thisBlob
          end if
        end do
      end do

      return
      end
c==============================================================================
c KillBlobs
c =========
c Read the blobs as for Compare so that blobId's are assigned in the same
c manner. Test each blob id for the size of the blob and if it is below the
c size limit, then delete the blob by over writing with the kill value.
Ca 6/18/97 RWA - Added an inverseUniverse switch so that we can KEEP the 
Ca blobs of a certain size and kill everything else.

      subroutine KILLBLOBS(new, old, image, dx, dy, dz, blobList, count,
     $     maxCnt, killSize, killValue, radiusUnblob, thisZ,
     $     inverseUniverse)

      implicit none
c Variables passed in
      integer*4 dx, dy, dz, maxCnt, killSize, radiusUnblob, radUnSq
      integer*4 thisZ, new(dx,dy), old(dx,dy), inverseUniverse
      real*4 killValue
c Variables returned
      integer*4 count, thisDist, xcen, ycen, zcen
      integer*4 zDistSq, yDistSq
      real*4 image(dx,dy)
      type blobstructure
        sequence
        integer*4 id
        integer*4 size
        real*4    minInt
        real*4    maxInt
      end type blobstructure
      type(blobstructure) blobList(maxCnt)
c Local variables
      integer*4 blobs(27), bCnt, thisBlob, x, y, thisSize

      radUnSq = radiusUnblob * radiusUnblob
      xcen = (dx / 2) + 1
      ycen = (dy / 2) + 1
      zcen = (dz / 2) + 1
      zDistSq = (thisZ - zcen) * (thisZ - zcen)
      do y=1, dy
        yDistSq= (y-ycen)*(y-ycen)
        do x=1, dx
          if((new(x,y).lt.0)) then      ! blob
            bCnt = 0
            CALL TESTCONNECTION(old, dx, dy, x, y, blobs, bCnt)
            CALL TESTCONNECTION(new, dx, dy, x, y, blobs, bCnt)
            if(bCnt.gt.0) then            ! connected blob
            thisBlob = blobs(1)            ! any connected one will do
            else                        ! unconnected blob
            count = count + 1            ! create new entry for it
            thisBlob = count
            end if
            new(x,y) = thisBlob
            CALL FINDBLOBSIZE(blobList, maxCnt, thisBlob, thisSize)
            thisDist = ((x-xcen)*(x-xcen))+yDistSq+zDistSq
              if(inverseUniverse .eq. 1) then
                 if(thisSize.gt.killsize.and.(thisDist.ge.radUnSq))then
Ca                  Do nothing
                 else
                    image(x,y) = killValue  ! Kill everything else
                 endif
              else
                 if(thisSize.lt.killsize.and.(thisDist.ge.radUnSq))then
                    image(x,y) = killValue
                 endif
            endif
          end if
        end do
      end do

      return
      end
c==============================================================================
c TestConnection
c ==============
c Test connection of point (x,y) with connected points (x-1 to x+1, y-1 to y+1).
c There may be more than one blob connected, and there may be more than one
c pixel from a blob connected. Return all unique blobs that are connected to
c x,y in new slice.

      subroutine TESTCONNECTION(array, dx, dy, px, py, blobs, b)

      implicit none
c Variables passed in
      integer*4 dx, dy, px, py
      integer*4 array(dx,dy)
c Variables returned
      integer*4 blobs(9), b
c Local variables
      integer*4 c, minx, maxx, miny, maxy, x, y, thisBlob

      minx = max(px-1,1)      ! check we don't go over the edge
      maxx = min(px+1,dx)
      miny = max(py-1,1)
      maxy = min(py+1,dy)

      do y=miny, maxy
        do x=minx, maxx
          thisBlob = array(x,y)
          if(thisBlob.gt.0) then            ! a connection
            do c=1, b                        ! check if...
              if(thisBlob.eq.blobs(c)) goto 20! ...already connected,  ignore
            end do
            b = b + 1                        ! a new connection, save it
            blobs(b) = thisBlob
20            continue
          end if
        end do
      end do

      return
      end
c==============================================================================
c ResolveConnections
c ==================
c 'blobs' is a list of blob ids that are connected. We go through the 'blobList'
c array and make all the connected blob id's point to the lowest blob id, which
c points to itself. We also combine the sizes of connected blobs into the size
c of the blob with the lowest id, and set the other connected blob size fileds
c to zero. This is so we don't have to update blob sizes in different places as
c blobs are added to.

      subroutine RESOLVECONNECTIONS(blobs, bCnt, blobList
     &                              , count, lowBlob, combinedBlobs)

      implicit none
c Variables passed in
      integer*4 bCnt, blobs(bCnt), count
c Variables returned
      integer*4 lowBlob, combinedBlobs
      type blobstructure
        sequence
        integer*4 id
        integer*4 size
        real*4    minInt
        real*4    maxInt
      end type blobstructure
      type(blobstructure) blobList(count)
c Local variables
      integer*4 thisBlob, temp, b

      if(bCnt.lt.1)  return

      lowBlob = blobs(1)
      do b=1, bCnt
        thisBlob = blobs(b)
        do while(blobList(thisBlob)%id .ne. thisBlob)
          thisBlob = blobList(thisBlob)%id
        end do
        if(thisBlob.lt.lowBlob) lowBlob = thisBlob
      end do
      do b=1, bCnt
        thisBlob = blobs(b)
        do while(blobList(thisBlob)%id .ne. thisBlob)
          temp = blobList(thisBlob)%id
          blobList(thisBlob)%id = lowBlob ! point to the lowest blob id
          thisBlob = temp
        end do
        if(thisBlob.ne.lowBlob) then
          blobList(lowBlob)%size = blobList(lowBlob)%size + 
     &                                   blobList(thisBlob)%size
          blobList(thisBlob)%size = 0          ! add its volume to low blob also
          if(blobList(thisBlob)%minInt < blobList(lowBlob)%minInt) then
            blobList(lowBlob)%minInt = blobList(thisBlob)%minInt
            blobList(thisBlob)%minInt = 0.0
          end if
          if(blobList(thisBlob)%maxInt > blobList(lowBlob)%maxInt) then
            blobList(lowBlob)%maxInt = blobList(thisBlob)%maxInt
            blobList(thisBlob)%maxInt = 0.0
          end if
          blobList(thisBlob)%id = lowBlob    ! point directly to the lowest blob id
          combinedBlobs = combinedBlobs + 1
        end if
      end do

      return
      end
c==============================================================================
c FindBlobSize
c ============
c Given a blob id (the entry index into the blobList array) return the size of
c the blob. Check if the blob has been combined with other blobs that come
c before it in the blobList array by inspecting the id field - if it points
c to another id then follow the chain of linked id's down to the lowest id
c where the total blob size is recoreded in its size field.

      subroutine FINDBLOBSIZE(blobList, maxCnt, aBlob, size)

      implicit none
c Variables passed in
      integer*4 maxCnt, aBlob
      type blobstructure
        sequence
        integer*4 id
        integer*4 size
        real*4    minInt
        real*4    maxInt
      end type blobstructure
      type(blobstructure) blobList(maxCnt)
c Variables returned
      integer*4 size, bBlob

      bBlob = aBlob
      do while(blobList(bBlob)%id .ne. bBlob)
        bBlob = blobList(bBlob)%id
      end do
      size = blobList(bBlob)%size

      return
      end
c==============================================================================
c ReadMapSlice
c ============
c Read the next slice from the map.

      subroutine READMAPSLICE_ROB (dType        ! data type: 3=real*4, -3=int*2
     &                        , dx, dy, dz      ! dimensions of slice
     &                        , scale           ! global scale for int*2 map
     &                        , array           ! address for 2d slice
     &                        , mapBuffer, z)

      implicit none
c Variables passed in
      integer*4 dType, dx, dy, dz
      real*4    scale
      real*4    mapBuffer(dx,dy,dz)
      integer*4 z
c Variables returned
      real*4    array(dx,dy)
c Local variables
      integer*4 x, y
      integer*2 arrInt(10240)

      dType = 0  ! Default it for now
      do y=1, dy
        if(dType.eq.-3) then
          do x=1, dx
            array(x,y) = real(arrInt(x)) * scale
          end do
        else
          do x=1, dx
            array(x,y)= mapBuffer(x,y,z)
          enddo
        endif
      end do

      goto 99
      
 99   return
      end
c====================================================================
c==============================================================================
c WriteMapSlice_rob
c =============
c Write the next slice to the map.

      subroutine WRITEMAPSLICE_ROB(dType        ! data type: 3=real*4, -3=int*2
     &                        , dx, dy, dz      ! dimensions of slice
     &                        , scale           ! global scale for int*2 map
     &                        , array           ! address of 2d slice
     &                        , mapBuffer, z)

      implicit none
c Variables passed in
      integer*4 z
c Variables returned
      integer*4 dType, dx, dy, dz
      real*4    array(dx,dy), scale
      real*4    mapBuffer(dx,dy,dz)
c Local variables
      integer*4 x, y
      integer*2 arrInt(10240)

      dType = 0      ! Default it for now
      do y=1, dy
        if(dType.eq.-3) then
          do x=1, dx
            arrInt(x) = nint(array(x,y) / scale)
          end do
        else
          do x=1, dx
              mapBuffer(x,y,z) = array(x,y)
          enddo
        endif
      end do

      goto 99

99      return
      end
