#include <limits.h>
#include <math.h>
#include "../include/common.h"
#include "../include/appodize.h"
#include "robem_def.h"

void calcStats(float *inbuffer, float *img_bkg, float *img_min,
	       float *img_max, float *img_avg, float *img_std,
	       int width, int height, int boxWidth, int fileType, 
	       int bkgrndMethod, int boxMethod, int packRadius, 
	       int boxRadius, int featherRadius, float recipSizeFactor, 
	       int tryColors)
{
  // function name: calcStats
  // Calculates statistics from the image

  int i, j, tot=0, totBkg=0;
  float pixelValue, totValues=0.0, totBkgValues=0.0;
  double var=0.0, sumsq=0.0;
  
  // Init all
  *img_bkg = 0;
  *img_min = INT_MAX;
  *img_max = INT_MIN;

  // Accumulate intermediate values

  for (i=0; i<height; i++) {
    for (j=0; j<width; j++) { // Look at all pixels
      pixelValue = inbuffer[(i*width)+j];
      *img_min = MIN(*img_min,pixelValue);
      *img_max = MAX(*img_max,pixelValue);
//GCA float buffer is assumed to have positive and negative values
//      if(pixelValue > 0) { // Only use non-zeros
	sumsq += pixelValue*pixelValue;
	totValues += pixelValue;
	tot++;
//      }
      if(i==0 || j==0 || i==(height-1) || j==(width-1)) {
	// Accum for RECT background
	totBkgValues += pixelValue;
	totBkg++;
      }
    }
  }
  
  // Calc the background 
  if(fileType==BYTEPORTABLE || fileType==EMPORTABLE || fileType==REBOX)
    *img_bkg = (float)
      calcFloatBackground(inbuffer, width, height, boxWidth,
			  fileType, bkgrndMethod, boxMethod, packRadius,
			  boxRadius, featherRadius, recipSizeFactor, tryColors);
  else if(fileType==RECTPACKED && totBkg > 0)
    *img_bkg = (int)(totBkgValues / totBkg);
  else 
    *img_bkg = inbuffer[0]; // e.g. Don't recalc Packed boxes or MAPS!!

  // Calc stats
  *img_avg = totValues / (width * height);
  var = (sumsq / (width*height)) - (*img_avg * *img_avg);
  *img_std = sqrt(fabs(var));

} // end function calcStats
