// CTFDISP - Application to interactively plot a theoretical CTF curve
// based on various parameters editable by the user interface.
//
//   Rob Ashmore
//   Biology Department
//   Purdue University
//   April 1996

#include "../include/common.h"
#include "ctfdisp.h"
#include "../include/libUtil.h"



double deg2rad(double degrees)
{
  // function name: deg2rad
  // convert Degrees to radians.

  double rtnVal;

  rtnVal = 2.0*PI*degrees/360.0;
  return(rtnVal);

} // end function deg2rad



double rad2deg(double rads)
{
  // function name: rad2deg
  // convert Radians to degrees.

  double rtnVal;

  rtnVal = (rads / THISTWOPI) * 360.0;
  return(rtnVal);

} // end function  rad2deg



void findMaxMin(float *fft_buffer, float *maxF, float *minF, 
		float *offset, int logSwitch)
{
  // function name: findMaxMin
  // Find the min/max for this FFT buffer.

  int i, j, jindx;
  float thisFloat;

  if(logSwitch) { // Find how neagtive we go
    *maxF= -999999999999999999.9; 
    *minF = 999999999999999999.9;
    for(j=0; j<nrow; j++) {
      for(i=0; i<ncol; i++) {
	jindx   = j*fftWidth*2 + i*2;
	thisFloat= fft_buffer[jindx];	
	*minF = MIN(*minF,thisFloat);
      }
    }
  }
  *offset = *minF;
  
  *maxF= -999999999999999999.9; 
  *minF = 999999999999999999.9;
  for(j=0; j<nrow; j++) {
    for(i=0; i<ncol; i++) {
      jindx   = j*fftWidth*2 + i*2;
      
      if(logSwitch) {
	if(fft_buffer[jindx] != 0.0 || fft_buffer[jindx+1] != 0.0)
	  thisFloat= 
	    log((fft_buffer[jindx]*fft_buffer[jindx]) +
		(fft_buffer[jindx+1]*fft_buffer[jindx+1]));
	else 
	  thisFloat = 0.0;
      }
      else 
	thisFloat= fft_buffer[jindx];
      
      *maxF = MAX(*maxF, thisFloat);
      *minF = MIN(*minF, thisFloat);
    }
  }
  
  if(*maxF == *minF)
    *maxF = *minF + 0.01;
  
} // end function findMaxMin



void fft_dsp_c(float *fft, unsigned short *img, 
	       int fftSize1, int fftSize2,
	       int ncol, int nrow, 
	       float thresh_hi, float thresh_lo,
	       int hiPixel, int loPixel, 
	       int linear, int amps)
{
  // function name: fft_dsp_c
  //
  // Derived from the TSB fortran FFT_DSP().  We're trying to do
  // 1024x1024 FFT's now and display them in 512x512.  Also be aware
  // of the problesm involving 256x256 and 128x128 fft's...
  //
  // THE INTENSITIES FROM EMIMGFFT ARE DISPLAYED USING A 64 GREY-LEVEL
  // SCALE. THE TOP TWO BITS ARE USED BY THE MASK PROGRAM FOR
  // SUBSEQUENT PSEUDO-OPTICAL FILTERING (USING EMFFT).
  //
  // ONLY VALUES BETWEEN THRESH_LO AND THRESH_HI ARE MAPPED INTO THE
  // COLOR TABLE, WITH OUTLIERS SET TO THE OUTSIDE LIMITS OF THE CT.

  int    i, j, fftSize2D2, fftSize2T2, jT2;
  int    lastRow,lastCol,ii,jj, thisOutRow, dist;
  float  xmult, ymult, halfCol,halfRow;
  float  testlo, testhi, slopeb, value;
  
  testhi = hiPixel;
  testlo = loPixel;
  
  xmult = (float)ncol / (float)fftSize1;
  ymult = (float)nrow / (float)fftSize2;

  lastCol = MAX((int)xmult,1);
  lastRow = MAX((int)ymult,1);
  
  fftSize2D2 = fftSize2 / 2;
  fftSize2T2 = fftSize2 * 2;

  slopeb = (testhi-testlo)/(thresh_hi-thresh_lo);
  
  // FILL UP IMG ARRAY AFTER SCALING
  
  halfRow = fftSize1/2;
  halfCol = fftSize2D2/2;
  if(xmult >= 1.0 && ymult >= 1.0) {/* The FFT is smaller or equal to the display size */
    
    for(i=0; i<fftSize1; i++) {/* do i = 1,((fftSize2/2)+1) */
      
      if(i==0)
	thisOutRow = fftSize2D2 - 1;
      else if(i < fftSize2D2)
	thisOutRow = fftSize2D2 - 1 + i;
      else
	thisOutRow = i - fftSize2D2 - 1;
      thisOutRow = MAX(thisOutRow,0);
      thisOutRow = MIN(thisOutRow,(fftSize1-1));
      
      for(j=0; j<fftSize2D2; j++) { // do j = 1,jtest ! right side of fft
	
	jT2 = 2 * j;
	/*	      fft[(jT2+1+(i*fftSize2T2))] *= 
		      tempFactors[((int)(i*ymult))][2*(int)(j*xmult)];
		      fft[(jT2+(i*fftSize2T2))] *= 
		      tempFactors[((int)(i*ymult))][2*(int)(j*xmult)];
	*/
	value = fft[jT2+1+(i*fftSize2T2)] * fft[jT2+1+(i*fftSize2T2)] + 
	  fft[jT2+(i*fftSize2T2)] * fft[jT2+(i*fftSize2T2)];
	
	if(amps != 0) value = sqrt(value);
	if(linear==0) {
	  if(value != 0.0)
	    value = log10(value);
	  else 
	    value = 0.0;
	}
	
	value = (value - thresh_lo)*slopeb + testlo;
	
	if(value < testlo) 
	  value = testlo;
	if(value > testhi) 
	  value = testhi;
	
	dist = (int)sqrt((float)
			 ((i-halfRow)*(i-halfRow) +
			  (j-halfCol)*(j-halfCol)));

	if(XtIsManaged(ctfCorrectScreen) && dispCtfCorrect) // Adjust value
	  value *= ctf_vals[curCtf][dist].fudge;
	
	for(ii=0;ii<lastRow;ii++) // Expand it out
	  for(jj=0;jj<lastCol;jj++) // Expand cols
	    img[((int)(((thisOutRow*ymult)+ii)*ncol)+
		 (int)((j+fftSize2D2)*xmult)+jj)] = (BYTE)(value);
	
      }/* do j = 1,jtest ! right side of fft */
      
      
      for(j=fftSize2/2; j<fftSize2; j++) { // do j = (jtest+1),fftSize1 ! left side of fft
	jT2 = 2 * j;
	/* fft[(jT2+1+(i*fftSize2T2))] *= 
	   tempFactors[((int)(i*ymult))][2*(int)(j*xmult)];
	   fft[(jT2+(i*fftSize2T2))] *= 
	   tempFactors[((int)(i*ymult))][2*(int)(j*xmult)];
	*/
	value = fft[jT2+1+(i*fftSize2T2)] * fft[jT2+1+(i*fftSize2T2)] + 
	  fft[jT2+(i*fftSize2T2)] * fft[jT2+(i*fftSize2T2)];
	
	if(amps != 0) value = sqrt(value);
	if(linear==0) {
	  if(value != 0.0)
	    value = log10(value);
	  else 
	    value = 0.0;
	}
	
	value = (value - thresh_lo)*slopeb + testlo;
	
	if(value < testlo) value = testlo;
	if(value > testhi) value = testhi;
	
	dist = (int)sqrt((float)
			 ((i-halfRow)*(i-halfRow) +
			  (j-halfCol)*(j-halfCol)) );

	if(XtIsManaged(ctfCorrectScreen) && dispCtfCorrect) {/* ADjust value */
	  value *= ctf_vals[curCtf][dist].fudge;
	}
	
	for(ii=0;ii<lastRow;ii++) // Expand it out
	  for(jj=0;jj<lastCol;jj++) // Expand cols
	    img[((int)(((thisOutRow*ymult)+ii)*ncol)+
		 (int)((j-fftSize2D2)*xmult)+jj)] = 
	      (BYTE)(value);
	
      } /* do j = (jtest+1),fftSize1 ! left side of fft */
      
    } /* do i = 1,((fftSize2/2)+1) */
  } /* The FFT is smaller or equal to the display size */
  else if(xmult < 1.0 && ymult < 1.0) {/* The FFT is larger than the display size */
    int step, fftSize2D2mult;
    float totValue,numPnts;
    step = (int)(1.0 / xmult);
    fftSize2D2mult = (int)(fftSize2D2 * xmult);
    for(i=0;i<(fftSize1);i+=step) {/* do all rows */
      
      if(i==0)
	thisOutRow = fftSize2D2mult - 1;
      else if(i < fftSize2D2)
	thisOutRow = fftSize2D2mult - 1 + (i*xmult);
      else
	thisOutRow = (i*xmult) - fftSize2D2mult - 1;
      thisOutRow = MAX(thisOutRow,0);
      thisOutRow = MIN(thisOutRow,((int)(fftSize1*xmult)-1)); 
      
      for(j=0; j<fftSize2D2; j+=step) { // do right side of fft
	numPnts  = 0.0;
	totValue = 0.0;
	for(ii=0; ii<step; ii++) {
	  for(jj=0; jj<step; jj++) {
	    jT2 = 2 * (j+jj);
	    /* fft[(jT2+1+((i+ii)*fftSize2T2))] *= 
	       tempFactors[((int)(i*ymult))][2*(int)(j*xmult)];
	       fft[(jT2+((i+ii)*fftSize2T2))] *= 
	       tempFactors[((int)(i*ymult))][2*(int)(j*xmult)];
	    */
	    value = fft[jT2+1+((i+ii)*fftSize2T2)] * 
	      fft[jT2+1+((i+ii)*fftSize2T2)] + 
	      fft[jT2+((i+ii)*fftSize2T2)] * 
	      fft[jT2+((i+ii)*fftSize2T2)];
	    
	    if(amps != 0) value = sqrt(value);
	    if(linear==0) {
	      if(value != 0.0)
		value = log10(value);
	      else 
		value = 0.0;
	    }
	    
	    value = (value-thresh_lo)*slopeb + testlo;
	    
	    if(value < testlo) 
	      value = testlo;
	    if(value > testhi) 
	      value = testhi;
	    
	    totValue += value;
	    numPnts  += 1.0;
	  }
	}
	
	dist = (int)sqrt( (float)
			 ((i-halfRow)*(i-halfRow) +
			  (j-halfCol)*(j-halfCol)) );

	if(XtIsManaged(ctfCorrectScreen) && dispCtfCorrect) // Adjust value
	  totValue *= ctf_vals[curCtf][dist].fudge;
	
	img[((int)(thisOutRow*ncol)+(int)((j+fftSize2D2)*xmult))] = 
	  (BYTE)(totValue/numPnts);
      }
      
      
      for(j=fftSize2D2; j<fftSize2; j+=step) { // do left side of fft
	
	numPnts  = 0.0;
	totValue = 0.0;

	for(ii=0; ii<step; ii++) {
	  for(jj=0; jj<step; jj++) {
	    jT2 = 2 * (j+jj);
	    /*  fft[(jT2+1+((i+ii)*fftSize2T2))] *= 
		tempFactors[((int)(i*ymult))][2*(int)(j*xmult)];
		fft[(jT2+((i+ii)*fftSize2T2))] *= 
		tempFactors[((int)(i*ymult))][2*(int)(j*xmult)];
	    */
	    value = fft[jT2+1+((i+ii)*fftSize2T2)] * 
	      fft[jT2+1+((i+ii)*fftSize2T2)] + 
	      fft[jT2+((i+ii)*fftSize2T2)] * 
	      fft[jT2+((i+ii)*fftSize2T2)];
	    
	    if(amps != 0) value = sqrt(value);
	    
	    if(linear==0) {
	      if(value != 0.0)
		value = log10(value);
	      else 
		value = 0.0;
	    }
	    
	    value = (value - thresh_lo)*slopeb + testlo;
	    
	    if(value < testlo) 
	      value = testlo;
	    if(value > testhi) 
	      value = testhi;
	    
	    totValue += value;
	    numPnts  += 1.0;
	  }
	}
	
	dist = (int)sqrt((float)
			 ((i-halfRow)*(i-halfRow) +
			  (j-halfCol)*(j-halfCol)));

	if(XtIsManaged(ctfCorrectScreen) && dispCtfCorrect) // Adjust value
	  totValue *= ctf_vals[curCtf][dist].fudge;
	
	img[((int)(thisOutRow*ncol)+
	     (int)((j-fftSize2D2)*xmult))] = 
	  (BYTE)(totValue/numPnts);
	
      }/* do left side of fft */
      
    }/* do all rows) */
    
  }/* The FFT is larger than the display size */
  else
    printf("[FFT_DSP_C] Don't know how to do this...\n");

  
} // end function fft_dsp_c



void disp_fft(float *fftImage_this,
	      unsigned short *this_image_buffer1,
	      int firstTime)
{
  // function name: disp_fft
  // Rearrange the FFT so we can display it on the display.  
  // Map the fft values into grayscale.

  int i,j;
  static float offset, maxF, minF, slopeb, thisVal;
  
  // Find scaling if appropriate
  if(firstTime) {
    offset = 0.0;
    findMaxMin(fftImage_this, &maxF, &minF, &offset, TRUE);
    // hiFFTscroll and loFFTscroll keep the percentage they want of that
    slopeb = ((tryColors - 4) - 8)/((maxF-minF)+(maxF*.05));
  }
  
  // Let's try and sew this fft together so we can see it
  for(i=0; i<fftHeight; i++) {
    for(j=0; j<fftWidth*2; j+=2) {
      int thisPixel, fftRow, fftCol;
      fftRow = fftCol = 0;
      thisPixel = j * 0.5;
      if(i<fftHeight*0.5 && j<fftWidth) { // Upper left quarter
	fftRow= (i+(int)(fftHeight*0.5)) * fftWidth*2;
	fftCol= j+fftWidth;
      }
      else if(i<fftHeight*0.5 && j>=fftWidth) { // Upper right quarter
	fftRow= (i+(int)(fftHeight*0.5)) * fftWidth*2;
	fftCol= j-fftWidth;
      }
      else if(i>=fftHeight*0.5 && j<fftWidth) { // Lower left quarter
	fftRow= (i-(int)(fftHeight*0.5)) * fftWidth*2;
	fftCol= j+fftWidth;
      }
      else if(i>=fftHeight*0.5 && j>=fftWidth) { // Lower right quarter
	fftRow= (i-(int)(fftHeight*0.5)) * fftWidth*2;
	fftCol= j-fftWidth;
      }
      
      if((fftImage_this[fftRow+fftCol] == 0.0) &&
	 (fftImage_this[fftRow+fftCol+1] == 0.0))
	thisVal = 0;
      else 
	thisVal = log((fftImage_this[fftRow+fftCol]*
		       fftImage_this[fftRow+fftCol]) 
		      +
		      (fftImage_this[fftRow+fftCol+1]*
		       fftImage_this[fftRow+fftCol+1]) 
		      );
      // value = ((value - thresh_lo)*slopeb) + testlo;
      thisVal = (thisVal-minF);
      
      if(thisVal <= ((maxF-minF)*loFFTscroll))
	thisVal = BLACK+9;
      else if(thisVal >= ((maxF-minF)*hiFFTscroll))
	thisVal = tryColors-2;
      else
	thisVal *= slopeb;
      this_image_buffer1[(i*fftWidth)+thisPixel]= 
	(unsigned short)(thisVal)+8;
    }
  }

} // end function disp_fft



void applyFFT()
{
  // function name: applyFFT
  // Apply CTF to the FFT and back translate

  int img_bkg,jindx,minI,maxI;
  int ksign,iform;
  float minF,maxF,offset,sizeScaleFactor;
  float thisFloat,maxDesired,slopeb;
  float thisVal;
  int   units,fft_full;
  float pixelSize, tFactor, wave;
  float pi,radian,degree;
  float angMajor;
  float cc,beta,dE,voltage;
  float resoXlo, res_max, cut_max, resoXhi, res_min, cut_min;

  piraddeg_(&pi,&radian,&degree);
  
  /* Why does ncol keep getting trashed??? */
  if(ncol > 256) {/* Trashed */
    printf("T\n");
    ncol= 256;
  }

  // Initialize image to background value
  img_bkg = 0;
  for(j=0; j<nrow; j++) {
    for(i=0; i<ncol; i++) {
      fftImage_buffer[j*(ncol*2) + (i*2)]   = img_bkg;
      fftImage_buffer[j*(ncol*2) + (i*2)+1] = img_bkg;
    }
  }

  img_bkg = 0;
  for(j=0; j<nrow; j++)
    for(i=0; i<ncol; i++)
      fftImage_buffer[(j*(ncol*2))+(i*2)]= (float)
	(int_image_buffer1[(j*ncol)+i] - img_bkg);
  
  /* DO it the fortran way *******************************************/
  img_bkg = int_image_buffer1[0];  /* Use first pixel */
  img_fft_fill_rob__(int_image_buffer1, &ncol, &nrow, fftImage_buffer, 
		    &fftWidth, &fftHeight, &img_bkg);


  /* Do the first FFT */
  ksign = 1;  /* Do a "forward" FFT */
  iform = 0;  /* Data is real not imaginary WHY is it real??? */
  iform = 1;  /* Data is imaginary */
  fft_2d_rob__(fftImage_buffer, &fftWidth, &fftHeight, &ksign, &iform);

  offset = 0.0;
  sizeScaleFactor= 10000.00; 
  sizeScaleFactor = 0.0;
  findMaxMin(fftImage_buffer, &maxF, &minF, &offset, TRUE);
  if(maxF==minF)
    sizeScaleFactor= 1.0;
  else sizeScaleFactor = (float)maxOrig / (maxF-minF);

  /* slopeb = (testhi-testlo)/(thresh_hi-thresh_lo); */
  /* hiFFTscroll and loFFTscroll keep the percentage they want of that
   */
  slopeb = ((tryColors-2) - 8) / (maxF-minF);

  // Let's try and sew this fft together so we can see it
  for(i=0; i<fftHeight; i++) {
    for(j=0; j<fftWidth*2; j+=2) {
      int thisPixel, fftRow, fftCol;
      thisPixel = (int)(j * 0.5);
      fftRow = fftCol = 0;
      if(i<fftHeight*0.5 && j<fftWidth) { // Upper left quarter
	fftRow= (i+(int)(fftHeight*0.5)) * fftWidth*2;
	fftCol= j + fftWidth;
      }
      else if(i<fftHeight*0.5 && j>=fftWidth) { // Upper right quarter
	fftRow= (i+(int)(fftHeight*0.5)) * fftWidth*2;
	fftCol= j-fftWidth;
      }
      else if(i>=fftHeight*0.5 && j<fftWidth) { // Lower left quarter
	fftRow= (i-(int)(fftHeight*0.5)) * fftWidth*2;
	fftCol= j+fftWidth;
      }
      else if(i>=fftHeight*0.5 && j>=fftWidth) { // Lower right quarter
	fftRow= (i-(int)(fftHeight*0.5)) * fftWidth*2;
	fftCol= j-fftWidth;
      }
      
      if((fftImage_buffer[(fftRow+fftCol)]==0.0) &&
	 (fftImage_buffer[(fftRow+fftCol+1)]==0.0))
	thisVal = 0;
      else 
	thisVal = (float)log(
			     (fftImage_buffer[fftRow+fftCol]*
			      fftImage_buffer[fftRow+fftCol]) 
			     +
			     (fftImage_buffer[fftRow+fftCol+1]*
			      fftImage_buffer[fftRow+fftCol+1]) 
			     );
      /* value = ((value - thresh_lo)*slopeb) + testlo; */
      thisVal = thisVal-minF;
      
      if(thisVal <= (maxF-minF)*loFFTscroll)
	thisVal = BLACK+8;
      else if(thisVal >= (maxF-minF)*hiFFTscroll)
	thisVal = tryColors-2;
      else
	thisVal *= slopeb;
      raw_fft_image_buffer1[i*fftWidth + thisPixel]= 
	(unsigned short)thisVal;
      
    }
  }
  
  /* Try using the TSB library call ctf_multiply() to aplly the FFT */
  units     = 2; /* e.g. we're using nanometers for pixelSize */
  pixelSize = 1.0; /* ??? */
  pixelSize = pixSizeTextValue * 0.1;  /* Do in nm */
  tFactor = 0.0;
  fft_full = 1;
  /* We need to change the sign of angMajor here because our
   * XWindows based buffer is upside down.  When we write
   * to disk we need to use the sign just as given from
   * the interface */
  angMajor = 0.0 - majAngScrollValue ;

  cc      = chaberrationScrollValue; 
  beta    = spCoherenceScrollValue * spCoFactor; 
  dE      = tmCoherenceScrollValue;
  voltage = avoltageScrollValue;

  if(applyFFTswitch) {
    wave = wavel_(&avoltageScrollValue);
    ctf_multiply__(&ctfMode,
		   fftImage_buffer,
		   &fftHeight, &fftWidth, 
		   &units, 
		   &pixelSize,
		   &pi,&radian,
		   &wave,
		   &majDefocusScrollValue, &minDefocusScrollValue,
		   &angMajor,
		   &afractScrollValue,
		   &spaberrationScrollValue, 
		   &cc,&beta,&dE,&voltage,
		   &tFactor, &fft_full
		   );
  }
  
  if(XtIsManaged(ctfCorrectScreen) && filter[0] != 0) {
    ctf_divide__(&ctfMode_div, &filter[0], 
		 fftImage_buffer, 
		 &fftHeight, &fftWidth, 
		 &units, 
		 &pixelSize,
		 &pi,&radian,
		 &wave,
		 &majDefocusScrollValue, &minDefocusScrollValue,
		 &angMajor,
		 &afractScrollValue,
		 &spaberrationScrollValue, 
		 filt_fac,
		 &tFactor, &fft_full,
		 &cc,&beta,&dE,&voltage);
  }

  if(bandPass==PASSCTF && resXlo >= 0 && resXhi >= 0) { 
    resoXlo = resXlo/512.0*resolutionScrollValue;
    resoXhi = resXhi/512.0*resolutionScrollValue;
    res_max = fftWidth * pixelSize * resoXlo;
    res_min = fftWidth * pixelSize * resoXhi;
    cut_min = cut_max = 0.0;
    
    fft_hiplop__(fftImage_buffer,
		 &fftWidth, &fftHeight, 
		 &res_min, &res_max,
		 &cut_min, &cut_max,
		 &middlePass, &fft_full);
  }
  
  // Let's just scale as per the original above
  sizeScaleFactor = 0.0;

  findMaxMin(fftImage_buffer, &maxF, &minF, &offset, TRUE);
  maxDesired = maxF * 1000000.0;
  if(maxF==minF)
    sizeScaleFactor= 1.0;
  else 
    sizeScaleFactor = (float)(maxDesired) / (maxF-minF);

  // slopeb = (testhi-testlo)/(thresh_hi-thresh_lo);
  // hiFFTscroll and loFFTscroll keep the percentage they want of that

  slopeb = ((tryColors-2) - 8)/(maxF-minF);

  // Let's try and sew this fft together so we can see it
  for(i=0; i<fftHeight; i++) {
    for(j=0; j<(int)(fftWidth*2.0); j+=2) {
      int thisPixel,fftRow,fftCol;
      thisPixel = (int)((j) * 0.5);
      fftRow = fftCol = 0;
      if(i<fftHeight*0.5 && j<fftWidth) { // Upper left quarter
	fftRow= (i+(int)(fftHeight*0.5)) * fftWidth*2;
	fftCol= j+fftWidth;
      }
      else if(i<fftHeight*0.5 && j>=fftWidth) { // Upper right quarter
	fftRow= (i+(int)(fftHeight*0.5)) * fftWidth*2;
	fftCol= j-fftWidth;
      }
      else if(i>=fftHeight*0.5 && j<fftWidth) { // Lower left quarter
	fftRow= (i-(int)(fftHeight*0.5)) * fftWidth*2;
	fftCol= j+fftWidth;
      }
      else if(i>=fftHeight*0.5 && j>=fftWidth) { // Lower right quarter
	fftRow= (i-(int)(fftHeight*0.5)) * fftWidth*2;
	fftCol= j-fftWidth;
      }
      
      if((fftImage_buffer[fftRow+fftCol]==0.0) &&
	 (fftImage_buffer[fftRow+fftCol+1]==0.0))
	thisVal = 0;
      else 
	thisVal = (float)log(
			     (fftImage_buffer[fftRow+fftCol]*
			      fftImage_buffer[fftRow+fftCol])
			     +
			     (fftImage_buffer[fftRow+fftCol+1]*
			      fftImage_buffer[fftRow+fftCol+1])
			     );
      /* value = ((value - thresh_lo)*slopeb) + testlo; */
      thisVal = (thisVal-minF);
      if(thisVal <= ((maxF-minF)*loFFTscroll))
	thisVal = BLACK+8;
      else if(thisVal >= ((maxF-minF)*hiFFTscroll))
	thisVal = tryColors-2;
      else
	thisVal *= slopeb;
      fft_image_buffer1[((i*fftWidth)+thisPixel)]= 
	(unsigned short)thisVal;
      
    }
  }
  

  // Now put it back in "normal" space
  ksign = 0-1;     // Do a "reverse" FFT
  iform = 1;       // Data is imaginary 
  fft_2d_rob__(fftImage_buffer, &fftWidth, &fftHeight, &ksign, &iform);
  
  // Let's look at what scaling needs done
  // need to stretch properly here!!!
  sizeScaleFactor = 0.0;
  findMaxMin(fftImage_buffer, &maxF, &minF, &offset, FALSE);
  if(maxF==minF)
    sizeScaleFactor= 1.0;
  else 
    sizeScaleFactor = (tryColors-9.0) / (maxF-minF);

  /* Remember to add the img_bkg back in */
  offset = minF + (float)img_bkg;

  // Now put the back-transformed image into the display buffer
  for(j=0; j<nrow; j++) {
    for(i=0; i<ncol; i++) {
      
      jindx   = j*fftWidth*2 + i*2;
      
      // Remember to add the img_bkg back in
      thisFloat= fftImage_buffer[jindx] + (float)img_bkg;
      
      // if(imageType==BACKSTRETCH)
      back_stretch_fft_image_buffer1[((j*ncol)+i)]= 
	(unsigned short)((float)(thisFloat-offset) * sizeScaleFactor); 
      // imageType==BACK
      back_fft_image_buffer1[((j*ncol)+i)]= 
	(unsigned short)(thisFloat-offset); 
      
    }
  }

  minI = maxI = back_stretch_fft_image_buffer1[0];
  for(j=0; j<nrow; j++) {
    for(i=0; i<ncol; i++) {
      maxI = MAX(maxI,back_stretch_fft_image_buffer1[(j*ncol)+i]);
      minI = MIN(minI,back_stretch_fft_image_buffer1[(j*ncol)+i]);
    }
  }
  printf("After scaling stretched image min,max[%d,%d]\n",minI,maxI);

  minI = maxI = back_fft_image_buffer1[0];
  for(j=0; j<nrow; j++) {
    for(i=0; i<ncol; i++) {
      maxI = MAX(maxI,back_fft_image_buffer1[(j*ncol)+i]);
      minI = MIN(minI,back_fft_image_buffer1[(j*ncol)+i]);
    }
  }
  printf("Simulated image min,max[%d,%d]\n",minI,maxI);

  if(imageType==FFT)
    /* stuffImage(raw_fft_image_buffer1,image_width,image_height,256,FALSE); */
    stuffImage(raw_fft_image_buffer1,image_width,image_height,256,TRUE);
  else if(imageType==CTF)
    /* stuffImage(fft_image_buffer1,image_width,image_height,256,FALSE); */
    stuffImage(fft_image_buffer1,image_width,image_height,256,TRUE);
  else if(imageType==BACK) {
    /* stuffImage(back_fft_image_buffer1,image_width,
       image_height,256,FALSE); */
    stuffImage(back_fft_image_buffer1,image_width,image_height,256,TRUE);
  }
  else if(imageType==BACKSTRETCH) {
    /* stuffImage(back_stretch_fft_image_buffer1,image_width,
       image_height,256,FALSE); */
    stuffImage(back_stretch_fft_image_buffer1,image_width,
	       image_height,256,TRUE);
  }

  /* stuffImage(fft_image_buffer1,image_width,image_height,512,FALSE); */
  stuffImage(fft_image_buffer1,image_width,image_height,512,TRUE);
  
  
  for(i=0; i<fftHeight; i++)
    for(j=0; j<fftWidth*2; j++)
      fftImage_junk[(i*fftWidth*2)+j]= 1000.0;


  /* Try using the TSB library call ctf_multiply() to apply the FFT */
  units     = 2;   /* e.g. we're using nanometers for pixelSize */
  pixelSize = 1.0; /* ??? */
  pixelSize = pixSizeTextValue * 0.1;  /* Do in nm */
  tFactor = 0.0;
  fft_full = 1;
  /* We need to change the sign of angMajor here because our
   * XWindows based buffer is upside down.  When we write
   * to disk we need to use the sign just as given from
   * the interface */
  angMajor = 0.0 - majAngScrollValue ;

  cc      = chaberrationScrollValue; 
  beta    = spCoherenceScrollValue * spCoFactor; 
  dE      = tmCoherenceScrollValue;
  voltage = avoltageScrollValue;

  wave = wavel_(&avoltageScrollValue);
  ctf_multiply__(&ctfMode,
		fftImage_junk,
		&fftHeight, &fftWidth, 
		&units, 
		&pixelSize,
		&pi, &radian,
		&wave,
		&majDefocusScrollValue, &minDefocusScrollValue,
		&angMajor,
		&afractScrollValue,
		&spaberrationScrollValue, 
		&cc,&beta,&dE,&voltage,
		&tFactor, &fft_full
		);


  if(XtIsManaged(ctfCorrectScreen) && filter[0] != 0) { // Do ctf_divide
    ctf_divide__(&ctfMode_div, &filter[0], 
		 fftImage_buffer, 
		 &fftHeight, &fftWidth, 
		 &units, 
		 &pixelSize,
		 &pi,&radian,
		 &wave,
		 &majDefocusScrollValue, &minDefocusScrollValue,
		 &angMajor,
		 &afractScrollValue,
		 &spaberrationScrollValue, 
		 filt_fac,
		 &tFactor, &fft_full,
		 &cc,&beta,&dE,&voltage);
    // DO the ctf2D display also ...
    ctf_divide__(&ctfMode_div, &filter[0], 
		 fftImage_junk, 
		 &fftHeight, &fftWidth, 
		 &units, 
		 &pixelSize,
		 &pi,&radian,
		 &wave,
		 &majDefocusScrollValue, &minDefocusScrollValue,
		 &angMajor,
		 &afractScrollValue,
		 &spaberrationScrollValue, 
		 filt_fac,
		 &tFactor, &fft_full,
		 &cc,&beta,&dE,&voltage);
  }
  
  
  if(bandPass==PASSCTF && resXlo >=0 && resXhi >= 0) { // Let's band pass it    
    resoXlo = resXlo/512.0*resolutionScrollValue;
    resoXhi = resXhi/512.0*resolutionScrollValue;
    res_max = fftWidth * pixelSize * resoXlo;
    res_min = fftWidth * pixelSize * resoXhi;
    cut_min = cut_max = 0.0;
    
    fft_hiplop__(fftImage_junk, &fftWidth, &fftHeight, 
		 &res_min, &res_max,
		 &cut_min, &cut_max,
		 &middlePass, &fft_full);
  }
  
  /* Try this generalized rtn */
  disp_fft(fftImage_junk,ctf2D_image_buffer1,firstCtf2D);
  firstCtf2D = FALSE;
  firstCtf2D = TRUE;  /* Try this */


  if(XtIsManaged(ctfCorrectScreen)) // Already Drawn
    stuffImage(ctf2D_image_buffer1,image_width,image_height,768,TRUE); 
  else // Always do
      stuffImage(ctf2D_image_buffer1,image_width,image_height,768,TRUE); 

  DrawWindow();

  XFlush (display);

} // end function applyFFT



void calcCTF(int currentCtf)
{
  // function name: calcCTF
  // Calculate CTF curve based on scroll values. NOTE - the CTF
  // calculation is now done in ctf.f in the TSB fortran library.

  float wave,tmp,disp_tmp;
//  float thisCs, res_step, ixD4;
  float xscalef,yscalef,res_step2;
//  float spConstant1, spConstant2;
//  float tmConstant1, tmConstant2;
//  int   plotx,ploty,xhi,yhi;
  int   ix,ix1,ix2,iny,iy1,iy2,ycen,xcen;
  int   i,thisCtf;
  float tot;
  float pi,radian,degree;
  float beta, dE, d_star,psi,phi;
  float angMajor, singleDraw;
//#define THISCTFGC ((thisCtf==0) ? imageCTF_gcBLUE : imageCTF_gcBLUE)

  piraddeg_(&pi,&radian,&degree);

  /* RWa 7/10/96 let's try and limit the depth to which this is drawn 
   * in the y direction to 0.9 from 1.0
   */
  thisCtf = currentCtf;
  xscalef= 1;
  yscalef= 0.8;
  xcen = 256;
  ycen = 256;
  singleDraw = 250.0;
  if(XtIsManaged(ctfCorrectScreen)) { // DO special
      /* yscalef    = (float)((float)ctfCorrectHeight / maxCTFCorrectYvalue) /
	(float)ctfCorrectHeight; */
      yscalef= 1.0;
      singleDraw = (float)((float)ctfCorrectHeight / maxCTFCorrectYvalue);
      ycen = 512 - middleLine;
  }

//  xhi = yhi = 512;

  if(bandPass==NOPASS || bandPass==PASSCTF)
    {/* Don't do if we're band passing */
      /* Point to the one we're drawing */
      thisCtf= currentCtf;
      
      ctfDrawn[thisCtf]= TRUE;
      
      /* PHI = ANGLE OF FOCUS_MAJ WRT X-DIRECTION */
      /* We need to change the sign of angMajor here because our
       * XWindows based buffer is upside down.  When we write
       * to disk we need to use the sign just as given from
       * the interface */
      angMajor = 0.0 - majAngScrollValue ;
      angMajor = majAngScrollValue ; /* We need the sign in this space
				      * for focus_astig() */
      phi = angMajor * radian;

      /* Let's get our scroll bar values */
      cs      = spaberrationScrollValue; 
      cc      = chaberrationScrollValue; 

      pfract  = pfractScrollValue;
      afract  = afractScrollValue;
      defocus = defocusScrollValue;
      voltage = avoltageScrollValue;
      
      wavelength = 1230.0/sqrt(voltage + (1.0e-6)*(voltage*voltage));
      wave = wavelength/1000.0;		/*! convert to nm units */
   
      /* Now calc the spatial and temporal coherence constants */
//      spConstant1 = ( (PI * PI) * 
//		     (spCoherenceScrollValue * spCoherenceScrollValue));
//      spConstant2 = spaberrationScrollValue * (wave * wave);
//      tmConstant1 = ( (PI * PI) / (16.0 * log(2.0)));
//      tmConstant2 = (chaberrationScrollValue * chaberrationScrollValue ) *
//	(wave * wave) *
//	(tmCoherenceScrollValue * tmCoherenceScrollValue);
	 
//      thisCs   = cs*1000000.0;	  		/*! convert to nm units */
      
      /* RWA try setting this guy with the value we get from the scroll
       * bar for resolution.  (0.0 - 1.0)   7/10/96
       */
      res_limit= resolutionScrollValue;

      ix1 = xcen - (int)(255.0*xscalef);
      ix2 = xcen + (int)(255.0*xscalef);
      /* Try this to see if we can't get this calculated past 255 in X */
      ix2 = xcen + (int)(510.0*xscalef);


      res_step2  = res_limit/(ix2-ix1);

//      res_step = res_limit/(ctf_width*xscalef + 1.0);
      
      iny = 0;

      /* Let's see what defocus we want based on the angle chosen
       * from the interface. */
      psi =  userAngScrollValue * radian;

      focus_astig__(&defocus ,
		 &majDefocusScrollValue,
		 &minDefocusScrollValue,
		 &psi,
		 &phi);		    
      sprintf(outStr,"%.3f um",defocus);
      XmTextSetString(userDefocusValue,outStr);

      for( ix = (ix1+1);ix<(ix2);ix++) 
	{/* for ix */

//	  ixD4 = (float)((float)ix / 4.0);
                                 /* What exactly is our spatial 
					      * frequency?
					      * seems sometimes to 
					      * be /4.0  ??? */

	  d_star = ((float)(ix -ix1 -1) / 512.0) *
	    resolutionScrollValue; /* in nm-1 */
	  /* Convert to mRad or whatever*/
	  beta = spCoherenceScrollValue * spCoFactor; 
	  dE   = tmCoherenceScrollValue;

	  wavelength = wavel_(&voltage);
	  ctf_func__(&tmp, &d_star, &pi, &wavelength, &defocus,
		     &afract, &cs, &cc, &beta, &dE, &voltage);
	  /*    CTFMODE:  
	   *    = 0 DO NOTHING!     
	   *    = 1 CORRECT PHASES AND AMPLITUDES   
	   *    = 2 FLIP PHASES WHERE NECESSARY; LEAVE AMPLITUDES UNCHANGED 
	   *    = 3 ADJUST AMPLITUDES; LEAVE PHASES UNCHANGED      */
	  disp_tmp = tmp;
	  if(ctfMode == 3) // AMPs only
	    disp_tmp = 0.0 - fabs(tmp);
	  else if(ctfMode == 2) {/* PHASEs only */
	    if(tmp < 0.0)
	      disp_tmp = -1.0;
	    else if(tmp > 0.0)
	      disp_tmp = 1.0;
	  }
	  else if(ctfMode == 0) // None
	    disp_tmp = -1.0;
	  
	  ctf_vals[currentCtf][ix].tmp = tmp;
	  ctf_vals[currentCtf][ix].disp_tmp = disp_tmp;
	  
	  iy1       = (int)((disp_tmp)*yscalef*singleDraw) + ycen;

/*	  if(iny != 0) {
	    plotx     = res_step2*(ix-ix1-1);
	    ploty     = 0.0-tmp;
//	    write(10,'(2f10.5)') plotx,ploty 
	  }
*/	  
	  d_star = ((float)(ix - ix1) / 512.0) *
	    resolutionScrollValue; /* in nm-1 */
	  /* Convert to mRad or whatever*/
	  beta = spCoherenceScrollValue * spCoFactor; 
	  dE   = tmCoherenceScrollValue;

	  wavelength = wavel_(&voltage);
	  ctf_func__(&tmp, &d_star, &pi, &wavelength, &defocus, 
		     &afract, &cs, &cc, &beta, &dE, &voltage);
	  /*    CTFMODE:  
	   *    = 0 DO NOTHING!     
	   *    = 1 CORRECT PHASES AND AMPLITUDES   
	   *    = 2 FLIP PHASES WHERE NECESSARY; LEAVE AMPLITUDES UNCHANGED 
	   *    = 3 ADJUST AMPLITUDES; LEAVE PHASES UNCHANGED      */
	  disp_tmp = tmp;
	  if(ctfMode == 3) // AMPs only
	    disp_tmp = 0.0 - fabs(tmp);
	  else if(ctfMode == 2) { // PHASEs only
	    if(tmp < 0.0)
	      disp_tmp = -1.0;
	    else if(tmp > 0.0)
	      disp_tmp = 1.0;
	  }/* PHASEs only */
	  else if(ctfMode == 0) // NONE
	    disp_tmp = -1.0;
	  
	  iy2       = (int)((disp_tmp)*yscalef*singleDraw) + ycen;
	  
	  ctf_vals[currentCtf][ix].iy1 = iy1;
	  ctf_vals[currentCtf][ix].iy2 = iy2;
	  
	  
	  if(sumCurveSwitch) {/* Do they want the sum of the two curves drawn? */
	    ctf_vals[MAXCURVE][ix].tmp = 0.0;
	    ctf_vals[MAXCURVE][ix].iy1 = 0.0;
	    ctf_vals[MAXCURVE][ix].iy2 = 0.0;
	    ctf_vals[MAXCURVE][ix].fy1 = 0.0;
	    ctf_vals[MAXCURVE][ix].fy2 = 0.0;
	    tot= 0.0;
	    for(i=0; i<topCurve; i++) {/* Look at all */
	      float wt;
	      /* Find a way to use the weightings[] we calculated
	       * when param files read in to weight the SUM curve */
	      if(useSum[i]) {/* OK to use for sum */
		wt = weightings[i];
		ctf_vals[MAXCURVE][ix].tmp += fabs(wt*
						   ctf_vals[i][ix].tmp);
		ctf_vals[MAXCURVE][ix].disp_tmp += 
		  fabs(wt*ctf_vals[i][ix].disp_tmp);
		ctf_vals[MAXCURVE][ix].fy1 += (fabs(wt*
						    (float)(ctf_vals[i][ix].iy1-
							    ycen)));
		ctf_vals[MAXCURVE][ix].fy2 += (fabs(wt*
						    (float)(ctf_vals[i][ix].iy2-
							    ycen)));
		tot += wt;
	      }
	    }
	    if(tot > 0.0) {/* OK to calc */
	      ctf_vals[MAXCURVE][ix].tmp /= (float)tot;
	      ctf_vals[MAXCURVE][ix].disp_tmp /= (float)tot;
	      ctf_vals[MAXCURVE][ix].iy1 = (ycen -
					    (int)(ctf_vals[MAXCURVE][ix].fy1/
						  (float)tot));
	      ctf_vals[MAXCURVE][ix].iy2 = (ycen -
					    (int)(ctf_vals[MAXCURVE][ix].fy2/
						  (float)tot));
	    }
	  }
	  
	  if(resXlo >=0 && resXhi >= 0) {/* Only do if we've got values */
	    if((bandPass==PASSCTF) && middlePass && 
	       ((ix <=resXlo) && ix >=resXhi)) {/* Init these out */
	      ctf_vals[currentCtf][ix].tmp= 0.0;
	      ctf_vals[currentCtf][ix].disp_tmp= 0.0;
	      if(ix==resXlo)
		ctf_vals[currentCtf][ix].iy1= iy1;
	      else iy1= ctf_vals[currentCtf][ix].iy1= ycen;
	      if(ix==resXhi)
		ctf_vals[currentCtf][ix].iy2= iy2;
	      else 
		iy2= ctf_vals[currentCtf][ix].iy2= ycen;
	    }
	    else if((bandPass==PASSCTF) && !middlePass && 
		    ((ix >=resXlo) || (ix<=resXhi))) {/* Init these out */
	      ctf_vals[currentCtf][ix].tmp= 0.0;
	      ctf_vals[currentCtf][ix].disp_tmp= 0.0;
	      if(ix==resXlo)
		ctf_vals[currentCtf][ix].iy1= iy1;
	      else iy1= ctf_vals[currentCtf][ix].iy1= ycen;
	      if(ix==resXhi)
		ctf_vals[currentCtf][ix].iy2= iy2;
	      else 
		iy2= ctf_vals[currentCtf][ix].iy2= ycen;
	      if(resXhi > resXlo)
		iy1=iy2=ycen;
	    }
	  }
	  
	  if(iny != 1) {
	    /* Draw in image */
	    XDrawLine(display, ctf_image_window1, draw_gc[curCtf],
		      (ix -1), (ctf_height-iy1),
		      (ix), (ctf_height-iy2));
	  }
	}/* for ix */
    }/* Don't do if we're band passing */
  
  for(i=0;i<topCurve;i++) {/* Try all of them */
    if((i != curCtf) && alwaysDraw[i]) {/* Don't draw unless calculated */
      for( ix = (ix1+1);ix<ix2;ix++) {/* for ix */
	XDrawLine(display, ctf_image_window1, draw_gc[i],
		  (ix -1), (ctf_height-ctf_vals[i][ix].iy1),
		  (ix), (ctf_height-ctf_vals[i][ix].iy2));
      }
    }
  }
  
  /* Now do the SUM if needed */
  if(sumCurveSwitch) {/* do sum */
    thisCtf= MAXCURVE;
    for( ix = (ix1+1);ix<ix2;ix++) {/* for ix */
      XDrawLine(display, ctf_image_window1, imageCTF_gcBLACK,
		(ix -1), (ctf_height-ctf_vals[thisCtf][ix].iy1),
		(ix), (ctf_height-ctf_vals[thisCtf][ix].iy2));
    }
  }

  if(XtIsManaged(ctfCorrectScreen))
    {/* Try drawing those curves here */
      float thisFactor, lastFactor;
      lastFactor = 1.0;
      for( ix = (ix1+1);ix<ix2;ix++)
	{/* for ix */

	  thisFactor = 1.0;
	  if((ctfMode_div != 0) && ctfMeth1) {/* Do method 1 */
	    iy1 = 512 - (middleLine - (int)(
					    1.0 /
					    (fabs(ctf_vals[thisCtf][ix].tmp) 
					     +
					     filt_fac[0])
					    *
					    (float)singleDraw));
	    iy2 = 512 - (middleLine - (int)(
					    1.0 /
					    (fabs(ctf_vals[thisCtf][ix+1].tmp) +
					     filt_fac[0])
					    *
					    (float)singleDraw));
	    thisFactor *=  (1.0 /
			    (fabs(ctf_vals[thisCtf][ix+1].tmp) +
			     filt_fac[0]));
	    if(ctfMode_div == 2) {/* Do for PHASE ONLY case */
	      iy1 = iy2 = 182;
	    }
	    XDrawLine(display, ctf_image_window1, image_gcRED,
		      (ix -1), (ctf_height-iy1),
		      (ix), (ctf_height-iy2));
	  }
	  if((ctfMode_div != 0) && ctfMeth2) {/* Do method 2 */
	    iy1 = 512 - (middleLine - (int)(
					    (1.0 /
					     (fabs(ctf_vals[thisCtf][ix].tmp) +
					      (filt_fac[0] * (1.0-fabs(ctf_vals[thisCtf][ix].tmp)))))
					    
					    *
					    (float)singleDraw));
	    iy2 = 512 - (middleLine - (int)(
					    1.0 /
					    (fabs(ctf_vals[thisCtf][ix+1].tmp) +
					     (filt_fac[0] * (1.0-fabs(ctf_vals[thisCtf][ix+1].tmp))))
					    *
					    (float)singleDraw));
	    thisFactor *=  (
			    1.0 /
			    (fabs(ctf_vals[thisCtf][ix+1].tmp) +
			     (filt_fac[0] * (1.0-fabs(ctf_vals[thisCtf][ix+1].tmp))))
			    );
	    if(ctfMode_div == 2) {/* Do for PHASE ONLY case */
	      iy1 = iy2 = 182;
	    }
	      XDrawLine(display, ctf_image_window1, image_gcRED,
			(ix -1), (ctf_height-iy1),
			(ix), (ctf_height-iy2));
	    }
	  if((ctfMode_div != 0) && ctfMeth3) {/* Do method 3 */
	    iy1 = 512 - (middleLine - (int)(
					    (fabs(ctf_vals[thisCtf][ix].tmp)  /
					     ((fabs(ctf_vals[thisCtf][ix].tmp) * 
					       fabs(ctf_vals[thisCtf][ix].tmp) ) +
					      filt_fac[0]))
					    *
					    (float)singleDraw));
	    
	    iy2 = 512 - (middleLine - (int)(
					    (fabs(ctf_vals[thisCtf][ix+1].tmp)  /
					     ((fabs(ctf_vals[thisCtf][ix+1].tmp) * 
					       fabs(ctf_vals[thisCtf][ix+1].tmp) ) +
					      filt_fac[0]))
					    *
					    (float)singleDraw));
	    thisFactor *=  (
			    (fabs(ctf_vals[thisCtf][ix+1].tmp)  /
			     ((fabs(ctf_vals[thisCtf][ix+1].tmp) * 
			       fabs(ctf_vals[thisCtf][ix+1].tmp) ) +
			      filt_fac[0]))
			    );
	    if(ctfMode_div == 2) {/* Do for PHASE ONLY case */
	      iy1 = iy2 = 182;
	    }
	    XDrawLine(display, ctf_image_window1, image_gcRED,
		      (ix -1), (ctf_height-iy1),
		      (ix), (ctf_height-iy2));
	  }
	  if((ctfMode_div != 0) && ctfMeth4) {/* Do method 4 */
	    iy1 = 512 - (middleLine - (int)(
					    (fabs(ctf_vals[thisCtf][ix].tmp)  /
					     ((fabs(ctf_vals[thisCtf][ix].tmp) * 
					       fabs(ctf_vals[thisCtf][ix].tmp) ) +
					      (filt_fac[0] * (1.0-fabs(ctf_vals[thisCtf][ix].tmp)))))
					    *
					    (float)singleDraw));
	    
	    iy2 = 512 - (middleLine - (int)(
					    (fabs(ctf_vals[thisCtf][ix+1].tmp)  /
					     ((fabs(ctf_vals[thisCtf][ix+1].tmp) * 
					       fabs(ctf_vals[thisCtf][ix+1].tmp) ) +
					      (filt_fac[0] * (1.0-fabs(ctf_vals[thisCtf][ix+1].tmp)))))
					    *
					    (float)singleDraw));
	    thisFactor *=  (
			    (fabs(ctf_vals[thisCtf][ix+1].tmp)  /
			     ((fabs(ctf_vals[thisCtf][ix+1].tmp) * 
			       fabs(ctf_vals[thisCtf][ix+1].tmp) ) +
			      (filt_fac[0] * (1.0-fabs(ctf_vals[thisCtf][ix+1].tmp)))))
			    );
	    if(ctfMode_div == 2) {/* Do for PHASE ONLY case */
	      iy1 = iy2 = 182;
	    }/* Do for PHASE ONLY case */
	    XDrawLine(display, ctf_image_window1, image_gcRED,
		      (ix -1), (ctf_height-iy1),
		      (ix), (ctf_height-iy2));
	  }
	  if((ctfMode_div != 0) && ctfMeth5) {/* Do method 5 */
	    int units;
	    float realIX,fft_rad_test,pixelSize, tmp;
	    realIX = (float)ix+1.0;
	    pixelSize = pixSizeTextValue * 0.1;  /* Do in nm */
	    units = 2;  /* Always nm for us here */
	    ctf_firstpeak_pft__(&defocus, &wavelength, &realIX, &pixelSize, 
				&fftWidth, &cs, &pi, &units, &afractScrollValue);
	    /* 0.25 for 256 fft and 1024 theoretical display and
	     * do resolution in nm also   WHY do I have to fudge the 
	     * pixelSize?
	     */
	    tmp = ((resolutionScrollValue)*(1.0/4.0))*(pixelSize/0.5);
	    fft_rad_test = ((float)ix+1.0) * tmp;
	    if ((float)fft_rad_test <= realIX) {/* Do less than first node */
	      iy1 = 512 - (middleLine - (int)(
					      (1.0 /
					       (fabs(ctf_vals[thisCtf][ix].tmp) +
						(filt_fac[0] * (1.0-fabs(ctf_vals[thisCtf][ix].tmp)))))
					      
					      *
					      (float)singleDraw));
	      iy2 = 512 - (middleLine - (int)(
					      1.0 /
					      (fabs(ctf_vals[thisCtf][ix+1].tmp) +
					       (filt_fac[0] * (1.0-fabs(ctf_vals[thisCtf][ix+1].tmp))))
					      *
					      (float)singleDraw));
	      thisFactor *=  (
			      1.0 /
			      (fabs(ctf_vals[thisCtf][ix+1].tmp) +
			       (filt_fac[0] * (1.0-fabs(ctf_vals[thisCtf][ix+1].tmp))))
			      );
	    }
	    else {/* Do normal */
	      iy1 = 512 - (middleLine - (int)(
					      (fabs(ctf_vals[thisCtf][ix].tmp)  /
					       ((fabs(ctf_vals[thisCtf][ix].tmp) * 
						 fabs(ctf_vals[thisCtf][ix].tmp) ) +
						(filt_fac[1] * (1.0-fabs(ctf_vals[thisCtf][ix].tmp)))))
					      *
					      (float)singleDraw));
	      
	      iy2 = 512 - (middleLine - (int)(
					      (fabs(ctf_vals[thisCtf][ix+1].tmp)  /
					       ((fabs(ctf_vals[thisCtf][ix+1].tmp) * 
						 fabs(ctf_vals[thisCtf][ix+1].tmp) ) +
						(filt_fac[1] * (1.0-fabs(ctf_vals[thisCtf][ix+1].tmp)))))
					      *
					      (float)singleDraw));
	      thisFactor *=  (
			      (fabs(ctf_vals[thisCtf][ix+1].tmp)  /
			       ((fabs(ctf_vals[thisCtf][ix+1].tmp) * 
				 fabs(ctf_vals[thisCtf][ix+1].tmp) ) +
				(filt_fac[1] * (1.0-fabs(ctf_vals[thisCtf][ix+1].tmp)))))
			      );
	    }
	    if(ctfMode_div == 2) {/* Do for PHASE ONLY case */
	      iy1 = iy2 = 182;
	    }
	    XDrawLine(display, ctf_image_window1, image_gcRED,
		      (ix -1), (ctf_height-iy1),
		      (ix), (ctf_height-iy2));
	  }
	  
	  /* Save for later */
	  ctf_vals[thisCtf][ix].fudge = thisFactor;
	  if(ix==(ix1+1))
	    lastFactor = thisFactor;

	  if(dispCtfCorrect) {/* Now apply the factors above to ctf curve */
	    if(ctfMode_div == 3) {/* Do for AMP only */
	      if(ctfMode == 3) {/* DO for amps only */
		if(ctf_vals[currentCtf][ix].tmp > 0.0) {
		  iy1 = 512 - (middleLine + (int)(
						  (ctf_vals[thisCtf][ix].tmp * 
						   lastFactor)
						  *
						  (float)singleDraw));
		  
		  iy2 = 512 - (middleLine + (int)(
						  (ctf_vals[thisCtf][ix+1].tmp * 
						   thisFactor)
						  *
						  (float)singleDraw));
		}
		else {/* normal */
		  iy1 = 512 - (middleLine - (int)(
						  (ctf_vals[thisCtf][ix].tmp * 
						   lastFactor)
						  *
						  (float)singleDraw));
		  
		  iy2 = 512 - (middleLine - (int)(
						  (ctf_vals[thisCtf][ix+1].tmp * 
						   thisFactor)
						  *
						  (float)singleDraw));
		}
	      }
	      else {/* normal */
		iy1 = 512 - (middleLine - (int)(
						(ctf_vals[thisCtf][ix].tmp * 
						 lastFactor)
						*
						(float)singleDraw));
		
		iy2 = 512 - (middleLine - (int)(
						(ctf_vals[thisCtf][ix+1].tmp * 
						 thisFactor)
						*
						(float)singleDraw));
	      }
	    }
	    else if(ctfMode_div == 2) {/* Do for PHASE only */
	      iy1 = ctf_vals[currentCtf][ix].iy1;
	      iy2 = ctf_vals[currentCtf][ix].iy2;
	      if(ctfMode == 3) {/* DO for amps only */
		if(ctf_vals[currentCtf][ix].tmp > 0.0) {
		  iy1 = iy1 - (2*(middleLine - (ctf_height-iy1)));
		  iy2 = iy2 - (2*(middleLine - (ctf_height-iy2)));
		}
	      }
	      else {/* Don't do for amp only */
		if((ctf_height - iy1) < middleLine)
		  iy1 = iy1 - (2*(middleLine - (ctf_height-iy1)));
		if((ctf_height - iy2) < middleLine)
		  iy2 = iy2 - (2*(middleLine - (ctf_height-iy2)));
	      }
	    }
	    else {/* Do for amp and phase */
	      if(ctfMode == 3) {/* DO for amps only */
		if(ctf_vals[currentCtf][ix].tmp > 0.0) {
		  iy1 = 512 - (middleLine - (int)(
						  (fabs(ctf_vals[thisCtf][ix].tmp) * 
						   lastFactor)
						  *
						  (float)singleDraw));
		  
		  iy2 = 512 - (middleLine - (int)(
						  (fabs(ctf_vals[thisCtf][ix+1].tmp) * 
						   thisFactor)
						  *
						  (float)singleDraw));
		  
		}
		else {/* normal */
		  iy1 = 512 - (middleLine + (int)(
						  (fabs(ctf_vals[thisCtf][ix].tmp) * 
						   lastFactor)
						  *
						  (float)singleDraw));
		  
		  iy2 = 512 - (middleLine + (int)(
						  (fabs(ctf_vals[thisCtf][ix+1].tmp) * 
						   thisFactor)
						  *
						  (float)singleDraw));
		  
		}
	      }
	      else {/* normal */
		iy1 = 512 - (middleLine + (int)(
						(fabs(ctf_vals[thisCtf][ix].tmp) * 
						 lastFactor)
						*
						(float)singleDraw));
		
		iy2 = 512 - (middleLine + (int)(
						(fabs(ctf_vals[thisCtf][ix+1].tmp) * 
						 thisFactor)
						*
						(float)singleDraw));
		
	      }
	    }
	    if((ctfMode_div != 0) &&
	       (ctfMeth1 || ctfMeth2 || ctfMeth3 || ctfMeth4 || ctfMeth5 ))
	      XDrawLine(display, ctf_image_window1, image_gcYELLOW,
			(ix -1), (ctf_height-iy1),
			(ix), (ctf_height-iy2));
	  }/* Now apply the factors above to ctf curve */
	  lastFactor = thisFactor;
	}/* for ix */
      
      
    }/* Try drawing those curves here */
  
  
} // end function calcCTF



void mainEmmap_sub_UNIX(char *fName, char *outFile)
{
  // function name: mainEmmap_sub_UNIX
  // Read PIF files. Rtn originally taken from robem.

  int k, l, m, logicalRecordLength;
  int numCols, numRows, numSecs, nimg, file_id1, file_id2;
  float img_min, img_max;
//  int inWidth, inHeight;
  int junk, junk1, junk2, margin;
  BYTE thisBkg;
  short junkShort;
  float *float_buffer;
  float thisPixel, map_scalef;
  short inBuffer[10000];

  FILE *fIn, *fOut;
  char title[512];
  float scaleFactor, dupFactor, minPixel, maxPixel;
  BYTE thisByte;

  // Init
  fOut= fIn= 0;
  fIn = fopen(fName, "r");
  fOut= fopen(outFile, "w");
  
  if(fIn) { // Read the file
    
    // Read the file type header
    fread(&junk1,sizeof(short),1,fIn); 
    fread(&file_id1,sizeof(int),1,fIn); 
    fread(&junk2,sizeof(short),1,fIn); 
    
    file_id2= 0;
    file_id2 |= junk1;
    file_id2 |= (((file_id1) & 0177777) << 16);
    
    // Read off the rest of the 512 byte record
    fread(title, sizeof(char), 72, fIn);  
    
    // EndofRecord short???
    fread(&junkShort, sizeof(short), 1, fIn);  
    
    fread(&numCols, sizeof(int), 1, fIn);  
    fread(&numRows, sizeof(int), 1, fIn);  
    fread(&nimg, sizeof(int), 1, fIn);  
    fread(&img_min, sizeof(float), 1, fIn);  
    fread(&img_max, sizeof(float), 1, fIn);  

    // aoverb and map_abang map_scalef
    fread(&junk, sizeof(int), 1, fIn);  
    fread(&junk, sizeof(float), 1, fIn);  
    fread(&map_scalef, sizeof(int), 1, fIn);  
    
    // This isn't working right.  Need to convert these properly.
    // vaxtie3((unsigned int *)&map_scalef);
    // map_scalef /= 4.0;
    printf("Map scale[%f]\n",map_scalef);
    nimg = MIN(MAXBOX,nimg);
    
    // Read eor junk
    fread(&junkShort, sizeof(short), 1, fIn);  
    
    // a_cell,b_cell,c_cell
    fread(&junk, sizeof(float), 1, fIn);  
    fread(&junk, sizeof(float), 1, fIn);  
    fread(&junk, sizeof(float), 1, fIn);  
    
    // alp,bet,gam
    fread(&junk, sizeof(float), 1, fIn);  
    fread(&junk, sizeof(float), 1, fIn);  
    fread(&junk, sizeof(float), 1, fIn);  
    
    // Read eor junk
    fread(&junkShort,sizeof(short),1,fIn);  
    
    // dela,delb,delc
    fread(&junk, sizeof(float), 1, fIn);  
    fread(&junk, sizeof(float), 1, fIn);  
    fread(&junk, sizeof(float), 1, fIn);  
    
    // There's no t_matrix???
    for(l=0; l<6; l++)
      fread(&junk, sizeof(float), 1, fIn);   
    
    // Read eor junk
    fread(&junkShort, sizeof(short), 1, fIn);  
    
//    inHeight = numRows;
//    inWidth  = numCols;
    numSecs = nimg;
    
    // Allocate buffer
    printf("Attempting to allocate [%lu] bytes [%d,%d,%d]\n",
	   sizeof(unsigned short)*numCols*numRows*numSecs,
	   numCols, numRows, numSecs);
    
    float_buffer = (float *)malloc(sizeof(float)*
				   numCols*numRows*numSecs);

    if(float_buffer==NULL) {
      printf("Can't allocate enough memory.\n");
      goto bugOut;
    }
    
    // This is the length of each record read
    logicalRecordLength = (numCols);
    if(logicalRecordLength > 10000)
      printf("!!!!!!! Buffer overflow!!!!!!!\n");
    
    minPixel = 99999.9;
    maxPixel= -99999.9;

    for(l=0; l<numSecs; l++) { // sections
      fflush(stdout);
      for(k=numRows-1; k>=0; k--) { // Do all rows
	// Now read the "meat" of all the sections in the MAP file
	fread(inBuffer, sizeof(unsigned short), logicalRecordLength, fIn);
	// Read EOR short
	fread(&junk, sizeof(unsigned short), 1, fIn);
	
	for(m=0;m<numCols;m++) { // Convert
	  thisPixel=
	    *(float_buffer +((l*(numCols*numRows))+
			     (k*numCols)+m)) =
	    (float)((float)inBuffer[m] * 
		    map_scalef);
	  if(thisPixel > maxPixel)
	    maxPixel= thisPixel;
	  if(thisPixel < minPixel)
	    minPixel= thisPixel; 
	}
      }
    }
    
    fclose(fIn);
    
  }/* Read the file */
  
  scaleFactor = (float)(255.0 / ((float)maxPixel-minPixel));
  dupFactor = 512.0 / (float)numCols;
  dupFactor = 2.0;
  margin = (int)(512 - ((int)((int)numCols * (int)dupFactor))) / (int)2;

  thisBkg = (BYTE)((float_buffer[(169+4)]-minPixel)*scaleFactor);

  for(i=0; i<margin; i++) {
    thisByte = (BYTE)thisBkg;
    for(j=0; j<512; j++) {
      fwrite(&thisByte, sizeof(BYTE), 1, fOut);  
    }
  }

  for(i=0; i<numRows; i++) {
    for(l=0; l<(int)dupFactor; l++) {
      thisByte = (BYTE)thisBkg;
      for(j=0; j<margin; j++)
	fwrite(&thisByte,sizeof(BYTE),1,fOut);
      for(j=0; j<numCols; j++) {
	thisByte = (BYTE)((float_buffer[((i*numCols)+j)]-minPixel) * 
			  scaleFactor);
	for(k=0; k<(int)dupFactor; k++)
	  fwrite(&thisByte,sizeof(BYTE),1,fOut);  
      }
      thisByte = (BYTE)thisBkg;
      for(j=0; j<margin; j++)
	fwrite(&thisByte,sizeof(BYTE),1,fOut);  
      /* In case we've got some left over */
      if(((margin*2)+(numCols*(int)dupFactor)) <(512))
	for(k=0; k<(512-((margin*2)+(numCols*(int)dupFactor))); k++)
	  fwrite(&thisByte,sizeof(BYTE),1,fOut);  
    }
  }
  
  for(i=0;i<margin+1;i++) {
    for(j=0;j<512;j++) {
      thisByte = (BYTE)thisBkg;
      fwrite(&thisByte,sizeof(BYTE),1,fOut);  
    }
  }

 bugOut:;
  
  fclose(fIn);
  fclose(fOut);

  fOut= fopen("/bio/reo/rwa/devel/ctfDisp/circle.apx","w");
  for(i=0; i<512; i++) {
    for(j=0; j<512; j++) {
      float radius;
      radius = sqrt((256.0-i)*(256.0-i) + (256.0-j)*(256.0-j));
      if(radius > 128.0)
	thisByte= 50;
      else 
	thisByte=127;
      fwrite(&thisByte,sizeof(BYTE),1,fOut);  
    }
  }

  fclose(fOut);

} // end function mainEmmap_sub_UNIX



/************************************************************************
 * OS transfer point.  The main routine does all the one-time setup and *
 * then calls XtAppMainLoop.                                            *
 ************************************************************************/
int main(argc, argv)
int argc;                           /* Command line argument count. */
char *argv[];                       /* Pointers to command line args. */
{/* MAIN */
  char pathenv[1024];
  int mylen;
  String fallbacks[] =
    { "*fontList:-dec-terminal-bold-r-normal--14-140-75-75-c-80-iso8859-1=tag",
	NULL
	};

  /* Show them our version number */
  sprintf(versionStr,"%s",banner("ctfDisp"));

  inited= FALSE;

  /* Initialize MRM */
  MrmInitialize();                 

  toplevel_widget= XtVaAppInitialize(&app_context, 
				     argv[0],
				     (XrmOptionDescRec *) NULL, 
				     (Cardinal) 0, 
				     &argc, 
				     argv, 
				     fallbacks,
				     (ArgList) NULL);

  display = XtDisplay(toplevel_widget);
  
  /* Add our remapping of keys to widgets */
  XtAppAddActions(app_context,rwa_action_table, 1);


  // Construct ':' delimited path(s) to be searched for uid file
  memset(pathenv, '\0', 1024);
  strcpy(pathenv, argv[0]);
  for (i=strlen(pathenv)-1; i>=0; i--) {
    if(pathenv[i] == '/') {
      pathenv[i] = ':';
      break;
    }
    else
      pathenv[i] = '\0';
  }
  strcat(pathenv, getenv("PATH"));

  db_filename_vec[0] = (char *)malloc(sizeof(char)*512);
  memset(db_filename_vec[0], '\0', 512);
  j = 0;

  // Check each element of path for uid file
  for(i=0; i<strlen(pathenv); i++) {
    if(pathenv[i] != ':') {
      db_filename_vec[0][j] = pathenv[i];
      j++;
    } 
    else {
      strcat(db_filename_vec[0], "/ctfdisp.uid");
      if( (fIn = fopen(db_filename_vec[0],"r")) != NULL ) {
	fclose(fIn);
	break;
      } 
      else {
	j = 0;
	memset(db_filename_vec[0], '\0', 512);
      }
    }
  }

  if(j == 0) {
    printf("Unable to find ctfDisp.uid in PATH\n");
    printf("Make sure that the uid file is in same directory as ctfDisp\n");
    exit(0);
  }

  printf("Full path to UID file: [%s]\n",db_filename_vec[0]);

  // Save path for later use
  strcpy(path, db_filename_vec[0]);
  mylen = strlen(path);
  for (i=1; i<12; i++)
    path[mylen-i] = '\0';
  

  // Open UID file in hierarchy
  db_filename_num = (sizeof db_filename_vec / sizeof db_filename_vec [0]);
  if (MrmOpenHierarchy(db_filename_num, // Number of files
		       db_filename_vec, // Array of file names
		       NULL,            // Default OS extenstion
		       &s_MrmHierarchy) // Pointer to returned MRM ID
      !=MrmSUCCESS)
    s_error("can't open hierarchy");
  
  
  /* Register the items MRM needs to bind for us. */
  MrmRegisterNames(reglist, reglist_num);
  
  /* Go get the main part of the application. */
  if (MrmFetchWidget(s_MrmHierarchy, "S_MAIN_WINDOW", toplevel_widget,
		     &main_window_widget, &dummy_class) != MrmSUCCESS)
    s_error("can't fetch main window");

  /* Go get the infoScreen part of the application. */
  if (MrmFetchWidget(s_MrmHierarchy, "infoMessageBox", toplevel_widget,
		     &infoBox, &dummy_class) != MrmSUCCESS)
    s_error("can't fetch info window");
  XtUnmanageChild(XmMessageBoxGetChild(infoBox,XmDIALOG_HELP_BUTTON));

  /* Go get the ctfScreen part of the application. */
  if (MrmFetchWidget(s_MrmHierarchy, "ctfscreen", toplevel_widget,
		     &ctfscreen, &dummy_class) != MrmSUCCESS)
    s_error("can't fetch info window");

  /* Go get the ctfScreen part of the application. */
  if (MrmFetchWidget(s_MrmHierarchy, "resoExample", toplevel_widget,
		     &resoExample_screen, &dummy_class) != MrmSUCCESS)
    s_error("can't fetch info window");

  /* Go get the ctfCorrectScreen part of the application. */
  if (MrmFetchWidget(s_MrmHierarchy, "ctfCorrectScreen", toplevel_widget,
		     &ctfCorrectScreen, &dummy_class) != MrmSUCCESS)
    s_error("can't fetch info window");


  /* Initialize our C code */
  init_globals(argc, argv);

  /* Manage the main part and realize everything.  The interface comes up
   * on the display now.  Manage all of the screens now so we can alter
   * what we need to in XLibInitialize() - Unmanage the unneeded ones
   * thereafter.
   */
  XtManageChild(main_window_widget);
  XtManageChild(infoBox);
  XtManageChild(ctfscreen);
  XFlush(display);
  XSync(display,FALSE);

  XlibInitialize();
  XtUnmanageChild(infoBox);

  /* Create our cursors */
  watch_cursor= XCreateFontCursor(display, XC_watch);
  cross_cursor= XCreateFontCursor(display, XC_cross);
  XFlush(display);
  XSync(display,FALSE);

  inited= TRUE;

  /* Now setup user defaults */
  userDefaults();

  DrawCTFWindow(FALSE);
  
  applyFFT();

  /* See if they want to do some automatic operations */
  checkAutomatic();

  /* Sit around forever waiting to process X-events.  We never leave
   * XtAppMainLoop. From here on, we only execute our callback routines. */
  XtAppMainLoop(app_context);
  
  return(1);

}/* MAIN */




/*-----------------------------------------------------------------------*/
/*-----------------------------------------------------------------------*/
/*-----------------------------------------------------------------------*/
/*************************************************************************
 * NON-Callback Routines - These routines are of the more generic C      *
 * language variety and are not called directly from X.  The callback    *
 * routines that are called by X do make heavy use of these, however.    *
 *************************************************************************/
/*-----------------------------------------------------------------------*/
/*-----------------------------------------------------------------------*/
/*-----------------------------------------------------------------------*/

/*************************************************************************
 * checkAutomatic() - Check to see if they've requested anything from
 * the command line to be done automatically by robem.
 *
 * A NOTE ABOUT USING XVFB.  If we are running in batch mode we want
 * to have Xvfb running as a virtual XServer by doing this sometime on the
 * machine (probably at startup):
 *
 *     % Xvfb :1 -screen 1 1600x1200x16
 *
 * Then for the Xterm that's running robem in batch mode:
 *
 *     % setenv DISPLAY :1.1
 *
 * Then:
 *
 *     % robem -pre a.pif -noise 2.0 SHOT
 *
 * etc ...
 *
 * All XEvents then get served by the Xvfb process and the user can
 * go on with their other processing needs as this runs in background.
 *
 *
*/



void checkAutomatic()
{
  // function name: checkAutomatic

  // See if they want to preload something
  if(preLoadSwitch) {
    readType= IMAGE;
    gotFilename(0,0,0);
  }

} // end function checkAutomatic



void differentEndianCtf(BYTE *inStr, int length )
{
  // function name: differentEndianCtf
  // Convert this to a different ENDIAN bit of memory. 

  int i, lm1;
  BYTE thisStr[8];

  if(endianSwitch) {
    for(i=0; i<length; i++)
      *(thisStr + i)= *(inStr + i);
    lm1= length - 1;
    for(i=0; i<length; i++)
      *(inStr + i) = (*(thisStr + (lm1-i)) & 0377);
  }
  
} // end function differentEndianCtf



void mainCtfDisp_sub_UNIX(int *choice, unsigned short **int_buffer, 
		   fString_t *fName,
		   int *i, int *j, int *emWrt)
{
  // function name: mainCtfDisp_sub_UNIX
  // Read in any PIF images.

  int k,l,m,writeIt,logicalRecordLength,asked;
  int inWidth,inHeight;
  unsigned short junk,junk1,junk2;
  FILE *fIn,*fOut;
  char s1[1024];

  /* Init */
  fOut= fIn= 0;

  /******************************
   *   READING                  *
   ******************************/
  if(*choice == 'O') {/* Open a file */
    
    /* We are reading some type fo file here either .IMG (Purdue
     * INTEGER*2 format) or .PCK (Purdue packed) or .PPI
     * (Purdue Portable Image format).
     */
    
    XFlush(display);
    
    *i= *j= 0;
    fIn = fopen((*fName).addr,"r");
    if(fIn == NULL) {/*  Ooops can't read it!!! */
      sprintf(outStr, "Can't read [%s] ",(*fName).addr);
      asked= askThem(outStr,TRUE,XmDIALOG_INFORMATION);
      goto bugOut;
    }
    if(fIn) {/* Read the file */
      
      /* Read the file type header */
      fread(&junk1,sizeof(short),1,fIn); 
      fread(&purdueHeader.file_id1,sizeof(int),1,fIn); 
      fread(&junk2,sizeof(short),1,fIn); 
      
      purdueHeader.file_id2= 0;
      purdueHeader.file_id2 |= junk1;
      purdueHeader.file_id2 |= (((purdueHeader.file_id1) & 0177777) << 16);
      
      differentEndianCtf((BYTE *)&purdueHeader.file_id1, 4);
      differentEndianCtf((BYTE *)&purdueHeader.file_id2, 4);
      
      /* What type of image is it??? */
      fileType = 0;
      if(purdueHeader.file_id1==-1)
	fileType |= IMAGE;
      if(purdueHeader.file_id1==-32)
	fileType |= PORTABLE;
      
      /* Check for inconsistencies */
      if(fileType==0 ||
	 ((fileType & IMAGE) && (fileType & PORTABLE))) {/* COnfusion */
	asked= askThem("Confusing fileType in header.",FALSE,
		       XmDIALOG_INFORMATION);
	goto bugOut;
      }/* COnfusion */
      
      
      
      /* Branch on fileType */
      if(fileType & PORTABLE) {/* portable */
	
	/****************************************
	 * This guy is a Purdue Portable image  *
	 ****************************************/
	
	/* Get title */
	fread(s1,sizeof(char),72,fIn);
	
	/* More VMS/FORTRAN zaniness */
	fread(&purdueHeader.img_bkg,sizeof(int),1,fIn);  /* Why??? */
	fread(&purdueHeader.numCols,sizeof(short),1,fIn); 
	fread(&purdueHeader.numRows,sizeof(short),1,fIn);
	
	differentEndianCtf((BYTE *)&purdueHeader.img_bkg, 4);
	differentEndianCtf((BYTE *)&purdueHeader.numCols, 2);
	differentEndianCtf((BYTE *)&purdueHeader.numRows, 2);
	
	/* Flag large images for scrolling */
	largeImage= FALSE;
	if(purdueHeader.numCols > 1024)
	  largeImage= TRUE;
	if(purdueHeader.numRows > 1024)
	  largeImage= TRUE;
	
	fread(&purdueHeader.img_min,sizeof(int),1,fIn);
	fread(&purdueHeader.img_max,sizeof(int),1,fIn);
	
	/* Read in the float information in ASCII */
	fread(s1,sizeof(char),22,fIn);
	purdueHeader.img_avg= (float)strtod(s1,0);
	
	fread(s1,sizeof(char),22,fIn);
	purdueHeader.img_std= (float)strtod(s1,0);
	
	/* Read off this VMS junk we don't need. */
	fread(&purdueHeader.junk,sizeof(short),1,fIn);
	
	/* Clip this guy to 1k x 1k and let them look at 
	 * 1k x 1k pages at a time */
	inWidth = MAX(0,MIN(1024,purdueHeader.numCols));
	inHeight = MAX(0,MIN(1024,purdueHeader.numRows));
	
	/* get dynamic memory */
	if(*int_buffer != NULL)
	  free(*int_buffer);
	*int_buffer = (unsigned short *)malloc(((1024*
						 1024)*
						sizeof(unsigned short))+1);
	/* Init out the buffer */
	for(m=0;m<1024;m++) // Init them
	  memcpy((&int_image_buffer1[(m*1024)]),stufIntBuf,(2048));
	
	/* When these files are written by FORTRAM/RMS/VMS they sometimes
	 * get defaulted to a logical Record Length (RECL) of
	 * 2044.  Account for that here.
	 */
	logicalRecordLength = inWidth;
	headOff= ftell(fIn);
	/* Read the image data */
	for(k=0;k<inHeight;k++) {/* Read it a row at a time */
	  
	  /* DON'T Use the +k because EOR markers aren't
	   * present here.  Also * 2.0 because it's 
	   * shorts not chars */
	  thisOffset = headOff + (2.0 * 
				  (k*purdueHeader.numCols));
	  sysError= fseek(fIn,thisOffset,0);
	  if(sysError) {/* Can't go on */
	    asked= askThem("Error reading file.",
			   TRUE,XmDIALOG_INFORMATION);
	    goto bugOut;
	  }
	  fread((*int_buffer+(1024*k)),
		sizeof(unsigned short),(logicalRecordLength),fIn);
	  if(endianSwitch)
	    for(l=0;l<logicalRecordLength;l++)
	      differentEndianCtf((BYTE *)
				 (*int_buffer+(logicalRecordLength*k)+l),2);
	}
	      
	/* Tell who called us the image dimensions */
	*i= purdueHeader.numCols;
	*j= purdueHeader.numRows;
      }/* portable */
      
      
      fclose(fIn);
      
      
    }/* Read the file */
    
  }/* Open a file */

  
  /******************************
   *   WRITING                  *
   ******************************/
  else if (*choice =='W') {/* Write a file */
    writeIt= TRUE;
    fOut= NULL;
    fOut = fopen((*fName).addr,"r");
    if(fOut != NULL) {/*  Ooops we may be overwriting it!!! */
      sprintf(outStr,
	      "Do you really want to overwrite [%s]?",(*fName).addr);
      writeIt= askThem(outStr,TRUE,XmDIALOG_QUESTION);
    }
    fclose(fOut);
    
    if(writeIt) {/* OK to proceed */
      
      /* This doesn't work too well as UNIX doesn't do all
       * that file structure stuff that VMS does.  Just kludge
       * something for now - we'll try and use FORTRAN code on
       * VMS to write these.
       */
      
      /* Put a dummy title out there. */
      sprintf(s1,"%80s","Test title");
      
      /* Branch accordingly */
      if(*emWrt==IMAGE) {/* Write an image file */
	fOut = fopen((*fName).addr,"w");
	if(fOut == NULL) {/*  Ooops can't write it!!! */
	  sprintf(outStr,
		  "Can't write [%s] ",(*fName).addr);
	  askThem(outStr,TRUE,XmDIALOG_INFORMATION);
	  goto bugOut;
	}/*  Ooops can't write it!!! */
	
	/* It is writable */
	if(fOut) {/* Write the file */
	  
	  /* Why does VMS do this??? */
	  junk1= junk2= 2;
	  purdueHeader.file_id1= -1;
	  fwrite(&junk1,sizeof(short),1,fOut); 
	  fwrite(&purdueHeader.file_id1,sizeof(int),1,fOut); 
	  fwrite(&junk2,sizeof(short),1,fOut); 
	  
	  /* title */
	  fwrite(s1,sizeof(char),72,fOut);
	  fwrite(&purdueHeader.img_bkg,sizeof(int),1,fOut);
	  
	  /* Do the image dimensions etc... */
	  fwrite(i,sizeof(short),1,fOut); 
	  fwrite(j,sizeof(short),1,fOut);
	  fwrite(&purdueHeader.img_min,sizeof(int),1,fOut);
	  fwrite(&purdueHeader.img_max,sizeof(int),1,fOut);
	  
	  junk= 0;
	  fwrite(&purdueHeader.junk,sizeof(int),1,fOut);
	  fwrite(&purdueHeader.junk,sizeof(int),1,fOut);
	  
	  fwrite(&purdueHeader.junk,sizeof(short),1,fOut);
	  
	  /* Now wriet the image data as shorts */
	  for(k=0;k<*j;k++) {/* Write it a row at a time */ 
	    fwrite((*int_buffer+((*i)*k)),
		   sizeof(unsigned short),(*i),fOut);
	    
	    /* Now write that pesky EndOfRecord short that 
	     * FORTRAN/VMS
	     * puts at the end of each record 
	     */
	    fwrite(&junk,sizeof(unsigned short),(1),fOut);
	  }
	  
	  fclose(fOut);
	  
	}/* Write the file */
      }/* Write an image file */
      
      
    }/* OK to proceed */
    
    fclose(fOut);
    
  }/* Write a file */
  
 bugOut:;
  
  fflush(NULL);
  fclose(fIn);
  fclose(fOut);
  
  
} // end function mainCtfDisp_sub_UNIX



void init_globals(int argc, char **argv)
{
  // function name: init_globals
  // Init out anything that needs it before we start executing.  See
  // groundZero() for anything that needs initing later on in the
  // execution.

  int i;

  // Analyze switches
  endianSwitch = FALSE;
  preLoadSwitch = FALSE;
  for(i=0; i<argc; i++) {
    if(*(argv[i])=='-') {
      if(strstr(argv[i],"endian")!=0 || strstr(argv[i],"ENDIAN")!=0) { 
	endianSwitch= TRUE;
      }
      else if(strstr(argv[i],"-pre")!=0 || strstr(argv[i],"-PRE")!=0) {
	preLoadSwitch = TRUE;
	sprintf(preLoadStr,"%s",argv[i+1]);
      }
      else if(strstr(argv[i],"-usage")!=0 || strstr(argv[i],"-USAGE")!=0) {
	printf("\nUSAGE: ctfDisp -PRELOAD fname | -USAGE -ENDIAN \n\n");
	exit(0);
      }
    }
  }
  
  // Generic counters etc...
  virXoffset= virYoffset= xoffset= yoffset= 0;
  largeImage= FALSE;
  noCallbackScroll= FALSE;

  // Seed the random number generator
  srandom(1);
      
  // Filename descriptor for FORTRAN
  image_fileName.addr = (char *)calloc(2048, sizeof(char));  
  image_fileName.length = 2048;
  fileName.addr = (char *)calloc(2048, sizeof(char));  
  fileName.length = 2048;
  image_fileName.type = 14;
  image_fileName.class = 1;
  fileName.type = 14;
  fileName.class = 1;

  defocus    =     1000;
  cs         =      2.0;
  voltage    = 100000.0;
  pfract     =      1.0;
  afract     =      0.0;
  wavelength = 1230.0/sqrt(voltage + (1.0e-6)*(voltage*voltage));
  res_limit  =   1.0;   // ABSCISSA RESOLUTION LIMIT IN RECIPROCAL NM UNITS

  afractScrollValue = 0.05;
  pfractScrollValue = 0.95;
  resolutionScrollValue = 1.0;
  avoltageScrollValue = 100000.0;
  avoltageScrollValue = 200000.0;  /* Defautl to 200 kV for FEG */
  spaberrationScrollValue = 2.0;
  chaberrationScrollValue = 2.0;
  spCoherenceScrollValue = 0.0;
  tmCoherenceScrollValue = 0.0;
  wienerScrollValue = 0.0;
  pixSizeTextValue = 5.0;
  majDefocusScrollValue = 1.0;
  minDefocusScrollValue = 1.0;
  majAngScrollValue = 0.0;
  defocusScrollValue = 1.000;

  spCoExp = -3;
  spCoFactor = 0.001;

  for(i=0; i<MAXCURVE; i++) {
    curve[i].afractScrollValue     = afractScrollValue;
    curve[i].resolutionScrollValue = resolutionScrollValue;
    curve[i].avoltageScrollValue   = avoltageScrollValue;
    curve[i].spaberrationScrollValue = spaberrationScrollValue;
    curve[i].chaberrationScrollValue = chaberrationScrollValue;
    curve[i].spCoherenceScrollValue = spCoherenceScrollValue;
    curve[i].tmCoherenceScrollValue = tmCoherenceScrollValue;
    curve[i].majDefocusScrollValue = majDefocusScrollValue;
    curve[i].minDefocusScrollValue = minDefocusScrollValue;
    curve[i].majAngScrollValue = majAngScrollValue;
    curve[i].wienerScrollValue = wienerScrollValue;
    curve[i].defocusScrollValue    = defocusScrollValue;
    curve[i].pfractScrollValue     = pfractScrollValue;
    curve[i].spCoExp        = spCoExp;
    curve[i].spCoFactor     = spCoFactor;
    alwaysDraw[i]= FALSE;
    useSum[i]= TRUE;
    weightings[i] = 1.0;
  }

  maxParamWeight = 0.0;
  resoDraw = FALSE;
  applyFFTswitch = TRUE;
  sumCurveSwitch = FALSE;
  curCtf= 0;
  resoY= 0;

  bandPass=NOPASS;

  imageType= BACK;
  scrollResolutionExample = 256.0;

  defocusMin = -10000;
  defocusMax = 10000;

  resXlo = 0;
  resXhi = 99513;
  // Start with them turned off
  resXlo = -99;
  resXhi = -99;

  middlePass = FALSE;
  ctfDrawn[0]= TRUE;
  for(i=1;i<MAXCURVE;i++)
    ctfDrawn[i]= FALSE;

  dragUpdate = FALSE;    // Don't redraw curve on DRAG events for scroll bars
			 // unless that's what they want

  drawPointWhere = 14;
  drawPWX = 7;

  hiFFTscroll = 1.0;
  loFFTscroll = 0.0;

  pixSizeTextValue = 5.0;  // Angstroms

  ctfMeth1 = ctfMeth2 = ctfMeth3 = ctfMeth4 = ctfMeth5 = FALSE;
  fudgeFactor = 0.2;
  maxCTFCorrectYvalue = 5.0 + 1.0;  // Add in the negative 1.0 values
  dispCtfCorrect = FALSE;
  secondDisplay = 0;
  firstCtf2D= TRUE;

  // ctf_multiply() stuff
  filter[0] = filter[1] = filter[2] = filter[3] = filter[4] = filter[5] = 0;
  filt_fac[0] = filt_fac[1] = filt_fac[2] = filt_fac[3] = filt_fac[4] = 0.2;
  filt_fac[0] = 0.05;
  filt_fac[1] = 0.1;
  filt_fac[4] = 0.10;

  /*    CTFMODE = 0 DO NOTHING!                                               *
   *            = 1 CORRECT PHASES AND AMPLITUDES                             *
   *            = 2 FLIP PHASES WHERE NECESSARY; LEAVE AMPLITUDES UNCHANGED   *
   *            = 3 ADJUST AMPLITUDES; LEAVE PHASES UNCHANGED                 */
  ctfMode     = 1;
  ctfMode_div = 1;
  applyAmp = applyPhase = TRUE;

} // end function init_globals



void userDefaults()
{
  // function name: userDefaults
  // Read user specified defaults file and do appropriate.

  char inStr[512],key[256],value[256];
  FILE *fIn;
  XmString simpleString;

  fIn = fopen("~/.ctfDisprc","r");

  if(fIn) {
    
    while(!feof(fIn)) {
      get_str(fIn, '\n', inStr);
      
      sscanf(inStr,"%s",key);
      if(EQU(key,"Defocus")) {
	sscanf((inStr+(strlen(key))), "%s",value);
	defocusScrollValue = strtod(value,0);
      }
      else if(EQU(key,"majorDefocus")) {
	sscanf((inStr+(strlen(key))), "%s",value);
	majDefocusScrollValue = strtod(value,0);
      }
      else if(EQU(key,"minorDefocus")) {
	sscanf((inStr+(strlen(key))), "%s",value);
	minDefocusScrollValue = strtod(value,0);
      }
      else if(EQU(key,"E")) {
	sscanf((inStr+(strlen(key))), "%s",value);
	avoltageScrollValue = strtod(value,0) * 1000.0;
      }
      else if(EQU(key,"Afract")) {
	sscanf((inStr+(strlen(key))), "%s",value);
	afractScrollValue = strtod(value,0);
	pfractScrollValue = 1.0 - afractScrollValue;
      }
      else if(EQU(key,"Cs")) {
	sscanf((inStr+(strlen(key))), "%s",value);
	spaberrationScrollValue = strtod(value,0);
      }
      else if(EQU(key,"Cc")) {
	sscanf((inStr+(strlen(key))), "%s",value);
	chaberrationScrollValue = strtod(value,0);
      }
      else if(EQU(key,"Beta")) {
	sscanf((inStr+(strlen(key))), "%s",value);
	spCoherenceScrollValue = strtod(value,0);
      }
      else if(EQU(key,"BetaExponent")) {
	sscanf((inStr+(strlen(key))), "%s",value);
	spCoExp = (int)strtod(value,0);
	spCoExp = MIN(MAX(spCoExp,-99),0);
	sprintf(outStr,"1E%d",spCoExp);
	spCoFactor = strtod(outStr,0);
	
	if(spCoExp < 0)
	  sprintf(outStr," * E%03d",spCoExp);
	else 
	  sprintf(outStr," * E+%02d",spCoExp);
	
	simpleString = XmStringCreateLocalized(outStr);
	XtSetArg(arglist[0], XmNlabelString, simpleString);
	XtSetArg(arglist[1], XmNrecomputeSize, FALSE );
	XtSetValues(spCoExpLabelW, arglist, 2); 
	XmStringFree(simpleString); 
	XFlush(display);
      }
      else if(EQU(key,"dE")) {
	sscanf((inStr+(strlen(key))), "%s",value);
	tmCoherenceScrollValue = strtod(value,0);
      }
      else if(EQU(key,"Wiener")) {
	sscanf((inStr+(strlen(key))), "%s",value);
	wienerScrollValue = strtod(value,0);
      }
      else if(EQU(key,"AngMajor")) {
	sscanf((inStr+(strlen(key))), "%s",value);
	majAngScrollValue = strtod(value,0);
      }
      
    }
    
    for(i=0; i<MAXCURVE; i++) {
      curve[i].afractScrollValue     = afractScrollValue;
      curve[i].resolutionScrollValue = resolutionScrollValue;
      curve[i].avoltageScrollValue   = avoltageScrollValue;
      curve[i].spaberrationScrollValue = spaberrationScrollValue;
      curve[i].chaberrationScrollValue = chaberrationScrollValue;
      curve[i].spCoherenceScrollValue = spCoherenceScrollValue;
      curve[i].tmCoherenceScrollValue = tmCoherenceScrollValue;
      curve[i].majDefocusScrollValue = majDefocusScrollValue;
      curve[i].minDefocusScrollValue = minDefocusScrollValue;
      curve[i].majAngScrollValue = majAngScrollValue;
      curve[i].wienerScrollValue = wienerScrollValue;
      curve[i].defocusScrollValue    = defocusScrollValue;
      curve[i].pfractScrollValue     = pfractScrollValue;
      curve[i].spCoExp        = spCoExp;
      curve[i].spCoFactor     = spCoFactor;
    }
    
    fclose(fIn);
  }
  
  setCurveVals(TRUE);
  
} // end function userDefaults



void createDefaultColormap()
{
  // function name: createDefaultColormap
  // Do whatever is necessary to get us the desired colormap.

//  FILE *fColor;
  float scaleFactor;

  printf("Default colormap.\n");
  visual= DefaultVisual(display, DefaultScreen(display));
  visual_depth = (unsigned int)DefaultDepth(display, DefaultScreen(display));
  
  /* Allocate the color cells */
  pixels_return= (unsigned long *)malloc(sizeof(unsigned long)*256);
  colors_return= (unsigned long *)malloc(sizeof(unsigned long)*MAXCURVE);
  /* Allocate the color definition structs */
  if (random_defs == NULL) 
    random_defs = (XColor *) malloc (1025 * sizeof (XColor) );
  if (color_defs == NULL) 
    color_defs = (XColor *) malloc (1025 * sizeof (XColor) );
  if (gen_color_defs == NULL) 
    gen_color_defs = (XColor *) malloc (1025 * sizeof (XColor) );
      
//  printf("visual==[%d]\n",visual);
  
  color_map = XCreateColormap(display, DefaultRootWindow(display),
			      visual, AllocNone);
//  printf("DEC colormap = [%d]\n",color_map);
  
  if (color_map == (Colormap)NULL) {
    fprintf (stderr, "Could not create color map for visual\n");
    exit (1);
  }
  
  // Now let's query the DISPLAY until we get the maximum number of
  // contiguous colorcell entries in the colormap that we can .
  tryColors= 256;
  
  // Init the color defs to point to each entry
  for (i = 0; i <256 ; i++)  { // Make the color map gray-scale
    color_defs[i].pixel= i;
    gen_color_defs[i].pixel= i;
    pixels_return[i] = i;
  }
  for (i = 0; i <MAXCURVE ; i++) {
    random_defs[i].pixel = i;
    colors_return[i] = i;
  }
  
  // Find out what's already there in the color table
  XQueryColors(display, color_map, color_defs, 256);
  XQueryColors(display, color_map, gen_color_defs, 256);
  XQueryColors(display, color_map, random_defs, MAXCURVE);
  
//GCA: not clear the reason to write the file - commenting for now
/*  sprintf(outStr,"%s/cmap.robem",getenv("HOME"));

  fColor = fopen(outStr,"w");

  if(fColor) { // Write it
    fwrite(&tryColors,sizeof(int),1,fColor);
    for(i=0;i<tryColors;i++)
      fwrite(&pixels_return[i],sizeof(unsigned long),1,fColor);
  }
  else
    printf("ERROR - Cannot write colormap file.\n");

  fclose(fColor);
*/  
  /* Now change the colormap for drawing in pixels_return[GREEN,RED,BLUE] 
   * First calculate some fudge factors.
   */
  pixelColor= 65535.0/256.0;
  scaleFactor = 255.0 / (tryColors - 7.0);
  
  // Make these entries in the colormap GRAYSCALE
  for(i=6; i<tryColors; i++) { // Do the colors we have been assigned
    gen_color_defs[pixels_return[i]].pixel = pixels_return[i];
    gen_color_defs[pixels_return[i]].red   = (i-6)*scaleFactor*pixelColor;
    gen_color_defs[pixels_return[i]].green = (i-6)*scaleFactor*pixelColor;
    gen_color_defs[pixels_return[i]].blue  = (i-6)*scaleFactor*pixelColor;
    gen_color_defs[pixels_return[i]].flags = DoRed | DoGreen | DoBlue;
    color_defs[pixels_return[i]].pixel = pixels_return[i];
    color_defs[pixels_return[i]].red   = (i-6)*scaleFactor*pixelColor;
    color_defs[pixels_return[i]].green = (i-6)*scaleFactor*pixelColor;
    color_defs[pixels_return[i]].blue  = (i-6)*scaleFactor*pixelColor;
    color_defs[pixels_return[i]].flags = DoRed | DoGreen | DoBlue;
    color_defs[pixels_return[i]].pad = 0;
    gen_color_defs[pixels_return[i]].pad = 0;
    XAllocColor (display, color_map, &color_defs[i]);
    pixels_return[i] = color_defs[i].pixel;
    gen_color_defs[i].pixel =  i;
  }

  // Make these entries in the colormap GRAYSCALE
  for(i=0; i<MAXCURVE; i++) { // Do the colors we have been assigned
    float factor, thisVal;
    // Get our fraction of the random number
    factor= 0.75 / DENOMINATOR;
    
    // Just get me a random number always have a 0.25 floor so we
    // don't get many dark colors */
    random_defs[colors_return[i]].pixel = colors_return[i];
    thisVal = (0.25+(random() * factor)) * 254.0;
    random_defs[colors_return[i]].red   = (unsigned short)
      (thisVal * pixelColor);
    thisVal = (0.25+(random() * factor)) * 254.0;
    random_defs[colors_return[i]].green = (unsigned short)
      (thisVal * pixelColor);
    thisVal = (0.25+(random() * factor)) * 254.0;
    random_defs[colors_return[i]].blue  = (unsigned short)
      (thisVal * pixelColor);
    random_defs[colors_return[i]].flags = DoRed | DoGreen | DoBlue;
    random_defs[colors_return[i]].pad = 0;
    XAllocColor (display, color_map, &random_defs[i]);
    colors_return[i] = random_defs[i].pixel;
  }
  
  // Now define our robem colors
  color_defs[WHITE].pixel = WHITE;
  color_defs[WHITE].red   = 255 * pixelColor;
  color_defs[WHITE].green = 255 * pixelColor;
  color_defs[WHITE].blue  = 255 * pixelColor;
  color_defs[WHITE].flags = DoRed | DoGreen | DoBlue;
  XAllocColor (display, color_map, &color_defs[WHITE]);
  pixels_return[WHITE] = color_defs[WHITE].pixel;
  
  color_defs[YELLOW].pixel = YELLOW;
  color_defs[YELLOW].red   = 255 * pixelColor;
  color_defs[YELLOW].green = 255 * pixelColor;
  color_defs[YELLOW].blue  = 0 * pixelColor;
  color_defs[YELLOW].flags = DoRed | DoGreen | DoBlue;
  XAllocColor (display, color_map, &color_defs[YELLOW]);
  pixels_return[YELLOW] = color_defs[YELLOW].pixel;
  
  color_defs[GREEN].pixel = GREEN;
  color_defs[GREEN].red   = 0;
  color_defs[GREEN].green = 255 * pixelColor;
  color_defs[GREEN].blue  = 0;
  color_defs[GREEN].flags = DoRed | DoGreen | DoBlue;
  XAllocColor (display, color_map, &color_defs[GREEN]);
  pixels_return[GREEN] = color_defs[GREEN].pixel;
  
  color_defs[pixels_return[RED]].pixel = RED;
  color_defs[pixels_return[RED]].red   = 255 * pixelColor;
  color_defs[pixels_return[RED]].green = 0;
  color_defs[pixels_return[RED]].blue  = 0;
  color_defs[pixels_return[RED]].flags = DoRed | DoGreen | DoBlue;
  XAllocColor (display, color_map, &color_defs[pixels_return[RED]]);
  pixels_return[RED] = color_defs[RED].pixel;
  
  color_defs[BLUE].pixel = BLUE;
  color_defs[BLUE].red   = 0;
  color_defs[BLUE].green = 0;
  color_defs[BLUE].blue  = 255 * pixelColor;
  color_defs[BLUE].flags = DoRed | DoGreen | DoBlue;
  XAllocColor (display, color_map, &color_defs[BLUE]);
  pixels_return[BLUE] = color_defs[BLUE].pixel;
  
  color_defs[PLACEHOLDER].pixel = PLACEHOLDER;
  color_defs[PLACEHOLDER].red   = 0 * pixelColor;
  color_defs[PLACEHOLDER].green = 255 * pixelColor;
  color_defs[PLACEHOLDER].blue  = 0;
  color_defs[PLACEHOLDER].flags = DoRed | DoGreen | DoBlue;
  XAllocColor (display, color_map, &color_defs[PLACEHOLDER]);
  pixels_return[PLACEHOLDER] = color_defs[PLACEHOLDER].pixel;
  
  color_defs[BLACK].pixel = BLACK;
  color_defs[BLACK].red   = 0 * pixelColor;
  color_defs[BLACK].green = 0 * pixelColor;
  color_defs[BLACK].blue  = 0 * pixelColor;
  color_defs[BLACK].flags = DoRed | DoGreen | DoBlue;
  XAllocColor (display, color_map, &color_defs[BLACK]);
  pixels_return[BLACK] = color_defs[BLACK].pixel;

  color_defs[GRAY].pixel = GRAY;
  color_defs[GRAY].red   = 165 * pixelColor;
  color_defs[GRAY].green = 165 * pixelColor;
  color_defs[GRAY].blue  = 165 * pixelColor;
  color_defs[GRAY].flags = DoRed | DoGreen | DoBlue;
  XAllocColor (display, color_map, &color_defs[GRAY]);
  pixels_return[GRAY] = color_defs[GRAY].pixel;
  
} // end function createDefaultColormap



void XlibInitialize()
{
  // function name: XlibInitialize
  // Initialize for Xlib calls.  Setup the colormap we'll use and
  // graphics contexts.  Create all of the Images and allocate memory
  // for all of our XmDrawingAreas.  Add event handlers for motion
  // events where needed and setup our cursors.

  BYTE thisBuf[1024];
  FILE *fIn;

  // Init dimensions
  buffer_width= image_width = NUMCOLS;
  buffer_height= image_height = NUMROWS;
  ctf_width  = CTF_WIDTH;
  ctf_height = CTF_HEIGHT;
  image_colors = 3;

  // Allocate the color cells
  pixels_return= (unsigned long *)malloc(sizeof(int)*256);
  
  // Allocate the color definition structs
  if (color_defs == NULL) 
    color_defs = (XColor *) malloc (1025 * sizeof (XColor) );
  if (gen_color_defs == NULL) 
    gen_color_defs = (XColor *) malloc (1025 * sizeof (XColor) );
  

  // Get visual
  visual= DefaultVisual(display, DefaultScreen(display));
  visual_depth = (unsigned int)DefaultDepth(display, 
					    DefaultScreen(display));
  createDefaultColormap();
  
  // Create Image buffers for our drawingAreas
  // MAIN drawing area on main screen
  ctf_image_image1 = XCreateImage (display, visual, visual_depth, 
			       ZPixmap, 0, NULL, ctf_width, 
			       ctf_height, 32, 0);
  if (ctf_image_image1 == NULL) {
    fprintf (stderr, "XCreateImage (image) failed\n");
    exit(1);
  }
  
  image_image1 = XCreateImage (display, visual, visual_depth, 
			       ZPixmap, 0, NULL, image_width, 
			       image_height, 32, 0);
  if (image_image1 == NULL) {
    fprintf (stderr, "XCreateImage (image) failed\n");
    exit(1);
  }
  // Get Window pointers from MOTIF 
  image_window1= XtWindow(drawingWidget);
  ctf_image_window1= XtWindow(ctfDrawingWidget);
  
  // Now tell MOTIF what kindof events you'll listen to in this window.
   event_mask  = ButtonPressMask;
   event_mask |= ButtonReleaseMask   | KeyPressMask | KeyReleaseMask;
   event_mask |= PointerMotionMask   | LeaveWindowMask | ExposureMask;
   XSelectInput(display, image_window1, event_mask);
   XSelectInput(display, ctf_image_window1, event_mask);
   
   // Make this colormap is the one that's used!
   XInstallColormap (display, color_map);    // Install the color map
  
   // Allocate for our image storage *4 in case 32 bit colormap
  image_buffer1 = (BYTE *)malloc ((buffer_width*buffer_height)*4*
				  (sizeof(BYTE)*4));
  mirror_image_buffer1 = (BYTE *)malloc ((sizeof(BYTE)*4));
  // Just give it something for now
  ctf_image_buffer1 = (BYTE *)malloc ((ctf_width*ctf_height)*4*
				      (sizeof(BYTE)*4));

  // Couldn't get aholt of enough storage?
  if (image_buffer1 == NULL) {
    perror ("malloc image_buffer");
    exit(1);
  }

  /* OK tell the image where to find it's data */
  image_image1->data = (char *)image_buffer1;
  ctf_image_image1->data = (char *)ctf_image_buffer1;
  
  for(i=0;i<ctf_height;i++)
    for(j=0;j<ctf_width;j++)
      XPutPixel(ctf_image_image1, j, i, pixels_return[170]); 
  
  // This is an optimization thing - just blow this out to the screen
  // numRows times whenever you need to make sure there's no unwanted
  // pixels showing.
  
  for(j=0;j<NUMCOLS;j++)
    stufBuf[j]= BLACK+8;
  for(j=0;j<NUMCOLS;j++)  
    stufIntBuf[j]= (unsigned short)BLACK+8;  

  // Just to make sure let's init this
  for(j=0;j<buffer_height;j++)
    memcpy((&image_buffer1[(j*buffer_width)]),stufBuf,buffer_width);


  // Setup Graphics contexts
  fontstruct =  XLoadQueryFont(display, FONT);
  if (fontstruct  == NULL) {
    fprintf(stderr, "%s: display %s doesn't know font %s\n",
	    "ctfDisp", "DisplayString(display)", FONT);
    exit(1);
  }

  gc_mask = 0;
  gc_values.function = GXcopy;		gc_mask |= GCFunction;
  gc_values.plane_mask = AllPlanes;	gc_mask |= GCPlaneMask;
  /*gc_values.foreground = white_pixel;	gc_mask |= GCForeground;*/
  /*gc_values.background = black_pixel;	gc_mask |= GCBackground;*/
  gc_values.foreground = 1;	        gc_mask |= GCForeground;
  gc_values.background = black_pixel;	gc_mask |= GCBackground;
  gc_values.graphics_exposures = False;	gc_mask |= GCGraphicsExposures;
  gc_values.font = fontstruct->fid;     gc_mask |= GCFont; 
  image_gc = XCreateGC (display, image_window1, gc_mask, &gc_values);
  ctf_image_gc = XCreateGC (display, ctf_image_window1, gc_mask, &gc_values);
  
  gc_values.foreground = pixels_return[RED];  gc_mask |= GCForeground;
  imageCTF_gcRED = XCreateGC (display, ctf_image_window1, 
			      gc_mask, &gc_values);
  
  gc_values.foreground = pixels_return[SEAGREEN];  gc_mask |= GCForeground;
  imageCTF_gcSEAGREEN = XCreateGC (display, ctf_image_window1, 
				gc_mask, &gc_values);
  
  gc_values.foreground = pixels_return[GREEN];  gc_mask |= GCForeground;
  imageCTF_gcGREEN = XCreateGC (display, ctf_image_window1, 
				gc_mask, &gc_values);
  
  gc_values.foreground = pixels_return[BLUE];  gc_mask |= GCForeground;
  imageCTF_gcBLUE = XCreateGC (display, ctf_image_window1,
			       gc_mask, &gc_values);
  
  gc_values.foreground = pixels_return[BLACK];  gc_mask |= GCForeground;
  imageCTF_gcBLACK = XCreateGC (display, ctf_image_window1, 
				gc_mask, &gc_values);
  

  // Get us some random colors
  for(i=0;i<MAXCURVE;i++) { // Do all curves
    gc_values.foreground = colors_return[i];  gc_mask |= GCForeground;
    imageCTF_colors[i] = XCreateGC (display, ctf_image_window1, 
				    gc_mask, &gc_values);
  }

  for(i=0; i<MAXCURVE; i++) {
    if(i==0) { // do special
      gc_values.foreground = pixels_return[BLUE];  
      gc_mask |= GCForeground;
      draw_gc_pos[i] = pixels_return[BLUE];
    }
    else if(i==1) { // do special
      gc_values.foreground = pixels_return[RED];  
      gc_mask |= GCForeground;
      draw_gc_pos[i] = pixels_return[RED];
    }
    else if(i==2) { // do special
      gc_values.foreground = pixels_return[GREEN]; 
      gc_mask |= GCForeground;
      draw_gc_pos[i] = pixels_return[GREEN];
    }
    else if(i==3) { // do special
      gc_values.foreground = pixels_return[WHITE]; 
      gc_mask |= GCForeground;
      draw_gc_pos[i] = pixels_return[WHITE];
    }
    else { // do NON special
      gc_values.foreground = colors_return[i]; 
      gc_mask |= GCForeground;
      draw_gc_pos[i] = colors_return[i];
    }
    
    draw_gc[i] = XCreateGC (display, ctf_image_window1, 
			    gc_mask, &gc_values);
  }
  
  gc_values.foreground = pixels_return[GRAY];
  gc_mask |= GCForeground;
  imageCTF_gcGRAY = XCreateGC (display, ctf_image_window1, 
				gc_mask, &gc_values);
  
  gc_values.foreground = pixels_return[RED];  gc_mask |= GCForeground;
  image_gcRED = XCreateGC (display, image_window1, gc_mask, &gc_values);
  
  gc_values.foreground = pixels_return[SEAGREEN];  gc_mask |= GCForeground;
  image_gcSEAGREEN = XCreateGC (display, image_window1, gc_mask, &gc_values);
  
  gc_values.foreground = pixels_return[GREEN];  gc_mask |= GCForeground;
  image_gcGREEN = XCreateGC (display, image_window1, gc_mask, &gc_values);
  
  gc_values.foreground = pixels_return[BLUE];  gc_mask |= GCForeground;
  image_gcBLUE = XCreateGC (display, image_window1, gc_mask, &gc_values);
  
  gc_values.foreground = pixels_return[ORANGE];  gc_mask |= GCForeground;
  image_gcORANGE = XCreateGC (display, image_window1, gc_mask, &gc_values);
  
  gc_values.foreground = pixels_return[YELLOW];  gc_mask |= GCForeground;
  image_gcYELLOW = XCreateGC (display, image_window1, gc_mask, &gc_values);
  
  gc_values.foreground = pixels_return[BLACK];  gc_mask |= GCForeground;
  image_gcBLACK = XCreateGC (display, image_window1, gc_mask, &gc_values);
  
  // Display the image in the window created above
  image_width = DEF_DISPLAY_SIZE;   /* This changes whenever a new image  */
  image_height = DEF_DISPLAY_SIZE;  /* is read                            */
  display_width = image_width;      /* Make sure we draw it all */
  display_height = image_height;

  strcpy(outStr, path);
  strcat(outStr, "lilly_rev.apx");
  fIn= fopen(outStr,"r");

  image_width= image_height= 256;

  ncol= nrow= fftWidth= fftHeight= 256;
  
  fftImage_buffer = (float *)malloc(sizeof(float)*2*fftWidth*fftHeight);
  
  fftImage_junk = (float *)malloc(sizeof(float)*2*fftWidth*fftHeight);
  
  back_fft_image_buffer1 = (unsigned short *)malloc(sizeof(unsigned short)*
						    fftWidth*fftHeight);
  
  back_stretch_fft_image_buffer1 = (unsigned short *)malloc(sizeof(unsigned short)*
							    fftWidth*fftHeight);
  
  raw_fft_image_buffer1 = (unsigned short *)malloc(sizeof(unsigned short)*
						   fftWidth*fftHeight);
  
  fft_image_buffer1 = (unsigned short *)malloc(sizeof(unsigned short)*
					       fftWidth*fftHeight);
  
  int_image_buffer1 = (unsigned short *)malloc(image_width*image_height*
						sizeof(unsigned short));

  resoImage = (unsigned short *)malloc(image_width*image_height*
					sizeof(unsigned short));

  ctf2D_image_buffer1 = (unsigned short *)malloc(image_width*image_height*
						 sizeof(unsigned short));
  minOrig = 9999;
  maxOrig = 0-9999;
  for(y=0; y<512; y++) {
    fread(thisBuf,sizeof(BYTE),512,fIn);
    for(x=0; x<512; x++) {
      minOrig = MIN(minOrig,(int)thisBuf[x]);
      maxOrig = MAX(maxOrig,(int)thisBuf[x]);
      int_image_buffer1[(((int)(y*0.5)*image_width)+(int)(x*0.5))] = 
	(unsigned short)(thisBuf[x]);
      
      ctf2D_image_buffer1[(((int)(y*0.5)*256)+(int)(x*0.5))]= 
	(toggleThis) ? (unsigned short)PLACEHOLDER :
	(unsigned short)(PLACEHOLDER+5);
      
      toggleThis++;
      if(toggleThis > 2)
	toggleThis= 0;;
    }
  }

  fclose(fIn);

  // Remember the gold std for later
  for(y=0; y<256; y++)
    for(x=0; x<256; x++)
      resoImage[(y*256)+x] = int_image_buffer1[(y*256)+x];
  
  stuffImage(int_image_buffer1,image_width,image_height,0,TRUE);
  stuffImage(int_image_buffer1,image_width,image_height,256,FALSE);
  stuffImage(fft_image_buffer1,image_width,image_height,512,FALSE);
  stuffImage(ctf2D_image_buffer1,image_width,image_height,768,FALSE);

  DrawWindow();

  XPutImage (display, ctf_image_window1, ctf_image_gc, ctf_image_image1, 0, 0,
	     0, 0, ctf_width, ctf_height);
  

  /* Let's also look for motion events in these windows */
  XtAddEventHandler(ctfDrawingWidget, 
		    /* ButtonMotionMask,  */
		    PointerMotionMask, 
		    FALSE, (XtEventHandler)plotMotion, NULL);

  XFlush (display);
  
} // end function XlibInitialize



void s_error(char *problem_string)
{
  // function name: s_error
  // All errors are fatal

  printf("%s\n", problem_string);
  exit(0);

} // end function s_error



void setCurveVals(int doTextWidgets)
{
  // function name: setCurveVals
  // Set the text (if desired) and scroll widgets to reflect what
  // values we have for the different parameters.

  char outStr[128];

  if(!inited)
    return;

  if(doTextWidgets) {  
    sprintf(outStr,"%.2f/%.2f",afractScrollValue,pfractScrollValue);
    XmTextSetString(afractValue,outStr);
    sprintf(outStr,"%.2f mm",spaberrationScrollValue);
    XmTextSetString(spaberrationValue,outStr);
    sprintf(outStr,"%.2f mm",chaberrationScrollValue);
    XmTextSetString(chaberrationValue,outStr);
    sprintf(outStr,"%.3f mRad",spCoherenceScrollValue);
    XmTextSetString(spCoherenceValue,outStr);
    sprintf(outStr,"%.3f eV",tmCoherenceScrollValue);
    XmTextSetString(tmCoherenceValue,outStr);
    sprintf(outStr,"%.2f",userAngScrollValue);
    XmTextSetString(userAngValue,outStr);
    sprintf(outStr,"%.2f",majAngScrollValue);
    XmTextSetString(majAngValue,outStr);
    sprintf(outStr,"%.3f um",majDefocusScrollValue);
    XmTextSetString(majDefocusValue,outStr);
    sprintf(outStr,"%.3f um",minDefocusScrollValue);
    XmTextSetString(minDefocusValue,outStr);
    sprintf(outStr,"%.2f",wienerScrollValue);
    XmTextSetString(wienerValue,outStr);
    sprintf(outStr,"%.0f kV",avoltageScrollValue/1000.0);
    XmTextSetString(avoltageValue,outStr);
    sprintf(outStr,"%.3f um",defocusScrollValue);
    XmTextSetString(defocusValue,outStr); 
  }

  XtVaSetValues(afractScroll,
		XmNborderColor, draw_gc_pos[curCtf],
		XmNvalue, (int)(afractScrollValue*100),
		NULL);
  XtVaSetValues(spaberrationScroll,
		XmNborderColor, draw_gc_pos[curCtf],
		XmNvalue, (int)(spaberrationScrollValue*100),
		NULL);
  XtVaSetValues(chaberrationScroll,
		XmNborderColor, draw_gc_pos[curCtf],
		XmNvalue, (int)(chaberrationScrollValue*100),
		NULL);
  XtVaSetValues(spCoherenceScroll,
		XmNborderColor, draw_gc_pos[curCtf],
		XmNvalue, (int)(spCoherenceScrollValue*100),
		NULL);
  XtVaSetValues(tmCoherenceScroll,
		XmNborderColor, draw_gc_pos[curCtf],
		XmNvalue, (int)(tmCoherenceScrollValue*100),
		NULL);
  XtVaSetValues(majDefocusScroll,
		XmNborderColor, draw_gc_pos[curCtf],
		XmNvalue, (int)(majDefocusScrollValue*1000.0),
		NULL);
  XtVaSetValues(minDefocusScroll,
		XmNborderColor, draw_gc_pos[curCtf],
		XmNvalue, (int)(minDefocusScrollValue*1000.0),
		NULL);
  XtVaSetValues(userAngScroll,
		XmNvalue, (int)(userAngScrollValue*100),
		NULL);
  XtVaSetValues(majAngScroll,
		XmNborderColor, draw_gc_pos[curCtf],
		XmNvalue, (int)(majAngScrollValue*100),
		NULL);
  XtVaSetValues(wienerScroll,
		XmNborderColor, draw_gc_pos[curCtf],
		XmNvalue, (int)(wienerScrollValue*100),
		NULL);
  XtVaSetValues(afractScroll,
		XmNborderColor, draw_gc_pos[curCtf],
		XmNvalue, (int)(afractScrollValue*100),
		NULL);
  XtVaSetValues(avoltageScroll,
		XmNborderColor, draw_gc_pos[curCtf],
		XmNvalue, (int)(avoltageScrollValue/1000.0),
		NULL);
  XtVaSetValues(defocusScroll,
		XmNborderColor, draw_gc_pos[curCtf],
		XmNvalue, (int)(defocusScrollValue*1000.0),
		NULL);
  
  if(curve[curCtf].isParamFile) {
      XmString simpleString;
      
      sprintf(outStr,"%s",curve[curCtf].paramFname);
      simpleString = XmStringCreateLocalized(outStr);
      XtSetArg(arglist[0], XmNlabelString, simpleString);
      XtSetArg(arglist[1], XmNrecomputeSize, TRUE );
      XtSetValues(paramFileW, arglist, 2); 
      XmStringFree(simpleString); 
      XFlush(display);
      
      sprintf(outStr,"#Part[%d] PercTot[%.2f]",
	      curve[curCtf].numParticles,
	      curve[curCtf].percPartTot);
      simpleString = XmStringCreateLocalized(outStr);
      XtSetArg(arglist[0], XmNlabelString, simpleString);
      XtSetArg(arglist[1], XmNrecomputeSize, TRUE );
      XtSetValues(paramNPW, arglist, 2); 
      XmStringFree(simpleString); 
      XFlush(display); 
    }
    
} // end function setCurveVals



void stuffIt(unsigned char *buff, int len, unsigned char *intShort)
{
  // function name: stuffIt
  // Stuff the presumably shorter intShort buffer into the char buffer
  // buff.  Used when writing VMS file compatible files from our C
  // code here.  It's easy to write the file in a memory buffer here
  // and then just blow it out into the VMS structured file one
  // "chunk" of memory at a time.

  int i;

  for(i=0; i<len; i++)
    *(buff+i) = (unsigned char)*(intShort + i);

} // end function stuffIt



void stuffImage(unsigned short *thisBuffer, int i, int j, 
		int drawingOffset, int firstTime) 
{
  // function name: stuffImage

  // Take the integer buffer we've read from the disk file and stuff
  // it into our BYTE buffer for display.  Make the integer buffer
  // disk data "fit" into our colormap by scaling it.  e.g. You may
  // have data from 0 to 255 on the disk (per pixel) but we've only
  // got 237 colors available to us for our color table.
  //
  // We maintain a "mirror" buffer here for speed in rezooming the
  // image.  For example if we need to zoom the image it is easier to
  // read the data already converted to our colormap domain and in
  // character form than trying to reread the integer buffer and
  // reconverting it.  Keep that integer buffer around, though, to
  // stuff the box buffers etc...  We need to always write out the
  // original data NOT the colromap converted stuff when we write. Rtn
  // originated in robem.

  int x, y, thisOffset, yBuf; //,yOff;
  static int min,max;
  static float scaleFactor;
//  BYTE *p;
  unsigned short *q;
  int thisMin,thisMax;

  /* Reset offsets */
  xoffset= yoffset= 0;
  
  /* Now clip this guy to our colormap */ 
  scaleFactor = (((float)tryColors - 6.0) / 256.0);
  
  /* Init histogram and point to our integer and display 
   * buffers */
  for(y=0;y<257;y++)
    histogram[y]= 0;
//  p= (BYTE *) image_buffer1;
  q= (unsigned short *)  thisBuffer; 

  if(firstTime) {/* Only do this once for the original */
    min= 99999999;
    max = -999999;
    for(y=0;y<j;y++) {/* Tim draws from the bottom up - WE DON't */
      yBuf = y*i;
      for(x=0;x<i;x++) {/* Clip the entire image */
	
	q = (thisBuffer + (yBuf+x));
	min = MIN((*q),min);
	max = MAX((*q),max);
      }/* Clip the entire image */
      
    }/* Tim draws from the bottom up - WE DON't */
    
    /* Use 8.0 so we don't have to worry about boundaries (e.g. should be
     * 6.0...)
     */
    if(max==min)
      scaleFactor= 1.0;
    else scaleFactor = ((float)(tryColors-10.0) / (float)(max-min));
  }/* Only do this once for the original */

  if(thisBuffer==int_image_buffer1) {/* remember */
    saveMin = min;
    saveMax = max;
    scaleOrig = scaleFactor;
  }/* remember */
  if(thisBuffer==back_fft_image_buffer1) {/* remember */
    /* min = saveMin;
       max = saveMax; */
    scaleFactor = scaleOrig;
  }
  
  /* Convert integer to char within our colormap and put it
   * in the display buffer(s) */
  thisMin= 99999999;
  thisMax = -999999;
  for(y=0;y<j;y++) {/* Tim draws from the bottom up - WE DON't */
//    yOff= y*buffer_width;
    yBuf = y*i;
    for(x=0;x<i;x++) {/* Clip the entire image */
      
//      p = (image_buffer1 + (yOff+x) + drawingOffset);
      q = (thisBuffer + (yBuf+x));
      
      /* We've got to scale these for now because we may get one
       * that is larger than we've got colormap entries for.
       */
      thisMin = MIN((*q),thisMin);
      thisMax = MAX((*q),thisMax);
      thisOffset = (int)( (  ( (((float)*(q))-(float)min) * 
			       scaleFactor )   ) + 8.0);
      
      thisOffset = MIN(MAX(thisOffset,0),255);
      if((thisOffset >=0) && (thisOffset<256))
	{/* OK to do */
	  XPutPixel(image_image1, (x+drawingOffset), y, 
		    pixels_return[thisOffset]); 
	  
	  histogram[thisOffset]+= 1;
	}/* OK to do */
    }/* Clip the entire image */
  }/* Tim draws from the bottom up - WE DON't */
  
  image_width= i;
  image_height= j;
  
  
} // end function stuffImage



void filterFiles(char *filesStr)
{
  // function name: filterFiles
  // Set the file selection widget to filter for the fileStr files.

  XmString tcs;

  tcs = XmStringLtoRCreate(filesStr, XmSTRING_DEFAULT_CHARSET);
  XmFileSelectionDoSearch(fileSel_widget, tcs);
  /*  XtFree(tcs); */
  XFlush(display);
  
} // end function filterFiles



/***********************************************************************
 * CALLBACK routines - Rotuines called directly from X whenever the    *
 * appropriate event occurs.                                           *
 ***********************************************************************/



int askThem(char *infoStr, int systemModal, int dialogType)
{
  // function name: askThem
  // This is the generic display a message and get them to respond
  // screen.  We get a return value from globAsk which is set by
  // infoOK() or infoCancel() (see below).

  XmString simpleString;
  
  // Init and try to position the window
  globAsk = -99;
  XFlush(display);
  simpleString = XmStringCreateLocalized(infoStr);
  XtSetArg(arglist[0], XmNmessageString, simpleString);
  XtSetArg(arglist[1], XmNdialogType, dialogType);
  XtSetArg(arglist[2], XmNx, 150);
  XtSetArg(arglist[3], XmNy, 150);
  XtSetValues(infoBox, arglist, 4);
  XFlush(display);
  XmStringFree(simpleString);
  
  // Init it and pop it up sit here and wait until we get 
  // something back
  gFinished= FALSE;
  XtManageChild(infoBox);
  XtPopup(XtParent(infoBox), XtGrabNone);

  XmProcessTraversal(infoBox, XmTRAVERSE_CURRENT); 
  XtRealizeWidget(infoBox);  
  XFlush(display);

  // gFinished is set in the infoOK() or infoCancel() callback
  while(!gFinished)
    XtAppProcessEvent(app_context, XtIMAll);

  // Pop it down and return their response
  XtUnmanageChild(infoBox);
  XtPopdown(XtParent(infoBox));
  XtSetArg(arglist[0], XmNdialogStyle, XmDIALOG_MODELESS);
  XtSetValues(XtParent(infoBox), arglist, 1);
  XFlush(display);
  
  return(globAsk);

} // end function askThem



void infoOK(Widget widget, int *tag, 
	    XmAnyCallbackStruct *callback_data)
{
  // function name: infoOK
  // If the OK button is pressed from the infoscreen return here.  Set
  // gFinished to cause the value of globAsk be return to the starting
  // point (see askThem() above).

  globAsk = TRUE;
  XtUnmanageChild(infoBox);
  gFinished= TRUE;

} // end function infoOK



void infoCancel(Widget  widget, int *tag, 
		XmAnyCallbackStruct *callback_data)
{
  // function name: infoCancel
  // If the CANCEL button is pressed from the infoscreen return here.
  // Set gFinished to cause the value of globAsk be return to the
  // starting point (see askThem() above).

  globAsk = FALSE;
  XtUnmanageChild(infoBox);
  gFinished= TRUE;

} // end function infoCancel



void handle_cr(Widget widget)
{
  // function name: handle_cr
  // This isn't really a callback but it is called by a callback.
  // This routine changes the translation table interpretations for
  // the simple text widgets so it knows what to do when it sees a
  // carriage return.

  XtTranslations parsed_t_table;

  // Modify this widgets translation table
  parsed_t_table = XtParseTranslationTable(rwa_translation_table);
  XtOverrideTranslations(widget, parsed_t_table);

} // end function handle_cr



void read_activate(Widget  widget, int *tag,
		   XmAnyCallbackStruct *callback_data)
{
  // function name: read_activate
  // They selected the "Read" submenu. Branch on the appropriate.

  static int firstTime = FALSE;

  readType= *tag;
  if(readType == IMAGE) {
    filterFiles("/bio/baker9d7/test/ctf/*");  
    // SCRIPPS filterFiles("/ami/cryo/cryoin03/purdue/test/ctf/*");
    firstTime = FALSE;
  }
  else {
    if(!firstTime)
      filterFiles("*"); 
    firstTime = TRUE;
  }
  
  XtManageChild(fileSel_widget);
  
} // end function read_activate



void exit_activate(Widget  widget, int *tag,
		   XmAnyCallbackStruct *callback_data)
{
  // function name: exit_activate
  // They said they wanted to exit the program ask again to make sure.
  
  exit(0);
  
} // end function exit_activate



void formResize(Widget  widget, int *tag,
		XmDrawingAreaCallbackStruct *callback_data)
{
  // function name: formResize
  // When the main screen formDialog is resized call here. Get the
  // new window size and setup our globals.

  Window tWin;
  int width, height; 
  XWindowAttributes winAttrib;

  if(!inited)
    return;

  tWin = (Window)XtWindow(drawingWidget);
  
  // Get new window dimensions
  if(tWin > 0) { // Valid window
    XGetWindowAttributes(display, tWin, &winAttrib);

  // Remember the dimensions of this
    display_width  = winAttrib.width;
    display_height = winAttrib.height;
 
    xoffset = yoffset = 0;
  }
} // end function formResize



void plotWindow_create(Widget widget, int *tag,
		       XmAnyCallbackStruct *callback_data)
{
  // function name: plotWindow_create
  // Remember the drawing area window pointer for use later. Setup the
  // file selection widget here.

  Arg arglist[32];
  XmString tcs;
  XtCallbackRec callBck[5];    // Our callbacks
  
  // Remember the widget number for later
  drawingWidget= widget;
  
  // Setup the file selection widget
  fileSel_widget = (Widget)XmCreateFileSelectionDialog(main_window_widget,
						"Get file", NULL, 0);
  tcs = XmStringLtoRCreate("*.apx", XmSTRING_DEFAULT_CHARSET);

  XtSetArg(arglist[0], XmNx, 10);
  XtSetArg(arglist[1], XmNy, 10);
  XtSetArg(arglist[2], XmNdirMask, tcs);  
  XtSetArg(arglist[3], XmNdefaultPosition, FALSE); 
  XtSetValues(fileSel_widget, arglist, 4);

  // Add callbacks to file selection widget
  callBck[0].callback= gotFilename;
  callBck[0].closure= 0;
  callBck[1].callback= 0;
  callBck[1].closure= 0;
  XtAddCallbacks(fileSel_widget, XmNokCallback, callBck);
  callBck[0].callback= cancel_fileSel;
  callBck[0].closure= 0;
  callBck[1].callback= 0;
  callBck[1].closure= 0;
  XtAddCallbacks(fileSel_widget, XmNcancelCallback, callBck);
  XmStringFree(tcs);

} // end function plotWindow_create



void DrawWindow()
{
  // function name: DrawWindow
  // Redraw window e.g. on expose events or whenever it's prudent.

//  int draw_width, draw_height;
  int thisX, angMajInt;
  float angMajLineY;

  // Draw all of image
//  draw_width = display_width;
//  draw_height = display_height;

  XPutImage (display, image_window1, image_gc, image_image1, 0, 0,
	     0, 0, (image_width*4), image_height);

  // Draw the Angle Major line
  if(userAngScrollValue >= 90.0 && userAngScrollValue < 270.0) {
    thisX = 768;
    angMajLineY = 128.0 + 128.0*tan(deg2rad(userAngScrollValue));
  }
  else {
    thisX = 1024;
    angMajLineY = 128.0 - 128.0*tan(deg2rad(userAngScrollValue));
  }
  angMajInt = (int)angMajLineY;
  // Kludge this for now ...
  if(angMajLineY > 4000.0) {
    angMajInt = 256;
    thisX = 896;
  }
  else if(angMajLineY < -4000.0) {
    angMajInt = 0;
    thisX = 896;
  }

  XDrawLine(display, image_window1, imageCTF_gcRED,
	    896, 128, thisX, angMajInt);
  
  XFlush (display);
  
} // end function DrawWindow



void imageClick(Widget widget, char *tag,
		XmDrawingAreaCallbackStruct *draw_data)
{
  // function name: imageClick
  // Every time the mouse button is clicked call here. This routine
  // handles the selecting/deselecting of box points and ruler points
  // on the screen.
  //       Button1 is the Left mouse button
  //        Button2 is the Middle mouse button
  //        Button3 is the Right mouse button

  XEvent event;
  int y; 
  char ans;

  // Do appropriate depending on event type
  if(draw_data != 0) { // Only look at events meant for us
    event= *(draw_data->event);
    
    switch (event.type) {
    case ButtonRelease:
      
      break;
      
    case ButtonPress:
      // They are trying to find points
//      x = event.xbutton.x;      // Get x,y copords of point pos
      y = event.xbutton.y;
      
      resoY= y;
      DrawResoExample(resoY);
      
      break;
      
    case KeyPress:
      ans= XLookupKeysym((XKeyEvent *)&event, 0);
      printf ("Keyed[%c]\n",ans);
      
      break;
      
    default:
      break;
    }
  }
  
  XFlush(display);
  
} // end function imageClick



static void plotMotion(Widget widget, caddr_t unused,
		       register XMotionEvent *pEvent)
{
  // function name: plotMotion
  // Whenever the pointer moves across our window call here. This is
  // where we redraw the circles if they're moving a box or print the
  // coords of the pointer as they move.

  int x, y;
  float resoX,inverseResoX,offsetY,ctfVal;
  static int firstTime = TRUE;

  // Make all changes current
  XFlush(display);               
  
  switch (pEvent->type) { // Find out which event we have
    
  case MotionNotify:
    
    // Get coords of pointer
    x = pEvent->x;     
    y = pEvent->y;
    
    resoX= ((float)x)/5120.0*resolutionScrollValue;
    if(resoX != 0.0)
      inverseResoX = 1.0 / resoX;
    else 
      inverseResoX = 0.0;
    if(y > 256)
      offsetY= 0.0 - (y-256.0)/226.0;
    else if(y<256)
      offsetY= (256.0 - y)/226.0;
    else
      offsetY = 0.0;
    /* Fudge this for now - we don't really have a range from 1.0 to -1.0
      it's more like 0.88 to -0.89
    */
    if(offsetY > 0.0)
      offsetY /= 0.88;
    else offsetY /= 0.89;
    ctfVal =  ctf_vals[curCtf][x].tmp;
    if(!firstTime)
      XDrawString(display, ctf_image_window1, imageCTF_gcGRAY,
		  drawPWX, drawPointWhere, 
		  motionNumStr, strlen(motionNumStr));
    sprintf(motionNumStr,"[%.4f Ang-1 or %.2f Ang][CTF= %.2f,Pointer= %.2f]",
	    resoX, inverseResoX, ctfVal, offsetY);
    XDrawString(display, ctf_image_window1, imageCTF_gcBLUE,
		drawPWX, drawPointWhere, 
		motionNumStr, strlen(motionNumStr));
    firstTime = FALSE;
    break;
    
  default:
    break;
  }
  
  XFlush(display);
  
} // end function plotMotion



void plotClick(Widget widget, char *tag,
	       XmDrawingAreaCallbackStruct *draw_data)
{
  // function name: plotClick
  // Every time the mouse button is clicked call here. This routine
  // handles the selecting/deselecting of box points and ruler points
  // on the screen.
  //       Button1 is the Left mouse button
  //       Button2 is the Middle mouse button
  //       Button3 is the Right mouse button

  XEvent event;
  int x; //, y;
  char ans;
  
  // Do appropriate depending on event type
  if(draw_data != 0) {
    event= *(draw_data->event);
    
    switch (event.type) {
    case ButtonRelease:
      
      break;
      
    case ButtonPress:
      // They are trying to find points
      x = event.xbutton.x;      // Get x,y copords of point pos
//      y = event.xbutton.y;
      
      // resXlo = (int)(x);
      if(bandPass==PASSCTF) { // Let's do the PASSCTF case
	if(event.xbutton.button == Button1) { // Defining the low point for passing
	  resXlo = x;
	  if(resXhi < 0)
	    resXhi = 0;
	}
	else if(event.xbutton.button == Button2) { // Defining the hi point for passing
	  resXhi = x;
	  if(resXlo < 0)
	    resXlo = 99513;
	}
	else if(event.xbutton.button == Button3) { // Reset
	  resXlo = -99;
	  resXhi = -99;
	}
	
	if(middlePass && resXhi>resXlo) { // Possible confusion
	  sprintf(outStr,
		  "WARNING:  Inverted filter mask and LOPASS radius < HIPASS radius - all frequencies pass.");
	  askThem(outStr,FALSE,XmDIALOG_INFORMATION);
	}
	if(!middlePass && resXhi>resXlo) { // Possible confusion
	  sprintf(outStr,
		  "WARNING:  With LOPASS radius < HIPASS radius filter mask blocks out all frequencies.");
	  askThem(outStr,FALSE,XmDIALOG_INFORMATION);
	}
	
      }
      
      formResize(0,0,0); 
      
      DrawCTFWindow(FALSE);
      
      applyFFT();
      
      break;
      
    case KeyPress:
      ans= XLookupKeysym((XKeyEvent *)&event, 0);
      printf ("Keyed[%c]\n",ans);
      
      break;
      
    default:
      break;
    }
  }
  
  XFlush(display);
  
} // end function plotClick



void cancel_fileSel(Widget widget, char *tag,
		    XmFileSelectionBoxCallbackStruct *callback_data)
{
  // function name: cancel_fileSel 
  // They want to abort the selection
  
  XtUnmanageChild(fileSel_widget);
  
} // end function cancel_fileSel



void gotFilename (Widget w, char *client_data,
		  XmFileSelectionBoxCallbackStruct *callback_data)
{
  // function name: gotFilename
  // Get the filename they specified in file selection widget. Branch
  // according to what operation they want performed on this filename.

  char oldFilename[256], inStr[1024], inStr2[1024];
  int i, x, y;
  int pifFile, offset;
  off_t imgOffset;
  float spatialScale, *thisFBuf;
  forPifHead_t thisHead;
  fString_t fName,readWrite;
  BYTE *thisBuf, bufPixel;
  FILE *fIn,*fParam;

  if(preLoadSwitch) { // Preload it
    sprintf(fileName.addr,"%s",preLoadStr);
    preLoadSwitch = FALSE;  // Done with it
  }
  else { // normal
    // Pop down the file selection widget
    XtUnmanageChild(fileSel_widget);
    XFlush(display);
    
    // Get the filename
    sprintf(oldFilename,"%s",fileName.addr);
    if(!XmStringGetLtoR(callback_data->value, XmSTRING_DEFAULT_CHARSET, 
			&fileName.addr))
      printf("Error extracting filename.\n");
  }

  sprintf(oldFilename,"%s",fileName.addr);
  toggleThis= 0;
  pifFile = FALSE;
  maxParamWeight = 0.0;

  if(readType == PARAM || readType == PFTSEARCH || readType == EM3DR) {
    // They want to read a param files
    int units;
    float pixSize, volts, amp_fac, focus_maj, focus_min, ang_maj, cs_coeff;
    
    curCtf = 0;
    pixSize = 1.0;
    fIn = fopen(fileName.addr,"r");
    // Everyone has at least 6 lines
    get_str(fIn,'\n',inStr);
    get_str(fIn,'\n',inStr);
    get_str(fIn,'\n',inStr);
    get_str(fIn,'\n',inStr);
    get_str(fIn,'\n',inStr);
    get_str(fIn,'\n',inStr);
    if(readType == PARAM)
      get_str(fIn,'\n',inStr);   // Get seventh line
    if(readType == PFTSEARCH) { // Get seventh through tenth lines
      get_str(fIn,'\n',inStr);   
      get_str(fIn,'\n',inStr);   
      get_str(fIn,'\n',inStr);   
      get_str(fIn,'\n',inStr);   
    }
    // Now get paramfiles
    while(!feof(fIn)) {
      // Get param filename
      get_str(fIn,'\n',inStr);
      if(!EQU(inStr,"")) {
	fParam = fopen(inStr,"r");
	if(fParam) {
	  sprintf(curve[curCtf].paramFname,"%s",inStr);
	  get_str(fParam,'\n',inStr); // Get first line
	  get_str(fParam,',',inStr2);
	  pixSize = (float)strtod(inStr2,0);
	  get_str(fParam,',',inStr2);
	  units = (int)strtod(inStr2,0);
	  get_str(fParam,',',inStr2);
	  volts = (float)strtod(inStr2,0);
	  get_str(fParam,',',inStr2);
	  amp_fac = (float)strtod(inStr2,0);
	  get_str(fParam,',',inStr2);
	  focus_maj = (float)strtod(inStr2,0);
	  get_str(fParam,',',inStr2);
	  focus_min = (float)strtod(inStr2,0);
	  get_str(fParam,',',inStr2);
	  ang_maj = (float)strtod(inStr2,0);
	  get_str(fParam,',',inStr2);
	  cs_coeff = (float)strtod(inStr2,0);
	  
	  if(units != 1)
	    pixSize *= 0.1;  // Convert from nm to Angstroms
	  
	  afractScrollValue   = curve[curCtf].afractScrollValue = amp_fac;
	  avoltageScrollValue = curve[curCtf].avoltageScrollValue = volts;
	  spaberrationScrollValue = 
	    curve[curCtf].spaberrationScrollValue = cs_coeff;
	  majDefocusScrollValue = 
	    curve[curCtf].majDefocusScrollValue = focus_maj;
	  minDefocusScrollValue = 
	    curve[curCtf].minDefocusScrollValue = focus_min;
	  majAngScrollValue = curve[curCtf].majAngScrollValue = 
	    ang_maj;
	  defocusScrollValue    = 
	    curve[curCtf].defocusScrollValue= focus_maj;
	  pfractScrollValue     = 
	    curve[curCtf].pfractScrollValue = 1.0 - amp_fac;
	  
	  alwaysDraw[curCtf] = FALSE;
	  useSum[curCtf] = TRUE;
	  weightings[curCtf] = 0.0;
	  while(!feof(fParam)) { // Count lines
	    weightings[curCtf] += 1.0;
	    get_str(fParam,'\n',inStr);
	  }
	  weightings[curCtf]--;
	  maxParamWeight += weightings[curCtf];
	  fclose(fParam);
	  
	  curve[curCtf].isParamFile = TRUE;
	  curve[curCtf].numParticles = weightings[curCtf];
	  
	  // Instantiate this one
	  setCurveVals(TRUE);
	  DrawCTFWindow(FALSE); 
	  
	  curCtf++;
	}
      }
    }
    fclose(fIn);
    for(i=0;i<=curCtf;i++) {
      weightings[i] /= maxParamWeight;
      curve[i].percPartTot= weightings[i]*100.0;
    }
    
    // Let's just pick the last pixSize read and use that for our default
    sprintf(outStr,"%f",pixSize);
    XmTextSetString(pixSizeValue,outStr);
    
    topCurve = curCtf;
    curCtf--;
    curCtf = MAX(curCtf,0);
    sprintf(outStr,"%d",(curCtf+1));  // User sees 1 for 0
    XmTextSetString(curveTextWidget, outStr);
    sprintf(outStr,"%d",(curCtf+1));  // User sees 1 for 0
    XmTextSetString(topCurveTextWidget, outStr);
    XFlush(display);
    
    // Now set the toggles appropriately
    sumCurveSwitch = TRUE;
    XtVaSetValues(sumCurveWidget, XmNset, sumCurveSwitch, NULL);
    XtVaSetValues(alwaysDrawWidget, XmNset, alwaysDraw[curCtf], NULL);
    XtVaSetValues(useSumWidget, XmNset, useSum[curCtf], NULL);
    
    setCurveVals(TRUE);
    DrawCTFWindow(FALSE); 
    applyFFT();
  }
  else { // They're reading an image
    // Try to read this as a PIF file
    fName.addr = (char *)malloc(sizeof(char)*64);
    readWrite.addr = (char *)malloc(sizeof(char)*64);
    sprintf(fName.addr,"%s",oldFilename);
    sprintf(readWrite.addr,"r");
    i= thisHead.luNum = 4;
    pif_open_(fName.addr, &i, readWrite.addr);
    
    pif_read_gh_(&thisHead);
    
    if(!(thisHead.mode == 46||thisHead.mode == 6||thisHead.mode == 0)) {
      pif_close(&thisHead.luNum);
      printf("Not a mode==46 or  mode==6 or mode==0 PIF file.\n");      
      goto notPifFile;
    }
    
    thisHead.imgNum = 1;
    pif_read_dh_(&thisHead);
    
    if(thisHead.mode==46) {
      int i, j;  // do this local
      int ncol, nrow;
      float minF,maxF,scaleF;
      thisFBuf = (float *)malloc(sizeof(float)*thisHead.numRows*
				 thisHead.numCols);
      thisBuf = (BYTE *)malloc(sizeof(BYTE)*thisHead.numRows*
			       thisHead.numCols);
      thisHead.imgNum = 1;  /* Force first */
      pif_read_boxi4_(&thisHead, thisFBuf);
      ncol = thisHead.numCols;
      nrow = thisHead.numRows;
      // Now scale it down into BYTE
      minF = maxF = thisFBuf[0];
      for(i=0; i<nrow; i++) {
         for(j=0; j<ncol; j++) {
            maxF = MAX(thisFBuf[(i*ncol)+j], maxF);
            minF = MIN(thisFBuf[(i*ncol)+j], minF);
	   }
      }
      scaleF = (254.0) / (maxF - minF);
      for(i=0; i<nrow; i++) {
         for(j=0; j<ncol; j++) {
            thisBuf[(i*ncol)+j]= 
                  (BYTE)((int)((thisFBuf[(i*ncol)+j] - minF)*scaleF)); 
         }
      }
      free(thisFBuf);
    }
    else { // byte
      thisBuf = (BYTE *)malloc(sizeof(BYTE)*thisHead.numRows*
			       thisHead.numCols);
      imgOffset = 1024;
      pif_read_byte_image_(&thisHead.luNum, thisBuf, &imgOffset,
			   &thisHead.numCols,
			   &thisHead.numRows,
			   &thisHead.numSecs);
    }
    
    pif_close(&thisHead.luNum);
    
    spatialScale = (float)thisHead.numCols/256.0;
    
    minOrig = 9999;
    maxOrig = 0-9999;
    for(y=0; y<256; y++) {
      for(x=0; x<256; x++) {
	offset = ((int)(((int)(y*spatialScale))*
			thisHead.numCols)+
		  (int)(x*spatialScale));
	bufPixel = (BYTE)((int)thisBuf[offset]);
	minOrig = MIN(minOrig, (int)bufPixel);
	maxOrig = MAX(maxOrig, (int)bufPixel);
	// Data is stored on disk in TSB world so we must
	// switch in the Y direction
	int_image_buffer1[(255-y)*image_width + x] = (unsigned short)(bufPixel);
	toggleThis++;
	if(toggleThis > 2)
	  toggleThis= 0;;
      }
    }
    
    free(thisBuf);
    pifFile = TRUE;
    
  notPifFile:;
    
    if(!pifFile) { // must be an apx file
      fIn= fopen(oldFilename,"r");
      if(fIn) {
	minOrig = 9999;
	maxOrig = 0-9999;
	for(y=0; y<512; y++) {
	  BYTE thisBuf;
	  for(x=0; x<512; x++) {
	    fread(&thisBuf,sizeof(BYTE),1,fIn);
	    thisBuf = (BYTE)((int)thisBuf);
	    minOrig = MIN(minOrig,(int)thisBuf);
	    maxOrig = MAX(maxOrig,(int)thisBuf);
	    int_image_buffer1[(((int)(y*0.5)*image_width)+
			       (int)(x*0.5))] = 
	      (unsigned short)(thisBuf);

	    ctf2D_image_buffer1[(int)(y*0.5)*256 + (int)(x*0.5)] = 
	      (toggleThis) ? 
	      (unsigned short)PLACEHOLDER :
	      (unsigned short)(PLACEHOLDER+5);
	    
	    toggleThis++;
	    if(toggleThis > 2)
	      toggleThis= 0;;
	  }
	}
      }
      fclose(fIn);
    }
    
    // Remember the gold std for later
    for(y=0; y<256*256; y++)
      resoImage[y] = int_image_buffer1[y];
    
    stuffImage(int_image_buffer1,image_width,image_height,0,TRUE);
    stuffImage(int_image_buffer1,image_width,image_height,256,FALSE);
    stuffImage(fft_image_buffer1,image_width,image_height,512,FALSE);
    stuffImage(ctf2D_image_buffer1,image_width,image_height,768,FALSE);
    
    DrawWindow();
    
    XPutImage (display, ctf_image_window1, ctf_image_gc, ctf_image_image1, 
	       0, 0, 0, 0, ctf_width, ctf_height);
     
    XFlush (display);
    DrawCTFWindow(FALSE);
    applyFFT();
  }

} // end function gotFilename



void ctf_activate(Widget widget, int *tag,
		  XmAnyCallbackStruct *callback_data)
{
  // function name: ctf_activate
  // They said they wanted to ctf the image
  
  XtManageChild(ctfscreen);
  
} // end function ctf_activate



void ctf_reset_activate(Widget widget, int *tag, 
			XmAnyCallbackStruct *callback_data)
{
  // function name: ctf_reset_activate
  // They said they wanted to reset.

  XmString simpleString;

  defocus    = 1000;
  cs         = 2.0;
  voltage    = 100000.0;
  pfract     = 1.0;
  afract     = 0.0;
  wavelength = 1230.0/sqrt(voltage + (1.0e-6)*(voltage*voltage));
  res_limit  = 1.0;   // ABSCISSA RESOLUTION LIMIT IN RECIPROCAL NM UNITS

  // Values common to all scopes
  // resolutionScrollValue = 1.0;
  majDefocusScrollValue = 1.0;
  minDefocusScrollValue = 1.0;
  majAngScrollValue = 0.0;
  wienerScrollValue = 0.0;
  defocusScrollValue = 1.000;
  
  if(*tag==ALL) { // normal
    afractScrollValue = 0.0;
    pfractScrollValue = 1.0;
    avoltageScrollValue = 100000.0;
    avoltageScrollValue = 200000.0;   // Default to 200 kV for FEG
    spaberrationScrollValue = 2.0;
    chaberrationScrollValue = 2.0;
    spCoherenceScrollValue = 5.0;
    spCoExp = -3;
    spCoFactor = 0.001;
    tmCoherenceScrollValue = 0.0;
  }
  else if(*tag==FEG) { // CM200 feg
    afractScrollValue = 0.05;
    pfractScrollValue = 0.95;
    avoltageScrollValue = 200000.0;
    spaberrationScrollValue = 2.0;
    chaberrationScrollValue = 2.0;
    spCoherenceScrollValue = 5.0;
    spCoExp = -2;
    spCoFactor = 0.01;
    tmCoherenceScrollValue = 0.5;
  }
  else if(*tag==FEG300) { // feg 300
    afractScrollValue = 0.04;
    pfractScrollValue = 0.96;
    avoltageScrollValue = 300000.0;
    spaberrationScrollValue = 2.0;
    chaberrationScrollValue = 2.0;
    spCoherenceScrollValue = 5.0;
    spCoExp = -3;
    spCoFactor = 0.001;
    tmCoherenceScrollValue = 0.5;
  }
  else if(*tag==EM300) { // nermag
    afractScrollValue = 0.07;
    pfractScrollValue = 0.93;
    avoltageScrollValue = 100000.0;
    spaberrationScrollValue = 1.6;
    chaberrationScrollValue = 1.3;
    spCoherenceScrollValue = 5.0;
    spCoExp = -3;
    spCoFactor = 0.001;
    tmCoherenceScrollValue = 1.6;
  }
  else if(*tag==EM420W) { // em420
    afractScrollValue = 0.07;
    pfractScrollValue = 0.93;
    avoltageScrollValue = 120000.0;
    spaberrationScrollValue = 2.0;
    chaberrationScrollValue = 2.0;
    spCoherenceScrollValue = 1.0;
    // Now set the exponent to 00
    spCoFactor = 1.0;
    spCoExp = 0;
    tmCoherenceScrollValue = 1.6;
  }
  else if(*tag==EM420L) { // em420 LaB6
    afractScrollValue = 0.07;
    pfractScrollValue = 0.93;
    avoltageScrollValue = 120000.0;
    spaberrationScrollValue = 2.0;
    chaberrationScrollValue = 2.0;
    spCoherenceScrollValue = 0.5;
    spCoExp = 0;
    spCoFactor = 1.0;
    tmCoherenceScrollValue = 0.8;
    }
  else if(*tag==JEOL_EM_2010F_HT_FEG) { // JEOL_EM-2010F-HT_FEG
    afractScrollValue = 0.05;
    pfractScrollValue = 0.95;
    avoltageScrollValue = 200000.0;
    spaberrationScrollValue = 1.4;
    chaberrationScrollValue = 1.8;
    spCoherenceScrollValue = 5.0;
    spCoExp = -3;
    spCoFactor = 0.001;
    tmCoherenceScrollValue = 0.5;
  }
  else if(*tag==JEOL_EM_2010F_UHR_FEG) { // JEOL_EM-2010F-UHR_FEG
    afractScrollValue = 0.05;
    pfractScrollValue = 0.95;
    avoltageScrollValue = 200000.0;
    spaberrationScrollValue = 0.5;
    chaberrationScrollValue = 1.1;
    spCoherenceScrollValue = 5.0;
    spCoExp = -3;
    spCoFactor = 0.001;
    tmCoherenceScrollValue = 0.5;
  }
  else if(*tag==JEOL1010) { // JEOL1010
    afractScrollValue = 0.05;
    pfractScrollValue = 0.95;
    avoltageScrollValue = 100000.0;
    spaberrationScrollValue = 1.9;
    chaberrationScrollValue = 1.9;
    spCoherenceScrollValue = 5.0;
    spCoExp = -3;
    spCoFactor = 0.001;
    tmCoherenceScrollValue = 0.5;
  }
  else if(*tag==Hitachi_H9000) { // Hitachi_H9000
    afractScrollValue = 0.05;
    pfractScrollValue = 0.95;
    avoltageScrollValue = 120000.0;
    spaberrationScrollValue = 0.9;
    chaberrationScrollValue = 2.0;
    spCoherenceScrollValue = 0.5;
    spCoExp = -3;
    spCoFactor = 0.001;
    tmCoherenceScrollValue = 0.5;
  }
  else if(*tag==Hitachi_HF2000) { // Hitachi_HF2000
    afractScrollValue = 0.05;
    pfractScrollValue = 0.95;
    avoltageScrollValue = 200000.0;
    spaberrationScrollValue = 1.2;
    chaberrationScrollValue = 1.5;
    spCoherenceScrollValue = 0.2;
    spCoExp = -3;
    spCoFactor = 0.001;
    tmCoherenceScrollValue = 0.3;
  }
  else if(*tag==FEI_Tecnai_G2_Polara) { // FEI_Tecnai_G2_Polara
    afractScrollValue = 0.04;
    pfractScrollValue = 0.96;
    avoltageScrollValue = 300000.0;
    spaberrationScrollValue = 2.3;
    chaberrationScrollValue = 2.0;
    spCoherenceScrollValue = 0.1;
    spCoExp = -3;
    spCoFactor = 0.001;
    tmCoherenceScrollValue = 0.7;
  }
  else if(*tag==FEI_Tecnai_G2_Sphera) { // FEI_Tecnai_G2_Sphera
    afractScrollValue = 0.05;
    pfractScrollValue = 0.95;
    avoltageScrollValue = 200000.0;
    spaberrationScrollValue = 2.26;
    chaberrationScrollValue = 2.0;
    spCoherenceScrollValue = 1.0;
    spCoExp = -3;
    spCoFactor = 0.001;
    tmCoherenceScrollValue = 1.0;
  }
  else if(*tag==USER) { // user defaults
    userDefaults();
  }

  if(spCoExp < 0)
    sprintf(outStr," * E%03d",spCoExp);
  else 
    sprintf(outStr," * E+%02d",spCoExp);
  simpleString = XmStringCreateLocalized(outStr);
  XtSetArg(arglist[0], XmNlabelString, simpleString);
  XtSetArg(arglist[1], XmNrecomputeSize, FALSE );
  XtSetValues(spCoExpLabelW, arglist, 2); 
  XmStringFree(simpleString); 
  XFlush(display);
  
  curve[curCtf].afractScrollValue       = afractScrollValue;
  curve[curCtf].resolutionScrollValue   = resolutionScrollValue;
  curve[curCtf].avoltageScrollValue     = avoltageScrollValue;
  curve[curCtf].spaberrationScrollValue = spaberrationScrollValue;
  curve[curCtf].chaberrationScrollValue = chaberrationScrollValue;
  curve[curCtf].spCoherenceScrollValue  = spCoherenceScrollValue;
  curve[curCtf].tmCoherenceScrollValue  = tmCoherenceScrollValue;
  curve[curCtf].majDefocusScrollValue   = majDefocusScrollValue;
  curve[curCtf].minDefocusScrollValue   = minDefocusScrollValue;
  curve[curCtf].majAngScrollValue       = majAngScrollValue;
  curve[curCtf].wienerScrollValue       = wienerScrollValue;
  curve[curCtf].defocusScrollValue      = defocusScrollValue;
  curve[curCtf].pfractScrollValue       = pfractScrollValue;
  curve[curCtf].spCoExp                 = spCoExp;
  curve[curCtf].spCoFactor              = spCoFactor;

  setCurveVals(TRUE);
  XFlush(display);
  DrawCTFWindow(FALSE); 
  applyFFT();

} // end function ctf_reset_activate



void ctf_exit_activate(Widget  widget, int *tag,
		       XmAnyCallbackStruct *callback_data)
{
  // function name: ctf_exit_activate
  // They said they wanted to ctf the image
  
  XtUnmanageChild(ctfscreen);
  
} // end function ctf_exit_activate



void ctfWindow_create(Widget widget, int *tag,
		      XmAnyCallbackStruct *callback_data)
{
  // function name: ctfWindow_create
  // Remember the drawing area window pointer for use later. Setup the
  // file selection widget here.
  
  ctfDrawingWidget = widget;

} // end function ctfWindow_create



void DrawResoExample(int thisY)
{
  // function name: DrawResoExample
  // Redraw window e.g. on expose events or whenever it's prudent.

  int i, thisX, success; //, lastX;
  float scaleIt;
  float *tmpBuf;
  float minF, maxF, filler; //, range, offset;
  float rx, ry, ix, iy, rotation, scale, scaleF;

  tmpBuf = (float *)malloc(sizeof(float)*256*256);
      
  // resoImage is gold std
  minF = maxF = (float)resoImage[0];
  for(i=0; i<256*256; i++) {
    tmpBuf[i] = resoImage[i] ;
    minF = MIN(minF,tmpBuf[i]);
    maxF = MAX(maxF,tmpBuf[i]);
  }
  
//  range  = maxF - minF;
//  offset = (int)minF;
	  
  rx = ry = ix = iy = 0.0;
  rotation = 0.0;
  filler = (float)tmpBuf[0];
  
  scale = (scrollResolutionExample/256.0);
  file_width = file_height = 256;
  success= interp2d(tmpBuf, &file_width,  &file_height,
		    &rx, &ry, &ix, &iy,
		    &rotation, &scale, &filler);

  scale = (256.0/scrollResolutionExample);
  success= interp2d(tmpBuf, &file_width,  &file_height,
		    &rx, &ry, &ix, &iy,
		    &rotation, &scale, &filler);
  
  // Now stuff interpolated buffer back into BYTE
  minF = maxF = tmpBuf[0];
  for (i=0; i<256*256; i++) {
    minF = MIN(minF,tmpBuf[i]);
    maxF = MAX(maxF,tmpBuf[i]);
  }
  scaleF = 255.0 / (maxF - minF);
  
  for(i=0; i<256*256; i++)
    int_image_buffer1[i] = (tmpBuf[i]-minF) * scaleF;
  
  free(tmpBuf);
  
  stuffImage(int_image_buffer1,image_width,image_height,0,TRUE);
  applyFFT();
  DrawWindow();
  
  XFlush (display);
  
  if(resoDraw) { // Draw where they selected
    XDrawLine(display, image_window1, imageCTF_gcRED,
	      0, thisY, 256, thisY);
    
    // Now draw the resolution example
    scaleIt = 256.0/scrollResolutionExample;
//    lastX = 0;
    for(i=0; i<256; i++) {
      int thisIndex, thisXp1;
      thisIndex = (float)i / scaleIt;
      thisIndex = (float)thisIndex * scaleIt;
      thisX = int_image_buffer1[(thisY*256)+thisIndex];
      thisX = 256 - thisX;
      thisIndex = (float)i+1 / scaleIt;
      thisIndex = (float)thisIndex * scaleIt;
      thisXp1 = int_image_buffer1[(thisY*256)+thisIndex];
      thisXp1 = 256 - thisXp1;
      XDrawLine(display, image_window1, imageCTF_gcGREEN,
		i, thisX, i+1, thisXp1);
//      lastX= thisX;
    }
  }
  
} // end function DrawResoExample



void DrawCTFCorrectWindow()
{
  // function name: DrawCTFCorrectWindow
  // Redraw window e.g. on expose events or whenever it's prudent.

  int i, draw_width, draw_height;
  char numStr[16];

  ctfCorrectHeight    = 450;
  middleLine = 480-(int)(ctfCorrectHeight/maxCTFCorrectYvalue);
  topLine    = 480-(int)((ctfCorrectHeight/maxCTFCorrectYvalue)*2.0);

  // Draw all of image
  draw_width = ctf_width;
  draw_height = ctf_height;
  XPutImage(display, ctf_image_window1, ctf_image_gc, ctf_image_image1,
	     0, 0, 0, 0,  draw_width,  draw_height);
  XDrawLine(display, ctf_image_window1, imageCTF_gcGREEN,
	    1, 0, 1, 512);
  XDrawLine(display, ctf_image_window1, imageCTF_gcGREEN,
	    0, middleLine,
	    512, middleLine);
  XDrawLine(display, ctf_image_window1, imageCTF_gcGREEN,
	    0, topLine,
	    512, topLine);

  // Now draw the resolution line with tick marks
  XDrawLine(display, ctf_image_window1, imageCTF_gcGREEN,
	    1, 480, 511, 480);
  XDrawLine(display, ctf_image_window1, imageCTF_gcGREEN,
	    1, 30, 511, 30);

  for(i=0; i<512; i+=16) { // Draw ticks
    if(i%64 == 0) { // Big one
      // Draw Middle line tick
      XDrawLine(display, ctf_image_window1, imageCTF_gcGREEN,
		i, middleLine, i, middleLine+10);   // Make larger
      // Draw Top line tick
      XDrawLine(display, ctf_image_window1, imageCTF_gcGREEN,
		i, topLine, i, topLine+10);   // Make larger
      // Draw bottom tick
      XDrawLine(display, ctf_image_window1, imageCTF_gcGREEN,
		i, 480, i, 490);   // Make larger
      // Draw top tick
      XDrawLine(display, ctf_image_window1, imageCTF_gcGREEN,
		i, 20, i, 30);   // Make larger
      if(i>0 && i != 192 && (i<257)) { // Don't do the first
	sprintf(numStr,"%.3f",(((float)i)/
			       5120.0*resolutionScrollValue));
	XDrawString(display, ctf_image_window1, imageCTF_gcBLUE,
		    (i-20), 505, numStr, strlen(numStr));
      }
    }
    else { // Little
      XDrawLine(display, ctf_image_window1, imageCTF_gcGREEN,
		i, 480, i, 485);
      XDrawLine(display, ctf_image_window1, imageCTF_gcGREEN,
		i, 30, i, 25);
    }
  }

  i=511;
  // Draw bottom tick
  XDrawLine(display, ctf_image_window1, imageCTF_gcGREEN,
	    i, 480, i, 490);   // Make larger
  // Draw top tick
  XDrawLine(display, ctf_image_window1, imageCTF_gcGREEN,
	    i, 20, i, 30);   // Make larger
  sprintf(numStr,"%.3f",(((float)i+1.0)/
			 5120.0*resolutionScrollValue));
  XDrawString(display, ctf_image_window1, image_gcBLUE,
	      (i-25), 505, numStr, strlen(numStr));
  
  if(bandPass==PASSCTF) { // mark where the rexlo and rexhi are
    sprintf(numStr,"<");
    XDrawString(display, ctf_image_window1, image_gcBLACK,
		(resXlo), middleLine+4, numStr, strlen(numStr));
    sprintf(numStr,">");
    XDrawString(display, ctf_image_window1, image_gcBLACK,
		(resXhi-5), middleLine+4, numStr, strlen(numStr));
  }
  
  // Draw values of scale in Y direction
  sprintf(numStr,"%.1f",maxCTFCorrectYvalue - 1.0);
  XDrawString(display, ctf_image_window1, imageCTF_gcBLACK,
	      0, 30, numStr, strlen(numStr));
  sprintf(numStr,"%.1f",1.0);
  XDrawString(display, ctf_image_window1, imageCTF_gcBLACK,
	      0, topLine, numStr, strlen(numStr));
  sprintf(numStr,"%.1f",-1.0);
  XDrawString(display, ctf_image_window1, imageCTF_gcBLACK,
	      0, 480, numStr, strlen(numStr));
  
  XFlush (display);
  calcCTF(curCtf); // Draw the curve
  
} // end function DrawCTFCorrectWindow



void DrawCTFWindow(int both)
{
  // function name: DrawCTFWindow
  // Redraw window e.g. on expose events or whenever it's prudent.
  
  char numStr[16];
  int i, draw_width, draw_height;
  
  if(XtIsManaged(ctfCorrectScreen)) { // Do ctfCorrect display
    DrawCTFCorrectWindow();
    return;
  }
  
  // Draw all of image
  draw_width = ctf_width;
  draw_height = ctf_height;
  XPutImage(display, ctf_image_window1, ctf_image_gc, ctf_image_image1,
	    0, 0, 0, 0, draw_width, draw_height);
  XDrawLine(display, ctf_image_window1, imageCTF_gcGREEN,
	    1, 0, 1, 512);
  XDrawLine(display, ctf_image_window1, imageCTF_gcGREEN,
	    0, (ctf_height/2), 512, (ctf_height/2));
  
  // Now draw the resolution line with tick marks
  XDrawLine(display, ctf_image_window1, imageCTF_gcGREEN,
	    1, 480, 511, 480);
  XDrawLine(display, ctf_image_window1, imageCTF_gcGREEN,
	    1, 30, 511, 30);
  
  for(i=0; i<512; i+=16) { // Draw ticks
    if(i%64 == 0) { // Big one
      // Draw Middle line tick
      XDrawLine(display, ctf_image_window1, imageCTF_gcGREEN,
		i, 251, i, 261);   // Make larger
      
      // Draw bottom tick
      XDrawLine(display, ctf_image_window1, imageCTF_gcGREEN,
		i, 480, i, 490);   // Make larger
      
      // Draw top tick
      XDrawLine(display, ctf_image_window1, imageCTF_gcGREEN,
		i, 20, i, 30);   // Make larger
      if(i>0 && i != 192 && (i<257)) { // Don't do the first
	sprintf(numStr,"%.3f",(((float)i)/
			       5120.0*resolutionScrollValue));
	XDrawString(display, ctf_image_window1, imageCTF_gcBLUE,
		    (i-20), 505, numStr, strlen(numStr));
      }
    }
    else { // Little
      XDrawLine(display, ctf_image_window1, imageCTF_gcGREEN,
		i, 480, i, 485);
      XDrawLine(display, ctf_image_window1, imageCTF_gcGREEN,
		i, 30, i, 25);
    }
  }
  i=511;
  
  // Draw bottom tick
  XDrawLine(display, ctf_image_window1, imageCTF_gcGREEN,
	    i, 480, i, 490);   // Make larger
  
  // Draw top tick
  XDrawLine(display, ctf_image_window1, imageCTF_gcGREEN,
	    i, 20, i, 30);   // Make larger
  sprintf(numStr,"%.3f",(((float)i+1.0)/
			 5120.0*resolutionScrollValue));
  XDrawString(display, ctf_image_window1, image_gcBLUE,
	      (i-25), 505, numStr, strlen(numStr));
  
  // Draw Middle line Data Limit tick
  i = 5120.0 / (resolutionScrollValue*pixSizeTextValue*2.0);
  XDrawLine(display, ctf_image_window1, imageCTF_gcRED,
	    i, 251, i, 261);   // Make larger
  
  if(bandPass==PASSCTF) { // mark where the rexlo and rexhi are
    sprintf(numStr,"<");
    XDrawString(display, ctf_image_window1, image_gcBLACK,
		(resXlo), 260, numStr, strlen(numStr));
    sprintf(numStr,">");
    XDrawString(display, ctf_image_window1, image_gcBLACK,
		(resXhi-5), 260, numStr, strlen(numStr));
  }
  
  XFlush (display);
  calcCTF(curCtf); // Draw the curve
  
} // end function DrawCTFWindow



void ctfFormResize(Widget  widget, int *tag,
		   XmDrawingAreaCallbackStruct *callback_data)
{
  // function name: ctfFormResize
  // When the main screen formDialog is resized call here. Get the new
  // window size and setup our globals.

  Window tWin;
//  int width, height; 
  XWindowAttributes winAttrib;

  if(inited) {
    tWin = (Window)XtWindow(drawingWidget);  
    if(tWin > 0) { // If valid window, get new window dimensions
      XGetWindowAttributes(display,tWin,&winAttrib);
//      width  = winAttrib.width;
//      height = winAttrib.height;
    } 
  }
  
} // end function ctfFormResize



void textActivate(Widget widget, int *tag,
		  XmAnyCallbackStruct *callback_data)
{
  // function name: textActivate
  // When they press the carriage return in any of the text
  // widgets call here.

  char *thisStr;

  if(widget==ctfCorrectFFW) { 
    thisStr = XmTextGetString(ctfCorrectFFW);
    fudgeFactor = (float)strtod(thisStr,0);
    XtFree(thisStr);
    if(fudgeFactor <= 0.0) { // Reset it
      fudgeFactor = 0.000001;
      sprintf(outStr,"%f",fudgeFactor);
      XmTextSetString(ctfCorrectFFW,outStr);
    }
    filt_fac[0] = fudgeFactor;
    DrawCTFWindow(TRUE);
    applyFFT();
  }
  else if(widget==ctfCorrectFF2W) {
    thisStr = XmTextGetString(ctfCorrectFF2W);
    fudgeFactor = (float)strtod(thisStr,0);
    XtFree(thisStr);
    if(fudgeFactor <= 0.0) { // Reset it
      fudgeFactor = 0.000001;
      sprintf(outStr,"%f",fudgeFactor);
      XmTextSetString(ctfCorrectFF2W,outStr);
    }
    filt_fac[1] = fudgeFactor;
    DrawCTFWindow(TRUE);
    applyFFT();
  }
  else { // normal
    setCurveVals(FALSE);  // Don't redo the text widgets
    DrawCTFWindow(FALSE); 
    applyFFT();
  }
  
} // end function textActivate



void ctfValueCreate(Widget widget, int *tag,
		    XmAnyCallbackStruct *callback_data)
{
  // function name: ctfValueCreate
  
  if(*tag==AFRACT) {
    afractValue= widget;
    XmTextSetString(afractValue, "0.00/1.00");
    handle_cr(widget);
  }
  else if(*tag==PFRACT)
    pfractValue= widget;
  else if(*tag==RESOLUTION)
    resolutionValue= widget;
  else if(*tag==AVOLTAGE) {
    avoltageValue= widget;
    XmTextSetString(avoltageValue, "200 kV");
    handle_cr(widget);
  }
  else if(*tag==ABERRATION_S) {
    spaberrationValue= widget;
    XmTextSetString(spaberrationValue, "2.00 mm");
    handle_cr(widget);
  }
  else if(*tag==ABERRATION_C) {
    chaberrationValue= widget;
    XmTextSetString(chaberrationValue, "2.00 mm");
    handle_cr(widget);
  }
  else if(*tag==COHERENCE_S) {
    spCoherenceValue= widget;
    XmTextSetString(spCoherenceValue, "0.000 mRad");
    handle_cr(widget);
  }
  else if(*tag==COHERENCE_T) {
    tmCoherenceValue= widget;
    XmTextSetString(tmCoherenceValue, "0.000 eV");
    handle_cr(widget);
  }
  else if(*tag==MAJDEFOCUS) {
    majDefocusValue= widget;
    XmTextSetString(majDefocusValue, "0.000 um");
    handle_cr(widget);
  }
  else if(*tag==MINDEFOCUS) {
    minDefocusValue= widget;
    XmTextSetString(minDefocusValue, "0.000 um");
    handle_cr(widget);
  }
  else if(*tag==MAJANG) {
    majAngValue= widget;
    XmTextSetString(majAngValue, "0.00");
    handle_cr(widget);
  }
  else if(*tag==USERANG) {
    userAngValue= widget;
    XmTextSetString(userAngValue, "0.00");
    handle_cr(widget);
  }
  else if(*tag==USERDEF) {
    userDefocusValue= widget;
    handle_cr(widget);
  }
  else if(*tag==WIENER) {
    wienerValue= widget;
    XmTextSetString(wienerValue, "0.00");
    handle_cr(widget);
  }
  else if(*tag==PIXSIZE) {
    pixSizeValue= widget;
    handle_cr(widget);
  }
  else if(*tag==DEFOCUS) {
    defocusValue= widget;
    XmTextSetString(defocusValue, "1.200 um");
    handle_cr(widget);
  }
  else if(*tag==COHERENCE_S_EXP) // beta exponent
    spCoExpLabelW= widget;
  else if(*tag==ACTF)
    actfW= widget;
  else if(*tag==PCTF)
    pctfW= widget;
  else if(*tag==ACTFPCTF)
    actfPctfW= widget;
  else if(*tag==CTFCFF) { 
    ctfCorrectFFW = widget;
    handle_cr(widget);    
  }
  else if(*tag==CTFCFF2) { 
    ctfCorrectFF2W = widget;
    handle_cr(widget);    
  }
  else if(*tag==NONE)
    ctfcMethNoneW = widget;
  else if(*tag==CTFCMETH1)
    ctfcMeth1W = widget;
  else if(*tag==CTFCMETH2)
    ctfcMeth2W = widget;
  else if(*tag==CTFCMETH3)
    ctfcMeth3W = widget;
  else if(*tag==CTFCMETH4)
    ctfcMeth4W = widget;
  else if(*tag==CTFCMETH5)
    ctfcMeth5W = widget;
  
} // end function ctfValueCreate



void ctfValueChange(Widget widget, int *tag,
		    XmAnyCallbackStruct *callback_data)
{
  // function name: ctfValueChange
  
  char *paramText;
  float floatVal;
  
  if(!inited)
    return;

  paramText = XmTextGetString(widget); 
  if(*tag==AFRACT) { // Special case
    char tmp[25],*s1;
    sprintf(tmp,"%s",paramText);
    s1 = strchr(tmp,'/');
    if(s1)
      *s1 = '\0';
    floatVal= (float)strtod(tmp,0);
  }
  else
    floatVal= (float)strtod(paramText,0);
  XtFree(paramText);
  
  if(*tag==AVOLTAGE) {
    // Avoltage is expressed in kV in interface
    avoltageScrollValue = floatVal * 1000.0;
    curve[curCtf].avoltageScrollValue   = avoltageScrollValue;
  }
  else if(*tag==AFRACT) {
    afractScrollValue = floatVal;
    pfractScrollValue = 1.0 - afractScrollValue;
    curve[curCtf].afractScrollValue   = afractScrollValue;
  }
  else if(*tag==ABERRATION_S) {
    spaberrationScrollValue = floatVal;
    curve[curCtf].spaberrationScrollValue = spaberrationScrollValue;
  }
  else if(*tag==ABERRATION_C) {
    chaberrationScrollValue = floatVal;
    curve[curCtf].chaberrationScrollValue = chaberrationScrollValue;
  }
  else if(*tag==COHERENCE_S) {
    spCoherenceScrollValue = floatVal;
    curve[curCtf].spCoherenceScrollValue = spCoherenceScrollValue;
  }
  else if(*tag==COHERENCE_T) {
    tmCoherenceScrollValue = floatVal;
    curve[curCtf].tmCoherenceScrollValue = tmCoherenceScrollValue;
  }
  else if(*tag==MAJDEFOCUS) {
    majDefocusScrollValue = floatVal;
    curve[curCtf].majDefocusScrollValue = majDefocusScrollValue;
  }
  else if(*tag==MINDEFOCUS) {
    minDefocusScrollValue = floatVal;
    curve[curCtf].minDefocusScrollValue = minDefocusScrollValue;
  }
  else if(*tag==USERANG) {
    userAngScrollValue = floatVal;
  }
  else if(*tag==MAJANG) {
    majAngScrollValue = floatVal;
    curve[curCtf].majAngScrollValue = majAngScrollValue;
  }
  else if(*tag==WIENER) {
    wienerScrollValue = floatVal;
    curve[curCtf].wienerScrollValue = wienerScrollValue;
  }
  else if(*tag==PIXSIZE) {
    pixSizeTextValue = floatVal;
  }
  else if(*tag==DEFOCUS) {
    defocusScrollValue = floatVal;
    curve[curCtf].defocusScrollValue    = defocusScrollValue;
    majDefocusScrollValue = floatVal;
    curve[curCtf].majDefocusScrollValue = majDefocusScrollValue;
    minDefocusScrollValue = floatVal;
    curve[curCtf].minDefocusScrollValue = minDefocusScrollValue;
    sprintf(outStr,"%.3f um",majDefocusScrollValue);
    XmTextSetString(majDefocusValue,outStr);
    sprintf(outStr,"%.3f um",minDefocusScrollValue);
    XmTextSetString(minDefocusValue,outStr);
  }
  
  // DON'T DO THIS HERE - it messes up the scrolls!!!!
  // if(FALSE)
  // setCurveVals(FALSE);
  
} // end function ctfValueChange



void ctfScrollCreate(Widget  widget, int *tag,
		     XmAnyCallbackStruct *callback_data)
{
  // function name: ctfScrollCreate
  // Remember widget numbers
  
  if(*tag==AFRACT)
    afractScroll= widget;
  else if(*tag==PFRACT)
    pfractScroll= widget;
  else if(*tag==RESOLUTION)
    resolutionScroll= widget;
  else if(*tag==AVOLTAGE)
    avoltageScroll= widget;
  else if(*tag==ABERRATION_S)
    spaberrationScroll= widget;
  else if(*tag==ABERRATION_C)
    chaberrationScroll= widget;
  else if(*tag==COHERENCE_S)
    spCoherenceScroll= widget;
  else if(*tag==COHERENCE_T)
    tmCoherenceScroll= widget;
  else if(*tag==MAJDEFOCUS)
    majDefocusScroll= widget;
  else if(*tag==MINDEFOCUS)
    minDefocusScroll= widget;
  else if(*tag==MAJANG)
    majAngScroll= widget;
  else if(*tag==USERANG)
    userAngScroll= widget;
  else if(*tag==WIENER)
    wienerScroll= widget;
  else if(*tag==DEFOCUS)
    defocusScroll= widget;

} // end function ctfScrollCreate



void ctf_ioparms_activate(Widget widget, int *tag,
			  XmAnyCallbackStruct *callback_data)
{
  // function name: ctf_ioparms_activate
  // They want to exchange ctf params with the FFT screen of robem.
  // There is a corrsponding callback in robem.c that read/writes this
  // info.
  
  FILE *fIO;
  char *ctfText, inStr[256], fileStr[512];
  int i;
  
  if(*tag==0) { // Output
    sprintf(fileStr,"%s/ctf_ioparms.dat",getenv("HOME"));
    fIO = fopen(fileStr,"w");
    if(fIO) {
      ctfText = XmTextGetString(afractValue); 
      fprintf(fIO,"%s\n",ctfText);
      XtFree(ctfText);
      ctfText = XmTextGetString(spaberrationValue); 
      fprintf(fIO,"%s\n",ctfText);
      XtFree(ctfText);
      ctfText = XmTextGetString(majDefocusValue); 
      fprintf(fIO,"%s\n",ctfText);
      XtFree(ctfText);
      ctfText = XmTextGetString(minDefocusValue); 
      fprintf(fIO,"%s\n",ctfText);
      XtFree(ctfText);
      ctfText = XmTextGetString(avoltageValue); 
      fprintf(fIO,"%s\n",ctfText);
      XtFree(ctfText);
      ctfText = XmTextGetString(wienerValue); 
      fprintf(fIO,"%s\n",ctfText);
      XtFree(ctfText);
      ctfText = XmTextGetString(majAngValue); 
      fprintf(fIO,"%s\n",ctfText);
      XtFree(ctfText);
      
      fclose(fIO);
    }
    else askThem("Can't open ctf_ioparms.dat file.",
		 FALSE,XmDIALOG_INFORMATION);
  }
  else { // Input
    sprintf(fileStr,"%s/ctf_ioparms.dat",getenv("HOME"));
    fIO = fopen(fileStr,"r"); 
    if(fIO){
      get_str(fIO, '\n', inStr);
      afractScrollValue = strtod(inStr,0);
      pfractScrollValue = 1.0 - afractScrollValue;
      
      get_str(fIO, '\n', inStr);
      spaberrationScrollValue = strtod(inStr,0);
      
      get_str(fIO, '\n', inStr);
      defocusScrollValue = strtod(inStr,0);
      majDefocusScrollValue = strtod(inStr,0);
      
      get_str(fIO, '\n', inStr);
      minDefocusScrollValue = strtod(inStr,0);
      
      get_str(fIO, '\n', inStr);
      avoltageScrollValue = strtod(inStr,0) * 1000.0;
      
      get_str(fIO, '\n', inStr);
      wienerScrollValue = strtod(inStr,0);
      
      get_str(fIO, '\n', inStr);
      majAngScrollValue = strtod(inStr,0);
      
      for(i=0; i<MAXCURVE; i++) { // DO all curves
	curve[i].afractScrollValue     = afractScrollValue;
	curve[i].resolutionScrollValue = resolutionScrollValue;
	curve[i].avoltageScrollValue   = avoltageScrollValue;
	curve[i].spaberrationScrollValue = spaberrationScrollValue;
	curve[i].chaberrationScrollValue = chaberrationScrollValue;
	curve[i].spCoherenceScrollValue = spCoherenceScrollValue;
	curve[i].tmCoherenceScrollValue = tmCoherenceScrollValue;
	curve[i].majDefocusScrollValue = majDefocusScrollValue;
	curve[i].minDefocusScrollValue = minDefocusScrollValue;
	curve[i].majAngScrollValue = majAngScrollValue;
	curve[i].wienerScrollValue = wienerScrollValue;
	curve[i].defocusScrollValue    = defocusScrollValue;
	curve[i].pfractScrollValue     = pfractScrollValue;
	curve[i].spCoExp        = spCoExp;
	curve[i].spCoFactor     = spCoFactor;
      }
      
      fclose(fIO);
      
      setCurveVals(TRUE);
      
      if(inited) {
	// Only draw when we've got everything we nned to draw
	DrawCTFWindow(FALSE); 
	applyFFT();
      }
      
    }
    else askThem("Can't open ctf_ioparms.dat file.",
		 FALSE,XmDIALOG_INFORMATION);
  }

} // end function ctf_ioparms_activate



void ctfChange(Widget  widget, int *tag,
	       XmScrollBarCallbackStruct *callback_data)
{
  // function name: ctfChange
  // When the scroll on the ctfscreen screen changes value call here.

  float floatVal;
  XmString simpleString;
  
  if(*tag==AFRACT) { // changing the afract
    floatVal = (int)callback_data->value * 0.01;
    afractScrollValue = floatVal;    
    pfractScrollValue = 1.0 - floatVal;
    
    sprintf(outStr,"%.02f/%.02f",afractScrollValue,pfractScrollValue);
    XmTextSetString(afractValue,outStr);
    
    XtVaSetValues(actfW, XmNset, FALSE, NULL);
    XtVaSetValues(pctfW, XmNset, FALSE, NULL);
    XtVaSetValues(actfPctfW, XmNset, TRUE, NULL);
  }
  else if(*tag==PFRACT) { // changing the pfract
    // Unused for now
  }
  else if(*tag==RESOLUTION) { // changing the resolution
    floatVal = (float)callback_data->value * 0.01;
    sprintf(outStr,"%.2f",floatVal);
    XmTextSetString(resolutionValue,outStr);
    resolutionScrollValue = floatVal;
  }
  else if(*tag==AVOLTAGE) { // changing the avoltage
    floatVal = (float)callback_data->value;
    sprintf(outStr,"%.0f kV",floatVal);
    floatVal *= 1000.0;    // Let's do this in 1000's
    XmTextSetString(avoltageValue,outStr);
    avoltageScrollValue = floatVal;
  }
  else if(*tag==ABERRATION_S) { // changing the spherical aberration
    floatVal = (float)callback_data->value * 0.01;
    sprintf(outStr,"%.2f mm",floatVal);
    XmTextSetString(spaberrationValue,outStr);
    spaberrationScrollValue = floatVal;
  }
  else if(*tag==ABERRATION_C) {// changing the chromatic aberration
    floatVal = (float)callback_data->value * 0.01;
    sprintf(outStr,"%.2f mm",floatVal);
    XmTextSetString(chaberrationValue,outStr);
    chaberrationScrollValue = floatVal;
  }
  else if(*tag==COHERENCE_S) { // changing the spatialcoherence
    floatVal = (float)callback_data->value * 0.01;
    sprintf(outStr,"%.3f mRad",floatVal);
    XmTextSetString(spCoherenceValue,outStr);
    spCoherenceScrollValue = floatVal;
  }
  else if(*tag==COHERENCE_T) { // changing the temporal coherence
    floatVal = (float)callback_data->value * 0.01;
    sprintf(outStr,"%.3f eV",floatVal);
    XmTextSetString(tmCoherenceValue,outStr);
    tmCoherenceScrollValue = floatVal;
  }
  else if(*tag==MAJDEFOCUS) { // changing the majDefocus
    floatVal = (float)callback_data->value * 10.0;
    sprintf(outStr,"%.3f um",floatVal/10000.0);
    XmTextSetString(majDefocusValue,outStr);
    majDefocusScrollValue = floatVal/10000.0;
  }
  else if(*tag==MINDEFOCUS) { // changing the minDefocus
    floatVal = (float)callback_data->value * 10.0;
    sprintf(outStr,"%.3f um",floatVal/10000.0);
    XmTextSetString(minDefocusValue,outStr);
    minDefocusScrollValue = floatVal/10000.0;
  }
  else if(*tag==USERANG) { // changing the userAng
    floatVal = (float)callback_data->value * 0.01;
    sprintf(outStr,"%.2f",floatVal);
    XmTextSetString(userAngValue,outStr);
    userAngScrollValue = floatVal;
  }
  else if(*tag==MAJANG) { // changing the majAng
    floatVal = (float)callback_data->value * 0.01;
    sprintf(outStr,"%.2f",floatVal);
    XmTextSetString(majAngValue,outStr);
    majAngScrollValue = floatVal;
  }
  else if(*tag==WIENER) { // changing the wiener
    floatVal = (float)callback_data->value * 0.01;
    sprintf(outStr,"%.2f",floatVal);
    XmTextSetString(wienerValue,outStr);
    wienerScrollValue = floatVal;
  }
  else if(*tag==DEFOCUS) { // changing the DEFOCUS
    floatVal = (float)callback_data->value * 10.0;
    sprintf(outStr,"%.3f um",floatVal/10000.0);
    XmTextSetString(defocusValue,outStr);
    XmTextSetString(majDefocusValue,outStr);
    XmTextSetString(minDefocusValue,outStr);
    defocusScrollValue = floatVal/10000.0;
    majDefocusScrollValue = floatVal/10000.0;
    minDefocusScrollValue = floatVal/10000.0;
    if(callback_data->reason != XmCR_DRAG) {
      // Don't draw this everytime they drag e.g
      if(defocusScrollValue > 0.0)
	sprintf(outStr,"F (Under)");
      else sprintf(outStr,"F (Over)");
      simpleString = XmStringCreateLocalized(outStr);
      XtSetArg(arglist[0], XmNlabelString, simpleString);
      XtSetArg(arglist[1], XmNrecomputeSize, FALSE );
      XtSetValues(defocusLabel_widget, arglist, 2); 
      XmStringFree(simpleString); 
      XFlush(display);
    }  
  }
  
  curve[curCtf].afractScrollValue     = afractScrollValue;
  for(i=0; i<MAXCURVE; i++)
    curve[i].resolutionScrollValue = resolutionScrollValue;
  curve[curCtf].avoltageScrollValue   = avoltageScrollValue;
  curve[curCtf].spaberrationScrollValue = spaberrationScrollValue;
  curve[curCtf].chaberrationScrollValue = chaberrationScrollValue;
  curve[curCtf].spCoherenceScrollValue = spCoherenceScrollValue;
  curve[curCtf].tmCoherenceScrollValue = tmCoherenceScrollValue;
  curve[curCtf].majDefocusScrollValue = majDefocusScrollValue;
  curve[curCtf].minDefocusScrollValue = minDefocusScrollValue;
  curve[curCtf].majAngScrollValue = majAngScrollValue;
  curve[curCtf].wienerScrollValue = wienerScrollValue;
  curve[curCtf].defocusScrollValue    = defocusScrollValue;
  curve[curCtf].pfractScrollValue     = pfractScrollValue;
  curve[curCtf].spCoExp        = spCoExp;
  curve[curCtf].spCoFactor     = spCoFactor;
  
  if(callback_data->reason != XmCR_DRAG  || dragUpdate) { // update curve
    DrawCTFWindow(FALSE); 
    applyFFT();
  }
  
} // end function ctfChange



void actfPctf_change(Widget widget, int *tag,
		     XmToggleButtonCallbackStruct *callback_data)
{
  // function name: actfPctf_change
  // Whenever the actfPctf state is changed call here. Callback for
  // the colormap mode changes and also the lock on the main screen.
  
  if(*tag==ACTF) { // apply fft
    afractScrollValue = 0.0;
    pfractScrollValue = 1.0 - afractScrollValue;
    XtVaSetValues(afractScroll,
		  XmNborderColor, draw_gc_pos[curCtf],
		  XmNvalue, (int)(afractScrollValue*100),
		  NULL);
    sprintf(outStr,"%.2f/%.2f",afractScrollValue,pfractScrollValue);
    XmTextSetString(afractValue,outStr);
  }
  else if(*tag==PCTF) { // sum curves
    afractScrollValue = 1.0;
    pfractScrollValue = 1.0 - afractScrollValue;
    XtVaSetValues(afractScroll,
		  XmNborderColor, draw_gc_pos[curCtf],
		  XmNvalue, (int)(afractScrollValue*100),
		  NULL);
    sprintf(outStr,"%.2f/%.2f",afractScrollValue,pfractScrollValue);
    XmTextSetString(afractValue,outStr);
  }
  else if(*tag==ACTFPCTF) {
    // do nothing
  }

  DrawCTFWindow(FALSE); 
  applyFFT();
  
} // end function actfPctf_change



void toggle_change(Widget widget, int *tag,
		   XmToggleButtonCallbackStruct *callback_data)
{ 
  // function name: toggle_change
  // Whenever the toggle state is changed call here. Callback for the
  // colormap mode changes and also the lock on the main screen.
  
  XmString simpleString;
  
  if(*tag==APPLYFFT) { // apply fft
    applyFFTswitch = !applyFFTswitch;
    applyFFT();
  }
  else if(*tag==SUMCURVE) { // sum curves
    sumCurveSwitch = !sumCurveSwitch;
    DrawCTFWindow(FALSE); 
    applyFFT();
  }
  else if(*tag==RESODRAW)
    resoDraw= !resoDraw;
  else if(*tag==ALWAYSDRAW) { // always draw curve
    alwaysDraw[curCtf]= !alwaysDraw[curCtf];
    // Now set the toggle appropriately
    XtVaSetValues(alwaysDrawWidget,
		  XmNset, alwaysDraw[curCtf],
		  NULL);
  }
  else if(*tag==USESUM) { // use in sum calc
    useSum[curCtf]= !useSum[curCtf];
    // Now set the toggle appropriately
    XtVaSetValues(useSumWidget,
		  XmNset, useSum[curCtf],
		  NULL);
  }
  else if(*tag==BANDPASS) { // bandpass
    if(bandPass == PASSCTF)
      bandPass=NOPASS;
    else 
      bandPass=PASSCTF;
    DrawCTFWindow(FALSE); 
    applyFFT();
  }
  else if(*tag==DRAGUPDATE) { // dragupdate
    dragUpdate= !dragUpdate;
    if(dragUpdate)
      sprintf(outStr,"Redraw Curve on DRAG       ");
    else 
      sprintf(outStr,"Don't Redraw Curve on DRAG ");
    
    simpleString = XmStringCreateLocalized(outStr);
    XtSetArg(arglist[0], XmNlabelString, simpleString);
    XtSetArg(arglist[1], XmNrecomputeSize, FALSE );
    XtSetValues(dragUpdateWidget, arglist, 2); 
    XmStringFree(simpleString); 
  }
  else if(*tag==CORRECTAMPPHASE) { // amps phases
    ctfMode_div = 1;
    calcCTF(curCtf);
    DrawCTFWindow(TRUE); 
    applyFFT();
  }
  else if(*tag==CORRECTAMP) { // amps
    ctfMode_div = 3;
    calcCTF(curCtf);
    DrawCTFWindow(TRUE); 
    applyFFT();
  }
  else if(*tag==CORRECTPHASE) { // phases
    ctfMode_div = 2;
    calcCTF(curCtf);
    DrawCTFWindow(TRUE); 
    applyFFT();
  }
  else if(*tag==CORRECTNONE) { // none
    ctfMode_div = 0;
    calcCTF(curCtf);
    DrawCTFWindow(TRUE); 
    applyFFT();
  }
  else if(*tag==APPLYAMPPHASE) { // amps phases
    ctfMode = 1;
    calcCTF(curCtf);
    DrawCTFWindow(TRUE); 
    applyFFT();
  }
  else if(*tag==APPLYAMP) { // amps
    ctfMode = 3;
    calcCTF(curCtf);
    DrawCTFWindow(TRUE); 
    applyFFT();
  }
  else if(*tag==APPLYPHASE) { // phases
    ctfMode = 2;
    calcCTF(curCtf);
    DrawCTFWindow(TRUE); 
    applyFFT();
  }
  else if(*tag==APPLYNONE) { // none
    ctfMode = 0;
    calcCTF(curCtf);
    DrawCTFWindow(TRUE); 
    applyFFT();
  }

} // end function toggle_change



void createTog(Widget widget, int *tag,
	       XmAnyCallbackStruct *callback_data)
{
  // function name: createTog
  // Remember the toggle widget number for setting the toggle later.
  
  if(*tag==APPLYFFT)
    applyFFTwidget= widget;
  else if(*tag==CLEARCURVE)
    clearCurveWidget= widget;
  else if(*tag==ALWAYSDRAW)
    alwaysDrawWidget= widget;
  else if(*tag==USESUM)
    useSumWidget= widget;
  else if(*tag==SUMCURVE)
    sumCurveWidget= widget;
  else if(*tag==BANDPASS)
    bandPassWidget= widget;
  else if(*tag==DRAGUPDATE)
    dragUpdateWidget= widget;

} // end function createTog



void curveText_create(Widget widget, int *tag,
		      XmAnyCallbackStruct *callback_data)
{
  // function name: curveText_create
  // Remember the curve text widget number for use later. Init the
  // widget to a value of 1. (really curCtf== 0).

  if(*tag==TOPCURVE) { // topcurve
    topCurveTextWidget= widget;
    XmTextSetString(topCurveTextWidget, "1"); 
    topCurve= 1;
  }
  else { // The curve selector widget
    curveTextWidget= widget;
    XmTextSetString(curveTextWidget, "1"); 
    curCtf= 0;  // User sees 1 we do 0
  }
  
} // end function curveText_create



void curveText_changed(Widget  widget, int *tag,
		       XmAnyCallbackStruct *callback_data)
{
  // function name: curveText_changed
  // When they change the curveText widget they need to press the
  // "curve" button next to it to get the change instantiated. We
  // change the curve here but they need to call the curve_activate()
  // callback (called when the button is pressed) to get it displayed.

  char *curveText;

  if(*tag==TOPCURVE) { // topcurve
    curveText = XmTextGetString(topCurveTextWidget); 
    topCurve= (int)strtod(curveText,0);
    XtFree(curveText);
  }
  else { // Curve selectyor
    curveText = XmTextGetString(curveTextWidget); 
    
    curCtf = (int)strtod(curveText, 0);
    curCtf--;    // Humans number from 1
    if(curCtf <= 0)
      curCtf = 0;
    if(curCtf >= topCurve)
      curCtf = topCurve-1;
    
    // Now set the toggles appropriately
    XtVaSetValues(alwaysDrawWidget, XmNset, alwaysDraw[curCtf], NULL);
    XtVaSetValues(useSumWidget, XmNset, useSum[curCtf], NULL);
    
    afractScrollValue     = curve[curCtf].afractScrollValue;
    resolutionScrollValue = curve[curCtf].resolutionScrollValue;
    avoltageScrollValue   = curve[curCtf].avoltageScrollValue;
    spaberrationScrollValue = curve[curCtf].spaberrationScrollValue;
    chaberrationScrollValue = curve[curCtf].chaberrationScrollValue;
    spCoherenceScrollValue = curve[curCtf].spCoherenceScrollValue;
    tmCoherenceScrollValue = curve[curCtf].tmCoherenceScrollValue;
    majDefocusScrollValue = curve[curCtf].majDefocusScrollValue;
    minDefocusScrollValue = curve[curCtf].minDefocusScrollValue;
    majAngScrollValue = curve[curCtf].majAngScrollValue;
    wienerScrollValue = curve[curCtf].wienerScrollValue;
    defocusScrollValue    = curve[curCtf].defocusScrollValue;
    pfractScrollValue     = curve[curCtf].pfractScrollValue;
    spCoExp     = curve[curCtf].spCoExp;
    spCoFactor  = curve[curCtf].spCoFactor;
    
    setCurveVals(TRUE);
    
    if(inited) { // Only draw when we've got everything we need to draw
      DrawCTFWindow(FALSE); 
      applyFFT();
    }
    
    XtFree(curveText);
    
  }
  
} // end function curveText_changed



void curveArrowActivate(Widget widget, int *tag,
			XmAnyCallbackStruct *callback_data)
{
  // function name: curveArrowActivate
  // Instead of making them type the new curve just let them click the
  // arrow. Change the curve accordingly.
  
  char newVal[12];

  if(*tag==XmARROW_UP)
      curCtf++;
  else if(*tag==XmARROW_DOWN)
      curCtf--;
      
  curCtf = MAX(curCtf, 0);
  curCtf = MIN(curCtf, (topCurve-1));

  sprintf(newVal,"%d",(curCtf+1));  // User sees 1 for 0
  XmTextSetString(curveTextWidget, newVal);

} // end function curveArrowActivate



void display_activate( Widget  widget, int *tag,
		       XmAnyCallbackStruct *callback_data)
{
  // function name: display_activate
  // Put the appropriate buffer on display on the main screen.

  if(*tag != FFT && *tag != CTF && *tag != BACK && *tag != BACKSTRETCH)
    return;
  
  imageType = *tag;
  
  if(*tag==FFT)
    stuffImage(raw_fft_image_buffer1, image_width, image_height, 256, TRUE);
  else if(*tag==CTF)
    stuffImage(fft_image_buffer1, image_width, image_height, 256, TRUE);
  else if(*tag==BACK)
    stuffImage(back_fft_image_buffer1, image_width, image_height, 256, TRUE);
  else if(*tag==BACKSTRETCH)
    stuffImage(back_stretch_fft_image_buffer1, image_width,
	       image_height, 256, TRUE);
  
  DrawWindow();
  XFlush (display);
  
} // end function display_activate



void fftMinMaxChange(Widget widget, int *tag,
		     XmScrollBarCallbackStruct *callback_data)
{ 
  // function name: fftMinMaxChange
  // min/max has changed so redraw FFT

  if(*tag==0)
    hiFFTscroll = callback_data->value/100.0;
  else 
    loFFTscroll = callback_data->value/100.0;

  if(loFFTscroll >= hiFFTscroll)
    hiFFTscroll = loFFTscroll + 0.01;

  applyFFT();

} // end function fftMinMaxChange



void resoExampleChange(Widget widget, int *tag,
		       XmScrollBarCallbackStruct *callback_data)
{
  // function name: resoExampleChange
  // They have a new resolution value they want to try.

  int thisVal;
  char outStr[64];
  XmString simpleString;

  thisVal = (int)(callback_data->value);
  scrollResolutionExample = (float)thisVal;

  sprintf(outStr,"%.2f",(256.0/thisVal));
  simpleString = XmStringCreateLocalized(outStr);
  XtSetArg(arglist[0], XmNlabelString, simpleString);
  XtSetArg(arglist[1], XmNrecomputeSize, FALSE );
  XtSetValues(resoExampleLabel_widget, arglist, 2); 
  XmStringFree(simpleString); 
  XFlush(display);
  
  if(*tag != DRAG)
    DrawResoExample(resoY);

} // end function resoExampleChange



void resoExampleLabel_create(Widget  widget, int *tag,
			     XmAnyCallbackStruct *callback_data)
{
  // resoExampleLabel_create
  // Remember the widget number for later

  if(*tag==RESOLUTION)
    resoExampleLabel_widget= widget;
  else if(*tag==DEFOCUS)
    defocusLabel_widget= widget;
  else if(*tag==SECONDDISPLAY)
    secondDisplayButton = widget;
  else if(*tag==PARAMFILE)
    paramFileW = widget;
  else if(*tag==PARAMNP)
    paramNPW = widget;
  else if(*tag==CTFCFF2)
    ctfCorrectFF2LabelW = widget;

} // end function 



void resoExample_activate(Widget  widget, int *tag,
			  XmAnyCallbackStruct *callback_data)
{
  // function name: resoExample_activate
  // Get the resolution example interface up on the display.

  XtManageChild(resoExample_screen);

} // end function resoExample_activate



void resoExample_deactivate(Widget  widget, int *tag,
			    XmAnyCallbackStruct *callback_data)
{
  // function name: resoExample_deactivate
  // Get rid of reso example interface.

  XtUnmanageChild(resoExample_screen);

} // end function resoExample_deactivate



void reso_reset_activate(Widget  widget, int *tag,
			 XmAnyCallbackStruct *callback_data)
{
  // function name: reso_reset_activate
  // Reset to the appropriate resolution.

  char buttonStr[34];
  XmString simpleString;
  float oldResXlo, oldResXhi;

  // Redraw button text to indicate current resolution with '*'
  XtSetArg(arglist[0], XmNrecomputeSize, FALSE );

  if(*tag==R40ANG)
    sprintf(buttonStr,"40 Ang  *");
  else 
    sprintf(buttonStr,"40 Ang  ");
  simpleString = XmStringCreateLocalized(buttonStr);
  XtSetArg(arglist[1], XmNlabelString, simpleString);
  XtSetValues(resoButtonW[R40ANG], arglist, 2); 
  XmStringFree(simpleString); 
  
  if(*tag==R20ANG)
    sprintf(buttonStr,"20 Ang  *");
  else 
    sprintf(buttonStr,"20 Ang  ");
  simpleString = XmStringCreateLocalized(buttonStr);
  XtSetArg(arglist[1], XmNlabelString, simpleString);
  XtSetValues(resoButtonW[R20ANG], arglist, 2); 
  XmStringFree(simpleString); 

  if(*tag==R10ANG)
    sprintf(buttonStr,"10 Ang  *");
  else 
    sprintf(buttonStr,"10 Ang  ");
  simpleString = XmStringCreateLocalized(buttonStr);
  XtSetArg(arglist[1], XmNlabelString, simpleString);
  XtSetValues(resoButtonW[R10ANG], arglist, 2); 
  XmStringFree(simpleString); 

  if(*tag==R5ANG)
    sprintf(buttonStr,"5 Ang  *");
  else 
    sprintf(buttonStr,"5 Ang  ");
  simpleString = XmStringCreateLocalized(buttonStr);
  XtSetArg(arglist[1], XmNlabelString, simpleString);
  XtSetValues(resoButtonW[R5ANG], arglist, 2); 
  XmStringFree(simpleString); 

  if(*tag==R4ANG)
    sprintf(buttonStr,"4 Ang  *");
  else 
    sprintf(buttonStr,"4 Ang  ");
  simpleString = XmStringCreateLocalized(buttonStr);
  XtSetArg(arglist[1], XmNlabelString, simpleString);
  XtSetValues(resoButtonW[R4ANG], arglist, 2); 
  XmStringFree(simpleString); 

  if(*tag==R3ANG)
      sprintf(buttonStr,"3 Ang  *");
  else 
    sprintf(buttonStr,"3 Ang  ");
  simpleString = XmStringCreateLocalized(buttonStr);
  XtSetArg(arglist[1], XmNlabelString, simpleString);
  XtSetValues(resoButtonW[R3ANG], arglist, 2); 
  XmStringFree(simpleString); 

  if(*tag==R2ANG)
    sprintf(buttonStr,"2 Ang  *");
  else 
    sprintf(buttonStr,"2 Ang ");
  simpleString = XmStringCreateLocalized(buttonStr);
  XtSetArg(arglist[1], XmNlabelString, simpleString);
  XtSetValues(resoButtonW[R2ANG], arglist, 2); 
  XmStringFree(simpleString); 

  if(*tag==R1ANG)
    sprintf(buttonStr,"1 Ang  *");
  else 
    sprintf(buttonStr,"1 Ang  ");
  simpleString = XmStringCreateLocalized(buttonStr);
  XtSetArg(arglist[1], XmNlabelString, simpleString);
  XtSetValues(resoButtonW[R1ANG], arglist, 2); 
  XmStringFree(simpleString); 

  oldResXlo= (((float)resXlo)/512.0*resolutionScrollValue);
  oldResXhi= (((float)resXhi)/512.0*resolutionScrollValue);

  if(*tag==R40ANG)
    resolutionScrollValue = 10.0 / 40.0;
  else if(*tag==R20ANG)
    resolutionScrollValue = 10.0 / 20.0;
  else if(*tag==R10ANG)
    resolutionScrollValue = 10.0 / 10.0;
  else 
    resolutionScrollValue = 10.0 / (float)(*tag);

  resXlo= ((float)oldResXlo/resolutionScrollValue)*512.0;
  resXhi= ((float)oldResXhi/resolutionScrollValue)*512.0;


  for(i=0; i<MAXCURVE; i++)
    curve[i].resolutionScrollValue = resolutionScrollValue;

  // Make sure we get all curves recalculated
  for(i=0; i<topCurve; i++) {
    afractScrollValue     = curve[i].afractScrollValue;
    resolutionScrollValue = curve[i].resolutionScrollValue;
    avoltageScrollValue   = curve[i].avoltageScrollValue;
    spaberrationScrollValue = curve[i].spaberrationScrollValue;
    chaberrationScrollValue = curve[i].chaberrationScrollValue;
    spCoherenceScrollValue = curve[i].spCoherenceScrollValue;
    tmCoherenceScrollValue = curve[i].tmCoherenceScrollValue;
    majDefocusScrollValue = curve[i].majDefocusScrollValue ;
    minDefocusScrollValue = curve[i].minDefocusScrollValue  ;
    majAngScrollValue = curve[i].majAngScrollValue ;
    wienerScrollValue = curve[i].wienerScrollValue;
    defocusScrollValue    = curve[i].defocusScrollValue;
    pfractScrollValue     = curve[i].pfractScrollValue;
    spCoExp               = curve[i].spCoExp;
    spCoFactor            = curve[i].spCoFactor;
    calcCTF(i);
  }
  
  // Now set these back to original
  afractScrollValue     = curve[curCtf].afractScrollValue;
  resolutionScrollValue = curve[curCtf].resolutionScrollValue;
  avoltageScrollValue   = curve[curCtf].avoltageScrollValue;
  spaberrationScrollValue = curve[curCtf].spaberrationScrollValue;
  chaberrationScrollValue = curve[curCtf].chaberrationScrollValue;
  spCoherenceScrollValue = curve[curCtf].spCoherenceScrollValue;
  tmCoherenceScrollValue = curve[curCtf].tmCoherenceScrollValue;
  majDefocusScrollValue = curve[curCtf].majDefocusScrollValue ;
  minDefocusScrollValue = curve[curCtf].minDefocusScrollValue  ;
  majAngScrollValue = curve[curCtf].majAngScrollValue ;
  wienerScrollValue = curve[curCtf].wienerScrollValue;
  defocusScrollValue    = curve[curCtf].defocusScrollValue;
  pfractScrollValue     = curve[curCtf].pfractScrollValue;
  spCoExp               = curve[curCtf].spCoExp;
  spCoFactor            = curve[curCtf].spCoFactor;

  DrawCTFWindow(TRUE); 

  applyFFT();

} // end function reso_reset_activate



void reso_button_create(Widget widget, int *tag,
			XmAnyCallbackStruct *callback_data)
{
  // function name: reso_button_create
  // Remember widgets

  if(*tag == R40ANG || *tag == R20ANG || *tag == R10ANG || *tag == R5ANG ||
     *tag == R4ANG  || *tag == R3ANG  || *tag == R2ANG  || *tag == R1ANG)
    resoButtonW[*tag] = widget;

} // end function reso_button_create



void defocusArrowActivate(Widget widget, int *tag,
			  XmAnyCallbackStruct *callback_data)
{
  // function name: defocusArrowActivate
  // They're changing the defocus using the arrow widgets.
  
  char outStr[32];
  XmString simpleString;
  int increment, pageIncrement;

  if(*tag==XmARROW_UP) {
    defocusMin *= 2;
    defocusMax *= 2;
  }
  else if(*tag==XmARROW_DOWN) {
    defocusMin = defocusMin / 2;
    defocusMax = defocusMax / 2;
  }
  else 
    printf("Defocus unchanged.\n");
  
  if(defocusMin < -10000000)
    defocusMin= -10000000;
  if(defocusMax > 10000000)
    defocusMax= 10000000;
  
  if(defocusScrollValue > 0.0)
    sprintf(outStr,"F (Under)");
  else 
    sprintf(outStr,"F (Over)");
  simpleString = XmStringCreateLocalized(outStr);
  XtSetArg(arglist[0], XmNlabelString, simpleString);
  XtSetArg(arglist[1], XmNrecomputeSize, FALSE );
  XtSetValues(defocusLabel_widget, arglist, 2); 
  XmStringFree(simpleString); 
  XFlush(display);
  
  increment = (defocusMax-defocusMin)/2000;
  if(increment < 1)
    increment = 1;
  pageIncrement = (defocusMax-defocusMin)/200;
  if(pageIncrement < 10)
    pageIncrement = 10;
  
  XtVaSetValues(majDefocusScroll, XmNvalue, 
		(int)(majDefocusScrollValue* 1000.0), NULL);
  XtVaSetValues(majDefocusScroll, XmNincrement, 
		increment, NULL);
  XtVaSetValues(majDefocusScroll, XmNpageIncrement, 
		pageIncrement, NULL);
  XtVaSetValues(majDefocusScroll, XmNminimum, 
		defocusMin, NULL);
  XtVaSetValues(majDefocusScroll, XmNmaximum, 
		(int)((defocusMax+pageIncrement)*1.25), NULL);

  XtVaSetValues(minDefocusScroll, XmNvalue, 
		(int)(minDefocusScrollValue* 1000.0), NULL);
  XtVaSetValues(minDefocusScroll, XmNincrement, 
		increment, NULL);
  XtVaSetValues(minDefocusScroll, XmNpageIncrement, 
		pageIncrement, NULL);
  XtVaSetValues(minDefocusScroll, XmNminimum, 
		defocusMin, NULL);
  XtVaSetValues(minDefocusScroll, XmNmaximum, 
		(int)((defocusMax+pageIncrement)*1.25), NULL);

  XtVaSetValues(defocusScroll, XmNvalue, 
		(int)(defocusScrollValue* 1000.0), NULL);
  XtVaSetValues(defocusScroll, XmNincrement, 
		increment, NULL);
  XtVaSetValues(defocusScroll, XmNpageIncrement, 
		pageIncrement, NULL);
  XtVaSetValues(defocusScroll, XmNminimum, 
		defocusMin, NULL);
  XtVaSetValues(defocusScroll, XmNmaximum, 
		(int)((defocusMax+pageIncrement)*1.25), NULL);

} // end function defocusArrowActivate



void defocusSchertzerActivate(Widget widget, int *tag,
			      XmAnyCallbackStruct *callback_data)
{
  // function name: defocusSchertzerActivate
  // They've pressed the Schertzer button on the interface - 
  // setup for Schertzer defocus.

  float wave;

  wave = wavel_(&avoltageScrollValue);
  defocusScrollValue = sqrt ( (spaberrationScrollValue * wave) );
  XtVaSetValues(defocusScroll,
		XmNvalue, (int)(defocusScrollValue* 1000.0),
		NULL);
  majDefocusScrollValue = defocusScrollValue;
  XtVaSetValues(majDefocusScroll,
		XmNvalue, (int)(majDefocusScrollValue* 1000.0),
		NULL);
  minDefocusScrollValue = defocusScrollValue;
  XtVaSetValues(minDefocusScroll,
		XmNvalue, (int)(minDefocusScrollValue* 1000.0),
		NULL);

  curve[curCtf].defocusScrollValue    = defocusScrollValue;
  curve[curCtf].minDefocusScrollValue = minDefocusScrollValue;
  curve[curCtf].majDefocusScrollValue = majDefocusScrollValue;
  
  // Now setup the scrolls properly
  setCurveVals(TRUE);

  XFlush(display);
  
  DrawCTFWindow(FALSE); 

  applyFFT();

} // end function defocusSchertzerActivate



void ctf_pointFB_activate(Widget  widget, int *tag,
			  XmAnyCallbackStruct *callback_data)
{
  // function name: ctf_pointFB_activate
  // Change where they want the pointer info drawn.

  XDrawString(display, ctf_image_window1, imageCTF_gcGRAY,
	      (drawPWX), drawPointWhere, 
	      motionNumStr, strlen(motionNumStr));

  if(drawPointWhere==14) { // Middle
    drawPointWhere = 245;
    drawPWX = 128;
  }
  else { // Top
    drawPointWhere = 14;
    drawPWX = 7;
  }  

} // end function ctf_pointFB_activate



void spCoArrowActivate(Widget widget, int *tag,
		       XmAnyCallbackStruct *callback_data)
{
  // function name: spCoArrowActivate
  // Call here when spherical coherence (Beta) arrows clicked.

  char outStr[32];
  XmString simpleString;

  if(*tag==XmARROW_UP)
    spCoExp++;
  else if(*tag==XmARROW_DOWN)
    spCoExp--;
  else 
    printf("spCo unchanged.\n");

  spCoExp = MIN(MAX(spCoExp,-99),0);

  // Stuff the label   
  // spCoExpLabelW

  if(spCoExp < 0)
    sprintf(outStr," * E%03d",spCoExp);
  else 
    sprintf(outStr," * E+%02d",spCoExp);

  simpleString = XmStringCreateLocalized(outStr);
  XtSetArg(arglist[0], XmNlabelString, simpleString);
  XtSetArg(arglist[1], XmNrecomputeSize, FALSE );
  XtSetValues(spCoExpLabelW, arglist, 2); 
  XmStringFree(simpleString); 
  XFlush(display);

  sprintf(outStr,"1E%d", spCoExp);
  spCoFactor = strtod(outStr, 0);

  XFlush(display);
  DrawCTFWindow(FALSE); 
  applyFFT();

} // end function spCoArrowActivate



void bandPass_flip_activate(Widget  widget, int *tag,
			    XmToggleButtonCallbackStruct *callback_data)
{
  // function name: bandPass_flip_activate
  // Change the concept of where the bandpass is evaluated.

  middlePass= !middlePass;
  DrawCTFWindow(FALSE);
  applyFFT();

} // end function bandPass_flip_activate



void ctfCorrect_activate(Widget widget, int *tag,
			 XmToggleButtonCallbackStruct *callback_data)
{
  // function name: ctfCorrect_activate
  // Get ctf correction interface up on the screen.

  XtManageChild(ctfCorrectScreen);
  XtPopup(XtParent(ctfCorrectScreen), XtGrabNone);
  DrawCTFWindow(TRUE); // Redraw it

} // end function ctfCorrect_activate



void ctfCorrect_deactivate(Widget widget, int *tag,
			   XmToggleButtonCallbackStruct *callback_data)
{
  // function name: ctfCorrect_deactivate
  // Get rid of ctf correction interface.

  XtUnmanageChild(ctfCorrectScreen);
  DrawCTFWindow(TRUE); // Redraw it
  applyFFT();

} // end function ctfCorrect_deactivate



void ctfCorrect_toggle_change(Widget widget, int *tag,
			      XmToggleButtonCallbackStruct *callback_data)
{
  // function name: ctfCorrect_toggle_change
  // Toggle which method(s) to use for ctf correction.
  
  if(ctfMeth5 && *tag!=CTFCMETH5) { // Clean up from last time
    sprintf(outStr,"%.2f",filt_fac[1]);
    XmTextSetString(ctfCorrectFFW,outStr);
    sprintf(outStr,"   ");
    XmTextSetString(ctfCorrectFF2W,outStr);
    filt_fac[4] = filt_fac[0];
    filt_fac[0] = filt_fac[1];
    filt_fac[1] = filt_fac[4];
  }
  
  if(*tag==CTFCORRECT)
    dispCtfCorrect = !dispCtfCorrect;
  else // Turn switches
    ctfMeth1 = ctfMeth2 = ctfMeth3 = ctfMeth4 = ctfMeth5 = FALSE;
  
  if(*tag==NONE) { // Do nothing
    filter[0] = 0;
    XtSetArg(arglist[0], XmNsensitive, FALSE);
    XtSetValues(ctfCorrectFF2W, arglist, 1);  
  }
  else if(*tag==CTFCMETH1) {
    ctfMeth1 = TRUE;
    filter[0] = 5;
    XtSetArg(arglist[0], XmNsensitive, FALSE);
    XtSetValues(ctfCorrectFF2W, arglist, 1);  
    XtSetValues(ctfCorrectFF2LabelW, arglist, 1);  
  }
  else if(*tag==CTFCMETH2) {
    ctfMeth2 = TRUE;
    filter[0] = 4;
    XtSetArg(arglist[0], XmNsensitive, FALSE);
    XtSetValues(ctfCorrectFF2W, arglist, 1);  
    XtSetValues(ctfCorrectFF2LabelW, arglist, 1);  
  }
  else if(*tag==CTFCMETH3) {
    ctfMeth3 = TRUE;
    filter[0] = 3;
    XtSetArg(arglist[0], XmNsensitive, FALSE);
    XtSetValues(ctfCorrectFF2W, arglist, 1);  
    XtSetValues(ctfCorrectFF2LabelW, arglist, 1);  
  }
  else if(*tag==CTFCMETH4) {
    ctfMeth4 = TRUE;
    filter[0] = 2;
    XtSetArg(arglist[0], XmNsensitive, FALSE);
    XtSetValues(ctfCorrectFF2W, arglist, 1);  
    XtSetValues(ctfCorrectFF2LabelW, arglist, 1);  
  }
  else if(*tag==CTFCMETH5) {
    ctfMeth5 = TRUE;
    filter[0] = 1;
    XtSetArg(arglist[0], XmNsensitive, TRUE);
    XtSetValues(ctfCorrectFF2W, arglist, 1);  
    XtSetValues(ctfCorrectFF2LabelW, arglist, 1);  
    sprintf(outStr,"%.2f",filt_fac[0]);
    XmTextSetString(ctfCorrectFF2W,outStr);
    sprintf(outStr,"%.2f",filt_fac[1]);
    XmTextSetString(ctfCorrectFFW,outStr);
    filt_fac[4] = filt_fac[0];
    filt_fac[0] = filt_fac[1];
    filt_fac[1] = filt_fac[4];
  }
  
  DrawCTFWindow(TRUE); // Redraw it
  
  applyFFT();
  
} // end function ctfCorrect_toggle_change



void ctfCorrect_tryit_activate(Widget widget, int *tag,
			       XmToggleButtonCallbackStruct *callback_data)
{ 
  // function name: ctfCorrect_tryit_activate
  // Experimental - Empty for now
  
} // end function ctfCorrect_tryit_activate



void secondDisplay_activate(Widget widget, int *tag,
			    XmAnyCallbackStruct *callback_data)
{
  // function name: secondDisplay_activate
  // Instead of making them type the new curve just let them click the
  // arrow. Change the curve accordingly.
  
  int tag2;
  XmString simpleString;
  
  secondDisplay ++;
  if(secondDisplay > 3 || secondDisplay < 0)
    secondDisplay = 0;
  
  if(secondDisplay == 0) {
    tag2 = BACK;
    sprintf(outStr, "Simulated Image");
  }
  else if(secondDisplay == 1) {
    tag2 = BACKSTRETCH;
    sprintf(outStr, "Simulated Image Stretched");
  }
  else if(secondDisplay == 2) {
    tag2 = FFT;
    sprintf(outStr, "Original FFT");
  }
  else if(secondDisplay == 3) {
    tag2 = CTF;
    sprintf(outStr, "Ctf FFT");
  }

  simpleString = XmStringCreateLocalized(outStr);
  XtSetArg(arglist[0], XmNlabelString, simpleString);
  XtSetArg(arglist[1], XmNrecomputeSize, FALSE );
  XtSetValues(secondDisplayButton, arglist, 2); 
  XmStringFree(simpleString); 
  XFlush(display);
  
  display_activate(0, &tag2, 0);

} // end function secondDisplay_activate



void zoomActivate(Widget widget, int *tag,
		  XmAnyCallbackStruct *callback_data)
{
  // function name: zoomActivate
  // Instead of making them type the new curve just let them click the
  // arrow.  Change the curve accordingly.

  float oldVal,newVal;

  //  maxCTFCorrectYvalue = 5.0 + 1.0;  Add in the negative 1.0 values
  oldVal = maxCTFCorrectYvalue - 1.0;
  newVal = oldVal;

  if(*tag == XmARROW_UP)
    newVal = oldVal * 2.0;
  else if(*tag == XmARROW_DOWN)
    newVal = oldVal * 0.5;
  else 
    printf("Y scale Zoom unchanged.\n");
  
  if(newVal <= 2.5)
    newVal = 2.5;

  maxCTFCorrectYvalue = newVal + 1.0;

  DrawCTFWindow(TRUE);

} // end function zoomActivate
