C     Compute the value of the exponential decay arising from limits on
C     the temporal coherence, using the following formula that is
C     described by Zemlin in Ultramicroscopy 46: 25-32 (1992)
C                                                                            
C                  [ (cc * de * wavelength * d_star**2)**2]                  
C         et = exp [-(--------------------------------)   ]                  
C                  [ (      e                         )   ]                  
C                                                                            
C     Where cc is the chromatic aberration coefficient (in mm)           
C           de is the energy spread (in electron volts)                  
C           e  is the accelerating votage (in electron volts)            
C           wavelength is the electron wavelength (in nm)                
C           d_star is the spatial frequency (in nm-1)                    
C                                                                            
C     Calculations are performed with variables in nm units.                
C     
C     August 1999 Purdue University Alphastation 500/400, T. Baker.         
C     Modified 31-aug-1999 by TSB
C     General cleanup, 11/15/2006 by Robert Sinkovits

      function ctf_et (cc, de, e, wavelength, d_star)

      implicit none

      real, intent(in) :: cc, de, e, wavelength, d_star

      real ctf_et
      double precision d_et, d_star_sq, part1, d_cc, d_wave

C     Convert from mm to nm
      d_cc      = cc * 1000000.0

      d_wave    = wavelength
      d_star_sq = d_star * d_star
      part1     = d_cc * d_wave * (de/e) * d_star_sq

      if(abs(part1) > 7.0) then
         d_et = 0.0
      else
         d_et = exp( -part1*part1 )
      endif

      ctf_et = real(d_et)

      return
      end function ctf_et
