C  ****************************************************************************
C  *                                                                          *
C  *                *****     SUBROUTINE CTF_MULTIPLY     *****               *
C  *                                                                          *
C  *    MULTIPLY IMAGE FFT BY CTF.                                            *
C  *                                                                          *
C  *    PHI     = ANGLE (IN RADIANS) SPECIFYING THE DIRECTION OF FOCUS_MAJ    *
C  *               RELATIVE TO THE X-AXIS OF THE DIGITIZED IMAGE.             *
C  *    PSI     = ANGLE (IN RADIANS) OF A PARTICULAR FOURIER COMPONENT WHOSE  *
C  *               DEFOCUS VALUE IS TO BE CALCULATED.                         *
C  *    TEMPFAC = TEMPERATURE FACTOR, SPECIFIED IN ANGSTROMS SQUARED, IS      *
C  *               USED TO DAMPEN HIGH RESOUTION FEATURES.                    *
C  *                                                                          *
C  *    CTFMODE = 0 DO NOTHING!                                               *
C  *            = 1 MODIFY PHASES AND AMPLITUDES                              *
C  *            = 2 FLIP PHASES WHERE NECESSARY; LEAVE AMPLITUDES UNCHANGED   *
C  *            = 3 ADJUST AMPLITUDES; LEAVE PHASES UNCHANGED                 *
C  *                                                                          *
C  *    AUGUST 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.         *
C  *    MODIFIED 13-AUG-1999 BY TSB (Fixed XTEST,YTEST problems).             *
C  *    MODIFIED  2-MAY-2000 BY TSB (Added CTF_MODE switch).                  *
C  *    LAST MODIFIED  2-MAY-2000 BY TSB/RWA (Implement CTF decay).           *
C  ****************************************************************************
C
      SUBROUTINE CTF_MULTIPLY (CTFMODE, FFT, IDIM1, IDIM2, UNITS, 
     1 PIXSIZE, PI, RADIAN, WAVELENGTH, FOCUS_MAJ, FOCUS_MIN, ANG_MAJ, 
     2 AMP_FAC, Cs, Cc, BETA, dE, VOLTS, TEMPFAC, FFT_FULL)
C
      IMPLICIT NONE
C
      INTEGER*4 IDIM1, IDIM2, COUNT, IX, IY, IY2, XTEST, YTEST
      INTEGER*4 UNITS, CTFMODE
      REAL*4    FFT(1), YSQ, C, D_STAR, PIXSIZE, TMP
      REAL*4    FOCUS_MAJ, FOCUS_MIN, ANG_MAJ, PSI, PHI, X, Y
      REAL*4    PI, RADIAN, WAVELENGTH, dF, AMP_FAC, Cs
      REAL*4    TEMPFAC, T, TWOPI, FACTOR, Cc, BETA, dE, VOLTS
      LOGICAL   FFT_FULL, ASTIG
C  ----------------------------------------------------------------------------
C
      IF(CTFMODE.LE.0 .OR. CTFMODE.GT.3) RETURN
C
      TWOPI = 2.0*PI
      PHI   = ANG_MAJ * RADIAN        ! ANGLE OF FOCUS_MAJ WRT X-DIRECTION
C
      IF(UNITS.EQ.1) THEN
         TMP = 10.0/(IDIM1*PIXSIZE)                   ! CONVERT TO NM UNITS
       ELSE
         TMP = 1.0/(IDIM1*PIXSIZE)            ! ALREADY IN NM OR PIXEL UNITS
      ENDIF
C
      IF(FOCUS_MAJ.EQ.FOCUS_MIN) THEN
         ASTIG = .FALSE.
         dF    = FOCUS_MAJ
       ELSE
         ASTIG = .TRUE.
      ENDIF
C
      XTEST  = IDIM1/2
      YTEST  = IDIM2/2
      COUNT = -1
C
      IF(FFT_FULL) THEN
         IY2 = IDIM2
       ELSE
         IY2 = YTEST + 1
      ENDIF
C
      DO IY = 1,IY2                        ! PROCESS FFT A ROW AT A TIME
        IF(IY.GT.YTEST) THEN
           Y = REAL(IY-IDIM2-1)
         ELSE
           Y = REAL(IY-1)
        ENDIF
        YSQ = Y**2
        DO IX = 1,IDIM1                  ! PROCESS CURRENT ROW
          COUNT = COUNT + 2
          IF(IX.LE.XTEST) THEN
             X = REAL(IX-1)                  ! RIGHT HALF OF FFT
           ELSE
             X = REAL(IX-IDIM1-1)            ! LEFT HALF OF FFT
          ENDIF
          D_STAR = SQRT(YSQ + X**2) * TMP
          IF(ABS(X).GT.0.9) THEN            ! TRAP FOR FFT Y-AXIS DATA
             PSI = ATAN2 (Y,X)            ! ATAN2 GIVES ANGLE IN RADIANS
           ELSE
             IF(Y.GE.0.0) THEN
                PSI =  PI/2.0
              ELSE
                PSI = -PI/2.0
             ENDIF
          ENDIF
          IF(ASTIG) CALL FOCUS_ASTIG(dF,FOCUS_MAJ,FOCUS_MIN,PSI,PHI)
          call CTF_FUNC(C, D_STAR, PI, WAVELENGTH, dF, AMP_FAC, Cs, 
     $         Cc, BETA, dE, VOLTS)
          T = EXP( -((TEMPFAC/400.0) * D_STAR**2) )            ! ADJUSTED FOR NM
          FACTOR = ABS(T*C)
          IF(CTFMODE.EQ.1) THEN             ! ADJUST PHASES AND AMPLITUDES
             IF(C.GT.0.0) FACTOR = -FACTOR
             FFT(COUNT)   = FACTOR * FFT(COUNT)
             FFT(COUNT+1) = FACTOR * FFT(COUNT+1)
           ELSE IF(CTFMODE.EQ.2) THEN                     ! ADJUST PHASES ONLY
             IF(C.GT.0.0) THEN
               FFT(COUNT)   = -FFT(COUNT)
               FFT(COUNT+1) = -FFT(COUNT+1)
             ENDIF
           ELSE IF(CTFMODE.EQ.3) THEN                   ! ADJUST AMPLITUDES ONLY
             FFT(COUNT)   = FACTOR * FFT(COUNT)
             FFT(COUNT+1) = FACTOR * FFT(COUNT+1)
          ENDIF
        ENDDO
      ENDDO
C
      RETURN
      END
